#ifndef DHNETSDK_H
#define DHNETSDK_H


#if defined(WIN32) || defined(_WIN32) || defined(_WIN64)
    #include <windows.h>
    #ifdef NETSDK_EXPORTS
        #define CLIENT_NET_API  __declspec(dllexport)
    #else
        #define CLIENT_NET_API  __declspec(dllimport)
    #endif

    #define CALLBACK        __stdcall
    #define CALL_METHOD     __stdcall  //__cdecl

    #define INT64           __int64
    #define TP_U64          unsigned __int64

    #ifndef LLONG
        #ifdef _WIN64
            #define LLONG   INT64
        #else 
            #define LLONG   LONG
        #endif
    #endif

    #ifndef LDWORD
        #ifdef _WIN64
            #define LDWORD  INT64
        #else 
            #define LDWORD  DWORD
        #endif
    #endif
#else    //non-windows
    #define CLIENT_NET_API  extern "C"
    #define CALL_METHOD 
    #define CALLBACK

    #ifndef INTERNAL_COMPILE
        #define RELEASE_HEADER
    #endif
 
    #ifdef RELEASE_HEADER
        #define WORD        unsigned short
        #define DWORD       unsigned int
        #define LONG        int
        #define LPDWORD     DWORD*

        #ifdef __OBJC__
            #include "objc/objc.h"
        #else
            #define BOOL    int
        #endif

        #ifndef TRUE
        #define TRUE        1
        #endif

        #ifndef FALSE
        #define FALSE       0
        #endif
        #define BYTE        unsigned char
        #define UINT        unsigned int
        #define HDC         void*
        #define HWND        void*
        #define LPVOID      void*

        #ifndef NULL
        #define NULL        0
        #endif

        #define LLONG       long
        #define INT64       long long
        #define TP_U64      unsigned long long
        #define LDWORD      long 

        #ifndef MAX_PATH
        #define MAX_PATH    260
        #endif

        #ifndef DEF_RECT
        typedef struct  tagRECT
        {
            LONG left;
            LONG top;
            LONG right;
            LONG bottom;
        } RECT;
        #define DEF_RECT
        #endif
    #else	//Internal translation
        #include "../Platform/osIndependent.h"
        #define INT64       int64
        #define TP_U64      uint64
    #endif // RELEASE_HEADER
#endif // linux

#ifndef LDWORD
    #if defined(WIN32) || defined(_WIN32) || defined(_WIN64)
        #ifdef _WIN64
            #define LDWORD  __int64
        #else //WIN32 
            #define LDWORD  DWORD
        #endif
    #else    //linux
        #define LDWORD      long 
    #endif
#endif

#ifdef __cplusplus
extern "C" {
#endif

/************************************************************************
 ** Constant Definition
 ***********************************************************************/
#define DH_SERIALNO_LEN                     48          // Device SN string length
#define DH_MAX_DISKNUM                      256         // Max HDD number
#define DH_MAX_SDCARDNUM                    32          // Max SD card number
#define DH_MAX_BURNING_DEV_NUM              32          // Max burner amount
#define DH_BURNING_DEV_NAMELEN              32          // Burner name max length 
#define DH_MAX_LINK                         6           
#define DH_MAX_CHANNUM                      16          // Max channel amount
#define DH_MAX_DECODER_CHANNUM              64          // decoder device max channel number
#define DH_MAX_ALARMIN                      128         // Max alarm input amount 
#define DH_MAX_ALARMOUT                     64          // Max alarm output amount 
#define DH_MAX_RIGHT_NUM                    100         // Max user right amount
#define DH_MAX_GROUP_NUM                    20          // Max user group amount
#define DH_MAX_USER_NUM                     200         // Max user account amount
#define DH_RIGHT_NAME_LENGTH                32          // Right name length
#define DH_USER_NAME_LENGTH                 8           // User name length 
#define DH_USER_PSW_LENGTH                  8           // User password length 
#define DH_CUSTOM_NAME_LENGTH               32          // Custom name length
#define DH_USER_NAME_LEN_EX                 32          // User name length ,for extension
#define DH_USER_PSW_LEN_EX                  32          // User password length ,for extension
#define DH_MEMO_LENGTH                      32          // Note length 
#define DH_MAX_STRING_LEN                   128         
#define MAX_STRING_LINE_LEN                 6           // Max six rows
#define MAX_PER_STRING_LEN                  20          // Line max length 
#define DH_MAX_MAIL_NAME_LEN                64          // The user name length the new mail structure supported
#define DH_MAX_MAIL_PSW_LEN                 64          // Password length the new mail structhre supported
#define DH_SPEEDLIMIT_TYPE_LEN              32          // speed limit string max length
#define DH_VEHICLE_TYPE_LEN                 32          // vehicle info type max length
#define DH_VEHICLE_INFO_LEN                 32          // vehicle info string max length
#define DH_VEHICLE_DRIVERNO_LEN             32          // Driver no string max length=
#define DH_MAX_CROSSING_NUM                 128         // Max supported crossing num 
#define DH_MAX_CROSSING_ID                  32          // Max crossing ID length 
#define DH_MAX_CARD_INFO_LEN                256         // max card info length
#define DH_MAX_CHANNUM_EX                   32          // extended max channel number
#define DH_MAX_SAERCH_IP_NUM                256         // max number of IP for saerch
#define DH_MAX_HARDDISK_TYPE_LEN            32          // hard disk type maximum length
#define DH_MAX_HARDDISK_SERIAL_LEN          32          // hard disk serial maximum length
#define DH_MAX_SIM_LEN                      16          // value of SIM maximum length
#define DH_MAX_SIM_NUM                      10          // pick up SIM maximum number
#define DH_MAX_VERSION_LEN                  32          // version maximum length
#define DH_MAX_MDN_LEN                      36          // value of MDN maximum length
#define DH_MAX_NETINTERFACE_NUM             64          // support the card number
#define DH_EVENT_NAME_LEN                   128         // length of event name
#define DH_STORAGE_NAME_LEN                 128         // length of storage name
#define DH_MAX_CARPORTLIGHT_NUM             4           // carport light maximum number
#define DH_STATION_NAME_LEN                 32          // length of station name
#define PTZ_PRESET_NAME_LEN                 64          // Length of PTZ preset name
#define DH_MAX_GUARD_DETECT_ID_COUNT        64          // Max Station Detection ID Number
#define DH_MAX_VERSION_STR                  64          // Max length of version string
#define DH_MAX_AUDIO_MATRIX_OUTPUT          8           // The Maximum OutPut Channels Of Each Audio Matrix
#define DH_MAX_CHANNEL_COUNT                16          // Supports maximum number of channels
#define DH_COMMON_STRING_4                  4           // Common string length 4
#define DH_COMMON_STRING_8                  8           // Common string length 8
#define DH_COMMON_STRING_16                 16          // Common string length 16
#define DH_COMMON_STRING_32                 32          // Common string length 32
#define DH_COMMON_STRING_64                 64          // Common string length 64
#define DH_COMMON_STRING_128                128         // Common string length 128
#define DH_COMMON_STRING_256                256         // Common string length 256
#define DH_COMMON_STRING_512                512         // Common string length 512
#define DH_COMMON_STRING_1024               1024        // Common string length 1024
#define DH_COMMON_STRING_2048               2048        // Common string length 2048
#define DH_MAX_ACCESS_NAME_LEN              64          // Length of access name
#define DH_MAX_EXALARMCHANNEL_NAME_LEN      128         // Length of extension module alarm channel name
#define DH_MAX_ALARM_SUBSYSTEM_NUM          256         // Max Alarm Sub System Number
#define DH_MAX_BELL_NUM                     4           // Max Bell Number
#define DH_MAX_KEYBOARD_NUM                 256         // Max Keyboard Number
#define MAX_GOURP_NUM                       128         // Face library max number
#define DH_MAX_POS_EXCHANGE_INFO            64          // Max NET_POSEXCHANGE_INFO array Number
#define NET_INTERFACE_DEFAULT_TIMEOUT       3000        // Interface default timeout
#define DH_MAX_BUSCARD_NUM                  64          // The max of length of bus card number
#define DH_MAX_POS_MAC_NUM                  8           // The max of POS Mac number
#define DH_MAX_MARK_FILE_NAME_LEN           124         // the max of video control mark-file
#define NET_MAX_ATTACHMENT_NUM              8           // the max of car attachment number
#define NET_MAX_ANNUUALINSPECTION_NUM       8           // the max of car AnnualInspection number
#define NET_MAX_EVENT_PIC_NUM               6           // the max of traffic event picture number
#define NET_MAX_MONTH_NUM                   31          // the max of month number
#define NET_IPADDRSTR_LEN                   46          // the max length of ipaddr  ipv4-mapped-on-ipv6 
#define NET_MAX_AP_NUM                      3           // the max of access-point number
#define MAX_WEP_KEY_NUM                     4           // the max of WEP password number
#define MAX_CARNO_LEN                       20          // the max length of car no
#define MAX_COACHNO_LEN                     12          // the max length of coach no 
#define MAX_WORKPATTERN_NUM                 2           // the max of work pattern number
#define MAX_ANTIFLICKERMODE_COUNT           8           // The max number of anti-flicker mode
#define MAX_CATEGORY_LEN                    64          // Max category len

// Remote configuration structure corresponding constant 
#define DH_MAX_MAIL_ADDR_LEN                128           // Mail address max length
#define DH_MAX_MAIL_SUBJECT_LEN             64            // Mail subject max length 
#define DH_MAX_IPADDR_LEN                   16            // IP address string length 
#define DH_MAX_IPADDR_LEN_EX                40            // extension IP address support IPV6
#define DH_MACADDR_LEN                      40            // MACE address string length
#define DH_MAX_URL_LEN                      128           // URL string length 
#define DH_MAX_DEV_ID_LEN                   48            // Device serial number max length 
#define DH_MAX_HOST_NAMELEN                 64            // Host name length 
#define DH_MAX_HOST_PSWLEN                  32            // Password length 
#define DH_MAX_NAME_LEN                     16            // Universal name string length 
#define DH_MAX_ETHERNET_NUM                 2             // Ethernet max amount 
#define DH_MAX_ETHERNET_NUM_EX              10            // extended Ethernet max amount
#define DH_DEV_SERIALNO_LEN                 48            // Serial number string length 
#define DH_DEV_CLASS_LEN                    16            // Device type string (such as IPC) length 
#define DH_DEV_TYPE_LEN                     32            // Device type string length 
#define DH_N_WEEKS                          7             // The days in one week 
#define DH_N_TSECT                          6             // Time period amount 
#define DH_N_REC_TSECT                      6             // Record period amount 
#define DH_N_COL_TSECT                      2             // Color period amount 
#define DH_CHAN_NAME_LEN                    32            // Channel name length. DVR DSP capacity limit. Max 32 bytes.        
#define DH_N_ENCODE_AUX                     3             // Extra stream amount 
#define DH_N_TALK                           1             // Max audio talk channel amount 
#define DH_N_COVERS                         1             // Privacy mask zone amount 
#define DH_N_CHANNEL                        16            // Max channel amount 
#define DH_N_ALARM_TSECT                    2             // Alarm prompt period amount 
#define DH_MAX_ALARMOUT_NUM                 16            // Alarm output ports max amount 
#define DH_MAX_AUDIO_IN_NUM                 16            // Audio input ports max amount 
#define DH_MAX_VIDEO_IN_NUM                 16            // Video input ports max amount 
#define DH_MAX_ALARM_IN_NUM                 16            // Alarm input ports max amount 
#define DH_MAX_DISK_NUM                     16            // HDD max amount. Now the value is 16.
#define DH_MAX_DECODER_NUM                  16            // Decoder(485) max amount 
#define DH_MAX_232FUNCS                     10            // 232 COM function max amount 
#define DH_MAX_232_NUM                      2             // 232 COM port max amount 
#define DH_MAX_232_NUM_EX                   16            // extended 232 COM port max amount 
#define DH_MAX_DECPRO_LIST_SIZE             100           // Decoder protocol list max amount 
#define DH_FTP_MAXDIRLEN                    240           // FTP file folder max length 
#define DH_MATRIX_MAXOUT                    16            // Matrix output ports max amount
#define DH_TOUR_GROUP_NUM                   6             // Matrix output ports max amount 
#define DH_MAX_DDNS_NUM                     10            // DDNS max amount the device supported 
#define DH_MAX_SERVER_TYPE_LEN              32            // DDNS type and max string length 
#define DH_MAX_DOMAIN_NAME_LEN              256           // DDNS domain name and max string length 
#define DH_MAX_DDNS_ALIAS_LEN               32            // DDNS alias and max string length 
#define DH_MAX_DEFAULT_DOMAIN_LEN           60            // DDNS default domain, max sring length    
#define DH_MOTION_ROW                       32            // Motion detection zone row amount 
#define DH_MOTION_COL                       32            // Motion detection zone column amount 
#define DH_STATIC_ROW                       32            // Static detection zone row amount 
#define DH_STATIC_COL                       32            // Static detection zone column amount 
#define DH_FTP_USERNAME_LEN                 64            // FTP setup:user name max length 
#define DH_FTP_PASSWORD_LEN                 64            // FTP setup:password max length 
#define DH_TIME_SECTION                     2             // FTP setup:time periods in each day.
#define DH_FTP_MAX_PATH                     240           // FTP setup:file path max length 
#define DH_FTP_MAX_SUB_PATH                 128           // FTP setup:sub path max length
#define DH_INTERVIDEO_UCOM_CHANID           32            // Platform embedded setup:U China Net Communication (CNC)channel ID
#define DH_INTERVIDEO_UCOM_DEVID            32            // Platform embedded setup:UCNC device ID
#define DH_INTERVIDEO_UCOM_REGPSW           16            // Platform embedded setup:UCNC registration password 
#define DH_INTERVIDEO_UCOM_USERNAME         32            // Platform embedded setup:UCNC user name 
#define DH_INTERVIDEO_UCOM_USERPSW          32            // Platform embedded setup: UCNC password 
#define DH_INTERVIDEO_NSS_IP                32            // Platform embedded setup:ZTE Netview IP
#define DH_INTERVIDEO_NSS_SERIAL            32            // Serial Platform embedded setup:ZTE Netview  serial
#define DH_INTERVIDEO_NSS_USER              32            // User Platform embedded setup:ZTE Netview user
#define DH_INTERVIDEO_NSS_PWD               50            // Password Platform embedded setup:ZTE Netview password
#define DH_MAX_VIDEO_COVER_NUM              16            // Privacy mask zones max amount 
#define DH_MAX_WATERMAKE_DATA               4096          // Watermark data max length 
#define DH_MAX_WATERMAKE_LETTER             128           // Watermark text max length
#define DH_MAX_WLANDEVICE_NUM               10            // Max searched wireless device amount 
#define DH_MAX_WLANDEVICE_NUM_EX            32            // Max searched wireless device amount
#define DH_MAX_ALARM_NAME                   64            // Address length 
#define DH_MAX_REGISTER_SERVER_NUM          10            // Auto registration server amount 
#define DH_SNIFFER_FRAMEID_NUM              6             // 6 FRAME ID options
#define DH_SNIFFER_CONTENT_NUM              4             // 4 sniffer in each FRAME
#define DH_SNIFFER_CONTENT_NUM_EX           8             // 8 sniffer in each FRAME
#define DH_SNIFFER_PROTOCOL_SIZE            20            // protocol length
#define DH_MAX_PROTOCOL_NAME_LENGTH         20
#define DH_SNIFFER_GROUP_NUM                4             // 4 group sniffer setup 
#define MAX_PATH_STOR                       240           // Remote folder length 
#define DH_ALARM_OCCUR_TIME_LEN             40            // New alarm upload time length 
#define DH_VIDEO_OSD_NAME_NUM               64            // Overlay name length. Now it supports 32-digit English and 16-digit Chinese.
#define DH_VIDEO_CUSTOM_OSD_NUM             8             // The self-defined amount supported excluding time and channel.
#define DH_VIDEO_CUSTOM_OSD_NUM_EX          256           // The self-defined amount supported excluding time and channel.
#define DH_CONTROL_AUTO_REGISTER_NUM        100           // Targeted initiatives to support the number of registration servers
#define DH_MMS_RECEIVER_NUM                 100           // Support the number of messages the recipient
#define DH_MMS_SMSACTIVATION_NUM            100           // Support the number of SMS sender
#define DH_MMS_DIALINACTIVATION_NUM         100           // Support for dial-up number of the sender
#define DH_MAX_ALARMOUT_NUM_EX              32            // Alarm output amount max limit
#define DH_MAX_VIDEO_IN_NUM_EX              32            // Video input amount max limit
#define DH_MAX_ALARM_IN_NUM_EX              32            // Alarm input amount max limit
#define DH_MAX_IPADDR_OR_DOMAIN_LEN         64            // IP or domain name length
#define DH_MAX_CALLID                       32            // Call ID
#define DH_MAX_OBJECT_LIST                  16            // the upper limit number of object id that detected by device in intelligent analysis 
#define DH_MAX_RULE_LIST                    16            // the upper limit number of device rule in intelligent analysis 
#define DH_MAX_POLYGON_NUM                  16            // the max number of polygon's culmination
#define DH_MAX_DETECT_LINE_NUM              20            // rule detect line's max point number
#define DH_MAX_DETECT_REGION_NUM            20            // rule detect region's max point number
#define DH_MAX_TRACK_LINE_NUM               20            // object moving track's max point number
#define DH_MAX_CANDIDATE_NUM                50            // max candidate number
#define DH_MAX_PERSON_IMAGE_NUM             48            // max image of every person
#define DH_MAX_FENCE_LINE_NUM               2             // max fence line number
#define MAX_SMART_VALUE_NUM                 30            // the max number of smart disk information
#define DH_MACHINE_NAME_NUM                 64            // Device name length
#define DH_INTERVIDEO_AMP_DEVICESERIAL      48            // Platform embedded setup, AMP,Device serial num length
#define DH_INTERVIDEO_AMP_DEVICENAME        16            // Platform embedded setup, AMP,Device Name length
#define DH_INTERVIDEO_AMP_USER              32            // Platform embedded setup, AMP,User Name Length
#define DH_INTERVIDEO_AMP_PWD               32            // Platform embedded setup, AMP,password length
#define MAX_SUBMODULE_NUM                   32            // Max number of submodule information
#define DH_MAX_CARWAY_NUM                   8             // traffic snapshot,the max car way number
#define DH_MAX_SNAP_SIGNAL_NUM              3             // one car way max time for snapshot
#define DH_MAX_CARD_NUM                     128           // max number of card
#define DH_MAX_CARDINFO_LEN                 32            // max length of card information
#define DH_MAX_CONTROLER_NUM                64            // max supported controller number
#define DH_MAX_LIGHT_NUM                    32            // max control light group number
#define DH_MAX_SNMP_COMMON_LEN              64            // max Snmp read or write buffer
#define DH_MAX_DDNS_STATE_LEN               128           // max DDNS state buffer
#define DH_MAX_PHONE_NO_LEN                 16            // max phone len
#define DH_MAX_MSGTYPE_LEN                  32            // max type len
#define DH_MAX_MSG_LEN                      256           // max message len
#define DH_MAX_DRIVINGDIRECTION             256           // max drivingdirection len
#define DH_MAX_GRAB_INTERVAL_NUM            4             // max snapshot count
#define DH_MAX_FLASH_NUM                    5             // max flash count
#define DH_MAX_LANE_NUM                     8             // max number of road in every channel
#define DH_MAX_ISCSI_PATH_NUM               64            // max ISCSI remote path number
#define DH_MAX_WIRELESS_CHN_NUM             256           // max wireless channel number
#define DH_PROTOCOL3_BASE                   100           // protocol 3 base value
#define DH_PROTOCOL3_SUPPORT                11            // prococol 3 support
#define DH_MAX_CHANMASK                     64            // max channel mask
#define DH_MAX_STAFF_NUM                    20            // max of compressed video configuration staff number
#define DH_MAX_CALIBRATEBOX_NUM             10            // max of compressed video configuration calibrate region
#define DH_MAX_EXCLUDEREGION_NUM            10            // max of compressed video configuration exclude region
#define DH_MAX_POLYLINE_NUM                 20            // number of compressed video configuration poly line
#define DH_MAX_COLOR_NUM                    16            // color maximum number
#define MAX_OBJFILTER_NUM                   16            // max number of  filter type 
#define DH_MAX_SYNOPSIS_STATE_NAME          64            // string state length of compressed video 
#define DH_MAX_SYNOPSIS_QUERY_FILE_COUNT    10            // the file number limit
#define DH_MAX_SSID_LEN                     36            // length of SSID
#define DH_MAX_APPIN_LEN                    16            // length of PIN
#define DH_NETINTERFACE_NAME_LEN            260           // length of net port
#define DH_NETINTERFACE_TYPE_LEN            260           // length of net type
#define DH_MAX_CONNECT_STATUS_LEN           260           // length of connect status
#define DH_MAX_MODE_LEN                     64            // length of 3G mode support
#define DH_MAX_MODE_NUM                     64            // number of 3G mode support
#define DH_MAX_COMPRESSION_TYPES_NUM        16            // max number of video compression types
#define DH_MAX_CAPTURE_SIZE_NUM             64            // number of video resolution 
#define DH_NODE_NAME_LEN                    64            // length of node name
#define MAX_CALIBPOINTS_NUM                 256           // max number of point support
#define DH_MAX_ATTR_NUM                     32            // the maximum number display element attributes
#define DH_MAX_CLOUDCONNECT_STATE_LEN       128           // length of cloud connect state
#define DH_MAX_IPADDR_EX_LEN                128           // IP entended maximum length
#define DH_MAX_PLATE_NUMBER_LEN             32            // plate string length    
#define DH_MAX_AUTHORITY_LIST_NUM           16            // max in authority list   
#define DH_MAX_CITY_NAME_LEN                64            // max length of city name
#define DH_MAX_PROVINCE_NAME_LEN            64            // max length of province name
#define DH_MAX_PERSON_ID_LEN                32            // max length of person ID
#define MAX_FACE_AREA_NUM                   8             // max number if face area
#define MAX_FACE_DB_NUM                     8             // max number of face DB
#define MAX_EVENT_NAME                      128           // max event name
#define DH_MAX_ETH_NAME                     64            // max network card name
#define DH_MAX_PERSON_NAME_LEN              64            // max length of person name
#define DH_N_SCHEDULE_TSECT                 8             // element number of schedule    
#define DH_MAX_URL_NUM                      8             // max URL number 
#define DH_MAX_LOWER_MITRIX_NUM             16            // max lower mitrix number
#define DH_MAX_BURN_CHANNEL_NUM             32            // max burn channel number
#define DH_MAX_NET_STRORAGE_BLOCK_NUM       64            // max remote storage block number 
#define DH_MAX_CASE_PERSON_NUM              32            // max number of case person
#define DH_MAX_MULTIPLAYBACK_CHANNEL_NUM    64            // max channel number of multiplayback channel
#define DH_MAX_MULTIPLAYBACK_SPLIT_NUM      32            // max split number of multipalyback channel
#define DH_MAX_AUDIO_ENCODE_TYPE            64            // Max audio encoding type quantity
#define MAX_LOG_PATH_LEN                    260           // Max log path name length
#define MAX_CARD_RECORD_FIELD_NUM           16            // Card No. Max Domain Quantity
#define DH_BATTERY_NUM_MAX                  16            // max battery number    
#define DH_POWER_NUM_MAX                    16            // max power number    
#define DH_MAX_AUDIO_PATH                   260           // max path of audio file
#define DH_MAX_DOORNAME_LEN                 128           // max length of door access name
#define DH_MAX_CARDPWD_LEN                  64            // max length of door access name
#define NET_MAX_FISHEYE_MOUNTMODE_NUM       4             // Max fish eye installation mode number
#define NET_MAX_FISHEYE_CALIBRATEMODE_NUM   16            // Max fish eye correction mode number
#define NET_MAX_FISHEYE_EPTZCMD_NUM         64            // Max fish eye electronic PTZ operation number  
#define POINT_NUM_IN_PAIR                   2             // Point Number in Pair
#define MAX_POINT_PAIR_NUM                  128           // Point Max Quantity
#define CHANNEL_NUM_IN_POINT_GROUP          2             // Point Video Channel Number
#define MAX_POINT_GROUP_NUM                 32            // Point Group Max Quantity. Each two channel splicing requires one group of point. 
#define MAX_LANE_INFO_NUM                   32            // Max lane info number
#define MAX_LANE_DIRECTION_NUM              8             // Lane direction total
#define DH_MAX_MONITORWALL_NUM              32            // TV wall max quantity
#define DH_MAX_OPTIONAL_URL_NUM             8             // The max number of alternate URL
#define DH_MAX_CAMERA_CHANNEL_NUM           1024          // Max channel amount
#define MAX_FILE_SUMMARY_NUM                32            // Max file summary amount  
#define MAX_AUDIO_ENCODE_NUM                64            // Max audio encode amount
#define MAX_MONITORWALL_NAME_LEN            64            // Max length of TV Wall name
#define MAX_FLASH_LIGHT_NUM                 8             // Max flash light amount
#define MAX_STROBOSCOPIC_LIGHT_NUM          8             // Max stroboscopic light amount
#define MAX_MOSAIC_NUM                      8             // Max counts of mosaic 
#define MAX_MOSAIC_CHANNEL_NUM              256           // Max counts of channels which supported video mosaic
#define MAX_FIREWARNING_INFO_NUM            4             // Max Counts of fire-warning info
#define MAX_AXLE_NUM                        8             // Max counts of axle 
#define DH_MAX_BULLET_HOLES                 10            // MAX counts of bullet holes
#define MAX_PLATE_NUM                       64            // MAX Plate counts in picture
#define MAX_PREVIEW_CHANNEL_NUM             64            // MAX preview channel number
#define MAX_EVENT_RESTORE_UUID              36            // Event restore uuid array size
#define MAX_EVENT_RESTORE_CODE_NUM          8             // Max event restore code num
#define MAX_EVENT_RESOTER_CODE_TYPE         32            // Event restore code type array size
#define MAX_SNAP_TYPE                       3             // snap type number
#define MAX_MAINFORMAT_NUM                  4             // main format number
#define CUSTOM_TITLE_LEN                    1024          // custom title name len 
#define MAX_CUSTOM_TITLE_NUM                8             // max custom title num
#define FORMAT_TYPE_LEN                     16            // max length of  format type
#define MAX_CHANNEL_NAME_LEN                256           // max length of channel name
#define MAX_VIRTUALINFO_DOMAIN_LEN          64            // virtual information domain length
#define MAX_VIRTUALINFO_TITLE_LEN           64            // virtual information net title length
#define MAX_VIRTUALINFO_USERNAME_LEN        32            // virtual information username length
#define MAX_VIRTUALINFO_PASSWORD_LEN        32            // virtual information password length
#define MAX_VIRTUALINFO_PHONENUM_LEN        12            // virtual information mobile phone length
#define MAX_VIRTUALINFO_IMEI_LEN            16            // virtual information international mobile device indicator length
#define MAX_VIRTUALINFO_IMSI_LEN            16            // virtual information international mobile user indicator length
#define MAX_VIRTUALINFO_LATITUDE_LEN        16            // virtual information latitude length
#define MAX_VIRTUALINFO_LONGITUDE_LEN       16            // virtual information longitude length
#define MAX_VIRTUALINFO_NUM                 1024          // MAX virtual information number
#define MAX_CALL_ID_LEN                     64            // Call ID length
#define MAX_FACE_DATA_LEN                   2048          // Max face detection data len
#define MAX_FACE_DATA_NUM                   20            // Max face detection data number 
#define MAX_FINGERPRINT_NUM                 10            // Max fingerprint number
#define MAX_RINGFILE_NUM                    64            // max ring file num
#define MAX_VIDEOIN_CONFLICT_NUM            128           // max conflict of videoin number 
#define MAX_COURSE_LOGIC_CHANNEL            64            // max course-media logic channel number
#define MAX_COMMON_STRING_8                 8             // Universal String Length 8
#define MAX_COMMON_STRING_16                16            // Universal String Length 16
#define MAX_COMMON_STRING_32                32            // Universal String Length 32
#define MAX_COMMON_STRING_64                64            // Universal String Length 64
#define MAX_MAN_LIST_COUNT                  64            // Max count of man list
#define MAX_COMMON_STRING_128               128           // Universal String Length 128
#define MAX_STREAM_NUM                      4             // MAX Stream Number
#define MAX_CELL_PHONE_NUMBER_LEN           32            // max cell phone number
#define MAX_MAIL_LEN                        64            // max mail len
#define MAX_USER_NAME_LEN                   128           // max user name len
#define MAX_PWD_LEN                         128           // max password len
#define MAX_SECURITY_CODE_LEN               16            // max security code len
#define MAX_PWD_SPEC_CHARS_ARRAY_LEN        128           // max len of special chars list in passwd specification 
#define MAX_PWD_BASIC_CHARS_ARRAY_LEN       128           // max len of basic chars list supported in passwd specification        
#define MAX_COMMON_STRING_512               512           // Universal String Length 512
#define MAX_RFIDELETAG_CARDID_LEN           16            // max length of car id
#define MAX_RFIDELETAG_DATE_LEN             16            // max length of date
#define MAX_LINK_NAME_LEN                   16            // link name length
#define MAX_SERVER_ADDRESS_LEN              64            // server IP length
#define LINK_LAYER_VPN_NUM                  64            // link layer VPN config num
#define MAX_SERVER_IP_LEN                   32            // server IP length
#define MAX_SCENICSPOT_POINTS_NUM           256           // Total number of points
#define MAX_ACCESSSUBCONTROLLER_NUM         64            // the max count of access sub-controller
#define MAX_ACCESSDOOR_NUM                  128           // the max count of door number
#define MAX_ACCESS_READER_NUM               32            // the max count of reader
#define MAX_ACCESS_POINT_NUM                32            // the max count of Repeat route point
#define MAX_CONFIG_NAME_LEN                 128           // max length of configuration's name.
#define MAX_PLATE_NUMBER_LEN                64            // max plate number len
#define MAX_MASTER_OF_CAR_LEN               32            // max master of car len
#define MAX_USER_TYPE_LEN                   32            // max user type len
#define MAX_SUB_USER_TYPE_LEN				64			  // max sub user type len
#define MAX_REMARKS_LEN						64			  // max remarks info len
#define MAX_PARK_CHARGE_LEN                 32            // max park charge len
#define MAX_IN_TIME_LEN                     32            // max in time len
#define MAX_OUT_TIME_LEN                    32            // max out time len
#define MAX_CUSTOM_LEN                      128           // max custom len
#define MAX_DEAL_NUM_LEN                    32            // max deal num len
#define MAX_STORE_NO_LEN                    32            // max store name No len
#define MAX_STORE_NAME_LEN                  32            // max store name len
#define MAX_STORE_EMPLOYEE_ID_LEN           32            // max store employee ID len
#define MAX_PRODUCT_NO_LEN                  32            // max product name No len 
#define MAX_PRODUCT_NAME_LEN                32            // max product name len
#define MAX_PRODUCT_CATEGORY_LEN            32            // max product category len
#define DH_MAX_FINGER_PRINT                 10            // Max length of finger print array
#define MAX_SUBCHANNEL_NUM                  16            // max sensor
#define MAX_NAME_LENGTH                     32            // max name len
#define MAX_SNAP_URL_LEN                    128           // max snap URL len
#define MAX_CODE_LEN                        64            // max good code number len
#define MAX_PERSON_INFO_NUM                 4             // max person info num
#define MAX_GOOD_INFO_NUM                   128           // max good info num
#define MAX_SUB_TAG_NUM                     20            // max sub tag number
#define MAX_MANUFACTURER_LEN                32            // max manufacturer len 
#define MAX_MACHISTORY_SSID_LEN             24            // max mac history SSID len
#define MAX_MACHISTORY_SSID_NUM	            5             // max history SSID num
#define MAX_ROUTE_NUM                       16            // max Route number
#define MAX_MCU_NUM                         10            // max MCU number

// Search type, corresponding to CLIENT_QueryDevState
#define DH_DEVSTATE_COMM_ALARM            0x0001           // Search general alarm status(including external alarm,video loss, motion detection)
#define DH_DEVSTATE_SHELTER_ALARM         0x0002           // Search camera masking alarm status
#define DH_DEVSTATE_RECORDING             0x0003           // Search record status 
#define DH_DEVSTATE_DISK                  0x0004           // Search HDD information 
#define DH_DEVSTATE_RESOURCE              0x0005           // Search system resources status 
#define DH_DEVSTATE_BITRATE               0x0006           // Search channel bit stream 
#define DH_DEVSTATE_CONN                  0x0007           // Search device connection status 
#define DH_DEVSTATE_PROTOCAL_VER          0x0008           // Search network protocol version number , pBuf = int*
#define DH_DEVSTATE_TALK_ECTYPE           0x0009           // Search the audio talk format the device supported. Please refer to structure DHDEV_TALKFORMAT_LIST
#define DH_DEVSTATE_SD_CARD               0x000A           // Search SD card information(FOR IPC series)
#define DH_DEVSTATE_BURNING_DEV           0x000B           // Search burner information
#define DH_DEVSTATE_BURNING_PROGRESS      0x000C           // Search burning information
#define DH_DEVSTATE_PLATFORM              0x000D           // Search the embedded platform the device supported
#define DH_DEVSTATE_CAMERA                0x000E           // Search camera property information ( for IPC series),pBuf = DHDEV_CAMERA_INFO *,there can be several structure.
#define DH_DEVSTATE_SOFTWARE              0x000F           // Search device software version information
#define DH_DEVSTATE_LANGUAGE              0x0010           // Search the audio type the device supported 
#define DH_DEVSTATE_DSP                   0x0011           // Search DSP capacity description 
#define DH_DEVSTATE_OEM                   0x0012           // Search OEM information 
#define DH_DEVSTATE_NET                   0x0013           // Search network running status information 
#define DH_DEVSTATE_TYPE                  0x0014           // Search function attributes(for IPC serirs)
#define DH_DEVSTATE_SNAP                  0x0015           // Search snapshot function attribute(For IPC series)
#define DH_DEVSTATE_RECORD_TIME           0x0016           // Query the first time and the recent video time
#define DH_DEVSTATE_NET_RSSI              0x0017           // Query the wireless network signal strength,Please refer to structure DHDEV_WIRELESS_RSS_INFO
#define DH_DEVSTATE_BURNING_ATTACH        0x0018           // Burning options inquiry
#define DH_DEVSTATE_BACKUP_DEV            0x0019           // Query the list of backup device
#define DH_DEVSTATE_BACKUP_DEV_INFO       0x001a           // Query the backup device information
#define DH_DEVSTATE_BACKUP_FEEDBACK       0x001b           // backup rate of progress
#define DH_DEVSTATE_ATM_QUERY_TRADE       0x001c           // Query ATM trade type
#define DH_DEVSTATE_SIP                   0x001d           // Query sip state
#define DH_DEVSTATE_VICHILE_STATE         0x001e           // Query wifi state of mobile DVR
#define DH_DEVSTATE_TEST_EMAIL            0x001f           // Query Email Function
#define DH_DEVSTATE_SMART_HARD_DISK       0x0020           // Query Hard Disk Information
#define DH_DEVSTATE_TEST_SNAPPICTURE      0x0021           // Query Snap Picture Function
#define DH_DEVSTATE_STATIC_ALARM          0x0022           // Query static alarm state
#define DH_DEVSTATE_SUBMODULE_INFO        0x0023           // Query submodule information
#define DH_DEVSTATE_DISKDAMAGE            0x0024           // Query hard disk damage ability     
#define DH_DEVSTATE_IPC                   0x0025           // Query device supported IPC ability,see struct DH_DEV_IPC_INFO 
#define DH_DEVSTATE_ALARM_ARM_DISARM      0x0026           // Query alarm arm disarm state
#define DH_DEVSTATE_ACC_POWEROFF_ALARM    0x0027           // Query ACC power off state(return a DWORD type value, 1 means power off,0 means power on)
#define DH_DEVSTATE_TEST_FTP_SERVER       0x0028           // FTP server connect test
#define DH_DEVSTATE_3GFLOW_EXCEED         0x0029           // Query 3G Flow exceed state info(see struct DHDEV_3GFLOW_EXCEED_STATE_INFO)
#define DH_DEVSTATE_3GFLOW_INFO           0x002a           // Query 3G Flow info(see struct DH_DEV_3GFLOW_INFO)
#define DH_DEVSTATE_VIHICLE_INFO_UPLOAD   0x002b           // Vihicle information uploading
#define DH_DEVSTATE_SPEED_LIMIT           0x002c           // Speed limit alarm
#define DH_DEVSTATE_DSP_EX                0x002d           // Query DSP expended cap(struct DHDEV_DSP_ENCODECAP_EX)
#define DH_DEVSTATE_3GMODULE_INFO         0x002e           // Query 3G module info(struct DH_DEV_3GMODLE_INFO)
#define DH_DEVSTATE_MULTI_DDNS            0x002f           // Query multi DDNS status(struct DH_DEV_MULTI_DDNS_INFO)
#define DH_DEVSTATE_CONFIG_URL            0x0030           // Query Device URL info(struct DH_DEV_URL_INFO)
#define DH_DEVSTATE_HARDKEY               0x0031           // Query Hard key state(struct DHDEV_HARDKEY_STATE)
#define DH_DEVSTATE_ISCSI_PATH            0x0032           // Query ISCSI path(struct DHDEV_ISCSI_PATHLIST)
#define DH_DEVSTATE_DLPREVIEW_SLIPT_CAP   0x0033           // Query local preview split capability(struct DEVICE_LOCALPREVIEW_SLIPT_CAP)
#define DH_DEVSTATE_WIFI_ROUTE_CAP        0x0034           // Query wifi capablity(struct DHDEV_WIFI_ROUTE_CAP)
#define DH_DEVSTATE_ONLINE                0x0035           // Query device online state(return a DWORD value, 1-online, 0-offline)
#define DH_DEVSTATE_PTZ_LOCATION          0x0036           // Query ptz state(struct DH_PTZ_LOCATION_INFO)
#define DH_DEVSTATE_MONITOR_INFO          0x0037           // Query monitor state(state DHDEV_MONITOR_INFO)
#define DH_DEVSTATE_SUBDEVICE             0x0300           // Query subdevcie(fan,cpu...) state(struct CFG_DEVICESTATUS_INFO)
#define DH_DEVSTATE_RAID_INFO             0x0038           // Query RAID state(struct ALARM_RAID_INFO)  
#define DH_DEVSTATE_TEST_DDNSDOMAIN       0x0039           // test DDNS domain enable
#define DH_DEVSTATE_VIRTUALCAMERA         0x003a           // query virtual camera state(struct DHDEV_VIRTUALCAMERA_STATE_INFO)
#define DH_DEVSTATE_TRAFFICWORKSTATE      0x003b           // get device's state of video/coil module(struct DHDEV_TRAFFICWORKSTATE_INFO)
#define DH_DEVSTATE_ALARM_CAMERA_MOVE     0x003c           // get camera move alarm state(struct ALARM_CAMERA_MOVE_INFO)
#define DH_DEVSTATE_ALARM                 0x003e           // get external alarm(struct NET_CLIENT_ALARM_STATE) 
#define DH_DEVSTATE_VIDEOLOST             0x003f           // get video loss alarm(struct NET_CLIENT_VIDEOLOST_STATE) 
#define DH_DEVSTATE_MOTIONDETECT          0x0040           // get motion alarm(struct NET_CLIENT_MOTIONDETECT_STATE)
#define DH_DEVSTATE_DETAILEDMOTION        0x0041           // get detailed motion alarm(struct NET_CLIENT_DETAILEDMOTION_STATE)
#define DH_DEVSTATE_VEHICLE_INFO          0x0042           // get vehicle device state(struct DHDEV_VEHICLE_INFO)
#define DH_DEVSTATE_VIDEOBLIND            0x0043           // get blind alarm(struct NET_CLIENT_VIDEOBLIND_STATE)
#define DH_DEVSTATE_3GSTATE_INFO          0x0044           // Query 3G state(struct DHDEV_VEHICLE_3GMODULE)
#define DH_DEVSTATE_NETINTERFACE          0x0045           // Query net interface(struct DHDEV_NETINTERFACE_INFO)
#define DH_DEVSTATE_PICINPIC_CHN          0x0046           // Query picinpic channel(struct DWORD)
#define DH_DEVSTATE_COMPOSITE_CHN         0x0047           // Query splice screen(struct DH_COMPOSITE_CHANNEL)
#define DH_DEVSTATE_WHOLE_RECORDING       0x0048           // Query whole recording status(struct BOOL),as long as ther is a channel running,it indicates that the overall state
#define DH_DEVSTATE_WHOLE_ENCODING        0x0049           // Query whole encoding(struct BOOL),as long as ther is a channel running,it indicates that the overall state
#define DH_DEVSTATE_DISK_RECORDE_TIME     0x004a           // Query disk record time(pBuf = DEV_DISK_RECORD_TIME*)
#define DH_DEVSTATE_BURNER_DOOR           0x004b           // whether have pop-up optical dirve(struct NET_DEVSTATE_BURNERDOOR)
#define DH_DEVSTATE_GET_DATA_CHECK        0x004c           // get data validation process(struct NET_DEVSTATE_DATA_CHECK)
#define DH_DEVSTATE_ALARM_IN_CHANNEL      0x004f           // Query alarm input channel information(struct NET_ALARM_IN_CHANNEL)
#define DH_DEVSTATE_ALARM_CHN_COUNT       0x0050           // Query alarm channel number(struct NET_ALARM_CHANNEL_COUNT)
#define DH_DEVSTATE_PTZ_VIEW_RANGE        0x0051           // Query PTZ view range status(struct DH_OUT_PTZ_VIEW_RANGE_STATUS)
#define DH_DEVSTATE_DEV_CHN_COUNT         0x0052           // Query device channel information(struct NET_DEV_CHN_COUNT_INFO)
#define DH_DEVSTATE_RTSP_URL              0x0053           // Query RTSP URL list supported by device, struct DEV_RTSPURL_LIST
#define DH_DEVSTATE_LIMIT_LOGIN_TIME      0x0054           // Query online overtime of device logging in and return a BYTE, UNIT:MIN ,0 means no limite and The non-zero positive integer means restrictions on the number of minutes
#define DH_DEVSTATE_GET_COMM_COUNT        0x0055           // get com count (struct NET_GET_COMM_COUNT)
#define DH_DEVSTATE_RECORDING_DETAIL      0x0056           // Query recording status detail information(pBuf = NET_RECORD_STATE_DETAIL*)
#define DH_DEVSTATE_PTZ_PRESET_LIST       0x0057           // get state PTZ preset list (struct NET_PTZ_PRESET_LIST)
#define DH_DEVSTATE_EXTERNAL_DEVICE       0x0058           // exteral device information (pBuf = NET_EXTERNAL_DEVICE*)
#define DH_DEVSTATE_GET_UPGRADE_STATE     0x0059           // get device upgrade state(struct DHDEV_UPGRADE_STATE_INFO)
#define DH_DEVSTATE_MULTIPLAYBACK_SPLIT_CAP  0x005a        // get mulipalyback split (struct NET_MULTIPLAYBACK_SPLIT_CAP )
#define DH_DEVSTATE_BURN_SESSION_NUM      0x005b           // get burn session number(pBuf = int*)
#define DH_DEVSTATE_PROTECTIVE_CAPSULE    0x005c           // Search protective capsule status(corresponding structure ALARM_PROTECTIVE_CAPSULE_INFO)
#define DH_DEVSTATE_GET_DOORWORK_MODE     0x005d           // get access controlmode( corresponding NET_GET_DOORWORK_MODE)
#define DH_DEVSTATE_PTZ_ZOOM_INFO         0x005e           // Query PTZ optical zoom value( corresponding to DH_OUT_PTZ_ZOOM_INFO )
#define DH_DEVSTATE_POWER_STATE           0x0152           // Query power state(struct DH_POWER_STATUS)
#define DH_DEVSTATE_ALL_ALARM_CHANNELS_STATE    0x153      // Query alarm channel state(struct NET_CLIENT_ALARM_CHANNELS_STATE)
#define DH_DEVSTATE_ALARMKEYBOARD_COUNT   0x0154           // Query alarm keyboard count connected on com(struct NET_ALARMKEYBOARD_COUNT)
#define DH_DEVSTATE_EXALARMCHANNELS       0x0155           // Query mapping relationship of extension alarm module channel (struct NET_EXALARMCHANNELS)
#define DH_DEVSTATE_GET_BYPASS            0x0156           // Query channel bypass state(struct NET_DEVSTATE_GET_BYPASS)
#define DH_DEVSTATE_ACTIVATEDDEFENCEAREA  0x0157           // get active sector information(struct NET_ACTIVATEDDEFENCEAREA)
#define DH_DEVSTATE_DEV_RECORDSET         0x0158           // Query device recording information(struct NET_CTRL_RECORDSET_PARAM)
#define DH_DEVSTATE_DOOR_STATE            0x0159           // Query door access state(struct NET_DOOR_STATUS_INFO)
#define DH_DEVSTATE_ANALOGALARM_CHANNELS  0x1560           // analog alarm input channel mapping (struct NET_ANALOGALARM_CHANNELS)
#define DH_DEVSTATE_GET_SENSORLIST        0x1561           // Get device supported sensor list(corresponding NET_SENSOR_LIST)
#define DH_DEVSTATE_ALARM_CHANNELS        0x1562           // Search switch alarm template channel mapping relation(corresponding structure  NET_ALARM_CHANNELS)
                                                           // If device su not support search extension alarm module channel, may use this function to search extension channel logic channel no. and use as local alarm channel use
#define DH_DEVSTATE_GET_ALARM_SUBSYSTEM_ACTIVATESTATUS    0x1563     // Get current system enabling status( corresponding NET_GET_ALARM_SUBSYSTEM_ACTIVATE_STATUES)
#define DH_DEVSTATE_AIRCONDITION_STATE    0x1564           // Get air conditioning status(corresponding to NET_GET_AIRCONDITION_STATE)
#define DH_DEVSTATE_ALARMSUBSYSTEM_STATE  0x1565           // Get sub system status(corresponding to NET_ALARM_SUBSYSTEM_STATE)
#define DH_DEVSTATE_ALARM_FAULT_STATE     0x1566           // Get failure status(corresponding to NET_ALARM_FAULT_STATE_INFO)
#define DH_DEVSTATE_DEFENCE_STATE         0x1567           // Get zone status(corresponding to NET_DEFENCE_STATE_INFO, and bypass status change event, local alarm event, alarm signal event status description is different, cannot mix, for specific device use only)
#define DH_DEVSTATE_CLUSTER_STATE         0x1568           // Get collection status(corresponding to NET_CLUSTER_STATE_INFO)
#define DH_DEVSTATE_SCADA_POINT_LIST      0x1569           // Get spot chart path info(corresponding to NET_SCADA_POINT_LIST_INFO)
#define DH_DEVSTATE_SCADA_INFO            0x156a           // Get monitor spot info(corresponding to NET_SCADA_INFO)
#define DH_DEVSTATE_SCADA_CAPS            0x156b           // Get SCADA capacityset(corresponding to NET_SCADA_CAPS)
#define DH_DEVSTATE_GET_CODEID_COUNT      0x156c           // Get successful code item number(corresponding  NET_GET_CODEID_COUNT)
#define DH_DEVSTATE_GET_CODEID_LIST       0x156d           // Search code info(corresponding  NET_GET_CODEID_LIST)
#define DH_DEVSTATE_ANALOGALARM_DATA      0x156e           // Search analog channel data(corresponding  NET_GET_ANALOGALARM_DATA)
#define DH_DEVSTATE_VTP_CALLSTATE         0x156f           // Access the call state of the video phone (Corresponding to NET_GET_VTP_CALLSTATE)
#define DH_DEVSTATE_SCADA_INFO_BY_ID      0x1570           // query point info by device id(corresponding to NET_SCADA_INFO_BY_ID)
#define DH_DEVSTATE_SCADA_DEVICE_LIST     0x1571           // query scada device id(corresponding to NET_SCADA_DEVICE_LIST)
#define DH_DEVSTATE_DEV_RECORDSET_EX      0x1572           // Search device record set info (with binary data) (Corresponding to NET_CTRL_RECORDSET_PARAM)
#define DH_DEVSTATE_ACCESS_LOCK_VER       0x1573           // Get door locker software version (Corresponding to NET_ACCESS_LOCK_VER)
#define DH_DEVSTATE_MONITORWALL_TVINFO    0x1574           // get monitorwall TV info(Corresponding to NET_CTRL_MONITORWALL_TVINFO)
#define DH_DEVSTATE_GET_ALL_POS           0x1575           // get all configuration of users's Pos devices (Corresponding to NET_POS_ALL_INFO)
#define DH_DEVSTATE_GET_ROAD_LIST         0x1576           // get city and road code info(Corresponding to NET_ROAD_LIST_INFO)
#define DH_DEVSTATE_GET_HEAT_MAP          0x1577           // get heatmap information(Corresponding to NET_QUERY_HEAT_MAP)
#define DH_DEVSTATE_GET_WORK_STATE        0x1578           // get device work state (Corresponding to NET_QUERY_WORK_STATE)
#define DH_DEVSTATE_GET_WIRESSLESS_STATE  0x1579           // get wireless device work state(Corresponding to NET_GET_WIRELESS_DEVICE_STATE)
#define DH_DEVSTATE_GET_REDUNDANCE_POWER_INFO  0x157a      // get redundance power info(Corresponding to NET_GET_REDUNDANCE_POWER_INFO)

//////////////////////////////////////////////////////////////////////////
// To set structure. The macro of dwSize
#define _setdwSize(stu) ((stu).dwSize = sizeof(stu))

//////////////////////////////////////////////////////////////////////////

#define CFG_MAX_ACCESSCONTROL_NUM				8							// The Max Number of AccessControl
#define CFG_MAX_SUMMARY_LEN						1024						// The Max Length of the summary information superimposed on the JPEG picture
#define CFG_MAX_DBKEY_NUM						64							// The Max Number of DBKey
#define CFG_MAX_CHANNELNAME_LEN					64							// The Max Length of ChannelName
#define CFG_MAX_VIDEO_CHANNEL_NUM				256							// The Max Number of Video Channel
#define CFG_MAX_CHANNEL_COUNT					16                          // The Max Number of Channel
#define CFG_MAX_PSTN_SERVER_NUM                 8							// The Max Number of PSTN Server

//Three Status BOOL
typedef enum tagNET_THREE_STATUS_BOOL
{
    BOOL_STATUS_FALSE  = 0 , 
    BOOL_STATUS_TRUE       ,
    BOOL_STATUS_UNKNOWN    ,  //unknown
}NET_THREE_STATUS_BOOL;

//////////////////////////////////////////////////////////////////////////

// Air Conditioning Work Mode
typedef enum __EM_AIRCONDITION_MODE
{
    AIRCONDITION_MODE_UNKNOWN = 0,
    AIRCONDITION_MODE_AUTO,             // Auto
    AIRCONDITION_MODE_HOT,              // Cool
    AIRCONDITION_MODE_COLD,             // Heat
    AIRCONDITION_MODE_WET,              // Arefaction
    AIRCONDITION_MODE_WIND,             // Vent
} EM_AIRCONDITION_MODE;

// Air Conditioning Fan Mode
typedef enum __EM_AIRCONDITION_WINDMODE
{
    AIRCONDITION_WINDMODE_UNKNOWN = 0,
    AIRCONDITION_WINDMODE_STOP,          // Stop
    AIRCONDITION_WINDMODE_AUTO,          // Auto
    AIRCONDITION_WINDMODE_HIGH,          // High Speed
    AIRCONDITION_WINDMODE_MIDDLE,        // Medium Speed
    AIRCONDITION_WINDMODE_LOW,           // Low Speed
} EM_AIRCONDITION_WINDMODE;

// Air Conditioning Working Status Info
typedef struct tagNET_AIRCONDITION_STATE_INFO
{
    DWORD				        dwSize;
    BOOL                        bIsOnline;                      // Online
    BOOL                        bIsON;                          // Enable
    int                         nTemperature;                   // Temperature
    EM_AIRCONDITION_MODE        emAirconditionMode;             // Work Mode
    EM_AIRCONDITION_WINDMODE    emAirconditionWindMode;         // Air Fan Mode
    float                       fActualTemperature;             // Environment Temperature
}NET_AIRCONDITION_STATE_INFO;

// Get Air Working Status(corresponding to DH_DEVSTATE_AIRCONDITION_STATE command)
typedef struct tagNET_GET_AIRCONDITION_STATE
{
    DWORD				dwSize;
    char				szDeviceID[DH_MAX_DEV_ID_LEN];			// Device SN
    NET_AIRCONDITION_STATE_INFO stuAirConditionState;           // Air conditioning working status info
}NET_GET_AIRCONDITION_STATE;

// Switch alarm channel info
typedef struct tagNET_ALARM_CHANNEL_INFO 
{
	DWORD			dwSize;
	int				nSlot;					        // Root address, 0 means local channel, 1 means connected to 1st serial extension channel, 2, 3... and so on
	int				nChannel;				        // Non local alarm channel on extension module channel no., from 0 
	char			szName[DH_COMMON_STRING_128];	// Channel name
}NET_ALARM_CHANNEL_INFO;

// Local(far or close)switch alarm channel and logic channel no. mapping relation
typedef struct tagNET_ALARM_CHANNELS 
{
	DWORD			dwSize;
	int				nMaxAlarmChannels;		// Max channel
	int				nRetAlarmChannels;		// Return channel no.
	NET_ALARM_CHANNEL_INFO*	pstuChannelInfo;// Channel info, user allocate memory
}NET_ALARM_CHANNELS;

// Alarm Sub System Status Type
typedef enum __EM_ALARM_SUBSYSTEM_STATE_TYPE
{
    ALARM_SUBSYSTEM_STATE_UNKNOWN = 0,
	ALARM_SUBSYSTEM_STATE_ACTIVE,           // Activated
	ALARM_SUBSYSTEM_STATE_INACTIVE,         // Not activated
	ALARM_SUBSYSTEM_STATE_UNDISTRIBUTED,    // Not distributed
}EM_ALARM_SUBSYSTEM_STATE_TYPE;

// Alarm Sub System Status, CLIENT_QueryDevStatecorresponding to command DH_DEVSTATE_ALARMSUBSYSTEM_STATE
typedef struct tagNET_ALARM_SUBSYSTEM_STATE
{
    DWORD          dwSize;
	int            nMaxAlarmSubSystem;                                  // Valid Sub System Number
	EM_ALARM_SUBSYSTEM_STATE_TYPE  emState[DH_MAX_ALARM_SUBSYSTEM_NUM]; // Sub System Status Info
}NET_ALARM_SUBSYSTEM_STATE;


// Failure Type
typedef enum __EM_ALARM_FAULT_STATE_TYPE
{
    EM_ALARM_FAULT_STATE_ALL = 0,                       // All Types
    EM_ALARM_FAULT_STATE_CONNECTFAULT,                  // Connection Failure
    EM_ALARM_FAULT_STATE_BELLFAULT,                     // Bell Failure
    EM_ALARM_FAULT_STATE_KBFAULT,                       // Keyboard Failure
    EM_ALARM_FAULT_STATE_KBTAMPER,                      // Keyboard Vandal-proof
    EM_ALARM_FAULT_STATE_SENSORSHORTOUT,                // Sensor Short circuit
    EM_ALARM_FAULT_STATE_SENSORTAMPER,                  // Sensor Vandal-proof
}EM_ALARM_FAULT_STATE_TYPE;

// Alarm Failure Status, Input Parameter
typedef struct tagNET_IN_ALARM_FAULT_STATE
{
    DWORD           dwSize;
    EM_ALARM_FAULT_STATE_TYPE   emType;                 // Type to search
}NET_IN_ALARM_FAULT_STATE;

// Alarm Failure Status, Output Parameter
typedef struct tagNET_OUT_ALARM_FAULT_STATE
{
    DWORD           dwSize;
    int             nConnectFaultState;                 // 0: Unknown, 1: Normal, 2:Failure
    int             nBell;                              // Bell valid number 
    int             anBellState[DH_MAX_BELL_NUM];       // Bell failure status info 0: Unknown, 1: Normal, 2:Failure
    int             nFaultKeyboard;                     // Keyboard failure valid number
    int             anFaultKeyboard[DH_MAX_KEYBOARD_NUM];  // Keyboard failure info 0: Unknown, 1: Normal, 2:Failure
    int             nTamperKeyboard;                       // Keyboard valid vandal-proof number 
    int             anTamperKeyboard[DH_MAX_KEYBOARD_NUM]; // Keyboard vandal-proof info 0: Unknown, 1: Normal, 2:Failure
    int             nShortOutSensor;                    // Sensor valid short circuit number
    int             anShortOutSensor[DH_MAX_ALARMIN];   // Sensor short circuit info 0: Unknown, 1: Normal, 2:Failure
    int             nTamperSensor;                      // Sensor valid vandal-proof number
    int             anTamperSensor[DH_MAX_ALARMIN];     // Sensor vandal-proof info 0: Unknown, 1: Normal, 2:Failure
}NET_OUT_ALARM_FAULT_STATE;

// Alarm failure status, CLIENT_QueryDevStatecorresponding to command DH_DEVSTATE_ALARM_FAULT_STATE
typedef struct tagNET_ALARM_FAULT_STATE_INFO
{
    DWORD           dwSize;
    NET_IN_ALARM_FAULT_STATE    stuIn;                  // Input parameter
    NET_OUT_ALARM_FAULT_STATE   stuOut;                 // Output parameter
}NET_ALARM_FAULT_STATE_INFO;

// Zone Status Type
typedef enum __EM_DEFENCE_STATE_TYPE
{
    EM_DEFENCE_STATE_UNKNOWN = 0,
    EM_DEFENCE_STATE_BYPASS,                            // Bypass
    EM_DEFENCE_STATE_OPEN,                              // Open, for alarm under disarming
    EM_DEFENCE_STATE_CLOSE,                             // Close
    EM_DEFENCE_STATE_ALARMING,                          // Alarm, for alarm under arming
}EM_DEFENCE_STATE_TYPE;

// Zone Status Info Input Parameter
typedef struct tagNET_IN_DEFENCE_STATE_INFO 
{
    DWORD           dwSize;
    int             nDefenceNum;                        // Zone valid number, -1means search all zones
    int             anDefence[DH_MAX_ALARMIN];          // Zone number to search
}NET_IN_DEFENCE_STATE_INFO;

// Zone Status Info Output Parameter
typedef struct tagNET_OUT_DEFENCE_STATE_INFO
{
    DWORD           dwSize;
    int             nDefenceNum;                            // Zone valid number    
	EM_DEFENCE_STATE_TYPE  anDefenceState[DH_MAX_ALARMIN];  // Zone status info
}NET_OUT_DEFENCE_STATE_INFO;

// Zpne status info, CLIENT_QueryDevStatecorresponding to command DH_DEVSTATE_DEFENCE_STATE
typedef struct tagNET_DEFENCE_STATE_INFO 
{
    DWORD           dwSize;
    NET_IN_DEFENCE_STATE_INFO   stuIn;         // Input parameter
    NET_OUT_DEFENCE_STATE_INFO  stuOut;        // Output parameter
}NET_DEFENCE_STATE_INFO;

// Type of video phone call
typedef enum tagEM_NET_VTP_CALL_TYPE
{
    EM_NET_VTP_CALL_TYPE_SIP = 0,              // SIP calling
    EM_NET_VTP_CALL_TYPE_GSM,                  // GSM calling
    EM_NET_VTP_CALL_TYPE_PRIVATE,              // private protocol calling
}EM_NET_VTP_CALL_TYPE;

// Type of video phone call
typedef enum tagEM_NET_VTP_CALL_STATE_TYPE
{
    EM_NET_VTP_CALL_STATE_UNKNOWN = 0,
    EM_NET_VTP_CALL_STATE_IDLE,                // Initial State
    EM_NET_VTP_CALL_STATE_CALLING,             // State of calling(the opposite side not response)
    EM_NET_VTP_CALL_STATE_CALLED,              // State of called
    EM_NET_VTP_CALL_STATE_RINGING,             // State of calling back (the opposite side responsed but no answer)
    EM_NET_VTP_CALL_STATE_CONNECTING,          // State of answering 
    EM_NET_VTP_CALL_STATE_CONNECTING2,         // 2 states in the call (call state after change of media)
    EM_NET_VTP_CALL_STATE_DISCONNECT,          // State of call off
}EM_NET_VTP_CALL_STATE_TYPE;

// Obtain the calling state of video phone (Corresponding to DH_DEVSTATE_VTP_CALLSTATE)
typedef struct tagNET_GET_VTP_CALLSTATE 
{
    DWORD                       dwSize;
    EM_NET_VTP_CALL_TYPE        emCallType;                     // the call type to be queried, input parameter
    char                        szCallID[DH_COMMON_STRING_128]; // the unique identification to be queried, input parameter
    const char*                 szTargetID;                     // the device to be transmit,  ID= NULL equals to no transmitting,  input parameter
    EM_NET_VTP_CALL_STATE_TYPE  emCallState;                    // State of calling
}NET_GET_VTP_CALLSTATE;

// Configuration type,corresponding to CLIENT_QueryRemotDevState
#define DH_DEVSTATE_ALARM_REMOTE          0x1000        // get the remote device external alarm(struct ALARM_REMOTE_ALARM_INFO)
#define DH_DEVSTATE_ALARM_FRONTDISCONNECT 0x1001 // get the front IPC disconnect alarm (struct ALARM_FRONTDISCONNET_INFO)

// Configuration type,corresponding to CLIENT_GetDevConfig and CLIENT_SetDevConfig
#define DH_DEV_DEVICECFG                  0x0001        // Device property setup 
#define DH_DEV_NETCFG                     0x0002        // Network setup 
#define DH_DEV_CHANNELCFG                 0x0003        // Video channel setup
#define DH_DEV_PREVIEWCFG                 0x0004        // Preview parameter setup
#define DH_DEV_RECORDCFG                  0x0005        // Record setup
#define DH_DEV_COMMCFG                    0x0006        // COM property setup 
#define DH_DEV_ALARMCFG                   0x0007        // Alarm property setup
#define DH_DEV_TIMECFG                    0x0008        // DVR time setup 
#define DH_DEV_TALKCFG                    0x0009        // Audio talk parameter setup 
#define DH_DEV_AUTOMTCFG                  0x000A        // Auto matrix setup
#define DH_DEV_VEDIO_MARTIX               0x000B        // Local matrix control strategy setup
#define DH_DEV_MULTI_DDNS                 0x000C        //  Multiple DDNS setup 
#define DH_DEV_SNAP_CFG                   0x000D        // Snapshot corresponding setup 
#define DH_DEV_WEB_URL_CFG                0x000E        // HTTP path setup 
#define DH_DEV_FTP_PROTO_CFG              0x000F        // FTP upload setup 
#define DH_DEV_INTERVIDEO_CFG             0x0010        // Plaform embedded setup. Now the channel parameter represents the platform type. 
// channel=4:Bell alcatel;channel=10:ZTE Netview;channel=11:U CNC  channel = 51 AMP
#define DH_DEV_VIDEO_COVER                0x0011        // Privacy mask setup
#define DH_DEV_TRANS_STRATEGY             0x0012        // Transmission strategy. Video quality\Fluency
#define DH_DEV_DOWNLOAD_STRATEGY          0x0013        //  Record download strategy setup:high-speed\general download
#define DH_DEV_WATERMAKE_CFG              0x0014        // Video watermark setup
#define DH_DEV_WLAN_CFG                   0x0015        // Wireless network setup 
#define DH_DEV_WLAN_DEVICE_CFG            0x0016        // Search wireless device setup 
#define DH_DEV_REGISTER_CFG               0x0017        // Auto register parameter setup 
#define DH_DEV_CAMERA_CFG                 0x0018        // Camera property setup 
#define DH_DEV_INFRARED_CFG               0x0019        // IR alarm setup 
#define DH_DEV_SNIFFER_CFG                0x001A        // Sniffer setup 
#define DH_DEV_MAIL_CFG                   0x001B        // Mail setup 
#define DH_DEV_DNS_CFG                    0x001C        // DNS setup 
#define DH_DEV_NTP_CFG                    0x001D        // NTP setup
#define DH_DEV_AUDIO_DETECT_CFG           0x001E        // Audio detection setup 
#define DH_DEV_STORAGE_STATION_CFG        0x001F        // Storage position setup 
#define DH_DEV_PTZ_OPT_CFG                0x0020        // PTZ operation property(It is invalid now. Please use CLIENT_GetPtzOptAttr to get PTZ operation property.)
#define DH_DEV_DST_CFG                    0x0021        // Daylight Saving Time (DST)setup
#define DH_DEV_ALARM_CENTER_CFG           0x0022        // Alarm center setup
#define DH_DEV_VIDEO_OSD_CFG              0x0023        // VIdeo OSD setup
#define DH_DEV_CDMAGPRS_CFG               0x0024        // CDMA\GPRS configuration
#define DH_DEV_IPFILTER_CFG               0x0025        // IP Filter configuration
#define DH_DEV_TALK_ENCODE_CFG            0x0026        // Talk encode configuration
#define DH_DEV_RECORD_PACKET_CFG          0x0027        // The length of the video package configuration
#define DH_DEV_MMS_CFG                    0x0028        // SMS MMS configuration 
#define DH_DEV_SMSACTIVATION_CFG          0x0029        // SMS to activate the wireless connection configuration
#define DH_DEV_DIALINACTIVATION_CFG       0x002A        // Dial-up activation of a wireless connection configuration
#define DH_DEV_SNIFFER_CFG_EX             0x0030        // Capture network configuration
#define DH_DEV_DOWNLOAD_RATE_CFG          0x0031        // Download speed limit
#define DH_DEV_PANORAMA_SWITCH_CFG        0x0032        // Panorama switch alarm configuration
#define DH_DEV_LOST_FOCUS_CFG             0x0033        // Lose focus alarm configuration
#define DH_DEV_ALARM_DECODE_CFG           0x0034        // Alarm decoder configuration
#define DH_DEV_VIDEOOUT_CFG               0x0035        // Video output configuration
#define DH_DEV_POINT_CFG                  0x0036        // Preset enable configuration
#define DH_DEV_IP_COLLISION_CFG           0x0037        // IP conflication configurationIp
#define DH_DEV_OSD_ENABLE_CFG             0x0038        // OSD overlay enable configuration
#define DH_DEV_LOCALALARM_CFG             0x0039        // Local alarm configuration(Structure DH_ALARMIN_CFG_EX)
#define DH_DEV_NETALARM_CFG               0x003A        // Network alarm configuration(Structure DH_ALARMIN_CFG_EX)
#define DH_DEV_MOTIONALARM_CFG            0x003B        // Motion detection configuration(Structure DH_MOTION_DETECT_CFG_EX)
#define DH_DEV_VIDEOLOSTALARM_CFG         0x003C        // Video loss configuration(Structure DH_VIDEO_LOST_CFG_EX)
#define DH_DEV_BLINDALARM_CFG             0x003D        // Camera masking configuration(Structure DH_BLIND_CFG_EX)
#define DH_DEV_DISKALARM_CFG              0x003E        // HDD alarm configuration(Structure DH_DISK_ALARM_CFG_EX)
#define DH_DEV_NETBROKENALARM_CFG         0x003F        // Network disconnection alarm configuration(Structure DH_NETBROKEN_ALARM_CFG_EX)
#define DH_DEV_ENCODER_CFG                0x0040        // Digital channel info of front encoders(Hybrid DVR use,Structure DEV_ENCODER_CFG)
#define DH_DEV_TV_ADJUST_CFG              0x0041        // TV adjust configuration(Now the channel parameter represents the TV numble(from 0), Structure DHDEV_TVADJUST_CFG)
#define DH_DEV_ABOUT_VEHICLE_CFG          0x0042        // about vehicle configuration
#define DH_DEV_ATM_OVERLAY_ABILITY        0x0043        // ATM ability information
#define DH_DEV_ATM_OVERLAY_CFG            0x0044        // ATM overlay configuration
#define DH_DEV_DECODER_TOUR_CFG           0x0045        // Decoder tour configuration
#define DH_DEV_SIP_CFG                    0x0046        // SIP configuration
#define DH_DEV_VICHILE_WIFI_AP_CFG        0x0047        // wifi ap configuration
#define DH_DEV_STATICALARM_CFG            0x0048        // Static 
#define DH_DEV_DECODE_POLICY_CFG          0x0049        // decode policy configuration(Structure DHDEV_DECODEPOLICY_CFG,channel start with 0) 
#define DH_DEV_MACHINE_CFG                0x004A        // Device relative config (Structure DHDEV_MACHINE_CFG)
#define DH_DEV_MAC_COLLISION_CFG          0x004B        // MACconflication configuration(Structure ALARM_MAC_COLLISION_CFG)
#define DH_DEV_RTSP_CFG                   0x004C        // RTSP configuration(structure DHDEV_RTSP_CFG)
#define DH_DEV_232_COM_CARD_CFG           0x004E        // 232 com card signal event configuration(structure COM_CARD_SIGNAL_LINK_CFG)
#define DH_DEV_485_COM_CARD_CFG           0x004F        // 485 com card signal event configuration(structure COM_CARD_SIGNAL_LINK_CFG)
#define DH_DEV_FTP_PROTO_CFG_EX           0x0050        // extend FTP upload setup(Structure DHDEV_FTP_PROTO_CFG_EX)
#define DH_DEV_SYSLOG_REMOTE_SERVER       0x0051        // SYSLOG remote server config (Structure DHDEV_SYSLOG_REMOTE_SERVER)
#define DH_DEV_COMMCFG_EX                 0x0052        // Extended com configuration(structure DHDEV_COMM_CFG_EX)
#define DH_DEV_NETCARD_CFG                0x0053        // net card configuration(structure DHDEV_NETCARD_CFG)
#define DH_DEV_BACKUP_VIDEO_FORMAT        0x0054        // Video backup format(structure DHDEV_BACKUP_VIDEO_FORMAT)
#define DH_DEV_STREAM_ENCRYPT_CFG         0x0055        // stream encrypt configuration(structure DHEDV_STREAM_ENCRYPT)
#define DH_DEV_IPFILTER_CFG_EX            0x0056        // IP filter extended configuration(structure DHDEV_IPIFILTER_CFG_EX)
#define DH_DEV_CUSTOM_CFG                 0x0057        // custom configuration(structure DHDEV_CUSTOM_CFG)
#define DH_DEV_WLAN_DEVICE_CFG_EX         0x0058        // Search wireless device extended setup(structure DHDEV_WLAN_DEVICE_LIST_EX)
#define DH_DEV_ACC_POWEROFF_CFG           0x0059        // ACC power off configuration(structure DHDEV_ACC_POWEROFF_CFG)
#define DH_DEV_EXPLOSION_PROOF_CFG        0x005a        // explosion proof alarm configuration(structure DHDEV_EXPLOSION_PROOF_CFG)
#define DH_DEV_NETCFG_EX                  0x005b        // Network extended setup(struct DHDEV_NET_CFG_EX)
#define DH_DEV_LIGHTCONTROL_CFG           0x005c        // light control config(struct DHDEV_LIGHTCONTROL_CFG)
#define DH_DEV_3GFLOW_CFG                 0x005d        // 3G flow info config(struct DHDEV_3GFLOW_INFO_CFG)
#define DH_DEV_IPV6_CFG                   0x005e        // IPv6 config(struct DHDEV_IPV6_CFG)
#define DH_DEV_SNMP_CFG                   0x005f        // Snmp config(struct DHDEV_NET_SNMP_CFG)
#define DH_DEV_SNAP_CONTROL_CFG           0x0060        // snap control config(struct DHDEV_SNAP_CONTROL_CFG)
#define DH_DEV_GPS_MODE_CFG               0x0061        // GPS mode config(struct DHDEV_GPS_MODE_CFG)
#define DH_DEV_SNAP_UPLOAD_CFG            0x0062        // Snap upload config(struct DHDEV_SNAP_UPLOAD_CFG)
#define DH_DEV_SPEED_LIMIT_CFG            0x0063        // Speed limit config(struct DHDEV_SPEED_LIMIT_CFG)
#define DH_DEV_ISCSI_CFG                  0x0064        // iSCSI config(struct DHDEV_ISCSI_CFG), need reboot
#define DH_DEV_WIRELESS_ROUTING_CFG       0x0065        // wifi config(struc DHDEV_WIRELESS_ROUTING_CFG)
#define DH_DEV_ENCLOSURE_CFG              0x0066        // enclosure config(stuct DHDEV_ENCLOSURE_CFG)
#define DH_DEV_ENCLOSURE_VERSION_CFG      0x0067        // enclosure version config(struct DHDEV_ENCLOSURE_VERSION_CFG)
#define DH_DEV_RIAD_EVENT_CFG             0x0068        // Raid event config(struct DHDEV_RAID_EVENT_CFG)
#define DH_DEV_FIRE_ALARM_CFG             0x0069        // fire alarm config(struct DHDEV_FIRE_ALARM_CFG)
#define DH_DEV_LOCALALARM_NAME_CFG        0x006a        // local alarm name config(string like "Name1&&name2&&name3...")
#define DH_DEV_URGENCY_RECORD_CFG         0x0070        // urgency storage config(struct DHDEV_URGENCY_RECORD_CFG)
#define DH_DEV_ELEVATOR_ATTRI_CFG         0x0071        // elevator parameter config(struct DHDEV_ELEVATOR_ATTRI_CFG)
#define DH_DEV_ATM_OVERLAY_CFG_EX         0x0072        // TM overlay function. For latest ATM series product only.  Support devices of 32-channel or higher.( struct DHDEV_ATM_OVERLAY_CONFIG_EX)
#define DH_DEV_MACFILTER_CFG              0x0073        // MAC filter config(struct DHDEV_MACFILTER_CFG)
#define DH_DEV_MACIPFILTER_CFG            0x0074        // MAC,IP filter config(need ip,mac one to one corresponding)(struct DHDEV_MACIPFILTER_CFG)
#define DH_DEV_STREAM_ENCRYPT_TIME_CFG    0x0075        // stream encrypt(encryot plan)(struct DHEDV_STREAM_ENCRYPT)
#define DH_DEV_LIMIT_BIT_RATE_CFG         0x0076        // limit bit rate config(struct DHDEV_LIMIT_BIT_RATE) 
#define DH_DEV_SNAP_CFG_EX                0x0077        // snap extern config(struct DHDEV_SNAP_CFG_EX)
#define DH_DEV_DECODER_URL_CFG            0x0078        // decoder url config(struct DHDEV_DECODER_URL_CFG)
#define DH_DEV_TOUR_ENABLE_CFG            0x0079        // toyr enable config(struct DHDEV_TOUR_ENABLE_CFG)
#define DH_DEV_VICHILE_WIFI_AP_CFG_EX     0x007a        // wifi ap extern config(struct DHDEV_VEHICLE_WIFI_AP_CFG_EX)
#define DH_DEV_ENCODER_CFG_EX             0x007b        // encoder extern config(struct DEV_ENCODER_CFG_EX)

// Here are some ITC_HWS000 configurations 
#define DH_DEV_ITC_HWS000_RA_CFG            0x007c      // ITC_HWS000 RA config(corresponding int value)
#define DH_DEV_ITC_HWS000_HS_CFG            0x007d      // ITC_HWS000 HS config(corresponding int value)
#define DH_DEV_ITC_HWS000_LS_CFG            0x007e      // ITC_HWS000 LS config(corresponding int value)
#define DH_DEV_ITC_HWS000_FL_CFG            0x007f      // ITC_HWS000 FL config(corresponding int value)
#define DH_DEV_ITC_HWS000_OC_CFG            0x0080      // ITC_HWS000 OC config(corresponding int value)
#define DH_DEV_ITC_HWS000_LC_CFG            0x0081      // ITC_HWS000 LC config(corresponding int value)
#define DH_DEV_ITC_HWS000_AC_CFG            0x0082      // ITC_HWS000 AC config(corresponding int value)
#define DH_DEV_ITC_HWS000_FC_CFG            0x0083      // ITC_HWS000 FC config(corresponding int value)
#define DH_DEV_ITC_HWS000_ALL_CFG           0x0084      // Query all the values (corresponding struct DEV_ITC_HWS000_ALL_CFG)
#define DH_DEV_ITC_HWS000_CLEAR_COUNT_CFG   0x0085      // Clear the counts(OC,LC,AC,FC corresponding int value)

#define DH_DEV_USER_END_CFG                 1000
#define DH_DEV_ITC_HWS000_CFG_ITEM_NUM      8           // the max count of ITC_HWS000's config item
// the config for all the values
typedef struct tagDEV_ITC_HWS000_ALL_CFG
{
    DWORD dwSize;                                // struct size
    int nValue[DH_DEV_ITC_HWS000_CFG_ITEM_NUM];  // the array of values
                                                 // start form the index 0, a sequence of RA, HS, LS, FL, OC, LC, AC, FC
}DEV_ITC_HWS000_ALL_CFG;

// Alarm type, corresponding to CLIENT_StartListen
#define DH_COMM_ALARM                       0x1100      // General alarm(Including external alarm, video loss and motion detection)
#define DH_SHELTER_ALARM                    0x1101      // Camera masking alarm 
#define DH_DISK_FULL_ALARM                  0x1102      // HDD full alarm 
#define DH_DISK_ERROR_ALARM                 0x1103      // HDD malfunction alarm 
#define DH_SOUND_DETECT_ALARM               0x1104      // Audio detection alarm 
#define DH_ALARM_DECODER_ALARM              0x1105      // Alarm decoder alarm 

// Extensive alarm type,corresponding to CLIENT_StartListenEx
#define DH_ALARM_ALARM_EX                   0x2101      // External alarm 
#define DH_MOTION_ALARM_EX                  0x2102      // Motion detection alarm 
#define DH_VIDEOLOST_ALARM_EX               0x2103      // Video loss alarm 
#define DH_SHELTER_ALARM_EX                 0x2104      // Camera masking alarm 
#define DH_SOUND_DETECT_ALARM_EX            0x2105      // Audio detection alarm 
#define DH_DISKFULL_ALARM_EX                0x2106      // HDD full alarm 
#define DH_DISKERROR_ALARM_EX               0x2107      // HDD malfunction alarm 
#define DH_ENCODER_ALARM_EX                 0x210A      // Encoder alarm 
#define DH_URGENCY_ALARM_EX                 0x210B      // Emergency alarm 
#define DH_WIRELESS_ALARM_EX                0x210C      // Wireless alarm 
#define DH_NEW_SOUND_DETECT_ALARM_EX        0x210D      // New auido detection alarm. Please refer to DH_NEW_SOUND_ALARM_STATE for alarm information structure;
#define DH_ALARM_DECODER_ALARM_EX           0x210E      // Alarm decoder alarm 
#define DH_DECODER_DECODE_ABILITY           0x210F      // NVD:Decoding capacity
#define DH_FDDI_DECODER_ABILITY             0x2110      // Fiber encoder alarm
#define DH_PANORAMA_SWITCH_ALARM_EX         0x2111      // Panorama switch alarm
#define DH_LOSTFOCUS_ALARM_EX               0x2112      // Lost focus alarm
#define DH_OEMSTATE_EX                      0x2113      // oem state
#define DH_DSP_ALARM_EX                     0x2114      // DSP alarm
#define DH_ATMPOS_BROKEN_EX                 0x2115      // atm and pos disconnection alarm, 0:disconnection 1:connection
#define DH_RECORD_CHANGED_EX                0x2116      // Record state changed alarm
#define DH_CONFIG_CHANGED_EX                0x2117      // Device config changed alarm
#define DH_DEVICE_REBOOT_EX                 0x2118      // Device rebooting alarm
#define DH_WINGDING_ALARM_EX                0x2119      // CoilFault alarm
#define DH_TRAF_CONGESTION_ALARM_EX         0x211A      // traffic congestion alarm
#define DH_TRAF_EXCEPTION_ALARM_EX          0x211B      // traffic exception alarm
#define DH_EQUIPMENT_FILL_ALARM_EX          0x211C      // FlashFault alarm
#define DH_ALARM_ARM_DISARM_STATE           0x211D      // alarm arm disarm 
#define DH_ALARM_ACC_POWEROFF               0x211E      // ACC power off alarm
#define DH_ALARM_3GFLOW_EXCEED              0x211F      // Alarm of 3G flow exceed(see struct DHDEV_3GFLOW_EXCEED_STATE_INFO)
#define DH_ALARM_SPEED_LIMIT                0x2120      // Speed limit alarm 
#define DH_ALARM_VEHICLE_INFO_UPLOAD        0x2121      // Vehicle information uploading 
#define DH_STATIC_ALARM_EX                  0x2122      // Static detection alarm
#define DH_PTZ_LOCATION_EX                  0x2123      // ptz location info
#define DH_ALARM_CARD_RECORD_UPLOAD         0x2124      // card record info(struct ALARM_CARD_RECORD_INFO_UPLOAD)
#define DH_ALARM_ATM_INFO_UPLOAD            0x2125      // ATM trade info(struct ALARM_ATM_INFO_UPLOAD)
#define DH_ALARM_ENCLOSURE                  0x2126      // enclosure alarm(struct ALARM_ENCLOSURE_INFO)
#define DH_ALARM_SIP_STATE                  0x2127      // SIP state alarm(struct ALARM_SIP_STATE)
#define DH_ALARM_RAID_STATE                 0x2128      // RAID state alarm(struct ALARM_RAID_INFO)
#define DH_ALARM_CROSSING_SPEED_LIMIT       0x2129      // crossing speed limit alarm(struct ALARM_SPEED_LIMIT)
#define DH_ALARM_OVER_LOADING               0x212A      // over loading alarm(struct ALARM_OVER_LOADING)
#define DH_ALARM_HARD_BRAKING               0x212B      // hard brake alarm(struct ALARM_HARD_BRAKING)
#define DH_ALARM_SMOKE_SENSOR               0x212C      // smoke sensor alarm(struct ALARM_SMOKE_SENSOR)
#define DH_ALARM_TRAFFIC_LIGHT_FAULT        0x212D      // traffic light fault alarm(struct ALARM_TRAFFIC_LIGHT_FAULT) 
#define DH_ALARM_TRAFFIC_FLUX_STAT          0x212E      // traffic flux alarm(struct ALARM_TRAFFIC_FLUX_LANE_INFO)
#define DH_ALARM_CAMERA_MOVE                0x212F      // camera move alarm(struct ALARM_CAMERA_MOVE_INFO)
#define DH_ALARM_DETAILEDMOTION             0x2130      // detailed motion alarm(struct ALARM_DETAILEDMOTION_CHNL_INFO)
#define DH_ALARM_STORAGE_FAILURE            0x2131      // storage failure alarm(struct ALARM_STORAGE_FAILURE)
#define DH_ALARM_FRONTDISCONNECT            0x2132      // front IPC disconnect alarm(struct ALARM_FRONTDISCONNET_INFO)
#define DH_ALARM_ALARM_EX_REMOTE            0x2133      // remote external alarm
#define DH_ALARM_BATTERYLOWPOWER            0x2134      // battery low power alarm(struct ALARM_BATTERYLOWPOWER_INFO)
#define DH_ALARM_TEMPERATURE                0x2135      // temperature alarm(struct ALARM_TEMPERATURE_INFO)
#define DH_ALARM_TIREDDRIVE                 0x2136      // tired drive alarm(struct ALARM_TIREDDRIVE_INFO)
#define DH_ALARM_LOST_RECORD                0x2137      // Alarm of record loss (corresponding structure ALARM_LOST_RECORD)
#define DH_ALARM_HIGH_CPU                   0x2138      // Alarm of High CPU Occupancy rate (corresponding structure ALARM_HIGH_CPU) 
#define DH_ALARM_LOST_NETPACKET             0x2139      // Alarm of net package loss (corresponding structure ALARM_LOST_NETPACKET)
#define DH_ALARM_HIGH_MEMORY                0x213A      // Alarm of high memory occupancy rate(corresponding structure ALARM_HIGH_MEMORY)
#define DH_LONG_TIME_NO_OPERATION           0x213B      // WEB user have no operation for long time (no extended info)
#define DH_BLACKLIST_SNAP                   0x213C      // blacklist snap(corresponding to DH_BLACKLIST_SNAP_INFO)         
#define DH_ALARM_DISK                       0x213E      // alarm of disk(corresponding to ALARM_DISK_INFO)
#define DH_ALARM_FILE_SYSTEM                0x213F      // alarm of file system(corresponding to ALARM_FILE_SYSTEM_INFO)
#define DH_ALARM_IVS                        0x2140      // alarm of ivs(corresponding to ALARM_IVS_INFO)
#define DH_ALARM_GOODS_WEIGHT_UPLOAD        0x2141      // goods weight (corresponding to ALARM_GOODS_WEIGHT_UPLOAD_INFO)
#define DH_ALARM_GOODS_WEIGHT               0x2142      // alarm of goods weight(corresponding to ALARM_GOODS_WEIGHT_INFO)
#define DH_GPS_STATUS                       0x2143      // GPS orientation info(corresponding to NET_GPS_STATUS_INFO)
#define DH_ALARM_DISKBURNED_FULL            0x2144      // alarm disk burned full(corresponding to ALARM_DISKBURNED_FULL_INFO)
#define DH_ALARM_STORAGE_LOW_SPACE          0x2145      // storage low space(corresponding to ALARM_STORAGE_LOW_SPACE_INFO)
#define DH_ALARM_DISK_FLUX                  0x2160      // disk flux abnormal(corresponding to ALARM_DISK_FLUX)
#define DH_ALARM_NET_FLUX                   0x2161      // net flux abnormal(corresponding to ALARM_NET_FLUX)
#define DH_ALARM_FAN_SPEED                  0x2162      // fan speed abnormal(corresponding to ALARM_FAN_SPEED)
#define DH_ALARM_STORAGE_FAILURE_EX         0x2163      // storage mistake(corresponding to ALARM_STORAGE_FAILURE_EX)
#define DH_ALARM_RECORD_FAILED              0x2164      // record abnormal(corresponding to ALARM_RECORD_FAILED_INFO)
#define DH_ALARM_STORAGE_BREAK_DOWN         0x2165      // storage break down(corresponding to ALARM_STORAGE_BREAK_DOWN_INFO)
#define DH_ALARM_VIDEO_ININVALID            0x2166      // ALARM_VIDEO_ININVALID_INFO
#define DH_ALARM_VEHICLE_TURNOVER           0x2167      // vehicle turnover arm event(struct ALARM_VEHICEL_TURNOVER_EVENT_INFO)
#define DH_ALARM_VEHICLE_COLLISION          0x2168      // vehicle collision event(struct ALARM_VEHICEL_COLLISION_EVENT_INFO)
#define DH_ALARM_VEHICLE_CONFIRM            0x2169      // vehicle confirm information event(struct ALARM_VEHICEL_CONFIRM_INFO)
#define DH_ALARM_VEHICLE_LARGE_ANGLE        0x2170      // vehicle camero large angle event(struct ALARM_VEHICEL_LARGE_ANGLE)
#define DH_ALARM_TALKING_INVITE             0x2171      // device talking invite event (struct ALARM_TALKING_INVITE_INFO)
#define DH_ALARM_ALARM_EX2                  0x2175      // local alarm event (struct ALARM_ALARM_INFO_EX2, upgrade DH_ALARM_ALARM_EX)
#define DH_ALARM_VIDEO_TIMING               0x2176      // video timing detecting event(struct ALARM_VIDEO_TIMING)
#define DH_ALARM_COMM_PORT                  0x2177      // COM event(struct ALARM_COMM_PORT_EVENT_INFO)
#define DH_ALARM_AUDIO_ANOMALY              0x2178      // audio anomaly event(struct ALARM_AUDIO_ANOMALY)
#define DH_ALARM_AUDIO_MUTATION             0x2179      // audio mutation event(struct ALARM_AUDIO_MUTATION)
#define DH_EVENT_TYREINFO                   0x2180      // Tyre information report event (struct EVENT_TYRE_INFO)
#define DH_ALARM_POWER_ABNORMAL             0x2181      // Redundant power supplies abnormal alarm(struct ALARM_POWER_ABNORMAL_INFO)
#define DH_EVENT_REGISTER_OFF               0x2182      // On-board equipment active offline events(struct  EVENT_REGISTER_OFF_INFO)
#define DH_ALARM_NO_DISK                    0x2183      // No hard disk alarm(struct ALARM_NO_DISK_INFO)
#define DH_ALARM_FALLING                    0x2184      // The fall alarm(struct ALARM_FALLING_INFO)
#define DH_ALARM_PROTECTIVE_CAPSULE         0x2185      // Protective capsule event(corresponding structure ALARM_PROTECTIVE_CAPSULE_INFO)
#define DH_ALARM_NO_RESPONSE                0x2186      // Call Non-response alarm(corresponding to ALARM_NO_RESPONSE_INFO)
#define DH_ALARM_CONFIG_ENABLE_CHANGE       0x2187      // Config enable to change reported event(corresponding to structure  ALARM_CONFIG_ENABLE_CHANGE_INFO)
#define DH_EVENT_CROSSLINE_DETECTION        0x2188      // Cross warning line event( Corresponding to structure ALARM_EVENT_CROSSLINE_INFO )
#define DH_EVENT_CROSSREGION_DETECTION      0x2189      // Warning zone event(Corresponding to structure ALARM_EVENT_CROSSREGION_INFO )
#define DH_EVENT_LEFT_DETECTION             0x218a      // Abandoned object event(Corresponding to structure ALARM_EVENT_LEFT_INFO )
#define DH_EVENT_FACE_DETECTION             0x218b      // Human face detect event(Corresponding to structure ALARM_EVENT_FACE_INFO ) 
#define DH_ALARM_IPC                        0x218c      // IPC alarm,IPC upload local alarm via DVR or NVR(Corresponding to structure ALARM_IPC_INFO)
#define DH_EVENT_TAKENAWAYDETECTION         0x218d      // Missing object event(Corresponding to structure ALARM_TAKENAWAY_DETECTION_INFO)
#define DH_EVENT_VIDEOABNORMALDETECTION     0x218e      // Video abnormal event(Corresponding to structure ALARM_VIDEOABNORMAL_DETECTION_INFO)
#define DH_EVENT_MOTIONDETECT               0x218f      // Video motion detect event  (Corresponding to structure ALARM_MOTIONDETECT_INFO)
#define DH_ALARM_PIR                        0x2190      // PIR alarm (Corresponding to BYTE*, pBuf length dwBufLen)
#define DH_ALARM_STORAGE_HOT_PLUG           0x2191      // Storage hot swap event(Corresponding to structure ALARM_STORAGE_HOT_PLUG_INFO)
#define DH_ALARM_FLOW_RATE                  0x2192      // the event of rate of flow(Corresponding to structure ALARM_FLOW_RATE_INFO)
#define DH_ALARM_MOVEDETECTION              0x2193      // Move detection event(Corresponding to structure ALARM_MOVE_DETECTION_INFO)
#define DH_ALARM_WANDERDETECTION            0x2194      // WanderDetection event(Corresponding to structure ALARM_WANDERDETECTION_INFO)
#define DH_ALARM_CROSSFENCEDETECTION        0x2195      // cross fence(Corresponding to ALARM_CROSSFENCEDETECTION_INFO)
#define DH_ALARM_PARKINGDETECTION           0x2196      // parking detection event(Corresponding to ALARM_PARKINGDETECTION_INFO)
#define DH_ALARM_RIOTERDETECTION            0x2197      // Rioter detection event(Corresponding to ALARM_RIOTERDETECTION_INFO)
#define DH_ALARM_STORAGE_NOT_EXIST          0x3167      // A storage group does not exist(struct ALARM_STORAGE_NOT_EXIST_INFO)
#define DH_ALARM_NET_ABORT                  0x3169      // Network fault event(struct ALARM_NETABORT_INFO)
#define DH_ALARM_IP_CONFLICT                0x3170      // IP conflict event(struct ALARM_IP_CONFLICT_INFO)
#define DH_ALARM_MAC_CONFLICT               0x3171      // MAC conflict event(struct ALARM_MAC_CONFLICT_INFO)
#define DH_ALARM_POWERFAULT                 0x3172      // power fault event(struct ALARM_POWERFAULT_INFO)
#define DH_ALARM_CHASSISINTRUDED            0x3173      // Chassis intrusion, tamper alarm events(struct ALARM_CHASSISINTRUDED_INFO)
#define DH_ALARM_ALARMEXTENDED              0x3174      // Native extension alarm events(struct ALARM_ALARMEXTENDED_INFO)
#define DH_ALARM_ARMMODE_CHANGE_EVENT       0x3175      // Cloth removal state change events(struct ALARM_ARMMODE_CHANGE_INFO)
#define DH_ALARM_BYPASSMODE_CHANGE_EVENT    0x3176      // The bypass state change events(struct ALARM_BYPASSMODE_CHANGE_INFO)
#define DH_ALARM_ACCESS_CTL_NOT_CLOSE       0x3177      // Entrance guard did not close events(struct ALARM_ACCESS_CTL_NOT_CLOSE_INFO)
#define DH_ALARM_ACCESS_CTL_BREAK_IN        0x3178      // break-in event(struct ALARM_ACCESS_CTL_BREAK_IN_INFO)
#define DH_ALARM_ACCESS_CTL_REPEAT_ENTER    0x3179      //access Again and again event(struct ALARM_ACCESS_CTL_REPEAT_ENTER_INFO)
#define DH_ALARM_ACCESS_CTL_DURESS          0x3180      // Stress CARDS event(struct ALARM_ACCESS_CTL_DURESS_INFO)
#define DH_ALARM_ACCESS_CTL_EVENT           0x3181      // Access event(struct ALARM_ACCESS_CTL_EVENT_INFO)
#define DH_URGENCY_ALARM_EX2                0x3182      // Emergency ALARM EX2(upgrade DH_URGENCY_ALARM_EX,struct ALARM_URGENCY_ALARM_EX2, Artificially triggered emergency, general processing is linked external communication function requests for help
#define DH_ALARM_INPUT_SOURCE_SIGNAL        0x3183      // Alarm input source signal events (as long as there is input will generate the event, whether to play the current mode, unable to block, struct ALARM_INPUT_SOURCE_SIGNAL_INFO)
#define DH_ALARM_ANALOGALARM_EVENT          0x3184      // analog alarm(struct ALARM_ANALOGALARM_EVENT_INFO)
#define DH_ALARM_ACCESS_CTL_STATUS          0x3185      // Access control status event(corresponding structure ALARM_ACCESS_CTL_STATUS_INFO)
#define DH_ALARM_ACCESS_SNAP                0x3186      // Access control snapshot event(corresponding to ALARM_ACCESS_SNAP_INFO)
#define DH_ALARM_ALARMCLEAR                 0x3187      // Cancel alarm(corresponding to structure ALARM_ALARMCLEAR_INFO)
#define DH_ALARM_CIDEVENT                   0x3188      // CID event(corresponding to structure ALARM_CIDEVENT_INFO)
#define DH_ALARM_TALKING_HANGUP             0x3189      // Device hand up evnt(corresponding to structure ALARM_TALKING_HANGUP_INFO)
#define DH_ALARM_BANKCARDINSERT             0x318a      // Bank card evnt(corresponding to structure ALARM_BANKCARDINSERT_INFO)
#define DH_ALARM_RCEMERGENCY_CALL           0x318b      // Emergency call alarm event(corresponding to structure ALARM_RCEMERGENCY_CALL_INFO)
#define DH_ALARM_OPENDOORGROUP              0x318c      // Multi-people group unlock event(corresponding to  structure ALARM_OPEN_DOOR_GROUP_INFO)
#define DH_ALARM_FINGER_PRINT               0x318d      // get fingerprint event(corresponding to  structure ALARM_CAPTURE_FINGER_PRINT_INFO)
#define DH_ALARM_CARD_RECORD                0x318e      // card no. record event(corresponding to  structure  ALARM_CARD_RECORD_INFO)
#define DH_ALARM_SUBSYSTEM_STATE_CHANGE     0x318f      // sub system status change event(corresponding to  structure ALARM_SUBSYSTEM_STATE_CHANGE_INFO)
#define DH_ALARM_BATTERYPOWER_EVENT         0x3190      // battery scheduled warning event(corresponding to  structure ALARM_BATTERYPOWER_INFO)
#define DH_ALARM_BELLSTATUS_EVENT           0x3191      // bell status event(corresponding to  structure ALARM_BELLSTATUS_INFO)
#define DH_ALARM_DEFENCE_STATE_CHANGE_EVENT 0x3192      // zone status change event(corresponding to  structure ALARM_DEFENCE_STATUS_CHANGE_INFO), 
                                                        // customized need, and arm/disarm change event, bypass event status have different definitions,
                                                        // The status via CLIENT_QueryDevState() port DH_DEVSTATE_DEFENCE_STATE command get
#define DH_ALARM_TICKET_STATISTIC           0x3193      // ticket statistics info event(corresponding to  structure  ALARM_TICKET_STATISTIC)
#define DH_ALARM_LOGIN_FAILIUR              0x3194      // login failed event(corresponding to  structure ALARM_LOGIN_FAILIUR_INFO)
#define DH_ALARM_MODULE_LOST                0x3195      // expansion module offline event(corresponding to  structure  ALARM_MODULE_LOST_INFO)
#define DH_ALARM_PSTN_BREAK_LINE            0x3196      // PSTN offline event(corresponding to  structure ALARM_PSTN_BREAK_LINE_INFO)
#define DH_ALARM_ANALOG_PULSE               0x3197      // analog alarm event(instant event), specific sensor  trigger(corresponding to  structure ALARM_ANALOGPULSE_INFO)
#define DH_ALARM_MISSION_CONFIRM            0x3198      // task confirmation event(corresponding to  structure  ALARM_MISSION_CONFIRM_INFO)
#define DH_ALARM_DEVICE_MSG_NOTIFY          0x3199      // device to platform notice event(corresponding to  structure  ALARM_DEVICE_MSG_NOTIFY_INFO)
#define DH_ALARM_VEHICLE_STANDING_OVER_TIME 0x319A      // parking timeout event(corresponding to  structure  ALARM_VEHICLE_STANDING_OVER_TIME_INFO)
#define DH_ALARM_ENCLOSURE_ALARM            0x319B      // e-fence event(corresponding to  structure  ALARM_ENCLOSURE_ALARM_INFO)
#define DH_ALARM_GUARD_DETECT               0x319C      // station detection event, one in station first report the start event, and last on in station report stop event before leave (corresponding to  structure ALARM_GUARD_DETECT_INFO)
#define DH_ALARM_GUARD_INFO_UPDATE          0x319D      // station info update event, report if people in station(corresponding to  structure ALARM_GUARD_UPDATE_INFO)  
#define DH_ALARM_NODE_ACTIVE                0x319E      // Node activation event (corresponding to structure ALARM_NODE_ACTIVE_INFO)
#define DH_ALARM_VIDEO_STATIC               0x319F      // Video static detection event (corresponding to structure ALARM_VIDEO_STATIC_INFO)
#define DH_ALARM_REGISTER_REONLINE          0x31a0      // Active registration device re-login event (corresponding to structure ALARM_REGISTER_REONLINE_INFO)
#define DH_ALARM_ISCSI_STATUS               0x31a1      // ISCSI alarm event (corresponding to structure ALARM_ISCSI_STATUS_INFO)
#define DH_ALARM_SCADA_DEV_ALARM            0x31a2      // detection collection device alarm event (corresponding to structure ALARM_SCADA_DEV_INFO)
#define DH_ALARM_AUXILIARY_DEV_STATE        0x31a3      // Sub device status(corresponding structure ALARM_AUXILIARY_DEV_STATE)
#define DH_ALARM_PARKING_CARD               0x31a4      // Parking swipe card event(corresponding structure ALARM_PARKING_CARD)
#define DH_ALARM_PROFILE_ALARM_TRANSMIT     0x31a5      // Alarm transmit event(corresponding structure ALARM_PROFILE_ALARM_TRANSMIT_INFO)
#define DH_ALARM_VEHICLE_ACC                0x31a6      // Vehicle acc event(corresponding structure ALARM_VEHICLE_ACC_INFO)
#define DH_ALARM_TRAFFIC_SUSPICIOUSCAR      0x31a7      // suspiciouscar event(corresponding structure ALARM_TRAFFIC_SUSPICIOUSCAR_INFO)
#define DH_ALARM_ACCESS_LOCK_STATUS         0x31a8      // the event of latch state (corresponding structure  ALARM_ACCESS_LOCK_STATUS_INFO)
#define DH_ALARM_FINACE_SCHEME              0x31a9      // Finace scheme event(corresponding structure ALARM_FINACE_SCHEME_INFO)
#define DH_ALARM_HEATIMG_TEMPER             0x31aa      // Thermal temperature abnormal event alarm(Corresponding to structure ALARM_HEATIMG_TEMPER_INFO)
#define DH_ALARM_TALKING_IGNORE_INVITE      0x31ab      // Device cancel bidirectional talk query event(Corresponding to structure ALARM_TALKING_IGNORE_INVITE_INFO)
#define DH_ALARM_BUS_SHARP_TURN             0x31ac      // Vehicle Abrupt-turn event(Corresponding to structure ALARM_BUS_SHARP_TURN_INFO)
#define DH_ALARM_BUS_SCRAM                  0x31ad      // vehicle abrupt stop event(Corresponding to structure ALARM_BUS_SCRAM_INFO)
#define DH_ALARM_BUS_SHARP_ACCELERATE       0x31ae      // Vehicle abrupt speed up event(Corresponding to structure ALARM_BUS_SHARP_ACCELERATE_INFO)
#define DH_ALARM_BUS_SHARP_DECELERATE       0x31af      // Vehicle abrupt slow down event (Corresponding to structure ALARM_BUS_SHARP_DECELERATE_INFO)
#define DH_ALARM_ACCESS_CARD_OPERATE        0x31b0      // A&C data operation event (Corresponding to structure ALARM_ACCESS_CARD_OPERATE_INFO)
#define DH_ALARM_POLICE_CHECK               0x31b1      // Policeman check in event(Corresponding to structure ALARM_POLICE_CHECK_INFO)
#define DH_ALARM_NET                        0x31b2      // Network alarm event(Corresponding to structure ALARM_NET_INFO)
#define DH_ALARM_NEW_FILE                   0x31b3      // New file event(Corresponding to structure ALARM_NEW_FILE_INFO)
#define DH_ALARM_FIREWARNING                0x31b5      // Thermal fire position (Corresponding to structure ALARM_FIREWARNING_INFO)
#define DH_ALARM_RECORD_LOSS                0x31b6      // Record loss event: the HDD is OK, delete results from misoperation.  (Corresponding to structure ALARM_RECORD_LOSS_INFO)
#define DH_ALARM_VIDEO_FRAME_LOSS           0x31b7      // Frame loss event: it results from poor network environment or insufficient encode capability (Corresponding to structure ALARM_VIDEO_FRAME_LOSS_INFO)
#define DH_ALARM_RECORD_VOLUME_FAILURE      0x31b8      // Abnormal record results from HDD volume(Corresponding to structure ALARM_RECORD_VOLUME_FAILURE_INFO)
#define DH_EVENT_SNAP_UPLOAD                0x31b9      // Image upload completion event(Corresponding to structure EVENT_SNAP_UPLOAD_INFO)
#define DH_ALARM_AUDIO_DETECT               0x31ba      // Audio detect event(Corresponding to structure ALARM_AUDIO_DETECT )
#define DH_ALARM_UPLOADPIC_FAILCOUNT        0x31bb      // Failure data amount during the image upload process (Corresponding to structure ALARM_UPLOADPIC_FAILCOUNT_INFO)
#define DH_ALARM_POS_MANAGE                 0x31bc      // POS management event(Corresponding to ALARM_POS_MANAGE_INFO )
#define DH_ALARM_REMOTE_CTRL_STATUS         0x31bd      // remote control status(Corresponding to ALARM_REMOTE_CTRL_STATUS )
#define DH_ALARM_PASSENGER_CARD_CHECK       0x31be      // desuetude, passenger card check(Corresponding to structure ALARM_PASSENGER_CARD_CHECK )
#define DH_ALARM_SOUND                      0x31bf      // Sound event(Corresponding to ALARM_SOUND )
#define DH_ALARM_LOCK_BREAK                 0x31c0      // Lock break event(Corresponding to ALARM_LOCK_BREAK_INFO )
#define DH_ALARM_HUMAN_INSIDE               0x31c1      // Human Inside event((Corresponding to structure ALARM_HUMAN_INSIDE_INFO)
#define DH_ALARM_HUMAN_TUMBLE_INSIDE        0x31c2      // Human tumble Inside(Corresponding to structure ALARM_HUMAN_TUMBLE_INSIDE_INFO)
#define DH_ALARM_DISABLE_LOCKIN             0x31c3      // Lock entry trigger event(Corresponding to structure ALARM_DISABLE_LOCKIN_INFO)
#define DH_ALARM_DISABLE_LOCKOUT            0x31c4      // Lock go out trigger(Corresponding to structure ALARM_DISABLE_LOCKOUT_INFO)
#define DH_ALARM_UPLOAD_PIC_FAILED          0x31c5      // break rules data upload failed (Corresponding to ALARM_UPLOAD_PIC_FAILED_INFO )
#define DH_ALARM_FLOW_METER                 0x31c6      // flow meter info event (ALARM_FLOW_METER_INFO)
#define DH_ALARM_WIFI_SEARCH                0x31c7      // search around wifi device(Corresponding to ALARM_WIFI_SEARCH_INFO)
#define DH_ALARM_WIRELESSDEV_LOWPOWER       0x31C8      // lowpower of wirelessdevice(ALARM_WIRELESSDEV_LOWPOWER_INFO)
#define DH_ALARM_PTZ_DIAGNOSES               0x31c9     // PTZ Diagnoses event(Corresponding to ALARM_PTZ_DIAGNOSES_INFO)
#define DH_ALARM_FLASH_LIGHT_FAULT          0x31ca      // flash light fault event (Corresponding to ALARM_FLASH_LIGHT_FAULT_INFO)
#define DH_ALARM_STROBOSCOPIC_LIGTHT_FAULT  0x31cb      // stroboscopic light fault event (Corresponding to ALARM_STROBOSCOPIC_LIGTHT_FAULT_INFO)
#define DH_ALARM_HUMAM_NUMBER_STATISTIC     0x31cc      // NumberStat event (Corresponding to ALARM_HUMAN_NUMBER_STATISTIC_INFO)
#define DH_ALARM_VIDEOUNFOCUS               0x31ce      // Video unfocus (Corresponding ALARM_VIDEOUNFOCUS_INFO)
#define DH_ALARM_BUF_DROP_FRAME             0x31cd      // Video recond buffer drop frame event(Corresponding to ALARM_BUF_DROP_FRAME_INFO)
#define DH_ALARM_DOUBLE_DEV_VERSION_ABNORMAL 0x31cf     // Abnormal event when master broad'version and slave broad'version different  (Corresponding to ALARM_DOUBLE_DEV_VERSION_ABNORMAL_INFO)
#define DH_ALARM_DCSSWITCH                  0x31d0      // Switch with master and slave(Corresponding to ALARM_DCSSWITCH_INFO)
#define DH_ALARM_RADAR_CONNECT_STATE        0x31d1      // Radar connect state(Corresponding to ALARM_RADAR_CONNECT_STATE_INFO)
#define DH_ALARM_DEFENCE_ARMMODE_CHANGE     0x31d2      // Defence arming status change(Corresponding to ALARM_DEFENCE_ARMMODECHANGE_INFO)
#define DH_ALARM_SUBSYSTEM_ARMMODE_CHANGE   0x31d3      // Subsystem arming status change(Corresponding to ALARM_SUBSYSTEM_ARMMODECHANGE_INFO)
#define DH_ALARM_RFID_INFO                  0x31d4      // infrared detection information event (Corresponding ALARM_RFID_INFO)
#define DH_ALARM_SMOKE_DETECTION            0x31d5      // smoke detection(Corresponding ALARM_SMOKE_DETECTION_INFO)
#define DH_ALARM_BETWEENRULE_TEMP_DIFF      0x31d6      // TemperatureDifference Between Rule (Corresponding ALARM_BETWEENRULE_DIFFTEMPER_INFO)
#define DH_ALARM_TRAFFIC_PIC_ANALYSE        0x31d7      // Traffic picture analyse(Corresponding ALARM_PIC_ANALYSE_INFO)
#define DH_ALARM_HOTSPOT_WARNING            0x31d8      // Hotspot warning(Corresponding ALARM_HOTSPOT_WARNING_INFO)
#define DH_ALARM_COLDSPOT_WARNING           0x31d9      // coldspot warning(Corresponding ALARM_COLDSPOT_WARNING_INFO)
#define DH_ALARM_FIREWARNING_INFO           0x31da      // firewarning (Corresponding ALARM_FIREWARNING_INFO_DETAIL)
#define DH_ALARM_FACE_OVERHEATING           0x31db      // face overheating(Corresponding ALARM_FACE_OVERHEATING_INFO)
#define DH_ALARM_SENSOR_ABNORMAL            0x31dc      // Sensor abnormal(Corresponding ALARM_SENSOR_ABNORMAL_INFO)
#define DH_ALARM_PATIENTDETECTION           0x31de      // patient detection(Corresponding ALARM_PATIENTDETECTION_INFO)
#define DH_ALARM_RADAR_HIGH_SPEED           0x31df      // radar high speed detection(Corresponding to ALARM_RADAR_HIGH_SPEED_INFO)
#define DH_ALARM_POLLING_ALARM              0x31e0      // Polling Alarm (Corresponding to ALARM_POLLING_ALARM_INFO)
#define DH_ALARM_ITC_HWS000                 0x31e1      // the alarm event for ITC_HWS000 (Corresponding ALARM_ITC_HWS000)
#define DH_ALARM_TRAFFICSTROBESTATE         0x31e2      // Traffic Strobe State(Corresponding to ALARM_TRAFFICSTROBESTATE_INFO)
#define DH_ALARM_TELEPHONE_CHECK            0x31e3      // telephone number check event(Corresponding to ALARM_TELEPHONE_CHECK_INFO)
#define DH_ALARM_PASTE_DETECTION            0x31e4      // Paste Detection(Corresponding to ALARM_PASTE_DETECTION_INFO )
#define DH_ALARM_SHOOTINGSCORERECOGNITION   0x31e5      // the alarm event for Shooting (Corresponding to ALARM_PIC_SHOOTINGSCORERECOGNITION_INFO)
#define DH_ALARM_SWIPEOVERTIME              0x31e6      // the alarm event for swipe overtime(Corresponding to ALARM_SWIPE_OVERTIME_INFO)
#define DH_ALARM_DRIVING_WITHOUTCARD        0x31e7      // the alarm event for driving without card(Corresponding to ALARM_DRIVING_WITHOUTCARD_INFO)
#define DH_ALARM_TRAFFIC_PEDESTRIAN_RUN_REDLIGHT_DETECTION 0x31e8  //red light event (Corresponding to ALARM_TRAFFIC_PEDESTRIAN_RUN_REDLIGHT_DETECTION_INFO)
#define DH_ALARM_FIGHTDETECTION             0x31e9      //the alarm event for fight detection(Corresponding to NET_ALARM_FIGHTDETECTION)
#define DH_ALARM_OIL_4G_OVERFLOW            0x31ea      //the alarm event for fushan oil 4G over flow threshold(Corresponding to NET_ALARM_OIL_4G_OVERFLOW_INFO)
#define DH_ALARM_ACCESSIDENTIFY             0x31eb      //VTO access identify(Corresponding to NET_ALARM_ACCESSIDENTIFY_INFO)
#define DH_ALARM_POWER_SWITCHER_ALARM       0x31ec      // the alarm event for Abnormal power switcher (Corresponding to DEV_ALRAM_POWERSWITCHER_INFO)
#define DH_ALARM_SCENNE_CHANGE_ALARM        0x31ed      // the alarm event for scene change (Corresponding to ALARM_PIC_SCENECHANGE_INFO)
#define DH_ALARM_WIFI_VIRTUALINFO_SEARCH    0x31ef      // the alarm event for WIFI virtual information(Corresponding to ALARM_WIFI_VIRTUALINFO_SEARCH_INFO)
#define DH_ALARM_TRAFFIC_OVERSPEED          0x31f0      // traffic overspeed event(Corresponding to event  ALARM_TRAFFIC_OVERSPEED_INFO)
#define DH_ALARM_TRAFFIC_UNDERSPEED         0x31f1      // traffic underspeed event(Corresponding to event  ALARM_TRAFFIC_NDERSPEED_INFO)
#define DH_ALARM_TRAFFIC_PEDESTRAIN         0x31f2      // traffic pedestrain event(Corresponding to event  ALARM_TRAFFIC_PEDESTRAIN_INFO)
#define DH_ALARM_TRAFFIC_JAM                0x31f3      // traffic jam event(Corresponding to event  ALARM_TRAFFIC_JAM_INFO)
#define DH_ALARM_TRAFFIC_PARKING            0x31f4      // traffic parking event(Corresponding to event  ALARM_TRAFFIC_PARKING_INFO)
#define DH_ALARM_TRAFFIC_THROW              0x31f5      // traffic throw event(Corresponding to event  ALARM_TRAFFIC_THROW_INFO)
#define DH_ALARM_TRAFFIC_RETROGRADE         0x31f6      // traffic retrograde event(Corresponding to event  ALARM_TRAFFIC_RETROGRADE_INFO)
#define DH_ALARM_VTSTATE_UPDATE             0x31f7      // VTS state update(Corresponding to ALARM_VTSTATE_UPDATE_INFO)
#define DH_ALARM_CALL_NO_ANSWERED           0x31f8      // the alarm event for call no answer, under directly connected(Corresponding to NET_ALARM_CALL_NO_ANSWERED_INFO)
#define DH_ALARM_USER_LOCK_EVENT            0x31f9      // User Lock Alarm Event
#define DH_ALARM_RETROGRADE_DETECTION       0x31fa      // retrogade dection event(Corresponding to ALARM_RETROGRADE_DETECTION_INFO)
#define DH_ALARM_AIO_APP_CONFIG_EVENT       0x31fb      // AIO App config event(Corresponding to NET_ALARM_AIO_APP_CONFIG_EVENT)
#define DH_ALARM_RAID_STATE_EX              0x31fc      // RAID state alarm(Corresponding to struct ALARM_RAID_INFO_EX)
#define DH_ALARM_STORAGE_IPC_FAILURE        0x31fd      // IPC storage failure alarm(IPC SD Card Abnormal Alarm)(Corresponding to struct ALARM_STORAGE_IPC_FAILURE_INFO)
#define DH_ALARM_DEVICE_STAY                0x31fe      // Still amarm, if the device coordinates are not changed in the specified time, the still alarm information is triggered(Corresponding to struct ALARM_DEVICE_STAY_INFO)    
#define DH_ALARM_SUB_WAY_DOOR_STATE         0x31ff      // the door state of subway(Corresponding to ALARM_SUB_WAY_DOOR_STATE_INFO)
#define DH_ALARM_SUB_WAY_PECE_SWITCH        0x3200      // the PECE switch state of subway(Corresponding to ALARM_SUB_WAY_PECE_SWITCH_INFO)
#define DH_ALARM_SUB_WAY_FIRE_ALARM         0x3201      // the fire alarm of subway(Corresponding to ALARM_SUB_WAY_FIRE_ALARM_INFO)
#define DH_ALARM_SUB_WAY_EMER_HANDLE        0x3202      // the emer handle state(Corresponding to ALARM_SUB_WAY_EMER_HANDLE_INFO)
#define DH_ALARM_SUB_WAY_CAB_COVER          0x3203      // the cab cover state(Corresponding to ALARM_SUB_WAY_CAB_COVER_INFO)
#define DH_ALARM_SUB_WAY_DERA_OBST          0x3204      // the dera or obst of subway(Corresponding to ALARM_SUB_WAY_DERA_OBST_INFO)
#define DH_ALARM_SUB_WAY_PECU_CALL          0x3205      // the PECU call state(Corresponding to ALARM_SUB_WAY_PECU_CALL_INFO)
#define DH_ALARM_BOX                        0x3206      // the Alarm Box(Corresponding to ALARM_BOX_INFO)
#define DH_ALARM_DOOR_CLOSEDMANUALLY        0x3207      // door closed manually(Corresponding to  ALARM_DOOR_CLOSEDMANUALLY_INFO)
#define DH_ALARM_DOOR_NOTCLOSED_LONGTIME    0x3208      // door not closed long time(Corresponding to ALARM_DOOR_NOTCLOSED_LONGTIME_INFO)
#define DH_ALARM_UNDER_VOLTAGE              0x3209      // the under voltage alarm ( Corresponding to ALARM_UNDER_VOLTAGE_INFO )
#define DH_ALARM_OVER_VOLTAGE               0x320a      // the over voltage alarm  ( Corresponding to ALARM_OVER_VOLTAGE_INFO )
#define DH_ALARM_CUT_LINE                   0x320b      // the cut line alarm(Corresponding to ALARM_CUT_LINE_INFO)
#define DH_ALARM_VIDEOMOTION_EVENT          0x320c      // video motion event(Corresponding to ALARM_VIDEOMOTION_EVENT_INFO)
#define DH_ALARM_WIDE_VIEW_REGION_EVENT     0x320d      // WideViewRegions event(Corresponding to ALARM_WIDE_VIEW_REGION_EVENT_INFO)
#define DH_ALARM_FIBRE_OPTIC_ABORT          0x320e      // the fibre optic abort alarm(Corresponding to ALARM_FIBRE_OPTIC_ABORT)
#define DH_ALARM_TAIL_DETECTION             0x320f      // tail detection(Corresponding ALARM_TAIL_DETECTION_INFO)
#define DH_ALARM_BITRATES_OVERLIMIT         0x3210      // alarm when camera bitrate is over channel decoding specification(Corresponding to ALARM_BITRATES_OVERLIMIT_INFO)
#define DH_ALARM_RECORD_CHANGED_EX          0x3211      // Record state changed alarm(Corresponding to ALARM_RECORD_CHANGED_INFO_EX)
#define DH_ALARM_HIGH_DECIBEL               0x3212      // High decibel alarm(Corresponding to ALARM_HIGH_DECIBEL_INFO)
#define DH_ALARM_SHAKE_DETECTION            0x3213      // Shake detection alarm(Corresponding to ALARM_SHAKE_DETECTION_INFO)
#define DH_ALARM_TUMBLE_DETECTION           0x3214      // tumble detection alarm(Corresponding to ALARM_TUMBLE_DETECTION_INFO)
#define DH_ALARM_ACCESS_CTL_MALICIOUS       0x3215      // Open door with malice(Corresponding to ALARM_ACCESS_CTL_MALICIOUS)
#define DH_ALARM_ACCESS_CTL_USERID_REGISTER 0x3216      // UserID Register(Corresponding to ALARM_ACCESS_CTL_USERID_REGISTER)
#define DH_ALARM_ACCESS_CTL_REVERSELOCK     0x3217      // ReverseLock alarm(Corresponding to ALARM_ACCESS_CTL_REVERSELOCK)
#define DH_ALARM_ACCESS_CTL_USERID_DELETE   0x3218      // UserID delete(Corresponding to ALARM_ACCESS_CTL_USERID_DELETE)
#define DH_ALARM_ACCESS_DOOR_BELL           0x3219      // Door bell (Corresponding to ALARM_ACCESS_DOOR_BELL_INFO)
#define DH_ALARM_ACCESS_FACTORY_RESET       0x321a      // Wireless Dev Facroty Reset (Corresponding to ALARM_ACCESS_FACTORY_RESET_INFO)
#define DH_ALARM_POLICE_RECORD_PROGRESS     0x321b        // MPT record file transfer progress(Corresponding to ALARM_POLICE_RECORD_PROGRESS_INFO)
#define DH_ALARM_POLICE_PLUGIN              0x321c        // MPT plugin in or out event(Corresponding to ALARM_POLICE_PLUGIN_INFO)
#define DH_ALARM_GPS_NOT_ALIGNED            0x321d        // GPS not aligned alarm (Corresponding to ALARM_GPS_NOT_ALIGNED_INFO)
#define DH_ALARM_WIRELESS_NOT_CONNECTED     0x321e        // WireLess(include wifi, 3G/4G) not connected alarm (Corresponding to ALARM_WIRELESS_NOT_CONNECTED_INFO)
#define DH_ALARM_CABINET                    0x321f        // Cloud Cabinet event(Corresponding to ALARM_CABINET_INFO)
#define DH_SWITCH_SCREEN                    0x3220        // Switch screen event.
#define DH_ALARM_NEAR_DISTANCE_DETECTION    0x3221        // alarm of near distance detection (Corresponding to ALARM_NEAR_DISTANCE_INFO)
#define DH_ALARM_MAN_STAND_DETECTION        0x3222        // alarm of stereo standing (Corresponding to ALARM_MAN_STAND_INFO)
#define DH_ALARM_MAN_NUM_DETECTION          0x3223        // alarm of regional population statistics  (Corresponding to ALARM_MAN_NUM_INFO)
#define DH_MCS_GENERAL_CAPACITY_LOW         0x3224        // MCS general capacity low event(Corresponding to ALARM_MCS_GENERAL_CAPACITY_LOW_INFO)
#define DH_MCS_DATA_NODE_OFFLINE            0x3225        // MCS data node offline event(Corresponding to ALARM_MCS_DATA_NODE_OFFLINE_INFO)
#define DH_MCS_DISK_OFFLINE                 0x3226        // MCS disk offline event(Corresponding to ALARM_MCS_DISK_OFFLINE_INFO)
#define DH_MCS_DISK_SLOW                    0x3227        // MCS disk slow event(Corresponding to ALARM_MCS_DISK_SLOW_INFO)
#define DH_MCS_DISK_BROKEN                  0x3228        // MCS disk broken event(Corresponding to ALARM_MCS_DISK_BROKEN_INFO)
#define DH_MCS_DISK_UNKNOW_ERROR            0x3229        // MCS disk unknown error event(Corresponding to ALARM_MCS_DISK_UNKNOW_ERROR_INFO)
#define DH_MCS_METADATA_SERVER_ABNORMAL     0x322a        // MCS metadata server abnormal event(Corresponding to ALARM_MCS_METADATA_SERVER_ABNORMAL_INFO)
#define DH_MCS_CATALOG_SERVER_ABNORMAL      0x322b        // MCS catalog server abnormal event(Corresponding to ALARM_MCS_CATALOG_SERVER_ABNORMAL_INFO)
#define DH_MCS_GENERAL_CAPACITY_RESUME      0x322c        // MCS general capacity resume event(Corresponding to ALARM_MCS_GENERAL_CAPACITY_RESUME_INFO)
#define DH_MCS_DATA_NODE_ONLINE             0x322d        // MCS data node online event(Corresponding to ALARM_MCS_DATA_NODE_ONLINE_INFO)
#define DH_MCS_DISK_ONLINE                  0x322e        // MCS disk online event(Corresponding to ALARM_MCS_DISK_ONLINE_INFO)
#define DH_MCS_METADATA_SLAVE_ONLINE        0x322f        // MCS metadata slave online event(Corresponding to ALARM_MCS_METADATA_SLAVE_ONLINE_INFO)
#define DH_MCS_CATALOG_SERVER_ONLINE        0x3230        // MCS catalog server online event(Corresponding to ALARM_MCS_CATALOG_SERVER_ONLINE_INFO)
#define DH_ALARM_OFFLINE_LOGSYNC            0x3231        // alarm of off Line logsync(Corresponding to ALARM_OFFLINE_LOGSYNC_INFO)
#define DH_ALARM_UPGRADE_STATE              0x3232        // event of device upgrade(Corresponding to ALARM_UPGRADE_STATE)
#define DH_ALARM_LABELINFO                  0x3233        // IPC added new(2017.4),RFID tag information acquisition event (Corresponding to DH_ALARM_LABELINFO)
#define DH_ALARM_TIRED_PHYSIOLOGICAL        0x3234        // alarm of Tired Physiological(Corresponding to ALARM_TIRED_PHYSIOLOGICAL)
#define DH_ALARM_CALLING_WHEN_DRIVING       0x3235        // alarm of Calling When Driving(Corresponding to ALARM_CALLING_WHEN_DRIVING)
#define DH_ALARM_TRAFFIC_DRIVER_SMOKING     0x3236        // alarm of Traffic Driver Smoking(Corresponding to ALARM_TRAFFIC_DRIVER_SMOKING)
#define DH_ALARM_TRAFFIC_DRIVER_LOWER_HEAD  0x3237        // alarm of Traffic Driver Lower Head(Corresponding to ALARM_TRAFFIC_DRIVER_LOWER_HEAD)
#define DH_ALARM_TRAFFIC_DRIVER_LOOK_AROUND 0x3238        // alarm of Traffic Driver Look Around(Corresponding to ALARM_TRAFFIC_DRIVER_LOOK_AROUND)
#define DH_ALARM_TRAFFIC_DRIVER_LEAVE_POST  0x3239        // alarm of Traffic Driver Leave Post(Corresponding to ALARM_TRAFFIC_DRIVER_LEAVE_POST)
#define DH_ALARM_TRAFFIC_DRIVER_YAWN        0x323a        // alarm of Traffic Driver Yawn(Corresponding to ALARM_TRAFFIC_DRIVER_YAWN)
#define DH_ALARM_AUTO_INSPECTION            0x323b        // Device auto inspection(Corresponding to ALARM_AUTO_INSPECTION) 
#define DH_ALARM_TRAFFIC_VEHICLE_POSITION   0x323c        // Vehicle Position Event(Corresponding to ALARM_TRAFFIC_VEHICLE_POSITION)
#define DH_ALARM_FACE_VERIFICATION_ACCESS_SNAP     0x323d // alarm of face verification access snap(Corresponding to ALARM_FACE_VERIFICATION_ACCESS_SNAP_INFO)
#define DH_ALARM_VIDEOBLIND                 0x323e        // alarm of video blind(Corresponding to ALARM_VIDEO_BLIND_INFO)
#define DH_ALARM_DRIVER_NOTCONFIRM          0x323f        // alarm of driver not confirm(Corresponding to ALARM_DRIVER_NOTCONFIRM_INFO)
#define DH_ALARM_FACEINFO_COLLECT           0x3240        // alarm of face info collect(Corresponding to ALARM_FACEINFO_COLLECT_INFO)
#define DH_ALARM_HIGH_SPEED				  	0x3241			// alarm of high speed(Corresponding to ALARM_HIGH_SPEED_INFO)
#define DH_ALARM_VIDEO_LOSS					0x3242			// alarm of video loss(Corresponding to ALARM_VIDEO_LOSS_INFO)
#define DH_ALARM_MPTBASE_CONNECT            0x3243          // alarm of the connection status between device ant base(Corresponding to ALARM_MPTBASE_CONNECT) 
#define DH_ALARM_LATEST_SHUTDOWN            0x3244          // alarm of the latest status of shut down (Corresponding to ALARM_LATEST_SHUTDOWN)

#define DH_ALARM_ROBOT_COLLISION	        0x3245			// alarm of robot collision(Corresponding to ALARM_ROBOT_COLLISION)
#define DH_ALARM_ROBOT_FALLENDOWN		    0x3246			// alarm of robot fallendown(Corresponding to ALARM_ROBOT_FALLENDOWN)
#define DH_ALARM_ROBOT_UNRECOGNIZED2DCODE   0x3247			// alarm of robot unrecognized 2DCODE(Corresponding to ALARM_ROBOT_UNRECOGNIZED2DCODE) 
#define DH_ALARM_ROBOT_WRONG2DCODE	        0x3248			// alarm of wrong 2DCODE(Corresponding to ALARM_ROBOT_WRONG2DCODE)
#define DH_ALARM_ROBOT_ROADBLOCKED	        0x3249		    // alarm of robot roadblocked(Corresponding to ALARM_ROBOT_ROADBLOCKED) 
#define DH_ALARM_ROBOT_FAULT				0x324a			// alarm of robot fault(Corresponding to ALARM_ROBOT_FAULT)
#define DH_ALARM_ROBOT_OVERLOAD			    0x324b			// alarm of robot overload(Corresponding to ALARM_ROBOT_OVERLOAD)
#define DH_ALARM_ROBOT_YAWEXCEPTION		    0x324c			// alarm of robot yawexception(Corresponding to ALARM_ROBOT_YAWEXCEPTION)

#define DH_ALARM_VIDEO_TALK_PATH		    0x324d			// alarm of the video talk path(Corresponding to ALARM_VIDEO_TALK_PATH_INFO)

#define DH_ALARM_ROBOT_LOADTIMEOUT		    0x324e			// alarm of robot LoadTimeout(Corresponding to ALARM_ROBOT_LOADTIMEOUT)
#define DH_ALARM_ROBOT_UNLOADTIMEOUT		0x324f			// alarm of robot UnLoadTimeout(Corresponding to ALARM_ROBOT_UNLOADTIMEOUT)
#define DH_ALARM_ROBOT_MAPUPDATE	    	0x3250			// alarm of robot MapUpdate(Corresponding to ALARM_ROBOT_MAPUPDATE)

// Event type
#define DH_CONFIG_RESULT_EVENT_EX            0x3000        // Modify the return code of the setup. Please refer to DEV_SET_RESULT for returned structure.
#define DH_REBOOT_EVENT_EX                   0x3001        // Device reboot event. Current modification becomes valid until sending out the reboot command. 
#define DH_AUTO_TALK_START_EX                0x3002        // Device automatically invites to begin audio talk 
#define DH_AUTO_TALK_STOP_EX                 0x3003        // Device actively stop audio talk 
#define DH_CONFIG_CHANGE_EX                  0x3004        // Device setup changes.
#define DH_IPSEARCH_EVENT_EX                 0x3005        // IP search event, the return value format is:DevName::Manufacturer::MAC::IP::Port::DevType::POEPort::SubMask::GateWay&&DevName::Manufacturer::MAC::IP::Port::DevType::POEPort::SubMask::GateWay &&
#define DH_AUTO_RECONNECT_FAILD              0x3006        // reconnect failed
#define DH_REALPLAY_FAILD_EVENT              0x3007        // real play failed
#define DH_PLAYBACK_FAILD_EVENT              0x3008        // playback failed
#define DH_IVS_TRAFFIC_REALFLOWINFO          0x3009        // traffic real flow info ALARM_IVS_TRAFFIC_REALFLOW_INFO
#define DH_DEVICE_ABORT_EVENT                0x300a        // Client end is out, corresponding to structure  DEV_CLIENT_ABORT_INFO
#define DH_TALK_FAILD_EVENT                  0x300b        // failed to request talk, corresponding to  structure  DEV_TALK_RESULT
#define DH_START_LISTEN_FINISH_EVENT         0x300c        // start listen interface finish asyn notify event, info pointer is NULL
#define DH_YUEQINGLIGHTING_STATE_EVENT       0x300d        // Statistical light switch time, corresponding to DEV_YUEQINGLIGHTING_STATE_INFO
#define DH_ALARM_VIOLATE_NO_FLY_TIME         0x300e        // violate NoFlyTime event, corresponding to ALARM_VIOLATE_NO_FLY_TIME_INFO
#define DH_ALARM_BOX_ALARM			         0x300f		// alarm box alarm (corresponding to ALARM_BOX_ALARM_INFO )


// Alarm type of alarm upload function,corresponding to CLIENT_StartService.NEW_ALARM_UPLOAD structure.
#define DH_UPLOAD_ALARM                      0x4000        // External alarm         
#define DH_UPLOAD_MOTION_ALARM               0x4001        // Motion detection alarm 
#define DH_UPLOAD_VIDEOLOST_ALARM            0x4002        // Video loss alarm 
#define DH_UPLOAD_SHELTER_ALARM              0x4003        // Camera masking alarm 
#define DH_UPLOAD_SOUND_DETECT_ALARM         0x4004        // Audio detection alarm 
#define DH_UPLOAD_DISKFULL_ALARM             0x4005        // HDD full alarm 
#define DH_UPLOAD_DISKERROR_ALARM            0x4006        // HDD malfunction alarm 
#define DH_UPLOAD_ENCODER_ALARM              0x4007        // Encoder alarm 
#define DH_UPLOAD_DECODER_ALARM              0x400B        // Alarm decoder alarm 
#define DH_UPLOAD_EVENT                      0x400C        // Scheduled upload 
#define DH_UPLOAD_IVS                        0x400D        // intelligent alarm,corresponding to ALARM_UPLOAD_IVS_INFO
#define DH_UPLOAD_SMOKESENSOR_ALARM          0x400E        // Smoke alarm, struct ALARM_UPLOAD_SMOKESENSOR_INFO
#define DH_UPLOAD_ALARM_EX                   0x400F        // External alarm extension, corresponding structure  ALARM_UPLOAD_ALARMEX_INFO 
#define DH_UPLOAD_REMOTE_LOGIN_FAILD_ALARM   0x4010        // Remote login failed alarm, corresponding structure ALARM_UPLOAD_LOGIN_FAILED_INFO
#define DH_UPLOAD_CUSTOM                     0x4011        // Extension string is defined by the client.,pParam:extension data pointer,dwParamLen:extension data length
#define DH_UPLOAD_FRONTDISCONNECT_ALARM      0x4012        // front device disconnect alarm, corresponding structure NEW_ALARM_UPLOAD
#define DH_UPLOAD_ALARM_IPC                  0x4013        // IPC External alarm, corresponding structure NEW_ALARM_UPLOAD
#define DH_UPLOAD_NET_ALARM                  0x4014        // Net Alarm input(trigger through network,different from device's local alarm input), corresponding structure NEW_ALARM_UPLOAD
#define DH_UPLOAD_ALARM_RAID                 0x4015        // Raid abnormal event, corresponding structure NEW_ALARM_UPLOAD
#define DH_UPLOAD_SCENE_CHANGE               0x4016        // Scene change event, corresponding structure NEW_ALARM_UPLOAD         

// order Bus status corresponding to event report(CLIENT_AttachBusState)
#define DH_ALARM_BUS_DRIVER_CHECK           0x0001      // driver sign up event(corresponding to  structure  ALARM_BUS_DRIVER_CHECK_INFO)
#define DH_ALARM_BUS_IMPORT_SITE            0x0002      // in station(corresponding to  structure  ALARM_BUS_IMPORT_SITE_INFO)
#define DH_ALARM_BUS_EXPORT_SITE            0x0003      // out event (corresponding to  structure  ALARM_BUS_EXPORT_SITE_INFO)
#define DH_ALARM_BUS_ABNORMAL               0x0004      // Abnormal vehicle event (corresponding to structure ALARM_BUS_ABNORMAL_INFO)
#define DH_ALARM_BUS_DOOR_OPEN              0x0005      // Unlock car door event (corresponding to structure ALARM_VEHICLE_DOOR_OPEN_INFO)
#define DH_ALARM_BUS_CUR_MILEAGE            0x0006      // Current mileage event(corresponding to structure ALARM_BUS_CUR_MILEAGE_INFO)
#define DH_ALARM_BUS_CUR_OIL                0x0007      // Current oil event(corresponding to structure ALARM_BUS_CUR_OIL_INFO)
#define DH_ALARM_BUS_LOW_OIL                0x0008      // Low oil event(corresponding to structure ALARM_BUS_LOW_OIL_INFO)
#define DH_ALARM_BUS_PASSENGER_CARD_CHECK   0x0009      // Passenger card check event(corresponding to structure ALARM_PASSENGER_CARD_CHECK )
#define DH_ALARM_BUS_VEHICLE_STANDING_OVER_TIME 0x000a  // vehicle standing over time(corresponding to structure ALARM_VEHICLE_STANDING_OVER_TIME_INFO )
#define DH_ALARM_BUS_DRIVE_AFTER_WORK       0x000b      // not work time(corresponding to structure ALARM_DVRIVE_AFTER_WORK )
#define DH_ALARM_BUS_PAD_SHUTDOWN           0x000c      // PAD shutdown event; DVR notify PadShutdown event, PAD will delay shutdown automatically by receiving this event(corresponding to structure ALARM_BUS_PAD_SHUTDOWN_INFO)
#define DH_ALARM_BUS_TIRED_DRIVE_CHECK      0x000d      // tired drive event(corresponding to ALARM_TIRED_DRIVE_CHECK_INFO )
#define DH_ALARM_BUS_STEAL_OIL              0x000e      // steal oil event(corresponding to ALARM_BUS_STEAL_OIL_INFO)

// Asynchronous interface callback type
#define RESPONSE_DECODER_CTRL_TV            0x00000001    // refer to CLIENT_CtrlDecTVScreen interface
#define RESPONSE_DECODER_SWITCH_TV          0x00000002    // refer to CLIENT_SwitchDecTVEncoder interface
#define RESPONSE_DECODER_PLAYBACK           0x00000003    // refer to CLIENT_DecTVPlayback interface
#define RESPONSE_EXCHANGE_DATA              0X00000004    // correspoding CLIENT_ExchangeData interface
#define RESPONSE_ASYN_QUERY_RECORDFILE      0X00000005    // correspoding CLIENT_StartQueryRecordFile interface

// CLIENT_FileTransmit Interface transmission file type
#define DH_DEV_UPGRADEFILETRANS_START       0x0000        // Begin sending update file(Corresponding structure DHDEV_UPGRADE_FILE_INFO)
#define DH_DEV_UPGRADEFILETRANS_SEND        0x0001        // Send update file  
#define DH_DEV_UPGRADEFILETRANS_STOP        0x0002        // Stop sending update file
#define DH_DEV_BLACKWHITETRANS_START        0x0003        // begin to send blackwhite list(Corresponding structure DHDEV_BLACKWHITE_LIST_INFO)
#define DH_DEV_BLACKWHITETRANS_SEND         0x0004        // send blackwhite list
#define DH_DEV_BLACKWHITETRANS_STOP         0x0005        // stop to send blackwhite list
#define DH_DEV_BLACKWHITE_LOAD              0x0006        // blackwhite list load (Corresponding structure DHDEV_LOAD_BLACKWHITE_LIST_INFO)
#define DH_DEV_BLACKWHITE_LOAD_STOP         0x0007        // blackwhite list load stop
#define DH_DEV_FILETRANS_STOP               0x002B        // Stop file upload
#define DH_DEV_FILETRANS_BURN               0x002C        // Burn File Upload

// Resolution list. Use to AND & OR of resolution subnet mask 
#define    DH_CAPTURE_SIZE_D1       0x00000001
#define DH_CAPTURE_SIZE_HD1         0x00000002
#define DH_CAPTURE_SIZE_BCIF        0x00000004
#define DH_CAPTURE_SIZE_CIF         0x00000008
#define DH_CAPTURE_SIZE_QCIF        0x00000010    
#define DH_CAPTURE_SIZE_VGA         0x00000020    
#define DH_CAPTURE_SIZE_QVGA        0x00000040
#define DH_CAPTURE_SIZE_SVCD        0x00000080
#define DH_CAPTURE_SIZE_QQVGA       0x00000100
#define DH_CAPTURE_SIZE_SVGA        0x00000200
#define DH_CAPTURE_SIZE_XVGA        0x00000400
#define DH_CAPTURE_SIZE_WXGA        0x00000800
#define DH_CAPTURE_SIZE_SXGA        0x00001000
#define DH_CAPTURE_SIZE_WSXGA       0x00002000   
#define DH_CAPTURE_SIZE_UXGA        0x00004000
#define DH_CAPTURE_SIZE_WUXGA       0x00008000
#define DH_CAPTURE_SIZE_LFT         0x00010000
#define DH_CAPTURE_SIZE_720         0x00020000
#define DH_CAPTURE_SIZE_1080        0x00040000
#define DH_CAPTURE_SIZE_1_3M        0x00080000
#define DH_CAPTURE_SIZE_2M          0x00100000
#define DH_CAPTURE_SIZE_5M          0x00200000
#define DH_CAPTURE_SIZE_3M          0x00400000    
#define DH_CAPTURE_SIZE_5_0M        0x00800000
#define DH_CPTRUTE_SIZE_1_2M        0x01000000
#define DH_CPTRUTE_SIZE_1408_1024   0x02000000
#define DH_CPTRUTE_SIZE_8M          0x04000000    
#define DH_CPTRUTE_SIZE_2560_1920   0x08000000
#define DH_CAPTURE_SIZE_960H        0x10000000    
#define DH_CAPTURE_SIZE_960_720     0x20000000                

// Encode mode list. Use to work AND & OR operation of encode mode mask.
#define DH_CAPTURE_COMP_DIVX_MPEG4    0x00000001
#define DH_CAPTURE_COMP_MS_MPEG4      0x00000002
#define DH_CAPTURE_COMP_MPEG2         0x00000004
#define DH_CAPTURE_COMP_MPEG1         0x00000008
#define DH_CAPTURE_COMP_H263          0x00000010
#define DH_CAPTURE_COMP_MJPG          0x00000020
#define DH_CAPTURE_COMP_FCC_MPEG4     0x00000040
#define DH_CAPTURE_COMP_H264          0x00000080
#define DH_CAPTURE_COMP_H265          0x00000100
#define DH_CAPTURE_COMP_SVAC          0x00000200

// Alarm activation operation. Use to work AND & OR operation of alarm activation operation.
#define DH_ALARM_UPLOAD                0x00000001
#define DH_ALARM_RECORD                0x00000002
#define DH_ALARM_PTZ                   0x00000004
#define DH_ALARM_MAIL                  0x00000008
#define DH_ALARM_TOUR                  0x00000010
#define DH_ALARM_TIP                   0x00000020
#define DH_ALARM_OUT                   0x00000040
#define DH_ALARM_FTP_UL                0x00000080
#define DH_ALARM_BEEP                  0x00000100
#define DH_ALARM_VOICE                 0x00000200
#define DH_ALARM_SNAP                  0x00000400

// Restore default setup mask. Can use to AND & OR operation, interface CLIENT_ControlDevice enum type DH_CTRL_RESTOREDEFAULT
#define DH_RESTORE_COMMON              0x00000001    // General setup
#define DH_RESTORE_CODING              0x00000002    // Encode setup
#define DH_RESTORE_VIDEO               0x00000004    // Record setup
#define DH_RESTORE_COMM                0x00000008    // COM setup
#define DH_RESTORE_NETWORK             0x00000010    // network setup
#define DH_RESTORE_ALARM               0x00000020    // Alarm setup
#define DH_RESTORE_VIDEODETECT         0x00000040    // Video detection
#define DH_RESTORE_PTZ                 0x00000080    // PTZ control 
#define DH_RESTORE_OUTPUTMODE          0x00000100    // Output mode
#define DH_RESTORE_CHANNELNAME         0x00000200    // Channel name
#define DH_RESTORE_VIDEOINOPTIONS      0x00000400    // Camera attribute
#define DH_RESTORE_CPS                 0x00000800    // TrafficSnapshot
#define DH_RESTORE_INTELLIGENT         0x00001000    // Intelligent Component
#define DH_RESTORE_REMOTEDEVICE        0x00002000    // Remote device configuration
#define DH_RESTORE_DECODERVIDEOOUT     0x00004000    // decode video out
#define DH_RESTORE_LINKMODE            0x00008000    // link mode
#define DH_RESTORE_COMPOSITE           0x00010000    // split screen   
#define DH_RESTORE_ALL                 0x80000000    // Reset all

// PTZ property list. Lower four bytes subnet mask 
#define PTZ_DIRECTION                  0x00000001    // Direction
#define PTZ_ZOOM                       0x00000002    // Zoom
#define PTZ_FOCUS                      0x00000004    // Focus
#define PTZ_IRIS                       0x00000008    // Aperture
#define PTZ_ALARM                      0x00000010    // Alarm function 
#define PTZ_LIGHT                      0x00000020    // Light 
#define PTZ_SETPRESET                  0x00000040    // Set preset 
#define PTZ_CLEARPRESET                0x00000080    // Delete preset
#define PTZ_GOTOPRESET                 0x00000100    // Go to a preset
#define PTZ_AUTOPANON                  0x00000200    // Enable pan
#define PTZ_AUTOPANOFF                 0x00000400    // isable pan
#define PTZ_SETLIMIT                   0x00000800    // Set limit
#define PTZ_AUTOSCANON                 0x00001000    // Enable auto scan
#define PTZ_AUTOSCANOFF                0x00002000    // Disable auto scan 
#define PTZ_ADDTOUR                    0x00004000    // Add tour point
#define PTZ_DELETETOUR                 0x00008000    // Delete tour point
#define PTZ_STARTTOUR                  0x00010000    // Begin tour
#define PTZ_STOPTOUR                   0x00020000    // Stop tour
#define PTZ_CLEARTOUR                  0x00040000    // Delete tour
#define PTZ_SETPATTERN                 0x00080000    // Set pattern
#define PTZ_STARTPATTERN               0x00100000    // Enbale pattern
#define PTZ_STOPPATTERN                0x00200000    // Disable pattern
#define PTZ_CLEARPATTERN               0x00400000    // Delete pattern
#define PTZ_POSITION                   0x00800000    // Position 
#define PTZ_AUX                        0x01000000    // auxiliary button 
#define PTZ_MENU                       0x02000000    // Speed dome menu 
#define PTZ_EXIT                       0x04000000    // Exit speed dome menu 
#define PTZ_ENTER                      0x08000000    // Confirm
#define PTZ_ESC                        0x10000000    // Cancel 
#define PTZ_MENUUPDOWN                 0x20000000    // Menu up/down
#define PTZ_MENULEFTRIGHT              0x40000000    // Menu left/right 
#define PTZ_OPT_NUM                    0x80000000    // Operation amount

// Higher four bytes subnet mask
#define PTZ_DEV                        0x00000001    // PTZ control 
#define PTZ_MATRIX                     0x00000002    // Matrix control 

// Snapshot video encode type
#define CODETYPE_MPEG4                    0
#define CODETYPE_H264                     1
#define CODETYPE_JPG                      2

// Bit stream control control list
#define DH_CAPTURE_BRC_CBR                0
#define DH_CAPTURE_BRC_VBR                1
//#define DH_CAPTURE_BRC_MBR              2

//The frame type mask definition
#define FRAME_TYPE_MOTION                       0x00000001      // MD frame

// the type of intelligent analysis event 
#define EVENT_IVS_ALL                           0x00000001      // subscription all event
#define EVENT_IVS_CROSSLINEDETECTION            0x00000002      // cross line event(Corresponding to DEV_EVENT_CROSSLINE_INFO)
#define EVENT_IVS_CROSSREGIONDETECTION          0x00000003      // cross region event(Corresponding to DEV_EVENT_CROSSREGION_INFO)
#define EVENT_IVS_PASTEDETECTION                0x00000004      // past event(Corresponding to DEV_EVENT_PASTE_INFO)
#define EVENT_IVS_LEFTDETECTION                 0x00000005      // left event(Corresponding to DEV_EVENT_LEFT_INFO)
#define EVENT_IVS_STAYDETECTION                 0x00000006      // stay event(Corresponding to DEV_EVENT_STAY_INFO)
#define EVENT_IVS_WANDERDETECTION               0x00000007      // wander event(Corresponding to DEV_EVENT_WANDER_INFO)
#define EVENT_IVS_PRESERVATION                  0x00000008      // reservation event(Corresponding to DEV_EVENT_PRESERVATION_INFO) 
#define EVENT_IVS_MOVEDETECTION                 0x00000009      // move event(Corresponding to DEV_EVENT_MOVE_INFO)
#define EVENT_IVS_TAILDETECTION                 0x0000000A      // tail event(Corresponding to DEV_EVENT_TAIL_INFO)
#define EVENT_IVS_RIOTERDETECTION               0x0000000B      // rioter event(Corresponding to DEV_EVENT_RIOTERL_INFO)
#define EVENT_IVS_FIREDETECTION                 0x0000000C      // fire event(Corresponding to DEV_EVENT_FIRE_INFO)
#define EVENT_IVS_SMOKEDETECTION                0x0000000D      // smoke event(Corresponding to DEV_EVENT_SMOKE_INFO)
#define EVENT_IVS_FIGHTDETECTION                0x0000000E      // fight event(Corresponding to DEV_EVENT_FLOWSTAT_INFO)
#define EVENT_IVS_FLOWSTAT                      0x0000000F      // flow stat event(Corresponding to DEV_EVENT_FLOWSTAT_INFO)
#define EVENT_IVS_NUMBERSTAT                    0x00000010      // number stat event(Corresponding to DEV_EVENT_NUMBERSTAT_INFO)
#define EVENT_IVS_CAMERACOVERDDETECTION         0x00000011      // camera cover event
#define EVENT_IVS_CAMERAMOVEDDETECTION          0x00000012      // camera move event
#define EVENT_IVS_VIDEOABNORMALDETECTION        0x00000013      // video abnormal event(Corresponding to DEV_EVENT_VIDEOABNORMALDETECTION_INFO)
#define EVENT_IVS_VIDEOBADDETECTION             0x00000014      // video bad event
#define EVENT_IVS_TRAFFICCONTROL                0x00000015      // traffic control event(Corresponding to DEV_EVENT_TRAFFICCONTROL_INFO)
#define EVENT_IVS_TRAFFICACCIDENT               0x00000016      // traffic accident event(Corresponding to DEV_EVENT_TRAFFICACCIDENT_INFO)
#define EVENT_IVS_TRAFFICJUNCTION               0x00000017      // traffic junction event(Corresponding to DEV_EVENT_TRAFFICJUNCTION_INFO)
#define EVENT_IVS_TRAFFICGATE                   0x00000018      // traffic gate event(Corresponding to DEV_EVENT_TRAFFICGATE_INFO)
#define EVENT_TRAFFICSNAPSHOT                   0x00000019      // traffic snapshot(Corresponding to DEV_EVENT_TRAFFICSNAPSHOT_INFO)
#define EVENT_IVS_FACEDETECT                    0x0000001A      // face detection(Corresponding to DEV_EVENT_FACEDETECT_INFO)
#define EVENT_IVS_TRAFFICJAM                    0x0000001B      // traffic-Jam(Corresponding to DEV_EVENT_TRAFFICJAM_INFO)
#define EVENT_IVS_TRAFFIC_RUNREDLIGHT           0x00000100      // traffic-RunRedLight(Corresponding to DEV_EVENT_TRAFFIC_RUNREDLIGHT_INFO)
#define EVENT_IVS_TRAFFIC_OVERLINE              0x00000101      // traffic-Overline(Corresponding to DEV_EVENT_TRAFFIC_OVERLINE_INFO)
#define EVENT_IVS_TRAFFIC_RETROGRADE            0x00000102      // traffic-Retrograde(Corresponding to DEV_EVENT_TRAFFIC_RETROGRADE_INFO)
#define EVENT_IVS_TRAFFIC_TURNLEFT              0x00000103      // traffic-TurnLeft(Corresponding to DEV_EVENT_TRAFFIC_TURNLEFT_INFO)
#define EVENT_IVS_TRAFFIC_TURNRIGHT             0x00000104      // traffic-TurnRight(Corresponding to DEV_EVENT_TRAFFIC_TURNRIGHT_INFO)    
#define EVENT_IVS_TRAFFIC_UTURN                 0x00000105      // traffic-Uturn(Corresponding to DEV_EVENT_TRAFFIC_UTURN_INFO)
#define EVENT_IVS_TRAFFIC_OVERSPEED             0x00000106      // traffic-Overspeed(Corresponding to DEV_EVENT_TRAFFIC_OVERSPEED_INFO)
#define EVENT_IVS_TRAFFIC_UNDERSPEED            0x00000107      // traffic-Underspeed(Corresponding to DEV_EVENT_TRAFFIC_UNDERSPEED_INFO)
#define EVENT_IVS_TRAFFIC_PARKING               0x00000108      // traffic-Parking(Corresponding to DEV_EVENT_TRAFFIC_PARKING_INFO)
#define EVENT_IVS_TRAFFIC_WRONGROUTE            0x00000109      // traffic-WrongRoute(Corresponding to DEV_EVENT_TRAFFIC_WRONGROUTE_INFO)
#define EVENT_IVS_TRAFFIC_CROSSLANE             0x0000010A      // traffic-CrossLane(Corresponding to DEV_EVENT_TRAFFIC_CROSSLANE_INFO)
#define EVENT_IVS_TRAFFIC_OVERYELLOWLINE        0x0000010B      // traffic-OverYellowLine(Corresponding to DEV_EVENT_TRAFFIC_OVERYELLOWLINE_INFO)
#define EVENT_IVS_TRAFFIC_DRIVINGONSHOULDER     0x0000010C      // traffic-DrivingOnShoulder(Corresponding to DEV_EVENT_TRAFFIC_DRIVINGONSHOULDER_INFO)   
#define EVENT_IVS_TRAFFIC_YELLOWPLATEINLANE     0x0000010E      // traffic-YellowPlateInLane(Corresponding to DEV_EVENT_TRAFFIC_YELLOWPLATEINLANE_INFO)
#define EVENT_IVS_TRAFFIC_PEDESTRAINPRIORITY    0x0000010F      // Traffic offense-Pedestral has higher priority at the  crosswalk(Corresponding to DEV_EVENT_TRAFFIC_PEDESTRAINPRIORITY_INFO)
#define EVENT_IVS_CROSSFENCEDETECTION           0x0000011F      // cross fence(Corresponding to DEV_EVENT_CROSSFENCEDETECTION_INFO) 
#define EVENT_IVS_ELECTROSPARKDETECTION         0x00000110      // ElectroSpark event(Corresponding to DEV_EVENT_ELECTROSPARK_INFO) 
#define EVENT_IVS_TRAFFIC_NOPASSING             0x00000111      // no passing(Corresponding to DEV_EVENT_TRAFFIC_NOPASSING_INFO)
#define EVENT_IVS_ABNORMALRUNDETECTION          0x00000112      // abnormal run(Corresponding to DEV_EVENT_ABNORMALRUNDETECTION_INFO)
#define EVENT_IVS_RETROGRADEDETECTION           0x00000113      // retrograde(Corresponding to DEV_EVENT_RETROGRADEDETECTION_INFO)
#define EVENT_IVS_INREGIONDETECTION             0x00000114      // in region detection(Corresponding to DEV_EVENT_INREGIONDETECTION_INFO)
#define EVENT_IVS_TAKENAWAYDETECTION            0x00000115      // taking away things(Corresponding to DEV_EVENT_TAKENAWAYDETECTION_INFO)
#define EVENT_IVS_PARKINGDETECTION              0x00000116      // parking(Corresponding to DEV_EVENT_PARKINGDETECTION_INFO)
#define EVENT_IVS_FACERECOGNITION               0x00000117      // face recognition(Corresponding to DEV_EVENT_FACERECOGNITION_INFO)
#define EVENT_IVS_TRAFFIC_MANUALSNAP            0x00000118      // manual snap(Corresponding to DEV_EVENT_TRAFFIC_MANUALSNAP_INFO)
#define EVENT_IVS_TRAFFIC_FLOWSTATE             0x00000119      // traffic flow state(Corresponding to DEV_EVENT_TRAFFIC_FLOW_STATE)
#define EVENT_IVS_TRAFFIC_STAY                  0x0000011A      // traffic stay(Corresponding to DEV_EVENT_TRAFFIC_STAY_INFO)
#define EVENT_IVS_TRAFFIC_VEHICLEINROUTE        0x0000011B      // traffic vehicle route(Corresponding to DEV_EVENT_TRAFFIC_VEHICLEINROUTE_INFO)
#define EVENT_ALARM_MOTIONDETECT                0x0000011C      // motion detect(Corresponding to DEV_EVENT_ALARM_INFO)
#define EVENT_ALARM_LOCALALARM                  0x0000011D      // local alarm(Corresponding to DEV_EVENT_ALARM_INFO)
#define EVENT_IVS_PRISONERRISEDETECTION         0x0000011E      // prisoner rise detect(Corresponding to DEV_EVENT_PRISONERRISEDETECTION_INFO)
#define EVENT_IVS_TRAFFIC_TOLLGATE              0x00000120      // traffic tollgate(Corresponding to DEV_EVENT_TRAFFICJUNCTION_INFO)
#define EVENT_IVS_DENSITYDETECTION              0x00000121      // density detection of persons(Corresponding to DEV_EVENT_DENSITYDETECTION_INFO)
#define EVENT_IVS_VIDEODIAGNOSIS                0x00000122      // video diagnosis result(Corresponding to NET_VIDEODIAGNOSIS_COMMON_INFO and NET_REAL_DIAGNOSIS_RESULT)
#define EVENT_IVS_QUEUEDETECTION                0x00000123      // queue detection(Corresponding to DEV_EVENT_QUEUEDETECTION_INFO)
#define EVENT_IVS_TRAFFIC_VEHICLEINBUSROUTE     0x00000124      // take up in bus route(Corresponding to DEV_EVENT_TRAFFIC_VEHICLEINBUSROUTE_INFO)
#define EVENT_IVS_TRAFFIC_BACKING               0x00000125      // illegal astern(Corresponding to DEV_EVENT_IVS_TRAFFIC_BACKING_INFO) 
#define EVENT_IVS_AUDIO_ABNORMALDETECTION       0x00000126      // audio abnormity(Corresponding to DEV_EVENT_IVS_AUDIO_ABNORMALDETECTION_INFO)
#define EVENT_IVS_TRAFFIC_RUNYELLOWLIGHT        0x00000127      // run yellow light(Corresponding to DEV_EVENT_TRAFFIC_RUNYELLOWLIGHT_INFO)
#define EVENT_IVS_CLIMBDETECTION                0x00000128      // climb detection(Corresponding to DEV_EVENT_IVS_CLIMB_INFO) 
#define EVENT_IVS_LEAVEDETECTION                0x00000129      // leave detection(Corresponding to DEV_EVENT_IVS_LEAVE_INFO)
#define EVENT_IVS_TRAFFIC_PARKINGONYELLOWBOX    0x0000012A      // parking on yellow box(Corresponding to DEV_EVENT_TRAFFIC_PARKINGONYELLOWBOX_INFO)
#define EVENT_IVS_TRAFFIC_PARKINGSPACEPARKING   0x0000012B      // parking space parking(Corresponding to DEV_EVENT_TRAFFIC_PARKINGSPACEPARKING_INFO)
#define EVENT_IVS_TRAFFIC_PARKINGSPACENOPARKING 0x0000012C      // parking space no parking(Corresponding to DEV_EVENT_TRAFFIC_PARKINGSPACENOPARKING_INFO)
#define EVENT_IVS_TRAFFIC_PEDESTRAIN            0x0000012D      // passerby(Corresponding to DEV_EVENT_TRAFFIC_PEDESTRAIN_INFO)
#define EVENT_IVS_TRAFFIC_THROW                 0x0000012E      // throw(Corresponding to DEV_EVENT_TRAFFIC_THROW_INFO)
#define EVENT_IVS_TRAFFIC_IDLE                  0x0000012F      // idle(Corresponding to DEV_EVENT_TRAFFIC_IDLE_INFO)
#define EVENT_ALARM_VEHICLEACC                  0x00000130      // Vehicle ACC power outage alarm events(Corresponding to DEV_EVENT_ALARM_VEHICLEACC_INFO)
#define EVENT_ALARM_VEHICLE_TURNOVER            0x00000131      // Vehicle rollover alarm events(Corresponding to DEV_EVENT_VEHICEL_ALARM_INFO)
#define EVENT_ALARM_VEHICLE_COLLISION           0x00000132      // Vehicle crash alarm events(Corresponding to DEV_EVENT_VEHICEL_ALARM_INFO)
#define EVENT_ALARM_VEHICLE_LARGE_ANGLE         0x00000133      // On-board camera large Angle turn events
#define EVENT_IVS_TRAFFIC_PARKINGSPACEOVERLINE  0x00000134      // Parking line pressing events(Corresponding to DEV_EVENT_TRAFFIC_PARKINGSPACEOVERLINE_INFO)
#define EVENT_IVS_MULTISCENESWITCH              0x00000135      // Many scenes switching events(Corresponding to DEV_EVENT_IVS_MULTI_SCENE_SWICH_INFO)
#define EVENT_IVS_TRAFFIC_RESTRICTED_PLATE      0X00000136      // Limited license plate event(Corresponding to DEV_EVENT_TRAFFIC_RESTRICTED_PLATE)
#define EVENT_IVS_TRAFFIC_OVERSTOPLINE          0X00000137      // Cross stop line event(Corresponding to DEV_EVENT_TRAFFIC_OVERSTOPLINE)
#define EVENT_IVS_TRAFFIC_WITHOUT_SAFEBELT      0x00000138      // Traffic unfasten seat belt event(Corresponding to DEV_EVENT_TRAFFIC_WITHOUT_SAFEBELT) 
#define EVENT_IVS_TRAFFIC_DRIVER_SMOKING        0x00000139      // Driver smoking event(Corresponding to DEV_EVENT_TRAFFIC_DRIVER_SMOKING) 
#define EVENT_IVS_TRAFFIC_DRIVER_CALLING        0x0000013A      // Driver call event(Corresponding to DEV_EVENT_TRAFFIC_DRIVER_CALLING) 
#define EVENT_IVS_TRAFFIC_PEDESTRAINRUNREDLIGHT 0x0000013B      // Pedestrain red light(Corresponding to DEV_EVENT_TRAFFIC_PEDESTRAINRUNREDLIGHT_INFO)
#define EVENT_IVS_TRAFFIC_PASSNOTINORDER        0x0000013C      // Pass not in order(corresponding DEV_EVENT_TRAFFIC_PASSNOTINORDER_INFO)
#define EVENT_IVS_OBJECT_DETECTION              0x00000141      // Object feature detection event(Corresponding to DEV_EVENT_TRAFFIC_OBJECT_DETECTION) 
#define EVENT_ALARM_ANALOGALARM                 0x00000150      // Analog alarm channel alarm event(correspondingDEV_EVENT_ALARM_ANALOGALRM_INFO)
#define EVENT_IVS_CROSSLINEDETECTION_EX          0x00000151     // Warning lineexpansion event(Corresponding to DEV_EVENT_CROSSLINE_INFO_EX) 
#define EVENT_ALARM_COMMON                      0x00000152      // Normal Record
#define EVENT_ALARM_VIDEOBLIND                  0x00000153      // Video tampering event(Corresponding to DEV_EVENT_ALARM_VIDEOBLIND)
#define EVENT_ALARM_VIDEOLOSS                   0x00000154      // Video loss event
#define EVENT_IVS_GETOUTBEDDETECTION            0x00000155      // Event of getting out bed detection(Corresponding to DEV_EVENT_GETOUTBED_INFO)
#define EVENT_IVS_PATROLDETECTION               0x00000156      // Event of patrol detection(Corresponding to DEV_EVENT_PATROL_INFO)
#define EVENT_IVS_ONDUTYDETECTION               0x00000157      // Event of on duty detection(Corresponding to DEV_EVENT_ONDUTY_INFO)
#define EVENT_IVS_NOANSWERCALL                  0x00000158      // Event of VTO do not answer calling request
#define EVENT_IVS_STORAGENOTEXIST               0x00000159      // Event of storage do not exist
#define EVENT_IVS_STORAGELOWSPACE               0x0000015A      // Event of storage space low
#define EVENT_IVS_STORAGEFAILURE                0x0000015B      // Event of storage failure
#define EVENT_IVS_PROFILEALARMTRANSMIT          0x0000015C      // Event of profile alarm transmit
#define EVENT_IVS_VIDEOSTATIC                   0x0000015D      // Event of static video detect(corresponding DEV_EVENT_ALARM_VIDEOSTATIC_INFO)
#define EVENT_IVS_VIDEOTIMING                   0x0000015E      // Event of video timing detect(corresponding DEV_EVENT_ALARM_VIDEOTIMING_INFO)
#define EVENT_IVS_HEATMAP                       0x0000015F      // Heat map (Corresponding to )
#define EVENT_IVS_CITIZENIDCARD                 0x00000160      // ID info reading event (Corresponding to  DEV_EVENT_ALARM_CITIZENIDCARD_INFO)
#define EVENT_IVS_PICINFO                       0x00000161      // Image info event(Corresponding to DEV_EVENT_ALARM_PIC_INFO)
#define EVENT_IVS_NETPLAYCHECK                  0x00000162      // NetPlayCheck event(corresponding DEV_EVENT_ALARM_NETPLAYCHECK_INFO)
#define EVENT_IVS_TRAFFIC_JAM_FORBID_INTO       0x00000163      // Jam Forbid into  event(corresponding DEV_EVENT_ALARM_JAMFORBIDINTO_INFO)
#define EVENT_IVS_SNAPBYTIME                    0x00000164      // Snap by time event(corresponding DEV_EVENT_SNAPBYTIME)
#define EVENT_IVS_PTZ_PRESET                    0x00000165      // PTZ turn to preset event(corresponding to DEV_EVENT_ALARM_PTZ_PRESET_INFO)
#define EVENT_IVS_RFID_INFO                     0x00000166      // Event of infrared detect info(corresponding to DEV_EVENT_ALARM_RFID_INFO)
#define EVENT_IVS_STANDUPDETECTION              0X00000167      // Event of standing up detection
#define EVENT_IVS_QSYTRAFFICCARWEIGHT           0x00000168      // Event of QSYTrafficCarWeight (corresponding to DEV_EVENT_QSYTRAFFICCARWEIGHT_INFO)
#define EVENT_IVS_TRAFFIC_COMPAREPLATE          0x00000169      // Event of compare plate(corresponding to DEV_EVENT_TRAFFIC_COMPAREPLATE_INFO)
#define EVENT_IVS_SHOOTINGSCORERECOGNITION      0x0000016A      // Event of shooting score recognition(corresponding to CFG_IVS_SHOOTINGSCORERECOGNITION_INFO)
#define EVENT_IVS_TRAFFIC_FCC                   0x0000016B      // Event of refuel gas (corresponding to DEV_EVENT_TRAFFIC_FCC_INFO)
#define EVENT_IVS_TRAFFIC_TRANSFINITE           0x0000016C      // Event of traffic transfinite (corresponding to DEV_EVENT_TRAFFIC_TRANSFINITE_INFO)
#define EVENT_IVS_SCENE_CHANGE                  0x0000016D      // Event of scene change (corresponding to DEV_ALRAM_SCENECHANGE_INFO,CFG_VIDEOABNORMALDETECTION_INFO)
#define EVENT_IVS_LETRACK                       0x0000016E      // Event of simple track(no event data)
#define EVENT_IVS_OBJECT_ACTION                 0x0000016F      // Event of object action detection(no event data)
#define EVENT_IVS_TRAFFIC_ANALYSE_PRESNAP       0x00000170      // Event of presnap analyse(corresponding to DEV_EVENT_TRAFFIC_ANALYSE_PRESNAP_INFO)
#define EVENT_ALARM_EQSTATE                     0x00000171      // Event of electrical power quality state of smart switch (no event data)
#define EVENT_IVS_ALARM_IPC                     0x00000172      // Event of IPC used by DVR/NVR(no event data)
#define EVENT_IVS_POS_RECORD                    0x00000173      // Event of POS record
#define EVENT_IVS_NEAR_DISTANCE_DETECTION       0x00000174      // Event of near distance detection(corresponding to DEV_EVENT_NEAR_DISTANCE_DETECTION_INFO)
#define EVENT_IVS_OBJECTSTRUCTLIZE_PERSON       0x00000175      // Event of person feature detect(corresponding to DEV_EVENT_OBJECTSTRUCTLIZE_PERSON_INFO)
#define EVENT_IVS_OBJECTSTRUCTLIZE_NONMOTOR     0x00000176      // Event of nonmotor feature detect(corresponding to DEV_EVENT_OBJECTSTRUCTLIZE_NONMOTOR_INFO)
#define EVENT_IVS_TUMBLE_DETECTION              0x00000177      // Event of tumble detection(corresponding to DEV_EVENT_TUMBLE_DETECTION_INFO)
#define EVENT_IVS_TRAFFIC_ALL                   0x000001FF      // All event start with [TRAFFIC]
                                                                // EVENT_IVS_TRAFFICCONTROL -> EVENT_TRAFFICSNAPSHOT
                                                                // EVENT_IVS_TRAFFIC_RUNREDLIGHT -> EVENT_IVS_TRAFFIC_UNDERSPEED
#define EVENT_IVS_VIDEOANALYSE                  0x00000200      // All IVS events (Corresponding to )
#define EVENT_IVS_LINKSD                        0x00000201      // LinkSD events(Corresponding to )
#define EVENT_IVS_VEHICLEANALYSE                0x00000202      // Vehicle Analyse (Corresponding to DEV_EVENT_VEHICLEANALYSE)
#define EVENT_IVS_FLOWRATE                      0x00000203      // Flow rate events(Corresponding to DEV_EVENT_FLOWRATE_INFO)
#define EVENT_IVS_ACCESS_CTL                    0x00000204      // Acccess control events (Corresponding to DEV_EVENT_ACCESS_CTL_INFO)
#define EVENT_IVS_SNAPMANUAL                    0x00000205      // SnapManual events(Corresponding to DEV_EVENT_SNAPMANUAL)
#define EVENT_IVS_TRAFFIC_ELETAGINFO            0x00000206      // RFID electronic tag  events(Corresponding to DEV_EVENT_TRAFFIC_ELETAGINFO_INFO)
#define EVENT_IVS_TRAFFIC_TIREDPHYSIOLOGICAL    0x00000207      // physiological fatigue driving events(Corresponding to DEV_EVENT_TIREDPHYSIOLOGICAL_INFO)
#define EVENT_IVS_TRAFFIC_BUSSHARPTURN          0x00000208      // bus sharp trun events(Corresponding to DEV_EVENT_BUSSHARPTURN_INFO)
#define EVENT_IVS_CITIZEN_PICTURE_COMPARE       0x00000209      // Event of comparison with ID and citizen picture(corresponding to DEV_EVENT_CITIZEN_PICTURE_COMPARE_INFO)
#define EVENT_IVS_TRAFFIC_TIREDLOWERHEAD        0x0000020A      // Event of driver lower head(Corresponding to DEV_EVENT_TIREDLOWERHEAD_INFO)
#define EVENT_IVS_TRAFFIC_DRIVERLOOKAROUND      0x0000020B      // Event of driver look around(Corresponding to DEV_EVENT_DRIVERLOOKAROUND_INFO)
#define EVENT_IVS_TRAFFIC_DRIVERLEAVEPOST       0x0000020C      // Event of driver leave post(Corresponding to DEV_EVENT_DRIVERLEAVEPOST_INFO)
#define EVENT_IVS_MAN_STAND_DETECTION           0x0000020D      // stereo standing event(Corresponding to DEV_EVENT_MANSTAND_DETECTION_INFO)
#define EVENT_IVS_MAN_NUM_DETECTION             0x0000020E      // Event of regional population statistics (Corresponding to DEV_EVENT_MANNUM_DETECTION_INFO)
#define EVENT_IVS_STEREO_NUMBERSTAT             0x0000020F      // Event of passenger flow statistics(no event data)
#define EVENT_IVS_TRAFFIC_DRIVERYAWN            0x00000210      // Event of  driver yawn(Corresponding to DEV_EVENT_DRIVERYAWN_INFO)
#define EVENT_IVS_NUMBERSTAT_PLAN               0x00000211      // Passenger flow statistics plan(no event data,Speed Dome use,Corresponding to rule config CFG_NUMBERSTAT_INFO)
#define EVENT_IVS_HEATMAP_PLAN                  0x00000212      // Heat map plan(no event data,Speed Dome use,Corresponding to rule config CFG_IVS_HEATMAP_INFO)
#define EVENT_IVS_CALLNOANSWERED                0x00000213      // Event of call no answered
#define EVENT_IVS_IGNOREINVITE                  0x00000214      // Event of ignore invite
#define EVENT_IVS_HUMANTRAIT                    0x00000215      // Event of human trait(Corresponding to DEV_EVENT_HUMANTRAIT_INFO)
#define EVENT_ALARM_LE_HEADDETECTION            0x00000216      // alarm of Head detection
#define EVENT_IVS_FACEANALYSIS                  0x00000217      // Event of face analysis(no event data)
#define EVENT_IVS_TRAFFIC_TURNLEFTAFTERSTRAIGHT 0x00000218      // Event of turn left not give precedence to straight(Corresponding to DEV_EVENT_TURNLEFTAFTERSTRAIGHT_INFO)
#define EVENT_IVS_TRAFFIC_BIGBENDSMALLTURN      0x00000219      // Event of small turn on big bend(Corresponding to DEV_EVENT_BIGBENDSMALLTURN_INFO)
#define EVENT_IVS_ROAD_CONSTRUCTION             0x0000021A      // Event of road construction inspection (Corresponding to DEV_EVENT_ROAD_CONSTRUCTION_INFO)
#define EVENT_IVS_ROAD_BLOCK                    0x0000021B      // Event of road block  detection (Corresponding to DEV_EVENT_ROAD_BLOCK_INFO)
#define EVENT_IVS_TRAFFIC_QUEUEJUMP             0x0000021C      // Event of car jump a queue(Corresponding to DEV_EVENT_TRAFFIC_QUEUEJUMP_INFO)
#define EVENT_IVS_VEHICLE_SUSPICIOUSCAR         0x0000021D      // Event of Suspicious Car(Corresponding to DEV_EVENT_VEHICLE_SUSPICIOUSCAR_INFO)
#define EVENT_IVS_TRAFFIC_TURNRIGHTAFTERSTRAIGHT  0x0000021E    // Turn right to make a straight event(Corresponding to DEV_EVENT_TURNRIGHTAFTERSTRAIGHT_INFO)
#define EVENT_IVS_TRAFFIC_TURNRIGHTAFTERPEOPLE    0x0000021F    // Turn right and go straight to pedestrians(Corresponding to DEV_EVENT_TURNRIGHTAFTERPEOPLE_INFO)
#define EVENT_IVS_INSTALL_CARDREADER            0x00000220      // Install card reader event(Corresponding to DEV_EVENT_INSTALL_CARDREADER_INFO)
#define EVENT_ALARM_YALE_DROPBOX_BADTOKEN       0X00000221      // event of Yale token bad

//Storage Point
#define STOR_POINT_READ_WRITE                   0x00001001      // Read-Write group
#define STOR_POINT_READ_ONLY                    0x00001002      // Read-Only group
#define STOR_POINT_REDUNDANT                    0x00001003      // Redundant group
#define STOR_POINT_BACKUP                       0x00001004      // Backup group
#define STOR_POINT_DRAW_FRAME                   0x00001005      // Drawframe group
#define STOR_POINT_NAS_FTP                      0x00001006      // Remote group(protocol type:FTP)
#define STOR_POINT_NAS_NFS                      0x00001007      // Remote group(protocol type:NFS)
#define STOR_POINT_NAS_SMB                      0x00001008      // Remote group(protocol type:SMB)
#define STOR_POINT_NAS_ISCSI                    0x00001009      // Remote group(protocol type:ISCSI)
#define STOR_POINT_NAS_CLOUD                    0x0000100A      // Remote group(protocol type:Cloud)

//Traffic statistics event using macros
#define FLOWSTAT_ADDR_NAME                      16              //Has long place name

// interface CLIENT_OperateTrafficList, All kinds of string length definition
#define DH_TARFFIC_NAME_LEN                     16                
#define DH_CREATE_TIME_LEN                      32
#define DH_AUTHORITY_NUM                        16

// Error type code. Corresponding to the return value of CLIENT_GetLastError
#define _EC(x)                                    (0x80000000|x)
#define NET_NOERROR                              0          // No error 
#define NET_ERROR                                -1         // Unknown error
#define NET_SYSTEM_ERROR                        _EC(1)      // Windows system error
#define NET_NETWORK_ERROR                       _EC(2)      // Protocol error it may result from network timeout
#define NET_DEV_VER_NOMATCH                     _EC(3)      // Device protocol does not match 
#define NET_INVALID_HANDLE                      _EC(4)      // Handle is invalid
#define NET_OPEN_CHANNEL_ERROR                  _EC(5)      // Failed to open channel 
#define NET_CLOSE_CHANNEL_ERROR                 _EC(6)      // Failed to close channel 
#define NET_ILLEGAL_PARAM                       _EC(7)      // User parameter is illegal 
#define NET_SDK_INIT_ERROR                      _EC(8)      // SDK initialization error 
#define NET_SDK_UNINIT_ERROR                    _EC(9)      // SDK clear error 
#define NET_RENDER_OPEN_ERROR                   _EC(10)     // Error occurs when apply for render resources.
#define NET_DEC_OPEN_ERROR                      _EC(11)     // Error occurs when opening the decoder library 
#define NET_DEC_CLOSE_ERROR                     _EC(12)     // Error occurs when closing the decoder library 
#define NET_MULTIPLAY_NOCHANNEL                 _EC(13)     // The detected channel number is 0 in multiple-channel preview. 
#define NET_TALK_INIT_ERROR                     _EC(14)     // Failed to initialize record library 
#define NET_TALK_NOT_INIT                       _EC(15)     // The record library has not been initialized
#define NET_TALK_SENDDATA_ERROR                 _EC(16)     // Error occurs when sending out audio data 
#define NET_REAL_ALREADY_SAVING                 _EC(17)     // The real-time has been protected.
#define NET_NOT_SAVING                          _EC(18)     // The real-time data has not been save.
#define NET_OPEN_FILE_ERROR                     _EC(19)     // Error occurs when opening the file.
#define NET_PTZ_SET_TIMER_ERROR                 _EC(20)     // Failed to enable PTZ to control timer.
#define NET_RETURN_DATA_ERROR                   _EC(21)     // Error occurs when verify returned data.
#define NET_INSUFFICIENT_BUFFER                 _EC(22)     // There is no sufficient buffer.
#define NET_NOT_SUPPORTED                       _EC(23)     // The current SDK does not support this funcntion.
#define NET_NO_RECORD_FOUND                     _EC(24)     // There is no searched result.
#define NET_NOT_AUTHORIZED                      _EC(25)     // You have no operation right.
#define NET_NOT_NOW                             _EC(26)     // Can not operate right now. 
#define NET_NO_TALK_CHANNEL                     _EC(27)     // There is no audio talk channel.
#define NET_NO_AUDIO                            _EC(28)     // There is no audio.
#define NET_NO_INIT                             _EC(29)     // The network SDK has not been initialized.
#define NET_DOWNLOAD_END                        _EC(30)     // The download completed.
#define NET_EMPTY_LIST                          _EC(31)     // There is no searched result.
#define NET_ERROR_GETCFG_SYSATTR                _EC(32)     // Failed to get system property setup.
#define NET_ERROR_GETCFG_SERIAL                 _EC(33)     // Failed to get SN.
#define NET_ERROR_GETCFG_GENERAL                _EC(34)     // Failed to get general property.
#define NET_ERROR_GETCFG_DSPCAP                 _EC(35)     // Failed to get DSP capacity description.
#define NET_ERROR_GETCFG_NETCFG                 _EC(36)     // Failed to get network channel setup.
#define NET_ERROR_GETCFG_CHANNAME               _EC(37)     // Failed to get channel name.
#define NET_ERROR_GETCFG_VIDEO                  _EC(38)     // Failed to get video property.
#define NET_ERROR_GETCFG_RECORD                 _EC(39)     // Failed to get record setup
#define NET_ERROR_GETCFG_PRONAME                _EC(40)     // Failed to get decoder protocol name 
#define NET_ERROR_GETCFG_FUNCNAME               _EC(41)     // Failed to get 232 COM function name.
#define NET_ERROR_GETCFG_485DECODER             _EC(42)     // Failed to get decoder property
#define NET_ERROR_GETCFG_232COM                 _EC(43)     // Failed to get 232 COM setup
#define NET_ERROR_GETCFG_ALARMIN                _EC(44)     // Failed to get external alarm input setup
#define NET_ERROR_GETCFG_ALARMDET               _EC(45)     // Failed to get motion detection alarm
#define NET_ERROR_GETCFG_SYSTIME                _EC(46)     // Failed to get device time
#define NET_ERROR_GETCFG_PREVIEW                _EC(47)     // Failed to get preview parameter
#define NET_ERROR_GETCFG_AUTOMT                 _EC(48)     // Failed to get audio maintenance setup
#define NET_ERROR_GETCFG_VIDEOMTRX              _EC(49)     // Failed to get video matrix setup
#define NET_ERROR_GETCFG_COVER                  _EC(50)     // Failed to get privacy mask zone setup
#define NET_ERROR_GETCFG_WATERMAKE              _EC(51)     // Failed to get video watermark setup
#define NET_ERROR_GETCFG_MULTICAST              _EC(52)     // Failed to get config, omulticast port by channel
#define NET_ERROR_SETCFG_GENERAL                _EC(55)     // Failed to modify general property
#define NET_ERROR_SETCFG_NETCFG                 _EC(56)     // Failed to modify channel setup
#define NET_ERROR_SETCFG_CHANNAME               _EC(57)     // Failed to modify channel name
#define NET_ERROR_SETCFG_VIDEO                  _EC(58)     // Failed to modify video channel 
#define NET_ERROR_SETCFG_RECORD                 _EC(59)     // Failed to modify record setup 
#define NET_ERROR_SETCFG_485DECODER             _EC(60)     // Failed to modify decoder property 
#define NET_ERROR_SETCFG_232COM                 _EC(61)     // Failed to modify 232 COM setup 
#define NET_ERROR_SETCFG_ALARMIN                _EC(62)     // Failed to modify external input alarm setup
#define NET_ERROR_SETCFG_ALARMDET               _EC(63)     // Failed to modify motion detection alarm setup 
#define NET_ERROR_SETCFG_SYSTIME                _EC(64)     // Failed to modify device time 
#define NET_ERROR_SETCFG_PREVIEW                _EC(65)     // Failed to modify preview parameter
#define NET_ERROR_SETCFG_AUTOMT                 _EC(66)     // Failed to modify auto maintenance setup 
#define NET_ERROR_SETCFG_VIDEOMTRX              _EC(67)     // Failed to modify video matrix setup 
#define NET_ERROR_SETCFG_COVER                  _EC(68)     // Failed to modify privacy mask zone
#define NET_ERROR_SETCFG_WATERMAKE              _EC(69)     // Failed to modify video watermark setup 
#define NET_ERROR_SETCFG_WLAN                   _EC(70)     // Failed to modify wireless network information 
#define NET_ERROR_SETCFG_WLANDEV                _EC(71)     // Failed to select wireless network device
#define NET_ERROR_SETCFG_REGISTER               _EC(72)     // Failed to modify the actively registration parameter setup.
#define NET_ERROR_SETCFG_CAMERA                 _EC(73)     // Failed to modify camera property
#define NET_ERROR_SETCFG_INFRARED               _EC(74)     // Failed to modify IR alarm setup
#define NET_ERROR_SETCFG_SOUNDALARM             _EC(75)     // Failed to modify audio alarm setup
#define NET_ERROR_SETCFG_STORAGE                _EC(76)     // Failed to modify storage position setup
#define NET_AUDIOENCODE_NOTINIT                 _EC(77)     // The audio encode port has not been successfully initialized. 
#define NET_DATA_TOOLONGH                       _EC(78)     // The data are too long.
#define NET_UNSUPPORTED                         _EC(79)     // The device does not support current operation. 
#define NET_DEVICE_BUSY                         _EC(80)     // Device resources is not sufficient.
#define NET_SERVER_STARTED                      _EC(81)     // The server has boot up 
#define NET_SERVER_STOPPED                      _EC(82)     // The server has not fully boot up 
#define NET_LISTER_INCORRECT_SERIAL             _EC(83)     // Input serial number is not correct.
#define NET_QUERY_DISKINFO_FAILED               _EC(84)     // Failed to get HDD information.
#define NET_ERROR_GETCFG_SESSION                _EC(85)     // Failed to get connect session information.
#define NET_USER_FLASEPWD_TRYTIME               _EC(86)     // The password you typed is incorrect. You have exceeded the maximum number of retries.
#define NET_LOGIN_ERROR_PASSWORD                _EC(100)    // Password is not correct
#define NET_LOGIN_ERROR_USER                    _EC(101)    // The account does not exist
#define NET_LOGIN_ERROR_TIMEOUT                 _EC(102)    // Time out for log in returned value.
#define NET_LOGIN_ERROR_RELOGGIN                _EC(103)    // The account has logged in 
#define NET_LOGIN_ERROR_LOCKED                  _EC(104)    // The account has been locked
#define NET_LOGIN_ERROR_BLACKLIST               _EC(105)    // The account has been in the black list
#define NET_LOGIN_ERROR_BUSY                    _EC(106)    // Resources are not sufficient. System is busy now.
#define NET_LOGIN_ERROR_CONNECT                 _EC(107)    // Time out. Please check network and try again.
#define NET_LOGIN_ERROR_NETWORK                 _EC(108)    // Network connection failed.
#define NET_LOGIN_ERROR_SUBCONNECT              _EC(109)    // Successfully logged in the device but can not create video channel. Please check network connection.
#define NET_LOGIN_ERROR_MAXCONNECT              _EC(110)    // exceed the max connect number
#define NET_LOGIN_ERROR_PROTOCOL3_ONLY          _EC(111)    // protocol 3 support
#define NET_LOGIN_ERROR_UKEY_LOST               _EC(112)    // There is no USB or USB info error
#define NET_LOGIN_ERROR_NO_AUTHORIZED           _EC(113)    // Client-end IP address has no right to login
#define NET_LOGIN_ERROR_USER_OR_PASSOWRD        _EC(117)    // user or password error 
#define NET_RENDER_SOUND_ON_ERROR               _EC(120)    // Error occurs when Render library open audio.
#define NET_RENDER_SOUND_OFF_ERROR              _EC(121)    // Error occurs when Render library close audio 
#define NET_RENDER_SET_VOLUME_ERROR             _EC(122)    // Error occurs when Render library control volume
#define NET_RENDER_ADJUST_ERROR                 _EC(123)    // Error occurs when Render library set video parameter
#define NET_RENDER_PAUSE_ERROR                  _EC(124)    // Error occurs when Render library pause play
#define NET_RENDER_SNAP_ERROR                   _EC(125)    // Render library snapshot error
#define NET_RENDER_STEP_ERROR                   _EC(126)    // Render library stepper error
#define NET_RENDER_FRAMERATE_ERROR              _EC(127)    // Error occurs when Render library set frame rate.
#define NET_RENDER_DISPLAYREGION_ERROR          _EC(128)    // Error occurs when Render lib setting show region
#define NET_RENDER_GETOSDTIME_ERROR             _EC(129)    // An error occurred when Render library getting current play time
#define NET_GROUP_EXIST                         _EC(140)    // Group name has been existed.
#define NET_GROUP_NOEXIST                       _EC(141)    // The group name does not exist. 
#define NET_GROUP_RIGHTOVER                     _EC(142)    // The group right exceeds the right list!
#define NET_GROUP_HAVEUSER                      _EC(143)    // The group can not be removed since there is user in it!
#define NET_GROUP_RIGHTUSE                      _EC(144)    // The user has used one of the group right. It can not be removed. 
#define NET_GROUP_SAMENAME                      _EC(145)    // New group name has been existed
#define NET_USER_EXIST                          _EC(146)    // The user name has been existed
#define NET_USER_NOEXIST                        _EC(147)    // The account does not exist.
#define NET_USER_RIGHTOVER                      _EC(148)    // User right exceeds the group right. 
#define NET_USER_PWD                            _EC(149)    // Reserved account. It does not allow to be modified.
#define NET_USER_FLASEPWD                       _EC(150)    // password is not correct
#define NET_USER_NOMATCHING                     _EC(151)    // Password is invalid
#define NET_USER_INUSE                          _EC(152)    // account in use
#define NET_ERROR_GETCFG_ETHERNET               _EC(300)    // Failed to get network card setup.
#define NET_ERROR_GETCFG_WLAN                   _EC(301)    // Failed to get wireless network information.
#define NET_ERROR_GETCFG_WLANDEV                _EC(302)    // Failed to get wireless network device.
#define NET_ERROR_GETCFG_REGISTER               _EC(303)    // Failed to get actively registration parameter.
#define NET_ERROR_GETCFG_CAMERA                 _EC(304)    // Failed to get camera property 
#define NET_ERROR_GETCFG_INFRARED               _EC(305)    // Failed to get IR alarm setup
#define NET_ERROR_GETCFG_SOUNDALARM             _EC(306)    // Failed to get audio alarm setup
#define NET_ERROR_GETCFG_STORAGE                _EC(307)    // Failed to get storage position 
#define NET_ERROR_GETCFG_MAIL                   _EC(308)    // Failed to get mail setup.
#define NET_CONFIG_DEVBUSY                      _EC(309)    // Can not set right now. 
#define NET_CONFIG_DATAILLEGAL                  _EC(310)    // The configuration setup data are illegal.
#define NET_ERROR_GETCFG_DST                    _EC(311)    // Failed to get DST setup
#define NET_ERROR_SETCFG_DST                    _EC(312)    // Failed to set DST 
#define NET_ERROR_GETCFG_VIDEO_OSD              _EC(313)    // Failed to get video OSD setup.
#define NET_ERROR_SETCFG_VIDEO_OSD              _EC(314)    // Failed to set video OSD 
#define NET_ERROR_GETCFG_GPRSCDMA               _EC(315)    // Failed to get CDMA\GPRS configuration
#define NET_ERROR_SETCFG_GPRSCDMA               _EC(316)    // Failed to set CDMA\GPRS configuration
#define NET_ERROR_GETCFG_IPFILTER               _EC(317)    // Failed to get IP Filter configuration
#define NET_ERROR_SETCFG_IPFILTER               _EC(318)    // Failed to set IP Filter configuration
#define NET_ERROR_GETCFG_TALKENCODE             _EC(319)    // Failed to get Talk Encode configuration
#define NET_ERROR_SETCFG_TALKENCODE             _EC(320)    // Failed to set Talk Encode configuration
#define NET_ERROR_GETCFG_RECORDLEN              _EC(321)    // Failed to get The length of the video package configuration
#define NET_ERROR_SETCFG_RECORDLEN              _EC(322)    // Failed to set The length of the video package configuration
#define NET_DONT_SUPPORT_SUBAREA                _EC(323)    // Not support Network hard disk partition
#define NET_ERROR_GET_AUTOREGSERVER             _EC(324)    // Failed to get the register server information
#define NET_ERROR_CONTROL_AUTOREGISTER          _EC(325)    // Failed to control actively registration
#define NET_ERROR_DISCONNECT_AUTOREGISTER       _EC(326)    // Failed to disconnect actively registration
#define NET_ERROR_GETCFG_MMS                    _EC(327)    // Failed to get mms configuration
#define NET_ERROR_SETCFG_MMS                    _EC(328)    // Failed to set mms configuration
#define NET_ERROR_GETCFG_SMSACTIVATION          _EC(329)    // Failed to get SMS configuration
#define NET_ERROR_SETCFG_SMSACTIVATION          _EC(330)    // Failed to set SMS configuration
#define NET_ERROR_GETCFG_DIALINACTIVATION       _EC(331)    // Failed to get activation of a wireless connection
#define NET_ERROR_SETCFG_DIALINACTIVATION       _EC(332)    // Failed to set activation of a wireless connection
#define NET_ERROR_GETCFG_VIDEOOUT               _EC(333)    // Failed to get the parameter of video output
#define NET_ERROR_SETCFG_VIDEOOUT               _EC(334)    // Failed to set the configuration of video output
#define NET_ERROR_GETCFG_OSDENABLE              _EC(335)    // Failed to get osd overlay enabling
#define NET_ERROR_SETCFG_OSDENABLE              _EC(336)    // Failed to set OSD overlay enabling
#define NET_ERROR_SETCFG_ENCODERINFO            _EC(337)    // Failed to set digital input configuration of front encoders
#define NET_ERROR_GETCFG_TVADJUST               _EC(338)    // Failed to get TV adjust configuration
#define NET_ERROR_SETCFG_TVADJUST               _EC(339)    // Failed to set TV adjust configuration
#define NET_ERROR_CONNECT_FAILED                _EC(340)    // Failed to request to establish a connection
#define NET_ERROR_SETCFG_BURNFILE               _EC(341)    // Failed to request to upload burn files
#define NET_ERROR_SNIFFER_GETCFG                _EC(342)    // Failed to get capture configuration information
#define NET_ERROR_SNIFFER_SETCFG                _EC(343)    // Failed to set capture configuration information
#define NET_ERROR_DOWNLOADRATE_GETCFG           _EC(344)    // Failed to get download restrictions information
#define NET_ERROR_DOWNLOADRATE_SETCFG           _EC(345)    // Failed to set download restrictions information
#define NET_ERROR_SEARCH_TRANSCOM               _EC(346)    // Failed to query serial port parameters
#define NET_ERROR_GETCFG_POINT                  _EC(347)    // Failed to get the preset info
#define NET_ERROR_SETCFG_POINT                  _EC(348)    // Failed to set the preset info
#define NET_SDK_LOGOUT_ERROR                    _EC(349)    // SDK log out the device abnormally
#define NET_ERROR_GET_VEHICLE_CFG               _EC(350)    // Failed to get vehicle configuration
#define NET_ERROR_SET_VEHICLE_CFG               _EC(351)    // Failed to set vehicle configuration
#define NET_ERROR_GET_ATM_OVERLAY_CFG           _EC(352)    // Failed to get ATM overlay configuration
#define NET_ERROR_SET_ATM_OVERLAY_CFG           _EC(353)    // Failed to set ATM overlay configuration
#define NET_ERROR_GET_ATM_OVERLAY_ABILITY       _EC(354)    // Failed to get ATM overlay ability
#define NET_ERROR_GET_DECODER_TOUR_CFG          _EC(355)    // Failed to get decoder tour configuration
#define NET_ERROR_SET_DECODER_TOUR_CFG          _EC(356)    // Failed to set decoder tour configuration
#define NET_ERROR_CTRL_DECODER_TOUR             _EC(357)    // Failed to control decoder tour
#define NET_GROUP_OVERSUPPORTNUM                _EC(358)    // Beyond the device supports for the largest number of user groups
#define NET_USER_OVERSUPPORTNUM                 _EC(359)    // Beyond the device supports for the largest number of users 
#define NET_ERROR_GET_SIP_CFG                   _EC(368)    // Failed to get SIP configuration
#define NET_ERROR_SET_SIP_CFG                   _EC(369)    // Failed to set SIP configuration
#define NET_ERROR_GET_SIP_ABILITY               _EC(370)    // Failed to get SIP capability
#define NET_ERROR_GET_WIFI_AP_CFG               _EC(371)    // Failed to get "WIFI ap' configuration 
#define NET_ERROR_SET_WIFI_AP_CFG               _EC(372)    // Failed to set "WIFI ap" configuration  
#define NET_ERROR_GET_DECODE_POLICY             _EC(373)    // Failed to get decode policy 
#define NET_ERROR_SET_DECODE_POLICY             _EC(374)    // Failed to set decode policy 
#define NET_ERROR_TALK_REJECT                   _EC(375)    // refuse talk
#define NET_ERROR_TALK_OPENED                   _EC(376)    // talk has opened by other client
#define NET_ERROR_TALK_RESOURCE_CONFLICIT       _EC(377)    // resource conflict
#define NET_ERROR_TALK_UNSUPPORTED_ENCODE       _EC(378)    // unsupported encode type
#define NET_ERROR_TALK_RIGHTLESS                _EC(379)    // no right
#define NET_ERROR_TALK_FAILED                   _EC(380)    // request failed
#define NET_ERROR_GET_MACHINE_CFG               _EC(381)    // Failed to get device relative config
#define NET_ERROR_SET_MACHINE_CFG               _EC(382)    // Failed to set device relative config
#define NET_ERROR_GET_DATA_FAILED               _EC(383)    // get data failed
#define NET_ERROR_MAC_VALIDATE_FAILED           _EC(384)    // MAC validate failed
#define NET_ERROR_GET_INSTANCE                  _EC(385)    // Failed to get server instance 
#define NET_ERROR_JSON_REQUEST                  _EC(386)    // Generated json string is error
#define NET_ERROR_JSON_RESPONSE                 _EC(387)    // The responding json string is error
#define NET_ERROR_VERSION_HIGHER                _EC(388)    // The protocol version is lower than current version
#define NET_SPARE_NO_CAPACITY                   _EC(389)    // Hotspare disk operation failed. The capacity is low
#define NET_ERROR_SOURCE_IN_USE                 _EC(390)    // Display source is used by other output
#define NET_ERROR_REAVE                         _EC(391)    // advanced users grab low-level user resource
#define NET_ERROR_NETFORBID                     _EC(392)    // net forbid
#define NET_ERROR_GETCFG_MACFILTER              _EC(393)    // get MAC filter configuration error
#define NET_ERROR_SETCFG_MACFILTER              _EC(394)    // set MAC filter configuration error
#define NET_ERROR_GETCFG_IPMACFILTER            _EC(395)    // get IP/MAC filter configuration error
#define NET_ERROR_SETCFG_IPMACFILTER            _EC(396)    // set IP/MAC filter configuration error
#define NET_ERROR_OPERATION_OVERTIME            _EC(397)    // operation over time 
#define NET_ERROR_SENIOR_VALIDATE_FAILED        _EC(398)    // senior validation failure
#define NET_ERROR_DEVICE_ID_NOT_EXIST           _EC(399)    // device ID is not exist
#define NET_ERROR_UNSUPPORTED                   _EC(400)    // unsupport operation
#define NET_ERROR_PROXY_DLLLOAD                 _EC(401)    // proxy dll load error
#define NET_ERROR_PROXY_ILLEGAL_PARAM           _EC(402)    // proxy user parameter is not legal
#define NET_ERROR_PROXY_INVALID_HANDLE          _EC(403)    // handle invalid
#define NET_ERROR_PROXY_LOGIN_DEVICE_ERROR      _EC(404)    // login device error
#define NET_ERROR_PROXY_START_SERVER_ERROR      _EC(405)    // start proxy server error
#define NET_ERROR_SPEAK_FAILED                  _EC(406)    // request speak failed
#define NET_ERROR_NOT_SUPPORT_F6                _EC(407)    // unsupport F6
#define NET_ERROR_CD_UNREADY                    _EC(408)    // CD is not ready
#define NET_ERROR_DIR_NOT_EXIST                 _EC(409)    // Directory does not exist
#define NET_ERROR_UNSUPPORTED_SPLIT_MODE        _EC(410)    // The device does not support the segmentation model
#define NET_ERROR_OPEN_WND_PARAM                _EC(411)    // Open the window parameter is illegal
#define NET_ERROR_LIMITED_WND_COUNT             _EC(412)    // Open the window more than limit
#define NET_ERROR_UNMATCHED_REQUEST             _EC(413)    // Request command with the current pattern don't match
#define NET_RENDER_ENABLELARGEPICADJUSTMENT_ERROR  _EC(414) // Render Library to enable high-definition image internal adjustment strategy error
#define NET_ERROR_UPGRADE_FAILED                _EC(415)    // Upgrade equipment failure
#define NET_ERROR_NO_TARGET_DEVICE              _EC(416)    // Can't find the target device
#define NET_ERROR_NO_VERIFY_DEVICE              _EC(417)    // Can't find the verify device 
#define NET_ERROR_CASCADE_RIGHTLESS             _EC(418)    // No cascade permissions
#define NET_ERROR_LOW_PRIORITY                  _EC(419)    // low priority
#define NET_ERROR_REMOTE_REQUEST_TIMEOUT        _EC(420)    // The remote device request timeout
#define NET_ERROR_LIMITED_INPUT_SOURCE          _EC(421)    // Input source beyond maximum route restrictions
#define NET_ERROR_SET_LOG_PRINT_INFO            _EC(422)    // Failed to set log print
#define NET_ERROR_PARAM_DWSIZE_ERROR            _EC(423)    // "dwSize" is not initialized in input param
#define NET_ERROR_LIMITED_MONITORWALL_COUNT     _EC(424)    // TV wall exceed limit
#define NET_ERROR_PART_PROCESS_FAILED           _EC(425)    // Fail to execute part of the process
#define NET_ERROR_TARGET_NOT_SUPPORT            _EC(426)    // Fail to transmit due to not supported by target
#define NET_ERROR_VISITE_FILE                   _EC(510)    // Access to the file failed
#define NET_ERROR_DEVICE_STATUS_BUSY            _EC(511)    // Device busy
#define NET_USER_PWD_NOT_AUTHORIZED             _EC(512)    // Fail to change the password
#define NET_USER_PWD_NOT_STRONG                 _EC(513)    // Password strength is not enough
#define NET_ERROR_NO_SUCH_CONFIG                _EC(514)    // No corresponding setup
#define NET_ERROR_AUDIO_RECORD_FAILED           _EC(515)    // Failed to record audio
#define NET_ERROR_SEND_DATA_FAILED              _EC(516)    // Failed to send out data 
#define NET_ERROR_OBSOLESCENT_INTERFACE         _EC(517)    // Abandoned port 
#define NET_ERROR_INSUFFICIENT_INTERAL_BUF      _EC(518)    // Internal buffer is not sufficient 
#define NET_ERROR_NEED_ENCRYPTION_PASSWORD      _EC(519)    // verify password when changing device IP
#define NET_ERROR_NOSUPPORT_RECORD              _EC(520)    // device not support the record
#define NET_ERROR_SERIALIZE_ERROR               _EC(1010)   // Failed to serialize data
#define NET_ERROR_DESERIALIZE_ERROR             _EC(1011)   // Failed to deserialize data
#define NET_ERROR_LOWRATEWPAN_ID_EXISTED        _EC(1012)   // the wireless id is already existed
#define NET_ERROR_LOWRATEWPAN_ID_LIMIT          _EC(1013)   // the wireless id limited
#define NET_ERROR_LOWRATEWPAN_ID_ABNORMAL       _EC(1014)   // add the wireless id abnormaly
#define NET_ERROR_ENCRYPT                       _EC(1015)    // encrypt data fail
#define NET_ERROR_PWD_ILLEGAL                   _EC(1016)    // new password illegal
#define NET_ERROR_DEVICE_ALREADY_INIT           _EC(1017)    // device is already init
#define NET_ERROR_SECURITY_CODE                 _EC(1018)    // security code check out fail
#define NET_ERROR_SECURITY_CODE_TIMEOUT         _EC(1019)    // security code out of time
#define NET_ERROR_GET_PWD_SPECI                 _EC(1020)    // get passwd specification fail
#define NET_ERROR_NO_AUTHORITY_OF_OPERATION     _EC(1021)    // no authority of operation 
#define NET_ERROR_DECRYPT                       _EC(1022)    // decrypt data fail
#define NET_ERROR_2D_CODE                       _EC(1023)    // 2D code check out fail
#define NET_ERROR_INVALID_REQUEST               _EC(1024)    // invalid request
#define	NET_ERROR_PWD_RESET_DISABLE			    _EC(1025)	 // pwd reset disabled
#define NET_ERROR_PLAY_PRIVATE_DATA             _EC(1026)    // failed to display private data,such as rule box
#define NET_ERROR_ROBOT_OPERATE_FAILED          _EC(1027)    // robot operate failed

/************************************************************************
 ** Enumeration Definition
 ***********************************************************************/


//Card information types CLIENT_QueryNetStat interface using
typedef enum tagEM_NET_QUERY_TYPE
{
	NET_APP_DATA_STAT , // Statistics for protocol stack,(input struct NET_IN_NETAPP_NET_DATA_STAT,output struct NET_OUT_NETAPP_NET_DATA_STAT)
	NET_APP_LINK_STAT , // Access to physical link state,(input struct NET_IN_NETAPP_LINK_STATUS,output struct NET_OUT_NETAPP_LINK_STATUS)
}EM_NET_QUERY_TYPE;

// Catch a figure type CLIENT_CapturePictureEx interface using
typedef enum tagNET_CAPTURE_FORMATS
{
	NET_CAPTURE_BMP,
	NET_CAPTURE_JPEG, // 100% quality JPEG
	NET_CAPTURE_JPEG_70, // 70% quality JPEG
	NET_CAPTURE_JPEG_50,
	NET_CAPTURE_JPEG_30,
}NET_CAPTURE_FORMATS;

// Device type
typedef enum tagNET_DEVICE_TYPE 
{
	NET_PRODUCT_NONE = 0,
	NET_DVR_NONREALTIME_MACE,					// Non real-time MACE
	NET_DVR_NONREALTIME,						// Non real-time
	NET_NVS_MPEG1,								// Network video server
	NET_DVR_MPEG1_2,							// MPEG1 2-ch DVR
	NET_DVR_MPEG1_8,							// MPEG1 8-ch DVR
	NET_DVR_MPEG4_8,							// MPEG4 8-ch DVR
	NET_DVR_MPEG4_16,							// MPEG4 16-ch DVR
	NET_DVR_MPEG4_SX2,							// LB series DVR
	NET_DVR_MEPG4_ST2,							// GB  series DVR
	NET_DVR_MEPG4_SH2,							// HB  series DVR
	NET_DVR_MPEG4_GBE,							// GBE  series DVR
	NET_DVR_MPEG4_NVSII,						// II network video server
	NET_DVR_STD_NEW,							// New standard configuration protocol
	NET_DVR_DDNS,								// DDNS server
	NET_DVR_ATM,								// ATM series 
	NET_NB_SERIAL,								// 2nd non real-time NB series DVR
	NET_LN_SERIAL,								// LN  series 
	NET_BAV_SERIAL,								// BAV series
	NET_SDIP_SERIAL,							// SDIP series
	NET_IPC_SERIAL,								// IPC series
	NET_NVS_B,									// NVS B series
	NET_NVS_C,									// NVS H series 
	NET_NVS_S,									// NVS S series
	NET_NVS_E,									// NVS E series
	NET_DVR_NEW_PROTOCOL,						// Search device type from QueryDevState. it is in string format
	NET_NVD_SERIAL,								// NVD
	NET_DVR_N5,									// N5
	NET_DVR_MIX_DVR,							// HDVR
	NET_SVR_SERIAL,								// SVR series
	NET_SVR_BS,									// SVR-BS
	NET_NVR_SERIAL,								// NVR series
	NET_DVR_N51,                                // N51
	NET_ITSE_SERIAL,							// ITSE Intelligent Analysis Box
	NET_ITC_SERIAL,                             // Intelligent traffic camera equipment
	NET_HWS_SERIAL,                             // radar speedometer HWS
	NET_PVR_SERIAL,                             // portable video record
	NET_IVS_SERIAL,                             // IVS(intelligent video server series)
	NET_IVS_B,                                  // universal intelligent detect video server series 
	NET_IVS_F,                                  // face recognisation server
	NET_IVS_V,                                  // video quality diagnosis server
	NET_MATRIX_SERIAL,							// matrix
	NET_DVR_N52,								// N52
	NET_DVR_N56,								// N56
	NET_ESS_SERIAL,                             // ESS
	NET_IVS_PC,                                 // number statistic server
	NET_PC_NVR,                                 // pc-nvr
	NET_DSCON,									// screen controller
	NET_EVS,									// network video storage server
	NET_EIVS,									// an embedded intelligent video analysis system
	NET_DVR_N6,                                 // DVR-N6
	NET_UDS,                                    // K-Lite Codec Pack
	NET_AF6016,									// Bank alarm host
	NET_AS5008,									// Video network alarm host
	NET_AH2008,									// Network alarm host
	NET_A_SERIAL,								// Alarm host series
	NET_BSC_SERIAL,								// Access control series of products
    NET_NVS_SERIAL,                             // NVS series product
    NET_VTO_SERIAL,                             // VTO series product
    NET_VTNC_SERIAL,                            // VTNC series product
    NET_TPC_SERIAL,               				// TPC series product, it is the thermal device 
    NET_ASM_SERIAL,                             // ASM series product
    NET_VTS_SERIAL,                             // VTS series product
    NET_ARC2016C,                               // Alarm host-ARC2016C    
    NET_ASA,                                    // ASA Attendance machine
	NET_VTT_SERIAL,								// Industry terminal walkie-talkie
	NET_VTA_SERIAL,								// Alarm column
	NET_VTNS_SERIAL,							// SIP Server
	NET_VTH_SERIAL,								// Indoor unit
}NET_DEVICE_TYPE ;

// Language type
typedef enum __LANGUAGE_TYPE
{
	DH_LANGUAGE_ENGLISH,						// English 
	DH_LANGUAGE_CHINESE_SIMPLIFIED,				// Simplified Chinese
	DH_LANGUAGE_CHINESE_TRADITIONAL,			// Traditional Chinese
	DH_LANGUAGE_ITALIAN,						// Italian 
	DH_LANGUAGE_SPANISH,						// Spanish
	DH_LANGUAGE_JAPANESE,						// Japanese
	DH_LANGUAGE_RUSSIAN,						// Russian 
	DH_LANGUAGE_FRENCH,							// French
	DH_LANGUAGE_GERMAN, 						// German
	DH_LANGUAGE_PORTUGUESE,						// Portuguese
	DH_LANGUAGE_TURKEY,							// Turkey	
	DH_LANGUAGE_POLISH,							// Polish
	DH_LANGUAGE_ROMANIAN,						// Romanian 
	DH_LANGUAGE_HUNGARIAN,						// Hungarian
	DH_LANGUAGE_FINNISH,						// Finnish
	DH_LANGUAGE_ESTONIAN,						// Estonian	
	DH_LANGUAGE_KOREAN,							// Korean
	DH_LANGUAGE_FARSI,							// Farsi	
	DH_LANGUAGE_DANSK,							// Denmark
	DH_LANGUAGE_CZECHISH,						// Czechish
	DH_LANGUAGE_BULGARIA,						// Bulgaria
	DH_LANGUAGE_SLOVAKIAN,						// Slovakian
	DH_LANGUAGE_SLOVENIA,						// Slovenia
	DH_LANGUAGE_CROATIAN,						// Croatian
	DH_LANGUAGE_DUTCH,							// Dutch
	DH_LANGUAGE_GREEK,							// Greek
	DH_LANGUAGE_UKRAINIAN,						// Ukrainian
	DH_LANGUAGE_SWEDISH,						// Swedish
	DH_LANGUAGE_SERBIAN,						// Serbian
	DH_LANGUAGE_VIETNAMESE,						// Vietnamese
	DH_LANGUAGE_LITHUANIAN,						// Lithuanian
	DH_LANGUAGE_FILIPINO,						// Filipino
	DH_LANGUAGE_ARABIC,							// Arabic
	DH_LANGUAGE_CATALAN,						// Catalan
	DH_LANGUAGE_LATVIAN,						// Latvian
	DH_LANGUAGE_THAI,							// Thai
	DH_LANGUAGE_HEBREW,							// Hebrew
	DH_LANGUAGE_Bosnian,						// Bosnian
} DH_LANGUAGE_TYPE;

// Upgrade type
typedef enum __EM_UPGRADE_TYPE
{
	DH_UPGRADE_BIOS_TYPE = 1,					// BIOS  upgrade
	DH_UPGRADE_WEB_TYPE,						// WEB upgrade
	DH_UPGRADE_BOOT_YPE,						// BOOT upgrade
	DH_UPGRADE_CHARACTER_TYPE,					// Chinese character library
	DH_UPGRADE_LOGO_TYPE,						// LOGO
	DH_UPGRADE_EXE_TYPE,						// EXE such as player
    DH_UPGRADE_DEVCONSTINFO_TYPE,               // upgrade device information
    DH_UPGRADE_PERIPHERAL_TYPE,                 // Peripheral access from (such as car 287 chip)
	DH_UPGRADE_GEOINFO_TYPE,                    // Geographic information positioning chip
	DH_UPGRADE_MENU,                            // Menu (equipment operation interface of the picture)
	DH_UPGRADE_ROUTE,                           // Line files (such as bus lines)
	DH_UPGRADE_ROUTE_STATE_AUTO,                // Stops the audio (with line stops audio)
	DH_UPGRADE_SCREEN,                          // Scheduling screen (e.g., bus operation panel)
} EM_UPGRADE_TYPE;

// Record related (schedule,motion detection,alarm)
typedef enum __REC_TYPE
{
	DH_REC_TYPE_TIM = 0,
	DH_REC_TYPE_MTD,
	DH_REC_TYPE_ALM,
	DH_REC_TYPE_NUM,
} REC_TYPE;

// network type  
typedef enum __GPRSCDMA_NETWORK_TYPE
{
	DH_TYPE_AUTOSEL = 0,						// Automatic selection
	DH_TYPE_TD_SCDMA,							// TD-SCDMA network 
	DH_TYPE_WCDMA,								// WCDMA network
	DH_TYPE_CDMA_1x,							// CDMA 1.x network
	DH_TYPE_EDGE,								// GPRS network
	DH_TYPE_EVDO,								// EVDO network
	DH_TYPE_WIFI,
} EM_GPRSCDMA_NETWORK_TYPE;

// Interface type,responding to the interface CLIENT_SetSubconnCallBack
typedef enum __EM_INTERFACE_TYPE
{
	DH_INTERFACE_OTHER = 0x00000000,			// Unknown interface
	DH_INTERFACE_REALPLAY,						// Realtime monitoring interface
	DH_INTERFACE_PREVIEW,						// Realtime multiple-window preview
	DH_INTERFACE_PLAYBACK,						// Playback interface
	DH_INTERFACE_DOWNLOAD,						// Download interface
	DH_INTERFACE_REALLOADPIC,                   // Download picture interface
} EM_INTERFACE_TYPE;

// realplay disconnect event
typedef enum _EM_REALPLAY_DISCONNECT_EVENT_TYPE
{
	DISCONNECT_EVENT_REAVE,                     // resources is taked by advanced user
	DISCONNECT_EVENT_NETFORBID,                 // forbidden
	DISCONNECT_EVENT_SUBCONNECT,                // sublink disconnect
}EM_REALPLAY_DISCONNECT_EVENT_TYPE;

// event file's tag type
typedef enum __EM_EVENT_FILETAG
{
	DH_ATMBEFOREPASTE = 1,                      // Before ATM Paste
	DH_ATMAFTERPASTE,                           // After ATM Paste
}EM_EVENT_FILETAG;

// IPC type
typedef enum __EM_IPC_TYPE
{
	DH_IPC_PRIVATE,                             // private
	DH_IPC_AEBELL,                              // AEBell
	DH_IPC_PANASONIC,                           // panasonic
	DH_IPC_SONY,                                // sony
	DH_IPC_DYNACOLOR,                           // Dynacolor
	DH_IPC_TCWS = 5 ,                           // TCWS	
	DH_IPC_SAMSUNG,                             // Samsung
	DH_IPC_YOKO,                                // YOKO
	DH_IPC_AXIS,                                // AXIS
	DH_IPC_SANYO,							    // sanyo       
	DH_IPC_BOSH = 10,							// Bosch
	DH_IPC_PECLO,								// PECLO
	DH_IPC_PROVIDEO,							// Provideo
	DH_IPC_ACTI,								// ACTi
	DH_IPC_VIVOTEK,								// Vivotek
	DH_IPC_ARECONT = 15,                        // Arecont
	DH_IPC_PRIVATEEH,			                // PrivateEH	
	DH_IPC_IMATEK,					            // IMatek
	DH_IPC_SHANY,                               // Shany
	DH_IPC_VIDEOTREC,                           // videorec
	DH_IPC_URA = 20,                            // Ura
	DH_IPC_BITICINO,                            // Bticino 
	DH_IPC_ONVIF,                               // Onvif protocol type
	DH_IPC_SHEPHERD,                            // Shepherd
	DH_IPC_YAAN,                                // Yaan
	DH_IPC_AIRPOINT = 25,                       // Airpoint
	DH_IPC_TYCO,                                // TYCO
	DH_IPC_XUNMEI,								// Xunmei
	DH_IPC_HIKVISION,							// HIKVISION
	DH_IPC_LG,                                  // LG
	DH_IPC_AOQIMAN = 30,                        // Aoqiman
	DH_IPC_BAOKANG,                             // baokang  
	DH_IPC_WATCHNET,                            // Watchnet
	DH_IPC_XVISION,                             // Xvision
	DH_IPC_FUSITSU,                             // Fisitu
	DH_IPC_CANON = 35,							// Canon
	DH_IPC_GE,								    // GE
	DH_IPC_Basler,								// Basler
	DH_IPC_Patro,								// Patro
	DH_IPC_CPKNC,								// CPPLUS K series
	DH_IPC_CPRNC = 40,							// CPPLUS R series
	DH_IPC_CPUNC,								// CPPLUS U series
	DH_IPC_CPPLUS,								// cpplus oem 
	DH_IPC_XunmeiS,								// XunmeiS
	DH_IPC_GDDW,								// guangdong power grid
	DH_IPC_PSIA = 45,                           // PSIA
	DH_IPC_GB2818,                              // GB2818	
	DH_IPC_GDYX,                                // GDYX
	DH_IPC_OTHER,                               // custom
	DH_IPC_CPUNR,								// CPPLUS NVR
	DH_IPC_CPUAR = 50,							// CPPLUS DVR
	DH_IPC_AIRLIVE,                             // Airlive	
	DH_IPC_NPE,									// NPE	
	DH_IPC_AXVIEW,								// AXVIEW
    DH_IPC_DFWL,                                // DFWL
	DH_IPC_HYUNDAI = 56,						// HYUNDAI DVR
	DH_IPC_APHD,								// APHD
	DH_IPC_WELLTRANS ,							// WELLTRANS
	DH_IPC_CDJF,								// CDJF
	DH_IPC_JVC = 60,						    // JVC
	DH_IPC_INFINOVA,							// INFINOVA
	DH_IPC_ADT,									// ADT
	DH_IPC_SIVIDI,								// SIVIDI
	DH_IPC_CPUNP,								// CPPLUS PTZ
	DH_IPC_HX = 65,								// HX
	DH_IPC_TJGS,                                // TJGS
    DH_IPC_MULTICAST = 79,                      // Multicast
    DH_IPC_RVI = 84,							// RVi  
}EM_IPC_TYPE;

// H264 profile rank
typedef enum __EM_H264_PROFILE_RANK
{
    	DH_PROFILE_BASELINE = 1,                 // offer I/P frame, only support progressive and CAVLC
		DH_PROFILE_MAIN,                         // offer I/P/B frame, support progressiv and interlaced,offer CAVLC and CABAC
		DH_PROFILE_EXTENDED,                     // offer I/P/B/SP/SI frame,only support progressive and CAVLC
		DH_PROFILE_HIGH,                         // FRExt,base on Main_Profile:8x8 intra prediction, custom quant, lossless video coding, more yuv
}EM_H264_PROFILE_RANK;

typedef enum __EM_DISK_TYPE
{
    DH_DISK_READ_WRITE,                          // read write disk
		DH_DISK_READ_ONLY,                       // read only disk
		DH_DISK_BACKUP,	                         // back up disk or media disk
		DH_DISK_REDUNDANT,                       // redundancy disk
		DH_DISK_SNAPSHOT,	                     // snapshot disk
}EM_DISK_TYPE;

// stream encrypt algorithm work mode
typedef enum  __EM_ENCRYPT_ALOG_WORKMODE
{
		ENCRYPT_ALOG_WORKMODE_ECB,				// ECB mode
		ENCRYPT_ALOG_WORKMODE_CBC,				// CBC mode
		ENCRYPT_ALOG_WORKMODE_CFB,				// CFB mode
		ENCRYPT_ALOG_WORKMODE_OFB,				// OFB mode
}EM_ENCRYPT_ALOG_WORKMODE;

typedef enum __EM_MOBILE_PPP_STATE
{
	MOBILE_PPP_UP = 0,							// connect has being build
	MOBILE_PPP_DOWN,							// ppp connect has being cut		
	MOBILE_PPP_CONNECTING,						// be connecting		
	MOBILE_PPP_CLOSEING,						// be closing PPP connect
} EM_MOBILE_PPP_STATE;

typedef enum __EM_3GMOBILE_STATE
{
    MOBILE_MODULE_OFF,							// means 3g net card mod is closed   		
	MOBILE_MODULE_STARTING,						// means 3g net card mod is starting
	MOBILE_MODULE_WORKING,						// means 3g net card mod is working
}EM_3GMOBILE_STATE;

typedef enum tagEM_LOGIN_SPAC_CAP_TYPE
{
    EM_LOGIN_SPEC_CAP_TCP               = 0,    // TCP login, default
    EM_LOGIN_SPEC_CAP_ANY               = 1,    // No criteria login
    EM_LOGIN_SPEC_CAP_SERVER_CONN       = 2,    // auto sign up login
    EM_LOGIN_SPEC_CAP_MULTICAST         = 3,    // multicast login, default
    EM_LOGIN_SPEC_CAP_UDP               = 4,    // UDP method login
    EM_LOGIN_SPEC_CAP_MAIN_CONN_ONLY    = 6,    // only main connection login
    EM_LOGIN_SPEC_CAP_SSL               = 7,    // SSL encryption login

    EM_LOGIN_SPEC_CAP_INTELLIGENT_BOX   = 9,    // login IVS box remote device
    EM_LOGIN_SPEC_CAP_NO_CONFIG         = 10,   // login device do not config
    EM_LOGIN_SPEC_CAP_U_LOGIN           = 11,   // USB key device login
    EM_LOGIN_SPEC_CAP_LDAP              = 12,   // LDAP login
    EM_LOGIN_SPEC_CAP_AD                = 13,   // AD, ActiveDirectory,  login
    EM_LOGIN_SPEC_CAP_RADIUS            = 14,   // Radius  login 
    EM_LOGIN_SPEC_CAP_SOCKET_5          = 15,   // Socks5 login
    EM_LOGIN_SPEC_CAP_CLOUD             = 16,   // cloud login
    EM_LOGIN_SPEC_CAP_AUTH_TWICE        = 17,   // dual authentication loin
    EM_LOGIN_SPEC_CAP_TS                = 18,   // TS stream client login
    EM_LOGIN_SPEC_CAP_P2P               = 19,   // web private login
    EM_LOGIN_SPEC_CAP_MOBILE            = 20,   // mobile client login
    EM_LOGIN_SPEC_CAP_INVALID                   // invalid login


}EM_LOGIN_SPAC_CAP_TYPE;

/////////////////////////////////Monitor related/////////////////////////////////

// Preview type.Corresponding to CLIENT_RealPlayEx
typedef enum _RealPlayType
{
	DH_RType_Realplay = 0,						// Real-time preview
	DH_RType_Multiplay,							// Multiple-channel preview 
	DH_RType_Realplay_0,						// Real-time monitor-main stream. It is the same as DH_RType_Realplay
	DH_RType_Realplay_1,						// 1 Real-time monitor -- extra stream 1
	DH_RType_Realplay_2,						// 2 Real-time monitor -- extra stream 2
	DH_RType_Realplay_3,						// 3 Real-time monitor -- extra stream 3
	DH_RType_Multiplay_1,						// Multiple-channel preview--1-window 
	DH_RType_Multiplay_4,						// Multiple-channel preview--4-window
	DH_RType_Multiplay_8,						// Multiple-channel preview--8-window
	DH_RType_Multiplay_9,						// Multiple-channel preview--9-window
	DH_RType_Multiplay_16,						// Multiple-channel preview--16-window
	DH_RType_Multiplay_6,						// Multiple-channel preview--6-window
	DH_RType_Multiplay_12,						// Multiple-channel preview--12-window
    DH_RType_Multiplay_25,                      // Multi-window tour--25-windows
    DH_RType_Multiplay_36,                      // Multi-window preview--36-windows
    DH_RType_Realplay_Test = 255,               // test stream
} DH_RealPlayType;

/////////////////////////////////About PTZ/////////////////////////////////

// General PTZ control command
typedef enum _PTZ_ControlType
{
	DH_PTZ_UP_CONTROL = 0,						// Up
	DH_PTZ_DOWN_CONTROL,						// Down
	DH_PTZ_LEFT_CONTROL,						// Left
	DH_PTZ_RIGHT_CONTROL,						// Right
	DH_PTZ_ZOOM_ADD_CONTROL,					// + Zoom in 
	DH_PTZ_ZOOM_DEC_CONTROL,					// - Zoom out 
	DH_PTZ_FOCUS_ADD_CONTROL,					// + Zoom in 
	DH_PTZ_FOCUS_DEC_CONTROL,					// - Zoom out 
	DH_PTZ_APERTURE_ADD_CONTROL,				// + Aperture 
	DH_PTZ_APERTURE_DEC_CONTROL,				// - Aperture
    DH_PTZ_POINT_MOVE_CONTROL,					// Go to preset 
    DH_PTZ_POINT_SET_CONTROL,					// Set 
    DH_PTZ_POINT_DEL_CONTROL,					// Delete
    DH_PTZ_POINT_LOOP_CONTROL,					// Tour 
    DH_PTZ_LAMP_CONTROL							// Light and wiper 
} DH_PTZ_ControlType;

// PTZ control extensive command 
typedef enum _EXTPTZ_ControlType
{
	DH_EXTPTZ_LEFTTOP = 0x20,					// Upper left
	DH_EXTPTZ_RIGHTTOP,							// Upper right 
	DH_EXTPTZ_LEFTDOWN,							// Down left
	DH_EXTPTZ_RIGHTDOWN,						// Down right 
	DH_EXTPTZ_ADDTOLOOP,						// Add preset to tour		tour	 preset value
	DH_EXTPTZ_DELFROMLOOP,						// Delete preset in tour	tour	 preset value
    DH_EXTPTZ_CLOSELOOP,						// Delete tour				tour		
	DH_EXTPTZ_STARTPANCRUISE,					// Begin pan rotation		
	DH_EXTPTZ_STOPPANCRUISE,					// Stop pan rotation		
	DH_EXTPTZ_SETLEFTBORDER,					// Set left limit		
	DH_EXTPTZ_SETRIGHTBORDER,					// Set right limit	
	DH_EXTPTZ_STARTLINESCAN,					// Begin scanning			
    DH_EXTPTZ_CLOSELINESCAN,					// Stop scanning		
    DH_EXTPTZ_SETMODESTART,						// Start mode	mode line		
    DH_EXTPTZ_SETMODESTOP,						// Stop mode	mode line		
	DH_EXTPTZ_RUNMODE,							// Enable mode	Mode line		
	DH_EXTPTZ_STOPMODE,							// Disable mode	Mode line	
	DH_EXTPTZ_DELETEMODE,						// Delete mode	Mode line
	DH_EXTPTZ_REVERSECOMM,						// Flip
	DH_EXTPTZ_FASTGOTO,							// 3D position	X address(8192)	Y address(8192)	zoom(4)
	DH_EXTPTZ_AUXIOPEN,							// auxiliary open	Auxiliary point	
	DH_EXTPTZ_AUXICLOSE,						// Auxiliary close	Auxiliary point
	DH_EXTPTZ_OPENMENU = 0x36,					// Open dome menu 
	DH_EXTPTZ_CLOSEMENU,						// Close menu 
	DH_EXTPTZ_MENUOK,							// Confirm menu 
	DH_EXTPTZ_MENUCANCEL,						// Cancel menu 
	DH_EXTPTZ_MENUUP,							// menu up 
	DH_EXTPTZ_MENUDOWN,							// menu down
	DH_EXTPTZ_MENULEFT,							// menu left
	DH_EXTPTZ_MENURIGHT,						// Menu right 
	DH_EXTPTZ_ALARMHANDLE = 0x40,				// Alarm activate PTZ param1:Alarm input channel; param2:Alarm activation type  1-preset 2-scan 3-tour; param 3:activation value,such as preset value.
	DH_EXTPTZ_MATRIXSWITCH = 0x41,				// Matrix switch param1:monitor number(video output number);param2:video input number;param3:matrix number 
	DH_EXTPTZ_LIGHTCONTROL,						// Light controller
	DH_EXTPTZ_EXACTGOTO,						// 3D accurately positioning param1:Pan degree(0~3600); param2: tilt coordinates(0~900); param3:zoom(1~128)
	DH_EXTPTZ_RESETZERO,                        // Reset 3D positioning as zero
	DH_EXTPTZ_MOVE_ABSOLUTELY,                  // Absolute motion control commands, param4 corresponding struct PTZ_CONTROL_ABSOLUTELY
	DH_EXTPTZ_MOVE_CONTINUOUSLY,                // Continuous motion control commands, param4 corresponding struct PTZ_CONTROL_CONTINUOUSLY
	DH_EXTPTZ_GOTOPRESET,                       // PTZ control command, at a certain speed to preset locus, param4 corresponding struct PTZ_CONTROL_GOTOPRESET
	DH_EXTPTZ_SET_VIEW_RANGE = 0x49,           	// Set to horizon(param4 corresponding struct PTZ_VIEW_RANGE_INFO)
	DH_EXTPTZ_FOCUS_ABSOLUTELY = 0x4A,         	// Absolute focus(param4 corresponding struct PTZ_FOCUS_ABSOLUTELY)
	DH_EXTPTZ_HORSECTORSCAN = 0x4B,             // Level fan sweep(param4 corresponding PTZ_CONTROL_SECTORSCAN,param1,param2,param3 is invalid)
	DH_EXTPTZ_VERSECTORSCAN = 0x4C,             // Vertical sweep fan(param4correspondingPTZ_CONTROL_SECTORSCAN,param1,param2,param3 is invalid)
	DH_EXTPTZ_SET_ABS_ZOOMFOCUS = 0x4D,         // Set absolute focus, focus on value, param1 for focal length, range: [0-255], param2 as the focus, scope: [0-255], param3, param4 is invalid
    DH_EXTPTZ_SET_FISHEYE_EPTZ = 0x4E,          // Control fish eye PTZ, param4corresponding to structure PTZ_CONTROL_SET_FISHEYE_EPTZ 
    DH_EXTPTZ_SET_TRACK_START = 0x4F,           // Track start control(param4 corresponding to structure  PTZ_CONTROL_SET_TRACK_CONTROL,dwStop set as FALSE, param1, param2, param3 is invalid)
    DH_EXTPTZ_SET_TRACK_STOP = 0x50,            // Track stop control (param4 corresponding to structure PTZ_CONTROL_SET_TRACK_CONTROL,dwStop set as FALSE, param1, param2, param3  is invalid)
    DH_EXTPTZ_RESTART = 0x51,                   // To restart the PTZ(param1,param2,param3,param4 are all invalid ,dwStop set to FALSE )
	DH_EXTPTZ_INTELLI_TRACKMOVE = 0x52,         // Continuous motion control commands,track move,param4 corresponding struct PTZ_CONTROL_INTELLI_TRACKMOVE
    DH_EXTPTZ_SET_FOCUS_REGION = 0x53,          // set the focus region(param4 corresponding to PTZ_CONTROL_SET_FOCUS_REGION,dwStop set to FALSE,param1,param2,param3 are all invalid)
    DH_EXTPTZ_PAUSELINESCAN = 0x54,             // Pause Scan(param1,param2,param3,param4 are all invalid ,dwStop set to FALSE)

	DH_EXTPTZ_UP_TELE = 0x70,					// Up + TELE param1=speed (1-8) 
	DH_EXTPTZ_DOWN_TELE,						// Down + TELE
	DH_EXTPTZ_LEFT_TELE,						// Left + TELE
	DH_EXTPTZ_RIGHT_TELE,						// Right + TELE
	DH_EXTPTZ_LEFTUP_TELE,						// Upper left + TELE
	DH_EXTPTZ_LEFTDOWN_TELE,					// Down left + TELE
	DH_EXTPTZ_TIGHTUP_TELE,						// Upper right + TELE
	DH_EXTPTZ_RIGHTDOWN_TELE,					// Down right + TELE
	DH_EXTPTZ_UP_WIDE,							// Up + WIDE param1=speed (1-8) 
	DH_EXTPTZ_DOWN_WIDE,						// Down + WIDE
	DH_EXTPTZ_LEFT_WIDE,						// Left + WIDE
	DH_EXTPTZ_RIGHT_WIDE,						// Right + WIDE
	DH_EXTPTZ_LEFTUP_WIDE,						// Upper left + WIDE
	DH_EXTPTZ_LEFTDOWN_WIDE,					// Down left+ WIDE
	DH_EXTPTZ_TIGHTUP_WIDE,						// Upper right + WIDE
	DH_EXTPTZ_RIGHTDOWN_WIDE,					// Down right + WIDE
	DH_EXTPTZ_GOTOPRESETSNAP = 0x80,            // goto preset and snap
    DH_EXTPTZ_DIRECTIONCALIBRATION = 0x82,      // calibtate directiontwo direction
    DH_EXTPTZ_SINGLEDIRECTIONCALIBRATION = 0x83,// calibtate directionsingle direction,param4 corresponding to structure NET_IN_CALIBRATE_SINGLEDIRECTION  

    DH_EXTPTZ_TOTAL,							// max command value
} DH_EXTPTZ_ControlType;

// Fish eye electronic PTZ control command 
typedef enum tagNET_FISHEYE_EPTZ_CMD
{
  NET_FISHEYE_EPTZ_CMD_UNKOWN,          // unknown type 
  NET_FISHEYE_EPTZ_CMD_ZOOMIN,          // Zoom in , dwParam1 means step length, range 1-8
  NET_FISHEYE_EPTZ_CMD_ZOOMOUT,         // Zoom out, dwParam1 means step length ,  range 1-8
  NET_FISHEYE_EPTZ_CMD_UP,            	// Move up, dwParam1 means step length ,  range 1-8
  NET_FISHEYE_EPTZ_CMD_DOWN,           	// Move down, dwParam1 means step length ,  range 1-8
  NET_FISHEYE_EPTZ_CMD_LEFT,           	// Move left, dwParam1 means step length ,  range 1-8
  NET_FISHEYE_EPTZ_CMD_RIGHT,           // Move right, dwParam1 means step length ,  range 1-8
  NET_FISHEYE_EPTZ_CMD_ROTATECLOCK,     // Auto clockwise rotation , dwParam1 means step length ,  range 1-8
  NET_FISHEYE_EPTZ_CMD_ROTATEANTICLOCK, // Auto counterclockwise rotation, dwParam1 means step length ,  range 1-8
  NET_FISHEYE_EPTZ_CMD_STOP,           	// End auto rotation
  NET_FISHEYE_EPTZ_CMD_TAPVIEW,         // Display specific location, instant view, dwParam1 means eptz, E-PTZ ,  focus coordinate, as The focus, the rectified image was in the abscissa, take the value does not exceed the rectified output image width
  // dwParam2 means eptz, E-PTZ , The focus of the cordinate, the rectified image was focus on the ordinate, take the value does not exceed the rectified output image 
  NET_FISHEYE_EPTZ_CMD_SHOEREGION,      // Panel Zoom in , wParam1 means selected rectangle center horizontal corrdinate, 
  // dwParam2 means selected rectangle center horizontal coordinate, dwParam3 means selected rectangle width
  // dwParam4 means selected rectangle height
}NET_FISHEYE_EPTZ_CMD;

// Track Control Command
typedef enum tagNET_TRACK_CONTROL_CMD
{
    NET_TRACK_CONTROL_CMD_UP,                       // Move up, dwParam1 means step length range 1-8 
    NET_TRACK_CONTROL_CMD_DOWN,                     // Move down, dwParam1 means step length  range 1-8
    NET_TRACK_CONTROL_CMD_LEFT,                     // Move left, dwParam1 means step length  range 1-8
    NET_TRACK_CONTROL_CMD_RIGHT,                    // Move right, dwParam1 means step length  range 1-8
    NET_TRACK_CONTROL_CMD_SETPRESET,                // Set preset, dwParam1 means preset value
    NET_TRACK_CONTROL_CMD_CLEARPRESET,              // Clear preset, dwParam1 means preset value
    NET_TRACK_CONTROL_CMD_GOTOPRESET,               // Goto preset, dwParam1 means preset value
} NET_TRACK_CONTROL_CMD;

// Fish eye correction mode
typedef enum tagNET_CALIBRATE_MODE
{
  NET_FISHEYE_CALIBRATE_MODE_UNKOWN,           				// unknown mode 
  NET_FISHEYE_CALIBRATE_MODE_ORIGIAL,          				// Original image mode 
  NET_FISHEYE_CALIBRATE_MODE_CONFIG,           				// config mode
  NET_FISHEYE_CALIBRATE_MODE_PANORAMA,         				// global mode 
  NET_FISHEYE_CALIBRATE_MODE_DOUBLEPANORAMA,       			// dual global mode 
  NET_FISHEYE_CALIBRATE_MODE_ORIGIALPLUSTHREEEPTZREGION, 	// 1+3 mode , one original Fish eye plus 3 EPtz images, 
  NET_FISHEYE_CALIBRATE_MODE_SINGLE,           				// Single image EPtz mode , only one EPtz video, 
  NET_FISHEYE_CALIBRATE_MODE_FOUREPTZREGION,       			// 4 image mode , 4 EPtz control image, 
  NET_FISHEYE_CALIBRATE_MODE_NORMAL,           				// normal mode 
}NET_FISHEYE_CALIBRATE_MODE;

typedef enum tagNET_FISHEYE_TYPE
{
  NET_FISHEYE_UNKOWN =0,        	// unknown 
  NET_FISHEYE_CHIP,        			//only support device port Fish eye correction
  NET_FISHEYE_PLUGIN,        		//only support control Fish eye correction
  NET_FISHEYE_CHIP_PLUGIN,        	//support both
}NET_FISHEYE_TYPE;

// Fish eye installation mode 
typedef enum tagNET_FISHEYE_MOUNT_MODE
{
  NET_FISHEYE_MOUNT_MODE_UNKOWN,             // unknown mode          
  NET_FISHEYE_MOUNT_MODE_CEIL,               // ceiling mode 
  NET_FISHEYE_MOUNT_MODE_WALL,               // wall mount mode  
  NET_FISHEYE_MOUNT_MODE_FLOOR,              // ground mode  
}NET_FISHEYE_MOUNT_MODE;

/////////////////////////////////About Log /////////////////////////////////

// Log search type 
typedef enum _DH_LOG_QUERY_TYPE
{
	DHLOG_ALL = 0,								// All logs
	DHLOG_SYSTEM,								// System logs 
	DHLOG_CONFIG,								// Configuration logs 
	DHLOG_STORAGE,								// Storage logs
	DHLOG_ALARM,								// Alarm logs 
	DHLOG_RECORD,								// Record related
	DHLOG_ACCOUNT,								// Account related
	DHLOG_CLEAR,								// Clear log 
	DHLOG_PLAYBACK,								// Playback related 
	DHLOG_MANAGER                               // Concerning the front-end management and running
} DH_LOG_QUERY_TYPE;

// Log Type
typedef enum _DH_LOG_TYPE
{
	DH_LOG_REBOOT = 0x0000,						// Device reboot 
	DH_LOG_SHUT,								// Shut down device 
    DH_LOG_REPORTSTOP,
    DH_LOG_REPORTSTART,
	DH_LOG_UPGRADE = 0x0004,					// Device Upgrade
	DH_LOG_SYSTIME_UPDATE = 0x0005,             // system time update
	DH_LOG_GPS_TIME_UPDATE = 0x0006,			// GPS time update
	DH_LOG_AUDIO_TALKBACK,	  					// Voice intercom, true representative open, false on behalf of the closed
	DH_LOG_COMM_ADAPTER,						// Transparent transmission, true representative open, false on behalf of the closed
    DH_LOG_NET_TIMING,                          // Net sync
	DH_LOG_CONFSAVE = 0x0100,					// Save configuration 
	DH_LOG_CONFLOAD,							// Read configuration 
	DH_LOG_FSERROR = 0x0200,					// File system error
	DH_LOG_HDD_WERR,							// HDD write error 
	DH_LOG_HDD_RERR,							// HDD read error
	DH_LOG_HDD_TYPE,							// Set HDD type 
	DH_LOG_HDD_FORMAT,							// Format HDD
	DH_LOG_HDD_NOSPACE,							// Current working HDD space is not sufficient
	DH_LOG_HDD_TYPE_RW,							// Set HDD type as read-write 
	DH_LOG_HDD_TYPE_RO,							// Set HDD type as read-only
	DH_LOG_HDD_TYPE_RE,							// Set HDD type as redundant 
	DH_LOG_HDD_TYPE_SS,							// Set HDD type as snapshot
	DH_LOG_HDD_NONE,							// No HDD
	DH_LOG_HDD_NOWORKHDD,						// No work HDD
	DH_LOG_HDD_TYPE_BK,							// Set HDD type to backup HDD
	DH_LOG_HDD_TYPE_REVERSE,					// Set HDD type to reserve subarea
	DH_LOG_HDD_START_INFO = 0x20e ,             // note the boot-strap's hard disk info
	DH_LOG_HDD_WORKING_DISK,                    // note the disk number after the disk change
	DH_LOG_HDD_OTHER_ERROR,                     // note other errors of disk
	DH_LOG_HDD_SLIGHT_ERR,						// there has some little errors on disk
	DH_LOG_HDD_SERIOUS_ERR,                     // there has some serious errors on disk
	DH_LOG_HDD_NOSPACE_END,                     // the end of the alarm that current disk has no space left 
	DH_LOG_HDD_TYPE_RAID_CONTROL,               // Raid control
	DH_LOG_HDD_TEMPERATURE_HIGH,				// excess temperature
	DH_LOG_HDD_TEMPERATURE_LOW,					// lower die temperature
	DH_LOG_HDD_ESATA_REMOVE,					// remove eSATA
	DH_LOG_ALM_IN = 0x0300,						// External alarm begin 
	DH_LOG_NETALM_IN,							// Network alarm input 
	DH_LOG_ALM_END = 0x0302,					// External input alarm stop 
	DH_LOG_LOSS_IN,								// Video loss alarm begin 
	DH_LOG_LOSS_END,							// Video loss alarm stop
	DH_LOG_MOTION_IN,							// Motion detection alarm begin 
	DH_LOG_MOTION_END,							// Motion detection alarm stop 
	DH_LOG_ALM_BOSHI,							// Annunciator alarm input 
	DH_LOG_NET_ABORT = 0x0308,					// Network disconnected 
	DH_LOG_NET_ABORT_RESUME,					// Network connection restore 
	DH_LOG_CODER_BREAKDOWN,						// Encoder error
	DH_LOG_CODER_BREAKDOWN_RESUME,				// Encoder error restore 
	DH_LOG_BLIND_IN,							// Camera masking 
	DH_LOG_BLIND_END,							// Restore camera masking 
	DH_LOG_ALM_TEMP_HIGH,						// High temperature 
	DH_LOG_ALM_VOLTAGE_LOW,						// Low voltage
	DH_LOG_ALM_BATTERY_LOW,						// Battery capacity is not sufficient 
	DH_LOG_ALM_ACC_BREAK,						// ACC power off 
    DH_LOG_ALM_ACC_RES,
	DH_LOG_GPS_SIGNAL_LOST,						// GPS signal lost
	DH_LOG_GPS_SIGNAL_RESUME,					// GPS signal resume
	DH_LOG_3G_SIGNAL_LOST,						// 3G signal lost
	DH_LOG_3G_SIGNAL_RESUME,					// 3G signal resume
    DH_LOG_ALM_IPC_IN,							// IPC external alarms
	DH_LOG_ALM_IPC_END,							// IPC external alarms recovery
	DH_LOG_ALM_DIS_IN,							// Broken network alarm
	DH_LOG_ALM_DIS_END,							// Broken network alarm recovery
	DH_LOG_ALM_UPS_IN, 				            // UPS alarm 
    DH_LOG_ALM_UPS_END, 				        // UPS alarm resume 
    DH_LOG_ALM_NAS_IN,				            // NAS server abnormal alarm 
    DH_LOG_ALM_NAS_END,				            // NAS server abnormal alarm resume 
    DH_LOG_ALM_REDUNDANT_POWER_IN,              // Redundant power alarm 
    DH_LOG_ALM_REDUNDANT_POWER_END,             // Redundant alarm resume  
    DH_LOG_ALM_RECORD_FAILED_IN,				// Record failure alarm 
    DH_LOG_ALM_RECORD_FAILED_END,			    // Record failure alarm resume 
    DH_LOG_ALM_VGEXCEPT_IN,				        // Storage pool abnormal alarm 
    DH_LOG_ALM_VGEXCEPT_END,				    // Storage abnormal alarm resume 		
    DH_LOG_ALM_FANSPEED_IN,			            // Fan alarm starts
    DH_LOG_ALM_FANSPEED_END,			        // Fan alarm stops 
    DH_LOG_ALM_DROP_FRAME_IN,			        // Frame loss alarm starts 
    DH_LOG_ALM_DROP_FRAME_END,			        // Frame loss alarm stops
    DH_LOG_ALM_DISK_STATE_CHECK,		        // HDD pre-check tour alarm event log type 
    DH_LOG_ALARM_COAXIAL_SMOKE,		            // HDCVI smoke alarm event
    DH_LOG_ALARM_COAXIAL_TEMP_HIGH,	            // HDCVI temperature alarm event 
    DH_LOG_ALARM_COAXIAL_ALM_IN,		        // HDCVI external alarm event 
	DH_LOG_INFRAREDALM_IN = 0x03a0,				// Wireless alarm begin 
	DH_LOG_INFRAREDALM_END,						// Wireless alarm end 
	DH_LOG_IPCONFLICT,							// IP conflict 
	DH_LOG_IPCONFLICT_RESUME,					// IP restore
	DH_LOG_SDPLUG_IN,							// SD Card insert
	DH_LOG_SDPLUG_OUT,							// SD Card Pull-out
	DH_LOG_NET_PORT_BIND_FAILED,				// Failed to bind port
	DH_LOG_HDD_BEEP_RESET,                      // Hard disk error beep reset 
	DH_LOG_MAC_CONFLICT,                        // MAC conflict
	DH_LOG_MAC_CONFLICT_RESUME,                 // MAC conflict resume
	DH_LOG_ALARM_OUT,							// alarm out
	DH_LOG_ALM_RAID_STAT_EVENT,                 // RAID state event    
	DH_LOG_ABLAZE_ON,				            // Fire alarm, smoker or high temperature
	DH_LOG_ABLAZE_OFF,			                // Fire alarm reset 
	DH_LOG_INTELLI_ALARM_PLUSE,					// Intelligence pulse alarm
	DH_LOG_INTELLI_ALARM_IN,					// Intelligence alarm start
	DH_LOG_INTELLI_ALARM_END,					// Intelligence alarm end
	DH_LOG_3G_SIGNAL_SCAN,						// 3G signal scan
	DH_LOG_GPS_SIGNAL_SCAN,						// GPS signal scan
	DH_LOG_AUTOMATIC_RECORD = 0x0400,			// Auto record 
	DH_LOG_MANUAL_RECORD = 0x0401,				// Manual record 
	DH_LOG_CLOSED_RECORD,						// Stop record 
	DH_LOG_LOGIN = 0x0500,						// Log in 
	DH_LOG_LOGOUT,								// Log off 
	DH_LOG_ADD_USER,							// Add user
	DH_LOG_DELETE_USER,							// Delete user
	DH_LOG_MODIFY_USER,							// Modify user 
	DH_LOG_ADD_GROUP,							// Add user group 
	DH_LOG_DELETE_GROUP,						// Delete user group 
	DH_LOG_MODIFY_GROUP,						// Modify user group 
	DH_LOG_NET_LOGIN = 0x0508,					// Network Login
	DH_LOG_MODIFY_PASSWORD,						// Modify password
	DH_LOG_CLEAR = 0x0600,						// Clear log 
	DH_LOG_SEARCHLOG,							// Search log 
	DH_LOG_SEARCH = 0x0700,						// Search record 
	DH_LOG_DOWNLOAD,							// Record download
	DH_LOG_PLAYBACK,							// Record playback
	DH_LOG_BACKUP,								// Backup recorded file 
	DH_LOG_BACKUPERROR,							// Failed to backup recorded file
	DH_LOG_BACK_UPRT,							// Real-time backup, that is, copy CD
	DH_LOG_BACKUPCLONE,							// CD copy
	DH_LOG_DISK_CHANGED,						// Manual changed
	DH_LOG_IMAGEPLAYBACK,						// Image playback
	DH_LOG_LOCKFILE,							// Lock the video
	DH_LOG_UNLOCKFILE,							// Unlock the video
	DH_LOG_ATMPOS,								// Add log superposition of ATM card number
	DH_PLAY_PAUSE,								// Pause
	DH_PLAY_START,								// Start
	LOG_PLAY_STOP,								// Stop
	LOG_PLAY_BACK,								// Back
	LOG_PLAY_FAST,								// Fast
	LOG_PLAY_SLOW,								// Slow
	LOG_SMART_SEARCH,							// Search
	LOG_RECORD_SNAP,							// Snap
	LOG_ADD_TAG,								// Add tag
	LOG_DEL_TAG,								// Delete tag
	LOG_USB_IN,									// USB connected
	LOG_USB_OUT,								// USB disconnected
	LOG_BACKUP_FILE,							// Backup file
	LOG_BACKUP_LOG,								// Backup log
	LOG_BACKUP_CONFIG,							// Backup config

    DH_LOG_TIME_UPDATE  = 0x0800,               // Time update
    DH_LOG_REMOTE_STATE = 0x0850,               // remote diary 
    DH_LOG_USER_DEFINE = 0x0900,
	DH_LOG_TYPE_NR = 10,
} DH_LOG_TYPE;

// Extensive log type. Correponding to CLIENT_QueryLogEx, Condition (int nType = 1; parameter reserved = &nType)
typedef enum _DH_NEWLOG_TYPE
{
	DH_NEWLOG_REBOOT = 0x0000,					// Device reboot 
	DH_NEWLOG_SHUT,								// Shut down device
    DH_NEWLOG_REPORTSTOP,
    DH_NEWLOG_REPORTSTART,
	DH_NEWLOG_UPGRADE = 0x0004,					// Device upgrade
	DH_NEWLOG_SYSTIME_UPDATE = 0x0005,          // system time update
	DH_NEWLOG_GPS_TIME_UPDATE = 0x0006,			// GPS time update

	DH_NEWLOG_AUDIO_TALKBACK,	  				// Voice intercom, true representative open, false on behalf of the closed
	DH_NEWLOG_COMM_ADAPTER,						// Transparent transmission, true representative open, false on behalf of the closed	
	DH_NEWLOG_NET_TIMING,						// NTP

	DH_NEWLOG_CONFSAVE = 0x0100,				// Save configuration 
	DH_NEWLOG_CONFLOAD,							// Read configuration 
	DH_NEWLOG_FSERROR = 0x0200,					// File system error
	DH_NEWLOG_HDD_WERR,							// HDD write error 
	DH_NEWLOG_HDD_RERR,							// HDD read error
	DH_NEWLOG_HDD_TYPE,							// Set HDD type 
	DH_NEWLOG_HDD_FORMAT,						// Format HDD
	DH_NEWLOG_HDD_NOSPACE,						// Current working HDD space is not sufficient
	DH_NEWLOG_HDD_TYPE_RW,						// Set HDD type as read-write 
	DH_NEWLOG_HDD_TYPE_RO,						// Set HDD type as read-only
	DH_NEWLOG_HDD_TYPE_RE,						// Set HDD type as redundant 
	DH_NEWLOG_HDD_TYPE_SS,						// Set HDD type as snapshot
	DH_NEWLOG_HDD_NONE,							// No HDD
	DH_NEWLOG_HDD_NOWORKHDD,					// No work HDD
	DH_NEWLOG_HDD_TYPE_BK,						// Set HDD type to backup HDD
	DH_NEWLOG_HDD_TYPE_REVERSE,					// Set HDD type to reserve subareas
	DH_NEWLOG_HDD_START_INFO = 0x20e ,          // note the boot-strap's hard disk info
	DH_NEWLOG_HDD_WORKING_DISK,                 // note the disk number after the disk change
	DH_NEWLOG_HDD_OTHER_ERROR,                  // note other errors of disk
	DH_NEWLOG_HDD_SLIGHT_ERR,					// there has some little errors on disk
	DH_NEWLOG_HDD_SERIOUS_ERR,                  // there has some serious errors on disk
	DH_NEWLOG_HDD_NOSPACE_END,                  // the end of the alarm that current disk has no space left 

	DH_NEWLOG_HDD_TYPE_RAID_CONTROL,            // Raid operation
	DH_NEWLOG_HDD_TEMPERATURE_HIGH,				// excess temperature
	DH_NEWLOG_HDD_TEMPERATURE_LOW,				// lower die temperature
	DH_NEWLOG_HDD_ESATA_REMOVE,					// remove eSATA

	DH_NEWLOG_ALM_IN = 0x0300,					// External alarm begin 
	DH_NEWLOG_NETALM_IN,						// Network alarm input 
	DH_NEWLOG_ALM_END = 0x0302,					// External input alarm stop 
	DH_NEWLOG_LOSS_IN,							// Video loss alarm begin 
	DH_NEWLOG_LOSS_END,							// Video loss alarm stop
	DH_NEWLOG_MOTION_IN,						// Motion detection alarm begin 
	DH_NEWLOG_MOTION_END,						// Motion detection alarm stop 
	DH_NEWLOG_ALM_BOSHI,						// Annunciator alarm input 
	DH_NEWLOG_NET_ABORT = 0x0308,				// Network disconnected 
	DH_NEWLOG_NET_ABORT_RESUME,					// Network connection restore 
	DH_NEWLOG_CODER_BREAKDOWN,					// Encoder error
	DH_NEWLOG_CODER_BREAKDOWN_RESUME,			// Encoder error restore 
	DH_NEWLOG_BLIND_IN,							// Camera masking 
	DH_NEWLOG_BLIND_END,						// Restore camera masking 
	DH_NEWLOG_ALM_TEMP_HIGH,					// High temperature 
	DH_NEWLOG_ALM_VOLTAGE_LOW,					// Low voltage
	DH_NEWLOG_ALM_BATTERY_LOW,					// Battery capacity is not sufficient 
	DH_NEWLOG_ALM_ACC_BREAK,					// ACC power off 
    DH_NEWLOG_ALM_ACC_RES,
	DH_NEWLOG_GPS_SIGNAL_LOST,					// GPS signal lost
	DH_NEWLOG_GPS_SIGNAL_RESUME,				// GPS signal resume
	DH_NEWLOG_3G_SIGNAL_LOST,					// 3G signal lost
	DH_NEWLOG_3G_SIGNAL_RESUME,					// 3G signal resume

	DH_NEWLOG_ALM_IPC_IN,						// IPC external alarms
	DH_NEWLOG_ALM_IPC_END,						// IPC external alarms recovery 
	DH_NEWLOG_ALM_DIS_IN,						//Broken network alarm
	DH_NEWLOG_ALM_DIS_END,						// Broken network alarm recovery

	DH_NEWLOG_INFRAREDALM_IN = 0x03a0,			// Wireless alarm begin 
	DH_NEWLOG_INFRAREDALM_END,					// Wireless alarm end 
	DH_NEWLOG_IPCONFLICT,						// IP conflict 
	DH_NEWLOG_IPCONFLICT_RESUME,				// IP restore
	DH_NEWLOG_SDPLUG_IN,						// SD Card insert
	DH_NEWLOG_SDPLUG_OUT,						// SD Card Pull-out
	DH_NEWLOG_NET_PORT_BIND_FAILED,				// Failed to bind port
	DH_NEWLOG_HDD_BEEP_RESET,                   // Hard disk error beep reset
	DH_NEWLOG_MAC_CONFLICT,                     // MAC conflict
	DH_NEWLOG_MAC_CONFLICT_RESUME,              // MAC conflict resume
	DH_NEWLOG_ALARM_OUT,						// alarm of output state
	DH_NEWLOG_ALM_RAID_STAT_EVENT,              // RAID state change
	DH_NEWLOG_ABLAZE_ON,				        // fire alarm, smoke or temperature
	DH_NEWLOG_ABLAZE_OFF,			            // fire alarm resume
	DH_NEWLOG_INTELLI_ALARM_PLUSE,				// alarm of pulse type
	DH_NEWLOG_INTELLI_ALARM_IN,					// begin of alarm
	DH_NEWLOG_INTELLI_ALARM_END,				// end of alarm
	DH_NEWLOG_3G_SIGNAL_SCAN,					// 3G signal scan
	DH_NEWLOG_GPS_SIGNAL_SCAN,					// GPS signal scan
	DH_NEWLOG_AUTOMATIC_RECORD = 0x0400,		// Auto record 
	DH_NEWLOG_MANUAL_RECORD,					// Manual record 
	DH_NEWLOG_CLOSED_RECORD,					// Stop record 
	DH_NEWLOG_LOGIN = 0x0500,					// Log in 
	DH_NEWLOG_LOGOUT,							// Log off 
	DH_NEWLOG_ADD_USER,							// Add user
	DH_NEWLOG_DELETE_USER,						// Delete user
	DH_NEWLOG_MODIFY_USER,						// Modify user 
	DH_NEWLOG_ADD_GROUP,						// Add user group 
	DH_NEWLOG_DELETE_GROUP,						// Delete user group 
	DH_NEWLOG_MODIFY_GROUP,						// Modify user group
	DH_NEWLOG_NET_LOGIN = 0x0508,				// Network user login
	DH_NEWLOG_CLEAR = 0x0600,					// Clear log 
	DH_NEWLOG_SEARCHLOG,						// Search log 
	DH_NEWLOG_SEARCH = 0x0700,					// Search record 
	DH_NEWLOG_DOWNLOAD,							// Record download
	DH_NEWLOG_PLAYBACK,							// Record playback
	DH_NEWLOG_BACKUP,							// Backup recorded file 
	DH_NEWLOG_BACKUPERROR,						// Failed to backup recorded file

	DH_NEWLOG_BACK_UPRT,						// Real-time backup, that is, copy CD
	DH_NEWLOG_BACKUPCLONE,						// CD copy
	DH_NEWLOG_DISK_CHANGED,						// Manual  changed
	DH_NEWLOG_IMAGEPLAYBACK,					// image palyback
	DH_NEWLOG_LOCKFILE,							// Lock the video
	DH_NEWLOG_UNLOCKFILE,						// Unlock the video
	DH_NEWLOG_ATMPOS,							// Add log superposition of ATM card number

	DH_NEWLOG_TIME_UPDATE  = 0x0800,            // Time update
	DH_NEWLOG_REMOTE_STATE = 0x0850,            // remote diary 

    DH_NEWLOG_USER_DEFINE = 0x0900,
    DH_NEWLOG_TYPE_NR = 10,        
} DH_NEWLOG_TYPE;

///////////////////////////////About audio talk ///////////////////////////////

// Audio encode type 
typedef enum __TALK_CODING_TYPE
{
	DH_TALK_DEFAULT = 0,						// No-head PCM
	DH_TALK_PCM = 1,							// With head PCM
	DH_TALK_G711a,								// G711a
	DH_TALK_AMR,								// AMR
	DH_TALK_G711u,								// G711u
	DH_TALK_G726,								// G726
	DH_TALK_G723_53,							// G723_53
	DH_TALK_G723_63,							// G723_63
	DH_TALK_AAC,								// AAC
	DH_TALK_OGG,                                // OGG
    DH_TALK_G729 = 10,                          // G729
    DH_TALK_MPEG2,                              // MPEG2
    DH_TALK_MPEG2_Layer2,                       // MPEG2-Layer2
    DH_TALK_G722_1,                             // G.722.1
	DH_TALK_ADPCM = 21,                         // ADPCM
	DH_TALK_MP3   = 22,							// MP3
} DH_TALK_CODING_TYPE;

// Record file type
typedef enum __NET_RECORD_TYPE
{
	NET_RECORD_TYPE_ALL,                        // All the video
	NET_RECORD_TYPE_NORMAL,                     // common  video
	NET_RECORD_TYPE_ALARM,                      // External alarm video
	NET_RECORD_TYPE_MOTION,                     // DM alarm video
}NET_RECORD_TYPE;

// Audio talk way 
typedef enum __EM_USEDEV_MODE
{
	DH_TALK_CLIENT_MODE,						// Set client-end mode to begin audio talk 
	DH_TALK_SERVER_MODE,						// Set server mode to begin audio talk 
	DH_TALK_ENCODE_TYPE,						// Set encode format for audio talk 
	DH_ALARM_LISTEN_MODE,						// Set alarm subscribe way 
	DH_CONFIG_AUTHORITY_MODE,					// Set user right to realize configuration management
	DH_TALK_TALK_CHANNEL,						// set talking channel(0~MaxChannel-1)
	DH_RECORD_STREAM_TYPE,                      // set the stream type of the record for query(0-both main and extra stream,1-only main stream,2-only extra stream)  
	DH_TALK_SPEAK_PARAM,                        // set speaking parameter,corresponding to NET_SPEAK_PARAM
	DH_RECORD_TYPE,                             // Set by time video playback and download the video file TYPE (see.net RECORD TYPE)
	DH_TALK_MODE3,								// Set voice intercom parameters of three generations of equipment and the corresponding structure NET TALK the EX
    DH_PLAYBACK_REALTIME_MODE ,                 // set real time playback function(0-off, 1-on)
    DH_TALK_TRANSFER_MODE,                      // Judge the voice intercom if it was a forwarding mode, (corresponding to  NET_TALK_TRANSFER_PARAM)
    DH_TALK_VT_PARAM,                           // Set VT Talk param (corresponding to  NET_VT_TALK_PARAM)
    DH_TARGET_DEV_ID,                           // set target device identifier for searching system capacity information, (not zero - locate device forwards the information)
} EM_USEDEV_MODE;


typedef enum __EM_TALK_DATA_TYPE
{ 
	NET_TALK_DATA_LOCAL_AUDIO = 0,				// Local recording audio data from the library
	NET_TALK_DATA_RECV_AUDIO,					// Receiving device sending audio data
	NET_TALK_DATA_RESPOND,						// Intercom call response data
    NET_TALK_DATA_RECV_VIDEO,                   // Got device sent video data
}EM_TALK_DATA_TYPE;

typedef struct tagNET_TALK_VIDEO_FORMAT
{
	DWORD				dwSize;
	DWORD				dwCompression;			// Video compression format
	int					nFrequency;				// Video sampling frequency
}NET_TALK_VIDEO_FORMAT;
// Three generations of equipment parameters of voice intercom, corresponding to the CLIENT SetDeviceMode DH TALK MODE3 () command
typedef struct tagNET_TALK_EX
{
    DWORD               dwSize;
	int					nChannel;               // channel number 
	int                 nAudioPort;             // Audio transmission listener ports
	int					nWaitTime;              // Ms wait time, unit, use the default value is 0
    HWND				hVideoWnd;				// Visual talk video window
	NET_TALK_VIDEO_FORMAT stuVideoFmt;			// Video encode format
	char				szMulticastAddr[DH_MAX_IPADDR_LEN_EX]; // Multicast address
	WORD				wMulticastLocalPort;	// Multicast local port
	WORD				wMulticastRemotePort;	// Multicast remote port
}NET_TALK_EX;

// AMR Encode Type
typedef enum __EM_ARM_ENCODE_MODE
{
	DH_TALK_AMR_AMR475 = 1,						// AMR475
	DH_TALK_AMR_AMR515,							// AMR515
	DH_TALK_AMR_AMR59,							// AMR59
	DH_TALK_AMR_AMR67,							// AMR67
	DH_TALK_AMR_AMR74,							// AMR74
	DH_TALK_AMR_AMR795,							// AMR795
	DH_TALK_AMR_AMR102,							// AMR102
	DH_TALK_AMR_AMR122,							// AMR122
} EM_ARM_ENCODE_MODE;

typedef struct __NET_SPEAK_PARAM
{
	DWORD           dwSize;                     // struct size 
	int             nMode;                      // 0:talk back(default), 1: propaganda,from propaganda ro talk back,need afresh to configure
	int             nSpeakerChannel;            // reproducer channel
    BOOL            bEnableWait;                // Wait for device to responding or not when enable bidirectional talk. Default setup is no.TRUE:wait ;FALSE:no
} NET_SPEAK_PARAM;

//Open the forwarding mode of intercom or not 
typedef struct tagNET_TALK_TRANSFER_PARAM
{
    DWORD           dwSize;
    BOOL            bTransfer;                  // Open the forwarding mode of intercom or not, TRUE: yes, FALSE: no
}NET_TALK_TRANSFER_PARAM;

/////////////////////////////////Control Related/////////////////////////////////

// Control type    Corresponding to CLIENT_ControlDevice
typedef enum _CtrlType
{
	DH_CTRL_REBOOT = 0,							// Reboot device	
	DH_CTRL_SHUTDOWN,							// Shut down device
	DH_CTRL_DISK,								// HDD management
	DH_KEYBOARD_POWER = 3,						// Network keyboard
	DH_KEYBOARD_ENTER,
	DH_KEYBOARD_ESC,
	DH_KEYBOARD_UP,
	DH_KEYBOARD_DOWN,
	DH_KEYBOARD_LEFT,
	DH_KEYBOARD_RIGHT,
	DH_KEYBOARD_BTN0,
	DH_KEYBOARD_BTN1,
	DH_KEYBOARD_BTN2,
	DH_KEYBOARD_BTN3,
	DH_KEYBOARD_BTN4,
	DH_KEYBOARD_BTN5,
	DH_KEYBOARD_BTN6,
	DH_KEYBOARD_BTN7,
	DH_KEYBOARD_BTN8,
	DH_KEYBOARD_BTN9,
	DH_KEYBOARD_BTN10,
	DH_KEYBOARD_BTN11,
	DH_KEYBOARD_BTN12,
	DH_KEYBOARD_BTN13,
	DH_KEYBOARD_BTN14,
	DH_KEYBOARD_BTN15,
	DH_KEYBOARD_BTN16,
	DH_KEYBOARD_SPLIT,
	DH_KEYBOARD_ONE,
	DH_KEYBOARD_NINE,
	DH_KEYBOARD_ADDR,
	DH_KEYBOARD_INFO,
	DH_KEYBOARD_REC,
	DH_KEYBOARD_FN1,
	DH_KEYBOARD_FN2,
	DH_KEYBOARD_PLAY,
	DH_KEYBOARD_STOP,
	DH_KEYBOARD_SLOW,
	DH_KEYBOARD_FAST,
	DH_KEYBOARD_PREW,
	DH_KEYBOARD_NEXT,
	DH_KEYBOARD_JMPDOWN,
	DH_KEYBOARD_JMPUP,
    DH_KEYBOARD_10PLUS,
    DH_KEYBOARD_SHIFT,
    DH_KEYBOARD_BACK,
    DH_KEYBOARD_LOGIN ,                         // new network keyboard function
    DH_KEYBOARD_CHNNEL ,                        // switch video channel
	DH_TRIGGER_ALARM_IN = 100,					// Activate alarm input
	DH_TRIGGER_ALARM_OUT,						// Activate alarm output 
	DH_CTRL_MATRIX,								// Matrix control 
	DH_CTRL_SDCARD,								// SD card control(for IPC series). Please refer to HDD control
	DH_BURNING_START,							// Burner control:begin burning 
	DH_BURNING_STOP,							// Burner control:stop burning 
	DH_BURNING_ADDPWD,							// Burner control:overlay password(The string ended with '\0'. Max length is 8 bits. )
	DH_BURNING_ADDHEAD,							// Burner control:overlay head title(The string ended with '\0'. Max length is 1024 bytes. Use '\n' to Enter.)
	DH_BURNING_ADDSIGN,							// Burner control:overlay dot to the burned information(No parameter) 
	DH_BURNING_ADDCURSTOMINFO,					// Burner control:self-defined overlay (The string ended with '\0'. Max length is 1024 bytes. Use '\n' to Enter)
	DH_CTRL_RESTOREDEFAULT,						// restore device default setup 
	DH_CTRL_CAPTURE_START,						// Activate device snapshot
	DH_CTRL_CLEARLOG,							// Clear log
	DH_TRIGGER_ALARM_WIRELESS = 200,			// Activate wireless alarm (IPC series)
	DH_MARK_IMPORTANT_RECORD,					// Mark important record
	DH_CTRL_DISK_SUBAREA, 						// Network hard disk partition	
	DH_BURNING_ATTACH,							// Annex burning
	DH_BURNING_PAUSE,							// Burn Pause
	DH_BURNING_CONTINUE,						// Burn Resume
	DH_BURNING_POSTPONE,						// Burn Postponed
	DH_CTRL_OEMCTRL,							// OEM control
	DH_BACKUP_START,							// Start to device backup
	DH_BACKUP_STOP,								// Stop to device backup
	DH_VIHICLE_WIFI_ADD,						// Add WIFI configuration manually for car device
	DH_VIHICLE_WIFI_DEC,						// Delete WIFI configuration manually for car device
	DH_BUZZER_START,                            // Start to buzzer control 
	DH_BUZZER_STOP,                             // Stop to buzzer control
	DH_REJECT_USER,                             // Reject User
	DH_SHIELD_USER,                             // Shield User
	DH_RAINBRUSH,                               // Rain Brush 
	DH_MANUAL_SNAP,                             // manual snap (struct MANUAL_SNAP_PARAMETER)
	DH_MANUAL_NTP_TIMEADJUST,                   // manual ntp time adjust
	DH_NAVIGATION_SMS,                          // navigation info and note
	DH_CTRL_ROUTE_CROSSING,                     // route info
	DH_BACKUP_FORMAT,							// backup device format
	DH_DEVICE_LOCALPREVIEW_SLIPT,               // local preview split(struct DEVICE_LOCALPREVIEW_SLIPT_PARAMETER)    
	DH_CTRL_INIT_RAID,							// RAID init
	DH_CTRL_RAID,								// RAID control
	DH_CTRL_SAPREDISK,							// sapredisk control
	DH_WIFI_CONNECT,							// wifi connect(struct WIFI_CONNECT)
	DH_WIFI_DISCONNECT,							// wifi disconnect(struct WIFI_CONNECT)
	DH_CTRL_ARMED,                              // Arm/disarm operation 
	DH_CTRL_IP_MODIFY,                          // IP modify(struct DHCTRL_IPMODIFY_PARAM)                     
	DH_CTRL_WIFI_BY_WPS,                        // wps connect wifi(struct DHCTRL_CONNECT_WIFI_BYWPS)
	DH_CTRL_FORMAT_PATITION,					// format pattion (struct DH_FORMAT_PATITION)
	DH_CTRL_EJECT_STORAGE,						// eject storage device(struct DH_EJECT_STORAGE_DEVICE)
	DH_CTRL_LOAD_STORAGE,						// load storage device(struct DH_LOAD_STORAGE_DEVICE)
	DH_CTRL_CLOSE_BURNER,                       // close burner(struct NET_CTRL_BURNERDOOR) need wait 6s
	DH_CTRL_EJECT_BURNER,                       // eject burner(struct NET_CTRL_BURNERDOOR) need wait 4s
	DH_CTRL_CLEAR_ALARM,						// alarm elimination corresponding structure NET (CTRL CLEAR ALARM)
	DH_CTRL_MONITORWALL_TVINFO,					// TV wall information display corresponding structure NET (CTRL MONITORWALL TVINFO)
	DH_CTRL_START_VIDEO_ANALYSE,                //  start Intelligent VIDEO analysis (corresponding structure NET CTRL START VIDEO ANALYSE)
	DH_CTRL_STOP_VIDEO_ANALYSE,                 // STOP intelligent VIDEO analysis corresponding structure NET (CTRL STOP VIDEO ANALYSE)
	DH_CTRL_UPGRADE_DEVICE,                     // Controlled start equipment upgrades, independently complete the upgrade process by the equipment do not need to upgrade file
	DH_CTRL_MULTIPLAYBACK_CHANNALES,            // Multi-channel preview playback channel switching corresponding structure NET (CTRL MULTIPLAYBACK CHANNALES)
	DH_CTRL_SEQPOWER_OPEN,						// Turn on the switch power supply timing device output corresponding.net (CTRL SEQPOWER PARAM)
	DH_CTRL_SEQPOWER_CLOSE,						// Close the switch power supply timing device output corresponding.net (CTRL SEQPOWER PARAM)
	DH_CTRL_SEQPOWER_OPEN_ALL,					// Power timing group open the switch quantity output corresponding.net (CTRL SEQPOWER PARAM)
	DH_CTRL_SEQPOWER_CLOSE_ALL,					// Power sequence set close the switch quantity output corresponding.net (CTRL SEQPOWER PARAM)
	DH_CTRL_PROJECTOR_RISE,						// PROJECTOR up corresponding.net (CTRL PROJECTOR PARAM)
	DH_CTRL_PROJECTOR_FALL,						// PROJECTOR drop (corresponding to the.net CTRL PROJECTOR PARAM)
	DH_CTRL_PROJECTOR_STOP,						// PROJECTOR stop (corresponding to the.net CTRL PROJECTOR PARAM)
	DH_CTRL_INFRARED_KEY,						// INFRARED buttons (corresponding to the.net CTRL INFRARED KEY PARAM)
	DH_CTRL_START_PLAYAUDIO,					// Device START playback of audio file corresponding structure NET (CTRL START PLAYAUDIO)
	DH_CTRL_STOP_PLAYAUDIO,						// Equipment stop playback of audio file
	DH_CTRL_START_ALARMBELL,					// Corresponding structure NET open alarm (CTRL ALARMBELL)
	DH_CTRL_STOP_ALARMBELL,						// Close the warning signal corresponding structure NET (CTRL ALARMBELL)
	DH_CTRL_ACCESS_OPEN,						// OPEN ACCESS control - corresponding structure NET (CTRL ACCESS OPEN)	
	DH_CTRL_SET_BYPASS,							// Corresponding structure NET BYPASS function (CTRL SET BYPASS)
	DH_CTRL_RECORDSET_INSERT,					// Add records to record set number (corresponding to the.net CTRL you INSERT PARAM)
	DH_CTRL_RECORDSET_UPDATE,					// Update a record of the number (corresponding to the.net CTRL you PARAM)
	DH_CTRL_RECORDSET_REMOVE,					// According to the record set number to delete a record (corresponding to the.net CTRL you PARAM)
	DH_CTRL_RECORDSET_CLEAR,					// Remove all RECORDSET information corresponding.net (CTRL you PARAM)
	DH_CTRL_ACCESS_CLOSE,						// Entrance guard control - CLOSE corresponding structure NET (CTRL ACCESS CLOSE)
	DH_CTRL_ALARM_SUBSYSTEM_ACTIVE_SET,			// Alarm sub system activation setup(corresponding structure NET_CTRL_ALARM_SUBSYSTEM_SETACTIVE)
    DH_CTRL_FORBID_OPEN_STROBE,                 // Disable device open gateway(corresponding to structure  NET_CTRL_FORBID_OPEN_STROBE)
    DH_CTRL_OPEN_STROBE,                        // Enable gateway(corresponding to structure  NET_CTRL_OPEN_STROBE)
    DH_CTRL_TALKING_REFUSE,                     // Talk no response(corresponding to structure  NET_CTRL_TALKING_REFUSE)
    DH_CTRL_ARMED_EX,                           // arm-disarm operation(corresponding to structure CTRL_ARM_DISARM_PARAM_EX), CTRL_ARM_DISARM_PARAM upgrade, recommended
    DH_CTRL_REMOTE_TALK,                        // Remote talk control(corresponding to structure NET_CTRL_REMOTETALK_PARAM)
    DH_CTRL_NET_KEYBOARD = 400,                 // Net keyboard control(corresponding to structure  DHCTRL_NET_KEYBOARD)
    DH_CTRL_AIRCONDITION_OPEN,                  // Open air conditioner(corresponding to structure  NET_CTRL_OPEN_AIRCONDITION)
    DH_CTRL_AIRCONDITION_CLOSE,                 // Close air-conditioner(corresponding to structure  NET_CTRL_CLOSE_AIRCONDITION)
    DH_CTRL_AIRCONDITION_SET_TEMPERATURE,       // Set temperature (corresponding to structure  NET_CTRL_SET_TEMPERATURE)
    DH_CTRL_AIRCONDITION_ADJUST_TEMPERATURE,    // Adjust temperature(corresponding to structure  NET_CTRL_ADJUST_TEMPERATURE)
    DH_CTRL_AIRCONDITION_SETMODE,               // Set air work mode(corresponding to structure  NET_CTRL_ADJUST_TEMPERATURE)
    DH_CTRL_AIRCONDITION_SETWINDMODE,           // Set fan mode(corresponding to structure  NET_CTRL_AIRCONDITION_SETMODE)
    DH_CTRL_RESTOREDEFAULT_EX ,                 // Recover device default and set new protocol(corresponding to structure NET_CTRL_RESTORE_DEFAULT)
                                                // Recover config and use this enumeration first, if port failed,
                                                // and CLIENT_GetLastError return NET_UNSUPPORTED, try again DH_CTRL_RESTOREDEFAULT restore config
    DH_CTRL_NOTIFY_EVENT,                       // send event to device (corresponding to  structure  NET_NOTIFY_EVENT_DATA)
    DH_CTRL_SILENT_ALARM_SET,                   // mute alarm setup
	DH_CTRL_START_PLAYAUDIOEX,                  // device start sound report(corresponding to  structure  NET_CTRL_START_PLAYAUDIOEX)
    DH_CTRL_STOP_PLAYAUDIOEX,                   // device stop sound report
    DH_CTRL_CLOSE_STROBE,                       // close gateway(corresponding to  structure  NET_CTRL_CLOSE_STROBE)
	DH_CTRL_SET_ORDER_STATE,                    // set parking reservation status (corresponding to  structure  NET_CTRL_SET_ORDER_STATE)
	DH_CTRL_RECORDSET_INSERTEX,                 // add fingerprint record get record collection no.(corresponding to NET_CTRL_RECORDSET_INSERT_PARAM)
    DH_CTRL_RECORDSET_UPDATEEX,                 // update finger print record set no record(corresponding to NET_CTRL_RECORDSET_PARAM)
    DH_CTRL_CAPTURE_FINGER_PRINT,               // fingerprint collection (corresponding to structure NET_CTRL_CAPTURE_FINGER_PRINT)
	DH_CTRL_ECK_LED_SET,                           // Parking lot entrance/exit controller LED setup(corresponding structure  NET_CTRL_ECK_LED_SET_PARAM)
    DH_CTRL_ECK_IC_CARD_IMPORT,                    // Intelligent parking system in/out device IC card info import (corresponding structure  NET_CTRL_ECK_IC_CARD_IMPORT_PARAM)
    DH_CTRL_ECK_SYNC_IC_CARD,                      // Intelligent parking system in/out device IC card info sync command, receive this command, device will delete original IC card info(corresponding structure  NET_CTRL_ECK_SYNC_IC_CARD_PARAM)
    DH_CTRL_LOWRATEWPAN_REMOVE,                    // Delete specific wireless device(corresponding structure  NET_CTRL_LOWRATEWPAN_REMOVE)
    DH_CTRL_LOWRATEWPAN_MODIFY,                    // Modify wireless device info(corresponding structure  NET_CTRL_LOWRATEWPAN_MODIFY)	
    DH_CTRL_ECK_SET_PARK_INFO,                     // Set up the vehicle spot information of the machine at the passageway of the intelligent parking system (corresponding to  NET_CTRL_ECK_SET_PARK_INFO_PARAM)
    DH_CTRL_VTP_DISCONNECT,                        // hang up the video phone (corresponding to NET_CTRL_VTP_DISCONNECT)
    DH_CTRL_UPDATE_FILES,                          // the update of the remote multimedia files (corresponding to NET_CTRL_UPDATE_FILES)
    DH_CTRL_MATRIX_SAVE_SWITCH,                    // Save up the relationship between the hyponymy matrixes (corresponding to NET_CTRL_MATRIX_SAVE_SWITCH)
    DH_CTRL_MATRIX_RESTORE_SWITCH,                 // recover the relationship between the hyponymy matrixes (corresponding to NET_CTRL_MATRIX_RESTORE_SWITCH)
    DH_CTRL_VTP_DIVERTACK,                         // video talk phone divert ack(corresponding to NET_CTRL_VTP_DIVERTACK)
    DH_CTRL_RAINBRUSH_MOVEONCE,                    // Rain-brush brush one time, efficient when set as manual mode(corresponding to NET_CTRL_RAINBRUSH_MOVEONCE)
    DH_CTRL_RAINBRUSH_MOVECONTINUOUSLY,            // Rain-brush brush cyclic, efficient when set as manal mode(corresponding to NET_CTRL_RAINBRUSH_MOVECONTINUOUSLY)
    DH_CTRL_RAINBRUSH_STOPMOVE,                    // Rain-brush stop, efficient when set as manal mode(corresponding to NET_CTRL_RAINBRUSH_STOPMOVE)
    DH_CTRL_ALARM_ACK,                             // affirm the alarm event(corresponding to NET_CTRL_ALARM_ACK)                                                   // DH_CTRL_ALARM_ACK DO NOT call this method in callback interface
    DH_CTRL_RECORDSET_IMPORT,                      // Batch import record set info (Corresponding to NET_CTRL_RECORDSET_PARAM)
    DH_CTRL_DELIVERY_FILE,                         // Delivery file to the video output port, building intercom use, run at the same time(Corresponding to NET_CTRL_DELIVERY_FILE)
	DH_CTRL_FORCE_BREAKING,                        // Force breaking rule(Corresponding to NET_CTRL_FORCE_BREAKING)
	DH_CTRL_RESTORE_EXCEPT,						   // Restore the configuration except the prescribed config.
	DH_CTRL_SET_PARK_INFO,						   // Set park info, platform is set to camera,the content is used for the dot matrix display(corresponding to NET_CTRL_SET_PARK_INFO)
	DH_CTRL_CLEAR_SECTION_STAT, 				   // clear the statistics for the period and start again from 0 (Corresponding to NET_CTRL_CLEAR_SECTION_STAT_INFO)
	DH_CTRL_DELIVERY_FILE_BYCAR,				   // Send video and image files to video output, Used by car, The ad time is served separately(Corresponding NET_CTRL_DELIVERY_FILE_BYCAR)
                                                   // The following commands are only for  CLIENT_ControlDeviceEx 
    DH_CTRL_THERMO_GRAPHY_ENSHUTTER = 0x10000,     // Enable or disable thermal shutter, pInBuf= NET_IN_THERMO_EN_SHUTTER*, pOutBuf= NET_OUT_THERMO_EN_SHUTTER * 
    DH_CTRL_RADIOMETRY_SETOSDMARK,                 // NET_IN_RADIOMETRY_SETOSDMARK*, pOutBuf= NET_OUT_RADIOMETRY_SETOSDMARK *    
    DH_CTRL_AUDIO_REC_START_NAME,                  // Enable audio record and get audio name, pInBuf = NET_IN_AUDIO_REC_MNG_NAME *, pOutBuf = NET_OUT_AUDIO_REC_MNG_NAME *
    DH_CTRL_AUDIO_REC_STOP_NAME,                   // Close audio file and return file name, pInBuf ,NET_IN_AUDIO_REC_MNG_NAME *, pOutBuf = NET_OUT_AUDIO_REC_MNG_NAME *
    DH_CTRL_SNAP_MNG_SNAP_SHOT,                    // Manual snap, pInBuf = NET_IN_SNAP_MNG_SHOT *, pOutBuf = NET_OUT_SNAP_MNG_SHOT *
    DH_CTRL_LOG_STOP,                              // Forcedly sync buffer data to the database and close the database, pInBuf = NET_IN_LOG_MNG_CTRL *, pOutBuf = NET_OUT_LOG_MNG_CTRL *
    DH_CTRL_LOG_RESUME,                            // Resume database, pInBuf = NET_IN_LOG_MNG_CTRL *, pOutBuf = NET_OUT_LOG_MNG_CTRL *
    DH_CTRL_POS_ADD,                               // Add a POS device, pInBuf = NET_IN_POS_ADD *, pOutBuf = NET_OUT_POS_ADD *
    DH_CTRL_POS_REMOVE,                            // Del a POS device, pInBuf = NET_IN_POS_REMOVE *, pOutBuf = NET_OUT_POS_REMOVE *
    DH_CTRL_POS_REMOVE_MULTI,                      // Del several POS device, pInBuf = NET_IN_POS_REMOVE_MULTI *, pOutBuf = NET_OUT_POS_REMOVE_MULTI *
    DH_CTRL_POS_MODIFY,                            // Modify a POS device, pInBuf = NET_IN_POS_ADD *, pOutBuf = NET_OUT_POS_ADD *
    DH_CTRL_SET_SOUND_ALARM,                       // Trigger alarm with sound, pInBuf = NET_IN_SOUND_ALARM *, pOutBuf = NET_OUT_SOUND_ALARM *
	DH_CTRL_AUDIO_MATRIX_SILENCE,				   // audiomatrix silence,pInBuf = NET_IN_AUDIO_MATRIX_SILENCE*,  pOutBuf =  NET_OUT_AUDIO_MATRIX_SILENCE*
    DH_CTRL_MANUAL_UPLOAD_PICTURE,                 // manual upload picture, pInBuf = NET_IN_MANUAL_UPLOAD_PICTURE *, pOutBUf = NET_OUT_MANUAL_UPLOAD_PICTURE *
    DH_CTRL_REBOOT_NET_DECODING_DEV,               // reboot network decoding device, pInBuf = NET_IN_REBOOT_NET_DECODING_DEV *, pOutBuf = NET_OUT_REBOOT_NET_DECODING_DEV *
	DH_CTRL_SET_IC_SENDER,						   // ParkingControl about setting IC Sender, pInBuf = NET_IN_SET_IC_SENDER *, pOutBuf = NET_OUT_SET_IC_SENDER * 
    DH_CTRL_SET_MEDIAKIND,                         // set the media type ,e.g. audio only,video only , audio & video ,pInBuf = NET_IN_SET_MEDIAKIND *, pOutBuf = NET_OUT_SET_MEDIAKIND *
	DH_CTRL_LOWRATEWPAN_ADD,                       // Add wireless device info(pInBuf = NET_CTRL_LOWRATEWPAN_ADD *, pOutBUf = NULL)
	DH_CTRL_LOWRATEWPAN_REMOVEALL,                 // remove all the wireless device info(pInBuf = NET_CTRL_LOWRATEWPAN_REMOVEALL *, pOutBUf = NULL)
	DH_CTRL_SET_DOOR_WORK_MODE,                    // Set the work mode of door( pInBuf = NET_IN_CTRL_ACCESS_SET_DOOR_WORK_MODE *, pOutBUf = NULL)
    DH_CTRL_TEST_MAIL,                             // Test Mail(pInBuf = NET_IN_TEST_MAIL *, pOutBUf = NET_OUT_TEST_MAIL *)
    DH_CTRL_CONTROL_SMART_SWITCH,                  // Control smart switch pInBuf = NET_IN_CONTROL_SMART_SWITCH *, pOutBUf = NET_OUT_CONTROL_SMART_SWITCH *
	DH_CTRL_LOWRATEWPAN_SETWORKMODE,          	   // Set the work mode of the detector(pInBuf = NET_IN_CTRL_LOWRATEWPAN_SETWORKMODE *, pOutBUf = NULL)
} CtrlType;

// the parameter of saving the relationship between the hyponymy matrixes(corresponding to DH_CTRL_MATRIX_SAVE_SWITCH)
typedef struct tagNET_CTRL_MATRIX_SAVE_SWITCH 
{
    DWORD           dwSize;
    const char*     pszName;                        //the relationship between the hyponymy matrixes
} NET_CTRL_MATRIX_SAVE_SWITCH;

// recover the relationship between the hyponymy matrixes (corresponding to DH_CTRL_MATRIX_RESTORE_SWITCH)
typedef struct tagNET_CTRL_MATRIX_RESTORE_SWITCH
{
    DWORD           dwSize;
    const char*     pszName;                        // the relationship between the hyponymy matrixes
} NET_CTRL_MATRIX_RESTORE_SWITCH;

// Restore default config new protocol input parameter
typedef struct tagNET_CTRL_RESTORE_DEFAULT
{
    DWORD			dwSize;
    const char*		szCfgNames;		// Config name and'&&' as interval, config see dhconfigsdk.h CLIENT_GetNewDevConfig and CLIENT_SetNewDevConfig port command 
}NET_CTRL_RESTORE_DEFAULT;

// Restore except config new protocol input parameter
typedef struct tagNET_CTRL_RESTORE_EXCEPT
{
	DWORD			 dwSize;
	char			 szCfgName[DH_MAX_STRING_LEN];				// Config name, config see dhconfigsdk.h CLIENT_GetNewDevConfig and CLIENT_SetNewDevConfig port command 
}NET_CTRL_RESTORE_EXCEPT;

// Open air-conditioner parameter(corresponding to DH_CTRL_AIRCONDITION_OPEN)
typedef struct tagNET_CTRL_OPEN_AIRCONDITION
{
    DWORD               dwSize;
    char				szDeviceID[DH_MAX_DEV_ID_LEN];			// Device SN
} NET_CTRL_OPEN_AIRCONDITION;

// Close air-conditioner parameter(corresponding to command DH_CTRL_AIRCONDITION_CLOSE)
typedef struct tagNET_CTRL_CLOSE_AIRCONDITION
{
    DWORD               dwSize;
    char				szDeviceID[DH_MAX_DEV_ID_LEN];			// Device SN
} NET_CTRL_CLOSE_AIRCONDITION;

// Set temperature parameter (corresponding command DH_CTRL_AIRCONDITION_SET_TEMPERATURE)
typedef struct tagNET_CTRL_SET_TEMPERATURE
{
    DWORD               dwSize;
    char				szDeviceID[DH_MAX_DEV_ID_LEN];			// SN 
    int                 nTemperature;                           // Temperature value(unit: degree Celsius)
} NET_CTRL_SET_TEMPERATURE;

// Adjust temperature parameter(corresponding command DH_CTRL_AIRCONDITION_ADJUST_TEMPERATURE)
typedef struct tagNET_CTRL_ADJUST_TEMPERATURE
{
    DWORD               dwSize;
    char				szDeviceID[DH_MAX_DEV_ID_LEN];			// SN 
    int                 nTemperatureScale;                      // Set value
} NET_CTRL_ADJUST_TEMPERATURE;

// Set air condition working mode parameter (corresponding command DH_CTRL_AIRCONDITION_SETMODE)
typedef struct tagNET_CTRL_AIRCONDITION_SETMODE
{
    DWORD                       dwSize;
    char				        szDeviceID[DH_MAX_DEV_ID_LEN];	// SN 
    EM_AIRCONDITION_MODE        emAirconditionMode;             // Air conditioner working mode
    int                         nTemperature;                   // Set value (unit: degree Celsius)
} NET_CTRL_AIRCONDITION_SETMODE;

// Set fan mode parameter(corresponding command DH_CTRL_AIRCONDITION_SETWINDMODE)
typedef struct tagNET_CTRL_AIRCONDITION_SETWINDMODE
{
    DWORD                       dwSize;
    char				        szDeviceID[DH_MAX_DEV_ID_LEN];	// SN 
    EM_AIRCONDITION_WINDMODE    emAirconditionWindMode;         // Aircondition Wind Mode
} NET_CTRL_AIRCONDITION_SETWINDMODE;

// IO control command. Corresponding to CLIENT_QueryIOControlState
typedef enum _IOTYPE
{
	DH_ALARMINPUT = 1,							// Control alarm input 
	DH_ALARMOUTPUT = 2,							// ontrol alarm output 
	DH_DECODER_ALARMOUT = 3,					// Control alarm decoder output 
	DH_WIRELESS_ALARMOUT = 5,					// Control wireless alarm output 
	DH_ALARM_TRIGGER_MODE = 7,					// Alarm activation type(auto/manual/close). Use TRIGGER_MODE_CONTROL structure 
} DH_IOTYPE;

/////////////////////////////////Configuration Related/////////////////////////////////

// Resolution enumeration. For DH_DSP_ENCODECAP to use 
typedef enum _CAPTURE_SIZE
{
	CAPTURE_SIZE_D1,							// 704*576(PAL)  704*480(NTSC)
	CAPTURE_SIZE_HD1,							// 352*576(PAL)  352*480(NTSC)
	CAPTURE_SIZE_BCIF,							// 704*288(PAL)  704*240(NTSC)
	CAPTURE_SIZE_CIF,							// 352*288(PAL)  352*240(NTSC)
	CAPTURE_SIZE_QCIF,							// 176*144(PAL)  176*120(NTSC)
	CAPTURE_SIZE_VGA,							// 640*480
	CAPTURE_SIZE_QVGA,							// 320*240
	CAPTURE_SIZE_SVCD,							// 480*480
	CAPTURE_SIZE_QQVGA,							// 160*128
	CAPTURE_SIZE_SVGA,							// 800*592
	CAPTURE_SIZE_XVGA,							// 1024*768
	CAPTURE_SIZE_WXGA,							// 1280*800
	CAPTURE_SIZE_SXGA,							// 1280*1024  
	CAPTURE_SIZE_WSXGA,							// 1600*1024  
	CAPTURE_SIZE_UXGA,							// 1600*1200
	CAPTURE_SIZE_WUXGA,							// 1920*1200
	CAPTURE_SIZE_LTF,							// 240*192
	CAPTURE_SIZE_720,							// 1280*720
	CAPTURE_SIZE_1080,							// 1920*1080
	CAPTURE_SIZE_1_3M,							// 1280*960
	CAPTURE_SIZE_2M,							// 1872*1408
	CAPTURE_SIZE_5M,							// 3744*1408
	CAPTURE_SIZE_3M,							// 2048*1536
	CAPTURE_SIZE_5_0M,                          // 2432*2050
	CPTRUTE_SIZE_1_2M,							// 1216*1024
	CPTRUTE_SIZE_1408_1024,                     // 1408*1024
	CPTRUTE_SIZE_8M,                            // 3296*2472
	CPTRUTE_SIZE_2560_1920,                     // 2560*1920(5M)
	CAPTURE_SIZE_960H,                          // 960*576(PAL) 960*480(NTSC)
	CAPTURE_SIZE_960_720,                       // 960*720
	CAPTURE_SIZE_NHD,							// 640*360
	CAPTURE_SIZE_QNHD,							// 320*180
	CAPTURE_SIZE_QQNHD,							// 160*90
	CAPTURE_SIZE_960_540,						// 960*540
	CAPTURE_SIZE_640_352,						// 640*352
	CAPTURE_SIZE_640_400,						// 640*400
	CAPTURE_SIZE_320_192,						// 320*192	
	CAPTURE_SIZE_320_176,						// 320*176
	CAPTURE_SIZE_NR=255  
} CAPTURE_SIZE;

// Configuration file type. For CLIENT_ExportConfigFile to use. 
typedef enum __DH_CONFIG_FILE_TYPE
{
	DH_CONFIGFILE_ALL = 0,						// All configuration file 
	DH_CONFIGFILE_LOCAL,						// Local configuration file 
	DH_CONFIGFILE_NETWORK,						// Network configuration file 
	DH_CONFIGFILE_USER,							// User configuration file 
} DH_CONFIG_FILE_TYPE;

// NTP
typedef enum __DH_TIME_ZONE_TYPE
{
	DH_TIME_ZONE_0,								// {0, 0*3600,"GMT+00:00"}
	DH_TIME_ZONE_1,								// {1, 1*3600,"GMT+01:00"}
	DH_TIME_ZONE_2,								// {2, 2*3600,"GMT+02:00"}
	DH_TIME_ZONE_3,								// {3, 3*3600,"GMT+03:00"}
	DH_TIME_ZONE_4,								// {4, 3*3600+1800,"GMT+03:30"}
	DH_TIME_ZONE_5,								// {5, 4*3600,"GMT+04:00"}
	DH_TIME_ZONE_6,								// {6, 4*3600+1800,"GMT+04:30"}
	DH_TIME_ZONE_7,								// {7, 5*3600,"GMT+05:00"}
	DH_TIME_ZONE_8,								// {8, 5*3600+1800,"GMT+05:30"}
	DH_TIME_ZONE_9,								// {9, 5*3600+1800+900,"GMT+05:45"}
	DH_TIME_ZONE_10,							// {10, 6*3600,"GMT+06:00"}
	DH_TIME_ZONE_11,							// {11, 6*3600+1800,"GMT+06:30"}
	DH_TIME_ZONE_12,							// {12, 7*3600,"GMT+07:00"}
	DH_TIME_ZONE_13,							// {13, 8*3600,"GMT+08:00"}
	DH_TIME_ZONE_14,							// {14, 9*3600,"GMT+09:00"}
	DH_TIME_ZONE_15,							// {15, 9*3600+1800,"GMT+09:30"}
	DH_TIME_ZONE_16,							// {16, 10*3600,"GMT+10:00"}
	DH_TIME_ZONE_17,							// {17, 11*3600,"GMT+11:00"}
	DH_TIME_ZONE_18,							// {18, 12*3600,"GMT+12:00"}
	DH_TIME_ZONE_19,							// {19, 13*3600,"GMT+13:00"}
	DH_TIME_ZONE_20,							// {20, -1*3600,"GMT-01:00"}
	DH_TIME_ZONE_21,							// {21, -2*3600,"GMT-02:00"}
	DH_TIME_ZONE_22,							// {22, -3*3600,"GMT-03:00"}
	DH_TIME_ZONE_23,							// {23, -3*3600-1800,"GMT-03:30"}
	DH_TIME_ZONE_24,							// {24, -4*3600,"GMT-04:00"}
	DH_TIME_ZONE_25,							// {25, -5*3600,"GMT-05:00"}
	DH_TIME_ZONE_26,							// {26, -6*3600,"GMT-06:00"}
	DH_TIME_ZONE_27,							// {27, -7*3600,"GMT-07:00"}
	DH_TIME_ZONE_28,							// {28, -8*3600,"GMT-08:00"}
	DH_TIME_ZONE_29,							// {29, -9*3600,"GMT-09:00"}
	DH_TIME_ZONE_30,							// {30, -10*3600,"GMT-10:00"}
	DH_TIME_ZONE_31,							// {31, -11*3600,"GMT-11:00"}
	DH_TIME_ZONE_32,							// {32, -12*3600,"GMT-12:00"}
} DH_TIME_ZONE_TYPE;

typedef enum _SNAP_TYPE
{
	SNAP_TYP_TIMING = 0,
	SNAP_TYP_ALARM,
	SNAP_TYP_NUM,
} SNAP_TYPE;

typedef enum _CONNECT_STATE
{
	CONNECT_STATE_UNCONNECT  = 0,	// no connected 
	CONNECT_STATE_CONNECTING,		// connecting
	CONNECT_STATE_CONNECTED,		// connected
	CONNECT_STATE_EMPTY,			// has connected, but config information is empty
	CONNECT_STATE_CHANGED,			// config information has changed
	CONNECT_STATE_ERROR = 255,		// unknown state
} CONNECT_STATE;

// Snap mode
typedef enum tagDH_TRAFFIC_SNAP_MODE
{	
	DH_TRAFFIC_SNAP_MODE_COIL = 1,				       // Loop snap
	DH_TRAFFIC_SNAP_MODE_COIL_PICANALYSIS,		       // Loop snap   picture analysis
	DH_TRAFFIC_SNAP_MODE_STREAM,				       // Video snap
	DH_TRAFFIC_SNAP_MODE_STREAM_IDENTIFY,		       // Video snap and recognize
} DH_TRAFFIC_SNAP_MODE;

// carport light type
typedef enum 
{
	NET_CARPORTLIGHT_TYPE_RED,                           // red
	NET_CARPORTLIGHT_TYPE_GREEN,                         // green
}NET_CARPORTLIGHT_TYPE;

// carport light mode
typedef enum
{
	NET_CARPORTLIGHT_MODE_OFF,                           // off 
	NET_CARPORTLIGHT_MODE_ON,                            // on
	NET_CARPORTLIGHT_MODE_GLINT,                         // glint
}NET_CARPORTLIGHT_MODE;

/////////////////////////////////Intelligent transportation related/////////////////////////////////
//Black and white list type operation
typedef enum _EM_OPERATE_TYPE
{
	NET_TRAFFIC_LIST_INSERT	    ,								// Increase the record operation
	NET_TRAFFIC_LIST_UPDATE		,								// Record update operation
	NET_TRAFFIC_LIST_REMOVE		,								// Delete the record operation
	NET_TRAFFIC_LIST_MAX		,
}EM_RECORD_OPERATE_TYPE ;

//License plate type
typedef enum _EM_NET_PLATE_TYPE
{
	NET_PLATE_TYPE_UNKNOWN					,
	NET_PLATE_TYPE_NORMAL					,		// "Normal" Blue card black card
	NET_PLATE_TYPE_YELLOW					,		// "Yellow" yellow card
	NET_PLATE_TYPE_DOUBLEYELLOW				,		// "DoubleYellow" Double yellow back card
	NET_PLATE_TYPE_POLICE					,		// "Police" Police card
	NET_PLATE_TYPE_ARMED					,		// "Armed" Armed card
	NET_PLATE_TYPE_MILITARY					,		// "Military" Force plate
	NET_PLATE_TYPE_DOUBLEMILITARY			,		// "DoubleMilitary" Forces double
	NET_PLATE_TYPE_SAR						,		// "SAR" Hong Kong and Macao SAR plate	
	NET_PLATE_TYPE_TRAINNING				,		// "Trainning" Drivers Ed plate
	NET_PLATE_TYPE_PERSONAL					,		// "Personal" Personality plate
	NET_PLATE_TYPE_AGRI						,		// "Agri" Agri-using card
	NET_PLATE_TYPE_EMBASSY					,		// "Embassy" The embassy of plate
	NET_PLATE_TYPE_MOTO						,		// "Moto" Motorcycle plate
	NET_PLATE_TYPE_TRACTOR					,		// "Tractor" The tractor plate
	NET_PLATE_TYPE_OFFICIALCAR				,		// "OfficialCar " officer's car
	NET_PLATE_TYPE_PERSONALCAR				,		// "PersonalCar" private car
	NET_PLATE_TYPE_WARCAR					,		// "WarCar"  for military use
	NET_PLATE_TYPE_OTHER					,		// "Other" The other plate
	NET_PLATE_TYPE_CIVILAVIATION			,       // "Civilaviation" Civilaviation
	NET_PLATE_TYPE_BLACK					,       // "Black" 
	NET_PLATE_TYPE_PURENEWENERGYMICROCAR	,       // "PureNewEnergyMicroCar" pure new energy micro car
	NET_PLATE_TYPE_MIXEDNEWENERGYMICROCAR	,       // "MixedNewEnergyMicroCar" mixed new energy micro car
	NET_PLATE_TYPE_PURENEWENERGYLARGECAR	,       // "PureNewEnergyLargeCar" pure new energy large car
	NET_PLATE_TYPE_MIXEDNEWENERGYLARGECAR	,       // "MixedNewEnergyLargeCar" mixed new energy large car

}EM_NET_PLATE_TYPE;


//The license plate color
typedef enum _EM_NET_PLATE_COLOR_TYPE
{
	NET_PLATE_COLOR_OTHER 						,				//	other colors
	NET_PLATE_COLOR_BLUE						,				//	blue		"Blue"
	NET_PLATE_COLOR_YELLOW						,				//	yellow		"Yellow"	
	NET_PLATE_COLOR_WHITE						,				//	white		"White"
	NET_PLATE_COLOR_BLACK						,				//	black		"Black"
	NET_PLATE_COLOR_YELLOW_BOTTOM_BLACK_TEXT	,				//	Yellow Bottom Positive Figure	"YellowbottomBlackText"
	NET_PLATE_COLOR_BLUE_BOTTOM_WHITE_TEXT		,				//	blue-mask LCD" 
	NET_PLATE_COLOR_BLACK_BOTTOM_WHITE_TEXT		,				//	White on Black	"BlackBottomWhiteText"
	NET_PLATE_COLOR_SHADOW_GREEN				,				//  Shadow Green
	NET_PLATE_COLOR_YELLOW_GREEN				,				//  YellowGreen
}EM_NET_PLATE_COLOR_TYPE;

//vehicle type
typedef enum _EM_NET_VEHICLE_TYPE
{
	NET_VEHICLE_TYPE_UNKNOW					  ,				//  unknown type
	NET_VEHICLE_TYPE_MOTOR					  ,				// "Motor" Motor vehicles"		   
	NET_VEHICLE_TYPE_NON_MOTOR				  ,				// "Non-Motor"non-Motor vehicles"		
	NET_VEHICLE_TYPE_BUS					  ,				// "Bus"bus		
	NET_VEHICLE_TYPE_BICYCLE				  ,				// "Bicycle"Bicycle		
    NET_VEHICLE_TYPE_MOTORCYCLE               ,             // "Motorcycle"        
	NET_VEHICLE_TYPE_UNLICENSEDMOTOR		  ,				// "UnlicensedMotor": A motor vehicle without a license
	NET_VEHICLE_TYPE_LARGECAR				  ,				// "LargeCar"  LargeCar
	NET_VEHICLE_TYPE_MICROCAR				  ,				// "MicroCar" MicroCar
	NET_VEHICLE_TYPE_EMBASSYCAR				  ,				// "EmbassyCar" EmbassyCa
	NET_VEHICLE_TYPE_MARGINALCAR			  ,				// "MarginalCar" MarginalCar
	NET_VEHICLE_TYPE_AREAOUTCAR				  ,				// "AreaoutCar" AreaoutCar
	NET_VEHICLE_TYPE_FOREIGNCAR				  ,				// "ForeignCar" ForeignCar
	NET_VEHICLE_TYPE_DUALTRIWHEELMOTORCYCLE	  ,				// "DualTriWheelMotorcycle"Two or three rounds of motorcycle
	NET_VEHICLE_TYPE_LIGHTMOTORCYCLE		  , 			// "LightMotorcycle"  light motorcycle
	NET_VEHICLE_TYPE_EMBASSYMOTORCYCLE		  ,				// "EmbassyMotorcycle "The embassy of the motorcycle
	NET_VEHICLE_TYPE_MARGINALMOTORCYCLE		  ,				// "MarginalMotorcycle "Consulate motorcycle
	NET_VEHICLE_TYPE_AREAOUTMOTORCYCLE		  ,				// "AreaoutMotorcycle "Outside the motorcycle
	NET_VEHICLE_TYPE_FOREIGNMOTORCYCLE		  ,				// "ForeignMotorcycle "Foreign motorcycle
	NET_VEHICLE_TYPE_FARMTRANSMITCAR		  ,				// "FarmTransmitCar" agricultural vehicle
	NET_VEHICLE_TYPE_TRACTOR				  ,				// "Tractor" tractor
	NET_VEHICLE_TYPE_TRAILER				  ,				// "Trailer"  trailer
	NET_VEHICLE_TYPE_COACHCAR				  ,				// "CoachCar "Car coach
	NET_VEHICLE_TYPE_COACHMOTORCYCLE		  ,				// "CoachMotorcycle " coach Motorcycle
	NET_VEHICLE_TYPE_TRIALCAR				  ,				// "TrialCar" trial car 
    NET_VEHICLE_TYPE_TRIALMOTORCYCLE,                   	// "TrialMotorcycle "Test motorcycle
	NET_VEHICLE_TYPE_TEMPORARYENTRYCAR		  ,				// "TemporaryEntryCar"Temporary entry vehicle
	NET_VEHICLE_TYPE_TEMPORARYENTRYMOTORCYCLE ,				// "TemporaryEntryMotorcycle"Temporary entry of motorcycle
	NET_VEHICLE_TYPE_TEMPORARYSTEERCAR		  ,			    // "TemporarySteerCar"Temporary driving car
	NET_VEHICLE_TYPE_PASSENGERCAR			  ,				// "PassengerCar" passenger car
	NET_VEHICLE_TYPE_LARGETRUCK				  ,				// "LargeTruck" LargeTruck
	NET_VEHICLE_TYPE_MIDTRUCK				  ,				// "MidTruck" MidTruck
	NET_VEHICLE_TYPE_SALOONCAR				  ,				// "SaloonCar" SaloonCar
	NET_VEHICLE_TYPE_MICROBUS                 ,				// "Microbus" Microbus
	NET_VEHICLE_TYPE_MICROTRUCK				  ,				// "MicroTruck" MicroTruck
    NET_VEHICLE_TYPE_TRICYCLE				  ,				// "Tricycle" Tricycle
	NET_VEHICLE_TYPE_PASSERBY				  ,				// "Passerby" Passerby
}EM_NET_VEHICLE_TYPE;


//body color
typedef enum _EM_NET_VEHICLE_COLOR_TYPE
{
	NET_VEHICLE_COLOR_OTHER					,				//other color	
	NET_VEHICLE_COLOR_WHITE					,				//white		"White"
	NET_VEHICLE_COLOR_BLACK					,				//black		"Black"
	NET_VEHICLE_COLOR_RED					,				//red		"Red"
	NET_VEHICLE_COLOR_YELLOW				,				//yellow	"Yellow"
	NET_VEHICLE_COLOR_GRAY					,				//gray		"Gray"
	NET_VEHICLE_COLOR_BLUE					,				//blue		"Blue"
	NET_VEHICLE_COLOR_GREEN					,				//green		"Green"
	NET_VEHICLE_COLOR_PINK					,				//pink		"Pink"
	NET_VEHICLE_COLOR_PURPLE				,				//purple	"Purple"
	NET_VEHICLE_COLOR_BROWN					,				//brown		"Brown"
}EM_NET_VEHICLE_COLOR_TYPE;

//Open the coil
typedef enum _EM_NET_TRAFFIC_CAR_CONTROL_TYPE
{
	NET_CAR_CONTROL_OTHER					,
	NET_CAR_CONTROL_OVERDUE_NO_CHECK		,				// Overdue inspection				"OverdueNoCheck"
	NET_CAR_CONTROL_BRIGANDAGE_CAR			,				// Stolen vehicles					"BrigandageCar"
	NET_CAR_CONTROL_BREAKING				,				// Break rules and regulations		"Breaking"
	NET_CAR_CONTROL_CAUSETROUBLE_ESCAPE		,				// Hit and run						"CausetroubleEscape"
}EM_NET_TRAFFIC_CAR_CONTROL_TYPE;

typedef enum _EM_NET_AUTHORITY_TYPE
{
	NET_AUTHORITY_UNKNOW					,
	NET_AUTHORITY_OPEN_GATE					,				// open gate
}EM_NET_AUTHORITY_TYPE;

typedef enum _EM_NET_RECORD_TYPE
{
  NET_RECORD_UNKNOWN,

  NET_RECORD_TRAFFICREDLIST,              // Traffic white list account record
                                          // search criteria corresponding to FIND_RECORD_TRAFFICREDLIST_CONDITION structure 
                                          // record info corresponding to NET_TRAFFIC_LIST_RECORD structure 
  NET_RECORD_TRAFFICBLACKLIST,            // Traffic black list account record
                                          // search criteria corresponding to FIND_RECORD_TRAFFICREDLIST_CONDITION structure 
                                          // record info corresponding to NET_TRAFFIC_LIST_RECORD structure 
  NET_RECORD_BURN_CASE,                   // burning case record
                                          // search criteria corresponding to FIND_RECORD_BURN_CASE_CONDITION structure 
                                          // record info corresponding to NET_BURN_CASE_INFO structure 
  NET_RECORD_ACCESSCTLCARD,               // access control card
                                          // search criteria corresponding to FIND_RECORD_ACCESSCTLCARD_CONDITION structure 
                                          // record info corresponding to NET_RECORDSET_ACCESS_CTL_CARD structure 
  NET_RECORD_ACCESSCTLPWD,                // access control password
                                          // search criteria corresponding to FIND_RECORD_ACCESSCTLPWD_CONDITION structure 
                                          // record info corresponding to NET_RECORDSET_ACCESS_CTL_PWD
  NET_RECORD_ACCESSCTLCARDREC,            // access control in/out record
                                          // search criteria corresponding to FIND_RECORD_ACCESSCTLCARDREC_CONDITION structure 
                                          // record info corresponding to 
  NET_RECORD_ACCESSCTLHOLIDAY,            // holiday record set
                                          // search criteria corresponding to FIND_RECORD_ACCESSCTLHOLIDAY_CONDITION structure 
                                          // record info corresponding to 
  NET_RECORD_TRAFFICFLOW_STATE,           // search Traffic flow record 
                                          // search criteria corresponding to FIND_RECORD_TRAFFICFLOW_CONDITION structure 
                                          // record info corresponding to NET_RECORD_TRAFFIC_FLOW_STATE structure 
  NET_RECORD_VIDEOTALKLOG,                // call record 
                                          // search criteria corresponding to FIND_RECORD_VIDEO_TALK_LOG_CONDITION structure 
                                          // record info corresponding to NET_RECORD_VIDEO_TALK_LOG structure 
  NET_RECORD_REGISTERUSERSTATE,           // status record 
                                          // search criteria corresponding to FIND_RECORD_REGISTER_USER_STATE_CONDITION structure 
                                          // record info corresponding to NET_RECORD_REGISTER_USER_STATE structure 
  NET_RECORD_VIDEOTALKCONTACT,            // contact record 
                                          // search criteria corresponding to FIND_RECORD_VIDEO_TALK_CONTACT_CONDITION structure 
														// record info corresponding to NET_RECORD_VIDEO_TALK_CONTACT structure 
	NET_RECORD_ANNOUNCEMENT,							// Record info corresponding to structure NET_RECORD_ANNOUNCEMENT_INFO 
														// No search criteria 
														
	NET_RECORD_ALARMRECORD,								// Record info corresponding to structure NET_RECORD_ALARMRECORD_INFO 
														// No search criteria 

    NET_RECORD_COMMODITYNOTICE,                         // Issue commodiy record 
                                                        // Record info corresponding to structure NET_RECORD_COMMODITY_NOTICE 
    NET_RECORD_HEALTHCARENOTICE,                        // Medical info  
                                                        // Record info corresponding to structure NET_RECORD_HEALTH_CARE_NOTICE
    NET_RECORD_ACCESSCTLCARDREC_EX,                     // A&C entry-exit record(can select some critera to search. Please replace NET_RECORD_ACCESSCTLCARDREC)
                                                        // Search criteria corresponding to structure FIND_RECORD_ACCESSCTLCARDREC_CONDITION_EX
                                                        // Record info corresponding to structure NET_RECORDSET_ACCESS_CTL_CARDREC

    NET_RECORD_GPS_LOCATION,                            // GPS position information reocrd, support import and clear only.
                                                        // Record info corresponding to structure NET_RECORD_GPS_LOCATION_INFO 

    NET_RECORD_RESIDENT,                                // resident info
    													// Record info corresponding to structure FIND_RECORD_RESIDENT_CONDTION    													
                                                        // Record info corresponding to NET_RECORD_RESIDENT_INFO 

	NET_RECORD_SENSORRECORD,                            // sensor record
														// Search criteria corresponding to structure FIND_RECORD_SENSORRECORD_CONDITION 
														// Record info corresponding to structure NET_RECORD_SENSOR_RECORD
	NET_RECORD_ACCESSQRCODE,							// AccessQRCode record 
														// Record info corresponding to structure NET_RECORD_ACCESSQRCODE_INFO

  	NET_RECORD_ELECTRONICSTAG,						  	// electronic tag info record
														// Search criteria corresponding to structure FIND_RECORD_ELECTRONICSTAG_CONDITION
														// Record info corresponding to NET_RECORD_ELECTRONICSTAG_INFO

    NET_RECORD_ACCESS_BLUETOOTH,					    // Access blue tooth record
                                                        // Search blue tooth access record corresponding to structure FIND_RECORD_ACCESS_BLUETOOTH_INFO_CONDITION
                                                        // Record info corresponding to structure NET_RECORD_ACCESS_BLUETOOTH_INFO
}EM_NET_RECORD_TYPE;

// time type
typedef enum
{
	NET_TIME_TYPE_ABSLUTE,                                  // absolute time  
	NET_TIME_TYPE_RELATIVE,                                 // Relative time, relative to the video file header frame as the time basis points, the first frame corresponding to the UTC (0000-00-00 00:00:00)
}EM_TIME_TYPE;

// color type
typedef enum
{
	NET_COLOR_TYPE_RED,                                     // red
	NET_COLOR_TYPE_YELLOW,                                  // yellow
	NET_COLOR_TYPE_GREEN,                                   // green
	NET_COLOR_TYPE_CYAN,                                    // cyan
	NET_COLOR_TYPE_BLUE,                                    // glue
	NET_COLOR_TYPE_PURPLE,                                  // purple
	NET_COLOR_TYPE_BLACK,                                   // black
	NET_COLOR_TYPE_WHITE,                                   // white
	NET_COLOR_TYPE_MAX,
}EM_COLOR_TYPE;

/////////////////////////////////Face recognition related/////////////////////////////////
// Personnel type
typedef enum 
{
	PERSON_TYPE_UNKNOWN,
	PERSON_TYPE_NORMAL,                                     // common person
	PERSON_TYPE_SUSPICION,                                  // Suspects
	PERSON_TYPE_THIEF,                                      // Thieves
	PERSON_TYPE_VIP,                                        // VIP
	PERSON_TYPE_FATECHECK,                                  // FateCheck
    PERSON_TYPE_STAFF,                                      // Staff
}EM_PERSON_TYPE;

// ID type
typedef enum
{
	CERTIFICATE_TYPE_UNKNOWN,
	CERTIFICATE_TYPE_IC,                                    // ID
	CERTIFICATE_TYPE_PASSPORT,                              // passport 
}EM_CERTIFICATE_TYPE;

// Face recognition database operations
typedef enum
{
	NET_FACERECONGNITIONDB_UNKOWN, 
	NET_FACERECONGNITIONDB_ADD,							// Add personnel information and face samples, if researchers already exists, image data and the original data
	NET_FACERECONGNITIONDB_DELETE,						// Delete the personnel information and face samples
    NET_FACERECONGNITIONDB_MODIFY,                      // Modify person info and human face sample, must input person UID
    NET_FACERECONGNITIONDB_DELETE_BY_UID,               // Delete person info and human face via UID
}EM_OPERATE_FACERECONGNITIONDB_TYPE;

// Face contrast pattern
typedef enum 
{
	NET_FACE_COMPARE_MODE_UNKOWN,
	NET_FACE_COMPARE_MODE_NORMAL,                  // normal
	NET_FACE_COMPARE_MODE_AREA,                    // Specify the face region combination area
	NET_FACE_COMPARE_MODE_AUTO,                    // Intelligent model, the algorithm according to the situation of facial regions automatically select combination
}EM_FACE_COMPARE_MODE;

// Face region
typedef enum
{
	NET_FACE_AREA_TYPE_UNKOWN,
	NET_FACE_AREA_TYPE_EYEBROW,                    // eyebrow
	NET_FACE_AREA_TYPE_EYE,                        // eye 
	NET_FACE_AREA_TYPE_NOSE,                       // nose
	NET_FACE_AREA_TYPE_MOUTH,                      // mouth
	NET_FACE_AREA_TYPE_CHEEK,                      // face
}EM_FACE_AREA_TYPE;

// face data type
typedef enum
{
	NET_FACE_DB_TYPE_UNKOWN,
	NET_FACE_DB_TYPE_HISTORY,						// History database, storage is to detect the human face information, usually does not contain face corresponding personnel information
	NET_FACE_DB_TYPE_BLACKLIST,						// The blacklist database
	NET_FACE_DB_TYPE_WHITELIST,						// The whitelist database
	NET_FACE_DB_TYPE_ALARM  ,						// Alarm library 
}EM_FACE_DB_TYPE;

// Face recognition event type
typedef enum 
{
	NET_FACERECOGNITION_ALARM_TYPE_UNKOWN,
	NET_FACERECOGNITION_ALARM_TYPE_ALL,                // blacklist and whitelist
	NET_FACERECOGNITION_ALARM_TYPE_BLACKLIST,          // The blacklist
	NET_FACERECOGNITION_ALARM_TYPE_WHITELIST,          // The whitelist
}EM_FACERECOGNITION_ALARM_TYPE;

// Face recognition face type
typedef enum
{
	EM_FACERECOGNITION_FACE_TYPE_UNKOWN,
	EM_FACERECOGNITION_FACE_TYPE_ALL,                  // All the faces 
	EM_FACERECOGNITION_FACE_TYPE_REC_SUCCESS,          // recognition success
	EM_FACERECOGNITION_FACE_TYPE_REC_FAIL,             // recognition fail
}EM_FACERECOGNITION_FACE_TYPE;

// Frame type enumeration values  
typedef enum __EM_FRAME_TYPE
{
	EM_FRAME_UNKOWN,                                   // unknown type
	EM_FRAME_TYPE_MOTION,                              // DM frame, corresponding frame information structure NET MOTION FRAM INFO
}EM_FRAME_TYPE;

/////////////////////////////////Cancelled Type/////////////////////////////////

// Configuration type. The interface that uses the enumberaiton has been canceled. Please do not use. 
typedef enum _CFG_INDEX
{
    CFG_GENERAL = 0,							// General 
	CFG_COMM,									// COM
	CFG_NET,									// Network
	CFG_RECORD,									// Record
	CFG_CAPTURE,								// Video setup
	CFG_PTZ,									// PTZ
	CFG_DETECT,									// Motion detection
	CFG_ALARM,									// Alarm 
	CFG_DISPLAY,								// Display 
	CFG_RESERVED,								// Reserved to keet type consecutive
	CFG_TITLE = 10,								// channel title 
	CFG_MAIL = 11,								// Mail function 
	CFG_EXCAPTURE = 12,							// preview video setup
	CFG_PPPOE = 13,								// pppoe setup
	CFG_DDNS = 14,								// DDNS  setup
	CFG_SNIFFER	= 15,							// Network monitor capture setup
	CFG_DSPINFO	= 16,							// Encode capacity information
	CFG_COLOR = 126,							// Color setup information 
	CFG_ALL,									// Reserved 
} CFG_INDEX;


/************************************************************************
 ** Structure Definition 
 ***********************************************************************/


//display area relative to the original display area coordinates
typedef struct
{
    double				dleft;					//left
	double				dright;					//right
	double				dtop;					//top
	double				dbottom;				//bottom
} DH_DISPLAYRREGION;
// Time
typedef struct 
{
	DWORD				dwYear;					// Year
	DWORD				dwMonth;				// Month
	DWORD				dwDay;					// Date
	DWORD				dwHour;					// Hour
	DWORD				dwMinute;				// Minute
	DWORD				dwSecond;				// Second
} NET_TIME,*LPNET_TIME;

typedef struct 
{
	DWORD				dwYear;					// Year
	DWORD				dwMonth;				// Month
	DWORD				dwDay;					// Date
	DWORD				dwHour;					// Hour
	DWORD				dwMinute;				// Minute
	DWORD				dwSecond;				// Second
	DWORD               dwMillisecond;          // Millisecond
	DWORD               dwReserved[2];          // reserved
} NET_TIME_EX,*LPNET_TIME_EX;

// The time definition in the log information
typedef struct _DHDEVTIME
{
	DWORD				second		:6;			// Second	1-60		
	DWORD				minute		:6;			// Minute	1-60		
	DWORD				hour		:5;			// Hour		1-24		
	DWORD				day			:5;			// Date		1-31		
	DWORD				month		:4;			// Month	1-12		
	DWORD				year		:6;			// Year	2000-2063	
} DHDEVTIME, *LPDHDEVTIME;

typedef struct tagRANGE
{
    float               fMax;                               // Max value
    float               fMin;                               // Min value
    BOOL                abStep;                             // Enable step or not 
    float               fStep;                              // Step 
    BOOL                abDefault;                          // Enable default value or not 
    float               fDefault;                           // Default value 
    char reserved[16];
} RANGE;

// callback data(Asynchronous interface)
typedef struct __NET_CALLBACK_DATA 
{
	int					nResultCode;			// Result code;0:Success
	char				*pBuf;					// Receive data,buffer is opened by the user,from the interface parameters,length is nRetLen
	int					nRetLen;				// the length of receive data
	LLONG				lOperateHandle;			// Operating handle
	void*				userdata;				// User parameters
	char				reserved[16];
} NET_CALLBACK_DATA, *LPNET_CALLBACK_DATA;

///////////////////////////////Monitor Related Definition ///////////////////////////////

// Frame parameter structure of Callback video data frame 
typedef struct _tagVideoFrameParam
{
	BYTE				encode;					// Encode type 
	BYTE				frametype;				// I = 0, P = 1, B = 2...
	BYTE				format;					// PAL - 0, NTSC - 1
	BYTE				size;					// CIF - 0, HD1 - 1, 2CIF - 2, D1 - 3, VGA - 4, QCIF - 5, QVGA - 6 ,
												// SVCD - 7,QQVGA - 8, SVGA - 9, XVGA - 10,WXGA - 11,SXGA - 12,WSXGA - 13,UXGA - 14,WUXGA - 15; LFT - 16, 720 - 17, 1080 - 18,
												// 1_3M-19, 2M-20, 5M-21;when size equal to 255, width and height valid
	DWORD				fourcc;					// If it is H264 encode it is always 0,Fill in FOURCC('X','V','I','D') in MPEG 4;
	WORD				width;					// width pixel, valid when struct member "size"  equal to 255
	WORD				height;					// height pixel, valid when struct member "size"  equal to 255
	NET_TIME			struTime;				// Time information 
} tagVideoFrameParam;


// Frame parameter structure of audio data callback 
typedef struct _tagCBPCMDataParam
{
	BYTE				channels;				// Track amount 
	BYTE				samples;				// sample 0 - 8000, 1 - 11025, 2 - 16000, 3 - 22050, 4 - 32000, 5 - 44100, 6 - 48000
	BYTE				depth;					// Sampling depth. Value:8/16 show directly
	BYTE				param1;					// 0 - indication no symbol,1-indication with symbol
	DWORD				reserved;				// Reserved
} tagCBPCMDataParam;

// Data structure of channel video title overlay 
typedef struct _DH_CHANNEL_OSDSTRING
{
	BOOL				bEnable;				// Enable 
	DWORD				dwPosition[MAX_STRING_LINE_LEN];//Character position in each line. The value ranges from 1 to 9.Corresponding to the small keyboard.
												//		7upper left 	8upper		9upper right 
												//		4left			5middle 	6right 
												//		1down left		2down	    3down right 
	char				szStrings[MAX_STRING_LINE_LEN][MAX_PER_STRING_LEN];	// Max 6 lines. Each line max 20 bytes.
} DH_CHANNEL_OSDSTRING;

// Para struct of YUV callback
typedef struct _tagCBYUVDataParam
{
	long				nWidth;                 // Width of image
	long				nHeight;				// Height of image
    NET_TIME            stNetTime;              // time in YUV stream
    DWORD               reserved[2];            // reserved
} tagCBYUVDataParam;

///////////////////////////////Definition relate with playback///////////////////////////////
// Type of video search
typedef enum tagEmQueryRecordType
{
    EM_RECORD_TYPE_ALL              = 0,            // All the recorded video  
    EM_RECORD_TYPE_ALARM            = 1,            // The video of external alarm
    EM_RECORD_TYPE_MOTION_DETECT    = 2,            // The video of dynamic detection alarm
    EM_RECORD_TYPE_ALARM_ALL        = 3,            // All the alarmed video
    EM_RECORD_TYPE_CARD             = 4,            // query by the card number
    EM_RECORD_TYPE_CONDITION        = 5,            // query by condition
    EM_RECORD_TYPE_JOIN             = 6,            // combination query 
    EM_RECORD_TYPE_CARD_PICTURE     = 8,            // query pictures by the card number, used by HB-U,NVS
    EM_RECORD_TYPE_PICTURE          = 9,            // query pictures, used by HB-U,NVS
    EM_RECORD_TYPE_FIELD            = 10,           // query by field
    EM_RECORD_TYPE_INTELLI_VIDEO    = 11,			// Smart record search 
    EM_RECORD_TYPE_NET_DATA         = 15,           // query network data, used by Jinqiao Internet Bar
    EM_RECORD_TYPE_TRANS_DATA       = 16,           // query the video of serial data
    EM_RECORD_TYPE_IMPORTANT        = 17,           // query the important video
    EM_RECORD_TYPE_TALK_DATA        = 18,           // query the recording file
    EM_RECORD_TYPE_POS				= 19,			// query the pos record

    EM_RECORD_TYPE_INVALID          = 256,          // invalid query type
    
}EM_QUERY_RECORD_TYPE;

// Record file information
typedef struct
{
    unsigned int		ch;						// Channel number
    char				filename[124];			// File name 
	unsigned int        framenum;               // the total number of file frames
    unsigned int		size;					// File length, unit: Kbyte 
    NET_TIME			starttime;				// Start time 
    NET_TIME			endtime;				// End time 
    unsigned int		driveno;				// HDD number 
    unsigned int		startcluster;			// Initial cluster number 
	BYTE				nRecordFileType;		// Recorded file type  0:general record;1:alarm record ;2:motion detection;3:card number record ;4:image ; 19:Pos record ;255:all
	BYTE                bImportantRecID;		// 0:general record 1:Important record
	BYTE                bHint;					// Document Indexing
	BYTE                bRecType;               // 0-main stream record 1-sub1 stream record 2-sub2 stream record 3-sub3 stream record
} NET_RECORDFILE_INFO, *LPNET_RECORDFILE_INFO;

// info of enrichment record file 
typedef struct tagNET_SynopsisFileInfo
{
	DWORD				dwSize;					// struct size 
    char				szFileName[MAX_PATH];	// file name,like :\a.dav
    NET_TIME			stuStartTime;			// start time
    NET_TIME			stuEndTime;				// end time
	unsigned int		nTaskID;				// server mark,with szFileName[] a choise
	BYTE				bFileType;				// 1-record file, 2- source file
	BYTE				byMode;					// Download mode: 0 - by file downloads, 1 - according to the time to download, 2 - according to download file offset
	BYTE				bReserved[2];
	unsigned int		nFileLength;			// file length (byte)
	unsigned int		nStartFileOffset;		// Starting file offset, unit: KB
	unsigned int		nEndFileOffset;			// The end of the file offset, the unit: KB
    int                 nChannel;               // channel, used for NVR playback(no TaskID), invalid for concentration server
    int                 nCluster;               // Cluster, used for NVR playback(no TaskID), invalid for concentration server
}NET_SYNOPSISFILE_INFO, *LPNET_SYNOPSISFILE_INFO;

// Playback data callback function prototype
// pBuffer: data buffer, memory malloc or free was managed by SDK interior
typedef int (CALLBACK *fDataCallBack)(LLONG lRealHandle, DWORD dwDataType, BYTE *pBuffer, DWORD dwBufSize, LDWORD dwUser);

typedef struct __NET_MULTI_PLAYBACK_PARAM 
{
	DWORD                      dwSize; 
	int                        nChannels[DH_MAX_MULTIPLAYBACK_CHANNEL_NUM]; // Preview the channel number
	int                        nChannelNum;                         // Preview the channel number
	int                        nType;                               // Playback file type, 0: ordinary video; 1: alarm video; 2: the mobile detection; 3: video card number; Picture 4:
	NET_TIME                   stStartTime;                         // Playback start time
	NET_TIME                   stEndTime;                           // The playback end time
	int                        nFPS;                                // Frame rate, 1 ~ 25
	int                        nBitRate;                            // Code flow values, 192 ~ 1024
	char                       szResolution[DH_MAX_CAPTURE_SIZE_NUM]; // Resolution, "D1", "HD1", "2 CIF", "CIF", "QCIF"
	int                        nWaitTime;                           // Timeout waiting time
	HWND                       hWnd;                                // Video playback window handle
	fDataCallBack              fDownLoadDataCallBack;               // Video data correction
	LDWORD                     dwDataUser;                          // 
}NET_MULTI_PLAYBACK_PARAM;

// record state of everyday in one month
typedef struct
{
	BYTE 	    flag[32];						// has record this day 0-no, 1-yes
	BYTE		Reserved[64];					// reserved
}NET_RECORD_STATUS, *LPNET_RECORD_STATUS;

// Asynchronous query results callback function prototype, nError = 0 means query success, nError = 1 said memory application failure, timeout nError = 2, nError = 3 said equipment return data validation is not through, nError = 4 send query request failed
typedef void (CALLBACK *fQueryRecordFileCallBack)(LLONG lQueryHandle, LPNET_RECORDFILE_INFO pFileinfos, int nFileNum, int nError, void *pReserved, LDWORD dwUser);

// CLIENT_StartQueryRecordFile Interface input parameters
typedef struct tagNET_IN_START_QUERY_RECORDFILE
{ 
	DWORD               dwSize;                            // The structure size
	int                 nChannelId;                        // To query the channel number
	int                 nRecordFileType;                   // For video query types
	int                 nStreamType;                       // Query stream type, 0 to advocate complementary code stream, 1 - the main stream, 2 - auxiliary stream
	NET_TIME            stStartTime;                       // Query starting time
	NET_TIME            stEndTime;                         // Query the end time
    char*               pchCardid;                         // card id information,the space application is 512 bytes by the user
														   // if nRecordFileType is 4 or 5 or 10,memory is 256 at most
														   // if nRecordFileType is 8,memory is 20 at most
	int                 nWaitTime;                         // Timeout waiting time, ms
	fQueryRecordFileCallBack cbFunc;                       // The query results callback function 
	LDWORD              dwUser;                            // user info
	BOOL				bByTime;						   // weather query by time
}NET_IN_START_QUERY_RECORDFILE;

typedef struct tagNET_OUT_START_QUERY_RECORDFILE
{
	DWORD                dwSize;                           //The structure size
	LLONG                lQueryHandle;                     //return handle    
}NET_OUT_START_QUERY_RECORDFILE;

//  function prototype of play back progress 
// dwDownLoadSize == -1 means progress finished
// dwDownLoadSize ==- 2 means no authority of operation
typedef void (CALLBACK *fDownLoadPosCallBack)(LLONG lPlayHandle, DWORD dwTotalSize, DWORD dwDownLoadSize, LDWORD dwUser);
// record play back parameter in
typedef struct tagNET_IN_PLAY_BACK_BY_TIME_INFO
{
    NET_TIME            stStartTime;                       // Begin time
    NET_TIME            stStopTime;                        // End time
    HWND                hWnd;                              // Play window
    fDownLoadPosCallBack cbDownLoadPos;                    // Download pos callback
    LDWORD              dwPosUser;                         // Pos user
    fDataCallBack       fDownLoadDataCallBack;             // Download data callback
    LDWORD              dwDataUser;                        // Data user
    int                 nPlayDirection;                    // Playback direction
    int                 nWaittime;                         // Watiting time
    BYTE                bReserved[1024];                   // Reserved
}NET_IN_PLAY_BACK_BY_TIME_INFO;
// record play back parameter out
typedef struct tagNET_OUT_PLAY_BACK_BY_TIME_INFO
{
    BYTE                bReserved[1024];                   // Reserved
}NET_OUT_PLAY_BACK_BY_TIME_INFO;

// record play back speed
typedef enum tagEM_PLAY_BACK_SPEED
{
	EM_PLAY_BACK_SPEED_SLOW_16 = -4,               // 1/16x slow speed
	EM_PLAY_BACK_SPEED_SLOW_8,                     // 1/8x slow speed
	EM_PLAY_BACK_SPEED_SLOW_4,                     // 1/4x slow speed
	EM_PLAY_BACK_SPEED_SLOW_2,                     // 1/2x slow speed
	EM_PLAY_BACK_SPEED_NORMAL = 0,                 // normal speed
	EM_PLAY_BACK_SPEED_FAST_2,                     // 2x fast speed
	EM_PLAY_BACK_SPEED_FAST_4,                     // 4x fast speed
	EM_PLAY_BACK_SPEED_FAST_8,                     // 8x fast speed
	EM_PLAY_BACK_SPEED_FAST_16,                    // 16x fast speed
}EM_PLAY_BACK_SPEED;

typedef struct
{
	unsigned short      left;                   // 0~8192
    unsigned short      right;                  // 0~8192
    unsigned short      top;                    // 0~8192
    unsigned short      bottom;                 // 0~8192
} MotionDetectRect;

// Smart Playback Information
typedef struct 
{
	MotionDetectRect    motion_rect;             // MotionDetect area
    NET_TIME            stime;                   // PlayBack start time
    NET_TIME            etime;                   // PlayBack stop time
	BYTE                bStart;                  // 1,start,2:stop
    BYTE                reserved[116];
} IntelligentSearchPlay, *LPIntelligentSearchPlay;

// The first recording time
typedef struct  
{
	int					nChnCount;				// Channel amount
	NET_TIME			stuFurthestTime[16];	// The first recording time, valid value is 0 to (nChnCount-1).If there is no video, the first recording time is 0.
	DWORD				dwFurthestTimeAllSize;	// when channel >16,use this field.means pStuFurthestTimeAll memory size.
	NET_TIME*			pStuFurthestTimeAll;	// when channel >16,use this field.need user apply, memory size(nChnCount*sizeof(NET_TIME)).
	BYTE				bReserved[376];			// Reserved words
} NET_FURTHEST_RECORD_TIME;

// CLIENT_FindFramInfo Interface input parameters
typedef struct __NET_IN_FIND_FRAMEINFO_PRAM
{
	DWORD                 dwSize;               // The structure size 
	BOOL                  abFileName;           // Whether the file name as a valid query conditions, if the file name is valid, is don't have to fill the file information (stRecordInfo)
	char                  szFileName[MAX_PATH]; // file name
	NET_RECORDFILE_INFO   stuRecordInfo;        // file information
	DWORD                 dwFramTypeMask;       // Frame type mask, see the "frame type mask defined"
}NET_IN_FIND_FRAMEINFO_PRAM;

// CLIENT_FindFramInfo Interface input parameters
typedef struct __NET_OUT_FIND_FRAMEINFO_PRAM
{
	DWORD                 dwSize;               // The structure size 
	LLONG                 lFindHandle;          // File search handle
}NET_OUT_FIND_FRAMEINFO_PRAM;

// DM frame information
typedef struct __NET_MOTION_FRAME_INFO
{
	DWORD                 dwSize;               // The structure size 
	NET_TIME              stuTime;              // The current frame, timestamp
	int					  nMotionRow;		    // The number of rows dynamic detection area
	int					  nMotionCol;		    // The number of columns of dynamic detection area
	BYTE				  byRegion[DH_MOTION_ROW][DH_MOTION_COL];// Detection area, up to 32 * 32 area
}NET_MOTION_FRAME_INFO;

// file frame information
typedef struct __NET_FILE_FRAME_INFO
{
	DWORD                 dwSize;               // The structure size 
	int                   nChannelId;           // channel number 
	NET_TIME              stuStartTime;         // the starting time 
	NET_TIME              stuEndTime;           // the ending time
	WORD                  wRecType;             // 0- main stream video code 1-auxiliary stream video 2- auxiliary stream 3 auxiliary stream video
	WORD                  wFameType;            // Frame TYPE, see EM FRAM TYPE
	void*                 pFramInfo;            // Corresponding to the type of frame information, the space application is sizeof(NET_MOTION_FRAME_INFO)
}NET_FILE_FRAME_INFO;

// CLIENT_FindNextFramInfo Interface input parameters
typedef struct __NET_IN_FINDNEXT_FRAMEINFO_PRAM
{
	DWORD                 dwSize;               // The structure size
	int                   nFramCount;           // To query the frame number, 0, said all the frame information query conforms to the query conditions
}NET_IN_FINDNEXT_FRAMEINFO_PRAM;

// CLIENT_FindNextFramInfo Interface output parameters
typedef struct __NET_OUT_FINDNEXT_FRAMEINFO_PRAM
{
	DWORD                 dwSize;               // The structure size 
	NET_FILE_FRAME_INFO*  pFramInfos;           // Frame information, by the user application space, the space size of sizeof (.net FILE FRAM INFO) * nMaxFramCount
	int                   nMaxFramCount;        // The number of the frame of information in the user application
    int                   nRetFramCount;        // The actual returns the number of frame information
}NET_OUT_FINDNEXT_FRAMEINFO_PRAM;

#define	MAX_LABEL_ARRAY		1024

// label array
typedef struct tagNET_FILE_STREAM_TAG_INFO
{
	DWORD				dwSize;									// The structure size 
	NET_TIME			stuTime;								// time
	char				szContext[DH_COMMON_STRING_64];			// context, chinese must be used UTF8 encoding
	char				szUserName[DH_COMMON_STRING_32];		// user name, chinese must be used UTF8 encoding
	char				szChannelName[DH_COMMON_STRING_64];		// channel name, chinese must be used UTF8 encoding
} NET_FILE_STREAM_TAG_INFO;

// file type
typedef enum tagNET_FILE_STREAM_TYPE
{
	NET_FILE_STREAM_TYPE_UNKNOWN = 0,				// unknown
	NET_FILE_STREAM_TYPE_NORMAL,					// noamal
	NET_FILE_STREAM_TYPE_ALARM,						// alarm
	NET_FILE_STREAM_TYPE_DETECTION,					// detection
} NET_FILE_STREAM_TYPE;

// file stream tag info
typedef struct tagNET_FILE_STREAM_TAG_INFO_EX
{
	DWORD					dwSize;	
	NET_TIME				stuTime;									// time
	int						nMillisecond;								// millisecond
	int						nSequence;									// Sequence
	char					szContext[DH_COMMON_STRING_64];				// context, chinese must be used UTF8 encoding
	NET_TIME				stuStartTime;								// recode file start time 
	NET_TIME				stuEndTime;									// recode file end time 
	NET_FILE_STREAM_TYPE	emType;										// file type
	char					szUserName[DH_COMMON_STRING_32];			// user name, chinese must be used UTF8 encoding
	char					szChannelName[DH_COMMON_STRING_64];			// channel name, chinese must be used UTF8 encoding
} NET_FILE_STREAM_TAG_INFO_EX;


// CLIENT_FileStreamClearTags / CLIENT_FileStreamSetTags Interface input parameters
typedef struct tagNET_IN_FILE_STREAM_TAGS_INFO
{
	DWORD						dwSize;								// The structure size 
	int							nArrayCount;						// count of user malloced, unit: sizeof(NET_FILE_STREAM_TAG_INFO)
	NET_FILE_STREAM_TAG_INFO*	pstuTagInfo;						// label array, user malloc the memory,apply to sizeof(NET_FILE_STREAM_TAG_INFO)*nArrayCount
} NET_IN_FILE_STREAM_TAGS_INFO;

// CLIENT_FileStreamClearTags / CLIENT_FileStreamSetTags Interface output parameters
typedef struct tagNET_OUT_FILE_STREAM_TAGS_INFO
{
	DWORD						dwSize;					// The structure size 
} NET_OUT_FILE_STREAM_TAGS_INFO;


// CLIENT_FileStreamGetTags Interface input parameters
typedef struct tagNET_IN_FILE_STREAM_GET_TAGS_INFO
{
	DWORD						dwSize;					// The structure size 
} NET_IN_FILE_STREAM_GET_TAGS_INFO;

// CLIENT_FileStreamGetTags / CLIENT_FileStreamfilterTags Interface output parameters
typedef struct tagNET_OUT_FILE_STREAM_GET_TAGS_INFO
{
	DWORD							dwSize;								// The structure size 
	int								nMaxNumber;							// max tags Number
	int								nRetTagsNumber;						// returned  tags Number
	NET_FILE_STREAM_TAG_INFO_EX*	pstuTagInfo;						// label array
} NET_OUT_FILE_STREAM_GET_TAGS_INFO;

// CLIENT_FileStreamFilterTags Interface input parameters
typedef struct tagNET_IN_FILE_STREAM_FILTER_TAGS_INFO
{
	DWORD						dwSize;									// The structure size 
	char						szContext[DH_COMMON_STRING_64];			// context, chinese must be used UTF8 encoding
	char						szUserName[DH_COMMON_STRING_32];		// user name, chinese must be used UTF8 encoding
	char						szChannelName[DH_COMMON_STRING_64];		// channel name, chinese must be used UTF8 encoding
} NET_IN_FILE_STREAM_FILTER_TAGS_INFO;

// CLIENT_FileStreamfilterTags Interface output parameters
// typedef struct tagNET_OUT_FILE_STREAM_FILTER_TAGS_INFO
// {
// 	DWORD							dwSize;								// The structure size 
// 	NET_FILE_STREAM_TAG_INFO_EX		stuTagInfo[MAX_LABEL_ARRAY];		// label array
// 	int								nRetTagsCount;						// returned  tags counts
// } NET_OUT_FILE_STREAM_FILTER_TAGS_INFO;

// tag manager's sub tag info
typedef struct tagNET_TAGMANAGER_SUB_TAG_INFO
{
	NET_TIME					stuTime;								// sub tag's time
	char						szSubTagName[DH_COMMON_STRING_64];		// sub tag's name
	BYTE						byReserved[512];						// reserved
}NET_TAGMANAGER_SUB_TAG_INFO;

// tag manager's tag info. this tag info has start\end time of a tag, which is different with NET_FILE_STREAM_TAG_INFO_EX
typedef struct tagNET_TAGMANAGER_TAG_INFO
{
	char						szContext[DH_COMMON_STRING_64];			// tag's context	
	NET_TIME					stuStartTime;							// tag's startTime
	NET_TIME					stuEndTime;								// tag's endTime
	int							nSubTagVaildNum;						// sub tag array(stuSubTag)'s valid number
	NET_TAGMANAGER_SUB_TAG_INFO stuSubTag[MAX_SUB_TAG_NUM];				// sub tag's array
	BYTE						byReserved[1024];						// reserved
}NET_TAGMANAGER_TAG_INFO;

// condition info in find tag
typedef struct tagNET_FINDTAGS_CONDITION_INFO
{
	char						szContext[DH_COMMON_STRING_64];			// tag's context	
	BYTE						byReserved[1024];						// reserved
}NET_FINDTAGS_CONDITION_INFO;

// find tag info, CLIENT_FileStreamFindTags's in param 
typedef struct tagNET_IN_FINDTAGS_INFO
{
	DWORD							dwSize;
	BOOL							bConditionEnable;					// if this field is True, stuCondition is enabled, lToken is disenabled; if this field is False, stuCondition is disenabled, lToken is enabled
	NET_FINDTAGS_CONDITION_INFO		stuCondition;						// condition, used in first query
	UINT							lToken;								// query token, used from second query, get from first query's NET_OUT_FINDTAGS_INFO
	int								nOffset;							// offset
	int								nCount;								// count of a query, can not greater than 60
}NET_IN_FINDTAGS_INFO;

// tag manager's find tag info, CLIENT_FileStreamFindTags's out param
typedef struct tagNET_OUT_FINDTAGS_INFO
{
	DWORD							dwSize;
	UINT							lToken;									// query token, get from first query
	int								nTotal;									// tags' total count, queried in device
	int								nMaxTagsNumber;							// tag array(pstuTagInfo)'s max number, not less than NET_IN_FINDTAGS_INFO's nCount
	int								nRetTagsNumber;							// returned tags Number
	NET_TAGMANAGER_TAG_INFO*		pstuTagInfo;							// pointer to tag ayyay, user malloc the memory, the size is sizeof(NET_TAGMANAGER_TAG_INFO)*nMaxTagsNumber
}NET_OUT_FINDTAGS_INFO;

// tag manager's tag state
typedef struct tagNET_TAGMANAGER_TAGSTATE_INFO
{
	int							nChannel;								// Channel ID
	char						szContext[DH_COMMON_STRING_64];			// tag's context	
	NET_TIME					stuStartTime;							// start time of tag
	int							nSubTagVaildNum;						// sub tag array(stuSubTag)'s valid number
	NET_TAGMANAGER_SUB_TAG_INFO stuSubTag[MAX_SUB_TAG_NUM];				// sub tag's array
	BYTE						byReserved[1024];						// reserved
}NET_TAGMANAGER_TAGSTATE_INFO;

// tag manager's get tag state, CLIENT_TagManagerGetTagState's in param
typedef struct tagNET_IN_TAGMANAGER_GETTAGSTATE_INFO
{
	DWORD							dwSize;
}NET_IN_TAGMANAGER_GETTAGSTATE_INFO;

// tag manager's get tag state, CLIENT_TagManagerGetTagState's out param
typedef struct tagNET_OUT_TAGMANAGER_GETTAGSTATE_INFO
{
	DWORD							dwSize;
	int								nState;								// Tag's state, 0 means not in tag, 1 means in tag
	int								nMaxTagStaeNumber;					// tag array(pstuTagStateInfo) 's max number
	int								nRetTagStaeNumber;					// returned tags Number
	NET_TAGMANAGER_TAGSTATE_INFO*	pstuTagStateInfo;					// pointer to tag ayyay, user malloc the memory, the size is sizeof(NET_TAGMANAGER_TAGSTATE_INFO)*nMaxTagStaeNumber
}NET_OUT_TAGMANAGER_GETTAGSTATE_INFO;

// tag manager's start tag, CLIENT_TagManagerStartTag's in param
typedef struct tagNET_IN_TAGMANAGER_STARTTAG_INFO
{
	DWORD						dwSize;
	int							nChannel;								// Channel ID
	char						szContext[DH_COMMON_STRING_64];			// tag's context	
}NET_IN_TAGMANAGER_STARTTAG_INFO;

// tag manager's start tag, CLIENT_TagManagerStartTag's out param
typedef struct tagNET_OUT_TAGMANAGER_STARTTAG_INFO
{
	DWORD						dwSize;
}NET_OUT_TAGMANAGER_STARTTAG_INFO;

// tag manager's set sub tag, CLIENT_TagManagerSetSubTag's in param
typedef struct tagNET_IN_TAGMANAGER_SETSUBTAG_INFO
{
	DWORD						dwSize;
	int							nChannel;								// Channel ID
	char						szSubTagName[DH_COMMON_STRING_64];		// sub tag name	
}NET_IN_TAGMANAGER_SETSUBTAG_INFO;

// tag manager's set sub tag, CLIENT_TagManagerSetSubTag's out param
typedef struct tagNET_OUT_TAGMANAGER_SETSUBTAG_INFO
{
	DWORD						dwSize;
}NET_OUT_TAGMANAGER_SETSUBTAG_INFO;

// tag manager's stop tag, CLIENT_TagManagerStopTag's in param
typedef struct tagNET_IN_TAGMANAGER_STOPTAG_INFO
{
	DWORD						dwSize;
	int							nChannel;								// Channel ID
}NET_IN_TAGMANAGER_STOPTAG_INFO;

// tag manager's stop tag, CLIENT_TagManagerStopTag's out param
typedef struct tagNET_OUT_TAGMANAGER_STOPTAG_INFO
{
	DWORD						dwSize;
}NET_OUT_TAGMANAGER_STOPTAG_INFO;




///////////////////////////////Alarm Related Definition ///////////////////////////////

// General alarm informaiton 
typedef struct
{
	int					channelcount;
	int					alarminputcount;
	unsigned char		alarm[16];				// External alarm 
	unsigned char		motiondection[16];		// Motion detection 
	unsigned char		videolost[16];			// Video loss 
} NET_CLIENT_STATE;

// General alarm information
typedef struct
{
	int					channelcount;
	int					alarminputcount;
    unsigned char		alarm[32];				// External alarm 
	unsigned char		motiondection[32];		// Motion detection 
	unsigned char		videolost[32];			// Video loss 
	BYTE                bReserved[32];
} NET_CLIENT_STATE_EX;

// struct of input alarm
typedef struct
{
	DWORD              dwSize;
	int				   alarminputcount;
	DWORD              dwAlarmState[DH_MAX_CHANMASK]; //DWORD value is the state by bit of 32 channels,0-no alarm,1-alarm
}NET_CLIENT_ALARM_STATE;

// struct of video loss alarm
typedef struct
{
	DWORD              dwSize;
	int				   channelcount;
	DWORD              dwAlarmState[DH_MAX_CHANMASK]; //DWORD value is the state by bit of 32 channels,0-no alarm,1-alarm
}NET_CLIENT_VIDEOLOST_STATE;

// struct of motion alarm
typedef struct
{
	DWORD              dwSize;
	int				   channelcount;
	DWORD              dwAlarmState[DH_MAX_CHANMASK]; //DWORD value is the state by bit of 32 channels,0-no alarm,1-alarm
}NET_CLIENT_MOTIONDETECT_STATE;

// struct of blind alarm
typedef struct
{
	DWORD              dwSize;
	int				   channelcount;
	DWORD              dwAlarmState[DH_MAX_CHANMASK]; //DWORD value is the state by bit of 32 channels,0-no alarm,1-alarm
}NET_CLIENT_VIDEOBLIND_STATE;

// struct of querying detailed motion alarm
typedef struct
{
	DWORD              dwSize;
	int                nChannelID;				// channel id
	BOOL               bAlarm;					// alarm or not,value is TRUE/FALSE
	int                nLevel;					// alarm level,in 1/1000 as a unit
}NET_CLIENT_DETAILEDMOTION_STATE;

// Alarm IO control 
typedef struct 
{
	unsigned short		index;					// Port serial number 
	unsigned short		state;					// Port status 
} ALARM_CONTROL;

//Activation type 
typedef struct
{
	unsigned short		index;					// Port serial number 
	unsigned short		mode;					// Activation way(0:close.1:manual.2:auto); The SDK reserves the original setup if you do not set channel here. 
	BYTE				bReserved[28];			
} TRIGGER_MODE_CONTROL;

// Alarm decoder control 
typedef struct 
{
	int					decoderNo;				// Alarm decoder. It begins from 0. 
	unsigned short		alarmChn;				// Alarm output port. It begin from o., 
	unsigned short		alarmState;				// Alarm output status;1:open,0:close.
} DECODER_ALARM_CONTROL;

// Call No-response Alarm Event
typedef struct __ALARM_NO_RESPONSE_INFO
{
	DWORD		dwSize;
	char        szCallID[DH_MAX_CALLID];                             //Call ID
}ALARM_NO_RESPONSE_INFO;

// Alarm information of alarm upload function
typedef struct  
{
	DWORD				dwAlarmType;			// Alarm type,when dwAlarmType = DH_UPLOAD_EVENT,dwAlarmMask and bAlarmDecoderIndex are invalid.
	DWORD				dwAlarmMask;			// Alarm information subnet mask. Bit represents each alarm status
	char				szGlobalIP[DH_MAX_IPADDR_LEN];				// Client-end IP address 
	char				szDomainName[DH_MAX_DOMAIN_NAME_LEN];		// Client-end domain name
	int					nPort;					// The port client-end connected when upload alarm 
	char				szAlarmOccurTime[DH_ALARM_OCCUR_TIME_LEN];	// Alarm occurred time 
	BYTE				bAlarmDecoderIndex;		// It means which alarm decoder. It is valid when dwAlarmType = DH_UPLOAD_DECODER_ALARM.
    BYTE                bChannelIndex;          // Index of channelMask(0,1,2...)only bChannelIndex>0,dwHighAlarmMask is effective
                                                // eg, if bChannelIndex=1, dwHighAlarmMask represent alarm status of channel 33~64
    BYTE                bReserved[2];
    DWORD               dwHighAlarmMask;        // Alarm information subnet mask(channel>32). Bit represents each alarm status
    BYTE                bReservedSpace[8];
} NEW_ALARM_UPLOAD;

// Smoke alarm events, alarm center
typedef struct __ALARM_UPLOAD_SMOKESENSOR_INFO
{
	DWORD			dwSize;
	char			szGlobalIP[DH_MAX_IPADDR_LEN];				// Remote IP address
	char			szDomainName[DH_MAX_DOMAIN_NAME_LEN];		// The client domain name
	int				nPort;										// When alarm upload client connection port
	char			szAlarmOccurTime[DH_ALARM_OCCUR_TIME_LEN];	// The time of the alarm
	int             nChannel;									// Alarm channel Starting from 0
	BYTE            byHighTemperature;							// 1: high temperature alarm, 0: high temperature alarm
	BYTE            bySmoke;									// 1: smoke alarm, 0: smoke alarm over
}ALARM_UPLOAD_SMOKESENSOR_INFO;

// intelligent alarm center
typedef struct __ALARM_UPLOAD_IVS_INFO
{
	DWORD				dwSize;
	NET_TIME			stuTime;                                  // occur time	
	char				szDomainName[DH_MAX_DOMAIN_NAME_LEN];	  // client domain name
	int					nChannelID;                               // channel,from 0
	char				szType[MAX_PATH];                         // alarm type,check rule
	int					nState;                                   // alarm state, 0-reset,1-setting,2-pulse
    char                szRuleName[DH_COMMON_STRING_128];         // Rule Name  
    char                szIPAddress[DH_MAX_IPADDR_LEN_EX];        // Device IP Address  
    int                 nPort;                                    // Device Port No.
    char                szMacAddress[DH_MACADDR_LEN];             // Device-end mac address
    char                szPicFilePath[MAX_PATH];                  // Picture Storage Path
    int                 nPicFileNum;                              // Current alarm folder number
    int                 nUploadPicFileNum;                        // Uploaded FTP picture  number
    DWORD               dwChannelMask;                            // Snapshot video channel mask, 15 means this alarm has 1,2,3,4 channels 
} ALARM_UPLOAD_IVS_INFO;

// Alarm center external alarm extension info
typedef struct _ALARM_UPLOAD_ALARMEX_INFO
{
    DWORD           dwSize;
    NET_TIME        stuTime;                                    // Event time    
    char            szDomainName[DH_MAX_DOMAIN_NAME_LEN];       // Client domain name
    DWORD           dwAlarmMask;                                // Alarm info mask, by bit means each alarm channel status, 7 means 1,2,3 alarm input channel
    char            szDevMac[DH_MACADDR_LEN];                   // Device port mac address
    char            szPicFilePath[MAX_PATH];                    // Picture storage file path
    int             nPicFileNum;                                // Current alarm corresponding picture file quantity
    int             nUploadPicFileNum;                          // Uploaded FTP picture file quantity
    char            szAreaName[DH_COMMON_STRING_128];           // Zone name
    DWORD           dwChannelMask;                              // Snapshot video channel mask, 15 means this alarm has 1,2,3,4 channels 
}ALARM_UPLOAD_ALARMEX_INFO;

typedef enum _EM_LOGIN_FAILED_TYPE
{
    LOGIN_FAILED_UNKNOW = 0 ,
    LOGIN_FAILED_PASSWORD   ,           //Password error
    LOGIN_FAILED_USER       ,           //User error
    LOGIN_FAILED_TIMEOUT    ,           //Time out
    LOGIN_FAILED_RELOGGIN   ,           //Relogin
    LOGIN_FAILED_LOCKED     ,           //Account locked
    LOGIN_FAILED_BLACKLIST  ,           //Account in black list
    LOGIN_FAILED_ERROR_BUSY ,           //Resource is busy
    LOGIN_FAILED_VERSION    ,           //Vesion error 
}EM_LOGIN_FAILED_TYPE;

//Login failed alarm upload
typedef struct  __ALARM_UPLOAD_LOGIN_FAILED_INFO
{
    DWORD                   dwSize;
    EM_LOGIN_FAILED_TYPE    emFailedType;                       //Failed type
    char                    szDeviceIP[DH_COMMON_STRING_64];    //Device ip
    DWORD                   nDevicePort;                        //Device port
}ALARM_UPLOAD_LOGIN_FAILED_INFO;

// Recording-changed alarm information
typedef struct
{
	int					nChannel;				// Record channel number
	char				reserved[12];
} ALARM_RECORDING_CHANGED;

// Recording-changed alarm information(DH_ALARM_RECORD_CHANGED_EX)
typedef struct __ALARM_RECORD_CHANGED_INFO_EX
{
    int                 nAction;                // 0:start 1:stop
	int                 nChannel;               // channel
	BYTE                byReserved[1024];       // reserved 
} ALARM_RECORD_CHANGED_INFO_EX;

// MPT record file transfer progress(DH_ALARM_POLICE_RECORD_PROGRESS)
typedef struct tagALARM_POLICE_RECORD_PROGRESS_INFO
{
    int                 nAction;                // 0:pulse,-1:unknown
    int                 nUsbSlot;               // USB Port number
    UINT                nTotalSize;             // file total size,unit:KB
    UINT                nTransferSize;          // transfered size,unit,KB
    BYTE                byReserved[1024];       // reserved 
}ALARM_POLICE_RECORD_PROGRESS_INFO;

// MPT state type
typedef enum tagEM_MPT_STATE
{
    EM_MPT_STATE_UNKNOWN,                                       // unknown
    EM_MPT_STATE_PLUGIN_OUT,                                    // mpt plugin out
    EM_MPT_STATE_PLUGIN_IN_WORKING_PROPERLY,                    // mpt plugin in and working properly
    EM_MPT_STATE_PLUGIN_IN_STORAGE_ERROR,                       // mpt plugin in but storage error
}EM_MPT_STATE;

// MPT plugin in or out event(DH_ALARM_POLICE_PLUGIN)
typedef struct tagALARM_POLICE_PLUGIN_INFO
{
    int                 nAction;                                // 0:pulse,-1:unknown
    int                 nUsbSlot;                               // usb port number
    EM_MPT_STATE        eState;                                 // mpt state
    char                szSerialNumber[DH_COMMON_STRING_32];    // MPT SN
    BYTE                byReserved[1024];                       // reserved
}ALARM_POLICE_PLUGIN_INFO;

// Gps not aligned alarm(DH_ALARM_GPS_NOT_ALIGNED)
typedef struct tagALARM_GPS_NOT_ALIGNED_INFO
{
    int                                 nAction;                        // -1:unknown, 0:pulse, 1:start, 2:stop 
	NET_TIME_EX                         stuTime;                        // Event occurrence time
	BYTE                                byReserved[1024];               // reserved 
}ALARM_GPS_NOT_ALIGNED_INFO;

// WireLess(include wifi, 3G/4G) not connected alarm(DH_ALARM_WIRELESS_NOT_CONNECTED)
typedef struct tagALARM_WIRELESS_NOT_CONNECTED_INFO
{
    int                                 nAction;                        // -1:unknown, 0:pulse, 1:start, 2:stop 
	NET_TIME_EX                         stuTime;                        // Event occurrence time
	BYTE                                byReserved[1024];               // reserved 
}ALARM_WIRELESS_NOT_CONNECTED_INFO;

// offline log sync state
typedef enum tagEM_OFFLINE_LOGSYNC_STATE
{
    EM_OFFLINE_LOGSYNC_STATE_UNKNOWN,                             // unknown
    EM_OFFLINE_LOGSYNC_STATE_SUCCEEDED,                           // success
    EM_OFFLINE_LOGSYNC_STATE_FAILED,                              // failed
}EM_OFFLINE_LOGSYNC_STATE;

//event of offline log sync
typedef struct tagALARM_OFFLINE_LOGSYNC_INFO
{
    int                                 nAction;                        // -1:unknown, 0:pulse, 1:start, 2:stop   
	NET_TIME_EX                         stuTime;                        // event happen time
	int                                 nDevAddrs;                      // sub controller id
	EM_OFFLINE_LOGSYNC_STATE            emState;                        // synchro state
	BYTE                                byReserved[1024];               //  
}ALARM_OFFLINE_LOGSYNC_INFO;

#define NET_MAX_WINDINGID_NUM    8

// CoilFault alarm event
typedef struct __ALARM_WINGDING_INFO
{
	int                 nDriveWayID;            // road number
	int                 nWindingID;             // coil ID
	NET_TIME            stuTime;                // event happen time 
	int                 nState;                 // Device state,0 indicate fault recover,1 indicate fault happen
	DWORD				dwChannel;				// Channel of alarm
	char                reserve[28];
    int                 nWindingIDNum;          // Coil ID number
    int                 nWindingIDs[NET_MAX_WINDINGID_NUM];// Detailed coil ID 
} ALARM_WINGDING_INFO;

// traffic congestion alarm 
typedef struct __ALARM_TRAF_CONGESTION_INFO
{
	int                 nDriveWayID;            // road number
	int                 nCongestionLevel;       // traffic congestion grade:1,2,3,4,5,6;1 indicate the most one 
	NET_TIME            stuTime;                // event happen time 
	int                 nState;                 // Device state,0 indicate fault recover,1 indicate fault happen
	DWORD				dwChannel;				// Channel of alarm
	char                reserve[28];
} ALARM_TRAF_CONGESTION_INFO;

// traffic exception alarm
typedef struct __ALARM_TRAF_EXCEPTION_INFO
{
	int                 nDriveWayID;            // road number
	NET_TIME            stuTime;                // event happen time
	int                 nState;                 // Device state,0 indicate fault recover,1 indicate fault happen
	DWORD				dwChannel;				// Channel of alarm
	char                reserve[28];
} ALARM_TRAF_EXCEPTION_INFO;

// fill equipment alarm 
typedef struct __ALARM_EQUIPMENT_FILL_INFO
{
	int                 nDriveWayID;            // road number
	NET_TIME            stuTime;                // event happen time
	int                 nState;                 // Device state,0 indicate fault recover,1 indicate fault happen
	DWORD				dwChannel;				// Channel of alarm
	char                reserve[28];
} ALARM_EQUIPMENT_FILL_INFO;

// alarm arm disarm state info
typedef struct __ALARM_ARM_DISARM_STATE_INFO
{
	BYTE                bState;                 // arm-disarm,0 means disarm,1 means arm, 2 means ForceOn
	char                reserve[31];
} ALARM_ARM_DISARM_STATE_INFO;

// 3G flow exceed state info 
typedef struct __DHDEV_3GFLOW_EXCEED_STATE_INFO
{
	BYTE                bState;                 //  3G flow exceed state,0 means not exceed,1 means exceed
	char                reserve[31];
} DHDEV_3GFLOW_EXCEED_STATE_INFO;

// alarm of speed limit (DH_DEVSTATE_SPEED_LIMIT)
typedef struct __ALARM_SPEED_LIMIT
{
	char                szType[DH_SPEEDLIMIT_TYPE_LEN];   // alarm type oLowerSpeed, UpperSpeed
	int                 iSpeedLimit;            // limit speed KM/H 
	int                 iSpeed;                 // speed KM/H
	char                szCrossingID[DH_MAX_CROSSING_ID]; // crossing ID
	DWORD				dwLongitude;			// longitude(millionth degree, 0-360)
    DWORD				dwLatidude;				// latidude(millionth degree,0-180)
	DHDEVTIME           stTime;                 // time
	BYTE                bOffline;               // 0-in real time,1-to fill 
	char				reserve[19];
}ALARM_SPEED_LIMIT;

// alarm of over loading
typedef struct __ALARM_OVER_LOADING
{
    NET_TIME			stuCurTime;             // current time
    DWORD				dwLatidude;				// latidude(millionth degree,0-180)
    DWORD				dwLongitude;			// longitude(millionth degree, 0-360)
    char				szDriverNo[DH_VEHICLE_DRIVERNO_LEN];    // driver id
    DWORD				dwCurSpeed;             // current speed
    BYTE				byReserved[128]; 
}ALARM_OVER_LOADING;

// alarm of hard braking
typedef struct __ALARM_HARD_BRAKING
{
    NET_TIME			stuCurTime;             // current time
    DWORD				dwLatidude;				// latidude(millionth degree,0-180)
    DWORD				dwLongitude;			// longitude(millionth degree, 0-360)
    char				szDriverNo[DH_VEHICLE_DRIVERNO_LEN];    // driver id
    DWORD				dwCurSpeed;             // current speed
    BYTE				byReserved[128]; 
}ALARM_HARD_BRAKING;

// alarm of smoke sensor
typedef struct __ALARM_SMOKE_SENSOR
{
	NET_TIME			stuCurTime;             // current time
	int					nChannel;               // channel
	BYTE				byHighTemperature;      // 1:hight temperature alarm start,0:end
	BYTE				bySmoke;                // 1:smoke alarm start,0:end
	BYTE				byReservrd[126];
}ALARM_SMOKE_SENSOR;

// alarm of traffic light
typedef struct _LIGHT_INFO
{
	BYTE               byDirection;            // direction: 1-left,2-right,3-straight , 4-turn round
	BYTE               byState;                // light state: 1-fault,2-normal
	BYTE               byReserved[62];         // reserved
}LIGHT_INFO;

typedef struct __ALARM_TRAFFIC_LIGHT_FAULT 
{
	NET_TIME           stTime;                 // alarm time
	int                nInfoNumber;            // info number
	LIGHT_INFO         stLightInfo[8];         // info
	BYTE               byReserved[128];        // reserved
}ALARM_TRAFFIC_LIGHT_FAULT;

// Flux alarm channel information
typedef struct __ALARM_TRAFFIC_FLUX_LANE_INFO
{
	NET_TIME            stuCurTime;            // current time
	int                 nLane;                 // lane number
	int                 nState;                // state:1-traffic jam, 2-traffic jam recover, 3-normal 4-break, 5-break recover
	int                 nFlow;                 // flow value,unit:per/second
	BYTE                byReserved[124];       // reserved
}ALARM_TRAFFIC_FLUX_LANE_INFO;

// SIP state alarm
typedef struct __ALARM_SIP_STATE
{
	int					nChannelID;
	BYTE				byStatus;               //0:succeed,1:unregistered ,2:invalid,3:registing,4:talking
	BYTE				bReserved[63];			//reserved
}ALARM_SIP_STATE;

// vehicle info uploading (DH_DEVSTATE_VIHICLE_INFO_UPLOAD)
typedef struct __ALARM_VEHICLE_INFO_UPLOAD
{
    char                szType[DH_VEHICLE_TYPE_LEN];            //type: DriverCheck ,Driver check in or check out
	char                szCheckInfo[DH_VEHICLE_INFO_LEN];       //Check in, Check out
    char                szDirverNO[DH_VEHICLE_DRIVERNO_LEN];    //Driver NO. string
	DHDEVTIME           stTime;                                      // time
	BYTE                bOffline;                                    // 0-real time 1-to fill 
	char                reserved[59];                     
}ALARM_VEHICLE_INFO_UPLOAD;

// card record uploading
typedef struct __ALARM_CARD_RECORD_INFO_UPLOAD
{
	int					nChannel;				// channel id
	BOOL				bEnable;				// is recording
	char				szCardInfo[DH_MAX_CARD_INFO_LEN];	// card info
	NET_TIME			stuTime;				// card start time
	BOOL                bPreviewOverlayEn;      // preview overlay enable
	BYTE                byOverlayPos;           // overlay position,1-left top,2-left bottom,3-right top,4-right bottom
	char				reserved[59];
}ALARM_CARD_RECORD_INFO_UPLOAD;


typedef enum __ATMTradeTypes{
	TRADE_TYPE_INC = 0,							// INSERTCARD
	TRADE_TYPE_WDC,								// WITHDRAWCARD
	TRADE_TYPE_CKT,								// CHECKTIME
	TRADE_TYPE_INQ,								// Query
	TRADE_TYPE_CWD,								// draw 
	TRADE_TYPE_PIN,								// change password
	TRADE_TYPE_TFR,								// debit
	TRADE_TYPE_DEP,								// deposit
	TRADE_TYPE_NCINQ,							// card less query
	TRADE_TYPE_NCDEP,							// card less deposit
	TRADE_TYPE_OTHERS,							// other
	TRADE_TYPE_ALL,								// all
}DH_eATMTradeTypes;

typedef struct __ALARM_ATM_INFO_UPLOAD_CHNL
{
	int					nChannel;				// channel
	char				szATMID[32];			// ATM id
	char				szCardNo[32];			// card number, 6222421541208230456 
	char				szTradetime[32];		// trade time, 20111118112200 means 2011-11-18 11:22:00
	DH_eATMTradeTypes	emTradeType;			// trade type
	int					nAmount;				// trade mount	0-4294967296
	BYTE				byRerved[32];			// reserved
}ALARM_ATM_INFO_UPLOAD_CHNL;

// ATM trade info uploading info
typedef struct __ALARM_ATM_INFO_UPLOAD
{
	int							nCnt;
	ALARM_ATM_INFO_UPLOAD_CHNL	m_stAtmInfo[DH_MAX_CHANNUM];
}ALARM_ATM_INFO_UPLOAD;

// camera move alarm
typedef struct __ALARM_CAMERA_MOVE_INFO
{
	int                         nChannelNum;                        // the number of alarm channel
	unsigned char               alarmChannels[DH_MAX_CHANNUM_EX];   // the channel information of alarm happened
	BYTE                        byReserved[128];
}ALARM_CAMERA_MOVE_INFO;

// detailed motion alarm
typedef struct __ALARM_DETAILEDMOTION_CHNL_INFO
{
	DWORD              dwSize;
	int                nChannelID;    // channel
	BOOL               bAlarm;        // alarm or not,value is TRUE/FALSE
	int                nLevel;        // alarm level,in 1/1000 as a unit
}ALARM_DETAILEDMOTION_CHNL_INFO;

// storage failure alarm info
typedef struct __ALARM_STORAGE_FAILURE
{
	DWORD     dwSize;                                     // struct size
	UINT      ActionType;                                 // 0:stop 1:start
	char      szProtocol[DH_MAX_STRING_LEN];              // protocol type
	char      szServerAddr[DH_MAX_IPADDR_OR_DOMAIN_LEN];  // server device's ip 
	DWORD     dwPort;                                     // port number
	NET_TIME  stuTime;                                    // event happen time	
	int		  nChannel;									  // channel, from 1, 0 means does not distinguish
}ALARM_STORAGE_FAILURE;

// front IPC disconnect alarm info
typedef struct __ALARM_FRONTDISCONNET_INFO
{
	DWORD              dwSize;                           // struct size
	int                nChannelID;                       // channel id
	int                nAction;                          // 0:start 1:stop
	NET_TIME           stuTime;                          // event happen time
	char               szIpAddress[MAX_PATH];            // front IP's address
}ALARM_FRONTDISCONNET_INFO;

// battery low power alarm info
typedef struct __ALARM_BATTERYLOWPOWER_INFO
{
	DWORD             dwSize;                            // struct size
	int               nAction;                           // 0:start 1:stop
	int               nBatteryLeft;                      // battery left, unit:%
	NET_TIME          stTime;                            // event happen time
	int               nChannelID;                        // channel no. Mark sub-device battery. Begins with 0. 
}ALARM_BATTERYLOWPOWER_INFO;

// temperature alarm info
typedef struct __ALARM_TEMPERATURE_INFO
{
	DWORD              dwSize;                           // struct size
	char               szSensorName[DH_MACHINE_NAME_NUM];// sensor name
	int                nChannelID;                       // channel id
	int                nAction;                          // 0:start 1:stop
	float              fTemperature;                     // current temperature, unit:degree centigrade
	NET_TIME           stTime;                           // event happen time
}ALARM_TEMPERATURE_INFO;

// Fatigue Driving Alarm
typedef struct __ALARM_TIREDDRIVE_INFO
{
	DWORD             dwSize;                            // struct size
	int               nAction;                           // 0:start 1:stop
	int               nDriveTime;                        // drive time, unit:minute
	NET_TIME          stTime;                            // event happen time
}ALARM_TIREDDRIVE_INFO;

//Alarm of record loss
typedef struct __ALARM_LOST_RECORD
{
	DWORD     dwSize;                                     // Structure size
	UINT      ActionType;                                 // 0: Stop, 1: start
	UINT      nChannelID;                                 // Channel ID, start from 1
	UINT      nStreamType;                                // Bit Stream type, 0: main stream; 1: extra stream1; 2: extra stream2; 3: extra stream3; 4: snapshot stream
	NET_TIME  stuTime;                                    // Event occurrence time	
}ALARM_LOST_RECORD;

// Alarm of High CPU Occupancy rate, temporarily we set the max limit as 95%
typedef struct __ALARM_HIGH_CPU
{
	DWORD     dwSize;                                     // Structure size
	UINT      ActionType;                                 // 0: Stop, 1: start
	UINT      nUsed;                                      // 1000 times of  CPU Occupancy rate
	NET_TIME  stuTime;                                    // Event occurrence time
}ALARM_HIGH_CPU;


// Alarm of netpackage loss
typedef struct __ALARM_LOST_NETPACKET
{
	DWORD     dwSize;                                     // Structure size
	UINT      ActionType;                                 // 0: Stop, 1: start
	UINT      nChannelID;                                 // Channel ID, start from 1
	UINT      nStreamType;                                // Bit stream type, 0: main stream; 1: extra stream1; 2: extra stream2; 3: extra stream3; 4: snapshot stream
	char      szRemoteIP[DH_MAX_IPADDR_LEN];              // IP address of sending
	DWORD     dwRemotePort;                               // Port address of sending 
	NET_TIME  stuTime;                                    // Event occurrence time
}ALARM_LOST_NETPACKET;

// Alarm of High memory Occupancy rate, temporarily we set the max limit as 95%
typedef struct __ALARM_HIGH_MEMORY
{
	DWORD     dwSize;                                     // Structure size
	UINT      ActionType;                                 // 0: Stop, 1: start
	UINT      nUsed;                                      // 1000 times of  memory Occupancy rate
	NET_TIME  stuTime;                                    // Event occurrence time	
}ALARM_HIGH_MEMORY;

// black list snap info
typedef struct __DH_BLACKLIST_SNAP_INFO
{
	DWORD     dwSize;
	char      szPlateNumber[32];                          // number of plate
	NET_TIME  stuTime;                                    // time
}DH_BLACKLIST_SNAP_INFO;

// disk flux abnormal
typedef struct __ALARM_DISK_FLUX
{
	DWORD				dwSize;	
	DWORD				dwAction;							// 0-start, 1-stop
	DWORD				dwDataFlux;							// data flux,KB			
	NET_TIME			stuTime;							// time
} ALARM_DISK_FLUX;

// net flux abnormal
typedef struct __ALARM_NET_FLUX
{
	DWORD				dwSize;	
	DWORD				dwAction;							// 0-start, 1-stop
	DWORD				dwDataFlux;							// data flux,KB	
	NET_TIME			stuTime;							// time
} ALARM_NET_FLUX;

// fan speed abnormal
typedef struct __ALARM_FAN_SPEED
{
	DWORD				dwSize;	
	DWORD				dwAction;							// 0-start, 1-stop
	DWORD				dwIndex;							// index
	char				szName[DH_MACHINE_NAME_NUM];		// name
	NET_TIME			stuTime;							// time
	DWORD				dwCurrent;							// current speed
} ALARM_FAN_SPEED;

// disk alarm
typedef struct __ALARM_DISK_INFO 
{
	DWORD				dwSize;
	DWORD				nChannel;							// channel number
	DWORD				nHDDNumber;							// number, 0: main trank,1: extern trank1, 2: extern trank2 
	DWORD				nHDDState;							// state, 0: Unknown, 1: Running, 2: Offline, 3: Warning, 4: Failed
} ALARM_DISK_INFO;

// alarm of file system
typedef struct __ALARM_FILE_SYSTEM_INFO 
{
	DWORD				dwSize;
	char				szMountDir[MAX_PATH];				// mount name
	DWORD				nState;								// state, 0: Unkown, 1: Normal, 2: Error
} ALARM_FILE_SYSTEM_INFO;

// alarm of remote 
typedef struct __ALARM_REMOTE_ALARM_INFO
{
	DWORD      dwSize;
	int        nChannelID;                               // channel ID,from 1
	int        nState;                                   // state,0-reset,1-setting
}ALARM_REMOTE_ALARM_INFO;

// alarm of ivs
typedef struct __ALARM_IVS_INFO
{
	DWORD      dwSize;
	NET_TIME   stuTime;                                  // time	
	int        nChannelID;                               // channel ID,from 0
	char	   szType[MAX_PATH];                         // type
	int        nState;                                   // state,0-reset,1-setting,2-pulse
}ALARM_IVS_INFO;

// alarm of good weight
typedef struct __ALARM_GOODS_WEIGHT_INFO
{
	DWORD		dwSize;
	int			nAction;							// 0-start, 1-stop
	int			nAlarmType;							// 0-greater than dwMaxGoodsWeight, 1-less than dwMinGoodsWeight, 2-the change greater than dwAlarmWeight
	DWORD		dwGoodsWeight;						// good weight(kg)
	DWORD		dwSelfWeight;						// self weight(kg)
	DWORD		dwTotalWeight;						// total weight(kg)
	DWORD		dwStandardWeight;					// standard weight(kg)
	DWORD		dwWeightScale;						// weight scale
	DWORD		dwMaxGoodsWeight;					// the max(kg)
	DWORD		dwMinGoodsWeight;					// the min(kg)
	DWORD		dwAlarmWeight;						// alarm of threshold(kg)
	int			nWeightChange;						// whight change when gather(kg)
	int			nCheckTime;							// gather time(s)
} ALARM_GOODS_WEIGHT_INFO;

// goods weight upload
typedef struct __ALARM_GOODS_WEIGHT_UPLOAD_INFO 
{
	DWORD		dwSize;
	DWORD		dwGoodsWeight;						// goods weight(kg)
	DWORD		dwSelfWeight;						// self weight(kg)
	DWORD		dwTotalWeight;						// total weight(kg)
	DWORD		dwStandardWeight;					// standard weight(kg)
	DWORD		dwWeightScale;						// weight scale	
    char        szCardNO[DH_MAX_CARDINFO_LEN];      // card number
} ALARM_GOODS_WEIGHT_UPLOAD_INFO;

// source of date
typedef enum tagEM_DATE_SOURCE
{
    EM_DATE_SOURCE_GPS,                                     // GPS 
    EM_DATE_SOURCE_INERTIALNAVIGATION,                      // inertial navigation date
}EM_DATE_SOURCE;

// GPS status info
typedef struct _NET_GPS_STATUS_INFO
{
    NET_TIME                revTime;				    // time
    char                    DvrSerial[50];			    // device number
	BYTE                    byRserved1[6];              // align
    double                  longitude;				    // longitude(1/1000000,range[0-360])
    double                  latidude;				    // latitude(1/1000000,range[0-180])
    double                  height;					    // highness(m)
    double                  angle;					    // angle(north is source point,clockwise is positive)
    double                  speed;					    // speed(sea mile,speed/1000*1.852km/h)
    WORD                    starCount;				    // star count
	BYTE                    byRserved2[2];              // align
	NET_THREE_STATUS_BOOL   antennaState;			    // antenna state(true good, false bad) valid when emDateSource is EM_DATE_SOURCE_GPS
    NET_THREE_STATUS_BOOL   orientationState;		    // orientation state(true orientation, false not) valid when emDateSource is EM_DATE_SOURCE_GPS
	int                     workStae;                   // working state(true normal, false abnormity)
                                                        // valid when emDateSource is EM_DATE_SOURCE_GPS
    int                     nAlarmCount;                // alarm count
    int                     nAlarmState[128];           // alarm type valid when emDateSource is EM_DATE_SOURCE_GPS
    BYTE                    bOffline;                   // 0- real time 1-fill 
    BYTE                    bSNR;                       // SNR for GPS, range: 0~100, 0 for unusable
	BYTE                    byRserved3[2];              // align
	EM_DATE_SOURCE          emDateSource;               // source of date
    BYTE                    byRserved[124];             // reserved bytes
} NET_GPS_STATUS_INFO,*LPNET_GPS_STATUS_INFO;

// alarm of disk burned full
typedef struct __ALARM_DISKBURNED_FULL_INFO
{
	DWORD		dwSize;
	int         nIndex;                             //index
}ALARM_DISKBURNED_FULL_INFO;

// alarm of storage low space
typedef struct tagALARM_STORAGE_LOW_SPACE_INFO 
{
	DWORD				dwSize;
	int					nAction;						// 0:start 1:stop
	char				szName[DH_EVENT_NAME_LEN];		// name
	char				szDevice[DH_STORAGE_NAME_LEN];	// device name
	char				szGroup[DH_STORAGE_NAME_LEN];	// group name 
	INT64				nTotalSpace;					// total space byte
	INT64				nFreeSpace;						// free space byte
	int					nPercent;						// used percent 
} ALARM_STORAGE_LOW_SPACE_INFO;

// storage error
typedef enum __EM_STORAGE_ERROR
{
	STORAGE_ERROR_NONE,							// no error
    STORAGE_ERROR_PATITION,						// patition error 		
	STORAGE_ERROR_INIT_FS,						// init system file error	
	STORAGE_ERROR_READ_DATA,					// read data error
	STORAGE_ERROR_WRITE_DATA,					// write data error
	STORAGE_ERROR_RAID_FAILED,					// RAID error
	STORAGE_ERROR_RAID_DEGRADED,				// RAID degranded 
    STORAGE_ERROR_ISCSI_FAILED,                 // iSCSI error
} EM_STORAGE_ERROR;

// alarm of storage failure
typedef struct __ALARM_STORAGE_FAILURE_EX
{
	DWORD				dwSize;
	int					nAction;						// 0:start 1:stop
	char				szName[DH_EVENT_NAME_LEN];		// name 
	char				szDevice[DH_STORAGE_NAME_LEN];	// device name
	char				szGroup[DH_STORAGE_NAME_LEN];	// group name
	char				szPath[MAX_PATH];				// path
	EM_STORAGE_ERROR	emError;						// error type
    int                 nPhysicNo;                      // disk No, start from 1 
    NET_TIME_EX			stuTime;						// Event occurrence time 
} ALARM_STORAGE_FAILURE_EX;

// alarm of record failed 
typedef struct __ALARM_RECORD_FAILED_INFO 
{
	DWORD				dwSize;
	int					nAction;						// 0:start 1:stop
	int					nIndex;							// index
} ALARM_RECORD_FAILED_INFO;

// storage break down
typedef struct __ALARM_STORAGE_BREAK_DOWN_INFO 
{
	DWORD				dwSize;
	int					nAction;						// 0:start 1:stop
} ALARM_STORAGE_BREAK_DOWN_INFO;

// hot plug action mode
typedef enum tagEM_STORAGE_HOT_PLUG_ACTION
{
    HOT_PLUG_ACTION_UNKNOW = 0  ,
    HOT_PLUG_ACTION_ADD         ,       //Insert device
    HOT_PLUG_ACTION_REMOVE      ,       //Remove device 
}EM_STORAGE_HOT_PLUG_ACTION;
// storage hot plug action event (corresponding to DH_ALARM_STORAGE_HOT_PLUG)
typedef struct __ALARM_STORAGE_HOT_PLUG_INFO
{
    int                         nAction;                            // 0:Start 1:stop
    EM_STORAGE_HOT_PLUG_ACTION  emHotPLugAction;                    // Hot swap operation type 
    char                        szDevice[DH_STORAGE_NAME_LEN];      // Storage device name 
    char                        szMediaType[DH_COMMON_STRING_32];   // Media type 
    char                        szBusType[DH_COMMON_STRING_32];     // BUS type 
    char                        szMountOn[DH_COMMON_STRING_128];    // Device mount position 
    int                         nPhysicNo;                          // Physical SN. Begins with 1. 
    int                         nLogicNo;                           // Logic SN
    BYTE                        bReserved[256];                     // Reserved byte. To be developed .
}ALARM_STORAGE_HOT_PLUG_INFO;

// flow rate event(corresponding to DH_ALARM_FLOW_RATE)
typedef struct tagALARM_FLOW_RATE_INFO
{
	int				nAction;				// 0:paulse, 1:start, 2:stop
	int				nChannelID;				// channelId
	UINT			nFlowRate;				// the flow rate(unit:MB)
	BYTE			bReserved[512];			// Reserved byte. 
} ALARM_FLOW_RATE_INFO;


typedef enum tagEM_NET_UPS_STATUS
{
	EM_NET_UPS_SYS_SIGN=0	 	,	//System temperature the sign bit. 1: negative temperature; 0:positive temperature
	EM_NET_UPS_SYS_SHUTDOWN 	,	//System shutdown. 1: shutdown activated state
	EM_NET_UPS_SYS_TEST 		,	//System tests. 1: said tests
	EM_NET_UPS_SYS_TYPE 		,	//UPS type. 1: backup machine; 0: said online machine
	EM_NET_UPS_SYS_FAULT 		,	//UPS fault. 1: UPS internal fault; Zero: normal
	EM_NET_UPS_ELE_SUPPLY 		,	//The bypass/inverter state. 1: AC power supply; 0: batteries
	EM_NET_UPS_VOL_LOW 			,	//Low battery voltage. 1: low battery voltage; Zero: it means the battery is normal
	EM_NET_UPS_BYPASS_STATUS	,	//Mains failure. 1: mains failure; Zero: it means the mains is normal
	EM_NET_UPS_MAX = 64			,	//
}EM_NET_UPS_STATUS;

typedef struct tagNET_UPS_INFO
{
	DWORD dwSize;
	float fInputVoltage;						//Specific reference input voltage intelligent transportation special power communication protocol (V1.2), blue shield special definition
	float fInputAbnormalVoltage;				//Abnormal input voltage
	float fOutputVoltage;						//output voltage
	float fOutputCurrent;						//Output current percentage, the unit (%)
	float fInputFrequency;						//incoming frequency
	float fVoltage;								//voltage
	float fTemp;								//temperature
	BYTE  bStatusInfo[EM_NET_UPS_MAX];			//Set UPS STATUS, see.net UPS STATUS enumeration
	char  szVersionInfo[DH_MAX_VERSION_STR];	//version
}NET_UPS_INFO;

typedef struct tagALARM_COMM_PORT_EVENT_INFO
{
	DWORD           dwSize;
	UINT      		nEventAction;	// Event Action, 0=Impulse Event,1=Begin a Continued Event,2=the Event End;
	NET_UPS_INFO	stUPSInfo;		// UPS's COM Information
}ALARM_COMM_PORT_EVENT_INFO;

// ininvalid of video input channel(example:)DH_ALARM_VIDEO_ININVALID
typedef struct __ALARM_VIDEO_ININVALID_INFO 
{
	DWORD               dwSize;                         // struct size
	int                 nChannelID;                     // channel ID,from 0
} ALARM_VIDEO_ININVALID_INFO;


// No Information of Event in Storage Group
typedef struct tagALARM_STORAGE_NOT_EXIST_INFO 
{
    DWORD           dwSize;
    int             nAction;                            // 0:start 1:stop
    char            szGroup[DH_STORAGE_NAME_LEN];       // Storage Droup Name
    NET_TIME        stuTime;                            // the Time of Event triggering)
}ALARM_STORAGE_NOT_EXIST_INFO;

//the Type of Network Fault Event)
typedef enum __EM_NETABORT_EVENT_TYPE
{
	EM_NETABORT_EVENT_TYPE_WIRE = 0,				// Wired Network Fault Event 
	EM_NETABORT_EVENT_TYPE_WIRELESS,				// Wireless Network Fault Event 
	EM_NETABORT_EVENT_TYPE_3G,						// 3G Network Fault Event
}EM_NETABORT_EVENT_TYPE;

// Network Fault Event
typedef struct tagALARM_NETABORT_INFO
{
	DWORD			dwSize;
	int				nAction;						// 0=Start 1=Stop
	EM_NETABORT_EVENT_TYPE	emNetAbortType;			// Event Type
	NET_TIME		stuTime;						// Event Triggering Time
}ALARM_NETABORT_INFO;

// IP Clash Event
typedef struct tagALARM_IP_CONFLICT_INFO
{
	DWORD			dwSize;
	int				nAction;						// 0=Start 1=Stop
	NET_TIME		stuTime;						// Event Triggering Time)
}ALARM_IP_CONFLICT_INFO;


// MAC Clash Evnet)
typedef struct tagALARM_MAC_CONFLICT_INFO
{
	DWORD			dwSize;
	int				nAction;						// 0=Start 1=Stop
	NET_TIME		stuTime;						// Event Triggering Time
}ALARM_MAC_CONFLICT_INFO;


// Power Type
typedef enum __EM_POWER_TYPE
{
	EM_POWER_TYPE_MAIN = 0,							// Main Power
	EM_POWER_TYPE_BACKUP,							// Spare Power
}EM_POWER_TYPE;

// the Type of Power Fault Event
typedef enum __EM_POWERFAULT_EVENT_TYPE
{
	EM_POWERFAULT_EVENT_LOST = 0					// Power Down
}EM_POWERFAULT_EVENT_TYPE;

// Power Fault Event
typedef struct tagALARM_POWERFAULT_INFO
{
	DWORD					dwSize;			
	EM_POWER_TYPE			emPowerType;			// Power Type
	EM_POWERFAULT_EVENT_TYPE	emPowerFaultEvent;	// Power Fault Event
	NET_TIME				stuTime;				// Alarm Event Begin Time 
	int						nAction;				// 0=Start 1=Stop 
}ALARM_POWERFAULT_INFO;

// Tamper Alarm Event 
typedef struct tagALARM_CHASSISINTRUDED_INFO
{
	DWORD				dwSize;
	int					nAction;				// 0=Start 1=Stop 
	NET_TIME			stuTime;				// Alarm Event Begin Time
	int                 nChannelID;             // Channel ID
	char				szReaderID[DH_COMMON_STRING_32];// Reader ID	
    UINT 			    nEventID;				//event id
}ALARM_CHASSISINTRUDED_INFO;

// Expand Module Alarm Event
typedef struct tagALARM_ALARMEXTENDED_INFO
{
	DWORD				dwSize;
	int                 nChannelID;             //  Channel ID
	int					nAction;				//  0=Start 1=Stop (1=stop) 
	NET_TIME			stuTime;				//  Alarm Event Begin Time
}ALARM_ALARMEXTENDED_INFO;


// Interphone's Initiator 
typedef enum __EM_TALKING_CALLER
{
	EM_TALKING_CALLER_UNKNOWN = 0,				// Uunbeknown Initiator 
	EM_TALKING_CALLER_PLATFORM,					// Interphone's initiator is Platform
}EM_TALKING_CALLER;

// Invite Evnet remote device protocol
typedef enum tagTALKINGINVITE_REMOTEDEVICE_PROTOCOL
{
    EM_TALKINGINVITE_REMOTEDEVICE_PROTOCOL_UNKNOWN = 0,     // UNKNOWN;
    EM_TALKINGINVITE_REMOTEDEVICE_PROTOCOL_HIKVISION        // HIKVISION
}EM_TALKINGINVITE_REMOTEDEVICE_PROTOCOL;

#define MAX_REMOTEDEVICEINFO_IPADDR_LEN         128       // max remote device IP address length
#define MAX_REMOTEDEVICEINFO_USERNAME_LEN       128       // max remote device username length
#define MAX_REMOTEDEVICEINFO_USERPSW_LENGTH     128       // max remote device password length

// Invite Evnet remote device information
typedef struct tagTALKINGINVITE_REMOTEDEVICEINFO
{
    char		                            szIP[MAX_REMOTEDEVICEINFO_IPADDR_LEN];	        // device's IP
	int	                                    nPort;							                // port
	EM_TALKINGINVITE_REMOTEDEVICE_PROTOCOL  emProtocol;                                     // protocol type
	char		                            szUser[MAX_REMOTEDEVICEINFO_USERNAME_LEN];	    // user name
	char		                            szPassword[MAX_REMOTEDEVICEINFO_USERPSW_LENGTH];// password
	char                                    szReverse[1024];                                // reverse word
}TALKINGINVITE_REMOTEDEVICEINFO;

// Alarm Event TypeDH_ALARM_TALKING_INVITE Device ask Other Side InitiateInterphone Event )Corresponding Data Description Information 
typedef struct tagALARM_TALKING_INVITE_INFO
{
    DWORD                                   dwSize;
    EM_TALKING_CALLER                       emCaller;                       // Interphone's Initiator is Device Desired 
    NET_TIME                                stuTime;                        // Event Triggering Time
    char                                    szCallID[DH_COMMON_STRING_64];	// Unique ID for calling
    int                                     nLevel;						    // The level of current calling device 
    TALKINGINVITE_REMOTEDEVICEINFO          stuRemoteDeviceInfo;            // remote device info
}ALARM_TALKING_INVITE_INFO;

// Alarm Event Type DH_ALARM_TALKING_HANGUP corresponding data description
typedef struct tagALARM_TALKING_IGNORE_INVITE_INFO
{
    DWORD                dwSize;
} ALARM_TALKING_IGNORE_INVITE_INFO;
// data describe info corresponding alarm event mode:DH_ALARM_TALKING_HANGUP
typedef struct tagALARM_TALKING_HANGUP_INFO 
{
    DWORD               dwSize;
    NET_TIME            stuTime;                // Event trigger time
}ALARM_TALKING_HANGUP_INFO;

// Station Update Event Status
typedef enum tagEM_GUARD_EVENT_STATE
{
	EM_GUARD_EVENT_STATE_UNKNOWN = 0,	// unknown status
	EM_GUARD_EVNET_STATE_ONGUARD,		// on guard
	EM_GUARD_EVENT_STATE_OFFGUARD		// off guard
}EM_GUARD_EVENT_STATE;

// station info update event ,DH_ALARM_GUARD_INFO_UNPDATE data description info
typedef struct tagALARM_GUARD_UPDATE_INFO
{
	DWORD				 dwSize;
	EM_GUARD_EVENT_STATE emEventState;							// station update event status
	int					 nIDCount;								// trigger detection event staff ID number
	int					 nIDArray[DH_MAX_GUARD_DETECT_ID_COUNT];// staff ID group
	NET_TIME			 stuTime;								// event trigger time
}ALARM_GUARD_UPDATE_INFO;

// station detection event ,DH_ALARM_GUARD_DETECTcorresponding to data description info
typedef struct tagALARM_GUARD_DETECT_INFO
{
	DWORD				dwSize;
	int					nAction;								// 0: start 1: stop
	int					nIndex;									// station detection config mark	
	int					nIDCount;								// trigger detection event staff ID number
	int					nIDArray[DH_MAX_GUARD_DETECT_ID_COUNT];	// staff ID group number
	NET_TIME			stuTime;								// event trigger time
}ALARM_GUARD_DETECT_INFO;

//Bank card event DH_ALARM_BANKCARDINSERT corresponding data info
typedef struct tagALARM_BANKCARDINSERT_INFO 
{
    DWORD               dwSize;
    int                 nAction;                // -1:unknown 0: start 1:stop
}ALARM_BANKCARDINSERT_INFO;

#define MAX_CARD_RECORD_FIELD_NUM 16            // card no. max domain quantity

// event trigger record,stop storage strategy
typedef enum tagNET_RECORD_SAVEFLAG
{
    NET_RECORD_SAVEFLAG_UNKNOWN = 0,            //unknown type
	NET_RECORD_SAVEFLAG_MARK,                   //save record and lock
	NET_RECORD_SAVEFLAG_SAVE,                   //only save record
}NET_RECORD_SAVEFLAG;

//card no. record event DH_ALARM_CARD_RECORD corresponding to group decryption info
typedef struct tagALARM_CARD_RECORD_INFO 
{
    DWORD               dwSize;
    int                 nAction;                // -1:unknown 0:start 1:stop
    int                 nChannelID;             // channel no., from 0
    unsigned int        nFieldCount;            // domain quantity
    char                szFields[MAX_CARD_RECORD_FIELD_NUM][DH_COMMON_STRING_256]; // domain info
    NET_RECORD_SAVEFLAG emRecordSaveFlag;       // event trigger record, stop storage strategy ,see  NET_RECORD_SAVEFLAG
}ALARM_CARD_RECORD_INFO;

// CLIENT_ControlDevice port DH_CTRL_NOTIFY_EVNENT command parameter(send event to device)
typedef struct tagALARM_NET_INFO
{
    DWORD               dwSize;
    int                 nAction;                // -1:Unknown, 0:Start 1:stop
    int                 nChannelID;             // Channel No. Begins with 0. 
} ALARM_NET_INFO;

//CLIENT_ControlDevice port DH_ALARM_VIDEOMOTION_EVENT command parameter(send event to device) 
typedef struct tagALARM_VIDEOMOTION_EVENT_INFO
{
	DWORD				 dwSize;
	int                  nAction;             // -1:Unknown, 0:Start 1:stop
	int                  nChannelID;          // Channel No. Begins with 0.
}ALARM_VIDEOMOTION_EVENT_INFO;

////CLIENT_ControlDevice port DH_ALARM_WIDE_VIEW_REGION_EVENT command parameter(send event to device)   
typedef struct tagALARM_WIDE_VIEW_REGION_EVENT_INFO
{
    DWORD				 dwSize;
    int                  nAction;             // -1:Unknown 0:Pulse   
}ALARM_WIDE_VIEW_REGION_EVENT_INFO;

// DH_CTRL_NOTIFY_EVNENT command parameter of CLIENT_ControlDevice interface (send event to device)
typedef struct tagNET_NOTIFY_EVENT_DATA
{
    DWORD                       dwSize;
    LONG                        lCommand;       // event type
    void*                       pEventData;     // event content,and lCommandcorresponding to structure indicator
}NET_NOTIFY_EVENT_DATA;

// Sensor's Sense Method Enumeration Type
typedef enum tagNET_SENSE_METHOD
{
	NET_SENSE_UNKNOWN = -1,		//Unknowed type
	NET_SENSE_DOOR=0,			//Door Contact
	NET_SENSE_PASSIVEINFRA,		//Passive Infrared
	NET_SENSE_GAS,				//Gase Induce)
	NET_SENSE_SMOKING,			//Smoking Induce
	NET_SENSE_WATER,			//Wwater Induce)
	NET_SENSE_ACTIVEFRA,		//Initiative Infrared
	NET_SENSE_GLASS,			//Glass Broken
	NET_SENSE_EMERGENCYSWITCH,	//Emergency switch
	NET_SENSE_SHOCK,			//Shock
	NET_SENSE_DOUBLEMETHOD,		//Double Method(Infrare+Microwave)
	NET_SENSE_THREEMETHOD,		//Three Method
	NET_SENSE_TEMP,				//Temperature
	NET_SENSE_HUMIDITY,			//Humidity
	NET_SENSE_WIND,             //Wind
	NET_SENSE_CALLBUTTON,		//Call button
    NET_SENSE_GASPRESSURE,      //Gas Pressure
    NET_SENSE_GASCONCENTRATION, //Gas Concentration
    NET_SENSE_GASFLOW,          //Gas Flow
	NET_SENSE_OTHER,			//Other
    NET_SENSE_OIL,              //oil detection, gasoline, diesel vehicles detection
    NET_SENSE_MILEAGE,          //mileage detection
    NET_SENSE_URGENCYBUTTON,    //Urgency button
    NET_SENSE_STEAL,            //Steal
    NET_SENSE_PERIMETER,        //Permeter
    NET_SENSE_PREVENTREMOVE,    //Prevent remove
    NET_SENSE_DOORBELL,         //Door bell
    NET_SENSE_ALTERVOLT,        //Alter voltage sensor
    NET_SENSE_DIRECTVOLT,       //Direct voltage sensor
    NET_SENSE_ALTERCUR,         //Alter current sensor
    NET_SENSE_DIRECTCUR,        //Direct current sensor
    NET_SENSE_RSUGENERAL,       //RSU general analog sensor, 4~20mA or 0~5V
    NET_SENSE_RSUDOOR,          //RSU door sensor
    NET_SENSE_RSUPOWEROFF,      //RSU power off sensor	
    NET_SENSE_TEMP1500 ,        //1500 temperature sensor	
    NET_SENSE_TEMPDS18B20 ,     //DS18B20 temperature sensor	
    NET_SENSE_HUMIDITY1500,     //1500 humidity sensor
    NET_SENSE_INFRARED,         //Infrared sensor
    NET_SENSE_FIREALARM,        //firealarm sensor
	NET_SENSE_CO2,				//Determination of CO2, typical value:0~5000ppm
	NET_SNESE_SOUND,			//Noise detection,typical value:30~130dB
	NET_SENSE_PM25,				//PM2.5 detection,typical value:0~1000ug/m3
	NET_SENSE_SF6,				 //Determination of SF6,typical value:0~3000ppm
	NET_SENSE_O3,				 //O3 detection,typical value:0~100ppm
	NET_SENSE_AMBIENTLIGHT,		 //ambient light detection,typical value:0~20000Lux




	NET_SENSE_NUM               // enum total
}NET_SENSE_METHOD;

//corresponding Enumeration Type of Sensor's Sense Method's unit 
typedef enum tagEM_SENSE_METHOD_UNIT
{
	EM_SENSE_UNIT_UNKOWN =-1,		 // Unknowed type
	EM_SENSE_TEMP_CENTIGRADE = 0,    // Temperature's unit: Centigrade
	EM_SENSE_TEMP_FAHRENHEIT,        // Temperature's unit: Fahrenheit
	EM_SENSE_WIND_SPEED,             // Wind speed's unit:  m/s
	EM_SENSE_HUMIDITY_PERCENT,       // Humidity unit: Percent 
}EM_SENSE_METHOD_UNIT;	

// Protect/Cancel Protect Mode 
typedef enum tagNET_ALARM_MODE
{	
	NET_ALARM_MODE_UNKNOWN = -1,	// Unknown
	NET_ALARM_MODE_DISARMING,		// Cancel Protect 
	NET_ALARM_MODE_ARMING,			// Install protect
	NET_ALARM_MODE_FORCEON,			// Forceon protect
    NET_ALARM_MODE_PARTARMING,      // Parting protect
}NET_ALARM_MODE;

// Arm/Disarm scene mode
typedef enum tagNET_SCENE_MODE
{
	NET_SCENE_MODE_UNKNOWN,			// Unknown scene
	NET_SCENE_MODE_OUTDOOR,			// Outdoor mode
	NET_SCENE_MODE_INDOOR,			// Inner mode
    NET_SCENE_MODE_WHOLE,           // global mode
    NET_SCENE_MODE_RIGHTNOW,        // immediate mode
    NET_SCENE_MODE_SLEEPING,        // sleeping mode
    NET_SCENE_MODE_CUSTOM,          // custom mode
}NET_SCENE_MODE;

// trigger mode
typedef enum tagNET_EM_TRIGGER_MODE
{
    NET_EM_TRIGGER_MODE_UNKNOWN = 0,
    NET_EM_TRIGGER_MODE_NET,            // Network user(Platform or Web)
    NET_EM_TRIGGER_MODE_KEYBOARD,       // keyboard
    NET_EM_TRIGGER_MODE_REMOTECONTROL,  // remote control
}NET_EM_TRIGGER_MODE;

// Protect Transformate Event's Information
typedef struct tagALARM_ARMMODE_CHANGE_INFO
{
	DWORD				dwSize;
	NET_TIME			stuTime;			// Alarm Event Begin Time
	NET_ALARM_MODE		bArm;				// Statue of Transformated
	NET_SCENE_MODE		emSceneMode;		// ContextualMode
    DWORD               dwID;               // ID number, remote control number or keypad address, emTriggerMode=0 when  belong to NET_EM_TRIGGER_MODE_NET
    NET_EM_TRIGGER_MODE emTriggerMode;      // trigger mode
}ALARM_ARMMODE_CHANGE_INFO;

// defence zone type
typedef enum
{
	NET_DEFENCEAREA_TYPE_UNKNOWN,		// Unknown Type Defence Area
	NET_DEFENCEAREA_TYPE_ALARM,			// Switching Value Defence Area 
}NET_DEFENCEAREA_TYPE;

// Bypass Statue Type
typedef enum
{
	NET_BYPASS_MODE_UNKNOW,			//UnknownBypass Statue
	NET_BYPASS_MODE_BYPASS,			//Bypass
	NET_BYPASS_MODE_NORMAL,			//Normal
	NET_BYPASS_MODE_ISOLATED,		//Isolated
}NET_BYPASS_MODE;

// The Information of Bypass's Statue Change Event
typedef struct tagALARM_BYPASSMODE_CHANGE_INFO
{
	DWORD                   dwSize;
	int	                    nChannelID;                 // Channel ID
	NET_TIME                stuTime;                    // Alarm Event Begin Time
	NET_DEFENCEAREA_TYPE    emDefenceType;              // Defence Area Type
	int                     nIsExtend;                  // Whether or not It Is Expand, Channel, Defence Area , 1=Expand Channel, 0=Non Expand Channel)
	NET_BYPASS_MODE	        emMode;	                    // Changed Mode 
    DWORD                   dwID;                       // ID number, remote control number or keypad address, emTriggerMode=0 when belong to NET_EM_TRIGGER_MODE_NET
    NET_EM_TRIGGER_MODE     emTriggerMode;              // trigger mode
}ALARM_BYPASSMODE_CHANGE_INFO;


// Emergency Event, Correspond DH_URGENCY_ALARM_EX2,DH_URGENCY_ALARM_EX Update, That is Artificially Triggered Emergency,General Treatment is Linkage External Communications Functions for Help, 
typedef struct tagALARM_URGENCY_ALARM_EX2 
{
	DWORD		dwSize;
	NET_TIME	stuTime;			// Event Begin Time
    DWORD           nID;            // used to mark different emergency event
}ALARM_URGENCY_ALARM_EX2;

// Alarm Input Source Event Details(As Long As There Will Have to Change The Input Event, Regardless of the Current Mode of The Defence Zone Can not be Shielded)
typedef struct tagALARM_INPUT_SOURCE_SIGNAL_INFO
{
	DWORD		dwSize;
	int			nChannelID;             // Channel ID
	int			nAction;				// 0=Start 1=Stop 
	NET_TIME	stuTime;				// Alarm Event Begin Time
}ALARM_INPUT_SOURCE_SIGNAL_INFO;

// Analog Input Channel Alarm Event Information(Corresponding DH_ALARM_ANALOGALARM_EVENT), 
typedef struct tagALARM_ANALOGALARM_EVENT_INFO 
{
	DWORD		dwSize;
	int			nChannelID;             // Alarm Event Begin Time
	int			nAction;				// 0=Start 1=Stop 
	NET_TIME	stuTime;				// Alarm Event Begin Time
	NET_SENSE_METHOD	emSensorType;	// the Sensor's Type
	char		szName[DH_COMMON_STRING_128];	// Channel Name
    int         nIsValid;               // Data is valid or not,-1: unknown,0:invalid,1:valid
    int         nStatus;                // Data status, -1: unknown,0:noemal,1:invalid data(over limit),
                                        // 2:over threshold value1,3: over threshold value 2,4: over threshold value 3,5: over threshold value 4,
                                        // 6:below threshold value 1,7: below threshold value2,8: below threshold value3,9: below threshold value4
    float       fValue;                 // Detector value
    NET_TIME    stuCollectTime;         // Data collection time(UTC)
    NET_GPS_STATUS_INFO stGPSStatus;    // GPS status
}ALARM_ANALOGALARM_EVENT_INFO;


// Access Control Status Type
typedef enum tagNET_ACCESS_CTL_STATUS_TYPE
{
	NET_ACCESS_CTL_STATUS_TYPE_UNKNOWN = 0,
	NET_ACCESS_CTL_STATUS_TYPE_OPEN,		// Open
	NET_ACCESS_CTL_STATUS_TYPE_CLOSE,		// Close
    NET_ACCESS_CTL_STATUS_TYPE_ABNORMAL,    // Abnormal

}NET_ACCESS_CTL_STATUS_TYPE;

// Access control status event
typedef struct tagALARM_ACCESS_CTL_STATUS_INFO 
{
	DWORD		dwSize;
	int			nDoor;						// Door channel no.
	NET_TIME	stuTime;					// Event time
	NET_ACCESS_CTL_STATUS_TYPE	emStatus;	// Access control status
}ALARM_ACCESS_CTL_STATUS_INFO;

// Access Control Snapshot Type
typedef enum tagNET_ACCESS_SNAP_TYPE
{
    NET_ACCESS_SNAP_TYPE_UNKNOWN = 0,
    NET_ACCESS_SNAP_TYPE_CARD,                          // Swipe
    NET_ACCESS_SNAP_TYPE_PASSWORD,                      // Password
}NET_ACCESS_SNAP_TYPE;

// Access Control Snapshot Event
typedef struct tagALARM_ACCESS_SNAP_INFO
{
    DWORD           dwSize;
    NET_ACCESS_SNAP_TYPE emType;                        // Type
    char            szCardNO[DH_COMMON_STRING_32];      // Card no., swipe card valie
    char            szPassword[DH_COMMON_STRING_64];    // Password, valid when unlock by password    
    char            szFtpUrl[MAX_PATH];                 // ftp upload address, picturepath
    int             nImageNum;                          // Snapshot picture
    NET_TIME        stuTime;                            // Event time
}ALARM_ACCESS_SNAP_INFO;

// Clear Alarm Event
typedef struct tagALARM_ALARMCLEAR_INFO 
{
    DWORD           dwSize;
    int             nChannelID;                         // Channel No.
    NET_TIME        stuTime;                            // Alarm event time
    int             bEventAction;                       // Event Action,0=Impluse Event,1=Continued Event Start,2=Continued Event End;
}ALARM_ALARMCLEAR_INFO;
// CID Event
typedef struct tagALARM_CIDEVENT_INFO
{
    DWORD           dwSize;
    NET_TIME        stuTime;                            // Alarm event time
    char			szCID[DH_COMMON_STRING_32];         // CID code, standard protocolcode, string length is 16, string value is 0~F
                                                        // 4 bit user code + 2 bit recognition code + 1 bit event limit code + 3 bit event code + 2 bit system code + 3 bit zone code + 1 bit parity code
}ALARM_CIDEVENT_INFO;

// Emergency Help Event Type
typedef enum tagEM_RCEMERGENCY_CALL_TYPE
{
    EM_RCEMERGENCY_CALL_UNKNOWN = 0,
    EM_RCEMERGENCY_CALL_FIRE,                           // Fire
    EM_RCEMERGENCY_CALL_DURESS,                         // Forced
    EM_RCEMERGENCY_CALL_ROBBER,                         // Robber
    EM_RCEMERGENCY_CALL_MEDICAL,                        // Medical
    EM_RCEMERGENCY_CALL_EMERGENCY,                      // Emergency
}EM_RCEMERGENCY_CALL_TYPE;

// alarm method
typedef enum tagEM_RCEMERGENCY_MODE_TYPE
{
    EM_RCEMERGENCY_MODE_UNKNOWN = 0,
    EM_RCEMERGENCY_MODE_KEYBOARD,                       // keyboard
    EM_RCEMERGENCY_MODE_WIRELESS_CONTROL,               // remote control
}EM_RCEMERGENCY_MODE_TYPE;

// Emergency Help Event Details
typedef struct tagALARM_RCEMERGENCY_CALL_INFO 
{
    DWORD                   dwSize;
    int                     nAction;                    // 0:start 1:stop
    EM_RCEMERGENCY_CALL_TYPE  emType;                   // emergency type
    NET_TIME                stuTime;                    // event time
    EM_RCEMERGENCY_MODE_TYPE  emMode;                   // alarm method
    DWORD                   dwID;                       // for different emergency events (only emMode is remote control type, it is valid , means remote control no. , 0 means invalid ID)
}ALARM_RCEMERGENCY_CALL_INFO;

// multi-people unlock event(corresponding to DH_ALARM_OPENDOORGROUP type)
typedef struct tagALARM_OPEN_DOOR_GROUP_INFO
{
    DWORD           dwSize;
    int             nChannelID;                         // door channel(from 0)
    NET_TIME        stuTime;                            // event time
}ALARM_OPEN_DOOR_GROUP_INFO;

// get fingerprint event(corresponding to DH_ALARM_FINGER_PRINT type)
typedef struct tagALARM_CAPTURE_FINGER_PRINT_INFO 
{
    DWORD           dwSize;
    int             nChannelID;                         // door channel no.( from 0)
    NET_TIME        stuTime;                            // event time
    char            szReaderID[DH_COMMON_STRING_32];    // card reader ID
    int             nPacketLen;                         // single fingerprint data length
    int             nPacketNum;                         // fingerprint data number
    char*           szFingerPrintInfo;                  // fingerprint data(data total length as nPacketLen*nPacketNum)
}ALARM_CAPTURE_FINGER_PRINT_INFO;

// sub system status type
typedef enum __EM_SUBSYSTEM_STATE_TYPE
{
    EM_SUBSYSTEM_STATE_UNKNOWN = 0,                     // unknown
    EM_SUBSYSTEM_STATE_ACTIVE,                          // activated
    EM_SUBSYSTEM_STATE_INACTIVE,                        // not activated
}EM_SUBSYSTEM_STATE_TYPE;

// sub system statua change event
typedef struct tagALARM_SUBSYSTEM_STATE_CHANGE_INFO 
{
    DWORD                   dwSize;
    int                     nChannelID;                 // sub system no.(from 0)
    NET_TIME                stuTime;                    // event time
    EM_SUBSYSTEM_STATE_TYPE emState;                    // changed status
}ALARM_SUBSYSTEM_STATE_CHANGE_INFO;

// battery scheduled notice event
typedef struct tagALARM_BATTERYPOWER_INFO
{
    DWORD       dwSize;
    int         nChannel;                                  // battery no.
    NET_TIME    stuTime;                                   // event time
    int         nPercent;                                  // battery percentage
    float       fVoltage;                                  // Voltage unit:V
    BOOL        bCharging;                                 // Charge status,true:Charging false:not Charge
}ALARM_BATTERYPOWER_INFO;

//bell status event
typedef struct tagALARM_BELLSTATUS_INFO 
{
    DWORD       dwSize;
    int         nChannel;                                   // bell channel no. 
    NET_TIME    stuTime;                                    // event time
    int         nAction;                                    // 0:start 1:stop
}ALARM_BELLSTATUS_INFO;

// zone status change event, corresponding to event type DH_ALARM_DEFENCE_STATE_CHANGE_EVENT
typedef struct tagALARM_DEFENCE_STATUS_CHANGE_INFO 
{
    DWORD                   dwSize;
    int                     nChannelID;                     // zone ID, as alarm input channel no.(from 0)
    NET_TIME                stuTime;                        // event time
    EM_DEFENCE_STATE_TYPE   emStatus;                       // zone changed status
}ALARM_DEFENCE_STATUS_CHANGE_INFO;

// login failed event
typedef struct tagALARM_LOGIN_FAILIUR_INFO
{
    DWORD                   dwSize;
    int                     nAction;                        // 0:start 1:stop
    unsigned int            nSequence;                      // no.
    char                    szName[DH_EVENT_NAME_LEN];      // event name, fill in user name
    char                    szType[DH_COMMON_STRING_128];   // login type
    char                    szAddr[DH_MAX_IPADDR_EX_LEN];   // source IP address
    int                     nError;                         // user login failed error code
}ALARM_LOGIN_FAILIUR_INFO;

// ticket statistics info event corresponding to event type DH_ALARM_TICKET_STATISTIC
typedef struct tagALARM_TICKET_STATISTIC
{
    DWORD                   dwSize;
    NET_GPS_STATUS_INFO     stuGPSStatusInfo;               // GPS info
    NET_TIME                stuTime;                        // event report time
    int                     nNormalCount;                   // general ticket number
    int                     nDiscountCount;                 // discount ticket
    int                     nSubsidyCount;                  // yound and old ticket number
    int                     nDriverCount;                   // driver ticket number
    int                     nNormalPrice;                   // general ticket price
    int                     nDiscountPrice;                 // discount ticket price
    int                     nSubsidyPrice;                  // young and old ticket price
}ALARM_TICKET_STATISTIC;

#define MAX_ALARMEXT_MODULE_NUM     256                     // max expansionmodule quantity

// BUS type
typedef enum tagNET_BUS_TYPE
{
    NET_BUS_TYPE_UNKNOWN = 0,
    NET_BUS_TYPE_MBUS,                                      // M-BUS
    NET_BUS_TYPE_RS485,                                     // RS-485    
    NET_BUS_TYPE_CAN,                                       // CAN
}NET_BUS_TYPE;

// expansion module offline event  corresponding to event type DH_ALARM_MODULE_LOST
typedef struct tagALARM_MODULE_LOST_INFO 
{
    DWORD                   dwSize;
    NET_TIME                stuTime;                        // event report time
    int                     nSequence;                      // expansion module BUS no.(from 0)
    NET_BUS_TYPE            emBusType;                      // BUS type
    int                     nAddr;                          // offline expansion module quantity
    int                     anAddr[MAX_ALARMEXT_MODULE_NUM];// offline expansionmodule no.(from 0)
	char					szDevType[DH_COMMON_STRING_64];	// device type "SmartLock"when type of"AlarmDefence"Index address is Alarm number
	BOOL					bOnline;						// Online status. The default setup is false. False=offline, true=online
}ALARM_MODULE_LOST_INFO;

// PSTN offline event
typedef struct tagALARM_PSTN_BREAK_LINE_INFO
{
    DWORD                   dwSize;
    int                     nChannelID;                 // tel wire no.(from 0)
    int                     nAction;                    // 0:start  1:stop
    NET_TIME                stuTime;                    // event time
}ALARM_PSTN_BREAK_LINE_INFO;

// analog alarm event(instant event), specific sensot type(as overspeed, mealige, oil)trigger
typedef struct tagALARM_ANALOGPULSE_INFO 
{
    DWORD                   dwSize;
    int                     nChannelID;                     // analog alarm channel no.
    NET_TIME                stuEventTime;                   // alarm event time
    NET_SENSE_METHOD        emSense;                        // sensor type
    char                    szName[DH_COMMON_STRING_128];   // sensor name
    int                     nStatus;                        // data status, -1:unknown,0:normal,1:invalid data(over range),
                                                            // 2:over threshold 1,3:over threshold 2,4: over threshold 3,5 over threshold 4,
                                                            // 6:below threshold 1,7: below threshold 2,8: below threshold 3,9: below threshold 4
    float                   fValue;                         // senspr value
    NET_TIME                stuCollectTime;                 // collection time
    NET_GPS_STATUS_INFO     stuGPSStatusInfo;               // GPS info
}ALARM_ANALOGPULSE_INFO;

//dispatch info confirmation type 
typedef enum tagNET_MISSION_CONFIRM_TYPE
{
    NET_MISSION_CONFIRM_UNKNOWN = 0 ,                        // unknown 
    NET_MISSION_CONFIRM_YES ,                                // agree and execute
	NET_MISSION_CONFIRM_NO ,                                 // refuse
}NET_MISSION_CONFIRM_TYPE;

// task confirmation event(corresponding to command DH_ALARM_MISSION_CONFIRM)
typedef struct tagALARM_MISSION_CONFIRM_INFO 
{
    DWORD                   dwSize;
    char                    szTitle[DH_COMMON_STRING_256];    // task name
    char                    szMissionID[DH_COMMON_STRING_256];// task ID
    char                    szComment[DH_COMMON_STRING_256];  // replay content
    NET_GPS_STATUS_INFO     stuGPSStatusInfo;                 // GPS info
    NET_MISSION_CONFIRM_TYPE emConfirmType;                   // dispatch info confirmation type 
}ALARM_MISSION_CONFIRM_INFO;

// collect Node group  type 
typedef enum tagNET_CLUSTER_GROUP_TYPE
{
  NET_CLUSTER_GROUP_UNKNOWN,            // unknown 
  NET_CLUSTER_GROUP_IDLE,               // stand alone
  NET_CLUSTER_GROUP_MASTER,             // host
  NET_CLUSTER_GROUP_SLAVE,              // spare
}NET_CLUSTER_GROUP_TYPE;

// Node activate event , corresponding to event type DH_ALARM_NODE_ACTIVE
typedef struct tagALARM_NODE_ACTIVE_INFO 
{
  DWORD         dwSize;
  int           nChannelID;         		// collect node no.
  NET_TIME      stuTime;          			// event occurred event 
  NET_CLUSTER_GROUP_TYPE emGroup;          	//  current Node group
}ALARM_NODE_ACTIVE_INFO;

// collect Node status 
typedef struct tagNET_CLUSTER_STATE_INFO 
{
  DWORD          			dwSize;
  NET_CLUSTER_GROUP_TYPE 	emGroup;          //  current Node group
}NET_CLUSTER_STATE_INFO;

// device alarm mode 
typedef enum tagEM_ALARM_VIDEO_STATIC_MODE
{
  EM_ALARM_VIDEO_STATIC_MODE_UNKNOWN,         // unknown 
  EM_ALARM_VIDEO_STATIC_MODE_INDOOR,          // indoor mode "Indoor"
  EM_ALARM_VIDEO_STATIC_MODE_OUTDOOR,         // outdoor mode "Outdoor"
}EM_ALARM_VIDEO_STATIC_MODE;

// video statis detection event , corresponding to event type DH_ALARM_VIDEO_STATIC
typedef struct tagALARM_VIDEO_STATIC_INFO 
{
  DWORD            dwSize;
  DWORD            dwAction;            // event action, 0:Start, 1:Stop
  EM_ALARM_VIDEO_STATIC_MODE emMode;    // device alarm mode 
  int             nChannelID;           // video statis detection alarm channel no.
}ALARM_VIDEO_STATIC_INFO;

// active registration device re-login event , corresponding to event type DH_ALARM_REGISTER_REONLINE
typedef struct tagALARM_REGISTER_REONLINE_INFO
{
  DWORD      dwSize;
}ALARM_REGISTER_REONLINE_INFO;

// connection status 
typedef enum tagEM_ALARM_ISCSI_STATUS
{
  EM_ALARM_ISCSI_STATUS_UNKNOWN,         		// unknown 
  EM_ALARM_ISCSI_STATUS_CONNECTED,        		// connected
  EM_ALARM_ISCSI_STATUS_NOT_CONNECTED,      	// unconnected
  EM_ALARM_ISCSI_STATUS_CONNECT_FAILED,      	// connection failed
  EM_ALARM_ISCSI_STATUS_AUTHENTICATION_FAILED,  // authentication failed
  EM_ALARM_ISCSI_STATUS_CONNECT_TIMEOUT,     	// connection timed out
  EM_ALARM_ISCSI_STATUS_NOT_EXIST,        		// device does not exist
}EM_ALARM_ISCSI_STATUS;

// ISCSI alarm event , corresponding to event type DH_ALARM_ISCSI_STATUS
typedef struct tagALARM_ISCSI_STATUS_INFO
{
  DWORD         dwSize;
  int           nIndex;               				// means ISCSIdevice index no.
  char          szAddress[DH_COMMON_STRING_32];   	// server address
  char          szName[DH_COMMON_STRING_64];    	// Target name 
  char          szUserName[DH_COMMON_STRING_64];  	// username
  EM_ALARM_ISCSI_STATUS  emStatus;              	// connection status 
}ALARM_ISCSI_STATUS_INFO;

// detection collection device alarm event , corresponding to event type DH_ALARM_SCADA_DEV_ALARM
typedef struct tagALARM_SCADA_DEV_INFO 
{
    DWORD       dwSize;
    int         nChannel;                           // channel no.
    NET_TIME    stuTime;                            // event occur time
    char        szDevName[DH_COMMON_STRING_64];     // failure device name 
    char        szDesc[DH_COMMON_STRING_256];       // alarm description
    int         nAction;                            // -1:unknown, 0:pulse, 1:start, 2:stop
    char        szID[DH_COMMON_STRING_32];          // point ID, current len is 16 bytes
    char        szSensorID[DH_COMMON_STRING_32];    // sensor ID, current len is 16 bytes
    char        szDevID[DH_COMMON_STRING_32];       // device ID, current len is 16 bytes
    char        szPointName[DH_COMMON_STRING_64];   // point name, matching with spot chart
    int         nAlarmFlag;                         // 0:start, 1:stop
}ALARM_SCADA_DEV_INFO;

// vehicle ACC alarm event, correspond event: DH_ALARM_VEHICLE_ACC
typedef struct tagALARM_VEHICLE_ACC_INFO 
{
    DWORD                   dwSize;
    int                     nACCStatus;                         // ACC state, 0:invalid, 1:open, 2:close 
    int                     nAction;                            // event action, 0:Start, 1:Stop
    NET_GPS_STATUS_INFO     stuGPSStatusInfo;                   // GPS informmation
    int                     nConstantElectricStatus;            // Constant electric states, 0:unknown, 1:connect, 2:break                            
	NET_TIME_EX				stuTime;							// Event occurrence time 
}ALARM_VEHICLE_ACC_INFO;

typedef NET_ACCESS_CTL_STATUS_TYPE  NET_ACCESS_LOCK_STATUS_TYPE;

// locked status, correspond to DH_ALARM_ACCESS_LOCK_STATUS
typedef struct tagALARM_ACCESS_LOCK_STATUS_INFO 
{
    DWORD                       dwSize;
    int                         nChannel;                       // door channal (from 0)
    NET_TIME                    stuTime;                        // the time of event occurs
    NET_ACCESS_LOCK_STATUS_TYPE emLockStatus;                   // locked status
}ALARM_ACCESS_LOCK_STATUS_INFO;

// SEVER TYPE
typedef enum tagEM_ALARM_FINACE_SCHEME_BUSINESS_SEVER_TYPE
{
    EM_ALARM_FINACE_SCHEME_BUSINESS_SEVER_TYPE_UNKNOWN,         // Unknown
    EM_ALARM_FINACE_SCHEME_BUSINESS_SEVER_TYPE_ORGANIZATION,    // Organization wealth investment 
    EM_ALARM_FINACE_SCHEME_BUSINESS_SEVER_TYPE_SEAL,            // Sealed wealth investment 
    EM_ALARM_FINACE_SCHEME_BUSINESS_SEVER_TYPE_EXCLUSIVE,       // Exclusive wealth investment 
    EM_ALARM_FINACE_SCHEME_BUSINESS_SEVER_TYPE_GATHER,          // Combination wealth investment 
    EM_ALARM_FINACE_SCHEME_BUSINESS_SEVER_TYPE_EVEN,            // Capital protection wealth investment 
}EM_ALARM_FINACE_SCHEME_BUSINESS_SEVER_TYPE;

// Finace event, correspond to DH_ALARM_FINACE_SCHEME
typedef struct tagALARM_FINACE_SCHEME_INFO 
{
    DWORD                       dwSize;
    int                         nChannel;                                   // channel ID
    char                        szBusinessSeverID[DH_COMMON_STRING_64];     // business sever ID
    EM_ALARM_FINACE_SCHEME_BUSINESS_SEVER_TYPE  emBusinessSeverType;        // business sever type
    NET_TIME                    stuFinaceSchemeTime;                        // finance scheme time
    NET_TIME                    stuSchemeStartTime;                         // scheme start time
    NET_TIME                    stuSchemeEndTime;                           // scheme end time
    char                        szProductName[DH_COMMON_STRING_128];        // product name
    char                        szCardNo[DH_COMMON_STRING_64];              // card ID
    char                        szPersonID[DH_COMMON_STRING_64];            // persion ID
    char                        szPersonName[DH_COMMON_STRING_64];          // persion name
    char                        szOrgNo[DH_COMMON_STRING_64];               // subbranch No
    char                        szStuffID[DH_COMMON_STRING_64];             // stuff ID
    NET_TIME                    stuRecStartTime;                            // record start time
    NET_TIME                    stuRecEndTime;                              // record end time
}ALARM_FINACE_SCHEME_INFO;

//////////////////////////////////////////////////////////////////////////

// New Record Set Operation(Insert)Parameter
typedef struct tagNET_CTRL_RECORDSET_IN
{
	DWORD	dwSize;
	EM_NET_RECORD_TYPE	emType;					// Record Information Type
	void*	pBuf;								// Record Information Cache,The EM_NET_RECORD_TYPE Note is Details,the space application by the user
	int		nBufLen;							// Record Information Cache Size,please refer to the structure of EM_NET_RECORD_TYPE
}NET_CTRL_RECORDSET_INSERT_IN;

// Record New Operation(Insert) Parameter
typedef struct tagNET_CTRL_RECORDSET_OUT 
{
    DWORD           dwSize;
    int             nRecNo;                             // Record Number(The Device Come Back When New Insert )
}NET_CTRL_RECORDSET_INSERT_OUT;

// Record New Operation (Insert)Parameter 
typedef struct tagNET_CTRL_RECORDSET_INSERT_PARAM 
{
    DWORD                           dwSize;
    NET_CTRL_RECORDSET_INSERT_IN    stuCtrlRecordSetInfo;       // Record Information(User Write)
    NET_CTRL_RECORDSET_INSERT_OUT   stuCtrlRecordSetResult;     // Record Information(the Device Come Back)
}NET_CTRL_RECORDSET_INSERT_PARAM;

// Record Operation Parameter
typedef struct tagNET_CTRL_RECORDSET_PARAM 
{
    DWORD               dwSize;
    EM_NET_RECORD_TYPE  emType;                         // Record Information Type
    void*               pBuf;                           // New/Renew/Inquire,It is Record Information Cache, the EM_NET_RECORD_TYPE Note is Details)
                                                        // Delete,It is Record Number(Int Model)
    int                 nBufLen;                     	// Record Information Cache Size,please refer to the structure of EM_NET_RECORD_TYPE
}NET_CTRL_RECORDSET_PARAM;

// video play mode
typedef enum tagEM_VIDEO_PLAY_MODE_TYPE
{
    EM_VIDEO_PLAY_MODE_TYPE_UNKNOWN,                    // unknown
    EM_VIDEO_PLAY_MODE_TYPE_ONCE,                       // once
    EM_VIDEO_PLAY_MODE_TYPE_REPEAT,                     // repeat
}EM_VIDEO_PLAY_MODE_TYPE;

#define MAX_DELIVERY_FILE_NUM 128                       // the max delivery file num
#define DELIVERY_FILE_URL_LEN 128                       // the max len of url of delivery file

// the delivery file type
typedef enum tagEM_DELIVERY_FILE_TYPE
{
    EM_DELIVERY_FILE_TYPE_UNKNOWN,                      // unknown
    EM_DELIVERY_FILE_TYPE_VIDEO,                        // video
    EM_DELIVERY_FILE_TYPE_IMAGE,                        // image
}EM_DELIVERY_FILE_TYPE;

// the information of delivery file
typedef struct tagNET_DELIVERY_FILE_INFO
{
    EM_DELIVERY_FILE_TYPE   emFileType;                         // file type
    char                    szFileURL[DELIVERY_FILE_URL_LEN];   // url of file
    int                     nImageSustain;                      // the second of sustain for image(it is valid only when emFileType is EM_DELIVERY_FILE_TYPE_IMAGE)
    BYTE                    byReserved[1024];                   // reserved
}NET_DELIVERY_FILE_INFO;

// Delivery file to the video output port, building intercom use, run at the same time
typedef struct tagNET_CTRL_DELIVERY_FILE 
{
    DWORD                   dwSize;                             // struct size
    int                     nPort;                              // the port index
    EM_VIDEO_PLAY_MODE_TYPE emPlayMode;                         // the play mode
    NET_TIME                stuStartPlayTime;                   // the time of start play
    NET_TIME                stuStopPlayTime;                    // the time of stop play, it is valid only when emPlayMode is EM_VIDEO_PLAY_MODE_TYPE_REPEAT   
    int                     nFileCount;                         // file count
    NET_DELIVERY_FILE_INFO  stuFileInfo[MAX_DELIVERY_FILE_NUM]; // array of file info
}NET_CTRL_DELIVERY_FILE;

// Serving file information
typedef struct tagNET_DELIVERY_FILE_BYCAR_INFO
{
    EM_DELIVERY_FILE_TYPE   emFileType;                         // file type
    char                    szFileURL[DELIVERY_FILE_URL_LEN];   // url of file
    int                     nImageSustain;                      // the second of sustain for image(it is valid only when emFileType is EM_DELIVERY_FILE_TYPE_IMAGE)
    NET_TIME                stuStartPlayTime;                   // the time of start play
    NET_TIME                stuStopPlayTime;                    // the time of stop play, it is valid only when emPlayMode is EM_VIDEO_PLAY_MODE_TYPE_REPEAT
    BYTE                    byReserved[1024];                   // Reserved
}NET_DELIVERY_FILE_BYCAR_INFO;

// Send video and image files to video output, Used by car, The ad time si served separately
typedef struct tagNET_CTRL_DELIVERY_FILE_BYCAR
{
    DWORD                           dwSize;                             // struct size
    int                             nPort;                              // the port index
    EM_VIDEO_PLAY_MODE_TYPE         emPlayMode;                         // the play mode
    int                             nFileCount;                         // file count
    NET_DELIVERY_FILE_BYCAR_INFO    stuFileInfo[MAX_DELIVERY_FILE_NUM]; // array of file info
}NET_CTRL_DELIVERY_FILE_BYCAR;

// ״̬
typedef enum tagNET_ACCESSCTLCARD_STATE
{
    NET_ACCESSCTLCARD_STATE_UNKNOWN = -1,
    NET_ACCESSCTLCARD_STATE_NORMAL = 0,                 // Normal
    NET_ACCESSCTLCARD_STATE_LOSE   = 0x01,              // Lose
    NET_ACCESSCTLCARD_STATE_LOGOFF = 0x02,              // Logoff
    NET_ACCESSCTLCARD_STATE_FREEZE = 0x04,              // Freeze
    NET_ACCESSCTLCARD_STATE_ARREARAGE = 0x08,           // Arrears
    NET_ACCESSCTLCARD_STATE_OVERDUE = 0x10,             // Overdue
    NET_ACCESSCTLCARD_STATE_PREARREARAGE = 0x20,        // Pre-Arrears(still can open the door)
}NET_ACCESSCTLCARD_STATE;

// Card Type 
typedef enum tagNET_ACCESSCTLCARD_TYPE
{
    NET_ACCESSCTLCARD_TYPE_UNKNOWN = -1,
    NET_ACCESSCTLCARD_TYPE_GENERAL,                     // General Card
    NET_ACCESSCTLCARD_TYPE_VIP,                         // VIP Card
    NET_ACCESSCTLCARD_TYPE_GUEST,                       // Guest Card
    NET_ACCESSCTLCARD_TYPE_PATROL,                      // Patrol Card
    NET_ACCESSCTLCARD_TYPE_BLACKLIST,                   // Blacklist Card
    NET_ACCESSCTLCARD_TYPE_CORCE,                       // Corce Card
    NET_ACCESSCTLCARD_TYPE_POLLING,                     // Polling Card
    NET_ACCESSCTLCARD_TYPE_MOTHERCARD = 0xff,           // Mother Card
}NET_ACCESSCTLCARD_TYPE;

#define DH_MAX_DOOR_NUM               32                // Max Door Number 
#define DH_MAX_TIMESECTION_NUM        32                // Max Time Section Number
#define DH_MAX_CARDNAME_LEN           64                // access control naming max length
#define DH_MAX_CARDNO_LEN             32                // Max Card-Number Len
#define DH_MAX_USERID_LEN             32                // Max User ID Len
#define DH_MAX_IC_LEN                 32                // Max identify card len
#define DH_MAX_QRCODE_LEN             128               // Max QRCode len 

// fingerprint data, for sending only
typedef struct tagNET_ACCESSCTLCARD_FINGERPRINT_PACKET
{
    DWORD   dwSize;
    int     nLength;        // length of a finger print packet, unit: byte
    int     nCount;         // packet number 
    char*   pPacketData;    // all fingerprint packet in a single buffer, allocated and filled by user, nLength*nCount bytes
}NET_ACCESSCTLCARD_FINGERPRINT_PACKET;

// fingerprint data, for sending and receiving
typedef struct tagNET_ACCESSCTLCARD_FINGERPRINT_PACKET_EX
{
    int     nLength;        // length of a finger print packet, unit: byte
    int     nCount;         // packet number 
    char*   pPacketData;    // all fingerprint packet in a single buffer, allocated by user,the space application is over nLength * nCount
    int     nPacketLen;     // pPacketData buffer length, set by user
    int     nRealPacketLen; // The actual fingerprint size returned to the user, equal to nLength*nCount
    int     nDuressIndex;   // duress index of fingerprint group, range: 1~nCount
    BYTE    byReverseed[1020]; //Reserved size 
}NET_ACCESSCTLCARD_FINGERPRINT_PACKET_EX;

#define MAX_REPEATENTERROUTE_NUM   12 //max repeat enter route num 

//Entrance Guard Record Information
typedef struct tagNET_RECORDSET_ACCESS_CTL_CARD
{
    DWORD           dwSize;
    int             nRecNo;                                 // Record Number,Read-Only
    NET_TIME        stuCreateTime;                          // Creat Time
    char            szCardNo[DH_MAX_CARDNO_LEN];            // Card number
    char            szUserID[DH_MAX_USERID_LEN];            // User's ID
    NET_ACCESSCTLCARD_STATE       emStatus;                 // Card Stetue
    NET_ACCESSCTLCARD_TYPE        emType;                   // Card Type
    char            szPsw[DH_MAX_CARDPWD_LEN];              // Card Password
    int             nDoorNum;                               // Valid Door Number;
    int             sznDoors[DH_MAX_DOOR_NUM];              // Privileged Door Number,That is CFG_CMD_ACCESS_EVENT Configure Array Subscript
    int             nTimeSectionNum;                        // the Number of Effective Open Time
    int             sznTimeSectionNo[DH_MAX_TIMESECTION_NUM];  // Open Time Segment Index,That is CFG_ACCESS_TIMESCHEDULE_INFO Array subscript
    int             nUserTime;                              // Frequency of Use
    NET_TIME        stuValidStartTime;                      // Valid Start Time 
    NET_TIME        stuValidEndTime;                        // Valid End Time
    BOOL            bIsValid;                               // Wether Valid,True =Valid,False=Invalid
    NET_ACCESSCTLCARD_FINGERPRINT_PACKET stuFingerPrintInfo;// fingerprint data info (send only), DEPRECATED! use stuFingerPrintInfoEx instead
    BOOL            bFirstEnter;                            // has first card or not
    char            szCardName[DH_MAX_CARDNAME_LEN];        // card naming
    char            szVTOPosition[DH_COMMON_STRING_64];     // VTO link position
    BOOL            bHandicap;                              // Card for handicap, TRUE:yes, FALSE:no
    BOOL            bEnableExtended;                        // Enabled member stuFingerPrintInfoEx
    NET_ACCESSCTLCARD_FINGERPRINT_PACKET_EX     stuFingerPrintInfoEx; // fingerprint data info structure   
    int             nFaceDataNum;                           // face detection data number,can not > 20
    char            szFaceData[MAX_FACE_DATA_NUM][MAX_FACE_DATA_LEN];// face detection data
    char            szDynamicCheckCode[MAX_COMMON_STRING_16];       // dynamic check code
    int             nRepeatEnterRouteNum;                         // repeat enter route num 
    int             arRepeatEnterRoute[MAX_REPEATENTERROUTE_NUM]; // repeat enter route
    int             nRepeatEnterRouteTimeout;                     // repeat enter route timeout
    BOOL            bNewDoor;                                     // enable to new field, TRUE: user nNewDoorNum,nNewDoors
    int             nNewDoorNum;                                  // Valid Door Number;
    int             nNewDoors[MAX_ACCESSDOOR_NUM];                // Privileged Door Number, That is CFG_CMD_ACCESS_EVENT Configure Array Subscript
    int             nNewTimeSectionNum;                           // the Number of Effective Open Time
    int             nNewTimeSectionNo[MAX_ACCESSDOOR_NUM];        //Open Time Segment Index,That is CFG_ACCESS_TIMESCHEDULE_INFO Array subscript
    char            szCitizenIDNo[MAX_COMMON_STRING_32];          // ID card no
}NET_RECORDSET_ACCESS_CTL_CARD;

// Entrance Guard Record  Information
typedef struct tagNET_RECORDSET_ACCESS_CTL_PWD 
{
    DWORD           dwSize;
    int             nRecNo;                                 // Record Number,Read-Only
    NET_TIME        stuCreateTime;                          // Create Time
    char            szUserID[DH_MAX_USERID_LEN];            // User's ID
    char            szDoorOpenPwd[DH_MAX_CARDPWD_LEN];      // Open Password
    char            szAlarmPwd[DH_MAX_CARDPWD_LEN];         // Alarm Password
    int             nDoorNum;                               // Valid Door Number
    int             sznDoors[DH_MAX_DOOR_NUM];              // Privileged Door Number,That is CFG_CMD_ACCESS_EVENT Configure Array Subscript
    char            szVTOPosition[DH_COMMON_STRING_64];     // VTO link position    
    int             nTimeSectionNum;                        // the Number of Effective Open Time
    int             nTimeSectionIndex[DH_MAX_TIMESECTION_NUM]; // Open Timesection Index array,each element corresponds to each element of sznDoors
    BOOL            bNewDoor;                               // enable to new field,TRUE: user nNewDoorNum,nNewDoors
    int             nNewDoorNum;                            // Valid Door Number;
    int             nNewDoors[MAX_ACCESSDOOR_NUM];          // Privileged Door Number,That is CFG_CMD_ACCESS_EVENT Configure Array Subscript
    int             nNewTimeSectionNum;                     // the Number of Effective Open Time
    int             nNewTimeSectionNo[MAX_ACCESSDOOR_NUM];  // Open Time Segment Index,That is CFG_ACCESS_TIMESCHEDULE_INFO Array subscript
}NET_RECORDSET_ACCESS_CTL_PWD;


// Door Open Method(Entrance Guard Configure,One Way of Door Work )
typedef enum tagNET_DOOR_OPEN_METHOD
{
    NET_DOOR_OPEN_METHOD_UNKNOWN = 0,
    NET_DOOR_OPEN_METHOD_PWD_ONLY,                          // Password Open is Only
    NET_DOOR_OPEN_METHOD_CARD,                              // Card  open is Only
    NET_DOOR_OPEN_METHOD_PWD_OR_CARD,                       // Password or Card
    NET_DOOR_OPEN_METHOD_CARD_FIRST,                        // First Card then Password
    NET_DOOR_OPEN_METHOD_PWD_FIRST,                         // First Card then Password
    NET_DOOR_OPEN_METHOD_SECTION,                           // Sub-Periods
}NET_DOOR_OPEN_METHOD;

// Door Open Method(Entrance Guard Event,Entrance Guard get In/Out Record, Actual Open Door Method)
typedef enum tagNET_ACCESS_DOOROPEN_METHOD
{
    NET_ACCESS_DOOROPEN_METHOD_UNKNOWN = 0,
    NET_ACCESS_DOOROPEN_METHOD_PWD_ONLY,                    // Password
    NET_ACCESS_DOOROPEN_METHOD_CARD,                        // Card
    NET_ACCESS_DOOROPEN_METHOD_CARD_FIRST,                  // First Card Then Password
    NET_ACCESS_DOOROPEN_METHOD_PWD_FIRST,                   // First Password Then Card 
    NET_ACCESS_DOOROPEN_METHOD_REMOTE,                      // Long-Range Open,Such as Through theIndoor Unit or Unlock the Door Machine Platform
    NET_ACCESS_DOOROPEN_METHOD_BUTTON,                      // Open Door Button
    NET_ACCESS_DOOROPEN_METHOD_FINGERPRINT,                 // fingerprint lock
    NET_ACCESS_DOOROPEN_METHOD_PWD_CARD_FINGERPRINT,        // password+swipe card+fingerprint combination unlock
    NET_ACCESS_DOOROPEN_METHOD_PWD_FINGERPRINT = 10,        // password+fingerprint combination unlock
    NET_ACCESS_DOOROPEN_METHOD_CARD_FINGERPRINT = 11,       // swipe card+fingerprint combination unlock
    NET_ACCESS_DOOROPEN_METHOD_PERSONS = 12,                // multi-people unlock
    NET_ACCESS_DOOROPEN_METHOD_KEY = 13,                    // Key
    NET_ACCESS_DOOROPEN_METHOD_COERCE_PWD = 14,             // Use force password to open the door     
    NET_ACCESS_DOOROPEN_METHOD_FACE_RECOGNITION = 16,       // face recogniton to open the door
}NET_ACCESS_DOOROPEN_METHOD;

//attendance state
typedef enum tagNET_ATTENDANCESTATE
{
    NET_ATTENDANCESTATE_UNKNOWN,
    NET_ATTENDANCESTATE_SIGNIN,
    NET_ATTENDANCESTATE_GOOUT,
    NET_ATTENDANCESTATE_GOOUT_AND_RETRUN,
    NET_ATTENDANCESTATE_SIGNOUT,
    NET_ATTENDANCESTATE_WORK_OVERTIME_SIGNIN,
    NET_ATTENDANCESTATE_WORK_OVERTIME_SIGNOUT,
}NET_ATTENDANCESTATE;

// Access Control open door Direction
typedef enum tagNET_ENUM_DIRECTION_ACCESS_CTL
{
    NET_ENUM_DIRECTION_UNKNOWN,
    NET_ENUM_DIRECTION_ENTRY,                              // entry             
    NET_ENUM_DIRECTION_EXIT,                               // exit
}NET_ENUM_DIRECTION_ACCESS_CTL;

// Access Control card Record Information
typedef struct tagNET_RECORDSET_ACCESS_CTL_CARDREC
{
    DWORD           dwSize;
    int             nRecNo;                                 // Record Number,Read-Only
    char            szCardNo[DH_MAX_CARDNO_LEN];            // Card Number
    char            szPwd[DH_MAX_CARDPWD_LEN];              // Password
    NET_TIME        stuTime;                                // Swing Card Time
    BOOL            bStatus;                                // Swing Card Result,True is Success,False is Fail
    NET_ACCESS_DOOROPEN_METHOD    emMethod;                 // Open Door Method
    int             nDoor;                                  // Door Number,That is CFG_CMD_ACCESS_EVENT Configure Array Subscript 
    char            szUserID[DH_MAX_USERID_LEN];            // user ID
    int             nReaderID;                              // card reader ID (abandoned)
    char            szSnapFtpUrl[MAX_PATH];                 // unlock snap upload ftp url
    char            szReaderID[DH_COMMON_STRING_32];        // card reader ID
    NET_ACCESSCTLCARD_TYPE      emCardType;                         // Card Type
    int                         nErrorCode;                         // Reason of unlock failure, only because it is valid when bStatus is FALSE
                                                                    // 0x00 no error
                                                                    // 0x10 unauthorized
                                                                    // 0x11 card lost or cancelled
                                                                    // 0x12 no door right
                                                                    // 0x13 unlock mode error
                                                                    // 0x14 valid period error
                                                                    // 0x15 anti sneak into mode
                                                                    // 0x16 forced alarm not unlocked
                                                                    // 0x17 door NC status
                                                                    // 0x18 AB lock status
                                                                    // 0x19 patrol card
                                                                    // 0x1A device is under intrusion alarm status
                                                                    // 0x20 period error
                                                                    // 0x21 unlock period error in holiday period
                                                                    // 0x30 first card right check required
                                                                    // 0x40 card correct, input password error
                                                                    // 0x41 card correct, input password timed out
                                                                    // 0x42 card correct, input fingerprint error
                                                                    // 0x43 card correct, input fingerprint timed out
                                                                    // 0x44 fingerprint correct, input password error
                                                                    // 0x45 fingerprint correct, input password timed out
                                                                    // 0x50 group unlock sequence error
                                                                    // 0x51 test required for group unlock
                                                                    // 0x60 test passed, control unauthorized


    char           szRecordURL[MAX_COMMON_STRING_128];              // record url
    int            nNumbers;                                        // snap picture numbers
    NET_ATTENDANCESTATE         emAttendanceState;                  // attendance state 
    NET_ENUM_DIRECTION_ACCESS_CTL   emDirection;                    // open door direction
}NET_RECORDSET_ACCESS_CTL_CARDREC;

// Holiday Record Iinformation
typedef struct tagNET_RECORDSET_HOLIDAY
{
    DWORD           dwSize;
    int             nRecNo;                                 // Record Number,Read-Only
    int             nDoorNum;                               // Valid Door Number    
    int             sznDoors[DH_MAX_DOOR_NUM];              // Privileged Door Number,That is CFG_CMD_ACCESS_EVENT Configure Array Subscript
    NET_TIME        stuStartTime;                           // Start Time
    NET_TIME        stuEndTime;                             // End Time
    BOOL            bEnable;                                // Holiday Enable
    char            szHolidayNo[DH_COMMON_STRING_32];       // Holiday No
    char            szHolidayName[DH_COMMON_STRING_128];    // Holiday Name
}NET_RECORDSET_HOLIDAY;

// card province
typedef enum tagEM_CARD_PROVINCE
{
    EM_CARD_UNKNOWN            = 10,        // UNKNOWN
    EM_CARD_BEIJING            = 11,        // BeiJing
    EM_CARD_TIANJIN            = 12,        // TianJin
    EM_CARD_HEBEI              = 13,        // HeBei
    EM_CARD_SHANXI_TAIYUAN     = 14,        // ShanXi, the provincial capital is TaiYuan
    EM_CARD_NEIMENGGU          = 15,        // NeiMengGu
    EM_CARD_LIAONING           = 21,        // LiaoNing
    EM_CARD_JILIN              = 22,        // JiKin
    EM_CARD_HEILONGJIANG       = 23,        // HeiLongJiang
    EM_CARD_SHANGHAI           = 31,        // ShangHai
    EM_CARD_JIANGSU            = 32,        // JiangSu
    EM_CARD_ZHEJIANG           = 33,        // ZheJiang
    EM_CARD_ANHUI              = 34,        // AnHui 
    EM_CARD_FUJIAN             = 35,        // FuJian
    EM_CARD_JIANGXI            = 36,        // JiangXi
    EM_CARD_SHANDONG           = 37,        // ShanDong 
    EM_CARD_HENAN              = 41,        // HeNan
    EM_CARD_HUBEI              = 42,        // HuBei
    EM_CARD_HUNAN              = 43,        // HuNan
    EM_CARD_GUANGDONG          = 44,        // GuangDong
    EM_CARD_GUANGXI            = 45,        // GuangXi
    EM_CARD_HAINAN             = 46,        // HaiNan
    EM_CARD_CHONGQING          = 50,        // ChongQing
    EM_CARD_SICHUAN            = 51,        // SiChuan
    EM_CARD_GUIZHOU            = 52,        // GuiZhou
    EM_CARD_YUNNAN             = 53,        // YunNan
    EM_CARD_XIZANG             = 54,        // XiZang
    EM_CARD_SHANXI_XIAN        = 61,        // ShanXi , the provincial capital is XiAn
    EM_CARD_GANSU              = 62,        // GanSu
    EM_CARD_QINGHAI            = 63,        // QingHai
    EM_CARD_NINGXIA            = 64,        // NingXia
    EM_CARD_XINJIANG           = 65,        // XinJiang
    EM_CARD_XIANGGANG          = 71,        // XiangGang
    EM_CARD_AOMEN              = 82,        // AoMen
} EM_CARD_PROVINCE;

// the type of the car
typedef enum tagEM_CAR_TYPE
{
    EM_CAR_UNKNOWN,    			// unknown
	EM_CAR_BUS,					// bus 
	EM_CAR_BIG_TRUCK,			// big truck
	EM_CAR_MEDIUM_TRUCK,		// medium truck
	EM_CAR_CAR,					// car 
	EM_CAR_VAN,					// van
	EM_CAR_SMALL_TRUCK,			// small truck
	EM_CAR_TRICYCLE,			// tricycle
	EM_CAR_MOTORCYCLE,			// motocycle
	EM_CAR_PEDESTRIAN,			// pedestrian
	EM_CAR_SUVMPV,				// SUV-MPV
	EM_CAR_MEDIUM_BUS,			// medium bus
	EM_CAR_DANGE_VEHICLE,		// hazardous chemical vehicles
} EM_CAR_TYPE;

// the tpye of the plate
typedef enum tagEM_PLATE_TYPE
{
	EM_PLATE_UNKNOWN,				// unkonwn
	EM_PLATE_BIGCAR,				// big car (black words on yellow background)
	EM_PLATE_SMALLCAR,				// small car (white words on blue background)
	EM_PLATE_EMBASSYCAR,			// embassy car (white words on black background, and the word 'ʹ' is red)
	EM_PLATE_CONSULATECAR,			// consulate car (white words on black background, and the word '' is red)
	EM_PLATE_ABROADCAR,				// abroad car (white/red words on black background)
	EM_PLATE_FOREIGNCAR,			// foreign car (white words on black background)
	EM_PLATE_POLICE,				// police plate
	EM_PLATE_ARMEDPOLICE,			// armed police plate
	EM_PLATE_TROOPS,				// troops plate
	EM_PLATE_TROOPSDOUBLE,			// troops plate which is double layer
	EM_PLATE_YELLOWTAILDOUBLE,		// yellow tail plate which double layer
	EM_PLATE_COACHCAR,				// coach car plate
	EM_PLATE_PERSONALITY,			// personality plate
	EM_PLATE_AGRICULTURAL,			// agricultural plate
	EM_PLATE_MOTORCYCLE,			// motorcycle plate
	EM_PLATE_TRACTOR,				// tractor plate
	EM_PLATE_SMALLCAR_BLACK,		// small car (white words on black background)
	EM_PLATE_RED,					// red plate
	EM_PLATE_BLUE,					// blue plate
	EM_PLATE_WHITE,					// white plate
	EM_PLATE_PURE_NEW_SMALLCAR,		// pure electric and new enegry car
	EM_PLATE_BLEND_NEW_SMALLCAR,	// hybrid and new enegry car
	EM_PLATE_PURE_NEW_BIGCAR,		// pure electric and new enegry cart
	EM_PLATE_BLEND_NEW_BIGCAR,		// hybrid and new enegry cart
} EM_PLATE_TYPE;

// car color
typedef enum tagEM_CAR_COLOR_TYPE
{
	EM_CAR_COLOR_WHITE,				// white
	EM_CAR_COLOR_BLACK,				// black
	EM_CAR_COLOR_RED,				// red
	EM_CAR_COLOR_YELLOW,			// yellow
	EM_CAR_COLOR_GRAY,				// gray
	EM_CAR_COLOR_BLUE,				// blue
	EM_CAR_COLOR_GREEN,				// green
	EM_CAR_COLOR_PINK,				// pink
	EM_CAR_COLOR_PURPLE,			// purple
	EM_CAR_COLOR_DARK_PURPLE,		// dark purple
	EM_CAR_COLOR_BROWN,				// brown
	EM_CAR_COLOR_MAROON,			// marron
	EM_CAR_COLOR_SILVER_GRAY,		// silver gray
	EM_CAR_COLOR_DARK_GRAY,			// dark gray
	EM_CAR_COLOR_WHITE_SMOKE,		// white smoke
	EM_CAR_COLOR_DEEP_ORANGE,		// deep orange
	EM_CAR_COLOR_LIGHT_ROSE,		// light rose
	EM_CAR_COLOR_TOMATO_RED,		// tomato red
	EM_CAR_COLOR_OLIVE,				// olive
	EM_CAR_COLOR_GOLDEN,			// golden
	EM_CAR_COLOR_DARK_OLIVE,		// dark olive
	EM_CAR_COLOR_YELLOW_GREEN,		// yellow green
	EM_CAR_COLOR_GREEN_YELLOW,		// green yellow
	EM_CAR_COLOR_FOREST_GREEN,		// forest green
	EM_CAR_COLOR_OCEAN_BLUE,		// ocean blue
	EM_CAR_COLOR_DEEP_SKYBLUE,		// deep sky blue
	EM_CAR_COLOR_CYAN,				// cyan
	EM_CAR_COLOR_DEEP_BLUE,			// deep blue
	EM_CAR_COLOR_DEEP_RED,			// deep red
	EM_CAR_COLOR_DEEP_GREEN,		// deep green
	EM_CAR_COLOR_DEEP_YELLOW,		// deep yellow
	EM_CAR_COLOR_DEEP_PINK,			// deep pink
	EM_CAR_COLOR_DEEP_PURPLE,		// deep purple
	EM_CAR_COLOR_DEEP_BROWN,		// deep brown
	EM_CAR_COLOR_DEEP_CYAN,			// deep cyan
	EM_CAR_COLOR_ORANGE,			// orange
	EM_CAR_COLOR_DEEP_GOLDEN,		// deep golden
	EM_CAR_COLOR_OTHER	= 255,		// other
} EM_CAR_COLOR_TYPE;

// use property
typedef enum tagEM_USE_PROPERTY_TYPE
{
	EM_USE_PROPERTY_OTHER,					// other
	EM_USE_PROPERTY_NOTOPERATING,			// not operating
	EM_USE_PROPERTY_HIGWAY,					// higway
	EM_USE_PROPERTY_BUS,					// bus
	EM_USE_PROPERTY_TAXI,					// taxi
	EM_USE_PROPERTY_TOURISM,				// tourism
	EM_USE_PROPERTY_FREIGHT,				// freight
	EM_USE_PROPERTY_LEASE,					// lease
	EM_USE_PROPERTY_POLICE,					// for police
	EM_USE_PROPERTY_FIRE,					// for fire police
	EM_USE_PROPERTY_RESCUE,					// for rescue
	EM_USE_PROPERTY_ENGINEERING,			// engineering emergency
	EM_USE_PROPERTY_OPERATION_TO_NOT,		// form operating to not operating
	EM_USE_PROPERTY_TAXI_TO_NOT,			// form taxi to not taxi
	EM_USE_PROPERTY_COACH,					// for coach
	EM_USE_PROPERTY_KINDER_SCHOOLBUS,		// kindergarten school bus
	EM_USE_PROPERTY_PUPIL_SCHOOLBUS,		// pupil school bus
	EM_USE_PROPERTY_OTHER_SCHOOLBUS,		// other school bus
	EM_USE_PROPERTY_FOR_DANGE_VEHICLE,		// for dangerous goods transportation
} EM_USE_PROPERTY_TYPE;

// the info of RFID electronic tag 
typedef struct tagNET_RFIDELETAG_INFO
{
	BYTE					szCardID[MAX_RFIDELETAG_CARDID_LEN];			// card ID
	int						nCardType;										// card type, 0:issued by transport administration offices, 1:new factory preloaded card
	EM_CARD_PROVINCE		emCardPrivince;									// card privince
	char					szPlateNumber[DH_MAX_PLATE_NUMBER_LEN];			// plate number
	char					szProductionDate[MAX_RFIDELETAG_DATE_LEN];		// production data
	EM_CAR_TYPE				emCarType;										// car type
	int						nPower;											// power, unit:kilowatt-hour, range:0~254, 255 means larger than maximum power value can be stored
	int						nDisplacement;									// displacement, unit:100ml, range:0~254, 255 means larger than maximum displacement value can be stored
	int						nAntennaID;										// antenna ID, range:1~4
	EM_PLATE_TYPE			emPlateType;									// plate type
	char					szInspectionValidity[MAX_RFIDELETAG_DATE_LEN];	// validity of inspection, year-month
	int						nInspectionFlag;								// the flag of inspetion, 0:already inspection, 1:not inspection
	int						nMandatoryRetirement;							// the years form effective inspection preiod to compulsory discarding preiod
	EM_CAR_COLOR_TYPE		emCarColor;										// car color
	int						nApprovedCapacity;								// authorized capacity, unit:people, <0:incalid
	int						nApprovedTotalQuality;							// total weight, unit:100kg, range:0~0x3FF,  0x3FF1023:larger than maximum value can be stored, <0:invalid
	NET_TIME_EX				stuThroughTime;									// the time when the car is pass
	EM_USE_PROPERTY_TYPE	emUseProperty;									// use property
	char					szPlateCode[MAX_COMMON_STRING_8];				// Licensing code, UTF-8 encoding
	char					szPlateSN[MAX_COMMON_STRING_16];				// Plate number, serial number, UTF-8 code
	BYTE               		bReserved[104];                      			// Reserved bytes
} NET_RFIDELETAG_INFO;

// electronic tag record info
typedef struct tagNET_RECORD_ELECTRONICSTAG_INFO
{
	DWORD					dwSize;							// Structure size 
	int						nRecNo;							// Read set SN. Read-only. 
	BOOL					bMatch;							// whether the picture has been matched
	UINT					unDevID;						// device ID, from 0
	NET_RFIDELETAG_INFO		stuEleTagInfo;					// RFID electronic tag info
} NET_RECORD_ELECTRONICSTAG_INFO;

// GPS position record set info 
typedef struct tagNET_RECORD_GPS_LOCATION_INFO
{
    DWORD           dwSize;                                 // Structure size 
    int             nRecNo;                                 // Read set SN. Read-only. 
    double          dLongitude;                             // GPS longitude. Unit is degree
    double          dLatitude;                              // GPS latitude. Unit is degree. 
    char            szPlaceName[128];                       // GPS coordinates name 
    char            szRadius[16];                           // Radius
} NET_RECORD_GPS_LOCATION_INFO;

// resident record info
typedef struct tagNET_RECORD_RESIDENT_INFO
{
    DWORD           dwSize;                                 // Structure size
    int             nRecNo;                                 // Read set SN. Read-only
    char            szResidentName[DH_MAX_USERID_LEN];      // user name
    char            szCardNo[DH_MAX_CARDNO_LEN];            // card number
    NET_TIME        stuStartTime;                           // start time of available
    NET_TIME        stuEndTime;                             // end time of available    
    char 			szICNum[DH_MAX_IC_LEN];					// Identify Card
}NET_RECORD_RESIDENT_INFO;

//Sensor record information
typedef struct tagNET_RECORD_SENSOR_RECORD
{
	DWORD						dwSize;                     // Structure size
	UINT						uRecNo;                     // Record Number,Read-Only
	NET_TIME					stuTime;					// Acquisition time, field is time
	UINT						uDeviceID;                  // Acquisition device ID, High 8 bits mean instrument type,low 24 bits mean the instrument serial number in the group.For example:0-electricity,1-analog sensor.
	BYTE						byStatus;                   // Data status,0:normal,1:abnormal.
	BYTE						byExceptionCode;            // abnormal data status mask,associated with specific instrument.Valid by Status is 1.
															// When instrument type is electricity, bit0 means under voltage, bit1 means over voltage, bit2 means under current, 
															// bit3 means over current, bit4 means invalid(this state is not set with other state at the same time)
	BYTE						bySwitchOn;					// switch is on or off, 0:off,1:on
	BYTE						byReserved;                 // reserve
	int							nPowerVoltage;              // Power voltage, unit:mV
	int							nPowerCurrent;              // Power current, unit:mA
	int							nPowerValue;				// Power value, unit:mW
	BYTE						byReservedSpace[4];         // reserve
	double						dActualValue;				// analogue use,variable,associated with specific analog instrument.
	char						szName[128];				// instrument name
	NET_SENSE_METHOD			emSenseMethod;				// SenseMethod, refer to the specific enumeration definition.
	EM_SENSE_METHOD_UNIT		emUnit;						// unit, it is associated with emSenseMethod.
}NET_RECORD_SENSOR_RECORD;

//AccessQRCode information
typedef struct tagNET_RECORD_ACCESSQRCODE_INFO
{
	DWORD                       dwSize;                                 // Structure size
	int                         nRecNo;                                 // Record Number,Read-Only
	char                        szQRCode[DH_MAX_QRCODE_LEN];            // QRCode
	UINT						nLeftTimes;                             // left times
	NET_TIME        			stuStartTime;                           // start time of available
    NET_TIME        			stuEndTime;                             // end time of available  
}NET_RECORD_ACCESSQRCODE_INFO;

//Access blue tooth information
typedef struct tagNET_RECORD_ACCESS_BLUETOOTH_INFO
{
    DWORD                       dwSize;                                 // Structure size
    int                         nRecNo;                                 // Record Number,Read-Only
    char						szUserName[DH_COMMON_STRING_128];       // User name
    char						szPassword[DH_COMMON_STRING_128];       // Password
    char        			    szMac[DH_COMMON_STRING_32];             // Mac addr
    char        			    szNote[DH_COMMON_STRING_128];           // User note info
}NET_RECORD_ACCESS_BLUETOOTH_INFO;

// the condition of query bluetooth accsess info 
typedef struct tagFIND_RECORD_ACCESS_BLUETOOTH_INFO_CONDITION
{
    DWORD					dwSize;
    char					szUserName[DH_COMMON_STRING_128];           // user name
} FIND_RECORD_ACCESS_BLUETOOTH_INFO_CONDITION;


// call type 
typedef enum tagEM_VIDEO_TALK_LOG_CALLTYPE
{
  EM_VIDEO_TALK_LOG_CALLTYPE_UNKNOWN = 0,   // unknown 
  EM_VIDEO_TALK_LOG_CALLTYPE_INCOMING,      // call in
  EM_VIDEO_TALK_LOG_CALLTYPE_OUTGOING,      // call out
  EM_VIDEO_TALK_LOG_CALLTYPE_MAX,         	//
}EM_VIDEO_TALK_LOG_CALLTYPE;

// terminal status 
typedef enum tagEM_VIDEO_TALK_LOG_ENDSTATE
{
  EM_VIDEO_TALK_LOG_ENDSTATE_UNKNOWN = 0,     // unknown 
  EM_VIDEO_TALK_LOG_ENDSTATE_MISSED,       	  // missed
  EM_VIDEO_TALK_LOG_ENDSTATE_RECEIVED,        // answered
  EM_VIDEO_TALK_LOG_ENDSTATE_MAX,         	  //
}EM_VIDEO_TALK_LOG_ENDSTATE;

// opposite type 
typedef enum tagEM_VIDEO_TALK_LOG_PEERTYPE
{
  EM_VIDEO_TALK_LOG_PEERTYPE_UNKNOWN = 0, // unknown 
  EM_VIDEO_TALK_LOG_PEERTYPE_VTO,         // VTO
  EM_VIDEO_TALK_LOG_PEERTYPE_VTH,         // VTH
  EM_VIDEO_TALK_LOG_PEERTYPE_VTS,         // management centre server
}EM_VIDEO_TALK_LOG_PEERTYPE;

// call record record set info 
typedef enum tagNET_RECORD_READFLAG
{
	NET_RECORD_READFLAG_UNREADED = 0, //Unread
	NET_RECORD_READFLAG_READED,	      //Read
	NET_RECORD_READFLAG_UNKNOWN,      //Unknown
}NET_RECORD_READFLAG;
// video talk record set log
typedef struct tagNET_RECORD_VIDEO_TALK_LOG
{
  DWORD             dwSize;
  int               nRecNo;               				// record set no., read-only
  NET_TIME          stuCreateTime;           			// start time
  EM_VIDEO_TALK_LOG_CALLTYPE   emCallType;             	// call type 
  EM_VIDEO_TALK_LOG_ENDSTATE   emEndState;             	// terminal status 
  char              szPeerNumber[DH_COMMON_STRING_64]; 	// opposite number	may be short no. or middle no., long no.
  EM_VIDEO_TALK_LOG_PEERTYPE   emPeerType;             	// opposite type 
  char              szLocalNumber[DH_COMMON_STRING_64]; // local no.	may be short no. or middle no., long no.
  int               nTalkTime;             				// talk time	unit second
  int               nMessageTime;            			// message time	unit second
  char              szPicturePath[DH_COMMON_STRING_128];// 
}NET_RECORD_VIDEO_TALK_LOG;


// audio talk status 
typedef enum tagEM_REGISTER_USER_STATE
{
  EM_REGISTER_USER_STATE_UNKNOWN = 0,         // unknown 
  EM_REGISTER_USER_STATE_IDLE,                // initial status 
  EM_REGISTER_USER_STATE_CALLING,             // call status , no response, 
  EM_REGISTER_USER_STATE_RINGING,             // ring status , opposite response, but no response, 
  EM_REGISTER_USER_STATE_PREVIEWING,          // previewing
  EM_REGISTER_USER_STATE_LEAVINGMESSAGE,      // messaging
  EM_REGISTER_USER_STATE_TALKING,             // talking
  EM_REGISTER_USER_STATE_MAX,                 //
}EM_REGISTER_USER_STATE;

// device type
typedef enum tagEM_REGISTER_USER_STATE_DEV_TYPE
{
    EM_REGISTER_USER_STATE_DEV_TYPE_UNKNOWN = 0,            // unknown
    EM_REGISTER_USER_STATE_DEV_TYPE_DIGIT_VTH,              // digital VTH
    EM_REGISTER_USER_STATE_DEV_TYPE_ANALOG_VTH,             // analogue VTH
    EM_REGISTER_USER_STATE_DEV_TYPE_DIGIT_VTO,              // digital VTO
    EM_REGISTER_USER_STATE_DEV_TYPE_ANALOG_VTO,             // analogue VTO
    EM_REGISTER_USER_STATE_DEV_TYPE_MAX,                    //
}EM_REGISTER_USER_STATE_DEV_TYPE;

// support lock or not
typedef enum tagEM_REGISTER_USER_STATE_SUPPORTLOCK_TYPE
{
    EM_REGISTER_USER_STATE_SUPPORTLOCK_TYPE_UNKNOWN = 0,                // Unknown
    EM_REGISTER_USER_STATE_SUPPORTLOCK_TYPE_SUPPORTLOCK,                // Support
    EM_REGISTER_USER_STATE_SUPPORTLOCK_TYPE_NOT_SUPPORTLOCK,            // Do not support
}EM_REGISTER_USER_STATE_SUPPORTLOCK_TYPE;

typedef enum tagNET_MONITORSTATE_TYPE
{
	NET_MONITORSTATE_TYPE_UNMONITORED = 0,		// 0 unmonitor (initial status)
	NET_MONITORSTATE_TYPE_MONITORREQUEST,		// 1 receive monitor query 
	NET_MONITORSTATE_TYPE_MONITORED,			// 2 monitor status 
	NET_MONITORSTATE_TYPE_UNKNOWN,				// Unknown status 
}NET_MONITORSTATE_TYPE;
// state record set info
typedef struct tagNET_RECORD_REGISTER_USER_STATE
{
    DWORD                                   dwSize;
    int                                     nRecNo;	                        // record set no., read-only
    NET_TIME                                stuCreateTime;                  // create time
    char                                    szUserID[DH_MAX_USERID_LEN];    // user ID
    BOOL									bOnline;                        // online or not
    char                                    szAddress[DH_MAX_IPADDR_OR_DOMAIN_LEN];   // network address
    int                                     nPort;                          // port no.  
    EM_REGISTER_USER_STATE                  emVideoTalkState;               // audio talk status 
    NET_TIME                                stuOnlineTime;                  // online time
    NET_TIME                                stuOfflineTime;                 // offline time
    EM_REGISTER_USER_STATE_DEV_TYPE         emDevType;                      // device type
    EM_REGISTER_USER_STATE_SUPPORTLOCK_TYPE	emSupportLock;                  // whether support lock
	NET_MONITORSTATE_TYPE					emMonitorState;					// Monitor State
}NET_RECORD_REGISTER_USER_STATE;

// visual talk analog VTH input mode 
typedef enum tagEM_VIDEO_TALK_CONTACT_VTSLAVEBINDMODE
{
  EM_VIDEO_TALK_CONTACT_VTSLAVEBINDMODE_UNKNOWN,       		// unknown 
  EM_VIDEO_TALK_CONTACT_VTSLAVEBINDMODE_SLAVEADDRESS,     	// via SlaveAddress
  EM_VIDEO_TALK_CONTACT_VTSLAVEBINDMODE_SLAVEID_SLAVEPORT,  // via SlaveID+SlavePort
}EM_VIDEO_TALK_CONTACT_VTSLAVEBINDMODE;

// user type 
typedef enum tagEM_VIDEO_TALK_CONTACT_TYPE
{
  EM_VIDEO_TALK_CONTACT_TYPE_UNKNOWN,       // unknown 
  EM_VIDEO_TALK_CONTACT_TYPE_VTH,         	// "VTH" VTH
  EM_VIDEO_TALK_CONTACT_TYPE_VTO,         	// "VTO" VTO
}EM_VIDEO_TALK_CONTACT_TYPE;

// registration method
typedef enum tagEM_VIDEO_TALK_CONTACT_REGISTER_TYPE
{
  EM_VIDEO_TALK_CONTACT_REGISTER_TYPE_UNKNOWN,        // unknown 
  EM_VIDEO_TALK_CONTACT_REGISTER_TYPE_PUBLIC,         // "public"
  EM_VIDEO_TALK_CONTACT_REGISTER_TYPE_LOCAL,          // "local"
}EM_VIDEO_TALK_CONTACT_REGISTER_TYPE;

// contact record set info 
typedef struct tagNET_RECORD_VIDEO_TALK_CONTACT
{
  DWORD                 dwSize;
  int                   nRecNo;									// record  set no., read-only
  NET_TIME              stuCreateTime;							// create time
  char                  szFirstName[DH_COMMON_STRING_32];		// name
  char                  szFamilyName[DH_COMMON_STRING_32];		// last name
  char                  szVTShortNumber[DH_COMMON_STRING_16];	// visual talk short no.
  char                  szVTMiddleNumber[DH_COMMON_STRING_32];	// visual talk middle no.
  char                  szVTLongNumber[DH_COMMON_STRING_64];	// visual talk long no., or VTH analogue VTH SN
  char                  szVTNetAddress[DH_MAX_IPADDR_LEN_EX];	// visual talk network address
  char                  szVTOPosition[DH_COMMON_STRING_64];		// VTH link to VTO position
  EM_VIDEO_TALK_CONTACT_VTSLAVEBINDMODE  emVTSlaveBindMode;     // visual talk analogue VTH input mode 
  char                  szVTSlaveId[DH_COMMON_STRING_32];		// visual talk analogue  VTH input distributor address
  char                  szVTSlavePort[DH_COMMON_STRING_32];		// visual talk analogue  VTH input distributor port
  char                  szVTSlaveAddress[DH_COMMON_STRING_64];	// visual talk analogue VTH address
  char                  szNickname[DH_COMMON_STRING_64];		// nickname
  char                  szNotes[DH_COMMON_STRING_64];			// note
  EM_VIDEO_TALK_CONTACT_TYPE       emType;						// user type 
  EM_VIDEO_TALK_CONTACT_REGISTER_TYPE   emRegisterType;         // registration method
  char                  szVTHPassword[DH_COMMON_STRING_64];		// registration password
  char                  szVTOBuilding[DH_COMMON_STRING_64];		// building no.
  char                  szVTOUnit[DH_COMMON_STRING_64];			// unit no.
  char                  szGroupNumber[DH_COMMON_STRING_64];		// group
  int                   nVTSlaveId;                             // VTSlave ID
  int                   nVTSlavePort;                           // VTSlave Port
}NET_RECORD_VIDEO_TALK_CONTACT;

//Bulletin status
typedef enum tagNET_ANNOUNCE_STATE
{
	NET_ANNOUNCE_STATE_UNSENDED = 0, //Initial status (to be send)
	NET_ANNOUNCE_STATE_SENDED,		 //Sent
	NET_ANNOUNCE_STATE_EXPIRED,		 //Expired
	NET_ANNOUNCE_STATE_UNKNOWN,		 //Unknown
}NET_ANNOUNCE_STATE;

//The bulletin has been reviewed or not
typedef enum tagNET_ANNOUNCE_READFLAG
{
	NET_ANNOUNCE_READFLAG_UNREADED = 0, //Unread
	NET_ANNOUNCE_READFLAG_READED,	    //Read
	NET_ANNOUNCE_READFLAG_UNKNOWN,      //Unknown
}NET_ANNOUNCE_READFLAG;

// Bulletin record info 
typedef struct tagNET_RECORD_ANNOUNCEMENT_INFO
{
	DWORD									dwSize;									
	int										nRecNo;									// Record set SN. read only 
	NET_TIME                                stuCreateTime;                          // Creation time 
	NET_TIME								stuIssueTime;							// Bulletin released time 
	char									szAnnounceTitle[DH_COMMON_STRING_64];	// Bulletin title
	char									szAnnounceContent[DH_COMMON_STRING_256];// Bulletin contents 
	char									szAnnounceDoor[DH_COMMON_STRING_16];	// The room No. the bulletin to be sent to. 
	NET_TIME								stuExpireTime;							// Bulletin expire time 
	NET_ANNOUNCE_STATE						emAnnounceState;						// Bulletin status
	NET_ANNOUNCE_READFLAG					emAnnounceReadFlag;						// Bulletin has been reviewed or not
}NET_RECORD_ANNOUNCEMENT_INFO;


// Bulletin record info search criteria 
typedef struct tagFIND_RECORD_ANNOUNCEMENT_CONDITION
{
	DWORD                     dwSize;
	BOOL                      bTimeEnable;                      // Enable search period 
	NET_TIME                  stStartTime;                      // Start time
	NET_TIME                  stEndTime;                        // End time
}FIND_RECORD_ANNOUNCEMENT_CONDITION;


//Alarm record info 
typedef struct tagNET_RECORD_ALARMRECORD_INFO
{
	DWORD									dwSize;
	int										nRecNo;									// Record set SN. Read-only 
	NET_TIME                                stuCreateTime;							// Alarm time. UTC second. Read-only 
	int										nChannelID;								// Alarm channel 
	NET_SENSE_METHOD						emSenseMethod;							// Sensor respond type 
	char									szRoomNumber[DH_MAX_DOOR_NUM];			// Alarm room No. 
	NET_ANNOUNCE_READFLAG					emReadFlag;								// 0=unread, 1-=read 
}NET_RECORD_ALARMRECORD_INFO;

//Alarm record search criteria
typedef struct tagFIND_RECORD_ALARMRECORD_CONDITION
{
	DWORD                     dwSize;
	BOOL                      bTimeEnable;                      // Enable search period 
	NET_TIME                  stStartTime;                      // Start time
	NET_TIME                  stEndTime;                        // End time
}FIND_RECORD_ALARMRECORD_CONDITION;


// A&C not close event detailed info
typedef struct tagALARM_ACCESS_CTL_NOT_CLOSE_INFO 
{
    DWORD           dwSize;
    int             nDoor;                                  // Door Channel Number
    char            szDoorName[DH_MAX_DOORNAME_LEN];        // Entrance Guard Name
    NET_TIME        stuTime;                                // Alarm Event Triggered Time
    int             nAction;                                // 0=Start 1=Stop     
    UINT 		    nEventID;				                // event id
}ALARM_ACCESS_CTL_NOT_CLOSE_INFO;

// Break Event Detail Information
typedef struct tagALARM_ACCESS_CTL_BREAK_IN_INFO 
{
    DWORD           dwSize;
    int             nDoor;                                  // Door Channel Number
    char            szDoorName[DH_MAX_DOORNAME_LEN];        // Entrance Guard Name
    NET_TIME        stuTime;                                // Alarm Event Triggered Time    
    UINT 		    nEventID;				                // vent id
}ALARM_ACCESS_CTL_BREAK_IN_INFO;


// Repeatedly Entry  Event Detail Information
typedef struct tagALARM_ACCESS_CTL_REPEAT_ENTER_INFO 
{
    DWORD           dwSize;
    int             nDoor;                                  // Door Channel Number
    char            szDoorName[DH_MAX_DOORNAME_LEN];        // Entrance Guard Name
    NET_TIME        stuTime;                                // Alarm Event Triggered Time
	char            szCardNo[DH_MAX_CARDNO_LEN];            // Card number	
    UINT 			nEventID;				                // event id
}ALARM_ACCESS_CTL_REPEAT_ENTER_INFO;


// Forced Card Swing Card  Event Detail Information
typedef struct tagALARM_ACCESS_CTL_DURESS_INFO 
{
    DWORD           dwSize;
    int             nDoor;                                  // Door Channel Number
    char            szDoorName[DH_MAX_DOORNAME_LEN];        // Entrance Guard Name
    char            szCardNo[DH_MAX_CARDNO_LEN];            // Forced Card Number
    NET_TIME        stuTime;                                // Alarm Event Triggered Time    
    UINT 			nEventID;				                // event id
}ALARM_ACCESS_CTL_DURESS_INFO;


// Entrance Guard Event Type
typedef enum tagNET_ACCESS_CTL_EVENT_TYPE
{
    NET_ACCESS_CTL_EVENT_UNKNOWN = 0,
    NET_ACCESS_CTL_EVENT_ENTRY,                             // Get In
    NET_ACCESS_CTL_EVENT_EXIT,                              // Get Out
}NET_ACCESS_CTL_EVENT_TYPE;

// Entrance Guard Event
typedef struct tagALARM_ACCESS_CTL_EVENT_INFO 
{
    DWORD                       dwSize;
    int                         nDoor;                              // Door Channel Number
    char                        szDoorName[DH_MAX_DOORNAME_LEN];    // Entrance Guard Name
    NET_TIME                    stuTime;                            // Alarm Event Triggered Time
    NET_ACCESS_CTL_EVENT_TYPE   emEventType;                        // Entrance Guard Event Type
    BOOL                        bStatus;                            // Swing Card Result,True is Success,False is Fail
    NET_ACCESSCTLCARD_TYPE      emCardType;                         // Card Type
    NET_ACCESS_DOOROPEN_METHOD  emOpenMethod;                       // Open The Door Method
    char                        szCardNo[DH_MAX_CARDNO_LEN];        // Card Number
    char                        szPwd[DH_MAX_CARDPWD_LEN];          // Password
    char                        szReaderID[DH_COMMON_STRING_32];    // Reader ID
    char                        szUserID[DH_COMMON_STRING_64];      // unlock user
    char                        szSnapURL[DH_COMMON_STRING_256];    // snapshot picture storage address
    int                         nErrorCode;                         // Reason of unlock failure, only because it is valid when bStatus is FALSE
                                                                    // 0x00 no error
                                                                    // 0x10 unauthorized
                                                                    // 0x11 card lost or cancelled
                                                                    // 0x12 no door right
                                                                    // 0x13 unlock mode error
                                                                    // 0x14 valid period error
                                                                    // 0x15 anti sneak into mode
                                                                    // 0x16 forced alarm not unlocked
                                                                    // 0x17 door NC status
                                                                    // 0x18 AB lock status
                                                                    // 0x19 patrol card
                                                                    // 0x1A device is under intrusion alarm status
                                                                    // 0x20 period error
                                                                    // 0x21 unlock period error in holiday period
                                                                    // 0x30 first card right check required
                                                                    // 0x40 card correct, input password error
                                                                    // 0x41 card correct, input password timed out
                                                                    // 0x42 card correct, input fingerprint error
                                                                    // 0x43 card correct, input fingerprint timed out
                                                                    // 0x44 fingerprint correct, input password error
                                                                    // 0x45 fingerprint correct, input password timed out
                                                                    // 0x50 group unlock sequence error
                                                                    // 0x51 test required for group unlock
                                                                    // 0x60 test passed, control unauthorized
    int                         nPunchingRecNo;                     // punching record number 
	int							nNumbers;							// pic Numbers	
    NET_ACCESSCTLCARD_STATE     emStatus;		                    // Card Status
}ALARM_ACCESS_CTL_EVENT_INFO;

typedef enum tagEM_NET_RECORD_COMMODITY_NOTICE_SOURCE_TYPE
{
    EM_NET_RECORD_COMMODITY_NOTICE_SOURCE_TYPE_UNKNOWN,				// Unknown 
    EM_NET_RECORD_COMMODITY_NOTICE_SOURCE_TYPE_SUPERMARKET,			// Super market
    EM_NET_RECORD_COMMODITY_NOTICE_SOURCE_TYPE_RESTAURANT,			// Dining hall
    EM_NET_RECORD_COMMODITY_NOTICE_SOURCE_TYPE_FRUITSTORE,     		// Fruit store
}EM_NET_RECORD_COMMODITY_NOTICE_SOURCE_TYPE;

typedef enum tagEM_NET_RECORD_COMMODITY_NOTICE_BARGAIN_TYPE
{
    EM_NET_RECORD_COMMODITY_NOTICE_BARGAIN_TYPE_UNKNOWN,        // Unknown
    EM_NET_RECORD_COMMODITY_NOTICE_BARGAIN_TYPE_YES,            // Bargain
    EM_NET_RECORD_COMMODITY_NOTICE_BARGAIN_TYPE_NO,             // No Bargain
}EM_NET_RECORD_COMMODITY_NOTICE_BARGAIN_TYPE;
// commodity notice record set info
typedef struct tagNET_RECORD_COMMODITY_NOTICE
{
    DWORD                                           dwSize;
    int                                             nRecNo;                                 // record number 
    NET_TIME                                        stuSaleTime;                            // Sale Time
    char                                            szName[DH_COMMON_STRING_128];           // commodity name
    EM_NET_RECORD_COMMODITY_NOTICE_SOURCE_TYPE      emSource;                               // commodity notice source type
    char                                            szURL[DH_COMMON_STRING_256];            // commodity pictrue path
    float                                           fPrice;                                 // commodity price, unit: yuan
    char                                            szID[DH_COMMON_STRING_64];              // ID number
    EM_NET_RECORD_COMMODITY_NOTICE_BARGAIN_TYPE     emBargain;                              // Bargain
}NET_RECORD_COMMODITY_NOTICE;

typedef enum tagEM_NET_RECORD_HEALTH_CARE_NOTICE_OFFICE_TYPE
{
    EM_NET_RECORD_HEALTH_CARE_NOTICE_OFFICE_TYPE_UNKNOWN,           // Unknown
    EM_NET_RECORD_HEALTH_CARE_NOTICE_OFFICE_TYPE_DIGEST,            // Digest 
    EM_NET_RECORD_HEALTH_CARE_NOTICE_OFFICE_TYPE_STOMATOLOGY,       // Stomatology
}EM_NET_RECORD_HEALTH_CARE_NOTICE_OFFICE_TYPE;
// health care notice record set info
typedef struct tagNET_RECORD_HEALTH_CARE_NOTICE
{
    DWORD                                           dwSize;
    int                                             nRecNo;                                 // record number
    NET_TIME                                        stuStartTime;                           // start time
    NET_TIME                                        stuEndTime;                             // end time
    char                                            szHospital[DH_COMMON_STRING_128];       // Hospital
    EM_NET_RECORD_HEALTH_CARE_NOTICE_OFFICE_TYPE    emOffice;                               // Office
    char                                            szOfficeNumber[DH_COMMON_STRING_64];    // OfficeNumber
    char                                            szDoctorName[DH_COMMON_STRING_128];     // Doctor Name
    char                                            szID[DH_COMMON_STRING_64];              // ID 
}NET_RECORD_HEALTH_CARE_NOTICE;
///////////////////////// Alarm of intelligent transportation[CLIENT_StartListenEx] /////////////////////////////////////

//type DH_IVS_TRAFFIC_REALFLOWINFO(traffic real flow info)
typedef struct tagALARM_IVS_TRAFFIC_REALFLOW_INFO
{
	int                 nChannelID;                       // channel ID
	char                szName[128];                      // name
	char                bReserved1[4];                    // reserved
	double              PTS;                              // PTS(ms)
	NET_TIME_EX         UTC;                              // time of occurrence
	int                 nEventID;                         // event ID
	unsigned int        nSequence;                        // sequence
	BYTE                bEventAction;                     // event action,0 pulse,1 durable events begin, 2 durable events end
	BYTE                byReserved[3];
	int                 nLane;                            // lane number
	int                 nCount;                           // cars count
	int					nSpeed;							  // speed,km/h
	int                 nOverPercent;                     // over percent
	int                 nMetersUints;                     // meter unit 0:km,1:mile
	BYTE                bReserved[1024];                  // reserved
}ALARM_IVS_TRAFFIC_REALFLOW_INFO;

//EVNET type DH_YUEQINGLIGHTING_STATE_EVENT(Statistical light switch time) 
typedef struct tagDEV_YUEQINGLIGHTING_STATE_INFO
{
	DWORD               dwAction;                       // event ation, 0:Start, 1:Stop
	int                 nChannelID;                     // Channel ID (start from 0)
	NET_TIME_EX         UTC;							// the event happen time 
	BYTE                bReserved[1024];                // reserved
}DEV_YUEQINGLIGHTING_STATE_INFO;

// Alarm Event Type,DH_ALARM_VEHICLE_TURNOVER's Data Describe Information
typedef struct tagALARM_VEHICEL_TURNOVER_EVENT_INFO
{
    NET_GPS_STATUS_INFO stGPSStatusInfo;                // GPS Information
    BYTE                bEventAction;                   // Event Action,0=Impluse Event,1=Continued Event Start,2=Continued Event End;
    BYTE                bReserved[1023];                // Hold Byte,For Extension.
}ALARM_VEHICEL_TURNOVER_EVENT_INFO;

// Alarm Event Type,DH_ALARM_VEHICLE_COLLISION'sData Describe Information
typedef struct tagALARM_VEHICEL_COLLISION_EVENT_INFO
{
    NET_GPS_STATUS_INFO stGPSStatusInfo;                // GPS Information
    BYTE                bEventAction;                   // Event Action,0=Impluse Event,1=Continued Event Start,2=Continued Event End;
    BYTE                bReserved[1023];                // Hold Byte,For Extension.
}ALARM_VEHICEL_COLLISION_EVENT_INFO;

// Alarm Event Type,DH_ALARM_VEHICLE_CONFIRM'sData Describe Information
typedef struct tagALARM_VEHICEL_CONFIRM_INFO
{
    DWORD               dwStructSize;                   // StructSize
    NET_GPS_STATUS_INFO stGPSStatusInfo;                // GPS Information
    BYTE                bEventAction;                   // Event Action,0=Impluse Event,1=Continued Event Start,2=Continued Event End;
    BYTE                byRserved[3];                   // Hold Byte,For Extension.
    char                szInfo[128];                    // Upload Alarm Concrete Information
}ALARM_VEHICEL_CONFIRM_INFO;

//Alarm Event Type,DH_EVENT_REGISTER_OFF's Data Describe Information
typedef struct tagEVENT_REGISTER_OFF_INFO
{
    DWORD               dwStructSize;                   // StructSize
    NET_GPS_STATUS_INFO stGPSStatusInfo;                // GPS Information
    BYTE                bEventAction;                   // Event Action,0=Impluse Event,1=Continued Event Start,2=Continued Event End;
    BYTE                byRserved[3];                   // Aline
}EVENT_REGISTER_OFF_INFO;

// Alarm Event Type,DH_ALARM_VIDEO_TIMING's Data Describe Information
typedef struct tagALARM_VIDEO_TIMING
{
    DWORD               dwStructSize;                   // StructSize
    DWORD               dwAction;                       // Event Action,0=Pause,1=Start,2=Stop
    DWORD               dwChannelID;                    // Video Channel ID
}ALARM_VIDEO_TIMING;

// Alarm Event Type,DH_ALARM_VEHICLE_LARGE_ANGLE's Data Describe Information
typedef struct tagALARM_VEHICEL_LARGE_ANGLE
{
    DWORD               dwStructSize;                   // StructSize
    NET_GPS_STATUS_INFO stGPSStatusInfo;                // GPS Information
    BYTE                bEventAction;                   // Event Action,0=Impluse Event,1=Continued Event Start,2=Continued Event End;
    BYTE                byRserved[3];                   // Aline
    unsigned int        nSequence;                      // no.
}ALARM_VEHICEL_LARGE_ANGLE;

// Alarm Event Type,DH_ALARM_AUDIO_ANOMALY's Data Describe Information
typedef struct tagALARM_AUDIO_ANOMALY
{
    DWORD               dwStructSize;                   // StructSize
    DWORD               dwAction;                       // Event Action,0=Pause,1=Start,2=Stop
    DWORD               dwChannelID;                    // Audio Channel ID
    int                 nDecibel;                       // Audio sensitivity
    int                 nFrequency;                     // Audio frequency 
}ALARM_AUDIO_ANOMALY;

// Alarm Event Type,DH_ALARM_AUDIO_MUTATION's Data Describe Information
typedef struct tagALARM_AUDIO_MUTATION
{
    DWORD               dwStructSize;                   // StructSize
    DWORD               dwAction;                       // Event Action,0=Pause,1=Start,2=Stop
    DWORD               dwChannelID;                    // Audio Channel ID
}ALARM_AUDIO_MUTATION;

// Corresponding data description info of alarm event type (alarm event type) DH_ALARM_AUDIO_DETECT (audio detect event)
typedef struct tagALARM_AUDIO_DETECT
{
    DWORD               dwAction;                       // Event operation, 1:Start, 2:Stop
    DWORD               dwChannelID;                    // Audio channel No.
    char                reserved[512];                  // Reserved 
} ALARM_AUDIO_DETECT;

#define MAX_SENSORID_NUMBER 20                          // Sensor max amount 
#define MAX_TYRE_NUM        128                         // Max tyre amount 

// Tyre alarm mark enumeration 
typedef enum tagAlarmTyreFlag
{
    ALARM_TYRE_FLAG_NONE,                               // None Valid Data
    ALARM_TYRE_FLAG_NORMAL,                             // Normal
    ALARM_TYRE_FLAG_HIGHPRESSURE,                       // High Pressure
    ALARM_TYRE_FLAG_LOWERPRESSURE,                      // Low Pressure
    ALARM_TYRE_FLAG_HIGHTEMP,                           // High Temperature
    ALARM_TYRE_FLAG_HIGHTEMP_HIGHPRESSURE,              // High Pressure, High Temperature
    ALARM_TYRE_FLAG_HIGHTEMP_LOWERPRESSURE,             // High Temperature, Low Pressure

    ALARM_TYRE_FLAG_INVALID,                            // Illegal Flag Data
}EM_ALARM_TYER_FALG;

// Tyre Information Struct
typedef struct tagTYRE_INFO_UNIT
{
    DWORD               dwStructSize;                    // StructSize
    DWORD               dwSensorIDNum;                   // Sensor Number
    DWORD               dwSensorID[MAX_SENSORID_NUMBER]; // Sensor ID Information
    DWORD               dwTyreNum;                       // Tyre Number
    DWORD               dwTyrePlace;                     // Tyre at the First Few Shaft

    EM_ALARM_TYER_FALG  emAlarmFlag;                     // Tyre Alarm Flag

    int                 nTemp;                           // Temperature,Unit: degree Celsius
    int                 nTempLimit;                      // Temperature,Unit: degree Celsius

    float               fPressure;                       // Pressure,Unit:Kpa
    float               fUpperLimit;                     // Alarm Threshold Pressure Upper Limit
    float               fLowerLimit;                     // Alarm Threshold Pressure Lower Limit
    float               fVoltage;                        // Sensor Current Voltage,Unit:V
}TYRE_INFO_UNIT;

// Alarm Event Type,DH_EVENT_TYREINFO's Data Describe Information
typedef struct tagEVENT_TYRE_INFO
{
    DWORD               dwStructSize;                   // StructSize
    DWORD               dwAction;                       // Event Action,0=Pause,1=Start,2=Stop
    DWORD               dwTyreNum;                      // Tyre Number
    TYRE_INFO_UNIT      *pstuTyre;                      // Tyre Information
}EVENT_TYRE_INFO;

/////////////////////////////Audio Talk Related Definition/////////////////////////////

// Audio format information 
typedef struct
{
	BYTE				byFormatTag;			// Encode type such as 0:PCM
	WORD				nChannels;				// Track number 
	WORD				wBitsPerSample;			// Sampling depth 	
	DWORD				nSamplesPerSec;			// Sampling rate
} DH_AUDIO_FORMAT, *LPDH_AUDIO_FORMAT;

/////////////////////////////User Management Corresponding Definition /////////////////////////////

// Support the device with max 8 bits user name. Corresponding to the CLIENT_QueryUserInfo and CLIENT_OperateUserInfo.
// Right information 
typedef struct _OPR_RIGHT
{
	DWORD				dwID;
	char				name[DH_RIGHT_NAME_LENGTH];
	char				memo[DH_MEMO_LENGTH];
} OPR_RIGHT;

// User information 
typedef struct _USER_INFO
{
	DWORD				dwID;
	DWORD				dwGroupID;
	char				name[DH_USER_NAME_LENGTH];
	char				passWord[DH_USER_PSW_LENGTH];
	DWORD				dwRightNum;
	DWORD				rights[DH_MAX_RIGHT_NUM];
	char				memo[DH_MEMO_LENGTH];
	DWORD				dwReusable;				// Reuse or not;1:enable reuse;0:disable reuse 
} USER_INFO;

// User group information 
typedef struct _USER_GROUP_INFO
{
	DWORD				dwID;
	char				name[DH_USER_NAME_LENGTH];
	DWORD				dwRightNum;
	DWORD				rights[DH_MAX_RIGHT_NUM];
	char				memo[DH_MEMO_LENGTH];
} USER_GROUP_INFO;

// User information list 
typedef struct _USER_MANAGE_INFO
{
	DWORD				dwRightNum;				// Right information 
	OPR_RIGHT			rightList[DH_MAX_RIGHT_NUM];
	DWORD				dwGroupNum;				// User group information 
	USER_GROUP_INFO		groupList[DH_MAX_GROUP_NUM];
	DWORD				dwUserNum;				// User information 
	USER_INFO			userList[DH_MAX_USER_NUM];
	DWORD				dwSpecial;				// User account reuse; 1:support account to be reused. 0:Does not support account to be reused.
} USER_MANAGE_INFO;

// Support the device of max 8-bit or 16-bit name. Corresponding interface to CLIENT_QueryUserInfoEx and CLIENT_OperateUserInfoEx
#define DH_USER_NAME_LENGTH_EX		16			// User name length 
#define DH_USER_PSW_LENGTH_EX		16			// Password 

// Right information
typedef struct _OPR_RIGHT_EX
{
	DWORD				dwID;
	char				name[DH_RIGHT_NAME_LENGTH];
	char				memo[DH_MEMO_LENGTH];
} OPR_RIGHT_EX;

// User information 
typedef struct _USER_INFO_EX
{
	DWORD				dwID;
	DWORD				dwGroupID;
	char				name[DH_USER_NAME_LENGTH_EX];
	char				passWord[DH_USER_PSW_LENGTH_EX];
	DWORD				dwRightNum;
	DWORD				rights[DH_MAX_RIGHT_NUM];
	char				memo[DH_MEMO_LENGTH];
	DWORD				dwFouctionMask;			// Subnet mask,0x00000001 - support reuse  
	BYTE				byReserve[32];
} USER_INFO_EX;

// User group information 
typedef struct _USER_GROUP_INFO_EX
{
	DWORD				dwID;
	char				name[DH_USER_NAME_LENGTH_EX];
	DWORD				dwRightNum;
	DWORD				rights[DH_MAX_RIGHT_NUM];
	char				memo[DH_MEMO_LENGTH];
} USER_GROUP_INFO_EX;

// User information sheet 
typedef struct _USER_MANAGE_INFO_EX
{
	DWORD				dwRightNum;				// Right information 
	OPR_RIGHT_EX		rightList[DH_MAX_RIGHT_NUM];
	DWORD				dwGroupNum;				// User group information 
	USER_GROUP_INFO_EX  groupList[DH_MAX_GROUP_NUM];
	DWORD				dwUserNum;				// User information 
	USER_INFO_EX		userList[DH_MAX_USER_NUM];
	DWORD				dwFouctionMask;			// Subnet mask;0x00000001 - support reuse, 0x00000002 - Password has been modified , it needs to be verified.
	BYTE				byNameMaxLength;		// The supported user name max length 
	BYTE				byPSWMaxLength;			// The supported password max length
	BYTE				byReserve[254];
} USER_MANAGE_INFO_EX;

// Max support a device of 64-channel. Corresponding extension interface CLIENT_QueryUserInfoNew and CLIENT_OperateUserInfoNew
#define DH_NEW_MAX_RIGHT_NUM        1024
#define DH_NEW_USER_NAME_LENGTH	 128			// User name length
#define DH_NEW_USER_PSW_LENGTH	 128			// password

// Rights info
typedef struct _OPR_RIGHT_NEW
{
	DWORD               dwSize;
	DWORD				dwID;
	char				name[DH_RIGHT_NAME_LENGTH];
	char				memo[DH_MEMO_LENGTH];
} OPR_RIGHT_NEW;

// User info
typedef struct _USER_INFO_NEW
{
	DWORD               dwSize;
	DWORD				dwID;
	DWORD				dwGroupID;
	char				name[DH_NEW_USER_NAME_LENGTH];
	char				passWord[DH_NEW_USER_PSW_LENGTH];
	DWORD				dwRightNum;
	DWORD				rights[DH_NEW_MAX_RIGHT_NUM];
	char				memo[DH_MEMO_LENGTH];
	DWORD				dwFouctionMask;			// Sub mask,0x00000001 - Support account reusable
    NET_TIME            stuTime;                // Last Revise Time
    BYTE                byIsAnonymous;          // Whether Can Be Anonymous Login,0=Can't Be Anonymous Login,1=Can be Anonymous Login
    BYTE                byReserve[7];
} USER_INFO_NEW;

// User group info
typedef struct _USER_GROUP_INFO_NEW
{
	DWORD               dwSize;
	DWORD				dwID;
	char				name[DH_USER_NAME_LENGTH_EX];
	DWORD				dwRightNum;
	DWORD				rights[DH_NEW_MAX_RIGHT_NUM];
	char				memo[DH_MEMO_LENGTH];
} USER_GROUP_INFO_NEW;

// user group information expand,user group lengthen
typedef struct _USER_GROUP_INFO_EX2
{
    DWORD               dwSize;
    DWORD               dwID;
    char                name[DH_NEW_USER_NAME_LENGTH];
    DWORD               dwRightNum;
    DWORD               rights[DH_NEW_MAX_RIGHT_NUM];
    char                memo[DH_MEMO_LENGTH];
} USER_GROUP_INFO_EX2;

// User info list
typedef struct _USER_MANAGE_INFO_NEW
{
	DWORD               dwSize;
	DWORD				dwRightNum;				// Rights info
	OPR_RIGHT_NEW		rightList[DH_NEW_MAX_RIGHT_NUM];
	DWORD				dwGroupNum;				// User group info
	USER_GROUP_INFO_NEW groupList[DH_MAX_GROUP_NUM];
	DWORD				dwUserNum;				// User info
	USER_INFO_NEW		userList[DH_MAX_USER_NUM];
	DWORD				dwFouctionMask;			// Sub mask; 0x00000001 - Support account reusable,0x00000002 - Verification needed when change password
	BYTE				byNameMaxLength;		// Max user name length supported
	BYTE				byPSWMaxLength;			// Max password length supported
	BYTE				byReserve[254];
    USER_GROUP_INFO_EX2 groupListEx[DH_MAX_GROUP_NUM];      // User Group Information Expand
} USER_MANAGE_INFO_NEW;

///////////////////////////////Search Related Definition ///////////////////////////////

// The language types device supported 
typedef struct _DH_LANGUAGE_DEVINFO
{
	DWORD				dwLanguageNum;			// The language amount supported
	BYTE				byLanguageType[252];	// Enumeration value please refer to DH_LANGUAGE_TYPE
} DH_DEV_LANGUAGE_INFO, *LPDH_DEV_LANGUAGE_INFO;

// the IPC types device supported
typedef struct
{
	int                 nTypeCount;              // The IPC type amount supported
	BYTE                bSupportTypes[128];      // Enumeration value please refer to EM_IPC_TYPE
}DH_DEV_IPC_INFO;

//3G net flow info
typedef struct
{
	int					nStrategy;              // strategy,0: charged by flow every month 1:charged by time every month
	int                 nUplimit;               // up limit flow, by flow: MB, by time: h
	int                 nUsed;                  // have used flow, by flow: MB, by time: h
	BYTE                bReserved[64];          // reserved
}DH_DEV_3GFLOW_INFO;

// 3G module info
typedef struct
{
	BYTE			    byEthNum;				// 3G model number
	BYTE				byReserved[255];        // reserved
}DH_DEV_3GMODULE_INFO;

typedef struct
{
	DWORD				dwId;                            // ddns server id
	char				szState[DH_MAX_DDNS_STATE_LEN];	 // ddns status
	BYTE				bReserved[512];                  // reserved
}DH_DEV_DNS_INFO;
typedef struct
{
	DWORD				dwDdnsServerNum;	
	DH_DEV_DNS_INFO     stDdns[DH_MAX_DDNS_NUM];
}DH_DEV_MULTI_DDNS_INFO;

// device URL info
typedef struct
{
	char				szURLInfo[512];         // device URL info, by string
	BYTE				bReserved[512];
}DH_DEV_URL_INFO;
// HDD informaiton 
typedef struct
{
	DWORD				dwVolume;				// HDD capacity 
	DWORD				dwFreeSpace;			// HDD free space 
	BYTE				dwStatus;				// higher 4 byte instruct hdd type, see the enum struct EM_DISK_TYPE; lower four byte instruct HDD status,0-hiberation,1-active,2-malfucntion and etc.;Devide DWORD into four BYTE
	BYTE				bDiskNum;				// HDD number
	BYTE				bSubareaNum;			// Subarea number
	BYTE				bSignal;				// Symbol. 0:local. 1:remote
} NET_DEV_DISKSTATE,*LPNET_DEV_DISKSTATE;

// Device HDD informaiton 
typedef struct _DH_HARDDISK_STATE
{
	DWORD				dwDiskNum;				// Amount 
	NET_DEV_DISKSTATE	stDisks[DH_MAX_DISKNUM];// HDD or subarea information 
} DH_HARDDISK_STATE, *LPDH_HARDDISK_STATE;

typedef DH_HARDDISK_STATE	DH_SDCARD_STATE;	// SD card. Please refer to HDD information for data structure. 

// Audio encode information 
typedef struct  
{
	DH_TALK_CODING_TYPE	encodeType;				// Encode type 
	int					nAudioBit;				// Bit:8/16
	DWORD				dwSampleRate;			// Sampling rate such as 8000 or 16000
    int                 nPacketPeriod;          // Pack Period,Unit ms
    char                reserved[60];
} DHDEV_TALKDECODE_INFO;

// The audio talk type the device supported
typedef struct 
{
	int					nSupportNum;			// Amount
	DHDEV_TALKDECODE_INFO type[64];				// Encode type 
	char				reserved[64];
} DHDEV_TALKFORMAT_LIST;

// PTZ property information
#define  NAME_MAX_LEN 16
typedef struct 
{
	DWORD				dwHighMask;				// Operation mask high bit 
	DWORD				dwLowMask;				// Operation mask low bit 
	char				szName[NAME_MAX_LEN];	// Operation protocol name 
	WORD				wCamAddrMin;			// Channel address min value
	WORD				wCamAddrMax;			// Channel address max value
	WORD				wMonAddrMin;			// Monitor address min value
	WORD				wMonAddrMax;			// Monitor address max value
	BYTE				bPresetMin;				// Preset min value
	BYTE				bPresetMax;				// Preset max value
	BYTE				bTourMin;				// Auto tour min value
	BYTE				bTourMax;				// Auto tour max value
	BYTE				bPatternMin;			// Pattern min value
	BYTE				bPatternMax;			// Pattern max value
	BYTE				bTileSpeedMin;			// Tilt speed min value
	BYTE				bTileSpeedMax;			// Tilt speed max value
	BYTE				bPanSpeedMin;			// Pan speed min value
	BYTE				bPanSpeedMax;			// Pan speed max value
	BYTE				bAuxMin;				// Aux function min value
	BYTE				bAuxMax;				// Aux function max value
	int					nInternal;				// Command interval
	char				cType;					// Protocol type
	BYTE				bReserved_1;			// Reserved
	BYTE				bFuncMask;				// function mask :0x01 - support PTZ-inside function
	BYTE				bReserved_2;
	char				Reserved[4];
} PTZ_OPT_ATTR;

// CD driver tray state
typedef enum tagEM_NET_BURN_DEV_TRAY_TYPE
{
    EM_NET_BURN_DEV_TRAY_TYPE_UNKNOWN,    // unknown
	EM_NET_BURN_DEV_TRAY_TYPE_READY,      // ready
	EM_NET_BURN_DEV_TRAY_TYPE_OPEN,       // open
	EM_NET_BURN_DEV_TRAY_TYPE_NODISK,     // no disk
	EM_NET_BURN_DEV_TRAY_TYPE_NOT_READY,  // not ready
}EM_NET_BURN_DEV_TRAY_TYPE;

// CD driver using state
typedef enum tagEM_NET_BURN_DEV_OPERATE_TYPE
{
    EM_NET_BURN_DEV_OPERATE_TYPE_UNKNOWN, // unknown
	EM_NET_BURN_DEV_OPERATE_TYPE_WRITE,   // write
	EM_NET_BURN_DEV_OPERATE_TYPE_READ,    // read
	EM_NET_BURN_DEV_OPERATE_TYPE_IDLE,    // idle
}EM_NET_BURN_DEV_OPERATE_TYPE;

// Burner informaiton 
typedef struct _NET_DEV_BURNING
{
	DWORD				dwDriverType;			// Burner driver type;0:DHFS,1:DISK,2:CDRW
	DWORD				dwBusType;				// Bus type;0:USB,1:1394,2:IDE
	DWORD				dwTotalSpace;			// Total space(KB)
	DWORD				dwRemainSpace;			// Free space(KB)
	BYTE				dwDriverName[DH_BURNING_DEV_NAMELEN];	 // Burner driver name
	EM_NET_BURN_DEV_TRAY_TYPE    emTrayType;					 // CD driver Tray state
	EM_NET_BURN_DEV_OPERATE_TYPE emOperateType;					 // CD driver using state
} NET_DEV_BURNING, *LPNET_DEV_BURNING;

// Device burner informaiton 
typedef struct _DH_BURNING_DEVINFO
{
	DWORD				dwDevNum;						// Burner device amount
	NET_DEV_BURNING		stDevs[DH_MAX_BURNING_DEV_NUM];	// Each burner device information 
} DH_BURNING_DEVINFO, *LPDH_BURNING_DEVINFO;

// Burner progress 
typedef struct _DH_BURNING_PROGRESS
{
	BYTE				bBurning;				// Burner status;0:ready to burn,1:burner types is not correct. It does not match. 
												// 2:there is no burner available(remainSpace is 0),3:There is burning in process,4:Burner is not free(It is backup or buring or playback.) 
	BYTE				bRomType;				// CD type;0:private file system,1:Portable HDD or flash disk,2:CD
	BYTE				bOperateType;			// Operation type;0:free,1:backup,2:burning,3:playback from the cd 
	BYTE				bType;					// Backup pr burning status:0:stop or end,1:start,2:error,3:full,4:initializing
	NET_TIME			stTimeStart;			// Begin time 
	DWORD				dwTimeElapse;			// Burning time(second)
	DWORD				dwTotalSpace;			// Total space
	DWORD				dwRemainSpace;			// Free space
	DWORD				dwBurned;				// Burned capacity 
	WORD				dwStatus;				// Reserved
	WORD				wChannelMask;			// The burning channel mask 
} DH_BURNING_PROGRESS, *LPDH_BURNING_PROGRESS;

// Log information. Corresponding to CLIENT_QueryLog
typedef struct _DH_LOG_ITEM
{
    DHDEVTIME			time;					// Date 
    unsigned short		type;					// Log Type, corresponding to DH_LOG_TYPE
    unsigned char		reserved;				// Reserved
    unsigned char		data;					// Data 
    unsigned char		context[8];				// Content
} DH_LOG_ITEM, *LPDH_LOG_ITEM;

// Log information. Corresponding to CLIENT_QueryLogEx, parameter reserved(int nType=1;reserved=&nType;)
typedef struct _DH_NEWLOG_ITEM
{
	DHDEVTIME			time;					// Date
	WORD				type;					// New Log Type, corresponding to DH_NEWLOG_TYPE
	WORD				data;					// data
	char				szOperator[8]; 			// User name 
	BYTE				context[16];		    // Content	
} DH_NEWLOG_ITEM, *LPDH_NEWLOG_ITEM;

// Log information. Corresponding to CLIENT_QueryDeviceLog
typedef struct _DH_DEVICE_LOG_ITEM
{
	int					nLogType;				// Log type 
	DHDEVTIME			stuOperateTime;			// Date
	char				szOperator[16]; 		// Operator
	BYTE				bReserved[3];
	BYTE				bUnionType;				// union structure type,0:szLogContext;1:stuOldLog
	union
	{
		char			szLogContext[64];		// Log content
		struct 
		{
			DH_LOG_ITEM		stuLog;				// Old log structure 
			BYTE			bReserved[48];		// Reserved
		}stuOldLog;
	};
	char				reserved[16];
} DH_DEVICE_LOG_ITEM, *LPDH_DEVICE_LOG_ITEM;

// new Log information. Corresponding to CLIENT_QueryDeviceLog
typedef struct _DH_DEVICE_LOG_ITEM_EX
{
	int					nLogType;				// Log type 
	DHDEVTIME			stuOperateTime;			// Date
	char				szOperator[16]; 		// Operator
	BYTE				bReserved[3];
	BYTE				bUnionType;				// union structure type,0:szLogContext;1:stuOldLog
	union
	{
		char			szLogContext[64];		// Log content
		struct 
		{
			DH_LOG_ITEM		stuLog;				// Old log structure 
			BYTE			bReserved[48];		// Reserved
		}stuOldLog;
	};
	char                szOperation[32];        // Detail operation
	char				szDetailContext[4 * 1024];  // DetailContext
} DH_DEVICE_LOG_ITEM_EX, *LPDH_DEVICE_LOG_ITEM_EX;

// Record log informaiton. Corresponding to the context of log structure 
typedef struct _LOG_ITEM_RECORD
{
	DHDEVTIME			time;					// Time 
	BYTE				channel;				// Channel
	BYTE				type;					// Record type
	BYTE				reserved[2];
} LOG_ITEM_RECORD, *LPLOG_ITEM_RECORD;

typedef struct _QUERY_DEVICE_LOG_PARAM
{
	 DH_LOG_QUERY_TYPE	emLogType;				// Searched log type
	 NET_TIME			stuStartTime;			// The searched log start time
	 NET_TIME			stuEndTime;				// The searched log end time. 
	 int				nStartNum;				// The search begins from which log in one period. It shall begin with 0 if it is the first time search.
	 int				nEndNum;				// The ended log serial number in one search,the max returning number is 1024 
	 BYTE                nLogStuType;           // log struct type,0:DH_DEVICE_LOG_ITEM;1:DH_DEVICE_LOG_ITEM_EX
	BYTE                reserved[3];            // Reserved
	unsigned int        nChannelID;             // Channel no. 0:Compatible with previous all channel numbers. The channel No. begins with 1.1: The first channel.
	BYTE				bReserved[40];
} QUERY_DEVICE_LOG_PARAM;

// record information in the hard disk
typedef struct __DEV_DISK_RECORD_INFO 
{
	NET_TIME			stuBeginTime;			// The first time video
	NET_TIME			stuEndTime;				// Recently video
	char				reserved[128];
} DEV_DISK_RECORD_INFO;

// disk record time
typedef struct __DEV_DISK_RECORD_TIME
{
	NET_TIME             stuStartTime1;		    // start time 1
	NET_TIME	         stuEndTime1;			// end time 1
	BOOL	             bTwoPart;			    // have two part or not
	NET_TIME	         stuStartTime2;		    // start time 2
	NET_TIME	         stuEndTime2;			// end time 2
	BYTE			  	 bDiskNum;				// disk number 
	BYTE				 bSubareaNum;			// subarea num
	BYTE                 byReserved[62];        // reserved

}DEV_DISK_RECORD_TIME;

///////////////////////////////Control Related Definition///////////////////////////////
//new network keyboard control
typedef struct _NKB_PARAM
{
    BYTE                bAddressCode;           // for DVR and etc. device physical address no.,0xFF as broadcast address
    BYTE                bKeyStatus;             // status code 0/1 0 means press by key 1 means release key
    BYTE                bExtern1;               // expansion field 1
    BYTE                bExtern2;               // expansion field 2
    BYTE                bReserved[60];
}NKB_PARAM;

// HDD operation
typedef struct _DISKCTRL_PARAM
{
	DWORD				dwSize;					// Structure size. For version control.
	int					nIndex;					// Subscript of array stDisks of information structure DH_HARDDISK_STATE. It begins from 0. 
	int					ctrlType;				// Operation type
												// 0 -  clear data, 1 - set as read-write HDD , 2 -  set as read-only HDD
												// 3 - set as redundant , 4 - restore error , 5 -set as snapshot disk  
	NET_DEV_DISKSTATE	stuDisk;				// diskInfo, replace nIndex
} DISKCTRL_PARAM;

typedef struct  
{
	BYTE				bSubareaNum;			// The number of pre-partition
	BYTE				bIndex;					// Subscript of array stDisks of information structure DH_HARDDISK_STATE. It begins from 0. 
	BYTE				bSubareaSize[32];		// partition Size(Percentage)
	BYTE				bReserved[30];			// Reservations
} DISKCTRL_SUBAREA;

// Alarm status
typedef struct _ALARMCTRL_PARAM
{
	DWORD				dwSize;
	int					nAlarmNo;				// Alarm channel. It begins from 0. 
	int					nAction;				// 1:activate alarm,0:stop alarm  
} ALARMCTRL_PARAM;

// Matrix control 
typedef struct _MATRIXCTRL_PARAM
{
	DWORD				dwSize;
	int					nChannelNo;				// Video input number. It begins from  0.
	int					nMatrixNo;				// Matrix output number. It begins from 0
} MATRIXCTRL_PARAM;

// Burner control 
typedef struct _BURNING_PARM
{
	int					channelMask;			// channel subnet mask. Bit means the channel to be burned.
	BYTE				devMask;				// Burner subnet mask.  Use bit to represent according to the searched burner list 
	BYTE                bySpicalChannel;        // PIP channel(Channel No.+32)
	BYTE                byReserved[2];          // Reserved
} BURNNG_PARM;

// Annex burn
typedef struct _BURNING_PARM_ATTACH
{
	BOOL				bAttachBurn;			// Whether,0:Not; 1:Yes
	BYTE				bReserved[12];			// Reservations
} BURNING_PARM_ATTACH;

// Manual snap parameter
typedef struct  _MANUAL_SNAP_PARAMETER{
	int                 nChannel;               // snap channel,start with 0
	BYTE                bySequence[64];	        // snap sequence string
	BYTE                byReserved[60];         // reserved
}MANUAL_SNAP_PARAMETER;

// local preview split parameter
typedef struct _DEVICE_LOCALPREVIEW_SLIPT_PARAMETER
{
	int                 nSliptMode;            // split mode
	int                 nSubSliptNum;          // split num, start with 1
	BYTE                byReserved[64];        // reserved
}DEVICE_LOCALPREVIEW_SLIPT_PARAMETER;

// local preview split capability
typedef struct _DEVICE_LOCALPREVIEW_SLIPT_CAP
{
	int                 nModeNumber;
	int                 nSliptMode[36];        // split array
	BYTE                byReserved[64];        // reserved
}DEVICE_LOCALPREVIEW_SLIPT_CAP;

// Crossing information
typedef struct  _CROSSING_INFO
{
	char              szCrossingID[DH_MAX_CROSSING_ID];  // Crossing ID
	DWORD             dwLatitude;				// Latitude(0-180 Degree, North Latitude 30.183382 degree==120183382)
	DWORD             dwLongitude;				// Longitude(0-360 Degree,East Longitude 100.178274 degree==280178274)
	WORD              wSpeedLimit;				// limit speed(KM/H)
	BYTE 	          byReserved[22];
}CROSSING_INFO;

// Route crossing information
typedef struct  _ROUTE_CROSSING_INFO
{
	BYTE 			   byCrossingNum;             // valid number is 1~DH_MAX_CROSSING_NUM
	BYTE			   byReserved1; 
	WORD               wCrossingRange;            // Crossing range(m)
	CROSSING_INFO 	   stCrossingInfo[DH_MAX_CROSSING_NUM]; // Crossing information struct array
    BYTE			   byReserved2[2044];
}ROUTE_CROSSING_INFO;

// raid control
typedef struct _CTRL_RAID_INFO
{
	char				szAction[16];						// Add,Delete
	char				szName[16];							// Raid name
	int					nType;								// type 1:Jbod     2:Raid0      3:Raid1     4:Raid5
	int					nStatus;							// status 0:unknown ,1:active,2:degraded,3:inactive,4:recovering
	int					nCntMem;							// nMember count
	int					nMember[32];						// 1,2,3,... 
	int					nCapacity;							// capacity(G)
	int					nRemainSpace;						// remain space(M)
	int					nTank;								// tank 0:main,1:tank1,2:tank2.....
	char				reserved[128];
}CTRL_RAID_INFO;

// spake disk info
typedef struct _CTRL_SPARE_DISK_INFO
{
	char				szAction[16];						// Enable
	char				szScope[16];						// Global,Local
	char				szName[16];							// name
	unsigned int        nIndex;								// disk index
    const char*         pszDevName;                         // device name
    char                reserved[124];
}CTRL_SPARE_DISK_INFO;

typedef struct _CTRL_SPARE_DISK_INFO_A
{
	int						nCnt;							// count
	CTRL_SPARE_DISK_INFO	stSpaceDiskInfo[32];			// spare disk info array
}CTRL_SPARE_DISK_INFO_A;

// Alarm arm and disarm control
typedef struct __CTRL_ARM_DISARM_PARAM
{
	BYTE                bState;                             // Arm/disarm state, 0 means disarm, 1 means arm, 2 means forced arm
    char                reserve[3];
    const char*         szDevPwd;                           // User's Password
    NET_SCENE_MODE      emSceneMode;                        // Contextual Mode
    const char*         szDevID;                            // Forwarding the Target Device ID,Null=Non Forward
    char                reserve1[16];
} CTRL_ARM_DISARM_PARAM;

// alarm arm/disarm control expansion input parameter
typedef struct tagCTRL_ARM_DISARM_PARAM_EX_IN
{
    DWORD               dwSize;
    NET_ALARM_MODE      emState;                // arm/disarm status 
    const char*         szDevPwd;               // user password
    NET_SCENE_MODE      emSceneMode;            // scene mode 
    const char*         szDevID;                // transfer target device ID, as NULL means not transfer
} CTRL_ARM_DISARM_PARAM_EX_IN;

#define ARM_DISARM_ZONE_MAX (256)               // zone max number 

// alarm arm/disarm control expansion output parameter
typedef struct tagCTRL_ARM_DISARM_PARAM_EX_OUT
{
    DWORD               dwSize;
    DWORD               dwSourceNum;                    // alarm source input arming failed zone number 
    int                 nSource[ARM_DISARM_ZONE_MAX];   // alarm source input failed zone no.,zone no. start from 0
    DWORD               dwLinkNum;                      // link alarm arming failed zone number 
    int                 nLink[ARM_DISARM_ZONE_MAX];     // link alarm failed zone no., zone no. start from 0
} CTRL_ARM_DISARM_PARAM_EX_OUT;

// alarm arm/diarm control expansion(corresponding to DH_CTRL_ARMED_EX)
typedef struct tagCTRL_ARM_DISARM_PARAM_EX
{
    DWORD               dwSize;
    CTRL_ARM_DISARM_PARAM_EX_IN stuIn;                  // alarm arm/disarm control input
    CTRL_ARM_DISARM_PARAM_EX_OUT stuOut;                // alarm arm/disarm control output
} CTRL_ARM_DISARM_PARAM_EX;

typedef struct tagDHCTRL_CONNECT_WIFI_BYWPS_IN
{
	DWORD				dwSize;
	int                 nType;                         // WPS connect type,0:virtual buttons; 1:(device)pin; 2:(wifi hot point)pin
	char                szSSID[DH_MAX_SSID_LEN];       // when SSID,nType is 1 or 2,availability,biggest support 32-bit
	char                szApPin[DH_MAX_APPIN_LEN];     // APPIN, when nType = 2,availability,PIN is 8 digits,from the wife(hot point)
	char                szWLanPin[DH_MAX_APPIN_LEN];   // device pin,when nType = 1,availability.produced by the device when empty,not empty soecified by the user,need to increase in wifi
}DHCTRL_CONNECT_WIFI_BYWPS_IN;

typedef struct tagDHCTRL_CONNECT_WIFI_BYWPS_OUT
{
	DWORD				dwSize;
	char                szRetWLanPin[DH_MAX_APPIN_LEN];// return pin,when WPS's connect type is device end,this parameter is valid.
}DHCTRL_CONNECT_WIFI_BYWPS_OUT;

// CLIENT_ControlDevice interface DH_CTRL_WIFI_BY_WPS command parameter(WPS rapid configuration WIFI)
typedef struct tagDHCTRL_CONNECT_WIFI_BYWPS
{
	DWORD				dwSize;
	DHCTRL_CONNECT_WIFI_BYWPS_IN     stuWpsInfo;         // connect parameter(user to fill in)
	DHCTRL_CONNECT_WIFI_BYWPS_OUT    stuWpsResult;       // return data(device to return to)
} DHCTRL_CONNECT_WIFI_BYWPS;

// CLIENT_ControlDevice interface DH_CTRL_CLOSE_BURNER 
typedef struct tagNET_CTRL_BURNERDOOR
{
	DWORD		        dwSize;
	const char*         szBurnerName;                  // cd name,like "/dev/sda"
	BOOL                bResult;                       // operate result
    BOOL                bSafeEject;                    // Whether The Security Pop-up Drive, Data Save TRUE- pop Up Before, FALSE- Display)
}NET_CTRL_BURNERDOOR;

// CLIENT_ControlDevice interface DH_CTRL_START_PLAYAUDIO Order Parameter
typedef struct tagNET_CTRL_START_PLAYAUDIO
{
    DWORD               dwSize;
    char                szAudioPath[DH_MAX_AUDIO_PATH];
}NET_CTRL_START_PLAYAUDIO;

// report unit type
typedef enum tagNET_PLAYAUDIO_TYPE
{
    NET_PLAYAUDIO_TYPE_UNKNOWN = 0,
    NET_PLAYAUDIO_TYPE_PHRASE,                          // message type, no analysis, read in order, supported by audio file
    NET_PLAYAUDIO_TYPE_FILE,                            // play specific path audiofile(device complete path)
    NET_PLAYAUDIO_TYPE_PLATERNUM,                       // report plate no, read by plate format
    NET_PLAYAUDIO_TYPE_MONEY,                           // report amount, read byamount format
    NET_PLAYAUDIO_TYPE_DATE,                            // report date, read by date format
    NET_PLAYAUDIO_TYPE_TIME,                            // report time, read by time format
    NET_PLAYAUDIO_TYPE_EMPTY,                           // null type, stop one character time
}NET_PLAYAUDIO_TYPE;

// audio report content
typedef struct tagNET_CTRL_PLAYAUDIO_INFO
{
    DWORD               dwSize;
    NET_PLAYAUDIO_TYPE  emPlayAudioType;                // report unit type,see NET_PLAYAUDIO_TYPE
    char                szDetail[DH_COMMON_STRING_128]; // detailed content type detail:
                                                        // Phrase type:"Welcome"
                                                        // File type: "/home/parking.pcm"
                                                        // PlateNumbe type: "AA12345"
                                                        // Money type: "80.12 yuan"
                                                        // Date type: "2014/4/10"
                                                        // Time type : "1 day 10 hours 20 min 5 seconds
}NET_CTRL_PLAYAUDIO_INFO;

#define DH_MAX_PLAYAUDIO_COUNT    16  // max report content items
// CLIENT_ControlDevice port DH_CTRL_START_PLAYAUDIOEX command parameter
typedef struct tagNET_CTRL_START_PLAYAUDIOEX
{
    DWORD                       dwSize;
    unsigned int                nAudioCount;                            // report content quantity               
    NET_CTRL_PLAYAUDIO_INFO     stuAudioInfos[DH_MAX_PLAYAUDIO_COUNT];  // audioplay content
}NET_CTRL_START_PLAYAUDIOEX;

// CLIENT_ControlDevice port DH_CTRL_START_ALARMBELL, DH_CTRL_STOP_ALARMBELL command parameter
typedef struct tagNET_CTRL_ALARMBELL
{
    DWORD               dwSize;
    int                 nChannelID;             // Channel ID (start from 0)            
}NET_CTRL_ALARMBELL;

// Access controller -- Open door type
typedef enum tagEM_OPEN_DOOR_TYPE
{
	EM_OPEN_DOOR_TYPE_UNKNOWN = 0,
	EM_OPEN_DOOR_TYPE_REMOTE,						// Remote
	EM_OPEN_DOOR_TYPE_LOCAL_PASSWORD,				// Local_Password
	EM_OPEN_DOOR_TYPE_LOCAL_CARD,					// Local_Card
	EM_OPEN_DOOR_TYPE_LOCAL_BUTTON,					// Local_Button
}EM_OPEN_DOOR_TYPE;

// CLIENT_ControlDevice's param: DH_CTRL_ACCESS_OPEN
typedef struct tagNET_CTRL_ACCESS_OPEN
{
    DWORD               dwSize;
    int                 nChannelID;                     // Channel ID (start from 0) 
    const char*         szTargetID;                     // Target ID, NULL equals to not transmit    
    char                szUserID[MAX_COMMON_STRING_32];	// remote user id
	EM_OPEN_DOOR_TYPE	emOpenDoorType;                 // open door type
}NET_CTRL_ACCESS_OPEN;

// the enum for work mode of door
typedef enum tagEM_NET_DOOR_WORK_MODE_TYPE
{
    NET_DOOR_WORK_MODE_UNKNOWN,            // Unknown
	NET_DOOR_WORK_MODE_NORMAL,             // Normal
	NET_DOOR_WORK_MODE_SHUTDOWN,           // shutdown
	NET_DOOR_WORK_MODE_UNUSED,             // unused
	NET_DOOR_WORK_MODE_OPEN_DOOR_CONTINUE, // continued opening

}EM_NET_DOOR_WORK_MODE_TYPE;

// the input param for setting the work mode of door
typedef struct tagNET_IN_CTRL_ACCESS_SET_DOOR_WORK_MODE
{
	DWORD                   dwSize;
	EM_NET_DOOR_WORK_MODE_TYPE emWorkMode;    // the work mode
}NET_IN_CTRL_ACCESS_SET_DOOR_WORK_MODE;

// the output param for setting the work mode of door
typedef struct tagNET_OUT_CTRL_ACCESS_SET_DOOR_WORK_MODE
{
    DWORD                   dwSize;
}NET_OUT_CTRL_ACCESS_SET_DOOR_WORK_MODE;

// the input param for synchronizing the cabin led's time
typedef struct tagNET_IN_CTRL_SYNC_CABINLED_TIME
{
    DWORD                   dwSize;
}NET_IN_CTRL_SYNC_CABINLED_TIME;

// the output param for synchronizing the cabin led's time
typedef struct tagNET_OUT_CTRL_SYNC_CABINLED_TIME
{
    DWORD                   dwSize;
}NET_OUT_CTRL_SYNC_CABINLED_TIME;

// the enum for control mode of playing the cabin led
typedef enum tagEM_NET_CABIN_LED_PLAY_CONTROL_MODE
{
    NET_CABIN_LED_PLAY_CONTROL_MODE_UNKNOWN,          // Unknown
    NET_CABIN_LED_PLAY_CONTROL_MODE_NORMAL_TURN_ON,   // Normal mode (turn on the screen)
    NET_CABIN_LED_PLAY_CONTROL_MODE_NEXT_PROGRAM,     // Next program      
    NET_CABIN_LED_PLAY_CONTROL_MODE_PREVIOUS_PROGRAM, // Pre program
    NET_CABIN_LED_PLAY_CONTROL_MODE_PAUSE,            // Pause
    NET_CABIN_LED_PLAY_CONTROL_MODE_PAUSE_TURN_OFF,   // Pause (turn off the screen)
    NET_CABIN_LED_PLAY_CONTROL_MODE_PLAY_GO_ON,       // Go on playing
    NET_CABIN_LED_PLAY_CONTROL_MODE_GO_TO_TEST_ITSELF,// Go to test itself
    NET_CABIN_LED_PLAY_CONTROL_MODE_EXIT_TEST_ITSELF, // Exit testing itself
    NET_CABIN_LED_PLAY_CONTROL_MODE_SYSTEM_RESET,     // Reset the system
}EM_NET_CABIN_LED_PLAY_CONTROL_MODE;

// the input param for play and control the cabin led
typedef struct tagNET_IN_CTRL_CABINLED_PLAYCONTROL
{
    DWORD                                   dwSize;
    EM_NET_CABIN_LED_PLAY_CONTROL_MODE      emAction; // Play mode
}NET_IN_CTRL_CABINLED_PLAYCONTROL;

// the output param for play and control the cabin led
typedef struct tagNET_OUT_CTRL_CABINLED_PLAYCONTROL
{
    DWORD                   dwSize;
}NET_OUT_CTRL_CABINLED_PLAYCONTROL;

// the enum for cmd of modify the cabin led's content
typedef enum tagEM_NET_CABIN_LED_MODIFY_CONTENT_CMD_TYPE
{
    NET_CABIN_LED_MODIFY_CONTENT_CMD_TYPE_UNKNOWN,
    NET_CABIN_LED_MODIFY_CONTENT_CMD_TYPE_ADD       = 0, // Add
    NET_CABIN_LED_MODIFY_CONTENT_CMD_TYPE_MODIFY    = 1, // Modify
    NET_CABIN_LED_MODIFY_CONTENT_CMD_TYPE_DELETE    = 2, // Delete
}EM_NET_CABIN_LED_MODIFY_CONTENT_CMD_TYPE;

// the enum for effect of modify the cabin led's content
typedef enum tagEM_NET_CABIN_LED_MODIFY_CONTENT_EFFECT_TYPE
{
    NET_MODIFY_CONTENT_EFFECT_TYPE_UNKNOWN = -1,
    NET_MODIFY_CONTENT_EFFECT_TYPE_SHOW = 0,                      // Show normal
    NET_MODIFY_CONTENT_EFFECT_TYPE_MOVE_LEFT,                     // Move to left
    NET_MODIFY_CONTENT_EFFECT_TYPE_MOVE_RIGHT,                    // Move to right
    NET_MODIFY_CONTENT_EFFECT_TYPE_MOVE_UP,                       // Move to up
    NET_MODIFY_CONTENT_EFFECT_TYPE_MOVE_DOWN,                     // Move to down
    NET_MODIFY_CONTENT_EFFECT_TYPE_FILL_UP_LEFT,                  // Fill up the left
    NET_MODIFY_CONTENT_EFFECT_TYPE_FILL_UP_RIGHT,                 // Fill up the right
    NET_MODIFY_CONTENT_EFFECT_TYPE_FILL_UP_UP,                    // Fill up the up
    NET_MODIFY_CONTENT_EFFECT_TYPE_FILL_UP_DOWN,                  // Fill up the down
    NET_MODIFY_CONTENT_EFFECT_TYPE_FILL_UP_UP_LEFT,               // Fill up the up-left
    NET_MODIFY_CONTENT_EFFECT_TYPE_FILL_UP_DOWN_LEFT,             // Fill up the down-left
    NET_MODIFY_CONTENT_EFFECT_TYPE_FILL_UP_UP_RIGHT,              // Fill up the up-right
    NET_MODIFY_CONTENT_EFFECT_TYPE_FILL_UP_DOWN_RIGHT,            // Fill up the down-right
    NET_MODIFY_CONTENT_EFFECT_TYPE_FILL_UP_LEFT_RIGHT_TO_CENTER,  // Fill up from left-right to center
    NET_MODIFY_CONTENT_EFFECT_TYPE_FILL_UP_CENTER_TO_LEFT_RIGHT,  // Fill up from center to left-right
    NET_MODIFY_CONTENT_EFFECT_TYPE_FILL_UP_UP_DOWN_TO_CENTER,     // Fill up from up-down to center
    NET_MODIFY_CONTENT_EFFECT_TYPE_FILL_UP_CENTER_TO_UP_DOWN,     // Fill up from center to up-down
    NET_MODIFY_CONTENT_EFFECT_TYPE_FILL_UP_LEFT_CROSS_RIGHT,      // Fill up with cross left-right
    NET_MODIFY_CONTENT_EFFECT_TYPE_FILL_UP_UP_CROSS_DOWN,         // Fill up with cross up-down
    NET_MODIFY_CONTENT_EFFECT_TYPE_VERTICAL_BLINDS,               // Vertical_blinds
    NET_MODIFY_CONTENT_EFFECT_TYPE_HORIZONTAL_BLINDS,             // Horizontal_blinds  
    NET_MODIFY_CONTENT_EFFECT_TYPE_FILL_UP_CENTER_TO_SIDES,       // Fill up from center to sides
    NET_MODIFY_CONTENT_EFFECT_TYPE_FILL_UP_SIDES_TO_CENTER,       // Fill up from sides to center
    NET_MODIFY_CONTENT_EFFECT_TYPE_FILL_UP_CENTER_TO_CORNER,      // Fill up from center to corner
    NET_MODIFY_CONTENT_EFFECT_TYPE_FILL_UP_CORNER_TO_CENTER,      // Fill up from corner to center
    NET_MODIFY_CONTENT_EFFECT_TYPE_ROTATE_360,                    // Rotate 360
    NET_MODIFY_CONTENT_EFFECT_TYPE_ROTATE_180,                    // Rotate_180
}EM_NET_CABIN_LED_MODIFY_CONTENT_EFFECT_TYPE;

// the input param for modify the cabin led's content
typedef struct tagNET_IN_CTRL_CABINLED_MODIFY_CONTENT
{
    DWORD                                    dwSize;
    EM_NET_CABIN_LED_MODIFY_CONTENT_CMD_TYPE emCmd;             // Cmd type
    int                                      nMsgID;            // Msg id,range from 1 to 5
    EM_NET_CABIN_LED_MODIFY_CONTENT_EFFECT_TYPE emEffect;       // Effect type
    int                                      nSpeed;            // Show speed,range of 0 to 15
    int                                      nDuration;         // Duration time,Unit:second   
    char                                     szText[512];       // Text data
}NET_IN_CTRL_CABINLED_MODIFY_CONTENT;

// the output param for modify the cabin led's content
typedef struct tagNET_OUT_CTRL_CABINLED_MODIFY_CONTENT
{
    DWORD                   dwSize;
}NET_OUT_CTRL_CABINLED_MODIFY_CONTENT;

// the input param for get the cabin led's content
typedef struct tagNET_IN_CTRL_CABINLED_GET_CONTENT
{
    DWORD                   dwSize;    
    int                     nMsgID;            // Msg id,range from 1 to 5   
}NET_IN_CTRL_CABINLED_GET_CONTENT;

// the output param for get the cabin led's content
typedef struct tagNET_OUT_CTRL_CABINLED_GET_CONTENT
{
    DWORD                   dwSize;
    int                     nMsgID;            // Msg id,range from 1 to 5
    EM_NET_CABIN_LED_MODIFY_CONTENT_EFFECT_TYPE emEffect;          // Effect type
    int                     nSpeed;            // Show speed,range of 0 to 15
    int                     nDuration;         // Duration time,Unit:second
    char                    szText[512];       // Text data
}NET_OUT_CTRL_CABINLED_GET_CONTENT;

// Time period structure 															    
typedef struct tagDH_TSECT
{
    int				bEnable;	    // Current record period . Bit means the four Enable functions. From the low bit to the high bit:Motion detection record, alarm record and general record, when Motion detection and alarm happened at the same time can record.
									// used in NET_POS_EVENT_LINK, it means enable;
    int				iBeginHour;
    int				iBeginMin;
    int				iBeginSec;
    int				iEndHour;
    int				iEndMin;
    int				iEndSec;
} DH_TSECT, *LPDH_TSECT;

#define NET_CABINLED_SCHEDULE_TIME_SECTION_NUM 8  // the max count for time schedule of cabin led

// the input param for set the cabin led's time schedule
typedef struct tagNET_IN_CTRL_CABINLED_SET_SCHEDULE
{
    DWORD                       dwSize;    
    int                         nIndex;                                                 // the index, zero for Sunday 1-6 mins Monday to Saturday    
    DH_TSECT                    stuTimeSection[NET_CABINLED_SCHEDULE_TIME_SECTION_NUM]; // the array of time schedule
                                                                                        // the bEnable in DH_TSECT: zero for invalid msg id, greater than zero for valid msg id

}NET_IN_CTRL_CABINLED_SET_SCHEDULE;

// the out param for set the cabin led's time schedule
typedef struct tagNET_OUT_CTRL_CABINLED_SET_SCHEDULE
{
    DWORD dwSize;   
}NET_OUT_CTRL_CABINLED_SET_SCHEDULE;

// the input param for get the cabin led's time schedule
typedef struct tagNET_IN_CTRL_CABINLED_GET_SCHEDULE
{
    DWORD                       dwSize;    
    int                         nIndex;                                                 // the index, zero for Sunday 1-6 mins Monday to Saturday 
}NET_IN_CTRL_CABINLED_GET_SCHEDULE;

// the out param for get the cabin led's time schedule
typedef struct tagNET_OUT_CTRL_CABINLED_GET_SCHEDULE
{
    DWORD dwSize;
    DH_TSECT                    stuTimeSection[NET_CABINLED_SCHEDULE_TIME_SECTION_NUM]; // the array of time schedule
                                                                                        // the bEnable in DH_TSECT: zero for invalid msg id, greater than zero for valid msg id
}NET_OUT_CTRL_CABINLED_GET_SCHEDULE;

// the in param for get the cabin led's char encoding
typedef struct tagNET_IN_CTRL_CABINLED_GET_CHAR_ENCODING
{
    DWORD                       dwSize;                     // Size in bytes of the specified structure. This member must be set.    
}NET_IN_CTRL_CABINLED_GET_CHAR_ENCODING;

// the out param for get the cabin led's char encoding
typedef struct tagNET_OUT_CTRL_CABINLED_GET_CHAR_ENCODING
{
    DWORD dwSize;                                           // Size in bytes of the specified structure. This member must be set.
    int nCharacterEncoding;                                 // the char encoding, 0-GB2312 1-Unicode                         
}NET_OUT_CTRL_CABINLED_GET_CHAR_ENCODING;


// the type of control caibin led
typedef enum tagEM_CABIN_LED_CONTROL_TYPE
{
    DH_CABIN_LED_CONTROL_UNKNOWN,                       // Unknown                  
    DH_CABIN_LED_CONTROL_SYNC_TIME,                     // Synchronize the time of cabin led (pInParam corresponding structure NET_IN_CTRL_SYNC_CABINLED_TIME,pOutParam corresponding structure NET_OUT_CTRL_SYNC_CABINLED_TIME)
    DH_CABIN_LED_CONTROL_PLAY_CONTROL,                  // Control the play mode of cabin led (pInParam corresponding structure NET_IN_CTRL_CABINLED_PLAYCONTROL,pOutParam corresponding structure NET_OUT_CTRL_CABINLED_PLAYCONTROL)
    DH_CABIN_LED_CONTROL_MODIFY_CONTENT,                // Modify the content of cabin led (pInParam corresponding structure NET_IN_CTRL_CABINLED_MODIFY_CONTENT,pOutParam corresponding structure NET_OUT_CTRL_CABINLED_MODIFY_CONTENT)
    DH_CABIN_LED_CONTROL_GET_CONTENT,                   // Modify the content of cabin led(pInParam corresponding structureNET_IN_CTRL_CABINLED_GET_CONTENT ,pOutParam corresponding structureNET_OUT_CTRL_CABINLED_GET_CONTENT)
    DH_CABIN_LED_CONTROL_SET_SCHEDULE,                  // Set the time schedule of cabin led(pInParam corresponding structureNET_IN_CTRL_CABINLED_SET_SCHEDULE ,pOutParam corresponding structureNET_OUT_CTRL_CABINLED_SET_SCHEDULE)
    DH_CABIN_LED_CONTROL_GET_SCHEDULE,                  // Get the time schedjle of cabin led(pInParam corresponding structureNET_IN_CTRL_CABINLED_GET_SCHEDULE ,pOutParam corresponding structureNET_OUT_CTRL_CABINLED_GET_SCHEDULE)
    DH_CABIN_LED_CONTROL_GET_CHAR_ENCODING,             // Get the char encoding of cabin led(pInParam corresponding structure NET_IN_CTRL_CABINLED_GET_CHAR_ENCODING ,pOutParam corresponding structureNET_OUT_CTRL_CABINLED_GET_CHAR_ENCODING)

}EM_CABIN_LED_CONTROL_TYPE;


// CLIENT_ControlDevice's param: DH_CTRL_ACCESS_CLOSE
typedef struct tagNET_CTRL_ACCESS_CLOSE
{
    DWORD               dwSize;
    int                 nChannelID;             // Channel ID (start from 0) 
}NET_CTRL_ACCESS_CLOSE;

typedef enum tagNET_ALARM_TYPE
{
    NET_ALARM_LOCAL = 0,                        // SwitchingValue Defence Area's Alarm Information
    NET_ALARM_ALARMEXTENDED,                    // Expand Module Alarm Event)
    NET_ALARM_TEMP,                             // Temperature Alarm Event)
    NET_ALARM_URGENCY,                          // Emergency Alarm Event)
    NET_ALARM_RCEMERGENCYCALL,                  // Emergency call alarm event (corresponding to DH_ALARM_RCEMERGENCY_CALL event )
    NET_ALARM_ALL,                              // ALL Alarm Event
}NET_ALARM_TYPE;

// CLIENT_ControlDevice's param: DH_CTRL_CLEAR_ALARM
typedef struct tagNET_CTRL_CLEAR_ALARM
{
    DWORD               dwSize;
    int                 nChannelID;             // Defence Area's ID, -1 means all Channel
    NET_ALARM_TYPE      emAlarmType;            // Event Type
    const char*         szDevPwd;               // Landing Equipment Such As Password, Do not Use Encryption Disappear the Police, Direct Assignment of NULL
    BOOL                bEventType;             // Means if enable nEventType field, TRUE:nEventType replaces emAlarmType field, FALSE: use emAlarmType field , ignore nEventType field 
    int                 nEventType;             // Event type, corresponding fMessCallBack called lCommand field , as CLIENT_StartListenEx port got event type
                                                // Such as DH_ALARM_ALARM_EX2 means local alarm event
}NET_CTRL_CLEAR_ALARM;

// TV Wall Display Information Control Parameters
typedef struct tagNET_CTRL_MONITORWALL_TVINFO 
{
    DWORD               dwSize;
    int                 nMonitorWallID;         // TV Wall ID, fill in DH_CTRL_MONITORWALL_TVINFO and DH_DEVSTATE_MONITORWALL_TVINFO
    BOOL                bDecodeChannel;         // Display Decoding Channel Information, fill in DH_CTRL_MONITORWALL_TVINFO
    BOOL                bControlID;             // Display Screen ID, fill in DH_CTRL_MONITORWALL_TVINFO
    BOOL                bCameraID;              // Camera ID, fill in DH_CTRL_MONITORWALL_TVINFO
} NET_CTRL_MONITORWALL_TVINFO;

// CLIENT_ControlDevice's param: DH_CTRL_START_VIDEO_ANALYSE
typedef struct tagNET_CTRL_START_VIDEO_ANALYSE
{
    DWORD               dwSize; 
    int                 nChannelId;             // Channel ID  
}NET_CTRL_START_VIDEO_ANALYSE;

// CLIENT_ControlDevice's param: DH_CTRL_STOP_VIDEO_ANALYSE 
typedef struct tagNET_CTRL_STOP_VIDEO_ANALYSE
{
    DWORD               dwSize; 
    int                 nChannelId;             // Channel ID  
}NET_CTRL_STOP_VIDEO_ANALYSE;

// CLIENT_ControlDevice's param DH_CTRL_MULTIPLAYBACK_CHANNALES 
typedef struct tagNET_CTRL_MULTIPLAYBACK_CHANNALES
{
    DWORD               dwSize;
    LLONG               lPlayBackHandle;        // Playback Handle, CLIENT_MultiPlayBack Returns the Value
    int                 nChannels[DH_MAX_MULTIPLAYBACK_CHANNEL_NUM];// Preview Channel ID
    int                 nChannelNum;            // Preview Channel Number 
}NET_CTRL_MULTIPLAYBACK_CHANNALES;

// CLIENT_ControlDevice's param: DH_CTRL_SET_BYPASS 
typedef struct tagNET_CTRL_SET_BYPASS
{
    DWORD               dwSize;
    const char*         szDevPwd;               // Login Device Password
    NET_BYPASS_MODE     emMode;                 // Channel Statue
    int                 nLocalCount;            // Local Alarm Input Channel Number
    int*                pnLocal;                // Local Alarm Input Channel ID    
    int                 nExtendedCount;         // Expand Module Alarm Input Channel Number
    int*                pnExtended;             // Expand Module Alarm Input Channel ID
}NET_CTRL_SET_BYPASS;

// CLIENT_ControlDevice's param: DH_CTRL_SET_MEDIAKIND 
typedef struct tagNET_CTRL_SET_MEDIAKIND
{
    DWORD                dwSize;                        //  set dwSize to sizeof(NET_CTRL_SET_MEDIAKIND)
    int                  nMediaKind;                    //  media type,0:video and audio,1:video only,2:audio only
    DWORD                dwChannelCount;                //  channel count
    int                  nChannels[256];                //  video channel (start form 0)
} NET_IN_SET_MEDIAKIND;

// CLIENT_ControlDevice's param: DH_CTRL_SET_MEDIAKIND 
typedef struct tagNET_OUT_SET_MEDIAKIND
{
    DWORD                dwSize;                        //  set dwSize to sizeof(NET_OUT_SET_MEDIAKIND)
} NET_OUT_SET_MEDIAKIND;

// CLIENT_QueryDevState's param: DH_DEVSTATE_GET_BYPASS 
typedef struct tagNET_DEVSTATE_GET_BYPASS
{
    DWORD               dwSize;
    int                 nLocalCount;            // Local Alarm Input Channel Number
    int*                pnLocal;                // Local Alarm Input Channel ID    
    int                 nExtendedCount;         // Expand Module Alarm Input Channel Number
    int*                pnExtended;             // Expand Module Alarm Input Channel ID
    NET_BYPASS_MODE*    pemLocal;               // Local Alarm Input Channel Statue
    NET_BYPASS_MODE*    pemExtended;            // Expand Module Alarm Input Channel Statue
}NET_DEVSTATE_GET_BYPASS;

// CLIENT_QueryDevState's param: DH_DEVSTATE_BURNERDOOR 
typedef struct tagNET_DEVSTATE_BURNERDOOR
{
	DWORD		        dwSize;
	const char*         szBurnerName;                  // cd name,like "/dev/sda"
	bool                bEjected;                      // ejected or not
	BYTE				Reserved[3];			       // reserved
}NET_DEVSTATE_BURNERDOOR;

// CLIENT_QueryDevState iterface DH_DEVSTATE_GET_DATA_CHECK
typedef struct tagNET_DEVSTATE_DATA_CHECK
{
    DWORD		        dwSize;                     // struct size
    const char*         szBurnerName;               // cd name,like"/dev/s",if it is multiple track, including one of path is ok
    char                szState[DH_MAX_STRING_LEN]; // "NotStart" "Verifying" "Failed" "Succeed"
    int                 nPercent;                   // check percent:0-100,when state  = Verifying,availability.
}NET_DEVSTATE_DATA_CHECK;

// CLIENT_ListenServer backcall interface, fServiceCallBack support command mode
enum { 
	DH_DVR_DISCONNECT=-1,	                        // Device disconnection callback during the verification period
	DH_DVR_SERIAL_RETURN=1,                         // Device send out SN callback char* szDevSerial
	NET_DEV_AUTOREGISTER_RETURN,                    // when device registering,serial number and token to carry, corresponding NET_CB_AUTOREGISTER
    NET_DEV_NOTIFY_IP_RETURN,                       // Equipment is only reported IP, not used for active registration
};
typedef struct tagNET_CB_AUTOREGISTER
{
	DWORD			dwSize;                          // struct size
	char            szDevSerial[DH_DEV_SERIALNO_LEN];// device serial
	char            szToken[MAX_PATH];               // token
}NET_CB_AUTOREGISTER;

// public agency registration
typedef struct tagCLOUDSERVICE_CONNECT_PARAM
{
	DWORD               dwSize;   
	char                szDstIp[DH_MAX_IPADDR_EX_LEN];    // server IP
	int                 nDstPort;                         // server port 
	DWORD               dwConnectType;                    // connect type,0:main connect, 1:the tunnel connection,2:data connect,3:dynamic registration link(use 0xb4)
	char                szToken[MAX_PATH];                // the only id token
}NET_CLOUDSERVICE_CONNECT_PARAM;

typedef struct tagCLOUDSERVICE_CONNECT_RESULT
{
	DWORD               dwSize;  
	DWORD               dwConnectState;                   // connect state ,1 succeed, 2 failed
	char                szMessage[DH_MAX_CLOUDCONNECT_STATE_LEN]; // connect status info,"Success","Password Error","Network Error","Timeout"
}NET_CLOUDSERVICE_CONNECT_RESULT;
///////////////////////////////definition of configuration///////////////////////////////

//-------------------------------Device Property ---------------------------------
// Device information 
typedef struct
{
	BYTE				sSerialNumber[DH_SERIALNO_LEN];	// SN
	BYTE				byAlarmInPortNum;		// DVR alarm input amount
	BYTE				byAlarmOutPortNum;		// DVR alarm output amount
	BYTE				byDiskNum;				// DVR HDD amount 
	BYTE				byDVRType;				// DVR type.Please refer to NET_DEVICE_TYPE
    union
    {
	BYTE				byChanNum;				// DVR channel amount 
        BYTE            byLeftLogTimes;         // When login failed due to password error, notice user via this parameter, remaining login times, is 0 means this parameter is invalid
    };
} NET_DEVICEINFO, *LPNET_DEVICEINFO;

// Device extension info 
typedef struct
{
    BYTE                sSerialNumber[DH_SERIALNO_LEN];	    // serial number
    int	                nAlarmInPortNum;                    // count of DVR alarm input
    int	                nAlarmOutPortNum;                   // count of DVR alarm output
    int	                nDiskNum;                           // number of DVR disk
    int	                nDVRType;                           // DVR type, refer to NET_DEVICE_TYPE
    int                 nChanNum;                           // number of DVR channel
    BYTE                byLimitLoginTime;                   // Online Timeout, Not Limited Access to 0, not 0 Minutes Limit Said
    BYTE                byLeftLogTimes;                     // When login failed due to password error, notice user via this parameter, remaining login times, is 0 means this parameter is invalid
    BYTE                bReserved[2];                       // keep bytes, bytes aligned
    int                 nLockLeftTime;                      // when log in failed, the left time for users to unlock (seconds), -1 indicate the device haven't set the parameter 
    char                Reserved[24];                       // reserved
} NET_DEVICEINFO_Ex, *LPNET_DEVICEINFO_Ex;

//Device software version information. The higher 16-bit means the main version number and low 16-bit means second version number. 
typedef struct 
{
	DWORD				dwSoftwareVersion;
	DWORD				dwSoftwareBuildDate;
	DWORD				dwDspSoftwareVersion;
	DWORD				dwDspSoftwareBuildDate;
	DWORD				dwPanelVersion;
	DWORD				dwPanelSoftwareBuildDate;
	DWORD				dwHardwareVersion;
	DWORD				dwHardwareDate;
	DWORD				dwWebVersion;
	DWORD				dwWebBuildDate;
} DH_VERSION_INFO, *LPDH_VERSION_INFO;

// Device software version information. Corresponding to CLIENT_QueryDevState
typedef struct  
{
	char				szDevSerialNo[DH_DEV_SERIALNO_LEN];	// Serial number 
	char				byDevType;							// Device type, please refer to NET_DEVICE_TYPE
	char				szDevType[DH_DEV_TYPE_LEN];			// Device detailed type. String format. It may be null.
	int					nProtocalVer;						// Protocol version number 
	char				szSoftWareVersion[DH_MAX_URL_LEN];
	DWORD				dwSoftwareBuildDate;
    char                szPeripheralSoftwareVersion[DH_MAX_URL_LEN];// From the Slice Version Information, The String Format, May Be Empty
    DWORD               dwPeripheralSoftwareBuildDate;
    char                szGeographySoftwareVersion[DH_MAX_URL_LEN]; // Geographical Iinformation Positioning Chip Version Information, The String Format, May Be empty
    DWORD               dwGeographySoftwareBuildDate;
	char				szHardwareVersion[DH_MAX_URL_LEN];
	DWORD				dwHardwareDate;
	char				szWebVersion[DH_MAX_URL_LEN];
	DWORD				dwWebBuildDate;
    char                szDetailType[MAX_COMMON_STRING_64];          // Device detailed type. String format. It may be null.
	char				reserved[192];
} DHDEV_VERSION_INFO;

// DSP capacity description. Correspondign to CLIENT_GetDevConfig
typedef struct 
{
	DWORD				dwVideoStandardMask;	// video format mask. Bit stands for the video format device supported.
	DWORD				dwImageSizeMask;		// Resolution mask bit. Bit stands for the resolution setup devices supported.
	DWORD				dwEncodeModeMask;		// Encode mode mask bit. Bit stands for the encode mode devices supported.
	DWORD				dwStreamCap;			// The multiple-media function the devices supported
												// The first bit:main stream
												// The second bit:extra stream 1
												// The third bit:extra stream 2
												// The fifth bit: snapshot in .jpg format
	DWORD				dwImageSizeMask_Assi[8];// When the main stream is the corresponding resolution, the supported extra stream resolution mask.
	DWORD				dwMaxEncodePower;		// The highest encode capacity DSP supported
	WORD				wMaxSupportChannel;		// The max video channel amount each DSP supported.
	WORD				wChannelMaxSetSync;		// Max encode bit setup in each DSP channel  are synchronized or not ;0:does not synchronized,1:synchronized
} DH_DSP_ENCODECAP, *LPDH_DSP_ENCODECAP;

// DSP capacity description. Extensive type. Corresponding to CLIENT_QueryDevState
typedef struct 
{
	DWORD				dwVideoStandardMask;	// video format mask. Bit stands for the video format device supported.
	DWORD				dwImageSizeMask;		// Resolution mask bit. Bit stands for the resolution setup devices supported.
	DWORD				dwEncodeModeMask;		// Encode mode mask bit. Bit stands for the encode mode devices supported.
	DWORD				dwStreamCap;			// The multiple-media function the devices supported
												// The first bit:main stream
												// The second bit:extra stream 1
												// The third bit:extra stream 2
												// The fifth bit: snapshot in .jpg format
	DWORD				dwImageSizeMask_Assi[32];// When the main stream is the corresponding resolution, the supported extra stream resolution mask.
	DWORD				dwMaxEncodePower;		// The highest encode capacity DSP supported
	WORD				wMaxSupportChannel;		// The max video channel amount each DSP supported.
	WORD				wChannelMaxSetSync;		// Max encode bit setup in each DSP channel  are synchronized or not;0:do not synchronized,1:synchronized
	BYTE				bMaxFrameOfImageSize[32];// The max sampling frame rate in different resolution. Bit corresponding to the dwVideoStandardMask.
	BYTE				bEncodeCap;				// Symbol. The configuration shall meet the following requirements, otherwise the configuration is invalid.
												// 0:main stream encode capacity+extra stream encode capacity<= device encode capacity 
												// 1:main stream encode capacity +extra stream encode capacity<= device encode capacity 
												// Extra stream encode capacity <=main stream encode capacity 
												// Extra stream resolution<=main stream resolution 
												// Main stream resolution and extra stream resolution <=front-end video collection frame rate
												// 2:N5 Device
												// Extra stream encode capacity <=main stream encode capacity
												// Query  the supported resolution and the corresponding maximum frame rate
	char				reserved[95];
} DHDEV_DSP_ENCODECAP, *LPDHDEV_DSP_ENCODECAP;

// DSP extend capacity description. Corresponding to CLIENT_QueryDevState
typedef struct 
{
	DWORD				dwVideoStandardMask;	// video format mask. Bit stands for the video format device supported.
	DWORD				dwImageSizeMask;		// Resolution mask bit. Bit stands for the resolution setup devices supported.
	DWORD				dwEncodeModeMask;		// Encode mode mask bit. Bit stands for the encode mode devices supported.
	DWORD				dwStreamCap;			// The multiple-media function the devices supported.
												// The first bit:main stream
												// The second bit:extra stream 1
												// The third bit:extra stream 2
												// The forth bit:extra stream 3
												// The fifth bit: snapshot in .jpg format
	DWORD				dwImageSizeMask_Assi[3][64];// When the main stream is the corresponding resolution, the supported extra stream resolution mask, the 0,1,2 member in the group correspond extra stream1,2,3
	DWORD				dwMaxEncodePower;		// The highest encode capacity DSP supported
	WORD				wMaxSupportChannel;		// The max video channel amount each DSP supported.
	WORD				wChannelMaxSetSync;		// Max encode bit setup in each DSP channel  are synchronized or not;0:do not synchronized,1:synchronized
	BYTE				bMaxFrameOfImageSize[32];// The max sampling frame rate in different resolution. Bit corresponding to the dwVideoStandardMask.
	BYTE				bEncodeCap;				// Symbol. The configuration shall meet the following requirements, otherwise the configuration is invalid.
												// 0:main stream encode capacity+extra stream encode capacity<= device encode capacity 
												// 1:main stream encode capacity +extra stream encode capacity<= device encode capacity 
												// Extra stream encode capacity <=main stream encode capacity 
												// Extra stream resolution<=main stream resolution 
												// Main stream resolution and extra stream resolution <=front-end video collection frame rate
												// 2:N5 Device
												// Extra stream encode capacity <=main stream encode capacity
												// Query  the supported resolution and the corresponding maximum frame rate
	BYTE				btReserve1[3];			// reserved
	DWORD				dwExtraStream;			// bit0-main stream, bit1-extra stream1, bit2-extra stream2
	DWORD				dwCompression[3];		// extra stream compression

	char				reserved[108];
} DHDEV_DSP_ENCODECAP_EX, *LPDHDEV_DSP_ENCODECAP_EX;

// System information
typedef struct 
{
	DWORD				dwSize;
	/* The following are read only for device. */
	DH_VERSION_INFO		stVersion;
	DH_DSP_ENCODECAP	stDspEncodeCap;			// DSP  capacity description 
	BYTE				szDevSerialNo[DH_DEV_SERIALNO_LEN];	// SN
	BYTE				byDevType;				// device type. Please refer to enumeration NET_DEVICE_TYPE
	BYTE				szDevType[DH_DEV_TYPE_LEN];	// Device detailed type. String format. It may be empty.
	BYTE				byVideoCaptureNum;		// Video port amount
	BYTE				byAudioCaptureNum;		// Audio port amount 
	BYTE				byTalkInChanNum;		// NSP
	BYTE				byTalkOutChanNum;		// NSP
	BYTE				byDecodeChanNum;		// NSP
	BYTE				byAlarmInNum;			// Alarm input port amount
	BYTE				byAlarmOutNum;			// Alarm output amount port
	BYTE				byNetIONum;				// network port amount 
	BYTE				byUsbIONum;				// USB USB port amount
	BYTE				byIdeIONum;				// IDE amount 
	BYTE				byComIONum;				// COM amount 
	BYTE				byLPTIONum;				// LPT amount
	BYTE				byVgaIONum;				// NSP
	BYTE				byIdeControlNum;		// NSP
	BYTE				byIdeControlType;		// NSP
	BYTE				byCapability;			// NSP, expansible description 
	BYTE				byMatrixOutNum;			// video matrix output amount 
	/* The following are read-write part for device */
	BYTE				byOverWrite;			// Operate when HDD is full(overwrite/stop)
	BYTE				byRecordLen;			// Video file Package length
	BYTE				byDSTEnable;			// Enable  DTS or not  1--enable. 0--disable
	WORD				wDevNo;					// Device serial number. Remote control can use this SN to control.
	BYTE				byVideoStandard;		// Video format
	BYTE				byDateFormat;			// Date format
	BYTE				byDateSprtr;			// Date separator(0:".",1:"-",2:"/")
	BYTE				byTimeFmt;				// Time separator  (0-24H,1-12H)
	BYTE				byLanguage;				// Please refer to DH_LANGUAGE_TYPE for enumeration value
} DHDEV_SYSTEM_ATTR_CFG, *LPDHDEV_SYSTEM_ATTR_CFG;

// The returned information after modify device 
typedef struct
{
	DWORD				dwType;					// Type (type of GetDevConfig and SetDevConfig)
	WORD				wResultCode;			// Returned bit;0:succeeded,1:failed,2:illegal data,3:can not set right now,4:have no right 
	WORD   				wRebootSign;			// Reboot symbol;0:do not need to reboot,1:need to reboot to get activated 
	DWORD				dwReserved[2];			// Reserved 
} DEV_SET_RESULT;

// play result
typedef struct
{
	DWORD              	dwResultCode;           // result code,
                                                // 1:no authority,2:device not support,3:insufficient resources,4:get data error, 
                                                // 11:resouces are taked by advance users, 12:forbidden, 13:limited from reading disk data
                                                // 14:multi window preview already open, out of resource, compressed playback failed, 15: compressed playback already open
	LLONG               lPlayHandle;            // play handle
	BYTE               	byReserved[32];         // reserved   
	                    
}DEV_PLAY_RESULT;

// request audio tal error return info
typedef struct
{
    DWORD               dwResultCode;               // error return code CLIENT_GetLastError port return value
                                                    // NET_ERROR_TALK_REJECT _EC(375) // refuse talk
                                                    // NET_ERROR_TALK_RESOURCE_CONFLICIT _EC(377) // resource conflict
                                                    // NET_ERROR_TALK_UNSUPPORTED_ENCODE _EC(378) // not support talk encode format
                                                    // NET_ERROR_TALK_RIGHTLESS _EC(379) // no right
    LLONG               lTalkHandle;                // correspoinding handle
    BYTE                byReserved[32];             // reserved text                         
}DEV_TALK_RESULT;

//DST(Daylight Save Time) setup
typedef struct  
{
	int					nYear;					// Year[200 - 2037]
	int					nMonth;					// Month[1 - 12]
	int					nHour;					// Hour[0 - 23]
	int					nMinute;				// Minute [0 - 59]
	int					nWeekOrDay;				// [-1 - 4]0:It means it adopts the date calculation method. 
												// 1:in week: the first week ,2:the second week,3:the third week,4: The fourth week ,-1: the last week  
	union
	{
		int				iWeekDay;				// week[0 - 6](nWeekOrDay:calculated in week )0: Sunday, 1:Monday , 2:Tuesday ,3: Wednesday,4:Thirsday ,5: Friday,6:Saturday 
		int				iDay;					// date[1 - 31] (nWeekOrDay: in date)
	};
	
	DWORD				dwReserved[8];			// Reserved 
}DH_DST_POINT;

typedef struct  
{
	DWORD				dwSize;
	int					nDSTType;				// DST position way. 0: position in date  , 1:position in week  
	DH_DST_POINT        stDSTStart;             // Enable DTS
	DH_DST_POINT        stDSTEnd;				// End DTS
	DWORD				dwReserved[16];			// Reserved
}DHDEV_DST_CFG;


// Auto maintenance setup
typedef struct
{
	DWORD				dwSize;
	BYTE				byAutoRebootDay;		// Auto reboot;0:never, 1:each day,2:each Sunday,3:Each Monday ,......
	BYTE				byAutoRebootTime;		// 0:0:00,1:1:00,......23:23:00
	BYTE				byAutoDeleteFilesTime;	// Auto delete file;0:never,1:24H,2:48H,3:72H,4:96H,5:ONE WEEK,6:ONE MONTH
	BYTE				reserved[13];			// Reserved bit
} DHDEV_AUTOMT_CFG;

// car's disk info
typedef struct  
{
	DWORD				dwSize;										// struct size,must initialize
	DWORD				dwVolume;									// disk volume
	DWORD				dwFreeSpace;								// free space MB
	BYTE				byModle[DH_MAX_HARDDISK_TYPE_LEN];			// disk mode
	BYTE				bySerialNumber[DH_MAX_HARDDISK_SERIAL_LEN];	// serial number
}DHDEV_VEHICLE_DISK;

// car's 3G mode info,the largest support DH_MAX_SIM_NUM
typedef struct
{
	DWORD				dwSize;						// struct size,must initialize
	BYTE				szIMSI[DH_MAX_SIM_LEN];		// SIM's value = 460012002778636,top three for the country code MCC,4-6for the network code MNC,the last is MSIN,a total of not more than 15 characters
	BYTE				szMDN[DH_MAX_MDN_LEN];		// MDN
}DHDEV_VEHICLE_3GMODULE;

// car basic info
typedef struct 
{
	DWORD					dwSize;								// struct size,must initialize
	DWORD					dwSoftwareBuildDate;				// soft ware build date
	char					szVersion[DH_MAX_VERSION_LEN];		// version
	char					szDevSerialNo[DH_DEV_SERIALNO_LEN];	// device serial no.
	char					szDevType[DH_DEV_TYPE_LEN];			// device type,string format,can empty
	DWORD					dwDiskNum;							// disk number
	DHDEV_VEHICLE_DISK		stuHarddiskInfo[DH_MAX_DISKNUM];	// disk info
	DWORD					dw3GModuleNum;						// count of 3G module
	DHDEV_VEHICLE_3GMODULE	stu3GModuleInfo[DH_MAX_SIM_NUM];	// 3G module info
}DHDEV_VEHICLE_INFO;

// net interface,the largest support DH_MAX_NETINTERFACE_NUM
typedef struct tagDHDEV_NETINTERFACE_INFO
{
	int             dwSize;
	BOOL			bValid;								// valid or not
	BOOL			bVirtual;							// support virtual or not
	int             nSpeed;								// theory of speed (Mbps)
	int             nDHCPState;							// 0-disable, 1-getting, 2-get succeed
	char			szName[DH_NETINTERFACE_NAME_LEN];	// net port mane
	char			szType[DH_NETINTERFACE_TYPE_LEN];	// net type
	char			szMAC[DH_MACADDR_LEN];			    // MAC addr
	char			szSSID[DH_MAX_SSID_LEN];			// SSID, if only szType == "Wireless",availability
	char            szConnStatus[DH_MAX_CONNECT_STATUS_LEN]; // Wifi,3G connect status,"Inexistence" : not exist, "Down": close "Disconn": disconnect "Connecting" "Connected"
	int             nSupportedModeNum;                  // support mode number
	char            szSupportedModes[DH_MAX_MODE_NUM][DH_MAX_MODE_LEN];// support 3G net mode	"TD-SCDMA", "WCDMA", "CDMA1x", "EDGE", "EVDO"
} DHDEV_NETINTERFACE_INFO;
//-----------------------------Video Channel Property -------------------------------

// Zone;Each margin is total lenght :8192
typedef struct 
{
    long					left;
    long					top;
    long					right;
    long					bottom;
} DH_RECT, *LPDH_RECT;

typedef struct tagNET_RECT
{
    int             nLeft;
    int             nTop;
    int             nRight;
    int             nBottom;
} NET_RECT;

// 2 dimension point
typedef struct 
{
	short					nx;
	short					ny;
} DH_POINT, *LPDH_POINT;

// 
typedef struct
{
	int        nPointNum;                           // point number
	DH_POINT   stuPoints[DH_MAX_DETECT_REGION_NUM]; // point info
}DH_POLY_POINTS;

// struct of OSD attribute
typedef struct  tagENCODE_WIDGET
{
	DWORD				rgbaFrontground;		// Object front view. Use bit to represent:red, green, blue and transparency.
	DWORD				rgbaBackground;			// Object back ground. Use bit to represent:red, green, blue and transparency.
	DH_RECT				rcRect;					// Position
	BYTE				bShow;					// Enable display
	BYTE                bExtFuncMask;           // Extended function,mask 
                                                // bit0 indicate week showing,0-not show 1-show 
	BYTE				byReserved[2];
} DH_ENCODE_WIDGET, *LPDH_ENCODE_WIDGET;

// Channel audio property 
typedef struct 
{
	// Video property 
	BYTE				byVideoEnable;			// Enable video;1:open,0:close 
	BYTE				byBitRateControl;		// Bit stream control;Please refer to constant Bit Stream Control definition 
	BYTE				byFramesPerSec;			// Frame rate
	BYTE				byEncodeMode;			// Encode mode:please refer to constant Encode Mode definition
	BYTE				byImageSize;			// Resolution:please refer to constant Resolution definition.
	BYTE				byImageQlty:7;			// Level 1-6
	BYTE                byImageQltyType:1;       
	WORD				wLimitStream;			// Limit stream parameter
	// Audio property 
	BYTE				byAudioEnable;			// Enable audio;1:open,0:close
	BYTE				wFormatTag;				// Audio encode mode:0:G711A,1:PCM,2:G711U,3:AMR,4:AAC 
	WORD				nChannels;				// Track amount 
	WORD				wBitsPerSample;			// Sampling depth 	
	BYTE				bAudioOverlay;			// Enabling audio
	BYTE				bH264ProfileRank;       // H.264 Profile rank(when byEncodeMode is h264, this value works ), see enmu struct EM_H264_PROFILE_RANK,when this value is 0, it means nothing
	DWORD				nSamplesPerSec;			// Sampling rate 
	BYTE				bIFrameInterval;		// 0-149. I frame interval amount. Describe the P frame amount between two I frames.
	BYTE				bScanMode;				// NSP
	BYTE				bReserved_3;
	BYTE				bReserved_4;
} DH_VIDEOENC_OPT, *LPDH_VIDEOENC_OPT;

// Image color property  
typedef struct 
{
	DH_TSECT			stSect;
	BYTE				byBrightness;			// Brightness:0-100
	BYTE				byContrast;				// Contrast:0-100
	BYTE				bySaturation;			// Saturation:0-100
	BYTE				byHue;					// Hue:0-100
	BYTE				byGainEn;				// Enable gain
	BYTE				byGain;					// Gain:0-100
	BYTE				byGamma;                // gamma 0-100
	BYTE				byReserved[1];
} DH_COLOR_CFG, *LPDH_COLOR_CFG;

// Image channel property structure 
typedef struct 
{
	WORD				dwSize;
	BYTE				bNoise;
	BYTE				bMode;   // (vehicle's special need)model 1 (quality first): 4-way video resolution D1, frame rate 2fps, bit rate 128kbps(225MB per hour) 
	                             // Mode 2 (fluency preferred): 4-way video resolution CIF, frame rate of 12fps, bit rate 256kbps (550MB per hour) 
                                 // Mode 3 (Custom) video resolution can be defined by the user, limited the maximum capacity of 4CIF/25fps
	char				szChannelName[DH_CHAN_NAME_LEN];
	DH_VIDEOENC_OPT		stMainVideoEncOpt[DH_REC_TYPE_NUM];
	DH_VIDEOENC_OPT		stAssiVideoEncOpt[DH_N_ENCODE_AUX];		
	DH_COLOR_CFG		stColorCfg[DH_N_COL_TSECT];
	DH_ENCODE_WIDGET	stTimeOSD;
	DH_ENCODE_WIDGET	stChannelOSD;
	DH_ENCODE_WIDGET	stBlindCover[DH_N_COVERS];	// Single privacy mask zone
	BYTE				byBlindEnable;				// Privacy mask zone enable button;0x00:disable privacy mask ,0x01:privacy mask local preview ,0x10:privacy mask record and network preview,0x11: Privacy mask all
	BYTE				byBlindMask;				// Privacy mask zone subnet mask. The first bit; device local preview ;The second bit :record (and network preview ) */
	BYTE				bVolume;					// volume(0~100)
	BYTE				bVolumeEnable;				// volume enable
} DHDEV_CHANNEL_CFG, *LPDHDEV_CHANNEL_CFG;

// Preview image property 
typedef struct 
{
	DWORD				dwSize;
	DH_VIDEOENC_OPT		stPreView;
	DH_COLOR_CFG		stColorCfg[DH_N_COL_TSECT];
}DHDEV_PREVIEW_CFG;

// snap control configuration
typedef struct _config_snap_control
{
	BYTE               bySnapState[32];           // every channel's snap on-off: 0 Autumn(other configuration and event will control whether snap picture ); 1: turn on snap; 2: turn off snap
    BYTE               byReserved[480];
}DHDEV_SNAP_CONTROL_CFG;

enum _gps_mode
{
	GPS_OR_GLONASS_MODE,	// GPS,GLONASS
	GLONASS_MODE,			// GLONASS
	GPS_MODE,				// GPS
};
// DH_DEV_GPS_MODE_CFG configuration
typedef struct tagDHDEV_GPS_MODE_CFG
{
	BYTE				byGPSMode; // GPS mode
	BYTE				byRev[255];
}DHDEV_GPS_MODE_CFG;

// snap upload DH_DEV_SNAP_UPLOAD_CFG configuration
typedef struct tagDHDEV_SNAP_UPLOAD_CFG
{
	int					nUploadInterval;		// the interval time of upload picture(s)
	BYTE				byReserved[252];	
}DHDEV_SNAP_UPLOAD_CFG;

// DH_DEV_SPEED_LIMIT_CFG configuration
typedef struct tagDHDEV_SPEED_LIMIT_CFG
{
	BYTE       byEnable;						// Enable speed limit;1:enable,0:disable
	BYTE       byReserved1;      
	WORD       wMaxSpeed;						// upper speed(KM/H) 0: none limit speed, >0:limit speed
	WORD       wMinSpeed;						// low speed(KM/H) 0: none limit speed, >0:limit speed
	BYTE       byReserved2[122];  
}DHDEV_SPEED_LIMIT_CFG;

// wireless routing config
typedef struct
{
	BOOL		bEnable;						// enable
	char		szSSID[36];						// SSID
	BOOL		bHideSSID;						// hide SSID
	char		szIP[DH_MAX_IPADDR_LEN];		// IP
	char		szSubMark[DH_MAX_IPADDR_LEN];	// sub mark
	char		szGateWay[DH_MAX_IPADDR_LEN];	// gateway
	char		szCountry[32];					// country
	int			nSafeType;						// safe type: 1-no encryption; 2-WEP; 2-WPA-PSK; 3-WPA2-PSK
	int			nEncryption;					// encryption: WEP(1-auto 2-open 3-shareable); WPA-PSK/WPA2-PSK(4-TKIP 5-AES)
	char		szKey[32];						// key
	int			nChannel;						// channel
	BOOL		bAutoChannelSelect;				// auto channel select
}DHDEV_WIRELESS_ROUTING_CFG;

//-------------------------------COM property ---------------------------------

// COM basic property 
typedef struct
{
	BYTE				byDataBit;				// Data bit;0:5,1:6,2:7,3:8
	BYTE				byStopBit;				// Stop bit;0:1 bit,1:1.5 bit,2:2 bits
	BYTE				byParity;				// Parity;0: None,1: Odd;2: even;3:sign;4:empty
	BYTE				byBaudRate;				// Baud rate;0:300,1:600,2:1200,3:2400,4:4800,
												// 5:9600,6:19200,7:38400,8:57600,9:115200
} DH_COMM_PROP;

// 485 decoder setup 
typedef struct
{ 
	DH_COMM_PROP		struComm;
	BYTE				wProtocol;				// Protocol type. Corresponding to the subscript of Protocol Name List
	BYTE				bPTZType;				// 0-Compatible,local ptz 1-remote network ptz, the capability refer to DEV_ENCODER_CFG.
	BYTE				wDecoderAddress;		// Decoder address;0 - 255
	BYTE 				byMartixID;				// Matrix number 
} DH_485_CFG;

// 232 COM setup 
typedef struct
{
	DH_COMM_PROP		struComm;
	BYTE				byFunction;				// COM function,Corresponding to the subscript of Function Name list. 
	BYTE				byReserved[3];
} DH_RS232_CFG;

// COM configuration structure 
typedef struct
{
	DWORD				dwSize;

	DWORD				dwDecProListNum;										// Decoder protocol amount
	char				DecProName[DH_MAX_DECPRO_LIST_SIZE][DH_MAX_NAME_LEN];	// Protocol name list
	DH_485_CFG			stDecoder[DH_MAX_DECODER_NUM];							// Each decoder current property

	DWORD				dw232FuncNameNum;										// 232 function amount 
	char				s232FuncName[DH_MAX_232FUNCS][DH_MAX_NAME_LEN];			// Function name list 
	DH_RS232_CFG		st232[DH_MAX_232_NUM];									// Current 232 COM property 
} DHDEV_COMM_CFG;

// Extended COM configuration structure 
typedef struct
{
	DWORD				dwSize;
	
	DWORD				dwDecProListNum;			                           // Decoder protocol amount
	char				DecProName[DH_MAX_DECPRO_LIST_SIZE][DH_MAX_NAME_LEN];  // Protocol name list
	DH_485_CFG			stDecoder[DH_MAX_DECODER_NUM];						   // Each decoder current property
	
	DWORD				dw232FuncNameNum;									   // 232 function amount
	char				s232FuncName[DH_MAX_232FUNCS][DH_MAX_NAME_LEN];	       // Function name list
	DWORD               dw232ComNum;										   // 232 com amount  
	DH_RS232_CFG		st232[DH_MAX_232_NUM_EX];	                           // Current 232 COM property
} DHDEV_COMM_CFG_EX;

// Serial port status
typedef struct
{
	unsigned int		uBeOpened;
	unsigned int		uBaudRate;
	unsigned int		uDataBites;
	unsigned int		uStopBits;
	unsigned int		uParity;
	BYTE				bReserved[32];
} DH_COMM_STATE;

//-------------------------------Record configuration---------------------------------

// Scheduled record 
typedef struct 
{
	DWORD				dwSize;
	DH_TSECT			stSect[DH_N_WEEKS][DH_N_REC_TSECT];
	BYTE				byPreRecordLen;			// Pre-record button. Unit is second. 0 means disable pre-record 
	BYTE				byRedundancyEn;			// Record redundant enable button 
	BYTE                byRecordType;           // Video bit-stream type:0:Main Stream 1:Assist Stream1 2:Assist Stream2 3:Assist Stream3
	BYTE				byReserved;
} DHDEV_RECORD_CFG, *LPDH_RECORD_CFG;

// NTP setup 
typedef struct  
{
	BOOL				bEnable;				// Enable or not
	int					nHostPort;				// NTP  server default port is 123
	char				szHostIp[32];			// Host IP
	char				szDomainName[128];		// Domain name 
	int					nType;					// Read only ,0:IP,1:domain name ,2:IP and domain name 
	int					nUpdateInterval;		// Update time(minute)
	int					nTimeZone;				// Please refer to DH_TIME_ZONE_TYPE
	char				reserved[128];
} DHDEV_NTP_CFG;

// FTP upload setup 
typedef struct
{
	struct
	{
		DH_TSECT		struSect;				// Enable function is disabled during the period. Can ignore
		BOOL			bMdEn;					// Upload motion detection record
		BOOL			bAlarmEn;				// Upload external alarm record 
		BOOL			bTimerEn;				// Upload scheduled record 
		DWORD			dwRev[4];
	} struPeriod[DH_TIME_SECTION];
} DH_FTP_UPLOAD_CFG;

typedef struct
{
	DWORD				dwSize;
	BOOL				bEnable;							// Enable or not
	char				szHostIp[DH_MAX_IPADDR_LEN];		// Host IP
	WORD				wHostPort;							// Host port 
	char				szDirName[DH_FTP_MAX_PATH];			// FTP path
	char				szUserName[DH_FTP_USERNAME_LEN];	// User name
	char				szPassword[DH_FTP_PASSWORD_LEN];	// Password 
	int					iFileLen;							// File length
	int					iInterval;							// Time interval between two near files.
	DH_FTP_UPLOAD_CFG	struUploadCfg[DH_MAX_CHANNUM][DH_N_WEEKS];
	char 				protocol;							// 0-FTP 1-SMB 2-NFS,3-ISCSI
	char				NASVer;								// Network storage server version 0=Old FTP(There is time period in the user interface),1=NAS storage(There is no time period in the user interface. )
	DWORD				dwFunctionMask;						// Function capacity mask. Use bit to represent. Lower 16-bit:FTP,SMB,NFS, ISCSI and higher 16-bit:(Local storage)DISK,U
	BYTE                bDataType;                          // data type, 0-all, 1-video, 2-image
	BYTE 				reserved[123];
} DHDEV_FTP_PROTO_CFG;

// extend FTP upload setup (support setting the FTP's path, maximal picture number)
typedef struct 
{
    int            nMaxPictures;                             // the maximal number of the FTP can save,can set limit of the picture number in the every chennel's direct;
	                                                         // if the saved picture number extended the limit the old picture will be covered by new one; the value 0 indecate that it will save picture untill the disck was full.
    char           szPreChannelName[DH_FTP_MAX_SUB_PATH];    // the picture named mod
    char           szFTPChannelPath[DH_FTP_MAX_SUB_PATH];    // the FTP's sub direction
    char           szRev[128];                               // reserved
}DHDEV_FTP_CHANNEL_CFG; 


typedef struct 
{ 
	DHDEV_FTP_PROTO_CFG       stuFtpNormalSet;                	// the normal function of the FTP
	DHDEV_FTP_CHANNEL_CFG     stuFtpChannelSet[DH_MAX_CHANNUM]; // the extended function of the FTP
	char                      szRev[128];                    	// reserved
}DHDEV_FTP_PROTO_CFG_EX;

//-------------------------------Network Configuration---------------------------------

// Ethernet Configuration
typedef struct 
{
	char				sDevIPAddr[DH_MAX_IPADDR_LEN];	// DVR IP  address
	char				sDevIPMask[DH_MAX_IPADDR_LEN];	// DVR IP subnet mask    
	char				sGatewayIP[DH_MAX_IPADDR_LEN];	// Gateway address

	/*
	 * 1:10Mbps full-duplex
	 * 2:10Mbps auto-duplex
	 * 3:10Mbps half-duplex
	 * 4:100Mbps full-duplex
	 * 5:100Mbps auto-duplex
	 * 6:100Mbps half-duplex
	 * 7:auto
	 */
	// Divide DWORD into four to future development
	BYTE				dwNetInterface;			// NSP
	BYTE				bTranMedia;				// 0:cable,1:wireless
	BYTE				bValid;					// Use bit to represent, The first bit:1:valid 0:invalid;The second bit:0:Disable DHCP 1:Enable DHCP;The third bit:0:Do not support DHCP 1:Support DHCP
	BYTE				bDefaultEth;			// To be the default network card or not.  1: default 0:non-default 
	char				byMACAddr[DH_MACADDR_LEN];	// MAC address , read-only., 
} DH_ETHERNET; 

// Remote host setup 
typedef struct 
{
	BYTE				byEnable;				// Enable connection 
	BYTE				byAssistant;            // Only for PPPoE server,0:on the cabled network; 1:on the wireless network
	WORD				wHostPort;				// Remote host Port 
	char				sHostIPAddr[DH_MAX_IPADDR_LEN];		// Remote host IP address  				
	char				sHostUser[DH_MAX_HOST_NAMELEN];		// Remote host User name   
	char				sHostPassword[DH_MAX_HOST_PSWLEN];	// Remote host Password   
} DH_REMOTE_HOST;

// Mail setup 
typedef struct 
{
	char				sMailIPAddr[DH_MAX_IPADDR_LEN];		// Email server IP
	WORD				wMailPort;							// Email server port 
	WORD				wReserved;							// Reserved
	char				sSenderAddr[DH_MAX_MAIL_ADDR_LEN];	// Send out address 
	char				sUserName[DH_MAX_NAME_LEN];			// User name 
	char				sUserPsw[DH_MAX_NAME_LEN];			// User password 
	char				sDestAddr[DH_MAX_MAIL_ADDR_LEN];	// Destination address 
	char				sCcAddr[DH_MAX_MAIL_ADDR_LEN];		// CC address 
	char				sBccAddr[DH_MAX_MAIL_ADDR_LEN];		// BCC address 
	char				sSubject[DH_MAX_MAIL_SUBJECT_LEN];	// Subject 
} DH_MAIL_CFG;

// Network configuration structure 
typedef struct
{ 
	DWORD				dwSize; 
	char				sDevName[DH_MAX_NAME_LEN];	// Device host name
	WORD				wTcpMaxConnectNum;		// TCP max connection amount
	WORD				wTcpPort;				// TCP listening port 
	WORD				wUdpPort;				// UDP listening port 
	WORD				wHttpPort;				// HTTP port number 
	WORD				wHttpsPort;				// HTTPS port number 
	WORD				wSslPort;				// SSL port number 
	DH_ETHERNET			stEtherNet[DH_MAX_ETHERNET_NUM];	// Ethernet port 

	DH_REMOTE_HOST		struAlarmHost;			// Alarm server 
	DH_REMOTE_HOST		struLogHost;			// Log server 
	DH_REMOTE_HOST		struSmtpHost;			// SMTP server 
	DH_REMOTE_HOST		struMultiCast;			// Multiple-cast group 
	DH_REMOTE_HOST		struNfs;				// NFS server
	DH_REMOTE_HOST		struPppoe;				// PPPoE server 
	char				sPppoeIP[DH_MAX_IPADDR_LEN]; // returned IP after PPPoE registration 
	DH_REMOTE_HOST		struDdns;				// DDNS server
	char				sDdnsHostName[DH_MAX_HOST_NAMELEN];	// DDNS host name
	DH_REMOTE_HOST		struDns;				// DNS server 
	DH_MAIL_CFG			struMail;				// Email setup 
} DHDEV_NET_CFG;

// Ethernet extended Configuration
typedef struct 
{
	char				sDevIPAddr[DH_MAX_IPADDR_LEN];	// DVR IP  address
	char				sDevIPMask[DH_MAX_IPADDR_LEN];	// DVR IP subnet mask
	char				sGatewayIP[DH_MAX_IPADDR_LEN];	// Gateway address

	/*
	 * 1:10Mbps full-duplex
	 * 2:10Mbps auto-duplex
	 * 3:10Mbps half-duplex
	 * 4:100Mbps full-duplex
	 * 5:100Mbps auto-duplex
	 * 6:100Mbps half-duplex
	 * 7:auto
	 */
	// Divide DWORD into four to future development
	BYTE				dwNetInterface;			// NSP
	BYTE				bTranMedia;				// 0:cable,1:wireless
	BYTE				bValid;					// Use bit to represent, The first bit:1:valid 0:invalid;The second bit:0:Disable DHCP 1:Enable DHCP;The third bit:0:Do not support DHCP 1:Support DHCP
	BYTE				bDefaultEth;			//To be the default network card or not.  1: default 0:non-default 
	char				byMACAddr[DH_MACADDR_LEN];	// MAC address , read-only.
	BYTE                bMode;                  // mode, 1:balance, 2:multi, 3:fault-toerant
	BYTE                bReserved1[3];          // reserved   
    char                szEthernetName[DH_MAX_NAME_LEN];    // Card name, read only
    BYTE                bReserved[12];                      // reserved    
} DH_ETHERNET_EX; 

// Network extended configuration structure
typedef struct
{ 
	DWORD				dwSize; 
	char				sDevName[DH_MAX_NAME_LEN];	// Device host name
	WORD				wTcpMaxConnectNum;		// TCP max connection amount
	WORD				wTcpPort;				// TCP listening port
	WORD				wUdpPort;				// UDP listening port 
	WORD				wHttpPort;				// HTTP port number
	WORD				wHttpsPort;				// HTTPS port number
	WORD				wSslPort;				// SSL port number
	int                 nEtherNetNum;           // Ethernet port number
	DH_ETHERNET_EX		stEtherNet[DH_MAX_ETHERNET_NUM_EX];	// Ethernet info
	
	DH_REMOTE_HOST		struAlarmHost;			// Alarm server
	DH_REMOTE_HOST		struLogHost;			// Log server
	DH_REMOTE_HOST		struSmtpHost;			// SMTP server
	DH_REMOTE_HOST		struMultiCast;			// Multiple-cast group
	DH_REMOTE_HOST		struNfs;				// NFS server
	DH_REMOTE_HOST		struPppoe;				// PPPoE server
	char				sPppoeIP[DH_MAX_IPADDR_LEN]; // returned IP after PPPoE registration
	DH_REMOTE_HOST		struDdns;				// DDNS server
	char				sDdnsHostName[DH_MAX_HOST_NAMELEN];	// DDNS host name
	DH_REMOTE_HOST		struDns;				// DNS server
	DH_MAIL_CFG			struMail;				// Email setup
	BYTE                bReserved[128];         // reserved
} DHDEV_NET_CFG_EX;

// ddns configuraiton structure 
typedef struct
{
	DWORD				dwId;					// ddns server id
	BOOL				bEnable;				// Enable. There is only one valid ddns server at one time.
	char				szServerType[DH_MAX_SERVER_TYPE_LEN];	// Server type. www.3322.org.
	char				szServerIp[DH_MAX_DOMAIN_NAME_LEN];		// Server IP or domain name 
	DWORD				dwServerPort;			// Server port 
	char				szDomainName[DH_MAX_DOMAIN_NAME_LEN];	// DVR domain name such as jeckean.3322.org
	char				szUserName[DH_MAX_HOST_NAMELEN];		// User name
	char				szUserPsw[DH_MAX_HOST_PSWLEN];			// Password
	char				szAlias[DH_MAX_DDNS_ALIAS_LEN];			// Server alias 
	DWORD				dwAlivePeriod;							// DDNS alive period
	BYTE				ByMode;									// domain mode:0-manual,szDomainName enable; 1-default szDefaultDomainName enable
	char				szDefaultDomainName[DH_MAX_DEFAULT_DOMAIN_LEN];// default domain, real only
    BYTE                bReserved[3];                           // Text align
    char                szEmailUserName[DH_MAX_MAIL_NAME_LEN];	// Mail username
    char                reserved[128];
} DH_DDNS_SERVER_CFG;

typedef struct
{
	DWORD				dwSize;
	DWORD				dwDdnsServerNum;	
	DH_DDNS_SERVER_CFG	struDdnsServer[DH_MAX_DDNS_NUM];	
} DHDEV_MULTI_DDNS_CFG;

// Mail configuration structure 
typedef struct 
{
	char				sMailIPAddr[DH_MAX_DOMAIN_NAME_LEN];	// Mail server address(IP and domain name )
	char				sSubMailIPAddr[DH_MAX_DOMAIN_NAME_LEN];
	WORD				wMailPort;								// Mail server port 
	WORD				wSubMailPort;
	WORD				wReserved;								// Reserved 
	char				sSenderAddr[DH_MAX_MAIL_ADDR_LEN];		// From
	char				sUserName[DH_MAX_MAIL_NAME_LEN];		// User name
	char				sUserPsw[DH_MAX_MAIL_NAME_LEN];			// password
	char				sDestAddr[DH_MAX_MAIL_ADDR_LEN];		// To
	char				sCcAddr[DH_MAX_MAIL_ADDR_LEN];			// CC
	char				sBccAddr[DH_MAX_MAIL_ADDR_LEN];			// BCC
	char				sSubject[DH_MAX_MAIL_SUBJECT_LEN];		// Subject
	BYTE				bEnable;								// Enable 0:false,	1:true
	BYTE				bSSLEnable;								// SSL enable
	WORD				wSendInterval;							// Send interval,[0,3600]s
	BYTE				bAnonymous;								// Anonymous Options[0,1], 0:FALSE,1:TRUE.
	BYTE				bAttachEnable;							// Attach enable[0,1], 0:FALSE,1:TRUE.
	char				reserved[154];
} DHDEV_MAIL_CFG;

// DNS server setup 
typedef struct  
{
	char				szPrimaryIp[DH_MAX_IPADDR_LEN];
	char				szSecondaryIp[DH_MAX_IPADDR_LEN];
	char				reserved[256];
} DHDEV_DNS_CFG;

// Record download strategy setup 
typedef struct
{
	DWORD				dwSize;
	BOOL				bEnable;				// TRUE: high-speed download,FALSE:general download  
}DHDEV_DOWNLOAD_STRATEGY_CFG;

// Network transmission strategy setup 
typedef struct
{
	DWORD				dwSize;
	BOOL				bEnable;
	int					iStrategy;				// 0: video quality,1:fluency ,2: auto
}DHDEV_TRANSFER_STRATEGY_CFG;

// The corresponding parameter when setting log in
typedef struct  
{
	int					nWaittime;				// Waiting time(unit is ms), 0:default 5000ms.
	int					nConnectTime;			// Connection timeout value(Unit is ms), 0:default 1500ms.
	int					nConnectTryNum;			// Connection trial times, 0:default 1.
	int					nSubConnectSpaceTime;	// Sub-connection waiting time(Unit is ms), 0:default 10ms.
	int					nGetDevInfoTime;		// Access to device information timeout, 0:default 1000ms.
	int					nConnectBufSize;		// Each connected to receive data buffer size(Bytes), 0:default 250*1024
	int					nGetConnInfoTime;		// Access to sub-connect information timeout(Unit is ms), 0:default 1000ms.
	int                 nSearchRecordTime;      // Timeout value of search video (unit ms), default 3000ms
	int                 nsubDisconnetTime;      // dislink disconnect time,0:default 60000ms
	BYTE				byNetType;				// net type, 0-LAN, 1-WAN
	BYTE                byPlaybackBufSize;      // playback data from the receive buffer size(m),when value = 0,default 4M
    BYTE                bDetectDisconnTime;     // Pulse detect offline time(second) .When it is 0, the default setup is 60s, and the min time is 2s 
    BYTE                bKeepLifeInterval;      // Pulse send out interval(second). When it is 0, the default setup is 10s, the min internal is 2s. 
	int                 nPicBufSize;            // actual pictures of the receive buffer size(byte)when value = 0,default 2*1024*1024
	BYTE				bReserved[4];			// reserved
} NET_PARAM;

// Corresponding to CLIENT_SearchDevices
typedef struct 
{
	char				szIP[DH_MAX_IPADDR_LEN];			// IP
	int					nPort;								// Port
	char				szSubmask[DH_MAX_IPADDR_LEN];		// Subnet mask
	char				szGateway[DH_MAX_IPADDR_LEN];		// Gateway
	char				szMac[DH_MACADDR_LEN];				// MAC address
	char				szDeviceType[DH_DEV_TYPE_LEN];		// Device type
	BYTE        		byManuFactory;				    	// manufactory
	BYTE        		byIPVersion;                    	// 4: IPv4, szXXX is in format with dot; 6:IPv6, szXXX is 128-bit *16 bytes) numerical format
	BYTE				byInitStatus;						// init status
															// bit0~10-old device, can not be init; 1-not init; 2-already init
															// bit2~30-old device,reserved; 1-connect to public network disable; 2-connect to public network enable
															// bit4~50-old device,reserved; 1-connect to cellphone disable; 2-connect to cellphone enable
	BYTE				byPwdResetWay;						// the way supported for reset password:make sense when the device is init														
															// bit0-support reset password by cellphone; bit1-support reset password by mail; bit2-support reset password by XML file
	BYTE				bySpecialAbility;					// special ability of device ,bit0-2D Code:0 support  1 no support, bit1-PN:0 support  1 no support
    BYTE                bReserved[27];                      // reserved
} DEVICE_NET_INFO;

// Corresponding to CLIENT_StartSearchDevices
typedef struct 
{
	int                 iIPVersion;                      // 4 for IPV4, 6 for IPV6
	char				szIP[64];		                 // IP IPV4 like"192.168.0.1" IPV6 like "2008::1/64"
	int				    nPort;		                     // port
	char				szSubmask[64];	                 // Subnet mask(IPV6 do not have subnet mask)
	char				szGateway[64];	                 // Gateway
	char				szMac[DH_MACADDR_LEN];           // MAC address
	char				szDeviceType[DH_DEV_TYPE_LEN];	 // Device type
	BYTE                byManuFactory;				     // device manufactory, see EM_IPC_TYPE
	BYTE                byDefinition;                    // 1-Standard definition 2-High definition
	bool                bDhcpEn;                         // Dhcp, true-open, false-close
	BYTE                byReserved1;                     // reserved
	char                verifyData[88];                  // ECC data 
	char                szSerialNo[DH_DEV_SERIALNO_LEN]; // serial no
	char                szDevSoftVersion[DH_MAX_URL_LEN];// soft version    
    char                szDetailType[DH_DEV_TYPE_LEN];   // device detail type
	char                szVendor[DH_MAX_STRING_LEN];     // OEM type
	char                szDevName[DH_MACHINE_NAME_NUM];  // device name
	char                szUserName[DH_USER_NAME_LENGTH_EX];  // user name for log in device(it need be filled when modify device ip)
	char                szPassWord[DH_USER_NAME_LENGTH_EX];  // pass word for log in device(it need be filled when modify device ip)
	unsigned short		nHttpPort;							 // HTTP server port
	WORD                wVideoInputCh;                       // count of video input channel
	WORD                wRemoteVideoInputCh;                 // count of remote video input
	WORD                wVideoOutputCh;                      // count of video output channel 
	WORD                wAlarmInputCh;                       // count of alarm input
	WORD                wAlarmOutputCh;                      // count of alarm output
    BOOL                bNewWordLen;                         // TRUE:szNewPassWord Enable
    char                szNewPassWord[DH_COMMON_STRING_64];  // pass word for log in device(it need be filled when modify device ip)
    BYTE				byInitStatus;						 // init status
															// bit0~10-old device, can not be init; 1-not init; 2-already init
															// bit2~30-old device,reserved; 1-connect to public network disable; 2-connect to public network enable
															// bit4~50-old device,reserved; 1-connect to cellphone disable; 2-connect to cellphone enable
	BYTE				byPwdResetWay;						// the way supported for reset password:make sense when the device is init														
															// bit0-support reset password by cellphone; bit1-support reset password by mail; bit2-support reset password by XML file															
	BYTE				bySpecialAbility;					// special ability of device ,high eight bit, bit0-2D Code:0 support  1 no support, bit1-PN:0 support  1 no support
    char                szNewDetailType[DH_COMMON_STRING_64];// device detail type 
	BOOL				bNewUserName;						// TRUE:szNewUserName enable
	char				szNewUserName[DH_COMMON_STRING_64];	// new user name for login device(it need be filled when modify device ip)
	char                cReserved[41];						// reserved
}DEVICE_NET_INFO_EX;

// Corresponding to CLIENT_SearchDevicesByIPs
typedef struct
{
	DWORD               dwSize;                          // struct size 
	int                 nIpNum;                          // the IPs number for search
	char                szIP[DH_MAX_SAERCH_IP_NUM][64];  // the IPs for search
}DEVICE_IP_SEARCH_INFO;

// struct SNMP config struct
typedef struct tagNET_DEVICE_SEARCH_PARAM
{
    DWORD       dwSize;
    BOOL        bUseDefault;
    WORD        wBroadcastLocalPort;
    WORD        wBroadcastRemotePort;
    WORD        wMulticastRemotePort;
}NET_DEVICE_SEARCH_PARAM;
typedef struct
{
	BOOL                bEnable;                                // SNMP enable
	int                 iSNMPPort;                              // SNMP port
	char                szReadCommon[DH_MAX_SNMP_COMMON_LEN];   // read common
	char                szWriteCommon[DH_MAX_SNMP_COMMON_LEN];  // write common
	char                szTrapServer[64];                       // trap address
	int                 iTrapPort;                              // trap port
	BYTE				bSNMPV1;								// SNMP V1 enable
	BYTE				bSNMPV2;								// SNMP V2 enable
	BYTE				bSNMPV3;								// SNMP V3 enable
	char                szReserve[125];
}DHDEV_NET_SNMP_CFG;

// ISCSI server
typedef struct
{
	char				szServerName[32];				// name
	union
	{ 
		BYTE	c[4];
		WORD	s[2];
		DWORD	l;
	}					stuIP;							// IP
	int					nPort;							// port
	char				szUserName[32];					// username
	char				szPassword[32];					// password
	BOOL				bAnonymous;						// anonymous
}DHDEV_ISCSI_SERVER;

// ISCSI config
typedef struct
{
	BOOL				bEnable;						// enable
	DHDEV_ISCSI_SERVER	stuServer;						// server
	char				szRemotePath[240];				// remote path
	BYTE				reserved[256];
}DHDEV_ISCSI_CFG;

//-------------------------------Alarm Property ---------------------------------

// PTZ Activation
typedef struct 
{
	int					iType;                          // 0-None,1-Preset,2-Tour,3-Pattern
	int					iValue;
} DH_PTZ_LINK, *LPDH_PTZ_LINK;

//Alarm activation structure 
typedef struct 
{
	/* Message process way. There can be several process ways.
	 * 0x00000001 -  Alarm upload
	 * 0x00000002 -  Activation alarm 
	 * 0x00000004 -  PTZ activation
	 * 0x00000008 -  Send out mail
	 * 0x00000010 -  Local tour
	 * 0x00000020 -  Local prompt
	 * 0x00000040 -  Alarm output
	 * 0x00000080 - Ftp upload
	 * 0x00000100 -  Buzzer 
	 * 0x00000200 -  Video prompt
	 * 0x00000400 -  Snapshot
	*/

	/* The process way current alarm supported. Use bit mask to represent */
	DWORD				dwActionMask;

	/* Use bit mask to represent. The parameters of detailed operation are in its correspond configuration */
	DWORD				dwActionFlag;
	
	/* The output channel the alarm activated. The output alarm activated. 1 means activate current output. */ 
	BYTE				byRelAlarmOut[DH_MAX_ALARMOUT_NUM];
	DWORD				dwDuration;				/*  Alarm duration time*/

	/* Activation record */
	BYTE				byRecordChannel[DH_MAX_VIDEO_IN_NUM]; /*  The record channel alarm activated. 1 means activate current channel. */
	DWORD				dwRecLatch;				/*  Record duration time */

	/* Snapshot channel  */
	BYTE				bySnap[DH_MAX_VIDEO_IN_NUM];
	/* Tour channel */
	BYTE				byTour[DH_MAX_VIDEO_IN_NUM];

	/* PTZ activation  */
	DH_PTZ_LINK			struPtzLink[DH_MAX_VIDEO_IN_NUM];
	DWORD				dwEventLatch;			/* The latch time after activation begins. Unit is second. The value ranges from 0 to 15. Default value is 0. */
	/* The wireless output channel alarm activated.The output alarm activated. 1 means activate current output.*/ 
	BYTE				byRelWIAlarmOut[DH_MAX_ALARMOUT_NUM];
	BYTE				bMessageToNet;
	BYTE                bMMSEn;					/* SMS Alarm enabled */
	BYTE                bySnapshotTimes;		/* the number of sheets of drawings */
	BYTE				bMatrixEn;				/* enable matrix */
	DWORD				dwMatrix;				/* matrix mask */			
	BYTE				bLog;					/* enable log */
	BYTE				bSnapshotPeriod;		/* <Snapshot frame interval. System can snapshot regularly at the interval you specify here. The snapshot amount in a period of time also has relationship with the snapshot frame rate. 0 means there is no interval, system just snapshot continuously. */
	BYTE                byEmailType;            /* <0,picture,1,record> */
	BYTE                byEmailMaxLength;       /* <max record length,unit:MB> */
	BYTE                byEmailMaxTime;         /* <max time length, unit:second> */
	BYTE				byReserved[99];     
} DH_MSG_HANDLE;

// External alarm 
typedef struct
{
	BYTE				byAlarmType;			// Annunciator type,0: normal close,1:normal open 
	BYTE				byAlarmEn;				// Enable alarm 
	BYTE				byReserved[2];						
	DH_TSECT			stSect[DH_N_WEEKS][DH_N_REC_TSECT]; // NSP
	DH_MSG_HANDLE		struHandle;				// Process way 
} DH_ALARMIN_CFG, *LPDHDEV_ALARMIN_CFG; 

//Motion detection alarm 
typedef struct 
{
	BYTE				byMotionEn;				// Enable motion detection alarm 
	BYTE				byReserved;
	WORD				wSenseLevel;			// Sensitivity 
	WORD				wMotionRow;				// Row amount in motion detection zones
	WORD				wMotionCol;				// Column amount in motion detection zones 
	BYTE				byDetected[DH_MOTION_ROW][DH_MOTION_COL]; // Detection zones .Max 32*32 zones.
	DH_TSECT			stSect[DH_N_WEEKS][DH_N_REC_TSECT];	// NSP
	DH_MSG_HANDLE		struHandle;				// Process way 
} DH_MOTION_DETECT_CFG;

// Video loss alarm 
typedef struct
{
	BYTE				byAlarmEn;				// Enable video loss alarm 
	BYTE				byReserved[3];
	DH_TSECT			stSect[DH_N_WEEKS][DH_N_REC_TSECT];	// NSP
	DH_MSG_HANDLE		struHandle;				// Process way
} DH_VIDEO_LOST_CFG;

// Camera masking alarm 
typedef struct
{
	BYTE				byBlindEnable;						// Enable
	BYTE				byBlindLevel;						// Sensitivity 1-6
	BYTE				byReserved[2];
	DH_TSECT			stSect[DH_N_WEEKS][DH_N_REC_TSECT];	// NSP
	DH_MSG_HANDLE		struHandle;							// Process way 
} DH_BLIND_CFG;

// HDD information(External alarm )
typedef struct 
{
	BYTE				byNoDiskEn;								// Alarm when there is no HDD
	BYTE				byReserved_1[3];
	DH_TSECT			stNDSect[DH_N_WEEKS][DH_N_REC_TSECT];	// NSP
	DH_MSG_HANDLE		struNDHandle;							// Process way 

	BYTE				byLowCapEn;								// Alarm when HDD capacity is low 
	BYTE				byLowerLimit;							// Capacity threshold 0-99
	BYTE				byReserved_2[2];
	DH_TSECT			stLCSect[DH_N_WEEKS][DH_N_REC_TSECT];	// NSP
	DH_MSG_HANDLE		struLCHandle;							// Process way 

	BYTE				byDiskErrEn;							// HDD malfunction alarm 
	BYTE				bDiskNum;
	BYTE				byReserved_3[2];
	DH_TSECT			stEDSect[DH_N_WEEKS][DH_N_REC_TSECT];	// NSP
	DH_MSG_HANDLE		struEDHandle;							// Process way 
} DH_DISK_ALARM_CFG;

typedef struct
{
	BYTE				byEnable;
	BYTE				byReserved[3];
	DH_MSG_HANDLE		struHandle;
} DH_NETBROKEN_ALARM_CFG;

// Alarm deployment 
typedef struct
{
	DWORD dwSize;
	DH_ALARMIN_CFG 		struLocalAlmIn[DH_MAX_ALARM_IN_NUM];
	DH_ALARMIN_CFG		struNetAlmIn[DH_MAX_ALARM_IN_NUM];
	DH_MOTION_DETECT_CFG struMotion[DH_MAX_VIDEO_IN_NUM];
	DH_VIDEO_LOST_CFG	struVideoLost[DH_MAX_VIDEO_IN_NUM];
	DH_BLIND_CFG		struBlind[DH_MAX_VIDEO_IN_NUM];
	DH_DISK_ALARM_CFG	struDiskAlarm;
	DH_NETBROKEN_ALARM_CFG	struNetBrokenAlarm;
} DHDEV_ALARM_SCHEDULE;

#define DECODER_OUT_SLOTS_MAX_NUM 		16
#define DECODER_IN_SLOTS_MAX_NUM 		16

// Alarm decoder configuration
typedef struct  
{
	DWORD				dwAddr;									// Alarm decoder address
	BOOL				bEnable;								// Alarm decoder enable
	DWORD				dwOutSlots[DECODER_OUT_SLOTS_MAX_NUM];	// Now only support 8
	int					nOutSlotNum;							// Effective number of array elements.
	DH_TSECT			stSect[DH_N_WEEKS][DH_N_REC_TSECT];
	DH_MSG_HANDLE		struHandle[DECODER_IN_SLOTS_MAX_NUM];	// Now only support 8
	int					nMsgHandleNum;							// Effective number of array elements.
	BYTE				bReserved[120];
} DH_ALARMDEC_CFG;

// The setup of alarm upload
typedef struct  
{
	BYTE				byEnable;						// Enable upload
	BYTE				bReserverd;						// Reserved
	WORD				wHostPort;						// Alarm center listening port 
	char				sHostIPAddr[DH_MAX_IPADDR_LEN];	// Alarm center IP
	int					nByTimeEn;						// Enable scheduled upload.Use it to upload IP or domain name to the centre.
	int					nUploadDay;						/*  Set upload date 
														"Never = 0", "Everyday = 1", "Sunday = 2", 
														"Monday = 3", Tuesday = 4", "Wednesday = 5",
														"Thursday = 6", "Friday = 7", "Saturday = 8"*/	
	int					nUploadHour;					// Set upload time ,[0~23]o'clock
	DWORD				dwReserved[300]; 				// Reserved for future development 
} ALARMCENTER_UP_CFG;

// Panorama switch alarm configuration
typedef struct __DH_PANORAMA_SWITCH_CFG 
{
	BOOL				bEnable;				// Enabled
	int					nReserved[5];			// Reserved
	DH_TSECT			stSect[DH_N_WEEKS][DH_N_REC_TSECT];
	DH_MSG_HANDLE		struHandle;				// Process way
} DH_PANORAMA_SWITCH_CFG;

typedef struct __ALARM_PANORAMA_SWITCH_CFG 
{
	int					nAlarmChnNum;			// Number of alarm channels
	DH_PANORAMA_SWITCH_CFG stuPanoramaSwitch[DH_MAX_VIDEO_IN_NUM];
} ALARM_PANORAMA_SWITCH_CFG;

// Lose focus alarm configuration
typedef struct __DH_LOST_FOCUS_CFG
{
	BOOL				bEnable;				// Enabled
	int					nReserved[5];			// Reserved
	DH_TSECT			stSect[DH_N_WEEKS][DH_N_REC_TSECT];
	DH_MSG_HANDLE		struHandle;				// Process way
} DH_LOST_FOCUS_CFG;

typedef struct __ALARM_LOST_FOCUS_CFG 
{
	int					nAlarmChnNum;			// Number of alarm channels
	DH_LOST_FOCUS_CFG stuLostFocus[DH_MAX_VIDEO_IN_NUM];
} ALARM_LOST_FOCUS_CFG;

// IP collision detection alarm configuration
typedef struct __ALARM_IP_COLLISION_CFG
{
	BOOL				bEnable;				// Enable
	DH_MSG_HANDLE		struHandle;				// Alarm activation
	int                 nReserved[300];			// Reserved
} ALARM_IP_COLLISION_CFG;

// MACcollision detection configuration
typedef struct __ALARM_MAC_COLLISION_CFG
{
	BOOL				bEnable;				// Enable
	DH_MSG_HANDLE		struHandle;				// Alarm activation
	int                 nReserved[300];			// Reserved
} ALARM_MAC_COLLISION_CFG;

// 232/485 com card signal info configuration
typedef struct __COM_CARD_SIGNAL_INFO
{
	WORD                wCardStartPose;          // card number start location
	WORD                wCardLenth;              // card number length
	char                cStartCharacter[32];     // start string
	char                cEndCharacter[32];       // end string
	BYTE                byReserved[28];          // reserved
}COM_CARD_SIGNAL_INFO;

// 232/485 com card  linked configuration(when the info from the com fit the configuration, it will trigger the device snap picture)
typedef struct __COM_CARD_SIGNAL_LINK_CFG
{
	COM_CARD_SIGNAL_INFO  struCardInfo;          // card info
	DH_MSG_HANDLE         struHandle;            // event link
	BYTE                  byReserved[24];        // reserved
	
}COM_CARD_SIGNAL_LINK_CFG;

//------------------------------Multiple privacy mask zones--------------------------------

// Privacy mask information
typedef struct __VIDEO_COVER_ATTR
{
	DH_RECT				rcBlock;				// Privacy mask zone coordinates
	int					nColor;					// Privacy mask color
	BYTE				bBlockType;				// Mask type;0:Black block,1: mosaic
	BYTE				bEncode;				// Encode level cover mask;1:valid,0:invalid
	BYTE				bPriview;				// Preview mask; 1:valid,0:invalid
	char				reserved[29];			// Reserved 
} VIDEO_COVER_ATTR;

// Multiple privacy mask zones 
typedef struct __DHDEV_VIDEOCOVER_CFG 
{
	DWORD				dwSize;
	char				szChannelName[DH_CHAN_NAME_LEN]; // Read-only
	BYTE				bTotalBlocks;			// The mask zones supported
	BYTE				bCoverCount;			// The mask zones have been set 
	VIDEO_COVER_ATTR	CoverBlock[DH_MAX_VIDEO_COVER_NUM]; // The mask zones 
	char				reserved[30];			// Reserved 
}DHDEV_VIDEOCOVER_CFG;

// decode policy configuration
typedef struct __DHDEV_DECODEPOLICY_CFG 
{
	int					nMinTime;				// (read only):minimal delay time(ms)
	int					nMaxTime;				// (read only):maximal delay time(ms)
	int					nDeocdeBufTime;			// device decode delay time(ms)
	
	char				reserved[128];			// reserved
}DHDEV_DECODEPOLICY_CFG;

// Device relative configuration
typedef struct __DHDEV_MACHINE_CFG 
{
	char				szMachineName[DH_MACHINE_NAME_NUM];		// Device name or SN 
	char				szMachineAddress[DH_MACHINE_NAME_NUM];	// Device location
	char				reserved[128];							// reserved
}DHDEV_MACHINE_CFG;


////////////////////////////////IPC series ////////////////////////////////

// Set wireless network information 
typedef struct 
{
	int					nEnable;				// Enable wireless
	char				szSSID[36];				// SSID
	int					nLinkMode;				// connection mode;0:auto,1:adhoc,2:Infrastructure
	int					nEncryption;			// encrypt;0:off,2:WEP64bit,3:WEP128bit, 4:WPA-PSK-TKIP, 5: WPA-PSK-CCMP
	int					nKeyType;				// 0:Hex,1:ASCII
    int					nKeyID;					// Serial number
	union
	{
		char			szKeys[4][32];			// Four group passwords
		char			szWPAKeys[128];			// when nEncryption is 4 or 5, use szWPAKeys
	};
	int					nKeyFlag;
	BYTE                byConnectedFlag;        // 0: not connect, 1: connect 
	char				reserved[11];
} DHDEV_WLAN_INFO;

// Select to use one wireless device 
typedef struct  
{
	char				szSSID[36];
	int					nLinkMode;				// Connection mode;0:adhoc,1:Infrastructure
	int 				nEncryption;			// Encrypt;0:off,2:WEP64bit,3:WEP128bit
	char				reserved[48];
} DHDEV_WLAN_DEVICE;

// The searched wireless device list 
typedef struct  
{
	DWORD				dwSize;
	BYTE				bWlanDevCount;			// The wireless devices searched 
	DHDEV_WLAN_DEVICE	lstWlanDev[DH_MAX_WLANDEVICE_NUM];
	char				reserved[255];
} DHDEV_WLAN_DEVICE_LIST;

// wireless device expended configuration
typedef struct
{
	char                szSSID[36];                         // server id
	char                szMacAddr[18];                      // mac address 
	BYTE                byApConnected;                      // connect state 0: not connect,1: connected
	BYTE                byLinkMode;                         // connect mod 0:adhoc 1:Infrastructure;
	int                 nRSSIQuality;                       // rssi quality(dbm)
	unsigned int        unApMaxBitRate;                     // max transmit speed
	BYTE                byAuthMode;                         // attestation mod:0:OPEN;1:SHARED;2:WPA;3:WPA-PSK;4:WPA2;5:WPA2-PSK;
															// 6:WPA-NONE(only use in Adhoc mode),
															// 7-11 are mix mode,choose one of them can to be connected 
															// 7:WPA-PSK | WPA2-PSK; 8:WPA | WPA2; 9:WPA | WPA-PSK;
															// 10:WPA2 | WPA2-PSK; 11:WPA | WPA-PSK |WPA2 |WPA2-PSK //12: UnKnown
	BYTE                byEncrAlgr;                         // encrypt mod 0:off; 2:WEP64bit; 3:WEP128bit; 4:WEP; 5:TKIP; 6:AES(CCMP)
															// 7: TKIP+AES( mix Mode) 8: UnKnown
	BYTE                byLinkQuality;                      // link quality 0~100(%)
    BYTE                byReserved[129];                    // Reserved 
}DHDEV_WLAN_DEVICE_EX;

// The searched wireless device expended configuration list 
typedef struct  
{
	DWORD				dwSize;
	BYTE				bWlanDevCount;			// The wireless device number searched
	DHDEV_WLAN_DEVICE_EX  lstWlanDev[DH_MAX_WLANDEVICE_NUM_EX];
	char				reserved[255];
} DHDEV_WLAN_DEVICE_LIST_EX;

// Function Test
typedef struct
{
	int                 nResult;                  // 0:success,1:failure
	BYTE                reserved[32];
} DHDEV_FUNC_TEST;

// FTP server info
typedef struct
{
	char                szServerName[32];          // server name
	char                szIp[16];                  // IP address
	int                 nPort;                     // port number
	char                szUserName[32];            // user name
	char                szPassword[32];	           // pass word
	BOOL                bAnonymity;                // whether anonymity log in 
	BYTE                byReserved[256];           // reserved
}FTP_SERVER_CFG;

// ftp server connect test
typedef struct
{
	FTP_SERVER_CFG     stuFtpServerInfo;           // ftp server info(filled by user)
	DHDEV_FUNC_TEST    stuTestResult;              // ftp server connect state
	BYTE               byReserved[64];
}DHDEV_FTP_SERVER_TEST;

// DDNS domain info
typedef struct
{

	char				szServerType[DH_MAX_SERVER_TYPE_LEN];	// server type
	char				szServerIp[DH_MAX_DOMAIN_NAME_LEN];		// server ip or domain
	DWORD				dwServerPort;							// server port
	char				szDomainName[DH_MAX_DOMAIN_NAME_LEN];	// dvr domain,such as jeckean.3322.org
	char				szUserName[DH_MAX_HOST_NAMELEN];		// username
	char				szUserPsw[DH_MAX_HOST_PSWLEN];			// password
	BYTE                byReserved[256];           				// reserved
}DDNS_DOMAIN_INFO;


// DDNS domain test
typedef struct
{
	DDNS_DOMAIN_INFO   stuDomainInfo;				// DDNS domain
	DHDEV_FUNC_TEST    stuTestResult;				// test result
	char			   szMemo[128];					// test result memo
	BYTE               byReserved[64];
}DHDEV_DDNS_DOMAIN_TEST;


// hard disk's basic information
typedef struct 
{
	BYTE                byModle[32];				// model
	BYTE                bySerialNumber[32];			// serial number
	BYTE                byFirmWare[32];				// firmware no
	int                 nAtaVersion;				// ATA protocol version no
	int                 nSmartNum ;					// smart information no
	INT64				Sectors;	
	int                 nStatus;					// disk state 0-normal 1-abnormal
	int                 nReserved[33]; 
} DHDEV_DEVICE_INFO;

//smart information of harddisk, there may be many items up to more than 30
typedef struct
{
	BYTE				byId;						// ID
	BYTE				byCurrent;					// attribute values 
	BYTE				byWorst;					// maximum error value 
	BYTE				byThreshold;				// threshold value 
	char				szName[64];					// property name
	char				szRaw[8];					// actual value
	int					nPredict;					// state
	char				reserved[128];
} DHDEV_SMART_VALUE;

//search hard disk smart information
typedef struct
{
	BYTE               nDiskNum;       // disk number
	BYTE               byRaidNO;       // Raid sub disk, 0:single disk
	BYTE               byReserved[2];  
	DHDEV_DEVICE_INFO  deviceInfo;    
	DHDEV_SMART_VALUE  smartValue[MAX_SMART_VALUE_NUM];
} DHDEV_SMART_HARDDISK;

// submodule information
typedef struct
{
	char               szModuleName[64];			//  submodule name
	char               szHardWareVersion[32];		//  submodule HardWareVersion
	char               szSoftwareVersion[32];		//  submodule SoftWareVersion
	BYTE               reserved[128]; 
} DHDEV_SUBMODELE_VALUE;

// search submodule information
typedef struct
{
	int                    nSubModuleNum;							// nSubModuleNum
	DHDEV_SUBMODELE_VALUE  stuSubmoduleValue[MAX_SUBMODULE_NUM];	// Submodule information
	BYTE				   bReserved[256];
} DHDEV_SUBMODULE_INFO;

// Query harddisk damage ability
typedef struct
{
	BYTE                bDiskDamageLevel[DH_MAX_DISK_NUM];  // every disk's damage level
	BYTE                bReserved[128];
} DHDEV_DISKDAMAGE_INFO;

// SYSLOG remote server ability
typedef struct 
{
	char				szServerIp[DH_MAX_IPADDR_OR_DOMAIN_LEN];	//Server IP address
	int					nServerPort;								//Server port
	BYTE				bEnable;									//Server Enable
	BYTE				bReserved[255];								//Reserved bytes
}DHDEV_SYSLOG_REMOTE_SERVER;

// Video backup config parameter
typedef struct
{
	BYTE				backupVideoFormat;							//file type to backup, 0:dav, 1:asf
	BYTE				password[6];								//password
	BYTE				reversed[505];								//reserved
}DHDEV_BACKUP_VIDEO_FORMAT;


// Auto register config parameter
typedef struct  
{
	char				szServerIp[32];			// Registration server IP ; no use it,use szServerIpEx
	int					nServerPort;			// Port number 
	BYTE                byReserved[3];          // 
	BYTE                bServerIpExEn;          // Extend Registration server IP enable, 0-not enable, 1-enable
	char				szServerIpEx[60];       // Extend Registration server IP
} DHDEV_SERVER_INFO;

typedef struct  
{
	DWORD				dwSize;
	BYTE				bServerNum;				// The max IP amount supported
	DHDEV_SERVER_INFO	lstServer[DH_MAX_REGISTER_SERVER_NUM];
	BYTE				bEnable;				// Enable
	char				szDeviceID[32];			// Device ID
	char				reserved[94];
} DHDEV_REGISTER_SERVER;

// Camera property 
typedef struct __DHDEV_CAMERA_INFO
{
	BYTE				bBrightnessEn;			// Brightness adjustable;1:adjustable,0:can not be adjusted
	BYTE				bContrastEn;			// Contrast adjustable
	BYTE				bColorEn;				// Hue adjustable
	BYTE				bGainEn;				// Gain adjustable
	BYTE				bSaturationEn;			// Saturation adjustable
	BYTE				bBacklightEn;			// Backlight compensation adjustable
	BYTE				bExposureEn;			// Exposure option adjustable
	BYTE				bColorConvEn;			// Day/night switch 
	BYTE				bAttrEn;				// Property option; 1:Enable, 0:Disable
	BYTE				bMirrorEn;				// Mirror;1:support,0:do not support 
    BYTE				bFlipEn;				// Flip;1:support,0:do not support 
	BYTE				iWhiteBalance;			// White Balance 1 Support,0 :Do not support
	BYTE				iSignalFormatMask;		// Signal format mask,Bitwise:0-Inside(Internal input) 1- BT656 2-720p 3-1080i  4-1080p  5-1080sF
	BYTE				bRotate90;				// 90-degree rotation 1:support,0:do not support 
    BYTE				bLimitedAutoExposure;   // Support the time limit with automatic exposure 1:support,0:do not support 
    BYTE				bCustomManualExposure;  // support user-defined manual exposure time 1:support,0:do not support
	BYTE				bFlashAdjustEn;			// Support the flash lamp adjust
	BYTE				bNightOptions;			// Support day and night change
	BYTE                iReferenceLevel;    	// Support electric reference setting
	BYTE                bExternalSyncInput;     // Support external sync Input
	unsigned short      usMaxExposureTime;      // Max exposure time, unit:ms         
	unsigned short      usMinExposureTime;      // Min exposure time, unit:ms
	BYTE                bWideDynamicRange;      // Wide dynamic range,0-present not support,2~n max supported range value
	BYTE                bDoubleShutter;         // Double Shutter
	BYTE				byExposureCompensation; // 1 support, 0 not support
	BYTE				bRev[109];				// reserved 
} DHDEV_CAMERA_INFO;

// Special configuration for night,will automatically switch to night configuration when low light
typedef struct __DHDEV_NIGHTOPTIONS 
{
	BYTE  bEnable;								// 0-Do not switch,1-Switch 
												// Roughly sunrise and sunset time, before sunrise or after sunset, will use a special configuration for night.
												// 00:00:00 ~23:59:59
	BYTE  bSunriseHour;
	BYTE  bSunriseMinute;
	BYTE  bSunriseSecond;
	BYTE  bSunsetHour;
	BYTE  bSunsetMinute;
	BYTE  bSunsetSecond;  
	BYTE  bWhiteBalance ;						// White balance  0:Disabled,1:Auto 2:sunny 3:cloudy 4:home 5:office 6:night 7: Custom
	BYTE  bGainRed;								// Red gain adjustment, white balance to "Custom" mode will effect 0~100
	BYTE  bGainBlue;							// Green gain adjustment, white balance to "Custom" mode will effect 0~100
	BYTE  bGainGreen;							// Blue gain adjustment, white balance to "Custom" mode will effect 0~100
	BYTE  bGain;								// 0~100
	BYTE  bGainAuto;							// 0-Without auto gain 1-Auto gain
	BYTE  bBrightnessThreshold ;				// Brightness value 0~100
	BYTE  ReferenceLevel;                       // electric value 0~100   
	BYTE  bExposureSpeed;						// Range depends on the device capability: 0-Auto Exposure  1~n-1-manual exposure level;  n-Auto Exposure with time limit;  n+1-manual exposure customized time (n means exposure level)
	float ExposureValue1;						// Lower limit of automatic exposure time or customized manual exposure time, in milliseconds, value 0.1ms ~ 80ms
	float ExposureValue2;						// Automatic exposure time limit, in milliseconds, value 0.1ms ~ 80ms
	BYTE  bAutoApertureEnable;                  // Auto Aperture Enable,1:open,0:close
	BYTE  bWideDynamicRange;                    // wide dynamic value, it depends on max support value
	WORD  wNightSyncValue;						// night sync 0~360
	WORD  wNightSyncValueMillValue;             // night sync mill value 0~999
	BYTE res[10];								// Reserve
} DHDEV_NIGHTOPTIONS;

// camera attribute configration
typedef struct __DHDEV_CAMERA_CFG 
{ 
	DWORD				dwSize;
	BYTE				bExposure;				// Exposure mode;1-9:Manual exposure level  ,0: Auto exposure 
	BYTE				bBacklight;				// Backlight compensation:3:High,2:Medium,1:Low,0:close
	BYTE				bAutoColor2BW;			// Day/night mode;2:Open,1:Auto,0:Close
	BYTE				bMirror;				// Mirror;1:Open,0:Close
	BYTE				bFlip;					// Flip;1:Open,0:Close  support;0 :do not support 
	BYTE				bLensEn;				// the capacity of Auto Iris function 1:support;0 :do not support
	BYTE				bLensFunction;			// Auto aperture function: 1:Enable aperture  ;0: Disable aperture  
	BYTE				bWhiteBalance;			// White Balance 0:Disabled,1:Auto 2:sunny 3:cloudy 4:home 5:office 6:night
	BYTE				bSignalFormat;			// Signal format 0-Inside(Internal input) 1- BT656 2-720p 3-1080i  4-1080p  5-1080sF
	BYTE				bRotate90;				// 0-Not rotating,1-90 degrees clockwise,2-90 degrees counterclockwise
	BYTE                bReferenceLevel;        // electric value 0~100  
	BYTE                byReserve;              // Reserved
	float				ExposureValue1;			// Auto exposure or manual exposure limit custom time,Milliseconds(0.1ms~80ms)
	float				ExposureValue2;			// Auto exposure limit,Milliseconds(0.1ms~80ms)
	DHDEV_NIGHTOPTIONS	stuNightOptions;		// Configuration parameter options for night 
	BYTE				bGainRed;				// Red gain adjustment, effective when white balance to "Custom" mode,  0 ~ 100
	BYTE				bGainBlue;				// Green gain adjustment, effective when white balance to "Custom" mode,  0 ~ 100
	BYTE				bGainGreen;				// Blue gain adjustment, effective when white balance to "Custom" mode,  0 ~ 100
	BYTE				bFlashMode;				// Flash mode,0-close,1-always,2-auto 
	BYTE				bFlashValue;			// Flash work values,  0-0us, 1-64us, 2-128us,...15-960us
	BYTE				bFlashPole;				// Flash trigger mode 0 - low level 1 - high level
	BYTE                bExternalSyncPhase;     // External single input
	BYTE                bFlashInitValue;        // Flash brightness prevlue, 0~100
	WORD                wExternalSyncValue ;    // External value 0~360
	WORD                wExternalSyncValueMillValue; //  External SyncValue Mill Value0~999
	BYTE                bWideDynamicRange;           // wide dynamic range, it depends on max support value
	BYTE				byExposureCompensation;		// default compensation value,default is 7,range[0~14]
	char				bRev[54];				// Reserved
} DHDEV_CAMERA_CFG;

#define ALARM_MAX_NAME 64
// (wireless)IR alarm setup
typedef struct
{
	BOOL				bEnable;				// Enable alarm input
	char				szAlarmName[DH_MAX_ALARM_NAME];	// Alarm input name
	int					nAlarmInPattern;		// Annunciator input wave
	int					nAlarmOutPattern;		// Annunciator output wave
	char				szAlarmInAddress[DH_MAX_ALARM_NAME];// Alarm input address
	int					nSensorType;			// External device sensor type normal open or normal close 
	int					nDefendEfectTime;		// Deploy and cancel latch time. The alarm input becomes activated after the specified time.
	int					nDefendAreaType;		// Defend area 
	int					nAlarmSmoothTime;		// Alarm smooth time:system ignores the second alarm if one alarm inputs in two times. 
	char				reserved[128];
	DH_TSECT			stSect[DH_N_WEEKS][DH_N_REC_TSECT];
	DH_MSG_HANDLE		struHandle;				// Process way 
} DH_INFRARED_INFO;

// Wireless remote control setup 
typedef struct 
{
	BYTE				address[ALARM_MAX_NAME];// Remote control address
	BYTE				name[ALARM_MAX_NAME];	// Remote control name
	BYTE				reserved[32];			// Reserved string 
} DH_WI_CONFIG_ROBOT;

// Wireless alarm output setup 
typedef struct 
{
	BYTE				address[ALARM_MAX_NAME];// Alarm output address
	BYTE				name[ALARM_MAX_NAME];	// Alarm output name
	BYTE				reserved[32];			// Reserved string 
} DH_WI_CONFIG_ALARM_OUT;

typedef struct  
{
	DWORD				dwSize;
	BYTE				bAlarmInNum;			// Wireless alarm input amount 
	BYTE				bAlarmOutNum;			// Wireless alarm output amount 
	DH_WI_CONFIG_ALARM_OUT AlarmOutAddr[16];	// Alarm output address
	BYTE				bRobotNum;				// Remote control amount
	DH_WI_CONFIG_ROBOT  RobotAddr[16];			// Remote control address 
	DH_INFRARED_INFO	InfraredAlarm[16];
	char				reserved[256];
} DH_INFRARED_CFG;

// New audio detection alarm information 
typedef struct
{
	int					channel;				// Alarm channel number
	int					alarmType;				// Alarm type;0:Volume value is too low ,1:Volume value is too high. 
	unsigned int		volume;					// Volume
	BYTE                byState;                // volume alarm state, 0: alarm appear, 1: alarm disappear
	char				reserved[255];
} NET_NEW_SOUND_ALARM_STATE;

typedef struct  
{
	int					channelcount;			// Alarm channel amount 
	NET_NEW_SOUND_ALARM_STATE SoundAlarmInfo[DH_MAX_ALARM_IN_NUM];
} DH_NEW_SOUND_ALARM_STATE;

// Snapshot function property structure 
typedef struct 
{
	int					nChannelNum;			// Channel amount 
	DWORD				dwVideoStandardMask;	// Resolution(Bit). Please refer to enumeration CAPTURE_SIZE						
	int					nFramesCount;			// Valid length of Frequency[128] array
	char				Frames[128];			// Frame rate(value) 
												// -25:1f/25s;-24:1f/24s;-23:1f/23s;-22:1f/23s
												// ......
												// 0: invalid;1:1f/s;2:2f/s;3:13f/s
												// 4:4f/s;5:5f/s;17:17f/s;18:18f/s
												// 19:19f/s;20:20f/s
												// ......
												// 25: 25f/s
	int					nSnapModeCount;			// valid length of SnapMode[16] array
	char				SnapMode[16];			// (value)0:activate scheduled snapshot,1:Manually activate snapshot
	int					nPicFormatCount;		// Valid length of Format[16] array 
	char 				PictureFormat[16];		// (Value)0:BMP format,1:JPG format
	int					nPicQualityCount;		// valid length of Quality[32] array
	char 				PictureQuality[32];		// value
												// 100:Image quality 100%;80:Image quality 80%;60:Image quality60%
												// 50:Image quality50%;30:Image quality30%;10:Image quality10%
	char 				nReserved[128];			// Reserved
} DH_QUERY_SNAP_INFO;

typedef struct 
{
	int					nChannelCount;			// Channel amount 
	DH_QUERY_SNAP_INFO  stuSnap[DH_MAX_CHANNUM];
} DH_SNAP_ATTR_EN;


/* IP Filtering configuration */
#define DH_IPIFILTER_NUM			200			// IP
#define DH_IPIFILTER_NUM_EX			512			// IP

// IP Information
typedef struct 
{
	DWORD				dwIPNum;				// IP count
	char				SZIP[DH_IPIFILTER_NUM][DH_MAX_IPADDR_LEN]; // IP
	char				byReserve[32];			// Reserved
} IPIFILTER_INFO;

// IP Filtering configuration
typedef struct
{
	DWORD				dwSize;
	DWORD				dwEnable;				// Enable
	DWORD				dwType;					// The current list type;0:White List 1:Blacklist(The device can enable only one type of list)
	IPIFILTER_INFO		BannedIP;				// Black list
	IPIFILTER_INFO		TrustIP;				// Trusted sites
	char				byReserve[256];			// Reserved
} DHDEV_IPIFILTER_CFG;

// IP Information extended
typedef struct 
{
	DWORD				dwIPNum;				// IP count
	char				SZIP[DH_IPIFILTER_NUM_EX][DH_MAX_IPADDR_LEN]; // IP
	char				byReserve[32];			// Reserved
} IPIFILTER_INFO_EX;

// IP Filtering extended configuration
typedef struct
{
	DWORD				dwSize;
	DWORD				dwEnable;				// Enable
	DWORD				dwType;					// The current list type;0:White List 1:Blacklist(The device can enable only one type of list) can only user one kind of device
	IPIFILTER_INFO_EX	BannedIP;				// Black list
	IPIFILTER_INFO_EX	TrustIP;				// Trusted sites
	char				byReserve[256];			// Reserved
} DHDEV_IPIFILTER_CFG_EX;

/* MAC filter configuration */
#define DH_MACFILTER_NUM			512			// MAC

// MAC info
typedef struct 
{
	DWORD				dwSize;					// struct size
	DWORD				dwMacNum;				// MAC count
	char				szMac[DH_MACFILTER_NUM][DH_MACADDR_LEN]; // MAC
} MACFILTER_INFO;
// MAC filter configuration
typedef struct
{
	DWORD					dwSize;				// struct size
	DWORD					dwEnable;			// enable
	DWORD					dwType;				// current list type,0:white list, 1:blacklist (The device can enable only one type of list) can only user one kind of device
	MACFILTER_INFO			stuBannedMac;		// black list Mac
	MACFILTER_INFO			stuTrustMac;		// white list Mac
} DHDEV_MACFILTER_CFG;

/* MAC,IP filter configuration */
#define DH_MACIPFILTER_NUM			512			// MAC,IP
// MAC, IP filter configuration info
typedef struct
{
	DWORD	dwSize;					// struct size
	char	szMac[DH_MACADDR_LEN];	// mac
	char	szIp[DH_MAX_IPADDR_LEN];// ip
}MACIP_INFO;

// MAC,IP filter configuration
typedef struct
{
	DWORD					dwSize;								// struct size
	DWORD					dwEnable;							// enable
	DWORD					dwType;								// The current list type;0:White List 1:Blacklist(The device can enable only one type of list) can only user one kind of device
	DWORD					dwBannedMacIpNum;					// black list MAC,IP count (MAC,IP one to one)
	MACIP_INFO				stuBannedMacIp[DH_MACIPFILTER_NUM];	// black list Mac,IP
	DWORD					dwTrustMacIpNum;					// white list MAC,IP count(MAC,IP one to one)
	MACIP_INFO				stuTrustMacIp[DH_MACIPFILTER_NUM];	// white list Mac,IP
} DHDEV_MACIPFILTER_CFG;
typedef struct
{
	int                nCardNum;                // card number
	char 	           cardInfo [DH_MAX_CARD_NUM][DH_MAX_CARDINFO_LEN]; // card info string 
	BYTE               byReserve[32];           // reserved
}DHDEV_NETCARD_CFG;

// RTSP configuration
typedef struct
{
	WORD               wPort;                  // port number(can't be the same as tcp or udp's port number)
	BYTE               byReserved[62];         // reserved
}DHDEV_RTSP_CFG;

// stream encrypt configuration
// encrypt key info
typedef struct _ENCRYPT_KEY_INFO
{
	BYTE        byEncryptEnable;       //  encrypt enable, 0: not encrypt, 1:encrypt
	BYTE        byReserved[3]; 
	union
	{
		BYTE    byDesKey[8];           // des key
		BYTE    by3DesKey[3][8];       // 3des key
		BYTE    byAesKey[32];          // aes key
		
	};
}ENCRYPT_KEY_INFO;

// encrypt algorithm
typedef struct _ALGO_PARAM_INFO
{
	WORD       wEncryptLenth;       // encrypt length, for example: wEncryptLenth = 128,the effictive encprypt key value only include byAesKey[0]~[15] in the  ENCRYPT_KEY_INFO struct
									// when the encrypt algorithm is AES,it only support 3 kind length such as 128,192,256
									// when the encrypt algorithm is DES,it has a fixed value 64
									// when the encrypt algorithm is DES,it means the encrypt key's number(2 or 3 encrypt key)
	BYTE       byAlgoWorkMode ;     // work mode, see EM_ENCRYPT_ALOG_WORKMODE 
	BYTE       reserved[13];        // reserved
}ALGO_PARAM_INFO;

// stream encrypt configuration
typedef struct _DHEDV_STREAM_ENCRYPT
{
    BYTE                    byEncrptAlgoType;        // encrypt algorithm type:00: AES,01:DES,02: 3DES
    BYTE                    byReserved1[3];
    ALGO_PARAM_INFO         stuEncrptAlgoparam;      // encrypt algorithm param
    ENCRYPT_KEY_INFO        stuEncryptKeys[32];      // each channel's encrypt key's info
	BYTE					byEncrptPlanEnable;		 // encrpt plan enable
	BYTE					byReserved3[3];
	NET_TIME				stuPreTime;				 // encrpt plan beginning time
	BYTE					reserved2[1360];
}DHEDV_STREAM_ENCRYPT;

// limit stream configuration
typedef struct _DHDEV_BIT_RATE
{
	DWORD                   nExpectCodeRate;          // limit stream (kps)
	BYTE                    byReserved[64];           // reserved
}DHDEV_LIMIT_BIT_RATE;
// custom configuration
typedef struct _DHDEV_CUSTOM_CFG
{
	char                   szData[1024];            // custom configuration information
	char                   reserved[3072];          // reserved
}DHDEV_CUSTOM_CFG;

/*audio talk configuration*/
typedef struct
{
	DWORD				dwSize;
	int					nCompression;			// Compression type,value,refer to DH_TALK_CODING_TYPE,please set the compression type according to the mode the device supports
	int					nMode;					// Encode mode, value, 0 means not support the compression type
												// Set the responding type according to compression type
												// like AMR, please refer to EM_ARM_ENCODE_MODE
	char				byReserve[256];			// Reserved
} DHDEV_TALK_ENCODE_CFG;

// According to the mobile function
// (Event triggers message)MMS Configuration Structure
typedef struct
{
	DWORD				dwSize;
	DWORD				dwEnable;				// Enable
	DWORD				dwReceiverNum;			// Receiver amount
	char				SZReceiver[DH_MMS_RECEIVER_NUM][32];	// Receiver,normally cellphone
    BYTE                byType;					// Message type 0:MMS;1:SMS
	char                SZTitle[32];			// Message title
	char				byReserve[223];			// Reserved
} DHDEV_MMS_CFG;

// (Message triggers wireless connection configuration)
typedef struct  
{
	DWORD				dwSize;
	DWORD				dwEnable;				// Enable
	DWORD				dwSenderNum;			// Sender amount
	char				SZSender[DH_MMS_SMSACTIVATION_NUM][32];	// Sender, normally the cellphone number
 	char				byReserve[256];			// Reserved
}DHDEV_SMSACTIVATION_CFG;

// (Dailing triggers the wireless connection)
typedef struct  
{
	DWORD				dwSize;
	DWORD				dwEnable;				// Enable
	DWORD				dwCallerNum;			// Sender amount
	char				SZCaller[DH_MMS_DIALINACTIVATION_NUM][32];	// Sender, normally the cellphone
 	char				byReserve[256];			// Reserved
}DHDEV_DIALINACTIVATION_CFG;
// Above is with the mobile phone functions


// Wireless network signal strength structure
typedef struct
{
	DWORD				dwSize;
	DWORD				dwTDSCDMA;				// TD-SCDMA strength,range:0-100
	DWORD				dwWCDMA;				// WCDMA strength,range:0-100
	DWORD				dwCDMA1x;				// CDMA1x strength,range:0-100
	DWORD				dwEDGE;					// EDGE strength,range:0-100
	DWORD				dwEVDO;					// EVDO strength,range:0-100
	int					nCurrentType;			// Current type
												// 0	The device can't support this
												// 1    TD_SCDMA
												// 2	WCDMA
												// 3	CDMA_1x
												// 4	EDGE
												// 5	EVDO
                                                // 6 	TD-LTE
    DWORD               dwTDLTE;                // TD-LTE Strength, Range:0:100
	char				byReserve[248];			// Reserved
} DHDEV_WIRELESS_RSS_INFO;

typedef struct _DHDEV_SIP_CFG
{
	BOOL				bUnregOnBoot;			//Unregister on Reboot
	char				szAccoutName[64];		//Account Name
	char				szSIPServer[128];		//SIP Server
	char				szOutBoundProxy[128];	//Outbound Proxy
	DWORD				dwSIPUsrID;				//SIP User ID
	DWORD				dwAuthID;				//Authenticate ID
	char				szAuthPsw[64];			//Authenticate Password
	char				szSTUNServer[128];		//STUN Server
	DWORD				dwRegExp;				//Registration Expiration
	DWORD				dwLocalSIPPort;			//Local SIP Port
	DWORD				dwLocalRTPPort;			//Local RTP Port
	BOOL				bEnable;				//Enable
	char				szNotifyID[128];        //Notify ID
	NET_TIME			stuRegTime;             //register time, read only 
	BYTE				bReserved[868];		    //reserved
} DHDEV_SIP_CFG;

typedef struct _DHDEV_SIP_STATE
{
	int					nStatusNum;				//SIP state number 
	BYTE				byStatus[64];           //0:on line,1:off line,2:invalid sip,3:registering,4:talking
	BYTE				bReserved[64];		    //reserved
}DHDEV_SIP_STATE;

typedef struct _DHDEV_HARDKEY_STATE
{
	BOOL				bState;                 // 0:Hard Key disconnect, 1:Hard Key connect
	BYTE				bReserved[64];          // reserved
}DHDEV_HARDKEY_STATE;

typedef struct _DHDEV_ISCSI_PATHLIST
{
	int					nCount;
	char				szPaths[DH_MAX_ISCSI_PATH_NUM][MAX_PATH_STOR];	// remote path
} DHDEV_ISCSI_PATHLIST;

// wifi capability country
typedef struct _DHDEV_WIFI_ROUTE_CAP_COUNTRY
{
	char				szCountry[32];			// country
	int					nValidChnNum;			// valid channel number
	int					nValideChannels[32];	// valid channel array
	char				reserved[64];			// reserved
} DHDEV_WIFI_ROUTE_CAP_COUNTRY;

typedef struct _DHDEV_WIFI_ROUTE_CAP
{
	int					nCountryNum;				// country number
	DHDEV_WIFI_ROUTE_CAP_COUNTRY stuCountry[256];	// country config
	char				reserved[256];				// reserved
} DHDEV_WIFI_ROUTE_CAP;

//  monitor info
typedef struct _MONITOR_INFO
{
	int					nPresetObjectNum;        // preset object number
	int					nActualObjectNum;        // actual object number
	char				reserved[64]; 
}MONITOR_INFO;

typedef struct _DHDEV_MONITOR_INFO
{
	int					nChannelNumber;
	MONITOR_INFO		stMonitorInfo[64];			//  monitor info
	char				reserved[128];
}DHDEV_MONITOR_INFO;

//Multi Channel Preview Playback Segmentation Ability
typedef struct tagNET_MULTIPLAYBACK_SPLIT_CAP
{
    DWORD            dwSize;
    int              nSliptMode[DH_MAX_MULTIPLAYBACK_SPLIT_NUM]; // Support segmentation Mode  
    int              nModeNumber;       
}NET_MULTIPLAYBACK_SPLIT_CAP;

/***************************** PTZ preset configuration ***********************************/
typedef struct _POINTEANBLE
{
	BYTE				bPoint;					// Preset range[1,80], invalid setting is 0.
	BYTE				bEnable;				// Valid or not,0 invalid,1 valid
	BYTE				bReserved[2];
} POINTEANBLE;

typedef struct _POINTCFG
{
	char				szIP[DH_MAX_IPADDR_LEN];// ip
	int					nPort;					// Port	
	POINTEANBLE			stuPointEnable[80];		// Preset enable
	BYTE				bReserved[256];
}POINTCFG;

typedef struct _DHDEV_POINT_CFG
{
	int					nSupportNum;			// Read-only parameter, shall return to sdk when set, it means supported pre-set number
	POINTCFG			stuPointCfg[16];		// 2D config means point info.
	BYTE				bReserved[256];			// Reserved
}DHDEV_POINT_CFG;
////////////////////////////////Mobile DVR supported////////////////////////////////

// GPS information(Mobile device)
typedef struct _GPS_Info
{
    NET_TIME			revTime;				// position time 
	char				DvrSerial[50];			// Device serial number
    double				longitude;				// Longitude
    double				latidude;				// Latitude 
    double				height;					// Height(meter)
    double				angle;					// Angle(The north pole is the original point and clockwise is +)
    double				speed;					// Speed(Kilometer/hour)
    WORD				starCount;				// Starcount
    BOOL				antennaState;			// Antenna status(true=good,false =bad)
    BOOL				orientationState;		// Orientation status (true=position,false = no position )
} GPS_Info,*LPGPS_Info;

// alarm state info
typedef struct 
{
	int                nAlarmCount;             // alarm count
	int                nAlarmState[128];        // alarm state
	BYTE               byRserved[128];          // reserved
}ALARM_STATE_INFO;

// Gps location info
typedef struct tagNET_GPS_LOCATION_INFO
{
	GPS_Info	        stuGpsInfo;               // GPS info
	ALARM_STATE_INFO    stuAlarmStateInfo;        // Alarm state info
	int				    nTemperature;		              // Temperature(Unit:0.1 centigrade)
	int					nHumidity;				  // Humidity(Unit:0.1%)
	unsigned int		nIdleTime;				  // Idle time(Unit:second)
	unsigned int        nMileage;				  // Mileage(Unit:0.1km)
	int					nVoltage;				  // Set Voltage value(Unit:0.1V)
	BYTE			    byReserved[1024];  
}NET_GPS_LOCATION_INFO, *LPNET_GPS_LOCATION_INFO;

// Snapshot parameter structure 
typedef struct _snap_param
{
	unsigned int		Channel;				// Snapshot channel
	unsigned int		Quality;				// Image quality:level 1 to level 6
	unsigned int		ImageSize;				// Video size;0:QCIF,1:CIF,2:D1
	unsigned int		mode;					// Snapshot mode;0:request one frame,1:send out requestion regularly,2: Request consecutively
	unsigned int		InterSnap;				// Time unit is second.If mode=1, it means send out requestion regularly. The time is valid.
	unsigned int		CmdSerial;				// Request serial number
	unsigned int		Reserved[4];
} SNAP_PARAMS, *LPSNAP_PARAMS;

// Snapshot function setup 
typedef struct 
{
	DWORD				dwSize;
	BYTE				bTimingEnable;				// Schedule snapshot button(The alarm snapshot button is in alarm activation configuration)
	BYTE                bPicIntervalHour;           // timing snapshot interval hour
	short	            PicTimeInterval;			// The time intervals of timing capture (s). At present, the capture device supports the largest time interval is 30 minutes 
	DH_VIDEOENC_OPT		struSnapEnc[SNAP_TYP_NUM];  // Snapshot encode setup. The resolution, video quality, frame rate setup and fram rate are all negative. It means the snapshot times in each second. 
} DHDEV_SNAP_CFG;

// snap function extern configuration
typedef struct 
{
	DWORD				dwSize;
	BYTE				bTimingEnable;				// timing diagram switch(in tach of the alarm configuration)
	BYTE                bPicIntervalHour;           // the number of hours interval
	short	            PicTimeInterval;			// time interval(s),the biggest capture device support interval 30min now                        
	DH_VIDEOENC_OPT		struSnapEnc[SNAP_TYP_NUM];  // snap encode configuration,support resolution ,image quality and frame rate setting,frame rate is negative,say for a second to grasp figure number
	DWORD               dwTrigPicIntervalSecond;    // 
	BYTE                byRserved[256];             // reserved
} DHDEV_SNAP_CFG_EX;
//wifi state of car device
typedef struct  
{
	char	szSSID[128];							//SSID
	BOOL	bEnable;								//If enable wifi function, 0:Disable 1:Enable
	int		nSafeType;								//Verify type
													//0:OPEN 
													//1:RESTRICTE
													//2:WEP
													//3:WPA
													//4:WPA2
													//5:WPA-PSK
													//6:WPA2-PSK
	int		nEncryprion;							//Encryption method
													//0:OPEN
													//1:TKIP
													//2:WEP
													//3:AES
													//4:NONE(without verify)
													//5:AUTO
	                                                //6:SHARED
	int		nStrength;								//AP site signal
	char	szHostIP[128];							//host address
	char	szHostNetmask[128];						//host mask
	char	szHostGateway[128];						//host gate  way
	int		nPriority;								//Priority,(1-32)
	int     nEnDHCP;                                //0:disable 1:enable(default value is 1)
	BYTE	bReserved[1016];
} DHDEV_VEHICLE_WIFI_STATE;

typedef struct
{
	char	szSSID[128];							//SSID
	int		nPriority;								//Priority,(1-32)
	int		nSafeType;								//Verify type
													//0:OPEN 
													//1:RESTRICTE
													//2:WEP
													//3:WPA
													//4:WPA2
													//5:WPA-PSK
													//6:WPA2-PSK
	int		nEncryprion;							//Encryption method
													//0:OPEN
													//1:TKIP
													//2:WEP
													//3:AES
													//4:NONE(No verify)
													//5:AUTO
	                                                //6:SHARED
	char	szKey[128];								//Connection key
	char	szHostIP[128];							//Host address
	char	szHostNetmask[128];						//Host mask
	char	szHostGateway[128];						//Host gateway
	int     nEnDHCP;                                //0:disable 1:enable(default value is 1)
	BYTE    byKeyIndex;                             //Verify index of WEP,0:no support,1-4 is index
	BYTE	bReserved[1019];
} DHDEV_VEHICLE_WIFI_CONFIG;
typedef struct
{
	char    szSSID[128];                            // SSID   
	BYTE    bReserved[256];                         // reserved
}WIFI_CONNECT;
// IP modify configuration
typedef struct __DHCTRL_IPMODIFY_PARAM
{
	int                 nStructSize;
	char				szRemoteIP[DH_MAX_IPADDR_OR_DOMAIN_LEN];		// device IP
	char				szSubmask[DH_MAX_IPADDR_LEN];	                // submask
	char				szGateway[DH_MAX_IPADDR_OR_DOMAIN_LEN];	        // gatway
	char				szMac[DH_MACADDR_LEN];			                // MAC addr
	char				szDeviceType[DH_DEV_TYPE_LEN];	                // device type
}DHCTRL_IPMODIFY_PARAM;

typedef struct 
{
	BOOL	bIsScan;								//0:Not scan wifi (Manually added), 1: scanned wifi
	char	szSSID[128];							//SSID
	int		nSafeType;								//Verify type
													//0:OPEN 
													//1:RESTRICTE
													//2:WEP
													//3:WPA
													//4:WPA2
													//5:WPA-PSK
													//6:WPA2-PSK
	int		nEncryprion;							//Encryption method
													//0:OPEN
													//1:TKIP
													//2:WEP
													//3:AES
													//4:NONE(No verify)	
													//5:AUTO
	                                                //6:SHARED
	char	szKey[128];								//Connection key
	int		nStrength;								//AP site signal
	int		nMaxBitRate;							//AP site maximum transmission rate, with units, read-only
	int		nIsCnnted;								//If success connect read-only
	int		nIsSaved;								//If save read-only
	int		nPriority;								//Priority,(1-32)
	char	szHostIP[128];							//Host address
	char	szHostNetmask[128];						//Host network mask
	char	szHostGateway[128];						//Host gateway
	int		nWifiFreq;								//Radio frequency, using channel logo
	int     nEnDHCP;                                //0:disable 1:enable(default value is 1)
	BYTE    byKeyIndex;                             //Verify index of WEP,0:no support,1-4 is index
	BYTE	bReserved[1019];
}DHDEV_VEHICLE_SINGLE_WIFI_AP_CFG;

typedef struct 
{
	BOOL	bEnable;								//if enable wifi, 0:disable, 1:enable
	int		nWifiApNum;								//Effective number of structure:DHDEV_VEHICLE_WIFI_AP_CFG 
	DHDEV_VEHICLE_SINGLE_WIFI_AP_CFG struWifiAp[64];//Single WIFI AP configration
	int	nReserved[512];								//reserved
}DHDEV_VEHICLE_WIFI_AP_CFG;

typedef struct  
{	
	BOOL   bEnable;									//to enable wift, 0:disable 1:enable
	int    nRetWifiApNum;							//get number of DHDEV_VEHICLE_WIFI_AP_CFG
	int    nMaxWifiApNum;							//apply number of DHDEV_VEHICLE_SINGLE_WIFI_AP_CFG
	DHDEV_VEHICLE_SINGLE_WIFI_AP_CFG* pWifiAp;		//a WIFI AP configuration
	int nReserved[512];								//reserved
}DHDEV_VEHICLE_WIFI_AP_CFG_EX;
// GPS log structure
typedef struct _DH_DEVICE_GPS_LOG_ITEM
{
	DHDEVTIME       stuDevTime;             // device time  
	DWORD		    dwLatidude;				// Longitude(0-180 degree)north Longitude 30.183382 = (30.183382 + 90) * 100000 = 120183382
	DWORD		    dwLongitude;			// Latitude(0-360 degree)east Latitude 120.178274 =(120.178274 + 180) * 100000 = 300178274
	DWORD           dwSpeed;                // speed,dwSpeed/1000*1.852km/h
	DWORD           dwHight;                // hight,m
	DWORD           dwAngle;                // direction,0~360,the north is it's origin, clockwise
	DHDEVTIME       stuGpsTime;             // GPS time 
	BYTE            bAntStatus; 		    // GPS antenna state,0 means good;!=0 meams there has some exception 
	BYTE            bOriStatus; 			// orientation state, != 0 means orientation sucess;
	BYTE            bSatCount; 				// satellite count
	BYTE            bGPSStatus; 			// GPS state,0:not orientation, 1:non differential position information 2:differential position information
	DWORD           dwTemp;                 // temperature(centigrade),if real value is 30.0 centigrade,this param will be valued as 30000
	DWORD           dwHumidity;             // humidity(%),if real value is 30.0%,this param will be valued as 30000
	BYTE            bReserved[24];          // reserved
    
}DH_DEVICE_GPS_LOG_ITEM;

// query GPS param
typedef struct _QUERY_GPS_LOG_PARAM
{
	NET_TIME			stuStartTime;			// start time
	NET_TIME			stuEndTime;				// end time
	int					nStartNum;				// start index,the first time to query an be valued with 0
	BYTE				bReserved[20];
} QUERY_GPS_LOG_PARAM;

typedef struct _GPS_TEMP_HUMIDITY_INFO
{
	double              dTemperature;          // temperature(centigrade),if real value is 30.0 centigrade,this param will be valued as 30000
	double              dHumidity;             // humidity(%),if real value is 30.0%,this param will be valued as 30000
	BYTE                bReserved[128];        // reserved
}GPS_TEMP_HUMIDITY_INFO;

// enclosure type
typedef enum
{
	ENCLOSURE_LIMITSPEED = 0x01,				// speed limit
	ENCLOSURE_DRIVEALLOW = 0x02,				// drive allow
	ENCLOSURE_FORBIDDRIVE = 0x04,				// forbid drive
	ENCLOSURE_LOADGOODS = 0x08,					// load goods
	ENCLOSURE_UPLOADGOODS = 0x10,				// upload goods
    ENCLOSURE_FLYALLOW = 0x20,                  // fly allow
    ENCLOSURE_MANUALFORBIDFLY = 0x40,           // manual forbid fly
    ENCLOSURE_FIXEDFORBIDFLY = 0x80,            // fixed forbid fly (airdrome ) - no configuration
    ENCLOSURE_FiXEDLIMITFLY = 0x100,             // fixed limit fly - no configuration
}ENCLOSURE_TYPE;

typedef enum
{
	 ENCLOSURE_ALARM_DRIVEIN ,	                 // drive in
	 ENCLOSURE_ALARM_DRIVEOUT,	                 // drive out
	 ENCLOSURE_ALARM_OVERSPEED,				     // over speed
	 ENCLOSURE_ALARM_SPEEDCLEAR,				 // speed clear
     ENCLOSURE_ALARM_FLYNEAR,                    // fly near
}ENCLOSURE_ALARM_TYPE;

typedef struct 
{
	DWORD				dwLongitude;			// longitude
    DWORD				dwLatidude;				// latidude
}GPS_POINT;

// enclosure config
typedef struct _DHDEV_ENCLOSURE_CFG
{
	UINT				unType;                 // mask
	BYTE				bRegion[8];             // front 4 bit means country, province, city, town
	UINT				unId;                   // rect id
	UINT				unSpeedLimit;           // speed limit(km/h)
	UINT				unPointNum;             // rect point number
	GPS_POINT			stPoints[128];			// rect point info
	char          		szStationName[DH_STATION_NAME_LEN];  //station name
    BYTE                bDisenable;             // disable. 0 - enable ; 1 - diable
    BYTE                bShape;                 // shape.  0 - polygon; 1 - circular;
    BYTE                bLimitType;             // limit type. 0-no limit; 1 - altitude limit; 2 - radius limist; 3 - both altitude and radius limit 
    BYTE                bAction;                // action . 0 - no; 1 - only report; 2 - hover; 3 - home return;
    int                 nLimitAltitude;         // limit altitude. unit: cm. 0 - invalid
    int                 nAlarmAltitude;         // alarm altitude. unit: cm. 0 - invalid
    UINT                unLimitRadius;          // limit radius. unit: cm. 0 - invalid
    UINT                unAlarmRadius;          // alarm radius. unit: cm. 0 - invalid
    BYTE                reserved[12];           // reserved
}DHDEV_ENCLOSURE_CFG;

// enclosure version config
typedef struct _DHDEV_ENCLOSURE_VERSION_CFG
{
	UINT				unType;                 // type mask,such as LIMITSPEED | DRIVEALLOW
    UINT				unVersion[32];          // version
    int					nReserved;              // reserved 
}DHDEV_ENCLOSURE_VERSION_CFG;

// enclosure alarm info
typedef struct __ALARM_ENCLOSURE_INFO
{
	int					nTypeNumber;                    // type number
	BYTE				bType[16];						// type
	int					nAlarmTypeNumber;               // alarm type number
	BYTE				bAlarmType[16];                 // alarm type
	char				szDriverId[DH_VEHICLE_DRIVERNO_LEN];  // drive id
	UINT				unEnclosureId;      	        // enclosure id
	UINT				unLimitSpeed;	                // speed limit(km/h)
	UINT				unCurrentSpeed;                 // current speed
	NET_TIME			stAlarmTime;                    // alarm time
	DWORD				dwLongitude;					// longitude
	DWORD				dwLatidude;						// latidude
	BYTE          		bOffline;                       // 0-real time 1-tofill  
    UINT                unTriggerCount;                 // trigger count
	BYTE				byReserved[115];                // reserved
}ALARM_ENCLOSURE_INFO;

// RAID state
#define DH_MAX_RAID_NUM  16
#define DH_MAX_RAID_DEVICE_NAME 16
typedef struct __RAID_STATE_INFO
{
	char				szName[16];					// Raid name
	BYTE				byType;						// type 1:Jbod     2:Raid0      3:Raid1     4:Raid5
	BYTE				byStatus;					// status 0:unknown ,1:active,2:degraded,3:inactive,4:recovering
	BYTE                byReserved[2];
	int					nCntMem;					// nMember number
	int					nMember[32];				// 1,2,3,.
	int					nCapacity;					// capacity(G)
	int					nRemainSpace;				// remain space(M)
	int					nTank;						// Tank 0:main,1:tank1,2:tank2 ...
	char				reserved[32];
}RAID_STATE_INFO;

typedef struct __ALARM_RAID_INFO
{
	int              nRaidNumber;                   // RAID number
	RAID_STATE_INFO  stuRaidInfo[DH_MAX_RAID_NUM];  // RAID info
	char             reserved[128];
}ALARM_RAID_INFO;

// RAID error type
typedef enum tagEM_RAID_ERROR
{
	EM_RAID_ERROR_UNKNOW,                                       // unknow
	EM_RAID_ERROR_FAILED,								        // RAID failed
	EM_RAID_ERROR_DEGRADED,                                     // RAID degraded
} EM_RAID_ERROR;

// NEW RAID alram,used in DH_ALARM_RAID_STATE_EX Alarm
typedef struct tagALARM_RAID_INFO_EX
{
	DWORD                nAction;                               // 0-start, 1-stop
	char                 szDevice[DH_MAX_RAID_DEVICE_NAME];     // device name
	EM_RAID_ERROR        emErrorType;                           // error type
	BYTE                 byReserved[1024];                      // rerserved
}ALARM_RAID_INFO_EX;

// Structure of IPC storage failure alarm(IPC SD Card Abnormal Alarm) Corresponding to DH_ALARM_STORAGE_IPC_FAILURE alarm
typedef struct tagALARM_STORAGE_IPC_FAILURE_INFO
{
    int                  nAction;                               // 0-start, 1-stop
    int                  nChannelID;                            // channel ID
    BYTE                 byReserved[1024];                      // rerserved
}ALARM_STORAGE_IPC_FAILURE_INFO;

//Still amarm, if the device coordinates are not changed in the specified time, the static alarm information is triggered. Corresponding to DH_ALARM_DEVICE_STAY
typedef struct tagALARM_DEVICE_STAY_INFO
{
	int					nAction;								// 0-start, 1-stop
	int					nChannelID;								// channel ID
	BYTE				byReserved[1024];						// rerserved
}ALARM_DEVICE_STAY_INFO;
// sub way door state type
typedef enum tagEM_SUB_WAY_DOOR_STATE_TYPE
{
    EM_SUB_WAY_DOOR_STATE_TYPE_UNKNOWN,     // Unkown
    EM_SUB_WAY_DOOR_STATE_TYPE_LOST,        // Lost
    EM_SUB_WAY_DOOR_STATE_TYPE_ISOLATION,   // Isolation
    EM_SUB_WAY_DOOR_STATE_TYPE_UNLOCK,      // Unlock
    EM_SUB_WAY_DOOR_STATE_TYPE_NORMAL,      // normal
}EM_SUB_WAY_DOOR_STATE_TYPE;


// sub way carriage number type
typedef enum tagEM_SUB_WAY_CARRIAGE_NUMBER_TYPE
{
    EM_SUB_WAY_CARRIAGE_NUMBER_TYPE_UNKNOWN, // Unknown
    EM_SUB_WAY_CARRIAGE_NUMBER_TYPE_TC1,     // TC1
    EM_SUB_WAY_CARRIAGE_NUMBER_TYPE_M1,      // M1
    EM_SUB_WAY_CARRIAGE_NUMBER_TYPE_M2,      // M2
    EM_SUB_WAY_CARRIAGE_NUMBER_TYPE_TC2,     // TC2
}EM_SUB_WAY_CARRIAGE_NUMBER_TYPE;

// the door state of subway Corresponding to DH_ALARM_SUB_WAY_DOOR_STATE alarm
typedef struct tagALARM_SUB_WAY_DOOR_STATE_INFO
{
    int                                 nAction;             // -1:unknown, 0:pulse, 1:start, 2:stop
    int					                nEventID;			 // Event ID
    double				                dbPTS;				 // Time stamp (Unit is ms)
    NET_TIME_EX			                stuTime;			 // Event occurrence time    
    int                                 nDoorNumber;         // door number
    EM_SUB_WAY_DOOR_STATE_TYPE          emDoorState;         // door state
    EM_SUB_WAY_CARRIAGE_NUMBER_TYPE     emCarriageNumber;    // carriage number
    BYTE                                byReserved[1024];    // reserved
}ALARM_SUB_WAY_DOOR_STATE_INFO;

// the PECE switch state of subway Corresponding to DH_ALARM_SUB_WAY_PECE_SWITCH alarm
typedef struct tagALARM_SUB_WAY_PECE_SWITCH_INFO
{
    int                                 nAction;             // -1:unknown, 0:pulse, 1:start, 2:stop
    int					                nEventID;			 // Event ID
    double				                dbPTS;				 // Time stamp (Unit is ms)
    NET_TIME_EX			                stuTime;			 // Event occurrence time
    int                                 nDoorNumber;         // door number
    int                                 nState;              // state,1:open,0:close,other:unknown
    EM_SUB_WAY_CARRIAGE_NUMBER_TYPE     emCarriageNumber;    // carriage number
    BYTE                                byReserved[1024];    // reserved
}ALARM_SUB_WAY_PECE_SWITCH_INFO;

// the fire alarm of subway Corresponding to DH_ALARM_SUB_WAY_FIRE_ALARM alarm
typedef struct tagALARM_SUB_WAY_FIRE_ALARM_INFO
{
    int                                 nAction;             // -1:unknown, 0:pulse, 1:start, 2:stop
    int					                nEventID;			 // Event ID
    double				                dbPTS;				 // Time stamp (Unit is ms)
    NET_TIME_EX			                stuTime;			 // Event occurrence time
    int                                 nSourceNumber;       // source number
    BOOL                                bState;              // state,TRUE:alarm,FALSE:no alarm    
    BYTE                                byReserved[1024];    // reserved
}ALARM_SUB_WAY_FIRE_ALARM_INFO;

// the emer handle state Corresponding to DH_ALARM_SUB_WAY_EMER_HANDLE alarm
typedef struct tagALARM_SUB_WAY_EMER_HANDLE_INFO
{
    int                                 nAction;             // -1:unknown, 0:pulse, 1:start, 2:stop
    int					                nEventID;			 // Event ID
    double				                dbPTS;				 // Time stamp (Unit is ms)
    NET_TIME_EX			                stuTime;			 // Event occurrence time
    BOOL                                bState;              // state,TRUE:handling,FALSE:no handling
    EM_SUB_WAY_CARRIAGE_NUMBER_TYPE     emCarriageNumber;    // carriage number
    BYTE                                byReserved[1024];    // reserved
}ALARM_SUB_WAY_EMER_HANDLE_INFO;

// the cab cover state Corresponding to DH_ALARM_SUB_WAY_CAB_COVER alarm
typedef struct tagALARM_SUB_WAY_CAB_COVER_INFO
{
    int                                 nAction;             // -1:unknown, 0:pulse, 1:start, 2:stop
    int					                nEventID;			 // Event ID
    double				                dbPTS;				 // Time stamp (Unit is ms)
    NET_TIME_EX			                stuTime;			 // Event occurrence time
    int                                 nState;              // state,1:open,0:close,other:unknown
    EM_SUB_WAY_CARRIAGE_NUMBER_TYPE     emCarriageNumber;    // carriage number
    BYTE                                byReserved[1024];    // reserved
}ALARM_SUB_WAY_CAB_COVER_INFO;

// the dera or obst of subway Corresponding to DH_ALARM_SUB_WAY_DERA_OBST alarm
typedef struct tagALARM_SUB_WAY_DERA_OBST_INFO
{
    int                                 nAction;             // -1:unknown, 0:pulse, 1:start, 2:stop
    int					                nEventID;			 // Event ID
    double				                dbPTS;				 // Time stamp (Unit is ms)
    NET_TIME_EX			                stuTime;			 // Event occurrence time
    BOOL                                bState;              // state, TRUE:state checked,FALSE:no state checked
    EM_SUB_WAY_CARRIAGE_NUMBER_TYPE     emCarriageNumber;    // carriage number
    BYTE                                byReserved[1024];    // reserved
}ALARM_SUB_WAY_DERA_OBST_INFO;

// the PECU call state Corresponding to DH_ALARM_SUB_WAY_PECU_CALL alarm
typedef struct tagALARM_SUB_WAY_PECU_CALL_INFO
{
    int                                 nAction;             // -1:unknown, 0:pulse, 1:start, 2:stop
    int					                nEventID;			 // Event ID
    double				                dbPTS;				 // Time stamp (Unit is ms)
    NET_TIME_EX			                stuTime;			 // Event occurrence time
    int                                 nCallNumber;         // call number
    BOOL                                bState;              // state,TRUE:alarming,FALSE:no alarming
    EM_SUB_WAY_CARRIAGE_NUMBER_TYPE     emCarriageNumber;    // carriage number
    BYTE                                byReserved[1024];    // reserved
}ALARM_SUB_WAY_PECU_CALL_INFO;

// door closed manually  Corresponding to DH_ALARM_DOOR_CLOSEDMANUALLY  alarm
typedef struct tagALARM_DOOR_CLOSEDMANUALLY_INFO
{
    int                                 nAction;             // -1:unknown 1:start 2:stop
    int					                nEventID;			 // Event ID
    double				                dbPTS;			     // Time stamp (Unit is ms)
    NET_TIME_EX			                stuTime;			 // Event occurrence time
    int                                 nDoor;               // door channel id
    char                                szDoorName[DH_MAX_DOORNAME_LEN]; //door name
    BYTE                                byReserved[1024];    // reserved
}ALARM_DOOR_CLOSEDMANUALLY_INFO;

// door not closed long time Corresponding to DH_ALARM_DOOR_NOTCLOSED_LONGTIME alarm
typedef struct tagALARM_DOOR_NOTCLOSED_LONGTIME_INFO
{
    int                                 nAction;             // -1:unknown 1:start 2:stop
    int					                nEventID;			 // Event ID
    double				                dbPTS;			     // Time stamp (Unit is ms)
    NET_TIME_EX			                stuTime;			 // Event occurrence time
    int                                 nDoor;               // door channel id
    char                                szDoorName[DH_MAX_DOORNAME_LEN]; // door name
    BYTE                                byReserved[1024];    // reserved
}ALARM_DOOR_NOTCLOSED_LONGTIME_INFO;

// the cut line alarm info, Corresponding to DH_ALARM_CUT_LINE alarm
typedef struct tagALARM_CUT_LINE_INFO
{
	int									nChannel;			 // Channel ID
	int                                 nAction;             // -1:unknown, 0:pulse, 1:start, 2:stop
    NET_TIME_EX			                stuTime;			 // Event occurrence time
    BYTE                                byReserved[1024];    // reserved
} ALARM_CUT_LINE_INFO;

// the fibre optic abort info,Corresponding to DH_ALARM_FIBRE_OPTIC_ABORT alarm
typedef struct tagALARM_FIBRE_OPTIC_ABORT
{
	int									nAlarmIn;			 // Alarm input NO.;
	int                                 nAction;             // 1:start, 2:stop 
	NET_TIME_EX							UTC;				 // Event occurrence time
	BYTE                                byReserved[1024];    // reserved
}ALARM_FIBRE_OPTIC_ABORT;

// near distance detection alarm infoCorresponding to DH_ALARM_NEAR_DISTANCE_DETECTION alarm
typedef struct tagALARM_NEAR_DISTANCE_INFO
{
	int									nChannel;			 // channel ID
	int                 				nAction;             // event action, 0: Pulse, 1: Start, 2: Stop
	int					                nEventID;			 // event ID
    double				                dbPTS;			     // PTS(ms)
    NET_TIME_EX			                stuTime;			 // time of occurrence
    BYTE                                byReserved[1024];    //  reserved
} ALARM_NEAR_DISTANCE_INFO;

// list of petsons within the region(man stand)
typedef struct tagMAN_STAND_LIST_INFO
{
	DH_POINT			stuCenter;			// standing personnel coordinate, 8192 coordinate system
	char                szReversed[128];    // reserved
} MAN_STAND_LIST_INFO;

// stereo standing alarm info, Corresponding to DH_ALARM_MAN_STAND_DETECTION alarm
typedef struct tagALARM_MAN_STAND_INFO
{
	int							nChannel;			 		// channel ID
	int                 		nAction;             		// event action, 0: Pulse, 1: Start, 2: Stop
	int					        nEventID;			 		// event ID
    double				        dbPTS;			     		// PTS(ms)
    NET_TIME_EX			        stuTime;			 		// time of occurrence

	int							nManListCount;				// number of regional personnet list
	MAN_STAND_LIST_INFO			stuManList[MAX_MAN_LIST_COUNT];	// list of petsons within the region
    BYTE                        byReserved[1024];    		// reserved
} ALARM_MAN_STAND_INFO;

// list of petsons within the region(man num)
typedef struct tagMAN_NUM_LIST_INFO
{
	DH_RECT				stuBoudingBox;			// personnel bounding box, 8192 coordinate system
	int					nStature;				// persoonel  height, unit cm
	char                szReversed[128];    	// reserved
} MAN_NUM_LIST_INFO;

// the statistics of people in stereo vision alarm info, Corresponding to DH_ALARM_MAN_NUM_DETECTION alarm
typedef struct tagALARM_MAN_NUM_INFO
{
	int							nChannel;			 		// channel ID
	int                 		nAction;             		// event action, 0: Pulse, 1: Start, 2: Stop
	int					        nEventID;			 		// event ID
    double				        dbPTS;			     		// PTS(ms)
    NET_TIME_EX			        stuTime;			 		// time of occurrence
    int							nManListCount;				// number of regional personnet list
	MAN_NUM_LIST_INFO			stuManList[MAX_MAN_LIST_COUNT];	// list of petsons within the region
    BYTE                        byReserved[1024];    		// reserved
} ALARM_MAN_NUM_INFO;


//tail detetcion info,Corresponding to DH_ALARM_TAIL_DETECTION alarm
typedef struct tagALARM_TAIL_DETECTION_INFO       
{     
	int									nAction;                   		// Event operation. 1: Start, 2: Stop
	int									nChannelID;						// Channel No.
    double								dbPTS;							// Time stamp (Unit is ms)
    NET_TIME_EX							stuTime;						// Event occurrence time 
    int									nEventID;						// Event ID
	BYTE                				byReserved[1024];   			// reserve text  
}ALARM_TAIL_DETECTION_INFO; 

//alarm when camera bitrate is over channel decoding specification, corresponding to DH_ALARM_BITRATES_OVERLIMIT alarm 
typedef struct tagALARM_BITRATES_OVERLIMIT_INFO        
{      
	int                                 nChannel;                       // video channel NO.          
	int                                 nAction;                        // event action,1:alarm start,2:alarm stop;   
	int                                 nLimitBps;                      // limit Bps,UNIT:Kbps 
	int                                 nCurrentBps;                    // Current Bps,UNIT:Kbps 
	BYTE                                byReserved[1024];               // reserved  
}ALARM_BITRATES_OVERLIMIT_INFO; 

//High decibel alarm info, corresponding to DH_ALARM_HIGH_DECIBEL alarm
typedef struct tagALARM_HIGH_DECIBEL_INFO
{
	int                                 nAudioChannel;                  // Audio channel NO. 
	int                                 nAction;                        // -1:unknown, 0:pulse, 1:start, 2:stop
	NET_TIME_EX                         stuTime;                        // Event occurrence time
    BYTE                                byReserved[1024];               // reserved
}ALARM_HIGH_DECIBEL_INFO; 

//Shake detection alarm info, corresponding to DH_ALARM_SHAKE_DETECTION alarm
typedef struct tagALARM_SHAKE_DETECTION_INFO
{          
	int                                 nAction;                        // -1:unknown, 0:pulse, 1:start, 2:stop   
    NET_TIME_EX                         stuTime;                        // Event occurrence time
	BYTE                                byReserved[1024];               // reserved    
}ALARM_SHAKE_DETECTION_INFO;

// access method
typedef enum tagNET_ACCESS_METHOD
{
    NET_ACCESS_METHOD_UNKNOWN,                                          // unknown
    NET_ACCESS_METHOD_CARD,                                             // card
    NET_ACCESS_METHOD_PASSWORD,                                         // password
    NET_ACCESS_METHOD_FINGERPRINT,                                      // fingerprint
}NET_ACCESS_METHOD;

//open door with malice,  corresponding to DH_ALARM_ACCESS_CTL_MALICIOUS
typedef struct tagALARM_ACCESS_CTL_MALICIOUS
{
    int                                 nAction;                        // -1:unknown, 0:pulse, 1:start, 2:stop   
    NET_TIME_EX                         stuTime;                        // event occurrence time
    NET_ACCESS_METHOD                   emMethod;                       // open door method
    char                                szSerialNum[MAX_COMMON_STRING_32];     // sn
    BYTE                                byReserved[1024];               // reserved  
}ALARM_ACCESS_CTL_MALICIOUS;

// user register alarm, corresponding to DH_ALARM_ACCESS_CTL_USERID_REGISTER
typedef struct tagALARM_ACCESS_CTL_USERID_REGISTER
{
    int                                 nAction;                        // -1:unknown, 0:pulse, 1:start, 2:stop   
    NET_TIME_EX                         stuTime;                        // event occurrence time
    char                                UserID[DH_USER_NAME_LEN_EX];    // user id
    NET_ACCESS_METHOD                   emMethod;                       // register method
    char                                szSerialNum[MAX_COMMON_STRING_32];     // intelligence lock serial number
    BYTE                                byReserved[1024];               // reserved 
}ALARM_ACCESS_CTL_USERID_REGISTER;

// lock type
typedef enum tagNET_ACCESS_CTL_LOCKTYPE
{
    NET_ACCESS_CTL_LOCKTYPE_UNKNOWN,                                    // unknown
    NET_ACCESS_CTL_LOCKTYPE_DEADBOLT,                                   // dead bolt
    NET_ACCESS_CTL_LOCKTYPE_STAYBOLT,                                   // stay bolt
}NET_ACCESS_CTL_LOCKTYPE;

// reverer lock alarm, corresponding to DH_ALARM_ACCESS_CTL_REVERSELOCK
typedef struct tagALARM_ACCESS_CTL_REVERSELOCK
{
    int                                 nAction;                        // -1:unknown, 0:pulse, 1:start, 2:stop     
    NET_TIME_EX                         stuTime;                        // event occurrence time
    BOOL                                bStatus;                        // lock status
    NET_ACCESS_CTL_LOCKTYPE             emMethod;                       // lock type
    char                                szSerialNum[MAX_COMMON_STRING_32];     // intelligence lock serial number
    BYTE                                byReserved[1024];               // reserved 
}ALARM_ACCESS_CTL_REVERSELOCK;

// delate user alarm , corresponding to DH_ALARM_ACCESS_CTL_USERID_DELETE
typedef struct tagALARM_ACCESS_CTL_USERID_DELETE
{
    int                                 nAction;                        // -1:unknown, 0:pulse, 1:start, 2:stop  
    NET_TIME_EX                         stuTime;                        // event occurrence time
    NET_ACCESS_METHOD                   emMethod;                       // register method
    char                                szSerialNum[MAX_COMMON_STRING_32];     // intelligence lock serial number
    BYTE                                byReserved[1024];               // reserved 
}ALARM_ACCESS_CTL_USERID_DELETE;

//event type DH_ALARM_ACCESS_DOOR_BELL (Door bell event ) corresponding data description info
typedef struct tagALARM_ACCESS_DOOR_BELL_INFO
{
	int                     nChannelID;                     			//channel ID,start from 0
	NET_TIME_EX             stuTime;                                 	//utc time
	char                    szName[DH_MAX_NAME_LEN];        		    // access control name
	char                    szWirelessDevSN[DH_MAX_ACCESS_NAME_LEN];    // wirelesss dev serial number
	BYTE			        bReserved[512];
}ALARM_ACCESS_DOOR_BELL_INFO;

//event type DH_ALARM_ACCESS_FACTORY_RESET (Access control reset factory event) corresponding data description info
typedef struct tagALARM_ACCESS_FACTORY_RESET_INFO
{
	NET_TIME_EX             stuTime;                        			//utc time
	char                    szSmartLockSN[DH_MAX_ACCESS_NAME_LEN];      //smart lock serial number
    BYTE			        bReserved[512];
}ALARM_ACCESS_FACTORY_RESET_INFO;


//event type DH_SWITCH_SCREEN (Switch Screen event) corresponding data description info
typedef struct tagDH_SWITCH_SCREEN_INFO
{
	int            nChannelID;                                    //channel ID, start from 0
	BYTE           bReserved[512];                               
}DH_SWITCH_SCREEN_INFO;

#define MAX_EVENT_INFO_LEN			128          // event info len

//	MCS general capacity low info, correspondging to DH_MCS_GENERAL_CAPACITY_LOW
typedef struct tagALARM_MCS_GENERAL_CAPACITY_LOW_INFO
{
	char		szEventInfo[MAX_EVENT_INFO_LEN];	       // Event info 
	BYTE		byReserved[1024];						   // Reserved
}ALARM_MCS_GENERAL_CAPACITY_LOW_INFO;

// MCS data node offline info,corresponding to DH_MCS_DATA_NODE_OFFLINE
typedef struct tagALARM_MCS_DATA_NODE_OFFLINE_INFO
{
	char		szEventInfo[MAX_EVENT_INFO_LEN];			// Event info 
	BYTE		byReserved[1024];							// Reserved
}ALARM_MCS_DATA_NODE_OFFLINE_INFO;

// MCS disk offline info, corresponding to DH_MCS_DISK_OFFLINE
typedef struct tagALARM_MCS_DISK_OFFLINE_INFO
{
	char		szEventInfo[MAX_EVENT_INFO_LEN];			// Event info 
	BYTE		byReserved[1024];							// Reserved
}ALARM_MCS_DISK_OFFLINE_INFO;

// MCS disk slow info,corresponding to DH_MCS_DISK_SLOW
typedef struct tagALARM_MCS_DISK_SLOW_INFO
{
	char		szEventInfo[MAX_EVENT_INFO_LEN];			// Event info
	BYTE		byReserved[1024];							// Reserved
}ALARM_MCS_DISK_SLOW_INFO;

// MCS disk broken info,corresponding to DH_MCS_DISK_BROKEN
typedef struct tagALARM_MCS_DISK_BROKEN_INFO
{
	char		szEventInfo[MAX_EVENT_INFO_LEN];			// Event info
	BYTE		byReserved[1024];							// Reserved
}ALARM_MCS_DISK_BROKEN_INFO;

// MCS disk unknow error info, corresponding to DH_MCS_DISK_UNKNOW_ERROR
typedef struct tagALARM_MCS_DISK_UNKNOW_ERROR_INFO
{
	char		szEventInfo[MAX_EVENT_INFO_LEN];			// Event info
	BYTE		byReserved[1024];							// Reserved
}ALARM_MCS_DISK_UNKNOW_ERROR_INFO;

// MCS metadata server abnormal info,corresponding to DH_MCS_METADATA_SERVER_ABNORMAL
typedef struct tagALARM_MCS_METADATA_SERVER_ABNORMAL_INFO
{
	char		szEventInfo[MAX_EVENT_INFO_LEN];			// Event info
	BYTE		byReserved[1024];							// Reserved
}ALARM_MCS_METADATA_SERVER_ABNORMAL_INFO;

// MCS catalog server abnormal info,corresponding to DH_MCS_CATALOG_SERVER_ABNORMAL
typedef struct tagALARM_MCS_CATALOG_SERVER_ABNORMAL_INFO
{
	char		szEventInfo[MAX_EVENT_INFO_LEN];			// Event info
	BYTE		byReserved[1024];							// Reserved
}ALARM_MCS_CATALOG_SERVER_ABNORMAL_INFO;

// MCS general capacity resume info,corresponding to DH_MCS_GENERAL_CAPACITY_RESUME
typedef struct tagALARM_MCS_GENERAL_CAPACITY_RESUME_INFO
{
	char		szEventInfo[MAX_EVENT_INFO_LEN];			// Event info
	BYTE		byReserved[1024];							// Reserved
}ALARM_MCS_GENERAL_CAPACITY_RESUME_INFO;

// MCS data node online info,corresponding to DH_MCS_DATA_NODE_ONLINE
typedef struct tagALARM_MCS_DATA_NODE_ONLINE_INFO
{
	char		szEventInfo[MAX_EVENT_INFO_LEN];			// Event info
	BYTE		byReserved[1024];							// Reserved
}ALARM_MCS_DATA_NODE_ONLINE_INFO;

// MCS disk online info,corresponding to DH_MCS_DISK_ONLINE
typedef struct tagALARM_MCS_DISK_ONLINE_INFO
{
	char		szEventInfo[MAX_EVENT_INFO_LEN];			// Event info
	BYTE		byReserved[1024];							// Reserved
}ALARM_MCS_DISK_ONLINE_INFO;

// MCS metadata slave online info,corresponding to DH_MCS_METADATA_SLAVE_ONLINE
typedef struct tagALARM_MCS_METADATA_SLAVE_ONLINE_INFO
{
	char		szEventInfo[MAX_EVENT_INFO_LEN];			// Event info
	BYTE		byReserved[1024];							// Reserved
}ALARM_MCS_METADATA_SLAVE_ONLINE_INFO;

// MCS catalog server online info,corresponding to DH_MCS_CATALOG_SERVER_ONLINE
typedef struct tagALARM_MCS_CATALOG_SERVER_ONLINE_INFO
{
	char		szEventInfo[MAX_EVENT_INFO_LEN];			// Event info
	BYTE		byReserved[1024];							// Reserved
}ALARM_MCS_CATALOG_SERVER_ONLINE_INFO;

// Cabinet switch action type 
typedef enum tagEM_NET_CABINET_SWITCH_ACTION_TYPE
{
    NET_CABINET_SWITCH_ACTION_TYPE_UNKNOWN,							// unknown
	NET_CABINET_SWITCH_ACTION_TYPE_OPEN,							// open cabinet
	NET_CABINET_SWITCH_ACTION_TYPE_CLOSE,							// close cabinet
} EM_NET_CABINET_SWITCH_ACTION_TYPE;

// max number of video surveillance channels associated with the storage device
#define MAX_LINK_CHANNEL_NUMBER		16

// Cloud Cabinet Event (DH_ALARM_CABINET)
typedef struct tagALARM_CABINET_INFO
{
    int									nAction;                                // 0:Pulse,1:Start,2:Stop,3:State
	NET_TIME_EX							stuTime;								// event occurrence time
	EM_NET_CABINET_SWITCH_ACTION_TYPE	emType;									// Cabinet switch action type 
	char								szSerialNo[MAX_COMMON_STRING_32];		// item serial number
	int									nLinkChannelNum;						// number of video surveillance channels associated with the storage device
	int									nLinkChannel[MAX_LINK_CHANNEL_NUMBER];	// store the video surveillance channels associated with the storage device
    BYTE								byReserved[1024];                       // reserved
} ALARM_CABINET_INFO;

//////////////////////////////////ATM support//////////////////////////////////

typedef struct
{
	int					Offset;					// Symbol position bit Offset
	int					Length;					// Symbol position length
	char				Key[16];				// Symbol position value 
} DH_SNIFFER_FRAMEID;

typedef struct 
{
	int					Offset;					// Symbol position bit offset 
	int					Offset2;				// It is invalid now 
	int					Length;					// The symbol position length
	int					Length2;				// It is invalid now
	char				Title[12];  			// Title value 
	char                Key[12];                // Key value
} DH_SNIFFER_CONTENT;

// Network sniffer setup 
typedef struct 
{
	DH_SNIFFER_FRAMEID	snifferFrameId;			// Each FRAME ID option
	DH_SNIFFER_CONTENT	snifferContent[DH_SNIFFER_CONTENT_NUM];	// The corresponding four sniffers in each FRAME
} DH_SNIFFER_FRAME;

// Configuration structure of each sniffer 
typedef struct
{
	char				SnifferSrcIP[DH_MAX_IPADDR_LEN];	// Sniffer source address 	
	int					SnifferSrcPort;						// Sniffer source port 
	char				SnifferDestIP[DH_MAX_IPADDR_LEN];	// Sniffer destination address 
	int					SnifferDestPort;					// Sniffer destination port 
	char				reserved[28];						// Reserved string 
	DH_SNIFFER_FRAME	snifferFrame[DH_SNIFFER_FRAMEID_NUM];// 6 FRAME options 
	int					displayPosition;					// Display position 
	int					recdChannelMask;					// Channel mask
} DH_ATM_SNIFFER_CFG;

typedef struct  
{
	DWORD				dwSize;
	DH_ATM_SNIFFER_CFG	SnifferConfig[4];
	char				reserved[256];						// Reserved string 
} DHDEV_SNIFFER_CFG;

typedef DH_SNIFFER_FRAMEID DH_SNIFFER_FRAMEID_EX;
typedef DH_SNIFFER_CONTENT DH_SNIFFER_CONTENT_EX;

// Capture network configuration
typedef struct  
{
	DH_SNIFFER_FRAMEID	snifferFrameId;								// Each FRAME ID Options
	DH_SNIFFER_CONTENT	snifferContent[DH_SNIFFER_CONTENT_NUM_EX];	// Each FRAME Corresponding to the contents of capture	
} DH_SNIFFER_FRAME_EX;

// Capture each of the corresponding structure
typedef struct
{
	char				SnifferSrcIP[DH_MAX_IPADDR_LEN];					// Source address capture		
	int					SnifferSrcPort;										// Capture source port
	char				SnifferDestIP[DH_MAX_IPADDR_LEN];					// Destination address capture
	int					SnifferDestPort;									// Capture the target port
	DH_SNIFFER_FRAME_EX	snifferFrame[DH_SNIFFER_FRAMEID_NUM];				// 6 FRAME Options
	int					displayPosition;									// Display Position
	int					recdChannelMask;									// Channel mask
	BOOL				bDateScopeEnable;									// Data sources enable
	BOOL				bProtocolEnable;									// Protocol enable
	char				szProtocolName[DH_SNIFFER_PROTOCOL_SIZE];			// Protocol name
	int					nSnifferMode;										// Capture mode; 0:net,1:232.
	int					recdChannelMask1;									// Channel submask  32 ~ 63 channel
	char				reserved[252];
} DH_ATM_SNIFFER_CFG_EX;

// Atm trade type
#define ATM_MAX_TRADE_TYPE_NAME	64
#define ATM_MAX_TRADE_NUM		1024

typedef struct __DH_ATM_QUERY_TRADE   
{
	int					nTradeTypeNum;										// number of trade types
	int					nExceptionTypeNum;									// number of exception events
	char				szSupportTradeType[ATM_MAX_TRADE_NUM][ATM_MAX_TRADE_TYPE_NAME];    // trade events
	char				szSupportExceptionType[ATM_MAX_TRADE_NUM][ATM_MAX_TRADE_TYPE_NAME];// exception events
} DH_ATM_QUERY_TRADE, *LPDH_ATM_QUERY_TRADE;

/////////////////////////////////NVD support/////////////////////////////////
#define nEncoderID nDecoderID
#define byEncoderID byDecoderID

// Decoder information
typedef struct __DEV_DECODER_INFO 
{
	char			szDecType[64];			// type
	int				nMonitorNum;			// TV number
	int				nEncoderNum;			// Decoder channel number
	BYTE			szSplitMode[16];		// Supported by a number of TV screen partition
	BYTE            bMonitorEnable[16];		// TV enable
	BYTE            bTVTipDisplay;          // TV tip display enable 0:not support 1:support.
	BYTE            reserved1[3];
	BYTE            byLayoutEnable[48];     // every channel's tip display enable
    DWORD           dwLayoutEnMask[2];      // Each decoding channel displays overlay info enable mask, from low to high support 64 channels, while dwLayoutEnMask[0] is low 32 bit
	char			reserved[4];
} DEV_DECODER_INFO, *LPDEV_DECODER_INFO;

// Encoder information
#ifndef NANJINGDITIE_NVD
// device encoder information
typedef struct __DEV_ENCODER_INFO 
{
	char			szDevIp[DH_MAX_IPADDR_LEN];			// IP address of Front-end DVR 
	WORD			wDevPort;							// Port of Front-end DVR
	BYTE			bDevChnEnable;                      // Decoder channel enable
	BYTE			byDecoderID;						// deprecated, to get same info, use dwDecoderID instead
	char			szDevUser[DH_USER_NAME_LENGTH_EX];	// User Name
	char			szDevPwd[DH_USER_PSW_LENGTH_EX];	// Password
	int				nDevChannel;						// Channel Number
	int				nStreamType;						// Stream type; 0:Main Stream, 1:Sub-stream
	BYTE			byConnType;							// -1: auto, 0:TCP, 1:UDP, 2:Multicast
	BYTE			byWorkMode;							// 0:Direct Connect, 1:transmit
	WORD			wListenPort;						// Listening port services, for transmit
	DWORD			dwProtoType;						// Protocol type
														// 0:compatible with each other.
														// 1:private 2nd protocol
														// 2:private system integration protocol
														// 3:private DSS protocol
														// 4:private rtsp protocol
	char			szDevName[64];						// Front device name
	BYTE            byVideoInType;                      // video source type:0-SD,1-HD		
	char			szDevIpEx[DH_MAX_IPADDR_OR_DOMAIN_LEN];// szDevIp extended, IP or domain name
	BYTE            bySnapMode;                         // snap mode(when nStreamType==2 effective) 0:That request a frame,1:Time to send a request
	BYTE            byManuFactory;						// The target device manufacturers,See the enum struct EM_IPC_TYPE
	BYTE            byDeviceType;                       // The target device's device type,0:IPC
	BYTE            byDecodePolicy;                     // The target device's decode policy
														// 0:LatencyLevel3 1:LatencyLevel2
														// 2:LatencyLevel1 3:MiddleLevel
														// 4:FluencyLevel3 5:FluencyLevel2
														// 6:FluencyLevel1
	BYTE            bReserved[3];                       // reserved
	DWORD           dwHttpPort;                         // http port 0-65535
	DWORD           dwRtspPort;                         // RTSP port 0-65535
	char			szChnName[32];						// remote channel name
	DWORD           dwDecoderID;                        // The corresponding channel number decoder
} DEV_ENCODER_INFO, *LPDEV_ENCODER_INFO;

#else

// encoder info
typedef struct __DEV_ENCODER_INFO 
{
	char			szDevIp[DH_MAX_IPADDR_LEN];			// IP address of Front-end DVR 
	WORD			wDevPort;							// Port of Front-end DVR
	BYTE			bDevChnEnable;                      // Decoder channel enable
	BYTE			byDecoderID;						// deprecated, to get same info, use dwDecoderID instead
	char			szDevUser[DH_USER_NAME_LENGTH_EX];	// User Name
	char			szDevPwd[DH_USER_PSW_LENGTH_EX];	// Password
	int				nDevChannel;						// Channel Number
	int				nStreamType;						// Stream type; 0:Main Stream, 1:Sub-stream
	BYTE			byConnType;							// -1: auto, 0:TCP, 1:UDP, 2:Multicast
	BYTE			byWorkMode;							// 0:Direct Connect, 1:transmit
	WORD			wListenPort;						// Listening port services, for transmit
	DWORD			dwProtoType;						// Protocol type
														// 0:compatible with each other.
														// 1:private 2nd protocol
														// 2:private system integration protocol
														// 3:private DSS protocol
														// 4:private rtsp protocol
	char			szDevName[32];						// Front device name
	BYTE            byVideoInType;                      // video source type:0-SD,1-HD		
	char			szDevIpEx[DH_MAX_IPADDR_OR_DOMAIN_LEN];// szDevIp extended, IP or domain name
	BYTE            bySnapMode;                         // snap mode(when nStreamType==2 effective) 0:That request a frame,1:Time to send a request
	BYTE            byManuFactory;						// The target device manufacturers,See the enum struct EM_IPC_TYPE
	BYTE            byDeviceType;                       // The target device's device type,0:IPC
	BYTE            byDecodePolicy;                     // The target device's decode policy
														// 0:LatencyLevel3 1:LatencyLevel2
														// 2:LatencyLevel1 3:MiddleLevel
														// 4:FluencyLevel3 5:FluencyLevel2
														// 6:FluencyLevel1
	BYTE            bReserved[3];                       // reserved
	DWORD           dwHttpPort;                         // http port 0-65535
	DWORD           dwRtspPort;                         // RTSP port 0-65535
	char			szChnName[32];						// remote channel name
	char			szMcastIP[DH_MAX_IPADDR_LEN];       // multicast address
    DWORD           dwDecoderID;                        // The corresponding channel number decoder
	char            reserved[124];
} DEV_ENCODER_INFO, *LPDEV_ENCODER_INFO;

#endif

// decoder protocol rtsp url configuration
typedef struct __DHDEV_DECODER_URL_CFG
{
	DWORD			dwSize;
	char			szMainStreamUrl[MAX_PATH];			// main stream url
	char			szExtraStreamUrl[MAX_PATH];			// extra stream url
} DHDEV_DECODER_URL_CFG;

typedef enum tagDH_SPLIT_DISPLAY_TYPE
{
    DH_SPLIT_DISPLAY_TYPE_GENERAL=1,          // Common display types
    DH_SPLIT_DISPLAY_TYPE_PIP=2,              // PIP Display Type
    DH_SPLIT_DISPLAY_TYPE_CUSTOM=3,           // Custom Display Type
} DH_SPLIT_DISPLAY_TYPE;

// CLIENT_CtrlDecTVScreen Interface parameters
typedef struct tagDH_CTRL_DECTV_SCREEN
{
    DWORD           dwSize;                             // The size of the structure
    int             nSplitType;                         // Split mode
    BYTE *          pEncoderChannel;                    // Display Channel, dwDisplayType = DH_SPLIT_DISPLAY_TYPE_GENERAL when Effect
														// the space application by the user, the caller can not be less than the length of the allocated memory size nSplitType
    BYTE            byGroupNo;                          // Input channel group No, when dwDisplayType = DH_SPLIT_DISPLAY_TYPE_PIP said PIP display types are valid
    char            reserved[3];                        // Reserved
    DWORD           dwDisplayType;                      // Display type; see specific display DH_SPLIT_DISPLAY_TYPE (note each mode Content is determined by the "PicInPic", the contents of each mode is displayed by the old rules NVD decision (ie DisChn field decision). Compatible, without which an item, the default display for the general category Type, i.e., "General"
}DH_CTRL_DECTV_SCREEN;

// TV parameters 
typedef struct __DEV_DECODER_TV 
{
	int				nID;								// TV ID
	BOOL			bEnable;							// Enable, open or close
	int				nSplitType;							// Partition number
	DEV_ENCODER_INFO stuDevInfo[16];					// All encoder information
	BYTE			bySupportSplit[10];					// Split mode supported
    BYTE            byGroupNo;     						// Input channel group No, when dwDisplayType = DH_SPLIT_DISPLAY_TYPE_PIP said PIP display types are valid
    char			reserved[1];                        // Reserved bytes
    DWORD           dwDisplayType;                      // Display type; see specific display DH_SPLIT_DISPLAY_TYPE (note each mode 
                                                        // Content is determined by the "PicInPic", the contents of each mode is displayed by the old rules NVD decision (ie DisChn field decision). Compatible, without which an item, the default display for the general category 
                                                        // Type, i.e., "General")

} DEV_DECODER_TV, *LPDEV_DECODER_TV;

// Screen combination of information
typedef struct __DEC_COMBIN_INFO
{
	int				nCombinID;							// Combin ID
	int             nSplitType;							// Partition number
	BYTE            bDisChn[16];						// Display channel
	char			reserved[16];
} DEC_COMBIN_INFO, *LPDEC_COMBIN_INFO;

// Tour Information
#define DEC_COMBIN_NUM 			32						// the number of tour combination
typedef struct __DEC_TOUR_COMBIN 
{
	int				nTourTime;							// Tour Interval(s)
	int				nCombinNum;							// the number of combination
	BYTE			bCombinID[DEC_COMBIN_NUM];			// Combination Table
	char			reserved1[32];
	BYTE			bCombinState[DEC_COMBIN_NUM];		// Combination option state,0:close;1:open
	char			reserved2[32];
} DEC_TOUR_COMBIN, *LPDEC_TOUR_COMBIN;

// Decoder Playback type
typedef enum __DEC_PLAYBACK_MODE
{
	Dec_By_Device_File = 0,								// Front-end DVR--By File
	Dec_By_Device_Time,									// Front-end DVR--By Time
} DEC_PLAYBACK_MODE;

// Decoder Playback control type
typedef enum __DEC_CTRL_PLAYBACK_TYPE
{
	Dec_Playback_Seek = 0,								// Drag
	Dec_Playback_Play,									// Play
	Dec_Playback_Pause,									// Pause
	Dec_Playback_Stop,									// Stop
} DEC_CTRL_PLAYBACK_TYPE;

// tour ctrol type
typedef enum __DEC_CTRL_TOUR_TYPE
{
	Dec_Tour_Stop = 0,									// stop
	Dec_Tour_Start,										// start
	Dec_Tour_Pause,										// pause
	Dec_Tour_Resume,									// resume
} DEC_CTRL_TOUR_TYPE;

// Playback by file Conditions
typedef struct __DEC_PLAYBACK_FILE_PARAM 
{
	char			szDevIp[DH_MAX_IPADDR_LEN];			// IP address of Front-end DVR
	WORD			wDevPort;							// Port of Front-end DVR
	BYTE			bDevChnEnable;                      // Decoder channel enable
	BYTE			byDecoderID;						// The corresponding channel number
	char			szDevUser[DH_USER_NAME_LENGTH_EX];	// User Name
	char			szDevPwd[DH_USER_PSW_LENGTH_EX];	// Password
	NET_RECORDFILE_INFO stuRecordInfo;					// Record file information
	char			reserved[12];
} DEC_PLAYBACK_FILE_PARAM, *LPDEC_PLAYBACK_FILE_PARAM;

// Playback by time Conditions
typedef struct __DEC_PLAYBACK_TIME_PARAM 
{
	char			szDevIp[DH_MAX_IPADDR_LEN];			// IP address of Front-end DVR
	WORD			wDevPort;							// Port of Front-end DVR
	BYTE			bDevChnEnable;                      // Decoder channel enable
	BYTE			byDecoderID;						// The corresponding channel number
	char			szDevUser[DH_USER_NAME_LENGTH_EX];	// User Name
	char			szDevPwd[DH_USER_PSW_LENGTH_EX];	// Password
	int				nChannelID;
	NET_TIME		startTime;
	NET_TIME		endTime;
	char			reserved[12];
} DEC_PLAYBACK_TIME_PARAM, *LPDEC_PLAYBACK_TIME_PARAM;

// Current decoding channel status(including channel status, decoding stream info and etc.)
typedef struct __DEV_DECCHANNEL_STATE
{
	BYTE			byDecoderID;						// Responding decoding channel number
	BYTE            byChnState;                         // Current decoding channel in opertion status:0 -free,1 -realtime monitoring,2 - playback 3 - Decode Tour
	BYTE			byFrame;                            // Current data frame rate
	BYTE            byReserved;                         // Reserved
	int				nChannelFLux;						// Decoding channel data amount
	int             nDecodeFlux;						// Decoding data amount
	char            szResolution[16];                   // Current data resolution
	char			reserved[256];
} DEV_DECCHANNEL_STATE, *LPDEV_DECCHANNEL_STATE;

// Device TV display info
typedef struct __DEV_VIDEOOUT_INFO
{
	DWORD				dwVideoStandardMask;			// NSP,video standard mask,according to the bit which shows video format(not support now)
	int					nVideoStandard;                 // NSP,current format(not support now,please use DHDEV_SYSTEM_ATTR_CFG by VideoStandard to read and config the video format)
	DWORD				dwImageSizeMask;				// Resolution mask,according the bit which shows video resolution
	int                 nImageSize;                     // Current resolution
	char				reserved[256];
}DEV_VIDEOOUT_INFO, *LPDEV_VIDEOOUT_INFO;

// TV adjust
typedef struct __DEV_TVADJUST_CFG
{
	int					nTop;							// top(0 - 100)
	int					nBotton;						// botton(0 - 100)
	int					nLeft;							// left(0 - 100)
	int                 nRight;							// right(0 - 100)
	int					reserved[32];
}DHDEV_TVADJUST_CFG, *LPDHDEV_TVADJUST_CFG;

//PTZ control coordinate unit 
typedef struct tagPTZ_SPEED_UNIT
{
    float                  fPositionX;           //PTZ horizontal speed, normalized to -1~1 
    float                  fPositionY;           //PTZ vertical speed, normalized to -1~1 
    float                  fZoom;                //PTZ aperture magnification, normalized to 0~1 
    char                   szReserve[32];        //Reserved
}PTZ_SPEED_UNIT;

//PTZ control coordinate unit 
typedef struct tagPTZ_SPACE_UNIT
{
    int                    nPositionX;           //PTZ horizontal motion position, effective range:[0,3600]
    int                    nPositionY;           //PTZ vertical motion position, effective range:[-1800,1800]
    int                    nZoom;                //PTZ aperture change position, the effective range:[0,128]
    char                   szReserve[32];        //Reserved
}PTZ_SPACE_UNIT;

// decoder tour configuration
typedef struct __DEV_DECODER_TOUR_SINGLE_CFG
{
	char		szIP[128];								// Front-end device's ip.Such as"10.7.5.21". Support retention of the domain name, end by '\0'.
	int			nPort;									// Front-end device's port.(0, 65535).
	int			nPlayChn;								// front-end device's Request channel [1, 16].
	int			nPlayType;								// front-end device's Stream type, 0:main; 1:sub.
	char		szUserName[32];							// front-end device's user name,end by '\0'.
	char		szPassword[32];							// front-end device's Password,end by '\0'.
	int			nInterval;								// Round tour interval [10, 120],s.
	DWORD		nManuFactory;							// device's factory(enum see IPC_TYPE)
	UINT		nHttpPport;								// device's http port
	UINT		nRtspPort;								// device's rtsp port
	BYTE		byServiceType;							// service type -1:auto,0:TCP;1:UDP;2:Multicast 
	BYTE		bReserved[51];							// reserved.
}DHDEV_DECODER_TOUR_SINGLE_CFG;

typedef struct __DEV_DECODER_TOUR_CFG
{
	int								nCfgNum;			// the number of Configure structures. The biggest support 32. Specific number of support inquiries through capacity.
	DHDEV_DECODER_TOUR_SINGLE_CFG	tourCfg[64];		// Polling configuration array, the effective number of structures by the members of the "nCfgNum" designation. Keep left to expand 32.
	BYTE							bReserved[256];		// reserved.
}DHDEV_DECODER_TOUR_CFG;

/////////////////////////////////intelligent support/////////////////////////////////
// picture info
typedef struct  
{
	DWORD                dwOffSet;                      // current picture file's offset in the binary file, byte
	DWORD                dwFileLenth;                   // current picture file's size, byte
	WORD                 wWidth;                        // picture width, pixel
	WORD                 wHeight;                       // picture high, pixel
    char*                pszFilePath;                   // File path
                                                        // User use this field need to apply for space for copy and storage
    BYTE            	 bIsDetected;					// When submit to the server, the algorithm has checked the image or not 
	BYTE				 bReserved[7];					// 12<--16
	DH_POINT			 stuPoint;						// The upper left corner of the figure is in the big picture. Absolute coordinates are used				
}DH_PIC_INFO;

// Object corresponding to picture file information (including images path)
typedef struct tagDH_PIC_INFO_EXX
{
    DWORD           dwSize;
    char            szFilePath[DH_COMMON_STRING_256];   // In view of historical reasons, the member is valid when be set or to get
                                                        // File path
    DH_PIC_INFO     stuPicInfo;                         // picture information
}DH_PIC_INFO_EX2;

// picture info
typedef struct  
{
	DWORD                dwOffSet;                      // current picture file's offset in the binary file, byte
	DWORD                dwFileLenth;                   // current picture file's size, byte
	WORD                 wWidth;                        // picture width, pixel
	WORD                 wHeight;                       // picture high, pixel
 	char                 szFilePath[64];                // File path
    BYTE            	 bIsDetected;					// When submit to the server, the algorithm has checked the image or not 
    BYTE            	 bReserved[11];					// Reserved
}DH_PIC_INFO_EX3;

// Extension fields added int64, forced 4 byte alignment
#pragma pack(push)
#pragma pack(4)

// Struct of object info for video analysis 
typedef struct 
{
	int					nObjectID;						// Object ID,each ID represent a unique object
	char				szObjectType[128];				// Object type
	int					nConfidence;					// Confidence(0~255),a high value indicate a high confidence
	int					nAction;						// Object action:1:Appear 2:Move 3:Stay 4:Remove 5:Disappear 6:Split 7:Merge 8:Rename
	DH_RECT				BoundingBox;					// BoundingBox
	DH_POINT			Center;							// The shape center of the object
	int					nPolygonNum;					// the number of culminations for the polygon
	DH_POINT			Contour[DH_MAX_POLYGON_NUM];	// a polygon that have a exactitude figure
	DWORD				rgbaMainColor;					// The main color of the object;the first byte indicate red value, as byte order as green, blue, transparence, for example:RGB(0,255,0),transparence = 0, rgbaMainColor = 0x00ff0000.
    char				szText[128];					// the interrelated text of object,such as number plate,container number
                                                        // "ObjectType","Vehicle" or "Logo", try to use Logo.Vehicle is used to be compatible with old product, means logo, support:
                                                        // "Unknown" Unknown 
                                                        // "Audi" Audi
                                                        // "Honda" Honda
                                                        // "Buick" Buick
                                                        // "Volkswagen" Volkswagen
                                                        // "Toyota" Toyota
                                                        // "BMW" BMW
                                                        // "Peugeot" Peugeot
                                                        // "Ford" Ford
                                                        // "Mazda" Mazda
                                                        // "Nissan" Nissan
                                                        // "Hyundai" Hyundai
                                                        // "Suzuki" Suzuki
                                                        // "Citroen" Citroen
                                                        // "Benz" Benz
                                                        // "BYD" BYD
                                                        // "Geely" Geely
                                                        // "Lexus" Lexus
                                                        // "Chevrolet" Chevrolet
                                                        // "Chery" Chery
                                                        // "Kia" Kia
                                                        // "Charade" Charade
                                                        // "DF" DF
                                                        // "Naveco" Naveco
                                                        // "SGMW" SGMW
                                                        // "Jinbei" Jinbei

                                                        // "JAC" JAC
                                                        // "Emgrand" Emgrand
                                                        // "ChangAn" ChangAn
                                                        // "Great Wall" Great Wall
                                                        // "Skoda" Skoda
                                                        // "BaoJun" BaoJun
                                                        // "Subaru" Subaru
                                                        // "LandWind" LandWind
                                                        // "Luxgen" Luxgen
                                                        // "Renault" Renault
                                                        // "Mitsubishi" Mitsubishi
                                                        // "Roewe" Roewe
                                                        // "Cadillac" Cadillac
                                                        // "MG" MG
                                                        // "Zotye" Zotye
                                                        // "ZhongHua" ZhongHua
                                                        // "Foton" Foton
                                                        // "SongHuaJiang" SongHuaJiang
                                                        // "Opel" Opel
                                                        // "HongQi" HongQi
                                                        // "Fiat" Fiat
                                                        // "Jaguar" Jaguar
                                                        // "Volvo" Volvo
                                                        // "Acura" Acura
                                                        // "Porsche" Porsche

                                                        // "Jeep" Jeep
                                                        // "Bentley" Bentley
                                                        // "Bugatti" Bugatti
                                                        // "ChuanQi" ChuanQi
                                                        // "Daewoo" Daewoo
                                                        // "DongNan" DongNan
                                                        // "Ferrari" Ferrari
                                                        // "Fudi" Fudi
                                                        // "Huapu" Huapu
                                                        // "HawTai" HawTai
                                                        // "JMC" JMC
                                                        // "JingLong" JingLong
                                                        // "JoyLong" JoyLong
                                                        // "Karry" Karry"
                                                        // "Chrysler" Chrysler
                                                        // "Lamborghini" Lamborghini
                                                        // "RollsRoyce" RollsRoyce
                                                        // "Linian" Linian
                                                        // "LiFan" LiFan
                                                        // "LieBao" LieBao
                                                        // "Lincoln" Lincoln
                                                        // "LandRover" LandRover
                                                        // "Lotus" Lotus
                                                        // "Maserati" Maserati
                                                        // "Maybach" Maybach

                                                        // "Mclaren" Mclaren
                                                        // "Youngman" Youngman
                                                        // "Tesla" Tesla
                                                        // "Rely" Rely
                                                        // "Lsuzu" Lsuzu
                                                        // "Yiqi" Yiqi
                                                        // "Infiniti" Infiniti
                                                        // "YuTong" YuTong
                                                        // "AnKai" AnKai
                                                        // "Canghe" Canghe
                                                        // "HaiMa" HaiMa
                                                        // "Crown" Crown
                                                        // "HuangHai" HuangHai
                                                        // "JinLv" JinLv
                                                        // "JinNing" JinNing
                                                        // "KuBo" KuBo
                                                        // "Europestar" Europestar
                                                        // "MINI" MINI
                                                        // "Gleagle" Gleagle
                                                        // "ShiDai" ShiDai
                                                        // "ShuangHuan" ShuangHuan
                                                        // "TianYe" TianYe
                                                        // "WeiZi" WeiZi
                                                        // "Englon" Englon
                                                        // "ZhongTong" ZhongTong

                                                        // "Changan" Changan
                                                        // "Yuejin" Yuejin
                                                        // "Taurus" Taurus
                                                        // "Alto" Alto
                                                        // "Weiwang" Weiwang
                                                        // "Chenglong" Chenglong
                                                        // "Haige" Haige
                                                        // "Shaolin" Shaolin
                                                        // "Beifang" Beifang
                                                        // "Beijing" Beijing
                                                        // "Hafu" Hafu
														
														// "BeijingTruck" BeiQi
														// "Besturn" Besturn
														// "ChanganBus" Changan
														// "Dodge" Dodge
														// "DongFangHong" Dongfanghong
														// "DongFengTruck" DongFeng
														// "DongFengBus" DongFeng
														// "MultiBrand" MutiBrand
														// "FotonTruck" Foton
														// "FotonBus" Foton
														// "GagcTruck" Gagc
														// "HaFei" HaFei
														// "HowoBus" Howo
														// "JACTruck" JAC
														// "JACBus" JAC
														// "JMCTruck" JMC
														// "JieFangTruck" JieFang
														// "JinBeiTruck" JinBei
														// "KaiMaTruck" KaiMa
														// "CoasterBus" Coaster
														// "MudanBus" Mudan
														// "NanJunTruck" NanJun
														// "QingLing" QingLing
														// "NissanCivilian" Nissan Civilian
														// "NissanTruck" Nissan
														// "MitsubishiFuso" Mitsubishi Fuso
														// "SanyTruck" Sany
														// "ShanQiTruck" ShanQi
														// "ShenLongBus" ShenLong
														// "TangJunTruck" TangJun
														// "MicroTruck" Micro
														// "VolvoBus" Volvo
														// "LsuzuTruck" Lsuzu
														// "WuZhengTruck" WuZheng
														// "Seat" Seat
														// "YangZiBus" YangZi
														// "YiqiBus" Yiqi
														// "YingTianTruck" YingTian
														// "YueJinTruck" YueJin
														// "ZhongDaBus" ZhongDa
														// "ZxAuto" ZxAuto
														// "ZhongQiWangPai" ZhongQiWangPai
														// "WAW" WAW
														// "BeiQiWeiWang" BeiQiWeiWang
														// "BYDDaimler"	BYDDaimler
														// "ChunLan" ChunLan
														// "DaYun" DaYun
														// "DFFengDu" DFFengDu
														// "DFFengGuang" DFFengGuang
														// "DFFengShen" DFFengShen
														// "DFFengXing" DFFengXing
														// "DFLiuQi" DFLiuQi
														// "DFXiaoKang" DFXiaoKang
														// "FeiChi" FeiChi
														// "FordMustang" FordMustang
														// "GuangQi" GuangQi
														// "GuangTong" GuangTong
														// "HuiZhongTruck" HuiZhongTruck
														// "JiangHuai" JiangHuai
														// "SunWin" SunWin
														// "ShiFeng" ShiFeng
														// "TongXin" TongXin
														// "WZL" WZL
														// "XiWo" XiWo
														// "XuGong" XuGong
														// "JingGong" JingGong
														// "SAAB" SAAB
														// "SanHuanShiTong" SanHuanShiTong
														// "KangDi" KangDi
														// "YaoLong" YaoLong

	char                szObjectSubType[64];            // object sub type,different object type has different sub type:
														// Vehicle Category:"Unknown","Motor","Non-Motor","Bus","Bicycle","Motorcycle"
														// Plate Category:"Unknown","mal","Yellow","DoubleYellow","Police","Armed",
														// "Military","DoubleMilitary","SAR","Trainning"
														// "Personal" ,"Agri","Embassy","Moto","Tractor","Other"
														// HumanFace Category:"Normal","HideEye","HideNose","HideMouth","TankCar"
    WORD                wSubBrand;                      // Specifies the sub-brand of vehicle,the real value can be found in a mapping table from the development manual 
	BYTE                byReserved1;                 
	bool                bPicEnble;                      // picture info enable
	DH_PIC_INFO         stPicInfo;                      // picture info
	bool				bShotFrame;						// is shot frame
	bool				bColor;							// rgbaMainColor is enable
	BYTE				byReserved2;
	BYTE                byTimeType;                     // Time indicates the type of detailed instructions, EM_TIME_TYPE
    NET_TIME_EX			stuCurrentTime;					// in view of the video compression,current time(when object snap or reconfnition, the frame will be attached to the frame in a video or pictures,means the frame in the original video of the time)
	NET_TIME_EX			stuStartTime;					// strart time(object appearing for the first time)
	NET_TIME_EX			stuEndTime;						// end time(object appearing for the last time)
	DH_RECT				stuOriginalBoundingBox;			// original bounding box(absolute coordinates)
	DH_RECT             stuSignBoundingBox;             // sign bounding box coordinate
	DWORD				dwCurrentSequence;				// The current frame number (frames when grabbing the object)
	DWORD				dwBeginSequence;				// Start frame number (object appeared When the frame number, 
	DWORD				dwEndSequence;					// The end of the frame number (when the object disappearing Frame number)
	INT64				nBeginFileOffset;				// At the beginning of the file offset, Unit: Word Section (when objects began to appear, the video frames in the original video file offset relative to the beginning of the file, 
	INT64				nEndFileOffset;					// At the end of the file offset, Unit: Word Section (when the object disappeared, video frames in the original video file offset relative to the beginning of the file)
	BYTE                byColorSimilar[NET_COLOR_TYPE_MAX];// Object color similarity, the range :0-100, represents an array subscript Colors, see EM_COLOR_TYPE, 
 	BYTE                byUpperBodyColorSimilar[NET_COLOR_TYPE_MAX]; // When upper body color similarity (valid object type man , 
 	BYTE                byLowerBodyColorSimilar[NET_COLOR_TYPE_MAX]; // Lower body color similarity when objects (object type human valid , 
    int                 nRelativeID;                        // ID of relative object
	char				szSubText[20];						// "ObjectType"is "Vehicle" or "Logo",  means a certain brand under LOGO, such as Audi A6L, since there are so many brands, SDK sends this field in real-time ,device filled as real.
    WORD                wBrandYear;                         // Specifies the model years of vehicle. the real value can be found in a mapping table from the development manual 
} DH_MSG_OBJECT;

// intrusion direction
typedef enum tagEM_MSG_OBJ_PERSON_DIRECTION
{
    EM_MSG_OBJ_PERSON_DIRECTION_UNKOWN,         // unknown direction
    EM_MSG_OBJ_PERSON_DIRECTION_LEFT_TO_RIGHT,  // from left to right
    EM_MSG_OBJ_PERSON_DIRECTION_RIGHT_TO_LEFT   // from right ro left
}EM_MSG_OBJ_PERSON_DIRECTION;

// Video analysis object info expansion structure 
typedef struct tagDH_MSG_OBJECT_EX
{
    DWORD               dwSize;
    int                 nObjectID;                  // object ID, each ID means a exclusive object
    char                szObjectType[128];          // object  type 
    int                 nConfidence;                // confidence coefficient (0~255),  value the bigger means  confidence coefficient the higher
    int                 nAction;                    // object  motion :1:Appear 2:Move 3:Stay 4:Remove 5:Disappear 6:Split 7:Merge 8:Rename
    DH_RECT             BoundingBox;                // box
    DH_POINT            Center;                     // object model center
    int                 nPolygonNum;                // polygon vertex number 
    DH_POINT            Contour[DH_MAX_POLYGON_NUM];// relatively accurate outline the polygon  
    DWORD               rgbaMainColor;              // means plate, vehicle body and etc. object major color, by byte means , are red, green, blue and transparency , such as:RGB value is (0,255,0), transparency is 0, its value is 0x00ff0000.
    char                szText[128];                // same as DH_MSG_OBJECT corresponding field
    char                szObjectSubType[64];        // object sub type , according to different object  types , may use the following sub type :
                                                    // same as DH_MSG_OBJECT field
    BYTE                byReserved1[3];
    bool                bPicEnble;                  // object corresponding to picture file info or not
    DH_PIC_INFO         stPicInfo;                  // object corresponding to picture info 
    bool                bShotFrame;                 // snapshot recognition result or not 
    bool                bColor;                     // object  color (rgbaMainColor) usable or not
    BYTE                bLowerBodyColor;            // lower color (rgbaLowerBodyColor) usable or not
    BYTE                byTimeType;                 // time means type ,  see EM_TIME_TYPE note 
    NET_TIME_EX         stuCurrentTime;             // for video compression,  current time stamp, object snapshot or recognition,  attach this recognition frame in one vire frame or jpegpicture, this frame appearance time in original video, 
    NET_TIME_EX         stuStartTime;               // start time stamp, object start appearance, 
    NET_TIME_EX         stuEndTime;                 // end time stamp, object last aapearance, 
    DH_RECT             stuOriginalBoundingBox;     // box(absolute coordinate)
    DH_RECT             stuSignBoundingBox;         // LGO coordinate box
    DWORD               dwCurrentSequence;          // current frame no., snapshot this object frame, 
    DWORD               dwBeginSequence;            // start frame no., object start appearance frame no., 
    DWORD               dwEndSequence;              // end frame no., object disappearance frame no., 
    INT64               nBeginFileOffset;           // start file shift, unit: byte, object start appearance, video in original video file moves toward file origin, 
    INT64               nEndFileOffset;             // End file shift, unit: byte, object disappearance, video in original video file moves toward file origin, 
    BYTE                byColorSimilar[NET_COLOR_TYPE_MAX]; // object  color similarity, take  value range :0-100, group subscript value represents certain color ,  see EM_COLOR_TYPE
    BYTE                byUpperBodyColorSimilar[NET_COLOR_TYPE_MAX]; // upper object  color  similarity (object  type as human is valid )
    BYTE                byLowerBodyColorSimilar[NET_COLOR_TYPE_MAX]; // lower object  color  similarity (object  type as human is valid )
    int                 nRelativeID;                // related object ID
	char				szSubText[20];              // "ObjectType"is "Vehicle"or "Logo",  means LOGO lower brand, such as Audi A6L, since there are many brands, SDK shows this field in real-time,device filled as real.

	int					nPersonStature;	            // Intrusion staff height, unit cm
	EM_MSG_OBJ_PERSON_DIRECTION	emPersonDirection;  // Staff intrusion direction
    DWORD               rgbaLowerBodyColor;         // Use direction same as rgbaMainColor,object  type as human is valid 	
} DH_MSG_OBJECT_EX;


// Video analysis object info extension structure, extension version 2. 
typedef struct tagDH_MSG_OBJECT_EX2
{
    DWORD				dwSize;
    int                 nObjectID;                          // Object ID. Each ID presents one object. 
    char                szObjectType[128];                  // Object type 
    int                 nConfidence;                        // Confiidence(0-255). The higher the value is, the higher the confidence is. 
    int                 nAction;                            // Object operation. 1:Appear 2:Move 3:Stay 4:Remove 5:Disappear 6:Split 7:Merge 8:Rename
    DH_RECT             BoundingBox;                        // Surrounding rectangle 
    DH_POINT            Center;                             // Object size centre
    int                 nPolygonNum;                        // Top amount of the polygon
    DH_POINT            Contour[DH_MAX_POLYGON_NUM];        // Polygon of generaly accurate frame 
    DWORD               rgbaMainColor;                      // The plate and the vehicle body main color. Use byte to present: red, green, blue and transparent. When RGB value is (0,255,0), transparent is 0, the value is 0x00ff0000.
    char                szText[128];                        // The same as the string of the DH_MSG_OBJECT
    char                szObjectSubType[64];                // Object sub-type. It has the following sub-tyes. 
															// The same as the string of the DH_MSG_OBJECT
    BYTE                byReserved1[3];
    bool                bPicEnble;                          // There is image file info of the corresponding object 
    DH_PIC_INFO         stPicInfo;                          // Image info of the object 
    bool                bShotFrame;                         // Has been snapped or not 
    bool                bColor;                             // Object color (rgbaMainColor) is usable or not. 
    BYTE                bLowerBodyColor;                    // The lower part color (rgbaLowerBodyColor) is usable or not 
    BYTE                byTimeType;                         // Time type. Please refer to EM_TIME_TYPE.
    NET_TIME_EX         stuCurrentTime;                     // For video synopsis. Current time stampl (When snap or recognize the object, use the recognition intelligent frame on one video frame or JPEG.  It is the appearing time of the frame on the original video. )
    NET_TIME_EX         stuStartTime;                       // Start time stamp(When the object first appear )
    NET_TIME_EX         stuEndTime;                         // End time (When the object last appear )
    DH_RECT             stuOriginalBoundingBox;             // Surrounding box(Absolute coordinates)
    DH_RECT             stuSignBoundingBox;                 // Vehicle symbol surrounding box 
    DWORD               dwCurrentSequence;                  // Current frame SN(Frame when snap the object )
    DWORD               dwBeginSequence;                    // Start frame SN (The frame SN when the object start appearing)
    DWORD               dwEndSequence;                      // End frame SN (The frame SN when the object disappering)
    INT64               nBeginFileOffset;                   // The file offset when start. Unit:byte. (When the object appearing, the video frame offset value comparing with the file start positon in the original video)
    INT64               nEndFileOffset;                     // The file offset when stop. Unit: byte. (When the object disappearing, the video frame offset value comparing with the file start position in the original video)
    BYTE                byColorSimilar[NET_COLOR_TYPE_MAX]; // Object color similarity level. The valur ranges from 0 to 100. The underline value of the array represents one color. Plase refer to EM_COLOR_TYPE.
    BYTE                byUpperBodyColorSimilar[NET_COLOR_TYPE_MAX]; // The top body color similarity leve; (When the object is the human)
    BYTE                byLowerBodyColorSimilar[NET_COLOR_TYPE_MAX]; // The lower body color similarity leve; (When the object is the human)
    int                 nRelativeID;                        // Related object ID
    char				szSubText[20];						// When "ObjectType" is "Vehicle" or "Logo", it represents one car series under the card symbol such as Audio A6L. Since there are too many card series, SDK use the network to realize COM transmission (szSubText) to realize this function. 
    
    int					nPersonStature;						// Intrusion person height. Unit is cm. 
    EM_MSG_OBJ_PERSON_DIRECTION	emPersonDirection;			// Intrusion person direction 
    DWORD               rgbaLowerBodyColor;                 // The same usage as the rgbaMainColor, it is valid when the object type is human. 

    //Video synopsis extra info 
    int                 nSynopsisSpeed;                             // Synopsis speed threshold. There are ten levels (1 to 10). 5 means only reserve the object of speed higher than 5. It is a relative unit.  
                                                                    // When it is 0, the string is invalid.
    int                 nSynopsisSize;                              // Synopsis dimension threshold. There are ten levels (1 to 10). 3 means only reserve the object of speed higher than 3. It is a relative unit.  
                                                                    // When it is o, the string is invalid. 
    BOOL                bEnableDirection;                           // When it is True, filter the object moving direction.
                                                                    // When it is False, do not filter the object moving direction. 
    DH_POINT            stuSynopsisStartLocation;                   // Synopsis moving direction, start coordinates. The point coordinates [0,8192), it is valid when bEnableDirection is True. 
    DH_POINT            stuSynopsisEndLocation;                     // Synopsis moving direction, stop coordinates. The point coordinates [0,8192), it is valid when bEnableDirection is True. 
    BYTE                byReserved[2048];                           // Extension byte
} DH_MSG_OBJECT_EX2;
#pragma pack(pop)


// snapshot info
typedef struct
{
	short              snSpeed;                          // current car speed,km/h
	short              snCarLength;                      // current car length, dm
	float              fRedTime;                         // current red light time, s.ms
	float              fCapTime;                         // current car way snapshot time, s.ms 
	BYTE               bSigSequence;                     // current snapshot Sequence
	BYTE               bType;                            // current snapshot type
														 // 0: radar up speed limit;1: radar low speed limit;2: car detector up speed limit;3:car detector low speed limit
														 // 4: reverse;5: break red light;6: red light on;7: red light off;8: snapshot or traffic gate
	BYTE               bDirection;                       // breaking type :01:left turn;02:straight;03:right
	BYTE               bLightColor;                      // current car way traffic light state,0: green, 1: red, 2: yellow
	BYTE               bSnapFlag[16];                    // snap flag from device
}DH_SIG_CARWAY_INFO;

// Vehicle detector redundancy info
typedef struct
{
	BYTE                byRedundance[8];                // The vehicle detector generates the snap signal redundancy info
	BYTE                bReserved[120];                 // Reserved field
}DH_SIG_CARWAY_INFO_EX;


// car way info
typedef struct  
{
	BYTE                bCarWayID;                           // current car way id 
	BYTE                bReserve[2];                         // reserved
	BYTE                bSigCount;                           // being snapshotted
	DH_SIG_CARWAY_INFO  stuSigInfo[DH_MAX_SNAP_SIGNAL_NUM];  // the snapshot info	
	BYTE                bReserved[12];                       // reserved
}DH_CARWAY_INFO;

// event file info
typedef struct
{
	BYTE               bCount;                               // the file count in the current file's group
	BYTE               bIndex;                               // the index of the file in the group
	BYTE               bFileTag;                             // file tag, see the enum struct EM_EVENT_FILETAG
	BYTE               bFileType;                            // file type,0-normal 1-compose 2-cut picture
	NET_TIME_EX        stuFileTime;                          // file time
	DWORD              nGroupId;                             // the only id of one group file
}DH_EVENT_FILE_INFO;

// pic resolution 
typedef struct
{
	unsigned short   snWidth;    // width
 	unsigned short   snHight;    // hight
}DH_RESOLUTION_INFO;

//  color RGBA
typedef struct tagDH_COLOR_RGBA
{
    int	        nRed;           // red
    int         nGreen;         // green
    int         nBlue;          // blue
    int         nAlpha;         // transparent
} DH_COLOR_RGBA, NET_COLOR_RGBA;

#define	NET_COUNTRY_LENGTH		3		// short for country
#define NET_COMMENT_LENGTH		100		// comment length
#define NET_GROUPID_LENGTH		64		// group id length
#define NET_GROUPNAME_LENGTH	128		// group name length
#define NET_FEATUREVALUE_LENGTH	128		// feature value length

// person info
typedef struct tagFACERECOGNITION_PERSON_INFO
{
	char                szPersonName[DH_MAX_NAME_LEN];		// name                 
	WORD				wYear;								// birth year
	BYTE				byMonth;							// birth month
	BYTE				byDay;								// birth day
	char                szID[DH_MAX_PERSON_ID_LEN];			// the unicle ID for the person
	BYTE                bImportantRank;						// importance level,1~10,the higher value the higher level
	BYTE                bySex;								// sex, 0-man, 1-female
	WORD                wFacePicNum;						// picture number
	DH_PIC_INFO         szFacePicInfo[DH_MAX_PERSON_IMAGE_NUM]; // picture info
	BYTE                byType;                                         // Personnel types, see EM_PERSON_TYPE
	BYTE                byIDType;                                       // Document types, see EM_CERTIFICATE_TYPE
	BYTE                bReserved1[2];                                  // Byte alignment
	char                szProvince[DH_MAX_PROVINCE_NAME_LEN];           // province
	char                szCity[DH_MAX_CITY_NAME_LEN];                   // city
	char                szPersonNameEx[DH_MAX_PERSON_NAME_LEN];	        // Name, the name is too long due to the presence of 16 bytes can not be Storage problems, the increase in this parameter
    char                szUID[DH_MAX_PERSON_ID_LEN];                    // person unique ID
	char				szCountry[NET_COUNTRY_LENGTH];					// country
	BYTE				byIsCustomType;									// using person type: 0 using byType, 1 using szPersonName
	char				*pszComment;									// comment info
	char				*pszGroupID;									// Group ID
	char				*pszGroupName;									// Group Name
	char				*pszFeatureValue;								// the face feature
	BYTE				bGroupIdLen;									// len of pszGroupID
	BYTE				bGroupNameLen;									// len of pszGroupName
	BYTE				bFeatureValueLen;								// len of pszFeatureValue
	BYTE                bReserved[5];
}FACERECOGNITION_PERSON_INFO;


// cadidate person info
typedef struct tagCANDIDATE_INFO
{
	FACERECOGNITION_PERSON_INFO  stPersonInfo;            // person info
	BYTE                         bySimilarity;            // similarity
	BYTE                         byRange;                 // Range officer's database, see EM_FACE_DB_TYPE
	BYTE                         byReserved1[2];
	NET_TIME                     stTime;                  // When byRange historical database effectively, which means that the query time staff appeared
	char                         szAddress[MAX_PATH];     // When byRange historical database effectively, which means that people place a query appears
	BOOL                         bIsHit;                  // Is hit, means the result face has compare result in database
	DH_PIC_INFO_EX3              stuSceneImage;           // Scene Image
	int							 nChannelID;			  // ChannelId
    BYTE                         byReserved[32];          // Reserved bytes
}CANDIDATE_INFO;



// calss type  
typedef enum tagEM_CLASS_TYPE        
{
	EM_CLASS_UNKNOWN                	= 0,         
    EM_CLASS_VIDEO_SYNOPSIS            	= 1,        
    EM_CLASS_TRAFFIV_GATE            	= 2,           
    EM_CLASS_ELECTRONIC_POLICE        	= 3,        
    EM_CLASS_SINGLE_PTZ_PARKING        	= 4,        
    EM_CLASS_PTZ_PARKINBG            	= 5,          
    EM_CLASS_TRAFFIC                	= 6,    // "Traffic"       
    EM_CLASS_NORMAL                    	= 7,    // "Normal"       
    EM_CLASS_PRISON                    	= 8,    // "Prison"       
    EM_CLASS_ATM                    	= 9,    // "ATM"       
    EM_CLASS_METRO                    	= 10,       
    EM_CLASS_FACE_DETECTION            	= 11,   // "FaceDetection"       
    EM_CLASS_FACE_RECOGNITION        	= 12,   // "FaceRecognition"       
    EM_CLASS_NUMBER_STAT            	= 13,   // "NumberStat"       
    EM_CLASS_HEAT_MAP                	= 14,   // "HeatMap"       
    EM_CLASS_VIDEO_DIAGNOSIS        	= 15,   // "VideoDiagnosis"       
    EM_CLASS_VIDEO_ENHANCE            	= 16,        
    EM_CLASS_SMOKEFIRE_DETECT        	= 17,          
    EM_CLASS_VEHICLE_ANALYSE        	= 18,   // "VehicleAnalyse"       
    EM_CLASS_PERSON_FEATURE            	= 19, 
    EM_CLASS_SDFACEDETECTION			= 20,	// "SDFaceDetect"  
	EM_CLASS_HEAT_MAP_PLAN				= 21,	// "HeatMapPlan" 
	EM_CLASS_NUMBERSTAT_PLAN			= 22,	// "NumberStatPlan"	
	EM_CLASS_ATMFD						= 23, 	// "ATMFD"
	EM_CLASS_HIGHWAY					= 24,	// "Highway"
	EM_CLASS_CITY						= 25,	// "City"
	EM_CLASS_LETRACK					= 26,	// "LeTrack"
	EM_CLASS_SCR						= 27,	// "SCR"
	EM_CLASS_STEREO_VISION              = 28,   // "StereoVision"
	EM_CLASS_HUMANDETECT                = 29,   // "HumanDetect"
	EM_CLASS_FACE_ANALYSIS				= 30,	// "FaceAnalysis"
} EM_CLASS_TYPE;

//same as EM_SCENE_TYPE
typedef enum tagEM_SCENE_CLASS_TYPE
{
	EM_SCENE_CLASS_UNKNOW,			// unknow
	EM_SCENE_CLASS_NORMAL,			// "Normal"
	EM_SCENE_CLASS_TRAFFIC,			// "Traffic" 
	EM_SCENE_CLASS_TRAFFIC_PATROL,	// "TrafficPatrol" 
	EM_SCENE_CLASS_FACEDETECTION,	// "FaceDetection" 
	EM_SCENE_CLASS_ATM,				// "ATM"
	EM_SENCE_CLASS_INDOOR,			// "Indoor"  
	EM_SENCE_CLASS_FACERECOGNITION,	// "FaceRecognition" 
	EM_SENCE_CLASS_PRISON,			// "Prison" 
	EM_SENCE_CLASS_NUMBERSTAT,		// "NumberStat" 
	EM_SENCE_CLASS_HEAT_MAP,		// "HeatMap" 
	EM_SENCE_CLASS_VIDEODIAGNOSIS,	// "VideoDiagnosis" 
	EM_SENCE_CLASS_VEHICLEANALYSE,	// "VehicleAnalyse" 
	EM_SENCE_CLASS_COURSERECORD,	// "CourseRecord" 
	EM_SENCE_CLASS_VEHICLE,			// "Vehicle" 
	EM_SENCE_CLASS_STANDUPDETECTION,// "StandUpDetection" 
	EM_SCENE_CLASS_GATE,			// "Gate"
	EM_SCENE_CLASS_SDFACEDETECTION,	// "SDFaceDetect"
	EM_SCENE_CLASS_HEAT_MAP_PLAN,	// "HeatMapPlan"
	EM_SCENE_CLASS_NUMBERSTAT_PLAN,	// "NumberStatPlan"
	EM_SCENE_CLASS_ATMFD,			// "ATMFD"
	EM_SCENE_CLASS_HIGHWAY,			// "Highway" 
	EM_SCENE_CLASS_CITY,			// "City" 
	EM_SCENE_CLASS_LETRACK,			// "LeTrack" 
	EM_SCENE_CLASS_SCR,				// "SCR"
	EM_SCENE_CLASS_STEREO_VISION,   // "StereoVision"
	EM_SCENE_CLASS_HUMANDETECT,		// "HumanDetect"
	EM_SCENE_CLASS_FACEANALYSIS,	// "FaceAnalysis"
} EM_SCENE_CLASS_TYPE;

// intelli event comm info
typedef struct tagEVENT_INTELLI_COMM_INFO
{
	EM_CLASS_TYPE		emClassType;								// class type
	int					nPresetID;									// Preset ID
	BYTE                bReserved[124];                     		// reserved
} EVENT_INTELLI_COMM_INFO;

// part of traffic car info
typedef struct tagEVENT_TRAFFIC_CAR_PART_INFO
{
    char                szMachineName[128];                  // Local or remote device name    
    char                szRoadwayNo[32];                     // road way number
    BYTE                bReserved[352];                      // reserved
}EVENT_TRAFFIC_CAR_PART_INFO;


// traffic car move direction type
typedef enum tagEM_TRAFFICCAR_MOVE_DIRECTION
{
    EM_TRAFFICCAR_MOVE_DIRECTION_UNKNOWN,                           // unknown
    EM_TRAFFICCAR_MOVE_DIRECTION_STRAIGHT,                          // straight
    EM_TRAFFICCAR_MOVE_DIRECTION_TURN_LEFT,                         // turn left
    EM_TRAFFICCAR_MOVE_DIRECTION_TURN_RIGHT,                        // turn right
    EM_TRAFFICCAR_MOVE_DIRECTION_TURN_AROUND,                       // turn around
}EM_TRAFFICCAR_MOVE_DIRECTION;

// TrafficCar
typedef struct tagDEV_EVENT_TRAFFIC_TRAFFICCAR_INFO
{
	char               szPlateNumber[32];               // plate number
	char               szPlateType[32];                 // Plate type: "Unknown" =Unknown; "Normal"=Blue and black plate. "Yellow"=Yellow plate. "DoubleYellow"=Double-layer yellow plate 
												        // "Police"=Police plate ; "Armed"= =Military police plate; "Military"=Army plate; "DoubleMilitary"=Army double-layer 
												        // "SAR" =HK SAR or Macao SAR plate; "Trainning" =rehearsal plate; "Personal"=Personal plate; "Agri"=Agricultural plate
												        // "Embassy"=Embassy plate; "Moto"=Moto plate ; "Tractor"=Tractor plate; "Other"=Other plate 
	char               szPlateColor[32];                // plate color, "Blue","Yellow", "White","Black","YellowbottomBlackText","BluebottomWhiteText","BlackBottomWhiteText","ShadowGreen","YellowGreen"
	char               szVehicleColor[32];              // vehicle color, "White", "Black", "Red", "Yellow", "Gray", "Blue","Green"
	int                nSpeed;                          // speed, Km/H
	char               szEvent[64];                     // trigger event type
	char               szViolationCode[32];             // violation code, see TrafficGlobal.ViolationCode
	char               szViolationDesc[64];             // violation describe
	int                nLowerSpeedLimit;                // lower speed limit
	int                nUpperSpeedLimit;                // upper speed limit
	int                nOverSpeedMargin;                // over speed margin, km/h 
	int                nUnderSpeedMargin;               // under speed margin, km/h 
	int                nLane;                           // lane	
	int                nVehicleSize;                    // vehicle size, see VideoAnalyseRule's describe
                                                        // Bit 0:"Light-duty", small car
                                                        // Bit 1:"Medium", medium car
                                                        // Bit 2:"Oversize", large car
                                                        // Bit 3:"Minisize", mini car
                                                        // Bit 4:"Largesize", long car
	float              fVehicleLength;                  // vehicle length, m
	int                nSnapshotMode;                   // snap mode 0-normal,1-globle,2-near,4-snap on the same side,8-snap on the reverse side,16-plant picture
	char               szChannelName[32];               // channel name
	char               szMachineName[256];              // Machine name
	char               szMachineGroup[256];             // machine group
	char               szRoadwayNo[64];                 // road way number	
	char               szDrivingDirection[3][DH_MAX_DRIVINGDIRECTION];   
																			// DrivingDirection: for example ["Approach", "Shanghai", "Hangzhou"]
													     					// "Approach" means driving direction,where the car is more near;"Leave"-means where if mor far to the car
														 					// the second and third param means the location of the driving direction
	char              *szDeviceAddress;                 // device address,OSD superimposed onto the image,from TrafficSnapshot.DeviceAddress,'\0'means end.
	char			   szVehicleSign[32];				// Vehicle identification, such as "Unknown" - unknown "Audi" - Audi, "Honda" - Honda ...
	DH_SIG_CARWAY_INFO_EX stuSigInfo;                   // Generated by the vehicle inspection device to capture the signal redundancy
	char			  *szMachineAddr;					// Equipment deployment locations
	float              fActualShutter;                  // Current picture exposure time, in milliseconds
	BYTE               byActualGain;                    // Current picture gain, ranging from 0 to 100
	BYTE			   byDirection;						// Lane Direction,0 - south to north 1- Southwest to northeast 2 - West to east, 3 - Northwest to southeast 4 - north to south 5 - northeast to southwest 6 - East to West 7 - Southeast to northwest 8 - Unknown 9-customized
	BYTE			   byReserved[2];
	char*			   szDetailedAddress;                   // Address, as szDeviceAddress supplement, 
    char               szDefendCode[DH_COMMON_STRING_64];   // waterproof  
    int                nTrafficBlackListID;                 // Link black list data recorddefualt main keyID, 0, invalid, > 0, black list data record
    DH_COLOR_RGBA      stuRGBA;                             // bofy color RGBA
    NET_TIME           stSnapTime;                          // snap time
    int                nRecNo;                              // Rec No
    char               szCustomParkNo[DH_COMMON_STRING_32 + 1]; // self defined parking space number, for parking, 
    BYTE               byReserved1[3];
    int                nDeckNo;                             // Metal plate No. 
    int                nFreeDeckCount;                      // Free metal plate No.
    int                nFullDeckCount;                      // Occupized metal plate No. 
    int                nTotalDeckCount;                     // Total metal plate No. 
    char               szViolationName[64];                 // violation name
	unsigned int	   nWeight;								// Weight of car(kg)
	char               szCustomRoadwayDirection[32];		// custom road way, valid when byDirection is 9
    BYTE               byPhysicalLane;                      // the physical lane number,value form 0 to 5
    BYTE               byReserved2[3];
    EM_TRAFFICCAR_MOVE_DIRECTION emMovingDirection;         // moving direction
    NET_TIME		   stuEleTagInfoUTC;					// corresponding to throughTime
    BYTE               bReserved[552];                      // Reserved bytes
}DEV_EVENT_TRAFFIC_TRAFFICCAR_INFO;

#pragma pack(push)
#pragma pack(4)
// GPS Infomation
typedef struct tagNET_GPS_INFO
{
    unsigned int                    nLongitude;         	// Longitude(unit:1/1000000 degree)
                                                            // west Longitude: 0 - 180000000            practical value = 180*1000000 - dwLongitude
                                                            // east Longitude: 180000000 - 360000000    practical value = dwLongitude - 180*1000000
															// eg: Longitude:300168866  (300168866 - 180*1000000)/1000000  equal east Longitude 120.168866 degree

    unsigned int					nLatidude;              // Latidude(unit:1/1000000 degree)
                                                            // north Latidude: 0 - 90000000				practical value = 90*1000000 - dwLatidude
                                                            // south Latidude: 90000000 - 180000000	practical value = dwLatidude - 90*1000000
															// eg: Latidude:120186268 (120186268 - 90*1000000)/1000000 equal south Latidude 30. 186268 degree
    double                          dbAltitude;             // altitude,unit:m
    double                          dbSpeed;                // Speed,unit:km/H
    double                          dbBearing;              // Bearing,unit:
    BYTE                            bReserved[8];           // Reserved bytes
}NET_GPS_INFO;
#pragma pack(pop)

// the describe of EVENT_IVS_CROSSLINEDETECTION's data
typedef struct tagDEV_EVENT_CROSSLINE_INFO 
{
	int					nChannelID;						// ChannelId
	char				szName[128];					// event name
	char                bReserved1[4];                  // byte alignment
	double				PTS;							// PTS(ms)
	NET_TIME_EX			UTC;							// the event happen time
	int					nEventID;						// event ID
	DH_MSG_OBJECT		stuObject;						// have being detected object
	DH_EVENT_FILE_INFO  stuFileInfo;                    // event file info
	DH_POINT            DetectLine[DH_MAX_DETECT_LINE_NUM];    // rule detect line
	int                 nDetectLineNum;                        // rule detect line's point number
	DH_POINT            TrackLine[DH_MAX_TRACK_LINE_NUM];      // object moveing track
	int                 nTrackLineNum;                         // object moveing track's point number
	BYTE                bEventAction;                   // Event action,0 means pulse event,1 means continuous event's begin,2means continuous event's end;
	BYTE                bDirection;                     // direction, 0-left to right, 1-right to left
	BYTE                byReserved[1];
	BYTE				byImageIndex;					// Serial number of the picture, in the same time (accurate to seconds) may have multiple images, starting from 0
	DWORD               dwSnapFlagMask;	                // flag(by bit),see NET_RESERVED_COMMON
	int                 nSourceIndex;                   // the source device's index,-1 means data in invalid
	char                szSourceDevice[MAX_PATH];       // the source device's sign(exclusive),field said local device does not exist or is empty
    unsigned int        nOccurrenceCount;               // event trigger accumulated times
    EVENT_INTELLI_COMM_INFO     stuIntelliCommInfo;     // intelli comm info
    BYTE                bReserved[476];	                // reserved
	
} DEV_EVENT_CROSSLINE_INFO;

// event type EVENT_IVS_CROSSLINEDETECTION_EX(warning line event )corresponding to  datadescriptioninfo 
typedef struct tagDEV_EVENT_CROSSLINE_INFO_EX 
{
    DWORD               dwSize;
    int                 nChannelID;                     // channel no.
    char                szName[128];                    // event name 
    char                bReserved1[4];                  // field align
    float				PTS;                            // time stamp(unit is ms)
    NET_TIME_EX         UTC;                            // event occurance time
    int                 nEventID;                       // event ID
    DH_MSG_OBJECT_EX    stuObject;                      // detection object 
    DH_EVENT_FILE_INFO  stuFileInfo;                    // event corresponding to file info 
    DH_POINT            DetectLine[DH_MAX_DETECT_LINE_NUM]; // rule detection line
    int                 nDetectLineNum;                 // rule detection line top number
    DH_POINT            TrackLine[DH_MAX_TRACK_LINE_NUM];// object motion track
    int                 nTrackLineNum;                  // object motion track top number
    BYTE                bEventAction;                   // event  motion , 0 means pulse event ,1 means continuity event  start ,2 means  continuity  event end;
    BYTE                bDirection;                     // means  intrusion direction, 0- leto to right, 1- right to left
    BYTE                byReserved[1];
    BYTE                byImageIndex;                   // picture no., same one time(accurate to second)may have multiple pictures , start from 0  
    DWORD               dwSnapFlagMask;                 // snapshot mark(by bit), see NET_RESERVED_COMMON
    int                 nSourceIndex;                   // event source device index,-1 means invalid data,-1 means invalid data
    char                szSourceDevice[MAX_PATH];       // event source device SN, field do not existor is null means local device 
    unsigned int        nOccurrenceCount;               // event trigger accumulated times
	int					nMsgObjArrayCount;              // detection object info number 
	DH_MSG_OBJECT_EX*	pMsgObjArray;                   // detection object info group indicator
	EVENT_INTELLI_COMM_INFO     stuIntelliCommInfo;     // intelli comm info
} DEV_EVENT_CROSSLINE_INFO_EX;

// the describe of EVENT_IVS_CROSSREGIONDETECTION's data
typedef struct tagDEV_EVENT_CROSSREGION_INFO 
{
	int					nChannelID;						// ChannelId
	char				szName[128];					// event name
	char                bReserved2[4];                  // byte alignment
	double				PTS;							// PTS(ms)
	NET_TIME_EX			UTC;							// the event happen time
	int					nEventID;						// event ID
	DH_MSG_OBJECT		stuObject;						// have being detected object
	DH_EVENT_FILE_INFO  stuFileInfo;                    // event file info
	DH_POINT            DetectRegion[DH_MAX_DETECT_REGION_NUM];    // rule detect region
	int                 nDetectRegionNum;                          // rule detect region's point number
	DH_POINT            TrackLine[DH_MAX_TRACK_LINE_NUM];          // object moving track
	int                 nTrackLineNum;                             // object moving track's point number
	BYTE                bEventAction;                   // Event action,0 means pulse event,1 means continuous event's begin,2means continuous event's end;
	BYTE                bDirection;                     // direction, 0-in, 1-out,2-apaer,3-leave
	BYTE                bActionType;                    // action type,0-appear 1-disappear 2-in area 3-cross area
	BYTE				byImageIndex;					// Serial number of the picture, in the same time (accurate to seconds) may have multiple images, starting from 0
    DWORD               dwSnapFlagMask;	                // flag(by bit),see NET_RESERVED_COMMON
	int                 nSourceIndex;                   // the source device's index,-1 means data in invalid
	char                szSourceDevice[MAX_PATH];       // the source device's sign(exclusive),field said local device does not exist or is empty
    unsigned int        nOccurrenceCount;               // event trigger times
	BYTE				bReserved[536];					// reserved
	int                 nObjectNum;                     // Detect object amount
	DH_MSG_OBJECT		stuObjectIDs[DH_MAX_OBJECT_LIST];// Detected object
	int                 nTrackNum;                      // Locus amount(Corresponding to the detected object amount.)
	DH_POLY_POINTS      stuTrackInfo[DH_MAX_OBJECT_LIST];// Locus info(Corresponding to the detected object)
	EVENT_INTELLI_COMM_INFO     stuIntelliCommInfo;      // intelli comm info
} DEV_EVENT_CROSSREGION_INFO;

//Special zone type 
typedef enum tagEM_SPECIAL_REGION_TYPE
{
    EM_SPEICAL_REGION_UNKNOW = 0        , // Unknown
    EM_SPEICAL_REGION_HIGH_LIGHT        , // Extra high light:ATM keyboard pane
    EM_SPEICAL_REGION_REGULAR_BLINK     , // Regular flash:ATM insert card zone 
    EM_SPEICAL_REGION_IRREGULAR_BLINK   , // Irregular flash:ATM screen 
}EM_SPECIAL_REGION_TYPE;

// the describe of EVENT_IVS_PASTEDETECTION's data
typedef struct tagDEV_EVENT_PASTE_INFO 
{
	int					nChannelID;						// ChannelId
	char				szName[128];					// event name
	char                bReserved1[4];                  // byte alignment
	double				PTS;							// PTS(ms)
	NET_TIME_EX			UTC;							// the event happen time
	int					nEventID;						// event ID
	DH_MSG_OBJECT		stuObject;						// have being detected object
	DH_EVENT_FILE_INFO  stuFileInfo;                    // event file info
	int                 nDetectRegionNum;				// rule detect region's point number
	DH_POINT            DetectRegion[DH_MAX_DETECT_REGION_NUM];    // rule detect region
	BYTE                bEventAction;                   // Event action,0 means pulse event,1 means continuous event's begin,2means continuous event's end;
	BYTE                byReserved[2];
	BYTE				byImageIndex;					// Serial number of the picture, in the same time (accurate to seconds) may have multiple images, starting from 0
    DWORD               dwSnapFlagMask;	                // flag(by bit),see NET_RESERVED_COMMON
    unsigned int        nOccurrenceCount;               // event trigger accumilated times 
    EM_SPECIAL_REGION_TYPE  emSpecialRegion;			// Special zone type
    EVENT_INTELLI_COMM_INFO stuIntelliCommInfo;         // intelli comm info
    BYTE                    bReserved[748];				// Reserved string. To be delevloped 
	
} DEV_EVENT_PASTE_INFO;

// the describe of EVENT_IVS_LEFTDETECTION's data
typedef struct tagDEV_EVENT_LEFT_INFO 
{
	int					nChannelID;						// ChannelId
	char				szName[128];					// event name
	char                bReserved1[4];                  // byte alignment
	double				PTS;							// PTS(ms)
	NET_TIME_EX			UTC;							// the event happen time
	int					nEventID;						// event ID
	DH_MSG_OBJECT		stuObject;						// have being detected object
	DH_EVENT_FILE_INFO  stuFileInfo;                    // event file info
	BYTE                bEventAction;                   // Event action,0 means pulse event,1 means continuous event's begin,2means continuous event's end;
	BYTE                byReserved[2];
	BYTE				byImageIndex;					// Serial number of the picture, in the same time (accurate to seconds) may have multiple images, starting from 0
	int                 nDetectRegionNum;				// detect region point number
	DH_POINT            DetectRegion[DH_MAX_DETECT_REGION_NUM];    // detect region
    DWORD               dwSnapFlagMask;	                // flag(by bit),see NET_RESERVED_COMMON
	int                 nSourceIndex;                   // the source device's index,-1 means data in invalid
	char                szSourceDevice[MAX_PATH];       // the source device's sign(exclusive),field said local device does not exist or is empty
	unsigned int    	nOccurrenceCount;          		// event trigger accumilated times
  	EVENT_INTELLI_COMM_INFO     stuIntelliCommInfo;     // intelli comm info
	short				nPreserID;						// Event triggered preset period, starting from 1 (no unknown)
	char				szPresetName[64];				// Preset name for event triggered
	BYTE				bReserved[422];			  		// Reserved
	
} DEV_EVENT_LEFT_INFO;

// the describe of EVENT_IVS_PRESERVATION's data
typedef struct tagDEV_EVENT_PRESERVATION_INFO 
{
	int					nChannelID;						// ChannelId
	char				szName[128];					// event name
	char                bReserved1[4];                  // byte alignment
	double				PTS;							// PTS(ms)
	NET_TIME_EX			UTC;							// the event happen time
	int					nEventID;						// event ID
	DH_MSG_OBJECT		stuObject;						// have being detected object
	DH_EVENT_FILE_INFO  stuFileInfo;                    // event file info
	BYTE                bEventAction;                   // Event action,0 means pulse event,1 means continuous event's begin,2means continuous event's end;
    BYTE                byReserved[2];
    BYTE                byImageIndex;                   // Serial number of the picture, in the same time (accurate to seconds) may have multiple images, starting from 0
	int                 nDetectRegionNum;				// detect region point number
	DH_POINT            DetectRegion[DH_MAX_DETECT_REGION_NUM];    // detect region
    DWORD               dwSnapFlagMask;	                // flag(by bit),see NET_RESERVED_COMMON
	int                 nSourceIndex;                   // the source device's index,-1 means data in invalid
	char                szSourceDevice[MAX_PATH];       // the source device's sign(exclusive),field said local device does not exist or is empty
	unsigned int    	nOccurrenceCount;          		// event trigger accumilated times 
  	EVENT_INTELLI_COMM_INFO     stuIntelliCommInfo;     // pintelli comm info
	BYTE				bReserved[488];			  		// Reserved
	
} DEV_EVENT_PRESERVATION_INFO;

// the describe of EVENT_IVS_STAYDETECTION's data
typedef struct tagDEV_EVENT_STAY_INFO 
{
	int					nChannelID;						// ChannelId
	char				szName[128];					// event name
	char                bReserved1[4];                  // byte alignment
	double				PTS;							// PTS(ms)
	NET_TIME_EX			UTC;							// the event happen time
	int					nEventID;						// event ID
	DH_MSG_OBJECT		stuObject;						// have being detected object
	DH_EVENT_FILE_INFO  stuFileInfo;                    // event file info
	BYTE                bEventAction;                   // Event action,0 means pulse event,1 means continuous event's begin,2means continuous event's end;
    BYTE                byReserved[2];
    BYTE                byImageIndex;                   // Serial number of the picture, in the same time (accurate to seconds) may have multiple images, starting from 0
	int                 nDetectRegionNum;				// detect region point number
	DH_POINT            DetectRegion[DH_MAX_DETECT_REGION_NUM];    // detect region
    DWORD               dwSnapFlagMask;	                // flag(by bit),see NET_RESERVED_COMMON
	int                 nSourceIndex;                   // the source device's index,-1 means data in invalid
	char                szSourceDevice[MAX_PATH];       // the source device's sign(exclusive),field said local device does not exist or is empty
	unsigned int    	nOccurrenceCount;          		// event trigger accumilated times 
  	EVENT_INTELLI_COMM_INFO     stuIntelliCommInfo;     // intelli comm info
	BYTE				bReserved[488];			  		// Reserved
	
} DEV_EVENT_STAY_INFO;

// the describe of EVENT_IVS_WANDERDETECTION's data
typedef struct tagDEV_EVENT_WANDER_INFO 
{
	int					nChannelID;						// ChannelId
	char				szName[128];					// event name
	char                bReserved1[4];                  // byte alignment
	double				PTS;							// PTS(ms)
	NET_TIME_EX			UTC;							// the event happen time
	int					nEventID;						// event ID
	DH_EVENT_FILE_INFO  stuFileInfo;                    // event file info
	BYTE                bEventAction;                   // Event action,0 means pulse event,1 means continuous event's begin,2means continuous event's end;
    BYTE                byReserved[2];                  // 
    BYTE                byImageIndex;                   // Serial number of the picture, in the same time (accurate to seconds) may have multiple images, starting from 0
	int                 nObjectNum;                     // detected objects number
	DH_MSG_OBJECT		stuObjectIDs[DH_MAX_OBJECT_LIST];	// detected objects
	int                 nTrackNum;                      // track number
	DH_POLY_POINTS      stuTrackInfo[DH_MAX_OBJECT_LIST];// track info
	int                 nDetectRegionNum;				// detect region point number
	DH_POINT            DetectRegion[DH_MAX_DETECT_REGION_NUM];    // detect region
    DWORD               dwSnapFlagMask;	                // flag(by bit),see NET_RESERVED_COMMON
	int                 nSourceIndex;                   // the source device's index,-1 means data in invalid
	char                szSourceDevice[MAX_PATH];       // the source device's sign(exclusive),field said local device does not exist or is empty
	unsigned int    	nOccurrenceCount;          		// event trigger accumilated times 
  	EVENT_INTELLI_COMM_INFO     stuIntelliCommInfo;     // intelli comm info
	short				nPreserID;						// Event triggered preset period, starting from 1 (no unknown)
	char				szPresetName[64];				// Preset name for event triggered
	BYTE				bReserved[558];					// reserved
	
} DEV_EVENT_WANDER_INFO;

// the describe of EVENT_IVS_MOVEDETECTION's data
typedef struct tagDEV_EVENT_MOVE_INFO 
{
	int					nChannelID;						// ChannelId
	char				szName[128];					// event name
	char                bReserved1[4];                  // byte alignment
	double				PTS;							// PTS(ms)
	NET_TIME_EX			UTC;							// the event happen time
	int					nEventID;						// event ID
	DH_MSG_OBJECT		stuObject;						// have being detected object
	DH_EVENT_FILE_INFO  stuFileInfo;                    // event file info
	BYTE                bEventAction;                   // Event action,0 means pulse event,1 means continuous event's begin,2means continuous event's end;
    BYTE                byReserved[2];
    BYTE                byImageIndex;                   // Serial number of the picture, in the same time (accurate to seconds) may have multiple images, starting from 0
	int                 nDetectRegionNum;				// detect region point 
	DH_POINT            DetectRegion[DH_MAX_DETECT_REGION_NUM];    // detect region
    DWORD               dwSnapFlagMask;	                // flag(by bit),see NET_RESERVED_COMMON
	int                 nSourceIndex;                   // the source device's index,-1 means data in invalid
	char                szSourceDevice[MAX_PATH];       // the source device's sign(exclusive),field said local device does not exist or is empty
	int                 nTrackLineNum;                  // Object trajectories vertices              
	DH_POINT            stuTrackLine[DH_MAX_TRACK_LINE_NUM]; // Object trajectories			
	unsigned int    	nOccurrenceCount;          		// event trigger accumilated times 
	EVENT_INTELLI_COMM_INFO     stuIntelliCommInfo;     // intelli comm info
	BYTE				bReserved[404];					// Reserved bytes, leave extended
	 
} DEV_EVENT_MOVE_INFO;

// the describe of EVENT_IVS_TAILDETECTION's data
typedef struct tagDEV_EVENT_TAIL_INFO 
{
	int					nChannelID;						// ChannelId
	char				szName[128];					// event name
	char                bReserved1[4];                  // byte alignment
	double				PTS;							// PTS(ms)
	NET_TIME_EX			UTC;							// the event happen time
	int					nEventID;						// event ID
	DH_MSG_OBJECT		stuObject;						// have being detected object
	DH_EVENT_FILE_INFO  stuFileInfo;                    // event file info
	BYTE                bEventAction;                   // Event action,0 means pulse event,1 means continuous event's begin,2means continuous event's end;
    BYTE                byReserved[2];
    BYTE                byImageIndex;                   // Serial number of the picture, in the same time (accurate to seconds) may have multiple images, starting from 0
	int                 nDetectRegionNum;				// detect region point 
	DH_POINT            DetectRegion[DH_MAX_DETECT_REGION_NUM];    //  detect region
    DWORD               dwSnapFlagMask;	                // flag(by bit),see NET_RESERVED_COMMON
	int                 nSourceIndex;                   // the source device's index,-1 means data in invalid
	char                szSourceDevice[MAX_PATH];       // the source device's sign(exclusive),field said local device does not exist or is empty
	unsigned int    	nOccurrenceCount;          		// event trigger accumilated times 
	EVENT_INTELLI_COMM_INFO     stuIntelliCommInfo;     // intelli comm info
	BYTE				bReserved[488];					
	
} DEV_EVENT_TAIL_INFO;

// the describe of EVENT_IVS_RIOTERDETECTION's data
typedef struct tagDEV_EVENT_RIOTER_INFO 
{
	int					nChannelID;						// ChannelId
	char				szName[128];					// event name
	char                bReserved1[4];                  // byte alignment
	double				PTS;							// PTS(ms)
	NET_TIME_EX			UTC;							// the event happen time
	int					nEventID;						// event ID
	int					nObjectNum;						// have being detected object number
	DH_MSG_OBJECT		stuObjectIDs[DH_MAX_OBJECT_LIST];// have being detected object list
	DH_EVENT_FILE_INFO  stuFileInfo;                    // event file info
	BYTE                bEventAction;                   // Event action,0 means pulse event,1 means continuous event's begin,2means continuous event's end;
    BYTE                byReserved[2];                  // 
    BYTE                byImageIndex;                   // Serial number of the picture, in the same time (accurate to seconds) may have multiple images, starting from 0
	int                 nDetectRegionNum;				// detect region point 
	DH_POINT            DetectRegion[DH_MAX_DETECT_REGION_NUM];    // detect region

    DWORD               dwSnapFlagMask;	                // flag(by bit),see NET_RESERVED_COMMON
	int                 nSourceIndex;                   // the source device's index,-1 means data in invalid
	char                szSourceDevice[MAX_PATH];       // the source device's sign(exclusive),field said local device does not exist or is empty
	unsigned int    	nOccurrenceCount;          		// event trigger accumilated times 
  	EVENT_INTELLI_COMM_INFO     stuIntelliCommInfo;     // intelli comm info
	BYTE				bReserved[492];					
	
} DEV_EVENT_RIOTERL_INFO;

// the describe of EVENT_IVS_FIGHTDETECTION's data
typedef struct tagDEV_EVENT_FIGHT_INFO 
{
	int					nChannelID;						// ChannelId
	char				szName[128];					// event name
	char                bReserved1[4];                  // byte alignment
	double				PTS;							// PTS(ms)
	NET_TIME_EX			UTC;							// the event happen time
	int					nEventID;						// event ID
	int					nObjectNum;						// have being detected object number
	DH_MSG_OBJECT		stuObjectIDs[DH_MAX_OBJECT_LIST];// have being detected object list
	DH_EVENT_FILE_INFO  stuFileInfo;                    // event file info
	BYTE                bEventAction;                   // Event action,0 means pulse event,1 means continuous event's begin,2means continuous event's end;
    BYTE                byReserved[2];                  // 
    BYTE                byImageIndex;                   // Serial number of the picture, in the same time (accurate to seconds) may have multiple images, starting from 0
	int                 nDetectRegionNum;				// detect region point 
	DH_POINT            DetectRegion[DH_MAX_DETECT_REGION_NUM];    // detect region
	
    DWORD               dwSnapFlagMask;	                // flag(by bit),see NET_RESERVED_COMMON
	int                 nSourceIndex;                   // the source device's index,-1 means data in invalid
	char                szSourceDevice[MAX_PATH];       // the source device's sign(exclusive),field said local device does not exist or is empty
	unsigned int    	nOccurrenceCount;          		// event trigger accumilated times 
  	EVENT_INTELLI_COMM_INFO     stuIntelliCommInfo;     // intelli comm info
	BYTE				bReserved[492];					
} DEV_EVENT_FIGHT_INFO;

// the describe of EVENT_IVS_FIREDETECTION's data
typedef struct tagDEV_EVENT_FIRE_INFO 
{
	int					nChannelID;						// ChannelId
	char				szName[128];					// event name
	char                bReserved1[4];                  // byte alignment
	double				PTS;							// PTS(ms)
	NET_TIME_EX			UTC;							// the event happen time
	int					nEventID;						// event ID
	DH_MSG_OBJECT		stuObject;						// have being detected object
	DH_EVENT_FILE_INFO  stuFileInfo;                    // event file info
	BYTE                bEventAction;                   // Event action,0 means pulse event,1 means continuous event's begin,2means continuous event's end;
    BYTE                byReserved[2];                  // 
    BYTE                byImageIndex;                   // Serial number of the picture, in the same time (accurate to seconds) may have multiple images, starting from 0
	int                 nDetectRegionNum;				// detect region point
	DH_POINT            DetectRegion[DH_MAX_DETECT_REGION_NUM];    // detect region
	
    DWORD               dwSnapFlagMask;	                // flag(by bit),see NET_RESERVED_COMMON
	int                 nSourceIndex;                   // the source device's index,-1 means data in invalid
	char                szSourceDevice[MAX_PATH];       // the source device's sign(exclusive),field said local device does not exist or is empty
	unsigned int    	nOccurrenceCount;          		// event trigger accumilated times 
  	EVENT_INTELLI_COMM_INFO     stuIntelliCommInfo;     // intelli comm info
	BYTE				bReserved[488];					// Reserved
	
} DEV_EVENT_FIRE_INFO;

// Event type EVENT_IVS_GETOUTBEDDETECTION(event of getting out of bed in detention center) data block corresponding description information
typedef struct tagDEV_EVENT_GETOUTBED_INFO 
{
    // Public field
    int                 nChannelID;                         // Channel Id
    char                szName[128];                        // Event name
    char                bReserved1[4];                      // byte alignment
    double              PTS;                                // Timestamp(millisecond)
    NET_TIME_EX         UTC;                                // The event happen time
    int                 nEventID;                           // Event ID
    // Event correspomds to field
    DH_MSG_OBJECT       stuObject;                          // Object said the object information who get off the bed
    int                 nDetectRegionNum;                   // Number of vertex indetection region for rules
    DH_POINT            DetectRegion[DH_MAX_DETECT_REGION_NUM]; // The detection region for rules
    DH_EVENT_FILE_INFO  stuFileInfo;                        // Event correspomds to file information
    BYTE                bEventAction;                       // Event action,0 means pulse event,1 means continuous event's begin,2means continuous event's end

    BYTE                byReserved[2];                      // Reserve bytes
    BYTE                byImageIndex;                       // Serial number of the picture, in the same time (accurate to seconds) may have multiple images, starting from 0
    DWORD               dwSnapFlagMask;                     // flag(by bit),see NET_RESERVED_COMMON    
    BYTE                bReserved[1024];                    // Reserve bytes,left to expand 
} DEV_EVENT_GETOUTBED_INFO;

// Event type EVENT_IVS_PATROLDETECTION(guard detection event) data block corresponding description information
typedef struct tagDEV_EVENT_PATROL_INFO 
{
    // Public field
    int                 nChannelID;                         // Channel Id
    char                szName[128];                        // Event name
    char                bReserved1[4];                      // byte alignment
    double              PTS;                                // Timestamp(millisecond)
    NET_TIME_EX         UTC;                                // the event happen time
    int                 nEventID;                           // Event ID
    // Event correspomds to field
    DH_MSG_OBJECT       stuObject;                          // Object said the information of patrolling object
    int                 nDetectRegionNum;                   // Number of vertex indetection region
    DH_POINT            DetectRegion[DH_MAX_DETECT_REGION_NUM]; // The detection region for rules
    DH_EVENT_FILE_INFO  stuFileInfo;                        // Event correspomds to file information
    BYTE                bEventAction;                       // Event action,0 means pulse event,1 means continuous event's begin,2means continuous event's end    
    BYTE                byReserved[2];                      // Reserve bytes
    BYTE                byImageIndex;                       // Serial number of the picture, in the same time (accurate to seconds) may have multiple images, starting from 0
    DWORD               dwSnapFlagMask;                     // flag(by bit),see NET_RESERVED_COMMON    
    BYTE                bReserved[1024];                    // Reserve bytes,left to expand
} DEV_EVENT_PATROL_INFO;

// Event type EVENT_IVS_ONDUTYDETECTION(guard detection event) data block corresponding description information
typedef struct tagDEV_EVENT_ONDUTY_INFO
{
    // Public field
    int                 nChannelID;                         // Channel Id
    char                szName[128];                        // Event name
    char                bReserved1[4];                      // byte alignment
    double              PTS;                                // Timestamp(millisecond)
    NET_TIME_EX         UTC;                                // the event happen time
    int                 nEventID;                           // Event ID
    // Field corresponding to the event
    DH_MSG_OBJECT       stuObject;                          // Object said the information of patrolling object
    int                 nDetectRegionNum;                   // Number of vertex indetection region for rules
    DH_POINT            DetectRegion[DH_MAX_DETECT_REGION_NUM]; // The detection region for rules
    DH_EVENT_FILE_INFO  stuFileInfo;                        // Event correspomds to file information
    BYTE                bEventAction;                       // Event action,0 means pulse event,1 means continuous event's begin,2means continuous event's end
    
    BYTE                byReserved[2];                      // Reserve bytes,left to expand
    BYTE                byImageIndex;                       // Serial number of the picture, in the same time (accurate to seconds) may have multiple images, starting from 0    DWORD               dwSnapFlagMask;                     // flag(by bit),see NET_RESERVED_COMMON    
    DWORD               dwSnapFlagMask;                     // snap flag mask(by bit), see NET_RESERVED_COMMON    
    BYTE                bReserved[1024];                    // Reserve bytes,left to expand
} DEV_EVENT_ONDUTY_INFO;

// the describe of EVENT_IVS_FIREDETECTION's data
typedef struct tagDEV_EVENT_ELECTROSPARK_INFO 
{
	int					nChannelID;						// Channel Id
	char				szName[128];					// event name
	char                bReserved1[4];                  // byte alignment
	double				PTS;							// PTS(ms)
	NET_TIME_EX			UTC;							// the event happen time
	int					nEventID;						// event ID
	DH_MSG_OBJECT		stuObject;						// have being detected object
	DH_EVENT_FILE_INFO  stuFileInfo;                    // event file info
	BYTE                bEventAction;                   // Event action,0 means pulse event,1 means continuous event's begin,2means continuous event's end
    BYTE                byReserved[2];
    BYTE                byImageIndex;                   // Serial number of the picture, in the same time (accurate to seconds) may have multiple images, starting from 0
    DWORD               dwSnapFlagMask;	                // flag(by bit),see NET_RESERVED_COMMON
	BYTE				bReserved[972];					// reserved
	
} DEV_EVENT_ELECTROSPARK_INFO;

// the describe of EVENT_IVS_SMOKEDETECTION's data
typedef struct tagDEV_EVENT_SMOKE_INFO 
{
	int					nChannelID;						// ChannelId
	char				szName[128];					// event name
	char                bReserved1[4];                  // byte alignment
	double				PTS;							// PTS(ms)
	NET_TIME_EX			UTC;							// the event happen time
	int					nEventID;						// event ID
	DH_MSG_OBJECT		stuObject;						// have being detected object
	DH_EVENT_FILE_INFO  stuFileInfo;                    // event file info
	BYTE                bEventAction;                   // Event action,0 means pulse event,1 means continuous event's begin,2means continuous event's end;
    BYTE                byReserved[2];
    BYTE                byImageIndex;                   // Serial number of the picture, in the same time (accurate to seconds) may have multiple images, starting from 0
    DWORD               dwSnapFlagMask;	                // flag(by bit),see NET_RESERVED_COMMON
	unsigned int    	nOccurrenceCount;				// event trigger accumilated times 
  	EVENT_INTELLI_COMM_INFO     stuIntelliCommInfo;     // intelli comm info
	PTZ_SPACE_UNIT		stuPtzPosition;					// ptz coordinate and zoom
	BYTE				bReserved[792];					
	
} DEV_EVENT_SMOKE_INFO;

// the describe of EVENT_IVS_FLOWSTAT's data
typedef struct tagDEV_EVENT_FLOWSTAT_INFO 
{
	int					nChannelID;						// ChannelId
	char				szName[128];					// event name
	char                bReserved1[4];                  // byte alignment
	double				PTS;							// PTS(ms)
	NET_TIME_EX			UTC;							// the event happen time
	int					nEventID;						// event ID
	int					nNumberLeft;					// the number of person across from left
	int					nNumberRight;					// the number of person across from right
	int					nUpperLimit;					// upper limit
	DH_EVENT_FILE_INFO  stuFileInfo;                    // event file info
	BYTE                bEventAction;                   // Event action,0 means pulse event,1 means continuous event's begin,2means continuous event's end;
    BYTE                byReserved[2];
    BYTE                byImageIndex;                   // Serial number of the picture, in the same time (accurate to seconds) may have multiple images, starting from 0
    DWORD               dwSnapFlagMask;	                // flag(by bit),see NET_RESERVED_COMMON
	unsigned int    	nOccurrenceCount;          		// event trigger accumilated times 
	BYTE				bReserved[972];					

} DEV_EVENT_FLOWSTAT_INFO;

// the describe of EVENT_IVS_NUMBERSTAT's data
typedef struct tagDEV_EVENT_NUMBERSTAT_INFO 
{
	int					nChannelID;						// ChannelId
	char				szName[128];					// event name
	char                bReserved2[4];                  // byte alignment
	double				PTS;							// PTS(ms)
	NET_TIME_EX			UTC;							// the event happen time
	int					nEventID;						// event ID
	int					nNumber;						// the number of object which is in the area
	int					nUpperLimit;					// upper limit
	DH_EVENT_FILE_INFO  stuFileInfo;                    // event file info
	BYTE                bEventAction;                   // Event action,0 means pulse event,1 means continuous event's begin,2means continuous event's end;
    BYTE                bReserved1[2];                  // 
    BYTE                byImageIndex;                   // Serial number of the picture, in the same time (accurate to seconds) may have multiple images, starting from 0
	int                 nEnteredNumber;                 // entered object number
	int                 nExitedNumber;                  // exited object number
    DWORD               dwSnapFlagMask;	                // flag(by bit),see NET_RESERVED_COMMON
	unsigned int    	nOccurrenceCount;          		// event trigger accumilated times 
  	EVENT_INTELLI_COMM_INFO     stuIntelliCommInfo;     // intelli comm info
	BYTE				bReserved[828];					// Reserved
} DEV_EVENT_NUMBERSTAT_INFO;

// the describe of EVENT_IVS_CROSSFENCEDETECTION's data
typedef struct tagDEV_EVENT_CROSSFENCEDETECTION_INFO 
{
	int					nChannelID;						// ChannelId
	char				szName[128];					// event name
	char                bReserved1[4];                  // byte alignment
	double				PTS;							// PTS(ms)
	NET_TIME_EX			UTC;							// the event happen time
	int					nEventID;						// event ID
	DH_MSG_OBJECT		stuObject;						// have being detected object
	int					nUpstairsLinePointNumber;		               // Upstairs Line Point Number
	DH_POINT		    stuUpstairsLine[DH_MAX_DETECT_LINE_NUM];	   // Upstairs Line info
	int					nDownstairsLinePointNumber;		               // Downstairs Line Point Number
	DH_POINT		    stuDownstairsLine[DH_MAX_DETECT_LINE_NUM];     // Downstairs Line info
	int                 nTrackLineNum;                                 // track line point number                
	DH_POINT            TrackLine[DH_MAX_TRACK_LINE_NUM];              // track line info
	DH_EVENT_FILE_INFO  stuFileInfo;                    // event file info
	BYTE                bEventAction;                   // Event action,0 means pulse event,1 means continuous event's begin,2means continuous event's end;
	BYTE                bDirection;                     // direction, 0-left to right, 1-right to left
    BYTE                byReserved[1];
    BYTE                byImageIndex;                   // Serial number of the picture, in the same time (accurate to seconds) may have multiple images, starting from 0
    DWORD               dwSnapFlagMask;	                // flag(by bit),see NET_RESERVED_COMMON
	int                 nSourceIndex;                   // the source device's index,-1 means data in invalid
	char                szSourceDevice[MAX_PATH];       // the source device's sign(exclusive),field said local device does not exist or is empty
	unsigned int    	nOccurrenceCount;              	// event trigger accumilated times 
	EVENT_INTELLI_COMM_INFO     stuIntelliCommInfo;     // intelli comm info
	BYTE				bReserved[616];					// Reserved
} DEV_EVENT_CROSSFENCEDETECTION_INFO;

// the describe of EVENT_IVS_INREGIONDETECTION's data
typedef struct tagDEV_EVENT_INREGIONDETECTION_INFO 
{
	int					nChannelID;						// ChannelId
	char				szName[128];					// event name
	char                bReserved1[4];                  // byte alignment
	double				PTS;							// PTS(ms)
	NET_TIME_EX			UTC;							// the event happen time
	int					nEventID;						// event ID
	int                 nObjectNum;                     // have being detected objects number
	DH_MSG_OBJECT		stuObjectIDs[DH_MAX_OBJECT_LIST];	// have being detected objects
	int                 nTrackNum;                      // track line number
	DH_POLY_POINTS      stuTrackInfo[DH_MAX_OBJECT_LIST];// track lines info
	int                 nDetectRegionNum;				// detect regions number
	DH_POINT            DetectRegion[DH_MAX_DETECT_REGION_NUM];    // detect regions
	DH_EVENT_FILE_INFO  stuFileInfo;                    // event file info
	BYTE                bEventAction;                   // Event action,0 means pulse event,1 means continuous event's begin,2means continuous event's end;
    BYTE                byReserved[2];
    BYTE                byImageIndex;                   // Serial number of the picture, in the same time (accurate to seconds) may have multiple images, starting from 0
    DWORD               dwSnapFlagMask;	                // flag(by bit),see NET_RESERVED_COMMON
	BYTE				bReserved[1016];				// Reserved
	
} DEV_EVENT_INREGIONDETECTION_INFO;

// the describe of EVENT_IVS_TAKENAWAYDETECTION's data
typedef struct tagDEV_EVENT_TAKENAWAYDETECTION_INFO
{
	int					nChannelID;						// ChannelId
	char				szName[128];					// event name
	char                bReserved1[4];                  // byte alignment
	double				PTS;							// PTS(ms)
	NET_TIME_EX			UTC;							// the event happen time
	int					nEventID;						// event ID
	DH_MSG_OBJECT		stuObject;						// have being detected object
	int                 nDetectRegionNum;				// detect region's point number
	DH_POINT            DetectRegion[DH_MAX_DETECT_REGION_NUM];    // detect region info
	DH_EVENT_FILE_INFO  stuFileInfo;                    // event file info
	BYTE                bEventAction;                   // Event action,0 means pulse event,1 means continuous event's begin,2means continuous event's end;
    BYTE                byReserved[2];
    BYTE                byImageIndex;                   // Serial number of the picture, in the same time (accurate to seconds) may have multiple images, starting from 0
    DWORD               dwSnapFlagMask;	                // flag(by bit),see NET_RESERVED_COMMON
	int                 nSourceIndex;                   // the source device's index,-1 means data in invalid
	char                szSourceDevice[MAX_PATH];       // the source device's sign(exclusive),field said local device does not exist or is empty
	unsigned int    	nOccurrenceCount;               // event trigger accumilated times 
  	EVENT_INTELLI_COMM_INFO     stuIntelliCommInfo;     // intelli comm info
	short				nPreserID;						// Event triggered preset period, starting from 1 (no unknown)
	char				szPresetName[64];				// Preset name for event triggered
	BYTE				bReserved[550];				  	// Reserved
} DEV_EVENT_TAKENAWAYDETECTION_INFO;

// the describe of EVENT_IVS_VIDEOABNORMALDETECTION's data
typedef struct tagDEV_EVENT_VIDEOABNORMALDETECTION_INFO
{
	int					nChannelID;						// ChannelId
	char				szName[128];					// event name
	char                bReserved1[4];                  // byte alignment
	double				PTS;							// PTS(ms)
	NET_TIME_EX			UTC;							// the event happen time
	int					nEventID;						// event ID
	DH_EVENT_FILE_INFO  stuFileInfo;                    // event file info
	BYTE                bEventAction;                   // Event action,0 means pulse event,1 means continuous event's begin,2means continuous event's end;
	BYTE                bType;                          // type, 0-video lost, 1-video freeze, 2-video blind, 3-camera moving, 4-too dark, 5-too light, 6-image unbalance, 7-Noise
    BYTE                byReserved[1];
    BYTE                byImageIndex;                   // Serial number of the picture, in the same time (accurate to seconds) may have multiple images, starting from 0
    DWORD               dwSnapFlagMask;	                // flag(by bit),see NET_RESERVED_COMMON
	int                 nSourceIndex;                   // the source device's index,-1 means data in invalid
	char                szSourceDevice[MAX_PATH];       // the source device's sign(exclusive),field said local device does not exist or is empty
	unsigned int    	nOccurrenceCount;               // event trigger accumilated times 
  	EVENT_INTELLI_COMM_INFO     stuIntelliCommInfo;     // intelli comm info
	BYTE				bReserved[620];					// Reserved
} DEV_EVENT_VIDEOABNORMALDETECTION_INFO;

// the describe of EVENT_IVS_PARKINGDETECTION's data
typedef struct tagDEV_EVENT_PARKINGDETECTION_INFO
{
	int					nChannelID;						// ChannelId
	char				szName[128];					// event name
	char                bReserved1[4];                  // byte alignment
	double				PTS;							// PTS(ms)
	NET_TIME_EX			UTC;							// the event happen time
	int					nEventID;						// event ID
	DH_MSG_OBJECT		stuObject;						// have being detected object
	int                 nDetectRegionNum;				// detect region's point number
	DH_POINT            DetectRegion[DH_MAX_DETECT_REGION_NUM];    // detect region info
	DH_EVENT_FILE_INFO  stuFileInfo;                    // event file info
	BYTE                bEventAction;                   // Event action,0 means pulse event,1 means continuous event's begin,2means continuous event's end;
    BYTE                byReserved[2];
    BYTE                byImageIndex;                   // Serial number of the picture, in the same time (accurate to seconds) may have multiple images, starting from 0
    DWORD               dwSnapFlagMask;	                // flag(by bit),see NET_RESERVED_COMMON
	int                 nSourceIndex;                   // the source device's index,-1 means data in invalid
	char                szSourceDevice[MAX_PATH];       // the source device's sign(exclusive),field said local device does not exist or is empty
	unsigned int    	nOccurrenceCount;               // event trigger accumilated times 
  	EVENT_INTELLI_COMM_INFO     stuIntelliCommInfo;   	// intelli comm info
	BYTE				bReserved[616];				  	// Reserved 
} DEV_EVENT_PARKINGDETECTION_INFO;

// the describe of EVENT_IVS_ABNORMALRUNDETECTION's data
typedef struct tagDEV_EVENT_ABNORMALRUNDETECTION 
{
	int					nChannelID;						// ChannelId
	char				szName[128];					// event name
	char                bReserved1[4];                  // byte alignment
	double				PTS;							// PTS(ms)
	NET_TIME_EX			UTC;							// the event happen time
	int					nEventID;						// event ID
	DH_MSG_OBJECT		stuObject;						// have being detected object
	double              dbSpeed;                        // speed ,km/h
	double              dbTriggerSpeed;                 // triggerSpeed,km/h
	int                 nDetectRegionNum;				// detect region's point number
	DH_POINT            DetectRegion[DH_MAX_DETECT_REGION_NUM];    // detect region info
	int                 nTrackLineNum;                         // track line point number              
	DH_POINT            TrackLine[DH_MAX_TRACK_LINE_NUM];      // track line info
	DH_EVENT_FILE_INFO  stuFileInfo;                    // event file info
	BYTE                bEventAction;                   // Event action,0 means pulse event,1 means continuous event's begin,2means continuous event's end;
	BYTE                bRunType;                       // type, 0-run fast, 1-sudden speedup, 2-sudden speed-down
    BYTE                byReserved[1];
    BYTE                byImageIndex;                   // Serial number of the picture, in the same time (accurate to seconds) may have multiple images, starting from 0
    DWORD               dwSnapFlagMask;	                // flag(by bit),see NET_RESERVED_COMMON
	int                 nSourceIndex;                   // the source device's index,-1 means data in invalid
	char                szSourceDevice[MAX_PATH];       // the source device's sign(exclusive),field said local device does not exist or is empty
	unsigned int    	nOccurrenceCount;              	// event trigger accumilated times 
    EVENT_INTELLI_COMM_INFO     stuIntelliCommInfo;     // intelligent things info
    BYTE                bReserved[616];                 // Reserved bytes.
	
} DEV_EVENT_ABNORMALRUNDETECTION_INFO;

// the describe of EVENT_IVS_RETROGRADEDETECTION's data
typedef struct tagDEV_EVENT_RETROGRADEDETECTION_INFO 
{
	int					nChannelID;						// ChannelId
	char				szName[128];					// event name
	char                bReserved1[4];                  // byte alignment
	double				PTS;							// PTS(ms)
	NET_TIME_EX			UTC;							// the event happen time
	int					nEventID;						// event ID
	DH_MSG_OBJECT		stuObject;						// have being detected object
	int                 nTrackLineNum;                           // track line point number                   
	DH_POINT            TrackLine[DH_MAX_TRACK_LINE_NUM];        // track line info
	int                 nDirectionPointNum;                      // direction point number
	DH_POINT            stuDirections[DH_MAX_DETECT_LINE_NUM];   // direction info
	int                 nDetectRegionNum;				         // detect region's point number
	DH_POINT            DetectRegion[DH_MAX_DETECT_REGION_NUM];  // detect region info
	DH_EVENT_FILE_INFO  stuFileInfo;                    // event file info
	BYTE                bEventAction;                   // Event action,0 means pulse event,1 means continuous event's begin,2means continuous event's end;
    BYTE                byReserved[2];
    BYTE                byImageIndex;                   // Serial number of the picture, in the same time (accurate to seconds) may have multiple images, starting from 0
    DWORD               dwSnapFlagMask;	                // flag(by bit),see NET_RESERVED_COMMON
	int                 nSourceIndex;                   // the source device's index,-1 means data in invalid
	char                szSourceDevice[MAX_PATH];       // the source device's sign(exclusive),field said local device does not exist or is empty
	unsigned int    	nOccurrenceCount;              	// event trigger accumilated times 
	EVENT_INTELLI_COMM_INFO     stuIntelliCommInfo;     // intelli comm info
	BYTE				bReserved[616];				  	// Reserved 
} DEV_EVENT_RETROGRADEDETECTION_INFO;

//sex type of dectected human face
typedef enum tagEM_DEV_EVENT_FACEDETECT_SEX_TYPE
{
    EM_DEV_EVENT_FACEDETECT_SEX_TYPE_UNKNOWN,                   // unknown
    EM_DEV_EVENT_FACEDETECT_SEX_TYPE_MAN,                       // male
    EM_DEV_EVENT_FACEDETECT_SEX_TYPE_WOMAN,                     // female
}EM_DEV_EVENT_FACEDETECT_SEX_TYPE;

//feature type of detected human face
typedef enum tagEM_DEV_EVENT_FACEDETECT_FEATURE_TYPE
{
    EM_DEV_EVENT_FACEDETECT_FEATURE_TYPE_UNKNOWN,               // unknown
    EM_DEV_EVENT_FACEDETECT_FEATURE_TYPE_WEAR_GLASSES,          // wearing glasses
    EM_DEV_EVENT_FACEDETECT_FEATURE_TYPE_SMILE,                 // smile
	EM_DEV_EVENT_FACEDETECT_FEATURE_TYPE_ANGER,                 // anger
    EM_DEV_EVENT_FACEDETECT_FEATURE_TYPE_SADNESS,               // sadness
    EM_DEV_EVENT_FACEDETECT_FEATURE_TYPE_DISGUST,               // disgust
    EM_DEV_EVENT_FACEDETECT_FEATURE_TYPE_FEAR,                  // fear
    EM_DEV_EVENT_FACEDETECT_FEATURE_TYPE_SURPRISE,              // surprise
    EM_DEV_EVENT_FACEDETECT_FEATURE_TYPE_NEUTRAL,               // neutral
    EM_DEV_EVENT_FACEDETECT_FEATURE_TYPE_LAUGH,                 // laugh
    EM_DEV_EVENT_FACEDETECT_FEATURE_TYPE_NOGLASSES,				// not wear glasses
    EM_DEV_EVENT_FACEDETECT_FEATURE_TYPE_HAPPY,					// happy
    EM_DEV_EVENT_FACEDETECT_FEATURE_TYPE_CONFUSED,				// confused
    EM_DEV_EVENT_FACEDETECT_FEATURE_TYPE_SCREAM,				// scream
    EM_DEV_EVENT_FACEDETECT_FEATURE_TYPE_WEAR_SUNGLASSES,       // wearing sun glasses
}EM_DEV_EVENT_FACEDETECT_FEATURE_TYPE;

// race type
typedef enum tagEM_RACE_TYPE
{
	EM_RACE_UNKNOWN,			// unknown
	EM_RACE_NODISTI,			// no disringuish
	EM_RACE_YELLOW,				// yellow
	EM_RACE_BLACK,				// black
	EM_RACE_WHITE,				// white
} EM_RACE_TYPE;

// eyes state
typedef enum tagEM_EYE_STATE_TYPE
{
	EM_EYE_STATE_UNKNOWN,		// unknown
	EM_EYE_STATE_NODISTI,		// no disringuish
	EM_EYE_STATE_CLOSE,			// close eyes
	EM_EYE_STATE_OPEN,			// open eyes
} EM_EYE_STATE_TYPE;

// mouth state
typedef enum tagEM_MOUTH_STATE_TYPE
{
	EM_MOUTH_STATE_UNKNOWN,		// unknown
	EM_MOUTH_STATE_NODISTI,		// no disringuish
	EM_MOUTH_STATE_CLOSE,		// close mouth
	EM_MOUTH_STATE_OPEN,		// open nouth
} EM_MOUTH_STATE_TYPE;

// mask state
typedef enum tagEM_MASK_STATE_TYPE
{
	EM_MASK_STATE_UNKNOWN,		// unknown
	EM_MASK_STATE_NODISTI,		// no disringuish
	EM_MASK_STATE_NOMASK,		// no mask
	EM_MASK_STATE_WEAR,			// wearing mask
} EM_MASK_STATE_TYPE;

// beard state
typedef enum tagEM_BEARD_STATE_TYPE
{
	EM_BEARD_STATE_UNKNOWN,		// unknown
	EM_BEARD_STATE__NODISTI,	// no disringuish
	EM_BEARD_STATE_NOBEARD,		// no beard
	EM_BEARD_STATE_HAVEBEARD,	// have beard
} EM_BEARD_STATE_TYPE;


#define DH_MAX_FACEDETECT_FEATURE_NUM          32                      // max number of detected human face feature

// the data of face
typedef struct tagNET_FACE_DATA
{
	EM_DEV_EVENT_FACEDETECT_SEX_TYPE 		emSex;						// sex type
	int        								nAge;						// age, invalid if it is -1
    unsigned int        					nFeatureValidNum;           // invalid number in array emFeature
    EM_DEV_EVENT_FACEDETECT_FEATURE_TYPE    emFeature[DH_MAX_FACEDETECT_FEATURE_NUM];   // human face features
	EM_RACE_TYPE							emRace;						// race
	EM_EYE_STATE_TYPE						emEye;						// eyes state
	EM_MOUTH_STATE_TYPE						emMouth;					// mouth state
	EM_MASK_STATE_TYPE 						emMask;						// mask state
	EM_BEARD_STATE_TYPE						emBeard;					// beard state
	int										nAttractive;				// Attractive value, -1: invalid, 0:no disringuishrange: 1-100, the higher value, the higher charm
    BYTE                					bReserved[128];             // Reserved 
} NET_FACE_DATA;

// the describe of EVENT_IVS_FACERECOGNITION's data
typedef struct tagDEV_EVENT_FACERECOGNITION_INFO
{
	int					nChannelID;						// ChannelId
	char				szName[128];					// event name
    int                 nEventID;                       // event ID
	NET_TIME_EX			UTC;							// the event happen time
	DH_MSG_OBJECT		stuObject;						// have being detected object
	int                 nCandidateNum;                  // candidate number
    CANDIDATE_INFO      stuCandidates[DH_MAX_CANDIDATE_NUM]; // candidate info
	BYTE                bEventAction;                   // Event action,0 means pulse event,1 means continuous event's begin,2means continuous event's end;
    BYTE                byImageIndex;                   // Serial number of the picture, in the same time (accurate to seconds) may have multiple images, starting from 0
	BYTE				byReserved1[2];				    // reserved
	BOOL                bGlobalScenePic;                // The existence panorama
	DH_PIC_INFO         stuGlobalScenePicInfo;          // Panoramic Photos
    char                szSnapDevAddress[MAX_PATH];     // Snapshot current face aadevice address  
    unsigned int        nOccurrenceCount;               // event trigger accumilated times 
    EVENT_INTELLI_COMM_INFO     stuIntelliCommInfo;     // intelligent things info
    NET_FACE_DATA		stuFaceData;					// the data of face
    BYTE                bReserved[300];                 // Reserved bytes.
}DEV_EVENT_FACERECOGNITION_INFO;

// Event type EVENT_IVS_DENSITYDETECTION(Population amount detect) corresponding data block description info
typedef struct tagDEV_EVENT_DENSITYDETECTTION_INFO
{
	int					nChannelID;						// Channel No.
	char				szName[128];					// Event name
	char                bReserved1[4];                  // byte alignment
	double				PTS;							// Time stamp(ms)
	NET_TIME_EX			UTC;							// Event occurred time
	int					nEventID;						// Event ID
	int					nObjectNum;						// Detected object amount
	DH_MSG_OBJECT		stuObjectIDs[DH_MAX_OBJECT_LIST];	// Detected object list
	DH_EVENT_FILE_INFO  stuFileInfo;                    // The corresponding file info of the event
	BYTE                bEventAction;                   // Event operation.0=pulse event,1=begin of the durative event,2=end of the durative event;
	BYTE                byReserved[2];                  // Reserved field
    BYTE                byImageIndex;                   // Serial number of the picture, in the same time (accurate to seconds) may have multiple images, starting from 0
	int                 nDetectRegionNum;				// Acme amount of the rule detect zone
	DH_POINT            DetectRegion[DH_MAX_DETECT_REGION_NUM];    // Rule detect zone
	
    DWORD               dwSnapFlagMask;	                // Snap flag(by bit).please refer to NET_RESERVED_COMMON
	int                 nSourceIndex;                   // the source device's index,-1 means data in invalid
	char                szSourceDevice[MAX_PATH];       // the source device's sign(exclusive),field said local device does not exist or is empty
    unsigned int        nOccurrenceCount;               // event trigger accumilated times 
	EVENT_INTELLI_COMM_INFO     stuIntelliCommInfo;		// intelligent things info
	int					nUnit;							// Numerical density check test unit
	int					nValue;							// Density value, the unit is Level, the value of 0~3,respectively(very thin, poor, dense, very dense)units for Percent, 1~100
	BYTE                bReserved[612];                 // Reserved bytes.
	
}DEV_EVENT_DENSITYDETECTION_INFO;

// Event type  EVENT_IVS_QUEUEDETECTION(queue detection)corresponding data block description info
typedef struct tagDEV_EVENT_QUEUEDETECTION_INFO 
{
	int					nChannelID;						// channel ID
	char				szName[128];					// event name
	char                bReserved2[4];                  // byte alignment
	double				PTS;							// PTS(ms)
	NET_TIME_EX			UTC;							// the event happen time
	int					nEventID;						// event ID
	DH_EVENT_FILE_INFO  stuFileInfo;                    // event file info
	BYTE                bEventAction;                   // Event action,0 means pulse event,1 means continuous event's begin,2means continuous event's end;
	BYTE				bReserved1[2];				    // reserved
    BYTE                byImageIndex;                   // Serial number of the picture, in the same time (accurate to seconds) may have multiple images, starting from 0
	DH_POINT            stuDetectLine[2];               // detect line
    unsigned int        nOccurrenceCount;               // event trigger accumilated times 
    BYTE                bReserved[1012];                // reserved
}DEV_EVENT_QUEUEDETECTION_INFO;

// Event type EVENT_IVS_TRAFFICCONTROL(traffic control)corresponding data block description info
typedef struct tagDEV_EVENT_TRAFFICCONTROL_INFO 
{
	int					nChannelID;						// ChannelId
	char				szName[128];					// event name
	char                bReserved1[4];                  // byte alignment
	double				PTS;							// PTS(ms)
	NET_TIME_EX			UTC;							// the event happen time
	int					nEventID;						// event ID
	DH_MSG_OBJECT		stuObject;						// have being detected object
	DH_EVENT_FILE_INFO  stuFileInfo;                    // event file info
	BYTE                bEventAction;                   // Event action,0 means pulse event,1 means continuous event's begin,2means continuous event's end;
	BYTE                byReserved[2];
    BYTE                byImageIndex;                   // Serial number of the picture, in the same time (accurate to seconds) may have multiple images, starting from 0
    DWORD               dwSnapFlagMask;	                // flag(by bit),see NET_RESERVED_COMMON
    NET_GPS_INFO        stuGPSInfo;                     // GPS info ,use in mobile DVR/NVR
	BYTE				bReserved[932];					// Reserved field. For extension use. 

} DEV_EVENT_TRAFFICCONTROL_INFO;

// the describe of EVENT_IVS_TRAFFICACCIDENT's data
typedef struct tagDEV_EVENT_TRAFFICACCIDENT_INFO 
{
	int					nChannelID;						// ChannelId
	char				szName[128];					// event name
	char                bReserved1[4];                  // byte alignment
	double				PTS;							// PTS(ms)
	NET_TIME_EX			UTC;							// the event happen time
	int					nEventID;						// event ID
	int					nObjectNum;						// have being detected object number
	DH_MSG_OBJECT		stuObjectIDs[DH_MAX_OBJECT_LIST];// have being detected object list
	DH_EVENT_FILE_INFO  stuFileInfo;                    // event file info
	BYTE                bEventAction;                   // Event action,0 means pulse event,1 means continuous event's begin,2means continuous event's end;
	BYTE                byReserved[2];
    BYTE                byImageIndex;                   // Serial number of the picture, in the same time (accurate to seconds) may have multiple images, starting from 0
    DWORD               dwSnapFlagMask;	                // flag(by bit),see NET_RESERVED_COMMON
    EVENT_TRAFFIC_CAR_PART_INFO   stuTrafficCarPartInfo;// part of traffic car info
    BYTE                bReserved[464];                 // reserved

} DEV_EVENT_TRAFFICACCIDENT_INFO;

#define DH_EVENT_MAX_CARD_NUM       16    // Incident reporting information includes the maximum number of cards
#define DH_EVENT_CARD_LEN           36    // Card Name Length
 
// Incidents reported to carry the card information
typedef struct tagEVENT_CARD_INFO
{
    char szCardNumber[DH_EVENT_CARD_LEN];     // Card number string
    BYTE bReserved[32];	                      // Reserved bytes, leave extended
}EVENT_CARD_INFO;

typedef enum tagEM_OPEN_STROBE_STATE
{
    NET_OPEN_STROBE_STATE_UNKOWN,                   // unknown
    NET_OPEN_STROBE_STATE_CLOSE,                    // close
    NET_OPEN_STROBE_STATE_AUTO,                     // auto open   
    NET_OPEN_STROBE_STATE_MANUAL,                   // manual open
}EM_OPEN_STROBE_STATE;

typedef enum tagEM_VEHICLE_DIRECTION
{
    NET_VEHICLE_DIRECTION_UNKOWN,                   // unknown 
    NET_VEHICLE_DIRECTION_HEAD,                     // head    
    NET_VEHICLE_DIRECTION_TAIL,                     // rear  
}EM_VEHICLE_DIRECTION;

//NTP status 
typedef enum tagEM_NTP_STATUS
{
    NET_NTP_STATUS_UNKNOWN = 0 ,
    NET_NTP_STATUS_DISABLE     , 
    NET_NTP_STATUS_SUCCESSFUL  , 
    NET_NTP_STATUS_FAILED      , 
}EM_NTP_STATUS;

#define COMMON_SEAT_MAX_NUMBER        8             // Max seat number

typedef enum tagEM_COMMON_SEAT_TYPE
{
    COMMON_SEAT_TYPE_UNKNOWN    = 0,                // unknown
    COMMON_SEAT_TYPE_MAIN       = 1,                // main seat
    COMMON_SEAT_TYPE_SLAVE      = 2,                // slave seat
}EM_COMMON_SEAT_TYPE;

// illegal state type of driver
typedef struct tagEVENT_COMM_STATUS                 
{
    BYTE bySmoking;                                 // smoking
    BYTE byCalling;                                 // calling
    char szReserved[14];                            // reversed
}EVENT_COMM_STATUS;

typedef enum tagNET_SAFEBELT_STATE
{
    SS_NUKNOW   = 0 ,				// Unknow
	SS_WITH_SAFE_BELT ,				// WithSafeBelt   
	SS_WITHOUT_SAFE_BELT ,			// WithoutSafeBelt 
}NET_SAFEBELT_STATE;

//sun shade state
typedef enum tagNET_SUNSHADE_STATE
{
    SS_NUKNOW_SUN_SHADE	= 0 ,		// Unknow
	SS_WITH_SUN_SHADE ,				// WithSunShade  
	SS_WITHOUT_SUN_SHADE ,			// WithoutSunShade
}NET_SUNSHADE_STATE;

// driver's illegal info
typedef struct tagEVENT_COMM_SEAT
{
    BOOL                    bEnable;                // whether seat info detected
    EM_COMMON_SEAT_TYPE     emSeatType;             // seat type
    EVENT_COMM_STATUS       stStatus;               // illegal state
	NET_SAFEBELT_STATE      emSafeBeltStatus;       // safe belt state
    NET_SUNSHADE_STATE      emSunShadeStatus;       // sun shade state
    char                    szReserved[24];         // reversed
}EVENT_COMM_SEAT;

typedef enum tagEM_COMM_ATTACHMENT_TYPE      
{       
	COMM_ATTACHMENT_TYPE_UNKNOWN    = 0,            // Unknown type       
	COMM_ATTACHMENT_TYPE_FURNITURE  = 1,            // Furniture       
	COMM_ATTACHMENT_TYPE_PENDANT    = 2,            // Pendant       
	COMM_ATTACHMENT_TYPE_TISSUEBOX  = 3,            // TissueBox       
	COMM_ATTACHMENT_TYPE_DANGER     = 4,            // Danger     
	COMM_ATTACHMENT_TYPE_PERFUMEBOX = 5,			// perfumebox
 }EM_COMM_ATTACHMENT_TYPE; 

// car attachment
typedef struct tagEVENT_COMM_ATTACHMENT
{
    EM_COMM_ATTACHMENT_TYPE     emAttachmentType;       // type
	NET_RECT                    stuRect;                // coordinate
	BYTE						bReserved[20];		    // reserved
}EVENT_COMM_ATTACHMENT;

// traffic event snap picture info
typedef struct tagEVENT_PIC_INFO
{
    DWORD                       nOffset;                // offset
    DWORD                       nLength;                // length of picture
}EVENT_PIC_INFO;

typedef struct tagEVENT_COMM_INFO
{
    EM_NTP_STATUS				emNTPStatus;										//NTP time sync status 
    int							nDriversNum;										//driver info number
    DH_MSG_OBJECT_EX*			pstDriversInfo;										//driver info data 
    char*						pszFilePath;										//writing path for loacl disk or sd card, or write to default path if NULL
    char*						pszFTPPath;											//ftp path
    char*						pszVideoPath;										//ftp path for assocated video
    EVENT_COMM_SEAT				stCommSeat[COMMON_SEAT_MAX_NUMBER];					//Seat info
	int							nAttachmentNum;										// Car Attachment number
	EVENT_COMM_ATTACHMENT		stuAttachment[NET_MAX_ATTACHMENT_NUM];				// Car Attachment 
	int							nAnnualInspectionNum;								// Annual Inspection number
	NET_RECT					stuAnnualInspection[NET_MAX_ANNUUALINSPECTION_NUM];	// Annual Inspection
    float                       fHCRatio;                                           // The ratio of HC,unit,%/1000000
    float                       fNORatio;                                           // The ratio of NO,unit,%/1000000
    float                       fCOPercent;                                         // The percent of CO,unit,% ,range from 0 to 100
    float                       fCO2Percent;                                        // The percent of CO2,unit: % ,range from 0 to 100     
    float                       fLightObscuration;                                  // The obscuration of light,unit,% ,range from 0 to 100
    int                         nPictureNum;                                        // Original pictures info number
    EVENT_PIC_INFO              stuPicInfos[NET_MAX_EVENT_PIC_NUM];                 // Original pictures info data
    float                       fTemperature;                                       // Temperature,unit: centigrade
    int                         nHumidity;                                          // Humidity,unit: %  
    float                       fPressure;                                          // Pressure,unit: Kpa
    float                       fWindForce;                                         // Wind force,unit: m/s
    UINT                        nWindDirection;                                     // Wind direction,unit: degree,range:[0,360]
    float                       fRoadGradient;                                      // Road gradient,unit: degree
    float                       fAcceleration;                                      // Acceleration,unit: m/s2   
    NET_RFIDELETAG_INFO			stuRFIDEleTagInfo;									// RFID electronics tag info
	BYTE						bReserved[704];								    	// reserved
    char						szCountry[20];										// Country
}EVENT_COMM_INFO;

//custom weight info
typedef struct tagEVENT_CUSTOM_WEIGHT_INFO
{
	DWORD        dwRoughWeight;                    // Rough Weight,unit:KG
    DWORD        dwTareWeight;                     // Tare Weight,unit:KG
    DWORD        dwNetWeight;                      // Net Weight,unit:KG
	BYTE		 bReserved[28];					   // reserved 
}EVENT_CUSTOM_WEIGHT_INFO;

//custom info in 
typedef struct tagEVENT_JUNCTION_CUSTOM_INFO
{
    EVENT_CUSTOM_WEIGHT_INFO    stuWeightInfo;      // custom weight info
	BYTE						bReserved[60];		// reserved 
}EVENT_JUNCTION_CUSTOM_INFO;


// Event Type EVENT_IVS_TRAFFICJUNCTION (transportation card traffic junctions old rule event / video port on the old electric alarm event rules) corresponding to the description of the data block, 
// Due to historical reasons, if you want to deal with bayonet event, DEV_EVENT_TRAFFICJUNCTION_INFO and EVENT_IVS_TRAFFICGATE be processed together to prevent police and video electrical coil electric alarm occurred while the case access platform, 
// Also EVENT_IVS_TRAFFIC_TOLLGATE only support the new bayonet events, 
typedef struct tagDEV_EVENT_TRAFFICJUNCTION_INFO 
{
	int					nChannelID;						// ChannelId
	char				szName[128];					// event name
    BYTE                byMainSeatBelt;                 // main driver, seat, safety belt , 1-fastened, 2-unfastened
    BYTE                bySlaveSeatBelt;                // co-drvier, seat, safety belt, 1-fastened, 2-unfastened
    BYTE                byVehicleDirection;             // Current snapshot is head or rear, see  EM_VEHICLE_DIRECTION
    BYTE                byOpenStrobeState;              // Open status, see EM_OPEN_STROBE_STATE 
	double				PTS;							// PTS(ms)
	NET_TIME_EX			UTC;							// the event happen time
	int					nEventID;						// event ID
	DH_MSG_OBJECT		stuObject;						// have being detected object
	int					nLane;							// road number
	DWORD				dwBreakingRule;					// BreakingRule's mask,first byte: crash red light; 
	                                                    // secend byte:break the rule of driving road number; 
	                                                    // the third byte:converse; the forth byte:break rule to turn around;
														// the five byte:traffic jam; the six byte:traffic vacancy; 
														// the seven byte: Overline; defalt:trafficJunction                                                        
	NET_TIME_EX			RedLightUTC;					// the begin time of red light
	DH_EVENT_FILE_INFO  stuFileInfo;                    // event file info
	int                 nSequence;                      // snap index,such as 3,2,1,1 means the last one,0 means there has some exception and snap stop
	int                 nSpeed;                         // car's speed (km/h)
	BYTE                bEventAction;                   // Event action,0 means pulse event,1 means continuous event's begin,2means continuous event's end;
	BYTE                byDirection;                    // Intersection direction 1 - denotes the forward 2 - indicates the opposite
    BYTE                byLightState;                   // LightState means red light status:0 unknown,1 green,2 red,3 yellow
	BYTE                byReserved;                  	// reserved
    BYTE                byImageIndex;                   // Serial number of the picture, in the same time (accurate to seconds) may have multiple images, starting from 0
	DH_MSG_OBJECT       stuVehicle;                     // vehicle info
    DWORD               dwSnapFlagMask;	                // flag(by bit),see NET_RESERVED_COMMON
	DH_RESOLUTION_INFO  stuResolution;                  // picture resolution
	char                szRecordFile[DH_COMMON_STRING_128]; // Alarm corresponding original video file information
	EVENT_JUNCTION_CUSTOM_INFO   stuCustomInfo;         // custom info
    BYTE                byPlateTextSource;              // the source of plate text, 0:Local,1:Server
	BYTE				bReserved1[3];				    // Reserved bytes, leave extended_
    NET_GPS_INFO        stuGPSInfo;                     // GPS info ,use in mobile DVR/NVR
	BYTE				bReserved[196];				    // Reserved bytes, leave extended_
    int                 nTriggerType;                   // Trigger Type:0 vehicle inspection device, 1 radar, 2 video
	DEV_EVENT_TRAFFIC_TRAFFICCAR_INFO stTrafficCar;     // Traffic vehicle info
	DWORD               dwRetCardNumber;                // Card Number
    EVENT_CARD_INFO     stuCardInfo[DH_EVENT_MAX_CARD_NUM];  // Card information
    EVENT_COMM_INFO     stCommInfo;                     // public info 
} DEV_EVENT_TRAFFICJUNCTION_INFO;


// the describe of EVENT_IVS_TRAFFICGATE's data
// owing to history, if you want to deal with TRAFFICGATE,DEV_EVENT_TRAFFICJUNCTION_INFO/EVENT_IVS_TRAFFICGATE must be handle together;
// in addition: EVENT_IVS_TRAFFIC_TOLLGATE only support new tollgate event configuration
typedef struct tagDEV_EVENT_TRAFFICGATE_INFO 
{
	int					nChannelID;						// ChannelId
	char				szName[128];					// event name
    BYTE                byOpenStrobeState;              // Open gateway status, see EM_OPEN_STROBE_STATE
	char                bReserved1[3];                  // byte alignment
	double				PTS;							// PTS(ms)
	NET_TIME_EX			UTC;							// the event happen time
	int					nEventID;						// event ID
	DH_MSG_OBJECT		stuObject;						// have being detected object
	int					nLane;							// road number
	int					nSpeed;							// the car's actual rate(Km/h)
	int					nSpeedUpperLimit;				// rate upper limit(km/h)
	int					nSpeedLowerLimit;				// rate lower limit(km/h) 
	DWORD				dwBreakingRule;					// BreakingRule's mask,first byte: Retrograde; 
	                                                    // second byte:Overline; the third byte:Overspeed; 
														// the forth byte:UnderSpeed;the five byte: crash red light;the six byte:passing(trafficgate)
	                                                    // the seven byte: OverYellowLine; the eight byte: WrongRunningRoute; the nine byte: YellowVehicleInRoute; default: trafficgate
	DH_EVENT_FILE_INFO  stuFileInfo;                    // event file info
	DH_MSG_OBJECT       stuVehicle;                     // vehicle info
	BYTE                szManualSnapNo[64];             // manual snap sequence string                 
	int                 nSequence;                      // snap index,such as 3,2,1,1 means the last one,0 means there has some exception and snap stop
	BYTE                bEventAction;                   // Event action,0 means pulse event,1 means continuous event's begin,2means continuous event's end;
	BYTE                byReserved[3];                  // reserved
	BYTE                szSnapFlag[16];                 // snap flag from device
	BYTE                bySnapMode;                     // snap mode,0-normal 1-globle 2-near 4-snap on the same side 8-snap on the reverse side 16-plant picture
	BYTE                byOverSpeedPercentage;          // over speed percentage
	BYTE                byUnderSpeedingPercentage;      // under speed percentage
	BYTE                byRedLightMargin;               // red light margin, s
	BYTE                byDriveDirection;               // drive direction,0-"Approach",where the car is more near,1-"Leave",means where if mor far to the car
	char                szRoadwayNo[32];                // road way number
	char                szViolationCode[16];            // violation code
	char                szViolationDesc[128];           // violation desc
	DH_RESOLUTION_INFO  stuResolution;                  // picture resolution
	char                szVehicleType[32];              // car type,"Motor", "Light-duty", "Medium", "Oversize", "Huge", "Other" 
	BYTE                byVehicleLenth;                 // car length, m
    BYTE                byLightState;                               // LightState means red light status:0 unknown,1 green,2 red,3 yellow
	BYTE                byReserved1;                 // reserved
    BYTE                byImageIndex;                   // Serial number of the picture, in the same time (accurate to seconds) may have multiple images, starting from 0
	int                 nOverSpeedMargin;               // over speed margin, km/h 
	int                 nUnderSpeedMargin;              // under speed margin, km/h 
	char                szDrivingDirection[3][DH_MAX_DRIVINGDIRECTION]; //
                                                      	// "DrivingDirection" : ["Approach", "Shanghai", "Hangzhou"],
	                                                  	// "Approach" means driving direction,where the car is more near;"Leave"-means where if mor far to the car
	                                                  	// the second and third param means the location of the driving direction
	char                szMachineName[256];             // machine name
	char                szMachineAddress[256];          // machine address
	char                szMachineGroup[256];            // machine group
    DWORD               dwSnapFlagMask;	                // flag(by bit),see NET_RESERVED_COMMON
	DH_SIG_CARWAY_INFO_EX stuSigInfo;                   // The vehicle detector generates the snap signal redundancy info
	char                szFilePath[MAX_PATH];           // File path
	NET_TIME_EX			RedLightUTC;					// the begin time of red light
	char              * szDeviceAddress;                // device address,OSD superimposed onto the image,from TrafficSnapshot.DeviceAddress,'\0'means end.
	float               fActualShutter;                 // Current picture exposure time, in milliseconds
	BYTE                byActualGain;                   // Current picture gain, ranging from 0 to 1000
    BYTE                byDirection;                    // 0-S to N  1-SW to NE 2-W to E 3-NW to SE 4-N to S 5-NE to SW 6-E to W 7-SE to NW 8-Unknown
    BYTE                bReserve;                       // Reserved bytes, byte alignment
    BYTE                bRetCardNumber;                 // Card Number
    EVENT_CARD_INFO     stuCardInfo[DH_EVENT_MAX_CARD_NUM];// Card information
    char               szDefendCode[DH_COMMON_STRING_64];// Waterproof
    int                nTrafficBlackListID;             // Link to balcklist main keyID, 0, invalid, > 0, blacklist data record
    EVENT_COMM_INFO     stCommInfo;                     // public info 
    BYTE                bReserved[452];	                // Reserved bytes, leave extended
} DEV_EVENT_TRAFFICGATE_INFO;

//the describe of EVENT_TRAFFICSNAPSHOT's data
typedef struct tagDEV_EVENT_TRAFFICSNAPSHOT_INFO 
{
	int					nChannelID;						// ChannelId
	char				szName[128];					// event name
	char                bReserved1[4];                  // byte alignment
	double				PTS;							// PTS(ms)
	NET_TIME_EX			UTC;							// the event happen time
	int					nEventID;						// event ID
	BYTE                bReserv[3];                     // reserved 
	BYTE                bCarWayCount;                   // car way number being snapshotting
	DH_CARWAY_INFO      stuCarWayInfo[DH_MAX_CARWAY_NUM]; // car way info being snapshotting
	DH_EVENT_FILE_INFO  stuFileInfo;                    // event file info
	BYTE                bEventAction;                   // Event action,0 means pulse event,1 means continuous event's begin,2means continuous event's end;
    BYTE                byReserved[2];
    BYTE                byImageIndex;                   // Serial number of the picture, in the same time (accurate to seconds) may have multiple images, starting from 0
    DWORD               dwSnapFlagMask;	                // flag(by bit),see NET_RESERVED_COMMON
	BYTE				bReserved[344];		    		// Reserved 
    EVENT_COMM_INFO     stCommInfo;                     // public info 
} DEV_EVENT_TRAFFICSNAPSHOT_INFO;

//the describe of EVENT_IVS_TRAFFIC_RUNREDLIGHT's data
typedef struct tagDEV_EVENT_TRAFFIC_RUNREDLIGHT_INFO
{
	int					nChannelID;						  // channel ID
	char				szName[128];					  // event name
	char                bReserved1[4];                    // byte alignment
	double				PTS;							  // PTS(ms)
	NET_TIME_EX			UTC;				              // the event happen time
	int					nEventID;			              // event ID
	int					nLane;				              // Corresponding Lane number
	DH_MSG_OBJECT		stuObject;	              	      // have being detected object
	DH_MSG_OBJECT       stuVehicle;                       // vehicle info
	DH_EVENT_FILE_INFO  stuFileInfo;                      // event file info
	int					nLightState;	              	  // state of traffic light 0:unknown 1:green 2:red 3:yellow
	int					nSpeed;			              	  // speed,km/h
	int                 nSequence;                        // snap index,such as 3,2,1,1 means the last one,0 means there has some exception and snap stop
	BYTE                bEventAction;					  // Event action,0 means pulse event,1 means continuous event's begin,2means continuous event's end;
    BYTE                byReserved[2];
    BYTE                byImageIndex;					  // Serial number of the picture, in the same time (accurate to seconds) may have multiple images, starting from 0
    DWORD               dwSnapFlagMask;					  // flag(by bit),see NET_RESERVED_COMMON
	NET_TIME_EX         stRedLightUTC;					  // time of red light starting
	DH_RESOLUTION_INFO  stuResolution;					  // picture resolution
	BYTE                byRedLightMargin;				  // red light margin, s
    BYTE                byAlignment[3];					  // Align string
    int                 nRedLightPeriod;                  // Red light period. The unit is ms. 
    NET_GPS_INFO        stuGPSInfo;                       // GPS info ,use in mobile DVR/NVR
    BYTE                bReserved[928];                   // Reserved string 
	DEV_EVENT_TRAFFIC_TRAFFICCAR_INFO stTrafficCar;       // Traffic vehicle info
	EVENT_COMM_INFO   	stCommInfo;                 	  // public info 
} DEV_EVENT_TRAFFIC_RUNREDLIGHT_INFO;

//Data description info of the event type EVENT_IVS_TRAFFIC_PEDESTRAINRUNREDLIGHT(trafic-pedestrian redlight running
typedef struct tagDEV_EVENT_TRAFFIC_PEDESTRAINRUNREDLIGHT_INFO
{
    int                 nChannelID;                                 // Channel No.
    char                szName[128];                                // Event name
    char                bReserved1[4];                              // Align string
    double              PTS;                                        // Time stamp (unit is ms)
    NET_TIME_EX         UTC;                                        // Event occurrence time 
    int                 nEventID;                                   // Event ID
    int                 nLane;                                      // Corresponding lane No. 
    DH_MSG_OBJECT       stuObject;                                  // Pedestrian info 
    DH_EVENT_FILE_INFO  stuFileInfo;                                // Corresponding file inof of the event  
    int                 nSequence;                                  // Snap SN. Such as 3,2,1,1 means stop snap. 0= abnormal stop. 
    BYTE                bEventAction;                               // Event operation. 0=pulse event. 1=continious event begin, 2=continuous event stop
    BYTE                byReserved[2];
    BYTE                byImageIndex;                               // Image SN. There can be several images at the same time (unit:second). Begins with 0. 
    DWORD               dwSnapFlagMask;                             // Snap mark (by byte).Please refer to  NET_RESERVED_COMMON 
    DH_RESOLUTION_INFO  stuResolution;                              // Corresponding image resolution 
    BYTE                bReserved[1024];                            // Reserved string 
    EVENT_COMM_INFO     stCommInfo;                                 // Public info 
}DEV_EVENT_TRAFFIC_PEDESTRAINRUNREDLIGHT_INFO;

//Data description info of the event type EVENT_IVS_TRAFFIC_PASSNOTINORDER(trafic-pass not in order)
typedef struct tagDEV_EVENT_TRAFFIC_PASSNOTINORDER_INFO
{
    int                 nChannelID;                                 // Channel No.
    char                szName[DH_EVENT_NAME_LEN];                  // Event name
    char                bReserved1[4];                              // Align string
    double              PTS;                                        // Time stamp (unit is ms)
    NET_TIME_EX         UTC;                                        // Event occurrence time 
    int                 nEventID;                                   // Event ID
    int                 nLane;                                      // Corresponding lane No. 
    DH_MSG_OBJECT       stuObject;                                  // plate info
    DH_MSG_OBJECT       stuVehicle;                                 // Vehicle info 
    DH_EVENT_FILE_INFO  stuFileInfo;                                // Corresponding file inof of the event  
    int                 nSequence;                                  // Snap SN. Such as 3,2,1,1 means stop snap. 0 - abnormal stop. 
    BYTE                bEventAction;                               // Event operation. 0-pulse event. 1-continious event begin, 2-continuous event stop
    BYTE                byReserved[2];
    BYTE                byImageIndex;                               // Image SN. There can be several images at the same time (unit:second). Begins with 0. 
    DWORD               dwSnapFlagMask;                             // Snap mark (by byte).Please refer to  NET_RESERVED_COMMON 
    DH_RESOLUTION_INFO  stuResolution;                              // Corresponding image resolution 
    DEV_EVENT_TRAFFIC_TRAFFICCAR_INFO stTrafficCar;                 // Traffic Car information
    EVENT_COMM_INFO     stCommInfo;                                 // Public info
    NET_GPS_INFO        stuGPSInfo;                                 // GPS info ,use in mobile DVR/NVR
    BYTE                bReserved[984];                             // Reserved string
}DEV_EVENT_TRAFFIC_PASSNOTINORDER_INFO;

//the describe of EVENT_IVS_TRAFFIC_OVERLINE's data
typedef struct tagDEV_EVENT_TRAFFIC_OVERLINE_INFO
{
	int					nChannelID;			// channel ID
	char				szName[128];		// event name
	char                bReserved1[4];      // byte alignment
	double				PTS;				// PTS(ms)
	NET_TIME_EX			UTC;				// the event happen time
	int					nEventID;			// event ID
	int					nLane;				// Corresponding Lane number
	DH_MSG_OBJECT		stuObject;			// have being detected object
	DH_MSG_OBJECT       stuVehicle;         // vehicle info
	DH_EVENT_FILE_INFO  stuFileInfo;        // event file info
	int                 nSequence;          // snap index,such as 3,2,1,1 means the last one,0 means there has some exception and snap stop
	int                 nSpeed;             // speed,km/h
	BYTE                bEventAction;		// Event action,0 means pulse event,1 means continuous event's begin,2means continuous event's end;
    BYTE                byReserved[2];
    BYTE                byImageIndex;       // Serial number of the picture, in the same time (accurate to seconds) may have multiple images, starting from 0
    DWORD               dwSnapFlagMask;	    // flag(by bit),see NET_RESERVED_COMMON
	DH_RESOLUTION_INFO  stuResolution;      // picture resolution	
    NET_GPS_INFO        stuGPSInfo;         // GPS info ,use in mobile DVR/NVR
	BYTE				bReserved[968];	    // Reserved 
	DEV_EVENT_TRAFFIC_TRAFFICCAR_INFO stTrafficCar;       // Traffic vehicle info
  EVENT_COMM_INFO   stCommInfo;                 // public info 
} DEV_EVENT_TRAFFIC_OVERLINE_INFO;


//the describe of EVENT_IVS_TRAFFIC_RETROGRADE's data
typedef struct tagDEV_EVENT_TRAFFIC_RETROGRADE_INFO
{
	int					nChannelID;			// channel ID
	char				szName[128];		// event name
	char                bReserved1[4];      // byte alignment
	double				PTS;				// PTS(ms)
	NET_TIME_EX			UTC;				// the event happen time
	int					nEventID;			// event ID
	int					nLane;				// Corresponding Lane number
	DH_MSG_OBJECT		stuObject;			// have being detected object
	DH_MSG_OBJECT       stuVehicle;         // vehicle info
	DH_EVENT_FILE_INFO  stuFileInfo;        // event file info
	int                 nSequence;          // snap index: such as 3,2,1,1 means the last one,0 means there has some exception and snap stop
	int                 nSpeed;             // speed, km/h
	BYTE                bEventAction;		// Event action,0 means pulse event,1 means continuous event's begin,2means continuous event's end;
    BYTE                byReserved[2];
    BYTE                byImageIndex;        // Serial number of the picture, in the same time (accurate to seconds) may have multiple images, starting from 0
    DWORD               dwSnapFlagMask;	     // flag(by bit),see NET_RESERVED_COMMON
	DH_RESOLUTION_INFO  stuResolution;       // picture resolution
	BOOL                bIsExistAlarmRecord; // a corresponding alarm recording; false: no corresponding alarm recording
	DWORD               dwAlarmRecordSize;   // Video size
	char                szAlarmRecordPath[DH_COMMON_STRING_256]; 	// Video Path
	EVENT_INTELLI_COMM_INFO     stuIntelliCommInfo;              	// intelli comm info
    NET_GPS_INFO        stuGPSInfo;                                 // GPS info ,use in mobile DVR/NVR
	BYTE				bReserved[484];								// Reserved bytes
	DEV_EVENT_TRAFFIC_TRAFFICCAR_INFO stTrafficCar;    			 	// Traffic vehicle info
	int                 nDetectNum;									// Acme amount of the rule detect zone
	DH_POINT            DetectRegion[DH_MAX_DETECT_REGION_NUM];    	// Rule detect zone 
	EVENT_COMM_INFO   	stCommInfo;         						// public info 
} DEV_EVENT_TRAFFIC_RETROGRADE_INFO;

//the describe of EVENT_IVS_TRAFFIC_TURNLEFT's data
typedef struct tagDEV_EVENT_TRAFFIC_TURNLEFT_INFO
{
	int					nChannelID;			// channel ID
	char				szName[128];		// event name
	char                bReserved1[4];      // byte alignment
	double				PTS;				// PTS(ms)
	NET_TIME_EX			UTC;				// the event happen time
	int					nEventID;			// event ID
	int					nLane;				// Corresponding Lane number
	DH_MSG_OBJECT		stuObject;			// have being detected object
	DH_MSG_OBJECT       stuVehicle;         // vehicle info
	DH_EVENT_FILE_INFO  stuFileInfo;        // event file info
	int                 nSequence;          // snap index: such as 3,2,1,1 means the last one,0 means there has some exception and snap stop
	int                 nSpeed;             // speed,km/h
	BYTE                bEventAction;		// Event action,0 means pulse event,1 means continuous event's begin,2means continuous event's end;
    BYTE                byReserved[2];
    BYTE                byImageIndex;                   // Serial number of the picture, in the same time (accurate to seconds) may have multiple images, starting from 0
    DWORD               dwSnapFlagMask;	    			// flag(by bit),see NET_RESERVED_COMMON
	DH_RESOLUTION_INFO  stuResolution;      			// picture resolution
    NET_GPS_INFO        stuGPSInfo;                     // GPS info ,use in mobile DVR/NVR
	BYTE				bReserved[968];				    // Reserved 
	DEV_EVENT_TRAFFIC_TRAFFICCAR_INFO stTrafficCar;     // Traffic vehicle info
	EVENT_COMM_INFO   	stCommInfo;                 	// public info 
} DEV_EVENT_TRAFFIC_TURNLEFT_INFO;


//the describe of EVENT_IVS_TRAFFIC_TURNRIGHT's data
typedef struct tagDEV_EVENT_TRAFFIC_TURNRIGHT_INFO
{
	int					nChannelID;			// channel ID
	char				szName[128];		// event name
	char                bReserved1[4];      // byte alignment
	double				PTS;				// PTS(ms)
	NET_TIME_EX			UTC;				// the event happen time
	int					nEventID;			// event ID
	int					nLane;				// Corresponding Lane number
	DH_MSG_OBJECT		stuObject;			// have being detected object
	DH_MSG_OBJECT       stuVehicle;         // vehicle info
	DH_EVENT_FILE_INFO  stuFileInfo;        // event file info
	int                 nSequence;          // snap index: such as 3,2,1,1 means the last one,0 means there has some exception and snap stop
	int                 nSpeed;             // speed,km/h
	BYTE                bEventAction;		// Event action,0 means pulse event,1 means continuous event's begin,2means continuous event's end;
    BYTE                byReserved[2];
    BYTE                byImageIndex;                   // Serial number of the picture, in the same time (accurate to seconds) may have multiple images, starting from 0
    DWORD               dwSnapFlagMask;	    			// flag(by bit),see NET_RESERVED_COMMON
	DH_RESOLUTION_INFO  stuResolution;      			// picture resolution
    NET_GPS_INFO        stuGPSInfo;                     // GPS info ,use in mobile DVR/NVR
	BYTE				bReserved[968];				    // Reserved 
	DEV_EVENT_TRAFFIC_TRAFFICCAR_INFO stTrafficCar;     // Traffic vehicle info
	EVENT_COMM_INFO   	stCommInfo;                 	// public info 
} DEV_EVENT_TRAFFIC_TURNRIGHT_INFO;

//the describe of EVENT_IVS_TRAFFIC_UTURN's data
typedef struct tagDEV_EVENT_TRAFFIC_UTURN_INFO 
{
	int					nChannelID;						  // channel ID
	char				szName[128];					  // event name
	char                bReserved1[4];                    // byte alignment
	double				PTS;							  // PTS(ms)
	NET_TIME_EX			UTC;						   	  // the event happen time
	int					nEventID;					      // event ID
	int					nLane;							  // Corresponding Lane number
	DH_MSG_OBJECT		stuObject;						  // have being detected object
	DH_MSG_OBJECT       stuVehicle;                       // vehicle info
	DH_EVENT_FILE_INFO  stuFileInfo;					  // event file info
	int                 nSequence;                        // snap index: such as 3,2,1,1 means the last one,0 means there has some exception and snap stop
	int                 nSpeed;             			  // speed,km/h
	BYTE                bEventAction;		              // Event action,0 means pulse event,1 means continuous event's begin,2means continuous event's end;
    BYTE                byReserved[2];
    BYTE                byImageIndex;                     // Serial number of the picture, in the same time (accurate to seconds) may have multiple images, starting from 0
    DWORD               dwSnapFlagMask;	                  // flag(by bit),see NET_RESERVED_COMMON
	DH_RESOLUTION_INFO  stuResolution;                    // picture resolution
    NET_GPS_INFO        stuGPSInfo;                       // GPS info ,use in mobile DVR/NVR
	BYTE				bReserved[968];				      // Reserved 
	DEV_EVENT_TRAFFIC_TRAFFICCAR_INFO stTrafficCar;       // Traffic vehicle info
	EVENT_COMM_INFO   	stCommInfo;                 	  // public info 
}DEV_EVENT_TRAFFIC_UTURN_INFO;

//the describe of EVENT_IVS_TRAFFIC_OVERSPEED's data
typedef struct tagDEV_EVENT_TRAFFIC_OVERSPEED_INFO 
{
	int					nChannelID;						  // channel ID
	char				szName[128];					  // event name
	char                bReserved1[4];                    // byte alignment
	double				PTS;							  // PTS(ms)
	NET_TIME_EX			UTC;						   	  // the event happen time
	int					nEventID;					      // event ID
	int					nLane;							  // Corresponding Lane number
	DH_MSG_OBJECT		stuObject;						  // have being detected object
	DH_MSG_OBJECT       stuVehicle;                       // vehicle info
	DH_EVENT_FILE_INFO  stuFileInfo;					  // event file info
    int                 nSpeed;                           // vehicle speed Unit:Km/h
	int					nSpeedUpperLimit;			      // Speed Up limit Unit:km/h
	int					nSpeedLowerLimit;				  // Speed Low limit Unit:km/h 
	int                 nSequence;                        // snap index:such as 3,2,1,1 means the last one,0 means there has some exception and snap stop
	BYTE                bEventAction;		              // Event action,0 means pulse event,1 means continuous event's begin,2 means continuous event's end;
    BYTE                byReserved[2];
    BYTE                byImageIndex;                     // Serial number of the picture, in the same time (accurate to seconds) may have multiple images, starting from 0
    DWORD               dwSnapFlagMask;	                  // flag(by bit),see NET_RESERVED_COMMON
	DH_RESOLUTION_INFO  stuResolution;                    // picture resolution
    char                szFilePath[MAX_PATH];             // Faile path
    EVENT_INTELLI_COMM_INFO     stuIntelliCommInfo;       // intelli comm info
    NET_GPS_INFO        stuGPSInfo;                       // GPS info ,use in mobile DVR/NVR
	BYTE                bReserved[576];				      // Reserved 
	DEV_EVENT_TRAFFIC_TRAFFICCAR_INFO stTrafficCar;       // Traffic vehicle info
	EVENT_COMM_INFO   	stCommInfo;                 	  // public info 
}DEV_EVENT_TRAFFIC_OVERSPEED_INFO;

//the describe of EVENT_IVS_TRAFFIC_UNDERSPEED's data
typedef struct tagDEV_EVENT_TRAFFIC_UNDERSPEED_INFO 
{
	int					nChannelID;						  // channel ID
	char				szName[128];					  // event name
	char                bReserved2[4];                    // byte alignment
	double				PTS;							  // PTS(ms)
	NET_TIME_EX			UTC;						   	  // the event happen time
	int					nEventID;					      // event ID
	int					nLane;							  // Corresponding Lane number
	DH_MSG_OBJECT		stuObject;						  // have being detected object
	DH_MSG_OBJECT       stuVehicle;                       // vehicle info
	DH_EVENT_FILE_INFO  stuFileInfo;					  // event file info
    int                 nSpeed;                           // vehicle speed Unit:Km/h
	int					nSpeedUpperLimit;			      // Speed Up limit Unit:km/h
	int					nSpeedLowerLimit;				  // Speed Low limit Unit:km/h 
	int                 nSequence;                        // snap index: such as 3,2,1,1 means the last one,0 means there has some exception and snap stop
	BYTE                bEventAction;		              // Event action,0 means pulse event,1 means continuous event's begin,2means continuous event's end;
    BYTE                bReserved1[2];                    // reserved
    BYTE                byImageIndex;                     // Serial number of the picture, in the same time (accurate to seconds) may have multiple images, starting from 0
	int                 nUnderSpeedingPercentage;         // under speed percentage
    DWORD               dwSnapFlagMask;	                  // flag(by bit),see NET_RESERVED_COMMON
	DH_RESOLUTION_INFO  stuResolution;                    // picture resolution
	EVENT_INTELLI_COMM_INFO     stuIntelliCommInfo;       // intelligence common information
    NET_GPS_INFO        stuGPSInfo;                       // GPS info ,use in mobile DVR/NVR
	BYTE                bReserved[832];				  	  // Reserved 
	DEV_EVENT_TRAFFIC_TRAFFICCAR_INFO stTrafficCar;       // Traffic vehicle info
    EVENT_COMM_INFO   stCommInfo;                 		  // public info 
}DEV_EVENT_TRAFFIC_UNDERSPEED_INFO;

//the describe of EVENT_IVS_TRAFFIC_JAM_FORBID_INTO's data
typedef struct tagDEV_EVENT_ALARM_JAMFORBIDINTO_INFO
{
	int                 nChannelID;                         // channel ID
    char                szName[DH_EVENT_NAME_LEN];          // event name
    char                bReserved1[4];                      // byte alignment
    DWORD               PTS;                                // PTS(ms)
    NET_TIME_EX         UTC;                                // the event happen time
    int                 nEventID;                           // event ID
	DH_EVENT_FILE_INFO  stuFileInfo;                        // event file info
	int					nMark;								// frame flag
	int					nSource;						  	// video source
	int					nSequence;						    // snap index: such as 3,2,1,1 means the last one,0 means there has some exception and snap stop
	int					nFrameSequence;						// the sequence of frame
	int					nLane;							    // Corresponding Lane number
	BYTE                byImageIndex;                   	// Serial number of the picture, in the same time (accurate to seconds) may have multiple images, starting from 0
	DH_MSG_OBJECT       stuObject;                      	// have being detected object
    NET_GPS_INFO        stuGPSInfo;                         // GPS info ,use in mobile DVR/NVR
	BYTE                bReserved[984];	                    // Reserved 
	DEV_EVENT_TRAFFIC_TRAFFICCAR_INFO stuTrafficCar;        // Traffic vehicle info
	EVENT_COMM_INFO     stCommInfo;                     	// public info 
    DH_MSG_OBJECT       stuVehicle;                         // Traffic vehicle info
    DH_RESOLUTION_INFO  stuResolution;                      // picture resolution
} DEV_EVENT_ALARM_JAMFORBIDINTO_INFO;

//analyse info
typedef struct tagEVENT_PIC_ANALYSE_INFO
{
	DH_MSG_OBJECT       stuObject;                                  // plate info
    DH_MSG_OBJECT       stuVehicle;                                 // vehicle info
	EVENT_COMM_INFO     stuCommInfo;                     			// public info 
    BYTE                bReserved[1024];                            // Reserved
}EVENT_PIC_ANAKYSE_INFO;

//the describe of DH_ALARM_TRAFFIC_PIC_ANALYSE's data
typedef struct tagDEV_ALARM_PIC_ANALYSE_INFO
{
    int						nIndex;                              
    char					szName[128];                                // event nament
    double					PTS;                                        // PTS(ms)
    NET_TIME_EX				UTC;                                        // the event happen time
    int						nEventID;                                   // event ID
    DH_MSG_OBJECT			stuObject;                                  // plate info
    DH_MSG_OBJECT			stuVehicle;                                 // vehicle info
    int						nGroupID;									// the only id of one group 
	int						nCountInGroup;								// the file count in the current file's group
	int						nIndexInGroup;								// the index of the file in the group
	char					szFilePath[MAX_PATH];						// the path of the file
	EVENT_COMM_INFO			stuCommInfo;                     			// public info 
	EVENT_PIC_ANAKYSE_INFO	stuAnalyseInfo;								// analyse info
	BYTE					bReserved[1024];							// Reserved 
} ALARM_PIC_ANALYSE_INFO;
typedef struct tagNET_BULLET_HOLES_INFO
{
    int                 nObjectID;                          // Object ID,each ID represent a unique object
    char                szObjectType[128];                  // Object type
    DH_RECT             stuBoundingBox;                     // BoundingBox
    DH_POINT            stuCenter;                          // The shape center of the object
	int				    nSequence;							// Sequence
	int                 nScore;							    // Score
    BYTE                byReserved[512];                    // Reserved
}NET_BULLET_HOLES_INFO;
typedef struct tagDEV_ALARM_PIC_SHOOTINGSCORERECOGNITION_INFO
{
    int						nIndex;                                     // channel ID
    char					szName[128];                                // event nament
    double					dbPTS;                                      // PTS(ms)
    NET_TIME_EX				stuUTC;                                     // the event happen time
    int						nEventID;                                   // event ID
    int						nGroupID;									// he only id of one group 
	int						nCountInGroup;								// the file count in the current file's group
	int						nIndexInGroup;								// the index of the file in the group
	char					szFilePath[MAX_PATH];						// the path of the file
	DH_POINT				stuDetectRegion;							// detect region
	int						nValidBulletHolesNum;						// bullet holes number
	NET_BULLET_HOLES_INFO   stuBulletHolesInfo[DH_MAX_BULLET_HOLES];	// detect bullet holes info 
	BYTE					bReserved[1024];							// Reserved
} ALARM_PIC_SHOOTINGSCORERECOGNITION_INFO;

//the describe of DH_ALARM_OIL_4G_OVERFLOW's data
typedef struct tagNET_ALARM_OIL_4G_OVERFLOW_INFO
{	
	int					            nChannelID;						// channel ID	
    double				            dbPTS;							// PTS(ms) 
    NET_TIME_EX			            stuTime;						// the event happen time
	int								nFlowRateCur;					// flow already used of the month(unit:MB)	
	int								nFlowRateMax;					// flow threshold of the month(unit:MB)	
	BYTE                			byReserved[1024];   			// Reserved
}NET_ALARM_OIL_4G_OVERFLOW_INFO;

//the describe of DH_ALARM_POWER_SWITCHER_ALARM's data
typedef struct tagDEV_ALARM_POWERSWITCHER_INFO
{
	int                     nEventAction;                       // Continuous event action: 1 means start,2 means stop;
	int                     nEventIndex;                        // Power supply switch serial number, in accord with the subscript serial number of PowerSwitcher configuration
	char					szName[128];                        // Power switcher name, corresponding with PowerSwitcher configuration name
	NET_TIME_EX				stuUTC;                             // The event happen time
	int                     nPowerVoltage;                      // Power voltage, uint:mV
	int                     nPowerCurrent;                      // Power current, uint:mA
	int                     nPowerValue;                        // Power value, uint:mW
	int						nSwitchOn;							// Switch on or off: 0 means switch off,1 means switch on 
	int                     nPowerState;                        // Power state of mask, 0 means normal,bit0 means under voltage, bit1 means over voltage, bit2 means under current, bit3 means over current, bit4 means invalid(this state is not set with other state at the same time)
	BYTE					byReserved[1024];				    // Reserved
}DEV_ALRAM_POWERSWITCHER_INFO;

//the describe of DH_ALARM_SCENNE_CHANGE_ALARM's data
typedef struct tagDEV_ALARM_PIC_SCENECHANGE_INFO
{
    int						nChannelID;                                 // channel ID
	int						nEventAction;								// Continuous event action: 1 means start,2 means stop;
    double					dbPTS;                                      // PTS(ms)
    NET_TIME_EX				stuUTC;                                     // The event happen time
    int						nEventID;                                   // event ID
	BYTE					bReserved[1024];							// Reserved
} ALARM_PIC_SCENECHANGE_INFO;

//the describe of EVENT_IVS_SCENE_CHANGE's data
typedef struct tagDEV_ALRAM_SCENECHANGE_INFO
{
    int							nChannelID;									// channel ID
	int							nEventAction;								// Continuous event action: 1 means start,2 means stop;
    double						dbPTS;										// PTS(ms)
    NET_TIME_EX					stuUTC;										// The event happen time
    int							nEventID;									// event ID

    DH_EVENT_FILE_INFO			stuFileInfo;								// event file info
    BYTE						byImageIndex;								// Serial number of the picture, in the same time (accurate to seconds) may have multiple images, starting from 0
    DWORD						dwSnapFlagMask;								// flag(by bit),see NET_RESERVED_COMMON 
    BYTE						bReserved[1024];							// Reserved 
} DEV_ALRAM_SCENECHANGE_INFO;

//Corresponding to data block description of event type EVENT_IVS_SNAPMANUAL(SnapManual time event) 
typedef struct tagDEV_EVENT_SNAPMANUAL
{
	int                 nChannelID;                                   // Channel number
    char                szName[DH_EVENT_NAME_LEN];                    // Event name
    char                bReserved1[4];                                // Byte alignment
    double              PTS;                                          // Timestamp (in milliseconds)
    NET_TIME_EX         UTC;                                          // Time for the event occurred
    int                 nEventID;                                     // Event ID
    DH_EVENT_FILE_INFO	stuFileInfo;								  // event file info
    BYTE				byImageIndex;							      // Serial number of the picture, in the same time (accurate to seconds) may have multiple images, starting from 0
    DWORD				dwSnapFlagMask;								  // flag(by bit),see NET_RESERVED_COMMON 
    BYTE				bReserved[1024];							  // Reserved 
}DEV_EVENT_SNAPMANUAL;

// event of swipe overtime
typedef struct tagALARM_SWIPE_OVERTIME_INFO
{                            
    NET_TIME_EX             stuTime;                            // the event happen time
    BYTE					bReserved[1024];
}ALARM_SWIPE_OVERTIME_INFO;

// event of driving without card
typedef struct tagALARM_DRIVING_WITHOUTCARD_INFO
{
    NET_TIME_EX             stuTime;                            // the event happen time
    BYTE					bReserved[1024];
}ALARM_DRIVING_WITHOUTCARD_INFO;

// multi faces detect info
typedef struct tagNET_FACE_INFO
{
    int                 nObjectID;                          // object id
    char                szObjectType[128];                  // object type
    int                 nRelativeID;                        // same with the source picture id
    DH_RECT             BoundingBox;                        // bounding box
    DH_POINT            Center;                             // object center
} NET_FACE_INFO;

//the describe of EVENT_IVS_FACEDETECT's data
typedef struct tagDEV_EVENT_FACEDETECT_INFO 
{
	int					nChannelID;						  // channel ID
	char				szName[128];					  // event name
	char                bReserved1[4];                    // byte alignment
	double				PTS;							  // PTS(ms)
	NET_TIME_EX			UTC;						   	  // the event happen time
	int					nEventID;					      // event ID
	DH_MSG_OBJECT		stuObject;						  // have being detected object
	DH_EVENT_FILE_INFO  stuFileInfo;					  // event file info
	BYTE                bEventAction;                     // Event action: 0 means pulse event,1 means continuous event's begin,2means continuous event's end;
	BYTE                reserved[2];                      // reserved
    BYTE                byImageIndex;                     // Serial number of the picture, in the same time (accurate to seconds) may have multiple images, starting from 0
	int                 nDetectRegionNum;				  // detect region point number
	DH_POINT            DetectRegion[DH_MAX_DETECT_REGION_NUM];    // detect region
    DWORD               dwSnapFlagMask;	                  // flag(by bit),see NET_RESERVED_COMMON
    char                szSnapDevAddress[MAX_PATH];       // snapshot current face device address
    unsigned int        nOccurrenceCount;                 // event trigger accumilated times 
    EM_DEV_EVENT_FACEDETECT_SEX_TYPE emSex;                         // sex type
    int        			nAge;                                       // age, invalid if it is -1
    unsigned int        nFeatureValidNum;                           // invalid number in array emFeature
    EM_DEV_EVENT_FACEDETECT_FEATURE_TYPE    emFeature[DH_MAX_FACEDETECT_FEATURE_NUM];   // human face features
    int                 nFacesNum;                                  // number of stuFaces
    NET_FACE_INFO       stuFaces[10];                               // when nFacesNum > 0, stuObject invalid
    EVENT_INTELLI_COMM_INFO     stuIntelliCommInfo;                 // public info 
	EM_RACE_TYPE				emRace;								// race
	EM_EYE_STATE_TYPE			emEye;								// eyes state
	EM_MOUTH_STATE_TYPE			emMouth;							// mouth state
	EM_MASK_STATE_TYPE 			emMask;								// mask state
	EM_BEARD_STATE_TYPE			emBeard;							// beard state
	int							nAttractive;						// Attractive value, -1: invalid, 0:no disringuishrange: 1-100, the higher value, the higher charm
    BYTE                		bReserved[868];                   	// Reserved 
} DEV_EVENT_FACEDETECT_INFO;

// the describe of EVENT_IVS_TRAFFICJAM's data
typedef struct tagDEV_EVENT_TRAFFICJAM_INFO 
{
	int					nChannelID;						// channel ID
	char				szName[128];					// event name
	char                bReserved1[4];                  // byte alignment
	double				PTS;							// PTS(ms)
	NET_TIME_EX			UTC;							// the event happen time
	int					nEventID;						// event ID
	int					nLane;							// Corresponding Lane number
	DH_EVENT_FILE_INFO  stuFileInfo;                    // event file info              
	BYTE                bEventAction;                   // Event action,0 means pulse event,1 means continuous event's begin,2means continuous event's end;
	BYTE                bJamLenght;                     // Mean congestion length (percentage of total lane length) 0-100
	BYTE                reserved;                    	// reserved
    BYTE                byImageIndex;                   // Serial number of the picture, in the same time (accurate to seconds) may have multiple images, starting from 0
	NET_TIME_EX         stuStartJamTime;				// the time of starting jam 
	int                 nSequence;                      // snap index: such as 3,2,1,1 means the last one,0 means there has some exception and snap stop(this param work when bEventAction=2) 
	int                 nAlarmIntervalTime;             // interval time of alarm(s).(this is a continuous event,if the interval time of recieving next event go beyond this param, we can judge that this event is over with exception)
    DWORD               dwSnapFlagMask;	                // flag(by bit),see NET_RESERVED_COMMON
	DH_RESOLUTION_INFO  stuResolution;                  // picture resolution
    int                 nJamRealLength;                 // means actual jam length, unit is meter
	BYTE				bReserved[1008];				// Reserved 
	DEV_EVENT_TRAFFIC_TRAFFICCAR_INFO stTrafficCar;     // Traffic vehicle info	
    EVENT_COMM_INFO     stCommInfo;                     // public info 
} DEV_EVENT_TRAFFICJAM_INFO;

// the describe of EVENT_IVS_TRAFFIC_PARKING's data
typedef struct tagDEV_EVENT_TRAFFIC_PARKING_INFO 
{
	int					nChannelID;						// channel ID
	char				szName[128];					// event name
	char                bReserved1[4];                  // byte alignment
	double				PTS;							// PTS(ms)
	NET_TIME_EX			UTC;							// the event happen time
	int					nEventID;						// event ID
	DH_MSG_OBJECT		stuObject;						// have being detected object
	DH_MSG_OBJECT       stuVehicle;                     // vehicle info
	int					nLane;							// Corresponding Lane number
	DH_EVENT_FILE_INFO  stuFileInfo;                    // event file info               
	BYTE                bEventAction;                   // Event action,0 means pulse event,1 means continuous event's begin,2means continuous event's end;
	BYTE                reserved[2];                    // Reserved bytes
    BYTE                byImageIndex;                   // Serial number of the picture, in the same time (accurate to seconds) may have multiple images, starting from 0
	NET_TIME_EX         stuStartParkingTime;            // the time of starting parking
	int                 nSequence;                      // snap index: such as 3,2,1,1 means the last one,0 means there has some exception and snap stop(this param work when bEventAction=2) 
	int                 nAlarmIntervalTime;             // interval time of alarm(s) (this is a continuous event,if the interval time of recieving next event go beyond this param, we can judge that this event is over with exception)
	int                 nParkingAllowedTime;            // the time of legal parking
	int                 nDetectRegionNum;				// detect region point number
	DH_POINT            DetectRegion[DH_MAX_DETECT_REGION_NUM];    // detect region point number
    DWORD               dwSnapFlagMask;	                // flag(by bit),see NET_RESERVED_COMMON
	DH_RESOLUTION_INFO  stuResolution;                  // picture resolution
	BOOL                bIsExistAlarmRecord;            // true:corresponding alarm recording; false: no corresponding alarm recording
	DWORD               dwAlarmRecordSize;              // Video size
	char                szAlarmRecordPath[DH_COMMON_STRING_256]; // Video Path
    char                szFTPPath[DH_COMMON_STRING_256];// FTP path 
    EVENT_INTELLI_COMM_INFO     stuIntelliCommInfo;     // intelli comm info
	BYTE				byPreAlarm;						// weather is PreAlarm event,0 :traffic parking event,1 :preAlarm event
	BYTE				bReserved2[3];				    // Reserved 
    NET_GPS_INFO        stuGPSInfo;                     // GPS info ,use in mobile DVR/NVR
	BYTE				bReserved[228];				    // Reserved 
	DEV_EVENT_TRAFFIC_TRAFFICCAR_INFO stTrafficCar;     // Traffic vehicle info
    EVENT_COMM_INFO     stCommInfo;                     // public info 
} DEV_EVENT_TRAFFIC_PARKING_INFO;

//the describe of EVENT_IVS_TRAFFIC_WRONGROUTE's data
typedef struct tagDEV_EVENT_TRAFFIC_WRONGROUTE_INFO 
{
	int					nChannelID;						// channel ID
	char				szName[128];					// event name
	char                bReserved1[4];                  // byte alignment
	double				PTS;							// PTS(ms)
	NET_TIME_EX			UTC;							// the event happen time
	int					nEventID;						// event ID
	DH_MSG_OBJECT		stuObject;						// have being detected object
	DH_MSG_OBJECT       stuVehicle;                     // vehicle info
	int					nLane;							// Corresponding Lane number
	DH_EVENT_FILE_INFO  stuFileInfo;                    // event file info               
	BYTE                bEventAction;                   // Event action,0 means pulse event,1 means continuous event's begin,2means continuous event's end;
    BYTE                byReserved[2];           
    BYTE                byImageIndex;                   // Serial number of the picture, in the same time (accurate to seconds) may have multiple images, starting from 0
	int                 nSpeed;                         // speed,km/h
    DWORD               dwSnapFlagMask;	                // flag(by bit),see NET_RESERVED_COMMON
	DH_RESOLUTION_INFO  stuResolution;                  // picture resolution
    NET_GPS_INFO        stuGPSInfo;                     // GPS info ,use in mobile DVR/NVR
	BYTE				bReserved[972];				    // Reserved 
	DEV_EVENT_TRAFFIC_TRAFFICCAR_INFO stTrafficCar;     // Traffic vehicle info
    EVENT_COMM_INFO     stCommInfo;                     // public info 
} DEV_EVENT_TRAFFIC_WRONGROUTE_INFO;

//the describe of EVENT_IVS_TRAFFIC_CROSSLANE's data
typedef struct tagDEV_EVENT_TRAFFIC_CROSSLANE_INFO 
{
	int					nChannelID;						// channel ID
	char				szName[128];					// event name
	char                bReserved1[4];                  // byte alignment
	double				PTS;							// PTS(ms)
	NET_TIME_EX			UTC;							// the event happen time
	int					nEventID;						// event ID
	DH_MSG_OBJECT		stuObject;						// have being detected object
	DH_MSG_OBJECT       stuVehicle;                     // vehicle info
	int					nLane;							// Corresponding Lane number
	DH_EVENT_FILE_INFO  stuFileInfo;                    // event file info                 
	BYTE                bEventAction;                   // Event action,0 means pulse event,1 means continuous event's begin,2means continuous event's end;
    BYTE                byReserved[2];       
    BYTE                byImageIndex;                   // Serial number of the picture, in the same time (accurate to seconds) may have multiple images, starting from 0
	int                 nSpeed;                         // speed,km/h
    DWORD               dwSnapFlagMask;	                // flag(by bit),see NET_RESERVED_COMMON
	DH_RESOLUTION_INFO  stuResolution;                  // picture resolution
	EVENT_INTELLI_COMM_INFO     stuIntelliCommInfo;     // intelli comm info
    NET_GPS_INFO        stuGPSInfo;                     // GPS info ,use in mobile DVR/NVR
	BYTE                bReserved[836];                 // Reserved bytes.
    DEV_EVENT_TRAFFIC_TRAFFICCAR_INFO stuTrafficCar;    // traffic vehicle info
    EVENT_COMM_INFO     stCommInfo;                     // public info 
} DEV_EVENT_TRAFFIC_CROSSLANE_INFO;

//the describe of EVENT_IVS_TRAFFIC_OVERYELLOWLINE's data
typedef struct tagDEV_EVENT_TRAFFIC_OVERYELLOWLINE_INFO 
{
	int					nChannelID;						// channel ID
	char				szName[128];					// event name
	char                bReserved1[4];                  // byte alignment
	double				PTS;							// PTS(ms)
	NET_TIME_EX			UTC;							// the event happen time
	int					nEventID;						// event ID
	DH_MSG_OBJECT		stuObject;						// have being detected object
	DH_MSG_OBJECT       stuVehicle;                     // vehicle info
	int					nLane;							// Corresponding Lane number
	DH_EVENT_FILE_INFO  stuFileInfo;                    // event file info                 
	BYTE                bEventAction;                   // Event action,0 means pulse event,1 means continuous event's begin,2means continuous event's end;
    BYTE                byReserved[2];  
    BYTE                byImageIndex;                   // Serial number of the picture, in the same time (accurate to seconds) may have multiple images, starting from 0
	int                 nSpeed;                         // speed,km/h
    DWORD               dwSnapFlagMask;	                // flag(by bit),see NET_RESERVED_COMMON
	DH_RESOLUTION_INFO  stuResolution;                  // picture resolution
	BOOL                bIsExistAlarmRecord;            // true:corresponding alarm recording; false: no corresponding alarm recording
	DWORD               dwAlarmRecordSize;              // Video size
	char                szAlarmRecordPath[DH_COMMON_STRING_256]; // Video Path
	EVENT_INTELLI_COMM_INFO     stuIntelliCommInfo;     // intelli comm info
	BYTE				bReserved[532];					// Reserved 
	DEV_EVENT_TRAFFIC_TRAFFICCAR_INFO stTrafficCar;     // Traffic vehicle info
	
	int                 nDetectNum;				                   // Acme amount of the rule detect zone 
	DH_POINT            DetectRegion[DH_MAX_DETECT_REGION_NUM];    // Rule detect zone 	
    EVENT_COMM_INFO     stCommInfo;                                // public info 
} DEV_EVENT_TRAFFIC_OVERYELLOWLINE_INFO;

//the describe of EVENT_IVS_TRAFFIC_DRIVINGONSHOULDER's data
typedef struct tagDEV_EVENT_TRAFFIC_DRIVINGONSHOULDER_INFO
{
	int					nChannelID;						// channel ID
	char				szName[128];					// event name
	char                bReserved1[4];                  // byte alignment
	double				PTS;							// PTS(ms)
	NET_TIME_EX			UTC;							// the event happen time
	int					nEventID;						// event ID
	DH_MSG_OBJECT		stuObject;						// have being detected object
	DH_MSG_OBJECT       stuVehicle;                     // vehicle info
	int					nLane;							// Corresponding Lane number
	DH_EVENT_FILE_INFO  stuFileInfo;                    // event file info                                
	BYTE                bEventAction;                   // Event action,0 means pulse event,1 means continuous event's begin,2means continuous event's end;
    BYTE                byReserved[2];      
    BYTE                byImageIndex;                   // Serial number of the picture, in the same time (accurate to seconds) may have multiple images, starting from 0
	int                 nSpeed;                         // speed,km/h
    DWORD               dwSnapFlagMask;	                // flag(by bit),see NET_RESERVED_COMMON
	DH_RESOLUTION_INFO  stuResolution;                  // picture resolution
    DEV_EVENT_TRAFFIC_TRAFFICCAR_INFO stTrafficCar;     // traffic car information
    EVENT_COMM_INFO     stCommInfo;                     // public info 
	
} DEV_EVENT_TRAFFIC_DRIVINGONSHOULDER_INFO;

//the describe of EVENT_IVS_TRAFFIC_YELLOWPLATEINLANE's data
typedef struct tagDEV_EVENT_TRAFFIC_YELLOWPLATEINLANE_INFO
{
	int					nChannelID;						// channel ID
	char				szName[128];					// event name
	char                bReserved1[4];                  // byte alignment
	double				PTS;							// PTS(ms)
	NET_TIME_EX			UTC;							// the event happen time
	int					nEventID;						// event ID
	DH_MSG_OBJECT		stuObject;						// have being detected object
	DH_MSG_OBJECT       stuVehicle;                     // vehicle info
	int					nLane;							// Corresponding Lane number
	DH_EVENT_FILE_INFO  stuFileInfo;                    // event file info                                               
	BYTE                bEventAction;                   // Event action,0 means pulse event,1 means continuous event's begin,2means continuous event's end;
    BYTE                byReserved[2];     
    BYTE                byImageIndex;                   // Serial number of the picture, in the same time (accurate to seconds) may have multiple images, starting from 0
	int                 nSpeed;                         // speed,km/h
    DWORD               dwSnapFlagMask;	                // flag(by bit),see NET_RESERVED_COMMON
	DH_RESOLUTION_INFO  stuResolution;                  // picture resolution
	BYTE				bReserved[1016];				// Reserved 
	DEV_EVENT_TRAFFIC_TRAFFICCAR_INFO stTrafficCar;     // Traffic vehicle info
    EVENT_COMM_INFO     stCommInfo;                     // public info 
	
} DEV_EVENT_TRAFFIC_YELLOWPLATEINLANE_INFO;


//the describe of EVENT_IVS_TRAFFIC_NOPASSING's data
typedef struct tagDEV_EVENT_TRAFFIC_NOPASSING_INFO
{
	int					nChannelID;						// channel ID
	char				szName[DH_EVENT_NAME_LEN];		// event name
    int                 nTriggerType;                   // Trigger Type, 0 vehicle inspection device, 1 radar, 2 video
	DWORD				PTS;							// PTS(ms)
	NET_TIME_EX			UTC;							// the event happen time
	int					nEventID;						// event ID
	int                 UTCMS;                          // 
	int                 nMark;                          // 
	int                 nSequence;                      // snap index: such as 3,2,1,1 means the last one,0 means there has some exception and snap stop
	DH_EVENT_FILE_INFO  stuFileInfo;                    // event file info  
	BYTE                bEventAction;                   // Event action,0 means pulse event,1 means continuous event's begin,2means continuous event's end;
	DEV_EVENT_TRAFFIC_TRAFFICCAR_INFO   stTrafficCar;   // TrafficCar info
    DWORD               dwSnapFlagMask;	                // flag(by bit),see NET_RESERVED_COMMON
	DH_RESOLUTION_INFO  stuResolution;                  // picture resolution
    BYTE                byImageIndex;                   // Serial number of the picture, in the same time (accurate to seconds) may have multiple images, starting from 0
    BYTE                byReserved1[3];
    int                 nLane;                          // Corresponding lane number
    DH_MSG_OBJECT		stuObject;						// Object to be detected
    DH_MSG_OBJECT       stuVehicle;                     // car body information 
    int                 nFrameSequence;                 // Video analysis frame number
    int                 nSource;                        // Data source address of the video analysis
    NET_GPS_INFO        stuGPSInfo;                     // GPS info ,use in mobile DVR/NVR
    BYTE				byReserved[984];                // Reserved bytes
    EVENT_COMM_INFO     stCommInfo;                     // public info 
}DEV_EVENT_TRAFFIC_NOPASSING_INFO;
typedef struct tagDH_TRAFFICFLOWSTAT
{
	char				szMachineAddress[256];			// same as DEV_EVENT_TRAFFIC_TRAFFICCAR_INFO.MachineGroup
	char				szMachineName[256];				// same as DEV_EVENT_TRAFFIC_TRAFFICCAR_INFO.MachineName
	char				szDrivingDirection[3][32];		// DrivingDirection "Approach" means driving direction,where the car is more near;"Leave"-means where if mor far to the car,the scend and third param means the location of the driving direction
	int					nLane;							// lane number
	NET_TIME_EX			UTC;							// Statistic time
	int					nPeriod;						// Statistic period, m
	int					nVehicles;						// passed vehicle number
	float				fAverageSpeed;					// average speed, km/h
	float				fAverageLength;					// average length, m
	float				fTimeOccupyRatio;				// time occupy ratio,
	float				fSpaceOccupyRatio;				// space occupy ratio,
	float				fSpaceHeadway;					// space between two cars,m
	float				fTimeHeadway;					// time between two cars, s
	float				fDensity;						// car density, every km
	int					nOverSpeedVehicles;				// over speed vehicle number
	int					nUnderSpeedVehicles;			// under speed vehicle number
	int					nLargeVehicles;					// big car number  
	int					nMediumVehicles;				// mid car number
	int					nSmallVehicles;					// small car number
	int					nMotoVehicles;					// moto car number
	int					nLongVehicles;					// long vehicle number
}DH_TRAFFICFLOWSTAT;
// the describe of EVENT_IVS_TRAFFIC_FLOWSTATE's data
typedef struct tagDEV_EVENT_TRAFFIC_FLOWSTAT_INFO
{
	char				szName[128];					// name
	double				PTS;							// time stamp(ms)
	NET_TIME_EX			UTC;							// occurrence time
	int					nEventID;						// event id
	int					nLaneCnt;						// channel number
	DH_TRAFFICFLOWSTAT	stTrafficFlowStats[DH_MAX_LANE_NUM];//traffic flow state info
	char				Reserved[4];					// byte alignment
}DEV_EVENT_TRAFFIC_FLOWSTAT_INFO;

//the describe of EVENT_IVS_TRAFFIC_MANUALSNAP's data
typedef struct tagDEV_EVENT_TRAFFIC_MANUALSNAP_INFO
{
	int					nChannelID;						// channel ID
	char				szName[128];					// event name
	char                bReserved1[4];                  // byte alignment
	double				PTS;							// PTS(ms)
	NET_TIME_EX			UTC;							// the event happen time
	int					nEventID;						// event ID
	int					nLane;							// lane number
	BYTE                szManualSnapNo[64];             // manual snap number 
	DH_MSG_OBJECT		stuObject;						// have being detected object
	DH_MSG_OBJECT       stuVehicle;                     // have being detected vehicle
	DEV_EVENT_TRAFFIC_TRAFFICCAR_INFO   stTrafficCar;   // TrafficCar info
	DH_EVENT_FILE_INFO  stuFileInfo;                    // event file info
	BYTE                bEventAction;                   // Event action,0 means pulse event,1 means continuous event's begin,2means continuous event's end;
    BYTE                byReserved[2];
    BYTE                byImageIndex;                   // Serial number of the picture, in the same time (accurate to seconds) may have multiple images, starting from 0
    DWORD               dwSnapFlagMask;	                // flag(by bit),see NET_RESERVED_COMMON
	DH_RESOLUTION_INFO  stuResolution;                  // picture resolution
	BYTE				bReserved[1016];				// 
    EVENT_COMM_INFO     stCommInfo;                     // public info 
}DEV_EVENT_TRAFFIC_MANUALSNAP_INFO;

//the describe of EVENT_IVS_TRAFFIC_STAY's data
typedef struct tagDEV_EVENT_TRAFFIC_STAY_INFO
{
	int					nChannelID;						// channel id
	char				szName[128];					// event name
	char                bReserved1[4];                  // byte alignment
	double				PTS;							// PTS(ms)
	NET_TIME_EX			UTC;							// the event happen time
	int					nEventID;						// event ID
	DH_MSG_OBJECT		stuObject;						// object info
	DH_MSG_OBJECT       stuVehicle;                     // vehicle info
	int					nLane;							// lane number
	int					nSequence;						// snap index
	DEV_EVENT_TRAFFIC_TRAFFICCAR_INFO   stTrafficCar;   // TrafficCar info
	DH_EVENT_FILE_INFO  stuFileInfo;                    // event file info  
	BYTE                bEventAction; 					// Event action,0 means pulse event,1 means continuous event's begin,2means continuous event's end;
    BYTE                byReserved0[2];
    BYTE                byImageIndex;                   // Serial number of the picture, in the same time (accurate to seconds) may have multiple images, starting from 0
    DWORD               dwSnapFlagMask;	                // flag(by bit),see NET_RESERVED_COMMON
	DH_RESOLUTION_INFO  stuResolution;                  // picture resolution
	BYTE                byReserved[1012];           
    EVENT_COMM_INFO     stCommInfo;                     // public info 
}DEV_EVENT_TRAFFIC_STAY_INFO;

//the describe of EVENT_IVS_TRAFFIC_VEHICLEINROUTE's data
typedef struct tagDEV_EVENT_TRAFFIC_VEHICLEINROUTE_INFO
{
	int					nChannelID;						// channel id
	char				szName[128];					// event name
	char                bReserved1[4];                  // byte alignment
	double				PTS;							// PTS(ms)
	NET_TIME_EX			UTC;							// the event happen time
	int					nEventID;						// event ID
	DH_MSG_OBJECT		stuObject;						// object info
	DH_MSG_OBJECT       stuVehicle;                     // vehicle info
	int					nLane;							// lane number
	int					nSequence;						// snap index
	int					nSpeed;							// speed
	DEV_EVENT_TRAFFIC_TRAFFICCAR_INFO   stTrafficCar;   // TrafficCar info
	DH_EVENT_FILE_INFO  stuFileInfo;                    // event file info            
	BYTE                bEventAction;                   // Event action,0 means pulse event,1 means continuous event's begin,2means continuous event's end;
    BYTE                byReserved0[2];
    BYTE                byImageIndex;                   // Serial number of the picture, in the same time (accurate to seconds) may have multiple images, starting from 0
    DWORD               dwSnapFlagMask;	                // flag(by bit),see NET_RESERVED_COMMON
	DH_RESOLUTION_INFO  stuResolution;                  // picture resolution
	EVENT_INTELLI_COMM_INFO     stuIntelliCommInfo;     // intelli comm info
	BYTE                byReserved[884];           
    EVENT_COMM_INFO     stCommInfo;                     // public info 
}DEV_EVENT_TRAFFIC_VEHICLEINROUTE_INFO;

// the describe of EVENT_ALARM_LOCALALARM and EVENT_ALARM_MOTIONALARM's data
typedef struct tagDEV_EVENT_ALARM_INFO
{
	int					nChannelID;						// channel id
	char				szName[128];					// evnent name
	char				Reserved[4];					// byte alignment
	double				PTS;							// PTS(ms)
	NET_TIME_EX			UTC;							// the event happen time
	int					nEventID;						// evnet ID        
    
    DH_EVENT_FILE_INFO  stuFileInfo;                    // event file info 
	EVENT_COMM_INFO     stCommInfo;                     // public info 
}DEV_EVENT_ALARM_INFO;

// the describe of EVENT_ALARM_VIDEOBLIND's data
typedef struct tagDEV_EVENT_ALARM_VIDEOBLIND
{
    int					nChannelID;						// channel id
    char				szName[128];					// evnent name
    char				Reserved[4];					// byte alignment
    double				PTS;							// PTS(ms)
    NET_TIME_EX			UTC;							// the event happen time
    int					nEventID;						// evnet ID        
    
    NET_TIME_EX         stuTime;                        // Action happens time,accurate to seconds

    BYTE                byReserved2[512];			    // reserve text,for extension
}DEV_EVENT_ALARM_VIDEOBLIND;

// Alarm event type EVENT_ALARM_ANALOGALARM(analog alarm channel alarm),
typedef struct tagDEV_EVENT_ALARM_ANALOGALRM_INFO 
{
    int                 nChannelID;                                 // (video)channel no.
    char                szName[DH_COMMON_STRING_128];               // analog alarm channel name
    char                Reserved[4];                                // reserve text align	
    double              PTS;                                        // time stamp(unit is ms)
    NET_TIME_EX         UTC;                                        // event time
    int                 nEventID;                                   // event ID 
    DH_EVENT_FILE_INFO  stuFileInfo;                                // event corresponding file info   
    int                 nIndex;                                     // analog alarm channel no.
    NET_SENSE_METHOD    emSensor;                                   // sensor type
    int                 nStatus;                                    // data status, -1:unknown,0:normal,1:invalid data(over meansure),
                                                                    // 2:over threshold1,3:over threshold2,4:over threshold3,5: over threshold 4,
                                                                    // 6:below threshold 1,7: below threshold 2,8: below threshold 3,9: below threshold 4
    float               fValue;                                     // detector value
    NET_TIME            stuCollectTime;                             // data collection time(UTC)
    DWORD               dwSnapFlagMask;                             // snapshotmark(by bit), see NET_RESERVED_COMMON
    BYTE                bEventAction;                               // event action, 0 means impulse event,1 means continuous event start,2 means continuous event end;
    BYTE                byReserved2[1023];                          // reserve text,for extension
}DEV_EVENT_ALARM_ANALOGALRM_INFO;

//EVENT_ALARM_VEHICLEACC(ACC outage alarm)
typedef struct tagDEV_EVENT_ALARM_VEHICLEACC_INFO
{
	int					nChannelID;						// (Channel number)
	char				szName[DH_COMMON_STRING_128];	// (Event Name)
	char				Reserved[4];					// (Reserved byte alignment)
	double				PTS;							// (Timestamp (in milliseconds)
	NET_TIME_EX			UTC;							// (Time the event occurred)
	int					nEventID;						// (Event ID)   
	NET_GPS_STATUS_INFO stGPSStatusInfo;				// (GPS information)
	int					nACCStatus;						// (ACC status 0: Invalid (compatible with), 1: On, 2: Off)                                                                                 	
	BYTE                bEventAction;                   // (Event action, 0 represents the pulse event, an event starts, said persistent, 2 for persistent event ends;)
	BYTE				bConstantElectricStatus;		// (Often charged state 0: Invalid (compatible with), 1: Connection 2: Disconnect)						
	BYTE                bReserved[1022];                // (Reserved bytes, left extensions.)
}DEV_EVENT_ALARM_VEHICLEACC_INFO;


// alarm event type EVENT_ALARM_VEHICLE_TURNOVER(vehicle side turn) , EVENT_ALARM_VEHICLE_COLLISION(collision)
typedef struct tagDEV_EVENT_VEHICEL_ALARM_INFO
{
	int					nChannelID;						// (Channel number)
	char				szName[128];					// (Event name )
	char				Reserved[4];					// (Reserved bytes, left extensions.)
	double				PTS;							// (Timestamp (in milliseconds))
	NET_TIME_EX			UTC;							// (Time for the event occurred )
	int					nEventID;						// (Event ID)
	NET_GPS_STATUS_INFO stGPSStatusInfo;				// (GPS information)
	DH_EVENT_FILE_INFO  stuFileInfo;                    // (Event corresponding to file information)
	BYTE                bEventAction;                   // (Event action, 0 represents the pulse event, 1 persistent event starts, 2 persistent event ends;)
	BYTE                byReserved[2];					// (With Byte alignment)
	BYTE				byImageIndex;					// (Serial number of the picture, in the same time (accurate to seconds) may have multiple images, starting from 0, )
	DWORD               dwSnapFlagMask;                 // (Grab flag (bit), see specific NET_RESERVED_COMMON)
	DH_RESOLUTION_INFO  stuResolution;                  // (Corresponding to the resolution of the picture, 
	BYTE                bReserved[1024];                // (Reserved bytes, left extensions.)
}DEV_EVENT_VEHICEL_ALARM_INFO;

// the describe of  EVENT_IVS_PRISONERRISEDETECTION's data
typedef struct tagDEV_EVENT_PRISONERRISEDETECTION_INFO
{
	int					nChannelID;						  // channel id
	char				szName[128];					  // evnent name
	char                bReserved1[4];                    // byte alignment
	double				PTS;							  // PTS(ms)
	NET_TIME_EX			UTC;						   	  // he event happen time
	int					nEventID;					      // evnet ID           
	DH_MSG_OBJECT		stuObject;						  // object info
	int                 nDetectRegionNum;				  // region point number
	DH_POINT            DetectRegion[DH_MAX_DETECT_REGION_NUM];    // region
	DH_EVENT_FILE_INFO  stuFileInfo;                      // event file info
	double				dInitialUTC;			  		  // UTC init time
	BYTE                bEventAction;                     // Event action,0 means pulse event,1 means continuous event's begin,2means continuous event's end;
    BYTE                byReserved[2];
    BYTE                byImageIndex;                     // Serial number of the picture, in the same time (accurate to seconds) may have multiple images, starting from 0
    DWORD               dwSnapFlagMask;	                  // flag(by bit),see NET_RESERVED_COMMON
	int                 nSourceIndex;                     // the source device's index,-1 means data in invalid
	char                szSourceDevice[MAX_PATH];         // the source device's sign(exclusive),field said local device does not exist or is empty
    unsigned int        nOccurrenceCount;                 // event trigger accumilated times 
    EVENT_INTELLI_COMM_INFO     stuIntelliCommInfo;       // intelligent things info
    BYTE                bReserved[616];                   // Reserved bytes.
}DEV_EVENT_PRISONERRISEDETECTION_INFO;

//Event type  EVENT_IVS_TRAFFIC_PEDESTRAINPRIORITY(Pedestal has higher priority at the  crosswalk) corresponding data block description info
typedef struct tagDEV_EVENT_TRAFFIC_PEDESTRAINPRIORITY_INFO
{
	int                 nChannelID;                       // Channel No.
	char                szName[128];                      // Event name
	char                bReserved1[4];                    // byte alignment
	double              PTS;                              // Time stamp(ms)
	NET_TIME_EX         UTC;                              // Event occurred time
	int                 nEventID;                         // Event ID
	DH_MSG_OBJECT       stuObject;                        // Detected object
	DH_MSG_OBJECT       stuVehicle;                       // Vehicle body info
	DH_EVENT_FILE_INFO  stuFileInfo;                      // The corresponding file info of the event
	int                 nLane;                            // Corresponding lane No.
	double				dInitialUTC;			  		  // Event initial UTC time 	UTC is the second of the event UTC (1970-1-1 00:00:00)
	BYTE                bEventAction;                     // Event operation.0=pulse event,1=begin of the durative event,2=end of the durative event;
    BYTE                byReserved[2];
    BYTE                byImageIndex;                     // Serial number of the picture, in the same time (accurate to seconds) may have multiple images, starting from 0
	DWORD               dwSnapFlagMask;                   // Snap flag(by bit),please refer to NET_RESERVED_COMMON		
	DEV_EVENT_TRAFFIC_TRAFFICCAR_INFO   stTrafficCar;     // The record of the database of the traffic vehicle 
	DH_RESOLUTION_INFO  stuResolution;                    // picture resolution
    NET_GPS_INFO        stuGPSInfo;                       // GPS info ,use in mobile DVR/NVR
	BYTE                bReserved[984];                   // Reserved field for future extension.
    EVENT_COMM_INFO     stCommInfo;                       // public info 
}DEV_EVENT_TRAFFIC_PEDESTRAINPRIORITY_INFO;

//Event type  EVENT_IVS_TRAFFIC_VEHICLEINBUSROUTE(vehicle in bus route)corresponding data block description info
typedef struct tagDEV_EVENT_TRAFFIC_VEHICLEINBUSROUTE_INFO
{
	int                 nChannelID;                       // channel ID
	char                szName[128];                      // event name
	char                bReserved1[4];                    // byte alignment
	double              PTS;                              // Time stamp(ms)
	NET_TIME_EX         UTC;                              // Event occurred time
	int                 nEventID;                         // Event ID
	DH_MSG_OBJECT       stuObject;                        // Detected object
	DH_MSG_OBJECT       stuVehicle;                       // Vehicle body info
	DH_EVENT_FILE_INFO  stuFileInfo;                      // The corresponding file info of the event
	int                 nLane;                            // Corresponding lane No.
	int					nSequence;						  // snap index: such as 3,2,1,1 means the last one,0 means there has some exception and snap stop
	int					nSpeed;							  // speed km/h
	BYTE                bEventAction;                     // Event operation.0=pulse event,1=begin of the durative event,2=end of the durative event;
    BYTE                byReserved[2];
    BYTE                byImageIndex;                     // Serial number of the picture, in the same time (accurate to seconds) may have multiple images, starting from 0
	DWORD               dwSnapFlagMask;                   // Snap flag(by bit),please refer to NET_RESERVED_COMMON		
	DEV_EVENT_TRAFFIC_TRAFFICCAR_INFO stTrafficCar;       // The record of the database of the traffic vehicle 
	DH_RESOLUTION_INFO  stuResolution;                    // picture resolution
    NET_GPS_INFO        stuGPSInfo;                       // GPS info ,use in mobile DVR/NVR
	BYTE                bReserved[980];                   // reserved
    EVENT_COMM_INFO     stCommInfo;                       // public info 
}DEV_EVENT_TRAFFIC_VEHICLEINBUSROUTE_INFO;

//Event type  EVENT_IVS_TRAFFIC_BACKING(traffic backing)corresponding data block description info
typedef struct tagDEV_EVENT_IVS_TRAFFIC_BACKING_INFO
{
	int                 nChannelID;                       // channel ID
	char                szName[128];                      // event name
	char                bReserved1[4];                    // byte alignment
	double              PTS;                              // Time stamp(ms)
	NET_TIME_EX         UTC;                              // Event occurred time
	int                 nEventID;                         // Event ID
	DH_MSG_OBJECT       stuObject;                        // Detected object
	DH_MSG_OBJECT       stuVehicle;                       // Vehicle body info
	DH_EVENT_FILE_INFO  stuFileInfo;                      // The corresponding file info of the event
	int                 nLane;                            // Corresponding lane No.
	int					nSequence;						  // snap index: such as 3,2,1,1 means the last one,0 means there has some exception and snap stop
	int					nSpeed;							  // speed km/h
	BYTE                bEventAction;                     // Event operation.0=pulse event,1=begin of the durative event,2=end of the durative event;
    BYTE                byReserved[2];
    BYTE                byImageIndex;                     // Serial number of the picture, in the same time (accurate to seconds) may have multiple images, starting from 0
	DWORD               dwSnapFlagMask;                   // Snap flag(by bit),please refer to NET_RESERVED_COMMON		
	DEV_EVENT_TRAFFIC_TRAFFICCAR_INFO stTrafficCar;       // The record of the database of the traffic vehicle 
	DH_RESOLUTION_INFO  stuResolution;                    // picture resolution
	EVENT_INTELLI_COMM_INFO     stuIntelliCommInfo;       // intelli comm info
    NET_GPS_INFO        stuGPSInfo;                       // GPS info ,use in mobile DVR/NVR
	BYTE                bReserved[848];                  // reserved
    EVENT_COMM_INFO     stCommInfo;                       // public info 
}DEV_EVENT_IVS_TRAFFIC_BACKING_INFO;

//Event type EVENT_IVS_AUDIO_ABNORMALDETECTION(audio abnormal detection)corresponding data block description info
typedef struct tagDEV_EVENT_IVS_AUDIO_ABNORMALDETECTION_INFO
{
	int                 nChannelID;                       // channel ID
	char                szName[128];                      // event name
	char                bReserved1[4];                    // byte alignment
	double              PTS;                              // Time stamp(ms)
	NET_TIME_EX         UTC;                              // Event occurred time
	int                 nEventID;                         // Event ID
	DH_EVENT_FILE_INFO  stuFileInfo;                      // The corresponding file info of the event
	int                 nDecibel;                         // decubel
	int                 nFrequency;                       // frequency
	BYTE                bEventAction;                     // Event operation.0=pulse event,1=begin of the durative event,2=end of the durative event;
    BYTE                byReserved[2];
    BYTE                byImageIndex;                     // Serial number of the picture, in the same time (accurate to seconds) may have multiple images, starting from 0
	DWORD               dwSnapFlagMask;                   // Snap flag(by bit),please refer to NET_RESERVED_COMMON	
	DH_RESOLUTION_INFO  stuResolution;                    // picture resolution
	BYTE                bReserved[1024];                  // reserved
}DEV_EVENT_IVS_AUDIO_ABNORMALDETECTION_INFO;

//Event type  EVENT_IVS_TRAFFIC_RUNYELLOWLIGHT(traffic run yellow light)corresponding data block description info
typedef struct tagDEV_EVENT_TRAFFIC_RUNYELLOWLIGHT_INFO
{
	int					nChannelID;						  // channel ID
	char				szName[128];					  // event name
	char                bReserved1[4];                    // byte alignment
	double				PTS;							  // Time stamp(ms)
	NET_TIME_EX			UTC;				              // Event occurred time
	int					nEventID;			              // Event ID
	int					nLane;				              // Corresponding lane No.
	DH_MSG_OBJECT		stuObject;	              	      // have being detected object
	DH_MSG_OBJECT       stuVehicle;                       // Vehicle body info
	DH_EVENT_FILE_INFO  stuFileInfo;                      // The corresponding file info of the event 
	int					nLightState;	              	  // state of traffic light 0:unknown 1:green 2:red 3:yellow
	int					nSpeed;			              	  // speed km/h
	int                 nSequence;                        // snap index: such as 3,2,1,1 means the last one,0 means there has some exception and snap stop
	BYTE                bEventAction;					  // Event operation.0=pulse event,1=begin of the durative event,2=end of the durative event;
    BYTE                byReserved[2];
    BYTE                byImageIndex;                     // Serial number of the picture, in the same time (accurate to seconds) may have multiple images, starting from 0
    DWORD               dwSnapFlagMask;	                  // Snap flag(by bit),please refer to NET_RESERVED_COMMON		
	NET_TIME_EX         stYellowLightUTC;                 // begin time of yellow light
	unsigned int        nYellowLightPeriod;               // yellow light period time (s)
	DH_RESOLUTION_INFO  stuResolution;                    // picture resolution
 	BYTE                byRedLightMargin;                 // time interval(s)
	char                szSourceDevice[MAX_PATH];         // the source device's sign(exclusive),field said local device does not exist or is empty
	DEV_EVENT_TRAFFIC_TRAFFICCAR_INFO stTrafficCar;       // Traffic vehicle info
	BYTE				bReserved[1024];	              // reserved
    EVENT_COMM_INFO     stCommInfo;                       // public info 
} DEV_EVENT_TRAFFIC_RUNYELLOWLIGHT_INFO;

//Event type  EVENT_IVS_LEAVEDETECTION(leave check)corresponding data block description info
typedef struct tagDEV_EVENT_IVS_LEAVE_INFO
{
	int					nChannelID;						// channel ID
	char				szName[128];					// event name
	char                bReserved1[4];                  // byte alignment
	double				PTS;							// Time stamp(ms)
	NET_TIME_EX			UTC;							// Event occurred time
	int					nEventID;						// Event ID
	DH_MSG_OBJECT		stuObject;						// Detected object
	DH_EVENT_FILE_INFO  stuFileInfo;                    // The corresponding file info of the event
	DH_RESOLUTION_INFO  stuResolution;                  // picture resolution
	int                 nDetectRegionNum;				// Rule Detect Area Top Number
	DH_POINT            DetectRegion[DH_MAX_DETECT_REGION_NUM];    //Rule Detect Area
	BYTE                bEventAction;                   // Event operation.0=pulse event,1=begin of the durative event,2=end of the durative event;
    BYTE                byImageIndex;                   // Serial number of the picture, in the same time (accurate to seconds) may have multiple images, starting from 0
	EVENT_INTELLI_COMM_INFO     stuIntelliCommInfo;     // intelligent things info
	BYTE                bReserved[894];                 // Reserved bytes.
} DEV_EVENT_IVS_LEAVE_INFO;

//Event type  EVENT_IVS_CLIMBDETECTION(climb check)corresponding data block description info
typedef struct tagDEV_EVENT_IVS_CLIMB_INFO
{
	int					nChannelID;						// channel ID
	char				szName[128];					// event name
	char                bReserved1[4];                  // byte alignment
	double				PTS;							// Time stamp(ms)
	NET_TIME_EX			UTC;							// Event occurred time
	int					nEventID;						// event ID
	DH_MSG_OBJECT		stuObject;						// Detected object
	DH_EVENT_FILE_INFO  stuFileInfo;                    // The corresponding file info of the event
	DH_RESOLUTION_INFO  stuResolution;                  // picture resolution
	int                 nDetectLineNum;                 // Acme amount of the rule detect zone
	DH_POINT            DetectLine[DH_MAX_DETECT_LINE_NUM];    // Rule detect zone 
	BYTE                bEventAction;                   // Event operation.0=pulse event,1=begin of the durative event,2=end of the durative event;
	BYTE				byImageIndex;					// Serial number of the picture, in the same time (accurate to seconds) may have multiple images, starting from 0
    unsigned int        nOccurrenceCount;               // event trigger accumilated times 
    EVENT_INTELLI_COMM_INFO     stuIntelliCommInfo;     // intelligent things info
    BYTE                bReserved[890];                 // Reserved bytes.
}DEV_EVENT_IVS_CLIMB_INFO;

//EVENT_IVS_MULTISCENESWITCH Event Type EVENT_IVS_MULTISCENESWITCH (multi-scene change event) corresponding to the description of the data block
typedef struct tagDEV_EVENT_IVS_MULTI_SCENE_SWICH_INFO
{
	int					nChannelID;						// (Channel number)
	char				szName[128];					// (Event name)
	char                bReserved1[4];                  // 
	double				PTS;							// (Timestamp (in milliseconds))
	NET_TIME_EX			UTC;							// (Time for the event occurred )
	int					nEventID;						// (Event ID)
	BYTE                bEventAction;                   // (Event action, 0 represents the pulse event, 1 persistent event starts, 2 persistent event ends;)
	BYTE				bReserved[1027];	            // (Reserved bytes)
} DEV_EVENT_IVS_MULTI_SCENE_SWICH_INFO;

//Event type  EVENT_IVS_TRAFFIC_PARKINGONYELLOWBOX(parking on yellow)corresponding data block description info
typedef struct tagDEV_EVENT_TRAFFIC_PARKINGONYELLOWBOX_INFO
{
	int					nChannelID;						// channel ID 
	char				szName[128];					// event name
	char                bReserved1[8];                  // byte alignment
	DWORD				PTS;							// Time stamp(ms)
	NET_TIME_EX			UTC;							// Event occurred time
	int					nEventID;						// Event ID
	int					nLane;							// Corresponding lane No.
	DH_MSG_OBJECT		stuObject;						// Detected object
	DH_MSG_OBJECT       stuVehicle;                     // Vehicle body info
	DH_EVENT_FILE_INFO  stuFileInfo;                    // The corresponding file info of the event

	int					nInterval1;						// the first and second time interval(s)
	int					nInterval2;						// 3rd and 2nd delay time, unit is second
	int					nFollowTime;					// follow time,if a car and a car before entering the pornographic websites,is less than this value,just as with car to enter, to enter the case if the parkingis not illegal

	BYTE                bEventAction;                   // Event operation.0=pulse event,1=begin of the durative event,2=end of the durative event;
    BYTE                byReserved[2];
    BYTE                byImageIndex;                   // Serial number of the picture, in the same time (accurate to seconds) may have multiple images, starting from 0
	DWORD               dwSnapFlagMask;	                // Snap flag(by bit),please refer to NET_RESERVED_COMMON		
	DH_RESOLUTION_INFO  stuResolution;                  // picture resolution
	DEV_EVENT_TRAFFIC_TRAFFICCAR_INFO stTrafficCar;     // Traffic vehicle info
    NET_GPS_INFO        stuGPSInfo;                     // GPS info ,use in mobile DVR/NVR
	BYTE				bReserved[984];                 // reserved
    EVENT_COMM_INFO     stCommInfo;                     // public info 
}DEV_EVENT_TRAFFIC_PARKINGONYELLOWBOX_INFO;

//Parking lot info
typedef struct tagDEV_TRAFFIC_PARKING_INFO
{
    int           nFeaturePicAreaPointNum;                  // Feature image point number 
    DH_POINT      stFeaturePicArea[DH_MAX_POLYGON_NUM];     // Feature image info
    BYTE          bReserved[572];                           // Reserved string 
}DEV_TRAFFIC_PARKING_INFO;


//Event type  EVENT_IVS_TRAFFIC_PARKINGSPACEPARKING(parking space parking)corresponding data block description info
typedef struct tagDEV_EVENT_TRAFFIC_PARKINGSPACEPARKING_INFO
{
	int					nChannelID;						// channel ID
    char                szName[DH_EVENT_NAME_LEN];      // event name
	char                bReserved1[8];                  // byte alignment
	DWORD				PTS;							// Time stamp(ms)
	NET_TIME_EX			UTC;							// Event occurred time
	int					nEventID;						// Event ID
	int					nLane;							// Corresponding lane No.
	DH_MSG_OBJECT		stuObject;						// Detected object
	DH_MSG_OBJECT       stuVehicle;                     // Vehicle body info
	DH_EVENT_FILE_INFO  stuFileInfo;                    // The corresponding file info of the event
	
	int                 nSequence;                      // snap index: such as 3,2,1,1 means the last one,0 means there has some exception and snap stop
	BYTE                bEventAction;                   // Event operation.0=pulse event,1=begin of the durative event,2=end of the durative event;
    BYTE                byReserved[2];
    BYTE                byImageIndex;                   // Serial number of the picture, in the same time (accurate to seconds) may have multiple images, starting from 0
	DWORD               dwSnapFlagMask;	                // Snap flag(by bit),please refer to NET_RESERVED_COMMON	
	DH_RESOLUTION_INFO  stuResolution;                  // picture resolution
	DEV_EVENT_TRAFFIC_TRAFFICCAR_INFO stTrafficCar;     // Traffic vehicle info
	int                 nParkingSpaceStatus;            // Comprehensive status of parking spaces, 0-occupied, 1-empty, 2-on line
    DEV_TRAFFIC_PARKING_INFO stTrafficParingInfo;       // Traffic parking information
    BYTE                byPlateTextSource;              // The source of plate text, 0:Local,1:Server
    BYTE                bReserved[379];                 // Reserved
    EVENT_COMM_INFO     stCommInfo;                     // public info 
}DEV_EVENT_TRAFFIC_PARKINGSPACEPARKING_INFO;

//Event type  EVENT_IVS_TRAFFIC_PARKINGSPACENOPARKING(parking space no parking)corresponding data block description info
typedef struct tagDEV_EVENT_TRAFFIC_PARKINGSPACENOPARKING_INFO
{
	int					nChannelID;						// channel ID
    char                szName[DH_EVENT_NAME_LEN];      // event name
	char                bReserved1[8];                  // byte alignment
	DWORD				PTS;							// Time stamp(ms)
	NET_TIME_EX			UTC;							// Event occurred time
	int					nEventID;						// Event ID
	int					nLane;							// Corresponding lane No.
	DH_MSG_OBJECT		stuObject;						// Detected object
	DH_MSG_OBJECT       stuVehicle;                     // Vehicle body info
	DH_EVENT_FILE_INFO  stuFileInfo;                    // The corresponding file info of the event
	
	int                 nSequence;                      // snap index: such as 3,2,1,1 means the last one,0 means there has some exception and snap stop
	BYTE                bEventAction;                   // Event operation.0=pulse event,1=begin of the durative event,2=end of the durative event;
    BYTE                byReserved[2];
    BYTE                byImageIndex;                   // Serial number of the picture, in the same time (accurate to seconds) may have multiple images, starting from 0
	DWORD               dwSnapFlagMask;	                // Snap flag(by bit),please refer to NET_RESERVED_COMMON	
	DH_RESOLUTION_INFO  stuResolution;                  // picture resolution
	DEV_EVENT_TRAFFIC_TRAFFICCAR_INFO stTrafficCar;     // Traffic vehicle info
    DEV_TRAFFIC_PARKING_INFO stTrafficParingInfo;       // traffic parking information
    BYTE                byPlateTextSource;              // the source of plate text, 0:Local,1:Server
    BYTE                bReserved[383];                 // Reserved
    EVENT_COMM_INFO     stCommInfo;                     // public info 
}DEV_EVENT_TRAFFIC_PARKINGSPACENOPARKING_INFO;

// EVENT_IVS_TRAFFIC_PARKINGSPACEOVERLINE Corresponding data block description
typedef struct tagDEV_EVENT_TRAFFIC_PARKINGSPACEOVERLINE_INFO
{
	int					nChannelID;						// Channel number, 
    char                szName[DH_EVENT_NAME_LEN];      // Event name, 
	DWORD				PTS;							// Timestamp (in milliseconds), 
	NET_TIME_EX			UTC;							// The time of events, 
	int					nEventID;						// (Event ID)
	int					nLane;							// (Corresponding lane number)
	DH_MSG_OBJECT		stuObject;						// (object detected)
	DH_MSG_OBJECT       stuVehicle;                     // (Car Body Information)
	DH_EVENT_FILE_INFO  stuFileInfo;                    // Event corresponding to file information, 
	
	int                 nSequence;                      // Means capture serial number, such as 3,2,1,1 capture end, 0 indicates abnormal end, 
	BYTE                byEventAction;                  // Event action, 0 represents the pulse event, 1 means persistent event starts, 2 means persistent event ends;..., 
	BYTE				byImageIndex;					// (Serial chip, the same time (accurate to seconds) may have multiple images, starting from 0)
	BYTE                byReserved1[2];
	DWORD               dwSnapFlagMask;	                // (Grab flag (bit), see specific NET_RESERVED_COMMON)
	DH_RESOLUTION_INFO  stuResolution;                  // (the resolution of relative picture)
	DEV_EVENT_TRAFFIC_TRAFFICCAR_INFO stuTrafficCar;    // (Transportation Vehicle Information)
    DEV_TRAFFIC_PARKING_INFO stTrafficParingInfo;       // traffic parking information
    BYTE                byReserved[384];                // Reserved
    EVENT_COMM_INFO     stCommInfo;                     // public info 
}DEV_EVENT_TRAFFIC_PARKINGSPACEOVERLINE_INFO;

// EVENT_IVS_TRAFFIC_TURNLEFTAFTERSTRAIGHT (turn left not give precedence to straight)corresponding data block description info
typedef struct tagDEV_EVENT_TURNLEFTAFTERSTRAIGHT_INFO
{
	int                 nChannelID;                                 // channel ID
    char                szName[DH_EVENT_NAME_LEN];                  // event name
    char                Reserved[4];                                // byte alignment    
    double              PTS;                                        // Time stamp(ms)
    NET_TIME_EX         UTC;                                        // Event occurred time
    DWORD               nEventID;                                   // Event ID
	DH_EVENT_FILE_INFO  stuFileInfo;                                // Event corresponding to file information
    DH_MSG_OBJECT       stuObject;                                  // plate info
    DH_MSG_OBJECT       stuVehicle;                                 // car body info
	DEV_EVENT_TRAFFIC_TRAFFICCAR_INFO   stTrafficCar;               // Traffic car info
    int                 nLane;                                      // Corresponding lane number
    int                 nSequence;                                  // snap index,such as 3,2,1,1 means the last one,0 means there has some exception and snap stop
    BYTE                byEventAction;                              // Event action, 0 represents the pulse event, 1 means persistent event starts, 2 means persistent event ends
	BYTE				byImageIndex;					            // (Serial chip, the same time (accurate to seconds) may have multiple images, starting from 0)
	BYTE                byReserved1[2];
	DWORD               dwSnapFlagMask;	                            // (Grab flag (bit), see specific NET_RESERVED_COMMON)
	DH_RESOLUTION_INFO  stuResolution;                              // (the resolution of relative picture)
    NET_GPS_INFO        stuGPSInfo;                                 // GPS info ,use in mobile DVR/NVR
	BYTE                byReserved[984];                            // Reserved
	EVENT_COMM_INFO     stCommInfo;                                 // public info 
}DEV_EVENT_TURNLEFTAFTERSTRAIGHT_INFO;

// EVENT_IVS_TRAFFIC_BIGBENDSMALLTURN (small turn on big bend)corresponding data block description info
typedef struct tagDEV_EVENT_BIGBENDSMALLTURN_INFO
{
	int                 nChannelID;                                 // channel ID
    char                szName[DH_EVENT_NAME_LEN];                  // event name
    char                Reserved[4];                                // byte alignment    
    double              PTS;                                        // Time stamp(ms)
    NET_TIME_EX         UTC;                                        // Event occurred time
    DWORD               nEventID;                                   // Event ID
	DH_EVENT_FILE_INFO  stuFileInfo;                                // Event corresponding to file information
    DH_MSG_OBJECT       stuObject;                                  // plate info
    DH_MSG_OBJECT       stuVehicle;                                 // car body info
	DEV_EVENT_TRAFFIC_TRAFFICCAR_INFO   stTrafficCar;               // Traffic car info
    int                 nLane;                                      // Corresponding lane number
    int                 nSequence;                                  // snap index,such as 3,2,1,1 means the last one,0 means there has some exception and snap stop
    BYTE                byEventAction;                              // Event action, 0 represents the pulse event, 1 means persistent event starts, 2 means persistent event ends
	BYTE				byImageIndex;					            // (Serial chip, the same time (accurate to seconds) may have multiple images, starting from 0)
	BYTE                byReserved1[2];
	DWORD               dwSnapFlagMask;	                            // (Grab flag (bit), see specific NET_RESERVED_COMMON)
	DH_RESOLUTION_INFO  stuResolution;                              // (the resolution of relative picture)
	BYTE                byReserved[1024];                           // Reserved
	EVENT_COMM_INFO     stCommInfo;                                 // public info 
}DEV_EVENT_BIGBENDSMALLTURN_INFO;

// EVENT_IVS_ROAD_CONSTRUCTION (Event of road construction inspection) corresponding data block description info
typedef struct tagDEV_EVENT_ROAD_CONSTRUCTION_INFO
{
	int                 nChannelID;                                 // channel ID
    char                szName[DH_EVENT_NAME_LEN];                  // event name
    char                Reserved[4];                                // byte alignment   
    double              PTS;                                        // Time stamp(ms)
    NET_TIME_EX         UTC;                                        // Event occurred time
    DWORD               nEventID;                                   // Event ID
    int                 nEventAction;                               // Event operation.0=pulse event,1=begin of the durative event,2=end of the durative event;
	EM_CLASS_TYPE		emClassType;								// class type
	DH_RECT				stuBoudingBox;								// the bouding box of construction location,0-8192
	BYTE                byReserved[512];                            // Reserved
} DEV_EVENT_ROAD_CONSTRUCTION_INFO;


// EVENT_IVS_ROAD_BLOCK (Event of road block  detection) corresponding data block description info
typedef struct tagDEV_EVENT_ROAD_BLOCK_INFO
{
	int                 nChannelID;                                 // channel ID
    char                szName[DH_EVENT_NAME_LEN];                  // event name
    char                Reserved[4];                                // byte alignment     
    double              PTS;                                        // Time stamp(ms)
    NET_TIME_EX         UTC;                                        // Event occurred time
    DWORD               nEventID;                                   // Event ID
    int                 nEventAction;                               // Event operation.0=pulse event,1=begin of the durative event,2=end of the durative event;
	EM_CLASS_TYPE		emClassType;								// class type
	DH_RECT				stuBoudingBox;								// the bouding box of roadblock location, 0-8192
	BYTE                byReserved[512];                            // Reserved
} DEV_EVENT_ROAD_BLOCK_INFO;

// EVENT_IVS_INSTALL_CARDREADER (install card reader event) corresponding data block description info
typedef struct tagDEV_EVENT_INSTALL_CARDREADER_INFO
{
    int                 nChannelID;									// Channel id
    char                szName[128];								// Event name
    char                Reserved[4];								// Byte alignment
    double              PTS;										// Time stamp(ms)
    NET_TIME_EX         UTC;										// Event occurred time
    int                 nEventID;									// Event ID        
    BYTE                byReserved[1024];							// Reserve,for extension
}DEV_EVENT_INSTALL_CARDREADER_INFO;

// EVENT_IVS_TRAFFIC_QUEUEJUMP (car jump a queue) corresponding data block description info
typedef struct tagDEV_EVENT_TRAFFIC_QUEUEJUMP_INFO
{
	int                 nChannelID;                                 // channel ID
    char                szName[DH_EVENT_NAME_LEN];                  // event name
    char                Reserved[4];                                // byte alignment    
    double              PTS;                                        // Time stamp(ms)
    NET_TIME_EX         UTC;                                        // Event occurred time
    DWORD               nEventID;                                   // Event ID
	DH_EVENT_FILE_INFO  stuFileInfo;                                // Event corresponding to file information
    DH_MSG_OBJECT       stuObject;                                  // plate info
    DH_MSG_OBJECT       stuVehicle;                                 // car body info
	DEV_EVENT_TRAFFIC_TRAFFICCAR_INFO   stTrafficCar;               // Traffic car info
    int                 nLane;                                      // Corresponding lane number
    int                 nSequence;                                  // snap index,such as 3,2,1,1 means the last one,0 means there has some exception and snap stop
    BYTE                byEventAction;                              // Event action, 0 represents the pulse event, 1 means persistent event starts, 2 means persistent event ends
	BYTE				byImageIndex;					            // (Serial chip, the same time (accurate to seconds) may have multiple images, starting from 0)
	BYTE                byReserved1[2];
	DWORD               dwSnapFlagMask;	                            // (Grab flag (bit), see specific NET_RESERVED_COMMON)
	DH_RESOLUTION_INFO  stuResolution;                              // (the resolution of relative picture)
	EVENT_COMM_INFO     stCommInfo;                                 // public info 
	BYTE                byReserved[1024];                           // Reserved
}DEV_EVENT_TRAFFIC_QUEUEJUMP_INFO;

// EVENT_IVS_VEHICLE_SUSPICIOUSCAR (Suspicious car) corresponding data block description info
typedef struct tagDEV_EVENT_VEHICLE_SUSPICIOUSCAR_INFO
{
    int                 nChannelID;                                 // channel ID
    char                szName[DH_EVENT_NAME_LEN];                  // event name
    char                Reserved[4];                                // byte alignment    
    double              PTS;                                        // Time stamp(ms)
    NET_TIME_EX         UTC;                                        // Event occurred time
    DWORD               nEventID;                                   // Event ID
    DH_EVENT_FILE_INFO  stuFileInfo;                                // Event corresponding to file information
    DH_MSG_OBJECT       stuObject;                                  // plate info
    DH_MSG_OBJECT       stuVehicle;                                 // car body info
    int                 nSequence;                                  // snap index,such as 3,2,1,1 means the last one,0 means there has some exception and snap stop
    BYTE                byEventAction;                              // Event action, 0 represents the pulse event, 1 means persistent event starts, 2 means persistent event ends
    BYTE                byImageIndex;                               // (Serial chip, the same time (accurate to seconds) may have multiple images, starting from 0)
    BYTE                byReserved1[2];                             // Reserved
    DWORD               dwSnapFlagMask;                             // (Grab flag (bit), see specific NET_RESERVED_COMMON) 
    DH_RESOLUTION_INFO  stuResolution;                              // (the resolution of relative picture)
    NET_GPS_INFO        stuGPSInfo;                                 // GPS info ,use in mobile DVR/NVR
    BYTE                byReserved[4096];                           // Reserved
}DEV_EVENT_VEHICLE_SUSPICIOUSCAR_INFO;

// Pedestrain info
typedef struct tagEVENT_VEHICLE_INFO
{
	char				szCategory[MAX_CATEGORY_LEN];				// Object type,Passerby mean pedestrain
	BYTE				byReserved[64];								// Reserved			
}EVENT_VEHICLE_INFO;

//Event type  EVENT_IVS_TRAFFIC_PEDESTRAIN(pedestrain)corresponding data block description info
typedef struct tagDEV_EVENT_TRAFFIC_PEDESTRAIN_INFO
{
    int                 nChannelID;                     // channel ID
    char                szName[DH_EVENT_NAME_LEN];      // event name
    char                bReserved1[8];                  // byte alignment
    DWORD               PTS;                            // Time stamp(ms)
    NET_TIME_EX         UTC;                            // Event occurred time
    int                 nEventID;                       // Event ID
    DH_EVENT_FILE_INFO  stuFileInfo;                    // The corresponding file info of the event
    DH_RESOLUTION_INFO  stuResolution;                  // picture resolution
    DWORD               dwSnapFlagMask;                 // Snap flag(by bit)0 bit:"*",1 bit:"Timing",2 bit:"Manual",3 bit:"Marked",4 bit:"Event",5 bit:"Mosaic",6 bit:"Cutout" 
    BYTE                bEventAction;                   // Event operation.0=pulse event,1=begin of the durative event,2=end of the durative event;
    BYTE                bReserved2[2];
	BYTE				byImageIndex;					// Serial number of the picture, in the same time (accurate to seconds) may have multiple images, starting from 0
    int                 nLane;                          // Corresponding lane No.
    DH_MSG_OBJECT       stuObject;                      // Detected object
    EVENT_INTELLI_COMM_INFO     stuIntelliCommInfo;     // intelli comm info
    EVENT_TRAFFIC_CAR_PART_INFO   stuTrafficCarPartInfo;   // part of traffic car info
	EVENT_VEHICLE_INFO	stuVehicle;						// pedestrain info
    BYTE                bReserved[252];                 // reserved
    EVENT_COMM_INFO     stCommInfo;                     // public info 
}DEV_EVENT_TRAFFIC_PEDESTRAIN_INFO;

//Event type  EVENT_IVS_TRAFFIC_THROW(throw)corresponding data block description info
typedef struct tagDEV_EVENT_TRAFFIC_THROW_INFO
{
    int                 nChannelID;                     // channel ID
    char                szName[DH_EVENT_NAME_LEN];      // event name
    char                bReserved1[8];                  // byte alignment
    DWORD               PTS;                            // Time stamp(ms)
    NET_TIME_EX         UTC;                            // Event occurred time
    int                 nEventID;                       // Event ID
    DH_EVENT_FILE_INFO  stuFileInfo;                    // The corresponding file info of the event
    DH_RESOLUTION_INFO  stuResolution;                  // picture resolution
    DWORD               dwSnapFlagMask;                 // Snap flag(by bit)0 bit:"*",1 bit:"Timing",2 bit:"Manual",3 bit:"Marked",4 bit:"Event",5 bit:"Mosaic",6 bit:"Cutout" 
    BYTE                bEventAction;                   // Event operation.0=pulse event,1=begin of the durative event,2=end of the durative event;
    BYTE                bReserved2[2];
	BYTE				byImageIndex;					// Serial number of the picture, in the same time (accurate to seconds) may have multiple images, starting from 0
    int                 nLane;                          // Corresponding lane No.
    DH_MSG_OBJECT       stuObject;                      // Detected object
    EVENT_INTELLI_COMM_INFO     stuIntelliCommInfo;     // intelligent things info
    EVENT_TRAFFIC_CAR_PART_INFO   stuTrafficCarPartInfo;// part of traffic car info
    NET_GPS_INFO        stuGPSInfo;                     // GPS info ,use in mobile DVR/NVR
    BYTE                bReserved[340];                 // reserved 
    EVENT_COMM_INFO     stCommInfo;                     // public info 
}DEV_EVENT_TRAFFIC_THROW_INFO;

//Event type  EVENT_IVS_TRAFFIC_IDLE(idle)corresponding data block description info
typedef struct tagDEV_EVENT_TRAFFIC_IDLE_INFO
{
    int                 nChannelID;                     // channel ID
    char                szName[DH_EVENT_NAME_LEN];      // event name
    char                bReserved1[8];                  // byte alignment
    DWORD               PTS;                            // Time stamp(ms)
    NET_TIME_EX         UTC;                            // Event occurred time
    int                 nEventID;                       // Event ID
    DH_EVENT_FILE_INFO  stuFileInfo;                    // The corresponding file info of the event
    DH_RESOLUTION_INFO  stuResolution;                  // picture resolution
    DWORD               dwSnapFlagMask;                 // Snap flag(by bit)0 bit:"*",1 bit:"Timing",2 bit:"Manual",3 bit:"Marked",4 bit:"Event",5 bit:"Mosaic",6 bit:"Cutout" 
    BYTE                bEventAction;                   // Event operation.0=pulse event,1=begin of the durative event,2=end of the durative event;
    BYTE                bReserved2[2];
	BYTE				byImageIndex;					// Serial number of the picture, in the same time (accurate to seconds) may have multiple images, starting from 0
    int                 nLane;                          // Corresponding lane No.
    BYTE                bReserved[1024];                // reserved
    EVENT_COMM_INFO     stCommInfo;                     // public info 
}DEV_EVENT_TRAFFIC_IDLE_INFO;

// Event type EVENT_IVS_TRAFFIC_TURNRIGHTAFTERSTRAIGHT(Turn right to make a straight event)
typedef struct tagDEV_EVENT_TURNRIGHTAFTERSTRAIGHT_INFO
{
	int                 nChannelID;									// Channel ID
	char				szName[DH_EVENT_NAME_LEN];					// Event name
	char				bReserved[4];								// Reserved
	double				dPTS;										// Timestamp (in milliseconds)
	NET_TIME_EX			stuUTC;										// Event occurred time
	int					nEventID;									// Event ID
	DH_EVENT_FILE_INFO  stuFileInfo;                                // The corresponding file info of the event
	DH_MSG_OBJECT       stuObject;                                  // Detected object
	DH_MSG_OBJECT       stuVehicle;                                 // Pedestrain info   
	DEV_EVENT_TRAFFIC_TRAFFICCAR_INFO   stTrafficCar;               // Traffic car info
	int					nLane;										// Corresponding lane No.
	int					nSequence;									// Snap index,such as 3,2,1,1 means the last one,0 means there has some exception and snap stop
	BYTE				byReserved[1024];							// Reserved
	EVENT_COMM_INFO     stCommInfo;                                 // Public info 
}DEV_EVENT_TURNRIGHTAFTERSTRAIGHT_INFO;

// Event type EVENT_IVS_TRAFFIC_TURNRIGHTAFTERPEOPLE(Turn right and go straight to pedestrians)
typedef struct tagDEV_EVENT_TURNRIGHTAFTERPEOPLE_INFO
{
	int                 nChannelID;									// Channel ID
	char				szName[DH_EVENT_NAME_LEN];					// Event name
	char				bReserved[4];								// Reserved
	double				dPTS;										// Timestamp (in milliseconds)
	NET_TIME_EX			stuUTC;										// Event occurred time
	int					nEventID;									// Event ID
	DH_EVENT_FILE_INFO  stuFileInfo;                                // The corresponding file info of the event
	DH_MSG_OBJECT       stuObject;                                  // Detected object
	DH_MSG_OBJECT       stuVehicle;                                 // Pedestrain info
	DEV_EVENT_TRAFFIC_TRAFFICCAR_INFO   stTrafficCar;               // Traffic car info
	int					nLane;										// Corresponding lane No.
	int					nSequence;									// Snap index,such as 3,2,1,1 means the last one,0 means there has some exception and snap stop
	BYTE				byReserved[1024];							// Reserved
	EVENT_COMM_INFO     stCommInfo;                                 // Public info 
}DEV_EVENT_TURNRIGHTAFTERPEOPLE_INFO;

#define MAX_DRIVING_DIR_NUM          16                             //  lane direction max quantity

// direction
typedef enum tagNET_FLOWSTAT_DIRECTION
{
	DRIVING_DIR_UNKNOW = 0 ,		// (Before compatible)
	DRIVING_DIR_APPROACH ,			// (Uplink, the vehicle away from the device deployment point is getting closer)
	DRIVING_DIR_LEAVE ,				// (Go down, that the vehicle is farther away from  equipment deployment point)
}NET_FLOWSTAT_DIRECTION;

// road direction
typedef enum tagNET_ROAD_DIRECTION
{
    ROAD_DIR_UNKNOW,             // unknown  
    ROAD_DIR_TURNLEFT,           // left turn 
    ROAD_DIR_TURNRIGHT,          // right turn
    ROAD_DIR_STRAIGHT,           // direction
    ROAD_DIR_UTURU,              // u turn
    ROAD_DIR_NUM,    
}NET_ROAD_DIRECTION;

//Vehicle flow statistics lane direction information
typedef struct tagNET_TRAFFIC_FLOWSTAT_INFO_DIR
{
	NET_FLOWSTAT_DIRECTION		emDrivingDir;							//(Driving direction)
	char						szUpGoing[FLOWSTAT_ADDR_NAME];			//(Uplink locations)
	char						szDownGoing[FLOWSTAT_ADDR_NAME];		//(Go down location)
	BYTE						reserved[32];							//(Reserved bytes)
	
}NET_TRAFFIC_FLOWSTAT_INFO_DIR;

//road jam status
typedef enum tagNET_TRAFFIC_JAM_STATUS
{
    JAM_STATUS_UNKNOW  =0 ,  //unknown 
    JAM_STATUS_CLEAR      ,  //clear
    JAM_STATUS_JAMMED     ,  //jammed
}NET_TRAFFIC_JAM_STATUS;

//road rank
typedef enum tagNET_TRAFFIC_ROAD_RANK
{
	ROAD_RANK_UNKNOWN	=0	,
	ROAD_RANK_RAPID		=1	,	//rapid
	ROAD_RANK_TRUNK		=2	,	//trunk
	ROAD_RANK_SUBTRUNK	=3	,	//subtrunk
	ROAD_RANK_BRANCH	=4	,	//branch
}NET_TRAFFIC_ROAD_RANK;


typedef struct tagNET_TRAFFIC_FLOW_STATE 
{
	int								nLane;				// (Lane number)
	DWORD							dwState;			// (State value)
														// (1 - heavy traffic)
														// (2-heavy traffic recovery)
														// (3-normal)
														// (4 - Flow is too  little)
														// (5-Traffic too low recovery)
	DWORD							dwFlow;				// (Flow value, units: vehicles)
	DWORD							dwPeriod;			// Corresponding statistical time of the flow value,Unit:minute,Used with dwPeriodByMili.
	NET_TRAFFIC_FLOWSTAT_INFO_DIR	stTrafficFlowDir;	// (Lane direction information)
	int					            nVehicles;			// (Total number of passing vehicles)
	float				            fAverageSpeed;		// (Average speed, unit km / h)
	float				            fAverageLength;		// (The average vehicle length, unit meters)
	float				            fTimeOccupyRatio;	// (Share of the time , i.e., The ratio of the sum time for the vehicle passing the cross-section  in  the unit  time and per unit time )
	float				            fSpaceOccupyRatio;	// (Share of the space ,is the result that the average driving distance intervals vehicle is divided the sum of the length of the vehicle measured by the percentage 
	float			            	fSpaceHeadway;		// (Headway, the distance between adjacent vehicles in m / vehicle)
	float				            fTimeHeadway;		// (Headway in seconds / vehicle)
	float				            fDensity;			// (Vehicle density, the number of vehicles per kilometer, unit vehicles / km)
	int					            nOverSpeedVehicles;	// (The number of Speeding vehicles)
	int					            nUnderSpeedVehicles;// (The number of low speeding vehicles)
	int				            	nLargeVehicles;		// (Carts traffic (9 m <car length <12 m), vehicle / unit time)
	int				            	nMediumVehicles;	// (Medium car Traffic 6 m <car length <9 meters), vehicle / unit time
	int				            	nSmallVehicles;		// (Car Traffic 4 m <car length <6 meters), vehicle / unit time
	int				            	nMotoVehicles;		// (Motorized traffic (mini-car, car length <4 m), vehicle / unit time, 
	int				            	nLongVehicles;		// (long traffic (car length> = 12 m), vehicle / unit time, 
	int                             nVolume;            // (Traffic, vehicles / unit time, the number of vehicles which pass through the lane, the road and other vehicles, caculated in one hour)
	int                             nFlowRate;          // (Flow rate of the car, Vehicles / hour, equivalent hours for Vehicle through the lane, a section or a section of the road)
	int                             nBackOfQueue;       // (Queue length, unit: m, distance from the signalized intersection stop line between the upstream end of the line vehicle)(proposed repeal)
	int                             nTravelTime;        // (Travel time, unit: second, a road vehicle used by a certain time. Including all parking delays)
	int                             nDelay;             // (Delay unit: seconds, extra travel time for the driver, passenger or pedestrian spend)
    BYTE                            byDirection[MAX_DRIVING_DIR_NUM]; // lane direction, see NET_ROAD_DIRECTION
    BYTE                            byDirectionNum;     // lane direction quantity
    BYTE                            reserved1[3];       // text align
    NET_TRAFFIC_JAM_STATUS          emJamState;         // road jam status
    //  Traffic statisitcs according to vehicle type
    int                             nPassengerCarVehicles;                      // Passenger vehicle statistics amount (amount/hour)
    int                             nLargeTruckVehicles;                        // Large truck statistics amount 
    int                             nMidTruckVehicles;                          // Medium truck statistics amount (amount/hour)
    int                             nSaloonCarVehicles;                         // Car statistics amount (amount/hour)
    int                             nMicrobusVehicles;                          // Minivan statistics amount (amount/hour)
    int                             nMicroTruckVehicles;                        // Small van statistics amount (amount/hour)
    int                             nTricycleVehicles;                          // Tricycle statistics amount (amount/hour)
    int                             nMotorcycleVehicles;                        // Motor statistics amount (amount/hour)
    int                             nPasserbyVehicles;                          // Pedestrian statistics amount (amount/hour)
	NET_TRAFFIC_ROAD_RANK			emRank;										// road rank
	int								nState;										// State value
																				// (1 - heavy traffic)
																				// (2-heavy traffic recovery)
																				// (3-normal)
																				// (4 - Flow is too  little)
																				// (5-Traffic too low recovery)
    BOOL                            bOccupyHeadCoil;                            // indicating whether the head coil is occupyied
    BOOL                            bOccupyTailCoil;                            // indicating whether the tail coil is occupyied
    BOOL                            bStatistics;                                // indicating whether the statistics is valid
	int								nLeftVehicles;								// (Total nubmer of turn left Vehicles, unit: min)
	int								nRightVehicles;								// (Total number of turn right Vehicles, unit: min)
	int								nStraightVehicles;							// (Total number of straight-head Vehicles,unit: min)
	int								nUTurnVehicles;								// (Total number of U-turn Vehicles,unit: min)
    DH_POINT                        stQueueEnd;                                 // the last car coordinate in a quene of lane,coordinate value 0~8192
	double							dBackOfQueue;								// (Queue length, unit: m, distance from the signalized intersection stop line between the upstream end of the line vehicle)
	DWORD							dwPeriodByMili;								// Corresponding statistical time of the flow millisecond value,Value is not more than 60000.Used with dwPeriod,statistical total time of the flow value:dwPeriod*60*1000+dwPeriodByMili(Unit:millisecond)
	int								nBusVehicles;								// Bus vehicle statistics amount (amount/hour)
	int								nMPVVehicles;								// MPV vehiclestatistics amount (amount/hour)
	int								nMidPassengerCarVehicles;					// midpassenger car vehicle statistics amount (amount/hour)
	int								nMiniCarriageVehicles;						// mini carriage vehicle statistics amount (amount/hour)
	int								nOilTankTruckVehicles;						// oil tank trunk vehicle statistics amount (amount/hour)
	int								nPickupVehicles;							// pick up vehicle statistics amount (amount/hour)
	int								nSUVVehicles;								// SUV vehicle statistics amount (amount/hour)
	int								nSUVorMPVVehicles;							// SUV or MPV vehicle statistics amount (amount/hour)
	int								nTankCarVehicles;							// tank car vehicle statistics amount (amount/hour)
	int								nUnknownVehicles;							// unknown vehicle statistics amount (amount/hour)
	BYTE                            reserved[724];								// Reserved string 
}NET_TRAFFIC_FLOW_STATE;
 
// trafficflowstatinfo (Corresponding data block description)
typedef struct tagNET_TRAFFICFLOWSTATINFO
{
	DH_TRAFFICFLOWSTAT				stuTrafficFlowStat;	// Traffic Flow Stat
	
	double							dbBackOfQueue;		// Queue length
	NET_TRAFFIC_ROAD_RANK			emRank;				// road rank
	int								nState;				// State value
														// (1 - heavy traffic)
														// (2-heavy traffic recovery)
														// (3-normal)
														// (4 - Flow is too  little)
														// (5-Traffic too low recovery)
    BYTE							reserved[1024];		// Reserved string 
}NET_TRAFFICFLOWSTATINFO;


// EVENT_IVS_TRAFFIC_FLOWSTATE (Corresponding data block description)
typedef struct tagDEV_EVENT_TRAFFIC_FLOW_STATE
{
    int                 		nChannelID;                     // (Channel number) 
    char                		szName[DH_EVENT_NAME_LEN];      // (Event name)
    char                		bReserved1[8];                  // (Byte alignment)
    DWORD               		PTS;                            // (Timestamp (in milliseconds))
    NET_TIME_EX         		UTC;                            // (Time for the event occurred)
    int                 		nEventID;                       // (Event ID)
	int							nSequence;						// (No.)
	int							nStateNum;						// (the number of traffic state)
	NET_TRAFFIC_FLOW_STATE 		stuStates[DH_MAX_LANE_NUM];		// (Flow state, each lane corresponding to an element in the array)
	EVENT_INTELLI_COMM_INFO     stuIntelliCommInfo;     		// intelli comm info
	BYTE                		bReserved[892];                // (Reserved bytes)
}DEV_EVENT_TRAFFIC_FLOW_STATE;

// EVENT_IVS_VIDEOSTATIC(Corresponding to data block description) 
typedef struct tagDEV_EVENT_ALARM_VIDEOSTATIC_INFO 
{
    int                 nChannelID;                                 // Channel number
    char                szName[DH_EVENT_NAME_LEN];                  // Event name
    char                bReserved1[8];                              // Byte alignment
    DWORD               PTS;                                        // Timestamp (in milliseconds)
    NET_TIME_EX         UTC;                                        // Time for the event occurred
    int                 nEventID;                                   // Event ID
    EM_ALARM_VIDEO_STATIC_MODE  emMode;                             // the mode 
    BYTE                bEventAction;                               // Even action
    BYTE                bReserved[1027];                            // Reserved bytes
}DEV_EVENT_ALARM_VIDEOSTATIC_INFO;

// EVENT_IVS_VIDEOTIMINGCorresponding to data block description) 
typedef struct tagDEV_EVENT_ALARM_VIDEOTIMING_INFO 
{
    int                 nChannelID;                                 // Channel number
    char                szName[DH_EVENT_NAME_LEN];                  // Event name
    char                bReserved1[8];                              // Byte alignment
    DWORD               PTS;                                        // Timestamp (in milliseconds)
    NET_TIME_EX         UTC;                                        // Time for the event occurred
    int                 nEventID;                                   // Event ID
    BYTE                bEventAction;                               // the mode 
    BYTE                bReserved[1027];                            // Reserved bytes
}DEV_EVENT_ALARM_VIDEOTIMING_INFO;


//event source information
typedef struct tagDEV_SRC_INFO
{
    char           szSrcEvent[DH_COMMON_STRING_32];// source event
    NET_TIME_EX    stSrcTime;                      // the time of source event
    int            iSrcChannel;                    // the channel of source event
    BYTE           reserved[512];                  // Reserved bytes
}DEV_SRC_INFO;

//EVENT_IVS_LINKSD(Corresponding to data block description)
typedef struct tagDEV_EVENT_LINK_SD
{
    // common fields
    int                 nChannelID;                         // Channel number
    char                szName[DH_EVENT_NAME_LEN];          // Event name
    char                bReserved1[8];                      // Byte alignment
    DWORD               PTS;                                // Timestamp (in milliseconds)
    NET_TIME_EX         UTC;                                // Time for the event occurred
    int                 nEventID;                           // Event ID

    // event fields
    DEV_SRC_INFO       *pstSrcInfo;                         // event source information
    NET_TIME_EX         stStartTime;                        // The start time of the preset
    NET_TIME_EX         stEndTime;                          // The end time of the preset
    int                 iPresetID;                          // The preset ID
    BYTE                reserved[512];                      // Reserved bytes
}DEV_EVENT_LINK_SD;

// EVENT_IVS_FLOWRATE(Corresponding to flow rate)
typedef struct tagDEV_EVENT_FLOWRATE_INFO
{
	// common fields
    int                 nChannelID;                         // Channel number
    char                szName[DH_EVENT_NAME_LEN];          // Event name
    char                bReserved1[8];                      // Byte alignment
    double              dbPTS;                              // Timestamp (in milliseconds)
    NET_TIME_EX         stuTime;                            // Time for the event occurred
    int                 nEventID;                           // Event ID

	// event fields
	unsigned int 		nFlowRate;							// flow rate
	BYTE                reserved[1024];                     // Reserved bytes
} DEV_EVENT_FLOWRATE_INFO;


typedef enum tagEM_CITIZENIDCARD_SEX_TYPE
{
    EM_CITIZENIDCARD_SEX_TYPE_UNKNOWN,						// Unknown 
        EM_CITIZENIDCARD_SEX_TYPE_MALE,						// Male
        EM_CITIZENIDCARD_SEX_TYPE_FEMALE,					// Female
        EM_CITIZENIDCARD_SEX_TYPE_UNTOLD,					// Untold
}EM_CITIZENIDCARD_SEX_TYPE;

// Media file search criteria
typedef struct tagDEV_EVENT_ALARM_CITIZENIDCARD_INFO
{
    int                 nChannelID;                                 // Channel No.
    char                szName[DH_EVENT_NAME_LEN];                  // Event name
    char                bReserved1[8];                              // Align type 
    DWORD               PTS;                                        // Time stamp (unit:ms)
    NET_TIME_EX         UTC;                                        // Event occurrence time 
    int                 nEventID;                                   // Event ID
    ///////////////////////////////The above are the public string//////////////////////////////
    int                 nGroupID;                                   // nGroupID event group ID. The nGroupID is the same for the snap process of the same object 
    int                 nCountInGroup;                              // nCountInGroup snap amount of one event group 
    int                 nIndexInGroup;                              // IndexInGroup snap SN of one event group 
    char                szCitizen[DH_COMMON_STRING_64];             // Name
    EM_CITIZENIDCARD_SEX_TYPE   emSex;                              // Gender 
    int                 nEthnicity;                                 // Nationality 
    // 0 invalid data 
    // 1 Han
    // 2 Mongolian
    // 3 Hui
    // 4 Tibetan
    // 5 Uygur
    // 6 Miao
    // 7 Yi
    // 8 Zhuang
    // 9 Bouyei
    // 10 Korean
    // 11 Manchu
    // 12 Dong
    // 13 Yao
    // 14 Bai
    // 15 Tujia
    // 16 Hani
    // 17 Kazak
    // 18 Dai
    // 19 Li
    // 20 Lisu
    // 21 Va
    // 22 She
    // 23 Gaoshan
    // 24 Lahu
    // 25 Shui
    // 26 Dongxiang
    // 27 Naxi
    // 28 Jingpo
    // 29 Kirgiz
    // 30 Tu
    // 31 Daur
    // 32 Mulam
    // 33 Qoiang
    // 34 Blang
    // 35 Salar
    // 36 Maonan
    // 37 Gelo
    // 38 Xibe
    // 39 Achang
    // 40 Pumi
    // 41 Tajik
    // 42 Nu
    // 43 Ozbek
    // 44 Russian
    // 45 Ewenkl
    // 46 Deang
    // 47 Bonan
    // 48 Yugur
    // 49 Jing
    // 50 Tatar
    // 51 Drung
    // 52 Oroqen
    // 53 Hezhen
    // 54 Moinba
    // 55 Lhoba
    // 56 Jino
    NET_TIME            stuBirth;                                   // Birth date 
    char                szAddress[DH_COMMON_STRING_256];            // Address
    char                szNumber[DH_COMMON_STRING_64];              // ID
    char                szAuthority[DH_COMMON_STRING_256];          // Issued authority 
    NET_TIME            stuStart;                                   // Start date 
    BOOL                bLongTimeValidFlag;                         // When the value is TRUE, the stop date menas always valid, at this time, the  stuEnd is null. 
    // When the value is FALSE, refer to stuEnd value for stop date.
    NET_TIME            stuEnd;                                     // Stop date, it is valid when bLongTimeValidFlag is FALSE.
    char                szReversed[1024];                           // Reserved string 
}DEV_EVENT_ALARM_CITIZENIDCARD_INFO;

// Corresponding data description info of event type EVENT_IVS_PICINFO (image info event) 
typedef struct tagDEV_EVENT_ALARM_PIC_INFO 
{
    int                 nChannelID;                                 // Channel No.
    char                szName[DH_EVENT_NAME_LEN];                  // Event name 
    char                bReserved1[8];                              // Align byte 
    DWORD               PTS;                                        // Time stamp (Unit:ms)
    NET_TIME_EX         UTC;                                        // Event occurrence time (Not for mobile device )
    int                 nEventID;                                   // Event ID
	///////////////////////////////Comon information above //////////////////////////////
	NET_TIME_EX         stuTime;                                    // Event occurrence time (Device time, it may not be utc time).
    DWORD               dwEventType;                                // Event  type
    DWORD               dwSpeed;                                    // Speed. Unit is km/h.
	DWORD               dwSpeedHighLine;                            // Speed limit (max speed)on highway. Unit is km/h.
    DWORD               dwDisk;                                     // Disk No.
    DWORD               dwCluster;                                  // Cluster No.
    DWORD               dwPartition;                                // Partition No.	
    char                szSnapAddr[DH_COMMON_STRING_128];           // Snap position. Valid 64-byte 
    char                szPicID[DH_COMMON_STRING_32];               // Image only ID
    char                szPlate[DH_COMMON_STRING_16];               // Plate 
    char                szReversed[2000];                           // Reserved string 
}DEV_EVENT_ALARM_PIC_INFO;

// Corresponding data description info of event type EVENT_IVS_ACCESS_CTL (Access control info event) 
typedef struct tagDEV_EVENT_ACCESS_CTL_INFO 
{
    int                         nChannelID;                         // Door Channel Number
    char                        szName[DH_EVENT_NAME_LEN];          // Entrance Guard Name
	char                		bReserved1[4];                      // Align byte 
    double              		PTS;                                // Time stamp (Unit:ms)
    NET_TIME_EX         		UTC;                                // Event occurrence time 
    int                 		nEventID;                           //  Event ID
    DH_MSG_OBJECT           	stuObject;                          // have being detected object
	DH_EVENT_FILE_INFO			stuFileInfo;	                    // The corresponding file info of the event
    NET_ACCESS_CTL_EVENT_TYPE   emEventType;                        // Entrance Guard Event Type
    BOOL                        bStatus;                            // Swing Card Result,True is Success,False is Fail
    NET_ACCESSCTLCARD_TYPE      emCardType;                         // Card Type
    NET_ACCESS_DOOROPEN_METHOD  emOpenMethod;                       // Open The Door Method
    char                        szCardNo[DH_MAX_CARDNO_LEN];        // Card Number
    char                        szPwd[DH_MAX_CARDPWD_LEN];          // Password
    char                        szReaderID[DH_COMMON_STRING_32];    // Reader ID
    char                        szUserID[DH_COMMON_STRING_64];      // unlock user
    char                        szSnapURL[DH_COMMON_STRING_128];    // snapshot picture storage address
    int                         nErrorCode;                         // Reason of unlock failure, only because it is valid when bStatus is FALSE
                                                                    // 0x00 no error
                                                                    // 0x10 unauthorized
                                                                    // 0x11 card lost or cancelled
                                                                    // 0x12 no door right
                                                                    // 0x13 unlock mode error
                                                                    // 0x14 valid period error
                                                                    // 0x15 anti sneak into mode
                                                                    // 0x16 forced alarm not unlocked
                                                                    // 0x17 door NC status
                                                                    // 0x18 AB lock status
                                                                    // 0x19 patrol card
                                                                    // 0x1A device is under intrusion alarm status
                                                                    // 0x20 period error
                                                                    // 0x21 unlock period error in holiday period
                                                                    // 0x30 first card right check required
                                                                    // 0x40 card correct, input password error
                                                                    // 0x41 card correct, input password timed out
                                                                    // 0x42 card correct, input fingerprint error
                                                                    // 0x43 card correct, input fingerprint timed out
                                                                    // 0x44 fingerprint correct, input password error
                                                                    // 0x45 fingerprint correct, input password timed out
                                                                    // 0x50 group unlock sequence error
                                                                    // 0x51 test required for group unlock
                                                                    // 0x60 test passed, control unauthorized
    int                         nPunchingRecNo;                     // punching record number
	int							nNumbers;							// picture Numbers
    BYTE						byImageIndex;	                    // Serial number of the picture, in the same time (accurate to seconds) may have multiple images, starting from 0
	BYTE                        byReserved[3];                      // Align byte 
	DWORD						dwSnapFlagMask;						// Snap flag(by bit)0 bit:"*",1 bit:"Timing",2 bit:"Manual",3 bit:"Marked",4 bit:"Event",5 bit:"Mosaic",6 bit:"Cutout"     
    NET_ATTENDANCESTATE         emAttendanceState;                  // Attendance state 
    BYTE						bReserved[1020];					// Reserved string 
}DEV_EVENT_ACCESS_CTL_INFO;


//Event EVENT_IVS_NETPALYCHECK(Corresponding to data block description)
typedef struct tagDEV_EVENT_ALARM_NETPLAYCHECK_INFO
{
	int                 nChannelID;                                 // Channel number
    char                szName[DH_EVENT_NAME_LEN];                  // Event name
    char                bReserved1[8];                              // Byte alignment
    DWORD               PTS;                                        // Timestamp (in milliseconds)
    NET_TIME_EX         UTC;                                        // Time for the event occurred
    int                 nEventID;                                   // Event ID
    ///////////////////////////////Comon information above//////////////////////////////
	NET_TIME_EX         stuTime;                                    // Event occurrence time (Device time, it may not utc).
	char				szIdentityCard[DH_COMMON_STRING_32];		// Identity Card
	char				szNetBarName[DH_COMMON_STRING_32];			// Internet Bar Name
	char				szNetBarID[DH_COMMON_STRING_32];			// Internet Bar ID
	char				szNetBarAddr[DH_COMMON_STRING_64];			// Network bar address 
}DEV_EVENT_ALARM_NETPLAYCHECK_INFO;

//Corresponding to data block description of event type EVENT_IVS_SNAPBYTIME(snap by time event) 
typedef struct tagDEV_EVENT_SNAPBYTIME
{
	int                 nChannelID;                                 // Channel number
    char                szName[DH_EVENT_NAME_LEN];                  // Event name
    char                bReserved1[4];                              // Byte alignment
    double              PTS;                                        // Timestamp (in milliseconds)
    NET_TIME_EX         UTC;                                        // Time for the event occurred
    int                 nEventID;                                   // Event ID
    ///////////////////////////////Comon information above, In addition to other fields outside the nChannelID is to set aside public space//////////////////////////////
    NET_TIME_EX         stuSnapTime;                                // snap time
    char                szReversed[2048];                           // Reserved string 
}DEV_EVENT_SNAPBYTIME;

// Corresponding to data block description of event type EVENT_IVS_TRAFFIC_TIREDPHYSIOLOGICAL(physiological fatigue driving events)
typedef struct tagDEV_EVENT_TIREDPHYSIOLOGICAL_INFO
{
	int                 nChannelID;                                   // Channel ID
    char                szName[DH_EVENT_NAME_LEN];                    // Event name
    char                bReserved1[4];                                // Byte alignment
    double              PTS;                                          // Timestamp (in milliseconds)
    NET_TIME_EX         UTC;                                          // Time for the event occurred
    int                 nEventID;                                     // Event ID
    
    int                 nAction;                                      // event action, 0: Pulse, 1: Start, 2: Stop
	NET_GPS_STATUS_INFO stuGPSStatus;                                 // GPS info
    BYTE				bReserved[1024];							  // Reserved 
} DEV_EVENT_TIREDPHYSIOLOGICAL_INFO;

// Corresponding to data block description of event type EVENT_IVS_TRAFFIC_TIREDLOWERHEAD(Event of driver lower head)
typedef struct tagDEV_EVENT_TIREDLOWERHEAD_INFO
{
	int                 nChannelID;                                   // Channel ID
    char                szName[DH_EVENT_NAME_LEN];                    // Event name
    char                bReserved1[4];                                // Byte alignment
    double              PTS;                                          // Timestamp (in milliseconds)
    NET_TIME_EX         UTC;                                          // Time for the event occurred
    int                 nEventID;                                     // Event ID
    
	NET_GPS_STATUS_INFO stuGPSStatus;                                 // GPS info
    BYTE				bReserved[1024];							  // Reserved 
} DEV_EVENT_TIREDLOWERHEAD_INFO;

// Corresponding to data block description of event type EVENT_IVS_TRAFFIC_DRIVERLOOKAROUND(Event of driver look around)
typedef struct tagDEV_EVENT_DRIVERLOOKAROUND_INFO
{
	int                 nChannelID;                                   // Channel ID
    char                szName[DH_EVENT_NAME_LEN];                    // Event name
    char                bReserved1[4];                                // Byte alignment
    double              PTS;                                          // Timestamp (in milliseconds)
    NET_TIME_EX         UTC;                                          // Time for the event occurred
    int                 nEventID;                                     // Event ID
    
	NET_GPS_STATUS_INFO stuGPSStatus;                                 // GPS info
    BYTE				bReserved[1024];							  // Reserved 
} DEV_EVENT_DRIVERLOOKAROUND_INFO;

// Corresponding to data block description of event type EVENT_IVS_TRAFFIC_DRIVERLEAVEPOST(Event of driver leave post)
typedef struct tagDEV_EVENT_DRIVERLEAVEPOST_INFO
{
	int                 nChannelID;                                   // Channel ID
    char                szName[DH_EVENT_NAME_LEN];                    // Event name
    char                bReserved1[4];                                // Byte alignment
    double              PTS;                                          // Timestamp (in milliseconds)
    NET_TIME_EX         UTC;                                          // Time for the event occurred
    int                 nEventID;                                     // Event ID
    
	NET_GPS_STATUS_INFO stuGPSStatus;                                 // GPS info
    BYTE				bReserved[1024];							  // Reserved 
} DEV_EVENT_DRIVERLEAVEPOST_INFO;

// Corresponding to data block description of event type EVENT_IVS_TRAFFIC_DRIVERYAWN (Event of driver yawn)
typedef struct tagDEV_EVENT_DRIVERYAWN_INFO
{
	int                 nChannelID;                                   // Channel ID
    char                szName[DH_EVENT_NAME_LEN];                    // Event name
    char                szReserved1[4];                               // Byte alignment
    double              PTS;                                          // Timestamp (in milliseconds)
    NET_TIME_EX         UTC;                                          // Time for the event occurred
    int                 nEventID;                                     // Event ID

	NET_GPS_STATUS_INFO stuGPSStatus;                                 // GPS info
    BYTE				byReserved[1024];							  // Reserved 
} DEV_EVENT_DRIVERYAWN_INFO;

// Detect Object
typedef enum tagEM_DETECT_OBJECT
{
	EM_DETECT_OBJECT_UNKNOWN,					// Unknown
	EM_DETECT_OBJECT_HUMAN_BODY_AND_FACE,		// Both human body and human face
	EM_DETECT_OBJECT_HUMAN_BODY,				// Only human body
	EM_DETECT_OBJECT_HUMAN_FACE,				// Only human face
}EM_DETECT_OBJECT;

// Human image info
typedef struct tagHUMAN_IMAGE_INFO
{
	unsigned int       nOffSet;					// Offset 		
	unsigned int	   nLength;					// image length,Unit:byte
	BYTE			   byReserved[64];			// Reserved
}HUMAN_IMAGE_INFO;

// Face image info
typedef struct tagFACE_IMAGE_INFO
{
	unsigned int	   nOffSet;					// Offset  
	unsigned int	   nLength;					// image length,Unit:byte
	BYTE			   byReserved[64];			// Reserved
}FACE_IMAGE_INFO;

// Corresponding to data block description of event type EVENT_IVS_HUMANTRAIT(Event of human trait)
typedef struct tagDEV_EVENT_HUMANTRAIT_INFO
{
	int					nChannelID;									  // Channel ID
	char				szName[DH_EVENT_NAME_LEN];					  // Event name
	int                 nEventID;                                     // Event ID
	double              PTS;                                          // Timestamp (in milliseconds)
	NET_TIME_EX         UTC;                                          // Time for the event occurred
	int                 nAction;                                      // 1:Start 2:Stop

	EM_CLASS_TYPE		emClassType;								  // Class type 
	int					nGroupID;									  // Event group ID,A detection of multiple human characteristics at the same nGroupID
	int					nCountInGroup;								  // the captured human number within an event group,A detection of multiple human characteristics at the same nCountInGroup
	int					nIndexInGroup;								  // capture Index of an event group,starting from 1
	HUMAN_IMAGE_INFO	stuHumanImage;								  // Human body image info
	FACE_IMAGE_INFO		stuFaceImage;								  // Human face image info
	EM_DETECT_OBJECT	emDetectObject;								  // Detect Object
	BYTE				byReserved[876];							  // Reserved 
}DEV_EVENT_HUMANTRAIT_INFO;

// Corresponding to data block description of event type EVENT_IVS_TRAFFIC_BUSSHARPTURN(bus sharp trun events)
typedef struct tagDEV_EVENT_BUSSHARPTURN_INFO
{
	int                 nChannelID;                                   // Channel ID
    char                szName[DH_EVENT_NAME_LEN];                    // Event name
    char                bReserved1[4];                                // Byte alignment
    double              PTS;                                          // Timestamp (in milliseconds)
    NET_TIME_EX         UTC;                                          // Time for the event occurred
    int                 nEventID;                                     // Event ID

    int                 nAction;                                      // event action, 0: Pulse, 1: Start, 2: Stop
	NET_GPS_STATUS_INFO stuGPSStatus;                                 // GPS info
    BYTE				bReserved[1024];							  // Reserved 
} DEV_EVENT_BUSSHARPTURN_INFO;

// Corresponding to data block description of event type EVENT_IVS_TRAFFIC_ELETAGINFO(RFID electronic tag  events)
typedef struct tagDEV_EVENT_TRAFFIC_ELETAGINFO_INFO
{
	
	int                 nChannelID;							// channel ID
    char                szName[DH_EVENT_NAME_LEN];			// Event name
    char                bReserved1[4];						// Byte alignment
    double              PTS;								// Timestamp (in milliseconds)
    NET_TIME_EX         UTC;								// Time for the event occurred
    int                 nEventID;							// Event ID
    ///////////////////////////////Comon information above//////////////////////////////
    int                 nAction;                            // event action, 0: Pulse, 1: Start, 2: Stop
    NET_RFIDELETAG_INFO	stuRFIDEleTagInfo;					// RFID electronic tag  info
	int					nRetChannelNum;						// Number of return channels
	int					nLinkChannel[DH_MAX_CHANNUM];		// Associated camera channel, the channel number starts at 0
	BYTE				bReserved[956];						// Reserved string 
} DEV_EVENT_TRAFFIC_ELETAGINFO_INFO;


// media file query condition
typedef enum __EM_FILE_QUERY_TYPE
{
	DH_FILE_QUERY_TRAFFICCAR,							// Vehicle information,corresponding structure is MEDIA_QUERY_TRAFFICCAR_PARAM
	DH_FILE_QUERY_ATM,									// ATM information
	DH_FILE_QUERY_ATMTXN,								// ATM transaction information 
	DH_FILE_QUERY_FACE,  								// Face info
    DH_FILE_QUERY_FILE,                                 // file info, corresponding to NET_IN_MEDIA_QUERY_FILE and NET_OUT_MEDIAFILE_FILE
	DH_FILE_QUERY_TRAFFICCAR_EX,						// Transportation vehicle information, expand DH_FILE_QUERY_TRAFFICCAR, support more fields,corresponding structure is MEDIA_QUERY_TRAFFICCAR_PARAM_EX
    DH_FILE_QUERY_FACE_DETECTION,                       // face recognition event info MEDIAFILE_FACE_DETECTION_PARAM  and MEDIAFILE_FACE_DETECTION_INFO
    DH_FILE_QUERY_IVS_EVENT,                            // ivs event info MEDIAFILE_IVS_EVENT_PARAM and MEDIAFILE_IVS_EVENT_INFO
    DH_FILE_QUERY_ANALYSE_OBJECT,                       // analyse object info MEDIAFILE_ANALYSE_OBJECT_PARAM and MEDIAFILE_ANALYSE_OBJECT_INFO
	DH_FILE_QUERY_MPT_RECORD_FILE,                      // query mpt record file,corresponding to MEDIAFILE_MPT_RECORD_FILE_PARAM and MEDIAFILE_MPT_RECORD_FILE_INFO
} EM_FILE_QUERY_TYPE;

typedef enum _EM_RECORD_SNAP_FLAG_TYPE
{
    FLAG_TYPE_TIMING ,                  //Schele
    FLAG_TYPE_MANUAL ,                  //Manual
    FLAG_TYPE_MARKED ,                  //Important
    FLAG_TYPE_EVENT  ,                  //Event
    FLAG_TYPE_MOSAIC ,                  //Combined
    FLAG_TYPE_CUTOUT ,                  //Cut
    FLAG_TYPE_LEAVE_WORD ,              //Message
    FLAG_TYPE_TALKBACK_LOCAL_SIDE ,     //Talk Local
    FLAG_TYPE_TALKBACK_REMOTE_SIDE ,    //Talk Remote
    FLAG_TYPE_SYNOPSIS_VIDEO ,          //Compressed Video
    FLAG_TYPE_ORIGINAL_VIDEO ,          //Original Video
    FLAG_TYPE_PRE_ORIGINAL_VIDEO ,      //Processed
    FLAG_TYPE_BLACK_PLATE ,             //Blacklist Picture
    FLAG_TYPE_ORIGINAL_PIC ,            //Original Picture
    FLAG_TYPE_CARD,                     //card no. record
    FLAG_TYPE_MAX = 128, 
}EM_RECORD_SNAP_FLAG_TYPE;

// ATM trsaction type
typedef enum
{
    ATM_TRADE_ALL,                      // all types
    ATM_TRADE_ENQUIRY,                  // search
    ATM_TRADE_WITHDRAW,                 // withdraw
    ATM_TRADE_MODIFY_PASSWORD,          // change password
    ATM_TRADE_TRANSFER,                 // transfer
    ATM_TRADE_DEPOSIT,                  // deposit
    ATM_TRADE_CARDLESS_ENQUIRY,         // search without card
    ATM_TRADE_CARDLESS_DEPOSIT,         // deposit without card
    ATM_TRADE_OTHER,                    // other
}EM_ATM_TRADE_TYPE;

// card no. record info
typedef struct
{
    DWORD               dwSize;
    int                 nType;          				// type, 0-Card, 1-Field
    char                szCardNo[DH_MAX_CARD_INFO_LEN]; // card no.
    EM_ATM_TRADE_TYPE   emTradeType;    				// transaction type
    char                szAmount[DH_COMMON_STRING_64]; 	// transaction amount, nullstring means no limit amount
    int                 nError;         				// error code, 0-all errors, 1-retain cash, 2-retain card
    int                 nFieldCount;    				// domain quantity, by domain search is valid
    char                szFields[MAX_CARD_RECORD_FIELD_NUM][DH_COMMON_STRING_256];   // domain info, by domain search is valid
	char				szChange[DH_COMMON_STRING_32];	// change
}NET_RECORD_CARD_INFO;

#define MAX_IVS_EVENT_NUM    256
#define MAX_QUERY_USER_NUM   4                 // max number of user

// the order of result order 
typedef enum tagEM_RESULT_ORDER_TYPE
{
    EM_RESULT_ORDER_UNKNOWN,                    // unknown order type
    EM_RESULT_ORDER_ASCENT_BYTIME,              // ascent by time
    EM_RESULT_ORDER_DESCENT_BYTIME,             // descent by time   
}EM_RESULT_ORDER_TYPE;

// record info, corresponde to CLIENT_FindFileEx, search condition
// support paths search in curent
typedef struct  
{
    DWORD               dwSize;                 // size 
    char*               szDirs;                 // working directory list,can inquire multiple directory at a atime,separated by ";",example "/mnt/dvr/sda0;/mnt/dvr/sda1",if szDirs==null or szDirs == "" ,means search all
    int					nMediaType;		        // file info,0:any type,1:search jpg image,2:search dav
    int                 nChannelID;             // Channel start from 0, -1 means search all channel
    NET_TIME            stuStartTime;           // start time	
    NET_TIME            stuEndTime;             // end time
    int                 nEventLists[MAX_IVS_EVENT_NUM]; // Event type list, see intelligent analysis event type
    int                 nEventCount;            // event total
    BYTE                byVideoStream;          // video stream 0-unknown  1-main 2-sub 1 3-sub 2 4- sub 3 
    BYTE                bReserved[3];           // aligh text
    EM_RECORD_SNAP_FLAG_TYPE emFalgLists[FLAG_TYPE_MAX]; // Record or snapshot file mark, not set mark to search all files
    int                 nFalgCount;             // total mark
    NET_RECORD_CARD_INFO stuCardInfo;           // card no. record info, emFalgLists including card no. video is valid
    int                 nUserCount;             // user total
    char                szUserName[MAX_QUERY_USER_NUM][DH_NEW_USER_NAME_LENGTH]; // user name
    EM_RESULT_ORDER_TYPE emResultOrder;         // result order
}NET_IN_MEDIA_QUERY_FILE;

// record info, corresponde to CLIENT_FindFileEx, search result
typedef struct 
{
    char szKey[DH_COMMON_STRING_64]   ;      // Abstract name 
    char szValue[DH_COMMON_STRING_512] ;     // Abstract contents 
    BYTE bReserved[256];                     // Reserved string    
}NET_FILE_SUMMARY_INFO;
//  record information corresponding to DH_FILE_QUERY_FILE command when use CLIENT_FindFileEx interface
typedef struct  
{
    DWORD               dwSize;                 // size
    int					nChannelID;				// channel ID,from 0,-1 means search all
    NET_TIME			stuStartTime;			// start time
    NET_TIME			stuEndTime;				// end time
    unsigned int		nFileSize;				// size of file
    BYTE				byFileType;				// file type 1:jpg, 2: dav
    BYTE                byDriveNo;              // deprecated, to get same info, use nDriveNo instead
    BYTE                byPartition;            // zone no.
    BYTE                byVideoStream;          // video stream 0-unknown  1-main 2-sub 1 3-sub 4-sub 
    unsigned int        nCluster;               // cluster
	char				szFilePath[MAX_PATH];	// FilePath
    int                 nEventLists[MAX_IVS_EVENT_NUM]; // Link event list,see event intelligent analysis event type
    int                 nEventCount;            //event total
    EM_RECORD_SNAP_FLAG_TYPE emFalgLists[FLAG_TYPE_MAX]; // record or snapshot file mark 
    int                 nFalgCount;             //mark total
    unsigned int        nDriveNo;               // disk driver number
    char szSynopsisPicPath[DH_COMMON_STRING_512];              // Snap file path when pre-process the file
    int                 nSynopsisMaxTime;                      // Video synopsis max time. Unit is second. 
    int                 nSynopsisMinTime;                      // Video synopsis min time. Unit is second. 
    int                 nFileSummaryNum;                       // file summary number
    NET_FILE_SUMMARY_INFO   stFileSummaryInfo[MAX_FILE_SUMMARY_NUM];        // file summary info   
}NET_OUT_MEDIA_QUERY_FILE;

//The corresponding search criteria of  DH_MEDIA_QUERY_TRAFFICCARtypedef struct  
typedef struct
{
	int					nChannelID;						// The channel number begins with 0. -1 is to search information of all channels .
	NET_TIME			StartTime;						// Start time 	
	NET_TIME			EndTime;						// End time 
	int					nMediaType;						// File type:0=search any type.1=search jpg file
	int					nEventType;						// deprecated, to get same info, use pEventType instead
	char				szPlateNumber[32];				// Vehicle plate. "\0" is to search any plate number.
	int					nSpeedUpperLimit;				// The searched vehicle speed range. Max speed unit is km/h
	int					nSpeedLowerLimit;				// The searched vehicle speed range. Min speed unit is km/h
	BOOL				bSpeedLimit;					// Search according to the speed or not.  TRUE: search according to the speed.nSpeedUpperLimit and nSpeedLowerLimit is valid.
    DWORD				dwBreakingRule;					// Illegal type:
														// When event type is EVENT_IVS_TRAFFICGATE
														//		bit1: Retrograde;   bit2: Overline; 
														//		bit3: Overspend; 	bit4:Under speed; 
														//		bit5: RunRedLight;
														// When event type is EVENT_IVS_TRAFFICJUNCTION
														//		bit1: RunRedLight;  bit2: WrongLan;  
														//		bit3: Retrograde; 	bit4:UTurn;
														//	    bit5: Overline;
	char                szPlateType[32];                // Plate type: "Unknown" =Unknown; "Normal"=Blue and black plate. "Yellow"=Yellow plate. "DoubleYellow"=Double-layer yellow plate 
												        // "Police"=Police plate ; "Armed"= =Military police plate; "Military"=Army plate; "DoubleMilitary"=Army double-layer 
												        // "SAR" =HK SAR or Macao SAR plate; "Trainning" =rehearsal plate; "Personal"=Personal plate; "Agri"=Agricultural plate
												        // "Embassy"=Embassy plate; "Moto"=Moto plate ; "Tractor"=Tractor plate; "Other"=Other plate 
	char                szPlateColor[16];               // plate color, "Blue","Yellow", "White","Black"
	char				szVehicleColor[16];		        // vehicle color:"White", "Black", "Red", "Yellow", "Gray", "Blue","Green"
	char				szVehicleSize[16];		        // vehicle type:"Light-duty";"Medium"; "Oversize"
	int                 nGroupID;                       // id of event group(it works when >= 0)
	short               byLane;                         // lane number(it works when >= 0)
	BYTE				byFileFlag;						// file flag, 0xFF-use nFileFlagEx, 0-all record, 1-timing file, 2-manual, 3-event, 4-important, 5-mosaic
	BYTE				byRandomAccess;                 // The need for random jumps in the query process, 0 - no need 1 - need
	int					nFileFlagEx;					// file flag, bit0-timing, bit1-manual, bit2-event, bit3-important, bit4-mosaic, 0xFFFFFFFF-all
	int					nDirection;				        // direction(to the direction of car)	0-north 1-northeast 2-east 3-southeast 4-south 5-southwest 6-west 7-northwest 8-unknown -1-all directions
	char*               szDirs;                         // working directory list,can inquire multiple directory at a atime,separated by ";",example "/mnt/dvr/sda0;/mnt/dvr/sda1",if szDirs==null or szDirs == "" ,means search all
	int*                pEventTypes;                    // Check the event type to be an array of pointers, event type, see "intelligent analysis event type", if the query is NULL considered all events (buffer required to apply by the user)
	int                 nEventTypeNum;                  // Event Type array size
	char*				pszDeviceAddress;               // Device address, NULL indicates that the field does not work
	char*				pszMachineAddress;				// Machine deployment locations, NULL indicates that the field does not work
	char*				pszVehicleSign;					// Vehicle identification, such as "Unknown" - unknown, "Audi" - Audi, "Honda" - Honda ... NULL indicates that the field does not work
	WORD                wVehicleSubBrand;               // Specifies the sub-brand of vehicle,the real value can be found in a mapping table from the development manual
    WORD                wVehicleYearModel;              // Specifies the model years of vehicle. the real value can be found in a mapping table from the development manual
	int					bReserved[28];					// Reserved field for future extension.
} MEDIA_QUERY_TRAFFICCAR_PARAM;

// The media file information searched by DH_MEDIA_QUERY_TRAFFICCAR
typedef struct
{
    unsigned int		ch;						// Channel number
    char				szFilePath[128];		// File path 
    unsigned int		size;					// File length 
    NET_TIME			starttime;				// Start time
    NET_TIME			endtime;				// End time
    unsigned int		nWorkDirSN;				// Working directory serial number									
	BYTE				nFileType;				// File type.  1:picture  2:video 
	BYTE                bHint;					// File location index
	BYTE                bDriveNo;               // drive number
	BYTE                bReserved2;
	unsigned int        nCluster;               // cluster number
	BYTE				byPictureType;			// picture type or file flag, 0-Normal, 1-Mosaic, 2-Cutout. more flags information ref to MEDIAFILE_TRAFFICCAR_INFO_EX's filed emFalgLists
	BYTE                byVideoStream;			// video stream 0-unknown  1-main 2-sub1 3-sub2 4-sub3 
	BYTE                byPartition;			// accurate positioning No.
	BYTE                bReserved[1];           // Reserved field for future extension. 

	//The following contents is the vehicle information 
	char				szPlateNumber[32];		// Vehicle plate number
	char				szPlateType[32];		// Plate type: "Unknown" =Unknown; "Normal"=Blue and black plate. "Yellow"=Yellow plate. "DoubleYellow"=Double-layer yellow plate 
												// "Police"=Police plate ; "Armed"= =Military police plate; "Military"=Army plate; "DoubleMilitary"=Army double-layer 
												// "SAR" =HK SAR or Macao SAR plate; "Trainning" =rehearsal plate; "Personal"=Personal plate; "Agri"=Agricultural plate
												// "Embassy"=Embassy plate; "Moto"=Moto plate ; "Tractor"=Tractor plate; "Other"=Other plate 
	char				szPlateColor[16];		// Plate color:"Blue","Yellow", "White","Black"
	char				szVehicleColor[16];		// Vehicle color:"White", "Black", "Red", "Yellow", "Gray", "Blue","Green"
	int					nSpeed;					// Speed. The unit is Km/H
	int					nEventsNum;				// Activation event amount 
	int					nEvents[32];			// Activation event list. The number refers to the corresponding event. Please refer to Intelligent Analytics Event Type.		
	DWORD				dwBreakingRule;			// Detailed offense type subnet mask. The first bit means redlight offense, the second bit is illegal straight/left-turn/right-turn driving. 
	                                            // The third bit is the wrong way driving; the four bit is illegal U-turn. Otherwise default value is intersection accident. 
	char				szVehicleSize[16];		// Vehicle type:"Light-duty"=small;"Medium"=medium; "Oversize"=large
	char				szChannelName[DH_CHAN_NAME_LEN];// Local or remote channel name
	char				szMachineName[DH_MAX_NAME_LEN];	// Local or remote device name
	int					nSpeedUpperLimit;	    // up limit of speed, km/h
	int					nSpeedLowerLimit;		// lower limit of speed km/h	
	int                 nGroupID;               // id of event group
	BYTE                byCountInGroup;         // total count of the event group
	BYTE                byIndexInGroup;         // the index of this event
	BYTE                byLane;                 // lane number
	BYTE                bReserved1[21];			// reserved
    NET_TIME            stSnapTime;             // snap time
	int					nDirection;				// direction,MEDIA_QUERY_TRAFFICCAR_PARAM
	char                szMachineAddress[MAX_PATH]; // machine address
} MEDIAFILE_TRAFFICCAR_INFO, *LPMEDIAFILE_TRAFFICCAR_INFO;

// refuel type
typedef enum tagEM_REFUEL_TYPE
{
    EM_REFUEL_TYPE_UNKNOWN = 0,							// unknown
	EM_REFUEL_TYPE_NINETY_EIGHT,						// "98#"
	EM_REFUEL_TYPE_NINETY_SEVEN,						// "97#"
	EM_REFUEL_TYPE_NINETY_FIVE,							// "95#"
	EM_REFUEL_TYPE_NINETY_THREE,                        // "93#"
	EM_REFUEL_TYPE_NINETY,								// "90#"
	EM_REFUEL_TYPE_TEN,									// "10#"
	EM_REFUEL_TYPE_FIVE,								// "5#"
	EM_REFUEL_TYPE_ZERO,								// "0#"
	EM_REFUEL_TYPE_NEGATIVE_TEN,						// "-10#"
	EM_REFUEL_TYPE_NEGATIVE_TWENTY,						// "-20#"
	EM_REFUEL_TYPE_NEGATIVE_THIRTY_FIVE,				// "-35#"
	EM_REFUEL_TYPE_NEGATIVE_FIFTY,						// "-50#"
}EM_REFUEL_TYPE;

// DH_MEDIA_QUERY_TRAFFICCAR_EX Corresponding query
typedef struct tagMEDIA_QUERY_TRAFFICCAR_PARAM_EX
{
	DWORD				dwSize;
	MEDIA_QUERY_TRAFFICCAR_PARAM stuParam;	        	// The basic query parameters
} MEDIA_QUERY_TRAFFICCAR_PARAM_EX;

// DH_MEDIA_QUERY_TRAFFICCAR_EX Check out the file information
typedef struct tagMEDIAFILE_TRAFFICCAR_INFO_EX
{
	DWORD				dwSize;
	MEDIAFILE_TRAFFICCAR_INFO stuInfo;			        	// Basic Information
	char				szDeviceAddr[DH_COMMON_STRING_256];	// Device Address
	char				szVehicleSign[DH_COMMON_STRING_32];	// Vehicle identification, such as "Unknown" - unknown, "Audi" - Audi, "Honda" - Honda ..
    char                szCustomParkNo[DH_COMMON_STRING_64];// self defined parking space number, for parking, 
    WORD                wVehicleSubBrand;                   // Specifies the sub-brand of vehicle,the real value can be found in a mapping table from the development manual
    WORD                wVehicleYearModel;                  // Specifies the model years of vehicle. the real value can be found in a mapping table from the development manual
	NET_TIME			stuEleTagInfoUTC;					// corresponding to throughTime in electronic tag info
	EM_RECORD_SNAP_FLAG_TYPE emFalgLists[FLAG_TYPE_MAX];	// record or snapshot file mark 
    int                 nFalgCount;							// mark total	
} MEDIAFILE_TRAFFICCAR_INFO_EX;

// FINDNEXT Find input parameter
typedef struct __NET_FINDNEXT_RESERVED
{
	DWORD               dwSize;                 // Structure size
	
	unsigned int        nBeginNumber;           // Search begin number, start from begin number, 0<=beginNumber<= totalCount-1
}NET_FINDNEXT_RESERVED;

// Enquiry jump condition
typedef struct __NET_FINDING_JUMP_OPTION_INFO
{
	DWORD           dwSize;
	int             nOffset;                            // Query results offset relative to the first query results position offset current query
}NET_FINDING_JUMP_OPTION_INFO;

// DH_FILE_QUERY_FACE Corresponding face recognition service search parameter
typedef struct __MEDIAFILE_FACERECOGNITION_PARAM
{
	DWORD               dwSize;                 		// Structure size

	// Search filter criteria
   	NET_TIME			stStartTime;			       	// start time 
    NET_TIME			stEndTime;				       	// closing time
	char                szMachineAddress[MAX_PATH];    	// Place to support fuzzy matching
	int                 nAlarmType;                    	// To query the type of alarm, see EM_FACERECOGNITION_ALARM_TYPE
    BOOL                abPersonInfo;                   // staff info is valid or not
    FACERECOGNITION_PERSON_INFO stPersonInfo;           // staff info
    int                 nChannelId;                     // channel no.  
    int                 nGroupIdNum;                    // staff group  
    char                szGroupId[MAX_GOURP_NUM][DH_COMMON_STRING_64]; // staff group ID 
}MEDIAFILE_FACERECOGNITION_PARAM;

typedef struct  tagDH_PIC_INFO_EX
{
	DWORD                dwSize;                        // structure size
	DWORD                dwFileLenth;                   // file size,unit:bite
	char                 szFilePath[MAX_PATH];          // file path   
}DH_PIC_INFO_EX;

typedef struct  tagNET_PIC_INFO
{
    DWORD                dwFileLenth;                   // file size,unit:bite
    char                 szFilePath[MAX_PATH];          // file path 
	BYTE                 bReserved[256];                // reserved
}NET_PIC_INFO;

typedef struct __NET_CANDIDAT_PIC_PATHS
{
	DWORD                dwSize;                        	// structure size
	int                  nFileCount;                    	// actual file amount
	DH_PIC_INFO_EX       stFiles[DH_MAX_PERSON_IMAGE_NUM];	// file information
}NET_CANDIDAT_PIC_PATHS;

typedef struct tagNET_CANDIDAT_PIC_PATHS_EX
{
    int                  nFileCount;                      // actual file amount
    NET_PIC_INFO	     stFiles[DH_MAX_PERSON_IMAGE_NUM];// file information
	BYTE                 bReserved[256];				  // reserved
}NET_CANDIDAT_PIC_PATHS_EX;

// corresponding facial recognition service  DH_FILE_QUERY_FACE FINDNEXT search returned parameter
typedef struct __MEDIAFILE_FACERECOGNITION_INFO
{
	DWORD               dwSize;                                   // Structure size
	BOOL                bGlobalScenePic;                          // The existence panorama
	DH_PIC_INFO_EX      stGlobalScenePic;                         // Panoramic image file path
	DH_MSG_OBJECT		stuObject;						          // the target face object information
	DH_PIC_INFO_EX      stObjectPic;                              // the target face file path
	int                 nCandidateNum;                            // Face Matching the current number of candidates
    CANDIDATE_INFO      stuCandidates[DH_MAX_CANDIDATE_NUM];      // Face candidates to match this informatio
	NET_CANDIDAT_PIC_PATHS stuCandidatesPic[DH_MAX_CANDIDATE_NUM];// The current face matching candidates to the image file path
	NET_TIME            stTime;                                   // time for an alarm
	char                szAddress[MAX_PATH];                      // Place for an alarm
    int                 nChannelId;                               // channel no.   
}MEDIAFILE_FACERECOGNITION_INFO; 

typedef enum __EM_FACEPIC_TYPE
{
    NET_FACEPIC_TYPE_UNKOWN,            // unknown type
    NET_FACEPIC_TYPE_GLOBAL_SENCE,      // face full size picture
    NET_FACEPIC_TYPE_SMALL,             // face small pictrure
}EM_FACEPIC_TYPE;

// face detection of glasses 
typedef enum tagEM_FACEDETECT_GLASSES_TYPE
{
    EM_FACEDETECT_GLASSES_UNKNOWN,          // unknown type
    EM_FACEDETECT_WITH_GLASSES,             // wear glasses
    EM_FACEDETECT_WITHOUT_GLASSES,          // without glasses
}EM_FACEDETECT_GLASSES_TYPE;

#define  NET_MAX_FRAMESEQUENCE_NUM     2
#define  NET_MAX_TIMESTAMP_NUM         2

typedef struct __MEDIAFILE_FACE_DETECTION_DETAIL_PARAM
{
    DWORD               dwSize;
    DWORD               dwObjectId;                     // object ID
    DWORD               dwFrameSequence;                // frame no.
    NET_TIME_EX         stTime;                         // time
}MEDIAFILE_FACE_DETECTION_DETAIL_PARAM;
 
// DH_FILE_QUERY_FACE_DETECTION corresponding face recognition service search parameter
typedef struct __MEDIAFILE_FACE_DETECTION_PARAM
{
    DWORD               dwSize;                         // structure size
    
    // search filter
    int                 nChannelID;                     // channel no.
    NET_TIME            stuStartTime;                   // start time
    NET_TIME            stuEndTime;                     // end time
    EM_FACEPIC_TYPE     emPicType;                      // picture type, see  EM_FACEPIC_TYPE
    BOOL                bDetailEnable;                  // if has detailed info
    MEDIAFILE_FACE_DETECTION_DETAIL_PARAM stuDetail;    // parameter detailed info
    EM_DEV_EVENT_FACEDETECT_SEX_TYPE emSex;             // sex
    BOOL                bAgeEnable;                     // age range is enabled
    int                 nAgeRange[2];                   // age range
    int                 nEmotionValidNum;               // invalid number in array emEmotion, 0 means all emotion
    EM_DEV_EVENT_FACEDETECT_FEATURE_TYPE    emEmotion[DH_MAX_FACEDETECT_FEATURE_NUM];   // human emotion
    EM_FACEDETECT_GLASSES_TYPE  emGlasses;              // wear glasses info  
}MEDIAFILE_FACE_DETECTION_PARAM;

// DH_FILE_QUERY_FACE_DETECTIONcorresponding face recognition service FINDNEXT search return parameter
typedef struct __MEDIAFILE_FACE_DETECTION_INFO
{
    DWORD               dwSize;                                     // structure size

    unsigned int        ch;                                         // channel no.
    char                szFilePath[128];                            // file path
    unsigned int        size;                                       // file length
    NET_TIME            starttime;                                  // start time
    NET_TIME            endtime;                                    // end time
    unsigned int        nWorkDirSN;                                 // working directory no.                                    
    BYTE                nFileType;                                  // file type  1:jpg picture
    BYTE                bHint;                                      // file positioning index
    BYTE                bDriveNo;                                   // disk no.
    BYTE                byPictureType;                              // picture type, 0-normal, 1-combined, 2-cut
    unsigned int        nCluster;                                   // cluster no.
    
    EM_FACEPIC_TYPE     emPicType;                                  // picture type, see EM_FACE_PIC_TYPE
    DWORD               dwObjectId;                                 // object ID
    DWORD               dwFrameSequence[NET_MAX_FRAMESEQUENCE_NUM]; // frame no.,group has two elements, one means small picture, two means large picture
    int                 nFrameSequenceNum;                          // frame no.quantity 
    NET_TIME_EX         stTimes[NET_MAX_TIMESTAMP_NUM];             // time, group has two elements, one means small picture, two means large picture
    int                 nTimeStampNum;
    int                 nPicIndex;                                  // Picture number in the cluster
                                                                    // For packaged multiple images in the same cluster,provides indexed mode to position pictures
    EM_DEV_EVENT_FACEDETECT_SEX_TYPE emSex;                         // sex type
    int                 nAge;                                       // age
    EM_DEV_EVENT_FACEDETECT_FEATURE_TYPE    emEmotion;              // human emotion
    EM_FACEDETECT_GLASSES_TYPE  emGlasses;                          // wear glasses info
}MEDIAFILE_FACE_DETECTION_INFO;

//Warning zone detected operation type 
typedef enum tagNET_CROSSREGION_ACTION_INFO
{
    EM_CROSSREGION_ACTION_UNKNOW = 0    , 
    EM_CROSSREGION_ACTION_INSIDE        ,   //In the area
    EM_CROSSREGION_ACTION_CROSS         ,   //Cross area
    EM_CROSSREGION_ACTION_APPEAR        ,   //Appear
    EM_CROSSREGION_ACTION_DISAPPEAR     ,   //Disappear 
}NET_CROSSREGION_ACTION_INFO;

//  DH_FILE_QUERY_IVS_EVENT corresponding face recognition service search parameter
typedef struct __MEDIAFILE_IVS_EVENT_PARAM
{
    DWORD               dwSize;                         // structure size
    int                 nChannelID;                     // channel no.
    NET_TIME            stuStartTime;                   // start time
    NET_TIME            stuEndTime;                     // end time
    int                 nMediaType;                     // file type, 0: any, 1:jpg , 2:dav
    int                 nVideoStream;                   // video stream  0-unknown 1-master 2-sub1 3-sub2 4-sub3 
    int                 nEventLists[MAX_IVS_EVENT_NUM]; // link event list,see event intelligent analysis event type
    int                 nEventCount;                    // event total count
    EM_RECORD_SNAP_FLAG_TYPE emFalgLists[FLAG_TYPE_MAX];// video or snap picture flags
    int                 nFalgCount;                     // flag total
    int                 nRuleType;                      // rule type, see event intelligent analysis event type
    NET_CROSSREGION_ACTION_INFO emAction;               // event action type
}MEDIAFILE_IVS_EVENT_PARAM;

// DH_FILE_QUERY_IVS_EVENT corresponding face recognition service FINDNEXT search return parameter
typedef struct __MEDIAFILE_IVS_EVENT_INFO
{
    DWORD               dwSize;                          // structure size
    int                 nChannelID;                      // channel no.
    NET_TIME            stuStartTime;                    // start time
    NET_TIME            stuEndTime;                      // end time
    char                szFilePath[128];                 // file path
    int                 nMediaType;                      // file type, 0: any, 1:jpg , 2:dav
    unsigned int        nFileLength;                     // file length
    unsigned int        nCluster;                        // cluster no.
    unsigned int        nHint;                           // hint
    unsigned int        nDriveNo;                        // drive no.       
    int                 nVideoStream;                    // video stream  0-unknown 1-master 2-sub1 3-sub2 4-sub3 
    int                 nEventLists[MAX_IVS_EVENT_NUM];  // link event list,see event intelligent analysis event type
    int                 nEventCount;                     // event total count
    EM_RECORD_SNAP_FLAG_TYPE emFalgLists[FLAG_TYPE_MAX]; // video or snap picture flags
    int                 nFalgCount;                      // flag total count
    int                 nRuleType;                       // rule type, see event intelligent analysis event type
    NET_CROSSREGION_ACTION_INFO emAction;                // event action type
}MEDIAFILE_IVS_EVENT_INFO;

// analyse object type
typedef enum tagEM_ANALYSE_OBJECT_TYPE
{
    EM_ANALYSE_OBJECT_TYPE_UNKNOWN,           // unknown 
    EM_ANALYSE_OBJECT_TYPE_HUMAN,             // human 
    EM_ANALYSE_OBJECT_TYPE_VEHICLE,           // vehicle
    EM_ANALYSE_OBJECT_TYPE_FIRE,              // fire
    EM_ANALYSE_OBJECT_TYPE_SMOKE,             // smoke
    EM_ANALYSE_OBJECT_TYPE_PLATE,             // plate             
    EM_ANALYSE_OBJECT_TYPE_HUMANFACE,         // human face
    EM_ANALYSE_OBJECT_TYPE_CONTAINER,         // container
    EM_ANALYSE_OBJECT_TYPE_ANIMAL,            // animal
    EM_ANALYSE_OBJECT_TYPE_TRAFFICLIGHT,      // traffic light
    EM_ANALYSE_OBJECT_TYPE_PASTEPAPER,        // paster paper
    EM_ANALYSE_OBJECT_TYPE_HUMANHEAD,         // human head
    EM_ANALYSE_OBJECT_TYPE_ENTITY,            // entity
}EM_ANALYSE_OBJECT_TYPE;

// DH_FILE_QUERY_ANALYSE_OBJECT corresponding face recognition service search parameter
typedef struct __MEDIAFILE_ANALYSE_OBJECT_PARAM
{
    DWORD               dwSize;                         // structure size
    int                 nChannelID;                     // channel no.
    NET_TIME            stuStartTime;                   // start time
    NET_TIME            stuEndTime;                     // stop time
    int                 nMediaType;                     // file type, 0: any, 1:jpg , 2:dav
    int                 nVideoStream;                   // video stream  0-unknown 1-master 2-sub1 3-sub2 4-sub3 
    int                 nEventLists[MAX_IVS_EVENT_NUM]; // link event list,see event intelligent analysis event type
    int                 nEventCount;                    // event total count
    EM_RECORD_SNAP_FLAG_TYPE emFalgLists[FLAG_TYPE_MAX];// video or snap picture flags
    int                 nFalgCount;                     // flag total count
    EM_ANALYSE_OBJECT_TYPE emObject;                    // object type
}MEDIAFILE_ANALYSE_OBJECT_PARAM;

// DH_FILE_QUERY_ANALYSE_OBJECT corresponding face recognition service FINDNEXT search return parameter
typedef struct __MEDIAFILE_ANALYSE_OBJECT_INFO
{
    DWORD               dwSize;                          // structure size
    int                 nChannelID;                      // channel no.
    NET_TIME            stuStartTime;                    // start time
    NET_TIME            stuEndTime;                      // stop time
    char                szFilePath[128];                 // file path
    int                 nMediaType;                      // file type, 0: any, 1:jpg , 2:dav
    unsigned int        nFileLength;                     // file length
    unsigned int        nCluster;                        // cluster no.
    unsigned int        nHint;                           // hint
    unsigned int        nDriveNo;                        // drive no. 
    int                 nVideoStream;                    // video stream  0-unknown 1-master 2-sub1 3-sub2 4-sub3 
    int                 nEventLists[MAX_IVS_EVENT_NUM];  // link event list,see event intelligent analysis event type
    int                 nEventCount;                     // event total count
    EM_RECORD_SNAP_FLAG_TYPE emFalgLists[FLAG_TYPE_MAX]; // video or snap picture flags
    int                 nFalgCount;                      // flag total count
    EM_ANALYSE_OBJECT_TYPE emObject;                     // object type
}MEDIAFILE_ANALYSE_OBJECT_INFO;

// the param of find the MPT record file(corresponding to DH_FILE_QUERY_MPT_RECORD_FILE)
typedef struct __MEDIAFILE_MPT_RECORD_FILE_PARAM
{
    DWORD               dwSize;                         // struct size

    // the common file info
    int                 nChannelID;                                 // channel id
    NET_TIME            stuStartTime;                               // start time
    NET_TIME            stuEndTime;                                 // end time
    int                 nMediaType;                                 // file type,0:all type, 1:jpg, 2:dav
    int                 nVideoStream;                               // video stream, 0-unknown 1-master 2-sub1 3-sub2 4-sub3

    // the MPT record file info
    char                szPoliceID[DH_COMMON_STRING_32];            // police id,empty means this field is invalid
    int                 nCrimeType;                                 // crime type,zero means this field is invalid
    int                 nCrimeTag;                                  // crime tag,zero means this field is invalid
    char                szSerialNumber[DH_COMMON_STRING_32];        // SN of MPT,empty means this field is invalid   
}MEDIAFILE_MPT_RECORD_FILE_PARAM;

// the MPT record file info(corresponding to DH_FILE_QUERY_MPT_RECORD_FILE)  
typedef struct __MEDIAFILE_MPT_RECORD_FILE_INFO
{
    DWORD               dwSize;                          // struct size
    int                 nChannelID;                      // channel id
    NET_TIME            stuStartTime;                    // start time
    NET_TIME            stuEndTime;                      // end time
    char                szFilePath[DH_COMMON_STRING_128];// file path
    int                 nMediaType;                      // file type,0:all type, 1:jpg, 2:dav
    int                 nVideoStream;                    // video stream, 0-unknown 1-master 2-sub1 3-sub2 4-sub3
    unsigned int        nFileLength;                     // file length
    unsigned int        nCluster;                        // cluster
    unsigned int        nHint;                           // file hit
    unsigned int        nDriveNo;                        // drive number

    int                 nCrimeType;                      // crime type
    int                 nCrimeTag;                       // crime tag
    char                szComment[DH_COMMON_STRING_256]; // comment for mark
}MEDIAFILE_MPT_RECORD_FILE_INFO;

// query video synopsis param
typedef struct __MEDIA_QUERY_VIDEOSYNOPSIS_PARAM
{
	DWORD               dwSize;
	NET_TIME			StartTime;						// start time	
	NET_TIME			EndTime;						// end time
	int					nMediaType;						// file type,0:arbitrariness,1:image,2:record
	int                 nQueryType;                     // query type,1:source video file 2:reduce video file
}MEDIA_QUERY_VIDEOSYNOPSIS_PARAM;

// query video synopsis param
typedef struct __MEDIAFILE_VIDEOSYNOPSIS_INFO
{
    char				szFilePath[128];		// file path
    unsigned int		size;					// file size
    NET_TIME			starttime;				// start time
    NET_TIME			endtime;				// end time
    unsigned int		nWorkDirSN;				// work dir serial number						
	BYTE				nFileType;				// file type  1:jpg
	BYTE                bHint;					// index file location
	BYTE                bDriveNo;               // drive no.
	BYTE                bReserved2;
	unsigned int        nCluster;               // cluster
	BYTE				byPictureType;			// picture type, 0-normal, 1-synthesis, 2-cutout
	BYTE                bReserved[3];           // reserved

	// video source file info
	int                nTaskID;                 // task ID
	char               szCurrentState[DH_MAX_SYNOPSIS_STATE_NAME]; //video synopsis state
	int                nProgress;               // sorresponding state
	int                nObjectNum;              // object number

	// video synopsis file info
	int                nDurationTime;           // duration time (s)
}MEDIAFILE_VIDEOSYNOPSIS_INFO;
// correlate with NET_IN_SNAPSHOT, flash control,array,each element corresponding to a flash light configuration
typedef struct __NET_FLASHCONTROL
{
	DWORD				dwSize;					// struct size
	int					nMode;					// work mode 0-no flash,1-always flash,2-auto flash
}NET_FLASHCONTROL;

// Capture client type
typedef enum tagSNAP_CLIENT_TYPE
{
    SNAP_CLIENT_TYPE_COMMON,               // Corresponding to "Common" type, the default type
    SNAP_CLIENT_TYPE_PARKINGSPACE,         // Correspondence "ParkingSpace" type, parking
}SNAP_CLIENT_TYPE;

#define DH_MAX_USER_DEFINE_INFO        1024

// when nTriggerType==2, client snap info
typedef struct _NET_CLIENT_SNAP_INFO
{
	DWORD				dwSize;					// struct info 
	unsigned int        nFrameSequence;         // frame sequence
	double              PTS;                    // pts,64 bit
	char				szUserDefinedInfo[DH_MAX_USER_DEFINE_INFO];	// String, custom client, "\ 0" at the end
	SNAP_CLIENT_TYPE    emSNAP_CLIENT_TYPE;     // Client Type
	DWORD               dwRetCardNumber;        // card amount
    EVENT_CARD_INFO     stuCardInfo[DH_EVENT_MAX_CARD_NUM];  // card information
}NET_CLIENT_SNAP_INFO;

// CLIENT_TrafficSnapByNetwork's input param
typedef struct __NET_IN_SNAPSHOT
{
	DWORD				dwSize;					// struct size
	int					nTriggerType;			// trigger type	0-unknown 1-zhongmeng net trigger
	int					nLaneID;				// lane id
	int					nGroupID;				// group id
	int					nGrabTimes;				// picture number
	int					nStartPicNum;			// the start picture id
	int					nDirection;				// road direction 0-north 1-east north 2-east 3-east south 4-south 5-west south 6-west 7-west north 8-unknown
	int					nGrabWaitTime;			// pGrabWaitTime group member number
	DWORD*				pGrabWaitTime;			// interval time between two picture, the space is nGrabWaitTime by the user
	int					nLowerSpeedLimit;		// lower speed limit, km/h
	int					nUpperSpeedLimit;		// upper speed limit, km/h
	int					nSpeed;					// speed, km/h
	int					nViolationNo;			// violation number 0-not transgress
												// 1-black shit
												// 2-over speed not over 50% 
												// 3-over speed between 50% and 100% 
												// 4-over speed over 100% 
												// 5-retrograde
												// 6-run red
												// 7-under speed
												// 8-no passing,wrong route
	int					nRedLightTime;			// red light time,s
	int					nFlashControl;			// pFlashControl group member number
	NET_FLASHCONTROL*   pFlashControl;			// flash control,the space is sizeof(NET_FLASHCONTROL)*nFlashControl by the user
	DWORD				dwUser;					// user data
	NET_CLIENT_SNAP_INFO stClientInfo;          // the incoming snap parameter
}NET_IN_SNAPSHOT;

// CLIENT_TrafficSnapByNetwork's output param
typedef struct __NET_OUT_SNAPSHOT
{
	DWORD				dwSize;					// structure size
}NET_OUT_SNAPSHOT;

// interface(CLIENT_TrafficForceLightState)input parameter
typedef struct __NET_IN_FORCELIGHTSTATE
{
	DWORD				dwSize;					// struct size
	unsigned int        nDirection;             // 0 bit:"Straight",1 bit:"TurnLeft",2 bit:"TurnRight",3 bit:"U-Turn"
}NET_IN_FORCELIGHTSTATE;

// interface (CLIENT_TrafficForceLightState)output parameter
typedef struct __NET_OUT_FORCELIGHTSTATE
{
	DWORD				dwSize;					// struct size
}NET_OUT_FORCELIGHTSTATE;
// CLIENT_StartTrafficFluxStat's callback function
typedef int  (CALLBACK *fFluxStatDataCallBack)(LLONG lFluxStatHandle, DWORD dwEventType, void* pEventInfo, BYTE *pBuffer, DWORD dwBufSize, LDWORD dwUser, int nSequence, void *reserved);

// CLIENT_StartTrafficFluxStat's input param
typedef struct __NET_IN_TRAFFICFLUXSTAT
{
	DWORD				dwSize;					// structure size
	fFluxStatDataCallBack		cbData;			// callback function pointer
	LDWORD				dwUser;					// user data
}NET_IN_TRAFFICFLUXSTAT;

// CLIENT_StartTrafficFluxStat's output param
typedef struct __NET_OUT_TRAFFICFLUXSTAT
{
	DWORD				dwSize;					// structure size	
}NET_OUT_TRAFFICFLUXSTAT;

// CLIENT_StartFindFluxStat's input param
typedef struct __NET_IN_TRAFFICSTARTFINDSTAT
{
	DWORD				dwSize;					// structure size
	NET_TIME			stStartTime;			// start time, temporarily  
	NET_TIME			stEndTime;				// end time, temporarily 
	int					nWaittime;				// the time to wait result
}NET_IN_TRAFFICSTARTFINDSTAT;

// CLIENT_StartFindFluxStat's output param
typedef struct __NET_OUT_TRAFFICSTARTFINDSTAT
{
	DWORD				dwSize;					// structure size
	DWORD               dwTotalCount;           // The total amount that matched current search criteria                 
}NET_OUT_TRAFFICSTARTFINDSTAT;

// CLIENT_DoFindFluxStat's input param
typedef struct __NET_IN_TRAFFICDOFINDSTAT
{
	DWORD				dwSize;					// structure size
	unsigned int		nCount;					// the number of flow Statistic for query
	int					nWaittime;				// the time to wait result
}NET_IN_TRAFFICDOFINDSTAT;

typedef struct
{
	DWORD				dwSize;					// structure size

	int					nStatInfo;				// the number of statistic info
	DH_TRAFFICFLOWSTAT *pStatInfo;				// the statistic pointer, the space application by the user, length is sizeof(DH_TRAFFICFLOWSTAT)*nStatInfo
}DH_TRAFFICFLOWSTAT_OUT;

// CLIENT_DoFindFluxStat's out param
typedef struct __NET_OUT_TRAFFICDOFINDSTAT
{
	DWORD				dwSize;					// structure size
	
	DH_TRAFFICFLOWSTAT_OUT stStatInfo;			// the statistic pointer
}NET_OUT_TRAFFICDOFINDSTAT;

// interface(CLIENT_StartFindNumberStat)'s input param
typedef struct __NET_IN_FINDNUMBERSTAT
{
	DWORD				dwSize;					// size
	int                 nChannelID;             // channel ID
	NET_TIME			stStartTime;			// start time
	NET_TIME			stEndTime;				// end time
	int                 nGranularityType;       // granularity type, 0:minute,1:hour,2:day,3:week,4:month,5:quarter,6:year
	int					nWaittime;				// wait time    
    UINT                nPlanID;                // Plan ID,Speed Dome use,start from 1
}NET_IN_FINDNUMBERSTAT;

// CLIENT_StartFindNumberStat's output param
typedef struct __NET_OUT_FINDNUMBERSTAT
{
	DWORD				dwSize;					 
	DWORD               dwTotalCount;           // total count
}NET_OUT_FINDNUMBERSTAT;

// CLIENT_DoFindNumberStat's input param
typedef struct __NET_IN_DOFINDNUMBERSTAT
{
	DWORD				dwSize;					 
	unsigned int        nBeginNumber;           // [0, totalCount-1]
	unsigned int		nCount;					// count
	int					nWaittime;				// wait time
}NET_IN_DOFINDNUMBERSTAT;

typedef struct __DH_NUMBERSTAT
{
	DWORD    dwSize;
	int      nChannelID;                           // channel id
	char     szRuleName[DH_CHAN_NAME_LEN];         // rule name
	NET_TIME stuStartTime;                         // start time
	NET_TIME stuEndTime;                           // end time
    int      nEnteredSubTotal;                     // entered total
	int      nExitedSubtotal;                      // entered total
    int      nAvgInside;                           // average number inside
	int      nMaxInside;                           // max number inside
    int      nEnteredWithHelmet;                   //people enter with helmet count
    int      nEnteredWithoutHelmet;                //people enter without helmet count
    int      nExitedWithHelmet;                    //people exit with helmet count
    int      nExitedWithoutHelmet;                 //people exit without helmet count
}DH_NUMBERSTAT;

// CLIENT_DoFindNumberStat's ouput param
typedef struct __NET_OUT_DOFINDNUMBERSTAT
{
	DWORD				dwSize;					             
	int                 nCount;                              // count
	DH_NUMBERSTAT       *pstuNumberStat;                     // state array, the space application by the user
    int                 nBufferLen;                          // the space application yb the user, the length unit is the dwsize of DH_NUMBERSTAT
}NET_OUT_DOFINDNUMBERSTAT;

// video statistical subtotal
typedef struct tagNET_VIDEOSTAT_SUBTOTAL 
{
    int                 nTotal;                         // count since device operation
    int                 nHour;                          // count in the last hour
    int                 nToday;                         // count for today
    int                 nOSD;                           // count for today, on screen display 
    char                reserved[252];
} NET_VIDEOSTAT_SUBTOTAL;

// video statistical summary
typedef struct tagNET_VIDEOSTAT_SUMMARY
{
    int                     nChannelID;                 // channel ID
    char                    szRuleName[32];             // rule name
    NET_TIME_EX             stuTime;                    // time of this statistics
    NET_VIDEOSTAT_SUBTOTAL  stuEnteredSubtotal;         // subtotal for the entered
    NET_VIDEOSTAT_SUBTOTAL  stuExitedSubtotal;          // subtotal for the exited
    char                    reserved[512];
} NET_VIDEOSTAT_SUMMARY;

// video statistical summary callback function type, lAttachHandle is the return value of CLIENT_AttachVideoStatSummary
typedef void (CALLBACK *fVideoStatSumCallBack) (LLONG lAttachHandle, NET_VIDEOSTAT_SUMMARY* pBuf, DWORD dwBufLen, LDWORD dwUser);

// input param for CLIENT_AttachVideoStatSummary
typedef struct tagNET_IN_ATTACH_VIDEOSTAT_SUM
{
    DWORD                   dwSize;
    int                     nChannel;                    // video channel ID       
    fVideoStatSumCallBack   cbVideoStatSum;              // video statistical summary callback
    LDWORD                  dwUser;						 // user data             
} NET_IN_ATTACH_VIDEOSTAT_SUM;

// output param for CLIENT_AttachVideoStatSummary
typedef struct tagNET_OUT_ATTACH_VIDEOSTAT_SUM
{
    DWORD                   dwSize;
} NET_OUT_ATTACH_VIDEOSTAT_SUM;

//// intelligent traffic detector

// CLIENT_GetParkingSpaceStatus's input param
typedef struct tagNET_IN_GET_PARKINGSPACE_STATUS
{
	DWORD                dwSize;                         // struct size
	DWORD                dwWaitTime;                     // wait time
	int                  nChannelID;                     // channel ID
	int *                pLaneID;                        // range[0,255], pLaneID==NULL means all parking space, the max number is DH_PRODUCTION_DEFNITION.nMaxRoadWays
	int                  nLaneCount;                     // apply to sizeof(int)*nLaneCount memory
} NET_IN_GET_PARKINGSPACE_STATUS;

typedef struct tagNET_LANE_PARKINGSPACE_STATUS
{
	DWORD                dwSize;                         // struct size
	int                  nLaneID;                        // lane ID
	unsigned int         nPictureId;                     // picture ID,get picture data
	DEV_EVENT_TRAFFIC_TRAFFICCAR_INFO stTrafficCar;      // traffic car info
} NET_LANE_PARKINGSPACE_STATUS;

// CLIENT_GetParkingSpaceStatus's output param
typedef struct tagNET_OUT_GET_PARKINGSPACE_STATUS
{
	DWORD                dwSize;                         // struct size
	NET_LANE_PARKINGSPACE_STATUS* pStatus;               // status, the space application by the user
	int                  nMaxStatus;                     // need apply to sizeof(NET_LANE_PARKINGSPACE_STATUS)*nMaxStatus, and nMaxStatus == DH_PRODUCTION_DEFNITION.nMaxRoadWays memory
	int                  nRetStatus;                     // actual returns status number of parking spaces
} NET_OUT_GET_PARKINGSPACE_STATUS;

// CLIENT_AttachParkingSpaceData's input param
typedef struct tagNET_CB_PARKINGSPACE_DATA
{
	DWORD                dwSize;                         // struct size
	void*                pPicBuffer;                     // image binary data
	unsigned int         nPicLength;                     // picture length
} NET_CB_PARKINGSPACE_DATA;

// CLIENT_AttachParkingSpaceData callback function , pBuf is json and image data info , nBufLen is pBuf length,for forwarding services
typedef int (CALLBACK *fNotifySnapData)(LLONG lParkingHandle, NET_CB_PARKINGSPACE_DATA* pDiagnosisInfo, void* pBuf, int nBufLen, LDWORD dwUser);

// CLIENT_AttachParkingSpaceData's input param
typedef struct tagNET_IN_ATTACH_PARKINGSPACE
{
	DWORD                dwSize;                         // struct size
	DWORD                dwWaitTime;                     // wait time
	int                  nChannelID;                     // channel ID
	unsigned int         nPictureId;                     // picture ID, can look for NET_LANE_PARKINGSPACE_STATUS.nPictureId
	
	fNotifySnapData      cbNotifySnapData;               // callback function
	LDWORD               dwUser;                         // user-defined parameters
} NET_IN_ATTACH_PARKINGSPACE;

// fingerprint data
typedef struct tagNET_CB_FINGERPRINT_INFO
{
	UINT 		nIndex;                                     // fingerprint index,range 0 -9,count form left pinkty to right pinky
	char        szFingerprintData[DH_COMMON_STRING_2048];   // fingerprint data,base64 string           
	BYTE        bReserved[256];                             // reserved  bytes
}NET_CB_FINGERPRINT_INFO;

// person info collect info
typedef struct tagNET_CB_PERSON_INFO
{
    UINT                 nSID;                           // SID
	UINT 	             nPicLength;                     // person picture length
	char                 szUserID[DH_MAX_USERID_LEN];    // user id
    UINT                 nRetFaceDataNum;                // return face detection data number 
    char                 szFaceData[MAX_FACE_DATA_NUM][MAX_FACE_DATA_LEN];  // person face detection data    
	char                 szPassword[DH_COMMON_STRING_64];//card password
    char                 szCardNo[DH_MAX_CARDNO_LEN];    // card no
    char                 szCardName[DH_COMMON_STRING_32];// card name
    UINT                 nFingerPrintNum;                // return fingerprint num
    NET_CB_FINGERPRINT_INFO stuFingerPrintInfo[MAX_FINGERPRINT_NUM];//fingerprint info
	BYTE                 bReserved[2048];                // reserved  bytes
}NET_CB_PERSON_INFO;

// callback function of person info collect 
typedef int (CALLBACK *fNotifyPersonInfo)(LLONG lLoginID, LLONG lPersonHandle, NET_CB_PERSON_INFO *pstuPersonInfo, BYTE *pBuffer, DWORD dwBufSize, LDWORD dwUser, void *reserved);

// CLIENT_AttachPersonInfoCollect  in param
typedef struct tagNET_IN_ATTACH_PERSONINFOCOLLECT
{
	DWORD 				dwSize;                         // struct size
	DWORD               dwWaitTime;                     // wait time
	fNotifyPersonInfo   cbNotifyPersonInfo;             // callback function
	LDWORD              dwUser;                         // user-defined parameters
}NET_IN_ATTACH_PERSONINFOCOLLECT;

//CLIENT_AttachPersonInfoCollect out param
typedef struct tagNET_OUT_ATTACH_PERSONINFOCOLLECT
{
	DWORD               dwSize;                         // struct size
}NET_OUT_ATTACH_PERSONINFOCOLLECT;

// CLIENT_DetachPersonInfoColloect in param
typedef struct tagNET_IN_DETACH_PERSONINFOCOLLECT
{
	DWORD 				dwSize;                        //struct size
	DWORD               dwWaitTime;                    // wait time 
	LLONG               lPersonHandle;                 // CLIENT_AttachPersonInfoCollect return handle
}NET_IN_DETACH_PERSONINFOCOLLECT;

//CLIENT_DetachPersonInfoColloect out param
typedef struct tagNET_OUT_DETACH_PERSONINFOCOLLECT
{
	DWORD               dwSize;                        // struct size
}NET_OUT_DETACH_PERSONINFOCOLLECT;

/******************************************************************************
function	:attach person info collect
return :	return attach handle,use to detach opearator
******************************************************************************/
CLIENT_NET_API LLONG CALL_METHOD CLIENT_AttachPersonInfoCollect(LLONG lLoginID, NET_IN_ATTACH_PERSONINFOCOLLECT *pstuInParam, NET_OUT_ATTACH_PERSONINFOCOLLECT *pstuOutParam);

/******************************************************************************
function	:detach person info collect
pstuInParam's lPersonHandle is returned by CLIENT_AttachPersonInfoCollect
******************************************************************************/
CLIENT_NET_API BOOL CALL_METHOD CLIENT_DetachPersonInfoColloect(NET_IN_DETACH_PERSONINFOCOLLECT* pstuInParam, NET_OUT_DETACH_PERSONINFOCOLLECT* pstuOutParam);

//NET_CAR_PASS_INFOS related enumerated type
typedef enum tagNET_CAR_SIZE
{
    CAR_SIZE_UNKNOWN  = 0 ,     //unknown
    CAR_SIZE_SAMLL        ,     //small car
    CAR_SIZE_BIG          ,     //big car 
}NET_CAR_SIZE;

typedef enum tagNET_CAR_ACTION
{
    CAR_ACTION_UNKNOWN = 0 , 
    CAR_ACTION_ENTER       ,  //Enter the detection area
    CAR_ACTION_LEAVE       ,  //Leave the detection area
}NET_CAR_ACTION;


//CLIENT_AttachCarPassInfo callback function returns the structure of the car pass information typedef struct tagNET_CAR_PASS_INFO
typedef struct tagNET_CAR_PASS_INFO
{
     DWORD                dwSize; 
       
     DWORD                nID;                              // Car ID
     DWORD                nLaneNumber;                      // Start from0,for example, if there is 3 lane, the number of the lane is 0 1 2 respectively
     DWORD                nRoadwayNumber;                   // Custom lane number, User configurable lane number, the current value in the range of 1-16
     DWORD                nSpeed;                           // Car speed (km/h)
     NET_CAR_SIZE         emCarSize;                        // Car size type
     NET_CAR_ACTION       emCarAction;                      // Car action type
     NET_TIME_EX          stuTime;                          // Action happens time,accurate to seconds
}NET_CAR_PASS_INFO;

// Port CLIENT_AttachCarPassInfo callback function
// nInfoNum the number of car pass information
// pstuCarPassInfos the array of car pass information number
typedef int (CALLBACK *fNotifyCarPassInfo)(LLONG lCarPassHandle, NET_CAR_PASS_INFO* pstuCarPassInfos,  int nInfoNum ,   LDWORD dwUser);

// Port CLIENT_AttachCarPassInfo input parameter
typedef struct tagNET_IN_ATTACH_CAR_PASS_INFO
{
    DWORD                dwSize;                         // he size of the structure is too small,should be assigned
    int                  nChannelID;                     // Channel ID

    fNotifyCarPassInfo   cbNotifyCarPassInfo;            // Callback function, when thereis a car information,a callback to the top
    LDWORD               dwUser;                         // User-defined parameters
}NET_IN_ATTACH_CAR_PASS_INFO;

//Port CLIENT_AttachCarPassInfo output parameter
typedef struct tagNET_OUT_ATTACH_CAR_PASS_INFO
{
    DWORD       dwSize;         // The size of the structure is too small,should be assigned
}NET_OUT_ATTACH_CAR_PASS_INFO;

// CLIENT_AttachParkingSpaceData's output param
typedef struct tagNET_OUT_ATTACH_PARKINGSPACE
{
	DWORD                dwSize;                         // struct size
} NET_OUT_ATTACH_PARKINGSPACE;

// CLIENT_DetachParkingSpaceData's input param
typedef struct tagNET_IN_DETACH_PARKINGSPACE
{
	DWORD                dwSize;                         // struct size
	DWORD                dwWaitTime;                     // wait time
	LLONG                lParkingHandle;                 // CLIENT_AttachParkingSpaceData return handle
} NET_IN_DETACH_PARKINGSPACE;
// CLIENT_DetachParkingSpaceData's output param
typedef struct tagNET_OUT_DETACH_PARKINGSPACE
{
	DWORD                dwSize;                         // struct size
} NET_OUT_DETACH_PARKINGSPACE;

// Batch download file

// Download progress callback function prototypes,  nError Indicates that an error occurred during the download of, 1-insufficient buffer, 2-parity errof of return data, 3-failed to download current file, 4-failed to create corresponding storage file
// When download according to the file,ID setting by the User is dwFileID, When download according to the conditions,ID setting by the User is dwConditionID                    
typedef void (CALLBACK *fMultiFileDownLoadPosCB)(LLONG lDownLoadHandle, DWORD dwID, DWORD dwFileTotalSize, DWORD dwDownLoadSize, int nError, LDWORD dwUser, void* pReserved);

//Ways to download file
typedef enum tagEM_DOWNLOAD_FILE_TYPE
{
   EM_DOWNLOAD_BY_FILENAME = 0      ,       //Download according to file name
   EM_DOWNLOAD_BY_CONDITION         ,       //Download according to query conditions
}EM_DOWNLOAD_FILE_TYPE;

//Download time type
typedef enum tagEM_DOWNLOAD_TIME_TYPE
{
    EM_DOWNLOAD_TIME_SNAP = 0       ,         //Snap time
    EM_DOWNLOAD_TIME_STORAGE        ,         //Storage time
}EM_DOWNLOAD_TIME_TYPE;

//Download data type
typedef enum tagEM_DOWNLOAD_DATA_TYPE
{
    EM_DOWNLOAD_DATA_RECORD = 0 ,               //Record
    EM_DOWNLOAD_DATA_PICTURE ,                  //Picture
}EM_DOWNLOAD_DATA_TYPE;

// CLIENT_DownLoadMultiFile  port parameter
typedef struct tagNET_DOWNLOADFILE_INFO
{
	DWORD               dwSize;
	DWORD               dwFileID;                       // Document ID, assigned by the user
	int                 nFileSize;                      // Query to the file size
	char                szSourceFilePath[MAX_PATH];     // Query to the file path
	char                szSavedFileName[MAX_PATH];      // Save the file path
}NET_DOWNLOADFILE_INFO;

typedef struct tagNET_DOWNLOAD_CONDITION_INFO
{
    DWORD                   dwSize;
    DWORD                   dwConditionID;                  // Condition ID, assign by the User
    int                     nChannel;                       // Channel ID
    NET_TIME                stuStartTime;                   // Start time
    NET_TIME                stuEndTime;                     // End time
    EM_DOWNLOAD_TIME_TYPE   emTimeType;                     // Time type
    EM_DOWNLOAD_DATA_TYPE   emDataType;                     // Data type 
    char                    szSavedFileName[MAX_PATH];      //Save the file path
}NET_DOWNLOAD_CONDITION_INFO;

typedef struct tagNET_IN_DOWNLOAD_MULTI_FILE
{
	DWORD               dwSize;
    // Download according to file name
    int                 nFileCount;                     // The number of files to be downloaded
	NET_DOWNLOADFILE_INFO* pFileInfos;                  // File information to be downloaded
														// the space application by the user,apply to sizeof(NET_DOWNLOADFILE_INFO)*nFileCount   
	fMultiFileDownLoadPosCB cbPosCallBack;              // Progress callback function
	LDWORD              dwUserData;                     // User data
    EM_DOWNLOAD_FILE_TYPE           emDownloadType;     // Download ways
    //Download according to query conditions
    int                             nConditionCount;    // Number of download conditions,emDownloadType is EM_DOWNLOAD_BY_CONDITION valid
    NET_DOWNLOAD_CONDITION_INFO*    pConditionInfos;    // Download conditions,emDownloadType is EM_DOWNLOAD_BY_CONDITION valid      
														// the space application by the user,apply to sizeof(NET_DOWNLOAD_CONDITION_INFO)*nConditionCount 
}NET_IN_DOWNLOAD_MULTI_FILE;

typedef struct tagNET_OUT_DOWNLOAD_MULTI_FILE
{
	DWORD               dwSize;
	LLONG               lDownLoadHandle;                // Download the handle
}NET_OUT_DOWNLOAD_MULTI_FILE;

typedef struct __NET_VIDEOANALYSE_STATE
{
	DWORD              dwSize;
	DWORD              dwProgress;                   		// Analysis of progress, 0-100
	char               szState[DH_COMMON_STRING_64]; 		// Channel Status,Running":Run, "Stop":Stop, "NoStart":Not started, "Failed":failed , "Successed":successed
	char               szFailedCode[DH_COMMON_STRING_64]; 	// Error code
}NET_VIDEOANALYSE_STATE;

//Real-time video analysis progress status callbacks
typedef int (CALLBACK *fVideoAnalyseState)(LLONG lAttachHandle, NET_VIDEOANALYSE_STATE* pAnalyseStateInfos, LDWORD dwUser, void* pReserved);

// CLIENT_AttachVideoAnalyseState  Interface input parameters
typedef struct __NET_IN_ATTACH_VIDEOANALYSE_STATE 
{
	DWORD              dwSize;
	int                nChannleId;            // Channel number
	fVideoAnalyseState cbVideoAnalyseState;   // Video analysis status callback function
	LDWORD             dwUser;                // User Information
}NET_IN_ATTACH_VIDEOANALYSE_STATE;

// CLIENT_AttachVideoAnalyseState  Interface output parameters
typedef struct __NET_OUT_ATTACH_VIDEOANALYSE_STATE 
{
	DWORD              dwSize;
	LLONG              lAttachHandle;         // Analysis of the progress of the analysis progress handle that uniquely identifies a particular channel
}NET_OUT_ATTACH_VIDEOANALYSE_STATE;

// Bidirectional talk status 
typedef enum tagEM_TALK_STATE
{
    EM_TALK_STATE_UNKNOWN,                 // Unknown 
    EM_TALK_STATE_INVITING,                // Caller 
    EM_TALK_STATE_RINGING,                 // Callee
    EM_TALK_STATE_ANSWER,                  // Answer
    EM_TALK_STATE_REFUSE,                  // Refuse
    EM_TALK_STATE_HANGUP,                  // Hang up 
    EM_TALK_STATE_BUSYING,                 // Busy
    EM_TALK_STATE_CANCEL,                  // Cancel calling 
} EM_TALK_STATE;

#define NET_CALL_NUM_MAX           128         // Max communication No. amount 

// Bidirectional talk status of call function CLIENT_AttachTalkState
typedef struct tagNET_TALK_STATE
{
    DWORD                dwSize; 
    char                 szCallID[DH_COMMON_STRING_64];  // Call unique symbole
    int                  nAudioPort;                     // Audio port 
    int                  nVideoPort;                     // Video port 
	char                 szMediaAddr[DH_MAX_IPADDR_LEN]; // Getting stream address 
    EM_TALK_STATE        emState;                        // Status 
    int                  nNumberCount;                   // Call No. amount 
    char                 szNumbers[NET_CALL_NUM_MAX][DH_COMMON_STRING_32]; // Call No. list 
} NET_TALK_STATE;

// Call function of  CLIENT_AttachTalkState 
typedef void (CALLBACK *fNotifyTalkState)(LLONG lAttachHandle, NET_TALK_STATE* pstuState, int nLen, LDWORD dwUser);

// Input parameters of CLIENT_AttachTalkState 
typedef struct tagNET_IN_ATTACH_TALK_STATE
{
    DWORD                dwSize;                         // Structure size, must have a value 
    int                  nChnId;                         // Channel No. It is to mark bidirectional talk unit. 
    fNotifyTalkState     cbCallBack;                     // Call function. Call the upper-level when there is bidirectional talk in process. 
    LDWORD               dwUser;                         // Customized paramters 
}NET_IN_ATTACH_TALK_STATE;

// Output parameters of the CLIENT_AttachTalkState
typedef struct tagNET_OUT_ATTACH_TALK_STATE
{
    DWORD       dwSize;                                  // Structure size. Must have a value
    char        szCallID[DH_COMMON_STRING_64];           // Call unique mark 
}NET_OUT_ATTACH_TALK_STATE;

///////////////////////////////// IVS server video analysis module /////////////////////////////////
// Video analysis  report result detect type definition
#define NET_DIAGNOSIS_DITHER                    "VideoDitherDetection"                       // Video vibration detect  Corresponding structure body(NET_VIDEO_DITHER_DETECTIONRESULT)
#define NET_DIAGNOSIS_STRIATION                 "VideoStriationDetection"                    // Video stria detect  Corresponding structure body(NET_VIDEO_STRIATION_DETECTIONRESULT)
#define NET_DIAGNOSIS_LOSS                      "VideoLossDetection"                         // Video loss detect  Corresponding structure body(NET_VIDEO_LOSS_DETECTIONRESULT)
#define NET_DIAGNOSIS_COVER                     "VideoCoverDetection"                        // Camera masking detect Corresponding structure body(NET_VIDEO_COVER_DETECTIONRESULT)
#define NET_DIAGNOSIS_FROZEN                    "VideoFrozenDetection"                       // Video freeze detect Corresponding structure body(NET_VIDEO_FROZEN_DETECTIONRESULT)
#define NET_DIAGNOSIS_BRIGHTNESS                "VideoBrightnessDetection"                   // Video brightness abnormal detect Corresponding structure body(NET_VIDEO_BRIGHTNESS_DETECTIONRESULT)
#define NET_DIAGNOSIS_CONTRAST                  "VideoContrastDetection"                     // Video contrast abnormal detect  Corresponding structure body(NET_VIDEO_CONTRAST_DETECTIONRESULT)
#define NET_DIAGNOSIS_UNBALANCE                 "VideoUnbalanceDetection"                    // Video color cast detect Corresponding structure body(NET_VIDEO_UNBALANCE_DETECTIONRESULT)
#define NET_DIAGNOSIS_NOISE                     "VideoNoiseDetection"                        // Video noise detect Corresponding structure body(NET_VIDEO_NOISE_DETECTIONRESULT)
#define NET_DIAGNOSIS_BLUR                      "VideoBlurDetection"                         // Video blur detect Corresponding structure body(NET_VIDEO_BLUR_DETECTIONRESULT)
#define NET_DIAGNOSIS_SCENECHANGE               "VideoSceneChangeDetection"                  // Video scene change detect Corresponding structure body(NET_VIDEO_SCENECHANGE_DETECTIONRESULT)
#define NET_DIAGNOSIS_VIDEO_DELAY               "VideoDelay"                                 // Video Delay, Corresponding structure body(NET_VIDEO_DELAY_DETECTIONRESUL)
#define NET_DIAGNOSIS_PTZ_MOVING                "PTZMoving"                                  // PTZ Moving, Corresponding structure body(NET_PTZ_MOVING_DETECTIONRESULT)
#define NET_DIAGNOSIS_BLACK_WHITE				"VideoBlackAndWhite"						 // black and white image detection, Corresponding structure body(NET_BLACK_WHITE_DETECTIONRESULT)
#define NET_DIAGNOSIS_DRAMATIC_CHANGE			"VideoDramaticChange"						 // dramatic change of scene detection, Corresponding structure body(NET_DIAGNOSIS_DRAMATIC_DETECTIONRESULT)

typedef enum tagNET_STATE_TYPE
{
	NET_EM_STATE_ERR,        // Others
	NET_EM_STATE_NORMAL,     // "Normal"  
	NET_EM_STATE_WARNING,    // "Warning" 
	NET_EM_STATE_ABNORMAL,   // "Abnormal" 
}NET_STATE_TYPE;
// video stream type
typedef enum tagNET_STREAM_TYPE
{
	NET_EM_STREAM_ERR,                  // Others
	NET_EM_STREAM_MAIN,					// "Main"-Main stream
	NET_EM_STREAM_EXTRA_1,				// "Extra1"-Extra stream 1
	NET_EM_STREAM_EXTRA_2,				// "Extra2"-Extra stream 2
	NET_EM_STREAM_EXTRA_3,				// "Extra3"-Extra stream 3
	NET_EM_STREAM_SNAPSHOT,				// "Snapshot"-Snap bit stream
	NET_EM_STREAM_OBJECT,				// "Object"-Object stream
	NET_EM_STREAM_AUTO,                 // "Auto"
	NET_EM_STREAM_PREVIEW,              // "Preview"
	NET_EM_STREAM_NONE,					// No video stream (audio only)
}NET_STREAM_TYPE;
// Video diagnosis type
typedef enum tagNET_VIDEODIAGNOSIS_RESULT_TYPE
{
	NET_EM_ROTATION   ,					// "Rotation"	-Video analysis of polling
	NET_EM_REAL		  ,					// "Real" -Real-time video analysis
	NET_EM_NR_UNKNOW  ,					// Undefined
}NET_VIDEODIAGNOSIS_RESULT_TYPE;
//Video causes of diagnostic error
typedef enum tagNET_VIDEODIAGNOSIS_FAIL_TYPE
{
	NET_EM_NO_ERROR				 ,			// Diagnostic success
	NET_EM_DISCONNECT			 ,			// "Disconnect"				- End devices can be connected
	NET_EM_CH_NOT_EXIST			 ,			// "ChannelNotExist"		- Channel does not exist
	NET_EM_LOGIN_OVER_TIME		 ,			// "LoginOverTime"			- Login Timeout
	NET_EM_NO_VIDEO				 ,			// "NoVideo"				- No video successful login
	NET_EM_NO_RIGHT				 ,			// "NoRight"				- No operating authority
	NET_EM_PLATFROM_LOGIN_FAILED ,			// "PlatformLoginFailed"	- Login failed platform
	NET_EM_PLATFROM_DISCONNECT 	 ,			// "PlatformDisconnect"		- Disconnect platform
	NET_EM_GET_STREAM_OVER_TIME  ,			// "GetStreamOverTime"		- Get stream timeout
    NET_EM_GET_NO_ENOUGH_STREAM  ,          // "NoEnoughStream"         - no enought stream
    NET_EM_DECODE_STREAM_FAILED  ,          // "DecodeStreamFailed"     - decode stream failed
    NET_EM_GET_OFF_LINE          ,          // "OffLine"                - device offline
	NET_EM_NF_UNKNOW			 ,			// Other reasons, as detailed in the structure described in the reason for the failure 
    NET_EM_NOT_SD                ,          // "NotSD"                  - not SD device, PTZ detect invalid
}NET_VIDEODIAGNOSIS_FAIL_TYPE;

// General long character ended with '\0'
typedef struct tagNET_ARRAY
{
	DWORD                dwSize;                         // Current structure body size 
	char*                pArray;                         // Buffer zone. Now the min value is 260 byte.Caller shall apply for the memory. The filling in data shall ended with '\0'.
	DWORD                dwArrayLen;                     // Buffer space length
}NET_ARRAY;

#define MAX_BACKPIC_COUNT	8	// max count of background pictures

// Video analysis result report general data
typedef struct tagNET_VIDEODIAGNOSIS_COMMON_INFO
{
	DWORD                dwSize;                         // Current structure body size 
	int                  nDiagnosisID;                   // Detect channel
	NET_ARRAY            stProject;                      // Project name
	NET_ARRAY            stTask;                         // Task name
	NET_ARRAY            stProfile;                      // Parameter list name
	NET_ARRAY            stDeviceID;                     // Device unique ID
	NET_TIME             stStartTime;                    // Start time
	NET_TIME             stEndTime;                      // End time
	int                  nVideoChannelID;                // Video channel No. The channel of the front-end device such as DVR,IPC.
	NET_STREAM_TYPE      emVideoStream;                  // Video bit stream
	NET_VIDEODIAGNOSIS_RESULT_TYPE	emResultType;					// Diagnosis type
	BOOL							bCollectivityState;             // Diagnostic results
	NET_VIDEODIAGNOSIS_FAIL_TYPE	emFailedCause;					// Reasons for failure
	char                            szFailedCode[DH_COMMON_STRING_64]; 		// Describe the reason for the failure
    char                            szResultAddress[DH_COMMON_STRING_128];  // Diagnosis result storage address
    int                             nFrameRate;                             // Frame Rate(kb/s)  update by day
    int                             nFrameWidth;                            // Frame Width       update by day
    int                             nFrameHeight;                           // Frame Height      update by day
	int								nBackPic;								// counts of background pictures
	char							szBackPicAddress[MAX_BACKPIC_COUNT][DH_COMMON_STRING_128];	// the address of these background pictures
}NET_VIDEODIAGNOSIS_COMMON_INFO;

// The result of detect type (NET_DIAGNOSIS_DITHER)  Video vibration detect -- Video change,wind or vibration,rotation including the PTZ movement.
typedef struct tagNET_VIDEO_DITHER_DETECTIONRESULT
{
	DWORD                dwSize;                         // Current structure body size 
	int                  nValue;                         // Detect result value 
	NET_STATE_TYPE       emState;                        // Detect result status  Usually smaller than is normal. Larger than is abnormal and the value in the middle is warning. 
	int                  nDuration;                      // Status lasts time  Detect item last time. It is null right now. 
}NET_VIDEO_DITHER_DETECTIONRESULT;

// The result of detect type (NET_DIAGNOSIS_STRIATION) Video stria detect  -- There is abnormal stria on the camera resulting from the interference.
typedef struct tagNET_VIDEO_STRIATION_DETECTIONRESULT
{
	DWORD                dwSize;                         // Current structure body size 
	int                  nValue;                         // Detect result value 
	NET_STATE_TYPE       emState;                        // Detect result status
	int                  nDuration;                      // Status lasts time
}NET_VIDEO_STRIATION_DETECTIONRESULT;

// The result of detect type (NET_DIAGNOSIS_LOSS) Video loss detect  -- Result from power failure or disconnection.
typedef struct tagNET_VIDEO_LOSS_DETECTIONRESULT
{
	DWORD                dwSize;                         // Current structure body size 
	NET_STATE_TYPE       emState;                        // Detect result status
	int                  nDuration;                      // Status lasts time
}NET_VIDEO_LOSS_DETECTIONRESULT;

// The result of detect type (NET_DIAGNOSIS_COVER) Camera masking detect -- The camera masking occurred
typedef struct tagNET_VIDEO_COVER_DETECTIONRESULT
{
	DWORD                dwSize;                         // Current structure body size 
	int                  nValue;                         // Detect result value 
	NET_STATE_TYPE       emState;                        // Detect result status
	int                  nDuration;                      // Status lasts time
}NET_VIDEO_COVER_DETECTIONRESULT;

// The result of detect type (NET_DIAGNOSIS_FROZEN) Video freeze detect -- The video idles for specified time is regarded as freeze.
typedef struct tagNET_VIDEO_FROZEN_DETECTIONRESULT
{
	DWORD                dwSize;                         // Current structure body size 
	NET_STATE_TYPE       emState;                        // Detect result status
	int                  nDuration;                      // Status lasts time
}NET_VIDEO_FROZEN_DETECTIONRESULT;

// The result of detect type (NET_DIAGNOSIS_BRIGHTNESS) Video brightness abnormal detect --The following contents are some camera improper setup detect.
typedef struct tagNET_VIDEO_BRIGHTNESS_DETECTIONRESULT
{
	DWORD                dwSize;                         // Current structure body size 
	int                  nValue;                         // Detect result value 
	NET_STATE_TYPE       emState;                        // Detect result status
	int                  nDuration;                      // Status lasts time
}NET_VIDEO_BRIGHTNESS_DETECTIONRESULT;

// The result of detect type (NET_DIAGNOSIS_CONTRAST) Video contrast abnormal detect
typedef struct tagNET_VIDEO_CONTRAST_DETECTIONRESULT
{
	DWORD                dwSize;                         // Current structure body size 
	int                  nValue;                         // Detect result value 
	NET_STATE_TYPE       emState;                        // Detect result status
	int                  nDuration;                      // Status lasts time
}NET_VIDEO_CONTRAST_DETECTIONRESULT;

// The result of detect type (NET_DIAGNOSIS_UNBALANCE) Video color cast detect
typedef struct tagNET_VIDEO_UNBALANCE_DETECTIONRESULT
{
	DWORD                dwSize;                         // Current structure body size 
	int                  nValue;                         // Detect result value 
	NET_STATE_TYPE       emState;                        // Detect result status
	int                  nDuration;                      // Status lasts time
}NET_VIDEO_UNBALANCE_DETECTIONRESULT;

// The result of detect type (NET_DIAGNOSIS_NOISE) Video noise detect
typedef struct tagNET_VIDEO_NOISE_DETECTIONRESULT
{
	DWORD                dwSize;                         // Current structure body size 
	int                  nValue;                         // Detect result value 
	NET_STATE_TYPE       emState;                        // Detect result status
	int                  nDuration;                      // Status lasts time
}NET_VIDEO_NOISE_DETECTIONRESULT;

// The result of detect type (NET_DIAGNOSIS_BLUR) Video blur detect
typedef struct tagNET_VIDEO_BLUR_DETECTIONRESULT
{
	DWORD                dwSize;                         // Current structure body size 
	int                  nValue;                         // Detect result value 
	NET_STATE_TYPE       emState;                        // Detect result status
	int                  nDuration;                      // Status lasts time
}NET_VIDEO_BLUR_DETECTIONRESULT;

// The result of detect type (NET_DIAGNOSIS_SCENECHANGE) Video scene change detect
typedef struct tagNET_VIDEO_SCENECHANGE_DETECTIONRESULT
{
	DWORD                dwSize;                         // Current structure body size 
	int                  nValue;                         // Detect result value 
	NET_STATE_TYPE       emState;                        // Detect result status
	int                  nDuration;                      // Status lasts time
}NET_VIDEO_SCENECHANGE_DETECTIONRESULT;

//  PTZ moving result type        
typedef enum tagEM_PTZ_MOVING_RESULT_TYPE  
{          
    EM_PTZ_MOVING_UNKNOWN ,      // unknown       
    EM_PTZ_MOVING_NORMAL  ,      // normal       
    EM_PTZ_MOVING_NOTOBEY ,      // not tobey       
    EM_PTZ_MOVING_NOTMOVE ,      // not move       
    EM_PTZ_MOVING_NOTDETECT ,    // not detect       
}EM_PTZ_MOVING_RESULT_TYPE; 

// The result of detect type (NET_DIAGNOSIS_PTZ_MOVING) PTZ moving detect
typedef struct tagNET_PTZ_MOVING_DETECTIONRESULT
{
    DWORD                       dwSize;                 // Current structure body size 
    EM_PTZ_MOVING_RESULT_TYPE   emPTZMovingUp;          // moving up detect result 
    EM_PTZ_MOVING_RESULT_TYPE   emPTZMovingDown;        // moving down detect result 
    EM_PTZ_MOVING_RESULT_TYPE   emPTZMovingLeft;        // moving left detect result 
    EM_PTZ_MOVING_RESULT_TYPE   emPTZMovingRight;       // moving right detect result 
    EM_PTZ_MOVING_RESULT_TYPE   emPTZMovingZoomWide;    // moving zoom wide detect result 
    EM_PTZ_MOVING_RESULT_TYPE   emPTZMovingZoomTele;    // moving zoom tele detect result     
}NET_PTZ_MOVING_DETECTIONRESULT;

// The result of detect type(NET_DIAGNOSIS_BLACK_WHITE)  black and white image detection
typedef struct tagNET_BLACK_WHITE_DETECTIONRESULT
{
	DWORD                dwSize;                 // Current structure body size 
	int                  nValue;                 // Detect result value 
    NET_STATE_TYPE       emState;                // Detect result status
    int                  nDuration;              // Status lasts time
} NET_BLACK_WHITE_DETECTIONRESULT;

// The result of detect type(NET_DIAGNOSIS_DRAMATIC_CHANGE) dramatic change of scene
typedef struct tagNET_DIAGNOSIS_DRAMATIC_DETECTIONRESULT
{
	DWORD                dwSize;                 // Current structure body size 
	int                  nValue;                 // Detect result value 
    NET_STATE_TYPE       emState;                // Detect result status
    int                  nDuration;              // Status lasts time
} NET_DIAGNOSIS_DRAMATIC_DETECTIONRESULT;


// The result of detect type(NET_DIAGNOSIS_VIDEO_DELAY)video delay detect
typedef struct tagNET_VIDEO_DELAY_DETECTIONRESUL
{
    DWORD       dwSize;             // Current structure body size 
    int         nSignalDelay;       // single delay(-1 mean detect failed),  unit ms
    int         nStreamDelay;       // video stream delay(-1 mean detect failed), unit ms
    int         nIFrameDelay;       // IFrame delay(-1 mean detect failed), unit ms
}NET_VIDEO_DELAY_DETECTIONRESUL;

typedef struct tagNET_DIAGNOSIS_RESULT_HEADER
{
	DWORD                dwSize;                         // Current structure body size 
	
	char                 szDiagnosisType[MAX_PATH];      // Analysis type. Please refer to dhnetsdk.h for "video analysis report result type definition"  such as NET_DIAGNOSIS_DITHER
	int                  nDiagnosisTypeLen;              // The structure body size of current analysis type
}NET_DIAGNOSIS_RESULT_HEADER;

// cbVideoDiagnosis Call parameter type
typedef struct tagNET_REAL_DIAGNOSIS_RESULT
{
	DWORD                dwSize;                         // Current structure body size 
	
    NET_VIDEODIAGNOSIS_COMMON_INFO* pstDiagnosisCommonInfo;  //Video analysisi general info
	
	int					 nTypeCount;					 // Analysis result data analysis type amount
	void*                pDiagnosisResult;               // The analysis result data for once. The format is as NET_DIAGNOSIS_RESULT_HEADER+analysis type1+NET_DIAGNOSIS_RESULT_HEADER+analysis type 2+...
	DWORD                dwBufSize;                      // Buffer length
}NET_REAL_DIAGNOSIS_RESULT;

// Video analysis result report call function
typedef int (CALLBACK *fRealVideoDiagnosis)(LLONG lDiagnosisHandle, NET_REAL_DIAGNOSIS_RESULT* pDiagnosisInfo, void* pBuf, int nBufLen, LDWORD dwUser);

//The input parameter of interface  CLIENT_StartVideoDiagnosis
typedef struct tagNET_IN_VIDEODIAGNOSIS
{
	DWORD                dwSize;                         // Current structure body size 
	int                  nDiagnosisID;                   // Video analysis ID begins with 0
	DWORD                dwWaitTime;                     // Interface time out waiting time
	
	fRealVideoDiagnosis  cbVideoDiagnosis;               // Video analysis result call function.
	LDWORD                dwUser;                        // User customized parameter
}NET_IN_VIDEODIAGNOSIS;

// The output parameter of interface CLIENT_StartVideoDiagnosis 
typedef struct tagNET_OUT_ATTACH_REALDIAGNOSIS
{
	DWORD                dwSize;                         // Current structure body size 
	LLONG                 lDiagnosisHandle;              // Subscribe handle
}NET_OUT_VIDEODIAGNOSIS;

// The input parameter of interface CLIENT_StartFindDiagnosisResult 
typedef struct tagNET_IN_FIND_DIAGNOSIS
{
	DWORD                dwSize;                         // Current structure body size 
	int                  nDiagnosisID;                   // Video analysis ID begins with 0
	DWORD                dwWaitTime;                     // Interface time out waiting time
	
	NET_ARRAY            stuDeviceID;                    // Device unique ID.pArray:NULL means does not search by device ID.
	NET_TIME             stuStartTime;                   // Start time
	NET_TIME             stuEndTime;                     // End time
	int                  nVideoChannel;                  // Video channel No.,-1: All channel No.
	int                  nTypeCount;                     // Analysis type amount.0:Do not use analysis type to search.
	NET_ARRAY*           pstDiagnosisTypes;              // Analysis type array. It is the analysis type to be searched. 
														 // the space application by the user,apply to sizeof(NET_ARRAY)*nTypeCount
	char                 szProjectName[MAX_PATH];        // Project name
} NET_IN_FIND_DIAGNOSIS;

// The output parameter of interface CLIENT_StartFindDiagnosisResult
typedef struct tagNET_OUT_FIND_DIAGNOSIS
{
	DWORD                dwSize;                         // Current structure body size 
	LLONG                 lFindHandle;                   // Search handle
	DWORD                dwTotalCount;                   // The total amount that matched the criteria
}NET_OUT_FIND_DIAGNOSIS;

// The input parameter of interface CLIENT_DoFindDiagnosisResult 
typedef struct tagNET_IN_DIAGNOSIS_INFO
{
	DWORD                dwSize;                         // Current structure body size 
	int                  nDiagnosisID;                   // Video analysis ID begins with 0
	DWORD                dwWaitTime;                     // Interface time out waiting time
	
	int                  nFindCount;                     // The video analysis result amount of each search
	int                  nBeginNumber;                   // Search initial SN 0<=beginNumber<= totalCount-1
}NET_IN_DIAGNOSIS_INFO;

// CLIENT_StartRealTimeProject Interface input parameters
typedef struct tagNET_IN_START_RT_PROJECT_INFO
{
	DWORD                dwSize;                         // This structure size
	char*                pszProjectInfo;                 // Real-time schedule information by calling dhconfigsdk.dll get the package in the interface CLIENT_PacketData
                            	                         // Package command corresponding CFG_CMD_VIDEODIAGNOSIS_REALPROJECT                      
}NET_IN_START_RT_PROJECT_INFO;

// CLIENT_StartRealTimeProject Interface output parameters
typedef struct tagNET_OUT_START_RT_PROJECT_INFO
{
	DWORD                dwSize;                         // This structure size
}NET_OUT_START_RT_PROJECT_INFO;

// CLIENT_StopRealTimeProject Interface output parameters
typedef struct tagNET_IN_STOP_RT_PROJECT_INFO
{
	DWORD                dwSize;                         // This structure size
	char*                 pszProjectNames;               // Program name to the name of the separator && as the example:Project1&&Project2   
}NET_IN_STOP_RT_PROJECT_INFO; 

// CLIENT_StopRealTimeProject Interface output parameters
typedef struct tagNET_OUT_STOP_RT_PROJECT_INFO
{
	DWORD                dwSize;                         // This structure size
}NET_OUT_STOP_RT_PROJECT_INFO;

// carport light info
typedef struct tagNET_CARPORTLIGHT_INFO
{
	DWORD               dwSize;  
	NET_CARPORTLIGHT_TYPE emType;                        // carport type
	NET_CARPORTLIGHT_MODE emMode;                        // light way
}NET_CARPORTLIGHT_INFO;

// carpor light status
typedef struct tagNET_CARPORTLIGHT_STATUS
{
	DWORD                dwSize;                         		// struct size 
	int                  nLightNum;                      		// light num
	NET_CARPORTLIGHT_INFO stuLights[DH_MAX_CARPORTLIGHT_NUM]; 	// light info,don't repeat
	int                  nFiringTime;                    		// firing time(s)
	int                  nHoldTime;                      		// hold time(s), -1 means keep,0 auto control
}NET_CARPORTLIGHT_STATUS;

// CLIENT_GetCarPortLightStatus's interface input
typedef struct tagNET_IN_GET_CARPORTLIGHT_STATUS
{
	DWORD                dwSize;                         // struct size 
	int                  nChannelId;                     // channel ID
}NET_IN_GET_CARPORTLIGHT_STATUS;

// CLIENT_GetCarPortLightStatus's interface output
typedef struct tagNET_OUT_GET_CARPORTLIGHT_STATUS
{
	DWORD                dwSize;                         // struct size 
	NET_CARPORTLIGHT_STATUS stuLightStatus;              // light status
}NET_OUT_GET_CARPORTLIGHT_STATUS;

// CLIENT_SetCarPortLightStatus's interface input
typedef struct tagNET_IN_SET_CARPORTLIGHT_STATUS
{
	DWORD                dwSize;                         // struct size 
	int                  nChannelId;
	NET_CARPORTLIGHT_STATUS stuLightStatus;              // light status
}NET_IN_SET_CARPORTLIGHT_STATUS;

// CLIENT_SetCarPortLightStatus's interface output
typedef struct tagNET_OUT_SET_CARPORTLIGHT_STATUS
{
	DWORD                dwSize;                         // struct size 
	
}NET_OUT_SET_CARPORTLIGHT_STATUS;
typedef struct tagNET_DEV_VIDEODIAGNOSIS_MULTI_INFO
{
	DWORD                					dwSize;                	// Current structure body size 
	
    NET_VIDEODIAGNOSIS_COMMON_INFO*         pstDiagnosisCommonInfo; // Video analysis general info. You need to allocate the following pointer of the structure body.
	
	BOOL                 					abDither;				// It is to stand for current analysis item is valid or not in this result.
	NET_VIDEO_DITHER_DETECTIONRESULT*       pstDither;  			// Video vibration detect
	
	BOOL                 					abStration;
	NET_VIDEO_STRIATION_DETECTIONRESULT*    pstStration;			// Video stria detect
	
	BOOL                 					abLoss;
	NET_VIDEO_LOSS_DETECTIONRESULT*         pstLoss;    			// Video loss detect 
	
	BOOL                 					abCover;
	NET_VIDEO_COVER_DETECTIONRESULT*        pstCover;   			// Camera masking detect
	
	BOOL                 					abFrozen;
	NET_VIDEO_FROZEN_DETECTIONRESULT*       pstFrozen;  			// Video freeze detect
	
	BOOL                 					abBrightness;
	NET_VIDEO_BRIGHTNESS_DETECTIONRESULT*   pstBrightness;			// Video brightness abnormal detect 
	
	BOOL                 					abContrast;
	NET_VIDEO_CONTRAST_DETECTIONRESULT*     pstContrast;			// Video contrast abnormal detect 
	
	BOOL                 					abUnbalance;
	NET_VIDEO_UNBALANCE_DETECTIONRESULT*    pstUnbalance;			// Video color cast detect
	
	BOOL                 					abNoise;
	NET_VIDEO_NOISE_DETECTIONRESULT*        pstNoise;   			// Video noise detect 
	
	BOOL                 					abBlur;
	NET_VIDEO_BLUR_DETECTIONRESULT*         pstBlur;    			// Video blur detect
	
	BOOL                 					abSceneChange;
	NET_VIDEO_SCENECHANGE_DETECTIONRESULT*  pstSceneChange;			// Video scene change detect 

    BOOL                 					abVideoDelay;
    NET_VIDEO_DELAY_DETECTIONRESUL*       	pstVideoDelay;        	// Video delay detect

    BOOL                					abPTZMoving;
    NET_PTZ_MOVING_DETECTIONRESULT*       	pstPTZMoving;         	// PTZ moving detect
    
    BOOL									abBlackAndWhite;
	NET_BLACK_WHITE_DETECTIONRESULT*	  	pstBlackAndWhite;		// black and white image detect
	
	BOOL									abDramaticChange;
	NET_DIAGNOSIS_DRAMATIC_DETECTIONRESULT* pstDramaticChange;		// dramatic change of scene detect
}NET_VIDEODIAGNOSIS_RESULT_INFO;

// The output parameter of interface CLIENT_DoFindDiagnosisResult
typedef struct tagNET_OUT_DIAGNOSIS_INFO
{
	DWORD                dwSize;                         // Current structure body size 
	
	int                  nInputNum;                      // The amount of the NET_DEV_RESULT_VIDEODIAGNOSIS_INFO you allocate
	int                  nReturnNum;                     // Returned amount
	NET_VIDEODIAGNOSIS_RESULT_INFO*  pstDiagnosisResult; // Result data You need to allocate the pointer of the structure body
														 // apply to sizeof(NET_VIDEODIAGNOSIS_RESULT_INFO)*nInputNum
}NET_OUT_DIAGNOSIS_INFO;

///////////////////////////////// master-slave device control module////////////////////////////
///////////////////////////////// CLIENT_OperateMasterSlaveDevice /////////////////////////////////
#define        MASTERSLAVE_CMD_START                   "masterSlaveTracker.start"                   // start
#define        MASTERSLAVE_CMD_STOP                    "masterSlaveTracker.stop"                    // stop
#define        MASTERSLAVE_CMD_MANUALTRACK             "masterSlaveTracker.manualSelectObjectTrack" // manual select object
#define        MASTERSLAVE_CMD_POINTTRACK              "masterSlaveTracker.selectPointTrack"        // points tracking
#define        MASTERSLAVE_CMD_GETCALIBPOINTS          "masterSlaveTracker.getCalibratePoints"      // get calibrate points
#define        MASTERSLAVE_CMD_CALIBRATE               "masterSlaveTracker.calibrate"               // save calibrate points
#define        MASTERSLAVE_CMD_ADDCALIBPOINT           "masterSlaveTracker.addCalibratePoint"       // add calibrate points
#define        MASTERSLAVE_CMD_REMOVECALIBPOINT        "masterSlaveTracker.removeCalibratePoint"    // remove calibrate points
#define        MASTERSLAVE_CMD_MARKMAXZOOM             "masterSlaveTracker.markSceneMaxZoom"        // mark scene max zoom

// master-slave device pair points
typedef struct __NET_MS_PAIRPOINTS
{
	int                nStructSize;
    DH_POINT           stMasterPoint;    // master point,range[0,8192]
    DH_POINT           stSlavePoint;     // slave point,range[0,8192]
}NET_MS_PAIRPOINTS;
//MASTERSLAVE_CMD_START
typedef struct __NET_IN_MS_START
{
	int                nStructSize;
}NET_IN_MS_START;

//MASTERSLAVE_CMD_START
typedef struct __NET_OUT_MS_START
{
	int                nStructSize;
}NET_OUT_MS_START;

//MASTERSLAVE_CMD_STOP
typedef struct __NET_IN_MS_STOP
{
	int                nStructSize;
}NET_IN_MS_STOP;

//MASTERSLAVE_CMD_STOP
typedef struct __NET_OUT_MS_STOP
{
	int                nStructSize;
}NET_OUT_MS_STOP;

//MASTERSLAVE_CMD_MANUALTRACK
typedef struct __NET_IN_MS_MANUALTRACK
{
	int                nStructSize;
	DWORD              dwObject;          // -1 any position of the entire region, >=0 specify the object(can be obtained from the device to return to intelligent frame)
}NET_IN_MS_MANUALTRACK;

//MASTERSLAVE_CMD_MANUALTRACK
typedef struct __NET_OUT_MS_MANUALTRACK
{
	int                nStructSize;
}NET_OUT_MS_MANUALTRACK;

//MASTERSLAVE_CMD_POINTTRACK
typedef struct __NET_IN_MS_POINTTRACK
{
	int                nStructSize;
	DH_POINT           stTrackerPoint;    // Tracker point,range[0,8192]
}NET_IN_MS_POINTTRACK;

//MASTERSLAVE_CMD_POINTTRACK
typedef struct __NET_OUT_MS_POINTTRACK
{
	int                nStructSize;
}NET_OUT_MS_POINTTRACK;

//MASTERSLAVE_CMD_GETCALIBPOINTS
typedef struct __NET_IN_MS_GETCALIBPOINTS
{
	int                nStructSize;
}NET_IN_MS_GETCALIBPOINTS;

//MASTERSLAVE_CMD_GETCALIBPOINTS
typedef struct __NET_OUT_MS_GETCALIBPOINTS
{
	int                        nStructSize;
	int                        nPointsNum;                           // practicality points num
	NET_MS_PAIRPOINTS          stPairPoints[MAX_CALIBPOINTS_NUM];    // pair points
}NET_OUT_MS_GETCALIBPOINTS;

//MASTERSLAVE_CMD_CALIBRATE
typedef struct __NET_IN_MS_CALIBRATE
{
	int                nStructSize;
}NET_IN_MS_CALIBRATE;

//MASTERSLAVE_CMD_CALIBRATE
typedef struct __NET_OUT_MS_CALIBRATE
{
	int                nStructSize;      
}NET_OUT_MS_CALIBRATE;

//MASTERSLAVE_CMD_ADDCALIBPOINT
typedef struct __NET_IN_MS_ADDCALIBPOINT
{
	int                        nStructSize;
	BOOL                       bSlavePointEn;        // enable, TURE able;FASLE disable
	NET_MS_PAIRPOINTS          stPairPoints;         // master-slave camera pair point
}NET_IN_MS_ADDCALIBPOINT;

//MASTERSLAVE_CMD_ADDCALIBPOINT
typedef struct __NET_OUT_MS_ADDCALIBPOINT
{
	int                nStructSize;
	DH_POINT           stPoint;      // slave point
}NET_OUT_MS_ADDCALIBPOINT;


//MASTERSLAVE_CMD_REMOVECALIBPOINT
typedef struct __NET_IN_MS_REMOVECALIBPOINT
{
	int                nStructSize;
	DH_POINT           stPoint;      // master point
}NET_IN_MS_REMOVECALIBPOINT;

//MASTERSLAVE_CMD_REMOVECALIBPOINT
typedef struct __NET_OUT_MS_REMOVECALIBPOINT
{
	int                nStructSize;      
}NET_OUT_MS_REMOVECALIBPOINT;

//MASTERSLAVE_CMD_MARKMAXZOOM
typedef struct __NET_IN_MS_MARKMAXZOOM
{
	int                nStructSize; 
}NET_IN_MS_MARKMAXZOOM;

//MASTERSLAVE_CMD_MARKMAXZOOM
typedef struct __NET_OUT_MS_MARKMAXZOOM
{
	int                nStructSize; 
}NET_OUT_MS_MARKMAXZOOM;

///////////////////////////////// CLIENT_OperateCommDevice Interface parameters defined /////////////////////////////////
#define		COMMDEV_GET_EXTERNLDATA							"devComm.getExternalData"		//get serial external data for 3rd party general data

#define        MAX_EXTERN_DATA_LEN                2048

//COMMDEV_GET_EXTERNLDATA Input parameters
typedef struct __COMMDEV_IN_GET_EXTERNDATA
{
    DWORD        dwSize;

}COMMDEV_IN_GET_EXTERNDATA;

//COMMDEV_GET_EXTERNLDATA Output parameters
typedef struct __COMMDEV_OUT_GET_EXTERNDATA
{
	DWORD		dwSize;
	char		szGetData[MAX_EXTERN_DATA_LEN];				//External serial devices to collect data (data format based on custom projects)
}COMMDEV_OUT_GET_EXTERNDATA;


/////////////////////////////////CLIENT_OperateVideoAnalyseDevice Interface parameter /////////////////////////////////
#define     NET_SET_MODULESTATE                         "devVideoAnalyse.setModuleState"                        // Set module status
#define		NET_TEST_WITH_SCREENPOINTS					"devVideoAnalyse.testCalibrateWithScreenPoints"			// Detect depth of the field according to the coordinates on the screen. It is the actual distance between these two points. Please complete the parameter setup before you test.
#define		NET_TEST_WITH_METRICLENGTH					"devVideoAnalyse.testCalibrateWithMetricLength"		    // Detect parameter of depth of the field according to actual lenght,start point,and direction coordinates.
#define		NET_GET_INTERNALOPTIONS						"devVideoAnalyse.getInternalOptions"					// Get internal option
#define		NET_TUNE_INTERNALOPTIONS					"devVideoAnalyse.tuneInternalOptions"					// Debug internal item
#define		NET_RESET_INTERNALOPTIONS					"devVideoAnalyse.resetInternalOptions"					// Reset all debug internal item
#define     NET_SET_PTZ_PRESET_STATUS                   "devVideoAnalyse.setPtzPresetStatus"                    // get preset info

// Line type
typedef enum tagNET_EM_LINETYPE
{
	NET_EM_HORIZONTAL,
	NET_EM_VERTICAL,
}NET_EM_LINETYPE;

// PTZ preset status type
typedef enum tagEM_DH_PTZ_PRESET_STATUS
{
    EM_DH_PTZ_PRESET_STATUS_UNKNOWN,        // unknown
    EM_DH_PTZ_PRESET_STATUS_REACH,          // preset reach the point
    EM_DH_PTZ_PRESET_STATUS_UNREACH,        // preset do not reach the point yet
}EM_DH_PTZ_PRESET_STATUS;

//PTZ positioning information alarm
typedef struct
{
	int     nChannelID;             // Channel number
	int     nPTZPan;                // Horizontal movement of the head position, effective range: [0,3600]
	int     nPTZTilt;               // PTZ vertical position, the effective range: [-1800,1800]
	int     nPTZZoom;               // PTZ iris position changes, effective range: [0,128]
	BYTE    bState;                 // PTZ motion, 0 - Unknown 1 - Movement 2 - Idle
	BYTE    bAction;                // PTZ movement, 0 - preset 1 - line scan 2 - Cruise 3 - patrol track 4 - horizontal rotation
	BYTE    bFocusState;            // PTZ focus state, 0 - unknown 1 - state of motion 2 - Idle
	BYTE    bEffectiveInTimeSection; //In the period of validity of the preset state
									 //If the current is preset reported preset period of time, compared with one, otherwise 0
	int     nPtzActionID;           // Cruise ID number
	DWORD   dwPresetID;             // PTZ preset number where
	float   fFocusPosition;         // Focus position
	BYTE	bZoomState;				// ZOOM PTZ status, 0 - Unknown,1-ZOOM, 2 - Idle
	BYTE	bReserved[3];			// Alignment
	DWORD   dwSequence;             // Packet sequence number, used to verify whether the loss
	DWORD   dwUTC;                  // Corresponding UTC (1970-1-1 00:00:00) seconds.
    EM_DH_PTZ_PRESET_STATUS emPresetStatus; // preset status
	int	    nZoomValue;			    // real zoom value ,expanded 100 times
	int     reserved[244];          // Reserved field
}DH_PTZ_LOCATION_INFO;

// NET_SET_PTZ_PRESET_STATUS
typedef struct __NET_IN_PTZ_PRESET_STATUS
{
    DWORD                    dwSize;
    DH_PTZ_LOCATION_INFO     stPTZStatus;
}NET_IN_PTZ_PRESET_STATUS;

typedef struct __NET_OUT_PTZ_PRESET_STATUS
{
    DWORD    dwSize;
}NET_OUT_PTZ_PRESET_STATUS;

// NET_TEST_WITH_SCREENPOINTS
typedef struct __NET_IN_CALIBRATE_BY_POINTS
{
	DWORD               dwSize;                 // Current structure body size 
	DH_POINT            stStartPoint;           // Start point of the line 	 The point of the line is within [0,8191].
	DH_POINT            stEndPoint;             // End point 	The point of the line is within [0,8191]
	NET_EM_LINETYPE     emType;                 // Line type	em_Horizontal("Horizontal")  em_Vertical("Vertical")
}NET_IN_CALIBRATE_BY_POINTS;

typedef struct __NET_OUT_CALIBRATE_BY_POINTS
{
	DWORD               dwSize;                 // Current structure body size 	
	double              dlength;                // The actual length between these two points. Unit is meter.
}NET_OUT_CALIBRATE_BY_POINTS;

// NET_TEST_WITH_METRICLENGTH
typedef struct __NET_IN_CALIBRATE_BY_LENGTH
{
	DWORD               dwSize;                 // Current structure body size 
	double				nLength;				// The actual length between these two points. Unit is meter.
	DH_POINT			stStartPoint;			// Start point of the line	The point of the line is within [0,8191]
	DH_POINT			stDirection;			// Line direction coordinates	For vertical line ony. The point of the line is within [0,8191]
	NET_EM_LINETYPE		emType;					// Line type	em_Horizontal("Horizontal")  em_Vertical("Vertical")				
}NET_IN_CALIBRATE_BY_LENGTH;

typedef struct __NET_OUT_CALIBRATE_BY_LENGTH
{
	DWORD               dwSize;                 // Current structure body size 
	DH_POINT			stEndPoint;				// Coordinates of the end of the line on the screen. 	The point of the line is within[0,8191]	
}NET_OUT_CALIBRATE_BY_LENGTH;

typedef struct __NET_INTERNAL_OPTION
{
	DWORD               dwSize;                 // Current structure body size 
	int					nIndex;					// Item SN
	NET_ARRAY           stKey;				    // Option name. 256 bytes including'\0'. Encryption data
	int					nValue;					// Option value
	int					nMinValue;				// Min value
	int					nMaxValue;				// Max value
}NET_INTERNAL_OPTION;

// Debug internal option NET_GET_INTERNALOPTIONS
typedef struct __NET_IN_GETINTERNAL_OPTIONS
{
	DWORD               dwSize;                 // Current structure body size
}NET_IN_GETINTERNAL_OPTIONS;

typedef struct __NET_OUT_GETINTERNAL_OPTIONS
{
	DWORD               dwSize;                 // Current structure body size 
	
	int					nTotalOptionNum;		// Option amount. Max value search capability.
	int					nReturnOptionNum;		// returned actual option amount.
	NET_INTERNAL_OPTION* pstInternalOption;	    // Option list. Caller shall allocate the address.
												// apply to sizeof(NET_INTERNAL_OPTION)*nTotalOptionNum
}NET_OUT_GETINTERNAL_OPTIONS;

// NET_TUNE_INTERNALOPTIONS
typedef struct __NET_IN_TUNEINTERNAL_OPTION
{
	DWORD               dwSize;                 // Current structure body size 
	int					nOptionCount;	    	// Option amount
	NET_INTERNAL_OPTION* pstInternalOption;	    // Option list. The amount stands for the capability.
												// the space appliacation by th user, apply to sizeof(NET_INTERNAL_OPTION)*nOptionCount
}NET_IN_TUNEINTERNAL_OPTION;

typedef struct __NET_OUT_TUNEINTERNAL_OPTION
{
	DWORD               dwSize;                 // Current structure body size 
}NET_OUT_TUNEINTERNAL_OPTION;

// NET_RESET_INTERNALOPTIONS
typedef struct __NET_IN_RESETINTERNAL_OPTION
{
	DWORD               dwSize;                 // Current structure body size 
}NET_IN_RESETINTERNAL_OPTION;

typedef struct __NET_OUT_RESETINTERNAL_OPTION
{
	DWORD               dwSize;                 // Current structure body size 
}NET_OUT_RESETINTERNAL_OPTION;

// NET_SET_MODULESTATE
typedef enum tagNET_EM_MODULESTATE
{
	NET_EM_MODULESTATE_OK,
	NET_EM_MODULESTATE_PAUSE,
}NET_EM_MODULESTATE;
typedef struct __NET_IN_SETMODULESTATE
{
	DWORD				dwSize;					// Current structure body size 
	NET_EM_MODULESTATE  emState;				// 0-Normal work.1-Pause. Need to rebuild background when you switch to the normal video since it may change.
	int                 nIndex;                 // Model SN.	-1 stands for all modules.
}NET_IN_SETMODULESTATE;

typedef struct __NET_OUT_SETMODULESTATE
{
	DWORD				dwSize;					// Current structure body size
}NET_OUT_SETMODULESTATE;

/////////////////////////////////CLIENT_DeleteDevConfig Interface parameter/////////////////////////////////
/*
 * CLIENT_DeleteDevConfig Input and output parameter
 */

typedef enum tagNET_EM_CONFIGOPTION
{
	NET_EM_CONFIGOPTION_OK,
	NET_EM_CONFIGOPTION_NEEDRESTART,
	NET_EM_CONFIGOPTION_NEEDREBOOT=2,
	NET_EM_CONFIGOPTION_WRITEFILEERROR=4,
	NET_EM_CONFIGOPTION_CAPSNOTSUPPORT=8,
	NET_EM_CONFIGOPTION_VALIDATEFAILED=16,
}NET_EM_CONFIGOPTION;

typedef struct tagNET_IN_DELETECFG
{
	DWORD         dwSize;						// Structure body size
	char*         szCommand;                    // Configuration command
}NET_IN_DELETECFG;

typedef struct tagNET_OUT_DELETECFG
{
	DWORD         dwSize;                       // Structure body size
    int           nError;                       // The error code device returns
    int           nRestart;                     // Device reboot or not
	DWORD         dwOptionMask;                 // Option. Please refer to enumeration NET_EM_CONFIGOPTION
}NET_OUT_DELETECFG;

/////////////////////////////////CLIENT_GetMemberNames interface parameter/////////////////////////////////
/*
 * CLIENT_GetMemberNames Input and output parameter
 */
typedef struct tagNET_IN_MEMBERNAME
{
	DWORD         dwSize;                       // Structure body size
	char*         szCommand;                    // Configuration command
}NET_IN_MEMBERNAME;

typedef struct tagNET_OUT_MEMBERNAME
{
	DWORD         dwSize;                       // Structure body size 
	int           nError;                       // The error code device returns
    int           nRestart;                     // Device reboot or not	
	int           nTotalNameCount;              // Name amount. Fromt the capability set. Caller fill in.
	int           nRetNameCount;                // The returned actual name amount
	NET_ARRAY*    pstNames;                     // Name array. The caller apply for the memory. The amount is nTotalNameCount.
												// the space application by the user,apply to sizeof(NET_ARRAY)*nTotalNameCount
}NET_OUT_MEMBERNAME;

////////////////////////////////////video compression//////////////////////////////////////
// staff type
typedef enum tagNET_EM_STAFF_TYPE
{
	NET_EM_STAFF_TYPE_ERR,
	NET_EM_STAFF_TYPE_HORIZONTAL,                       // "Horizontal" 
	NET_EM_STAFF_TYPE_VERTICAL,                         // "Vertical" 
	NET_EM_STAFF_TYPE_ANY,                              // "Any" 
	NET_EM_STAFF_TYPE_CROSS,                            // "Cross" 
}NET_EM_STAFF_TYPE;

// region type
typedef enum tagNET_EM_CALIBRATEAREA_TYPE
{
	NET_EM_CALIBRATEAREA_TYPE_ERR,
	NET_EM_CALIBRATEAREA_TYPE_GROUD,		            // "Groud" 
	NET_EM_CALIBRATEAREA_TYPE_HORIZONTAL,	            // "Horizontal" 		
	NET_EM_CALIBRATEAREA_TYPE_VERTICAL,		            // "Vertical" 
	NET_EM_CALIBRATEAREA_TYPE_ANY,			            // "Any"
}NET_EM_CALIBRATEAREA_TYPE;

// The concentrated output data subtype
typedef enum tagNET_EM_OUTPUT_SUB_TYPE
{
	NET_EM_OUTPUT_SUB_TYPE_ERR,
	NET_EM_OUTPUT_SUB_TYPE_NORMAL,                      // Normal playback speed (the parent type is concentrated Video valid)
	NET_EM_OUTPUT_SUB_TYPE_SYNOPSIS,                    // Concentrated by the playback speed (parent type is effective when concentrated video), quick release in the absence of an object, the object appears when playing at normal speed
}NET_EM_OUTPUT_SUB_TYPE;

// staff info
typedef struct tagNET_STAFF_INFO
{
	DH_POINT	        stuStartLocation;	            // start location
	DH_POINT	        stuEndLocation;		            // end location
	float			    nLenth;				            // length(m)
	NET_EM_STAFF_TYPE	emType;			                // type
}NET_STAFF_INFO;

// calibration area,common scenario use
typedef struct tagNET_CALIBRATEAREA_INFO
{
	int				    nLinePoint;					    // horizontal line point
	DH_POINT	        stuLine[DH_MAX_POLYLINE_NUM];	// horizontal line
	float			    fLenth;						    // lenth
	DH_POLY_POINTS	    stuArea;					    // area
	int				    nStaffNum;					    // number of vertical line
	NET_STAFF_INFO	    stuStaffs[DH_MAX_STAFF_NUM];    // vertical line         
	NET_EM_CALIBRATEAREA_TYPE emType;			        // area type
}NET_CALIBRATEAREA_INFO;

typedef struct tagNET_LOCALFILE_INFO
{
	DWORD			    dwSize;
	char			    szFilePath[MAX_PATH];	        // file path
}NET_LOCALFILE_INFO;

typedef struct tagNET_REMOTEFILE_INFO
{
	DWORD			    dwSize;
	char			    szIP[MAX_PATH];	                // the remote device IP
	unsigned int	    nPort;				            // the remote device  port
	char			    szName[DH_MAX_NAME_LEN];        // the remote device  userName
	char			    szPassword[DH_MAX_NAME_LEN];    // the remote device  password
	BYTE			    byProtocolType;			        // the remote device protocol type,2-the second generation of private 3-the third generation of private
	BYTE			    byReserved[3];			        // byte alignment
	char			    szFilePath[MAX_PATH];           // file path,when byProtocolType == 3 effective.
	int				    nStartCluster;		  	        // start cluster,when byProtocolType == 2 effective.
	int				    nDriverNo;				        // driver no.when byProtocolType == 2 effective.
	
}NET_REMOTEFILE_INFO;

// file path info
typedef struct tagNET_FILEPATH_INFO
{
	DWORD				dwSize;
	BOOL				bFileType;			            // TRUE: local server's file; FALSE: remote device's file
	NET_LOCALFILE_INFO	stuLocalFile;		            // local file, when bFileType==TRUE effective
	NET_REMOTEFILE_INFO	stuRemoteFile;		            // remote file, when bFileType==FALSE effective
}NET_FILEPATH_INFO;

// global param info
typedef struct tagNET_GLOBAL_INFO
{
	DWORD			    dwSize;
	char			    szSceneType[DH_MAX_NAME_LEN];	// scene type,only support "Normal"
	int				    nStaffNum;						// staff num
	NET_STAFF_INFO	    stuStaffs[DH_MAX_STAFF_NUM];	// staffs
	int				    nCalibrateAreaNum;				// number of calibrate area
	NET_CALIBRATEAREA_INFO stuCalibrateArea[DH_MAX_CALIBRATEBOX_NUM]; // calibrate area(if the filed does not exist,return the entire tegion)
	int				    nJitter;		                // vidicon jitter, range[0-100],the value of static camera shake,the more powerful shake,the bigger the value.
	BOOL			    bDejitter;		                // if start to wobble
}NET_GLOBAL_INFO;

// module info
typedef struct tagNET_MODULE_INFO
{
	DWORD			    dwSize;										// size
	BYTE			    bSensitivity;								// sensitivity,range[1-10],the lower the value the sensitivity.
	BYTE			    bReserved[3];
	int				    nDetectRegionPoint;							// detect region point
	DH_POINT		    stuDetectRegion[DH_MAX_POLYGON_NUM]; 		// detect region
	int				    nExcludeRegionNum;							// number of exclude region 
	DH_POLY_POINTS	    stuExcludeRegion[DH_MAX_EXCLUDEREGION_NUM];	// exclude region
}NET_MODULE_INFO;

// object filter info
typedef struct tagNET_OBJFILTER_INFO
{
	DWORD		        dwSize;						    // size
	NET_TIME	        stuStartTime;				    // start time
	NET_TIME	        stuEndTime;					    // end time
	char		        szObjectType[DH_MAX_NAME_LEN];	// object type, support for the following:
											            // "Unknown", "Human", "Vehicle",
											            // "Fire", "Smoke", "Plate", "HumanFace", "Container", "Animal", "TrafficLight", "PastePaper", "HumanHead", "Entity"
	char		        szObjectSubType[64];		    // object sub type,support for the following:
											            // Vehicle Category:"Unknown"  "Motor" "Non-Motor" "Bus" "Bicycle" "Motorcycle" 
											            // Plate Category: "Unknown" "Normal" "Yellow" "DoubleYellow" "Police" "Armed" 
											            // "Military" "DoubleMilitary" "SAR" "Trainning" 
											            // "Personal" "Agri" "Embassy" "Moto" "Tractor" "Other"
											            // HumanFace Category:"Normal" "HideEye" "HideNose" "HideMouth" 
	DWORD		        dwRGBMainColor[DH_MAX_COLOR_NUM];	// main color,object to the person,said the upper part of the color,in bytes,is red,green,blue and diaphaneity,example:RGB value is(0,255,0),when diaphaneity = 0, the value = 0x00ff0000
	DWORD		        dwLowerBodyColor[DH_MAX_COLOR_NUM];	// object to the person,said the lower half of the color,in bytes,is red,green,blue and diaphaneity,example:RGB value is(0,255,0),when diaphaneity = 0, the value = 0x00ff0000
	int			        nMinSize;					    // min siez, m
	int			        nMaxSize;					    // max size, m
	int			        nMinSpeed;					    // min speed km/h
	int			        nMaxSpeed;					    // max speed km/h
	DWORD		        dwRGBMainColorNum;			    // object is the person,dwRGBMainColor effective number of colors
	DWORD		        dwLowerBodyColorNum;		    // object is the person, and bRGBMainColor==true,is the lower half of the effective color
	BOOL		        bRGBMainColor;				    // true-dwRGBMainColor the upper part of the color,dwLowerBodyColor the lower half of the color; false-dwRGBMainColor means main color,dwLowerBodyColor nullity
}NET_OBJFILTER_INFO;

// param of rule
typedef struct tagNET_VIDEOSYNOPSISRULE_INFO
{
	DWORD				dwSize;				            // struct size
	BYTE				byOutputType;		            // out tyope,1-snapshot;2-condensatal video;3-snapshot with video
	BYTE				byDensity;			            // density, 1-10, the more value,the density higher
	bool				bShowTime;			            // show time
	bool				bShowBox;			            // show object box
	bool				bEnableRecordResult;            // the result recoded or not
    BYTE                byTime;                         // Length after video synposis, unit is minute
    BYTE                byReserved[2];
	NET_OBJFILTER_INFO	stuFilter[MAX_OBJFILTER_NUM];   // fileter info
	DWORD				dwObjFilterNum;		            // number of filter

	// NET_OUT_QUERY_VIDEOSYNOPSIS , CFG_CMD_ANALYSERULE transfer dhconfigsdk.dll get CLIENT_ParseData
	// NET_IN_ADD_VIDEOSYNOPSIS ,CFG_CMD_ANALYSERULEtransfer  dhconfigsdk.dll get CLIENT_PacketData
	char*		        szAnalyseRuleInfo;				// JSON rule info
	int                 nBufferLen;                     // NET_OUT_QUERY_VIDEOSYNOPSIS,apply szAnalyseRuleInfo length
	BOOL				bCustomResolution;				// Custom resolution enabled, TRUE-resolution is determined by emResolution, FALSE-concentrate production based on the original video resolution video
	CAPTURE_SIZE		emResolution;					// Resolution, bCustomResolution is TRUE only valid
	BOOL				bEnableMaxUsedCPU;				// Are maximize occupy CPU resource requirements
	BOOL				bEnableSmoothBorder;			// Target is smooth border
	NET_EM_OUTPUT_SUB_TYPE emOutputSubType;             // After concentrating the output data sub-type, see NET_EM_OUTPUT_SUB_TYPE
	int					nExtractFrameRate;				// Video frame rate pumping concentrated 1-32, the value, the higher the pumping rate of the frame, the client presents the faster playback
														// This field is concentrated only in the output data sub-type (emOutputSubType) is valid NET_EM_OUTPUT_SUB_TYPE_SYNOPSIS
	//Video synposis extra info 
    int                 nSynopsisSpeed;                 // Synposis speed threshold. The level ranges from 1 to 10. 5 means only reserve the object of speed higher than 5. It is a relative unit.
                                                        // Current string is null if it is o. 
    int                 nSynopsisSize;                  // Synposis dimensions threshold. There are ten levels (1 to 10). 3 means only reserve the object of speed higher than 3. It is a relative unit.
                                                        // Current string is null if it is o. 
    BOOL                bEnableDirection;               // When it is True,  filter the object moving direction. 
                                                        // When it is False, do not filter the object moving direction. 
    DH_POINT            stuSynopsisStartLocation;       // Synopsis moving direction, start coordinates. The point coordinates [0,8192), it is valid when bEnableDirection is True. 
    DH_POINT            stuSynopsisEndLocation;         // Synopsis moving direction, stop coordinates. The point coordinates [0,8192), it is valid when bEnableDirection is True. 
}NET_VIDEOSYNOPSISRULE_INFO;

// add video synopsis input param
typedef struct tagNET_IN_ADD_VIDEOSYNOPSIS
{
	DWORD			    dwSize;			                // size
	NET_FILEPATH_INFO*	pFilePathInfo;	                // file path info
														// the space application by the user,apply to sizeof(NET_FILEPATH_INFO)*dwFileCount														// the space application by the user,apply to 

	DWORD				dwFileCount;	                // effective number of files
	NET_GLOBAL_INFO		stuGlobalInfo;	                // global info
	NET_MODULE_INFO		stuModuleInfo;	                // module info
	NET_VIDEOSYNOPSISRULE_INFO	stuRuleInfo;	        // rule info
	unsigned int		nWaitTime;		                // wait time(ms)
}NET_IN_ADD_VIDEOSYNOPSIS;

// add video synopsis output param
typedef struct tagNET_OUT_ADD_VIDEOSYNOPSIS
{
	DWORD	            dwSize;			                // size
	DWORD*	            pnTaskID;		                // TaskID array,users are assigned space.
	DWORD	            nTaskIDCount;	                // count of TaskID
}NET_OUT_ADD_VIDEOSYNOPSIS;

// pause video synopsis input param
typedef struct tagNET_IN_PRE_HANDLE_VIDEOSYNOPSIS
{
    DWORD                   dwSize;                         // The structure size
    NET_FILEPATH_INFO*      pFilePathInfo;                  // File position info. Space allocated by the user,apply to sizeof(NET_FILEPATH_INFO)*dwFileCount
    DWORD                   dwFileCount;                    // Valid file amount. 
    unsigned int            nWaitTime;                      // Waiting time out, unit is ms.
    char                    szTaskName[DH_COMMON_STRING_64];// the task name,can be empty
}NET_IN_PRE_HANDLE_VIDEOSYNOPSIS;

// Video synopsis pre-process task outputs parameters 	
typedef struct tagNET_OUT_PRE_HANDLE_VIDEOSYNOPSIS
{
    DWORD               dwSize;                             // Structure size 
    DWORD*              pnTaskID;                           // TaskID array. Space allocated by user. It is corresponding to the added task file. 0=failed to add the corresponding task. 
															// the space application by the user, apply to sizeof(DWORD)*nTaskIDCount
	DWORD               nTaskIDCount;                       // TaskID amount 
}NET_OUT_PRE_HANDLE_VIDEOSYNOPSIS;

// Input parameters when implement the video synopsis task 
typedef struct tagNET_IN_RUN_VIDEOSYNOPSIS_TASK
{
    DWORD                       dwSize;                     // Structure size 
    DWORD                       nTaskID;                    // Task ID. CLIENT_PreHandleVideoSynopsisTask output function and then return 
    NET_GLOBAL_INFO             stuGlobalInfo;              // Global info 
    NET_MODULE_INFO             stuModuleInfo;              // Module info 
    NET_VIDEOSYNOPSISRULE_INFO  stuRuleInfo;                // Rule info 
    unsigned int                nWaitTime;                  // Waiting time out. Unit is ms
}NET_IN_RUN_VIDEOSYNOPSIS_TASK;

//  Output parameters when implementing the video synopsis task 
typedef struct tagNET_OUT_RUN_VIDEOSYNOPSIS_TASK
{
    DWORD               dwSize;                             // Structure size 
    DWORD               nPlayID;                            // Playback ID,0=interface failed, otherwise input CLIENT_PlayBackBySynopsisFile to playback video
    DWORD               nTime;                              // Video synopsis time. Unit:second. 
}NET_OUT_RUN_VIDEOSYNOPSIS_TASK;

// Input parameters when parsing the video synopsis task 
typedef struct tagNET_IN_PAUSE_VIDEOSYNOPSIS
{
	DWORD	            dwSize;			                // size 
	BOOL                bPause;                         // TRUE-pause  FALSE-resume 
	DWORD*	            pnTaskID;		                // TaskID array,users are assigned space.apply to sizeof(DWORD)*nTaskIDCount
	DWORD	            nTaskIDCount;	                // count of TaskID
	DWORD	            nWaitTime;		                // wait time(ms)
}NET_IN_PAUSE_VIDEOSYNOPSIS;

//  remove video synopsis input param
typedef enum tagNET_SYNOPSIS_REMOVE_TYPE
{
    EM_SYNOPSIS_REMOVE_UNKNOW =0 , // Unknown 
    EM_SYNOPSIS_REMOVE_PRETASK   , // Pre-process task 
    EM_SYNOPSIS_REMOVE_SYNO      , // Video synopsis task 
}NET_SYNOPSIS_REMOVE_TYPE;

// Input parameters when deleting video synopsis task 
typedef struct tagNET_IN_REMOVE_VIDEOSYNOPSIS
{
	DWORD	            dwSize;			                // size
	DWORD*	            pnTaskID;		                // TaskID array,users are assigned space.apply to sizeof(DWORD)*nTaskIDCount
	DWORD	            nTaskIDCount;	                // count of TaskID
	DWORD	            nWaitTime;		                // wait time(ms)
    NET_SYNOPSIS_REMOVE_TYPE    emRemoveType;			// Task type to be deleted 
}NET_IN_REMOVE_VIDEOSYNOPSIS;

// return objece info of video synopsis
typedef struct tagNET_REAL_SYNOPSIS_OBJECT_INFO
{
	DWORD               dwSize;
	
	DWORD               dwTaskID;                       // task ID
    DH_MSG_OBJECT_EX2   stuObjectInfo;                  // Object detailed info 
}NET_REAL_SYNOPSIS_OBJECT_INFO;

typedef struct tagNET_REAL_SYNOPSIS_STATE_INFO
{
	DWORD               dwSize;
	
	DWORD               dwTaskID;                       // task ID   
	int                 nObjectNum;                     // number of object
	int                 nTaolProgress;                  // progress of condensation,0~100
	char                szState[DH_MAX_SYNOPSIS_STATE_NAME]; // state of condensation,"Downloading"  "Synopsising"
                                                        // "DownloadFailed""DownloadSucceeded"
	                                                  	// "SynopsisFailed" ,"Succeeded" 
		                                                // "DownloadPause" ,"SynopsisPause" 
	int           		nProgressOfCurrentState;        // progress of current state,0~100
	char				szFailedCode[DH_MAX_STRING_LEN];// Failure code, szState="SynopsisFailed"Effective
														// "OutOfMemory" Out of memory;
														// "TooMany_TargetInVideo"  Too many goals the entire video;
														// "InvalidFilePointer" File pointer is invalid;
														// "InvalidFilePath" File path is invalid;
														// "CreateDirectoryFailed"  Create a folder path fails;
														// "WriteDataFailed" Intermediate file failed to write Tube
														// "DataSizeTooLarge" Tube file is too large
														// "Detect_Nothing" Scene without moving target
														// "OpenDataFailed" Failed to open the intermediate file
														// "InvalidSynopsisBackgroud" Invalid concentrated background
														// "ColorRetrieval" Wrong color retrieve configuration
														// "UnsupportRuleType" Unsupported rule type
														// "RuleNumberExceedLimit" Number of rules overrun
														// "NoFunctionCfgInfo" But did not specify the configuration information corresponding function
														// "FunctionNumberExceedLimit" The number of feature quantity of each rule overrun
														// "PointsExceedLimit" Point line or area overrun
														// "BadFunctionType" Error function type
														// "RulePointNumber" Points rule error
														// "MaskRegionNumberExceedLimit" Shielded area overrun quantity
														// "CameraSceneSwitch" Cameras scene change
														// "TooManyTargeInFrame" Excessive number of single-frame target
														// "InalidSynopsisDesity" Concentrated density invalid
														// "ExtrackColorFeatureFailed" Failed to extract color features
														// "JpegEncodeFrameFailed" Jpeg encoder failure
														// "JpegParamRestFailed" Jpeg encoding parameters fail reset
														// "JpegDecodeFrameFailed" Jpeg decoding failure
														// "RetrieveObjectIdInvalid" Retrieving Target ID is invalid
														// "RetrieveImageInvalid" Invalid image retrieval
														// "NetworkAnomaly" DISH Network anomaly map
														// "ObjectNumberLimit" Limit the number of goals over analysis
														// "CreateChannelFailed" Create a channel failure
														// "ReadUGFileFailed" Read UG file failed
														// "GetSynopsisInfoFailed" Failed to obtain concentrated Video
														// "ObjectNumberExceedLimit" The number of targets in the scene over the limit
														// "RebuildBackgroudFailed" Background reconstruction failure
														// "NotExistMiddleFile" Intermediate file does not exist
														// "NotExistSourceFile" Original file does not exist
														// "NotDog" No dongle
														// "NotEnoughFreeDisk" Disk space is not enough
														// "StartDecodeFail" Start decoding failure
														// "DecodeTimeOut" Decoding wait timeout
														// "EncodeTimeOut" Coding wait timeout
														// "ExactTimeOut" Extract snapshots wait timeout
														// "ReadMidlleFileFail" Failed to read the intermediate file
														// "ExactPictureFail" Failed to extract pictures
    char 				szPicPath[DH_COMMON_STRING_512];// The snap image file path get by pre-process file.  
                                                        // Support HTTP URL means :http://www.dahuate.com/1.jpg
                                                        // Support FTP URL means: "ftp://ftp.dahuate.com/1.jpg"
                                                        // Server local path  
                                                        // a)"C:/pic/1.jpg" 
                                                        // b)"/mnt//2010/8/11/dav/15:40:50.jpg"
    int         		nMaxTime;                       // Supported video synopsis max time. Unit is second. 
    int         		nMinTime;                       // Supported video synopsis min time. Unit is second. 
}NET_REAL_SYNOPSIS_STATE_INFO;

//callback function of synopsis video info
typedef int (CALLBACK *fVideoSynopsisObjData)(LLONG lRealLoadHandle, NET_REAL_SYNOPSIS_OBJECT_INFO* pSynopsisObjectInfo, void* pBuf, int nBufLen, LDWORD dwUser, void* pReserved);

//callback function of synopsis video state
typedef int (CALLBACK *fVideoSynopsisState)(LLONG lRealLoadHandle, NET_REAL_SYNOPSIS_STATE_INFO* pSynopsisStateInfos, int nInfoCount, LDWORD dwUser, void* pReserved);



// CLIENT_RealLoadObjectData's interface define
typedef struct tagNET_IN_REALLOAD_OBJECTDATA
{
	DWORD               dwSize;                         // size
	
	int                 nTaskIDNum;                     // number of taskID
	DWORD               *pTaskIDs;                      // material taskID, the space application by the user, apply to sizeof(DWORD)*nTaskIDNum
	BOOL                bNeedPicFile;                   // need download corresponding to the picture 
	int                 nWaitTime;                      // wait time(ms)
	fVideoSynopsisObjData cbVideoSynopsisObjData;       // callback function's pointer
	LDWORD         		dwUser;
}NET_IN_REALLOAD_OBJECTDATA;

typedef struct tagNET_OUT_REALLOAD_OBJECTDATA
{
	DWORD               dwSize;                         // size

	LLONG               lRealLoadHandle;                // return subscriptal handle
}NET_OUT_REALLOAD_OBJECTDATA;

// CLIENT_StopLoadObjectData's interface define
typedef struct tagNET_IN_STOPLOAD_OBJECTDATA
{
	DWORD               dwSize;

	int                 nTackIDNum;                     // number of tackID
	DWORD               *pTaskIDs;                      // specific taskID, the space application by the user, apply to sizeof(DWORD)*nTaskIDNum
}NET_IN_STOPLOAD_OBJECTDATA;

// CLIENT_RealLoadSynopsisState's interface define
typedef struct tagNET_IN_REALLAOD_SYNOPSISSTATE
{
	DWORD               dwSize;                         // size
	
	int                 nTaskIDNum;                     // number of taskID
	DWORD               *pTaskIDs;                      // specific taskID,the space application by the user, apply to sizeof(DWORD)*nTaskIDNum  
	int                 nWaitTime;                      // wait time(ms)
	fVideoSynopsisState cbVideoSynopsisState;           // callback function's pointer
	LDWORD         		dwUser;
}NET_IN_REALLAOD_SYNOPSISSTATE;

typedef struct tagNET_OUT_REALLOAD_SYNOPSISSTATE
{
	DWORD               dwSize;                         // size
	
	LLONG               lRealLoadHandle;                // return subscriptal handle
}NET_OUT_REALLOAD_SYNOPSISSTATE;

// CLIENT_StopLoadSynopsisState's interface define
typedef struct tagNET_IN_STOPLOAD_SYNOPSISSTATE
{
	DWORD               dwSize;
	
	int                 nTackIDNum;                     // number of taskID
	DWORD               *pTaskIDs;                      // specific taskID, the space application by the user, apply to sizeof(DWORD)*nTaskIDNum  
}NET_IN_STOPLOAD_SYNOPSISSTATE;

// type of video synopsis's query
typedef enum tagEM_VIDEOSYNOPSIS_QUERY_TYPE
{
	EM_VIDEOSYNOPSIS_TASK,	                            // task info
	EM_VIDEOSYNOPSIS_OBJ,	                            // snapshot
}EM_VIDEOSYNOPSIS_QUERY_TYPE;

// query the task of video synopsis information
typedef struct tagNET_QUERY_VSTASK_INFO
{
	DWORD	            dwSize;			                // size
	DWORD	            dwTaskID;		                // taskID,when value = 0,query all task
}NET_QUERY_VSTASK_INFO;

// query conditions of video objects in the task information
typedef struct tagNET_QUERY_VSOBJECT_INFO
{
	DWORD			    dwSize;			                // size
	DWORD*			    pdwTaskID;                		// taskID,when value = -1,query all task
	DWORD			    dwTaskIDCount;                	// count of taskID
	DH_MSG_OBJECT	    stuObjInfo;	                	// object info,when objectID!=0,query all
}NET_QUERY_VSOBJECT_INFO;

//--CLIENT_QueryVideoSynopsisInfo's interface input define
typedef struct tagNET_IN_QUERY_VIDEOSYNOPSIS
{
	DWORD	            dwSize;							// size
	EM_VIDEOSYNOPSIS_QUERY_TYPE emQueryType;            // query type
	DWORD	            dwCount;						// count of query
	DWORD	            dwBeginNumber;					// begin number
	void*	            pQueryInfo;						// information query condition
											            // when emQueryType==EM_VIDEOSYNOPSIS_TASK, corresponding to NET_QUERY_VSTASK_INFO,apply to sizeof(NET_QUERY_VSTASK_INFO)*dwCount
											            // when emQueryType==EM_VIDEOSYNOPSIS_OBJ, corresponding to NET_QUERY_VSOBJECT_INFO,apply to sizeo(NET_QUERY_VSOBJECT_INFO)*dwCount

	DWORD	            dwWaitTime;						// wait time(ms)
}NET_IN_QUERY_VIDEOSYNOPSIS;

// query to the task of each synopsis video information
typedef struct tagNET_VSTASK_INFO
{
	DWORD		        dwSize;			                // size
	DWORD				dwTaskID;		                // task ID
	NET_MODULE_INFO		stuModuleInfo;                	// module info
	NET_VIDEOSYNOPSISRULE_INFO	stuRuleInfo;           	// info of task rule
	NET_GLOBAL_INFO		stuGlobalInfo;	                // global param
	NET_TIME			stuCreateTime;	                // create time
	char				szCurrState[DH_MAX_SYNOPSIS_STATE_NAME];  // current state,"Downloading" "Synopsising" 
																  // "DownloadFailed" "DownloadSucceeded" 
																  // "SynopsisFailed" "Succeeded" 
																  // "DownloadPause" "SynopsisPause" 
	char				szCreateUser[DH_MAX_NAME_LEN];	// the creator
	int					nProgressOfCurrentState;		// progress of current state [0~100]
	char				szLocalFilePath[MAX_PATH];		// local file path
	int					nObjectNum;						// number of object
	NET_FILEPATH_INFO	stuVideoSourceFilePath;			// path of video source file
	NET_FILEPATH_INFO	stuSynopsisVideoFilePath;		// path of synopsis video file
	char                szFailedCode[DH_MAX_STRING_LEN];// Concentrated video failed error code, szCurrState when "SynopsisFailed" Effective
                                                        // "OutOfMemory" Out of memory;
                                                        // "TooMany_TargetInVideo"  Too many goals the entire video;
                                                        // "InvalidFilePointer" File pointer is invalid;
                                                        // "InvalidFilePath" File path is invalid;
                                                        // "CreateDirectoryFailed"  Create a folder path fails;
                                                        // "WriteDataFailed" Intermediate file failed to write Tube
                                                        // "DataSizeTooLarge" Tube file is too large
                                                        // "Detect_Nothing" Scene without moving target
                                                        // "OpenDataFailed" Failed to open the intermediate file
                                                        // "InvalidSynopsisBackgroud" Invalid concentrated background
                                                        // "ColorRetrieval" Wrong color retrieve configuration
                                                        // "UnsupportRuleType" Unsupported rule type
                                                        // "RuleNumberExceedLimit" Number of rules overrun
                                                        // "NoFunctionCfgInfo" But did not specify the configuration information corresponding function
                                                        // "FunctionNumberExceedLimit" The number of feature quantity of each rule overrun
                                                        // "PointsExceedLimit" Point line or area overrun
                                                        // "BadFunctionType" Error function type
                                                        // "RulePointNumber" Points rule error
                                                        // "MaskRegionNumberExceedLimit" Shielded area overrun quantity
                                                        // "CameraSceneSwitch" Cameras scene change
                                                        // "TooManyTargeInFrame" Excessive number of single-frame target
                                                        // "InalidSynopsisDesity" Concentrated density invalid
                                                        // "ExtrackColorFeatureFailed" Failed to extract color features
                                                        // "JpegEncodeFrameFailed" Jpeg encoder failure
                                                        // "JpegParamRestFailed" Jpeg encoding parameters fail reset
                                                        // "JpegDecodeFrameFailed" Jpeg decoding failure
                                                        // "RetrieveObjectIdInvalid" Retrieving Target ID is invalid
                                                        // "RetrieveImageInvalid" RetrieveImageInvalid
                                                        // "NetworkAnomaly" NetworkAnomaly
                                                        // "ObjectNumberLimit" bjectNumberLimit
                                                        // "CreateChannelFailed" CreateChannelFailed
                                                        // "ReadUGFileFailed" ReadUGFileFailed
                                                        // "GetSynopsisInfoFailed" GetSynopsisInfoFailed
                                                        // "ObjectNumberExceedLimit" ObjectNumberExceedLimit
                                                        // "RebuildBackgroudFailed" RebuildBackgroudFailed
                                                        // "NotExistMiddleFile" NotExistMiddleFile
                                                        // "NotExistSourceFile" NotExistSourceFile
                                                        // "NotDog" NotDog
                                                        // "NotEnoughFreeDisk" NotEnoughFreeDisk
                                                        // "StartDecodeFail"StartDecodeFail
                                                        // "DecodeTimeOut" DecodeTimeOut
                                                        // "EncodeTimeOut" EncodeTimeOut
                                                        // "ExactTimeOut" ExactTimeOut
                                                        // "ReadMidlleFileFail" ReadMidlleFileFail
                                                        // "ExactPictureFail" ExactPictureFail
    char                        szTaskName[DH_COMMON_STRING_64];    //  the task name
}NET_VSTASK_INFO;

// each object in the synopsis video task info
typedef struct tagNET_VSOBJECT_INFO
{
	DWORD				dwSize;			                // size 
	DWORD				dwTaskID;		                // task ID
	DH_MSG_OBJECT		stuObjInfo;		                // object info
	NET_FILEPATH_INFO	stuFilePathInfo;                // file info
	DWORD				dwFileLength;	                // file length
}NET_VSOBJECT_INFO;

//--CLIENT_QueryVideoSynopsisInfo's interface output define
typedef struct tagNET_OUT_QUERY_VIDEOSYNOPSIS
{
	DWORD	            dwSize;			                // size
	DWORD	            dwTotalCount;	                // total count
	DWORD            	dwFoundCount;	                // found count
	void*	            pResult;                		// return result info
					                		            // if emQueryType==EM_VIDEOSYNOPSIS_TASK,corresponding to NET_VSTASK_INFO,apply to sizeof(NET_VSTASK_INFO)*dwMaxCount
							                            // if emQueryType==EM_VIDEOSYNOPSIS_OBJ,corresponding to NET_VSOBJECT_INFO,apply to sizeof(NET_VSOBJECT_INFO)*dwMaxCount
	DWORD	            dwMaxCount;		                // hope tp get the number
}NET_OUT_QUERY_VIDEOSYNOPSIS;

// CLIENT_FindSynopsisFile's interface define 
// query type of file
typedef enum tagNET_EM_QUERY_SYNOPSIS_FILE
{
	DH_FILE_QUERY_VIDEO,                                // normal record file info,corresponding to:NET_SYNOPSIS_QUERY_VIDEO_PARAM, return result:NET_SYNOPSISFILE_VIDEO_INFO
	DH_FILE_QUERY_SYNOPSISVIDEO,                        // synopsis video file,corresponding to:NET_QUERY_SYNOPSISVIDEO_PARAM,return result:NET_QUERY_SYNOPSISVIDEO_INFO
}NET_EM_QUERY_SYNOPSIS_FILE;

typedef struct __NET_SYNOPSIS_QUERY_VIDEO_PARAM
{
	DWORD               dwSize;
	DWORD				dwQueryMask;					// query type mask, according to from low to high,the first bit is period of time, the second is file path 
	NET_TIME			stuStartTime;					// start time
	NET_TIME			stuEndTime;						// end time
	char				szFilePath[DH_MAX_SYNOPSIS_QUERY_FILE_COUNT][MAX_PATH];	// file path
	DWORD				dwFileCount;					// file count
}NET_SYNOPSIS_QUERY_VIDEO_PARAM;

typedef struct  
{
	DWORD               dwSize;
	int                 nTaskId;                        // taskID
	DWORD               dwOutPutTypeMask;               // output type in current,the first bit is snapshot,the second bit is synopsis video 
	char                szCurrentState[DH_MAX_SYNOPSIS_STATE_NAME]; // current state
}NET_VIDEOSYNOPSIS_TASK_INFO;

// DH_FILE_QUERY_VIDEO return video file info
typedef struct
{
	DWORD               dwSize;
    unsigned int		nchannelId;						// channel ID
    char				szFilePath[MAX_PATH];		    // file path
    unsigned int		nFileLenth;					    // file lenth(byte)
    NET_TIME			stuStarttime;				    // start time 
    NET_TIME			stuEndtime;				        // end time
    unsigned int		nWorkDirSN;				        // work dir SN	
	unsigned int        nCluster;                       // cluster				
	BYTE                bHint;					        // hint
	BYTE                bDriveNo;                       // drive no.
	BYTE                bReserved[18];                  // reserved
	
	int                 nTaskInfoNum;                   // task info number
	NET_VIDEOSYNOPSIS_TASK_INFO stuTaskInfo[16];        // video synopsis task info
    char szSynopsisPicPath[DH_COMMON_STRING_512];       // The snap image file path get by pre-process file.  
    int                 nSynopsisMaxTime;               // Supported video synopsis max time. Unit is second. 
    int                 nSynopsisMinTime;               // Supported video synopsis min time. Unit is second. 
}NET_SYNOPSISFILE_VIDEO_INFO;

// DH_FILE_QUERY_SYNOPSISVIDEO synopsis viedo file query param
typedef struct __NET_QUERY_SYNOPSISVIDEO_PARAM
{
	DWORD               dwSize;
	int                 nTaskID;                        // taskID
}NET_QUERY_SYNOPSISVIDEO_PARAM;

// DH_FILE_QUERY_SYNOPSISVIDEO query result
typedef struct __NET_QUERY_SYNOPSISVIDEO_INFO
{
	DWORD               dwSize;
    char				szFilePath[MAX_PATH];        	// file path
    unsigned int		nFileLenth;					    // file lenth
	int                 nDurationTime;                  // duration time(s)
}NET_QUERY_SYNOPSISVIDEO_INFO;

// CLIENT_FindSynopsisFile's interface param
typedef struct tagNET_IN_FIND_SYNOPSISFILE
{
	DWORD               dwSize;                          
	NET_EM_QUERY_SYNOPSIS_FILE emQueryType;             // query's type
	void*               pQueryCondition;                // query's type,the space application by the user
														// if query type is DH_FILE_QUERY_VIDEO, corresponding to NET_SYNOPSIS_QUERY_VIDEO_PARAM structure, apply to sizeof(NET_SYNOPSIS_QUERY_VIDEO_PARAM)
														// if query type is DH_FILE_QUERY_SYNOPSISVIDEO, corresponding to NET_QUERY_SYNOPSISVIDEO_PARAM structure, apply to sizeof(NET_QUERY_SYNOPSISVIDEO_PARAM)
	int                 nWaitTime;                      // wait time(ms)
}NET_IN_FIND_SYNOPSISFILE;

typedef struct tagNET_OUT_FIND_SYNOPSISFILE
{
	DWORD               dwSize;
	LLONG               lFindHandle;                    // find handle         
}NET_OUT_FIND_SYNOPSISFILE;

// CLIENT_FindNextSynopsisFile's interface param
typedef struct tagNET_IN_FINDNEXT_SYNOPSISFILE
{
	DWORD               dwSize;
    int                 nFileCount;                     // count of file
	NET_EM_QUERY_SYNOPSIS_FILE emQueryType;             // query's type
	void*               pSynopsisFileInfo;              // info of file buf,the space application by the user, apply to nMaxlen
	int                 nMaxlen;                        // size of buf
	int                 nWaitTime;                      // wait time(ms)
}NET_IN_FINDNEXT_SYNOPSISFILE;

typedef struct tagNET_OUT_FINDNEXT_SYNOPSISFILE
{
	DWORD               dwSize;
	int                 nRetFileCount;                  // the actual return information artucle number,return<nFileCount(input param) corresponding period of the finished file query
}NET_OUT_FINDNEXT_SYNOPSISFILE;

// download progress's callback,nError means downloading appear error,1 - cashe is insufficient,2-data validation error,3.download failed,4,create file failed
typedef void (CALLBACK *fSynopsisFileDownLoadPosCB)(LLONG lDownLoadHandle, DWORD dwFileID, DWORD dwFileTotalSize, DWORD dwDownLoadSize, int nError, LDWORD dwUser, void* pReserved);

// CLIENT_DownLoadSynosisFile's interface param
typedef  NET_DOWNLOADFILE_INFO NET_SYNOPSIS_DOWNLOADFILE_INFO;

typedef struct tagNET_IN_DOWNLOAD_SYNOPSISFILE
{
	DWORD               dwSize;
    int                 nFileCount;                     // count of file
	NET_SYNOPSIS_DOWNLOADFILE_INFO* pFileInfos;         // file info,the space application by the user, apply to sizeof(NET_SYNOPSIS_DOWNLOADFILE_INFO)*nFileCount
	fSynopsisFileDownLoadPosCB cbPosCallBack;           // post of callback function
	LDWORD              dwUserData;                     // user's data
	int                 nWaitTime;                      // wait time(ms)
}NET_IN_DOWNLOAD_SYNOPSISFILE;

typedef struct tagNET_OUT_DOWNLOAD_SYNOPSISFILE
{
	DWORD               dwSize;
	LLONG               lDownLoadHandle;                // handle of download
}NET_OUT_DOWNLOAD_SYNOPSISFILE;

// path of file's info 
typedef struct	tagNET_SET_FILEPATH_INFO
{
	DWORD		        dwSize;
	char		        szFilePath[MAX_PATH];	        // can be a folder ,can be the file,current server only supports the dav format file
}NET_SET_FILEPATH_INFO;

// CLIENT_SetFilePathInfo()interface input param
typedef struct tagNET_IN_SET_FILEPATHINFO
{
	DWORD		        dwSize;
	DWORD		        dwCount;		                // count of added file
	void*	        	pFilePathInfo;                	// look for NET_SET_FILEPATH_INFO
													    // the space application by the user, apply to sizeof(NET_SET_FILEPATH_INFO)*dwCount
	DWORD		        dwWaitTime;		                // wait time(ms)
}NET_IN_SET_FILEPATHINFO;


// fAddFileStateCB param
typedef struct tagNET_CB_ADDFILESTATE
{
	DWORD		        dwSize;
	const char*         szFileName;                     // file name
	const char*         szState;                        // analyse file's state, "Successed" ; "Failed" ;
}NET_CB_ADDFILESTATE;

// burning device callback function, pBuf->dwSize == nBufLen
typedef void (CALLBACK *fAddFileStateCB) (LLONG lLoginID, LLONG lAttachHandle, NET_CB_ADDFILESTATE* pBuf,  int nBufLen, LDWORD dwUser);

// CLIENT_AttacAddFileState()interface input param
typedef struct tagNET_IN_ADDFILE_STATE
{
	DWORD		        dwSize;
	fAddFileStateCB     cbAttachState;                 // listenint to increase callback file state
	LDWORD              dwUser;                        // user's data
}NET_IN_ADDFILE_STATE;
typedef struct tagNET_OUT_ADDFILE_STATE
{
	DWORD		        dwSize;
}NET_OUT_ADDFILE_STATE;

///////////////////////////////////Face recognition module related structures///////////////////////////////////////
typedef struct tagNET_UID_CHAR
{
    char szUID[DH_MAX_PERSON_ID_LEN];  //UID contents 
}NET_UID_CHAR;

// CLIENT_OperateFaceRecognitionDBInterface input parameters
typedef struct __NET_IN_OPERATE_FACERECONGNITIONDB
{
	DWORD             					dwSize;
	EM_OPERATE_FACERECONGNITIONDB_TYPE  emOperateType;  // Type of operation
	FACERECOGNITION_PERSON_INFO         stPersonInfo;   // Personnel information

	// Picture binary data
    DWORD            					nUIDNum;		// UID amount              
    NET_UID_CHAR     					*stuUIDs;		// Person unique mark. Generated by the client if it is the first time. Different from the ID string. 
														// the space application by the user, apply to sizeof(NET_UID_CHAR)*nUIDNum
	char                				*pBuffer;		// Buffer address
	int                 				nBufferLen;		// Buffer data length
}NET_IN_OPERATE_FACERECONGNITIONDB;

// CLIENT_OperateFaceRecognitionDB port output parameter
typedef struct __NET_OUT_OPERATE_FACERECONGNITIONDB
{
    DWORD               dwSize;
}NET_OUT_OPERATE_FACERECONGNITIONDB;

typedef struct __NET_FACE_MATCH_OPTIONS
{
	DWORD               dwSize;
	unsigned int        nMatchImportant;               // Important level 1 to 10 staff, the higher the number the more important (check important level greater than or equal to this level of staff)
	EM_FACE_COMPARE_MODE emMode;                       // Face comparison mode, see EM_FACE_COMPARE_MODE
	int                 nAreaNum;                      // Face the number of regional
	EM_FACE_AREA_TYPE   szAreas[MAX_FACE_AREA_NUM];    // Regional groupings of people face is NET_FACE_COMPARE_MODE_AREA effective when emMode
	int                 nAccuracy;                     // Recognition accuracy (ranging from 1 to 10, with the value increases, the detection accuracy is improved, the detection rate of decline. Minimum value of 1 indicates the detection speed priority, the maximum is 10, said detection accuracy preferred. Temporarily valid only for face detection)
	int                 nSimilarity;                   // Similarity (must be greater than the degree of acquaintance before the report; expressed as a percentage, from 1 to 100)
	int                 nMaxCandidate;                 // Reported the largest number of candidate (based on similarity to sort candidates to take the maximum number of similarity report)
	
}NET_FACE_MATCH_OPTIONS;

typedef struct __NET_FACE_FILTER_CONDTION
{
	DWORD               dwSize;
	NET_TIME			stStartTime;			       // Start time
    NET_TIME			stEndTime;				       // End Time
	char                szMachineAddress[MAX_PATH];    // Place to support fuzzy matching
	int                 nRangeNum;                     // The actual number of database
	BYTE                szRange[MAX_FACE_DB_NUM];      // To query the database type, see EM_FACE_DB_TYPE
	EM_FACERECOGNITION_FACE_TYPE emFaceType;           // Face to query types, see EM_FACERECOGNITION
    int                 nGroupIdNum;                   // staff group  
    char                szGroupId[MAX_GOURP_NUM][DH_COMMON_STRING_64]; // staff group ID 
    NET_TIME            stBirthdayRangeStart;          // start birthday time
    NET_TIME            stBirthdayRangeEnd;            // end birthday time
}NET_FACE_FILTER_CONDTION;
// CLIENT_StartFindFaceRecognitionInterface input parameters
typedef struct __NET_IN_STARTFIND_FACERECONGNITION
{
	DWORD               dwSize;
	BOOL                bPersonEnable;                 // Personnel information query is valid
	FACERECOGNITION_PERSON_INFO stPerson;              // Personnel information query
	NET_FACE_MATCH_OPTIONS stMatchOptions;             // Face Matching Options
	NET_FACE_FILTER_CONDTION stFilterInfo;             // Query filters
  
	// Picture binary data
	char                *pBuffer;                      // Buffer address
	int                 nBufferLen;                    // Buffer data length

    int                 nChannelID;                    // Channel ID     
}NET_IN_STARTFIND_FACERECONGNITION;

// CLIENT_StartFindFaceRecognitionInterface output parameters
typedef struct __NET_OUT_STARTFIND_FACERECONGNITION
{
	DWORD               dwSize;
	int                 nTotalCount;                   // Record number of returns that match the query criteria
	LLONG               lFindHandle;                   // Query handle
    int                 nToken;                        // The search token received 
}NET_OUT_STARTFIND_FACERECONGNITION;

// CLIENT_StartMultiFindFaceRecognition input parameters
typedef struct tagNET_IN_STARTMULTIFIND_FACERECONGNITION
{
	DWORD               dwSize;
	int*				pChannelID;					   // Channel ID
	int					nChannelCount;				   // number of channel
	BOOL                bPersonEnable;                 // Personnel information query is valid
	FACERECOGNITION_PERSON_INFO stPerson;              // Personnel information query
	NET_FACE_MATCH_OPTIONS stMatchOptions;             // Face Matching Options
	NET_FACE_FILTER_CONDTION stFilterInfo;             // Query filters
	
	// Picture binary data
	char                *pBuffer;                      // Buffer address
	int                 nBufferLen;                    // Buffer data length  
}NET_IN_STARTMULTIFIND_FACERECONGNITION;

// CLIENT_StartMultiFindFaceRecognition output parameters
typedef struct tagNET_OUT_STARTMULTIFIND_FACERECONGNITION
{
	DWORD               dwSize;
	int                 nTotalCount;                   // Record number of returns that match the query criteria
	LLONG               lFindHandle;                   // Query handle
	int                 nToken;                        // The search token received
}NET_OUT_STARTMULTIFIND_FACERECONGNITION;

// CLIENT_StartMultiFindFaceRecognitionRecord Corresponding Multimodal face recognition
typedef struct tagNET_IN_STARTMULTIFIND_FACERECONGNITIONRECORD 
{
    DWORD               dwSize;                         // Structure size
	
														// Search filter criteria
    NET_TIME            stStartTime;                    // start time
    NET_TIME            stEndTime;                      // closing time
    char                szMachineAddress[MAX_PATH];     // Place to support fuzzy matching     
    int                 nAlarmType;                     // To query the type of alarm, see EM_FACERECOGNITION_ALARM_TYPE
    BOOL                abPersonInfo;                   // staff info is valid or not
    FACERECOGNITION_PERSON_INFO stPersonInfo;           // staff info
    int*                pChannelID;                     // channel no.
	int					nChannelCount;					// number of channels
    int                 nGroupIdNum;                    // staff group  
    char                szGroupId[MAX_GOURP_NUM][DH_COMMON_STRING_64]; // staff group ID 
}NET_IN_STARTMULTIFIND_FACERECONGNITIONRECORD;

// CLIENT_StartMultiFindFaceRecognitionRecord Corresponding Multimodal face recognition
typedef struct tagNET_OUT_STARTMULTIFIND_FACERECONGNITIONRECORD
{
	DWORD               dwSize;
    int                 nTotalCount;                   // Record number of returns that match the query criteria
	LLONG               lFindHandle;                   // Query handle
	int                nToken;                        // The search token received
}NET_OUT_STARTMULTIFIND_FACERECONGNITIONRECORD;

//CLIENT_DoFindFaceRecognitionRecord Inpute parameters
typedef struct tagNET_IN_DOFIND_FACERECONGNITIONRECORD
{
	DWORD               dwSize;
    int                 nTotalCount;                    // Record number of returns that match the query criteria	
	LLONG               lFindHandle;                   // Query handle 
	int					nBeginNumber;					// begin Number
}NET_IN_DOFIND_FACERECONGNITIONRECORD;

// Face Recognition data
typedef struct tagNET_DOFIND_FACERECONGNITIONRECORD_INFO
{
    BOOL                    bGlobalScenePic;                        // Structure size
    NET_PIC_INFO		    stGlobalScenePic;                       // Panoramic image file path
    DH_MSG_OBJECT           stuObject;                              // the target face object information
    NET_PIC_INFO			stObjectPic;                            // the target face file path
    int                     nCandidateNum;                          // Face Matching the current number of candidates
    CANDIDATE_INFO          stuCandidates[DH_MAX_CANDIDATE_NUM];    // Face candidates to match this informatio
    NET_CANDIDAT_PIC_PATHS_EX  stuCandidatesPic[DH_MAX_CANDIDATE_NUM]; // The current face matching candidates to the image file path
    NET_TIME                stTime;                                 // time for an alarm  
    char                    szAddress[MAX_PATH];                    // Place for an alarm
    int                     nChannelId;                             // channel no 
	BYTE					bReserved[256];							// Reserved
}NET_DOFIND_FACERECONGNITIONRECORD_INFO;

//CLIENT_DoFindFaceRecognitionRecord Out parameters
typedef struct tagNET_OUT_DOFIND_FACERECONGNITIONRECORD
{
	DWORD											dwSize;
	NET_DOFIND_FACERECONGNITIONRECORD_INFO			*stuResults;		// return date
	int												nResultNum;			// The number of memory applications for face recognition information
	int												nTotalCount;		// Actual number of returns
}NET_OUT_DOFIND_FACERECONGNITIONRECORD;


#define MAX_FIND_COUNT  20

// the format of the image returned in the query results
typedef enum tagEM_NEEDED_PIC_RETURN_TYPE
{
    EM_NEEDED_PIC_TYPE_UNKOWN,            // unknow
    EM_NEEDED_PIC_TYPE_HTTP_URL,          // http url
    EM_NEEDED_PIC_TYPE_BINARY_DATA,       // binary data
    EM_NEEDED_PIC_TYPE_HTTP_AND_BINARY,   // http url and binary data
}EM_NEEDED_PIC_RETURN_TYPE;

// CLIENT_DoFindFaceRecognition Interface input parameters
typedef struct __NET_IN_DOFIND_FACERECONGNITION
{
	DWORD               dwSize;
	LLONG               lFindHandle;                   	// Query handle
	int                 nBeginNum;                     	// Queries starting serial number
	int                 nCount;                        	// The current number of records you want to search for
	EM_NEEDED_PIC_RETURN_TYPE	emDataType;				// the format of the image returned in the query results
}NET_IN_DOFIND_FACERECONGNITION;

// CLIENT_DoFindFaceRecognitionInterface output parameters
typedef struct __NET_OUT_FINDNEXT_FACERECONGNITION
{
	DWORD               dwSize;
	int                 nCadidateNum;                  // The actual number of candidate information structure returned
	CANDIDATE_INFO      stCadidateInfo[MAX_FIND_COUNT];// An array of candidate information
	
	// Picture binary data
	char                *pBuffer;                      // Buffer address
	int                 nBufferLen;                    // Buffer data length
}NET_OUT_DOFIND_FACERECONGNITION;

// CLIENT_DetectFaceInterface input parameters
typedef struct __NET_IN_DETECT_FACE
{
	DWORD               dwSize; 
	DH_PIC_INFO         stPicInfo;                     // Big picture information

	// Picture binary data
	char                *pBuffer;                      // Buffer address
	int                 nBufferLen;                    // Buffer data length
}NET_IN_DETECT_FACE;

// CLIENT_DetectFaceInterface output parameters
typedef struct __NET_OUT_DETECT_FACE
{
	DWORD               dwSize; 
	DH_PIC_INFO         *pPicInfo;                     // The detected face image information from the user space applications, apply to sizeof(DH_PIC_INFO)*nMaxPicNum
	int                 nMaxPicNum;                    // The maximum number of face image information
	int                 nRetPicNum;                    // The actual number of returning faces pictures
	
	// Picture binary data
	char                *pBuffer;                      // Buffer address, the user application space to store the detected face image data
	int                 nBufferLen;                    // Buffer data length
}NET_OUT_DETECT_FACE;

// staff group operation enumeration
typedef enum tagEM_OPERATE_FACERECONGNITION_GROUP_TYPE
{
  NET_FACERECONGNITION_GROUP_UNKOWN,
  NET_FACERECONGNITION_GROUP_ADD,           // add staff group info 
  NET_FACERECONGNITION_GROUP_MODIFY,         // modify staff group info  
  NET_FACERECONGNITION_GROUP_DELETE,         // delete staff group info 
}EM_OPERATE_FACERECONGNITION_GROUP_TYPE;

// staff group info 
typedef struct tagNET_FACERECONGNITION_GROUP_INFO
{
  DWORD       dwSize;
  EM_FACE_DB_TYPE   emFaceDBType;         			// staff group type ,  see  EM_FACE_DB_TYPE
  char        szGroupId[DH_COMMON_STRING_64];		// staff group ID, SN(cannot modify, invalid when adding operation)
  char        szGroupName[DH_COMMON_STRING_128]; 	// staff operation name 
  char        szGroupRemarks[DH_COMMON_STRING_256]; // staff group note info 
  int         nGroupSize;          					// current group staff number
  int		  nSimilarity;							// library similarity threshold
  int		  nRetChnCount; 						// rect channel count
  int		  nChannel[DH_MAX_CAMERA_CHANNEL_NUM];	// the list of channels
}NET_FACERECONGNITION_GROUP_INFO;

// add staff group info 
typedef struct tagNET_ADD_FACERECONGNITION_GROUP_INFO
{
  DWORD        dwSize;
  NET_FACERECONGNITION_GROUP_INFO stuGroupInfo;   // staff group info 
}NET_ADD_FACERECONGNITION_GROUP_INFO;

// delete staff group info 
typedef struct tagNET_DELETE_FACERECONGNITION_GROUP_INFO
{
  DWORD        dwSize;
  char         szGroupId[DH_COMMON_STRING_64];	// staff group ID, SN staff
}NET_DELETE_FACERECONGNITION_GROUP_INFO;

// modify staff group info 
typedef struct tagNET_MODIFY_FACERECONGNITION_GROUP_INFO
{
  DWORD        dwSize;
  NET_FACERECONGNITION_GROUP_INFO stuGroupInfo;   // staff group info 
}NET_MODIFY_FACERECONGNITION_GROUP_INFO;

// CLIENT_OperateFaceRecognitionGroup port input parameter
typedef struct tagNET_IN_OPERATE_FACERECONGNITION_GROUP
{
  DWORD        dwSize;
  EM_OPERATE_FACERECONGNITION_GROUP_TYPE emOperateType; // operation type, the space application by the user,please refer to the structure of operate type 
														// if operate type is NET_FACERECONGNITION_GROUP_ADD,corresponding to NET_ADD_FACERECONGNITION_GROUP_INFO
														// if operate type is NET_FACERECONGNITION_GROUP_MODIFY,corresponding to NET_MODIFY_FACERECONGNITION_GROUP_INFO
														// if operate type is NET_FACERECONGNITION_GROUP_DELETE,corresponding to NET_DELETE_FACERECONGNITION_GROUP_INFO
  void        *pOPerateInfo;         					// operation info 
}NET_IN_OPERATE_FACERECONGNITION_GROUP;  

// CLIENT_OperateFaceRecognitionGroup port output parameter
typedef struct tagNET_OUT_OPERATE_FACERECONGNITION_GROUP
{
  DWORD       dwSize;
  char        szGroupId[DH_COMMON_STRING_64]; // new record staff group ID, SN staff
}NET_OUT_OPERATE_FACERECONGNITION_GROUP;  

// max num of get face recognition server person type
#define MAX_FACE_RECOGNITION_SERVER_PERSON_TYPE		64

// CLIENT_GetFaceRecognitionPersonType input parameter
typedef struct tagNET_IN_GET_FACERECOGNITION_PERSON_TYPE
{
    DWORD               dwSize;				// struct size
} NET_IN_GET_FACERECOGNITION_PERSON_TYPE;   

// CLIENT_GetFaceRecognitionPersonType output parameter
typedef struct tagNET_OUT_GET_FACERECOGNITION_PERSON_TYPE
{
    DWORD               dwSize;																		// struct size
    char                szPersonType[MAX_FACE_RECOGNITION_SERVER_PERSON_TYPE][DH_COMMON_STRING_16];	// person type
	int					nPersonTypeNum;																// number of person type
	BYTE                byDefault[DH_COMMON_STRING_16];												// person type, see EM_PERSON_TYPE
	int					nDefaultNum;																// return person type NumBer																	// Personnel types, see EM_PERSON_TYPE
} NET_OUT_GET_FACERECOGNITION_PERSON_TYPE; 

// CLIENT_GetDevCaps (NET_FACERECOGNITIONSE_CAPS) input parameter
typedef struct tagNET_IN_FACERECOGNITIONSERVER_CAPSBILITYQUERY
{
    DWORD               dwSize;				// struct size
} NET_IN_FACERECOGNITIONSERVER_CAPSBILITYQUERY;


// CLIENT_GetDevCaps (NET_FACERECOGNITIONSE_CAPS) output parameter
typedef struct tagNET_OUT_FACERECOGNITIONSERVER_CAPSBILITYQUERY
{
    DWORD               dwSize;				// struct size
	BOOL				bmultiFind;			// support multi channel
	UINT				nmaxFaceType;		// Maximum number of face face types supported
	
} NET_OUT_FACERECOGNITIONSERVER_CAPSBILITYQUERY;  

// operate face recognition person type
typedef enum tagEM_OPERATE_FACERECOGNITION_PERSON_TYPE
{
    NET_PERSON_TYPE_UNKOWN,				// unknown
	NET_PERSON_TYPE_ADD,				// add face recongnition person type
	NET_PERSON_TYPE_MODIFY,				// modify face recongnition person type
	NET_PERSON_TYPE_DELETE,				// delete face recongnition person type
} EM_OPERATE_FACERECOGNITION_PERSON_TYPE;

// add face recongnition person type
typedef struct tagNET_ADD_FACERECOGNITION_PERSON_TYPE_INFO
{
    DWORD               dwSize;									// struct size
	char				szType[DH_COMMON_STRING_16];			// person type
} NET_ADD_FACERECOGNITION_PERSON_TYPE_INFO;   

// modify face recongnition person type
typedef struct tagNET_MODIFY_FACERECOGNITION_PERSON_TYPE_INFO
{
    DWORD               dwSize;									// struct size
	char				szType[DH_COMMON_STRING_16];			// person type
	char				szNewType[DH_COMMON_STRING_16];			// new person type
} NET_MODIFY_FACERECOGNITION_PERSON_TYPE_INFO;   

// delete face recongnition person type
typedef struct tagNET_DELETE_FACERECOGNITION_PERSON_TYPE_INFO
{
    DWORD               dwSize;									// struct size
	char				szType[DH_COMMON_STRING_16];			// person type
} NET_DELETE_FACERECOGNITION_PERSON_TYPE_INFO;   

// CLIENT_OperateFaceRecognitionPersonType input parameter
typedef struct tagNET_IN_OPERATE_FACERECOGNITION_PERSON_TYPE
{
    DWORD											dwSize;			// struct size
    EM_OPERATE_FACERECOGNITION_PERSON_TYPE			emOperateType;	// operate type
    void											*pOPerateInfo;	// the space application by the user,please refer to the structure of operate type 
																	// if operate type is NET_PERSON_TYPE_ADD,corresponding to NET_ADD_FACERECONGNITION_PERSON_TYPE_INFO
																	// if operate type is NET_PERSON_TYPE_MODIFY,corresponding to NET_MODIFY_FACERECONGNITION_PERSON_TYPE_INFO
																	// if operate type is NET_PERSON_TYPE_DELETE,corresponding to NET_DELETE_FACERECONGNITION_PERSON_TYPE_INFO
} NET_IN_OPERATE_FACERECOGNITION_PERSON_TYPE;   

// CLIENT_OperateFaceRecognitionPersonType output parameter (when delete face recongnition person type is efficacious)
typedef struct tagNET_OUT_OPERATE_FACERECOGNITION_PERSON_TYPE
{
    DWORD               dwSize;										// struct size
    char                szFailedCode[DH_COMMON_STRING_16];			// failed code
} NET_OUT_OPERATE_FACERECOGNITION_PERSON_TYPE;   

// CLIENT_FindGroupInfo port input parameter
typedef struct tagNET_IN_FIND_GROUP_INFO  
{
  DWORD       dwSize;
  char        szGroupId[DH_COMMON_STRING_64];// staff ID, SN staff, as null means search all staff group info 
}NET_IN_FIND_GROUP_INFO;

// CLIENT_FindGroupInfo port output parameter
typedef struct tagNET_OUT_FIND_GROUP_INFO  
{
  DWORD       dwSize;
  NET_FACERECONGNITION_GROUP_INFO *pGroupInfos;   // staff group info , apply space by user, apply to sizeof(NET_FACERECONGNITION_GROUP_INFO)*nMaxGroupNum
  int         nMaxGroupNum;         			  // current applied group size
  int         nRetGroupNum;         			  // device returned staff group number 
}NET_OUT_FIND_GROUP_INFO;


// CLIENT_SetGroupInfoForChannel port input parameter
typedef struct tagNET_IN_SET_GROUPINFO_FOR_CHANNEL
{
  DWORD       dwSize;
  int         nChannelID;         	// channel no.
  int         nGroupIdNum;          // staff group  
  char        szGroupId[MAX_GOURP_NUM][DH_COMMON_STRING_64]; // staff group ID 
}NET_IN_SET_GROUPINFO_FOR_CHANNEL;
// CLIENT_SetGroupInfoForChannel port output parameter
typedef struct tagNET_OUT_SET_GROUPINFO_FOR_CHANNEL
{
  DWORD        dwSize;
}NET_OUT_SET_GROUPINFO_FOR_CHANNEL;

typedef struct tagNET_CB_FACE_FIND_STATE
{
    DWORD               dwSize;     
    int                 nToken;         //Video synopsis task database main key ID 
    int                 nProgress;      //Normal value: 0-100. 1=Searched token does not exist (When subscribe a search that does not exist or already finished)
    int                 nCurrentCount;  //The human face amount that match current criteria
}NET_CB_FACE_FIND_STATE;
typedef void (CALLBACK *fFaceFindState)(LLONG lLoginID, LLONG lAttachHandle, NET_CB_FACE_FIND_STATE* pstStates, int nStateNum, LDWORD dwUser);

//CLIENT_AttachFaceFindState interface parameter in
typedef struct tagNET_IN_FACE_FIND_STATE
{
    DWORD           dwSize;             //Structure size. Must fill in. 
    int             nTokenNum;          //Search token. 0=subscribe all searched tasks.
    int            *nTokens;            //Search toke,the space application by the user, apply to sizeof(int)*nTokenNum
    fFaceFindState  cbFaceFindState;    //Call function 
    LDWORD          dwUser;             //User data 
}NET_IN_FACE_FIND_STATE;
// CLIENT_AttachFaceFindState interface parameter in
 typedef struct  tagNET_OUT_FACE_FIND_STATE
 {
    DWORD           dwSize;
 }NET_OUT_FACE_FIND_STATE;
// CLIENT_FaceRecognitionSetSearchImageInfo 
typedef struct tagNET_IN_FACE_RECOGNITION_SET_SEARCH_IMAGE_INFO
{
    DWORD               dwSize;
    int                 nChannel;                           // Channel ID
    char                szFilePath[DH_COMMON_STRING_256];   // File path
}NET_IN_FACE_RECOGNITION_SET_SEARCH_IMAGE_INFO;
// CLIENT_FaceRecognitionSetSearchImageInfo 
typedef struct tagNET_OUT_FACE_RECOGNITION_SET_SEARCH_IMAGE_INFO
{
    DWORD                dwSize;
}NET_OUT_FACE_RECOGNITION_SET_SEARCH_IMAGE_INFO;
//////////////////////////////// burn the upload////////////////////////////////

// fBurnCheckCallBack Parameter
typedef struct tagNET_CB_BURN_CHECK_STATE
{
	DWORD		        dwSize;
	const char*         szType;							// Message Type
														// "Checking": Check in
														// "CheckResult": CheckResult
	BOOL				bCheckResult;					// Calibration results for "CheckResult", TRUE-success, FALSE-failure
	BYTE				byProgress;						// Calibration schedule for "Checking", the percentage of 0 to 100
	BYTE				reserved[3];
} NET_CB_BURN_CHECK_STATE;

// Burn verification callback function prototype, lAttachHandle return value is CLIENT_AttachBurnCheckState
typedef void (CALLBACK *fBurnCheckCallBack)(LLONG lLoginID, LLONG lAttachHandle, NET_CB_BURN_CHECK_STATE* pstState, void* reserved, LDWORD dwUser);

// CLIENT_AttachBurnCheckState Input parameter interface (monitor burn parity status)
typedef struct tagNET_IN_ATTACH_BURN_CHECK 
{
	DWORD				dwSize;
	fBurnCheckCallBack  cbBurnCheck;					// Burn verification callback
	LDWORD              dwUser;							// User data
} NET_IN_ATTACH_BURN_CHECK;

// CLIENT_AttachBurnCheckState Output parameters of the interface (listening burning parity status)
typedef struct tagNET_OUT_ATTACH_BURN_CHECK
{
    DWORD            dwSize;
} NET_OUT_ATTACH_BURN_CHECK;

///////////////////////////////// logical device /////////////////////////////////

typedef struct tagNET_CB_CAMERASTATE
{
	DWORD		        dwSize;
	int                 nChannel;             // channel
	CONNECT_STATE       emConnectState;       // state of connect
}NET_CB_CAMERASTATE;

// CLIENT_AttachCameraState()callback function, pBuf->dwSize == nBufLen
typedef void (CALLBACK *fCameraStateCallBack) (LLONG lLoginID, LLONG lAttachHandle, const NET_CB_CAMERASTATE *pBuf, int nBufLen, LDWORD dwUser);

// CLIENT_AttachCameraState()input param
typedef struct tagNET_IN_CAMERASTATE
{
	DWORD		        dwSize;
	int *               pChannels;             // observation of the channel, if the value = -1,is boservate all channel
											   // the space application by the user,apply to  sizeof(int)*nChannels
	int                 nChannels;             // length of pChannels pointer
    fCameraStateCallBack cbCamera;             // state of callback function
	LDWORD				dwUser;                // user's data
}NET_IN_CAMERASTATE;
typedef struct tagNET_OUT_CAMERASTATE
{
	DWORD		        dwSize;
	int                 nObjectID;              // Object ID,each ID represent a unique object
}NET_OUT_CAMERASTATE;

////////////////////////////////Special version/////////////////////////////////

// Activate device to snapshot overlay card number information 
typedef struct __NET_SNAP_COMMANDINFO 
{
	char				szCardInfo[16];			// Card information
	char				reserved[64];			// Reserved 
} NET_SNAP_COMMANDINFO, LPNET_SNAP_COMMANDINFO;

typedef struct
{
	int					nChannelNum;			// Channel number 
	char				szUseType[32];			// Channel type 
	DWORD				dwStreamSize;			// Stream size(Unit:kb/s)
	char				reserved[32];			// Reserved 
} DHDEV_USE_CHANNEL_STATE;

typedef struct 
{
	char				szUserName[32];			// User name 
	char				szUserGroup[32];		// User group 
	NET_TIME			time;					// Log in time
	int					nOpenedChannelNum;		// Enabled channel amount 
	DHDEV_USE_CHANNEL_STATE	channelInfo[DH_MAX_CHANNUM];
	char                szIpAddress[DH_MAX_IPADDR_LEN_EX];  // ip
    char                reserved[24];
} DHDEV_USER_NET_INFO;

// Network running status information 
typedef	struct 
{
	int					nUserCount;				// User amount
	DHDEV_USER_NET_INFO	stuUserInfo[32];
	char				reserved[256];
}DHDEV_TOTAL_NET_STATE;

typedef struct
{
    char                szIpAddress[DH_MAX_IPADDR_LEN];  // ip
	char                szUserGroup[32];                 // User Group
	char                szUserName[32];                  // User Name
	char                reserved[64];
}DHDEV_USER_REJECT_INFO;

// Reject User
typedef struct
{ 
	int                       nUserCount;				// User Account
	DHDEV_USER_REJECT_INFO    stuUserInfo[10];     
	char				      reserved[256];
}DHDEV_REJECT_USER;

typedef struct
{
	char                szIpAddress[DH_MAX_IPADDR_LEN];  // ip
	char                szUserGroup[32];                 // User Group
	char                szUserName[32];                  // User Name
	int                 nForbiddenTime;                  // Forbidden Time
	char                reserved[64];
}DHDEV_USER_SHIELD_INFO;

// Shield User
typedef struct
{ 
	int                       nUserCount;          // User Account
	DHDEV_USER_SHIELD_INFO    stuUserInfo[10];     
	char				      reserved[256];
}DHDEV_SHIELD_USER;

// longitude latitude
typedef struct
{//  longitude first,then latitude
	char                chPreLogi;        // preflag of longitude:N(north),S(south),W(west),E(east)
	char                chPreLati;        // preflag of latitude:N(north),S(south),W(west),E(east)
	BYTE                reserved[6];      // 
	double              dbLongitude;      // longitude
	double              dbLatitude;       // latitude
}DHDEV_LONGI_LATI;

// NAVIGATION OR SMS
typedef struct
{
	char                szPhoneNum[DH_MAX_PHONE_NO_LEN];    // phone no
	NET_TIME            stMsgTime;                          // time of sending message
	char                szMsgType[DH_MAX_MSGTYPE_LEN];      // type:Navigation message-Navigation,short message--SMS
	char                szSmsContext[DH_MAX_MSG_LEN];       // message to send

	DHDEV_LONGI_LATI    stLogiLati;                         // longitude
	unsigned int        uFlag;                              // 01:marking true longitude
	char                szNavigationType[16];               // TNC,TXZ
	char                szAddress[32];                      // address
	char                szNavigationMode[32];               // mode
	DHDEV_LONGI_LATI    stPassLogiLati[5];                  // passing
	DHDEV_LONGI_LATI    stNoPassLogiLati[5];                // no passing
	BYTE				reserved[256];
}DHDEV_NAVIGATION_SMSS;

// Image watermark setup 
typedef struct __DHDEV_WATERMAKE_CFG 
{
	DWORD				dwSize;
	int					nEnable;				// Enable 
	int					nStream;				// Bit stream(1~n)0- All bit streams
	int					nKey;					// Data type(1- text,2- image )
	char				szLetterData[DH_MAX_WATERMAKE_LETTER];	// text
	char				szData[DH_MAX_WATERMAKE_DATA]; 			// Image data
	BYTE				bReserved[512];			// Reserved
} DHDEV_WATERMAKE_CFG;

// Storage position set structure.  Each channel set separately.Each channel can set several storage types including local,portableand remote storage.
typedef struct 
{
	DWORD				dwSize;
	DWORD				dwLocalMask;			// Local storage mask. Use bit to represent.
												// The first bit:system pre-record. The second bit:scheduled record. The third bit:motion detection record.
												// The fourth bit:alarm record. The fifth bit:card record. The sixth bit:manual record.
	DWORD				dwMobileMask;			// Moving storage mask. Storage mask such as local storage mask.
	int					RemoteType;				// Remote storage type 0: Ftp  1: Smb 
	DWORD				dwRemoteMask;			// Remote storage mask.  Storage mask such as local storage mask.
	DWORD				dwRemoteSecondSelLocal;	// Local storage mask when remote is abnormal
	DWORD				dwRemoteSecondSelMobile;// Moving storage mask when remote is abnormal.
	char				SubRemotePath[MAX_PATH_STOR]; // Remote path. Its length is 240
	DWORD				dwFunctionMask;			// Function shield bit. Use bit to represent bit0 = 1:Shield sanpshot event activate storage position function.
	DWORD				dwAutoSyncMask;			// If synchronous mask; after remote storage network recovery, the local storage data will automatically synchronized to the remote storage.
	BYTE				bAutoSyncRange;			// the time for synchronous from the network synchronous time. In hour. 0:all data  1:data in one hour n:data in n hours
	char				reserved[119];
} DH_STORAGE_STATION_CFG;

#define MAX_ALARM_DECODER_NUM 16
typedef struct  
{
	DWORD				dwAlarmDecoder;			// Now it supports max 8 alarm input ports. Reserved 8 bits for future development.
	BYTE				bDecoderIndex;			// It means it is from which alarm decoder.
	BYTE				bReserved[3];
} ALARM_DECODER;

// Alarm decoder alarm 
typedef struct
{
	int					nAlarmDecoderNum;
	ALARM_DECODER		stuAlarmDecoder[MAX_ALARM_DECODER_NUM];
	BYTE				bReserved[32];
} ALARM_DECODER_ALARM;

//DSP alarm
typedef struct  
{
	BOOL				bError;			//0,DSP normal 1,DSP abnormal
	DWORD				dwErrorMask;	//Bit shows,Nor 0 shows havening this error,0 shows no.(Now alarm has only 1 bit valid)
										//bit		DSP alarm
										//1			DSP downloading failure 
										//2			DSP error
										//3			Format not correct 
										//4			Resolution not support
										//5			Data format not support
										//6			Can't find I frame
	DWORD               dwDecChnnelMask;//Bit shows,alarm decoding channel number,dwErrorMask is DSP fault,incorrect format,incorrect resolution,data format not support
	DWORD               dwDecChnnelMask1;//Bit shows,33-64 is alarm of decoding channel, dwErrorMask is DSP mistake,format is wrong, effective when not support resolution or data format
	BYTE				bReserved[24];
} DSP_ALARM;

// Fiber-optic alarm
typedef struct  
{
	int		nFDDINum;
	BYTE	bAlarm[256];
} ALARM_FDDI_ALARM;

#define ALARM_PTZ_LOCATION_INFO DH_PTZ_LOCATION_INFO

// New audio detection alarm setup 
typedef struct
{
	BOOL				bEnable;				// Enable alarm input 
	int					Volume_min;				// Min volume
	int					Volume_max;				// Max volume
	char				reserved[128];	
	DH_TSECT			stSect[DH_N_WEEKS][DH_N_REC_TSECT];	
	DH_MSG_HANDLE		struHandle;				// Process way 
} DH_AUDIO_DETECT_INFO;

typedef struct  
{
	DWORD				dwSize;
	int					AlarmNum;
	DH_AUDIO_DETECT_INFO AudioDetectAlarm[DH_MAX_AUDIO_IN_NUM];
	char				reserved[256];
} DH_AUDIO_DETECT_CFG;

typedef struct
{
	BOOL				bTourEnable;			// Enable tour
	int					nTourPeriod;			// Tour interval. Unit is second. The value ranges from 5 to 300 
	DWORD				dwChannelMask;			// Tour channel. Use mas bit to represent.
	char				reserved[64];
}DH_VIDEOGROUP_CFG;

// Local matrix control strategy setup 
typedef struct
{
	DWORD				dwSize;
	int					nMatrixNum;				// Matrix amount(Read only )
	DH_VIDEOGROUP_CFG	struVideoGroup[DH_MATRIX_MAXOUT];
	char				reserved[32];
} DHDEV_VIDEO_MATRIX_CFG;   

// WEB path setup 
typedef struct 
{
	DWORD				dwSize;
	BOOL				bSnapEnable;					// Snapshot or not
	int					iSnapInterval;					// Snapshot interval
	char				szHostIp[DH_MAX_IPADDR_LEN];	// HTTP host IP
	WORD				wHostPort;
	int					iMsgInterval;					// Message sending out interval
	char				szUrlState[DH_MAX_URL_LEN];		// Status message upload URL
	char				szUrlImage[DH_MAX_URL_LEN];		// Image upload Url
	char				szDevId[DH_MAX_DEV_ID_LEN];		// Device web serial number
	BYTE				byReserved[2];
} DHDEV_URL_CFG;

// OEM search 
typedef struct  
{
	char				szVendor[DH_MAX_STRING_LEN];
	char				szType[DH_MAX_STRING_LEN];
	char				reserved[128];
} DHDEV_OEM_INFO;


//Video OSD
typedef struct 
{ 
	DWORD	rgbaFrontground;		// Object front view. Use bit to represent:red,green, blue and transparency.
	DWORD	rgbaBackground;			// Object background. Use bit to represent:red,green, blue and transparency
	RECT	rcRelativePos;			// Position. The proportion of border distance and whole length *8191
	BOOL	bPreviewBlend;			// Enable preview overlay	
	BOOL	bEncodeBlend;			// Enable encode overlay
	BYTE    bReserved[4];           // Reserved 
} DH_DVR_VIDEO_WIDGET;

typedef struct 
{
	DH_DVR_VIDEO_WIDGET	StOSD_POS; 								// OSD position and background color.
	char 				SzOSD_Name[DH_VIDEO_OSD_NAME_NUM]; 		// OSD name 
}DH_DVR_VIDEOITEM;
 
// OSD information in each channel
typedef struct 
{
	DWORD 				dwSize;
	DH_DVR_VIDEOITEM 	StOSDTitleOpt [DH_VIDEO_CUSTOM_OSD_NUM];// Each channel OSD information
	BYTE    			bReserved[16];							// Reserved
} DH_DVR_VIDEOOSD_CFG;

// CDMA/GPRS Configuration
// Time structure															    
typedef struct 
{
	BYTE				bEnable;					// Time period enable,1 shows this time period valid,0 shows invalid.
	BYTE				bBeginHour;
	BYTE				bBeginMin;
	BYTE				bBeginSec;
	BYTE				bEndHour;
	BYTE				bEndMin;
	BYTE				bEndSec;
	BYTE    			bReserved;					//Reserved
} DH_3G_TIMESECT, *LPDH_3G_TIMESECT;

typedef struct 
{
	DWORD 				dwSize;
	BOOL				bEnable;					// Wireless module enable
	DWORD               dwTypeMask;                 // Network type mask;Use bit to represent,The first bit:Automatic selection
													// The second bit:TD-SCDMA network;The third bit:WCDMA network;
													// The fourth bit:CDMA 1.x network;The fifth bit:GPRS network;The sixth bit:EVDO network
													// The seventh bit:EVDO network;The eighth bit:WIFI network;
	DWORD               dwNetType;                  // Types of wireless networks,EM_GPRSCDMA_NETWORK_TYPE
	char				szAPN[128];					// Access Point Name
	char				szDialNum[128];				// Dial-up number
	char				szUserName[128];			// Dial-up user name
	char				szPWD[128];					// Dial-up Password
	
	/* Read-only part */
	BOOL				iAccessStat;				// Wireless network registration status
    char				szDevIP[16];				// dial-up IP of Front-end equipment  
	char				szSubNetMask [16];			// Dial-up mask of Front-end equipment
	char				szGateWay[16];				// Dial-up Gateway of Front-end equipment

	int                 iKeepLive;					// Keep-alive time
	DH_3G_TIMESECT		stSect[DH_N_WEEKS][DH_N_TSECT];// 3G dial period,valid time period,start dialing;beyond this period,shut the dial.
	BYTE                byActivate;                 // Whether need to be active by voice or message
	
	BYTE                bySimStat;                  // SIM(UIM) State: 1, 0 (read-only data) be occupied the 1 byte
	char                szIdentify[128];            // identify mode
	bool                bPPPEnable;                 // PPP Dial-up,0- cut down ppp connect,1-ppp dial-up
	BYTE                bPPPState;                  // PPP Dial-up statr(real only),see EM_MOBILE_PPP_STATE
	BYTE                bNetCardState;              // net card state(read only),see EM_3GMOBILE_STATE
	char				szPhyCardNo[32];			// physics card no.
	char				Reserved[4];				// reserved
	BYTE				byPinShow;					// PIN show or not(read only), 0-don't support 1-show, 2-don't show
	BYTE				byPinEnable;				// PIN setting enable(read only), 0-disable, 1-enable
	BYTE				byPinCount;					// the rest of pin setting number(read only), 0~3
	//-------------------------------the following fields in setting up effective after dwSize------------------------------------
	char				szPinNo[16];				// value of PIN, a combination of letters or numbers
} DHDEV_CDMAGPRS_CFG;

// The length of the video package configuration
typedef struct  
{
	DWORD 				dwSize;
	int					nType;						// 0:by time,1:by size
	int					nValue;						// nType = 0:(minutes),nType = 1:(KB)
	char				Reserved[128];				// Reserved
} DHDEV_RECORD_PACKET_CFG;

// (Directional)register the server information
typedef struct __DEV_AUTOREGISTER_INFO 
{
	LONG			lConnectionID;					// Connection ID
	char			szServerIp[DH_MAX_IPADDR_LEN];	// Server IP
	int				nPort;							// Port:0- 65535
	int             nState;                         // Server status:0-Registration failure;1-Registration success; 2-Connection failed
	char			reserved[16];
} DEV_AUTOREGISTER_INFO;

typedef struct __DEV_SERVER_AUTOREGISTER 
{
	DWORD					dwSize;
	int						nRegisterSeverCount;									// Number of registered servers
	DEV_AUTOREGISTER_INFO	stuDevRegisterSeverInfo[DH_CONTROL_AUTO_REGISTER_NUM];	// Server information
	char					reserved[256];
} DEV_SERVER_AUTOREGISTER, *LPDEV_SERVER_AUTOREGISTER;

// Upload burning annex
typedef struct
{
	DWORD				dwSize;		
	char				szSourFile[MAX_PATH_STOR];	// Source file path
	int					nFileSize;					// Source file size,If less than or equal to 0, sdk to calculate file size.
	char				szBurnFile[MAX_PATH_STOR];	// After burning the file name
	BYTE				bReserved[64];
} DHDEV_BURNFILE_TRANS_CFG;

// Update file upload
typedef struct
{
	char                 szFile[MAX_PATH_STOR];      // Update file path
	int                  nFileSize;                  // Update file size  
	BYTE                 byChannelId;                // Channel No.
	BYTE                 byManufactryType;           // Manufacturer type    Please refer to EM_IPC_TYPE
	
	BYTE                 byReserved[126];            // Reserved      
}DHDEV_UPGRADE_FILE_INFO;

// upload black-white list
typedef struct  
{
	char                 szFile[MAX_PATH_STOR];     // path of file
	int                  nFileSize;                 // size of upgrade file
	BYTE                 byFileType;                // type of file.0-black list,1-white list 
	BYTE                 byAction;                  // action,0-overload,1-additional

	BYTE                 byReserved[126];           // reserved
}DHDEV_BLACKWHITE_LIST_INFO;

// download black-white list
typedef struct
{
	char                 szFile[MAX_PATH_STOR];     // path of file
	BYTE                 byFileType;                // type of file,0-black list,1-white of list
	BYTE                 byReserved[127];           // reserved
}DHDEV_LOAD_BLACKWHITE_LIST_INFO;
// Zhengzhou EC_U video data overlay enabling configuration
typedef struct 
{
	BOOL				bEnable;					// Enable
	int					nPort;						// Port
	BYTE				bReserved[64];
}DHDEV_OSD_ENABLE_CFG;

// about vehicle configuration
typedef struct	
{
	BOOL				bAutoBootEnable;				//auto boot enable
	DWORD				dwAutoBootTime;					//auto boot time, Unit is second. The value ranges from 0 to 82800.
	BOOL				bAutoShutDownEnable;			//auto shut down enable
	DWORD				dwAutoShutDownTime;				//auto shut down time ,  Unit is second. The value ranges from 0 to 82800.
	DWORD				dwShutDownDelayTime;			//shut down delay time, Unit is second.
	BYTE				bEventNoDisk;					//1,(Ignore); 2,(Reboot)
	BYTE				bWifiEnable;					//If support car wifi module.
	BYTE				bUpperSpeedEnable;				//if use over speed verdict
	BYTE				bLowerSpeedEnable;				//if use low speed verdict
	DWORD				dwUpperSpeedValue;				//Over speed area(1,1000).km/hour.
	DWORD               dwLowerSpeedValue;              //Low speed area(1,1000).km/hour.
    DWORD               dwUpperSpeedDelayTime;          //Upper Speed Delay Time
	DWORD               dwLowerSpeedDelayTime;          //Lower Speed Delay Time
	DWORD               dwAlarmMaskEnable;              //Search/set OSD alarm information enable, every bit present one external alarm channel, 0:close;1:open
	BOOL                bSpeedOverAlarmRecordEnable;    //Over Speed Alarm Record Enable
	BOOL                bSpeedLowAlarmRecordEnable;     //Low Speed Alarm Record Enable
	BYTE				bReserved[480];					//reserved byte.
} DHDEV_ABOUT_VEHICLE_CFG;

// atm: query overlay ability information
typedef struct  
{
	DWORD				dwDataSource;					//1:Network, 2:Comm, 3:Network&Comm
	char				szProtocol[32][33];				//protocol name
	BYTE				bReserved[256];
} DHDEV_ATM_OVERLAY_GENERAL;
 
// atm: overlay configuration
typedef struct 
{
	DWORD				dwDataSource;					// 1:Network, 2:Comm
	char				szProtocol[33];					// protocol name, from DHDEV_ATM_OVERLAY_GENERAL
	BYTE				bReserved_1[3];
	DWORD				dwChannelMask;					// channel mask
	BYTE				bMode;							// 1:Preview, 2:Encode, 3:Preview&Encode
	BYTE				bLocation;						// 1:LeftTop, 2:LeftBottom, 3:RightTop, 4:RightBottom
	BYTE                bReserved_3[2];
	int                 nLatchTime;                     // display latch time on overlay, instruct the video's close latch time, 0~65535
	BYTE                bReserved_4[3];
	BYTE                bRecordSrcKeyNum;               // key number
	int                 nRecordSrcKey[32];              // key of channels 
	BYTE				bReserved_2[120];
} DHDEV_ATM_OVERLAY_CONFIG;

// atm:overlay set configuration
typedef struct 
{
	DWORD               dwSize;                         // size
	DWORD				dwDataSource;					// 1:Net, 2:Com, 3:Com422
	char				szProtocol[33];					// protocol name
	BYTE				bReserved_1[3];
	DWORD				dwChannelMask[8];				// overlay channel,the mask means: dwChannelMask[0] is channel 1,so on
	BYTE				bMode;							// 1:Preview(preview), 2:Encode(encode), 3:Preview&Encode(preview and encode)
	BYTE				bLocation;						// 1:LeftTop, 2:LeftBottom , 3:RightTop , 4:RightBottom 
	BYTE                bReserved_3[2];           
	int                 nLatchTime;                     // latch time,0~65535s
	BYTE                bReserved_4[3];
	BYTE                bRecordSrcKeyNum;               // number of key word
	int                 nRecordSrcKey[256];             // key word
} DHDEV_ATM_OVERLAY_CONFIG_EX;

// radar configuration - car speed
typedef struct tagNET_RADAR_CARSPEED_INFO
{
    int                 nTriggerLower;                  // lowerbound of trigger value  range 1~255km/h
    int                 nTriggerUpper;                  // upperbound of trigger value  range 1~255km/h
    int                 nLimitLower;                    // lowerbound of limit value    range 5~255km/h
    int                 nLimitUpper;                    // upperbound of limit value    range 5~255km/h
} NET_RADAR_CARSPEED_INFO;

// dahua radar configuration
typedef struct tagDEV_DAHUA_RADAR_CONFIG
{
    int							nAngle;                         // Angle, useful when correcting speed error from radar sensor installation angle. Unit: degree, range: 0~45
    BOOL						bLowSpeed;                      // whether or not enable lowerbounds. if TRUE, the lowerbounds in NET_RADAR_CARSPEED_INFO are valid
    BOOL						bSpeedForSize;                  // whether or not discriminate different car size, if FALSE, only small car speed config (stuSmallCarSpeed) is valid
    NET_RADAR_CARSPEED_INFO	    stuSmallCarSpeed;               // small car speed, if bSpeedForSize is TRUE, nTriggerLower member is ALSO lowerbounds for medium car and large car
    NET_RADAR_CARSPEED_INFO     stuMediumCarSpeed;              // medium car speed, valid only if bSpeedForSize is TRUE, but nTriggerLower member is ALWAYS invalid.
    NET_RADAR_CARSPEED_INFO		stuBigCarSpeed;                 // large car speed, valid only if bSpeedForSize is TRUE, but nTriggerLower member is ALWAYS invalid.
    char						szName[256];                    // device installation address
    int							nSensitivity;                   // Sensitivity, range: 0~5, 0 is most sensitive
    int							nDetectMode;                    // Detect Mode, values: -1  nonsense, 0 approaching in front, 1 leaving in front, 2 approaching behind, 3 leaving behind, 4 in front (approaching or leaving), 5 behind (approaching or leaving)
	BYTE						bReserved[1024];
} DEV_DAHUA_RADAR_CONFIG;

// single radar configuration 
typedef struct tagDEV_RADAR_CONFIG
{
	DWORD							dwSize;
	BOOL							bEnable;                          		// whether to enable function
	int								nPort;							  		// serial port number
	DH_COMM_PROP					stuCommAttr;							// serial port attribute
	int								nAddress;								// device's address;if multiple devices are hung on the serial port,they are distinguished by addresses.
	int								nPreSpeedWait;							// waiting time in the case of speed first
	int								nDelaySpeedWait;						// waiting time in the case of speed later,  range (1 -- 5000ms)
	BOOL							bDahuaRadarEnable;						// if enable dahua radar configuration, range (1 -- 5000ms)
	DEV_DAHUA_RADAR_CONFIG			stuDhRadarConfig;						// Dahua radar parameter configuration
}DEV_RADAR_CONFIG;



#define DH_MAX_BACKUPDEV_NUM			16
#define DH_MAX_BACKUP_NAME				32

// backup device list
typedef struct 
{
	int					nBackupDevNum;												// number of backup devices
	char				szBackupDevNames[DH_MAX_BACKUPDEV_NUM][DH_MAX_BACKUP_NAME]; // backup device name
} DHDEV_BACKUP_LIST, *LPDHDEV_BACKUP_LIST;

// backup device type
typedef enum __BACKUP_TYPE
{
	BT_DHFS = 0,							// dvr file system
	BT_DISK,								// floating disk
	BT_CDRW									// CD
} DHDEV_BACKUP_TYPE;	

// backup device interface
typedef enum __BACKUP_BUS
{
	BB_USB = 0,								// usb
	BB_1394,								// 1394
	BB_IDE,									// ide
	BB_ESATA,								// esata
} DHDEV_BACKUP_BUS;	

typedef struct 
{
	char				szName[32];						// name
	BYTE				byType;							// see BACKUP_TYPE
	BYTE				byBus;							// see BACKUP_BUS
	UINT				nCapability;					// total capability(kBytes)
	UINT				nRemain;						// remain capability(kBytes)
	char				szDirectory[128];				// Remote Directory
} DHDEV_BACKUP_INFO, *LPDHDEV_BACKUP_INFO;

typedef struct 
{
	char				szName[32];						// name 
	UINT				nCapability;					// total capability(kBytes)
	UINT				nRemain;						// remain capability(kBytes)
} DHDEV_BACKUP_FEEDBACK, *LPDHDEV_BACKUP_FEEDBACK;

#define  DH_MAX_BACKUPRECORD_NUM 1024

typedef struct  
{
	char				szDeviceName[DH_MAX_BACKUP_NAME];			//name
	int					nRecordNum;									//number of records
	NET_RECORDFILE_INFO	stuRecordInfo[DH_MAX_BACKUPRECORD_NUM];		//record information
} BACKUP_RECORD, *LPBACKUP_RECORD;

// multiplay input param
typedef struct
{
	int                 iChannelID;      // channel id
	DH_RealPlayType     realplayType;    // real play type
	char                reserve[32];
}DHDEV_IN_MULTIPLAY_PARAM;

// multiplay output param
typedef struct
{
	int                 iChannelID;      // channel id
	LLONG                lRealHandle;    // real play handle
	char                reserve[32];
}DHDEV_OUT_MULTIPLAY_PARAM;
/////////////////////////////////Embedded platform/////////////////////////////////

// Platform embedded setup - U China Network Communication(CNC)
typedef struct
{
    BOOL				bChnEn;
    char				szChnId[DH_INTERVIDEO_UCOM_CHANID];
} DH_INTERVIDEO_UCOM_CHN_CFG;

typedef struct
{
	DWORD				dwSize;
	BOOL				bFuncEnable;			// Enable connection
	BOOL				bAliveEnable;			// Enable alive
	DWORD				dwAlivePeriod;			// Alive period. Unit is second. The value ranges from 0 to 3600.
	char				szServerIp[DH_MAX_IPADDR_LEN];			// CMS IP
	WORD				wServerPort;							// CMS Port
    char				szRegPwd[DH_INTERVIDEO_UCOM_REGPSW];	// Registration password 
	char				szDeviceId[DH_INTERVIDEO_UCOM_DEVID];	// Device id
	char				szUserName[DH_INTERVIDEO_UCOM_USERNAME];
	char				szPassWord[DH_INTERVIDEO_UCOM_USERPSW];
    DH_INTERVIDEO_UCOM_CHN_CFG  struChnInfo[DH_MAX_CHANNUM];	// Channel id,en
} DHDEV_INTERVIDEO_UCOM_CFG;

//  Platform embedded setup - Alcatel
typedef struct
{
	DWORD				dwSize;
    unsigned short		usCompanyID[2];			// Company ID,Value. the different three party service company. Now use the first array considering the four bytes in line.
    char				szDeviceNO[32];			// Strings including '\0' are 32 bytes.
    char				szVSName[32];			// Front-end device name. Strings including '\0' are 16 bytes.
    char				szVapPath[32];			// VAP path
    unsigned short		usTcpPort;				// TCP port,value:value ranges from 1 to 65535 
    unsigned short		usUdpPort;				// UDP port,Value:Value ranges from 1 to 65535
    bool				bCsEnable[4];			// Enable central server symbol. Value:true=enable,false=disable.Now only use the first array considering the four bytes in line.
    char				szCsIP[16];				// Central server IP address.Strings including '\0' are 16 bytes.
    unsigned short		usCsPort[2];			// Central server port. Value ranges from 1 to 65535.Now only use the first array considering the four bytes in line.
    bool				bHsEnable[4];			// Enable pulse server symbol. Value:true-enable,false=disable.Now only use the first array considering the four bytes in line.
    char				szHsIP[16];				// Pulse server IP address. Strings including '\0' are 16 bytes.
    unsigned short		usHsPort[2];			// Pulse server port. Value ranges from 1 to 65535.Now only use the first array considering the four bytes in line
    int					iHsIntervalTime;		// Pulse server interval. Value(unit is second)
    bool				bRsEnable[4];			// Enable registration server symbol. Value:,true=enable,false=disable.Now only use the first array considering the four bytes in line. 
    char				szRsIP[16];				// Registration server IP address. Strings including '\0' are 16 bytes.
    unsigned short		usRsPort[2];			// Registration server port. Value. The value ranges from 1 to 65535.Now use the first array considering the four bytes in line
    int					iRsAgedTime;			// Registration server valid duration. Value(unit is hour)
    char				szAuthorizeServerIp[16];// IP Authorization server IP
    unsigned short		usAuthorizePort[2];		// Authorization server port. Now only use the first array considering the four bytes in line
    char				szAuthorizeUsername[32];// Authorization server account
    char				szAuthorizePassword[36];// Authorization server password
    
    char				szIpACS[16];			// ACS( auto registration server) IP
    unsigned short		usPortACS[2];			// ACS Port,Now only use the first array considering the four bytes in line. 
    char				szUsernameACS[32];		// ACS user name
    char				szPasswordACS[36];		// ACS password
    bool				bVideoMonitorEnabled[4];// DVS regularly uploads front-end video monitor message or not.Value:true=enable,false=disable
    int					iVideoMonitorInterval;	// Upload interval(minute)
    
    char				szCoordinateGPS[64];	// GPS coordinates
    char				szPosition[32];			// Device position
    char				szConnPass[36];			// Device connection bit 
} DHDEV_INTERVIDEO_BELL_CFG;

//  Platform embedded setup - ZTE Netview
typedef struct  
{
	DWORD				dwSize;
	unsigned short		nSevPort;								// Server port. Value. The value ranges from 1 to 65535
    char				szSevIp[DH_INTERVIDEO_NSS_IP];			// Server IP address,string,including '\0' end symbol, there are total 32byte.
    char				szDevSerial[DH_INTERVIDEO_NSS_SERIAL];	// Front-end device serial number, string including '\0' end symbol, there are total 32byte.
    char				szUserName[DH_INTERVIDEO_NSS_USER];
    char				szPwd[DH_INTERVIDEO_NSS_PWD];
} DHDEV_INTERVIDEO_NSS_CFG;

// Platform embedded setup - AMP
typedef struct  
{
	char               szDevSerial[DH_INTERVIDEO_AMP_DEVICESERIAL];                  // Front Device Serial num(encoder serial num), read only	
	char               szDevName[DH_INTERVIDEO_AMP_DEVICENAME];                      // Front Device Name(encoder name), read only
	char               szRegUserName[DH_INTERVIDEO_AMP_USER];                        // Server Name
	char               szRegPwd[DH_INTERVIDEO_AMP_PWD];                              // Server Password
	BYTE			   bReserved[128];
} DHDEV_INTERVIDEO_AMP_CFG;  
////////////////////////////////HDVR special use//////////////////////////////////
// Alarm relay structure
typedef struct 
{
	/* Message triggered methods,multiple methods,including
	 * 0x00000001 - alarm upload
	 * 0x00000002 - triggering recording
	 * 0x00000004 - PTZ movement
	 * 0x00000008 - sending email
	 * 0x00000010 - local tour
	 * 0x00000020 - local tips
	 * 0x00000040 - alarm output
	 * 0x00000080 - ftp upload
	 * 0x00000100 - buzzer
	 * 0x00000200 - voice tips 
	 * 0x00000400 - snapshot
	*/

	/* Current alarm supporting methods, shown by bit mask */
	DWORD				dwActionMask;

	/* Triggering action,shown by bit mask,concrete action parameter is shows in the configuration */
	DWORD				dwActionFlag;
	
	/* Triggering alarm output channel,1 means triggering this output */ 
	BYTE				byRelAlarmOut[DH_MAX_ALARMOUT_NUM_EX];
	DWORD				dwDuration;				/* Alarm lasting period */

	/* Triggering recording */
	BYTE				byRecordChannel[DH_MAX_VIDEO_IN_NUM_EX]; /* Record channel triggered by alarm,1 means triggering this channel */
	DWORD				dwRecLatch;				/* Recording period */

	/* Snapshot channel */
	BYTE				bySnap[DH_MAX_VIDEO_IN_NUM_EX];
	/* Tour channel */
	BYTE				byTour[DH_MAX_VIDEO_IN_NUM_EX];

	/* PTZ movement */
	DH_PTZ_LINK			struPtzLink[DH_MAX_VIDEO_IN_NUM_EX];
	DWORD				dwEventLatch;			/* Event delay time, s for unit,range is 0~15,default is 0 */
	/* Alarm trigerring wireless output,alarm output,1 for trigerring output */ 
	BYTE				byRelWIAlarmOut[DH_MAX_ALARMOUT_NUM_EX];
	BYTE				bMessageToNet;
	BYTE                bMMSEn;                /*Message triggering alarm enabling*/
	BYTE                bySnapshotTimes;       /* the number of sheets of drawings */
	BYTE				bMatrixEn;				/*!< Matrix output enable */
	DWORD				dwMatrix;				/*!< Matrix mask */			
	BYTE				bLog;					/*!< Log enable,only used in WTN motion detection */
	BYTE				bSnapshotPeriod;		/*!<Snapshot frame interval. System can snapshot regularly at the interval you specify here. The snapshot amount in a period of time also has relationship with the snapshot frame rate. 0 means there is no interval, system just snapshot continuously.*/
	BYTE				byTour2[DH_MAX_VIDEO_IN_NUM_EX];/* Tour channel 32-63*/
	BYTE                byEmailType;             /*<0,picture,1,record>*/
	BYTE                byEmailMaxLength;        /*<max record length,unit:MB>*/
	BYTE                byEmailMaxTime;          /*<max time length, unit:second>*/
	BYTE				byReserved[475];   
} DH_MSG_HANDLE_EX;

// External alarm expansion
typedef struct
{
	BYTE				byAlarmType;			// Alarm type,0:normal closed,1:normal open
	BYTE				byAlarmEn;				// Alarm enable
	BYTE				byReserved[2];						
	DH_TSECT			stSect[DH_N_WEEKS][DH_N_REC_TSECT]; //NSP
	DH_MSG_HANDLE_EX	struHandle;				// Handle method
} DH_ALARMIN_CFG_EX, *LPDHDEV_ALARMIN_CFG_EX; 

// Motion detection alarm
typedef struct 
{
	BYTE				byMotionEn;				// Motion detection alarm enabling
	BYTE				byReserved;
	WORD				wSenseLevel;			// Sensitivity
	WORD				wMotionRow;				// Motion detection area rows
	WORD				wMotionCol;				// Motion detection area lines
	BYTE				byDetected[DH_MOTION_ROW][DH_MOTION_COL]; // Motion detection area,most 32*32
	DH_TSECT			stSect[DH_N_WEEKS][DH_N_REC_TSECT];		  // NSP
	DH_MSG_HANDLE_EX	struHandle;				//Handle method
} DH_MOTION_DETECT_CFG_EX;

// fire alarm
typedef struct
{
	BOOL                byFireEn;              // enable
	DH_MSG_HANDLE_EX    struHandle;            // handler
	BYTE                byReserved[128];
}DHDEV_FIRE_ALARM_CFG;

// Static detection alarm

typedef struct 
{
	BYTE				byStaticEn;				// Static detection alarm enabling
	BYTE				byLatch;                // detect delay (old struct)
	WORD				wSenseLevel;			// Sensitivity
	WORD				wStaticRow;				// Static detection area rows
	WORD				wStaticCol;				// Static detection area lines
	BYTE				byDetected[DH_STATIC_ROW][DH_STATIC_COL]; // Static detection area,most 32*32
	DH_TSECT			stSect[DH_N_WEEKS][DH_N_REC_TSECT];	//NSP
	DH_MSG_HANDLE_EX	struHandle;				//Handle method
	int                 nLatch;                 // detect delay (new struct)
	BYTE                bReserved[64];          // reserved
} DH_STATIC_DETECT_CFG_EX;

// ACC power off configuration
typedef struct _DHDEV_ACC_POWEROFF_CFG
{
	BOOL                bEnable;               // ACC power off configuration enable
	int                 nDelayTime;            // ACC power off delay time(m)
	DH_MSG_HANDLE_EX    struHandle;            // handle method
	BYTE                Reserved[128];         // reserved
}DHDEV_ACC_POWEROFF_CFG;

// explosion proof alarm configuration
typedef struct _DHDEV_EXPLOSION_PROOF_CFG
{
	BOOL                bEnable;               // explosion proof alarm configuration enable
	DH_MSG_HANDLE_EX    struHandle;            // handle method
	BYTE                Reserved[128];         // reserved
}DHDEV_EXPLOSION_PROOF_CFG;

// Raid evnet config
typedef struct _DHDEV_RAID_EVENT_CFG
{
	BOOL                bEnable;               // enable
	DH_MSG_HANDLE_EX    struHandle;            // handle
	BYTE                Reserved[128];         // 
}DHDEV_RAID_EVENT_CFG;

// Video loss alarm
typedef struct
{
	BYTE				byAlarmEn;				// Video loss alarm enabling
	BYTE				byReserved[3];
	DH_TSECT			stSect[DH_N_WEEKS][DH_N_REC_TSECT];	//NSP
	DH_MSG_HANDLE_EX	struHandle;				// Handle method
} DH_VIDEO_LOST_CFG_EX;

// Camera masking alarm
typedef struct
{
	BYTE				byBlindEnable;			// Enable
	BYTE				byBlindLevel;			// Sensitivity 1-6
	BYTE				byReserved[2];
	DH_TSECT			stSect[DH_N_WEEKS][DH_N_REC_TSECT];	//NSP
	DH_MSG_HANDLE_EX	struHandle;				// Handle method
} DH_BLIND_CFG_EX;

// HDD info(alarm)
typedef struct 
{
	BYTE				byNoDiskEn;				// No HDD alarm
	BYTE				byReserved_1[3];
	DH_TSECT			stNDSect[DH_N_WEEKS][DH_N_REC_TSECT]; //NSP
	DH_MSG_HANDLE_EX	struNDHandle;			// Handle method
	
	BYTE				byLowCapEn;				// Low capacity alarm
	BYTE				byLowerLimit;			// Lower limit,0-99
	BYTE				byReserved_2[2];
	DH_TSECT			stLCSect[DH_N_WEEKS][DH_N_REC_TSECT]; //NSP
	DH_MSG_HANDLE_EX	struLCHandle;			// Handle method
	
	BYTE				byDiskErrEn;			// HDD error alarm
	BYTE				bDiskNum;
	BYTE				byReserved_3[2];
	DH_TSECT			stEDSect[DH_N_WEEKS][DH_N_REC_TSECT]; //NSP
	DH_MSG_HANDLE_EX	struEDHandle;			// Handle method
} DH_DISK_ALARM_CFG_EX;

typedef struct
{
	BYTE				byEnable;
	BYTE				byReserved[3];
	DH_MSG_HANDLE_EX	struHandle;
} DH_NETBROKEN_ALARM_CFG_EX;

// Front encoder configuration parameter
typedef struct __DEV_ENCODER_CFG 
{
	int					nChannels;				// Digital channel number
	DEV_ENCODER_INFO	stuDevInfo[32];			// Digital channel info
	BYTE				byHDAbility;			// The max high definition video amount the digital channel supported. (0 means it does not support high definition setup.)
												// If it supports high definition setup, the high definition channel ranges from 0 to N-1. If your high definition channel number is more than N, the save operation may fail. 
	BYTE				bTVAdjust;				// TV adjust. 0:not support, 1:support.
	BYTE				bDecodeTour;			// Decode tour. 0:not support, greater than zero: it means the maximal number supported.
	BYTE				bRemotePTZCtl;			// Remote PTZ control. 0:not support, 1:support.
	char				reserved[12];
} DEV_ENCODER_CFG, *LPDEV_ENCODER_CFG;

// front end access configuratiion parameter expansion
typedef struct __DEV_ENCODER_CFG_EX 
{
	int					nChannels;				// number of channel
	DEV_ENCODER_INFO	stuDevInfo[128];	    // the front end of the every digital channel encoder information
	BYTE				byHDAbility;			// maximum number of hd video(0 means not support)

	BYTE				bTVAdjust;				// support TV adjust,0:not support,1:support.
	BYTE				bDecodeTour;			// support tour or not, 0:not support, >0:the current number of device support
	BYTE				bRemotePTZCtl;			// support remote PTZ control
	char				reserved[256];
} DEV_ENCODER_CFG_EX, *LPDEV_ENCODER_CFG_EX;
// Controller
typedef struct tagDHCONFIG_CONTROLER{
	DH_COMM_PROP        struComm;	                     // Com attribution
	BYTE                bLightGroup[DH_MAX_LIGHT_NUM];   // Controlled light group,light number start from 1,such as[1,3,8,0...0]present control light number 1,3,8 light
	BYTE	            bDeviceAddr;	                 // Controller address
	BYTE                bComPortType;                    // Com Type 0:485 COM, 1:232 COM
	BYTE                bReserved[6];		             // Reserved
} DH_CONFIG_CONTROLER;

// Light control config
typedef struct __DHDEV_LIGHTCONTROL_CFG
{
	DH_CONFIG_CONTROLER    struControlers[DH_MAX_CONTROLER_NUM]; // Control config
	BYTE                   bControlerNum;                        // Effective controller number
	BYTE                   bReserved[1023];                      // Reserved
} DHDEV_LIGHTCONTROL_CFG, *LPDHDEV_LIGHTCONTROL_CFG;

// 3G Flow info config
typedef struct
{
	int                 nStrategy;       // strategy, 0: charged by flow every month 1:charged by time every month
	int                 nUpLimit;        // up limit, by flow: MB, by time: h
	int                 nreserved[22];   // reserved
}DHDEV_3GFLOW_INFO_CFG;

// IPv6 config
typedef struct 
{
	char               szLinkAddr[44];	 // link address: string length = 44;(every host has an exclusive link address, read only)
	char               szHostIP[40]; 	 // host IP
	DWORD              dwPreFix;		 // net prefix, 1-128
	char               szGateWay[40];	 // gateway
	BOOL               bAutoGet;		 // enable of auto get ip 
	char               szPrimaryDns[40]; // primary DNS
	char               szSecondDns[40];	 // second DNS
    char               cReserved[256];   // reserved
}DHDEV_IPV6_CFG;

//Emergency storage configuration
typedef struct __DEV_URGENCY_RECORD_CFG
{
	DWORD dwSize;           // Structure body size
	BOOL bEnable;           // Enable or not. 1=enable.0=disable
	UINT nMaxRecordTime;    // Max record time. Unit is second.
}DHDEV_URGENCY_RECORD_CFG;

// Lift running parameter configuration
typedef struct __DEV_ELEVATOR_ATTRI_CFG
{
	DWORD  dwSize;           //Structure body size
	UINT   nMAXFloorNum;     //Max floor. Min is 2.
	UINT   nFloorDelayTime;  //Stay verification time ranges from 5-60. It regards as stop current floor if keep this time period.
	UINT   nIntervalTime;    //The max time for the lift to go up/down a floor. It regars the lift is malfunction if the time is out of the threshold. The lift stopsa between two floors.
}DHDEV_ELEVATOR_ATTRI_CFG;

// Virtual camera status search
typedef struct tagDHDEV_VIRTUALCAMERA_STATE_INFO
{
	DWORD              nStructSize;                  // Structure body size
	int                nChannelID;                   // Channel No.
	CONNECT_STATE      emConnectState;               // Connection status
	UINT               uiPOEPort;                    // The PoE port the virtual camera connected to. 0=It is not a PoE connection.
	char               szDeviceName[64];             // Device name
	char               szDeviceType[128];            // Device type
	char               szSystemType[128];            // system type
	char               szSerialNo[DH_SERIALNO_LEN];  // serial no
	int                nVideoInput;                  // video input number
	int                nAudioInput;                  // audio input number
	int                nAlarmOutput;                 // alarm output number
}DHDEV_VIRTUALCAMERA_STATE_INFO;

// Device working video/loop mode status info and etc search 
typedef struct tagDHDEV_TRAFFICWORKSTATE_INFO
{
	DWORD                nStructSize;      // Structure body size
	int                  nChannelID;       // Channel No.
	DH_TRAFFIC_SNAP_MODE emSnapMode;       // Snap mode
}DHDEV_TRAFFICWORKSTATE_INFO;

typedef enum tagEM_NET_DEFENCE_AREA_TYPE
{
    EM_NET_DEFENCE_AREA_TYPE_UNKNOW,                    // unknown
    EM_NET_DEFENCE_AREA_TYPE_INTIME,                    // realtime protection area  
    EM_NET_DEFENCE_AREA_TYPE_DELAY,                     // delay protection area
    EM_NET_DEFENCE_AREA_TYPE_FULLDAY,                   // 24h protection area
    EM_NET_DEFENCE_AREA_TYPE_Follow,                    // follow Protection area
    EM_NET_DEFENCE_AREA_TYPE_MEDICAL,                   // medical protection area
    EM_NET_DEFENCE_AREA_TYPE_PANIC,                     // panic Protection area
    EM_NET_DEFENCE_AREA_TYPE_FIRE,                      // fire Protection area
    EM_NET_DEFENCE_AREA_TYPE_FULLDAYSOUND,              // 24h sound Protection area
    EM_NET_DEFENCE_AREA_TYPE_FULLDATSLIENT,             // 24h slient Protection area
    EM_NET_DEFENCE_AREA_TYPE_ENTRANCE1,                 // entrance Protection area 1 
    EM_NET_DEFENCE_AREA_TYPE_ENTRANCE2,                 // entrance Protection area 2
    EM_NET_DEFENCE_AREA_TYPE_INSIDE,                    // inside Protection area
    EM_NET_DEFENCE_AREA_TYPE_OUTSIDE,                   // outside Protection area
    EN_NET_DEFENCE_AREA_TYPE_PEOPLEDETECT,              // people detect Protection area
} EM_NET_DEFENCE_AREA_TYPE;

// Local Alarm Event (DH_ALARM_ALARM_EX Update
typedef struct tagALARM_ALARM_INFO_EX2
{
    DWORD		                     dwSize;
    int			                     nChannelID;            // Channel ID
    int			                     nAction;				// 0=Start 1=Stop 
    NET_TIME	                     stuTime;				// Alarm Event Begin Time
    NET_SENSE_METHOD                 emSenseType;		    // The Sensor's Type
    DH_MSG_HANDLE_EX                 stuEventHandler;       // Handle method
    EM_NET_DEFENCE_AREA_TYPE         emDefenceAreaType;     // protection area type, refer to EM_NET_DEFENCE_AREA_TYPE 	
    UINT 							 nEventID;				// event id
    char                             szName[DH_COMMON_STRING_32];  // Channel Name
    int                              nCount;                 // evnet count
}ALARM_ALARM_INFO_EX2;

/////////////////////////////////ITS picture subscription interface parameter/////////////////////////////////
typedef struct RESERVED_DATA_INTEL_BOX
{
	DWORD  dwEventCount;	 //Event count
	DWORD* dwPtrEventType;	 //Point to continuous value of event type, user should request and fr,apply to sizeof(DWORD)*dwEventCount
    DWORD  dwInternalTime;   //picture upload interval time, unit:second
	BYTE   bReserved[1020];	 //Reserved
}ReservedDataIntelBox;

#define RESERVED_TYPE_FOR_INTEL_BOX 0x00000001
typedef struct RESERVED_PARA
{
	DWORD 	dwType;	//pData's type
					//when [dwType] is RESERVED_TYPE_FOR_INTEL_BOX, pData is address of type ReservedDataIntelBox					
					//when [dwType] is ...,[pData] is ...
	void*	pData;	//data,the space application by the user, please refer to corresponding structure of pData's type
}ReservedPara;
#define RESERVED_TYPE_FOR_COMMON   0x00000010
typedef struct tagNET_RESERVED_COMMON
{
	DWORD            dwStructSize;
	ReservedDataIntelBox* pIntelBox;    // include RESERVED_TYPE_FOR_INTEL_BOX
	DWORD            dwSnapFlagMask;	// snap flags(by bit),0bit:"*",1bit:"Timing",2bit:"Manual",3bit:"Marked",4bit:"Event",5bit:"Mosaic",6bit:"Cutout"
}NET_RESERVED_COMMON;

#define RESERVED_TYPE_FOR_PATH  0x00000100
// picture path type
typedef enum tagNET_PICTURE_PATH_TYPE
{
    NET_PATH_LOCAL_PATH     ,   // writing path for loacl disk or sd card
    NET_PATH_FTP_PATH       ,   // ftp path
    NET_PATH_VIDEO_PATH     ,   // ftp path for assocated video
}NET_PICTURE_PATH_TYPE;

#define MAX_PIC_PATH_NUM        16
typedef struct tagNET_RESERVED_PATH
{
    int                     nMaxPathNum;                        //picture path count, device will use default path if 0
    NET_PICTURE_PATH_TYPE   emPictruePaths[MAX_PIC_PATH_NUM];   //picture path type
}NET_RESERVED_PATH;

/////////////////////////////////Intelligent speed dome control interface parameter/////////////////////////////////
// Scene structure info
typedef struct 
{
	DWORD       dwSize;
	int         nScene;			//Scene SN
}DHDEV_INTELLI_SCENE_INFO;

// Scene margin limit position info
typedef struct 
{
	DWORD       dwSize;
	int         nScene;			//Scene SN
	int         nType;			//0:Top margin limit;1:Bottom margin limit;2:Left margin;3:Right margin
}DHDEV_INTELLI_SCENELIMIT_INFO;

// Manually track object structure body info
typedef struct
{
	DWORD       dwSize;
	int         nObjectID;		// Object ID -1 = Set the intelligent frame upload the position of any object out of the frame in the Web >=0: select the intelligent frame to send the object in the frame. 
	RECT        stuBound;		// Rectangle range. The coordinates of the point is within [0,8192].
}DHDEV_INTELLI_TRACKOBJECT_INFO;

typedef enum __TRACKCONTROL_TYPE
{
	DH_TRACK_MARKSCENE,				// Specified scene. Corresponding to the structure body of the DHDEV_INTELLI_SCENE_INFO
		DH_TRACK_GOTOSCENE,			// Go to scene. Corresponding to the structure body of the DHDEV_INTELLI_SCENE_INFO
		DH_TRACK_MARKSCENELIMIT,	// The margin limit position of the specified scene. Corresponding to the structure body of the DHDEV_INTELLI_SCENELIMIT_INFO
		DH_TRACK_GOTOSCENELIMIT,	// Go to the marin limit position of the scene. Corresponding to the structure body of the DHDEV_INTELLI_SCENELIMIT_INFO
		DH_TRACK_MARKSCENEMAXZOOM,	// The max track rate in the specified scene. Corresponding to the structure body of the DHDEV_INTELLI_SCENE_INFO
		DH_TRACK_OBJECT,			// The tracking object in the selected scene. Corresponding to the structure body of the DHDEV_INTELLI_TRACKOBJECT_INFO
		DH_TRACK_START,				// Begin intelligent track. No need to specify parameter information.
		DH_TRACK_STOP,				// Stop intelligent track. No need to specify parameter information.
		DH_TRACK_TYPE_NUM,
}DH_TRACKCONTROL_TYPE;

//Intelligent speed dome control input parameter
typedef struct tagNET_IN_CONTROL_INTELLITRACKER
{
    DWORD       dwSize;
    int         nChannelID;            // Channel ID
    DH_TRACKCONTROL_TYPE emCtrlType;   // Control type
    void*       pCtrlInfo;             // The corresponding inform structure pointer of the control type. Please refer to DH_TRACKCONTROL_TYPE for definition.the space application by the user.
    int         nMaxLen;               // Structure body size of the control information
	int         nWaittime;             // Wait time out time
}NET_IN_CONTROL_INTELLITRACKER;

//Intelligent speed dome control output parameter
typedef struct tagNET_OUT_CONTROL_INTELLITRACKER
{
    DWORD       dwSize;
}NET_OUT_CONTROL_INTELLITRACKER;

/////////////////////////////////Video Analysis Interface Parameters//////////////////////////////
#define MAX_TEMPLATEIMAGE_NUM               32                      // Limitation of valid images quantities.
 
//Get the inputs of video analyses target
 typedef struct tagNET_IN_VIDEOANALYSE_GETTEMPLATEIMAGE
{
    DWORD               dwSize;
    int                 nChannelID;                                 // channel ID
	char                szRule[DH_COMMON_STRING_128];               // The correspondence rule of template image
 }NET_IN_VIDEOANALYSE_GETTEMPLATEIMAGE;

//Get the outputs of video analyses target
typedef struct tagNET_OUT_VIDEOANALYSE_GETTEMPLATEIMAGE
{
    DWORD               dwSize;
    unsigned int        nImageNum;                                  // The number of valid images
    DH_PIC_INFO_EX2     stuImage[MAX_TEMPLATEIMAGE_NUM];            // Image information
	char*               pBuffer;                                    // The pointer of image buffer, applied by users
    int                 nBufLen;                                    // The size of image buffer, filled in by users
	int                 nRetLen;                                    // Get the length of the image data, no need to fill in by user, which can be used to decide whether the buffer is enough.
 
}NET_OUT_VIDEOANALYSE_GETTEMPLATEIMAGE;

//Setup the input parameters of video analyses target
typedef struct tagNET_IN_VIDEOANALYSE_SETTEMPLATEIMAGE
{
    DWORD               dwSize;
    int                 nChannelID;                                 // Channel ID
    char                szRule[DH_COMMON_STRING_128];               // The correspondence rule of template image
    unsigned int        nImageNum;                                  // The number of valid images

    DH_PIC_INFO_EX2     stuImage[MAX_TEMPLATEIMAGE_NUM];            // Image information
	char*               pBuffer;                                    // The pointer of image buffer, the space application by the user,applt to nBufLen
    int                 nBufLen;                                    // Image buffer size
}NET_IN_VIDEOANALYSE_SETTEMPLATEIMAGE;

//Setup the output parameters of video analyses target
typedef struct tagNET_OUT_VIDEOANALYSE_SETTEMPLATEIMAGE
{
    DWORD       dwSize;
}NET_OUT_VIDEOANALYSE_SETTEMPLATEIMAGE;

/////////////////////////////////Abandoned Type/////////////////////////////////

// Search device working status channel information. Corresponding interfaces have been abandoned. Do not user.
typedef struct
{
	BYTE				byRecordStatic;			// Channel is recording or not. ;0: does not record,1: manual record,2:auto record  
	BYTE				bySignalStatic;			// Connected signal status ;0:normal,1:signal loss  
	BYTE				byHardwareStatic;		// Channel hardware status;0:normal ,1:abnormal such as DSP is down , 
	char				reserve;
	DWORD				dwBitRate;				// Actual bit rate
	DWORD				dwLinkNum;				// The client-end connected amount 
	DWORD				dwClientIP[DH_MAX_LINK];// Client-end IP address 
} NET_DEV_CHANNELSTATE, *LPNET_DEV_CHANNELSTATE;

// Search device working status. The corresponding interfaces have been abandoned. Please do not use
typedef struct
{
	DWORD				dwDeviceStatic;						// Device status;0x00:normal ,0x01:CPU occupation is too high ,0x02:hardware error 
	NET_DEV_DISKSTATE	stHardDiskStatic[DH_MAX_DISKNUM]; 
	NET_DEV_CHANNELSTATE stChanStatic[DH_MAX_CHANNUM];		// Channel status
	BYTE				byAlarmInStatic[DH_MAX_ALARMIN];	// Alarm port status;0:no alarm,1: alarm
	BYTE				byAlarmOutStatic[DH_MAX_ALARMOUT]; 	// Alarm output port status;0:no alarm,1:alarm  
	DWORD				dwLocalDisplay;						// Local display status  ;0:normal,1:abnormal 
} NET_DEV_WORKSTATE, *LPNET_DEV_WORKSTATE;

// Protocol information
typedef struct 
{
	char				protocolname[12];		// Protocol name
	unsigned int		baudbase;				// Baud rate
	unsigned char		databits;				// Data bit
	unsigned char		stopbits;				// Stop bit
	unsigned char		parity;					// Pairty bit 
	unsigned char		reserve;
} PROTOCOL_INFO, *LPPROTOCOL_INFO;

// Audio talk parameter setup
typedef struct 
{
	// Audio input parameter
	BYTE				byInFormatTag;			// Encode type such as PCM
	BYTE				byInChannels;			// Track amount
	WORD				wInBitsPerSample;		// Sampling depth 				
	DWORD				dwInSamplesPerSec;		// Sampling rate 

	// Audio output parameter
	BYTE				byOutFormatTag;			// Encode type such as PCM
	BYTE				byOutChannels;			// Track amount
	WORD				wOutBitsPerSample;		// Sampling depth 		
	DWORD				dwOutSamplesPerSec;		// Sampling rate
} DHDEV_TALK_CFG, *LPDHDEV_TALK_CFG;

/////////////////////////////////// Matrix ///////////////////////////////////////

#define DH_MATRIX_INTERFACE_LEN		16		// Signal interface name length
#define DH_MATRIX_MAX_CARDS			128		// Matrix sub card max amount
#define DH_SPLIT_PIP_BASE			1000	// The basic value of the PIP split mode
#define	DH_MAX_SPLIT_MODE_NUM		64		// Max split matrix amount
#define DH_MATRIX_MAX_CHANNEL_IN	1500	// Matrix max input channel amount
#define DH_MATRIX_MAX_CHANNEL_OUT	256		// Matrix max output channel amount
#define DH_DEVICE_NAME_LEN			64		// Device name length
#define DH_MAX_CPU_NUM				16		// Max CPU amount
#define DH_MAX_FAN_NUM				16		// Max fan amount
#define DH_MAX_POWER_NUM			16		// Max power amount
#define DH_MAX_BATTERY_NUM          16      // Max battery quantity
#define DH_MAX_TEMPERATURE_NUM      256		// Max temperature sensor amount 
#define DH_MAX_ISCSI_NAME_LEN		128		// ISCSI Name length
#define DH_VERSION_LEN				64		// Version info length
#define DH_MAX_STORAGE_PARTITION_NUM 32		//  Storage partition max number
#define DH_STORAGE_MOUNT_LEN		64		// Mount length
#define DH_STORAGE_FILE_SYSTEM_LEN	16		// File system name length
#define DH_MAX_MEMBER_PER_RAID		32		// RAID max members
#define DH_DEV_ID_LEN_EX			128		// Device ID max length
#define DH_MAX_BLOCK_NUM			32		// max number of block
#define DH_MAX_SPLIT_WINDOW			128		// max number of split window
#define DH_FILE_TYPE_LEN			64		// length of file type
#define DH_DEV_ID_LEN				128		// length of device ID
#define DH_DEV_NAME_LEN				128		// max length of device name  
#define DH_TSCHE_DAY_NUM			8		// schedule the first dimension size, means days
#define DH_TSCHE_SEC_NUM			6		// schedule the second dimension size, means time
#define    DH_SPLIT_INPUT_NUM       256     // Judicial device level 2 switch level 1 split supported input channel

#define DH_DEVICE_ID_LOCAL		"Local"		// local device ID
#define DH_DEVICE_ID_REMOTE		"Remote"	// remote device ID
#define DH_DEVICE_ID_UNIQUE		"Unique"	// unique ID

// Split mode
typedef enum tagDH_SPLIT_MODE
{
	DH_SPLIT_1 = 1,							// 1-window
	DH_SPLIT_2 = 2, 						// 2-window
	DH_SPLIT_4 = 4, 						// 4-window
	DH_SPLIT_6 = 6, 						// 6-window
	DH_SPLIT_8 = 8, 						// 8-window
	DH_SPLIT_9 = 9, 						// 9-window
	DH_SPLIT_12 = 12, 						// 12-window
	DH_SPLIT_16 = 16, 						// 16-window
	DH_SPLIT_20 = 20, 						// 20-window
	DH_SPLIT_25 = 25, 						// 25-window
	DH_SPLIT_36 = 36, 						// 36-window
	DH_SPLIT_64 = 64, 						// 64-window
	DH_SPLIT_144 = 144, 					// 144-window
	DH_PIP_1 = DH_SPLIT_PIP_BASE + 1,		// PIP mode.1-full screen+1-small window
	DH_PIP_3 = DH_SPLIT_PIP_BASE + 3,		// PIP mode.1-full screen+3-small window
	DH_SPLIT_FREE = DH_SPLIT_PIP_BASE * 2,	// free open window mode,are free to create,close, window position related to the z axis
	DH_COMPOSITE_SPLIT_1 = DH_SPLIT_PIP_BASE * 3 + 1,	// integration of a split screen
	DH_COMPOSITE_SPLIT_4 = DH_SPLIT_PIP_BASE * 3 + 4,	// fusion of four split screen
	DH_SPLIT_3  = 10,                                // 3-window
	DH_SPLIT_3B = 11,								 // 3-window(bottom) 	
} DH_SPLIT_MODE;

#define DH_PROTOCOL_DAHUA2 DH_PROTOCOL_PRIVATE2
#define DH_PROTOCOL_DAHUA3 DH_PROTOCOL_PRIVATE3

// Device protocol type
typedef enum tagDH_DEVICE_PROTOCOL
{
	DH_PROTOCOL_PRIVATE2,					// private 2nd protocol
	DH_PROTOCOL_PRIVATE3,					// private 3rd protocol
	DH_PROTOCOL_ONVIF,						// Onvif	
	DH_PROTOCOL_VNC,						// virtual network computer
    DH_PROTOCOL_TS,                         // Standard TS
    
	DH_PROTOCOL_PRIVATE = 100,              // private protocol of private        
	DH_PROTOCOL_AEBELL,                     // aebell
	DH_PROTOCOL_PANASONIC,                  // panasonic       
	DH_PROTOCOL_SONY,                       // sony   
	DH_PROTOCOL_DYNACOLOR,                  // Dynacolor        
	DH_PROTOCOL_TCWS,						// tcsw        
	DH_PROTOCOL_SAMSUNG,                    // sansung        
	DH_PROTOCOL_YOKO,                       // YOKO        
	DH_PROTOCOL_AXIS,                       // axis        
	DH_PROTOCOL_SANYO,						// sanyo       		
	DH_PROTOCOL_BOSH,						// Bosch		
	DH_PROTOCOL_PECLO,						// Peclo		
	DH_PROTOCOL_PROVIDEO,					// Provideo		
	DH_PROTOCOL_ACTI,						// ACTi		
	DH_PROTOCOL_VIVOTEK,					// Vivotek		
	DH_PROTOCOL_ARECONT,					// Arecont        
	DH_PROTOCOL_PRIVATEEH,			        // PrivateEH	        
	DH_PROTOCOL_IMATEK,					    // IMatek        
	DH_PROTOCOL_SHANY,                      // Shany        
	DH_PROTOCOL_VIDEOTREC,                  // videotrec        
	DH_PROTOCOL_URA,						// Ura        
	DH_PROTOCOL_BITICINO,                   // Bticino         
	DH_PROTOCOL_ONVIF2,                     // Onvif's protocol type, same to DH_PROTOCOL_ONVIF    
	DH_PROTOCOL_SHEPHERD,                   // shepherd        
	DH_PROTOCOL_YAAN,                       // yaan      
	DH_PROTOCOL_AIRPOINT,					// Airpop        
	DH_PROTOCOL_TYCO,                       // TYCO        
	DH_PROTOCOL_XUNMEI,                     // xunmei      
	DH_PROTOCOL_HIKVISION,                  // hikvision        
	DH_PROTOCOL_LG,                         // LG        
	DH_PROTOCOL_AOQIMAN,					// aoqiman       
	DH_PROTOCOL_BAOKANG,                    // baokang            
	DH_PROTOCOL_WATCHNET,                   // Watchnet        
	DH_PROTOCOL_XVISION,                    // Xvision        
	DH_PROTOCOL_FUSITSU,                    // fusitsu        
	DH_PROTOCOL_CANON,						// Canon		
	DH_PROTOCOL_GE,							// GE		
	DH_PROTOCOL_Basler,						// basler		
	DH_PROTOCOL_Patro,						// patro	    
	DH_PROTOCOL_CPKNC,						// CPPLUS K series		
	DH_PROTOCOL_CPRNC,						// CPPLUS R series		
	DH_PROTOCOL_CPUNC,						// CPPLUS U series		
	DH_PROTOCOL_CPPLUS,						// CPPLUS IPC		
	DH_PROTOCOL_XunmeiS,					// xunmeis,protocal is Onvif		
	DH_PROTOCOL_GDDW,						// GDDW		
	DH_PROTOCOL_PSIA,                       // PSIA        
	DH_PROTOCOL_GB2818,                     // GB2818	        
	DH_PROTOCOL_GDYX,                       // GDYX        
	DH_PROTOCOL_OTHER,                      // others   
} DH_DEVICE_PROTOCOL;

// Split mode info
typedef struct tagDH_SPLIT_MODE_INFO
{
	DWORD				dwSize;
	DH_SPLIT_MODE		emSplitMode;			// Split mode
	int					nGroupID;				// Group SN
    DWORD               dwDisplayType;          // display type, seeDH_SPLIT_DISPLAY_TYPE, under each mode in note , displayed content depends on "PicInPic", each mode displayed content by NVD old rule, as depending on DisChntext, . Compatible, no this item, default is normal display type as"General", 
} DH_SPLIT_MODE_INFO;

// Split capability
typedef struct tagDH_SPLIT_CAPS 
{
	DWORD				dwSize;
	int					nModeCount;				// The split amount supported
	DH_SPLIT_MODE		emSplitMode[DH_MAX_SPLIT_MODE_NUM];	// The split mode supported
	int				    nMaxSourceCount;		// Max display source allocation amount
	int					nFreeWindowCount;		// count of free window support
	BOOL				bCollectionSupported;	// support collection
    DWORD               dwDisplayType;                          // mask means multiple display types, see DH_SPLIT_DISPLAY_TYPE, under each mode in note , displayed content depends on "PicInPic", each mode displayed content by NVD old rule, as depending on DisChntext, Compatible, no this item, default is normal display type as"General", 
    int                 nPIPModeCount;                          // PIP support split mode quantity
    DH_SPLIT_MODE       emPIPSplitMode[DH_MAX_SPLIT_MODE_NUM];  // PIP supported split mode
    int                 nInputChannels[DH_SPLIT_INPUT_NUM];     // supported input channel
    int                 nInputChannelCount;                     // supported input channel quantity, 0 means no input channel limit
    int                 nBootModeCount;                         // enable split mode quantity
    DH_SPLIT_MODE       emBootMode[DH_MAX_SPLIT_MODE_NUM];      // support enable default video split mode
} DH_SPLIT_CAPS;

// even the authentication information
typedef struct tagDH_CASCADE_AUTHENTICATOR
{
	DWORD				dwSize;
	char				szUser[DH_NEW_USER_NAME_LENGTH];		// user name
	char				szPwd[DH_NEW_USER_PSW_LENGTH];			// passwd
	char				szSerialNo[DH_SERIALNO_LEN];			// serial no.
} DH_CASCADE_AUTHENTICATOR;

typedef enum tagEM_SRC_PUSHSTREAM_TYPE
{   
    EM_SRC_PUSHSTREAM_AUTO,        // device automatic recognition according to bit stream head, default
    EM_SRC_PUSHSTREAM_HIKVISION,   // Hikvision private bit stream
    EM_SRC_PUSHSTREAM_PS,          // PS
    EM_SRC_PUSHSTREAM_TS,          // TS
    EM_SRC_PUSHSTREAM_SVAC,        // SVAC
}EM_SRC_PUSHSTREAM_TYPE;

// Display source
typedef struct tagDH_SPLIT_SOURCE
{
	DWORD				dwSize;
	BOOL			    bEnable;						    // Enable
	char			    szIp[DH_MAX_IPADDR_LEN];		    // IP, null means there is no setup.
	char			    szUser[DH_USER_NAME_LENGTH];	    // User name
	char			    szPwd[DH_USER_PSW_LENGTH];	    	// Password
	int			    	nPort;							    // Port
	int				    nChannelID;						    // Channel No.
	int				    nStreamType;					    // Video bit stream. -1-auto, 0-main stream, 1-extra stream 1, 2-extra stream 2, 3-extra stream 3
	int				    nDefinition;					    // Definition, 0-standard definition, 1-high definition
	DH_DEVICE_PROTOCOL  emProtocol;							// Protocol type
	char			    szDevName[DH_DEVICE_NAME_LEN];		// Device name
	int					nVideoChannel;						// Video input channel amount
	int					nAudioChannel;						// Audio input channel amount
	// For decoder only
	BOOL				bDecoder;							// Decoder or not.
	BYTE				byConnType;							// 0:TCP;1:UDP;2:multicast
	BYTE				byWorkMode;							// 0:connect directly; 1:transfer 
	WORD				wListenPort;						// isten port, valid with transfer; when byConnType is multicast, it is multiport
	char				szDevIpEx[DH_MAX_IPADDR_OR_DOMAIN_LEN];// szDevIp extension, front DVR Ip address (enter domain name)
	BYTE				bySnapMode;                         //  snapshot mode (valid when nStreamType==4) 0: request for single frame, 1: sechdule sending request
	BYTE				byManuFactory;						// Target device manufacturer. Refer to EM_IPC_TYPE for detailed information.
	BYTE				byDeviceType;                       //  target device type, 0:IPC
	BYTE				byDecodePolicy;                     // target device decode policy, 0:compatible with previous  
															// 1:real time level high 2: real time level medium
															// 3: real time level low 4: default level 
															// 5: fluency level high 6: fluency level medium
															// 7: fluency level low
	DWORD				dwHttpPort;                         // Http port number, 0-65535
	DWORD				dwRtspPort;                         // Rtsp port number, 0-65535
	char				szChnName[DH_DEVICE_NAME_LEN];		// Remote channel name, modifiable only when name read is not vacant
	char				szMcastIP[DH_MAX_IPADDR_LEN];       // Multicast IP address. Valid only when byConnType is multicast
	char				szDeviceID[DH_DEV_ID_LEN_EX];		// device ID, ""-null, "Local"  "Remote"
	BOOL				bRemoteChannel;						// is remote channel or not(read only)
	unsigned int		nRemoteChannelID;					// remote channel ID (read only), effective when bRemoteChannel=TRUE
	char				szDevClass[DH_DEV_TYPE_LEN];		// type of device, such as IPC, DVR, NVR and so on
	char				szDevType[DH_DEV_TYPE_LEN];			// device specific type, such as IPC-HF3300
	char				szMainStreamUrl[MAX_PATH];			// main stream url, effective when byManuFactory =D H_IPC_OTHER
	char				szExtraStreamUrl[MAX_PATH];			// extra stream url, effective when byManuFactory =D H_IPC_OTHER
	int					nUniqueChannel;						// unique channel ID, read only
	DH_CASCADE_AUTHENTICATOR stuCascadeAuth;				// ssascade authemyication, effective when device ID = "Local/Cascade/SerialNo",  SerialNo is device seral no.
    int                 nHint;                              // 0-normal video source, 1- alarm video source
    int                 nOptionalMainUrlCount;              // back main stream address quantity
    char                szOptionalMainUrls[DH_MAX_OPTIONAL_URL_NUM][MAX_PATH]; // backup main stream address list
    int                 nOptionalExtraUrlCount;             // backup sub stream address quantity
    char                szOptionalExtraUrls[DH_MAX_OPTIONAL_URL_NUM][MAX_PATH]; // backup sub stream address list
    //--------------------------------------------------------------------------------------
    //protocol follow-up add field
    int                 nInterval;                          // tour time intertval	unit:second
    char                szUserEx[DH_NEW_USER_NAME_LENGTH];  // user name
    char                szPwdEx[DH_NEW_USER_PSW_LENGTH];    // password
    EM_SRC_PUSHSTREAM_TYPE  emPushStream;                   // type of pushstream,effective when byConnType is TCP-Push or UDP-Push 
} DH_SPLIT_SOURCE;

// Video output capability set
typedef struct tagDH_VIDEO_OUT_CAPS 
{
	DWORD				dwSize;
	BOOL				bColor;							// Support color output setup or not
	BOOL				bMargin;						// Support margin setup or not
	int					nLayers;						// The max layers supported at the same time
	BOOL				bPowerMode;						// Support power control or not.
	int					bSplitMode;						// The video split mode supported. 0-1-window. 1-'#' mode(inclduing 1-window).2-Any mode
    unsigned int        nBackground;                    // support background mode, by bit, 0-color, 1-picture
    BOOL                bBindOutput;                    // If support output screen resoluton binding setup
    BOOL                bAudioSupport;                  // Support audio output or not
    DWORD               dwAudioMode;                    // The support audio mode, by bit, bit0-main controller output, bit1-daughter card output
} DH_VIDEO_OUT_CAPS;

// Color BCSH
typedef struct tagDH_COLOR_BCSH 
{
	int					nBirghtness;					// Brightness
	int					nContrast;						// Contrast
	int			        nSaturation;					// Saturation
	int					nHue;						    // Hue
} DH_COLOR_BCSH;

// Dimensions
typedef struct tagDH_SIZE 
{
	int					nWidth;							// Width
	int					nHeight;						// Height
} DH_SIZE;

// mode of hot plug
typedef struct tagDH_HOT_PLUG_MODE
{
	DWORD				dwSize;
	int					nMode;							// mode of hot plug, 0-hot plug, 1-force output mode
} DH_HOT_PLUG_MODE;
// Video output option 
typedef struct tagDH_VIDEO_OUT_OPT
{
	DWORD				dwSize;
	DH_RECT*			pstuMargin;						// Margin range
	DH_COLOR_BCSH*		pstuColor;						// Output color
	DH_COLOR_RGBA*		pstuBackground;					// Background color
	DH_SIZE*			pstuSize;						// Output dimensions
	DH_HOT_PLUG_MODE*	pstuHotPlugMode;				// mode of hot plug
} DH_VIDEO_OUT_OPT;

#define NET_VIDEOANALYSE_SCENES (32)
#define NET_VIDEOANALYSE_RULES (64)
// intelligent analysis
typedef struct tagNET_PD_VIDEOANALYSE
{
    BOOL                bSupport;                   // whether supported 
    char                szSupportScenes[NET_VIDEOANALYSE_SCENES][DH_COMMON_STRING_64];   // supported scenes name
    char                SupportRules[NET_VIDEOANALYSE_RULES][DH_COMMON_STRING_64];       // supported rules name
} NET_PD_VIDEOANALYSE;
// product definition
typedef struct tagDH_PRODUCTION_DEFNITION
{
	DWORD				dwSize;
	int					nVideoInChannel;				// Video input channel amount
	int					nVideoOutChannel;				// Video output channel amount
	int					nRemoteDecChannel;				// Remote decode channel amount
	char				szDevType[DH_DEV_TYPE_LEN];		// Device type
	char				szVendor[DH_MAX_NAME_LEN];		// OEM customer
	int					nOEMVersion;					// OEM version
	int					nMajorVerion;					// Main version No.
	int					nMinorVersion;					// SUb version No.
	int					nRevision;						// Revision version
	char				szWebVerion[DH_MAX_NAME_LEN];	// Web version
	char				szDefLanguage[DH_MAX_NAME_LEN];	// Default setup
	NET_TIME			stuBuildDateTime;				// Release time. Unit is second.
	int					nAudioInChannel;				// Audio input channel amount
	int					nAudioOutChannel;				// Audio output channel amount
	BOOL				bGeneralRecord;					// Support schedule storage or not.
	BOOL				bLocalStore;					// Support local storage or not.
	BOOL				bRemoteStore;					// Support network storage or not.
	BOOL				bLocalurgentStore;				// Support emergency local storage or not.
	BOOL				bRealtimeCompress;				// Support real-time compression storage or not.
	DWORD				dwVideoStandards;				// The video format supported. bit0-PAL, bit1-NTSC
	int					nDefVideoStandard;				// Default video format, 0-PAL, 1-NTSC
	int					nMaxExtraStream;				// Max extra stream channel amount
	int					nRemoteRecordChannel;			// Remote record channel amount
	int					nRemoteSnapChannel;				// Remote snap channel amount
	int					nRemoteVideoAnalyseChannel;		// Remote video analysis channel amount
	int					nRemoteTransmitChannel;			// Remote real-time stream transmit max channel amount
	int					nRemoteTransmitFileChannel;		// Remote transmit file channel amount
	int					nStreamTransmitChannel;			// Max network transmit channel amount
	int					nStreamReadChannel;				// Max read file channel amount
	int					nMaxStreamSendBitrate;			// Max bit stream network send capability, kbps
	int					nMaxStreamRecvBitrate;			// Max bit stream network interface capability, kbps
	BOOL				bCompressOldFile;				// Old compression file or not. Delete P frame and save I frame.
	BOOL				bRaid;							// Support RAID or not.
	int					nMaxPreRecordTime;				// Max pre-record time.Unit is second.
	BOOL				bPtzAlarm;						// Support PTZ alarm or not.
	BOOL				bPtz;							// Support PTZ or not.
	BOOL				bATM;							// Support corresponding function of the ATM or not.
	BOOL				b3G;							// Support 3G module or not.
	BOOL				bNumericKey;					// With number button or not.
	BOOL				bShiftKey;						// With Shift button or not.
	BOOL				bCorrectKeyMap;					// Number character map sheet is right or not.
	BOOL				bNewATM;						// The new 2nd ATM front panel.
	BOOL				bDecoder;						// Decoder or not
	DEV_DECODER_INFO	stuDecoderInfo;					// Decoder info. Valid when bDecoder=true.bDecoder=true
	int					nVideoOutputCompositeChannels;	// integration ceiling screen output channel
	BOOL                bSupportedWPS;                  // support WPS or not
	int					nVGAVideoOutputChannels;		// VGA video output channel number
	int					nTVVideoOutputChannels;			// TV video output channel number
	int					nMaxRemoteInputChannels;		// max number of remote channels
	int					nMaxMatrixInputChannels;		// max number of matrix channels
	int                 nMaxRoadWays;                   // max counts of road ways 1~6
	int                 nMaxParkingSpaceScreen;         // max counts of screen when docking with the camera 0~20

	int					nPtzHorizontalAngleMin;			// PTZ'horizontal minimum Angle, [0-360]
	int					nPtzHorizontalAngleMax;			// PTZ'horizontal maximum Angle, [0-360]
	int					nPtzVerticalAngleMin;			// PTZ'vertical  minimum Angle, [-90,90]
	int					nPtzVerticalAngleMax;			// PTZ'vertical  maximum Angle, [-90,90]
	BOOL				bPtzFunctionMenu;				// Whether to support PTZ's function menu 
	BOOL				bLightingControl;				// Whether to support lighting control 
	DWORD				dwLightingControlMode;			// Manual lighting control mode,bitwise,see NET_LIGHTING_CTRL_ON_OFF
	int					nNearLightNumber;				// dipped headlight group number, 0 means no support 
	int					nFarLightNumber;				// High beam group number, 0 means no support
	BOOL				bFocus;							// Whether to support control focus 
	BOOL				bIris;							// Whether to support control aperture 
	char				szPtzProtocolList[DH_COMMON_STRING_1024];	// PTZ support agreement list, can be more, each with '; 'delimited 
	BOOL				bRainBrushControl;				// Whether to support wiper control 
	int					nBrushNumber;					// Number of wiper, 0 means no support
	int					nLowerMatrixInputChannels[DH_MAX_LOWER_MITRIX_NUM];		// inferior video matrix input channel, the subscript corresponding matrix number 
	int					nLowerMatrixOutputChannels[DH_MAX_LOWER_MITRIX_NUM];	// inferior video matrix output channel, the subscript corresponding matrix number 
    BOOL                bSupportVideoAnalyse;           // support intelligent analuysis or not
    BOOL                bSupportIntelliTracker;         // support intelligen tracking or not
    DWORD               nSupportBreaking;               // device supported violation type mask(by bit Get )
                                                        // 0-run the red light 1-cross line  2-retrogradation  3-under speed 4-over speed 5-car in lane 6-yellow plate in lane 7-violation driving, left, right, u-turn, 
                                                        // 8-violation enter waiting zone 9-illegal parking 10-Traffic jam 11-driving not by lane 12-illegal lane change 13-cross yellow line 14-should driving
                                                        // 15-manual snapshot 16-illegal retention 17-crosswalk pedestration first 18-over flow 19-under flow 20-illegal in lane 21-illegal back car
                                                        // 22-cross stop line 23-run yellow light 24-yellow grid line parking 25-car in parking 26-car not in parking 27-car in parking cross line  28-limited plate
                                                        // 29-no pass 30-unfasten seat belt 31-drvier smoking
    DWORD               nSupportBreaking1;              // 0-driver call 1-trafic-pedestrian redlight running 2-Traffic Jam Forbid Into 3-Pass Not In Order
    NET_PD_VIDEOANALYSE stuVideoAnalyse;                // IVS
    BOOL                bTalkTransfer;                  // support talk-transfer or not
    BOOL				bCameraAttribute;				// support video in options or not
    BOOL				bPTZFunctionViaApp;				// support PTZ function or not
	BOOL				bAudioProperties;				// support audio properties or not
	BOOL				bIsCameraIDOsd;					// support camera id osd or not
	BOOL				bIsPlaceOsd;					// support place osd or not

	unsigned int		nMaxGeographyTitleLine;			// the max geography title line
} DH_PRODUCTION_DEFNITION;

// Manual lighting control mode 
#define NET_LIGHTING_CTRL_ON_OFF				0x01	// Direct switch mode 
#define NET_LIGHTING_CTRL_ADJUST_BRIGHTNESS		0x02	// Manually adjust brightness mode 
#define NET_LIGHTING_CTRL_ZOOM_PRIO				0x04	// Ratio prefer 


// Matrix sub card type. Various setups.
#define DH_MATRIX_CARD_MAIN				0x10000000		// main card
#define DH_MATRIX_CARD_INPUT			0x00000001		// input card 
#define DH_MATRIX_CARD_OUTPUT			0x00000002		// output card
#define DH_MATRIX_CARD_ENCODE			0x00000004		// encode card
#define DH_MATRIX_CARD_DECODE			0x00000008		// decode card
#define DH_MATRIX_CARD_CASCADE			0x00000010		// cascade card
#define DH_MATRIX_CARD_INTELLIGENT		0x00000020		// intelligent card
#define DH_MATRIX_CARD_ALARM            0x00000040      // alarm card 
#define DH_MATRIX_CARD_RAID             0x00000080      // Hdd Raid Card
#define DH_MATRIX_CARD_NET_DECODE       0x00000100      // net decode card

// Matrix sub card info
typedef struct tagDH_MATRIX_CARD
{
	DWORD				dwSize;
	BOOL				bEnable;					// Valid or not
	DWORD				dwCardType;					// Sub card type
	char		    	szInterface[DH_MATRIX_INTERFACE_LEN];	// Signal interface type, "CVBS", "VGA", "DVI"...
	char		    	szAddress[DH_MAX_IPADDR_OR_DOMAIN_LEN];	// Device IP or domain name. The sub card that has no network conneciton can be null.
	int			    	nPort;						// Port No. The sub card that has no network conneciton can be 0.
	int					nDefinition;				// Definition. 0=standard definition. 1=High definition
	int					nVideoInChn;				// Video input channel amount
	int					nAudioInChn;				// Audio input channel amount
	int					nVideoOutChn;				// Video output channel amount
	int				    nAudioOutChn;				// Audio output channel amount
	int			    	nVideoEncChn;				// Video encode channel amount
	int				    nAudioEncChn;				// Audio encode channel amount
	int			    	nVideoDecChn;				// Video decode channel amount
	int			    	nAudioDecChn;				// Audio decode channel amount
	int					nStauts;					// Status: 0-OK, 1-no respond, 2-network disconnection, 3-conflict, 4-upgrading now
	int					nCommPorts;					// COM amount
	int					nVideoInChnMin;				// Video input channel min value
	int					nVideoInChnMax;				// Video input channel max value
	int					nAudioInChnMin;				// Audio input channel min value
	int					nAudioInChnMax;				// Audio input channel max value
	int					nVideoOutChnMin;			// Video output channel min value
	int					nVideoOutChnMax;			// Video output channel max value
	int					nAudioOutChnMin;			// Audio output channel min value
	int					nAudioOutChnMax;			// Audio output channel max value	
	int					nVideoEncChnMin;			// Video encode channel min value
	int					nVideoEncChnMax;			// Video encode channel max value
	int					nAudioEncChnMin;			// Audio encode channel min value
	int					nAudioEncChnMax;			// Audio encode channel max value
	int					nVideoDecChnMin;			// Video decode channel min value
	int					nVideoDecChnMax;			// Video decode channel max value
	int					nAudioDecChnMin;			// Audio decode channel min value
	int					nAudioDecChnMax;			// Audio decode channel max value
	int					nCascadeChannels;			// number of cascade channel
	int					nCascadeChannelBitrate;		// cascade channel bitrate (Mbps)
	int					nAlarmInChnCount;			// Alarm input channel number 
	int					nAlarmInChnMin;				// Alarm input channel number minimum value 
	int					nAlarmInChnMax;				// Alarm input channel number maximum value 
	int					nAlarmOutChnCount;			// Alarm output channel number 
	int					nAlarmOutChnMin;			// Alarm output channel number minimum value
	int					nAlarmOutChnMax;			// Alarm output channel number maximum value
	int					nVideoAnalyseChnCount;		// Intelligent analysis of channel number 
	int					nVideoAnalyseChnMin;		// Intelligent analysis of channel number minimum value 
	int					nVideoAnalyseChnMax;		// Intelligent analysis of channel number maximum value
	int					nCommPortMin;				// minimum value of serial port number  
	int					nCommPortMax;				// maximum value of serial port number
    char                szVersion[DH_COMMON_STRING_32];         // Version info
    NET_TIME            stuBuildTime;                           // compile time
    char                szBIOSVersion[DH_COMMON_STRING_64];     // BIOS Version
    char				szMAC[DH_MACADDR_LEN];					// MAC address
} DH_MATRIX_CARD;

// Matrix sub card list
typedef struct tagDH_MATRIX_CARD_LIST 
{
	DWORD				dwSize;
	int					nCount;							// sub card  amount
	DH_MATRIX_CARD		stuCards[DH_MATRIX_MAX_CARDS];	// Sub card list 
} DH_MATRIX_CARD_LIST;

// Video output window
typedef struct tagDH_VIDEO_OUT_WINDOW
{
	DWORD				dwSize;
	int					nBackboardID;				// Backboard ID
	int					nScreenID;					// Screen ID
	int					nWindowID;					// Window ID
} DH_VIDEO_OUT_WINDOW;

// Set Backup up information, input parameter
typedef struct tagNET_IN_SETBACKUP_SOURCES_INFO
{
	DWORD				dwSize;
	int					nSetSourcesNum;				// 	Set Backup up num
	DH_SPLIT_SOURCE	*	pSource;					//  Video Sources(Memory is released by the user)
} NET_IN_SETBACKUP_SOURCES_INFO;

// Set Backup up information,inout parameter 
typedef struct tagNET_OUT_SETBACKUP_SOURCES_INFO
{
	DWORD				dwSize;
} NET_OUT_SETBACKUP_SOURCES_INFO;

// CLIENT_GetISCSITargets
typedef struct tagDH_IN_ISCSI_TARGETS 
{
	DWORD				dwSize;
	const char*			pszAddress;					// Server address
	int					nPort;						// port
	const char*			pszUser;					// Username
	const char*			pszPwd;						// Password
} DH_IN_ISCSI_TARGETS;

// ISCSI Target Info
typedef struct tagDH_ISCSI_TARGET 
{
	DWORD				dwSize;
	char				szName[DH_MAX_ISCSI_NAME_LEN];	// Name
	char				szAddress[DH_MAX_IPADDR_OR_DOMAIN_LEN];	// service address
	char				szUser[DH_NEW_USER_NAME_LENGTH];// user name
	int					nPort;							// port
	UINT				nStatus;						// status, 0- unknow, 1-connected, 2-un connected, 3-connect failed, 4-authentication failed, 5-connect time out	
} DH_ISCSI_TARGET;

// CLIENT_GetISCSITargets Interface output parameter
typedef struct tagDH_OUT_ISCSI_TARGETS
{
	DWORD				dwSize;
	DH_ISCSI_TARGET*	pstuTargets;				// iscsi array, the space application by the user, applt to sizeof(DH_ISCSI_TARGET)*nMaxCount
	int					nMaxCount;					// iscsi group size
	int					nRetCount;					// Received iscSi amount
} DH_OUT_ISCSI_TARGETS;

typedef enum tagDH_BITMAP_ACCURACY
{
    BA_DAY ,            //day 
    BA_HOUR ,           //day
    BA_5MIN ,           //5 miuntes
}DH_BITMAP_ACCURACY;

#define DH_MAX_BITMAPS        256                        // Bitmap bytes, DH_MAX_BITMAPS*8-bit
#define DH_MAX_CHN_NUM        256                        // The maximum channel number 
//CLIENT_GetBitmap Interface input parameters 
typedef struct tagDH_IN_BITMAP
{
    DWORD               dwSize;
    NET_TIME            stuStartTime;                   // start time, accurate to seconds 
    NET_TIME            stuEndTime;                     // Over time, accurate to seconds 
    int                 nChnList[DH_MAX_CHN_NUM];       // [1, ..., 10]	Channel number list ,-1 means all channels ,0 means channels end
    DH_BITMAP_ACCURACY  emAccuracy;                     // Precision of the bitmap     
}DH_IN_BITMAP;

typedef struct tagDH_BITMAP_INFO
{
    DWORD               dwSize;
    int                 nChnID;                         //Channel number, Channel number is 0, means the end of the list 
    BYTE                bBitmap[DH_MAX_BITMAPS];        //Bitmap (each bit means a query scale, from low to high filling) 
                                                        //support DH_MAX_BITMAPS * 8 scale
    int                 nBitNum;                        //the digits of bitmap 
}DH_BITMAP_INFO;

//CLIENT_GetBitmap  Interface output parameters 
typedef struct tagDH_OUT_BITMAP
{
    DWORD               dwSize;
    DH_BITMAP_INFO      *pstBitmapInfos;     // the space application by the user,apply to sizeof(DH_BITMAP_INFO)*nMaxBitmapInfo 
    int                 nGetBitmapInfo;
    int                 nMaxBitmapInfo;    
}DH_OUT_BITMAP;

// Storage device name
typedef struct tagDH_STORAGE_DEVICE_NAME 
{
	DWORD				dwSize;
	char				szName[DH_STORAGE_NAME_LEN];
} DH_STORAGE_DEVICE_NAME;

// RAID state
#define DH_RAID_STATE_ACTIVE		0x00000001
#define DH_RAID_STATE_INACTIVE		0x00000002
#define DH_RAID_STATE_CLEAN			0x00000004
#define DH_RAID_STATE_FAILED		0x00000008
#define DH_RAID_STATE_DEGRADED		0x00000010
#define DH_RAID_STATE_RECOVERING	0x00000020
#define DH_RAID_STATE_RESYNCING		0x00000040
#define DH_RAID_STATE_RESHAPING		0x00000080
#define DH_RAID_STATE_CHECKING		0x00000100
#define DH_RAID_STATE_NOTSTARTED	0x00000200

// RAID member info
typedef struct tagNET_RAID_MEMBER_INFO 
{
    DWORD               dwSize;
    DWORD               dwID;                                       // disk no., may use to describe disk cabinet slot
    BOOL                bSpare;                                     // partial hot device, true-partial hot device, false-RAID sub disk
} NET_RAID_MEMBER_INFO;

// RAID Info
typedef struct tagDH_STORAGE_RAID
{
	DWORD				dwSize;
	int					nLevel;										// level
	int					nState;										// RAID state combinationDH_RAID_STATE_ACTIVE | DH_RAID_STATE_DEGRADED
	int					nMemberNum;									// member amount
	char				szMembers[DH_MAX_MEMBER_PER_RAID][DH_STORAGE_NAME_LEN];	// RAID member
    float               fRecoverPercent;                            // Sync percentage, 0~100, RAID status has"Recovering" or "Resyncing" valid
    float               fRecoverMBps;                               // Sync speed, unit MBps, RAID status has"Recovering" or "Resyncing" valid
    float               fRecoverTimeRemain;                         // Sync remaining time, unit minute, RAID status has "Recovering" or "Resyncing" valid
    NET_RAID_MEMBER_INFO stuMemberInfos[DH_MAX_MEMBER_PER_RAID];    // RAID member info
	int                 nRaidDevices;                               // The number of RAID device
    int                 nTotalDevices;                              // The total count of RAID device
	int                 nActiveDevices;                             // The number of active device
	int                 nWorkingDevices;                            // The number of working device
	int                 nFailedDevices;                             // The number of failed device
	int                 nSpareDevices;                              // The number of hot-spare device
} DH_STORAGE_RAID;

// Storage partition info
typedef struct tagDH_STORAGE_PARTITION
{
	DWORD				dwSize;
	char				szName[DH_STORAGE_NAME_LEN];				// Name
	INT64				nTotalSpace;   							    // Total space(MB) 
	INT64				nFreeSpace;								    // free space(MB)
	char				szMountOn[DH_STORAGE_MOUNT_LEN];			// Mount point
	char				szFileSystem[DH_STORAGE_FILE_SYSTEM_LEN];	// File system
	int					nStatus;									// partition state, 0-LV not available, 1-LV available
} DH_STORAGE_PARTITION;

// storage tank info
typedef struct tagDH_STORAGE_TANK 
{
	DWORD				dwSize;
	int					nLevel;										// level, the host is 0 level
	int					nTankNo;									// extend port number from 0
    int                 nSlot;                                      // Corresponding cabinet board card no., start from 0
} DH_STORAGE_TANK;

// storage device status
#define NET_STORAGE_DEV_OFFLINE                 0                   // physical disk offline status
#define NET_STORAGE_DEV_RUNNING                 1                   // physicali disk running ststus
#define NET_STORAGE_DEV_ACTIVE                  2                   // RAID activity
#define NET_STORAGE_DEV_SYNC                    3                   // RAID sync
#define NET_STORAGE_DEV_SPARE                   4                   // RAID hotdevice(partial)
#define NET_STORAGE_DEV_FAULTY                  5                   // RAID invalid
#define NET_STORAGE_DEV_REBUILDING              6                   // RAID re-build
#define NET_STORAGE_DEV_REMOVED                 7                   // RAID remove
#define NET_STORAGE_DEV_WRITE_ERROR             8                   // RAID error
#define NET_STORAGE_DEV_WANT_REPLACEMENT        9                   // RAID need to be replaced
#define NET_STORAGE_DEV_REPLACEMENT             10                  // RAID is replacement
#define NET_STORAGE_DEV_GLOBAL_SPARE            11                  // Global hotdevice
#define NET_STORAGE_DEV_ERROR                   12                  // Error, partial available
#define NET_STORAGE_DEV_RAIDSUB                 13                  // The disk is single, originally is Raid sub disk, may auto add Raid after reboot

// Storage device info
typedef struct tagDH_STORAGE_DEVICE 
{
	DWORD				dwSize;
	char				szName[DH_STORAGE_NAME_LEN];				// name
	INT64				nTotalSpace;								// Total space (MB) 
	INT64				nFreeSpace;									// free space (MB) 
	BYTE				byMedia;									// Media, 0-DISK, 1-CDROM, 2-FLASH medium,  
	BYTE				byBUS;										// BUS, 0-ATA, 1-SATA, 2-USB, 3-SDIO, 4-SCSI main line 0-ATA, 1-SATA, 2-USB, 3-SDIO, 4-SCSI
	BYTE				byVolume;									// volume type, 0-physics, 1-Raid, 2- VG virtual 
	BYTE				byState;									// Physics disk state, 0-physics disk offline state 1-physics disk  2- RAID activity 3- RAID sync 4-RAID hotspare 5-RAID invalidation 6- RAID re-creation  7- RAID delete
	int					nPhysicNo;									// storage interface of devices of same type logic number
	int					nLogicNo;									// storage interface of devices of same type physics number
	char				szParent[DH_STORAGE_NAME_LEN];				// superior storage group name
	char				szModule[DH_STORAGE_NAME_LEN];				// device module
	char				szSerial[DH_SERIALNO_LEN];					// device serial number
	char				szFirmware[DH_VERSION_LEN];					// Firmware version
	int					nPartitionNum;								// partition number
	DH_STORAGE_PARTITION stuPartitions[DH_MAX_STORAGE_PARTITION_NUM];// partition info
	DH_STORAGE_RAID		stuRaid;									// Raid info, for RAID use only(byVolume == 1) 
	DH_ISCSI_TARGET		stuISCSI;									// Iscsi info, for iscsi use only (byVolume == 2)
	BOOL				abTank;										// tank enable
	DH_STORAGE_TANK		stuTank;									// tank info, effectice when abTank = TRUE
} DH_STORAGE_DEVICE; 

// OSD channel info
typedef struct tagNET_SPLIT_OSD 
{
    DWORD               dwSize;
    BOOL                bEnable;                    // enable
    DH_COLOR_RGBA       stuFrontColor;              // The foreground color 
    DH_COLOR_RGBA       stuBackColor;               // The background color 
    DH_RECT             stuFrontRect;               // Foreground area 
    DH_RECT             stuBackRect;                // Background region 
    BOOL                bRoll;                      // Whether the scroll display, applies to the text only 
    BYTE                byRollMode;                 // Scroll mode, applies only to text, 0 -from left to right,1-from right to left,2-from top to bottom,3-from bottom to top
    BYTE                byRoolSpeed;                // Rolling speed, applies to text  only , 0 ~ 4, the greater the value the faster the scrolling 
    BYTE                byFontSize;                 // The font size, applies to the text only  
    BYTE                byTextAlign;                // Alignment, 0 - left, 1 - centered, 2 - right 
    BYTE                byType;                     // OSD type, 0 - text, 1 - icon 
    BYTE                Reserved[3];                // retain byte
    char                szContent[MAX_PATH];        // OSD content, if the type is icon, the content is the name of the icon 
    float               fPitch;                     // field interval, 0.0 ~ 5.0
    char                szFontType[DH_COMMON_STRING_64]; // font type 
} NET_SPLIT_OSD;

// CLIENT_GetSplitOSD's interface input param(get split window input OSD info)
typedef struct tagDH_IN_SPLIT_GET_OSD
{
	DWORD					dwSize;
	int						nChannel;				// channel no.
	int						nWindow;				// window no.
} DH_IN_SPLIT_GET_OSD;

// CLIENT_GetSplitOSD's interface input param(get split window output OSD info)
typedef struct tagDH_OUT_SPLIT_GET_OSD
{
	DWORD					dwSize;
    int                 nOSDNum;                            // OSD number
    NET_SPLIT_OSD       stuOSD[DH_VIDEO_CUSTOM_OSD_NUM];    // OSD information
} DH_OUT_SPLIT_GET_OSD;

// CLIENT_SetSplitOSD's interface input param(setting split window input OSD info)
typedef struct tagDH_IN_SPLIT_SET_OSD
{
	DWORD					dwSize;
	int						nChannel;							// channel no.
	int						nWindow;							// window no.
    int                 	nOSDNum;                            // OSD number
    NET_SPLIT_OSD       	stuOSD[DH_VIDEO_CUSTOM_OSD_NUM];    // OSD information
} DH_IN_SPLIT_SET_OSD;

// CLIENT_SetSplitOSD's interface input param(setting split window output OSD info)
typedef struct tagDH_OUT_SPLIT_SET_OSD
{
	DWORD					dwSize;
} DH_OUT_SPLIT_SET_OSD;

// CLIENT_GetSplitOSDEx's interface input param(get split window input OSD info)
typedef struct tagNET_IN_SPLIT_GET_OSD_EX
{
	DWORD					dwSize;					// the size of this struct
	int						nChannel;				// channel no.
	int						nWindow;				// window no.
} NET_IN_SPLIT_GET_OSD_EX;

// CLIENT_GetSplitOSDEx's interface input param(get split window output OSD info)
typedef struct tagNET_OUT_SPLIT_GET_OSD_EX
{
	DWORD				dwSize;									// the size of this struct
    int                 nOSDNum;                            	// OSD number
    NET_SPLIT_OSD       stuOSD[DH_VIDEO_CUSTOM_OSD_NUM_EX];   	// OSD information
} NET_OUT_SPLIT_GET_OSD_EX;


// CLIENT_SetSplitOSDEx's interface input param(setting split window input OSD info)
typedef struct tagNET_IN_SPLIT_SET_OSD_EX
{
	DWORD               dwSize;									// the size of this struct
    int                 nChannel;                           	// channel no.
    int                 nWindow;                            	// window no.
    int                 nOSDNum;                            	// OSD number
    NET_SPLIT_OSD       stuOSD[DH_VIDEO_CUSTOM_OSD_NUM_EX];		// OSD information
}NET_IN_SPLIT_SET_OSD_EX;

// CLIENT_SetSplitOSDEx's interface input param(setting split window output OSD info)
typedef struct tagNET_OUT_SPLIT_SET_OSD_EX
{
    DWORD               dwSize;			// the size of this struct
} NET_OUT_SPLIT_SET_OSD_EX;


// Video output control method 
typedef enum
{
    EM_VIDEO_OUT_CTRL_CHANNEL,              // Logical channel number control mode,effective for physical screen and splicing screen 
    EM_VIDEO_OUT_CTRL_COMPOSITE_ID,         // Splice screen ID control mode, applies to splice screen only  
} EM_VIDEO_OUT_CTRL_TYPE;

// CLIENT_SetSplitSourceEx  The input parameters of the interface 
typedef struct tagNET_IN_SET_SPLIT_SOURCE 
{
    DWORD                   dwSize;
    EM_VIDEO_OUT_CTRL_TYPE  emCtrlType;         // Video output control method 
    int                     nChannel;           // Video output logical channel number,when the emCtrlType is EM_VIDEO_OUT_CTRL_CHANNEL effective
    const char*             pszCompositeID;     // Splicing screen ID, when the emCtrlType is EM_VIDEO_OUT_CTRL_CHANNEL effective
    int                     nWindow;            // winder number, -1 means all windows of the current segmentation mode 
    DH_SPLIT_SOURCE*        pstuSources;        // Video source information, when nWindow=-1, Video source is an array, and the number and the window number
												// the space application by the user,applt to sizeof(DH_SPLIT_SOURCE)*nSourceCount
	int                     nSourceCount;       // Video source number 
} NET_IN_SET_SPLIT_SOURCE;

// Set the return result of video source  
typedef struct tagNET_SET_SPLIT_SOURCE_RESULT 
{
    DWORD                   dwSize;
    int                     nPushPort;          // Monitor Port Number of Push Flow Pattern Equipment 
} NET_SET_SPLIT_SOURCE_RESULT;

// CLIENT_SetSplitSourceEx output parameters of the interface 
typedef struct tagNET_OUT_SET_SPLIT_SOURCE
{
    DWORD                   dwSize;
    NET_SET_SPLIT_SOURCE_RESULT* pstuResults;   // returned value after successful setting , corresponding the window array in NET_IN_SET_SPLIT_SOURCE, user allocates memory , apply to sizeof(NET_SET_SPLIT_SOURCE_RESULT)*nMaxResultCount,If don't need can be NULL 
    int                     nMaxResultCount;    // The size of the pstuResults array
    int                     nRetCount;          // The Number Of Return
} NET_OUT_SET_SPLIT_SOURCE;

//volume type enumeration
typedef enum tagNET_VOLUME_TYPE
{
    VOLUME_TYPE_ALL = 0      ,      //all volume
    VOLUME_TYPE_PHYSICAL     ,      //physical volume
    VOLUME_TYPE_RAID         ,      //Raid volume
    VOLUME_TYPE_VOLUME_GROUP ,      //VG virtual volume
    VOLUME_TYPE_ISCSI        ,      //iSCSI volume
    VOLUME_TYPE_INVIDUAL_PHY ,      //independent physical volume, this physical volume, is not added into,  RAID, virtual volume group, 
    VOLUME_TYPE_GLOBAL_SPARE ,      //global hot spare volume
    VOLUME_TYPE_MAX          ,
}NET_VOLUME_TYPE;


#define    MAX_DEVICE_VOLUME_NUMS        128                //volume type upper limit

//CLIENT_QueryDevInfo , NET_QUERY_DEV_STORAGE_NAMES type port input parameter
typedef struct tagNET_IN_STORAGE_DEV_NAMES
{
    DWORD               dwSize;
    NET_VOLUME_TYPE     emVolumeType;       //volume type to get
} NET_IN_STORAGE_DEV_NAMES;

//CLIENT_QueryDevInfo , NET_QUERY_DEV_STORAGE_NAMES type volume output parameter
typedef struct tagNET_OUT_STORAGE_DEV_NAMES
{
    DWORD               dwSize;
    int                 nDevNamesNum;                       							//got device storage module name quantity
    char                szStoregeDevNames[MAX_DEVICE_VOLUME_NUMS][DH_STORAGE_NAME_LEN]; //device name list
}NET_OUT_STORAGE_DEV_NAMES;

//CLIENT_QueryDevInfo , NET_QUERY_DEV_STORAGE_INFOS port input parameter
typedef struct tagNET_IN_STORAGE_DEV_INFOS
{
    DWORD               dwSize;
    NET_VOLUME_TYPE     emVolumeType;       //volume type to get
} NET_IN_STORAGE_DEV_INFOS;

//CLIENT_QueryDevInfo , NET_QUERY_DEV_STORAGE_INFOS port output parameter
typedef struct tagNET_OUT_STORAGE_DEV_INFOS
{
    DWORD               dwSize;
    int                 nDevInfosNum;                      			//device storage moduleinfo list to get
    DH_STORAGE_DEVICE   stuStoregeDevInfos[MAX_DEVICE_VOLUME_NUMS]; //device info list, dwsize of DH_STORAGE_DEVICE need to assign value
} NET_OUT_STORAGE_DEV_INFOS;

typedef enum tagNET_RECENCY_CAR_INFO
{
    RECENCY_CAR_INFO_UNKNOW = 0 ,
    RECENCY_CAR_INFOO_NEWEST    ,      //newest
    RECENCY_CAR_INFO_ODLEST     ,      //oldest
    RECENCY_CAR_INFO_MAX        ,
}NET_RECENCY_CAR_INFO;

//CLIENT_QueryDevInfo , NET_QUERY_RECENCY_JNNCTION_CAR_INFO port input parameter
typedef struct tagNET_IN_GET_RECENCY_JUNCTION_CAR_INFO
{
    DWORD                   dwSize;
    int                     nChannel;       //snapshot channel no.
    NET_RECENCY_CAR_INFO    emRecencyType;  //get vehicle info type, newest or latest
    DWORD                   nIndex;         //no. vehicle info, from 1, if it is 0, it alsomeans no. 1
}NET_IN_GET_RECENCY_JUNCTION_CAR_INFO;

//CLIENT_QueryDevInfo , NET_QUERY_RECENCY_JNNCTION_CAR_INFO port output parameter
typedef struct tagNET_OUT_GET_RECENCY_JUNCTION_CAR_INFO
{
    DWORD           dwSize;
    DEV_EVENT_TRAFFIC_TRAFFICCAR_INFO stTrafficCar;        // traffic vehicle info
}NET_OUT_GET_RECENCY_JUNCTION_CAR_INFO;

// 
#define NET_MAX_FISHEYE_WINDOW_NUM        8     // Max fish eye window number

// window position info 
typedef struct tagNET_FISHEYE_WINDOW_INFO
{
  DWORD      dwSize;
  DWORD      dwWindowID;            // window ID
  int        nFocusX;              	// EPtz, E-PTZ , focus horizontal coordinate
  int        nFocusY;              	// EPtz (E-PTZ , focus vertical coordinate  
  int        nHorizontalAngle;      // EPtz horizontal degree
  int        nVerticalAngle;        // EPtz vertical degree
}NET_FISHEYE_WINDOW_INFO;

// corresponding to CLIENT_QueryDevInfo port, NET_QUERY_DEV_FISHEYE_WININFO search Fish eye window info input parameter
typedef struct tagNET_IN_FISHEYE_WININFO
{
  DWORD       dwSize;
  int         nChannelId;           // channel no.
  int         nWindowNum;           // search window number
  int         nWindows[NET_MAX_FISHEYE_WINDOW_NUM]; // window id, cannot repeat
}NET_IN_FISHEYE_WININFO;

// corresponding to CLIENT_QueryDevInfo port, NET_QUERY_DEV_FISHEYE_WININFO search Fish eye window info output parameter
typedef struct tagNET_OUT_FISHEYE_WININFO
{
  DWORD       dwSize;
  int         nWindowNum;           //window number
  NET_FISHEYE_WINDOW_INFO stuWindows[NET_MAX_FISHEYE_WINDOW_NUM]; // windowdetail info 
}NET_OUT_FISHEYE_WININFO;

//CLIENT_QueryDevInfo , NET_QUERY_LANES_STATE port input parameter
typedef struct tagNET_IN_GET_LANES_STATE
{
  DWORD         dwSize;
  int           nLaneNumber; //-1 represents search all lane,>=0 means search specify lane
}NET_IN_GET_LANES_STATE;

//signal light indicator status 
typedef enum tagNET_TRAFFIC_LIGHT_STATUS
{
  LIGHT_STATUS_UNKNOWN = 0  ,  	//unknown 
  LIGHT_STATUS_RED      ,  		//red
  LIGHT_STATUS_GREEN     ,  	//green
  LIGHT_STATUS_YELLOW     ,  	//yellow
}NET_TRAFFIC_LIGHT_STATUS;

//signal light indicator info 
typedef struct tatNET_TRAFFIC_LIGHT_INFO
{
  DWORD            			dwSize;
  NET_TRAFFIC_LIGHT_STATUS  emStraightLightInfo;  //straight light status 
  NET_TRAFFIC_LIGHT_STATUS  emLeftLightInfo;      //left light status 
  NET_TRAFFIC_LIGHT_STATUS  emRightLightInfo;     //right light status 
  NET_TRAFFIC_LIGHT_STATUS  emUTurnLightInfo;     //u-turn light status 
}NET_TRAFFIC_LIGHT_INFO;

//lane direction
typedef enum tagNET_TRAFFIC_DIRECTION
{
  DIRECTION_UNKNOW  = 0,  // unknown 
  DIRECTION_STRAIGHT   ,   //straight
  DIRECTION_LEFT       ,   //left
  DIRECTION_RIGHT      ,   //right
  DIRECTION_UTURN      ,   //u-turn
}NET_TRAFFIC_DIRECTION;

//lane info 
typedef struct tagNET_TRAFFIC_LANE_INFO
{
  DWORD         dwSize;
  UINT          nLaneNumber;                  //lane no.,0 start 
  UINT          nSupportDirectionNum;         //lane can drive direction number
  NET_TRAFFIC_DIRECTION  emTrafficDirections[MAX_LANE_DIRECTION_NUM];  //lane direction, means this lane can drive in all directions
  NET_TRAFFIC_JAM_STATUS emJamState;          //road jam status
  //flow info 
  UINT          nLargeVehicleNum;             //large car quantity
  UINT          nMediumVehicleNum;            //medium car quantity
  UINT          nSmallVehicleNum;             //small car quantity
  UINT          nMotoNum;                     //motor quantity
}NET_TRAFFIC_LANE_INFO;

//CLIENT_QueryDevInfo , NET_QUERY_LANES_STATE port output parameter
typedef struct tagNET_OUT_GET_LANES_STATE
{
  DWORD         dwSize;
  int           nGetLaneInfoNum;          					// Get lane info quantity
  NET_TRAFFIC_LANE_INFO  stLaneInfos[MAX_LANE_INFO_NUM];    // Get lane info 
  NET_TRAFFIC_LIGHT_INFO stLightInfo;            			// signal light indicator status 
}NET_OUT_GET_LANES_STATE;

//CLIENT_QueryDevInfo , input param
typedef struct tagNET_IN_SYSTEM_INFO
{
    DWORD               dwSize;
} NET_IN_SYSTEM_INFO;

//CLIENT_QueryDevInfo , output param
typedef struct tagNET_OUT_SYSTEM_INFO
{
    DWORD               dwSize;
    BOOL                bHasRTC;                       // whether system has RTC(for recording system time),TRUE:Yes,FALSE:No
    int                 nRetMCUNum;                   // MCU number of return 	 	 
    char                szMCUVersion[MAX_MCU_NUM][DH_MAX_VERSION_LEN];         // MCU software version
}NET_OUT_SYSTEM_INFO;


// port  CLIENT_AttachLanesState input parameter
typedef struct tagNET_IN_REGDEV_NET_INFO
{
    DWORD               dwSize;
    char                szDevSerial[DH_DEV_SERIALNO_LEN];   // Device SN by auto register 
}NET_IN_REGDEV_NET_INFO;

// celluar net type
typedef enum tagNET_CELLUAR_NET_TYPE 
{
    EM_CELLUAR_NET_UNKNOW           =   -1  ,    //Unknown 
    EM_CELLUAR_NET_PRIVATE_3G_4G    =   0   ,    //Special 3G/4G network (such as the internal network of the public security authority)
    EM_CELLUAR_NET_COMMERCIAL_3G_4G =   1   ,    //Business 3G/4G network (Such as internet service provider)
    EM_CELLUAR_NET_MAX , 
}NET_CELLUAR_NET_TYPE;

//output param of CLIENT_QueryDevInfo according to NET_QUERY_REG_DEVICE_NET_INFO 
typedef struct tagNET_OUT_REGDEV_NET_INFO
{
    DWORD                   dwSize;
    NET_CELLUAR_NET_TYPE    emCelluarNetType;           //Network type when auto register 
}NET_OUT_REGDEV_NET_INFO;

// video channel type when query video channel info
typedef enum tagNET_VIDEO_CHANNEL_TYPE
{
    NET_VIDEO_CHANNEL_TYPE_ALL,                         // all types
    NET_VIDEO_CHANNEL_TYPE_INPUT,                       // input
    NET_VIDEO_CHANNEL_TYPE_OUTPUT,                      // output
} NET_VIDEO_CHANNEL_TYPE;

//input param of CLIENT_QueryDevInfo according to NET_QUERY_VIDEOCHANNELSINFO 
typedef struct tagNET_IN_GET_VIDEOCHANNELSINFO
{
    DWORD                               dwSize;
    NET_VIDEO_CHANNEL_TYPE              emType;         // video channel type to query                     
} NET_IN_GET_VIDEOCHANNELSINFO;

typedef struct tagNET_VIDEOCHANNELS_INPUT 
{
    int                     nThermographyCount;         // thermo channel count in nThermography
    int                     nThermography[64];          // thermo channel numbers
    int                     nMultiPreviewCount;         // multi preview channel count in nMultiPreview
    int                     nMultiPreview[4];	        // multi preview channel numbers
    int                     nPIPCount;                  // PIP channel count in nPIP
    int                     nPIP[4];    	            // PIP channel numbers
    int                     nCompressPlayCount;         // compress play channel count in nCompressPlay
    int                     nCompressPlay[4];	        // compress play channel numbers
	int						nSDCount;					// SD channel count in nSD
	int						nSD[64];					// SD channel numbers
    char                    reserved[252];
} NET_VIDEOCHANNELS_INPUT;

typedef struct tagNET_VIDEOCHANNELS_OUTPUT 
{
    int                     nVGACount;                  // VGA output count in nVGA
    int                     nVGA[128];                  // VGA output channel numbers
    int                     nTVCount;                   // TV output count in nTV
    int                     nTV[128];                   // TV output channel numbers
    char                    reserved[512];
} NET_VIDEOCHANNELS_OUTPUT;

//output param of CLIENT_QueryDevInfo according to NET_QUERY_VIDEOCHANNELSINFO
typedef struct tagNET_OUT_GET_VIDEOCHANNELSINFO
{
    DWORD                       dwSize;
    NET_VIDEOCHANNELS_INPUT     stInputChannels;        // input channel info        
    NET_VIDEOCHANNELS_OUTPUT    stOutputChannels;       // output channel info
} NET_OUT_GET_VIDEOCHANNELSINFO;

// CLIENT_QueryDevInfo , NET_QUERY_WORKGROUP_NAMES input
typedef struct tagNET_IN_WORKGROUP_NAMES
{
    DWORD                       dwSize;
} NET_IN_WORKGROUP_NAMES;

// max length for workgroup name
#define WORKGROUP_NAME_LEN      32

//CLIENT_QueryDevInfo , NET_QUERY_WORKGROUP_NAMES output
typedef struct tagNET_OUT_WORKGROUP_NAMES
{
    DWORD                       dwSize;
    int                         nCount;                         // workgroup count
    char                        szName[64][WORKGROUP_NAME_LEN]; // name of each workgroup
} NET_OUT_WORKGROUP_NAMES;

//CLIENT_QueryDevInfo , NET_QUERY_WORKGROUP_INFO input
typedef struct tagNET_IN_WORKGROUP_INFO
{
    DWORD                       dwSize;
    char                        szName[WORKGROUP_NAME_LEN];     // workgroup whose info you want to query                    
} NET_IN_WORKGROUP_INFO;

//CLIENT_QueryDevInfo , NET_QUERY_WORKGROUP_INFO input
typedef struct tagNET_OUT_WORKGROUP_INFO
{
    DWORD                       dwSize;
    int                         nState;                         // state: 0 nonsense, 1 normal, 2 damaged, 3 error
    int                         nTotalSpace;                    // Total Space Unit: MB, -1 for error
    int                         nFreeSpace;                     // Free Space Unit: MB -1 for error
} NET_OUT_WORKGROUP_INFO;

//CLIENT_QueryDevInfo , NET_QUERY_WLAN_ACCESSPOINT input
typedef struct tagNET_IN_WLAN_ACCESSPOINT
{
    DWORD                       dwSize;
    char                        szSSID[DH_MAX_SSID_LEN];        // SSID of wireless network whose info you want to query. if null, query all wireless network                    
} NET_IN_WLAN_ACCESSPOINT;

typedef struct tagNET_WLAN_ACCESSPOINT_INFO
{
    char                        szSSID[DH_MAX_SSID_LEN];        // SSID (name of wireless network)
    int                         nStrength;                      // signal strength, range: 0-100
    int                         nAuthMode;                      // attestation mod:0:OPEN;1:SHARED;2:WPA;3:WPA-PSK;4:WPA2;5:WPA2-PSK;
                                                                // 6:WPA-NONE(only use in Adhoc mode),
                                                                // 7-11 are mix mode,choose one of them can to be connected 
                                                                // 7:WPA-PSK | WPA2-PSK; 8:WPA | WPA2; 9:WPA | WPA-PSK;
                                                                // 10:WPA2 | WPA2-PSK; 11:WPA | WPA-PSK |WPA2 |WPA2-PSK //12: UnKnown
    int                         nEncrAlgr;                      // encrypt mod 0:off; 2:WEP64bit; 3:WEP128bit; 4:WEP; 5:TKIP; 6:AES(CCMP)
                                                                // 7: TKIP+AES( mix Mode) 8: UnKnown
    char                        reserved[1016];
} NET_WLAN_ACCESSPOINT_INFO;

//CLIENT_QueryDevInfo , NET_QUERY_WLAN_ACCESSPOINT output
typedef struct tagNET_OUT_WLAN_ACCESSPOINT
{
    DWORD                       dwSize;
    int                         nCount;                         // wireless network count
    NET_WLAN_ACCESSPOINT_INFO   stuInfo[64];                    // info for each network                       
} NET_OUT_WLAN_ACCESSPOINT;



// input for CLIENT_QueryDevInfo, NET_QUERY_TRAFFICRADAR_VERSION 
typedef struct tagNET_IN_TRAFFICRADAR_VERSION
{
    DWORD                   dwSize;
    int                     nChannel;                   // serial port number
} NET_IN_TRAFFICRADAR_VERSION;
// output of CLIENT_QueryDevInfo , NET_QUERY_TRAFFICRADAR_VERSION
typedef struct tagNET_OUT_TRAFFICRADAR_VERSION
{
    DWORD                   dwSize;
    char                    szVersion[DH_MAX_VERSION_LEN];  // version
} NET_OUT_TRAFFICRADAR_VERSION;

// input for CLIENT_AttachLanesState 
typedef NET_OUT_GET_LANES_STATE NET_CB_LANES_STATE;

// callback for CLIENT_AttachLanesState 
typedef void (CALLBACK *fNotifyLanesState)(LLONG lLanesStateHandle, NET_CB_LANES_STATE* pLanesStateInfo, LDWORD dwUser, void *reserved);

// port  CLIENT_AttachLanesState input parameter
typedef struct tagNET_IN_ATTACH_LANES_STATE
{
  DWORD       dwSize;              	// this structure size
  int         nChannelID;           // channel no.
  int         nLaneNumber;          // -1order all lanes, >=0 means order specific lane
  
  fNotifyLanesState  cbNotifyLanesState;       // call function
  LDWORD      dwUser;             	// user custom parameter
} NET_IN_ATTACH_LANES_STATE;

// port  CLIENT_AttachLanesState output parameter
typedef struct tagNET_OUT_ATTACH_LANES_STATE
{
  DWORD        dwSize;             // this structure size
} NET_OUT_ATTACH_LANES_STATE;


// CLIENT_MatrixSwitch The input parameters 
typedef struct tagNET_IN_MATRIX_SWITCH
{
    DWORD           dwSize;
    DH_SPLIT_MODE   emSplitMode;                // Segmentation Model 
    int*            pnOutputChannels;           // Output channel, can specify multiple output channel switch together at the same time, the content is consistent 
												// the space application by the user, apply to sizeof(int)*nOutputChannelCount
	int             nOutputChannelCount;        // Output channel number 
    int*            pnInputChannels;            // Input channel, each split window corresponding to one input channel 
												// the space application by the user, apply to sizeof(int)*nInputChannelCount
	int             nInputChannelCount;         // Input channel number
} NET_IN_MATRIX_SWITCH;

// CLIENT_MatrixSwitch Output Parameters 

typedef struct tagNET_OUT_MATRIX_SWITCH 
{
    DWORD            dwSize;
} NET_OUT_MATRIX_SWITCH;

// CLIENT_SplitSetMultiSource The Input Parameters of the Interface 
typedef struct tagNET_IN_SPLIT_SET_MULTI_SOURCE 
{
    DWORD                   dwSize;
    EM_VIDEO_OUT_CTRL_TYPE  emCtrlType;         // Video output control method 
    int                     nChannel;           // Video output logical channel number,Effective When emCtrlType is EM_VIDEO_OUT_CTRL_CHANNEL
    const char*             pszCompositeID;     // Splicing screen ID,When emCtrlType is EM_VIDEO_OUT_CTRL_COMPOSITE_ID
    BOOL                    bSplitModeEnable;   // Whether to change segmentation model 
    DH_SPLIT_MODE           emSplitMode;        // Segmentation model,Enable when bSplitModeEnable=TRUE
    int                     nGroupID;           // Segmentation group number,Effective when bSplitModeEnable=TRUE
    int*                    pnWindows;          // Window Number Array,the space application by the user, apply to sizeof(int)*nWindowCount
    int                     nWindowCount;       // Window Number
    DH_SPLIT_SOURCE*        pstuSources;        // Video source information, corresponding to each window, the number with the number of window.the space application by the user,apply to sizeof(DH_SPLIT_SOURCE)*nWindowCount
} NET_IN_SPLIT_SET_MULTI_SOURCE;

// CLIENT_SplitSetMultiSource The output parameters of the interface
typedef struct tagNET_OUT_SPLIT_SET_MULTI_SOURCE 
{
    DWORD                   dwSize;
} NET_OUT_SPLIT_SET_MULTI_SOURCE;

// Video split operation type
typedef enum tagNET_SPLIT_OPERATE_TYPE
{
    NET_SPLIT_OPERATE_SET_BACKGROUND,            // Set background, corresponding NET_IN_SPLIT_SET_BACKGROUND  and NET_OUT_SPLIT_SET_BACKBROUND
    NET_SPLIT_OPERATE_GET_BACKGROUND,            // get background, corresponding NET_IN_SPLIT_GET_BACKGROUND  and NET_OUT_SPLIT_GET_BACKGROUND
    NET_SPLIT_OPERATE_SET_PREPULLSRC,            // set pre stream srouce, corresponding to NET_IN_SPLIT_SET_PREPULLSRC and NET_OUT_SPLIT_SET_PREPULLSRC
    NET_SPLIT_OPERATE_SET_HIGHLIGHT,             // set source frame brightness switch, corresponding to NET_IN_SPLIT_SET_HIGHLIGHT and NET_OUT_SPLIT_SET_HIGHLIGHT
    NET_SPLIT_OPERATE_SET_ZORDER,                // adjust window Z order, corresponding to NET_IN_SPLIT_SET_ZORDER  and  NET_OUT_SPLIT_SET_ZORDER
    NET_SPLIT_OPERATE_SET_TOUR,                  // window tour control, corresponding to NET_IN_SPLIT_SET_TOUR  and  NET_OUT_SPLIT_SET_TOUR
    NET_SPLIT_OPERATE_GET_TOUR_STATUS,           // Get window tour status , corresponding to NET_IN_SPLIT_GET_TOUR_STATUS  and  NET_OUT_SPLIT_GET_TOUR_STATUS
    NET_SPLIT_OPERATE_GET_SCENE,                 // Get screen window info , corresponding to NET_IN_SPLIT_GET_SCENE  and  NET_OUT_SPLIT_GET_SCENE
    NET_SPLIT_OPERATE_OPEN_WINDOWS,              // batch window, corresponding to NET_IN_SPLIT_OPEN_WINDOWS  and  NET_OUT_SPLIT_OPEN_WINDOWS
    NET_SPLIT_OPERATE_SET_WORK_MODE,             // set work mode , corresponding to NET_IN_SPLIT_SET_WORK_MODE  and  NET_OUT_SPLIT_SET_WORK_MODE
    NET_SPLIT_OPERATE_GET_PLAYER,                // Get player example, corresponding to NET_IN_SPLIT_GET_PLAYER  and  NET_OUT_SPLIT_GET_PLAYER
    NET_WM_OPERATE_SET_WORK_MODE,                // Set window working mode, corresponding  NET_IN_WM_SET_WORK_MODE and NET_OUT_WM_SET_WORK_MODE
    NET_WM_OPERATE_GET_WORK_MODE,                // Get window working mode, corresponding  NET_IN_WM_GET_WORK_MODE and NET_OUT_WM_GET_WORK_MODE
    NET_SPLIT_OPERATE_CLOSE_WINDOWS,             // close batch windows NET_IN_SPLIT_CLOSE_WINDOWS o,a NET_OUT_SPLIT_CLOSE_WINDOWS
    NET_WM_OPERATE_SET_FISH_EYE_PARAM,           // set the output rules of the fish eyes, corresponding NET_IN_WM_SET_FISH_EYE_PARAM and NET_OUT_WM_SET_FISH_EYE_PARAM
	NET_WM_OPERATE_SET_CORRIDOR_MODE,			 // set the corridor mode of the window, corresponding NET_IN_WM_SET_CORRIDOR_MODE and NET_OUT_WM_SET_CORRIDOR_MODE
	NET_WM_OPERATE_GET_CORRIDOR_MODE,			 // get the corridor mode of the window, corresponding NET_IN_WM_GET_CORRIDOR_MODE and NET_OUT_WM_GET_CORRIDOR_MODE
	NET_WM_OPERATE_SET_VOLUME_COLUMN,			 // set volume column enable, corresponding NET_IN_WM_SET_VOLUME_COLUMN and NET_OUT_WM_SET_VOLUME_COLUMN
	NET_WM_OPERATE_GET_VOLUME_COLUMN,			 // get volume column enable, corresponding NET_IN_WM_GET_VOLUME_COLUMN and NET_OUT_WM_GET_VOLUME_COLUMN
	NET_WM_OPERATE_SET_BACKGROUND,				 // set the background of window, corresponding NET_IN_WM_SET_BACKGROUND and NET_OUT_WM_SET_BACKGROUND
	NET_WM_OPERATE_GET_BACKGROUND,				 // get the background of window, corresponding NET_IN_WM_GET_BACKGROUND and NET_OUT_WM_GET_BACKGROUND
} NET_SPLIT_OPERATE_TYPE;

// Set source frame brightness switch input parameter
typedef struct tagNET_IN_SPLIT_SET_HIGHLIGHT
{
    DWORD           dwSize; 
    int             nChannel;                   // video output channel
    int             nWindow;                    // window no.
    BOOL            bHighLightEn;               // frame brightness,TRUE-high brightness
    DH_COLOR_RGBA   stuColor;                   // frame color 
    int				nBlinkTimes;				// blink times of frame
    int				nBlinkInterval;				// the interval time between blink, unit: ms
}NET_IN_SPLIT_SET_HIGHLIGHT;

// set frame brightness enable switch output parameter
typedef struct tagNET_OUT_SPLIT_SET_HIGHLIGHT
{
    DWORD           dwSize;
}NET_OUT_SPLIT_SET_HIGHLIGHT;

// Set pre stream input parameter
typedef struct tagNET_IN_SPLIT_SET_PREPULLSRC 
{
    DWORD           dwSize;
    int             nChannel;                   // video output channel
    int             nWindow;                    // window no.
    int             nSrcCount;                  // pre stream source quantity
    DH_SPLIT_SOURCE* pSources;                  // re stream source info,the space application by the user, applt to sizeof(DH_SPLIT_SOURCE)*nSrcCount
} NET_IN_SPLIT_SET_PREPULLSRC;

// Set pre stream source return result
typedef struct tagNET_SPLIT_SET_PREPULLSRC_RESULT 
{
    DWORD           dwSize;
    BOOL            bResult;                    // set result, TRUE-successful, FALSE-failed
    DWORD           dwErrorCode;                // Error code
} NET_SPLIT_SET_PREPULLSRC_RESULT;

// Set pre source output parameter
typedef struct tagNET_OUT_SPLIT_SET_PREPULLSRC 
{
    DWORD           dwSize;
    int             nResultCount;               // result quantity, same with pre source    NET_SPLIT_SET_PREPULLSRC_RESULT* pResults;  // result
    NET_SPLIT_SET_PREPULLSRC_RESULT* pResults;  // result
} NET_OUT_SPLIT_SET_PREPULLSRC;

// Set video output background input parameter
typedef struct tagNET_IN_SPLIT_SET_BACKGROUND
{
    DWORD            dwSize;
    int              nChannel;                   // Video output channel no.
    BOOL             bEnable;                    // enable
    const char*      pszFileName;                // background name
} NET_IN_SPLIT_SET_BACKGROUND;

// Set video output background parameter
typedef struct tagNET_OUT_SPLIT_SET_BACKGROUND 
{
    DWORD            dwSize;
} NET_OUT_SPLIT_SET_BACKGROUND;

// get video output background input parameter
typedef struct tagNET_IN_SPLIT_GET_BACKGROUND 
{
    DWORD            dwSize;
    int              nChannel;                   // video output channel o.
} NET_IN_SPLIT_GET_BACKGROUND;

// get video output background output parameter
typedef struct tagNET_OUT_SPLIT_GET_BACKGROUND 
{
    DWORD			dwSize;
    BOOL            bEnable;                            // enable
    char            szFileName[DH_COMMON_STRING_256];   // background name
} NET_OUT_SPLIT_GET_BACKGROUND;

// the input parameter of setting window background
typedef struct tagNET_IN_WM_SET_BACKGROUND
{
	DWORD				dwSize;
	int					nChannel;							// video output channel ID, invalid when pszCompositeID == NULL
	char			   *pszCompositeID;						// composite ID
	int					nWindowID;							// windows ID
	BOOL				bEnable;							// whether set background or not
	char				szFileName[DH_COMMON_STRING_128];	// background name
} NET_IN_WM_SET_BACKGROUND;

// the output parameter of setting window background
typedef struct tagNET_OUT_WM_SET_BACKGROUND
{
	DWORD            	dwSize;
} NET_OUT_WM_SET_BACKGROUND;

//  the input parameter of getting window background
typedef struct tagNET_IN_WM_GET_BACKGROUND
{
	DWORD            	dwSize;
	int					nChannel;				// video output channel ID, invalid when pszCompositeID == NULL
	char			   *pszCompositeID;			// composite ID
	int					nWindowID;				// windows ID
} NET_IN_WM_GET_BACKGROUND;

//  the output parameter of getting window background
typedef struct tagNET_OUT_WM_GET_BACKGROUND
{
	DWORD            	dwSize;
	BOOL             	bEnable;                            // whether set background or not
    char             	szFileName[DH_COMMON_STRING_128];   // background name
} NET_OUT_WM_GET_BACKGROUND;


// window Z sequence
typedef enum tagNET_WINDOW_ZORDER
{
  NET_WINDOW_ZORDER_TOP,               // top
  NET_WINDOW_ZORDER_BOTTOM,            // bottom
  NET_WINDOW_ZORDER_UP,                // up
  NET_WINDOW_ZORDER_DOWN,              // down
} NET_WINDOW_ZORDER;

// window layer order
typedef struct tagDH_WND_ZORDER
{
  DWORD        	  dwSize;
  unsigned int    nWindowID;           // window no.
  unsigned int    nZOrder;             // Z order
} DH_WND_ZORDER;

// set window Z order input parameter
typedef struct tagNET_IN_SPLIT_SET_ZORDER 
{
  DWORD       		dwSize;
  int         		nChannel;            // output channel no. or cubeless video wall virtual channel no., pszCompositeID is NULL is valid 
  const char*     	pszCompositeID;      // cubeless video wall ID
  int         		nWindow;             // window no.
  NET_WINDOW_ZORDER emZOrder;            // adjust Z order
} NET_IN_SPLIT_SET_ZORDER;

// set window Z order output parameter, adjust one window Z order will affect all windows,  return adjusted all windows's Z order
typedef struct tagNET_OUT_SPLIT_SET_ZORDER
{
  DWORD        		dwSize;
  DH_WND_ZORDER*   	pZOders;            	// window order group, user allocate memory, apply to sizeof(DH_WND_ZORDER)*nMaxWndCount
  int         		nMaxWndCount;          	// window order group size
  int         		nWndCount;          	// return window quantity
} NET_OUT_SPLIT_SET_ZORDER;

// window tour motion 
typedef enum tagEM_NET_WINDOW_TOUR_ACTION 
{
  EM_NET_WND_TOUR_ACTION_START,           // start 
  EM_NET_WND_TOUR_ACTION_STOP,            // stop
} EM_NET_WINDOW_TOUR_ACTION;

// window tour control input parameter, corresponding to NET_SPLIT_OPERATE_SET_TOUR
typedef struct tagNET_IN_SPLIT_SET_TOUR 
{
  DWORD       dwSize;
  int         nChannel;            	// output channel no. or cubeless video wall virtual channel no., pszCompositeID is NULL is valid 
  const char* pszCompositeID;      	// cubeless video wall ID
  int         nWindow;            	// window no.
  EM_NET_WINDOW_TOUR_ACTION emAction;         // tour motion 
} NET_IN_SPLIT_SET_TOUR;

// window tour control output parameter, corresponding to NET_SPLIT_OPERATE_SET_TOUR
typedef struct tagNET_OUT_SPLIT_SET_TOUR 
{
  DWORD        dwSize;
} NET_OUT_SPLIT_SET_TOUR;

// Get window tour status input parameter, corresponding to NET_SPLIT_OPERATE_GET_TOUR_STATUS
typedef struct tagNET_IN_SPLIT_GET_TOUR_STATUS
{
  DWORD       dwSize;
  int         nChannel;            	// output channel no. or cubeless video wall virtual channel no., pszCompositeID is NULL is valid 
  const char* pszCompositeID;  		// cubeless video wall ID
  int         nWindow;             	// window no., -1 means all window
} NET_IN_SPLIT_GET_TOUR_STATUS;

// tour status 
typedef enum tagNET_TOUR_STATUS
{
  NET_TOUR_UNKNOWN,                // unknown 
  NET_TOUR_START,                  // touring
  NET_TOUR_STOP,                   // tour stop
}NET_TOUR_STATUS;

// window tour status info 
typedef struct tagNET_WINDOW_TOUR_STATUS_INFO 
{
  DWORD       dwSize;
  int         nWindow;            		 // window no.
  NET_TOUR_STATUS   emStatus;            // status 
} NET_WINDOW_TOUR_STATUS_INFO;

// Get window tour status output parameter, corresponding to NET_SPLIT_OPERATE_GET_TOUR_STATUS
typedef struct tagNET_OUT_SPLIT_GET_TOUR_STATUS
{
  DWORD        					dwSize;
  NET_WINDOW_TOUR_STATUS_INFO* 	pstuStatus;      // status info indicator, user allocation memory. search window no. is -1 means multi-window info group.
  int         					nMaxStatusCount; // status info max quantity, user input
  int         					nRetStatusCount; // status info actual quantity, search to get
} NET_OUT_SPLIT_GET_TOUR_STATUS;

// batch window input parameter, corresponding to NET_SPLIT_OPERATE_OPEN_WINDOWS
typedef struct tagNET_IN_SPLIT_OPEN_WINDOWS 
{
  DWORD       dwSize;
  int         nChannel;            	// output channel no. or cubeless video wall virtual channel no., pszCompositeID is NULL is valid 
  const char* pszCompositeID;       // cubeless video wall id
  int         nWindowNum;           // window quantity
  NET_RECT*   pstuWindowRects;      // window zone group,the space application by the user,apply to sizeof(NET_RECT)*nWindowNum
  BOOL        bDirectable;          // meet direct criteria 
} NET_IN_SPLIT_OPEN_WINDOWS;

// window info 
typedef struct tagNET_SPLIT_WINDOW_INFO
{
  DWORD       dwSize;
  int         nWindowID;          	// window no.
  int         nZOrder;            	// Z order
  char        szControlID[DH_DEV_ID_LEN];   // control no.
} NET_SPLIT_WINDOW_INFO;

// batch window output parameter, corresponding to NET_SPLIT_OPERATE_OPEN_WINDOWS
typedef struct tagNET_OUT_SPLIT_OPEN_WINDOWS 
{
  DWORD       dwSize;
  NET_SPLIT_WINDOW_INFO* pstuWindows;        // window info , user allocation memory, apply to sizeof(NET_SPLIT_WINDOW_INFO)*nMaxWindowCount
  int         nMaxWindowCount;        // window info max quantity, user input
  int         nRetWindowCount;        // window quantity
} NET_OUT_SPLIT_OPEN_WINDOWS;

// video split work mode 
typedef enum tagNET_SPLIT_WORK_MODE
{
  NET_SPLIT_WORK_MODE_UNKNOWN,            // unknown 
  NET_SPLIT_WORK_MODE_LOCAL,              // local normal mode 
  NET_SPLIT_WORK_MODE_REPLAY,             // playback mode 
}NET_SPLIT_WORK_MODE;

// setup work mode input parameter, corresponding to NET_SPLIT_OPERATE_SET_WORK_MODE
typedef struct tagNET_IN_SPLIT_SET_WORK_MODE 
{
  DWORD       dwSize;
  int         nChannel;            			// output channel no. or cubeless video wall virtual channel no., pszCompositeID is NULL is valid 
  const char* pszCompositeID;         		// cubeless video wall ID
  NET_SPLIT_WORK_MODE emMode;            	// work mode 
}NET_IN_SPLIT_SET_WORK_MODE;

// setup work mode output parameter, corresponding to NET_SPLIT_OPERATE_SET_WORK_MODE
typedef struct tagNET_OUT_SPLIT_SET_WORK_MODE 
{
  DWORD        dwSize;
}NET_OUT_SPLIT_SET_WORK_MODE;

// player type 
typedef enum tagNET_SPLIT_PLAYER_TYPE
{
  NET_SPLIT_PLAYER_TYPE_UNKNOWN,            // unknown 
  NET_SPLIT_PLAYER_TYPE_FILE_LIST,          // file list player
  NET_SPLIT_PLAYER_TYPE_FILE,             	// file player
}NET_SPLIT_PLAYER_TYPE;

// Get player actual input parameter, corresponding to NET_SPLIT_OPERATE_GET_PLAYER
typedef struct tagNET_IN_SPLIT_GET_PLAYER
{
  DWORD        dwSize;
  int          nChannel;            	// output channel no. or cubeless video wall virtual channel no., pszCompositeID is NULL is valid 
  const char*  pszCompositeID;         	// cubeless video wall ID
  NET_SPLIT_PLAYER_TYPE emType;         // player type 
  int          nWindow;            		// player window no.
}NET_IN_SPLIT_GET_PLAYER;

// Get player actual output parameter, corresponding to NET_SPLIT_OPERATE_GET_PLAYER
typedef struct tagNET_OUT_SPLIT_GET_PLAYER
{
  DWORD         dwSize;
  LLONG         lPlayerID;           // player actual ID
}NET_OUT_SPLIT_GET_PLAYER;

// Window Working Mode
typedef enum tagNET_WM_WORK_MODE
{
    NET_WM_WORK_MODE_UNKNOWN,                             // Unknown
    NET_WM_WORK_MODE_DISPLAY,                             // Preview mode
    NET_WM_WORK_MODE_REPLAY,                              // Playback mode  
}NET_WM_WORK_MODE;

// Set window working mode input parameter, corresponding  NET_WM_OPERATE_SET_WORK_MODE
typedef struct tagNET_IN_WM_SET_WORK_MODE
{
    DWORD                 dwSize;
    int                   nChannel;                       // Output channel no. or cubeless video wall virtual channel no., when pszCompositeID is NULL. valid
    const char*           pszCompositeID;                 // Cubeless video wall ID
    int                   nWindow;                        // Window no.
    NET_WM_WORK_MODE      emMode;                         // Window working mode
}NET_IN_WM_SET_WORK_MODE;

//Set window working mode output parameter, corresponding  NET_WM_OPERATE_SET_WORK_MODE
typedef struct tagNET_OUT_WM_SET_WORK_MODE
{
    DWORD                 dwSize;
}NET_OUT_WM_SET_WORK_MODE;

// Get window woking mode input parameter, corresponding  NET_WM_OPERATE_GET_WORK_MODE
typedef struct tagNET_IN_WM_GET_WORK_MODE
{
    DWORD                 dwSize;
    int                   nChannel;                       // Output channel no. or cubeless virtual channel no., when pszCompositeID is NULL, valid
    const char*           pszCompositeID;                 // Cubeless video wall ID
    int                   nWindow;                        // Window no.
}NET_IN_WM_GET_WORK_MODE;

//Get window working mode output parameter, corresponding  NET_WM_OPERATE_GET_WORK_MODE
typedef struct tagNET_OUT_WM_GET_WORK_MODE
{
    DWORD                 dwSize;
    NET_WM_WORK_MODE      emMode;                         // Window working mode
}NET_OUT_WM_GET_WORK_MODE;

// Set window corridor mode input parameter, corresponding  NET_WM_OPERATE_SET_CORRIDOR_MODE
typedef struct tagNET_IN_WM_SET_CORRIDOR_MODE
{
	DWORD                 dwSize;			// the value is the size of this struct
	int 				  nChannel;			// Output channel index or the virtual channel index of multi-screen display, it is valid when pszCompositeID =NULL
	const char* 		  pszCompositeID;	// Multi-screen display ID
	int 				  nWindow;			// window ID
	BOOL	  			  bIsCorridor;		// the corridor mode of the window(TRUE:open FALSE:close)
} NET_IN_WM_SET_CORRIDOR_MODE;

// Set window corridor mode output parameter, corresponding  NET_WM_OPERATE_SET_CORRIDOR_MODE
typedef struct tagNET_OUT_WM_SET_CORRIDOR_MODE
{
	DWORD                 dwSize;			// the value is the size of this struct
} NET_OUT_WM_SET_CORRIDOR_MODE;


// Get window corridor mode input parameter, corresponding  NET_WM_OPERATE_GET_CORRIDOR_MODE
typedef struct tagNET_IN_WM_GET_CORRIDOR_MODE
{
	DWORD                 dwSize;			// the value is the size of this struct
	int 				  nChannel;			// Output channel index or the virtual channel index of multi-screen display, it is valid when pszCompositeID =NULL
	const char* 		  pszCompositeID;	// Multi-screen display ID
	int 				  nWindow;			// window ID
} NET_IN_WM_GET_CORRIDOR_MODE;

// Get window corridor mode output parameter, corresponding  NET_WM_OPERATE_GET_CORRIDOR_MODE
typedef struct tagNET_OUT_WM_GET_CORRIDOR_MODE
{
	DWORD                 dwSize;			// the value is the size of this struct
	BOOL	  			  bIsCorridor;		// the corridor mode of window(TRUE:open FALSE:close)
} NET_OUT_WM_GET_CORRIDOR_MODE;

// set volume column enable intput parameter, corresponding NET_WM_OPERATE_SET_VOLUME_COLUMN
typedef struct tagNET_IN_WM_SET_VOLUME_COLUMN
{
	DWORD                 dwSize;			// size of struct
	int 				  nChannel;			// Output channel index or the virtual channel index of multi-screen display, it is valid when pszCompositeID =NULL
	const char* 		  pszCompositeID;	// Multi-screen display ID
	BOOL				  bIsEnable;	    // Display volume column(TRUE:Display FALSE:No Display)
} NET_IN_WM_SET_VOLUME_COLUMN;				

// set volume column enable output parameter, corresponding NET_WM_OPERATE_SET_VOLUME_COLUMN
typedef struct tagNET_OUT_WM_SET_VOLUME_COLUMN
{
	DWORD				   dwSize;			// size of struct
} NET_OUT_WM_SET_VOLUME_COLUMN;

// get volume column enable input parameter, corresponding NET_WM_OPERATE_GET_VOLUME_COLUMN
typedef struct tagNET_IN_WM_GET_VOLUME_COLUMN
{
	DWORD                 dwSize;			// size of struct
	int 				  nChannel;			// Output channel index or the virtual channel index of multi-screen display, it is valid when pszCompositeID =NULL
	const char* 		  pszCompositeID;	// Multi-screen display ID
} NET_IN_WM_GET_VOLUME_COLUMN;

// get volume column enable output parameter, corresponding NET_WM_OPERATE_GET_VOLUME_COLUMN
typedef struct tagNET_OUT_WM_GET_VOLUME_COLUMN
{
	DWORD				   dwSize;			// size of struct
	BOOL				   bIsEnable;		// Display volume column(TRUE:Display FALSE:No Display)	
} NET_OUT_WM_GET_VOLUME_COLUMN;

// The input for close batch windows,  corresponding to NET_SPLIT_OPERATE_CLOSE_WINDOWS
typedef struct tagNET_IN_SPLIT_CLOSE_WINDOWS
{
    DWORD               dwSize;
    int                 nChannel;                       // Output channel index or the virtual channel index of multi-screen display, it is valid when pszCompositeID =NULL
    const char*         pszCompositeID;                 // Multi-screen display ID
    int*                pnWindows;                      // The pointer of windows index array,the space application by the user,apply to sizeof(int)*nWindowCount
    int                 nWindowCount;                   // The number of windows
} NET_IN_SPLIT_CLOSE_WINDOWS;

// Close windows operation result
typedef struct tagNET_SPLIT_CLOSE_WINDOW_RESULT 
{
    BOOL                bResult;                        // Result
    char                reserved[256];                  // Reserved bytes
} NET_SPLIT_CLOSE_WINDOW_RESULT ;

// The output parameters of Close batch windows, corresponding to NET_SPLIT_OPERATE_CLOSE_WINDOWS
typedef struct tagNET_OUT_SPLIT_CLOSE_WINDOWS
{
    DWORD               dwSize;
    NET_SPLIT_CLOSE_WINDOW_RESULT* pstuResults;         // Result array, memory assigned by users, apply to sizeof(NET_SPLIT_CLOSE_WINDOW_RESULT)*nMaxResultCount. Set to Null if no need the return
    int                 nMaxResultCount;                // The maximum of result array, filled in by user.
	int                 nRetResultCount;                // The number of return result

} NET_OUT_SPLIT_CLOSE_WINDOWS;

// Display Screen Fisheye calibrate mode
typedef enum tagNET_WM_FISHEYE_CALIBRATE_MODE
{
    NET_WM_FISHEYE_CALIBRATE_MODE_UNKOWN ,            // unknown
    NET_WM_FISHEYE_CALIBRATE_MODE_CLOSE ,             // close 
    NET_WM_FISHEYE_CALIBRATE_MODE_ORIGINAL,           // original
    NET_WM_FISHEYE_CALIBRATE_MODE_PANORAMA,           // 1P
    NET_WM_FISHEYE_CALIBRATE_MODE_PAN_PLUS_ONE,       // 1P+1
    NET_WM_FISHEYE_CALIBRATE_MODE_DOUBLE_PANORAMA,    // 2P        
    NET_WM_FISHEYE_CALIBRATE_MODE_ORI_DOUBLE_PAN,     // 1+2p
    NET_WM_FISHEYE_CALIBRATE_MODE_ORI_PLUS_THREEE,    // 1+3
    NET_WM_FISHEYE_CALIBRATE_MODE_PAN_PLUS_THREEE,    // 1p+3
    NET_WM_FISHEYE_CALIBRATE_MODE_ORI_PLUS_TWO,       // 1+2
    NET_WM_FISHEYE_CALIBRATE_MODE_ORI_PLUS_FOUR,      // 1+4
    NET_WM_FISHEYE_CALIBRATE_MODE_PAN_PLUS_FOUR,      // 1p+4
    NET_WM_FISHEYE_CALIBRATE_MODE_PAN_PLUS_SIX,       // 1p+6
    NET_WM_FISHEYE_CALIBRATE_MODE_ORI_PLUS_EIGHT,     // 1+8
    NET_WM_FISHEYE_CALIBRATE_MODE_PAN_PLUS_EIGHT,     // 1P+8
}NET_WM_FISHEYE_CALIBRATE_MODE;

// Fisheye region parameter
typedef struct tagNET_WM_FISH_EYE_REGION_PARAM
{
    int     nCoordinateX;         // X Coordinate of window center to original circle
    int     nCoordinateY;         // Y Coordinate of window center to original circle
    int     nAngleH;              // Horizontal Angle of Fisheye calibrate region
    int     nAngleV;              // Vertical Angle of Fisheye calibrate region
    int 	nAvailable;			  // Available
    BYTE    Reserved[124];        // Reserverd
}NET_WM_FISH_EYE_REGION_PARAM;

#define MAX_FISH_EYE_REGION_NUM     9
// Set Fisheye init parameter, used for switching mode to last status
typedef struct tagNET_WM_SET_FISHEYE_INIT_PARAM
{
    BOOL                            bUseRegion;         // if True, using underside member to init; if FLASE, underside member is invaild; 
    int                             nCircular;          // Circular (Fisheye display mode with original, eg:1+3, 1+8 )
    int                             nPanorama;          // Panorama (Fisheye display mode with panorama, eg:1P, 2P )
	int                             nFishEyeRegionNum;  // FishEye region number 
    NET_WM_FISH_EYE_REGION_PARAM    stFishEyeRegions[MAX_FISH_EYE_REGION_NUM];    // FishEye region parameter information
    BYTE                            Reserved[1024];     // Reserved
}NET_WM_SET_FISHEYE_INIT_PARAM;

// Set FishEye EPTZ parameter
typedef struct tagNET_WM_SET_FISHEYE_EPTZ_PARAM
{
	int 							nOptWayType;		// operate way type 
	int 							nOptWinNum; 		// operate window number 
	int 							nOptWayData;		// operate way data
	BYTE                            Reserved[512];      // Reserved
}NET_WM_SET_FISHEYE_EPTZ_PARAM;


// The input parameters of Set FishEye parameter, corresponding to NET_IN_WM_SET_FISH_EYE_PARAM
typedef struct tagNET_IN_WM_SET_FISH_EYE_PARAM
{
    DWORD                           dwSize;
    int                             nChannel;           // output channel or Composite screen channel, valid when pszCompositeID is NULL 
    const char*                     pszCompositeID;     // Composite screen ID
    int                             nWindowID;          // Window ID corresponding to output screen

    NET_FISHEYE_MOUNT_MODE          emMount;            // Fisheye mount mode
    NET_WM_FISHEYE_CALIBRATE_MODE   emCalibrate;        // FishEye calibrate mode
    NET_WM_SET_FISHEYE_INIT_PARAM   stInitParam;        // FishEye init parameter
    NET_WM_SET_FISHEYE_EPTZ_PARAM	stEPtzParam;		// FishEye EPTZ parameter
}NET_IN_WM_SET_FISH_EYE_PARAM;

// The output parameters of Set FishEye parameter, corresponding to NET_OUT_WM_SET_FISH_EYE_PARAM
typedef struct tagNET_OUT_WM_SET_FISH_EYE_PARAM
{
    DWORD dwSize;
}NET_OUT_WM_SET_FISH_EYE_PARAM;


////////////////////////////////// System status////////////////////////////////////////

// CPU info
typedef struct tagDH_CPU_INFO
{
	DWORD				dwSize;
	int					nUsage;						// CPU usage
} DH_CPU_INFO;

// CPU status
typedef struct tagDH_CPU_STATUS
{
	DWORD				dwSize;
	BOOL				bEnable;					// Search succeeded or not
	int					nCount;						// CPU amount
	DH_CPU_INFO			stuCPUs[DH_MAX_CPU_NUM];	// CPU info
} DH_CPU_STATUS;

// Memory info
typedef struct tagDH_MEMORY_INFO
{
	DWORD				dwSize;
	DWORD				dwTotal;					// Total memory, M
	DWORD				dwFree;						// Free memory, M
} DH_MEMORY_INFO;

// Memory status
typedef struct tagDH_MEMORY_STATUS 
{
	DWORD				dwSize;
	BOOL				bEnable;					// Search succeeded or not
	DH_MEMORY_INFO		stuMemory;					// Memory info
} DH_MEMORY_STATUS;

// Fan info
typedef struct tagDH_FAN_INFO
{
	DWORD				dwSize;
	char				szName[DH_DEVICE_NAME_LEN];	// Name
	DWORD				nSpeed;						// Speed
} DH_FAN_INFO;

// Fan status
typedef struct tagDH_FAN_STATUS
{
	DWORD				dwSize;
	BOOL				bEnable;					// Search succeeded or not
	int					nCount;						// Fan amount
	DH_FAN_INFO			stuFans[DH_MAX_FAN_NUM];	// Fan status
} DH_FAN_STATUS;

// power current status type
typedef enum __EM_CURRENT_STATE_TYPE
{
    EM_CURRENT_STATE_UNKNOWN = 0,
    EM_CURRENT_STATE_OVER_CURRENT,                      // current too high
    EM_CURRENT_STATE_NORMAL,                            // current normal
    EM_CURRENT_STATE_UNDER_CURRENT,                     // current too low
}EM_CURRENT_STATE_TYPE;

// power voltage status type
typedef enum tagEM_VOLTAGE_STATE_TYPE
{
	EM_VOLTAGE_STATE_UNKNOWN,							// unknown
	EM_VOLTAGE_STATE_OVER,								// over 
	EM_VOLTAGE_STATE_NORMAL,							// normal
	EM_VOLTAGE_STATE_UNDER,								// under
} EM_VOLTAGE_STATE_TYPE;

// Power info
typedef struct tagDH_POWER_INFO
{
    DWORD               dwSize;
    BOOL                bPowerOn;                       // Power is on or not
    EM_CURRENT_STATE_TYPE emCurrentState;               // power current status
	EM_VOLTAGE_STATE_TYPE emVoltageState;				// power voltage status
} DH_POWER_INFO;


// battery in-place status
typedef enum __EM_BATTERY_EXIST_STATE
{
    EM_BATTERY_EXIST_STATE_UNKNOWN = 0,
    EM_BATTERY_EXIST_STATE_EXIST,                       // battery in-place
    EM_BATTERY_EXIST_STATE_MISSING,                     // battery lost
}EM_BATTERY_EXIST_STATE;

// battery status
typedef enum __EM_BATTERY_STATE
{
	EM_BATTERY_STATE_UNKNOWN = 0,
	EM_BATTERY_STATE_NORMAL,							// normal power
	EM_BATTERY_STATE_LOW,								// low power
}EM_BATTERY_STATE;

// Battery Information 
typedef struct tagDH_BATTERY_INFO
{
    DWORD               dwSize;
    int                 nPercent;                       // Battery Capacity Percentage
    BOOL                bCharging;                      // Whether real charging 
    EM_BATTERY_EXIST_STATE emExistState;                // battery in-place status
	EM_BATTERY_STATE	emState;						// battery power status	
	float				fVoltage;						// battery voltage
} DH_BATTERY_INFO;

// Power status
typedef struct tagDH_POWER_STATUS
{
	DWORD				dwSize;
	BOOL				bEnable;					// Search succeeded or not
	int					nCount;						// Power amount
	DH_POWER_INFO		stuPowers[DH_MAX_POWER_NUM];// Power status
    int                 nBatteryNum;                        // Battery Number
    DH_BATTERY_INFO     stuBatteries[DH_MAX_BATTERY_NUM];   // Battery Status 
} DH_POWER_STATUS;

// Temperature info
typedef struct tagDH_TEMPERATURE_INFO
{
	DWORD				dwSize;
	char				szName[DH_DEVICE_NAME_LEN];	// Sensor name
	float				fTemperature;				// Temperature
} DH_TEMPERATURE_INFO;

//Temperature status
typedef struct tagDH_TEMPERATURE_STATUS
{
	DWORD				dwSize;
	BOOL				bEnable;							// Search succeeded or not
	int					nCount;								// Temperature  amount
	DH_TEMPERATURE_INFO	stuTemps[DH_MAX_TEMPERATURE_NUM];	// Temperature  info
} DH_TEMPERATURE_STATUS;

// System status
typedef struct tagDH_SYSTEM_STATUS
{
	DWORD				dwSize;
	DH_CPU_STATUS*		pstuCPU;					// CPU status
	DH_MEMORY_STATUS*	pstuMemory;					// Memory status
	DH_FAN_STATUS*		pstuFan;					// Fan status
	DH_POWER_STATUS*	pstuPower;					// Power status
	DH_TEMPERATURE_STATUS*	pstuTemp;				// Temperature  status
} DH_SYSTEM_STATUS;

// Corresponding CLIENT_QueryDevState() Interface's DH_DEVSTATE_ALL_ALARM_CHANNELS_STATE Command Parameter 
// For Alarm Channel Status 
typedef enum tagNET_ALARM_CHANNEL_TYPE
{
    NET_ALARM_CHANNEL_TYPE_ALL,                             // All channels (including all of the following)
    NET_ALARM_CHANNEL_TYPE_ALARMIN,                         // Alarm input channel 
    NET_ALARM_CHANNEL_TYPE_ALARMOUT,                        // Alarm output channel 
    NET_ALARM_CHANNEL_TYPE_ALARMBELL,                       // Signal channel 
    NET_ALARM_CHANNEL_TYPE_EXALARMIN,                       // Extension module alarm input channel 
    NET_ALARM_CHANNEL_TYPE_EXALARMOUT,                      // Extension module and alarm output channel
}NET_ALARM_CHANNEL_TYPE;

typedef struct tagNET_CLIENT_ALARM_CHANNELS_STATE
{
    DWORD                       dwSize;
    NET_ALARM_CHANNEL_TYPE      emType;                     // The type of query alarm channel 	                                                                                                                                
    int                         nAlarmInCount;              // Alarm input number, specified by the user 
    int                         nAlarmInRetCount;           // The number of returned alarm input 
    BOOL*                       pbAlarmInState;             // Alarm state of input arrays, memory allocated by the user, each element represents a channel status, TRUE for input, FALSE for no input 
															// the space application by the user,apply to sizeof(BOOL)*nAlarmInCount
	int                         nAlarmOutCount;             // The number of alarm output,specified by the user 
    int                         nAlarmOutRetCount;          // The number of alarm output 
    BOOL*                       pbAlarmOutState;            // Alarm state of output arrays, memory allocated by the user, each element represents a channel status, TRUE for output, FALSE for no output 
															// the space application by the user,apply to sizeof(BOOL)*nAlarmOutCount
	int                         nAlarmBellCount;            // Alarm Number,specified by the user 
    int                         nAlarmBellRetCount;         // Returned Alarm Number 
    BOOL*                       pbAlarmBellState;           // Alarm state array, allocate memory by the user, each element represents a channel status, TRUE for output, FALSE for no output
															// the space application by the user,apply to sizeof(BOOL)*nAlarmBellCount
	int                         nExAlarmInCount;            // Extension module alarm input number, specified by the user 
    int                         nExAlarmInRetCount;         // The number of returned extension module alarm input 
    BOOL*                       pbExAlarmInState;           // Extension module alarm input state array, Memory allocated by the user, each element represents a channel status, TRUE for output, FALSE for no output 
															// the space application by the user,apply to sizeof(BOOL)*nExAlarmInCount
	int*                        pnExAlarmInDestionation;    // The location of the extension module alarm input effective channels 
															// the space application by the user,apply to sizeof(int)*nExAlarmInCount
	int                         nExAlarmOutCount;           // Extension module alarm output number, specified by the user
    int                         nExAlarmOutRetCount;        // Extension module alarm output number 
    BOOL*                       pbExAlarmOutState;          // Extension module alarm output state array, Memory allocated by the user, each element represents a channel status, TRUE for output, FALSE for no output 
															// the space application by the user,apply to sizeof(BOOL)*nExAlarmOutCount
	int*                        pnExAlarmOutDestionation;   // The location of the extension module alarm output effective channels
															// the space application by the user,apply to sizeof(int)*nExAlarmOutCount
}NET_CLIENT_ALARM_CHANNELS_STATE;


// the number of alarm keyboard connecting on a serial port
typedef struct tagNET_ALARMKEYBOARD_COUNT
{
    DWORD               dwSize;
    int                 nAlarmKeyboardCount;        // The number of alarm keyboard connected 
}NET_ALARMKEYBOARD_COUNT;

// the device type on com port
typedef enum tagEM_COM_PORT_DEVICE_TYPE
{
    EM_COM_PORT_DEVICE_TYPE_UNKNOWN,        // Unknown
    EM_COM_PORT_DEVICE_TYPE_PTZ,            // ptz
    EM_COM_PORT_DEVICE_TYPE_GPS,            // GPS
    EM_COM_PORT_DEVICE_TYPE_ALARMKEYBOARD,  // alarm keyboard
    EM_COM_PORT_DEVICE_TYPE_EXALARMBOX,     // Exalarm box
}EM_COM_PORT_DEVICE_TYPE;

// CLIENT_GetComPortDeviceChannelInfo's interface input param(get the device channel info on com port)
typedef struct tagNET_IN_GET_COM_PORT_DEVICE_CHANNEL_INFO
{
    DWORD               dwSize;
    EM_COM_PORT_DEVICE_TYPE emDeviceType;   // device type
}NET_IN_GET_COM_PORT_DEVICE_CHANNEL_INFO;

// CLIENT_GetComPortDeviceChannelInfo's interface output param(get the device channel info on com port)
typedef struct tagNET_OUT_GET_COM_PORT_DEVICE_CHANNEL_INFO
{
    DWORD               dwSize;    
    int                 nDeviceCount;               // device count(out param,include online and offline devices)
    int                 nMaxOnlineDeviceCount;      // max number of online device(in param)
    int                 nRetOnlineDeviceCount;      // online device number(out param)      
    int*                pOnlineChannel;             // online device channel array(in param,alloc by user)    
}NET_OUT_GET_COM_PORT_DEVICE_CHANNEL_INFO;

// Get the device channel info on com port
CLIENT_NET_API BOOL CALL_METHOD CLIENT_GetComPortDeviceChannelInfo(LLONG lLoginID,const NET_IN_GET_COM_PORT_DEVICE_CHANNEL_INFO* pInParam,NET_OUT_GET_COM_PORT_DEVICE_CHANNEL_INFO* pOutParam,int nWaitTime);

////////////////////////////////// screen control////////////////////////////////////////
// CLIENT_OpenSplitWindow's interface input param(open window)
typedef struct tagDH_IN_SPLIT_OPEN_WINDOW
{
	DWORD				dwSize;
	int					nChannel;					// channel no.
	DH_RECT				stuRect;					// windon position, 0~8192
	BOOL				bDirectable;				// coordinate whether meet the confitions
} DH_IN_SPLIT_OPEN_WINDOW;

// CLIENT_OpenSplitWindow's interface output param(open window)
typedef struct tagDH_OUT_SPLIT_OPEN_WINDOW
{
	DWORD				dwSize;
	unsigned int		nWindowID;					// window ID
	unsigned int		nZOrder;					// window order	
} DH_OUT_SPLIT_OPEN_WINDOW;

// CLIENT_CloseSplitWindow's interface input param(close window)
typedef struct tagDH_IN_SPLIT_CLOSE_WINDOW
{
	DWORD				dwSize;
	int					nChannel;					// channel ID
	UINT				nWindowID;					// window order
    const char*         pszCompositeID;             // multi-screen display ID
} DH_IN_SPLIT_CLOSE_WINDOW;

// CLIENT_CloseSplitWindow's interface output param(close window)
typedef struct tagDH_OUT_SPLIT_CLOSE_WINDOW
{
	DWORD				dwSize;
} DH_OUT_SPLIT_CLOSE_WINDOW;

// CLIENT_SetSplitWindowRect's interface input param(setting the window position)
typedef struct tagDH_IN_SPLIT_SET_RECT
{
	DWORD				dwSize;
	int					nChannel;					// channel ID
	UINT				nWindowID;					// window order
	DH_RECT				stuRect;					// window position, 0~8192
	BOOL				bDirectable;				// coordinate whether meet the confitions
} DH_IN_SPLIT_SET_RECT;

// CLIENT_SetSplitWindowRect's interface output param(setting the window position)
typedef struct tagDH_OUT_SPLIT_SET_RECT
{
	DWORD				dwSize;
} DH_OUT_SPLIT_SET_RECT;

// CLIENT_GetSplitWindowRect's interface input param(get window position)
typedef struct tagDH_IN_SPLIT_GET_RECT
{
	DWORD				dwSize;
	int					nChannel;					// channel ID
	UINT				nWindowID;					// window ID
} DH_IN_SPLIT_GET_RECT;

// CLIENT_GetSplitWindowRect's interface output param(get window position)
typedef struct tagDH_OUT_SPLIT_GET_RECT
{
	DWORD				dwSize;	
	DH_RECT				stuRect;					// window position, 0~8192
} DH_OUT_SPLIT_GET_RECT;

// CLIENT_SetSplitWindowLevels's interface input param(setting window order)
typedef struct tagDH_IN_SPLIT_SET_TOP_WINDOW
{
	DWORD				dwSize;
	int					nChannel;					// channel ID
	int					nWindowID;					// windown ID
} DH_IN_SPLIT_SET_TOP_WINDOW;

// CLIENT_SetSplitWindowLevels's interface output param(setting window order)
typedef struct tagDH_OUT_SPLIT_SET_TOP_WINDOW
{
	DWORD				dwSize;
	DH_WND_ZORDER*		pZOders;					// window order array,the space application by the user,apply to sizeof(DH_WND_ZORDER)*nMaxWndCount
	int					nMaxWndCount;				// window order array size
	int					nWndCount;					// count of returned window
} DH_OUT_SPLIT_SET_TOP_WINDOW;

// CLIENT_SetDisplayMode's interface input param(monitor wall display mode settings)
typedef struct tagDH_IN_WM_SET_DISPLAY_MODE 
{
	DWORD				dwSize;
	int					nMonitorWallID;				// monitor wall ID
	const char*			pszBlockID;					// block ID, NULL/""-all region
	int					nTVID;						// display unit ID, -1 means all unit
	UINT				nDisplayMode;				// display mode, 0-standard, 1-highlighting, 2-energy saving
} DH_IN_WM_SET_DISPLAY_MODE;

// CLIENT_SetDisplayMode's interface output param(monitor wall display mode settings)
typedef struct tagDH_OUT_WM_SET_DISPLAY_MODE
{
	DWORD				dwSize;
} DH_OUT_WM_SET_DISPLAY_MODE;

// all display unit in the area of the display mode
typedef struct tagDH_BLOCK_DISPLAY_MODE
{
	DWORD				dwSize;
	int*				pTVModes;					// TV display mode
	int					nMaxTVCount;				// max count of tV
	int					nRetTVCount;				// count of retTV
} DH_BLOCK_DISPLAY_MODE;

// CLIENT_GetDisplayMode's interface input param(get monitor wall display mode)
typedef struct tagDH_IN_WM_GET_DISPLAY_MODE 
{
	DWORD				dwSize;
	int					nMonitorWallID;				// monitor wall ID
	const char*			pszBlockID;					// block ID, NULL/""-means all
	int					nTVID;						// TV ID, -1 means all
} DH_IN_WM_GET_DISPLAY_MODE;

// CLIENT_GetDisplayMode's interface output param(get monitor wall display mode)
typedef struct tagDH_OUT_WM_GET_DISPLAY_MODE
{
	DWORD				dwSize;
	DH_BLOCK_DISPLAY_MODE* pBlockModes;				// display mode, the space appliacation by the user,apply to sizeof(DH_BLOCK_DISPLAY_MODE)*nMaxBlockCount
	int					nMaxBlockCount;				// count of array
	int					nRetBlockCount;				// count of ret 
} DH_OUT_WM_GET_DISPLAY_MODE;

// CLIENT_PowerControl's interface input param(monitor wall power control)
typedef struct tagDH_IN_WM_POWER_CTRL 
{
	DWORD				dwSize;
	int					nMonitorWallID;				// monitor wall ID
	const char*			pszBlockID;					// block ID, NULL/""- means all
	int					nTVID;						// TV ID, -1 means all
	BOOL				bPowerOn;					// power on or not
} DH_IN_WM_POWER_CTRL;

// CLIENT_PowerControl's interface input param(monitor wall power control)
typedef struct tagDH_OUT_WM_POWER_CTRL
{
	DWORD				dwSize;
} DH_OUT_WM_POWER_CTRL;

// CLIENT_LoadMonitorWallCollection's interface input param(load monitor wall plans)
typedef struct tagDH_IN_WM_LOAD_COLLECTION 
{
	DWORD				dwSize;
	int					nMonitorWallID;				// monitor wall ID
	const char*			pszName;					// name of plan
} DH_IN_WM_LOAD_COLLECTION;

// CLIENT_LoadMonitorWallCollection's interface output param(load monitor wall plans)
typedef struct tagDH_OUT_WM_LOAD_COLLECTION 
{
	DWORD				dwSize;
} DH_OUT_WM_LOAD_COLLECTION;

// CLIENT_SaveMonitorWallCollection's interface input param(save monitor wall plans)
typedef struct tagDH_IN_WM_SAVE_COLLECTION 
{
	DWORD				dwSize;
	int					nMonitorWallID;				// monitor wall ID
	const char*			pszName;					// name pf plan
    const char*         pszControlID;               // control id
} DH_IN_WM_SAVE_COLLECTION;

// CLIENT_SaveMonitorWallCollection's interface output param(load monitor wall plans)
typedef struct tagDH_OUT_WM_SAVE_COLLECTION 
{
	DWORD				dwSize;
} DH_OUT_WM_SAVE_COLLECTION;

// CLIENT_RenameMonitorWallCollection's interface input param(plan rename)
typedef struct tagDH_IN_WM_RENAME_COLLECTION 
{
	DWORD				dwSize;
	int					nMonitorWallID;				// monitor wall ID
	const char*			pszOldName;					// old name
	const char*			pszNewName;					// new name
} DH_IN_WM_RENAME_COLLECTION;

// CLIENT_RenameMonitorWallCollection's interface input param(plan rename)
typedef struct tagDH_OUT_WM_RENAME_COLLECTION
{
	DWORD				dwSize;
} DH_OUT_WM_RENAME_COLLECTION;

// information window areas
typedef struct tagDH_WINDOW_COLLECTION
{
	DWORD				dwSize;
	int					nWindowID;					// window ID
	BOOL				bWndEnable;					// enable
	DH_RECT				stuRect;					// rect, effect when free split mode
	BOOL				bDirectable;				// coordinate whether meet the conditions
	int					nZOrder;					// z order
	BOOL				bSrcEnable;					// source enable
	char				szDeviceID[DH_DEV_ID_LEN_EX]; // device ID
	int					nVideoChannel;				// video channel
	int					nVideoStream;				// video stream type
	int					nAudioChannel;				// audio channel
	int					nAudioStream;				// audio stream type
	int					nUniqueChannel;				// unique channel
} DH_WINDOW_COLLECTION;

// areas collection
typedef struct tagDH_BLOCK_COLLECTION 
{
	DWORD				dwSize;
	DH_SPLIT_MODE		emSplitMode;					// split mode
	DH_WINDOW_COLLECTION stuWnds[DH_MAX_SPLIT_WINDOW];	// window info array
	int					nWndsCount;						// count of window
	char				szName[DH_DEVICE_NAME_LEN];		// favorites name
    int                 nScreen;                       	// The output channel number, including the splicing screen 
    char                szCompositeID[DH_DEV_ID_LEN_EX]; // splicing video wall ID
    DH_WINDOW_COLLECTION* pstuWndsEx;                  	// Windows information array pointer, the memory assigned by user. Use when the size of stuWnds array is not enough.  
    int                 nMaxWndsCountEx;               	// The maximum number of windows, filled by user. pstuWndsEx the muber of the array element.
    int                 nRetWndsCountEx;               	// The number of return windows.
} DH_BLOCK_COLLECTION;

// TV wall display unit
typedef struct tagDH_MONITORWALL_OUTPUT 
{
    DWORD       dwSize;
    char        szDeviceID[DH_DEV_ID_LEN];     	// device ID, local time is""
    int         nChannel;              			// channel no.
    char        szName[DH_DEV_NAME_LEN];      	// screen name 
} DH_MONITORWALL_OUTPUT;

// TV wall display block
typedef struct tagDH_MONITORWALL_BLOCK
{
  DWORD         dwSize;
  char          szName[DH_DEV_NAME_LEN];    	// block name 
  char          szCompositeID[DH_DEV_ID_LEN];  	// splicing video wall ID
  char          szControlID[DH_DEV_ID_LEN];   	// control ID
  int           nSingleOutputWidth;       		// single display unit occupied grid column number
  int           nSingleOutputHeight;      		// single display unit occupied grid row number
  DH_RECT       stuRect;            			// zone coordinate
  DH_TSECT      stuPowerSchedule[DH_TSCHE_DAY_NUM][DH_TSCHE_SEC_NUM]; // switch on time chart, first-dimension each element means sunday~saturday and holiday
  DH_MONITORWALL_OUTPUT* pstuOutputs;          	// display unit data,  user allocation memory
  int           nMaxOutputCount;        		// display unit group size,user fill in
  int           nRetOutputCount;        		// return display unit quantity
} DH_MONITORWALL_BLOCK;

// tv wall config 
typedef struct tagDH_MONITORWALL 
{
  DWORD         dwSize;
  char          szName[DH_DEV_NAME_LEN];    // name 
  int           nGridLine;           		// grid row
  int           nGridColume;          		// grid column
  DH_MONITORWALL_BLOCK*  pstuBlocks;        // display block group, user allocation nmemory
  int           nMaxBlockCount;         	// display zone group size, user fill in
  int           nRetBlockCount;         	// return display block quantity
  BOOL          bDisable;            		// disable or not, 0-tv wall is valid , 1-tv wall is invalid
  char          szDesc[DH_COMMON_STRING_256];  // tv wall description info 
} DH_MONITORWALL;


// plan of monitor wall
typedef struct tagDH_MONITORWALL_COLLECTION 
{
	DWORD				dwSize;
	char				szName[DH_DEVICE_NAME_LEN];		// name
	DH_BLOCK_COLLECTION	stuBlocks[DH_MAX_BLOCK_NUM];	// area array
	int					nBlocksCount;					// area count
	char				szControlID[DH_DEV_ID_LEN_EX];	// control ID
    DH_MONITORWALL      stuMonitorWall;             	// tv wall config 
} DH_MONITORWALL_COLLECTION;

// CLIENT_GetMonitorWallCollections's interface input param(get plan information of monitor wall)
typedef struct tagDH_IN_WM_GET_COLLECTIONS 
{
	DWORD				dwSize;
	int					nMonitorWallID;				// monitor wall ID
} DH_IN_WM_GET_COLLECTIONS;

// CLIENT_GetMonitorWallCollections's interface output param(get plan information of monitor wall)
typedef struct tagDH_OUT_WM_GET_COLLECTIONS 
{
	DWORD				dwSize;	
	DH_MONITORWALL_COLLECTION* pCollections;		// plan of monitor wall array
	int					nMaxCollectionsCount;		// array size
	int					nCollectionsCount;			// max count of plan
} DH_OUT_WM_GET_COLLECTIONS;

// CLIENT_GetSplitWindowsInfo's interface input param
typedef struct tagDH_IN_SPLIT_GET_WINDOWS 
{
	DWORD				dwSize;
	int					nChannel;					// channel
} DH_IN_SPLIT_GET_WINDOWS;

// CLIENT_GetSplitWindowsInfo's interface output param
typedef struct tagDH_OUT_SPLIT_GET_WINDOWS 
{
	DWORD				dwSize;
	DH_BLOCK_COLLECTION	stuWindows;					// channel
} DH_OUT_SPLIT_GET_WINDOWS;

// CLIENT_LoadSplitCollection's interface input param(load collection)
typedef struct tagDH_IN_SPLIT_LOAD_COLLECTION 
{
	DWORD				dwSize;
	int					nChannel;					// channel
	const char*			pszName;					// name
} DH_IN_SPLIT_LOAD_COLLECTION;

// CLIENT_LoadSplitCollection's interface output param(load collection)
typedef struct tagDH_OUT_SPLIT_LOAD_COLLECTION 
{
	DWORD				dwSize;
} DH_OUT_SPLIT_LOAD_COLLECTION;

// CLIENT_SaveSplitCollection's interface input param(save collection)
typedef struct tagDH_IN_SPLIT_SAVE_COLLECTION 
{
	DWORD				dwSize;
	int					nChannel;					// channle
	const char*			pszName;					// name
} DH_IN_SPLIT_SAVE_COLLECTION;

// CLIENT_SaveSplitCollection's interface input param(save collection)
typedef struct tagDH_OUT_SPLIT_SAVE_COLLECTION 
{
	DWORD				dwSize;
} DH_OUT_SPLIT_SAVE_COLLECTION;

// CLIENT_RenameSplitCollection's interface input param(collection rename)
typedef struct tagDH_IN_SPLIT_RENAME_COLLECTION 
{
	DWORD				dwSize;
	int					nChannel;					// channel
	const char*			pszOldName;					// old name
	const char*			pszNewName;					// new name
} DH_IN_SPLIT_RENAME_COLLECTION;

// CLIENT_RenameSplitCollection's interface output param(collection rename)
typedef struct tagDH_OUT_SPLIT_RENAME_COLLECTION
{
	DWORD				dwSize;
} DH_OUT_SPLIT_RENAME_COLLECTION;

// CLIENT_GetSplitCollections's interface input param(get information of favorites)
typedef struct tagDH_IN_SPLIT_GET_COLLECTIONS 
{
	DWORD				dwSize;
	int					nChannel;					// channel
} DH_IN_SPLIT_GET_COLLECTIONS;

// CLIENT_GetSplitCollections's interface output param(get information of favorites)
typedef struct tagDH_OUT_SPLIT_GET_COLLECTIONS 
{
	DWORD				dwSize;	
	DH_BLOCK_COLLECTION* pCollections;				// array,the space application by the user,apply to sizeof(DH_BLOCK_COLLECTION)*nMaxCollectionsCount
	int					nMaxCollectionsCount;		// size
	int					nCollectionsCount;			// count
} DH_OUT_SPLIT_GET_COLLECTIONS;

// CLIENT_DeleteSplitCollection's interface input param(delete favorites)
typedef struct tagDH_IN_SPLIT_DELETE_COLLECTION 
{
	DWORD				dwSize;
	int					nChannel;					// channel
	const char**		ppszNames;					// array of favorites name
	int					nNameCount;					// size of array
} DH_IN_SPLIT_DELETE_COLLECTION;

// CLIENT_DeleteSplitCollection's interface input param(delete favorites)
typedef struct tagDH_OUT_SPLIT_DELETE_COLLECTION 
{
	DWORD				dwSize;
} DH_OUT_SPLIT_DELETE_COLLECTION;

// composite screen channel information
typedef struct tagDH_COMPOSITE_CHANNEL
{
	DWORD				dwSize;
	char				szMonitorWallName[DH_DEVICE_NAME_LEN];	// monitor wall name
	char				szCompositeID[DH_DEV_ID_LEN_EX];		// composite ID
	int					nVirtualChannel;						// virtual channel
} DH_COMPOSITE_CHANNEL;

// tour enable configuration
typedef struct tagDHDEV_TOUR_ENABLE
{
	DWORD				dwSize;
	BOOL				bEnable;					// enable
	BOOL				bStart;						// is touring or not(read only). bStart=FALSE when tour is able but doesn't set
} DHDEV_TOUR_ENABLE_CFG;

// CLIENT_SetDecodePolicy's interface input param(set the decoding policy)
typedef struct tagDH_IN_SET_DEC_POLICY 
{
	DWORD				dwSize;
	int					nChannel;			// channel
	int					nWindow;			// window no., -1 means all
	int					nPolicyLevel;		// policy level, a total of 5 file(-2, -1, 0, 1, 2), the greater the vale ,the bigger the fluid but delay
											// -2 real-time good, 2 fluency good, 0 default
} DH_IN_SET_DEC_POLICY;

// CLIENT_SetDecodePolicy's interface output param(set the decoding policy)
typedef struct tagDH_OUT_SET_DEC_POLICY 
{
	DWORD				dwSize;
} DH_OUT_SET_DEC_POLICY;

// CLIENT_GetDecodePolicy's interface input param(get the decoding policy)
typedef struct tagDH_IN_GET_DEC_POLICY 
{
	DWORD				dwSize;
	int					nChannel;			// channel
	int					nWindow;			// window ID, -1 means all
} DH_IN_GET_DEC_POLICY;

// CLIENT_GetDecodePolicy's interface output param(get the decoding policy)
typedef struct tagDH_OUT_GET_DEC_POLICY 
{
	DWORD				dwSize;
	int					nPolicyLevel;		// policy level, a total of 5 file(-2, -1, 0, 1, 2), the greater the vale ,the bigger the fluid but delay
											// -2 real-time good, 2 fluency good, 0 default
} DH_OUT_GET_DEC_POLICY;

// audio output mode
typedef enum
{
	DH_AUDIO_AUTO,							// automatic switch
	DH_AUDIO_DISABLE,						// all output disable 
	DH_AUDIO_FORCE,							// mandatory audio output to the user to specify a window
    DH_AUDIO_ENABLE_ONE,                    // Open the audio of target window, multi-channel of audio output is available    
    DH_AUDIO_DISABLE_ONE,                   // Close the audio of target window, multi-channel of audio output is available    
    DH_AUDIO_MULTI,                         // Multi-channel of audio output, be used when inquire not setting
} DH_AUDIO_OUTPUT_MODE;

// CLIENT_SetSplitAudioOuput's interface input param(set mode of audio output)
typedef struct tagDH_IN_SET_AUDIO_OUTPUT 
{
	DWORD				dwSize;
	int					nChannel;			// channel ID
	DH_AUDIO_OUTPUT_MODE emMode;			// mode
	int					nWindow;			// window no. , effect when emMode = DH_AUDIO_FORCE
} DH_IN_SET_AUDIO_OUTPUT;

// CLIENT_SetSplitAudioOuput's interface output param(set mode of audio output)
typedef struct tagDH_OUT_SET_AUDIO_OUTPUT
{
	DWORD				dwSize;
} DH_OUT_SET_AUDIO_OUTPUT;

// CLIENT_GetSplitAudioOuput's interface input param(get mode of audio output)
typedef struct tagDH_IN_GET_AUDIO_OUTPUT
{
	DWORD				dwSize;
	int					nChannel;			// channel ID
} DH_IN_GET_AUDIO_OUTPUT;

// CLIENT_GetSplitAudioOuput's interface output param(get mode of audio output)
typedef struct tagDH_OUT_GET_AUDIO_OUTPUT
{
	DWORD				dwSize;
	DH_AUDIO_OUTPUT_MODE emMode;				// mode of audio output
	int					nWindow;				// window no., effect when emMode = DH_AUDIO_FORCE
    int*                pMultiWindows;         	// The list of output windows number, valid when emMode = DH_AUDIO_MULTI, user assign the memory,apply to sizeof(int)*nMaxMultiWindowCount
    int                 nMaxMultiWindowCount;  	// ouput the maximum number of window list, filled by user
    int                 nRetMultiWindowCount;  	// Output the number of windows, effect  when  emMode=DH_AUDIO_MULTI
} DH_OUT_GET_AUDIO_OUTPUT;

// CLIENT_GetEncodePlan's interface output param(access to burn a CD to yer coding parameters)
typedef struct tagDH_IN_GET_ENCODE_PLAN
{
	DWORD				dwSize;
	unsigned int        nChannel;          // channel
	unsigned int        nExpectTime;       // expect time,int(min)
	unsigned int        nCapacity;         // capacity,int(M)
}DH_IN_GET_ENCODE_PLAN;
// CLIENT_GetEncodePlan's interface output param(access to burn a CD to yer coding parameters)
typedef struct tagDH_OUT_GET_ENCODE_PLAN
{
	DWORD				dwSize;
	int                 nResolutionType;    // emResolutionTypes counts
	CAPTURE_SIZE        emResolutionTypes[DH_MAX_CAPTURE_SIZE_NUM];// the scope of video resolution
	CAPTURE_SIZE        emResolution;       // the recommended video resolution
	unsigned int        nBitRate;           // bit rate
}DH_OUT_GET_ENCODE_PLAN;

// CLIENT_GetEncodePlanByResolution's interface input param(get expect time by resolution and CD capacity)
typedef struct tagNET_IN_GET_ENCODEPLANE_BY_RESOLUTION
{
	DWORD               dwSize;
	unsigned int        nChannel;               // channel id
	CAPTURE_SIZE        emResolution;			// video resolution
	unsigned int        nBitRate;               // bit rate
	unsigned int		nCapacity;				// capacity(M)
} NET_IN_GET_ENCODEPLANE_BY_RESOLUTION;

// CLIENT_GetEncodePlanByResolution's interface output param(get expect time by resolution and CD capacity)
typedef struct tagNET_OUT_GET_ENCODEPLANE_BY_RESOLUTION
{
	DWORD               dwSize;
	unsigned int        nExpectTime;			// expect time(min)
} NET_OUT_GET_ENCODEPLANE_BY_RESOLUTION;


// organization directory logical objects
typedef struct tagDH_ORGANIZATION_NODE_LOGIC_OBJECT 
{
	DWORD				dwSize;
	char				szName[DH_NODE_NAME_LEN];	// name
	char				szType[DH_DEV_TYPE_LEN];	// type
	char				szDevID[DH_DEV_ID_LEN_EX];	// deviceID
	char				szControlID[DH_DEV_ID_LEN_EX]; // control ID, read only
	int					nChannel;					// channel
} DH_ORGANIZATION_NODE_LOGIC_OBJECT;

// organization directory
typedef struct tagDH_ORGANIZATION_NODE_DIRECTORY
{
	DWORD				dwSize;
	char				szName[DH_NODE_NAME_LEN];		// name
	char				szControlID[DH_DEV_ID_LEN_EX];	// control ID
} DH_ORGANIZATION_NODE_DIRECTORY;

// info of organization directory node
typedef struct tagDH_ORGANIZATION_NODE 
{
	DWORD				dwSize;
	int					nNodeType;						// node type, 0-logical objects, 1-list
	DH_ORGANIZATION_NODE_LOGIC_OBJECT	stuLogicObject;	// logical object, effective when nNodeType==0
	DH_ORGANIZATION_NODE_DIRECTORY		stuDirectory;	// list,effective when nNodeType==1		
} DH_ORGANIZATION_NODE;

// add node param
typedef struct tagDH_ORGANIZATION_ADD_NODE_PARAM
{
	DWORD				dwSize;
	int					nPosition;					// position, -1-start, -2-end, 0~n
	DH_ORGANIZATION_NODE stuNode;					// info of node
} DH_ORGANIZATION_ADD_NODE_PARAM;

// CLIENT_OrganizationAddNodes's interface input param(add node)
typedef struct tagDH_IN_ORGANIZATION_ADD_NODES
{
	DWORD				dwSize;
	char*				pszPath;					// path
	DH_ORGANIZATION_ADD_NODE_PARAM* pstuNodes;		// node pointer,the space application by the user,apply to sizeof(DH_ORGANIZATION_ADD_NODE_PARAM)*nNodeCount
	int					nNodeCount;					// count of node
} DH_IN_ORGANIZATION_ADD_NODES;

// result of added node
typedef struct tagDH_ORGANIZATION_ADD_NODE_RESULT
{
	DWORD				dwSize;
	BOOL				bResult;					// result
	char				szPath[MAX_PATH];			// path, return node path when succeed,return add node path when failed
} DH_ORGANIZATION_ADD_NODE_RESULT;

// CLIENT_OrganizationAddNodess interface output param(add node)
typedef struct tagDH_OUT_ORGANIZATION_ADD_NODES
{
	DWORD				dwSize;
	DH_ORGANIZATION_ADD_NODE_RESULT* pstuResults;	// result pointer
	int					nMaxResultCount;			// size of result
	int					nRetResultCount;			// count of result
} DH_OUT_ORGANIZATION_ADD_NODES;

// node path
typedef struct tagDH_ORGANIZATION_NODE_PATH 
{
	DWORD				dwSize;
	char				szPath[MAX_PATH];			// path
} DH_ORGANIZATION_NODE_PATH;

// CLIENT_OrganizationDeleteNodes's interface input param(delete node)
typedef struct tagDH_IN_ORGANIZATION_DELETE_NODES
{
	DWORD				dwSize;
	DH_ORGANIZATION_NODE_PATH*	pstuPath;			// path array,the space application by the user,apply to sizeof(DH_ORGANIZATION_NODE_PATH)*nPathCount
	int					nPathCount;					// count
} DH_IN_ORGANIZATION_DELETE_NODES;

// CLIENT_OrganizationDeleteNodes's interface output param(delete node)
typedef struct tagDH_OUT_ORGANIZATION_DELETE_NODES 
{
	DWORD				dwSize;
} DH_OUT_ORGANIZATION_DELETE_NODES;

// CLIENT_OrganizationGetNodes's interface input param(get node info)
typedef struct tagDH_IN_ORGANIZATION_GET_NODES 
{
	DWORD				dwSize;
	char*				pszPath;					// path
	int					nLevel;						// level, 0-the level, 1-next level
} DH_IN_ORGANIZATION_GET_NODES;

// CLIENT_OrganizationGetNodes's interface output param(get node info)
typedef struct tagDH_OUT_ORGANIZATION_GET_NODES 
{
	DWORD				dwSize;
	DH_ORGANIZATION_NODE*	pstuNodes;				// node array,the space application by the user,apply to sizeof(DH_ORGANIZATION_NODE)*nMaxNodeCount
	int					nMaxNodeCount;				// size of array
	int					nRetNodeCount;				// count of node count
} DH_OUT_ORGANIZATION_GET_NODES;

// CLIENT_OrganizationSetNode's interface input param(get node info)
typedef struct tagDH_IN_ORGANIZATION_SET_NODE 
{
	DWORD				dwSize;
	char*				pszPath;					// path
	DH_ORGANIZATION_NODE stuNode;					// node info
} DH_IN_ORGANIZATION_SET_NODE;

// CLIENT_OrganizationSetNode's interface output param(set node)
typedef struct tagDH_OUT_ORGANIZATION_SET_NODE 
{
	DWORD				dwSize;
} DH_OUT_ORGANIZATION_SET_NODE;

// channel info of video input
typedef struct tagDH_VIDEO_INPUTS
{
	DWORD				dwSize;
	char				szChnName[DH_DEVICE_NAME_LEN];		// channel name
	BOOL				bEnable;							// enable
	char				szControlID[DH_DEV_ID_LEN_EX];		// control ID
	char				szMainStreamUrl[MAX_PATH];			// main stream url 
	char				szExtraStreamUrl[MAX_PATH];			// extra stream url
    int                 nOptionalMainUrlCount;              // spare main stream address quantity
    char                szOptionalMainUrls[DH_MAX_OPTIONAL_URL_NUM][MAX_PATH]; // spare main stream address list
    int                 nOptionalExtraUrlCount;             // spare sub stream address quantity
    char                szOptionalExtraUrls[DH_MAX_OPTIONAL_URL_NUM][MAX_PATH]; // spare substream address list
} DH_VIDEO_INPUTS;

typedef struct tagDH_REMOTE_DEVICE 
{
	DWORD				dwSize;
	BOOL				bEnable;							// enable
	char			    szIp[DH_MAX_IPADDR_LEN];		    // IP
	char			    szUser[DH_USER_NAME_LENGTH];	    // username
	char			    szPwd[DH_USER_PSW_LENGTH];	    	// password
	int			    	nPort;							    // port
	int				    nDefinition;					    // definition. 0-standard definition, 1-high definition
	DH_DEVICE_PROTOCOL  emProtocol;							// protocol type
	char			    szDevName[DH_DEVICE_NAME_LEN];		// device name
	int					nVideoInputChannels;				// count channel of video input
	int					nAudioInputChannels;				// count channel of audio input
	char				szDevClass[DH_DEV_TYPE_LEN];		// device type, such as IPC, DVR, NVR
	char				szDevType[DH_DEV_TYPE_LEN];			// device type, such as IPC-HF3300
	int					nHttpPort;							// Http port
	int					nMaxVideoInputCount;				// max count of video input
	int					nRetVideoInputCount;				// return count
	DH_VIDEO_INPUTS*	pstuVideoInputs;					// max count of audion input, user malloc the memory,apply to sizeof(DH_VIDEO_INPUTS)*nMaxVideoInputCount
	char				szMachineAddress[DH_MAX_CARD_INFO_LEN];	// machine address
	char				szSerialNo[DH_SERIALNO_LEN];		// serial no.
    int                 nRtspPort;                          // Rtsp Port

	/*use to new plaform for extension*/
	char                szUserEx[DH_USER_NAME_LEN_EX];      // username
    char                szPwdEx[DH_USER_PSW_LEN_EX];        // password
} DH_REMOTE_DEVICE;

typedef enum tagNET_LOGIC_CHANNEL_TYPE
{
    LOGIC_CHN_UNKNOWN,              // Unknow
    LOGIC_CHN_LOCAL,                // Local channel 
    LOGIC_CHN_REMOTE,               // Remote access channel 
    LOGIC_CHN_COMPOSE,              // Synthesis of channel, for the judicial equipment contains picture in picture channel and mixing channel
    LOGIC_CHN_MATRIX,               // matrix channel, including analog matrix and digital matrix
    LOGIC_CHN_CASCADE,              // cascading channel
} NET_LOGIC_CHN_TYPE;

// available according to the source of information
typedef struct tagDH_MATRIX_CAMERA_INFO
{
	DWORD				dwSize;
	char				szName[DH_DEV_ID_LEN_EX];		// name
	char				szDevID[DH_DEV_ID_LEN_EX];		// device ID
	char				szControlID[DH_DEV_ID_LEN_EX];	// control ID
	int					nChannelID;						// channel ID, DeviceID is unique
	int					nUniqueChannel;					// unique channel
	BOOL				bRemoteDevice;					// support remote device or not
	DH_REMOTE_DEVICE	stuRemoteDevice;				// info of remote device
	NET_STREAM_TYPE     emStreamType;                   // stream type
    NET_LOGIC_CHN_TYPE  emChannelType;                  // Channel Types 
} DH_MATRIX_CAMERA_INFO;

// CLIENT_MatrixGetCameras's interface input param
typedef struct tagDH_IN_MATRIX_GET_CAMERAS 
{
	DWORD				dwSize;
} DH_IN_MATRIX_GET_CAMERAS;

// CLIENT_MatrixGetCameras's interface output param
typedef struct tagDH_OUT_MATRIX_GET_CAMERAS 
{
	DWORD				dwSize;
	DH_MATRIX_CAMERA_INFO* pstuCameras;					// array
	int					nMaxCameraCount;				// size of source array,the space application by the user,apply to sizeof(DH_MATRIX_CAMERA_INFO)*nMaxCameraCount
	int					nRetCameraCount;				// return count
} DH_OUT_MATRIX_GET_CAMERAS;

//CLIENT_QueryDevInfo , NET_QUERY_DEV_REMOTE_DEVICE_INFO inquire  the input parameters of remote device information 
typedef struct tagNET_IN_GET_DEVICE_INFO
{
    DWORD                       dwSize;                                         // when this structure be used, dwSize assigned to sizeof(NET_IN_GET_DEVICE_INFO)
    char                        szDevice[DH_DEV_ID_LEN_EX];                 	// Device ID
    // Device attribute, when szDevice is null, the following do effect
    char                        szAttributeIP[DH_COMMON_STRING_32];             // device address
    int                         nAttributePort;                                 // device port
    char                        szAttributeUsername[DH_COMMON_STRING_128];      // user name
    char                        szAttributePassword[DH_COMMON_STRING_128];      // password
    char                        szAttributeManufacturer[DH_COMMON_STRING_128];  // manufacturer protocl
}NET_IN_GET_DEVICE_INFO;

//CLIENT_QueryDevInfo , NET_QUERY_DEV_REMOTE_DEVICE_INFO inquire the output parameter of remote device information 
typedef struct tagNET_OUT_GET_DEVICE_INFO
{
    DWORD                       dwSize;         // When this structure is used, dwSize should be assigned to sizeof(NET_OUT_GET_DEVICE_INFO)
	DH_REMOTE_DEVICE            stuInfo;        // Device information, the member of this structur,  dwSize, should be assigned by user
}NET_OUT_GET_DEVICE_INFO;

// CLIENT_MatrixSetCameras's interface input param
typedef struct tagDH_IN_MATRIX_SET_CAMERAS 
{
	DWORD				dwSize;
	DH_MATRIX_CAMERA_INFO* pstuCameras;					// array,the space application by the user,apply to sizeof(DH_MATRIX_CAMERA_INFO)*nCameraCount
	int					nCameraCount;				    // size of source array
} DH_IN_MATRIX_SET_CAMERAS;

// CLIENT_MatrixSetCameras's interface output param
typedef struct tagDH_OUT_MATRIX_SET_CAMERAS 
{
	DWORD				dwSize;
} DH_OUT_MATRIX_SET_CAMERAS;

// source of window display info 
typedef struct tagDH_SPLIT_WND_SOURCE 
{
	DWORD				dwSize;
	BOOL				bEnable;						// enable
	char				szDeviceID[DH_DEV_ID_LEN];		// device ID
	char				szControlID[DH_DEV_ID_LEN];		// control ID
	int					nVideoChannel;					// video channel ID
	int					nVideoStream;					// video stream type
	int					nAudioChannel;					// audio channel ID
	int					nAudioStream;					// audio stream type
	int					nUniqueChannel;					// unique channel,read only
	BOOL				bRemoteDevice;					// support remote device
	DH_REMOTE_DEVICE	stuRemoteDevice;				// info of remote device
} DH_SPLIT_WND_SOURCE;

// info of split window
typedef struct tagDH_SPLIT_WINDOW 
{
	DWORD				dwSize;
	BOOL				bEnable;						// enable
	int					nWindowID;						// window ID
	char				szControlID[DH_DEV_ID_LEN];		// control ID
	DH_RECT				stuRect;						// rect of window, effective when free split
	BOOL				bDirectable;					// coordinate whether meet the conditions
	int					nZOrder;						// Z order
	DH_SPLIT_WND_SOURCE stuSource;						// source info
} DH_SPLIT_WINDOW;

// split scene
typedef struct tagDH_SPLIT_SCENE 
{
	DWORD				dwSize;
	char				szCompositeID[DH_DEV_ID_LEN];	// composite ID
	char				szControlID[DH_DEV_ID_LEN];		// control ID
	DH_SPLIT_MODE		emSplitMode;					// split mode
	DH_SPLIT_WINDOW*	pstuWnds;						// info of array,the space application by the user,apply to sizeof(DH_SPLIT_WINDOW)*nMaxWndCount
	int					nMaxWndCount;					// size of array
	int					nRetWndCount;					// return count
} DH_SPLIT_SCENE;

// monitor wall scene
typedef struct tagDH_MONITORWALL_SCENE
{
	DWORD				dwSize;
	char				szName[DH_DEV_NAME_LEN];		// name of plan
	char				szControlID[DH_DEV_ID_LEN];		// control ID
	DH_MONITORWALL		stuMonitorWall;					// monitor wall configuration	
	DH_SPLIT_SCENE*		pstuSplitScene;					// array of split scene,the space application by the user,apply to sizeof(DH_SPLIT_SCENE)*nMaxSplitSceneCount
	int					nMaxSplitSceneCount;			// size of array
	int					nRetSplitSceneCount;			// return count
} DH_MONITORWALL_SCENE;

// CLIENT_MonitorWallGetScene's interface input param(get monitor wall scene)
typedef struct tagDH_IN_MONITORWALL_GET_SCENE 
{
	DWORD				dwSize;
	int					nMonitorWallID;					// monitor wall ID
} DH_IN_MONITORWALL_GET_SCENE;

// CLIENT_MonitorWallGetScene's interface output param(get monitor wall scene)
typedef struct tagDH_OUT_MONITORWALL_GET_SCENE 
{
	DWORD				dwSize;
	char				szName[DH_DEV_NAME_LEN];		// name of plan, can be null
	DH_MONITORWALL_SCENE stuScene;						// minotor wall scene
} DH_OUT_MONITORWALL_GET_SCENE;

// CLIENT_MonitorWallSetScene's interface input param(set monitor wall scene)
typedef struct tagDH_IN_MONITORWALL_SET_SCENE 
{
	DWORD				dwSize;
	int					nMonitorWallID;					// TV Wall No. 
	DH_MONITORWALL_SCENE stuScene;						// TV Wall Scene
} DH_IN_MONITORWALL_SET_SCENE;

// CLIENT_MonitorWallSetScene's interface output param(set monitor wall scene)
typedef struct tagDH_OUT_MONITORWALL_SET_SCENE 
{
	DWORD				dwSize;
} DH_OUT_MONITORWALL_SET_SCENE;

//CLIENT_QueryNetStat Port, input parameter when the Types of queries is NET_APP_DATA_STAT (statistics for protocol stack) 
typedef struct tagNET_IN_NETAPP_NET_DATA_STAT
{
    DWORD       dwSize;    
    char        szEthName[DH_MAX_ETH_NAME];                // The network card name
}NET_IN_NETAPP_NET_DATA_STAT;

//Receive relevant statistics, same meaning with the ifconfig 
typedef struct tagNET_NETAPP_RECEIVE_STAT
{
    DWORD        dwSize;
    DWORD        dwPackets;
    DWORD        dwBytes;
    DWORD        dwErrors;
    DWORD        dwDroped;
    DWORD        dwOverruns;
    DWORD        dwFrame;
}NET_NETAPP_RECEIVE_STAT;

//Transport statistics,same meaning with the ifconfig 
typedef struct tagNET_NETAPP_TRANSMIT_STAT
{
    DWORD        dwSize;
    DWORD        dwPackets;
    DWORD        dwBytes;
    DWORD        dwErrors;
    DWORD        dwDroped;
    DWORD        dwOverruns;
    DWORD        dwCarrier;
    DWORD        dwCollisions;
    DWORD        dwTxQueue;
}NET_NETAPP_TRANSMIT_STAT;

//CLIENT_QueryNetStat Port, output parameter when the Types of queries is NET_APP_DATA_STAT (statistics for protocol stack) 
typedef struct tagNET_OUT_NETAPP_NET_DATA_STAT
{                    
    DWORD                        dwSize;
    NET_NETAPP_RECEIVE_STAT      stuReceiveStat;        // Receive relevant statistics,same meaning with the ifconfig 
    NET_NETAPP_TRANSMIT_STAT     stuTransmitStat;       // Transport statistics,same meaning with the ifconfig 
    int                          nSpeed;                // Network speed, unit is Mbps
}NET_OUT_NETAPP_NET_DATA_STAT;

//CLIENT_QueryNetStatPort, input parameter when the Types of queries is NET_APP_DATA_STAT(access to physical link state)
typedef struct tagNET_IN_NETAPP_LINK_STATUS
{
    DWORD       dwSize;
    char        szEthName[DH_MAX_ETH_NAME];             // Network Card Name 
}NET_IN_NETAPP_LINK_STATUS;

//CLIENT_QueryNetStatPort, output parameter when the Types of queries is NET_APP_DATA_STAT(access to physical link state)
typedef struct tagNET_OUT_NETAPP_LINK_STATUS
{
    DWORD       dwSize;
    BOOL        bWorking;               // Whether working  
    BOOL        bIPConflict;            // Whether IP conflict 
}NET_OUT_NETAPP_LINK_STATUS;

// type of input signal
#define DH_VIDEO_SIGNAL_CVBS	0x0001
#define DH_VIDEO_SIGNAL_SDI		0x0002
#define DH_VIDEO_SIGNAL_VGA		0x0004 
#define DH_VIDEO_SIGNAL_DVI		0x0008
#define DH_VIDEO_SIGNAL_HDMI	0x0010 
#define DH_VIDEO_SIGNAL_YPBPR	0x0020 
#define DH_VIDEO_SIGNAL_SFP		0x0040 
#define DH_VIDEO_SIGNAL_HDCVI       0x0080 
#define DH_VIDEO_SIGNAL_DUAL_LINK   0x0100 

// caps of video inpt
typedef struct tagDH_VIDEO_IN_CAPS 
{
	DWORD					dwSize;
	DWORD					dwSignalType;				// type of signal in
	BOOL					bAutoFocusPeak;				// support auto focus peak or not
	BOOL					bElectricFocus;				// support electric focus or not
	BOOL					bSyncFocus;					// support sync focus
    BOOL                    bSetColor;                  // if support video color setup
    BOOL                    bGain;                      // support gain control or not
    BOOL                    bPhase;                     // support phase control or not
	int						nSubChannel;					// number of sensors
	DH_RECT					stuRect[MAX_SUBCHANNEL_NUM];	// location of each sensor
} DH_VIDEO_IN_CAPS;

// CLIENT_GetVideoInCaps's interface input param
typedef struct tagDH_IN_GET_VIDEO_IN_CAPS 
{
	DWORD			dwSize;
	int				nChannel;							// channel ID of video in
} DH_IN_GET_VIDEO_IN_CAPS;

// CLIENT_GetVideoInCaps's interface output param
typedef struct tagDH_OUT_GET_VIDEO_IN_CAPS 
{
	DWORD			dwSize;
	DH_VIDEO_IN_CAPS stuCaps;							// ability
} DH_OUT_GET_VIDEO_IN_CAPS;

// mode of video out
typedef struct tagDH_VIDEO_OUT_MODE 
{
	DWORD			dwSize;
	int				nWidth;								// horizontal resolution
	int				nHeight;							// vertical resolution
	int				nBPP;								// color depth
	int				nFormat;							// format of signel out, 0-Auto, 1-TV, 2-VGA, 3-DVI
	int				nRefreshRate;						// refersh rate
	int				nPhysicalPort;						// physical port, VGA, TV, DVI independent number
    int             nScanFormat;                       	// Scanning mode, 0 - line by line, 1 - interlaced 
} DH_VIDEO_OUT_MODE;

// CLIENT_EnumVideoOutModes's interface input param
typedef struct tagDH_IN_ENUM_VIDEO_OUT_MODES 
{
	DWORD			dwSize;
	int				nChannel;
} DH_IN_ENUM_VIDEO_OUT_MODES;

// CLIENT_EnumVideoOutModes's interface output param
typedef struct tagDH_OUT_ENUM_VIDEO_OUT_MODES 
{
	DWORD			dwSize;
	DH_VIDEO_OUT_MODE* pstuModes;						// mode array,the space application by the user,apply to sizeof(DH_VIDEO_OUT_MODE)*nMaxModeCount
	int				nMaxModeCount;						// count of mode
	int				nRetModeCount;						// return count
} DH_OUT_ENUM_VIDEO_OUT_MODES;

// minotor wall attribute type
typedef enum tagDH_OUTPUT_ATTRIBUTE_TYPE
{
	DH_OUTPUT_ATTRIBUTE_VIDEO, 
	DH_OUTPUT_ATTRIBUTE_YPBPR, 
	DH_OUTPUT_ATTRIBUTE_VGA, 
	DH_OUTPUT_ATTRIBUTE_DVI, 
	DH_OUTPUT_ATTRIBUTE_MENU, 
} DH_OUTPUT_ATTRIBUTE_TYPE;

// caps of monitor wall adjust (DH_OUTPUT_ATTRIBUTE_CAPS) Video ability
#define	DH_ATTR_VIDEO_BRIGHTNESS	0X00000001
#define	DH_ATTR_VIDEO_CONTRAST		0X00000002
#define	DH_ATTR_VIDEO_SATURATION	0X00000004
#define	DH_ATTR_VIDEO_SHAPENESS		0X00000008
#define	DH_ATTR_VIDEO_DENOISE		0X00000010

// caps of monitor wall adjust (DH_OUTPUT_ATTRIBUTE_CAPS) YPbPr ability
#define	DH_ATTR_YPBPR_BRIGHTNESS	0X00000001
#define	DH_ATTR_YPBPR_CONTRAST		0X00000002
#define	DH_ATTR_YPBPR_SATURATION	0X00000004
#define	DH_ATTR_YPBPR_SHAPENESS		0X00000008
#define	DH_ATTR_YPBPR_DENOISE		0X00000010

// caps of monitor wall adjust (DH_OUTPUT_ATTRIBUTE_CAPS) VGA ability
#define	DH_ATTR_VGA_BRIGHTNESS		0X00000001
#define	DH_ATTR_VGA_CONTRAST		0X00000002
#define	DH_ATTR_VGA_HORPOSITION		0X00000004
#define	DH_ATTR_VGA_VERPOSITION		0X00000008
#define	DH_ATTR_VGA_CLOCK			0X00000010

// caps of monitor wall adjust (DH_OUTPUT_ATTRIBUTE_CAPS) DVI ability
#define	DH_ATTR_DVI_BRIGHTNESS		0X00000001
#define	DH_ATTR_DVI_CONTRAST		0X00000002
#define	DH_ATTR_DVI_HORPOSITION		0X00000004
#define	DH_ATTR_DVI_VERPOSITION		0X00000008
#define	DH_ATTR_DVI_CLOCK			0X00000010

// caps of monitor wall adjust (DH_OUTPUT_ATTRIBUTE_CAPS) Menu ability
#define	DH_ATTR_MENU_MENU			0X00000001
#define	DH_ATTR_MENU_UP				0X00000002
#define	DH_ATTR_MENU_DOWN			0X00000004
#define	DH_ATTR_MENU_LEFT			0X00000008
#define	DH_ATTR_MENU_RIGHT			0X00000010
#define	DH_ATTR_MENU_FACTORYMENU	0X00000020

// caps of monitor wall adjust 
typedef struct tagDH_OUTPUT_ATTRIBUTE_CAPS
{
	DWORD			dwSize;
	BOOL			abVideo;			// Video enable
	DWORD			dwVideo;			// Video ability, by bite, specific to see DH_ATTR_VIDEO_BRIGHTNESS
	BOOL			abYPbPr;			// YPbPr enable
	DWORD			dwYPbPr;			// YPbPr ability, by bite, specific to see DH_ATTR_VGA_BRIGHTNESS
	BOOL			abVGA;				// VGA enable
	DWORD			dwVGA;				// VGA ability, by bite, specific to see DH_ATTR_VGA_BRIGHTNESS
	BOOL			abDVI;				// DVI enable
	DWORD			dwDVI;				// DVI ability, by bite, specific to see DH_ATTR_DVI_BRIGHTNESS
	BOOL			abMenu;				// Menu enable
	DWORD			dwMemu;				// Menu ability, by bite, specific to see DH_ATTR_MENU_MENU
} DH_OUTPUT_ATTRIBUTE_CAPS;

// CLIENT_MonitorWallGetAttributeCaps's interface input param
typedef struct tagDH_IN_MONITORWALL_GET_ARRT_CAPS
{
	DWORD			dwSize;
	int				nMonitorWallID;			// monitor wall ID
	const char*		pszCompositeID;			// composite ID
	int				nOutputID;				// output ID
} DH_IN_MONITORWALL_GET_ARRT_CAPS;

// CLIENT_MonitorWallGetAttributeCaps's interface output param
typedef struct tagDH_OUT_MONITORWALL_GET_ARRT_CAPS
{
	DWORD						dwSize;
	DH_OUTPUT_ATTRIBUTE_CAPS 	stuCaps;		// caps of attribute
} DH_OUT_MONITORWALL_GET_ARRT_CAPS;


// CLIENT_MonitorWallAutoAdjust's interface input param
typedef struct tagDH_IN_MONITORWALL_AUTO_ADJUST
{
	DWORD			dwSize;
	int				nMonitorWallID;			// monitor wall ID
	const char*		pszCompositeID;			// composite ID
	int*			pOutputs;				// array pointer,the space application by the user,apply to sizeof(int)*nOutputCount
	int				nOutputCount;			// count of pOutputs
} DH_IN_MONITORWALL_AUTO_ADJUST;

// CLIENT_MonitorWallAutoAdjust's interface output param
typedef struct tagDH_OUT_MONITORWALL_AUTO_ADJUST
{
	DWORD			dwSize;
} DH_OUT_MONITORWALL_AUTO_ADJUST;

// display unit attribute key/value
typedef struct tagDH_ATTR_PAIR
{
	DWORD	dwKey;					// attribute key
									// dwKey = DH_OUTPUT_ATTRIBUTE_VIDEO , uses DH_ATTR_VIDEO_BRIGHTNESS
									// dwKey = DH_OUTPUT_ATTRIBUTE_YPBPR, usesDH_ATTR_YPBPR_BRIGHTNESS
									// so on
	int		nValue;					// value of attribute, 0-reduce 1-increase, 2-no change
} DH_ATTR_PAIR;

// CLIENT_MonitorWallSetAttribute's interface input param
typedef struct tagDH_IN_MONITORWALL_SET_ATTR
{
	DWORD					dwSize;
	int						nMonitorWallID;					// monitor wall ID
	const char*				pszCompositeID;					// composite ID
	int						nOutputID;						// output ID
	DH_OUTPUT_ATTRIBUTE_TYPE emAttrType;					// attribute, the differ type, the stuAttrs differ also
	DH_ATTR_PAIR			stuAttrs[DH_MAX_ATTR_NUM];		// value of attribute
} DH_IN_MONITORWALL_SET_ATTR;

// CLIENT_MonitorWallSetAttribute's interface output param
typedef struct tagDH_OUT_MONITORWALL_SET_ATTR
{
	DWORD			dwSize;
} DH_OUT_MONITORWALL_SET_ATTR;

// CLIENT_MonitorWallSetBackLight's interface input param
typedef struct tagDH_IN_MONITORWALL_SET_BACK_LIGHT
{
	DWORD					dwSize;
	int						nMonitorWallID;					// monitor wall ID
	const char*				pszCompositeID;					// omposite ID
	int						nOutputID;						// output ID
	int						nMode;							// backlight model, 0-circulation patterns,1-not circulation
} DH_IN_MONITORWALL_SET_BACK_LIGHT;

// CLIENT_MonitorWallSetBackLight's interface output param
typedef struct tagDH_OUT_MONITORWALL_SET_BACK_LIGHT
{
	DWORD			dwSize;
} DH_OUT_MONITORWALL_SET_BACK_LIGHT;

// CLIENT_MonitorWallGetPowerSchedule Interface Input Parameters 
typedef struct tagNET_IN_WM_GET_POWER_SCHEDULE
{
    DWORD               dwSize;
    int                 nMonitorWallID;                     // TV Wall Serial Number 
    const char*         pszCompositeID;                     // Splicing Screen ID 
} NET_IN_MW_GET_POWER_SCHEDULE;

// CLIENT_MonitorWallGetPowerSchedule Interface Output Parameters 
typedef struct tagNET_OUT_MW_GET_POWER_SCHEDULE
{
    DWORD               dwSize;
    DH_TSECT            stuPowerSchedule[DH_TSCHE_DAY_NUM][DH_TSCHE_SEC_NUM];    // starting up schedule, the first dimension means Saturday and Sunday and other holidays 
} NET_OUT_MW_GET_POWER_SCHEDULE;

// CLIENT_MonitorWallSetPowerSchedule Interface Input Parameters
typedef struct tagNET_IN_MW_SET_POWER_SCHEDULE
{
    DWORD               dwSize;
    int                 nMonitorWallID;                     // TV wall serial number 
    const char*         pszCompositeID;                     // Splicing Screen ID, NULL means all the screen 
    DH_TSECT            stuPowerSchedule[DH_TSCHE_DAY_NUM][DH_TSCHE_SEC_NUM];    // starting up schedule, the first dimension means Saturday and Sunday and other holidays
} NET_IN_MW_SET_POWER_SCHEDULE;

// CLIENT_MonitorWallSetPowerSchedule Interface Output Parameters 
typedef struct tagNET_OUT_MW_SET_POWER_SCHEDULE
{
    DWORD               dwSize;
} NET_OUT_MW_SET_POWER_SCHEDULE;

// CLIENT_MonitorWallGetScrnCtrlParam Interface Intput Parameters 
typedef struct tagNET_IN_MW_GET_SCRN_CTRL_PARAM
{
    DWORD               dwSize;
    int                 nMonitorWallID;                     // TV wall Serial Number
    const char*         pszCompositeID;                     // Splicing Screen ID 
} NET_IN_MW_GET_SCRN_CTRL_PARAM;

#define DH_MAX_SCREEN_PORT_NUM        16                    // The biggest screen input port number 
#define DH_MAX_COMPSITE_SCREEN_NUM    256                   // The biggest splicing screen number 

// Screen Input Port Information
typedef struct tagNET_SCREEN_PORT_INFO
{
    DWORD               dwSize;
    char                szType[DH_COMMON_STRING_16];        // Port type, "DVI", "VGA", "HDMI"Etc., Allow the same type of multiple ports 
    char                szAddress[DH_COMMON_STRING_16];     // Port address 
    char                szDeviceID[DH_DEV_ID_LEN_EX];       // The video output device of a binding 
    int                 nOutputChannel;                     // Video output channel number 
} NET_SCREEN_PORT_INFO;


typedef struct tagNET_SCREEN_PORT_GROUP 
{
    DWORD                dwSize;
    int                  nPortNum;                          // Port Number
    NET_SCREEN_PORT_INFO stuPorts[DH_MAX_SCREEN_PORT_NUM];  // Port Information
} NET_SCREEN_PORT_GROUP;

// TV wall screen control parameters 
typedef struct tagNET_WM_SCRN_CTRL_PARAM
{
    DWORD                   dwSize;
    int                     nCommPort;                                  // Serial Number
    DH_COMM_PROP            stuCommProp;                                // Port Attributes
    char                    szProtocol[DH_COMMON_STRING_32];            // Port Protocol 
    int                     nResolutionNum;                             // Resolution Number
    CAPTURE_SIZE            emResolutions[DH_MAX_CAPTURE_SIZE_NUM];     // Video Resolution List 
    int                     nScreenCodeNum;                             // Screen Code Number 
    char                    szScreenCode[DH_MAX_COMPSITE_SCREEN_NUM][DH_COMMON_STRING_16];    // Screen code, using for serial command addressing,this encoding configuration by the user, can be the same 
    int                     nScreenPortsNum;                            // Number of screen input port information
    NET_SCREEN_PORT_GROUP   stuScreenPorts[DH_MAX_COMPSITE_SCREEN_NUM]; // Screen input port information     
} NET_WM_SCRN_CTRL_PARAM;

// CLIENT_MonitorWallGetScrnCtrlParam Interface Input Parameters 
typedef struct tagNET_OUT_MW_GET_SCRN_CTRL_PARAM
{
    DWORD                   dwSize;
    NET_WM_SCRN_CTRL_PARAM  stuScrnCtrlParam;           // Screen Control Parameters
} NET_OUT_MW_GET_SCRN_CTRL_PARAM;

// CLIENT_MonitorWallSetScrnCtrlParam Interface Input Parameters
typedef struct tagNET_IN_MW_SET_SCRN_CTRL_PARAM
{
    DWORD                   dwSize;
    int                     nMonitorWallID;             // TV Wall Serial Number 
    const char*             pszCompositeID;             // Splicing Screen ID 
    NET_WM_SCRN_CTRL_PARAM  stuScrnCtrlParam;           // Screen Control Parameters
} NET_IN_MW_SET_SCRN_CTRL_PARAM;

// CLIENT_MonitorWallSetScrnCtrlParam    Interface Output Parameters 
typedef struct tagNET_OUT_MW_SET_SCRN_CTRL_PARAM
{
    DWORD           dwSize;
} NET_OUT_MW_SET_SCRN_CTRL_PARAM;

// CLIENT_MonitorWallGetBackgroudColor Interface Input Parameters 
typedef struct tagNET_IN_MW_GET_BACKGROUDND_COLOR
{
    DWORD           dwSize;
    int             nMonitorWallID;             // TV Wall Serial Number
} NET_IN_MW_GET_BACKGROUDND_COLOR;

// CLIENT_MonitorWallGetBackgroudColor    Interface Output Parameters 
typedef struct tagNET_OUT_MW_GET_BACKGROUDND_COLOR
{
    DWORD           dwSize;
    DH_COLOR_RGBA   stuScreenColor;             // Screen Background Color 
    DH_COLOR_RGBA   stuWindowColor;             // Window Background Color 
} NET_OUT_MW_GET_BACKGROUDND_COLOR;

// CLIENT_MonitorWallSetBackgroudColor Interface Input Parameters 
typedef struct tagNET_IN_MW_SET_BACKGROUD_COLOR 
{
    DWORD           dwSize;
    int             nMonitorWallID;             // TV Wall Serial Number
    DH_COLOR_RGBA   stuScreenColor;             // Screen Background Color 
    DH_COLOR_RGBA   stuWindowColor;             // Window Background Color 
} NET_IN_MW_SET_BACKGROUD_COLOR;

// CLIENT_MonitorWallSetBackgroudColor Interface Output Parameters
typedef struct tagNET_OUT_MW_SET_BACKGROUD_COLOR 
{
    DWORD           dwSize;
} NET_OUT_MW_SET_BACKGROUD_COLOR;


// signal type
typedef enum tagEM_SIGNAL_TYPE
{
	EM_SIGNAL_TYPE_UNKNOWN,				// unknown
	EM_SIGNAL_TYPE_VIDEO,				// "Video"
	EM_SIGNAL_TYPE_VGA,				    // "VGA"
	EM_SIGNAL_TYPE_YPBPR,				// "YPbPr"
	EM_SIGNAL_TYPE_HDMI,				// "HDMI"
	EM_SIGNAL_TYPE_DVI,					// "DVI"
	EM_SIGNAL_TYPE_SDI,					// "SDI"
	EM_SIGNAL_TYPE_CVBS,				// "CVBS"
}EM_SIGNAL_TYPE;
// CLIENT_MonitorWallSwitchDisplaySignal interface input param
typedef struct tagNET_IN_MW_SWITCH_DISPLAY_SIGNAL
{	
	DWORD				dwSize;							// When this structure is used, dwSize should be assigned to sizeof(NET_IN_MW_SWITCH_DISPLAY_SIGNAL)
    int                 nMonitorWallID;                 // TV wall ID
	char*				pszCompositeID;					// Splicing ID
	int					nOutputID;						// Splicing Block Index ID, -1 means wall
	EM_SIGNAL_TYPE		emSignalType;					// signal type
	int					nIndex;							// index of signal interface
}NET_IN_MW_SWITCH_DISPLAY_SIGNAL;
// CLIENT_MonitorWallSwitchDisplaySignal interface output param
typedef struct tagNET_OUT_MW_SWITCH_DISPLAY_SIGNAL			
{
	DWORD				dwSize;							//When this structure is used, dwSize should be assigned to sizeof(NET_OUT_MW_SWITCH_DISPLAY_SIGNAL)
}NET_OUT_MW_SWITCH_DISPLAY_SIGNAL;
// tv wall operation type
// monitorwall operate type
typedef enum tagNET_MONITORWALL_OPERATE_TYPE
{
    NET_MONITORWALL_OPERATE_ADD,            // add tv wall, corresponding to  NET_IN_MONITORWALL_ADD and NET_OUT_MONITORWALL_ADD
    NET_MONITORWALL_OPERATE_CTRL_TOUR,      // scheme tour control, corresponding to  NET_IN_CTRL_COLLECTIONTOUR and NET_OUT_CTRL_COLLECTIONTOUR
    NET_MONITORWALL_OPERATE_GET_STATUS,     // get matrix current status, corresponding to  NET_IN_MONITORWALL_GET_STATUS NET_OUT_MONITORWALL_GET_STATUS
    NET_MONITORWALL_OPERATE_SET_COLL_SCHD,  // set scheme time schedule, corresponding to  NET_IN_MONITORWALL_SET_COLL_SCHD NET_OUT_MONITORWALL_SET_COLL_SCHD
    NET_MONITORWALL_OPERATE_GET_COLL_SCHD,  // get scheme schedule, corresponding to  NET_IN_MONITORWALL_GET_COLL_SCHD and NET_OUT_MONITORWALL_GET_COLL_SCHD
    NET_MONITORWALL_OPERATE_REMOVE,         // delete tv wall, corresponding to NET_IN_MONITORWALL_REMOVE  and  NET_OUT_MONITORWALL_REMOVE
    NET_MONITORWALL_OPERATE_SET_ENABLE,     // setup enable, corresponding to NET_IN_MONITORWALL_SET_ENABLE  and  NET_OUT_MONITORWALL_SET_ENABLE
    NET_MONITORWALL_OPERATE_GET_ENABLE,     // Get  enable, corresponding to NET_IN_MONITORWALL_GET_ENABLE  and  NET_OUT_MONITORWALL_GET_ENABLE
    NET_MONITORWALL_OPERATE_NAME_EXIST,     // tv wall exists or not, corresponding to NET_IN_MONITORWALL_NAME_EXIST  and  NET_OUT_MONITORWALL_NAME_EXIST	
	NET_MONITORWALL_OPERATE_RENAME,			// modify the monitorwall name, corresponding to NET_IN_MONITORWALL_RENAME and NET_OUT_MONITORWALL_RENAME
} NET_MONITORWALL_OPERATE_TYPE;

// add tv wall input parameter
typedef struct tagNET_IN_MONITORWALL_ADD
{
    DWORD dwSize;
    DH_MONITORWALL stuMonitorWall; // tv wall info
} NET_IN_MONITORWALL_ADD;

// add tv wall output parameter
typedef struct tagNET_OUT_MONITORWALL_ADD
{
    DWORD dwSize;
    unsigned int nMonitorWallID; // tv wall ID
} NET_OUT_MONITORWALL_ADD;

// CLIENT_OperateMonitorWallport input parameter=>NET_MONITORWALL_OPERATE_CTRL_TOUR
typedef struct tagNET_IN_CTRL_COLLECTIONTOUR 
{
    DWORD              dwSize;
    int                nChannel;                       // channel no.
    int                nAction;                        // tour action, 0:ebd, 1:start
} NET_IN_CTRL_COLLECTIONTOUR;

// CLIENT_OperateMonitorWallport output parameter=>NET_MONITORWALL_OPERATE_CTRL_TOUR
typedef struct tagNET_OUT_CTRL_COLLECTIONTOUR 
{
    DWORD                   dwSize;
} NET_OUT_CTRL_COLLECTIONTOUR;

// matrix operation status
typedef enum tagNET_MATRIX_STATUS
{
    MATRIX_STATUS_UNKNOWN,              // unknown 
    MATRIX_STATUS_TOUR,                 // scheme tour
    MATRIX_STATUS_NORMAL,               // normal tour
} NET_MATRIX_STATUS;

#define DH_MAX_COLLECTION_NUM   64      // max scheme
// matrix status info
typedef struct tagNET_MONITORWALL_STATUS_INFO
{
    DWORD       dwSize;
    int         nInterval;                      // tour interval
    int         nCollectionNum;                 // scheme tour
    char        szCollections[DH_MAX_COLLECTION_NUM][DH_DEVICE_NAME_LEN];         // scheme tour content
    char        szName[DH_DEVICE_NAME_LEN];     // current display scheme name
} NET_MONITORWALL_STATUS_INFO;

// // CLIENT_OperateMonitorWallport input parameter=>NET_MONITORWALL_OPERATE_GET_STATUS
typedef struct tagNET_IN_MONITORWALL_GET_STATUS 
{
    DWORD              dwSize;
    int                nChannel;                        //channel no.
} NET_IN_MONITORWALL_GET_STATUS;

// CLIENT_OperateMonitorWallport output parameter=>NET_MONITORWALL_OPERATE_GET_STATUS
typedef struct tagNET_OUT_MONITORWALL_GET_STATUS
{
    DWORD                           dwSize;
    NET_MATRIX_STATUS               emMatrixStatus;     // operation status
    NET_MONITORWALL_STATUS_INFO     stuStatusInfo;      // status info
} NET_OUT_MONITORWALL_GET_STATUS;

// tv wall scheme schedule
typedef struct tagMONITORWALL_COLLECTION_SCHEDULE
{
    DWORD               dwSize;
    char                szName[DH_DEVICE_NAME_LEN];     // schedule name
    DH_TSECT            stuSchedule[DH_TSCHE_DAY_NUM][DH_TSCHE_SEC_NUM];   // schedule, first each element means sunday~Saturday and holiday
} MONITORWALL_COLLECTION_SCHEDULE;

// CLIENT_OperateMonitorWallport input parameter=>NET_MONITORWALL_OPERATE_SET_COLL_SCHD
typedef struct tagNET_IN_MONITORWALL_SET_COLL_SCHD 
{
    DWORD               dwSize;
    int                 nCollectionNum;                 // scheme number
    MONITORWALL_COLLECTION_SCHEDULE stuCollShedule[DH_MAX_COLLECTION_NUM];  // scheme schedule
    int                 nMonitorWallID;                 // tv wall ID
} NET_IN_MONITORWALL_SET_COLL_SCHD;

// CLIENT_OperateMonitorWallport output parameter=>NET_MONITORWALL_OPERATE_SET_COLL_SCHD
typedef struct tagNET_OUT_MONITORWALL_SET_COLL_SCHD 
{
    DWORD               dwSize;
} NET_OUT_MONITORWALL_SET_COLL_SCHD;

// CLIENT_OperateMonitorWallport input parameter=>NET_MONITORWALL_OPERATE_GET_COLL_SCHD
typedef struct tagNET_IN_MONITORWALL_GET_COLL_SCHD 
{
    DWORD               dwSize;
    BOOL                bAllCollections;                // TRUE-search all scheme, no need to appoint scheme name. FALSE-search name corresponding scheme
    int                 nCollectionNum;                 // scheme, bAllCollections is FALSE valid
    char                szCollections[DH_MAX_COLLECTION_NUM][DH_DEVICE_NAME_LEN];   // scheme name
    int                 nMonitorWallID;                 // tv wall ID
} NET_IN_MONITORWALL_GET_COLL_SCHD;

// CLIENT_OperateMonitorWallport input parameter=>NET_MONITORWALL_OPERATE_GET_COLL_SCHD
typedef struct tagNET_OUT_MONITORWALL_GET_COLL_SCHD 
{
    DWORD               dwSize;
    int                 nCollectionNum;                 // scheme number
    MONITORWALL_COLLECTION_SCHEDULE stuCollShedule[DH_MAX_COLLECTION_NUM];  // scheme schedule
} NET_OUT_MONITORWALL_GET_COLL_SCHD;

// CLIENT_OperateMonitorWallport input parameter =>NET_MONITORWALL_OPERATE_REMOVE
typedef struct tagNET_IN_MONITORWALL_REMOVE 
{
    DWORD       dwSize;
    int         nMonitorWallNum;        // tv wall quantity to delete
    char        szNames[DH_MAX_MONITORWALL_NUM][DH_COMMON_STRING_128]; // tv wall  name 
} NET_IN_MONITORWALL_REMOVE;

// CLIENT_OperateMonitorWallport  output parameter =>NET_MONITORWALL_OPERATE_REMOVE
typedef struct tagNET_OUT_MONITORWALL_REMOVE 
{
    DWORD       dwSize;
} NET_OUT_MONITORWALL_REMOVE;

// tv wall enable info 
typedef struct tagNET_MONITORWALL_ENABLE_INFO
{
    DWORD       dwSize;
    BOOL        bEanble;            // enable 
    char        szName[DH_COMMON_STRING_128];  // tv wall  name 
} NET_MONITORWALL_ENABLE_INFO;

// CLIENT_OperateMonitorWallport input parameter =>NET_MONITORWALL_OPERATE_SET_ENABLE
typedef struct tagNET_IN_MONITORWALL_SET_ENABLE
{
    DWORD       dwSize;
    int         nMonitorWallNum;        // tv wall quantity to set
    NET_MONITORWALL_ENABLE_INFO stuEnable[DH_MAX_MONITORWALL_NUM]; // tv wall enable 
} NET_IN_MONITORWALL_SET_ENABLE;

// CLIENT_OperateMonitorWallport  output parameter =>NET_MONITORWALL_OPERATE_SET_ENABLE
typedef struct tagNET_OUT_MONITORWALL_SET_ENABLE
{
    DWORD        dwSize;
} NET_OUT_MONITORWALL_SET_ENABLE;

// CLIENT_OperateMonitorWallport input parameter =>NET_MONITORWALL_OPERATE_GET_ENABLE
typedef struct tagNET_IN_MONITORWALL_GET_ENABLE
{
    DWORD       dwSize;
    int         nMonitorWallNum;        //tv wall quantity to search, -1 means search alltv wall 
    char        szNames[DH_MAX_MONITORWALL_NUM][DH_COMMON_STRING_128]; // tv wall  name , nMonitorWallNum>0 is valid 
} NET_IN_MONITORWALL_GET_ENABLE;

// CLIENT_OperateMonitorWallport  output parameter =>NET_MONITORWALL_OPERATE_SET_ENABLE
typedef struct tagNET_OUT_MONITORWALL_GET_ENABLE
{
    DWORD       dwSize;
    int         nMonitorWallNum;        // tv wall quantity
    NET_MONITORWALL_ENABLE_INFO stuEnable[DH_MAX_MONITORWALL_NUM]; // tv wall enable 
} NET_OUT_MONITORWALL_GET_ENABLE;

// CLIENT_OperateMonitorWallport input parameter =>NET_MONITORWALL_OPERATE_NAME_EXIST
typedef struct tagNET_IN_MONITORWALL_NAME_EXIST 
{
    DWORD           dwSize;
    const char*     pszName;            // tv wall  name 
} NET_IN_MONITORWALL_NAME_EXIST;

// CLIENT_OperateMonitorWallport  output parameter =>NET_MONITORWALL_OPERATE_NAME_EXIST
typedef struct tagNET_OUT_MONITORWALL_NAME_EXIST 
{
    DWORD       dwSize;
    BOOL        bExist;            // tv wall exists or not
} NET_OUT_MONITORWALL_NAME_EXIST;

// CLIENT_OperateMonitorWall input parameter=>NET_MONITORWALL_OPERATE_RENAME(modify monitor wall name)
typedef struct tagNET_IN_MONITORWALL_RENAME
{
	DWORD               dwSize;
	unsigned int		unMonitorWallID;							// monitor wall ID
	char				szNewName[MAX_MONITORWALL_NAME_LEN];		// new name of monitor wall
} NET_IN_MONITORWALL_RENAME;

// CLIENT_OperateMonitorWall output parameter=>NET_MONITORWALL_OPERATE_RENAME(modify monitor wall name)
typedef struct tagNET_OUT_MONITORWALL_RENAME
{
	DWORD               dwSize;
} NET_OUT_MONITORWALL_RENAME;


/************************************************************************/
/*  U disk caught
/************************************************************************/

// CLIENT_StartSniffer's interface input param
typedef struct tagDH_IN_START_SNIFFER 
{
	DWORD		dwSize;
	const char*	pszNetInterface;				// name of network card
	const char*	pszPath;						// path of caught file, it is means the default path when pszPath = NULL
	int			nSaveType;						// type of file, 0-Wireshark/Tcpdump
    const char* pszFilter;                  	// Filter Conditions ,Such as "host 172.9.88.200 and port 8080 and tcp"
} DH_IN_START_SNIFFER;

// CLIENT_StartSniffer's interface output param
typedef struct tagDH_OUT_START_SNIFFER 
{
	DWORD		dwSize;
} DH_OUT_START_SNIFFER;

// grasp the package information
typedef struct tagDH_SNIFFER_INFO
{
	DWORD		dwSize;
	DWORD		nSnifferID;					// Sniffer ID
} DH_SNIFFER_INFO;

// CLIENT_GetSnifferInfo's interface input param
typedef struct tagDH_IN_GET_SNIFFER_INFO
{
	DWORD		dwSize;
	const char*	pszNetInterface;
} DH_IN_GET_SNIFFER_INFO;

// CLIENT_GetSnifferInfo's interface output param
typedef struct tagDH_OUT_GET_SNIFFER_INFO
{
	DWORD		dwSize;
	DH_SNIFFER_INFO	* pstuSniffers;			// array of Sniffer
	int			nMaxSnifferCount;			// size of array
	int			nRetSnifferCount;			// return count
} DH_OUT_GET_SNIFFER_INFO;

/************************************************************************/
/*  manage file
/************************************************************************/

// format the partition information 
typedef struct tagDH_FORMAT_PATITION 
{
	DWORD			dwSize;
	const char*		pszStorageName;							// storage name
	const char*		pszPatitionName;						// partition name
    const char*     pszFileSystem;                          // file system
} DH_FORMAT_PATITION;

// CLIENT_CreateRemoteFile's interface input param
typedef struct tagDH_IN_CREATE_REMOTE_FILE
{
	DWORD			dwSize;
	const char*		pszPath;								// path of file
	BOOL			bDirectory;								// is directory or not
} DH_IN_CREATE_REMOTE_FILE;

// CLIENT_CreateRemoteFile's interface output param
typedef struct tagDH_OUT_CREATE_REMOTE_FILE 
{
	DWORD			dwSize;
} DH_OUT_CREATE_REMOTE_FILE;

// CLIENT_RemoveRemoteFiles's interface input param
typedef struct tagDH_IN_REMOVE_REMOTE_FILES
{
	DWORD			dwSize;
	const char**	pszPath;								// pointer of file path
	int				nFileCount;								// count of file
} DH_IN_REMOVE_REMOTE_FILES;

// CLIENT_RemoveRemoteFiles's interface output param
typedef struct tagDH_OUT_REMOVE_REMOTE_FILES 
{
	DWORD		dwSize;
} DH_OUT_REMOVE_REMOTE_FILES;

// CLIENT_RenameRemoteFile's interface input param
typedef struct tagDH_IN_RENAME_REMOTE_FILE
{
	DWORD			dwSize;
	const char*		pszOldPath;								// old path
	const char*		pszNewPath;								// new path
} DH_IN_RENAME_REMOTE_FILE;

// CLIENT_RenameRemoteFile's interface output param
typedef struct tagDH_OUT_RENAME_REMOTE_FILE 
{
	DWORD			dwSize;
} DH_OUT_RENAME_REMOTE_FILE;

// file/catalog info
typedef struct tagDH_REMOTE_FILE_INFO 
{
	DWORD			dwSize;
	BOOL			bDirectory;								// is directory or not
	char			szPath[MAX_PATH];						// path
	NET_TIME		stuCreateTime;							// create time
	NET_TIME		stuModifyTime;							// modify time
	INT64			nFileSize;								// size of file
	char			szFileType[DH_FILE_TYPE_LEN];			// type of file
} DH_REMOTE_FILE_INFO;

// remote file query condition
typedef enum tagDH_REMOTE_FILE_COND
{
    DH_REMOTE_FILE_COND_NONE,                           // no condition
    DH_REMOTE_FILE_COND_VOICE,                          // voice file, CANNOT specify search path, ONLY file name obtained
} DH_REMOTE_FILE_COND;

// CLIENT_ListRemoteFile's interface input param
typedef struct tagDH_IN_LIST_REMOTE_FILE
{
	DWORD			dwSize;
	const char*		pszPath;								// path
	BOOL			bFileNameOnly;							// only for file name
    DH_REMOTE_FILE_COND emCondition;                        // query condition
} DH_IN_LIST_REMOTE_FILE;

// CLIENT_ListRemoteFile's interface output param
typedef struct tagDH_OUT_LIST_REMOTE_FILE 
{
	DWORD					dwSize;
	DH_REMOTE_FILE_INFO*	pstuFiles;						// array of file,the space application by the user,apply to sizeof(DH_REMOTE_FILE_INFO)*nMaxFileCount
	int						nMaxFileCount;					// size of array
	int						nRetFileCount;					// return count
} DH_OUT_LIST_REMOTE_FILE;

// set file property - property name
typedef enum tagNET_SET_FILEATTR_ACTION
{
    EM_SET_FILEATTR_ACTION_UNKNOWN = 0,                 // unknown
    EM_SET_FILEATTR_ACTION_MARK,                        // file lock
} NET_SET_FILEATTR_ACTION;

// CLIENT_SetFileAttribute port input paramete
typedef struct tagDH_IN_SET_FILEATTRIBUTE
{
    DWORD                   dwSize;
    unsigned int            nDriveNo;                       // disk no.
    unsigned int            nPartition;                     // partition no.
    unsigned int            nCluster;                       // cluster no.
    NET_SET_FILEATTR_ACTION emAction;                       // each property corresponding to one key, set value depends on value
                                                            // set key:Mark file lock ,corresponding to value, delay lock time(unit:day)
    char                    szValue[DH_COMMON_STRING_64];   // property valu
} DH_IN_SET_FILEATTRIBUTE;

// CLIENT_SetFileAttribute port output parameter
typedef struct tagDH_OUT_SET_FILEATTRIBUTE
{
    DWORD                   dwSize;
} DH_OUT_SET_FILEATTRIBUTE;

// manual pop-up storage device
typedef struct tagDH_EJECT_STORAGE_DEVICE
{
	DWORD				dwSize;	
	const char*			pszStorageName;						// storage name
} DH_EJECT_STORAGE_DEVICE;

//  manual load storage device
typedef struct tagDH_LOAD_STORAGE_DEVICE 
{
	DWORD				dwSize;
	const char*			pszStorageName;						// storage name
} DH_LOAD_STORAGE_DEVICE;

// CLIENT_UploadRemoteFile's interface input param(upload file to the device)
typedef struct tagDH_IN_UPLOAD_REMOTE_FILE
{
    DWORD               dwSize;
    const char*         pszFileSrc;			// path of source file
    const char*         pszFileDst;			// path of goal file
    const char*         pszFolderDst;       // target file path: may be NULL, NULL device use default path
    unsigned int        nPacketLen;         // file pack size(byte): 0 means not pack
} DH_IN_UPLOAD_REMOTE_FILE;

// CLIENT_UploadRemoteFile's interface output param(upload file to the device)
typedef struct tagDH_OUT_UPLOAD_REMOTE_FILE
{
	DWORD					dwSize;
} DH_OUT_UPLOAD_REMOTE_FILE;

// CLIENT_DownloadRemoteFile    Interface Input Parameters (the file download)
typedef struct tagDH_IN_DOWNLOAD_REMOTE_FILE
{
    DWORD               dwSize;
    const char*         pszFileName;                    // File Name Needs to Download 
    const char*         pszFileDst;                     // File Path 
} DH_IN_DOWNLOAD_REMOTE_FILE;

// CLIENT_DownloadRemoteFile Interface Output Parameters (the file download) 
typedef struct tagDH_OUT_DOWNLOAD_REMOTE_FILE
{
    DWORD               dwSize;
} DH_OUT_DOWNLOAD_REMOTE_FILE;

typedef struct tagNET_IN_DOWNLOAD_PIECE_FILE
{
	DWORD				dwSize;
	char				szFileName[MAX_PATH];		   // File name,support full path
	UINT				nOffSet;					   // Offset,unit:BYTE
	UINT				nNeedLength;				   // need length,unit:BYTE, suggest value is 32KB, not over 4MB
} NET_IN_DOWNLOAD_PIECE_FILE;
typedef struct tagNET_OUT_DOWNLOAD_PIECE_FILE
{
	DWORD				dwSize;
	UINT				nFileLength;	 // file length
	UINT				nPacketLength;   // binary data length this time, unit:BYTE,suggest value is 32KB, not over 4MB
	char*               szBuffer;        // file buffer,the space application by the user,size is nBufferLen                            
	UINT				nBufferLen;      // buffer len,unit:BYTE,the same to nNeedLength of NET_IN_DOWNLOAD_PIECE_FILE
} NET_OUT_DOWNLOAD_PIECE_FILE;
/************************************************************************/
/* cascade device
/************************************************************************/
// conditon of cascade device search
typedef struct tagDH_IN_SEARCH_CONDITON
{
	DWORD					dwSize;
	const char*				pMachineName;					// device name or no.
	const char*				pMachineAddr;					// machine address (XX.XX.XX)
	const char*				pSerialNo;						// serial no.
	const char*				pChannelName;					// channel name of video in
} DH_MATRIX_SEARCH_CONDITON;

// CLIENT_MatrixSearch's interface input param(search cascade device)
typedef struct tagDH_IN_MATIRX_SEARCH
{
	DWORD					dwSize;
	const char*				pSerialNo;						// servial no.,"Local""Root",other devices with a serial number
	DH_MATRIX_SEARCH_CONDITON stuCondition;					// condition of search, can be for a single or combination
} DH_IN_MATRIX_SEARCH;

// CLIENT_MatrixSearch's interface output param(search cascade device)
typedef struct tagDH_OUT_MATRIX_SEARCH
{
	DWORD					dwSize;
	DH_REMOTE_DEVICE*		pstuRemoteDevices;				// devices list,the space application by the user,apply to sizeof(DH_REMOTE_DEVICE)*nMaxDeviceCount
	int						nMaxDeviceCount;				// max count of device
	int						nRetDeviceCount;				// return count
} DH_OUT_MATRIX_SEARCH;

// CLIENT_GetMatrixTree's interface input param
typedef struct tagDH_IN_GET_MATRIX_TREE
{
	DWORD					dwSize;
	const char*				pSerialNo;						// servial no.,"Local""Root",other devices with a serial number
	int						nLevel;							// get device information,0-all, 1-Local, 2-Local+device under
															// 3-Local+device under+ the next next devuce
} DH_IN_GET_MATRIX_TREE;

// cascade device info
typedef struct tagDH_CASCADE_DEVICE_INFO
{
	DWORD					dwSize;
	int						nLevel;							// level
	char					szPath[MAX_PATH];				// set path,format:name1.name2.name3...
	DH_REMOTE_DEVICE		stuDeviceInfo;					// device info
} DH_CASCADE_DEVICE_INFO;

// CLIENT_GetMatrixTree's interface output param
typedef struct tagDH_OUT_GET_MATRIX_TREE
{
	DWORD					dwSize;
	DH_CASCADE_DEVICE_INFO*	pstuRemoteDevices;				// info of cascade device
	int						nMaxDeviceCount;				// max number of cascade device
	int						nRetDeviceCount;				// return count
} DH_OUT_GET_MATRIX_TREE;

// CLIENT_GetSuperiorMatrixList's interface input param
typedef struct tagDH_IN_GET_SUPERIOR_MATRIX_LIST
{
	DWORD					dwSize;
} DH_IN_GET_SUPERIOR_MATRIX_LIST;

// CLIENT_GetSuperiorMatrixList's interface output param
typedef struct tagDH_OUT_GET_SUPERIOR_MATRIX_LIST
{
	DWORD					dwSize;
	DH_REMOTE_DEVICE*		pstuRemoteDevices;				// device list
	int						nMaxDeviceCount;				// max count of device
	int						nRetDeviceCount;				// return count
} DH_OUT_GET_SUPERIOR_MATRIX_LIST;

////////////////////////////////////record backup to restore//////////////////////////////////////

// task of record backup to restore
typedef struct tagDH_RECORD_BACKUP_RESTORE_TASK
{
    DWORD				dwSize;
    unsigned int		nTaskID;                        // task ID
    char				szDeviceID[DH_DEV_ID_LEN_EX];   // device ID
    int			        nChannelID;                     // channek ID
    NET_TIME			stuStartTime;                   // start time
    NET_TIME			stuEndTime;                     // end time
    int			        nState;                         // state of backup, 0-wait, 1-working, 2-finish, 3-failed
} DH_REC_BAK_RST_TASK;

// CLIENT_AddRecordBackupRestoreTask's interface input param
typedef struct tagDH_IN_ADD_RECORD_BACKUP_RESTORE_TASK
{
    DWORD				dwSize;
    const char*			pszDeviceID;					// device ID
    int*		        pnChannels;						// array of channel,the space application by the user,apply to sizeof(int)*nChannelCount
	int					nChannelCount;					// size of array
    NET_TIME			stuStartTime;					// start time
    NET_TIME			stuEndTime;						// end time
} DH_IN_ADD_REC_BAK_RST_TASK;

// CLIENT_RemoveRecordBackupRestoreTask's interface input param
typedef struct tagDH_IN_REMOVE_RECORD_BACKUP_RESTORE_TASK
{
    DWORD				dwSize;
    unsigned int*		pnTaskIDs;						// array of task ID,the space application by the user,apply to sizeof(int)*nTaskCount
	int					nTaskCount;						// count of task
} DH_IN_REMOVE_REC_BAK_RST_TASK;

// CLIENT_QueryRecordBackupRestoreTask's interface input param
typedef struct tagDH_IN_QUERY_RECORD_BACKUP_RESTORE_TASK
{
    DWORD		        dwSize;
} DH_IN_QUERY_REC_BAK_RST_TASK;

// CLIENT_QueryRecordBackupRestoreTask's interface output param
typedef struct tagDH_OUT_QUERY_RECORD_BACKUP_RESTORE_TASK
{
    DWORD				 dwSize;
    DH_REC_BAK_RST_TASK* pTasks;						// array of task,the space application by the user,apply to sizeof(DH_REC_BAK_RST_TASK)*nMaxCount
    int					 nMaxCount;						// size of array
    int					 nReturnCount;					// return count
} DH_OUT_QUERY_REC_BAK_RST_TASK;

typedef struct tagDH_LOGIC_DEVICE_ADD_CAMERA_PARAM
{
	DWORD			dwSize;
	const char*		pszDeviceID;			// device ID
	int				nChannel;				// channel
} DH_LOGIC_DEVICE_ADD_CAMERA_PARAM;

typedef struct tagDH_LOGIC_DEVICE_ADD_CAMERA_RESULT
{
	DWORD			dwSize;
	char			szDeviceID[DH_DEV_ID_LEN];	// device ID
	int				nChannel;					// channel ID
	int				nUniqueChannel;				// uniform number
	int				nFailedCode;				// failure code, 0-succeed 1-Unique 2-have added
} DH_LOGIC_DEVICE_ADD_CAMERA_RESULT;

// CLIENT_AddLogicDeviceCamera's interface input param
typedef struct tagDH_IN_ADD_LOGIC_DEVICE_CAMERA
{
	DWORD			dwSize;
	DH_LOGIC_DEVICE_ADD_CAMERA_PARAM*	pCameras;	// array of source video ,the space application by the user,apply to sizeof(DH_LOGIC_DEVICE_ADD_CAMERA_PARAM)*nCameraCount
	int				nCameraCount;					// count of sourcevideo
} DH_IN_ADD_LOGIC_DEVICE_CAMERA;

// CLIENT_AddLogicDeviceCamera's interface output param
typedef struct tagDH_OUT_ADD_LOGIC_DEVICE_CAMERA 
{
	DWORD			dwSize;
	DH_LOGIC_DEVICE_ADD_CAMERA_RESULT* pResults;	// result,the space application by the user,apply to sizeof(DH_LOGIC_DEVICE_ADD_CAMERA_RESULT)*nMaxResultCount
	int				nMaxResultCount;				// size of array
	int				nRetResultCount;				// return count
} DH_OUT_ADD_LOGIC_DEVICE_CAMERA;

//array info
typedef struct tagDH_LOGIC_BYDEVICE_ADD_CAMERA_PARAM
{
    DWORD                   dwSize;
    int                     nUniqueChannel;             // UniqueChannel
    int                     nChannel;                   // channel         
} DH_LOGIC_BYDEVICE_ADD_CAMERA_PARAM;

//result info
typedef struct tagDH_LOGIC_BYDEVICE_ADD_CAMERA_RESULT
{
    DWORD                   dwSize;
    int                     nUniqueChannel;             // UniqueChannel
    int                     nFailedCode;                // FaileCode, 0-Success,1-unsupport           
} DH_LOGIC_BYDEVICE_ADD_CAMERA_RESULT;

// CLIENT_MatrixAddCamerasByDevice's interface input param
typedef struct tagDH_IN_ADD_LOGIC_BYDEVICE_CAMERA
{
    DWORD                   dwSize;
    char                    pszDeviceID[DH_DEV_ID_LEN]; // DeviceID
    DH_REMOTE_DEVICE        stuRemoteDevice;            // RemoteDevice Info
    int                     nCameraCount;               // source video number
    DH_LOGIC_BYDEVICE_ADD_CAMERA_PARAM* pCameras;       // array of source video,the space application by the user,apply to sizeof(DH_LOGIC_BYDEVICE_ADD_CAMERA_PARAM)*nCameraCount
}DH_IN_ADD_LOGIC_BYDEVICE_CAMERA;

// CLIENT_MatrixAddCamerasByDevice's interface output param
typedef struct tagDH_OUT_ADD_LOGIC_BYDEVICE_CAMERA
{
    DWORD                   dwSize;                         
    char                    szDeviceID[DH_DEV_ID_LEN];    // DeviceID
    int                     nMaxResultCount;              // ResultCount
    int                     nRetResultCount;              // return count
    DH_LOGIC_BYDEVICE_ADD_CAMERA_RESULT* pResults;        // result,the space application by the user,apply to sizeof(DH_LOGIC_BYDEVICE_ADD_CAMERA_RESULT)*nMaxResultCount
}DH_OUT_ADD_LOGIC_BYDEVICE_CAMERA;

// the cameras info
typedef struct tagNET_CAMERA_INFO
{	
    int                     nUniqueChannel;             // unigue channel, -1 means that the channelID is automatically assigned by the device
    unsigned int            nChannel;                   // channel ID  
    BYTE                	reserved[128];				// reserved
} NET_CAMERA_INFO;

// the camera groups info
typedef struct tagNET_CAMERA_GROUP_INFO
{
	char                    szDeviceID[DH_DEV_ID_LEN];	// device ID
    int                     nCameraCount;				// the count of cameras
    NET_CAMERA_INFO			*pCameras;       			// array of cameras, the space application by the user,apply to sizeof(NET_CAMERA_INFO)*nCameraCount
	DH_REMOTE_DEVICE        stuRemoteDevice;			// the info of remote device, effective when szDeviceID is invalid
	BYTE                	reserved[1024];				// reserved
} NET_CAMERA_GROUP_INFO;

// input param of CLIENT_MatrixAddCamerasByGroup (batch add cameras) 
typedef struct tagNET_IN_ADD_LOGIC_BYGROUP_CAMERA
{
	DWORD                   dwSize;
	int						nCameraGroup;				// the count of camera groups
	NET_CAMERA_GROUP_INFO	*pstCameraGroupInfo; 		//array of camera groups, the space application by the user,apply to sizeof(NET_CAMERA_GROUP_INFO)*nCameraGroup
} NET_IN_ADD_LOGIC_BYGROUP_CAMERA;

// the result inforamtion of cameras returned by adding camera groups
typedef struct tagNET_ADD_BYGROUP_RESULT_CAMERA
{
    unsigned int           	nUniqueChannel;             // unique channel
    unsigned int            nFailedCode;                // FaileCode, 0-Success,1-unsupport 
    BYTE					reserved[128];				// reserved
} NET_ADD_BYGROUP_RESULT_CAMERA;

//  the result inforamtion of camera groups returned by adding camera groups
typedef struct tagNET_ADD_BYGROUP_RESULT
{
	char                    			szDeviceID[DH_DEV_ID_LEN];    	// device ID
	int									nResultCam;						// the count of cameras, consistent with the nCameraCount in NET_CAMERA_GROUP_INFO
	NET_ADD_BYGROUP_RESULT_CAMERA		*pstResultCamera;				// array of cameras by adding camera groups,
																		// the space application by the user,apply to sizoef(NET_ADD_BYGROUP_RESULT_CAMERA)*nRetResultCamera
	BYTE								reserved[1024];					// reserved
} NET_ADD_BYGROUP_RESULT;

// output paramer of CLIENT_MatrixAddCamerasByGroup (batch add cameras) 
typedef struct tagNET_OUT_ADD_LOGIC_BYGROUP_CAMERA
{
	DWORD                   			dwSize;
	int									nGroupCount;			// the count of groups, consistent with the nCameraGroup in NET_IN_ADD_LOGIC_BYGROUP_CAMERA
	NET_ADD_BYGROUP_RESULT				*pstAddByGroupResult;	// array of camera groups  by adding camera groups,
																// the space application by the user,apply to sizoef(NET_ADD_BYGROUP_RESULT)*nMaxResultGroup
} NET_OUT_ADD_LOGIC_BYGROUP_CAMERA;

/************************************************************************/
/*                         Database Records                               */
/************************************************************************/
typedef struct tagNET_AUTHORITY_TYPE
{
  DWORD                       dwSize; 
  EM_NET_AUTHORITY_TYPE       emAuthorityType;                          //Permission Types 
  BOOL                        bAuthorityEnable;                         //Permission Enabled
}NET_AUTHORITY_TYPE;

// Information of recorded in transportation black and white list 
typedef struct tagNET_TRAFFIC_LIST_RECORD
{
  DWORD                      dwSize; 
  int                        nRecordNo;                                 // Queried Record Number 
  char                       szMasterOfCar[DH_MAX_NAME_LEN];            // Car Owner's Name
  char                       szPlateNumber[DH_MAX_PLATE_NUMBER_LEN];    // License Plate Number 
  EM_NET_PLATE_TYPE          emPlateType;                               // License Plate Type 
  EM_NET_PLATE_COLOR_TYPE    emPlateColor;                              // License Plate Color   
  EM_NET_VEHICLE_TYPE        emVehicleType;                             // Vehicle Type  
  EM_NET_VEHICLE_COLOR_TYPE  emVehicleColor;                            // Car Body Color
  NET_TIME                   stBeginTime;                               // Start Time 
  NET_TIME                   stCancelTime;                              // Undo Time
  int                        nAuthrityNum;                              // Permission Number
  NET_AUTHORITY_TYPE         stAuthrityTypes[DH_MAX_AUTHORITY_LIST_NUM];// Permissions List, White List Only 
  EM_NET_TRAFFIC_CAR_CONTROL_TYPE emControlType;                        // Monitor Type, Black List Only 
}NET_TRAFFIC_LIST_RECORD;

// Query Conditions Of Traffic Black And White List Account Records 
typedef struct
{
    DWORD                    dwSize;
    char                     szPlateNumber[DH_MAX_PLATE_NUMBER_LEN];      // License Plate Number
    char                     szPlateNumberVague[DH_MAX_PLATE_NUMBER_LEN]; // License Plate Number Fuzzy Query 
    int                      nQueryResultBegin;                           // Offset in the query results of first results returned   
    BOOL                     bRapidQuery;                                 // Whether support the quick query, TRUE: for quick, quick query time don't wait for all add, delete, change operation is completed. The default is non-quick query 
}FIND_RECORD_TRAFFICREDLIST_CONDITION;

// Suspicious vehicle Event, corresponding to DH_ALARM_TRAFFIC_SUSPICIOUSCAR
typedef struct tagALARM_TRAFFIC_SUSPICIOUSCAR_INFO
{
    DWORD                   dwSize;
    int                     nAction;                        // Event Action, -1:unknown, 0:Start, 1:Stop, 2:Pulse
    DH_MSG_OBJECT           stuVehicle;                     // Vehicle info
    NET_TRAFFIC_LIST_RECORD stuCarInfo;                     // Information of recorded in transportation black and white list 
    EVENT_COMM_INFO         stCommInfo;                     // common info
}ALARM_TRAFFIC_SUSPICIOUSCAR_INFO;

//Record Case Record Query Conditions 
typedef struct
{
    DWORD                    dwSize;
    NET_TIME                 stuStartTime;                      // Start Time 
    NET_TIME                 stuEndTime;                        // End Time
}FIND_RECORD_BURN_CASE_CONDITION;


// Entrance Card Record Query Conditions 
typedef struct tagFIND_RECORD_ACCESSCTLCARD_CONDITION
{
    DWORD                    dwSize;
    char                     szCardNo[DH_MAX_CARDNO_LEN];      // Card Number 
    char                     szUserID[DH_MAX_USERID_LEN];      // User ID 
    BOOL                     bIsValid;                         // Whether effective, TRUE: effective, FALSE: invalid 
    BOOL                     abCardNo;                         // Card inquire condition effects or not, for member szCardNo
    BOOL                     abUserID;                         // User ID inquire condition effects or not, for member  szUserID
    BOOL                     abIsValid;                        // IsValid inquire condition effects or not, for member  bIsValid
}FIND_RECORD_ACCESSCTLCARD_CONDITION;

// Access password record query conditions 
typedef struct tagFIND_RECORD_ACCESSCTLPWD_CONDITION
{
    DWORD                     dwSize;
    char                      szUserID[DH_MAX_USERID_LEN];      // User ID
}FIND_RECORD_ACCESSCTLPWD_CONDITION;

// Entrance guard access records query conditions 
typedef struct tagFIND_RECORD_ACCESSCTLCARDREC_CONDITION
{
    DWORD                     dwSize;
    char                      szCardNo[DH_MAX_CARDNO_LEN];      // User ID
    NET_TIME                  stStartTime;                      // Start Time 
    NET_TIME                  stEndTime;                        // End Time
}FIND_RECORD_ACCESSCTLCARDREC_CONDITION;

// A&C extry/exit search criteria
typedef struct tagFIND_RECORD_ACCESSCTLCARDREC_CONDITION_EX
{
    DWORD                     dwSize;
    BOOL                      bCardNoEnable;                    // Enable card search
    char                      szCardNo[DH_MAX_CARDNO_LEN];      // Card No.
    BOOL                      bTimeEnable;                      // Enable search by period
    NET_TIME                  stStartTime;                      // Start time 
    NET_TIME                  stEndTime;                        // End time 
}FIND_RECORD_ACCESSCTLCARDREC_CONDITION_EX;


// sensor record search criteria
typedef struct tagFIND_RECORD_SENSORRECORD_CONDITION
{
	DWORD					  dwSize;							
	NET_TIME                  stStartTime;                      // Start time, required 
    NET_TIME                  stEndTime;                        // End time, required 
	UINT					  uDeviceID;						// Acquisition device ID, optional.High 8 bits mean instrument type,low 24 bits mean the instrument serial number in the group.For example:0-electricity,1-analog sensor.0xffffffff means invalid
	BOOL                      bDeviceIDEnable;                  // Enbale search by device ID
	BOOL                      bStatusEnable;                    // Enable search by status
	BYTE					  byStatus;                         // Data status,optinal.0:normal,1:abnormal.0xff:invalid
	BYTE					  byReserved[3];					// Reserved
} FIND_RECORD_SENSORRECORD_CONDITION;

// Holiday Recordset Query Conditions 
typedef struct tagFIND_RECORD_ACCESSCTLHOLIDAY_CONDITION
{
    DWORD                     dwSize;	
    char            		  szHolidayNo[DH_COMMON_STRING_32];       // holiday no
}FIND_RECORD_ACCESSCTLHOLIDAY_CONDITION;

// electronic tag query conditions
typedef struct tagFIND_RECORD_ELECTRONICSTAG_CONDITION
{
	DWORD					dwSize;										// struct size
	NET_TIME                stStartTime;								// the start time of Through Time, required
    NET_TIME                stEndTime;									// the end time of Through Time, required
	BOOL					abDevId;									// whether the device ID is effective or not
	UINT					unDevID;									// device id ,from 0, optional
	BOOL					abMatchPic;									// whether the picture matched is effective or not
    BOOL					bMatch;										// whether the picture has been matched, optional
    BOOL					abPlateNumber;								// whether the plate number is effective or not
    char					szPlateNumber[DH_MAX_PLATE_NUMBER_LEN];		// plate number, optional
} FIND_RECORD_ELECTRONICSTAG_CONDITION;


//Record Resident info Query Conditions
typedef struct tagFIND_RECORD_RESIDENT_CONDTION
{
	DWORD 					  dwSize;
	char 					  szICNum[DH_MAX_IC_LEN];		   //Identify Card
}FIND_RECORD_RESIDENT_CONDTION;

// traffic flow record search filter
typedef struct tagFIND_RECORD_TRAFFICFLOW_CONDITION
{
    DWORD                     dwSize;
    BOOL                      abChannelId;                      // channel no. search filter is valid or not     
    int                       nChannelId;                       // channel no.
    BOOL                      abLane;                           // lane no. searchfilter is valid or not
    int                       nLane;                            // lane no.
    BOOL                      bStartTime;						// The start time search critiera is valid or not    
    NET_TIME                  stStartTime;                      // start time
    BOOL					  bEndTime;							// The end time search criteria is valid or not. 
    NET_TIME                  stEndTime;                        // end time 
    BOOL                      bStatisticsTime;                 // query if statistics time or notbStartTime and bEndTime TRUE
}FIND_RECORD_TRAFFICFLOW_CONDITION;

#define DH_MAX_CALLTYPE_LIST_NUM        16              // call  type search criteria listmax number 
#define DH_MAX_ENDSTATE_LIST_NUM        16              // final status  search criteria listmax number 

// call record search criteria 
typedef struct tagFIND_RECORD_VIDEO_TALK_LOG_CONDITION
{
    DWORD               dwSize;
    BOOL                bCallTypeEnable;                // call  type search criteria valid or not
    int                 nCallTypeListNum;               // corresponding to emCallTypeList valid enumeration number 
    EM_VIDEO_TALK_LOG_CALLTYPE   emCallTypeList[DH_MAX_CALLTYPE_LIST_NUM];  // call  type enable list
    BOOL                bEndStateEnable;                // final status  search criteria isvalid or not
    int                 nEndStateListNum;               // corresponding to emEndStateList valid enumeration number 
    EM_VIDEO_TALK_LOG_ENDSTATE   emEndStateList[DH_MAX_ENDSTATE_LIST_NUM];  // final status enable list
    BOOL                bTimeEnable;                    // Time enable
    NET_TIME            stStartTime;                    // Start Time
    NET_TIME            stEndTime;                      // End Time
}FIND_RECORD_VIDEO_TALK_LOG_CONDITION;

// status  record search criteria 
typedef struct tagFIND_RECORD_REGISTER_USER_STATE_CONDITION
{
    DWORD               dwSize;
    BOOL                bUserIDEnable;                  // userID search criteria is valid or not
    char                szUserID[DH_MAX_USERID_LEN];    // user ID
    BOOL                bOnlineEnable;                  // online search criteria is valid or not
    int					nOnline;            			// Online
    BOOL                bVideoTalkingEnable;            // audio talk status  search criteria  is valid or not 
    EM_REGISTER_USER_STATE     emVideoTalking;          // audio talk status 
}FIND_RECORD_REGISTER_USER_STATE_CONDITION;

// contact record search criteria 
typedef struct tagFIND_RECORD_VIDEO_TALK_CONTACT_CONDITION
{
    DWORD               dwSize;
    BOOL                bVTShortNumberEnable;                   // visual talk short no. search criteria  is valid or not 
    char                szVTShortNumber[DH_COMMON_STRING_32];   // visual talk short no.
}FIND_RECORD_VIDEO_TALK_CONTACT_CONDITION;

// Delivered commodies search criteria 
typedef struct tagFIND_RECORD_COMMODITY_NOTICE_CONDITION
{
    DWORD                           dwSize;
    BOOL                            bIDEnable;                                   // Enable ID Search
    char                            szID[DH_COMMON_STRING_64];                   // ID
}FIND_RECORD_COMMODITY_NOTICE_CONDITION;

// Medical check info search criteria 
typedef struct tagFIND_RECORD_HEALTH_CARE_NOTICE_CONDITION
{
    DWORD                           dwSize;
    BOOL                            bIDEnable;                                   // Enable ID Search
    char                            szID[DH_COMMON_STRING_64];                   // ID
}FIND_RECORD_HEALTH_CARE_NOTICE_CONDITION;


// traffic flow record
typedef struct tagNET_RECORD_TRAFFIC_FLOW_STATE 
{
    DWORD                           dwSize;
    int                             nRecordNum;                 // record no.
    int                             nChannel;                   // channel no.   
    int                             nLane;                      // lane no.
    int                             nVehicles;                  // pass vehicle total
    float                           fAverageSpeed;              // average speed, unit km/h
    float                           fTimeOccupyRatio;           // time occupancy rate, as within unit time passed vehicle used time total and unit occupied scale
    float                           fSpaceOccupyRatio;          // space occupancy, as percentage of total vehicle length to time interval average vehicle driving distance
    float                           fSpaceHeadway;              // head distance, with adjacent vehicle, unit meter/vehicle
    float                           fTimeHeadway;               // head distance, unit meter/vehicle
    int                             nLargeVehicles;             // truck flow(9m<vehicle length<12m), vehicle/unit time
    int                             nMediumVehicles;            // Midsize vehicle flow(6m<vehicle length<9m), vehicle/unit time
    int                             nSmallVehicles;             // car flow(4m<vehicle legnth<6m), vehicle/unit, 
    float                           fBackOfQueue;               // line length, unit: meter, from signal cross to front line end
}NET_RECORD_TRAFFIC_FLOW_STATE;

// CLIENT_FindRecord    Interface Input Parameters 
typedef struct _NET_IN_FIND_RECORD_PARAM
{
    DWORD                     dwSize;          // The Structure Size 
    EM_NET_RECORD_TYPE        emType;          // The record type to query
    void*                     pQueryCondition; // Query types corresponding to the query conditions 
											   // the space application by the user,according to query condition type,find corresponding structure,then ensure memory size
}NET_IN_FIND_RECORD_PARAM;

// CLIENT_FindRecord  Interface Output Parameters 
typedef struct _NET_OUT_FIND_RECORD_PARAM
{
    DWORD                     dwSize;          // Structure Size
    LLONG                     lFindeHandle;    // Query Log Handle,Uniquely identifies a certain query
}NET_OUT_FIND_RECORD_PARAM;

// CLIENT_FindNextRecord  Interface Input Parameters 
typedef struct _NET_IN_FIND_NEXT_RECORD_PARAM
{
    DWORD                     dwSize;          // Structure Size 
    LLONG                     lFindeHandle;    // Query Log Handle
    int                       nFileCount;      // The current number of records  need query 
}NET_IN_FIND_NEXT_RECORD_PARAM;

// CLIENT_FindNextRecord  Interface Output Parameters 
typedef struct _NET_OUT_FIND_NEXT_RECORD_PARAM
{
    DWORD                     dwSize;          // Structure Size 
    void*                     pRecordList;     // Record List, the user allocates memory, ensure structure by query record type(EM_NET_RECORD_TYPE) of NET_IN_FIND_RECORD_PARAM,then ensure memory size
    int                       nMaxRecordNum;   // Max list Record Number 
    int                       nRetRecordNum;   // Query to the number of records, when the query to the article number less than want to query the number, end 
}NET_OUT_FIND_NEXT_RECORD_PARAM;

typedef struct tagNET_INSERT_RECORD_INFO
{
    DWORD                     dwSize;
    NET_TRAFFIC_LIST_RECORD   *pRecordInfo;    // Record the content information,the space application by the user,apply to sizeof(NET_TRAFFIC_LIST_RECORD)                    
}NET_INSERT_RECORD_INFO;

typedef struct tagNET_UPDATE_RECORD_INFO
{
    DWORD                     dwSize;
    NET_TRAFFIC_LIST_RECORD   *pRecordInfo;    // Record the content information,the space application by the user,apply to sizeof(NET_TRAFFIC_LIST_RECORD)                      
}NET_UPDATE_RECORD_INFO;

typedef struct tagNET_REMOVE_RECORD_INFO
{
    DWORD                     dwSize;
    int                       nRecordNo;       // Queried Record Number    
}NET_REMOVE_RECORD_INFO;

typedef struct tagNET_IN_OPERATE_TRAFFIC_LIST_RECORD
{
    DWORD                     dwSize;
    
    EM_RECORD_OPERATE_TYPE    emOperateType;
    EM_NET_RECORD_TYPE        emRecordType;    // record type to operate 
    void                      *pstOpreateInfo; // the space application by the user,please refer to emRecordType to ensure corresponding structure,then ensure memory size
    
}NET_IN_OPERATE_TRAFFIC_LIST_RECORD;

//In current implementation of operation interface, only return nRecordNo operation, stRetRecord is temporarily unavailable 
typedef struct tagNET_OUT_OPERATE_TRAFFIC_LIST_RECORD
{
    DWORD                     dwSize;
    int                       nRecordNo;        //Record Number     
}NET_OUT_OPERATE_TRAFFIC_LIST_RECORD;

//Continuous control PTZ corresponding structure 
typedef struct tagPTZ_Control_Continuously
{
    PTZ_SPEED_UNIT         stuSpeed;              //PTZ speed 
    int                    nTimeOut;              //Continuous motion timeout, the unit is in seconds 
    char                   szReserve[64];         //Reserved
}PTZ_CONTROL_CONTINUOUSLY;

//Absolute control PTZ corresponding structure 
typedef struct tagPTZ_Control_Absolutely
{
    PTZ_SPACE_UNIT         stuPosition;           //PTZ Absolute Speed 
    PTZ_SPEED_UNIT         stuSpeed;              //PTZ Operation Speed
    char                   szReserve[64];         //Reserved
}PTZ_CONTROL_ABSOLUTELY;

//PTZ control coordinate unit,track move
typedef struct tagPTZ_LOCATION_SPACE_UNIT
{
    int                    nPositionX;           //PTZ horizontal motion position, effective range:[0,3600]
    int                    nPositionY;           //PTZ vertical motion position, effective range:[-1800,1800]
    int                    nZoom;                //PTZ zoom position
    char                   szReserve[32];        //Reserved
}PTZ_LOCATION_SPACE_UNIT;

//PTZ control coordinate unit,speed
typedef struct tagPTZ_LOCATION_SPEED_UNIT
{
    int                    nSpeedX;				//PTZ horizontal real Speed ,Left is negative,Right is positive,1000 means 10 degree per second,expanded 100 times
    int                    nSpeedY;				//PTZ vertical real Speed ,Left is negative,Right is positive,1000 means 10 degree per second,expanded 100 times
    char                   szReserve[32];       //Reserved
}PTZ_LOCATION_SPEED_UNIT;

//ptz Continuous movement,command DH_EXTPTZ_INTELLI_TRACKMOVE 
typedef struct tagPTZ_CONTROL_INTELLI_TRACKMOVE
{
	DWORD						dwSize;				   // Structure Size
	int							nChannelID;            // channel id
	int							nFlag;				   // move flag
													   // 0:location,speed  is invalid,position Zoom is valid
													   // 1:Continuous movement,speed is invalid,position Zoom is invalid
													   // 2:Continuous movement,speed is valid,position Zoom is invalid
	
	PTZ_LOCATION_SPACE_UNIT     stuPosition;           // PTZ Absolute move position 
    PTZ_LOCATION_SPEED_UNIT		stuSpeed;              // PTZ Operation Speed
}PTZ_CONTROL_INTELLI_TRACKMOVE;

//auxiliary	Auxiliary point	command: DH_EXTPTZ_AUXIOPEN,DH_EXTPTZ_AUXICLOSE, ptz control interface:CLIENT_DHPTZControlEx2
typedef struct tagPTZ_CONTROL_AUXILIARY
{
	DWORD						dwSize;
	char						szFunctionName[DH_COMMON_STRING_32];   // PTZ auxiliary function name,use interface CLIENT_DHPTZControlEx2 by command CFG_CAP_CMD_PTZ can get expected info(CFG_PTZ_PROTOCOL_CAPS_INFO)
}PTZ_CONTROL_AUXILIARY;

// Alarm input channel information 
typedef struct tagNET_ALARM_IN_CHANNEL
{
    DWORD                   dwSize;
    BOOL                    bValid;                             // whether effective,FALSE show the alarm channel is not used
    int                     nChannel;                           // Alarm channel number 
    char                    szDeviceID[DH_DEV_ID_LEN];          // Device ID, Local alarm channel: "Local",remote device : use uuid express
    char                    szName[DH_DEV_NAME_LEN];            // Alarm  Channel Names 
}NET_ALARM_IN_CHANNEL;

// Alarm Channel Number 
typedef struct tagNET_ALARM_CHANNEL_COUNT 
{
    DWORD                   dwSize;
    int                     nLocalAlarmIn;                      // Local Alarm Input Channel Number
    int                     nLocalAlarmOut;                     // Local Alarm Output Channel Number 
    int                     nRemoteAlarmIn;                     // Remote Alarm Input Channel Number 
    int                     nRemoteAlarmOut;                    // Remote Alarm Output Channel Number 
}NET_ALARM_CHANNEL_COUNT;

//With speed rotation site PTZ control corresponding to the preset structure 
typedef struct tagPTZ_Control_GotoPreset
{
    int                     nPresetIndex;           //Preset BIT Index 
    PTZ_SPEED_UNIT          stuSpeed;               //PTZ Operation Speed
    char                    szReserve[64];          //Reserved
}PTZ_CONTROL_GOTOPRESET;

//remove preset PTZ control corresponding to the preset structure
typedef struct tagPTZ_Control_RemovePreset
{
    int                     nPresetIndex;           //Preset BIT Index
    char                    szReserve[64];          //Reserved
}PTZ_CONTROL_REMOVEPRESET;

//With speed rotation site and snap PTZ control corresponding to the preset structure 
typedef struct tagPTZ_Control_GotoPresetSnap
{
    int                     nPresetIndex;           //Preset BIT Index 
	int						nChannel;               //channel NO
    PTZ_SPEED_UNIT          stuSpeed;               //PTZ Operation Speed
    char                    szReserve[64];          //Reserve
}PTZ_CONTROL_GOTOPRESETSNAP;

// CLIENT_SetTourSource   Interface input parameters (Settings window round tour shows source) 
typedef struct tagNET_IN_SET_TOUR_SOURCE 
{
    DWORD                   dwSize;
    int                     nChannel;               // Output Channel Number 
    int                     nWindow;                // Window Number
    DH_SPLIT_SOURCE*        pstuSrcs;               // Display Origin Array,can be round tour in the window,the space application by the user,apply to sizeof(DH_SPLIT_SOURCE)*nSrcCount  
    int                     nSrcCount;              // Display Origin Number
} NET_IN_SET_TOUR_SOURCE;

// CLIENT_SetTourSource    Interface output parameters (Settings window round tour shows source) 
typedef struct tagNET_OUT_SET_TOUR_SOURCE
{
    DWORD                   dwSize;
} NET_OUT_SET_TOUR_SOURCE;

// CLIENT_GetTourSource port input parameter 
typedef struct tagNET_IN_GET_TOUR_SOURCE 
{
    DWORD           dwSize;
    int             nChannel;           // output channel no., pszCompsiteIDis NULL, it is valid 
    const char*     pszCompositeID;     // splicing video wall ID
    int             nWindow;            // window no., -1 means all window
} NET_IN_GET_TOUR_SOURCE;

// window tour display source info 
typedef struct tagNET_SPLIT_TOUR_SOURCE 
{
    DWORD           dwSize;
    DH_SPLIT_SOURCE*    pstuSrcs;       // display source group, user allocation memory,apply to sizeof(DH_SPLIT_SOURCE)*nMaxSrcCount
    int             nMaxSrcCount;       // display source max quantity
    int             nRetSrcCount;       // return display source quantity
} NET_SPLIT_TOUR_SOURCE;

// CLIENT_GetTourSource port  output parameter 
typedef struct tagNET_OUT_GET_TOUR_SOURCE
{
    DWORD          dwSize;  
    NET_SPLIT_TOUR_SOURCE* pstuWndSrcs; // window tour info group, user allocation memory,the space application by the user,apply to sizeof(NET_SPLIT_TOUR_SOURCE)*nMaxWndCount
    int            nMaxWndCount;        // window max group, user fill in
    int            nRetWndCount;        // return window  quantity
} NET_OUT_GET_TOUR_SOURCE;

// split tour status info 
typedef struct tagNET_SPLIT_TOUR_STATUS_INFO
{
    DWORD          dwSize;
    int            nWindow;             // window no.
    NET_TOUR_STATUS   	emStatus;       // status 
    DH_SPLIT_WND_SOURCE stuSource;      //  current display source info 
}NET_SPLIT_TOUR_STATUS_INFO;

// tour status call function origin, lAttachHandle is CLIENT_AttachSplitTour return value 
typedef void (CALLBACK *fTourStatusCallBack)(LLONG lLoginID, LLONG lAttachHandle, NET_SPLIT_TOUR_STATUS_INFO* pstStatus, void* reserved, LDWORD dwUser);

// CLIENT_AttachSplitTourport input parameter 
typedef struct tagNET_IN_ATTACH_SPLIT_TOUR
{
    DWORD           dwSize;
    int             nChannel;           // output channel no., pszCompsiteIDis NULL, it is valid 
    const char*     pszCompositeID;     // splicing video wall ID
    fTourStatusCallBack cbStatus;       // tour status call function
    LDWORD          dwUser;             // tour status call function
}NET_IN_ATTACH_SPLIT_TOUR;

// CLIENT_AttachSplitTourport  output parameter 
typedef struct tagNET_OUT_ATTACH_SPLIT_TOUR 
{
    DWORD           dwSize;
}NET_OUT_ATTACH_SPLIT_TOUR;

// the work mode of this window
typedef enum tagEM_WINDOW_WORKMODE
{
	EM_WINDOW_DISPLAY,				// display(default)
	EM_WINDOW_REPLAY,				// replay
} EM_WINDOW_WORKMODE;

// display source infomation
typedef struct tagNET_SOURCE_INFO
{
	int 				nWindowID;					// current window ID
	int					nCaremaChannel;				// current video source channel ID
	NET_TIME			stuStartTime;				// the start time
	EM_WINDOW_WORKMODE	emWorkMode;					// work mode of the windows
	BYTE				bReserved[128];				// reserved byte
}NET_SOURCE_INFO;

// the display sources of windows
typedef struct tagNET_WINDOW_SOURCE_INFO
{
	DWORD				dwSize;
	int					nRetSrcCount;							// actual numer of source
	NET_SOURCE_INFO		*pstSourceInfo;							// display source infomation of each window
} NET_WINDOW_SOURCE_INFO;

// the call back function of windows sources, lAttachHandleis CLIENT_AttachWindowsSource return value
typedef void (CALLBACK *fWindowSourceCallBack)(LLONG lLoginID, LLONG lAttachHandle, NET_WINDOW_SOURCE_INFO* pstWindowSource, void* reserved, LDWORD dwUser);

// CLIENT_AttachWindowsSource input parameter 
typedef struct tagNET_IN_ATTACH_WINDOW_SOURCE
{
	DWORD					dwSize;
	int						nWindowID;					// window ID, -1 means the entire screen
	int 					nChannel;					// video out channel,  it is effective when pszCompositeID is NULL
	const char*         	pszCompositeID;             // composite ID
	fWindowSourceCallBack 	cbWindowSource;             // the call back function of windows sources
    LDWORD              	dwUser;                     // the param of window source call back
} NET_IN_ATTACH_WINDOW_SOURCE;

// CLIENT_AttachWindowsSource output parameter
typedef struct tagNET_OUT_ATTACH_WINDOW_SOURCE
{
    DWORD               dwSize;
}NET_OUT_ATTACH_WINDOW_SOURCE;

// input parameter of interface CLIENT_WindowRegionEnlarge 
typedef struct tagNET_IN_WINDOW_REGION_ENLARGE
{
	DWORD					dwSize;
    int                 	nChannel;           // video out channel or virtual channel
	int						nWindowID;			// window ID
	NET_RECT				stuRect;			// area of windw magnification, the coordinate system is virtual coodinate system.[0~8192]
} NET_IN_WINDOW_REGION_ENLARGE;

// out parameter of interface CLIENT_WindowRegionEnlarge
typedef struct tagNET_OUT_WINDOW_REGION_ENLARGE
{
	DWORD					dwSize;
} NET_OUT_WINDOW_REGION_ENLARGE;

// input parameter of interface CLIENT_WindowEnlargeReduction
typedef struct tagNET_IN_WINDOW_ENLARGE_REDUCTION
{
	DWORD					dwSize;
    int                 	nChannel;           // video out channel or virtual channel
	int						nWindowID;			// window ID
} NET_IN_WINDOW_ENLARGE_REDUCTION;

// outparameter of interface CLIENT_WindowEnlargeReduction
typedef struct tagNET_OUT_WINDOW_ENLARGE_REDUCTION
{
	DWORD					dwSize;
} NET_OUT_WINDOW_ENLARGE_REDUCTION;



// fAttachRecordInfoCB parameter, information of reported video file 
typedef struct tagNET_CB_RECORD_INFO
{
    DWORD                   dwSize;
    int                     nChannel;               // Channel number 
    char                    szFileName[MAX_PATH];   // The name of the video 
    DWORD                   dwType;                 // Video type,defined by bit as follows:
                                                    // Bit0-Timing video 
                                                    // Bit1-Dynamic test video 
                                                    // Bit2-Alarm video
                                                    // Bit3-Video card number
    DWORD                   dwState;                // Video status, 0 - packaging, 1 - to delete 
} NET_CB_RECORD_INFO;



// Video update callback function prototype, lAttachHandle is return value of CLIENT_AttachRecordInfo, n strip per time, pBuf->dwSize * n == nBufLen
typedef void (CALLBACK *fAttachRecordInfoCB)(LLONG lLoginID, LLONG lAttachHandle, NET_CB_RECORD_INFO* pBuf, int nBufLen, LDWORD dwUser);

// CLIENT_AttachRecordInfo  Input Parameters
typedef struct tagNET_IN_ATTACH_RECORD_INFO
{
    DWORD                   dwSize;
    int                     nInterval;              // Time Interval, Every Interval second, equipment send video information 
    int                     nDelay;                 // Report Delay, after equipment choice a random number between 1 and nDelay value, delay report video state, nDelayMust be smaller than nInterval
    fAttachRecordInfoCB     cbAttachRecordInfo;     // Video update callback function 
    LDWORD                  dwUser;                 // User data 
} NET_IN_ATTACH_RECORD_INFO;

typedef struct tagNET_OUT_ATTACH_RECORD_INFO
{
    DWORD                   dwSize;
} NET_OUT_ATTACH_RECORD_INFO;


//Subscribe to PTZ metadata interface and callback function prototypes 
//Pbufs at this stage mainly DH_PTZ_LOCATION_INFO type 
typedef void (CALLBACK *fPTZStatusProcCallBack)(LLONG lLoginID, LLONG lAttachHandle, void* pBuf, int nBufLen, LDWORD dwUser);

// Subscribe to PTZ metadata interface input parameters 
typedef struct tagNET_IN_PTZ_STATUS_PROC
{
    DWORD                   dwSize;
    int                     nChannel;              // PTZ Channel 
    fPTZStatusProcCallBack  cbPTZStatusProc;       // Callback function 
    LDWORD                  dwUser;                // User data
}NET_IN_PTZ_STATUS_PROC;

// Subscribe to PTZ metadata interface output parameters 
typedef struct tagNET_OUT_PTZ_STATUS_PROC
{
    DWORD                   dwSize;
}NET_OUT_PTZ_STATUS_PROC;

//PTZ conditions for visual structure
typedef struct tagDH_OUT_PTZ_VIEW_RANGE_STATUS
{
    DWORD      dwSize;
    double     dbDistance;                         // Visual range, the unit: m 
    int        nAngelH;                            // Horizontal viewing angles, 0~1800, unit: degrees 
    int        nAzimuthH;                          // Horizontal azimuth Angle, 0 ~ 3600, unit: degrees 
    int        nInclinationH;                      // Horizontal inclination angle, -900~900, unit: degree
}DH_OUT_PTZ_VIEW_RANGE_STATUS;

//Subscribe to yuntai horizon callback function prototype 
typedef void (CALLBACK *fViewRangeStateCallBack)(LLONG lLoginID, LLONG lAttachHandle, DH_OUT_PTZ_VIEW_RANGE_STATUS* pBuf, int nBufLen, LDWORD dwUser);

// Subscribe to the visible range input parameters 
typedef struct tagNET_IN_VIEW_RANGE_STATE
{
    DWORD                   dwSize;
    int                     nChannel;              // PTZ channel
    fViewRangeStateCallBack cbViewRange;           // State Callback Function 
    LDWORD                  dwUser;                // User Data
}NET_IN_VIEW_RANGE_STATE;

// Subscribe to the visible range output parameters 
typedef struct tagNET_OUT_VIEW_RANGE_STATE
{
    DWORD                   dwSize;
}NET_OUT_VIEW_RANGE_STATE;

// Weather info 
typedef struct tagNET_WEATHER_INFO
{
	float				    fWindSpeed;		       // Wind speed,Unit:m/s,Range:[0.0,60.0]	
	float					fTemperature;		   // Temperature,Unit:Centigrade,Range:[-40.0,+80.0]
	float					fHumidity;			   // postfix:%,Range[0.0,100.0]
	unsigned int			nLight;				   // Light,Unit:W/(m*m),Range:[0,2000]
	float					fWindDirection;		   // Wind direction,Unit:degree,Range:[0,360)
	float				    fAirPressure;		   // Air pressure,Unit:hPa,Range[10~1100]
	BYTE					byReserved[2048];	   // reserved
}NET_WEATHER_INFO;

// Subscribe to ptz weather info callback function
typedef void (CALLBACK *fWeatherInfoCallBack)(LLONG lLoginID, LLONG lAttachHandle, NET_WEATHER_INFO* pBuf, int nBufLen, LDWORD dwUser);

//  Subscribe to the weather info input parameters
typedef struct tagNET_IN_WEATHER_INFO
{
	DWORD                   dwSize;
	int                     nChannel;              // PTZ channel
	fWeatherInfoCallBack	cbWeatherInfo;         // Callback Function 
	LDWORD                  dwUser;                // User Data
}NET_IN_WEATHER_INFO;

// Subscribe to the weather info output parameters
typedef struct tagNET_OUT_WEATHER_INFO
{
	DWORD                   dwSize;
}NET_OUT_WEATHER_INFO;

// configuration of set view range by object input parameters
typedef struct tagNET_IN_SET_VIEW_RANGE_BY_OBJECT
{
	DWORD					dwSize;					
	int						nChannel;				// channel number(0 begin)
	int						nDistance;				// object distance(Unit:centimeter) 
	int						nWidth;					// object width(Unit:centimeter)
	int						nScreenRatio;			// screen Ratio
}NET_IN_SET_VIEW_RANGE_BY_OBJECT;

// configuration of set view range by object output parameters
typedef struct tagNET_OUT_SET_VIEW_RANGE_BY_OBJECT
{
    DWORD                   dwSize;
}NET_OUT_SET_VIEW_RANGE_BY_OBJECT;
//Set the PTZ vision information 
typedef struct tagPTZ_VIEW_RANGE_INFO
{
    int                     nStructSize;
    int                     nAzimuthH;              // Horizontal azimuth Angle, 0~3600, unit: degrees 
}PTZ_VIEW_RANGE_INFO;

//Query PTZ optical zoom value( corresponding to DH_DEVSTATE_PTZ_ZOOM_INFO ) 
typedef struct tagDH_OUT_PTZ_ZOOM_INFO
{
    int                     dwSize;
    int                     nZoomValue;              // zoom value
}DH_OUT_PTZ_ZOOM_INFO;

// Channel number information 
typedef struct tagNET_CHN_COUNT_INFO
{
    DWORD                   dwSize;
    int                     nMaxTotal;              // Equipment to the total number of channels (the sum of all valid channel number
    int                     nCurTotal;              // the number of configured on channels 
    int                     nMaxLocal;              // Maximum number of local channels, including motherboard and pluggable cartoon 
    int                     nCurLocal;              // configured local channel number 
    int                     nMaxRemote;             // Maximum number of remote channel 
    int                     nCurRemote;             // Configured remote channel number
} NET_CHN_COUNT_INFO;

// Equipment channel number information  
typedef struct tagNET_DEV_CHN_COUNT_INFO
{
    DWORD                   dwSize;
    NET_CHN_COUNT_INFO      stuVideoIn;             // Video Input Channel 
    NET_CHN_COUNT_INFO      stuVideoOut;            // Video Output Channel 
} NET_DEV_CHN_COUNT_INFO;

//  detailed information of video state
typedef struct tagNET_RECORD_STATE_DETAIL 
{
    DWORD                   dwSize;
    BOOL                    bMainStream;            // The main stream, TRUE - are video, FALSE - not in the video 
    BOOL                    bExtraStream1;          // Auxiliary stream 1, TRUE - are video, FALSE - not in the video 
    BOOL                    bExtraStream2;          // Auxiliary stream 2, TRUE - are video, FALSE - not in the video 
    BOOL                    bExtraStream3;          // Auxiliary stream 3, TRUE - are video, FALSE - not in the video 
} NET_RECORD_STATE_DETAIL;


//PTZ Absolute Focus Corresponding Structure 
typedef struct tagPTZ_Focus_Absolutely
{
    DWORD                    dwValue;               // PTZ Focused On Location, range (0~8191) 
    DWORD                    dwSpeed;               // PTZ Focused On Speed, the scope (0~7) 
    char                     szReserve[64];         // reserved 64 bytes 
}PTZ_FOCUS_ABSOLUTELY;

// CLIENT_PlayAudioFile   Interface Input Parameters 
typedef struct tagNET_IN_PLAY_AUDIO_FILE
{
    DWORD                   dwSize;
    const char*             pszFilePath;            // File Path
    DWORD                   dwOutput;               // Output Path, can be a variety of output, according to the bit, bit0-mic, bit1-speaker
} NET_IN_PLAY_AUDIO_FILE;

// CLIENT_PlayAudioFile   Interface Output Parameters 
typedef struct tagNET_OUT_PLAY_FILE_STREAM
{
    DWORD                   dwSize;
} NET_OUT_PLAY_AUDIO_FILE;

// RTSP URLInformation list structure 
typedef struct tagNET_DEV_RTSPURL_LIST
{
    DWORD                   dwSize;
    int                     nChannelID;                                 // Channel number (user input parameters) 
    int                     nUrlNum;                                    // Url Address Number 
    char                    szURLList[DH_MAX_URL_NUM][DH_MAX_URL_LEN];  // UrlAddress List 
}NET_DEV_RTSPURL_LIST;

// PTZ control - fan and corresponding structure 
typedef struct tagPTZ_Control_SectorScan
{
    int             nBeginAngle;                    // Staring Angle,Range:[-180,180]
    int             nEndAngle;                      // Ending Angle,Range:[-180,180]
    int             nSpeed;                         // Speed,Range:[0,255]
    char            szReserve[64];                  // Reserved 64 bytes 
}PTZ_CONTROL_SECTORSCAN;

// control Fish eye E-PTZ info 
typedef struct tagPTZ_CONTROL_SET_FISHEYE_EPTZ
{
    DWORD       dwSize;	                // structure size
    DWORD       dwWindowID;             // EPtz control window no.
    DWORD       dwCommand;              // E-PTZ command 
    DWORD       dwParam1;               // command corresponding to parameter 1
    DWORD       dwParam2;               // command corresponding to  parameter 2
    DWORD       dwParam3;               // command corresponding to  parameter 3
    DWORD       dwParam4;               // command corresponding to  parameter 4
}PTZ_CONTROL_SET_FISHEYE_EPTZ;

// Track Control Information
typedef struct tagPTZ_CONTROL_SET_TRACK_CONTROL
{
        DWORD           dwSize;                         // dwSize need to be assigned sizeof(PTZ_CONTROL_SET_TRACK_CONTROL)
        DWORD           dwChannelID;                    // channel number
        DWORD           dwCommand;                      // Control command, orresponding to enum NET_TRACK_CONTROL_CMD
        DWORD           dwParam1;                       // command corresponding to parameter 1
        DWORD           dwParam2;                       // command corresponding to parameter 2
        DWORD           dwParam3;                       // command corresponding to parameter 3
} PTZ_CONTROL_SET_TRACK_CONTROL;
//===============Area Scan, Ptz operator=============================

// Area Scan command of ptz (interface is CLIENT_DHPTZAreaScan)
typedef enum tagNET_AREASCAN_EPTZ_CMD
{
	NET_AREASCAN_EPTZ_CMD_SET,					// set Area Scan info command(szInBuffer according to PTZ_CONTROL_SET_AREA_SCAN_LIMIT,szOutBuffer,dwOutBufferSize is invalid)
	NET_AREASCAN_EPTZ_CMD_DEL,					// delete Area Scan info command(szInBuffer according to PTZ_CONTROL_DEL_AREA_SCAN,szOutBuffer,dwOutBufferSize is invalid)
	NET_AREASCAN_EPTZ_CMD_GET, 					// get Area Scan info command(szInBuffer according to PTZ_CONTROL_GET_AREA_SCAN_IN,szOutBuffer according to PTZ_CONTROL_GET_AREA_SCAN_OUT )
	NET_AREASCAN_EPTZ_CMD_START,				// start Area Scan command(szInBuffer according to PTZ_CONTROL_START_AREA_SCAN,szOutBuffer,dwOutBufferSize is invalid)
	NET_AREASCAN_EPTZ_CMD_STOP,					// stop Area Scan command(szInBuffer according to PTZ_CONTROL_STOP_AREA_SCAN,szOutBuffer,dwOutBufferSize is invalid)
}NET_AREASCAN_EPTZ_CMD;

// set area scan info 
typedef struct tagPTZ_CONTROL_SET_AREA_SCAN_LIMIT
{
    DWORD           dwSize;                         // the size of this struct
    int				nChannelID;                     // channel id
	int             nIndex;							// area number,the number form 1.
	char			szLimitMode[16];				// Area boundary, "Left": left boundary "Right":right boundary
	int             nZoom;							// Zoom,range:0~128, default 0
} PTZ_CONTROL_SET_AREA_SCAN_LIMIT;

// delete area scan info
typedef struct tagPTZ_CONTROL_DEL_AREA_SCAN
{
    DWORD           dwSize;                         // the size of this struct
    int				nChannelID;                     // channel id
	int             nIndex;							// area number,the number form 1.
} PTZ_CONTROL_DEL_AREA_SCAN;

// get area scan info,input param
typedef struct tagPTZ_CONTROL_GET_AREA_SCAN_IN
{
    DWORD           dwSize;                         // the size of this struct
    int				nChannelID;                     // channel id
	int             nSize;							// area scan numbers
} PTZ_CONTROL_GET_AREA_SCAN_IN;

// area state info
typedef struct tagNET_AREASCAN_STATE 
{
    BOOL            bValid;							// area state
	BYTE			Reserved[128];					// Reserved
}NET_AREASCAN_STATE;

// get area scan info,output param
typedef struct tagPTZ_CONTROL_GET_AREA_SCAN_OUT
{
    DWORD                       dwSize;                             // the size of this struct
	UINT				        nValidNum;						    // valid number of arrary stuStateInfo
    NET_AREASCAN_STATE			stuStateInfo[DH_COMMON_STRING_32];  // array of area state info
}PTZ_CONTROL_GET_AREA_SCAN_OUT;

// Area Scan start info
typedef struct tagPTZ_CONTROL_START_AREA_SCAN
{
    DWORD           dwSize;                         // the size of this struct
    int				nChannelID;                     // channel id
	int             nIndex;							// area number,the number form 1.
	int             nSpeed;							// horizontal scan speed,range 1~255
	int				nTitleAngle;					// Vertical scan angele, range 0~90 unit: degrees 
} PTZ_CONTROL_START_AREA_SCAN;

// Area Scan stop info
typedef struct tagPTZ_CONTROL_STOP_AREA_SCAN
{
    DWORD           dwSize;                         // the size of this struct
    int				nChannelID;                     // channel id
	int             nIndex;							// area number,the number form 1.
} PTZ_CONTROL_STOP_AREA_SCAN;

//===============Area Scan, Ptz operator=============================

// the type of region focus
typedef enum tagEM_NET_FOCUS_REGION_MODE
{
    EM_NET_FOCUS_REGION_MODE_GLOBAL,                // global
    EM_NET_FOCUS_REGION_MODE_AUTO,                  // auto
    EM_NET_FOCUS_REGION_MODE_INROIREGION,           // in the ROI region
    EM_NET_FOCUS_REGION_MODE_OUTROIREGION,          // out of the ROI region

}EM_NET_FOCUS_REGION_MODE;

// the info of focus region(corresponding to DH_EXTPTZ_SET_FOCUS_REGION)
typedef struct tagPTZ_CONTROL_SET_FOCUS_REGION
{
    DWORD                       dwSize;             // sizeof(PTZ_CONTROL_SET_FOCUS_REGION)   
    EM_NET_FOCUS_REGION_MODE    emMode;	            // focus mode
    NET_RECT                    stuRect;            // focus region,value form 0 to 8192                        
} PTZ_CONTROL_SET_FOCUS_REGION;


// CLIENT_TransmitInfoForWeb   Interface Extension Parameters 
typedef struct tagNET_TRANSMIT_EXT_INFO
{
    DWORD           dwSize;
    unsigned char*  pInBinBuf;                      // Binary Input Data Buffer 
    DWORD           dwInBinBufSize;                 // Binary Input Data Length 
    int             nJsonDataLen;                   // Json Output Data Length
    int             nBinDataLen;                    // Binary Output Data Length
} NET_TRANSMIT_EXT_INFO;

// Monitor CAN Bus Data 
// fAttachProxyCB Parameter 
typedef struct tagNET_CB_CANDATA
{
    DWORD           dwSize;
    int             nDataLength;                    // Common Data Length
    unsigned char*  pDataContent;                   // Common Data Content
}NET_CB_CANDATA;

// Monitor CAN bus data callback function prototype , lAttachHandle is return value of CLIENT_AttachCAN
typedef void (CALLBACK *fAttachCANCB) (LLONG lLoginID, LLONG lAttachHandle, NET_CB_CANDATA* pBuf, LDWORD dwUser);

// CLIENT_AttachCAN() Input Parameters 
typedef struct tagNET_IN_ATTACH_CAN
{
    DWORD          dwSize;
    int            nChannel;                        // Monitor CAN channel number 
    fAttachCANCB   cbAttachCAN;                     // Callback Registered Video Phone State Changes 
    LDWORD         dwUser;                          // User Data 
}NET_IN_ATTACH_CAN;

// CLIENT_AttachCAN() Output Parameters 
typedef struct tagNET_OUT_ATTACH_CAN
{
    DWORD          dwSize;
}NET_OUT_ATTACH_CAN;

// CLIENT_SendCAN() input parameter
typedef struct tagNET_IN_SEND_CAN
{
    DWORD          dwSize;
    int            nChannel;                       // CAN channel no.
	int            nSendDataLength;                // Send data length
    unsigned char* pSendData;                      // Send data content
}NET_IN_SEND_CAN;

// CLIENT_SendCAN() output parameter
typedef struct tagNET_OUT_SEND_CAN
{
    DWORD          dwSize;
}NET_OUT_SEND_CAN;

// parameter of fAttachDevCommCB
typedef struct tagNET_CB_DEVCOMMDATA
{
    DWORD           dwSize;
    int             nDataLength;                    // data length
    unsigned char*  pDataContent;                   // data content
}NET_CB_DEVCOMMDATA;

// callback prototype of listening to data from trans comm, lAttachHandle is return value of CLIENT_AttachDevComm
typedef void (CALLBACK *fAttachDevCommCB) (LLONG lLoginID, LLONG lAttachHandle, NET_CB_DEVCOMMDATA* pBuf, LDWORD dwUser);

// CLIENT_AttachDevComm() input parameter
typedef struct tagNET_IN_ATTACH_DEVCOMM
{
    DWORD               dwSize;
    int                 nChannel;                        // channel
    fAttachDevCommCB    cbAttachDevComm;                 // callback function
    LDWORD              dwUser;                          // user param
}NET_IN_ATTACH_DEVCOMM;

// CLIENT_AttachDevComm() output parameter
typedef struct tagNET_OUT_ATTACH_DEVCOMM
{
    DWORD          dwSize;
}NET_OUT_ATTACH_DEVCOMM;

// Gets the current equipment serial number, corresponding CLIENT_QueryDevState() Interface's DH_DEVSTATE_GET_COMM_COUNT command parameter 
typedef struct tagNET_GET_COMM_COUNT
{
    DWORD          dwSize;
    int            nChannelCout;                    // Number Of Accessed serial port
}NET_GET_COMM_COUNT;

// CLIENT_ExChangeData  port input  structure 
typedef struct tagNET_IN_EXCHANGEDATA
{
    DWORD           dwSize;
    int             nChannel;                       // serial port no
    BOOL            bFlag;                          // TRUE is to send data and wait for response, FALSE is to collect data
    int             nCollectTime;                   // Serial collection time length(unit:ms)
    int             nSendDataLength;                // send data length, bFlag = TRUE valid
    unsigned char*  pSendData;                      // send data content, bFlag = TRUE valid
}NET_IN_EXCHANGEDATA;

// CLIENT_ExChangeData port output  structure 
typedef struct tagNET_OUT_EXCHANGEDATA
{
    DWORD           dwSize;
}NET_OUT_EXCHANGEDATA;

// PTZ preset
typedef struct tagNET_PTZ_PRESET 
{
    int                     nIndex;                         // no.
    char                    szName[PTZ_PRESET_NAME_LEN];    // name
    char                    szReserve[64];                  // reserve 64 bytes
} NET_PTZ_PRESET;

// PTZ preset list
typedef struct tagNET_PTZ_PRESET_LIST 
{
    DWORD                   dwSize;
    DWORD                   dwMaxPresetNum;                 // preset max
    DWORD                   dwRetPresetNum;                 // return preset quantity
    NET_PTZ_PRESET          *pstuPtzPorsetList;             // preset list(apply according to max quantity),the space appliacation by the user,apply to sizeof(NET_PTZ_PRESET)*dwMaxPresetNum
} NET_PTZ_PRESET_LIST;

// event type EVENT_IVS_TRAFFIC_RESTRICTED_PLATE (limited plate event)corresponding data module description info
typedef struct tagDEV_EVENT_TRAFFIC_RESTRICTED_PLATE
{
    int                     nChannelID;                     // channel no.
    char                    szName[DH_EVENT_NAME_LEN];      // event name
    int                     nTriggerType;                   // TriggerType:, 0 vehicle detector, 1 radar, 2 video
    DWORD                   PTS;                            // time stamp(unit is ms)
    NET_TIME_EX             UTC;                            // event time
    int                     nEventID;                       // event ID
    int                     nSequence;                      // snapshot no., as 3,2,1,1 means snapshot end,0 means abnormal end
    BYTE                    byEventAction;                  // event action, 0 means impulse event,1 means continuous event start,2 means continuous event end;
    BYTE                    byImageIndex;                   // picture no., same time(to second)may have many pictures, from 0
    BYTE                    byReserved1[2];
    int                     nLane;                          // corresponding lane no.
    DH_MSG_OBJECT           stuObject;                      // detect object
    DH_MSG_OBJECT           stuVehicle;                     // vehicle info
    DH_EVENT_FILE_INFO      stuFileInfo;                    // event corresponding file info
    int                     nMark;                          // basic level generated trigger snapshot frame mark
    int                     nFrameSequence;                 // video analysis frame no.
    int                     nSource;                        // video analysis data surce address
    DWORD                   dwSnapFlagMask;                 // snapshot mark (by bit), see NET_RESERVED_COMMON    
    DH_RESOLUTION_INFO      stuResolution;                  // corresponding picture resolution
    DEV_EVENT_TRAFFIC_TRAFFICCAR_INFO stuTrafficCar;        // traffic vehicle info
    NET_GPS_INFO            stuGPSInfo;                     // GPS info ,use in mobile DVR/NVR
    BYTE                    byReserved[984];                // reserve text
    EVENT_COMM_INFO         stCommInfo;                     // public info 
}DEV_EVENT_TRAFFIC_RESTRICTED_PLATE;

// event type EVENT_IVS_TRAFFIC_OVERSTOPLINE (cross line event)corresponding to data module info
typedef struct tagDEV_EVENT_TRAFFIC_OVERSTOPLINE
{
    int                     nChannelID;                     // channel no.
    char                    szName[DH_EVENT_NAME_LEN];      // event name
    int                     nTriggerType;                   // TriggerType: trigger type, 0 detect, 1 radar, 2 video
    DWORD                   PTS;                            // time stamp( unit is ms)
    NET_TIME_EX             UTC;                            // event time
    int                     nEventID;                       // event ID
    int                     nSequence;                      // snapshot no.
    BYTE                    byEventAction;                  // event action, 
    BYTE                    byImageIndex;                   // picture no., within same time, may have more than one picture, start ftom 0
    BYTE                    byReserved1[2];
    int                     nLane;                          // corresponding to lane no.
    DH_MSG_OBJECT           stuObject;                      // detected object
    DH_MSG_OBJECT           stuVehicle;                     // vehicle body info
    DH_EVENT_FILE_INFO      stuFileInfo;                    // event corresponding to text
    int                     nMark;                          // bottom trigger snapshot frame mark
    int                     nFrameSequence;                 // video analysis no.
    int                     nSource;                        // video analysis data source address
    DWORD                   dwSnapFlagMask;                 // snapshot maek, see NET_RESERVED_COMMON    
    DH_RESOLUTION_INFO      stuResolution;                  // corresponding to picture resolution
    DEV_EVENT_TRAFFIC_TRAFFICCAR_INFO stuTrafficCar;        // traffic vehicle info
    int                     nSpeed;                         // vehicle speed,Km/h
    NET_GPS_INFO            stuGPSInfo;                     // GPS info ,use in mobile DVR/NVR
    BYTE                    byReserved[984];                // reserved text
    EVENT_COMM_INFO         stCommInfo;                     // public info 
}DEV_EVENT_TRAFFIC_OVERSTOPLINE;



// event type EVENT_IVS_TRAFFIC_WITHOUT_SAFEBELT(Traffic unfasten seat belt event event )corresponding to data block description info 
typedef struct tagDEV_EVENT_TRAFFIC_WITHOUT_SAFEBELT
{
    int           nChannelID;					// channel no.
    char          szName[DH_EVENT_NAME_LEN];    // event name 
    int           nTriggerType;                 // TriggerType:trigger type , 0 vehicle detector, 1 radar, 2 video
    DWORD         PTS;                          // time stamp(unit is ms)
    NET_TIME_EX   UTC;                          // event occurred time
    int           nEventID;                     // event ID
    int           nSequence;                    // means snaoshot no., as 3,2,1,1 means snapshotend ,0 means abnormal end
    BYTE          byEventAction;                // event  motion , 0 means pulse event ,1 means  continuity  event  start ,2 means  continuity  event end;  BYTE          byReserved1[2];
    BYTE          byImageIndex;                 // picture no., same time(accurate to second)may have multiple pictures , start from 0 
    BYTE          byReserved1[2];
    DH_EVENT_FILE_INFO   stuFileInfo;           // event corresponding to file info 
    int           nLane;                        // corresponding to lane no.
    int           nMark;                        // bottom generated trigger snapshot frame mark
    int           nFrameSequence;               // video analysis frame no.
    int           nSource;                      // video analysis data source address
    DH_MSG_OBJECT      stuObject;               // detection object 
    DH_MSG_OBJECT      stuVehicle;              // body info 
    DEV_EVENT_TRAFFIC_TRAFFICCAR_INFO stuTrafficCar;    // Traffic vehicle info 
    int           nSpeed;                       // vehicle actual speed,Km/h
    NET_SAFEBELT_STATE   emMainSeat;            // main driver seat belt status 
    NET_SAFEBELT_STATE   emSlaveSeat;           // co-drvier seat belt status 
    DWORD         dwSnapFlagMask;               // snapshot mark(by bit), see NET_RESERVED_COMMON  
    DH_RESOLUTION_INFO   stuResolution;         // corresponding to picture resolution
    NET_GPS_INFO        stuGPSInfo;             // GPS info ,use in mobile DVR/NVR
    BYTE          byReserved[984];              // reserved text
    EVENT_COMM_INFO     stCommInfo;             // public info 
}DEV_EVENT_TRAFFIC_WITHOUT_SAFEBELT;

// event type EVENT_IVS_TRAFFIC_DRIVER_SMOKING (driver smoke event )corresponding to data block description info 
typedef struct tagDEV_EVENT_TRAFFIC_DRIVER_SMOKING
{
    int           nChannelID;                   // channel no.
    char          szName[DH_EVENT_NAME_LEN];    // event name 
    int           nTriggerType;                 // TriggerType: trigger type , 0 vehicle detector, 1 radar, 2 video
    DWORD         PTS;                          // time stamp(unit is ms)
    NET_TIME_EX   UTC;                          // event occurred time
    int           nEventID;                     // event ID
    int           nSequence;                    // means snapshot no.,  as 3,2,1,1 means snapshot end,0 means abnormal end
    BYTE          byEventAction;                // event  motion , 0 means pulse  event ,1 means  continuity  event  start ,2 means  continuity  event end;  BYTE          byReserved1[2];
    BYTE          byImageIndex;                 // picture no., same time(accurate to second) may be multiple pictures , start from 0  
    BYTE          byReserved1[2];
    DH_EVENT_FILE_INFO   stuFileInfo;           // event corresponding to file info 
    int           nLane;                        // corresponding to lane no.
    int           nMark;                        // bottom occurred trigger snapshot mark
    int           nFrameSequence;               // video analysis frame no.
    int           nSource;                      // video analysis data source address
    DH_MSG_OBJECT      stuObject;               // detection received object 
    DH_MSG_OBJECT      stuVehicle;              // body info 
    DEV_EVENT_TRAFFIC_TRAFFICCAR_INFO stuTrafficCar;    // Traffic vehicle info 
    int           nSpeed;                       // car actual speed,Km/h
    DWORD         dwSnapFlagMask;               // snapshot mark(by bit), see NET_RESERVED_COMMON  
    DH_RESOLUTION_INFO   stuResolution;         // corresponding to picture resolution
    EVENT_COMM_INFO     stCommInfo;             // public info 
    NET_GPS_INFO        stuGPSInfo;             // GPS info ,use in mobile DVR/NVR
    BYTE          byReserved[984];              // reserve text
}DEV_EVENT_TRAFFIC_DRIVER_SMOKING;

// event type EVENT_IVS_TRAFFIC_DRIVER_CALLING(driver call event )corresponding to data block description info 
typedef struct tagDEV_EVENT_TRAFFIC_DRIVER_CALLING
{
    int           nChannelID;                   // channel no.
    char          szName[DH_EVENT_NAME_LEN];    // event name 
    int           nTriggerType;                 // TriggerType: trigger type , 0 vehicle detector, 1 radar, 2 video
    DWORD         PTS;                          // time stamp(unit is ms)
    NET_TIME_EX   UTC;                          // event occurred time
    int           nEventID;                     // event ID
    int           nSequence;                    // means snapshot no., as 3,2,1,1 means snapshot end,0 means abnormal end
    BYTE          byEventAction;                // event  motion , 0 means pulse  event ,1 means  continuity  event  start ,2 means  continuity  event end;  BYTE          byReserved1[2];
    BYTE          byImageIndex;                 // picture no., same time (accurate to second) maybe multiple picture , start from 0  
    BYTE          byReserved1[2];
    DH_EVENT_FILE_INFO   stuFileInfo;           // event corresponding to file info 
    int           nLane;                        // corresponding to lane no.
    int           nMark;                        // bottom trigger snapshot frame mark
    int           nFrameSequence;               // video analysis frame no.
    int           nSource;                      // video analysis data source address
    DH_MSG_OBJECT      stuObject;               // detection received object 
    DH_MSG_OBJECT      stuVehicle;              // body info 
    DEV_EVENT_TRAFFIC_TRAFFICCAR_INFO stuTrafficCar;    // Traffic vehicle info 
    int           nSpeed;                       // vehicle actual speed,Km/h
    DWORD         dwSnapFlagMask;               // snapshot mark(by bit), see NET_RESERVED_COMMON  
    DH_RESOLUTION_INFO   stuResolution;         // corresponding to picture resolution
    EVENT_COMM_INFO     stCommInfo;             // public info 
    NET_GPS_INFO        stuGPSInfo;             // GPS info ,use in mobile DVR/NVR
    BYTE                byReserved[984];        // reserve text
}DEV_EVENT_TRAFFIC_DRIVER_CALLING;

// event type EVENT_IVS_OBJECT_DETECTION(object feature detection event )corresponding to data description info 
typedef struct tagDEV_EVENT_TRAFFIC_OBJECT_DETECTION
{
    int	        nChannelID;                     // channel no.
    char        szName[128];                    // event name 
    char        bReserved1[4];                  // field align
    double      PTS;                            // time stamp(unit is ms)
    NET_TIME_EX UTC;                        	// event occurred time
    int         nEventID;                       // event ID
    DH_EVENT_FILE_INFO stuFileInfo;             // event corresponding to file info 
    BYTE        bEventAction;                   // event  motion , 0 means pulse event ,1 means  continuity  event  start ,2 means  continuity  event end;
    BYTE        byReserved[2];                  // reserved text
    BYTE        byImageIndex;                   // picture no., same time(accurate to second)may be multiple picture , start from 0 
    DWORD       dwSnapFlagMask;                 // snapshot mark(by bit), see NET_RESERVED_COMMON  
    int         nSourceIndex;                   // event source device index,-1 means data invalid
    char        szSourceDevice[MAX_PATH];       // event source device SN, field does not exist or null means local device 
    unsigned int    nOccurrenceCount;           // event trigger accumilated times 
    int         nObjectNum;                     // object info number
    DH_MSG_OBJECT_EX  *pstObjectInfo;           // object info data 
    EVENT_INTELLI_COMM_INFO     stuIntelliCommInfo;     // public info 
    BYTE        byReserved2[892];              	// reserved text
}DEV_EVENT_TRAFFIC_OBJECT_DETECTION;

// redundant power abnormal alarm
typedef struct tagALARM_POWER_ABNORMAL_INFO
{
    DWORD                   dwSize;
    int                     nChannelID;                     // channel no.
    NET_TIME                stuTime;                        // time
    DWORD                   dwAction;                       // event ation, 0:Start, 1:Stop
}ALARM_POWER_ABNORMAL_INFO;

// Tri-status enable
typedef enum tagNET_THREE_STATE_ENABLE
{
    EM_STATE_ENABLE_INVALID = 0,            // invalid
    EM_STATE_ENABLE_OFF,                    // off
    EM_STATE_ENABLE_ON,                     // on
} NET_THREE_STATE_ENABLE;

// config enable change reported event
typedef struct tagCONFIG_ENABLE_CHANGE_INFO
{
    DWORD                   dwSize;
    int                     nChannelID;                     // chanle no., start from 0
    NET_THREE_STATE_ENABLE  emMotionDetect;                 // motion detect,     0: invalid , 1: off, 2: on
	NET_THREE_STATE_ENABLE  emLossDetect;                   // video loss enable, 0: invalid , 1: off, 2: on	NET_THREE_STATE_ENABLE  emBlindDetect;                  // video tempering enable, 0: invalid , 1: off, 2: on	NET_THREE_STATE_ENABLE  emAlarmLocal;                   // external alarm enable, 0: invalid , 1: off, 2: on
	NET_THREE_STATE_ENABLE  emBlindDetect;                  // blind detect enable, 0: invalid , 1: off, 2: on
	NET_THREE_STATE_ENABLE  emAlarmLocal;                   // alarm local enable, 0: invalid , 1: off, 2: on
}ALARM_CONFIG_ENABLE_CHANGE_INFO;

// ITC HWS000 IT TYPE
typedef enum tagEM_ITC_HWS000_IT_TYPE
{
    EM_ITC_HWS000_IT_TYPE_UNKNOWN,      // unknown
    EM_ITC_HWS000_IT_TYPE_HIGH_SPEED,   // high speed
    EM_ITC_HWS000_IT_TYPE_LOW_SPEED,    // low speed
}EM_ITC_HWS000_IT_TYPE;

// ITC HWS000 RS TYPE
typedef enum tagEM_ITC_HWS000_RS_TYPE
{
    EM_ITC_HWS000_RS_TYPE_UNKNOWN,      // unknown
    EM_ITC_HWS000_RS_TYPE_ONLINE,       // online
    EM_ITC_HWS000_RS_TYPE_OFFLINE,      // offline
}EM_ITC_HWS000_RS_TYPE;

// ITC HWS000 FS TYPE
typedef enum tagEM_ITC_HWS000_FS_TYPE
{
    EM_ITC_HWS000_FS_TYPE_UNKNOWN,     // unknown
    EM_ITC_HWS000_FS_TYPE_ONLINE,      // online
    EM_ITC_HWS000_FS_TYPE_OFFLINE,     // offline
}EM_ITC_HWS000_FS_TYPE;

// the alarm event for ITC_HWS000
typedef struct tagALARM_ITC_HWS000
{
    DWORD dwSize;
    EM_ITC_HWS000_IT_TYPE emItType;   // the type of breaking rules
    EM_ITC_HWS000_RS_TYPE emRsType;   // the type of radar state
    EM_ITC_HWS000_FS_TYPE emFsType;   // the type of flashlight state
    int nOcNum;                       // the count of high speed
    int nLcNum;                       // the count of low speed
    int nAcNum;                       // the count of breaking rules
    int nFcNum;                       // the count of light flash
}ALARM_ITC_HWS000;

// car status
typedef enum tagEM_CAR_COME_STATUS
{
    EM_CAR_COME_STATUS_UNKNOWN,                         //unknown
    EM_CAR_COME_STATUS_COME_IN,                         //come in
    EM_CAR_COME_STATUS_COME_OUT,                        //come out
}EM_CAR_COME_STATUS;

// the event of telephonenumber check
typedef struct tagALARM_TELEPHONE_CHECK_INFO
{
    DWORD                       dwSize; 
    EM_CAR_COME_STATUS          emCarStatus;               //car status
    char                        szTelephoneNum[32];        //telephone number
}ALARM_TELEPHONE_CHECK_INFO;

#define UPDATE_VERSION_LEN        64

// get device upgrade status corresponding structure 
typedef struct tagDHDEV_UPGRADE_STATE_INFO
{
    int                  nState;                            // 0:None- no update found, default status; 1:Regular-general upgrade; 
                                                            // 2:Emergency-forced; 3:Upgrading-upgrading
    
    char                szOldVersion[UPDATE_VERSION_LEN];   // old version
    char                szNewVersion[UPDATE_VERSION_LEN];   // new version
    DWORD               dwProgress;                         // upgrading progress
    int                 reserved[256];                      // reserve text
}DHDEV_UPGRADE_STATE_INFO;

// Warning line intrusion direction 
typedef enum tagNET_CROSSLINE_DIRECTION_INFO
{
    EM_CROSSLINE_DIRECTION_UNKNOW = 0 , 
    EM_CROSSLINE_DIRECTION_LEFT2RIGHT ,   //From left to right
    EM_CROSSLINE_DIRECTION_RIGHT2LEFT ,   //From right to left
    EM_CROSSLINE_DIRECTION_ANY        ,   
}NET_CROSSLINE_DIRECTION_INFO;

// Warning line event (Corresponding to event  DH_EVENT_CROSSLINE_DETECTION)
typedef struct tagALARM_EVENT_CROSSLINE_INFO
{
    DWORD                           dwSize;                         
    int					            nChannelID;						// Channel No.
    double				            PTS;							// Time stamp (Unit is ms)
    NET_TIME_EX			            UTC;							// Event occurrence time 
    int					            nEventID;						// Event ID
    int                             nEventAction;                   // Event operation. 0=pulse event.1=continious event begin. 2=continuous event stop

    NET_CROSSLINE_DIRECTION_INFO    emCrossDirection;               // Intrusion direction 
    int                             nOccurrenceCount;               // Triggered amount 
    int                             nLevel;                         // Event type
}ALARM_EVENT_CROSSLINE_INFO;

// Move detection event(Corresponding to event  DH_ALARM_MOVEDETECTION)
typedef struct tagALARM_MOVE_DETECTION_INFO
{
	int                             nAction;                   		// Event operation. 0=pulse event.1=start. 2=stop
	int					            nChannelID;						// Channel No.
    double				            dbPTS;							// Time stamp (Unit is ms)
    NET_TIME_EX			            stuTime;						// Event occurrence time 
    int					            nEventID;						// Event ID

	int								nCount;							// Triggered amount 
	BYTE                 			byReserved[1024];               // reserve text
} ALARM_MOVE_DETECTION_INFO;

// WanderDetection event(Corresponding to event  DH_ALARM_WANDERDETECTION)
typedef struct tagALARM_WANDERDETECTION_INFO
{
	int                             nAction;                   		// Event operation. 0=pulse event.1=start. 2=stop
	int					            nChannelID;						// Channel No.
    double				            dbPTS;							// Time stamp (Unit is ms)
    NET_TIME_EX			            stuTime;						// Event occurrence time 
    int					            nEventID;						// Event ID

	int 							nCount;							// Triggered amount 
	int								nLevel;							// event level
	short							nPreserID;						// Event triggered preset period, starting from 1 (no unknown)
	char							szPresetName[64];				// Preset name for event triggered
	BYTE                 			byReserved[958];                // reserve text
} ALARM_WANDERDETECTION_INFO;

// Cross fencedetection event(Corresponding to event  DH_ALARM_CROSSFENCEDETECTION)
typedef struct tagALARM_CROSSFENCEDETECTION_INFO
{
	int                             nAction;                   		// Event operation. 0=pulse event.1=start. 2=stop
	int					            nChannelID;						// Channel No.
    double				            dbPTS;							// Time stamp (Unit is ms)
    NET_TIME_EX			            stuTime;						// Event occurrence time 
    int					            nEventID;						// Event ID

	NET_CROSSLINE_DIRECTION_INFO	emCrossDirection;               // Intrusion direction 
	int 							nCount;							// Triggered amount 
	BYTE                			byReserved[1024];   			// reserve text
} ALARM_CROSSFENCEDETECTION_INFO;

// red light event(Corresponding to event  ALARM_TRAFFIC_PEDESTRIAN_RUN_REDLIGHT_DETECTION_INFO)
typedef struct tagALARM_TRAFFIC_PEDESTRIAN_RUN_REDLIGHT_DETECTION_INFO
{
	int                             nAction;                   		// Event operation. 0=pulse event.1=start. 2=stop
	int					            nChannelID;						// Channel No.
    double				            dbPTS;							// Time stamp (Unit is ms)
    NET_TIME_EX			            stuTime;						// Event occurrence time 
    int					            nEventID;						// Event ID
	BYTE                			byReserved[1024];   			// reserve text
} ALARM_TRAFFIC_PEDESTRIAN_RUN_REDLIGHT_DETECTION_INFO;

//fight detection event(Corresponding to event  DH_ALARM_FIGHTDETECTION)
typedef struct tagNET_ALARM_FIGHTDETECTION
{	
	int                             nAction;                   		// Event operation, 0:Start, 1:Stop
	int					            nChannelID;						// Channel No., start from 0
    double				            dbPTS;							// Time stamp (Unit is ms)
    NET_TIME_EX			            stuTime;						// Event occurrence time   
	BYTE                			byReserved[1024];   			// reserve text
}NET_ALARM_FIGHTDETECTION;

#define VTO_BUILDING_LEN			8
#define VTO_UNIT_LEN				8

//VTO access identify(Corresponding to event DH_ALARM_ACCESSIDENTIFY)
typedef struct tagNET_ALARM_ACCESSIDENTIFY_INFO
{	
    NET_TIME_EX			            stuTime;						 // Event occurrence time     
	char 							szVTOBuilding[VTO_BUILDING_LEN]; // VTO building no
	char 							szVTOUnit[VTO_UNIT_LEN];		 // VTO uinit no 
	BYTE                			byReserved[1024];   			 // reserve text
}NET_ALARM_ACCESSIDENTIFY_INFO;

// call no answer event, under directly connected(Corresponding to event DH_ALARM_CALL_NO_ANSWERED)
typedef struct tagNET_ALARM_CALL_NO_ANSWERED_INFO
{
	NET_TIME_EX			            stuTime;						 // Event occurrence time 
	char 							szCallID[MAX_CALL_ID_LEN];		 // Call ID
	BYTE                			byReserved[1024];   			 // reserve text
} NET_ALARM_CALL_NO_ANSWERED_INFO;

// parking detection event(Corresponding to event  DH_ALARM_PARKINGDETECTION)
typedef struct tagALARM_PARKINGDETECTION_INFO
{
	int                             nAction;                   		// Event operation. 0=pulse event.1=start. 2=stop
	int					            nChannelID;						// Channel No.
    double				            dbPTS;							// Time stamp (Unit is ms)
    NET_TIME_EX			            stuTime;						// Event occurrence time 
    int					            nEventID;						// Event ID

	int 							nCount;							// Triggered amount 
	BYTE                			byReserved[1024];   			// reserve text
} ALARM_PARKINGDETECTION_INFO;

// Rioter detection event(Corresponding to event  DH_ALARM_RIOTERDETECTION)
typedef struct tagALARM_RIOTERDETECTION_INFO
{
	int                             nAction;                   		// Event operation. 0=pulse event.1=start. 2=stop
	int					            nChannelID;						// Channel No.
    double				            dbPTS;							// Time stamp (Unit is ms)
    NET_TIME_EX			            stuTime;						// Event occurrence time 
    int					            nEventID;						// Event ID

	int 							nCount;							// Triggered amount 
	BYTE                			byReserved[1024];   			// reserve text
} ALARM_RIOTERDETECTION_INFO;

//  traffic parking event(Corresponding to event  DH_ALARM_TRAFFIC_PARKING)
typedef struct tagALARM_TRAFFIC_PARKING_INFO 
{
	int                             nAction;                   		// Event operation. 0=pulse event.1=start. 2=stop
	int					            nChannelID;						// Channel No.
    double				            dbPTS;							// Time stamp (Unit is ms)
    NET_TIME_EX			            stuTime;						// Event occurrence time 
    int					            nEventID;						// Event ID

	BYTE                			byReserved[1024];   			// reserve text
} ALARM_TRAFFIC_PARKING_INFO;


// traffic jam event(Corresponding to event  DH_ALARM_TRAFFIC_JAM)
typedef struct tagALARM_TRAFFIC_JAM_INFO 
{
	int                             nAction;                   		// Event operation. 0=pulse event.1=start. 2=stop
	int					            nChannelID;						// Channel No.
    double				            dbPTS;							// Time stamp (Unit is ms)
    NET_TIME_EX			            stuTime;						// Event occurrence time 
    int					            nEventID;						// Event ID

	BYTE                			byReserved[1024];   			// reserve text
} ALARM_TRAFFIC_JAM_INFO;

// traffic pedestrain event(Corresponding to event  DH_ALARM_TRAFFIC_PEDESTRAIN)
typedef struct tagALARM_TRAFFIC_PEDESTRAIN_INFO
{
	int                             nAction;                   		// Event operation. 0=pulse event.1=start. 2=stop
	int					            nChannelID;						// Channel No.
    double				            dbPTS;							// Time stamp (Unit is ms)
    NET_TIME_EX			            stuTime;						// Event occurrence time 
    int					            nEventID;						// Event ID

	BYTE                			byReserved[1024];   			// reserve text
} ALARM_TRAFFIC_PEDESTRAIN_INFO;

// traffic throw event(Corresponding to event  DH_ALARM_TRAFFIC_THROW)
typedef struct tagALARM_TRAFFIC_THROW_INFO
{
	int                             nAction;                   		// Event operation. 0=pulse event.1=start. 2=stop
	int					            nChannelID;						// Channel No.
    double				            dbPTS;							// Time stamp (Unit is ms)
    NET_TIME_EX			            stuTime;						// Event occurrence time 
    int					            nEventID;						// Event ID

	BYTE                			byReserved[1024];   			// reserve text
} ALARM_TRAFFIC_THROW_INFO;

// traffic retrograde event(Corresponding to event  DH_ALARM_TRAFFIC_RETROGRADE)
typedef struct tagALARM_TRAFFIC_RETROGRADE_INFO
{
	int                             nAction;                   		// Event operation. 0=pulse event.1=start. 2=stop
	int					            nChannelID;						// Channel No.
    double				            dbPTS;							// Time stamp (Unit is ms)
    NET_TIME_EX			            stuTime;						// Event occurrence time 
    int					            nEventID;						// Event ID
 
	BYTE                			byReserved[1024];   			// reserve text
} ALARM_TRAFFIC_RETROGRADE_INFO;

// traffic overspeed event(Corresponding to event  DH_ALARM_TRAFFIC_OVERSPEED)
typedef struct tagALARM_TRAFFIC_OVERSPEED_INFO 
{
	int                             nAction;                   		// Event operation. 0=pulse event.1=start. 2=stop
	int					            nChannelID;						// Channel No.
    double				            dbPTS;							// Time stamp (Unit is ms)
    NET_TIME_EX			            stuTime;						// Event occurrence time 
    int					            nEventID;						// Event ID

	int								nSpeed;							// car speed
	int								nSpeedUpperLimit;				// speed upper limit
	int								nSpeedLowerLimit;				// speed lower limit
	BYTE                			byReserved[1024];   			// reserve text
} ALARM_TRAFFIC_OVERSPEED_INFO;

// traffic underspeed event(Corresponding to event  DH_ALARM_TRAFFIC_UNDERSPEED)
typedef struct tagALARM_TRAFFIC_UNDERSPEED_INFO 
{
	int                             nAction;                   		// Event operation. 0=pulse event.1=start. 2=stop
	int					            nChannelID;						// Channel No.
    double				            dbPTS;							// Time stamp (Unit is ms)
    NET_TIME_EX			            stuTime;						// Event occurrence time 
    int					            nEventID;						// Event ID

	int								nSpeed;							// car speed
	int								nSpeedUpperLimit;				// speed upper limit
	int								nSpeedLowerLimit;				// speed lower limit
	BYTE                			byReserved[1024];   			// reserve text
} ALARM_TRAFFIC_UNDERSPEED_INFO;


//retrogade detection event (Corresponding to event DH_ALARM_RETROGRADE_DETECTION)
typedef struct tagALARM_RETROGRADE_DETECTION_INFO
{
	int                             nAction;                   		// Event operation. 0=pulse event.1=start. 2=stop
	int					            nChannelID;						// Channel No.
    double				            dbPTS;							// Time stamp (Unit is ms)
    NET_TIME_EX			            stuTime;						// Event occurrence time 
    int					            nEventID;						// Event ID
	int 							nCount;							// Triggered amount 
	BYTE                			byReserved[1024];   			// reserve text
} ALARM_RETROGRADE_DETECTION_INFO;




// AIO App config(Corresponding to event DH_ALARM_AIO_APP_CONFIG_EVENT)
typedef struct tagALARM_AIO_APP_CONFIG_EVENT_INFO
{
	NET_TIME_EX			            stuTime;							// Event occurrence time 
	char 							szAddress[DH_MAX_IPADDR_LEN];		// ip address
	BYTE                			byReserved[1024];   				// reserve text
}ALARM_AIO_APP_CONFIG_EVENT_INFO;


//DH_ALARM_BOX(Corresponding to DH_ALARM_BOX)
typedef struct tagALARM_BOX_INFO
{
	int                             nAction;                   		// Event operation. 1=start. 2=stop
    double				            dbPTS;							// Time stamp (Unit is ms)
    NET_TIME_EX			            stuTime;						// Event occurrence time 
    int					            nEventID;						// 
	
	int 							nAlarmBoxIndex;					// alarm box index, start from 0
	int								nAlarmIndex;					// alarm box inside channel, start from 0
	BYTE                			byReserved[1024];   			// reserved
} ALARM_BOX_INFO;

// under voltage alarm ( Corresponding to event DH_ALARM_UNDER_VOLTAGE )
typedef struct tagALARM_UNDER_VOLTAGE_INFO
{
    int                             nAction;                   		// Event operation. 1=start. 2=stop
    int					            nChannelID;						// Channel
    double				            dbPTS;							// Time stamp (Unit is ms)
    NET_TIME_EX			            stuTime;						// Event occurrence time 
    int					            nEventID;						// Event ID
    BYTE                			byReserved[512];       			// Reserved
}ALARM_UNDER_VOLTAGE_INFO;

// over voltage alarm ( Corresponding to event DH_ALARM_OVER_VOLTAGE )
typedef struct tagALARM_OVER_VOLTAGE_INFO
{
    int                             nAction;                   		// Event operation. 1=start. 2=stop
    int					            nChannelID;						// Channel
    double				            dbPTS;							// Time stamp (Unit is ms)
    NET_TIME_EX			            stuTime;						// Event occurrence time 
    int					            nEventID;						// Event ID
    BYTE                			byReserved[512];       			// Reserved  
}ALARM_OVER_VOLTAGE_INFO;

// alarm of tired physiological ( Corresponding to event DH_ALARM_TIRED_PHYSIOLOGICAL )
typedef struct tagALARM_TIRED_PHYSIOLOGICAL
{
    int                             nAction;                   		// Event operation. 0=pulse event. 1=start. 2=stop
    int					            nChannelID;						// Channel
    double				            dbPTS;							// Time stamp (Unit is ms)
    NET_TIME_EX			            stuTime;						// Event occurrence time 
    int					            nEventID;						// Event ID

	NET_GPS_STATUS_INFO				stuGPSStatus;					// GPS info
    BYTE                			byReserved[512];       			// Reserved    
} ALARM_TIRED_PHYSIOLOGICAL;


// alarm of calling when driver ( Corresponding to event DH_ALARM_CALLING_WHEN_DRIVING )
typedef struct tagALARM_CALLING_WHEN_DRIVING
{
    int                             nAction;                   		// Event operation. 0=pulse event. 1=start. 2=stop
    int					            nChannelID;						// Channel
    double				            dbPTS;							// Time stamp (Unit is ms)
    NET_TIME_EX			            stuTime;						// Event occurrence time 
    int					            nEventID;						// Event ID

	NET_GPS_STATUS_INFO				stuGPSStatus;					// GPS info
    BYTE                			byReserved[512];       			// Reserved     
} ALARM_CALLING_WHEN_DRIVING;


// alarm of traffic driver smoking ( Corresponding to event DH_ALARM_TRAFFIC_DRIVER_SMOKING )
typedef struct tagALARM_TRAFFIC_DRIVER_SMOKING
{
    int                             nAction;                   		// Event operation. 0=pulse event. 1=start. 2=stop
    int					            nChannelID;						// Channel
    double				            dbPTS;							// Time stamp (Unit is ms)
    NET_TIME_EX			            stuTime;						// Event occurrence time 
    int					            nEventID;						// Event ID

	NET_GPS_STATUS_INFO				stuGPSStatus;					// GPS info
    BYTE                			byReserved[512];       			// Reserved      
} ALARM_TRAFFIC_DRIVER_SMOKING;


// alarm of traffic driver lower head ( Corresponding to event DH_ALARM_TRAFFIC_DRIVER_LOWER_HEAD )
typedef struct tagALARM_TRAFFIC_DRIVER_LOWER_HEAD
{
    int                             nAction;                   		// Event operation. 0=pulse event. 1=start. 2=stop
    int					            nChannelID;						// Channel
    double				            dbPTS;							// Time stamp (Unit is ms)
    NET_TIME_EX			            stuTime;						// Event occurrence time 
    int					            nEventID;						// Event ID

	NET_GPS_STATUS_INFO				stuGPSStatus;					// GPS info
    BYTE                			byReserved[512];       			// Reserved      
} ALARM_TRAFFIC_DRIVER_LOWER_HEAD;


// alarm of traffic driver look around ( Corresponding to event DH_ALARM_TRAFFIC_DRIVER_LOOK_AROUND )
typedef struct tagALARM_TRAFFIC_DRIVER_LOOK_AROUND
{
    int                             nAction;                   		// Event operation. 0=pulse event. 1=start. 2=stop
    int					            nChannelID;						// Channel
    double				            dbPTS;							// Time stamp (Unit is ms)
    NET_TIME_EX			            stuTime;						// Event occurrence time 
    int					            nEventID;						// Event ID

	NET_GPS_STATUS_INFO				stuGPSStatus;					// GPS info
    BYTE                			byReserved[512];       			// Reserved       
} ALARM_TRAFFIC_DRIVER_LOOK_AROUND;


// alarm of traffic driver leave post ( Corresponding to event DH_ALARM_TRAFFIC_DRIVER_LEAVE_POST )
typedef struct tagALARM_TRAFFIC_DRIVER_LEAVE_POST
{
    int                             nAction;                   		// Event operation. 0=pulse event. 1=start. 2=stop
    int					            nChannelID;						// Channel
    double				            dbPTS;							// Time stamp (Unit is ms)
    NET_TIME_EX			            stuTime;						// Event occurrence time 
    int					            nEventID;						// Event ID

	NET_GPS_STATUS_INFO				stuGPSStatus;					// GPS info
    BYTE                			byReserved[512];       			// Reserved     
} ALARM_TRAFFIC_DRIVER_LEAVE_POST;


// alarm of traffic driver yawn ( Corresponding to event DH_ALARM_TRAFFIC_DRIVER_YAWN )
typedef struct tagALARM_TRAFFIC_DRIVER_YAWN
{
    int                             nAction;                   		// Event operation. 0=pulse event. 1=start. 2=stop
    int					            nChannelID;						// Channel
    double				            dbPTS;							// Time stamp (Unit is ms)
    NET_TIME_EX			            stuTime;						// Event occurrence time 
    int					            nEventID;						// Event ID

	NET_GPS_STATUS_INFO				stuGPSStatus;					// GPS info
    BYTE                			byReserved[512];       			// Reserved       
} ALARM_TRAFFIC_DRIVER_YAWN;

// alarm of the connection status between the device and the base ( Corresponding to event DH_ALARM_MPTBASE_CONNECT )
typedef struct tagALARM_MPTBASE_CONNECT
{
	int 							nAction;						// Event operation. 1=start. 2=stop
	BYTE                			byReserved[512];       			// Reserved   
} ALARM_MPTBASE_CONNECT;

// the reason of shut down
typedef enum tagEM_SHUTDOWN_REASON_TYPE
{
	EM_SHUTDOWN_REASON_UNKNOWN, 				// unknown
	EM_SHUTDOWN_REASON_LOWERPOWER,				// lower power
	EM_SHUTDOWN_REASON_LOSTPOWER,				// lost power
	EM_SHUTDOWN_REASON_CRASH,					// system crash
} EM_SHUTDOWN_REASON_TYPE;

// alarm ot the latest status of shut down ( Corresponding to event DH_ALARM_LATEST_SHUTDOWN)
typedef struct tagALARM_LATEST_SHUTDOWN
{
	EM_SHUTDOWN_REASON_TYPE			emShutdownReason;				// the reason of shut down
	BYTE                			byReserved[512];       			// reserved 
} ALARM_LATEST_SHUTDOWN;

// Vehicle position alarm event(DH_ALARM_TRAFFIC_VEHICLE_POSITION)
typedef struct tagALARM_TRAFFIC_VEHICLE_POSITION
{
	int             nAction;                   						// Event operation. 0=pulse event. 1=start. 2=stop
	char			szEventName[DH_COMMON_STRING_128];				// Event Name
	unsigned int	nObjectID;										// Object ID
	char			szPlateNumber[DH_COMMON_STRING_128];			// car plate Number
	int				nPosition;										// The location of the license plate and bayonet(0:capture the distance between the bayonet and the license plate, >0:cross the snap line; <0:not over the capture line)
	BYTE			byReserved[512];       							// Reserved  
} ALARM_TRAFFIC_VEHICLE_POSITION;

// alarm of auto inspection (Corresponding to event DH_ALARM_AUTO_INSPECTION) 
typedef struct tagALARM_AUTO_INSPECTION
{
	double				            dbPTS;							// Time stamp (Unit is ms)
	NET_TIME_EX			            stuTime;						// Event occurrence time 
	int					            nEventID;						// Event ID

	NET_TIME_EX						stuStartTime;					// start time of auto inspection
	BOOL							bCheckResult;					// Device check result,FALSE means fail,TRUE means success
	BYTE							byReserved[1024];				// Reserved
}ALARM_AUTO_INSPECTION;

// Event type
typedef enum tagEM_FACE_VERIFICATION_ACCESS_SNAP_TYPE
{
	EM_FACE_VERIFICATION_ACCESS_SNAP_TYPE_UNKNOWN,			// unknown
	EM_FACE_VERIFICATION_ACCESS_SNAP_TYPE_ENTRY,			// entry
	EM_FACE_VERIFICATION_ACCESS_SNAP_TYPE_EXIT,				// exit
}EM_FACE_VERIFICATION_ACCESS_SNAP_TYPE;

// Person info
typedef struct tagNET_PERSON_INFO
{
	BOOL			bCheckResult;					// Facial check result, TRUE means success, FALSE means fail
	char			szName[MAX_NAME_LENGTH];		// Name, This field is valid Only when bCheckResult is TRUE 
	char			szSnapURL[MAX_SNAP_URL_LEN];    // Face snap URL
	BYTE			byReserved[512];				// Reserved
}NET_PERSON_INFO;

// Good info
typedef struct tagNET_GOOD_INFO
{
	char			szCode[MAX_CODE_LEN];			    // Good code number
	char			szSnapURL[MAX_SNAP_URL_LEN];		// Good snap URL
	BYTE			byReserved[512];				    // Reserved
}NET_GOOD_INFO;

// operation type
typedef enum tagEM_OPERATION_TYPE
{
	EM_OPERATION_TYPE_UNKNOWN,					// unknown
	EM_OPERATION_TYPE_FETCH,                    // fetch good
	EM_OPERATION_TYPE_STORE,					// store good
	EM_OPERATION_TYPE_EXCHANGE,					// exchange good
	EM_OPERATION_TYPE_CHECK,					// check good				
}EM_OPERATION_TYPE;

// alarm of face verification access snap(Corresponding to event DH_ALARM_FACE_VERIFICATION_ACCESS_SNAP)
typedef struct tagALARM_FACE_VERIFICATION_ACCESS_SNAP_INFO
{
	double										dbPTS;									// Time stamp (Unit is ms)
	NET_TIME_EX									stuTime;								// Event occurrence time 
	int											nEventID;								// Event ID

	int											nRecNo;									// Record number, it is the same to record number of AccessControl event	
	EM_FACE_VERIFICATION_ACCESS_SNAP_TYPE       emEventType;							// Event type
	NET_PERSON_INFO								stuPersonInfo[MAX_PERSON_INFO_NUM];		// Person info
	int											nRetPersonInfoNum;						// return num of person info
	NET_GOOD_INFO								stuGoodInfo[MAX_GOOD_INFO_NUM];			// Good info
	int											nRetGoodInfoNum;						// return num of good info
	EM_OPERATION_TYPE							emOperation;							// operation type
	BYTE										byReserved[1020];						// Reserved
}ALARM_FACE_VERIFICATION_ACCESS_SNAP_INFO;

// alarm of video blind, (Corresponding to event DH_ALARM_VIDEOBLIND)
typedef struct  tagALARM_VIDEO_BLIND_INFO
{
	int                             nAction;                   		// Event operation. 1=start. 2=stop
    int					            nChannelID;						// channel
    double				            dbPTS;							// Time stamp (Unit is ms)
    NET_TIME_EX			            stuTime;						// Event occurrence time 
    int					            nEventID;						// Event ID

    BYTE                			byReserved[512];       			// Reserved  
} ALARM_VIDEO_BLIND_INFO;

// alarm of driver not confirm, (Corresponding to event DH_ALARM_DRIVER_NOTCONFIRM)
typedef struct tagALARM_DRIVER_NOTCONFIRM_INFO
{
	int                             nAction;                   		// Event operation. 1=start. 2=stop
	NET_TIME_EX			            stuTime;						// Event occurrence time 
    double				            dbPTS;							// Time stamp (Unit is ms)
    BYTE                			byReserved[512];       			// Reserved  
} ALARM_DRIVER_NOTCONFIRM_INFO;

// alarm of violate, (Corresponding to event DH_ALARM_VIOLATE_NO_FLY_TIME)
typedef struct tagALARM_VIOLATE_NO_FLY_TIME_INFO
{
	int                             nAction;                   		// Event operation. 1=start. 2=stop  
    NET_TIME_EX			            stuTime;						// Event occurrence time 
    double				            dbPTS;							// Time stamp (Unit is ms)

    BYTE                			byReserved[512];       			// Reserved   
}ALARM_VIOLATE_NO_FLY_TIME_INFO;

//alarm box alarm ( Corresponding to event DH_ALARM_BOX_ALARM )
typedef struct tagALARM_BOX_ALARM_INFO
{
    int                             nAction;                   		    // Event operation. 1=start. 2=stop
    int					            nChannelID;						    // Channel id
    double				            dbPTS;							    // Time stamp (Unit is ms)
    NET_TIME_EX			            stuTime;						    // Event occurrence time
    int					            nEventID;						    // Event id


    char                            szAlarmChannelName[DH_COMMON_STRING_64];   // Alarm channel name
    BYTE                			byReserved[1024];   			    // Reserved
}ALARM_BOX_ALARM_INFO;

// alarm of face info collect. (Corresponding to event DH_ALARM_FACEINFO_COLLECT)
typedef struct tagALARM_FACEINFO_COLLECT_INFO
{
	
	int                             nAction;                   		// Event operation. 1=start. 2=stop
    NET_TIME_EX			            stuTime;						//  Event occurrence time 
	double				            dbPTS;							// Time stamp (Unit is ms)
	char							szUserID[DH_MAX_USERID_LEN];	// user ID
	BYTE                			byReserved[512];       			// Reserved  
} ALARM_FACEINFO_COLLECT_INFO;

// alarm of high speed, (Corresponding to event DH_ALARM_HIGH_SPEED)
typedef struct tagALARM_HIGH_SPEED_INFO
{
	int                             nAction;                   		// Event operation. 1=start. 2=stop
    NET_TIME_EX			            stuTime;						// Event occurrence time 
	double				            dbPTS;							// Time stamp (Unit is ms)
    NET_GPS_STATUS_INFO 			stGPSStatusInfo;                // GPS info
	int								nSpeedLimit;					// limit speed, km/h
	int								nCurSpeed;						// current speed, km/h
	BYTE                			byReserved[512];       			// Reserved      
} ALARM_HIGH_SPEED_INFO;

//  alarm of driver video loss, (Corresponding to event DH_ALARM_VIDEO_LOSS)
typedef struct tagALARM_VIDEO_LOSS_INFO
{
	int                             nAction;                   		// Event operation. 1=start. 2=stop
	int					            nChannelID;						// channel ID
	double				            dbPTS;							// Time stamp (Unit is ms)
	BYTE							byReserved1[4];					// alagin
	NET_TIME_EX			            stuTime;						// Event occurrence time 
	BYTE                			byReserved[512];       			// Reserved    
} ALARM_VIDEO_LOSS_INFO;

#define  VTS_STATE_LONGNUMBER_LEN	24
// VTS state
typedef enum tagVTS_STATE
{
    EM_VTS_STATE_UNKNOWN = 0	, 
	EM_VTS_STATE_BUSY			,								// "Busy" 
	EM_VTS_STATE_IDLE			,								// "Idle" 
}VTS_STATE;

// VTS state update
typedef struct tagALARM_VTSTATE_UPDATE_INFO
{
	int					            nChannelID;						// Channel No.
    NET_TIME			            stuTime;						// Event occurrence time 
	VTS_STATE						emState;						// device state
	char							szLongNum[VTS_STATE_LONGNUMBER_LEN];	// long nember
	BYTE                			byReserved[1024];   			// reserve text
} ALARM_VTSTATE_UPDATE_INFO;

//video talk path ( Corresponding to event DH_ALARM_VIDEO_TALK_PATH )
typedef struct tagALARM_VIDEO_TALK_PATH_INFO
{
    int                             nAction;                   		    // Event operation. 0=pulse event. 1=start. 2=stop
    int					            nChannelID;						    // Channel id
    double				            dbPTS;							    // Time stamp (Unit is ms)
    NET_TIME_EX			            stuTime;						    // Event occurrence time
	char							szSrcDevID[DH_COMMON_STRING_128];	// Src Device ID
	char							szDstDevID[DH_COMMON_STRING_128];	// Dst Device ID
    BYTE                			byReserved[1024];   			    // Reserved
} ALARM_VIDEO_TALK_PATH_INFO;

// Waring zone intrusion direction 
typedef enum tagNET_CROSSREGION_DIRECTION_INFO
{
    EM_CROSSREGION_DIRECTION_UNKNOW = 0 , 
    EM_CROSSREGION_DIRECTION_ENTER      ,   //Entry
    EM_CROSSREGION_DIRECTION_LEAVE      ,   //Exit
    EM_CROSSREGION_DIRECTION_APPEAR     ,   //Appear
    EM_CROSSREGION_DIRECTION_DISAPPEAR  ,   //Disappear
}NET_CROSSREGION_DIRECTION_INFO;


//Warning zone event( Corresponding to event DH_EVENT_CROSSREGION_DETECTION)
typedef struct tagALARM_EVENT_CROSSREGION_INFO
{
    DWORD                           dwSize;    
    int					            nChannelID;						// Channel No. 
    double				            PTS;							// Time stamp (Unit is ms)
    NET_TIME_EX			            UTC;							// Event occurrence time
	int					            nEventID;						// Event ID
    int                             nEventAction;                   // Event operation. 0=pulse event.1=continues event begin. 2=continuous event stop

    NET_CROSSREGION_DIRECTION_INFO  emDirection;                    // Warning zone intrusion direction
    NET_CROSSREGION_ACTION_INFO     emActionType;                   // Detected types in the warning zone 
    int                             nOccurrenceCount;               // Rule triggered amount
    int                             nLevel;                         // Event type
}ALARM_EVENT_CROSSREGION_INFO;


//Abandoned object event (Corresponding to event DH_EVENT_LEFT_DETECTION)
typedef struct tagALARM_EVENT_LEFT_INFO
{
    DWORD               dwSize;    
    int					nChannelID;						// Channel No.
    double				PTS;							// Time stamp (Unit is ms)
    NET_TIME_EX			UTC;							// Event occurrence time 
    int					nEventID;						// Event ID
    int                 nEventAction;                   // Event operation. 0=pulse event.1=continues event begin. 2=continuous event stop
    
    int                 nOccurrenceCount;               // Rule triggered amount
    int                 nLevel;                         // Event type 
	short				nPreserID;						// Event triggered preset period, starting from 1 (no unknown)
	char				szPresetName[64];				// Preset name for event triggered
}ALARM_EVENT_LEFT_INFO;

//Human face detect event( corresponding to event DH_EVENT_FACE_DETECTION)
typedef struct tagALARM_EVENT_FACE_INFO
{
    DWORD               dwSize;    
    int					nChannelID;						// Channel No.
    double				PTS;							// Time stamp (Unit is ms)
    NET_TIME_EX			UTC;							// Event occurrence time 
    int					nEventID;						// Event ID
    int                 nEventAction;                   // Event operation. 0=pulse event.1=continues event begin. 2=continuous event stop
}ALARM_EVENT_FACE_INFO;

   
//IPC alarm,local alarm IPC send out by DVR or NVR(Corresponding to event  DH_ALARM_IPC)
typedef struct tagALARM_IPC_INFO
{
    DWORD               dwSize;    
    int					nChannelID;						// Channel No. 
    int                 nEventAction;                   // Event operation. 0=pulse event.1=continues event begin. 2=continuous event stop
}ALARM_IPC_INFO;
/////////////////////////////////Burning Session /////////////////////////////////////////

// CLIENT_StartBurnSession port input parameter
typedef struct tagNET_IN_START_BURN_SESSION 
{
    DWORD               dwSize;
    unsigned int        nSessionID;                         // session ID
} NET_IN_START_BURN_SESSION;

typedef struct tagNET_OUT_START_BURN_SESSION 
{
    DWORD               dwSize;
} NET_OUT_START_BURN_SESSION;

// Burning session
typedef enum tagNET_BURN_MODE
{
    BURN_MODE_SYNC,                     // sync
    BURN_MODE_TURN,                     // turn
    BURN_MODE_CYCLE,                    // cycle
} NET_BURN_MODE;

// Extension Burning mode
typedef enum tagNET_BURN_EXTMODE
{
	BURN_EXTMODE_UNKNOWN,					 // unknown
    BURN_EXTMODE_NORMAL = 1,                 // Normal Burning
    BURN_EXTMODE_NODISK,                     // Nodist Burning
} NET_BURN_EXTMODE;

// burning stream format
typedef enum tagNET_BURN_RECORD_PACK
{
    BURN_PACK_DHAV,                     // DHAV
    BURN_PACK_PS,                       // PS
    BURN_PACK_ASF,                      // ASF
    BURN_PACK_MP4,                      // MP4
    BURN_PACK_TS,                       // TS
} NET_BURN_RECORD_PACK;

// CLIENT_StartBurn port input parameter
typedef struct tagNET_IN_START_BURN 
{
    DWORD                dwSize;
    DWORD                dwDevMask;                             // burning device mask, by bit means multiple burning device group
    int                  nChannels[DH_MAX_BURN_CHANNEL_NUM];    // burning channel group
    int                  nChannelCount;                         // burning channel number
    NET_BURN_MODE        emMode;                                // burning mode
    NET_BURN_RECORD_PACK emPack;                                // burning stream format
    NET_BURN_EXTMODE     emExtMode;                             // Extension Burning mode
} NET_IN_START_BURN;

// CLIENT_StartBurn port output parameter
typedef struct tagNET_OUT_START_BURN 
{
    DWORD               dwSize;
} NET_OUT_START_BURN;

// CLIENT_BurnMarkTag port input parameter
typedef struct tagNET_IN_BURN_MAAK_TAG
{
    DWORD               dwSize;
    const char*         pszDescInfo;                            // mark description info
} NET_IN_BURN_MARK_TAG;

// CLIENT_BurnMarkTag port output parameter
typedef struct tagNET_OUT_BURN_MAAK_TAG
{
    DWORD               dwSize;
} NET_OUT_BURN_MARK_TAG;

// CLIENT_BurnChangeDisk port input parameter
typedef struct tagNET_IN_BURN_CHANGE_DISK
{
    DWORD               dwSize;
    BOOL                nAction;                                // switch action, 0-start, 1-end
} NET_IN_BURN_CHANGE_DISK;

// CLIENT_BurnChangeDisk port output parameter
typedef struct tagNET_OUT_BURN_CHANGE_DISK
{
    DWORD               dwSize;
} NET_OUT_BURN_CHANGE_DISK;

// burning status
typedef enum tagNET_BURN_STATE
{
    BURN_STATE_STOP,                    // stop
    BURN_STATE_STOPING,                 // stoping
    BURN_STATE_INIT,                    // initialize
    BURN_STATE_BURNING,                 // burning
    BURN_STATE_PAUSE,                   // pause
    BURN_STATE_CHANGE_DISK,             // changing
    BURN_STATE_PREPARE_EXTRA_FILE,      // attachment initialize
    BURN_STATE_WAIT_EXTRA_FILE,         // wait for attachment burning
    BURN_STATE_UPLOAD_FILE_START,       // attachment burning
    BURN_STATE_CHECKING_DISK,           // detecting disk
    BURN_STATE_DISK_READY,              // disk is ready
} NET_BURN_STATE;

// burning error code
typedef enum tagNET_BURN_ERROR_CODE
{
    BURN_CODE_NORMAL,                   // normal
    BURN_CODE_UNKNOWN_ERROR,            // unknown error
    BURN_CODE_SPACE_FULL,               // burning full
    BURN_CODE_START_ERROR,              // start burning error
    BURN_CODE_STOP_ERROR,               // stop burning  error
    BURN_CODE_WRITE_ERROR,              // burning error
	BURN_CODE_UNKNOWN,					// unknown
} NET_BURN_ERROR_CODE;

// CLIENT_BurnGetState port input parameter
typedef struct tagNET_IN_BURN_GET_STATE
{
    DWORD                dwSize;
} NET_IN_BURN_GET_STATE;

// CD driver used state
typedef enum tagEM_NET_BURN_DEV_USED_STATE
{
	EM_NET_BURN_DEV_USED_STATE_UNKNOWN, // unknown
	EM_NET_BURN_DEV_USED_STATE_STOP,	// stop
	EM_NET_BURN_DEV_USED_STATE_BURNING,	// buring
}EM_NET_BURN_DEV_USED_STATE;

// burning device status
typedef struct tagNET_BURN_DEV_STATE 
{
    DWORD                dwSize;
    int                  nDeviceID;                             // ROM device ID
    char                 szDevName[DH_BURNING_DEV_NAMELEN];     // ROM devicename
    DWORD                dwTotalSpace;                          // ROM total capacity, uni KB
    DWORD                dwRemainSpace;                         // ROM remaining capacity, unit KB
	EM_NET_BURN_DEV_USED_STATE	            emUsedType;			// ROM used state
	NET_BURN_ERROR_CODE						emError;			// sing ROM error
} NET_BURN_DEV_STATE;

// CLIENT_BurnGetState port output parameter
typedef struct tagNET_OUT_BURN_GET_STATE
{
    DWORD                dwSize;
    NET_BURN_STATE       emState;                               // burning status
    NET_BURN_ERROR_CODE  emErrorCode;                           // error code
    DWORD                dwDevMask;                             // burning device mask, by bit means multiple device group
    int                  nChannels[DH_MAX_BURN_CHANNEL_NUM];    // burning channel group
    int                  nChannelCount;                         // burning channel number
    NET_BURN_MODE        emMode;                                // burning mode
    NET_BURN_RECORD_PACK emPack;                                // burning stream format
    int                  nFileIndex;                            // current burning file no.
    NET_TIME             stuStartTime;                          // burning start time
    NET_BURN_DEV_STATE   stuDevState[DH_MAX_BURNING_DEV_NUM];   // burning device status
    int                  nRemainTime;                           // Event operation. 0=pulse event.1=continues event begin. 2=continuous event stop
	NET_BURN_EXTMODE	 emExtMode;								// extra mode,when emExtMode = BURN_EXTMODE_NODISK,stuDevState maybe invalid
} NET_OUT_BURN_GET_STATE;

// fAttachBurnStateCB parameter
typedef struct tagNET_CB_BURNSTATE
{
    DWORD               dwSize;
    const char*         szState;                        // info type
                                                        //"UploadFileStart":may start to upload attachment
                                                        //"InitBurnDevice":initialize burning device
                                                        //"Burning":burning
                                                        //"BurnExtraFileStop":stop burning
                                                        //"BurnFilePause":pause burning
                                                        //"SpaceFull":burning space full
                                                        //"BurnFileError": burning error    
    const char*         szFileName;                     // current burning file name, use to "UploadFileStart" start upload attachment info
    unsigned int        dwTotalSpace;                   // total capacity, unit KB, use to "Burning" , display capacity or calculate progress
    unsigned int        dwRemainSpace;                  // free capacity, unit KB, useto "Burning"
    const char*         szDeviceName;                   // burning device name, use for different burning devices
    int                 nRemainTime;                    // Burning remaining time. Unit is second. -1=invalid 
}NET_CB_BURNSTATE;

// burning device callback function original, lAttachHandle is CLIENT_AttachBurnState return value,each 1 item, pBuf->dwSize == nBufLen
typedef void (CALLBACK *fAttachBurnStateCB) (LLONG lLoginID, LLONG lAttachHandle, NET_CB_BURNSTATE* pBuf, int nBufLen, LDWORD dwUser);

// burning device callback extension function original
typedef void (CALLBACK *fAttachBurnStateCBEx)(LLONG lLoginID, LLONG lAttachHandle, NET_OUT_BURN_GET_STATE* pBuf, int nBufLen, LDWORD dwUser);

// CLIENT_AttachBurnState() input parameter
typedef struct tagNET_IN_ATTACH_STATE
{
    DWORD                   dwSize;
    const char*             szDeviceName;                   // disk name, as "/dev/sda"
    fAttachBurnStateCB      cbAttachState;                  // burning listening callback
    LDWORD                  dwUser;                         // user data
    LLONG                   lBurnSession;                   // burning session handle, CLIENT_StartBurnSession retuen value. This value is 0, szDeviceName is valid, by burning device subscribe burning status
    fAttachBurnStateCBEx    cbAttachStateEx;                // extension burning listening callback
    LDWORD                  dwUserEx;                       // extension burninglistening callback user data
}NET_IN_ATTACH_STATE;

// CLIENT_AttachBurnState output parameter
typedef struct tagNET_OUT_ATTACH_STATE
{
    DWORD                   dwSize;
}NET_OUT_ATTACH_STATE;

// burning device callback function orginal, lUploadHandle is CLIENT_StartUploadFileBurned return value
typedef void (CALLBACK *fBurnFileCallBack) (LLONG lLoginID, LLONG lUploadHandle, int nTotalSize, int nSendSize, LDWORD dwUser);

// CLIENT_StartUploadFileBurned() input parameter
typedef struct tagNET_IN_FILEBURNED_START
{
    DWORD                   dwSize;
    const char*             szMode;                        // file upload method "append", append mode, now burning file name fixed as" FILE.zip ",filename ignored
    const char*             szDeviceName;                  // disk name, as "/dev/sda"
    const char*             szFilename;                    // local file name
    fBurnFileCallBack       cbBurnPos;                     // burning progress callback
    LDWORD                  dwUser;                        // user data
    LLONG                   lBurnSession;                  // burning handle, CLIENT_StartBurnSession return value This value is , szDeviceName valid, now by burning device subscribe burning status
}NET_IN_FILEBURNED_START;

// CLIENT_StartUploadFileBurned output parameter
typedef struct tagNET_OUT_FILEBURNED_START
{
    DWORD                   dwSize;
    char                    szState[DH_MAX_NAME_LEN];      // "start":system is ready,may start to upload; "busy":system busy, try later."error":system is not burning, return error, request failed
}NET_OUT_FILEBURNED_START;

// burning case info
typedef struct tagNET_BURN_CASE_INFO
{
    DWORD       dwSize;
    int         nChannel;                                   // channel no.
    NET_TIME    stuStartTime;                               // burning start time
    NET_TIME    stuEndTime;                                 // burning end time
    int         nIndex;                                     // sn
    int         nCode;                                      // no.
    int         nDiscNum;                                   // disk no.        
    char        szName[DH_COMMON_STRING_128];               // name
    char        szPlace[DH_COMMON_STRING_128];              // location
    char        szInvestigator[DH_MAX_CASE_PERSON_NUM][DH_COMMON_STRING_32]; // handled by
    char        szSuspects[DH_MAX_CASE_PERSON_NUM][DH_COMMON_STRING_32];     // invloved
    char        szMemo[DH_COMMON_STRING_256];               // note
    char        szVideoName[DH_COMMON_STRING_128];          // record name
    char        szRecorder[DH_COMMON_STRING_32];            // burned by
} NET_BURN_CASE_INFO;

// burning device callback function original, lAttachHandle is CLIENT_AttachBurnCase return value
typedef void (CALLBACK *fBurnCaseCallBack) (LLONG lAttachHandle, NET_BURN_CASE_INFO* pBuf, DWORD dwBufLen, void* pReserved, LDWORD dwUser);

// CLIENT_AttachBurnCase port input parameter
typedef struct tagNET_IN_ATTACH_BURN_CASE
{
    DWORD                dwSize;
    fBurnCaseCallBack    cbBurnCase;                        // case info callback function
    LDWORD               dwUser;                            // user data
} NET_IN_ATTACH_BURN_CASE;

// CLIENT_AttachBurnCase port output parameter
typedef struct tagNET_OUT_ATTACH_BURN_CASE
{
    DWORD                dwSize;
} NET_OUT_ATTACH_BURN_CASE;

// burning device state info
typedef enum tagEM_NET_BURN_DEV_BUS_TYPE
{
    EM_NET_BURN_DEV_BUS_TYPE_UNKNOWN,   // Unknown
    EM_NET_BURN_DEV_BUS_TYPE_IDE,       // IDE
    EM_NET_BURN_DEV_BUS_TYPE_USB,       // USB
    EM_NET_BURN_DEV_BUS_TYPE_1394,      // 1394
    EM_NET_BURN_DEV_BUS_TYPE_SATA,      // SATA
    EM_NET_BURN_DEV_BUS_TYPE_ESATA,     // ESATA
}EM_NET_BURN_DEV_BUS_TYPE;
// device burn medium type
typedef enum tagEM_NET_BURN_DEV_BK_TYPE
{
    EM_NET_BURN_DEV_BK_TYPE_UNKNOWN,    // Unknown
    EM_NET_BURN_DEV_BK_TYPE_DHFS,       // dahua file system
    EM_NET_BURN_DEV_BK_TYPE_DISK,       // Disk
    EM_NET_BURN_DEV_BK_TYPE_CDRW,       // cdrom
}EM_NET_BURN_DEV_BK_TYPE;
// device burn state information
typedef struct tagNET_BURN_DEV_STATE_INFO
{
    char            szName[DH_COMMON_STRING_256];               // burning device name
    unsigned int    nTotalSpace;                                // CD driver total space, unit KB
    unsigned int    nFreeSpace;                                 // CD driver free space, unit KB
    EM_NET_BURN_DEV_TRAY_TYPE        emTrayType;                // CD driver tray state
    EM_NET_BURN_DEV_OPERATE_TYPE     emOperateType;             // CD driver using state
    EM_NET_BURN_DEV_BUS_TYPE         emBusType;                 // burning device bus type
    EM_NET_BURN_DEV_BK_TYPE          emBkType;                  // burning device bk type
} NET_BURN_DEV_STATE_INFO;



// the info list of burning device's state
typedef struct tagNET_BURN_DEV_STATE_INFO_LIST
{
    unsigned int    nListCount; // list count
	NET_BURN_DEV_STATE_INFO *pstuNetBurnDevStateInfo;   // list info
}NET_BURN_DEV_STATE_INFO_LIST;

// burning device state callback function original,lAttachHandle is CLIENT_AttachBurnDevState return value
typedef void (CALLBACK *fBurnDevStateCallBack) (LLONG lAttachHandle, const NET_BURN_DEV_STATE_INFO_LIST* pBuf, DWORD dwBufLen, void* pReserved, LDWORD dwUser);

// CLIENT_AttachBurnDevState port input parameter
typedef struct tagNET_IN_ATTACH_BURN_DEV_STATE
{
    DWORD                       dwSize;
    fBurnDevStateCallBack       cbBurnDevState;             // burning device state callback function
    LDWORD                      dwUser;                     // user data
} NET_IN_ATTACH_BURN_DEV_STATE;

// CLIENT_AttachBurnDevState port output parameter
typedef struct tagNET_OUT_ATTACH_BURN_DEV_STATE
{
    DWORD                dwSize;
} NET_OUT_ATTACH_BURN_DEV_STATE;

/////////////////////////////////// Case backup burn files ///////////////////////////////////////

#define CASE_BACK_UP_CHANNEL_MAX_NUM        32  // the max channel num of backupping case
#define CASE_BACK_UP_FILE_TYPE_MAX_NUM      8   // the max file type num of backupping case
#define CASE_BACK_UP_INQUEST_INFO_MAX_NUM   8   // the max inquest info num of backupping case
#define CASE_BACK_UP_DEVICE_MAX_NUM         8   // the max device num of backupping case

// the file type for backupping case
typedef enum tagEM_CASE_BACK_UP_FILE_TYPE
{
    EM_CASE_BACK_UP_FILE_TYPE_UNKNOWN,          // Unknown
    EM_CASE_BACK_UP_FILE_TYPE_DAV,              // DAV
    EM_CASE_BACK_UP_FILE_TYPE_JPG,              // JPG
    EM_CASE_BACK_UP_FILE_TYPE_TXT,              // TXT
}EM_CASE_BACK_UP_FILE_TYPE;

// the mode of backupping case
typedef enum tagEM_CASE_BACK_UP_MODE_TYPE
{
    EM_CASE_BACK_UP_MODE_TYPE_UNKNOWN,          // Unknown
    EM_CASE_BACK_UP_MODE_TYPE_SYNC,             // Sync:all the devices burn the same files
    EM_CASE_BACK_UP_MODE_TYPE_CYCLE,            // Cycle:burn the files with devices on by one
}EM_CASE_BACK_UP_MODE_TYPE;

// the pack type of backupping case
typedef enum tagEM_CASE_BACK_UP_PACK_TYPE
{
    EM_CASE_BACK_UP_PACK_TYPE_UNKNOWN,          // Unknown
    EM_CASE_BACK_UP_PACK_TYPE_DHAV,             // DHAV
    EM_CASE_BACK_UP_PACK_TYPE_MP4,              // MP4
    EM_CASE_BACK_UP_PACK_TYPE_PS,               // PS   
}EM_CASE_BACK_UP_PACK_TYPE;

// the inquest info of backupping case
typedef struct tagNET_CASE_BACK_UP_INQUEST_INFO
{
    char szTitle[128];                          // Title
    char szText[128];                           // Text
    BYTE byReserved[512];                       // Reserved
}NET_CASE_BACK_UP_INQUEST_INFO;

// the conditon of backupping case
typedef struct tagNET_CASE_BACK_UP_QUERY_CONDITION
{
    int                             nChannelNum;                                        // channel num                            
    int                             nChannels[CASE_BACK_UP_CHANNEL_MAX_NUM];            // the array of channels
    NET_TIME                        stuStartTime;                                       // start time
    NET_TIME                        stuEndTime;                                         // end time
    int                             nFileTypeNum;                                       // file type num
    EM_CASE_BACK_UP_FILE_TYPE       emFileTypes[CASE_BACK_UP_FILE_TYPE_MAX_NUM];        // the array of file types
    char                            szCaseNumber[256];                                  // case number
    int                             nInquestInfoNum;                                    // inquest info num
    NET_CASE_BACK_UP_INQUEST_INFO   stuInquestInfo[CASE_BACK_UP_INQUEST_INFO_MAX_NUM];  // the array of inquest info
    BYTE                            byReserved[1024];                                   // reserved
}NET_CASE_BACK_UP_QUERY_CONDITION;

// the mode info of backupping case
typedef struct tagNET_CASE_BACK_UP_MODE
{
    EM_CASE_BACK_UP_MODE_TYPE   emBackupMode;                                           // backup mode
    int                         nDeviceNum;                                             // device num
    char                        szDeviceName[CASE_BACK_UP_DEVICE_MAX_NUM][256];         // the array of device
    EM_CASE_BACK_UP_PACK_TYPE   emPackType;                                             // backup file type
    BOOL                        bExtraFile;                                             // extra file enable or not
    BOOL                        bDataCheck;                                             // data check enable or not
    BYTE                        byReserved[1024];                                       // reserved
}NET_CASE_BACK_UP_MODE;

// the in param of start case backup
typedef struct tagNET_IN_START_CASE_BACK_UP
{
    DWORD                               dwSize;                                         // struct size
    NET_CASE_BACK_UP_QUERY_CONDITION    stuCondition;                                   // condition
    NET_CASE_BACK_UP_MODE               stuMode;                                        // backup mode
}NET_IN_START_CASE_BACK_UP;

// the error code type of start case backup
typedef enum tagEM_CASE_BACK_UP_START_ERROR_CODE
{
    EM_CASE_BACK_UP_START_ERROR_CODE_UNKNOWN,                                           // Unknown
    EM_CASE_BACK_UP_START_ERROR_CODE_NORMAL,                                            // Normal
    EM_CASE_BACK_UP_START_ERROR_CODE_BACKUPING,                                         // Backupping
    EM_CASE_BACK_UP_START_ERROR_CODE_NO_FINDFILE,                                       // can not find the file
    EM_CASE_BACK_UP_START_ERROR_CODE_DEVICE_BUSY,                                       // device is busy
    EM_CASE_BACK_UP_START_ERROR_CODE_NO_SUPPORT_MEDIA_FORMAT,                           // not supported media format
    EM_CASE_BACK_UP_START_ERROR_CODE_DEVICE_ERROR,                                      // device error
    EM_CASE_BACK_UP_START_ERROR_CODE_BACKUP_FILE_OVER_LIMIT,                            // Backup File Over Limit    
    EM_CASE_BACK_UP_START_ERROR_CODE_OTHER_ERROR,                                       // other error
}EM_CASE_BACK_UP_START_ERROR_CODE;

// the out param of start case backup
typedef struct tagNET_OUT_START_CASE_BACK_UP
{
    DWORD                               dwSize;                                         // struct size
    EM_CASE_BACK_UP_START_ERROR_CODE    emErrorCode;                                    // error code
}NET_OUT_START_CASE_BACK_UP;

// the in param of stop case backup
typedef struct tagNET_IN_STOP_CASE_BACK_UP
{
    DWORD                               dwSize;                                         // struct size
}NET_IN_STOP_CASE_BACK_UP;

// the out param of stop case backup
typedef struct tagNET_OUT_STOP_CASE_BACK_UP
{
    DWORD                               dwSize;                                         // struct size
}NET_OUT_STOP_CASE_BACK_UP;

// the state type of case backup
typedef enum tagEM_CASE_BACK_UP_STATE_TYPE
{
    EM_CASE_BACK_UP_STATE_TYPE_UNKNOWN,                                                 // Unknown
    EM_CASE_BACK_UP_STATE_TYPE_INIT,                                                    // Initing
    EM_CASE_BACK_UP_STATE_TYPE_STOP,                                                    // stoped
    EM_CASE_BACK_UP_STATE_TYPE_STOPPING,                                                // stopping
    EM_CASE_BACK_UP_STATE_TYPE_BACKUPING,                                               // backupping           
    EM_CASE_BACK_UP_STATE_TYPE_CHANGE_DISK,                                             // change disk
    EM_CASE_BACK_UP_STATE_TYPE_PREPARE_EXTRA_FILE,                                      // prepare of extra file
    EM_CASE_BACK_UP_STATE_TYPE_WAIT_EXTRA_FILE,                                         // wait for extra file
    EM_CASE_BACK_UP_STATE_TYPE_UPLOAD_FILE_START,                                       // upload extra file
}EM_CASE_BACK_UP_STATE_TYPE;

// the error code of state for case backup
typedef enum tagEM_CASE_BACK_UP_ERROR_CODE
{
    EM_CASE_BACK_UP_ERROR_CODE_UNKNOWN,                                                 // Unknown
    EM_CASE_BACK_UP_ERROR_CODE_NORMAL,                                                  // normal
    EM_CASE_BACK_UP_ERROR_CODE_SPACE_FULL,                                              // space full
    EM_CASE_BACK_UP_ERROR_CODE_START_ERROR,                                             // start error   
    EM_CASE_BACK_UP_ERROR_CODE_STOP_ERROR,                                              // stop error
    EM_CASE_BACK_UP_ERROR_CODE_WRITE_ERROR,                                             // write file error   
    EM_CASE_BACK_UP_ERROR_CODE_OTHER_ERROR,                                             // other error      
}EM_CASE_BACK_UP_ERROR_CODE;

// the info of state for case backup
typedef struct tagNET_CASE_BACK_UP_STATE
{
    EM_CASE_BACK_UP_STATE_TYPE  emBackUpState;                                          // backup state
    EM_CASE_BACK_UP_ERROR_CODE  emErrorCode;                                            // error code
    BYTE                        byReserved[512];                                        // reserved
}NET_CASE_BACK_UP_STATE;

// the file info for case backup
typedef struct tagNET_CASE_BACK_UP_FILE_INFO
{
    int                         nFileCount;                                             // the total file count
    int                         nCurFileIndex;                                          // the index of cur backup file 
    TP_U64                      nFileTotalLength;                                       // the total file length
    TP_U64                      nCurBackupTotalLength;                                  // the total length of back-upped files
    TP_U64                      nCurBackupFileTotalLength;                              // the lengh of cur backup file
    TP_U64                      nCurBackupFileLength;                                   // the back-upped length of cur backup file  
    BYTE                        byReserved[512];                                        // reserved
}NET_CASE_BACK_UP_FILE_INFO;

// the device info of case backup
typedef struct tagNET_CASE_BACK_UP_DEVICE_INFO
{
    int                         nDeviceID;                                              // device id
    char                        szDeviceName[256];                                      // device name
    int                         nTotalSpace;                                            // the total space, unit:KB
    int                         nRemainSpace;                                           // the left space, unit:KB
    EM_CASE_BACK_UP_STATE_TYPE  emBackUpState;                                          // the backup state
    EM_CASE_BACK_UP_ERROR_CODE  emErrorCode;                                            // error code
    BYTE                        byReserved[512];                                        // reserved
}NET_CASE_BACK_UP_DEVICE_INFO;

// the state of case backup for callback
typedef struct tagNET_OUT_CASE_BACK_UP_STATE
{
    DWORD                           dwSize;                                             // struct size
    NET_CASE_BACK_UP_STATE          stuBackUpState;                                     // state info
    NET_CASE_BACK_UP_FILE_INFO      stuFileInfo;                                        // file info
    int                             nDeviceNum;                                         // device num    
    NET_CASE_BACK_UP_DEVICE_INFO    stuDeviceInfo[CASE_BACK_UP_DEVICE_MAX_NUM];         // the array of device    
}NET_OUT_CASE_BACK_UP_STATE;

// the callback function for case backup state
typedef void (CALLBACK *fAttachCaseBackUpState)(LLONG lLoginID, LLONG lAttachHandle, const NET_OUT_CASE_BACK_UP_STATE* pBuf, int nBufLen, LDWORD dwUser);

// the in param for attach case backup state
typedef struct tagNET_IN_ATTACH_CASE_BACK_UP_STATE
{
    DWORD                           dwSize;                                             // struct size
    fAttachCaseBackUpState          cbAttachState;                                      // callback function
    LDWORD                          dwUser;                                             // user data
}NET_IN_ATTACH_CASE_BACK_UP_STATE;

// the out param for attach case backup state
typedef struct tagNET_OUT_ATTACH_CASE_BACK_UP_STATE
{
    DWORD                           dwSize;                                             // struct size
}NET_OUT_ATTACH_CASE_BACK_UP_STATE;

// the in param for detach case backup state
typedef struct tagNET_IN_DETACH_CASE_BACK_UP_STATE
{
    DWORD                           dwSize;                                             // struct size
}NET_IN_DETACH_CASE_BACK_UP_STATE;

// the out param for detach case backup state
typedef struct tagNET_OUT_DETACH_CASE_BACK_UP_STATE
{
    DWORD                           dwSize;                                             // struct size
}NET_OUT_DETACH_CASE_BACK_UP_STATE;

// the in param for get info of case backup 
typedef struct tagNET_IN_GET_CASE_BACK_UP_INFO
{
    DWORD                           dwSize;                                             // struct size
}NET_IN_GET_CASE_BACK_UP_INFO;

// the out param for get info of case backup
typedef struct tagNET_OUT_GET_CASE_BACK_UP_INFO
{
    DWORD                               dwSize;                                         // struct size
    NET_CASE_BACK_UP_QUERY_CONDITION    stuCondition;                                   // condition
    NET_CASE_BACK_UP_MODE               stuMode;                                        // backup mode
}NET_OUT_GET_CASE_BACK_UP_INFO;

// start case backup
CLIENT_NET_API BOOL CALL_METHOD CLIENT_StartBackUpCase(LLONG lLoginID, const NET_IN_START_CASE_BACK_UP* pstInParam, NET_OUT_START_CASE_BACK_UP *pstOutParam, int nWaitTime = NET_INTERFACE_DEFAULT_TIMEOUT);

// stop case backup
CLIENT_NET_API BOOL CALL_METHOD CLIENT_StopBackUpCase(LLONG lLoginID, const NET_IN_STOP_CASE_BACK_UP* pstInParam, NET_OUT_STOP_CASE_BACK_UP *pstOutParam, int nWaitTime = NET_INTERFACE_DEFAULT_TIMEOUT);

// attach the state of case backup
CLIENT_NET_API LLONG CALL_METHOD CLIENT_AttachBackUpCaseState(LLONG lLoginID, const NET_IN_ATTACH_CASE_BACK_UP_STATE* pstInParam, NET_OUT_ATTACH_CASE_BACK_UP_STATE *pstOutParam, int nWaitTime = NET_INTERFACE_DEFAULT_TIMEOUT);

// detach the state of case backup
CLIENT_NET_API BOOL CALL_METHOD CLIENT_DetachBackUpCaseState(LLONG lAttachHandle, const NET_IN_DETACH_CASE_BACK_UP_STATE* pstInParam, NET_OUT_DETACH_CASE_BACK_UP_STATE *pstOutParam, int nWaitTime = NET_INTERFACE_DEFAULT_TIMEOUT);

// get the info of case backup
CLIENT_NET_API BOOL CALL_METHOD CLIENT_GetCaseBackUpInfo(LLONG lLoginID, const NET_IN_GET_CASE_BACK_UP_INFO* pstInParam, NET_OUT_GET_CASE_BACK_UP_INFO *pstOutParam, int nWaitTime = NET_INTERFACE_DEFAULT_TIMEOUT);

/////////////////////////////////// Case backup burn files ///////////////////////////////////////

/////////////////////////////////// Compose channel ///////////////////////////////////////

#define NET_COMPOSE_CHANNEL_MAX_SPLIT_WINDOW_NUM 36     // the max split window num of compose channel

// the temp split window info
typedef struct tagNET_TEMP_SPLIT_WINDOW_INFO
{
    int nChannel;                                       // the channel number
    BOOL bAudio;                                        // the video and audio mixed or not
    DH_RECT stuPosition;                                // window position,The point coordinates [0,8192)
    BYTE byReserved[128];                               // reserved
}NET_TEMP_SPLIT_WINDOW_INFO;

// the in param of get the temp split info
typedef struct tagNET_IN_GET_TEMP_SPLIT
{
    DWORD dwSize;                                       // struct size
    int nComposeChannel;                                // the compose channel
}NET_IN_GET_TEMP_SPLIT;

// the out param of get the temp split info
typedef struct tagNET_OUT_GET_TEMP_SPLIT
{
    DWORD dwSize;                                       // struct size
    DH_SPLIT_MODE emSplitMode;                          // split mode,max to DH_SPLIT_36
    int nSplitNum;                                      // the window num,max to 36
    NET_TEMP_SPLIT_WINDOW_INFO stuWindowInfo[NET_COMPOSE_CHANNEL_MAX_SPLIT_WINDOW_NUM]; // the array of window
}NET_OUT_GET_TEMP_SPLIT;

// the in param of set the temp split info
typedef struct tagNET_IN_SET_TEMP_SPLIT
{
    DWORD dwSize;                                       // struct size
    int nComposeChannel;                                // the compose channel
    DH_SPLIT_MODE emSplitMode;                          // split mode,max to DH_SPLIT_36
    int nSplitNum;                                      // the window num,max to 36
    NET_TEMP_SPLIT_WINDOW_INFO stuWindowInfo[NET_COMPOSE_CHANNEL_MAX_SPLIT_WINDOW_NUM]; // the array of window
}NET_IN_SET_TEMP_SPLIT;

// the out param of set the temp split
typedef struct tagNET_OUT_SET_TEMP_SPLIT
{
    DWORD dwSize;                                       // struct size
}NET_OUT_SET_TEMP_SPLIT;

// the split window info
typedef struct tagNET_TEMP_SOURCE_WINDOW_INFO
{
    int nWindowID;                                      // window index,start from 0
    int nUniqueChannel;                                 // the unique channel
    BYTE byReserved[128];                               // reserved
}NET_TEMP_SOURCE_WINDOW_INFO;

// the in param of set the temp split source
typedef struct tagNET_IN_SET_TEMP_SOURCE
{
    DWORD dwSize;                                       // struct size
    int nComposeChannel;                                // the compose channel
    int nWindowNum;                                     // the window num
    NET_TEMP_SOURCE_WINDOW_INFO stuWindowInfo[NET_COMPOSE_CHANNEL_MAX_SPLIT_WINDOW_NUM]; // the array of window info
}NET_IN_SET_TEMP_SOURCE;

// the out param of set the temp split source
typedef struct tagNET_OUT_SET_TEMP_SOURCE
{
    DWORD dwSize;                                       // struct size   
}NET_OUT_SET_TEMP_SOURCE;

// set the temp split info
CLIENT_NET_API BOOL CALL_METHOD CLIENT_SetTempSplit(LLONG lLoginID, const NET_IN_SET_TEMP_SPLIT* pstInParam, NET_OUT_SET_TEMP_SPLIT *pstOutParam, int nWaitTime = NET_INTERFACE_DEFAULT_TIMEOUT);

// get the temp split info
CLIENT_NET_API BOOL CALL_METHOD CLIENT_GetTempSplit(LLONG lLoginID, const NET_IN_GET_TEMP_SPLIT* pstInParam, NET_OUT_GET_TEMP_SPLIT *pstOutParam, int nWaitTime = NET_INTERFACE_DEFAULT_TIMEOUT);

// set the temp split source info
CLIENT_NET_API BOOL CALL_METHOD CLIENT_SetTempSource(LLONG lLoginID, const NET_IN_SET_TEMP_SOURCE* pstInParam, NET_OUT_SET_TEMP_SOURCE *pstOutParam, int nWaitTime = NET_INTERFACE_DEFAULT_TIMEOUT);

/////////////////////////////////// Combined Channel ///////////////////////////////////////

/////////////////////////////////// Storage ///////////////////////////////////////

// remote storage writing-in result
typedef enum tagNET_STORAGE_WRITE_STATE
{
    NS_WRITE_UNKNOWN,                                   // unknown
    NS_WRITE_OK,                                        // successful
    NS_WRITE_DISK_IO_ERROR,                             // disk IO error
    NS_WRITE_DISK_NOT_READY,                            // disk not ready
    NS_WRITE_DISK_FAULT,                                // disk failure
    NS_WRITE_STREAM_NOT_EXIST,                          // data stream ID inexists
    NS_WRITE_MOUNT_DISK_ERROR,                          // disk load error
} NET_STORAGE_WRITE_STATE;

// remote module info
typedef struct tagNET_STORAGE_BLOCK 
{
    DWORD                   dwSize;
    unsigned int            nID;                        // CQFS module ID
    unsigned int            nStreamID;                  // data stream SN
    unsigned int            nRecycleTimestamp;          // CQFS recycle time stamp
    unsigned int            nBeginTimestamp;            // module internal data start time stamp
    unsigned int            nEndTimestamp;              // module internal data endtime stamp
    unsigned int            nLockCount;                 // lock value, 0 means not locked
    char                    szDiskUUID[DH_COMMON_STRING_64]; // disk UUID
} NET_STORAGE_BLOCK;

// remote storage info status
typedef struct tagNET_STORAGE_WRITE_INFO
{
    DWORD                    dwSize;
    NET_STORAGE_WRITE_STATE  emState;                   // writing in result
    int                      nBlockCount;               // module quantity
    NET_STORAGE_BLOCK        stuBlocks[DH_MAX_NET_STRORAGE_BLOCK_NUM]; // module info
} NET_STORAGE_WRITE_INFO;

//////////////////////////////////////////////////////////////////////////
/// \fn subscribe write-in remote storage info status callback function
/// \brief 
/// \author yang_xiuqing
/// \param  LLONG lAttachHandle [OUT] subscribe handle, CLIENT_NetStorageAttachWriteInfo return value 
/// \param  NET_STORAGE_WRITE_INFO * pBuf [OUT] remote storage info status
/// \param  int nBufLen [OUT] status info length
/// \param  LDWORD dwUser user data
/// \return n/a
///////////////////////////////////////////////////////////////////////////
typedef void (CALLBACK *fNetStorageAttachWriteInfoCB)(LLONG lAttachHandle, NET_STORAGE_WRITE_INFO* pBuf, int nBufLen, LDWORD dwUser);

// CLIENT_NetStorageAttachWriteInfo port input parameter
typedef struct tagNET_IN_STORAGE_ATTACH_WRITE_INFO
{
    DWORD                           dwSize;
    const char*                     pszName;                // remote storage name, value from NAS config
    fNetStorageAttachWriteInfoCB    cbISCSIBlcok;           // callback function
    LDWORD                          dwUser;                 // user data
} NET_IN_STORAGE_ATTACH_WRITE_INFO;

// CLIENT_NetStorageAttachWriteInfo port output parameter
typedef struct tagNET_OUT_STORAGE_ATTACH_WRITE_INFO
{
    DWORD                       dwSize;
} NET_OUT_STORAGE_ATTACH_WRITE_INFO;

// CLIENT_NetStorageGetWriteInfo port input parameter
typedef struct tagNET_IN_STORAGE_GET_WRITE_INFO 
{
    DWORD                       dwSize;
    const char*                 pszName;                    // remote storage name
} NET_IN_STORAGE_GET_WRITE_INFO;

// CLIENT_NetStorageGetWriteInfo port output parameter
typedef struct tagNET_OUT_STORAGE_GET_WRITE_INFO 
{
    DWORD                       dwSize;    
    int                         nBlockCount;                // module quantity
    NET_STORAGE_BLOCK           stuBlocks[DH_MAX_NET_STRORAGE_BLOCK_NUM]; // module info
} NET_OUT_STORAGE_GET_WRITE_INFO;

// RAID operation type
typedef enum tagNET_RAID_OPERATE_TYPE
{
    NET_RAID_OPERATE_ADD,                    // add RAID, corresponding structure  NET_IN_RAID_ADD  and NET_OUT_RAID_ADD
    NET_RAID_OPERATE_REMOVE,                 // delete RAID, corresponding structure  NET_IN_RAID_REMOVE  and NET_OUT_RAID_REMOVE
    NET_RAID_OPERATE_GET_SUBDEVICE,          // get RAID sub device info, corresponding structure  NET_IN_RAID_GET_SUBDEVICE  and NET_OUT_RAID_GET_SUBDEVICE
    NET_RAID_OPERATE_GET_SUBSMART,           // get RAID sub device SMART info, corresponding structure  NET_IN_RAID_GET_SUBSMART  and NET_OUT_RAID_GET_SUBSMART
    NET_RAID_OPERATE_MODIFY,                 // modify RAID, corresponding to  structure  NET_IN_RAID_MODIFY and NET_OUT_RAID_MODIFY
} NET_RAID_OPERATE_TYPE;

// RAID info
typedef struct tagNET_RAID_INFO 
{
    DWORD                dwSize;
    char                 szName[DH_COMMON_STRING_64];     // name, as time name returned by device
    char                 szLevel[DH_COMMON_STRING_16];    // level, such as "RAID0", "RAID5" and etc.
    int                  nMemberNum;                      // member quantity
    char                 szMembers[DH_MAX_MEMBER_PER_RAID][DH_COMMON_STRING_64]; // member info
} NET_RAID_INFO;

// add RAID return result
typedef struct tagNET_RAID_ADD_RESULT 
{
    DWORD                dwSize;
    BOOL                 bResult;
    DWORD                dwErrorCode;                    // failed error code
    char                 szName[DH_COMMON_STRING_64];    // name
} NET_RAID_ADD_RESULT;

// add RAID input parameter
typedef struct tagNET_IN_RAID_ADD 
{
    DWORD                dwSize;
    int                  nRaidNun;                        	// RAID quantity
    NET_RAID_INFO        stuRaids[DH_MAX_RAID_NUM];         // RAID info
} NET_IN_RAID_ADD;

// add RAID oiutput parameter
typedef struct tagNET_OUT_RAID_ADD 
{
    DWORD                dwSize;
    int                  nResultNum;                        // result
    NET_RAID_ADD_RESULT  stuResults[DH_MAX_RAID_NUM];       // RAID operation result
} NET_OUT_RAID_ADD;

// delete RAID return result
typedef struct tagNET_RAID_REMOVE_RESULT 
{
    DWORD                dwSize;
    BOOL                 bResult;
    DWORD                dwErrorCode;                       // failed error code
} NET_RAID_REMOVE_RESULT;

// delete RAID input parameter
typedef struct tagNET_IN_RAID_REMOVE 
{
    DWORD                dwSize;
    int                  nRaidNum;                         // RAID quantity
    char                 szRaids[DH_MAX_RAID_NUM][DH_COMMON_STRING_64];    // RAID name group
} NET_IN_RAID_REMOVE;

// delete RAID output parameter
typedef struct tagNET_OUT_RAID_REMOVE
{
    DWORD                dwSize;
    int                  nResultNum;                       // result
    NET_RAID_REMOVE_RESULT stuResults[DH_MAX_RAID_NUM];    // RAID operation result
} NET_OUT_RAID_REMOVE;

// get RAID sub device input parameter
typedef struct tagNET_IN_RAID_GET_SUBDEVICE 
{
    DWORD                dwSize;
    const char*          pszRaidName;                    // RAID name
} NET_IN_RAID_GET_SUBDEVICE;

// get RAID sub device output parameter
typedef struct tagNET_OUT_RAID_GET_SUBDEVICE 
{
    DWORD                dwSize;
    int                  nSubDeviceNum;                  // sub device quantity
    DH_STORAGE_DEVICE    stuSubDevices[DH_MAX_MEMBER_PER_RAID]; // sub device quantity
} NET_OUT_RAID_GET_SUBDEVICE;

// RAID sub device SMART info
typedef struct tagNET_RAID_SMART_INFO 
{
    DWORD                dwSize;
    unsigned int         nID;                            // property ID
    char                 szName[DH_COMMON_STRING_64];    // property name
    int                  nCurrent;                       // property value
    int                  nWorst;                         // max error value
    int                  nThreshold;                     // threshold
    int                  nPredict;                       // status
    char                 szRaw[DH_COMMON_STRING_16];     // actual value
} NET_RAID_SMART_INFO;

// get RAID sub device SMART info input parameter
typedef struct tagNET_IN_RAID_GET_SUBSMART
{
    DWORD                dwSize;
    const char*          pszSubDevName;                  // RAID sub device name 
} NET_IN_RAID_GET_SUBSMART;

// get RAID sub device SMART info output parameter
typedef struct tagNET_OUT_RAID_GET_SUBSMART
{
    DWORD                dwSize;
    int                  nSmartNum;                          // SMART info
    NET_RAID_SMART_INFO  stuSmartInfos[MAX_SMART_VALUE_NUM]; // SMART info
} NET_OUT_RAID_GET_SUBSMART;

// modify RAID return result
typedef struct tagNET_RAID_MODIFY_RESULT 
{
    DWORD                dwSize;
    BOOL                 bResult;
    DWORD                dwErrorCode;                       // failed error code
} NET_RAID_MODIFY_RESULT;

// modify RAID input parameter
typedef struct tagNET_IN_RAID_MODIFY
{
    DWORD                dwSize;
    int                  nRaidNun;                          // RAID quantity
    NET_RAID_INFO        stuRaids[DH_MAX_RAID_NUM];         // RAID info
} NET_IN_RAID_MODIFY;

// modify RAID output parameter
typedef struct tagNET_OUT_RAID_MODIFY 
{
    DWORD                dwSize;
    int                  nResultNum;                        // result
    NET_RAID_MODIFY_RESULT stuResults[DH_MAX_RAID_NUM];     // RAID operation result
} NET_OUT_RAID_MODIFY;


//////////////////////////////// external device //////////////////////////////////////////

// external device type
typedef enum tagNET_EXT_DEV_TYPE
{
    EXT_DEV_UNKNOWN,                    // unknow
    EXT_DEV_PROJECTOR,                  // projector
    EXT_DEV_SEQUENCE_POWER,            	// power sequence
} NET_EXT_DEV_TYPE;

// external device info
typedef struct tagNET_EXTERNAL_DEVICE 
{
    DWORD               dwSize;
    NET_EXT_DEV_TYPE    emType;                             // device type
    char                szDevID[DH_DEV_ID_LEN_EX];          // device ID, SN
    char                szDevName[DH_DEV_ID_LEN_EX];        // device name 
} NET_EXTERNAL_DEVICE;

// power sequence control parameter
typedef struct tagNET_CTRL_SEQPOWER_PARAM
{
    DWORD               dwSize;
    const char*         pszDeviceID;                        // device ID
    int                 nChannel;                           // output or output sequence
} NET_CTRL_SEQPOWER_PARAM;

// projector control parameter
typedef struct tagNET_CTRL_PROJECTOR_PARAM 
{
    DWORD               dwSize;
    const char*         pszDeviceID;                        // device ID
} NET_CTRL_PROJECTOR_PARAM;

// IR button
typedef struct tagNET_CTRL_INFRARED_KEY_PARAM 
{
    DWORD               dwSize;
    int                 nChannel;                           // IR control channel no.
    unsigned int        nKey;                               // IR button ID
} NET_CTRL_INFRARED_KEY_PARAM;

// no HDD alarm
typedef struct tagALARM_NO_DISK_INFO
{
    DWORD               dwSize;
    NET_TIME            stuTime;                            // time
    DWORD               dwAction;                           // event action, 0:Start, 1:Stop
}ALARM_NO_DISK_INFO;
//protective capsule status enumeration
typedef enum __EM_CAPSULE_STATE
{
    CAPSULE_STATE_UNKNOW ,      //undefine
    CAPSULE_STATE_NORMAL ,      //normal
    CAPSULE_STATE_TIME_OUT ,    //overtime
    CAPSULE_STATE_EMERGENCY_CALL,   //sos call
}EM_CAPSULE_STATE;

//protective capsule event
typedef struct tagALARM_PROTECTIVE_CAPSULE_INFO
{
    DWORD               dwSize;
    EM_CAPSULE_STATE    emCapsuleState; //protective capsule status
    DWORD               nLock;          //0:unlock, 1:lock; only when emCapsuleStateis CAPSULE_STATE_NORMAL valid
    DWORD               nInfrared;      //0:idle (no one),1:trigger(someone); only emCapsuleState is CAPSULE_STATE_NORMAL valid
}ALARM_PROTECTIVE_CAPSULE_INFO;

// Get Fish eye capacity input parameter 
typedef struct tagNET_IN_VIDEOIN_FISHEYE_CAPS
{
    DWORD       dwSize; 
    int         nChannel; //channel no.
}NET_IN_VIDEOIN_FISHEYE_CAPS;

// Get Fish eye capacity  output parameter 
typedef struct tagNET_OUT_VIDEOIN_FISHEYE_CAPS
{
    DWORD                   dwSize; 
    int                     nMountModeNum;              // supported installation mode number     
    NET_FISHEYE_MOUNT_MODE  emMountModes[NET_MAX_FISHEYE_MOUNTMODE_NUM]; // supported installation mode list
    int                     nCalibrateModeNum;          // supported Fish eye correction mode number  
    NET_FISHEYE_CALIBRATE_MODE emCalibrateModes[NET_MAX_FISHEYE_CALIBRATEMODE_NUM]; // supported Fish eye correction mode list
    int                     nEPtzCmdNum;                // supported Fish eye E-PTZ operation number 
    NET_FISHEYE_EPTZ_CMD    emEPtzCmds[NET_MAX_FISHEYE_EPTZCMD_NUM]; // supported Fish eye E-PTZ operationlist
    NET_FISHEYE_TYPE        emType;                     // Fish eye type 
}NET_OUT_VIDEOIN_FISHEYE_CAPS;

// get power sequence input parameter
typedef struct tagNET_IN_CAP_SEQPOWER 
{
    DWORD                dwSize;
    const char*          pszDeviceID;                       // device ID
} NET_IN_CAP_SEQPOWER;

// get power sequence output parameter
typedef struct tagNET_OUT_CAP_SEQPOWER
{
    DWORD                dwSize;
    int                  nChannelNum;                       // channel no.
} NET_OUT_CAP_SEQPOWER;

// get device encode config corresponding capacity input parameter
typedef struct tagNET_IN_ENCODE_CFG_CAPS
{
    DWORD               dwSize;           
    int                 nChannelId;                         // channel no.    
    int                 nStreamType;                        // stream type, 0:main stream, 1:sub stream1, 2:sub stream2, 3:sub stream3, 4:snapshot stream
															// can be left unset, whatever value it is set, you get results of main stream, sub stream1 and snapshot stream
    char*               pchEncodeJson;                      // Encode config, cia calling dhconfigsdk.dll port CLIENT_PacketData to get
                                                            // correspondingsampling command is CFG_CMD_ENCODE                 
}NET_IN_ENCODE_CFG_CAPS;

// stream config corresponding capacity
typedef struct tagNET_STREAM_CFG_CAPS
{
    DWORD               dwSize;
    int                 nAudioCompressionTypes[DH_MAX_AUDIO_ENCODE_TYPE]; // supported audio encode type, see DH_TALK_CODING_TYPE
    int                 nAudioCompressionTypeNum;                   // audio compression quantity
    int                 dwEncodeModeMask;                           // video encode mode mask, see "encode mode"
    DH_RESOLUTION_INFO  stuResolutionTypes[DH_MAX_CAPTURE_SIZE_NUM];// supportedvideo resolution, discard
    int                 nResolutionFPSMax[DH_MAX_CAPTURE_SIZE_NUM]; // different resolution max frame, subscript nResolutionTypescorresponding 
    int                 nResolutionTypeNum;                         // video resolution quantity
    int                 nMaxBitRateOptions;                         // max video stream(kbps) 
    int                 nMinBitRateOptions;                         // min video stream(kbps)
    BYTE                bH264ProfileRank[DH_PROFILE_HIGH];          // supported H.264 Profile level,refer to enumeration type EM_H264_PROFILE_RANK;  
    int                 nH264ProfileRankNum;                        // supported H.264 Profile level
    int                 nCifPFrameMaxSize;                          // when resolution is  cif, max p frame(Kbps)
    int                 nCifPFrameMinSize;                          // when currentresolution is cif, min p frame(Kbps)
    int                 nFPSMax;                                    // video max frame rate, as 0 , make nResolutionFPSMax as standard
    DH_RESOLUTION_INFO  stuIndivResolutionTypes[DH_MAX_COMPRESSION_TYPES_NUM][DH_MAX_CAPTURE_SIZE_NUM];// supportedvideo resolution
	BOOL				abIndivResolution;							// 0: stuResolutionTypes,nResolutionTypeNum is valid
                                                                    // 1: stuIndivResolutionTypes, nIndivResolutionNums is valid
    int                 nIndivResolutionNums[DH_MAX_COMPRESSION_TYPES_NUM];// number of video resolution quantity with different encoding mode  
}NET_STREAM_CFG_CAPS;

// get device encode config corresponding capacity output parameter
typedef struct tagNET_OUT_ENCODE_CFG_CAPS
{
    DWORD               dwSize;
    NET_STREAM_CFG_CAPS stuMainFormatCaps[DH_REC_TYPE_NUM];      	// main stream corresponding capacity
    NET_STREAM_CFG_CAPS stuExtraFormatCaps[DH_N_ENCODE_AUX];        // sub stream config corresponding capacity
    NET_STREAM_CFG_CAPS stuSnapFormatCaps[SNAP_TYP_NUM];           	// snap stream config corresponding capacity
    int                 nMainFormCaps;                              // number of main stream config
    int                 nExtraFormCaps;                             // number of Extra stream config
    int                 nSnapFormatCaps;                            // number of snap stream config
}NET_OUT_ENCODE_CFG_CAPS;

// Video detect device caps
typedef struct tagNET_IN_VIDEO_DETECT_CAPS
{
    DWORD               dwSize;           
    int                 nChannel;  //channel, start from 0
}NET_IN_VIDEO_DETECT_CAPS;

// media capability type
typedef enum tagNET_MEDIA_CAP_TYPE
{
    NET_MEDIA_CAP_TYPE_SENSORINFO,      // camera sensor info
} NET_MEDIA_CAP_TYPE;

// CLIENT_GetDevCaps NET_MEDIAMANAGER_CAPS input parameter
typedef struct tagNET_IN_MEDIAMANAGER_GETCAPS
{
    DWORD               dwSize;
    NET_MEDIA_CAP_TYPE  emType;         // capability type to query
} NET_IN_MEDIAMANAGER_GETCAPS;

// get snap config capbility input parameter	 
typedef struct tagNET_IN_SNAP_CFG_CAPS	 	 	 
{        	 	 	 
    int                 nChannelId;                     // channel num(start from 0) 	 	 	 
    BYTE                bReserved[1024];                // Reserved	 	 	 
}NET_IN_SNAP_CFG_CAPS;	 	 	 

#define DH_MAX_FPS_NUM                128                // max length of nFramesPerSecList array	 	 	 
#define DH_MAX_QUALITY_NUM            32                 // max length of nQualityList array	 	 	 

// get snap config capbility output parameter	 	 	 
typedef struct tagNET_OUT_SNAP_CFG_CAPS 	 	 	 
{	 	 	 
    int                 nResolutionTypeNum;                // valid length of stuResolutionTypes array	 	 	 
    DH_RESOLUTION_INFO  stuResolutionTypes[DH_MAX_CAPTURE_SIZE_NUM];	 	 	 
    DWORD               dwFramesPerSecNum;                 // valid length of nFramesPerSecList array		 	 	 
    int                 nFramesPerSecList[DH_MAX_FPS_NUM]; // -25:1f/25s;-24:1f/24s;-23:1f/23s;-22:1f/22s...
                                                           // 0: invalid;1:1f/s;2:2f/s;3:13f/s
                                                           // 4:4f/s;5:5f/s;17:17f/s;18:18f/s
                                                           // 19:19f/s;20:20f/s...
												           // 25: 25f/s 	  	 	 
    DWORD               dwQualityMun;                      // valid length of nQualityList array	 	 	 
    DWORD               nQualityList[DH_MAX_QUALITY_NUM];  // Image quality(1-6,6 means the best quality)
                                                           // 6:100%; 5:80%; 4:60% 3:50%; 2:30%; 1:10%	 	 
    DWORD               dwMode;                            // (Bit)0:activate scheduled snapshot,1:Manually activate snapshot	 	 	 
    DWORD               dwFormat;                          // (Bit)0:BMP format,1:JPG format 	 	 
    BYTE                bReserved[2048];                   // Reserved	 	 
} NET_OUT_SNAP_CFG_CAPS;

// camera sensor type
typedef enum tagNET_CAMERA_SENSOR
{
    NET_CAMERA_SENSOR_NORMAL,           // normal (visible light)
    NET_CAMERA_SENSOR_LEPTON,           // Lepton infrared
    NET_CAMERA_SENSOR_TAU,              // Tau infrared
} NET_CAMERA_SENSOR;

// camera sensor info
typedef struct tagNET_CAMERA_SENSORINFO
{
    NET_CAMERA_SENSOR   emSensorType;       // sensor type
    int                 nChannelsCount;     // count of channels of the sensor type
    int                 nChannels[512];     // channels of the sensor type
    char                reserved[512];
} NET_CAMERA_SENSORINFO;

// media info - camera sensor
typedef struct tagNET_MEDIA_SENSORINFO
{
    BOOL                    bSupport;               // whether or not support querying this capability. when TRUE, members below are valid
    int                     nSensorTypeCount;       // count of sensor types of the device
    NET_CAMERA_SENSORINFO   stuDetail[16];          // detailed info for each sensor type, nSensorTypeCount elements are in effect 
    char                    reserved[1024];
} NET_MEDIA_SENSORINFO;

// CLIENT_GetDevCaps NET_MEDIAMANAGER_CAPS output parameter
typedef struct tagNET_OUT_MEDIAMANAGER_GETCAPS
{
    DWORD                   dwSize;
    NET_MEDIA_SENSORINFO    stuSensorInfo;                  // camera sensor info
} NET_OUT_MEDIAMANAGER_GETCAPS;

// type of videoin caps
typedef enum tagNET_ENUM_VIDEOIN_CAP_TYPE
{
    NET_VIDEOIN_CAP_TYPE_CONFLICT,                          // conflict caps of videoin, CorrespondingStructure NET_OUT_VIDEOIN_CONFLICT_CAPS
}NET_ENUM_VIDEOIN_CAP_TYPE;

// CLIENT_GetDevCaps NET_VIDEOIN_CAPS input parameter
typedef struct tagNET_IN_VIDEOIN_CAPS
{
    DWORD                               dwSize;
    int                                 nChannel;           // channel id
    NET_ENUM_VIDEOIN_CAP_TYPE           emCapType;          // type of videoin caps
}NET_IN_VIDEOIN_CAPS;

// enumeration of conflict
typedef enum tagNET_ENUM_CONFLICT_TYPE
{
    NET_ENUM_CONFLICT_TYPE_UNKNOWN,                          // unknown
    NET_ENUM_CONFLICT_TYPE_MAIN,                             // Main
    NET_ENUM_CONFLICT_TYPE_EXTRA1,                           // EXTRA1
    NET_ENUM_CONFLICT_TYPE_EXTRA2,                           // EXTRA2
    NET_ENUM_CONFLICT_TYPE_TVOUT,                            // TVOUT
    NET_ENUM_CONFLICT_TYPE_DSP,                              // DSP
    NET_ENUM_CONFLICT_TYPE_SMARTENC,                         // Smart Enc
    NET_ENUM_CONFLICT_TYPE_SETGOP,                           // SET GOP
    NET_ENUM_CONFLICT_TYPE_ROI,                              // ROI
    NET_ENUM_CONFLICT_TYPE_CBR,                              // CBR
    NET_ENUM_CONFLICT_TYPE_SVC,                              // SVC
    NET_ENUM_CONFLICT_TYPE_MJPEG,                            // MJPEG
    NET_ENUM_CONFLICT_TYPE_ROTATE_90,                        // ROTATE 90
}NET_ENUM_CONFLICT_TYPE;

// Conflict
typedef struct tagNET_CONFLICT_TYPE
{
    NET_ENUM_CONFLICT_TYPE      emConflict1;                 // Conflict 1              
    NET_ENUM_CONFLICT_TYPE      emConflict2;                 // Conflict 2
    char                        reserved[64];
}NET_CONFLICT_TYPE;

typedef struct tagNET_VIDEOIN_CONFLICT_CAPS
{
    BOOL                        bConflict;                  // Enable conflict
    int                         nConflictNum;               // Conflict Number
    NET_CONFLICT_TYPE           stuConflict[MAX_VIDEOIN_CONFLICT_NUM];            // Conflict
}NET_VIDEOIN_CONFLICT_CAPS;

// CLIENT_GetDevCaps NET_VIDEOIN_CAPS output parameter
typedef struct tagNET_OUT_VIDEOIN_CAPS
{
    DWORD                       dwSize;
    NET_VIDEOIN_CONFLICT_CAPS   stuConflictCap;             // struct of conflict cap
}NET_OUT_VIDEOIN_CAPS;

// CLIENT_GetDevCaps  NET_VIDEO_MOSAIC_CAPS input parameter
typedef struct tagNET_IN_MEDIA_VIDEOMOSAIC_GETCAPS
{
	DWORD               dwSize;						        // the size of this struct
} NET_IN_MEDIA_VIDEOMOSAIC_GETCAPS;     

// CLIENT_GetDevCaps  NET_VIDEO_MOSAIC_CAPS output parameter
typedef struct tagNET_OUT_MEDIA_VIDEOMOSAIC_GETCAPS
{
	DWORD               dwSize;									// the size of this struct
	int					nSupportCount;							// counts of supports channels of the device
	short				snSupport[MAX_MOSAIC_CHANNEL_NUM];		// the channels which supported mosaic, -1 mean all
	int					nMosaicCount;							// the count of mosaic particles
	char				szMosaic[MAX_MOSAIC_NUM];				// the size of mosaic particles
	DH_SIZE				stuRectMax;								// the max size of mosaic block
	DH_SIZE				stuRectMin;								// the min size of mosaic block
} NET_OUT_MEDIA_VIDEOMOSAIC_GETCAPS;

// CLIENT_GetDevCaps NET_FACE_BOARD_CAPS input parameter
typedef struct tagNET_IN_FACEBOARD_CAPS
{
    DWORD               dwSize;						            // the size of this struct
}NET_IN_FACEBOARD_CAPS;

// CLIENT_GetDevCaps NET_VIDEO_IMAGECONTROL_CAPS input parameter
typedef struct tagNET_IN_VIDEO_IMAGECONTROL_CAPS
{
	DWORD               dwSize;									// the size of this struct 
	int					nChannel;								// channel ID
} NET_IN_VIDEO_IMAGECONTROL_CAPS;

// the stable caps type
typedef enum tagNET_EM_STABLE_CAPS_TYPE
{
	EM_STABLE_UNSPPORT,					// unsupport
	EM_STABLE_ELEC,						// electornic
	EM_STABLE_LIGHT,					// light
	EM_STAVLE_ELEC_AND_LIGHT,			// electornic and light
	EM_STABLE_CONTROL,					// control
	EM_STABLE_ELEC_AND_CONTROL,			// electornic and control
	EM_STABLE_LIGHT_AND_CONTROL,		// light and control
} NET_EM_STABLE_CAPS_TYPE;

// CLIENT_GetDevCaps NET_VIDEO_IMAGECONTROL_CAPS output parameter
typedef struct tagNET_OUT_VIDEO_IMAGECONTROL_CAPS
{
	DWORD               			dwSize;							// the size of this struct
	BOOL							bSupport;						// support Mirror or Flip or Rotate90 or not
	BOOL							bMirror;						// support mirror or not
	BOOL							bFlip;							// support flip or not
	BOOL							bRotate90;						// support rotate90 or not
	BOOL							bFreeze;						// support freeze or not
	NET_EM_STABLE_CAPS_TYPE			emStable;						// support stable or not
} NET_OUT_VIDEO_IMAGECONTROL_CAPS;

// CLIENT_GetDevCaps NET_VIDEOIN_EXPOSURE_CAPS input parameter
typedef struct tagNET_IN_VIDEOIN_EXPOSURE_CAPS
{
	DWORD               dwSize;									// the size of this struct
	int					nChannel;								// channel ID
} NET_IN_VIDEOIN_EXPOSURE_CAPS;

#define MAX_EXPOSURE_COUNT	8

// exposure mode
typedef enum tagNET_EM_EXPOSURE_MODE
{
	NET_EM_EXPOSURE_AUTO,					// auto
	NET_EM_EXPOSURE_LOWNICE,				// low nice
	NET_EM_EXPOSURE_ANTISHADOW,				// anti shadow
	NET_EM_EXPOSURE_MANUALRANGE	= 4,		// manual range
	NET_EM_EXPOSURE_APERTUREFIRST,			// aperture first
	NET_EM_EXPOSURE_MANUALFIXATION,			// manual fixation
	NET_EM_EXPOSURE_GIANFIRST,				// gian first
	NET_EM_EXPOSURE_SHUTTERFIRST,			// shutter first
	NET_EM_EXPOSURE_FLASHMATCH,				// flash match
} NET_EM_EXPOSURE_MODE;

// the anti ficker mode
typedef enum tagNET_EM_ANTIFLICKER_MODE
{
	NET_EM_ANTIFLICKER_OUTDOOR,			// outdoor
	NET_EM_ANTIFLICKER_50HZ,			// 50Hz
	NET_EM_ANTIFLICKER_60HZ,			// 60Hz
} NET_EM_ANTIFLICKER_MODE;


typedef struct tagNET_SPEED_CAPS
{
	int			nRetManual;										// the actual count of shutters in manual
	int			nManual[DH_COMMON_STRING_16];					// the actual list of shutters in manual
	int			nRetManual50Hz;									// the actual count of shutters in 50HZ
	int			nManual50Hz[DH_COMMON_STRING_16];				// the actual list of shutters in 50HZ
	int			nRetManual60Hz;									// the actual count of shutters in 60HZ
	int			nManual60Hz[DH_COMMON_STRING_16];				// the actual list of shutters in 60HZ
	int			nRetShutterPAL;									// the actual count of P shutters in shutter priority
	int			nShutterPAL[DH_COMMON_STRING_16];				// the actual list of P shutters in shutter priority
	int			nRetShutterNTSC;								// the actual count of N shutters in shutter priority
	int			nShutterNTSC[DH_COMMON_STRING_16];				// the actual list of N shutters in shutter priority
	BYTE		bReserved[128];									// reserved
} NET_SPEED_CAPS;

typedef struct tagNET_SLOW_SPEED_CAPS
{
	int			nRetPal;
	int			nPal[DH_COMMON_STRING_16];
	int			nRetNtsc;
	int			nNtsc[DH_COMMON_STRING_16];
	BYTE		bReserved[128];	
} NET_SLOW_SPEED_CAPS;

// CLIENT_GetDevCaps NET_VIDEOIN_EXPOSURE_CAPS output parameter
typedef struct tagNET_OUT_VIDEOIN_EXPOSURE_CAPS
{
	DWORD               		dwSize;									// the size of this struct
	BOOL						bSupport;								// support seting exposure or not
	int							nExposureMode;							// the actual count of exposure mode
	NET_EM_EXPOSURE_MODE		emExposureMode[MAX_EXPOSURE_COUNT];		// the supported exposure modes
	BOOL						bAntiFlicker;							// support seting anti flicker or not
	int							nAntiFlicker;							// the actual count of anti flicker mode
	NET_EM_ANTIFLICKER_MODE		emAntiFlicker[MAX_ANTIFLICKERMODE_COUNT];// the actual list of anti flicker mode
	int							nMinCompensation;						// the min value of compensation
	int							nMaxCompensation;						// the max value of compensation
	BOOL						bGainUpperLimit;						// support gain limit or not
	int							nMinGain;								// min value of gain
	int							nMaxGain;								// max value of gain
	BOOL						bSlowAutoExposure;						// support slow exposure or not
	int							nMinSlowAutoExposure;					// min value of slow exposure
	int							nMaxSlowAutoExposure;					// max value of slow exposure
	BOOL						bSlowShutter;							// support slow shutter or not
	float						fMinValueLow;							// the min value of auto exposure time lower limit
	float						fMaxValueLow;							// the max value of auto exposure time lower limit
	float						fMinValueUp;							// the min value of auto exposure time upper limit
	float						fMaxValueUp;							// the max value of auto exposure time upper limit
	NET_SPEED_CAPS				stuSpeedCaps;							// the capability of exposure speed
	NET_SLOW_SPEED_CAPS			stuSlowSpeedCaps;						// the capability of slow exposure speed
	BOOL						bIrisAuto;								// support auto iris or not
	int							nIrisMin;								// min value of iris
	int							nIrisMax;								// max value of iris
	BOOL						bSupportIrisRange;						// the capability of iris range
	BOOL						bDoubleExposure;						// support setting double shutter or not
	BOOL						bRecoveryTime;							// support auto recovery or not
} NET_OUT_VIDEOIN_EXPOSURE_CAPS;

// CLIENT_GetDevCaps NET_VIDEOIN_DENOISE_CAPS input parameter
typedef struct tagNET_IN_VIDEOIN_DENOISE_CAPS
{
	DWORD				dwSize; 								// the size of this struct
	int 				nChannel;								// channel ID
} NET_IN_VIDEOIN_DENOISE_CAPS;

// the type of 3D denoise
typedef enum tagNET_EM_3DSUPPORT_TYPE
{
	NET_EM_3DSUPPORT_UNKOWN,									// unknown
	NET_EM_3DSUPPORT_OFF,										// off 
	NET_EM_3DSUPPORT_AUTO,										// auto 
	NET_EM_3DSUPPORT_MANUAL,									// manual
} NET_EM_3DSUPPORT_TYPE;

// the mode of 3D denoise
typedef struct tagNET_3D_ALGORITHM_MODE
{
	// mask representationbit0 means noise algorithm bit0,bit1 means noise algorithm bit1,bit2...
	DWORD	dwSingleExposure;				// the capability of single shutter
	DWORD	dwDoubleExposureFullRate;		// the capability of full double denoise
	DWORD	dwDoubleExposureHalfRate;		// the capability of half double denoise
	DWORD	dsThreeExposure;				// the capability of three shutter
	BYTE	bReserved[128];					// reverd
} NET_3DALGORITHM_MODE;

#define		MAX_3DTYPE_COUNT			8			// max 3D denoise count 
#define		MAX_GROUP_COUNT				2			// max group of denoise 
#define		MAX_PROGRAM_COUNT			8			// max program count

// the capability of 2D denoise
typedef struct tagNET_SUPPORT2D_CAPS
{
	BOOL						bSupport2D;								// support 2D denoise or not
	int							n2DLevelMin;							// the min level of 2D denoise
	int							n2DLevelMax;							// the max level of 2D denoise
	int							nMaxRAWLevel;							// th max value 2D denoise which support RAW, 0 means not support
	BYTE						bReserved[128];							// reverd
} NET_SUPPORT2D_CAPS;

// the program of per denoise group
typedef struct tagNET_DENOISEGROUP_PROGRAM
{
	int							nProgramCount;							// the actual count of programs
	int							nProgram[MAX_PROGRAM_COUNT];			// the actual list of programs
} NET_DENOISEGROUP_PROGRAM;

// the capability of 3D denoise
typedef struct tagNET_SUPPORT3D_CAPS
{
	BOOL						bSupport3D;								// support 3D denose or not
	int							n3DTypeCount;							// the count of 3D denoise type
	NET_EM_3DSUPPORT_TYPE		em3DSupportType[MAX_3DTYPE_COUNT];		// the list of 3D denoise type
	int							n3DLevelMin;							// the min level of 3D denoise
	int							n3DLevelMax;							// the max level of 3D denoise
	int							nAutoLevelMin;							// the min level of auto 3D denoise
	int							nAutoLevelMax;							// the max level of auto 3D denoise
	NET_3DALGORITHM_MODE		stuAlgorithmSDMode;						// the algorithm mode supported by 3D denoise
	int							nGroupCount;							// the actual counts of groups
	NET_DENOISEGROUP_PROGRAM	stuDenoiseGroup[MAX_GROUP_COUNT]; 		// the list of groups
	int							nDenoiseScheme;							// the plame of denoise
	BYTE						bReserved[128];							// reverd
} NET_SUPPORT3D_CAPS;

// CLIENT_GetDevCaps NET_VIDEOIN_DENOISE_CAPS output parameter
typedef struct tagNET_OUT_VIDEOIN_DENOISE_CAPS
{
	DWORD						dwSize; 								// the size of this struct
	BOOL						bSupport;								// support setting denoise or not
	NET_SUPPORT2D_CAPS			stu2DCaps;								// the capability of 2D denoise 
	NET_SUPPORT3D_CAPS			stu3DCaps;								// the capability of 3D denoise
	BOOL						bSupportAlgorithm1;						// support denoise algorithm 1 or not
	int							nTnfLevelRangeMin;						// the min level of time renge
	int							nTnfLevelRangeMax;						// the max level of time renge
	int							nSnfLevelRangeMin;						// the min level of space renge
	int							nSnfLevelRangeMax;						// the min level of space renge
	int							nSeniotTypeCount;						// the count of seniot type
	NET_EM_3DSUPPORT_TYPE		emSeniorType[MAX_3DTYPE_COUNT];			// the list of seniot type
} NET_OUT_VIDEOIN_DENOISE_CAPS;

// CLIENT_GetDevCaps NET_VIDEOIN_BACKLIGHT_CAPS input parameter
typedef struct tagNET_IN_VIDEOIN_BACKLIGHT_CAPS
{
	DWORD				dwSize; 								// the size of this struct
	int 				nChannel;								// channel ID
} NET_IN_VIDEOIN_BACKLIGHT_CAPS;

// back mode
typedef enum tagNET_EM_BACK_MODE
{
	NET_EM_BACKLIGHT_MODE_UNKNOW,					// unknow
	NET_EM_BACKLIGHT_MODE_OFF,						// off
	NET_EM_BACKLIGHT_MODE_BACKLIGHT,				// back light
	NET_EM_BACKLIGHT_MODE_WIDEDYNAMIC,				// widedynamic
	NET_EM_BACKLIGHT_MODE_GLAREINHIBITION,			// glareinhibition
	NET_EM_BACKLIGHT_MODE_SSA,						// SSA
} NET_EM_BACK_MODE;

// back light mode
typedef enum tagNET_EM_BLACKLIGHT_MODE
{
	NET_EM_BLACKLIGHT_UNKNOW,						// unkonw
	NET_EM_BLACKLIGHT_DEFAULT,						// default
	NET_EM_BLACKLIGHT_REGION,						// region
} NET_EM_BLACKLIGHT_MODE;

// the mode of Intensity
typedef enum tagNET_EM_BACK_INTENSITY_MODE
{
	NET_EM_INTENSITY_UNKNOWN,			// unkonw
	NET_EM_INTENSITY_OFF,				// Disable 
	NET_EM_INTENSITY_AUTO,				// AutoIntensity 
	NET_EM_INTENSITY_MANUAL,			// ManualIntensity
} NET_EM_BACK_INTENSITY_MODE;

// the mode of hightlight suppression
typedef enum tagNET_EM_GLAREINHIBITION_MODE
{
	NET_EM_GLAREINHIBITION_UNKNOWN,		// unknown
	NET_EM_GLAREINHIBITION_DEFAULT,		// default
	NET_EM_GLAREINHIBITION_FPGA,		// Fpga
} NET_EM_GLAREINHIBITION_MODE;

#define MAX_MODE_COUNT		8		// the max count of hightlight suppression mode

// CLIENT_GetDevCaps NET_VIDEOIN_BACKLIGHT_CAPS output parameter
typedef struct tagNET_OUT_VIDEOIN_BACKLIGHT_CAPS
{
	DWORD						dwSize; 								// the size of this struct
	BOOL						bSupport;								// support setting bcaklight or not
	int							nBackModeCount;							// the actual count of black mode
	NET_EM_BACK_MODE			emBackMode[MAX_MODE_COUNT];				// the list of black mode
	int							nBackLightModeCount;					// the actual count of black light mode
	NET_EM_BLACKLIGHT_MODE		emBackLightMode[MAX_MODE_COUNT];		// the list of black light mode
	int							nWideDynamicRange;						// support wide dynamic range or not
	int							nSSAIntensity;							// the count of SSA modes
	NET_EM_BACK_INTENSITY_MODE	emIntensityMode[MAX_MODE_COUNT];		// the list of SSA modes
	NET_EM_GLAREINHIBITION_MODE	emGlareInhibition[MAX_MODE_COUNT];		// the list of glareInhibition modes
} NET_OUT_VIDEOIN_BACKLIGHT_CAPS;

// CLIENT_GetDevCaps NET_VIDEOIN_WHITEBALANCE_CAPS input parameter
typedef struct tagNET_IN_VIDEOIN_WHITEBALANCE_CAPS
{
	DWORD				dwSize; 								// the size of this struct
	int 				nChannel;								// channel ID
} NET_IN_VIDEOIN_WHITEBALANCE_CAPS;

// white balance type
typedef enum tagNET_EM_WHITEBALANCE_TYPE
{
	NET_EM_WHITEBALANCE_UNKNOW,					// unkonw
	NET_EM_WHITEBALANCE_DISABLE,				// Disable
	NET_EM_WHITEBALANCE_AUTO,					// Auto
	NET_EM_WHITEBALANCE_SUNNY,					// Sunny, about 6500K
	NET_EM_WHITEBALANCE_CLOUDY,					// Cloudy,7500K
	NET_EM_WHITEBALANCE_HOME,					// Home, about 5000K
	NET_EM_WHITEBALANCE_OFFICE,					// Office, about 4400K
	NET_EM_WHITEBALANCE_NIGHT,					// Night, about 2800K
	NET_EM_WHITEBALANCE_CUSTOM,					// Custom
	NET_EM_WHITEBALANCE_HIGHCOLORTEMP,			// HighColorTemperature
	NET_EM_WHITEBALANCE_LOWCOLORTEMP,			// LowColorTemperature
	NET_EM_WHITEBALANCE_AUTOCOLORTEMP,			// AutoColorTemperature
	NET_EM_WHITEBALANCE_CUSTOMCOLORTEMP,		// CustomColorTemperature
	NET_EM_WHITEBALANCE_INDOOR,					// Indoor
	NET_EM_WHITEBALANCE_OUTDOOR,				// Outdoor
	NET_EM_WHITEBALANCE_ATW,					// ATW
	NET_EM_WHITEBALANCE_MANUAL,					// Manual
	NET_EM_WHITEBALANCE_AUTOOUTDOOR,			// AutoOutdoor
	NET_EM_WHITEBALANCE_SODIUMAUTO,				// SodiumAuto
	NET_EM_WHITEBALANCE_SODIUM,					// Sodium, about 2000K
	NET_EM_WHITEBALANCE_MANUALDATUM,			// ManualDatum
	NET_EM_WHITEBALANCE_PARTWHITEBALANCE,		// PartWhiteBalance
	NET_EM_WHITEBALANCE_NATURAL,				// Natural, 2000K-12000K 
	NET_EM_WHITEBALANCE_STREETLAMP,				// StreetLamp, 1000K-5000K
} NET_EM_WHITEBALANCE_TYPE;

#define MAX_BALANCEMODES_COUNT			16		// the max counts of white balance modes

// CLIENT_GetDevCaps NET_VIDEOIN_WHITEBALANCE_CAPS output param
typedef struct tagNET_OUT_VIDEOIN_WHITEBALANCE_CAPS
{
	DWORD								dwSize; 								// the size of this struct
	BOOL								bSupport;								// support white balance or not
	int									nWhiteBalance;							// the cuonts of white balance supported
	NET_EM_WHITEBALANCE_TYPE			emWhiteBalance[MAX_BALANCEMODES_COUNT];	// the list of white balance supported
} NET_OUT_VIDEOIN_WHITEBALANCE_CAPS;

// CLIENT_GetDevCaps NET_VIDEOIN_DAYNIGHT_CAPS input param
typedef struct tagNET_IN_VIDEOIN_DAYNIGHT_CAPS
{
	DWORD				dwSize; 								// the size of this struct
	int 				nChannel;								// channel ID
} NET_IN_VIDEOIN_DAYNIGHT_CAPS;

// ICR switch type
typedef enum tagNET_EM_ICR_TYPE
{
	NET_EM_ICR_UNKONOW,			// unknow
	NET_EM_ICR_ELECTRON,		// electronics
	NET_EM_ICR_MECHANISM,		// mechanics
} NET_EM_ICR_TYPE;

// color to black
typedef enum tagNET_EM_COLORBLACK_MODE
{
	NET_EM_COLORBLACK_UNKNOWN,					// unknown
	NET_EM_COLORBLACK_COLOR,					// color
	NET_EM_COLORBLACK_BRIGHTNESS,				// automatic switching based on brightness
	NET_EM_COLORBLACK_BLACKWHITE,				// alawys black and white
	NET_EM_COLORBLACK_PHOTORESISTOR,			// photosensitive resistor
	NET_EM_COLORBLACK_GAIN,						// switch accord to gain
	NET_EM_COLORBLACK_ALARMINPUT,				// external alarm input 
	NET_EM_COLORBLACK_IO,						// external I/O input
} NET_EM_COLORBLACK_MODE;

// CLIENT_GetDevCaps NET_VIDEOIN_DAYNIGHT_CAPS output parameter
typedef struct tagNET_OUT_VIDEOIN_DAYNIGHT_CAPS
{
	DWORD						dwSize; 								// the size of this struct
	BOOL						bSupport;								// the ability to set night and day
	int							nICRType;								// the actual counts of ICR type
	NET_EM_ICR_TYPE				emICRType[MAX_MODE_COUNT];				// the list of ICR types
	int							nColorBlackMode;						// the actual counts of color to black mode
	NET_EM_COLORBLACK_MODE		emColorBlackMode[MAX_MODE_COUNT];		// the list of color to black modes
	int							nSensitivityRangeMin;					// the min of color to black sensitivity
	int							nSensitivityRangeMax;					// the max of color to black sensitivity
	int							nDelayRangeMin;							// the min delay of dat and night switching
	int							nDelayRangeMax;							// the max delay of dat and night switching
} NET_OUT_VIDEOIN_DAYNIGHT_CAPS;

// CLIENT_GetDevCaps NET_VIDEOIN_ZOOM_CAPS input parameter
typedef struct tagNET_IN_VIDEOIN_ZOOM_CAPS
{
	DWORD				dwSize; 								// the size of this struct
	int 				nChannel;								// channel ID
} NET_IN_VIDEOIN_ZOOM_CAPS;

// CLIENT_GetDevCaps NET_VIDEOIN_ZOOM_CAPS output parameter
typedef struct tagNET_OUT_VIDEOIN_ZOOM_CAPS
{
	DWORD						dwSize; 								// the size of this struct

	BOOL						bSupport;								// support zoom or not
	int							nSpeedRangeMin;							// th min speed of the zoom
	int 						nSpeedRangeMax;							// the max speed of the zoom
	BOOL						bDigitalZoomSupport;					// support digital zoom or not
	int							nZoomLimitRangeMin;						// the min data of the zoom in this speed	
	int							nZoomLimitRangeMax;						// the max data of the zoom in this speed
} NET_OUT_VIDEOIN_ZOOM_CAPS;

// CLIENT_GetDevCaps NET_VIDEOIN_FOCUS_CAPS input parameter
typedef struct tagNET_IN_VIDEOIN_FOCUS_CAPS
{
	DWORD				dwSize; 								// the size of this struct
	int 				nChannel;								// channel ID
} NET_IN_VIDEOIN_FOCUS_CAPS;

// focus mode
typedef enum tagNET_EM_FOCUS_MODE
{
	NET_EM_FOCUS_OFF,				// off 
	NET_EM_FOCUS_ASSIST,			// assist
	NET_EM_FOCUS_AUTO,				// auto
	NET_EM_FOCUS_SEMI_AUTO,			// semi auto
	NET_EM_FOCUS_MANUAL,			// manual
} NET_EM_FOCUS_MODE;

// focus limit select mode
typedef enum tagNET_EM_FOCUS_LIMITSELECT_MODE
{
	NET_EM_FOCUS_LIMITSELECT_UNKNOW,			// unknow
	NET_EM_FOCUS_LIMITSELECT_MANUAL,			// manual
	NET_EM_FOCUS_LIMITSELECT_AUTO,				// auto
} NET_EM_FOCUS_LIMITSELECT_MODE;

// focus type
typedef enum tagNET_EM_FOCUS_TYPE
{
	NET_EM_FOCUS_UNKNOWN,						// unknown
	NET_EM_FOCUS_AUTOTRACE,						// auto trace
} NET_EM_FOCUS_TYPE;

// CLIENT_GetDevCaps NET_VIDEOIN_FOCUS_CAPS output parameter
typedef struct tagNET_OUT_VIDEOIN_FOCUS_CAPS
{
	DWORD							dwSize; 								// the size of this struct
	BOOL							bSupport;								// support focus or not
	int								nFcousMode;								// the actual counts of focus modes
	NET_EM_FOCUS_MODE				emFocusMode[MAX_MODE_COUNT];			// the list of focus modes
	int								nLimitMode;								// the actual counts of focus limit modes
	NET_EM_FOCUS_LIMITSELECT_MODE	emLimitMode[MAX_MODE_COUNT];			// the list counts of focus limit modes
	BOOL							bSupportFocusRegion;					// support domain focus or not
	BOOL							bSensitivity;							// support focus sensitivity or not
	BOOL							bIRCorrection;							// support infrared focus corredtion or not
	BOOL							bFocusLimit;							// support focus limit or not
	int								nFocusTypeCount;						// the actual counts of focus modes
	NET_EM_FOCUS_TYPE				emFocusType[MAX_MODE_COUNT];			// the list counts of focus modes
} NET_OUT_VIDEOIN_FOCUS_CAPS;

//// CLIENT_GetDevCaps NET_VIDEOIN_SHARPNESS_CAPS input parameter
typedef struct tagNET_IN_VIDEOIN_SHARPNESS_CAPS
{
	DWORD				dwSize; 								// the size of this struct
	int 				nChannel;								// channel ID
} NET_IN_VIDEOIN_SHARPNESS_CAPS;

// sharpness mode
typedef enum tagNET_EM_SHARPNESS_MODE
{
	NET_EM_SHARPNESS_AUTO,				// auto
	NET_EM_SHARPNESS_MANAUL,			// manual
}NET_EM_SHARPNESS_MODE;

// CLIENT_GetDevCaps NET_VIDEOIN_SHARPNESS_CAPS output paremeter
typedef struct tagNET_OUT_VIDEOIN_SHARPNESS_CAPS
{
	DWORD						dwSize; 								// the size of this struct
	BOOL						bSupport;								// support domain sharpness or not
	int							nSharpnessMode;							// the actual counts of sharpness modes
	NET_EM_SHARPNESS_MODE		emSharpnessMode[MAX_MODE_COUNT];		// the this counts of sharpness modes
	int							nSharpnessMin;							// the min value of sharpness
	int							nSharpnessMax;							// the max value of sharpness

	BOOL						bSupportRestrain;						// support sharpness restrain or not
	int							nRestrainLevelMin;						// the min level of sharpness restrain
	int							nRestrainLevelMax;						// the max level of sharpness restrain
} NET_OUT_VIDEOIN_SHARPNESS_CAPS;

// CLIENT_GetDevCaps NET_VIDEOIN_COLOR_CAPS input parameter
typedef struct tagNET_IN_VIDEOIN_COLOR_CAPS
{
	DWORD				dwSize; 								// the size of this struct
	int 				nChannel;								// channel ID
} NET_IN_VIDEOIN_COLOR_CAPS;

// the style type of picture
typedef enum tagNET_EM_STYLE_TYPE
{
	NET_EM_STYLE_UNKONWON,			// unknown
	NET_EM_STYLE_GENTLE,			// gentle
	NET_EM_STYLE_STANDARD,			// stanard
	NET_EM_STYLE_FLAMBOYANT,		// flamboyant
} NET_EM_STYLE_TYPE;

// the capability of picture style
typedef struct tagNET_COLOR_STYLE_CAPS
{
	BOOL				bSupport;								// support seting picture style or not
	int					nStyleType;								// the actual counts of picture styles
	NET_EM_STYLE_TYPE	emStyleType[MAX_MODE_COUNT];			// the list counts of picture styles
	BYTE				bReserved[128];							// reserve
} NET_COLOR_STYLE_CAPS;

#define MAX_GRAYVALUE_COUNT			8  // the max count og gray value

// the capability of gray scale
typedef struct tagNET_GRAY_SCALE_CAPS
{
	BOOL				bSupport;								// support set gray scale or not
	int					nValueCount;							// the actual counts of gray value
	int					nValue[MAX_GRAYVALUE_COUNT][2];		// the list counts of gray value
	BYTE				bReserved[128];							// reserve
} NET_GRAY_SCALE_CAPS;

// CLIENT_GetDevCaps NET_VIDEOIN_COLOR_CAPS output parameter
typedef struct tagNET_OUT_VIDEOIN_COLOR_CAPS
{
	DWORD					dwSize; 							// the size of this struct
	BOOL					bSupport;							// support seting image attributes or not
	BOOL					bBrightness;						// support seting brightness or not
	BOOL					bContrast;							// support seting contrast or not
	BOOL					bSaturation;						// support seting stayration or not
	BOOL					bHue;								// support seting HUE or not
	BOOL					bGamma;								// support seting gamma or not
	BOOL					bChromaSuppress;					// support seting chroma suppress or not
	NET_COLOR_STYLE_CAPS	stuColorStype;						// support seting color style or not
	NET_GRAY_SCALE_CAPS		stuGrayScale;						// support seting gray scale or not
} NET_OUT_VIDEOIN_COLOR_CAPS;

// CLIENT_GetDevCaps NET_VIDEOIN_RAWFRAME_CAPS input parameter
typedef struct tagNET_IN_VIDEOIN_RAWFRAME_CAPS
{
	DWORD					dwSize;
}NET_IN_VIDEOIN_RAWFRAME_CAPS;

// YUV Data format capability set
typedef struct tagNET_RAWFRAMETYPE_DATA
{
	int						nListNum;										// The number of actual returns
	char					szList[DH_COMMON_STRING_16][DH_COMMON_STRING_8];// List of supported YUV data formats
	BYTE					byReserved[1024];
}NET_RAWFRAMETYPE_DATA;

// CLIENT_GetDevCaps NET_VIDEOIN_RAWFRAME_CAPS output parameter
typedef struct tagNET_OUT_VIDEOIN_RAWFRAME_CAPS
{
	DWORD								dwSize;
	NET_RAWFRAMETYPE_DATA				stuFrameData;
}NET_OUT_VIDEOIN_RAWFRAME_CAPS;

// temperature unit
typedef enum tagEM_TEMPERATURE_UNIT
{
	EM_TEMPERATURE_CENTIGRADE,			// centigrade
	EM_TEMPERATURE_FAHRENHEIT,			// fahrenheit
	EM_TEMPERATURE_KELVIN,				// kelvin
} EM_TEMPERATURE_UNIT;

#define MAX_UNIT_COUNT		8		// the max count of temperature units

// CLIENT_GetDevCaps NET_FACE_BOARD_CAPS output parameter
typedef struct tagNET_OUT_FACEBOARD_CAPS
{
    DWORD               	dwSize;						            // the size of this struct
    BOOL                	bHasBattery;                            // if has battery
    BOOL                	bSupportPowerVoltageDetect;             // if support power voltage detection
    BOOL					bTemperatures;							// support temperatures or not
    BOOL					bOSDTemperatureUnit;					// support temperature unit or not
    int						nRetUnitCount;							// the counts of temperature unit supported
    EM_TEMPERATURE_UNIT		emTempreatureUnit[MAX_UNIT_COUNT];		// the supported units of temperature
}NET_OUT_FACEBOARD_CAPS;

typedef enum tagEM_DETECT_VERSION_TYPE
{
    EM_DETECT_VERSION_UNKNOW = 0 , 
    EM_DETECT_VERSION_V1_0 ,           //Ver1 motion detection
    EM_DETECT_VERSION_V3_0 ,           //Ver3 motion detection
}EM_DETECT_VERSION_TYPE;
#define    MAX_DETECT_VERSION_NUM           64

typedef enum tagEM_BLIND_DETECT_VERSION_TYPE
{
    EM_BLIND_DETECT_VERSION_UNKNOW = 0 ,
    EM_BLIND_DETECT_VERSION_FULL_SCREEN ,   //Full screen detection
    EM_BLIND_DETECT_VERSION_MULTI_WINDOW ,  //Multi window detection
}EM_BLIND_DETECT_VERSION_TYPE;
#define    MAX_BLIND_DETECT_VERSION_NUM     64

typedef struct tagNET_OUT_VIDEO_DETECT_CAPS
{
        DWORD                       dwSize;         
        BOOL                        bSupportBlind;          //Is support BlindScreen detect, TRUE:yes, FALSE:no
        BOOL                        bSupportLoss;           //Is support VideoLoss detect, TRUE:yes, FALSE:no
        BOOL                        bSupportMotion;         //Is support VideoMotion detect, TRUE:yes, FALSE:no
        BOOL                        bMotionResult;          //Is support to get motion detect result in every region
        DWORD                       nMotionColumns;         //Columns of motion detect detect region
        DWORD                       nMotionRows;            //Rows of motion detect detect region
        DWORD                       nMotionDetectWindow;    //Video windows of motion detect
        DWORD                       nBlindColumns;          //Columns of video blind detect region
        DWORD                       nBlindRows;             //Rows of video blind detect region
        DWORD                       nBlindDetectWindow;     //Video windows of video blind detect
        BOOL                        bPositionDetect;        //Is support motion position detect
        DWORD                       nDetectVersionNum;      //Number of supported motion detect versions
        EM_DETECT_VERSION_TYPE      emDetectVersions[MAX_DETECT_VERSION_NUM];               //Supported motion detect versions info
        DWORD                       nBlindDetectVersionNum; //Number of supported video blind detect versions
        EM_BLIND_DETECT_VERSION_TYPE emBlindDetectVersions[MAX_BLIND_DETECT_VERSION_NUM];   //Supported video blind detect versions info
        BOOL                        bMotionLinkPtzPreset;   //Is support motion detect linked ptz preset, TRUE:yes, FALSE:no
        BOOL                        bMotionLinkPtzTour;     //Is support motion detect linked ptz tour, TRUE:yes, FALSE:no
        BOOL                        bMotionLinkPtzPattern;  //Is support motion detect linked ptz pattern, TRUE:yes, FALSE:no
        BOOL                        bUnFocusDetect;         //Is support unfocus detect, TRUE:yes, FALSE:no
        BOOL                        bAlarmDetect;           //Is support detect and trigger alarm when motion detect is working, TRUE:yes, FALSE:no
		BOOL						bSupportMoveDetect;		//Is support Moved detect, TRUE:yes, FALSE:no
}NET_OUT_VIDEO_DETECT_CAPS;

// fall event type
typedef struct tagALARM_FALLING_INFO
{
    DWORD               dwStructSize;                               // structure size
    BYTE                bEventAction;                               // event action, 0 means impulse event,1 means continuous event start,2 means continuous event end;
    BYTE                byRserved[3];                               // align                                                                                                                                                                                                                                                                        
}ALARM_FALLING_INFO;

//order Bus status call function model
typedef void (CALLBACK *fBusStateCallBack)(LLONG lAttachHandle, LONG lCommand, char *pBuf, DWORD dwBufLen, LDWORD dwUser);

// swipe card type
typedef enum tagNET_DRIVER_CHECK_TYPE
{
    NET_DRIVER_CHECK_TYPE_UNKNOWN,                                  // unknow
    NET_DRIVER_CHECK_TYPE_SIGNIN,                                   // sign up
    NET_DRIVER_CHECK_TYPE_SIGNOUT,                                  // sign out
} NET_DRIVER_CHECK_TYPE;

// swipe card
typedef enum tagNET_DRIVER_CHECK_METHOD
{
    NET_DRIVER_CHECK_METHOD_UNKNOWN,                                // unknown
    NET_DRIVER_CHECK_METHOD_POS,                                    // POS device
    NET_DRIVER_CHECK_METHOD_HAND,                                   // manually input
} NET_DRIVER_CHECK_METHOD;

// alarm event type DH_ALARM_BUS_DRIVER_CHECK( driver recognition card detection event)corresponding to data description info
typedef struct tagALARM_BUS_DRIVER_CHECK_INFO
{
    DWORD                   dwSize;
    char                    szCarNo[DH_MAX_PLATE_NUMBER_LEN];       // plate
    char                    szDriverName[DH_COMMON_STRING_16];      // driver name
    char                    szDriverID[DH_MAX_PERSON_ID_LEN];       // ID
    char                    szOrganize[DH_COMMON_STRING_128];       // licence issuing organization name
    NET_TIME_EX             stUsefulLife;                           // licence validity
    NET_GPS_STATUS_INFO     stGPSStatusInfo;                        // GPS info
    NET_TIME_EX             stCheckTime;                            // sign up time
    NET_DRIVER_CHECK_METHOD emCheckMethod;                          // sign up method
    NET_DRIVER_CHECK_TYPE   emCheckType;                            // sign up type
}ALARM_BUS_DRIVER_CHECK_INFO;

// alarm event type DH_ALARM_DEVICE_MSG_NOTIFY(device to platform notice event)corresponding to data description info
typedef struct tagALARM_DEVICE_MSG_NOTIFY_INFO
{
    DWORD                   dwSize;
    char                    szMsg[DH_COMMON_STRING_256];            // message content
    NET_GPS_STATUS_INFO     stuGPSStatusInfo;                       // GPS info
} ALARM_DEVICE_MSG_NOTIFY_INFO;

// alarm event type DH_ALARM_VEHICLE_STANDING_OVER_TIME(parking timeout alarm)corresponding to  data description info
typedef struct tagALARM_VEHICLE_STANDING_OVER_TIME_INFO
{
    DWORD                   dwSize;
    NET_GPS_STATUS_INFO     stuGPSStatusInfo;                       // GPS info
    NET_TIME_EX             stuTime;                                // first occurance time
    NET_TIME_EX             stuUtc;                                 // current event occurrence time
    DWORD                   dwUtc;                                  // current event occurrence time
    BOOL                    bEventConfirm;                          // confirm needed or not, invalid in CLIENT_StartListenEx callback
} ALARM_VEHICLE_STANDING_OVER_TIME_INFO;

// direction
typedef enum tagNET_LINE_DIRECTION
{
    NET_LINE_DIRECTION_UNKNOWN,                                     // unknown
    NET_LINE_DIRECTION_POSTIVE,                                     // firward direction
    NET_LINE_DIRECTION_NEGATIVE,                                    // opposite direction
    NET_LINE_DIRECTION_AROUND,                                      // loop
    NET_LINE_DIRECTION_DISPERSE,                                    // disperse
} NET_LINE_DIRECTION;

// in/out station status
typedef enum tagNET_BUS_STATE
{
    NET_BUS_STATE_UNKNOWN,                                         // unknown
    NET_BUS_STATE_ILLEGAL,                                         // illegal
    NET_BUS_STATE_LEGAL,                                           // legal
} NET_BUS_STATE;

// stop announcement method
typedef enum tagNET_PORT_TYPE
{
    NET_PORT_TYPE_UNKOWN,                                           // unknown
    NET_PORT_TYPE_MANUAL,                                           // manual stop announcement
    NET_PORT_TYPE_GPS,                                              // GPS stopannouncement
} NET_PORT_TYPE;

// mobile unlock event data type 
typedef enum tagEM_VEHICLE_DATA_TYPE
{
    EM_VEHICLE_DOOR_OPEN_DATA_UNKNOWN = 0,
    EM_VEHICLE_DOOR_OPEN_DATA_REALTIME,               // real-time data
    EM_VEHICLE_DOOR_OPEN_DATA_HISTORY,                // re-send data
}EM_VEHICLE_DATA_TYPE;

// alarm event type DH_ALARM_BUS_IMPORT_SITE(in station event)corresponding to data description info
typedef struct tagALARM_BUS_IMPORT_SITE_INFO
{
    DWORD                   dwSize;
    char                    szSiteID[DH_COMMON_STRING_64];          // station mark
    DWORD                   dwSiteNum;                              // stop order
    NET_LINE_DIRECTION      emDirection;                            // direction
    char                    szLineID[DH_COMMON_STRING_64];          // maek
    NET_TIME_EX             stuTime;                                // in/out station time
    int                     nTime;                                  // used to confirm event time, UTC, unit is second
    NET_BUS_STATE           emState;                                // if in/out station on time
    NET_PORT_TYPE           emType;                                 // stop announcement method
    NET_GPS_STATUS_INFO     stuGPSStatusInfo;                       // GPS info
    DWORD                   dwSiteCount;                            // total stop point
    char                    szSiteName[DH_COMMON_STRING_64];        // stop name 
    char                    szDesignation[DH_COMMON_STRING_64];     // path no.
    EM_VEHICLE_DATA_TYPE    emDataType;                             // event data type 
    BOOL                    bNeedConfirm;                           // confirm or not, via callCLIENT_BusConfirmEventport to confirm
} ALARM_BUS_IMPORT_SITE_INFO;

//alarm event type DH_ALARM_BUS_EXPORT_SITE(out station event)corresponding to data description info
typedef struct tagALARM_BUS_EXPORT_SITE_INFO
{
    DWORD                   dwSize;
    char                    szSiteID[DH_COMMON_STRING_64];          // stop mark
    DWORD                   dwSiteNum;                              // order
    NET_LINE_DIRECTION      emDirection;                            // direction
    char                    szLineID[DH_COMMON_STRING_64];          // mark
    NET_TIME_EX             stuTime;                                // in/out station time
    int                     nTime;                                  // used to confirm  event time, UTC, unit is second
    NET_BUS_STATE           emState;                                // if out station on time
    NET_PORT_TYPE           emType;                                 // stop announcement method
    NET_GPS_STATUS_INFO     stuGPSStatusInfo;                       // GPS info
    DWORD                   dwSiteCount;                            // total stop point
    char                    szSiteName[DH_COMMON_STRING_64];        // stop name 
    char                    szDesignation[DH_COMMON_STRING_64];     // path no.
    EM_VEHICLE_DATA_TYPE    emDataType;                             // event data type 
    BOOL                    bNeedConfirm;                           // confirm or not, via call CLIENT_BusConfirmEventport to confirm
} ALARM_BUS_EXPORT_SITE_INFO;

// abnormal event type 
typedef enum tagEM_ALARM_BUS_ABNORMAL_EVENT_TYPE
{
    EM_ALARM_BUS_ABNORMAL_EVENT_TYPE_UNKNOWN,      // unknown 
    EM_ALARM_BUS_ABNORMAL_EVENT_TYPE_RUNNING,      // recover operation, "Running"
    EM_ALARM_BUS_ABNORMAL_EVENT_TYPE_MEAL,         // eat, "Meal"
    EM_ALARM_BUS_ABNORMAL_EVENT_TYPE_BLOCK,        // jam, "Block"
    EM_ALARM_BUS_ABNORMAL_EVENT_TYPE_CALL,         // call, "Call"
    EM_ALARM_BUS_ABNORMAL_EVENT_TYPE_BREAKDOWN,    // breakdown, "Breakdown"
    EM_ALARM_BUS_ABNORMAL_EVENT_TYPE_DISCONTINUED, // stop operation, "Discontinued"
    EM_ALARM_BUS_ABNORMAL_EVENT_TYPE_ROBING,       // robing, "Robing"
    EM_ALARM_BUS_ABNORMAL_EVENT_TYPE_OVERLOAD,     // overload, "Overload"
    EM_ALARM_BUS_ABNORMAL_EVENT_TYPE_DISPUTE,      // dispute, "Dispute"
    EM_ALARM_BUS_ABNORMAL_EVENT_TYPE_ACCIDENT,     // event, "Accident"
    EM_ALARM_BUS_ABNORMAL_EVENT_TYPE_OVERSPEED,    // over speed, "OverSpeed"
    EM_ALARM_BUS_ABNORMAL_EVENT_TYPE_RENTAL,       // rental, "Rental"
    EM_ALARM_BUS_ABNORMAL_EVENT_TYPE_MAINTENANCE,  // maintenance, "Maintenance"
    EM_ALARM_BUS_ABNORMAL_EVENT_TYPE_CLOSURE,      // closure, "Closure"
    EM_ALARM_BUS_ABNORMAL_EVENT_TYPE_PUBSECURITY,       // "PubSecurity"
    EM_ALARM_BUS_ABNORMAL_EVENT_TYPE_ENTER_PARK,        // "EnterPark"
    EM_ALARM_BUS_ABNORMAL_EVENT_TYPE_LEAVE_PARK,        // "LeavePark"
    EM_ALARM_BUS_ABNORMAL_EVENT_TYPE_REFUEL,            // "Refuel"
    EM_ALARM_BUS_ABNORMAL_EVENT_TYPE_FIRE,              // "Fire"
    EM_ALARM_BUS_ABNORMAL_EVENT_TYPE_HELP,              // "Help"
}EM_ALARM_BUS_ABNORMAL_EVENT_TYPE;

// alarm event type DH_ALARM_BUS_ABNORMAL(vehicle abnormal event )corresponding to data description info 
typedef struct tagALARM_BUS_ABNORMAL_INFO
{
    DWORD                   dwSize;
    EM_ALARM_BUS_ABNORMAL_EVENT_TYPE    emEventType;            // abnormal event type 
    char                    szCarNo[DH_MAX_PLATE_NUMBER_LEN];   // plate
    char                    szLineID[DH_COMMON_STRING_64];      // path mark
    NET_LINE_DIRECTION      emLineDirection;                    // path direction
    NET_TIME_EX             stuTime;                            // occur time
    NET_GPS_STATUS_INFO     stuGPSStatusInfo;                   // GPSinfo 
} ALARM_BUS_ABNORMAL_INFO;

// alarm event type DH_ALARM_ENCLOSURE_ALARM(e-fence event)corresponding to data description info
typedef struct tagALARM_ENCLOSURE_ALARM_INFO
{
    DWORD                   dwSize;
    DWORD                   dwAlarmType;                            // alarm type, by bit means, 
                                                                    // 0:LimitSpeed, 1:DriveAllow, 2:ForbidDrive, 3:LoadGoods, 4:UploadGoods
    DWORD                   dwAlarmDetail;                          // alarm description, by bit means,
                                                                    // 0:DriveIn, 1:DriveOut, 2:Overspeed, 3:SpeedClear
    NET_BUS_STATE           emState;                                // if trigger event by schedules time
    DWORD                   dwDriverNo;                             // driver no.
    DWORD                   dwEnclosureID;                          // fence ID
    DWORD                   dwLimitSpeed;                           // speed limit
    DWORD                   dwCurrentSpeed;                         // current speed
    NET_TIME_EX             stuTime;                                // current time
    NET_GPS_STATUS_INFO     stuGPSStatusInfo;                       // GPS info
} ALARM_ENCLOSURE_ALARM_INFO;

// mobile door status 
typedef NET_ACCESS_CTL_STATUS_TYPE NET_VEHICLE_DOOR_STATUS;

// mobile unlock event , corresponding to event type DH_VEHICLE_DOOR_OPEN
typedef struct tagALARM_VEHICLE_DOOR_OPEN_INFO 
{
    DWORD                   dwSize;
    NET_VEHICLE_DOOR_STATUS emStatus;           // door status 
    int                     nDoor;              // door no., 1:front door,2:middle door,3:rear door
    NET_TIME                stuTime;            // event time, UTC
    int                     nTime;              // used to confirm  event used time, UTC, unit is second
    NET_GPS_STATUS_INFO     stuGPSStatusInfo;   // GPSinfo 
    EM_VEHICLE_DATA_TYPE    emDataType;         // event  data  type 
    BOOL                    bNeedConfirm;       // confirm or not, via call CLIENT_BusConfirm Eventport to confirm
}ALARM_VEHICLE_DOOR_OPEN_INFO;

//The current event, the corresponding event type  DH_ALARM_BUS_CUR_MILEAGE
typedef struct tagALARM_BUS_CUR_MILEAGE_INFO 
{
    DWORD                       dwSize;
    BOOL                        bNeedConfirm;           // Whether need confirm, confirm it by CLIENT_BusConfirmEvent
    int                         nTime;                  // Time for confirming the event, UTC, The unit is in seconds
    EM_VEHICLE_DATA_TYPE        emDataType;             // The type of event data
    NET_TIME                    stuTime;                // Event time, UTC
    NET_GPS_STATUS_INFO         stuGPSStatusInfo;       // GPS information, only longitude and latitude /speed and direction angle valid
    NET_TIME                    stuStartTime;           // The beginning of the current period of time, UTC
    NET_GPS_STATUS_INFO         stuStartGPSStatusInfo;  // The informations of the GPS on the beginning of the current period of time
    unsigned int                nMileage;               // Number of miles the current period, unit:0.1km
}ALARM_BUS_CUR_MILEAGE_INFO;

// The current consumption situation, A corresponding event type DH_ALARM_BUS_CUR_OIL
typedef struct tagALARM_BUS_CUR_OIL_INFO 
{
    DWORD                       dwSize;
    BOOL                        bNeedConfirm;           // Whether need confirm, confirm it by CLIENT_BusConfirmEvent
    int                         nTime;                  // Time for confirmmingevent, UTC, unit:seconds
    EM_VEHICLE_DATA_TYPE        emDataType;             // The type of the event data
    NET_TIME                    stuTime;                // Event time, UTC
    NET_GPS_STATUS_INFO         stuGPSStatusInfo;       // GPS information, only longitude and latitude /speed and direction angle valid
    NET_TIME                    stuStartTime;           // The begining time of the current, UTC
    NET_GPS_STATUS_INFO         stuStartGPSStatusInfo;  // The GPS informations at the beginning time of the current, only longitude and latitude /speed and direction angle valid
    unsigned int                nCurOil;                // Current oil, unit: 0.1L
    unsigned int                nOilTankage;            // Tank capacity, unit: 0.1L
    int                         nOilChange;             // The changes oil of current time, unit: 0.1L, Positive means refuel, Negative means oil consumption
    char                        szCarNO[DH_MAX_PLATE_NUMBER_LEN]; // plate
}ALARM_BUS_CUR_OIL_INFO;

// Low oil alarm events, Corresponding event type DH_ALARM_BUS_LOW_OIL
typedef struct tagALARM_BUS_LOW_OIL_INFO 
{
    DWORD                       dwSize;
    BOOL                        bNeedConfirm;           // Whether need confirm, confirm it by CLIENT_BusConfirmEvent
    int                         nTime;                  // Time for confirming the event, UTC, The unit is in seconds
    EM_VEHICLE_DATA_TYPE        emDataType;             // The type of the event data
    NET_TIME                    stuTime;                // Event time, UTC
    NET_GPS_STATUS_INFO         stuGPSStatusInfo;       // GPS information, only longitude and latitude /speed and direction angle valid
    unsigned int                nCurOil;                // Current oil, unit: 0.1L
    unsigned int                nOilLine;               // Oil threshold, unit: 0.1L
    unsigned int                nOilTankage;            // Tank capacity, unit: 0.1L
    char                        szCarNO[DH_MAX_PLATE_NUMBER_LEN]; // plate
}ALARM_BUS_LOW_OIL_INFO;

// Steal oil alarm events, Corresponding evnet type DH_ALARM_BUS_STEAL_OIL
typedef struct tagALARM_BUS_STEAL_OIL_INFO 
{
    DWORD                       dwSize;
    BOOL                        bNeedConfirm;           // Whether need confirm, confirm it by CLIENT_BusConfirmEvent
    int                         nTime;                  // Time for confirming the event, UTC, The unit is in seconds
    EM_VEHICLE_DATA_TYPE        emDataType;             // The type of the event data
    NET_TIME                    stuTime;                // Event time, UTC
    NET_GPS_STATUS_INFO         stuGPSStatusInfo;       // GPS information, only longitude and latitude /speed and direction angle valid
    unsigned int                nCurOil;                // Current oil, unit: 0.1L
    unsigned int                nOilTankage;            // Tank capacity, unit: 0.1L
    char                        szCarNO[DH_MAX_PLATE_NUMBER_LEN]; // plate
}ALARM_BUS_STEAL_OIL_INFO;

// order Bus status input structure
typedef struct tagNET_IN_BUS_ATTACH
{
    DWORD                   dwSize;
    fBusStateCallBack       cbBusState;                             // status call function
    LDWORD                  dwUser;                                 // user data
}NET_IN_BUS_ATTACH;

// order Bus status output structure
typedef struct tagNET_OUT_BUS_ATTACH
{
    DWORD                dwSize;
}NET_OUT_BUS_ATTACH;

// extention module alarm channel info
typedef struct tagNET_EXALARMCHANNELS_INFO
{
    DWORD               dwSize;
    int                 nExAlarmBoxNum;                             // extension alarm box channel no.
    int                 nChannelNum;                                // ths channelno. On alarm box
    char                szChannelName[DH_MAX_EXALARMCHANNEL_NAME_LEN];// alarm channel name 
} NET_EXALARMCHANNELS_INFO;

// CLIENT_QueryDevState port input parameter
typedef struct tagNET_EXALARMCHANNELS
{
    DWORD                       dwSize;
    int                         nExAlarmInCount;                    // extension  module alarm input channel quantity, need user search
    int                         nRetExAlarmInCount;                 // extension  module alarm input channel return quantity
    NET_EXALARMCHANNELS_INFO*   pstuExAlarmInInfo;                  // extension  module alarm input channel info
																	// the space application by the user, apply to sizeof(NET_EXALARMCHANNELS_INFO)*nExAlarmInCount
	int                         nExAlarmOutCount;                   // extension  module alarm output channel quantity, need user to search
    int                         nRetExAlarmOutCount;                // extension  module alarm output channel return quantity
    NET_EXALARMCHANNELS_INFO*   pstuExAlarmOutInfo;                 // extension  module alarm output channel info
																	// the space application by the user, apply to sizeof(NET_EXALARMCHANNELS_INFO)*nExAlarmOutCount
} NET_EXALARMCHANNELS;

// activate zone info
typedef struct tagNET_ACTIVATEDDEFENCEAREA_INFO
{
    DWORD                       dwSize;
    int                         nChannel;                           // zone channel no.
    NET_TIME                    stuActivationTime;                  // zone activation time 
}NET_ACTIVATEDDEFENCEAREA_INFO;
// CLIENT_QueryDevState port input parameter
typedef struct tagNET_ACTIVATEDEFENCEAREA
{
    DWORD                       dwSize;
    int                         nAlarmInCount;                      // search local alarm input channel quantity,  need use to specify
    int                         nRetAlarmInCount;                   // local alarm input channel actual activation quantity
    NET_ACTIVATEDDEFENCEAREA_INFO* pstuAlarmInDefenceAreaInfo;      // local alarm input channel info
																	// the space application by the user,apply to sizeof(NET_ACTIVATEDDEFENCEAREA_INFO)*nAlarmInCount
    int                         nExAlarmInCount;                    // search extension module alarm input quantity. Use specify
    int                         nRetExAlarmInCount;                 // extension  module alarm input channel actual activation quantity
    NET_ACTIVATEDDEFENCEAREA_INFO* pstuExAlarmInDefenceAreaInfo;    // extension  module alarm input channel info
																	// the space application by the user,apply to sizeof(NET_ACTIVATEDDEFENCEAREA_INFO)*nExAlarmInCount
}NET_ACTIVATEDDEFENCEAREA;

// access control status type
typedef enum tagEM_NET_DOOR_STATUS_TYPE
{
    EM_NET_DOOR_STATUS_UNKNOWN,
    EM_NET_DOOR_STATUS_OPEN,                            // door unlock
    EM_NET_DOOR_STATUS_CLOSE,                           // door lock
    EM_NET_DOOR_STATUS_BREAK,                           // door abnormal unlock
}EM_NET_DOOR_STATUS_TYPE;

// access control status info(CLIENT_QueryDevState port input parameter)
typedef struct tagNET_DOOR_STATUS_INFO
{
    DWORD                       dwSize;
    int                         nChannel;               // access control channel no.
    EM_NET_DOOR_STATUS_TYPE     emStateType;            // access control status info
}NET_DOOR_STATUS_INFO;

// CLIENT_QueryRecordCount port input parameter
typedef struct _NET_IN_QUEYT_RECORD_COUNT_PARAM
{
    DWORD                       dwSize;                 // structure size
    LLONG                       lFindeHandle;           // search handle
}NET_IN_QUEYT_RECORD_COUNT_PARAM;

// CLIENT_QueryRecordCount port output parameter
typedef struct _NET_OUT_QUEYT_RECORD_COUNT_PARAM
{
    DWORD                       dwSize;                 // structure size
    int                         nRecordCount;           // device return record item
}NET_OUT_QUEYT_RECORD_COUNT_PARAM;

// analog alarm input channel info
typedef struct tagNET_ANALOGALARM_CHANNELS_INFO 
{
    DWORD               dwSize;
    int                 nSlot;                          	// root address, 0 means local channel, 1 means connection to 1st serial extention channel, 2,3...and so on
    int                 nLevel1;                        	// 1st level cascading address, means connection to no.nSlot serial no.nLevel1 detector, from 0
    int                 nLevel2;                        	// 2nd level cascading address,means cascading address, means connection to no.nLevel1 node no.nLevel2 detector, from 0, -1 means inexist
    char                szName[DH_COMMON_STRING_128];   	// channel name 
	char				szSenseMethod[DH_COMMON_STRING_32];	// Sensor sensing mode
	BYTE				byIsNull;							// Distinguish weather the channel is empty, 0 is not vacant, 1 is vacant
}NET_ANALOGALARM_CHANNELS_INFO;

// analog alarm input channel mapping relation (correspondingDH_DEVSTATE_ANALOGALARM_CHANNELS command)
typedef struct tagNET_ANALOGALARM_CHANNELS 
{
    DWORD                          dwSize;
    int                            nMaxAnalogAlarmChannels; // max channel
    int                            nRetAnalogAlarmChannels; // return channel
    NET_ANALOGALARM_CHANNELS_INFO* pstuChannelInfo;         // channel info, userallocate memory, apply to sizeof(NET_ANALOGALARM_CHANNELS_INFO)*nMaxAnalogAlarmChannels
	int							   nVideoChannel;			// specify the video channel number
}NET_ANALOGALARM_CHANNELS;

// sensor data info
typedef struct tagNET_ANALOGALARM_SENSE_INFO 
{
    DWORD                   dwSize;
    int                     nChannelID;                     // channel no.(from 0)
    NET_SENSE_METHOD        emSense;                        // sensor type
    float                   fData;                          // sensor value
    NET_TIME                stuTime;                        // collection time
    int                     nStatus;                        // data status, -1:unknown,0:normal,1data invalid(over measure),
                                                            // 2: over threshold1,3: over threshold 2,4: over threshold 3,5: over threshold 4,
                                                            // 6:below threshold1,7: below threshold 2,8: below threshold 3,9: below threshold 4
    NET_GPS_STATUS_INFO     stuGpsSatus;                    // GPS status 
    int                     nUint;                          // Unit of the sensor value, it is valid when emSense is the following value
                                                            // NET_SENSE_TEMP, refer to NET_TEMPERATURE_UNIT for the value. 
	int						nVideoChannel;					// video channel
}NET_ANALOGALARM_SENSE_INFO;

//subscribe analog alarm channel data callback function original
typedef void (CALLBACK *fAnalogAlarmDataCallBack)(LLONG lLoginID, LLONG lAttachHandle, NET_ANALOGALARM_SENSE_INFO* pInfo, int nBufLen, LDWORD dwUser);
// CLIENT_AttachAnalogAlarmData() port input parameter
typedef struct tagNET_IN_ANALOGALARM_DATA 
{
    DWORD                       dwSize;
    int                         nChannelId;                 // from 0, -1 means all channels
    fAnalogAlarmDataCallBack    cbCallBack;                 // data callback function
    LDWORD                      dwUser;                     // user custom parameter

}NET_IN_ANALOGALARM_DATA;

// CLIENT_AttachAnalogAlarmData()port output parameter
typedef struct tagNET_OUT_ANALOGALARM_DATA 
{
    DWORD    dwSize;
}NET_OUT_ANALOGALARM_DATA;

// order  record change port definition
#define DH_MAX_RECORD_INSERT_NUM              128
#define DH_MAX_RECORD_DELETE_NUM              128
#define DH_MAX_RECORD_UPDATE_NUM              128

typedef struct tagNET_RECORDUPDATER_INFO
{
    DWORD           dwSize;
    int             nInsertNum;                   // record info , inserted  record number 
    int             nInsertNumberList[DH_MAX_RECORD_INSERT_NUM];  // means new record  index       
    int             nDeleteNum;                   // record info , deleted record number 
    int             nDeleteNumberList[DH_MAX_RECORD_DELETE_NUM];  // means deleted record  index , -1 means delete all record     
    int             nUpdateNum;                   // record info , updated record number 
    int             nUpdateNumberList[DH_MAX_RECORD_UPDATE_NUM];  // means updated record  index      
}NET_RECORDUPDATER_INFO;

//order record change port 
typedef void (CALLBACK *fRecordUpdaterCallBack)(LLONG lLoginID, LLONG lAttachHandle, NET_RECORDUPDATER_INFO* pInfo, int nBufLen, LDWORD dwUser);

// CLIENT_AttachRecordUpdater()port input parameter 
typedef struct tagNET_IN_RECORDUPDATER_DATA
{
    DWORD               dwSize;
    EM_NET_RECORD_TYPE  emType;           // record  set info type 
    fRecordUpdaterCallBack   cbRecordUpdater;         // record update call function
    LDWORD              dwUser;           // user defnition parameter 
}NET_IN_RECORDUPDATER_DATA;

// CLIENT_AttachRecordUpdater()port  output parameter 
typedef struct tagNET_OUT_RECORDUPDATER_DATA
{
    DWORD               dwSize;
}NET_OUT_RECORDUPDATER_DATA;


// search device supported sensor type 
// CLIENT_QueryDevState port DH_DEVSTATE_GET_SENSORLIST  command parameter
#define MAX_SUPPORT_SENSORTYPE_NUM    128                   // max supported sensor device type

typedef struct tagNET_SENSOR_LIST 
{
    DWORD       dwSize;
    int         nSupportSensorNum;                          // return sensor device type
    char        szSensorList[MAX_SUPPORT_SENSORTYPE_NUM][DH_COMMON_STRING_64];
}NET_SENSOR_LIST;

// CLIENT_QueryDevLogCount get log item input parameter
typedef struct tagNET_IN_GETCOUNT_LOG_PARAM
{
    DWORD                       dwSize;
    QUERY_DEVICE_LOG_PARAM      stuQueryCondition;          // search record filter
} NET_IN_GETCOUNT_LOG_PARAM;

// CLIENT_QueryDevLogCount get log item output parameter
typedef struct tagNET_OUT_GETCOUNT_LOG_PARAM
{
    DWORD                       dwSize;
    int                         nLogCount;                  // log quantity(device return)
} NET_OUT_GETCOUNT_LOG_PARAM;


// SDK global log print
typedef struct tagLogSetPrintInfo
{
    DWORD           dwSize;
    BOOL            bSetFilePath;                           // reset log path
    char            szLogFilePath[MAX_LOG_PATH_LEN];        // log path(default"./sdk_log/sdk_log.log")
    BOOL            bSetFileSize;                           // reset log size
    unsigned int    nFileSize;                              // each log file size(default size 10240), unit:bit
    BOOL            bSetFileNum;                            // reset log file number
    unsigned int    nFileNum;                               // log file quantity(default size 10)
    BOOL            bSetPrintStrategy;                      // reset log print strategy
    unsigned int    nPrintStrategy;                         // log out strategy, 0: output to file(defualt); 1:output to window
}LOG_SET_PRINT_INFO;

//////////////////////////////////////////////////////////////////////////
//       Low Rate Wireless Personal Area Network low speed wireless private network begin
//////////////////////////////////////////////////////////////////////////

// Wireless Device Type
typedef enum tagNET_WIRELESS_DEVICE_TYPE
{
	NET_WIRELESS_DEVICE_TYPE_UNKNOWN = 0,				
	NET_WIRELESS_DEVICE_TYPE_KEYBOARD,			// Wireless keyboard
	NET_WIRELESS_DEVICE_TYPE_DEFENCE,			// Wireless zone 
	NET_WIRELESS_DEVICE_TYPE_REMOTECONTROL,		// Wireless remote control 
	NET_WIRELESS_DEVICE_TYPE_MAGNETOMER,		// Wireless door sensor 
	NET_WIRELESS_DEVICE_TYPE_ALARMBELL,			// Wireless alarm bell
	NET_WIRELESS_DEVICE_TYPE_SWITCHER,          // Wireless switcher
    NET_WIRELESS_DEVICE_TYPE_SMARTLOCK,         // Wireless smart lock
    NET_WIRELESS_DEVICE_TYPE_REPEATER,          // Wireless Repeater
} NET_WIRELESS_DEVICE_TYPE;

// Wireless Device Mode
typedef enum tagEM_WIRELESS_DEVICE_MODE
{
    EM_WIRELESS_DEVICE_MODE_UNKNOWN = 0,               
    EM_WIRELESS_DEVICE_MODE_NORMAL,                // Normal  
    EM_WIRELESS_DEVICE_MODE_POLLING,               // Polling    It only to be valid when emType is NET_WIRELESS_DEVICE_TYPE_REMOTECONTROL
} EM_WIRELESS_DEVICE_MODE;

// the type of sense method
typedef enum tagEM_CODEID_SENSE_METHOD_TYPE
{
    EM_CODEID_SENSE_METHOD_TYPE_UNKOWN,         // Unknown
    EM_CODEID_SENSE_METHOD_TYPE_DOOR_MAGNETISM, // Door Magnetism
    EM_CODEID_SENSE_METHOD_TYPE_GAS_SENSOR,     // Gas Sensor
    EM_CODEID_SENSE_METHOD_TYPE_CURTAIN_SENSOR, // Curtain Sensor
    EM_CODEID_SENSE_METHOD_TYPE_MOBILE_SENSOR,  // Mobile Sensor
    EM_CODEID_SENSE_METHOD_TYPE_PASSIVEINFRA,   // Passive Infrared Sensor
    EM_CODEID_SENSE_METHOD_TYPE_URGENCY_BUTTON, // Urgency Button
    EM_CODEID_SENSE_METHOD_TYPE_SMOKING_SENSOR, // Smoking Sensor
    EM_CODEID_SENSE_METHOD_TYPE_DOUBLEMETHOD,   // DoubleMethod(Infrare+Microwave)
    EM_CODEID_SENSE_METHOD_TYPE_WATER_SENSOR,   // Water Sensor
    EM_CODEID_SENSE_METHOD_TYPE_THREEMETHOD,    // Three Sensor
}EM_CODEID_SENSE_METHOD_TYPE;

#define DH_WIRELESS_DEVICE_SERIAL_NUMBER_MAX_LEN 32    // the max length of wireless device SN

// Code info
typedef struct tagNET_CODEID_INFO
{
	DWORD						dwSize;
	TP_U64	                    nWirelessId;					   // Wireless ID no.
	NET_WIRELESS_DEVICE_TYPE	emType;							   // Wireless device type
	char						szName[DH_USER_NAME_LENGTH];	   // Username
	BOOL						bEnable;						   // Enable this device
    char                        szCustomName[DH_COMMON_STRING_64]; // Custom Name
    int                         nChannel;                          // It only to be valid when emType is NET_WIRELESS_DEVICE_TYPE_DEFENCE
    EM_WIRELESS_DEVICE_MODE     emMode;                            // Wireless Device Mode.
    EM_CODEID_SENSE_METHOD_TYPE emSenseMethod;                     // The sense method
    char                        szSerialNumber[DH_WIRELESS_DEVICE_SERIAL_NUMBER_MAX_LEN]; // Wireless Device SN
}NET_CODEID_INFO;

// Code error type
typedef enum tagNET_CODEID_ERROR_TYPE
{	
	NET_CODEID_ERROR_TYPE_RIGHT = 0,			// Code correct
	NET_CODEID_ERROR_TYPE_ALREADYEXIST,			// Exists
	NET_CODEID_ERROR_TYPE_OTHER,				// Other error
} NET_CODEID_ERROR_TYPE;

// Order wireless code info call function origin, lAttachHandle is CLIENT_AttachLowRateWPAN return valud
typedef void (CALLBACK *fAttachLowRateWPANCB) (LLONG lLoginID, LLONG lAttachHandle, NET_CODEID_INFO* stuBuf, NET_CODEID_ERROR_TYPE emError, LDWORD dwUser);

// CLIENT_AttachLowRateWPAN() input parameter
typedef struct tagNET_IN_ATTACH_LOWRATEWPAN
{
	DWORD					dwSize;
	fAttachLowRateWPANCB	cbAttachLowRateWPANCB;         // Code data call
	LDWORD					dwUser;                        // User Data
}NET_IN_ATTACH_LOWRATEWPAN;

// CLIENT_AttachLowRateWPAN() output parameter
typedef struct tagNET_OUT_ATTACH_LOWRATEWPAN
{
	DWORD		        dwSize;
}NET_OUT_ATTACH_LOWRATEWPAN;

// Delete specific wireless device
// CLIENT_ControlDevice port DH_CTRL_LOWRATEWPAN_REMOVE command parameter
typedef struct tagNET_CTRL_LOWRATEWPAN_REMOVE
{
	DWORD					dwSize;
	TP_U64			        nWirelessId;				// Wireless device ID
    char                    szSerialNumber[DH_WIRELESS_DEVICE_SERIAL_NUMBER_MAX_LEN]; // Wireless device SN
}NET_CTRL_LOWRATEWPAN_REMOVE;

// Delete all wireless device
// CLIENT_ControlDevice port DH_CTRL_LOWRATEWPAN_REMOVEALL command parameter
typedef struct tagNET_CTRL_LOWRATEWPAN_REMOVEALL
{
	DWORD					dwSize;
}NET_CTRL_LOWRATEWPAN_REMOVEALL;

// Modify wireless device info
// CLIENT_ControlDevice port DH_CTRL_LOWRATEWPAN_MODIFY command parameter
typedef struct tagNET_CTRL_LOWRATEWPAN_MODIFY
{
	DWORD					dwSize;
	NET_CODEID_INFO			stuCodeIDInfo;			// According to CodeIDInfo CodeID, modify its UserandEnable indo, cannot modify others
}NET_CTRL_LOWRATEWPAN_MODIFY;

//Setting the working mode of the detector
//CLIENT_ControlDevice port DH_CTRL_LOWRATEWPAN_SETWORKMODE command parameter
typedef struct tagNET_IN_CTRL_LOWRATEWPAN_SETWORKMODE
{
	DWORD                   dwSize;
	int                     nMode;                 //mode 0 :power saving mode,  1 : working mode

}NET_IN_CTRL_LOWRATEWPAN_SETWORKMODE;

// car pass status
typedef enum tagEM_CARPASS_STATUS
{
	EM_CARPASS_STATUS_UNKNOWN,     // Unknown status     
	EM_CARPASS_STATUS_CARPASS,     // car pass status
	EM_CARPASS_STATUS_NORMAL,      // no car status
}EM_CARPASS_STATUS;

//Set park info, corresponding DH_CTRL_SET_PARK_INFO command parameter
typedef struct tagNET_CTRL_SET_PARK_INFO
{
	DWORD				   dwSize;
	char				   szPlateNumber[MAX_PLATE_NUMBER_LEN];         // Plate number
	unsigned int		   nParkTime;									// park time,Unit:minute
	char				   szMasterofCar[MAX_MASTER_OF_CAR_LEN];		// Master of car
	char				   szUserType[MAX_USER_TYPE_LEN];				// User type,not general,Used in entrance capture machine
																		// monthlyCardUser means monthly card user,yearlyCardUser means yearly card user,longTimeUser means long time user/VIP,casualUser means casual user/Visitor
	unsigned int		   nRemainDay;									// Remain day
	char				   szParkCharge[MAX_PARK_CHARGE_LEN];			// Park charge
	unsigned int		   nRemainSpace;								// Remain space
	unsigned int		   nPassEnable;									// 0:car is not allowed to pass,1:car is allowed to pass
	NET_TIME			   stuInTime;									// car in time
	NET_TIME			   stuOutTime;									// car out time
	EM_CARPASS_STATUS	   emCarStatus;									// car pass status
	char				   szCustom[MAX_CUSTOM_LEN];					// custom field,default:null
	char				   szSubUserType[MAX_SUB_USER_TYPE_LEN];		// Sub user type of szUserType
	char				   szRemarks[MAX_REMARKS_LEN];					// Remarks info
}NET_CTRL_SET_PARK_INFO;

// clear the statistics for the period and start again from 0
// corresponding DH_CTRL_CLEAR_SECTION_STAT command parameter
typedef struct tagNET_CTRL_CLEAR_SECTION_STAT_INFO
{
	DWORD                   dwSize;
	int						nChannel;		// channel ID
} NET_CTRL_CLEAR_SECTION_STAT_INFO;

// Get code successful total items
typedef struct tagNET_GET_CODEID_COUNT
{
	DWORD				dwSize;
	int					nCodeIDCount;		// Code successful total items
}NET_GET_CODEID_COUNT;

// Get code info
typedef struct tagNET_GET_CODEID_LIST
{
	DWORD				dwSize;
	int					nStartIndex;			// Start index symbol, start first search may set to 0
	int					nQueryNum;				// The gotten code items, this value issmaller than or equal to capacity set nMaxPageSize field valud
	int					nRetCodeIDNum;			// Actual returned code items
	NET_CODEID_INFO*	pstuCodeIDInfo;			// Get code content, memory is allocated by user, cannot be lower than nQueryNum*sizeof(NET_CODEID_INFO)
}NET_GET_CODEID_LIST;

// add code info
typedef struct tagNET_CTRL_LOWRATEWPAN_ADD
{
	DWORD dwSize;
	NET_CODEID_INFO stuCodeIDInfo;             // code info data
}NET_CTRL_LOWRATEWPAN_ADD;

// wireless device online state
typedef enum tagEM_WIRELESS_DEVICE_ONLINE_STATE
{   
    EM_WIRELESS_DEVICE_STATE_UNKNOWN = 0,      // unknown
    EM_WIRELESS_DEVICE_STATE_OUTLINE,          // outline
    EM_WIRELESS_DEVICE_STATE_ONLINE,           // online
} EM_WIRELESS_DEVICE_ONLINE_STATE;

// wireless device power state
typedef enum tagEM_WIRELESS_DEVICE_POWER_STATE
{   
    EM_WIRELESS_DEVICE_POWER_UNKNOWN = 0,      // unknown
    EM_WIRELESS_DEVICE_POWER_NORMAL,           // normal
    EM_WIRELESS_DEVICE_POWER_LOW,              // low
} EM_WIRELESS_DEVICE_POWER_STATE;

// wireless device info
typedef struct tagNET_WIRELESS_DEVICE_INFO
{
    char                                szSerialNumber[DH_WIRELESS_DEVICE_SERIAL_NUMBER_MAX_LEN]; // device serial NO
    EM_WIRELESS_DEVICE_ONLINE_STATE     emOnlineState;          // online state of device: 0-unknown, 1-outline, 2-online 
    EM_WIRELESS_DEVICE_POWER_STATE      emPowerState;           // current power state of device: 0-unknown power, 1-normal power, 2-low power
    BYTE                                byReserved[128];        // reserved
}NET_WIRELESS_DEVICE_INFO;

// get wireless devices info
typedef struct tagNET_GET_WIRELESS_DEVICE_STATE
{
    DWORD               dwSize;
    int                 nStartIndex;            // Start index symbol, start first search may set to 0, 
    int                 nQueryNum;              // The gotten device state items, this value is smaller than or equal to capacity set nMaxPageSize field value
    int                 nRetQueryNum;           // Actually returned items, smaller than nQueryNum
    NET_WIRELESS_DEVICE_INFO *pstuDeviceInfo;   // Get state content, memory is allocated by user, cannot be lower than nQueryNum*sizeof(NET_WIRELESS_DEVICE_INFO)
}NET_GET_WIRELESS_DEVICE_STATE;

// the state of redundance power
typedef enum tagEM_REDUNDANCE_POWER_STATE
{   
    EM_REDUNDANCE_POWER_STATE_UNKNOWN = 0,       // Unknown
    EM_REDUNDANCE_POWER_STATE_ON,                // On
    EM_REDUNDANCE_POWER_STATE_OFF,               // Off
    EM_REDUNDANCE_POWER_STATE_ON_FAULT,          // on fault
} EM_REDUNDANCE_POWER_STATE;

#define MAX_REDUNDANCE_POWER_NUM 16              // the max count of redundance power    

// get redundance power info
typedef struct tagNET_GET_REDUNDANCE_POWER_INFO
{
    DWORD                     dwSize;                                   // size in bytes of this struct
    int                       nPowerNum;                                // the num of power
    EM_REDUNDANCE_POWER_STATE emPowerState[MAX_REDUNDANCE_POWER_NUM];   // the array of power state
}NET_GET_REDUNDANCE_POWER_INFO;

// test mail input,  DH_CTRL_TEST_MAIL control type of CLIENT_ControlDeviceEx
typedef struct tagNET_IN_TEST_MAIL
{
    DWORD dwSize;
}NET_IN_TEST_MAIL;

// test mail output, DH_CTRL_TEST_MAIL control type of CLIENT_ControlDeviceEx
typedef struct tagNET_OUT_TEST_MAIL
{
    DWORD dwSize;
}NET_OUT_TEST_MAIL;

// control smart switch input , DH_CTRL_CONTROL_SMART_SWITCH control type of CLIENT_ControlDeviceEx 
typedef struct tagNET_IN_CONTROL_SMART_SWITCH
{
    DWORD                       dwSize;
    char                        szSerialNumber[DH_WIRELESS_DEVICE_SERIAL_NUMBER_MAX_LEN]; // serial number
    BOOL                        bEnable; 	// TRUE - switch on, FALSE - switch off
    int                         nDelayTime; // delay time ,  equal to zero means immediately. unit: min
}NET_IN_CONTROL_SMART_SWITCH;

// control smart switch output , DH_CTRL_CONTROL_SMART_SWITCH control type of CLIENT_ControlDeviceEx 
typedef struct tagNET_OUT_CONTROL_SMART_SWITCH
{
    DWORD                       dwSize;
}NET_OUT_CONTROL_SMART_SWITCH;

//////////////////////////////////////////////////////////////////////////
//       Low Rate Wireless Personal Area Network end
//////////////////////////////////////////////////////////////////////////

// Analog channel data search(corresponding  DH_DEVSTATE_ANALOGALARM_DATA command)
typedef struct tagNET_GET_ANALOGALARM_DATA 
{
    DWORD                       dwSize;
    int                         nChannelID;     // Analog channel no. to search(start from 0)
    NET_ANALOGALARM_SENSE_INFO  stuInfo;        // Get analog channel data
}NET_GET_ANALOGALARM_DATA;

// Monitor control and data collection device type
typedef enum tagEM_NET_SCADA_DEV_TYPE
{
    EM_NET_SCADA_DEV_TYPE_UNKNOWN = 0,                         // Unknown
    EM_NET_SCADA_DEV_TYPE_ALL,                                 // All-type device
    EM_NET_SCADA_DEV_TYPE_UPS,                                 // UPS
} EM_NET_SCADA_DEV_TYPE;

// Monitor control and data collection device spot path info input parameter, search criteria
typedef struct tagNET_IN_SCADA_POINT_LIST_INFO 
{
    DWORD                           dwSize;
    char                            szDevType[DH_COMMON_STRING_64]; // Device type
}NET_IN_SCADA_POINT_LIST_INFO;

#define MAX_SCADA_POINT_LIST_INDEX          8               // Max SCADADev config subscript number , as max channel

// Spot chart path info
typedef struct tagNET_SCADA_POINT_LIST
{
    DWORD                           dwSize;
    int               nIndexValidNum;                    // Valid config subscript number 
    int               nIndex[MAX_SCADA_POINT_LIST_INDEX];// SCADADev config config subscript value , start from 0
    char              szPath[DH_COMMON_STRING_256];      // chart complete path
}NET_SCADA_POINT_LIST;

#define MAX_SCADA_POINT_LIST_INFO_NUM       256               // max spot path number 
#define MAX_SCADA_POINT_LIST_ALARM_INFO_NUM 256				  // max spot chart alarm number

// Monitor control and data collection device spot chart path info output parameter, search result
typedef struct tagNET_OUT_SCADA_POINT_LIST_INFO 
{
    DWORD                           dwSize;
    int                             nList;                  // Valid spot chart path info number 
    NET_SCADA_POINT_LIST            stuList[MAX_SCADA_POINT_LIST_INFO_NUM];// spot chart path info
}NET_OUT_SCADA_POINT_LIST_INFO;

// Monitor control and data collection device spot chart path info, (corresponding to DH_DEVSTATE_SCADA_POINT_LIST command )
typedef struct tagNET_SCADA_POINT_LIST_INFO 
{
    DWORD                           dwSize;
    NET_IN_SCADA_POINT_LIST_INFO    stuIn;                  // search criteria 
    NET_OUT_SCADA_POINT_LIST_INFO   stuOut;                 // search result 
}NET_SCADA_POINT_LIST_INFO;

// Spot type
typedef enum tagEM_NET_SCADA_POINT_TYPE
{
    EM_NET_SCADA_POINT_TYPE_UNKNOWN = 0,                       // unknown
    EM_NET_SCADA_POINT_TYPE_ALL,                               // all type
    EM_NET_SCADA_POINT_TYPE_YC,                                // analog input
    EM_NET_SCADA_POINT_TYPE_YX,                                // switch input
    EM_NET_SCADA_POINT_TYPE_YT,                                // analog output
    EM_NET_SCADA_POINT_TYPE_YK,                                // switch ouput
} EM_NET_SCADA_POINT_TYPE;

// SCADA monitor spot search criteria 
typedef struct tagNET_IN_SCADA_INFO 
{
    DWORD                   dwSize;
    EM_NET_SCADA_POINT_TYPE emPointType;                    // spot type to search
}NET_IN_SCADA_INFO;

#define MAX_SCADA_YX_NUM                128                 // max yx type spot number
#define MAX_SCADA_YC_NUM                128                 // max yc type spot number
#define MAX_SCADA_POINT_INFO_NUM        8                   // max spot chart number

// Spot chart info 
typedef struct tagNET_SCADA_POINT_INFO 
{
    DWORD                   dwSize;
    char                    szDevName[DH_COMMON_STRING_64]; // device name
    int                     nYX;                            // valid count of YX type spot in anYX
    unsigned int            anYX[MAX_SCADA_YX_NUM];         // value of YX type spots
    int                     nYC;                            // valid count of YC type spot in afYC
    float                   afYC[MAX_SCADA_YC_NUM];         // value of YC type spots
}NET_SCADA_POINT_INFO;

// SCADA monitor spot search result 
typedef struct tagNET_OUT_SCADA_INFO 
{
    DWORD                   dwSize;
    int                     nPointInfoNum;                  // valid spot chart number 
    NET_SCADA_POINT_INFO    stuPointInfo[MAX_SCADA_POINT_INFO_NUM];// spot chart info
}NET_OUT_SCADA_INFO;

// Monitor control and data collection device spot chart path monitor spot info (corresponding to DH_DEVSTATE_SCADA_INFO command )
typedef struct tagNET_SCADA_INFO
{
    DWORD                   dwSize;
    NET_IN_SCADA_INFO       stuIn;                          // search criteria
    NET_OUT_SCADA_INFO      stuOut;                         // search result
}NET_SCADA_INFO;

// SCADA capacity type
typedef enum tagEM_NET_SCADA_CAPS_TYPE
{
    EM_NET_SCADA_CAPS_TYPE_UNKNOWN,
    EM_NET_SCADA_CAPS_TYPE_ALL,                                // all type
    EM_NET_SCADA_CAPS_TYPE_DEV,                                // DevInfo
} EM_NET_SCADA_CAPS_TYPE;

// Monitor control and data collection device spot chart path capacity info search criteria 
typedef struct tagNET_IN_SCADA_CAPS
{
    DWORD                   dwSize;
    EM_NET_SCADA_CAPS_TYPE  emType;                         // search type
} NET_IN_SCADA_CAPS;

#define MAX_NET_SCADA_CAPS_TYPE (16)
#define MAX_NET_SCADA_CAPS_NAME (16)

// Monitor control and data collection device spot chart path type capacity info 
typedef struct tagNET_OUT_SCADA_CAPS_ITEM
{
    DWORD                   dwSize;
    char                    szDevType[DH_COMMON_STRING_32];    	// device type
    int                     nValidName;                         // valid device name number
    char                    szDevName[MAX_NET_SCADA_CAPS_NAME][DH_COMMON_STRING_32];// unique device names
} NET_OUT_SCADA_CAPS_ITEM;

// Monitor control and data collection device spot chart path capacity info search result 
typedef struct tagNET_OUT_SCADA_CAPS
{
    DWORD                   dwSize;
    int                     nValidType;                         // valid device type number 
    NET_OUT_SCADA_CAPS_ITEM stuItems[MAX_NET_SCADA_CAPS_TYPE];  // Max 16 types 
} NET_OUT_SCADA_CAPS;

// Monitor control and data collection device spot chart path capacity info (corresponding to DH_DEVSTATE_SCADA_CAPS command )
typedef struct tagNET_SCADA_CAPS
{
    DWORD                   dwSize;
    NET_IN_SCADA_CAPS       stuIn;                          // search criteria 
    NET_OUT_SCADA_CAPS      stuOut;                         // search result 
} NET_SCADA_CAPS;

// spot info(queried by device id, sensor id)
typedef struct tagNET_SCADA_POINT_BY_ID_INFO 
{
    DWORD                   dwSize;
    EM_NET_SCADA_POINT_TYPE emType;                         // spot type
    char                    szID[DH_COMMON_STRING_64];      // id
    int                     nMeasuredVal;                   // measured value, valid if spot type is YX
    float                   fMeasureVal;                    // measured value, valid if spot type is YC
    int                     nSetupVal;                      // setup value, valid if spot type is YK
    float                   fSetupVal;                      // setup value, valid if spot type is YT
    int                     nStatus;                        // data status, -1:unknown, 0:normal, 1:alarm level1, 2:alarm level2, 3:alarm level3, 4:alarm level4, 5:operation event, 6:invalid data
    NET_TIME                stuTime;                        // collect time
}NET_SCADA_POINT_BY_ID_INFO;

#define MAX_SCADA_ID_OF_SENSOR_NUM  128                     // Max count of spot id

// get spot info by sensor id, spot id(corresponding to DH_DEVSTATE_SCADA_INFO_BY_ID)
typedef struct tagNET_SCADA_INFO_BY_ID
{
    DWORD                   dwSize;
    char                    szSensorID[DH_COMMON_STRING_64];// input param, sensor ID
    int                     nIDs;                           // input param, valid count of spot id in szIDs
    char                    szIDs[MAX_SCADA_ID_OF_SENSOR_NUM][DH_COMMON_STRING_64];// input param, spot IDs
    int                     nMaxCount;                      // input param, count of NET_SCADA_POINT_BY_ID_INFO in pstuInfo
    int                     nRetCount;                      // output param, returned count of NET_SCADA_POINT_BY_ID_INFO from device, may be larger than nMaxCount
    NET_SCADA_POINT_BY_ID_INFO* pstuInfo;                   // input/output param, user allocates the memory, apply to sizeof(NET_SCADA_POINT_BY_ID_INFO)*nMaxCount
}NET_SCADA_INFO_BY_ID;

// scada device id info
typedef struct tagNET_SCADA_DEVICE_ID_INFO 
{
    char                    szDeviceID[DH_COMMON_STRING_64];// device id
    char                    szDevName[DH_COMMON_STRING_64]; // device name, the same as "szDevName" in CFG_SCADA_DEV_INFO
    BYTE                    reserve[1024];
}NET_SCADA_DEVICE_ID_INFO;

// query scada device IDs(corresponding to DH_DEVSTATE_SCADA_DEVICE_LIST)
typedef struct tagNET_SCADA_DEVICE_LIST
{
    DWORD                       dwSize;
    int                         nMax;                       // count of NET_SCADA_DEVICE_ID_INFO in pstuDeviceIDInfo
    int                         nRet;                       // returned count of NET_SCADA_DEVICE_ID_INFO from device, may be larger than nMax
    NET_SCADA_DEVICE_ID_INFO*   pstuDeviceIDInfo;           // scada device id info, user allocates the memory,apply to sizeof(NET_SCADA_DEVICE_ID_INFO)*nMax
}NET_SCADA_DEVICE_LIST;

// monitor spot info 
typedef struct tagNET_SCADA_NOTIFY_POINT_INFO
{
    DWORD                       dwSize; 
    char                        szDevName[DH_COMMON_STRING_64];     // device name , and getInfo Get name identical
    EM_NET_SCADA_POINT_TYPE     emPointType;                        // spot type 
    char                        szPointName[DH_COMMON_STRING_64];   // spot name, and spot chart take value identical
    float                       fValue;                             // value, valid if spot type is YC
    int                         nValue;                             // value, valid if spot type is YX
    char                        szFSUID[DH_COMMON_STRING_64];       // FSUIID(Field Supervision Unit), device itself
    char                        szID[DH_COMMON_STRING_64];          // spot ID
    char                        szSensorID[DH_COMMON_STRING_64];    // sensor ID
    NET_TIME_EX                 stuCollectTime;                     // collect time
} NET_SCADA_NOTIFY_POINT_INFO;

// monitor spot info list
typedef struct tagNET_SCADA_NOTIFY_POINT_INFO_LIST
{
    DWORD                           dwSize;
    int                             nList;                        // monitor spot info number 
    NET_SCADA_NOTIFY_POINT_INFO     stuList[MAX_SCADA_POINT_LIST_INFO_NUM]; // monitor spot info
} NET_SCADA_NOTIFY_POINT_INFO_LIST;

// order monitor spot info  callback prototype
typedef void (CALLBACK *fSCADAAttachInfoCallBack)(LLONG lLoginID, LLONG lAttachHandle, NET_SCADA_NOTIFY_POINT_INFO_LIST *pInfo, int nBufLen, LDWORD dwUser);

// CLIENT_SCADAAttachInfo()'s input param
typedef struct tagNET_IN_SCADA_ATTACH_INFO
{
    DWORD                           dwSize;
    fSCADAAttachInfoCallBack        cbCallBack;                 // callback
    EM_NET_SCADA_POINT_TYPE         emPointType;                // spot type
    LDWORD                          dwUser;                     // user's param
} NET_IN_SCADA_ATTACH_INFO;

// CLIENT_SCADAAttachInfo()'s output param
typedef struct tagNET_OUT_SCADA_ATTACH_INFO
{
    DWORD    dwSize;
} NET_OUT_SCADA_ATTACH_INFO;

//////////////////////////////////////////////////////////////////////////
#define MAX_SCADA_ID_NUM                            1024    // max number of spot id

// spot threshold info
typedef struct tagNET_SCADA_ID_THRESHOLD_INFO 
{
    DWORD                   dwSize;
    EM_NET_SCADA_POINT_TYPE emPointType;                // spot type
    char                    szID[DH_COMMON_STRING_32];  // spot ID
    float                   fThreshold;                 // threshold to trigger alarm
    float                   fAbsoluteValue;             // absolute value of threshold
    float                   fRelativeValue;             // relative value of threshold
    int                     nStatus;                    // data state, -1:unknown, 0:normal, 1:alarm level 1, 2:alarm level 2, 3:alarm level 3, 4:alarm level 4, 5:operation event, 6:invalid data
}NET_SCADA_ID_THRESHOLD_INFO;

// CLIENT_SCADAGetThreshold's input param
typedef struct tagNET_IN_SCADA_GET_THRESHOLD 
{
    DWORD           dwSize;
    char            szDeviceID[DH_COMMON_STRING_64];                // device id
    int             nIDs;                                           // valid spot count
    char            szIDs[MAX_SCADA_ID_NUM][DH_COMMON_STRING_32];   // spot IDs to query
}NET_IN_SCADA_GET_THRESHOLD;

// CLIENT_SCADAGetThreshold's output param
typedef struct tagNET_OUT_SCADA_GET_THRESHOLD 
{
    DWORD                           dwSize;
    int                             nMax;               // number user malloced, unit:sizeof(NET_SCADA_ID_THRESHOLD_INFO)
    int                             nRet;               // return number, unit:sizeof(NET_SCADA_ID_THRESHOLD_INFO)
    NET_SCADA_ID_THRESHOLD_INFO*    pstuThresholdInfo;  // spot threshold info, user malloc the memory,apply to sizeof(NET_SCADA_ID_THRESHOLD_INFO)*nMax
}NET_OUT_SCADA_GET_THRESHOLD;

//////////////////////////////////////////////////////////////////////////
// CLIENT_SCADASetThreshold's input param
typedef struct tagNET_IN_SCADA_SET_THRESHOLD 
{
    DWORD                           dwSize;
    char                            szDeviceID[DH_COMMON_STRING_64];// device id
    int                             nMax;               // count of user malloced, unit: sizeof(NET_SCADA_ID_THRESHOLD_INFO)
    NET_SCADA_ID_THRESHOLD_INFO*    pstuThresholdInfo;  // spots threshold info, user malloc the memory,apply to sizeof(NET_SCADA_ID_THRESHOLD_INFO)*nMax
}NET_IN_SCADA_SET_THRESHOLD;

// CLIENT_SCADASetThreshold's output param
typedef struct tagNET_OUT_SCADA_SET_THRESHOLD 
{
    DWORD           dwSize;
    int             nSuccess;                                           // valid count of successfully set id
    char            szSuccessID[MAX_SCADA_ID_NUM][DH_COMMON_STRING_32]; // IDs that set successfully
    int             nFail;                                              // valid count of failed set id
    char            szFailID[MAX_SCADA_ID_NUM][DH_COMMON_STRING_32];    // IDs that set failed
}NET_OUT_SCADA_SET_THRESHOLD;

//////////////////////////////////////////////////////////////////////////
// CLIENT_StartFindSCADA's input param
typedef struct tagNET_IN_SCADA_START_FIND 
{
    DWORD           dwSize;
    NET_TIME        stuStartTime;                   // start time, must provide
    BOOL            bEndTime;                       // to set the end time, TRUE: must provide stuEndTime, FLASE: do not limit the end time
    NET_TIME        stuEndTime;                     // end time
    char            szDeviceID[DH_COMMON_STRING_64];// DeviceID, must provide
    char            szID[DH_COMMON_STRING_32];      // spot ID, must provide
}NET_IN_SCADA_START_FIND;

// CLIENT_StartFindSCADA's output param
typedef struct tagNET_OUT_SCADA_START_FIND 
{
    DWORD           dwSize;
    DWORD           dwTotalCount;                   // total count matching the finding condition
}NET_OUT_SCADA_START_FIND;

// CLIENT_DoFindSCADA's input param
typedef struct tagNET_IN_SCADA_DO_FIND 
{
    DWORD                       dwSize;
    int                         nStartNo;           // start number
    int                         nCount;             // number to query
}NET_IN_SCADA_DO_FIND;

// CLIENT_DoFindSCADA's output param
typedef struct tagNET_OUT_SCADA_DO_FIND 
{
    DWORD                       dwSize;
    int                         nRetNum;            // return number
    NET_SCADA_POINT_BY_ID_INFO* pstuInfo;           // result, user malloc the memroy, apply to sizeof(NET_SCADA_POINT_BY_ID_INFO)*nMaxNum
    int	                        nMaxNum;            // number of user malloced, unit: sizeof(NET_SCADA_POINT_BY_ID_INFO)
}NET_OUT_SCADA_DO_FIND;

// spot alarm event info
typedef struct tagNET_SCADA_NOTIFY_POINT_ALARM_INFO
{
    DWORD           dwSize;
    char            szDevID[DH_COMMON_STRING_16];               // device ID
    char            szPointID[DH_COMMON_STRING_128];            // spot ID
    BOOL            bAlarmFlag;                                 // alarm flag
    NET_TIME        stuAlarmTime;                               // alarm time
    int	            nAlarmLevel;                                // alarm level, 0~6, 
    int             nSerialNo;                                  // alarm number
    char            szAlarmDesc[DH_COMMON_STRING_128];          // alarm descriptioin
} NET_SCADA_NOTIFY_POINT_ALARM_INFO;

// spot alarm event list info
typedef struct tagNET_SCADA_NOTIFY_POINT_ALARM_INFO_LIST
{
    DWORD                               dwSize;
    int	                                nList;                                          // spot alarm event number
    NET_SCADA_NOTIFY_POINT_ALARM_INFO   stuList[MAX_SCADA_POINT_LIST_ALARM_INFO_NUM];   // spot alarm event info
} NET_SCADA_NOTIFY_POINT_ALARM_INFO_LIST;

// spot alarm event info callback prototype
typedef void (CALLBACK *fSCADAAlarmAttachInfoCallBack)(LLONG lAttachHandle, NET_SCADA_NOTIFY_POINT_ALARM_INFO_LIST *pInfo, int nBufLen, LDWORD dwUser);

// CLIENT_SCADAAlarmAttachInfo()'s input param
typedef struct tagNET_IN_SCADA_ALARM_ATTACH_INFO
{
    DWORD                           dwSize;
    fSCADAAlarmAttachInfoCallBack   cbCallBack;                 // callback
    LDWORD                          dwUser;	                    // user's param
} NET_IN_SCADA_ALARM_ATTACH_INFO;

// CLIENT_SCADAAlarmAttachInfo()'s output param
typedef struct tagNET_OUT_SCADA_ALARM_ATTACH_INFO
{
    DWORD                       dwSize;
} NET_OUT_SCADA_ALARM_ATTACH_INFO;

// spot param and value info to be set
typedef struct tagNET_SCADA_POINT_SET_INFO
{
    DWORD                       dwSize;
    EM_NET_SCADA_POINT_TYPE	    emType;                         // spot type, only support YK,YT
    char                        szPointID[DH_COMMON_STRING_64];	// spot ID
    int	                        nSetupVal;                      // value, if emType is YK
    float                       fSetupVal;                      // value, if emType is YT
} NET_SCADA_POINT_SET_INFO;

// set scada spot param and value,CLIENT_SetSCADAInfo()'s input param
typedef struct tagNET_IN_SCADA_POINT_SET_INFO_LIST
{
    DWORD                       dwSize;
    char                        szDevID[DH_COMMON_STRING_32];           // device ID
    int	                        nPointNum;                              // spot number
    NET_SCADA_POINT_SET_INFO    stuList[MAX_SCADA_ID_OF_SENSOR_NUM];    // spot info to set
} NET_IN_SCADA_POINT_SET_INFO_LIST;

// set scada spot param and value,CLIENT_SetSCADAInfo()'s output param
typedef struct tagNET_OUT_SCADA_POINT_SET_INFO_LIST
{
    DWORD                   dwSize;
    int                     nSuccess;                                                       // valid count of spot that set ok
    char                    szSuccessID[MAX_SCADA_ID_OF_SENSOR_NUM][DH_COMMON_STRING_32];   // spot list that set ok
    int                     nFail;                                                          // valid count of spot that set failed
    char                    szFailID[MAX_SCADA_ID_OF_SENSOR_NUM][DH_COMMON_STRING_32];      // spot list that set failed
} NET_OUT_SCADA_POINT_SET_INFO_LIST;


// get current sub system enable status(corresponding DH_DEVSTATE_GET_ALARM_SUBSYSTEM_ACTIVATESTATUS)
typedef struct tagNET_GET_ALARM_SUBSYSTEM_ACTIVATE_STATUES
{
	DWORD				dwSize;
	int					nChannelId;			// sub system no.
	BOOL				bActive;			// sub system enable status, , TRUE  means enable,FALSE means disable
}NET_GET_ALARM_SUBSYSTEM_ACTIVATE_STATUES;

// alarm sub system activation setup parameter(correspondingDH_CTRL_ALARM_SUBSYSTEM_ACTIVE_SET command)
typedef struct tagNET_CTRL_ALARM_SUBSYSTEM_SETACTIVE 
{
	DWORD				dwSize;
	int					nChannelId;			// sub system no.
	BOOL				bActive;			// sub system enable status ,TRUE means enable, FALSE means disable
}NET_CTRL_ALARM_SUBSYSTEM_SETACTIVE;

// Forbid open strobe param(corresponding to  DH_CTRL_FORBID_OPEN_STROBE)
typedef struct tagNET_CTRL_FORBID_OPEN_STROBE
{
    DWORD               dwSize;             
    int                 nChannelID;        // channel no.    
    int                 nTime;             // forbid open gateway duration, unit:s, as after this operation corresponding to time cannot open device gateway 
}NET_CTRL_FORBID_OPEN_STROBE;

#define MAX_PLATENUMBER_LEN    64           // max palte no length
// open gateway parameter(corresponding to DH_CTRL_OPEN_STROBE command)
typedef struct tagNET_CTRL_OPEN_STROBE
{
	DWORD				dwSize;
	int					nChannelId;			                	// channel no.
	char				szPlateNumber[MAX_PLATENUMBER_LEN];		// plate no.
}NET_CTRL_OPEN_STROBE;

// asynchronism open gateway callback function
// nError :0 means query success, 1 means memory application failure, 2 means timeout, 3 means equipment return data validation is not through, 4 means send query request failed
typedef void (CALLBACK *fOpenStrobeCallBack)(LLONG lLoginID, NET_CTRL_OPEN_STROBE stuOpenStrobe, int nError, LDWORD dwUser);


// asynchronism open gateway parameter(corresponding to DH_CTRL_OPEN_STROBE command)
typedef struct tagNET_CTRL_OPEN_STROBE_ASYN
{
    DWORD					dwSize;								// struct size
	NET_CTRL_OPEN_STROBE	stuOpenStrobe;						// open gateway parameter
	fOpenStrobeCallBack		cbCallBack;							// data callback function
	LDWORD					dwUser;								// user data
	int						nWaitTime;							// Wait time
} NET_CTRL_OPEN_STROBE_ASYN;

// close gateway parameter(corresponding to DH_CTRL_CLOSE_STROBE command)
typedef struct tagNET_CTRL_CLOSE_STROBE
{
    DWORD               dwSize;
    int                 nChannelId;                         // channel no.
}NET_CTRL_CLOSE_STROBE;

// Force breaking rule parameter(corresponding to DH_CTRL_FORCE_BREAKING command)
typedef struct tagNET_CTRL_FORCE_BREAKING
{
	DWORD               dwSize;
    int                 nChannelId;                         // channel no.
	int                 nLaneMask;                          // lane mask
	DWORD               dwBreakingType;                     // breaking type, the type of intelligent analysis event in dhnetsdk.h               
}NET_CTRL_FORCE_BREAKING;

// playback refuse listening parameter(corresponding to DH_CTRL_TALKING_REFUSE command)
typedef struct tagNET_CTRL_TALKING_REFUSE 
{
    DWORD           dwSize;
    int             nChannelID;                 // talk channel no.
}NET_CTRL_TALKING_REFUSE;

// the state of remote talk control
typedef enum tagEM_REMOTETALK_CTRL_STATE
{
    EM_REMOTETALK_CTRL_STATE_UNKNOWN = 0,                                // unknown
    EM_REMOTETALK_CTRL_STATE_INVITE,                                     // invite talk
}EM_REMOTETALK_CTRL_STATE;

// the information of remote talk control (corresponding to DH_CTRL_REMOTE_TALK command)
typedef struct tagNET_CTRL_REMOTETALK_PARAM
{
    DWORD                       dwSize;                 // size of this structure
    EM_REMOTETALK_CTRL_STATE    emAction;               // the state of remote talk control
}NET_CTRL_REMOTETALK_PARAM;


// parking reservation status
typedef enum tagEM_NET_ORDER_STATE
{
    EM_NET_ORDER_STATE_UNKNOWN,
    EM_NET_ORDER_STATE_FREE,                            // Free
    EM_NET_ORDER_STATE_ORDERED,                         // Ordered
}EM_NET_ORDER_STATE;

// parking reservation status info
typedef struct tagNET_ORDER_STATE_INFO
{
    DWORD               dwSize;
    unsigned int        nLane;                          // parking no.
    EM_NET_ORDER_STATE  emOrderState;                   // reservation status
}NET_ORDER_STATE_INFO;

// set parking reservation status parameter(corresponding to DH_CTRL_SET_ORDER_STATE command)
typedef struct tagNET_CTRL_SET_ORDER_STATE
{
    DWORD                   dwSize;
    NET_ORDER_STATE_INFO*   pOrderStateInfo;              // parking reservation info,user allocation space,apply to sizeof(NET_ORDER_STATE_INFO)*nStateCount
    unsigned int            nStateCount;                  // valid reservation number
}NET_CTRL_SET_ORDER_STATE;

// Parking lot entrance/exit controller LED roll screen subtitle speed
typedef enum tagNET_ECK_LED_SPEED_TYPE 
{
    NET_ECK_LED_SPEED_UNKNOWN = 0,
    NET_ECK_LED_SPEED_LOW,                              // Slow
    NET_ECK_LED_SPEED_NORMAL,                           // Normal
    NET_ECK_LED_SPEED_HIGH,                             // Fast
}NET_ECK_LED_SPEED_TYPE;

// Parking lot entrance/exit controller LED setup parameter DH_CTRL_ECK_LED_SET
typedef struct tagNET_CTRL_ECK_LED_SET_PARAM 
{
    DWORD                   dwSize;
    NET_ECK_LED_SPEED_TYPE  emLEDSpeed;                 // Subtitle rolling speed
    int                     nCount;                     // Valid display times, range 0-255, 0 means permanent
    char                    szCustomData[DH_COMMON_STRING_512];// LED rolling screen initial characters, 0-256 bytes
}NET_CTRL_ECK_LED_SET_PARAM;


// Intelligent parking system entrance/exit device IC card user type
typedef enum tagNET_ECK_IC_CARD_USER_TYPE
{
    NET_ECK_IC_CARD_USER_UNKNOWN,
    NET_ECK_IC_CARD_USER_ALL,                           // all type
    NET_ECK_IC_CARD_USER_TEMP,                          // Temporary user
    NET_ECK_IC_CARD_USER_LONG,                          // Long-term user
    NET_ECK_IC_CARD_USER_ADMIN,                         // admin
    NET_ECK_IC_CARD_USER_BLACK_LIST,                    // black list
}NET_ECK_IC_CARD_USER_TYPE;

//Intelligent parking lot system entrance/exit device abnormal pass car record type
typedef enum tagNET_ECK_CAR_PASS_FLAG
{
    NET_ECK_CAR_PASS_FLAG_NORMAL = 0,                   // Normal
    NET_ECK_CAR_PASS_FLAG_ABNORMAL = 1,                 // Abnormal
    NET_ECK_CAR_PASS_FLAG_ALL,                          // All
} NET_ECK_CAR_PASS_FLAG;

// Intelligent parking system entrance/exit device IC card info
typedef struct tagNET_ECK_IC_CARD 
{
    DWORD                   dwSize;
    DWORD                   dwCard;                     // card no.
    NET_ECK_IC_CARD_USER_TYPE emUserType;               // User type
}NET_ECK_IC_CARD;

#define NET_MAX_ECK_IC_CARD_IMPORT_NUM      50          // Intelligent parking system entrance/exit device max inport IC card quantity

// Intelligent parking system entrance/exit deviceIC card info import parameter DH_CTRL_ECK_IC_CARD_IMPORT
typedef struct tagNET_CTRL_ECK_IC_CARD_IMPORT_PARAM
{
    DWORD               dwSize;
    int                 nCardNum;                       // IC card quantity
    NET_ECK_IC_CARD     stuCardInfo[NET_MAX_ECK_IC_CARD_IMPORT_NUM];// IC card info
}NET_CTRL_ECK_IC_CARD_IMPORT_PARAM;

// Intelligent parking system entrance/exit deviceIC card info sync info parameter DH_CTRL_ECK_SYNC_IC_CARD
typedef struct tagNET_CTRL_ECK_SYNC_IC_CARD_PARAM
{
    DWORD               dwSize;
    int                 nCount;                         // Client IC card info quantity
}NET_CTRL_ECK_SYNC_IC_CARD_PARAM;

#define ECK_SCREEN_NUM_MAX (8)                         // The largest number of screens entrance and exits intelligent parking system 
// Set parking informations on the entrance and exits on the intelligent parking system DH_CTRL_ECK_SET_PARK_INFO
typedef struct tagNET_CTRL_ECK_SET_PARK_INFO_PARAM
{
    DWORD           dwSize;
    int             nScreenNum;                             // Screen numbers, not exceeding  ECK_SCREEN_NUM_MAX
    int             nScreenIndex[ECK_SCREEN_NUM_MAX];       // Screen series , every element means the screen series
    int             nFreeParkNum[ECK_SCREEN_NUM_MAX];       // Corresponding to the free parking number under the screen adminstration
                                                            // The length and the subscript accordance with nScreenIndex, every element means the free parking }NET_CTRL_ECK_SET_PARK_INFO_PARAM;
}NET_CTRL_ECK_SET_PARK_INFO_PARAM;

// fingerprint collection (corresponding to DH_CTRL_CAPTURE_FINGER_PRINT command )
typedef struct tagNET_CTRL_CAPTURE_FINGER_PRINT 
{
    DWORD      dwSize;
    int        nChannelID;                      // access control no.(start from 0)
    char       szReaderID[DH_COMMON_STRING_32]; // card reader ID
}NET_CTRL_CAPTURE_FINGER_PRINT;

// Hang up video phone(Corresponding DH_CTRL_VTP_DISCONNECT)
typedef struct tagNET_CTRL_VTP_DISCONNECT 
{
    DWORD                   dwSize;
    const char*             szTargetID;                     // The device ID to be forward , NULL means don't forward
    EM_NET_VTP_CALL_TYPE    emCallType;                     // Call type
    char                    szCallID[DH_COMMON_STRING_128]; // Intercom unique identifier}NET_CTRL_VTP_DISCONNECT;
}NET_CTRL_VTP_DISCONNECT;

// client(video talk phone) ack type
typedef enum tagNET_EM_VTP_ACK
{
    NET_EM_VTP_OK,                              // client calls right after get the event
    NET_EM_VTP_BUSY,                            // client busy, do not call
}NET_EM_VTP_ACK;

// client(video talk phone) divert ack
typedef struct tagNET_CTRL_VTP_DIVERTACK
{
    DWORD                   dwSize;
    EM_NET_VTP_CALL_TYPE    emCallType;                     // Calling type to query
    const char*             szCallID;                       // Unique ID for calling
    NET_EM_VTP_ACK          emAck;                          // Client's ack
}NET_CTRL_VTP_DIVERTACK;

// The operation model of remote multimedia files
typedef enum tagNET_EM_SPEAK_FILE_ACTION
{
    NET_EM_SPEAK_FILE_ACTION_UPDATE = 0,                    // Update the original file
    NET_EM_SPEAK_FILE_ACTION_ADD,                           // Added files
    NET_EM_SPEAK_FILE_ACTION_DEL,                           // Delete files
}NET_EM_SPEAK_FILE_ACTION;

// The operation informations of the remote multimedia files 
typedef struct tagNET_SPEAK_FILE_INFO
{
    DWORD                       dwSize;
    char                        szFilePath[MAX_PATH];       // File path, use the address setted up by CFG_CMD_NASEX firstly    NET_EM_SPEAK_FILE_ACTION    emAction;                   // Operation type, operation for the files appointed by szFilePath 
    NET_EM_SPEAK_FILE_ACTION    emAction;                   // The operation model of remote multimedia files
    BOOL                        bSave;                      // Whether update or save the files in local device 
}NET_SPEAK_FILE_INFO;

#define MAX_SPEAK_FILE_NUM      32                          // The max number of the files for remote

// Update the files for remote
typedef struct tagNET_CTRL_UPDATE_FILES
{
    DWORD                   dwSize;
    DWORD                   dwFileNum;                      // The valid file number
    NET_SPEAK_FILE_INFO     stuFileInfo[MAX_SPEAK_FILE_NUM];// The multimedia file information
}NET_CTRL_UPDATE_FILES;


// (corresponding to DH_CTRL_RAINBRUSH_MOVEONCE)
typedef struct tagNET_CTRL_RAINBRUSH_MOVEONCE
{
    DWORD           dwSize;
    int             nChannel;       // Rain-brush channel
}NET_CTRL_RAINBRUSH_MOVEONCE;

// (corresponding to DH_CTRL_RAINBRUSH_MOVECONTINUOUSLY)
typedef struct tagNET_CTRL_RAINBRUSH_MOVECONTINUOUSLY
{
    DWORD           dwSize;
    int             nChannel;       // Rain-brush channel
    unsigned int    nInterval;      // Interval
}NET_CTRL_RAINBRUSH_MOVECONTINUOUSLY;

// (corresponding to DH_CTRL_RAINBRUSH_STOPMOVE)
typedef struct tagNET_CTRL_RAINBRUSH_STOPMOVE
{
    DWORD           dwSize;
    int             nChannel;       // Rain-brush channel
}NET_CTRL_RAINBRUSH_STOPMOVE;

// (corresponding to DH_CTRL_ALARM_ACK)
typedef struct tagNET_CTRL_ALARM_ACK
{
    DWORD                   dwSize;
    int                     nEventID;   // Event ID
}NET_CTRL_ALARM_ACK;
////////////////////////////////system task subscription//////////////////////////////////////////
// zone status
typedef struct __NET_PARTITION_STATE
{
    DWORD             dwSize;
    int               nStatus;                  // zone status, 0-normal, 1-error 
    double            dbTotalSize;              // zone total capacity, byte is unit
    double            dbRemainSize;             // free capacity, byte is unit
    
}NET_PARTITION_STATE;

// HDD status
typedef struct __NET_HDD_STATE
{
    DWORD             dwSize; 
    int               nState;                   // HDD status, 0-normal, 1-error   
    double            dbTotalSize;              // HDD total capacity, byte is unit
    NET_PARTITION_STATE stuPartitions[DH_MAX_STORAGE_PARTITION_NUM]; // zone status
    int               nPartitionNum;              // zone quantity
}NET_HDD_STATE;

// channel status
typedef struct __NET_CHANNLE_STATE
{
    DWORD             dwSize;
    BYTE              byRecState;               // record status, 1-record, 0-close
    BYTE              byVideoInState;           // video input status, 1-video in, 0-no video  
    BYTE              byReserved[2];            // text align
}NET_CHANNLE_STATE;

// PAD/DVR connection state type
typedef enum tagNET_PAD_CONNECT_STATE
{
    NET_PAD_CONNECT_STATE_UNKNOWN,             // unknown
    NET_PAD_CONNECT_STATE_UNCONNECTED,         // unconnected
    NET_PAD_CONNECT_STATE_CONNECTED,           // connected
} NET_PAD_CONNECT_STATE;

// Home key state
typedef enum tagNET_HOME_STATE
{
    NET_HOME_STATE_UNKNOWN,                    // unknown
    NET_HOME_STATE_UNPRESSED,                  // unpressed
    NET_HOME_STATE_PRESSED,                    // pressed
} NET_HOME_STATE;

// device self-check info
typedef struct __NET_SELFCHECK_INFO
{
    DWORD             dwSize;
    int               nAlarmIn;                 // alarm input channel number
    int               nAlarmOut;                // alarm output channel number  
    NET_TIME          stuTime;                  // report time
    char              szPlateNo[DH_MAX_PLATE_NUMBER_LEN]; // plate
    char              szICCID[DH_MAX_SIM_LEN];  // SIM card no.
    BYTE              byOrientation;            // position status, 0-not position, 1-position 
    BYTE              byACCState;               // ACC status, 0-close, 1-open
    BYTE              byConstantElecState;      // power on, 0-normal, 1-disconnect, 2-less pressure, 3-over pressure
    BYTE              byAntennaState;           // channel signal status, 0-normal, 1-unknown failure, 2-disaonnect, 3-short circuit
    
    // external device status
    BYTE              byReportStation;          // station announcer status, 0-disconnect, 1-normal, 2-abnormal
    BYTE              byControlScreen;          // dispatch status, 0-disconnect, 1-normal, 2-abnormal
    BYTE              byPOS;                    // POS status, 0-disconnect, 1-normal, 2-abnormal
    BYTE              byCoinBox;                // box status, 0-disconnect, 1-normal, 2-abnormal
    
    // capacity set
    BOOL              bTimerSnap;               // schedule snapshot, TRUE-support, FALSE-not support
    BOOL              bElectronEnclosure;       // e-fence, TRUE-support, FALSE-not support
    BOOL              bTeleUpgrade;             // remote upgrade, TRUE-support, FALSE-not support   
    
    NET_HDD_STATE     stuHddStates[DH_MAX_DISKNUM]; //HDD status
    int               nHddNum;                  // HDD quantity
    
    NET_CHANNLE_STATE* pChannleState;           // CLIENT_AttachMission interface,channel status, is a group,  memoryis applied by sdk internally, released by sdk intenally
												// CLIENT_GetSelfCheckInfo interface,output param,memory is applied by the user,apply to sizeof(NET_CHANNLE_STATE)*nChannelMax
	int               nChannleNum;              // channel quantity
    int               nChannelMax;              // pChannleState max counts mallcoed by caller
    NET_PAD_CONNECT_STATE         emConnState;  // PAD/DVR connection state
    NET_HOME_STATE                emHomeState;  // Home key state
	char              szICCIDExInfo[DH_COMMON_STRING_256];  // Extend SIM card no.
	BYTE              by3GState;                // 3G/4G State, 0-disconnect, 1-connect, 2-module not find
	BYTE              byWifiState;              // Wifi State, 0-disconnect, 1-connect, 2-module not find
	BYTE              byGpsState;               // Gps State, 0-disconnect, 1-connect, 2-module not find
	BYTE              byBlackBoxState;          // BlackBox State, 0-disconnect, 1-normal
	int               nCpuUsage;                // CPU Usage, percent
	int               nTemperature;             // Device inner temperature, Celsius
}NET_SELFCHECK_INFO;

typedef void (CALLBACK *fMissionInfoCallBack)(LLONG lAttachHandle, DWORD dwType, void* pMissionInfo, void* pReserved, LDWORD dwUserData);

typedef enum EM_MISSION_TYPE
{
    NET_MISSION_TYPE_UNKOWN,                    // unknown
    NET_MISSION_TYPE_SELFCHECK,                 // device self-check, corresponding structure  NET_SELFCHECK_INFO
}EM_MISSION_TYPE;

//CLIENT_AttachMissionport input parameter
typedef struct __NET_IN_ATTACH_MISSION_PARAM
{
    DWORD             dwSize;
    EM_MISSION_TYPE   emMissionType;            // task type
    fMissionInfoCallBack cbMissionInfofunc;     // task info callback function
    LDWORD            dwUser;                   // user data 
}NET_IN_ATTACH_MISSION_PARAM;

// CLIENT_DetachMissionport output parameter
typedef struct NET_OUT_ATTACH_MISSION_PARAM
{
    DWORD             dwSize;
    LLONG             lAttachHandle;           // subscribe handle
}NET_OUT_ATTACH_MISSION_PARAM;

#define MAX_EVENT_TO_CONFIRM_NUM    8    // max to confirm event number 

//to confirm event info 
typedef struct tagNET_EVENT_INFO_TO_CONFIRM 
{
    DWORD           dwSize;
    DWORD           dwEventType;        // event type , corresponding to CLIENT_AttachBusStateport event type 
    NET_TIME        stuTime;            // event time, UTC
    LONG            lTime;              // event time, UTC, unit is second
}NET_EVENT_INFO_TO_CONFIRM;

// vehicle emergency alarm confirm input parameter 
typedef struct tagNET_IN_BUS_CONFIRM_EVENT 
{
    DWORD           dwSize;
    int             nEvent;             // to confirm event valid quantity
    NET_EVENT_INFO_TO_CONFIRM  stuEventInfo[MAX_EVENT_TO_CONFIRM_NUM];// to confirm event info 
}NET_IN_BUS_CONFIRM_EVENT;

// vehicle emergency alarm confirm  output parameter 
typedef struct tagNET_OUT_BUS_CONFIRM_EVENT 
{
    DWORD           dwSize;
}NET_OUT_BUS_CONFIRM_EVENT;

// CLIENT_SetDevicePosition interface parameter input 
typedef struct tagNET_IN_SET_DEVICE_POSITION
{
    DWORD                   dwSize;                             // Struture size. Must fill in when initialization 
    char                    szPosition[DH_COMMON_STRING_512];   // Address info 
}NET_IN_SET_DEVICE_POSITION;

// CLIENT_SetDevicePosition interface paramter output
typedef struct tagNET_OUT_SET_DEVICE_POSITION
{
    DWORD                   dwSize;                             // Structure size. Must fill in when initialization
}NET_OUT_SET_DEVICE_POSITION;

// CLIENT_GetDevicePosition interface paramter input
typedef struct tagNET_IN_GET_DEVICE_POSITION
{
    DWORD                   dwSize;                             // Structure size. Must fill in when initialization
}NET_IN_GET_DEVICE_POSITION;

// CLIENT_GetDevicePosition interface paramter output
typedef struct tagNET_OUT_GET_DEVICE_POSITION
{
    DWORD                   dwSize;                             // Structure size. Must fill in when initialization
    char                    szPosition[DH_COMMON_STRING_512];   // Address info 
}NET_OUT_GET_DEVICE_POSITION;

// CLIENT_SnapPictureByEvent interface paramter input
typedef struct tagNET_IN_SNAP_BY_EVENT 
{
    DWORD                   dwSize;                             // Struture size. Must fill in when initialization
    int                     nChannel;                           // Video channel No. Begins with 0. 
    DWORD                   dwEventID;                          // Refer to event type of CLIENT_RealLoadPicEx
}NET_IN_SNAP_BY_EVENT;

// CLIENT_SnapPictureByEvent interface paramter output
typedef struct tagNET_OUT_SNAP_BY_EVENT 
{
    DWORD                   dwSize;                             // Struture size. Must fill in when initialization
}NET_OUT_SNAP_BY_EVENT;
// CLIENT_StartQueryLog input parameter
typedef struct tagNET_IN_START_QUERYLOG
{
    DWORD               dwSize;
} NET_IN_START_QUERYLOG;

// CLIENT_StartQueryLog ouput parameter
typedef struct tagNET_OUT_START_QUERYLOG
{
    DWORD               dwSize;
}NET_OUT_START_QUERYLOG;

// Detail info of log
typedef struct tagNET_LOG_MESSAGE
{
    DWORD               dwSize;
    char				szLogMessage[DH_COMMON_STRING_1024];    // detailed info
} NET_LOG_MESSAGE;

// Info of log
typedef struct tagNET_LOG_INFO
{
    DWORD               dwSize;
    NET_TIME            stuTime;                        // time 
    char                szUserName[DH_COMMON_STRING_32];// operator
    char                szLogType[DH_COMMON_STRING_128];// type
    NET_LOG_MESSAGE	    stuLogMsg;                      // detailed info
} NET_LOG_INFO;

// CLIENT_QueryNextLog input parameter
typedef struct tagNET_IN_QUERYNEXTLOG
{
    DWORD               dwSize;
    int                 nGetCount;      // count of log item try to query
}NET_IN_QUERYNEXTLOG;

// CLIENT_QueryNextLog ouput parameter
typedef struct tagNET_OUT_QUERYNEXTLOG
{
    DWORD               dwSize;
    int                 nMaxCount;      // count of log item malloced by user, should be NET_IN_GETNEXTLOG*nGetCount
    NET_LOG_INFO*       pstuLogInfo;    // buffer for log item malloced by user, should be nMaxCount*sizeof(NET_LOG_INFO)
    int                 nRetCount;      // return count of log item
}NET_OUT_QUERYNEXTLOG;

// Asynchronous callback function registered equipment prototype (when nError back to 11, reserved fields reserved for int type parameters and used to invoke the CLIENT_StartLoginEx landed the port)
typedef void (CALLBACK *fHaveLogin)(LLONG lLoginID, char *pchDVRIP, LONG nDVRPort, BOOL bOnline, NET_DEVICEINFO_Ex stuDeviceInfo, int nError, LDWORD dwUser, void *reserved);

// CLIENT_StartLoginEx  input parameter
typedef struct tagNET_IN_STARTLOGINEX 
{
    DWORD               dwSize;
    const char*         szIp;           // device ip
    DWORD               dwPort;         // login port
    const char*         szName;         // username
    const char*         szPwd;          // password
    fHaveLogin          cbLogin;        // login result call
    LDWORD              dwUser;         // call user parameter
}NET_IN_STARTLOGINEX;

// CLIENT_StartLoginEx output parameter
typedef struct tagNET_OUT_STARTLOGINEX 
{
    DWORD       dwSize;
}NET_OUT_STARTLOGINEX;

// the protocol type of device
typedef enum tagEM_DEV_PROTOCOL_TYPE
{
	EM_DEV_PROTOCOL_UNKNOWN,			// unknown
	EM_DEV_PROTOCOL_V2,					// private v2
	EM_DEV_PROTOCOL_V3,					// private v3
} EM_DEV_PROTOCOL_TYPE;

// video splicing operation type
typedef enum tagNET_VIDEOJOIN_OPERATE_TYPE
{
    NET_VIDEOJOIN_GET_STATUS,               // get splicing status, corresponding to  NET_IN_VIDEOJOIN_GET_STATUS and NET_OUT_VIDEOJOIN_GET_STATUS
    NET_VIDEOJOIN_GET_CALIBRATE_POINTS,     // get calibrate point , corresponding to  tagNET_IN_VIDEOJOIN_GET_CALIBRATE_POINTS and  tagNET_OUT_VIDEOJOIN_GET_CALIBRATE_POINTS
    NET_VIDEOJOIN_CALIBRATE,                // mark ,  corresponding to  NET_IN_VIDEOJOIN_CALIBRATE and NET_OUT_VIDEOJOIN_CALIBRATE
}NET_VIDEOJOIN_OPERATE_TYPE;

// video splicing status
typedef enum tagNET_VIDEOJOIN_STATUS
{
    NET_VIDEOJOIN_STATUS_UNKNOWN,               // unknow
    NET_VIDEOJOIN_STATUS_NOTSTART,              // not started
    NET_VIDEOJOIN_STATUS_DOING,                 // running
    NET_VIDEOJOIN_STATUS_FINISH,                // complate
}NET_VIDEOJOIN_STATUS;

// video splicing mark mode
typedef enum tagNET_VIDEOJOIN_MODE
{
    NET_VIDEOJOIN_MODE_UNKNOWN,                 // unknow
    NET_VIDEOJOIN_MODE_AUTO,                    // auto
    NET_VIDEOJOIN_MODE_MANUAL,                  // manual
}NET_VIDEOJOIN_MODE;

// CLIENT_OperateVideoJoin input parameter, corresponding to NET_VIDEOJOIN_GET_STATUS
typedef struct tagNET_IN_VIDEOJOIN_GET_STATUS
{
    DWORD           dwSize;
}NET_IN_VIDEOJOIN_GET_STATUS;

// CLIENT_OperateVideoJoin output parameter, corresponding to NET_VIDEOJOIN_GET_STATUS
typedef struct tagNET_OUT_VIDEOJOIN_GET_STATUS
{
    DWORD           dwSize;
    NET_VIDEOJOIN_STATUS emStatus;              // splicing status
}NET_OUT_VIDEOJOIN_GET_STATUS;

// camera mark pair, means one pair of marks
typedef struct tagNET_POINT_PAIR 
{
    DH_POINT        stuPoints[POINT_NUM_IN_PAIR];           // camera mark
}NET_POINT_PAIR;

// video splicing mark group, each group means two channels and multiple mark pairs
typedef struct tagNET_VIDEOJOIN_POINT_GROUP
{
    DWORD           dwSize;
    int             nPointPairNum;                          // mark quantity
    NET_POINT_PAIR  stuPointPairs[MAX_POINT_PAIR_NUM];      // mark
    int             nChannels[CHANNEL_NUM_IN_POINT_GROUP];  // video channel no., via data mark to correspond , mark 0 channel corresponding to NET_POINT_PAIR mark 0 point
}NET_VIDEOJOIN_POINT_GROUP;

// CLIENT_OperateVideoJoin input parameter, corresponding to NET_VIDEOJOIN_GET_CALIBRATE_POINTS
typedef struct tagNET_IN_VIDEOJOIN_GET_CALIBRATE_POINTS
{
    DWORD           dwSize;
}NET_IN_VIDEOJOIN_GET_CALIBRATE_POINTS;

// CLIENT_OperateVideoJoin output parameter, corresponding to NET_VIDEOJOIN_GET_CALIBRATE_POINTS
typedef struct tagNET_OUT_VIDEOJOIN_GET_CALIBRATE_POINTS
{
    DWORD               dwSize;
    NET_VIDEOJOIN_MODE  emMode;                                 // mark mode
    int                 nPointGroupNum;                         // mark point quantity
    NET_VIDEOJOIN_POINT_GROUP stuPointGroups[MAX_POINT_GROUP_NUM]; // mark pointgroup, under manual mode is valid
}NET_OUT_VIDEOJOIN_GET_CALIBRATE_POINTS;

// CLIENT_OperateVideoJoin input parameter, corresponding to NET_VIDEOJOIN_CALIBRATE
typedef struct tagNET_IN_VIDEOJOIN_CALIBRATE
{
    DWORD               dwSize;
    NET_VIDEOJOIN_MODE  emMode;                                 // mark mode
    int                 nPointGroupNum;                         // mark point group quantity
    NET_VIDEOJOIN_POINT_GROUP stuPointGroups[MAX_POINT_GROUP_NUM]; // mark pointgroup, manual mode valid
}NET_IN_VIDEOJOIN_CALIBRATE;

// CLIENT_OperateVideoJoin output mode, corresponding to NET_VIDEOJOIN_CALIBRATE
typedef struct tagNET_OUT_VIDEOJOIN_CALIBRATE
{
    DWORD           dwSize;
}NET_OUT_VIDEOJOIN_CALIBRATE;

// Get screen window info input parameter , corresponding to NET_SPLIT_OPERATE_GET_SCENE
typedef struct tagNET_IN_SPLIT_GET_SCENE 
{
    DWORD           dwSize;
    int             nChannel;               // output channel no. or cubeless video wall virtual channel no., pszCompositeIDis NULL, it is valid 
    const char*     pszCompositeID;         // cubeless video wall ID
}NET_IN_SPLIT_GET_SCENE;

// Get  screen window info  output parameter , corresponding to NET_SPLIT_OPERATE_GET_SCENE
typedef struct tagNET_OUT_SPLIT_GET_SCENE 
{
    DWORD           dwSize;
    DH_SPLIT_SCENE  stuScene;               // window info 
}NET_OUT_SPLIT_GET_SCENE;

// tv wall tour status 
typedef enum tagEM_NET_WM_TOUR_STATUS
{
    EM_NET_WM_TOUR_STATUS_UNKNOWN,          // unknown 
    EM_NET_WM_TOUR_STATUS_START,            // touring
    EM_NET_WM_TOUR_STATUS_STOP,             // tour stop
} EM_NET_WM_TOUR_STATUS;

// tv wall scheme tour status info 
typedef struct tagNET_WM_TOUR_STATUS_INFO 
{
    DWORD          dwSize;
    EM_NET_WM_TOUR_STATUS  emStatus;        // tour status 
    DH_MONITORWALL_SCENE  stuScene;         // scheme info 
} NET_WM_TOUR_STATUS_INFO;

// tv wall scheme tour status call function origin, lAttachHandle is CLIENT_MonitorWallAttachTour return value 
typedef void (CALLBACK *fMonitorWallTourStatusCallBack)(LLONG lLoginID, LLONG lAttachHandle, NET_WM_TOUR_STATUS_INFO* pstStatus, void* reserved, LDWORD dwUser);

// CLIENT_MonitorWallAttachTour port input parameter 
typedef struct tagNET_IN_WM_ATTACH_TOUR 
{
    DWORD       dwSize;
    int       	nMonitorWallID;             // tv wall ID
    fMonitorWallTourStatusCallBack cbStatus;// tour status  call function 
    LDWORD     	dwUser;                     // tour status call parameter 
} NET_IN_WM_ATTACH_TOUR;

// CLIENT_MonitorWallAttachTour port  output parameter 
typedef struct tagNET_OUT_WM_ATTACH_TOUR 
{
    DWORD      dwSize;
} NET_OUT_WM_ATTACH_TOUR;

// CLIENT_SnapManagerConfirmUpload port input parameter
typedef struct tagNET_IN_SNAP_MANAGER_CONFIRM_UPLOAD
{
    DWORD           dwSize;
    char            szFilePath[DH_COMMON_STRING_256];       // File path, used for SN of picture
    BOOL            bResult;                                // If picture upload is complete
}NET_IN_SNAP_MANAGER_CONFIRM_UPLOAD;

// CLIENT_SnapManagerConfirmUpload port output parameter
typedef struct tagNET_OUT_SNAP_MANAGER_CONFIRM_UPLOAD
{
    DWORD           dwSize;
}NET_OUT_SNAP_MANAGER_CONFIRM_UPLOAD;

// Video phone status informations
typedef struct tagNET_VTP_CALL_STATE_INFO 
{
    DWORD                       dwSize;
    char                        szNumber[DH_COMMON_STRING_128]; // Call number
    char                        szCallID[DH_COMMON_STRING_128]; // Intercom unique identifier
    EM_NET_VTP_CALL_STATE_TYPE  emCallState;                    // Call status
}NET_VTP_CALL_STATE_INFO;

// The callback function prototypes of subscribe to video phone
typedef void (CALLBACK *fVTPCallStateCallBack)(LLONG lAttachHandle, NET_VTP_CALL_STATE_INFO *pInfo, int nBufLen, LDWORD dwUser);

// Input parameter about the status informations of subscribe to video phone
typedef struct tagNET_IN_VTP_CALL_STATE_ATTACH 
{
    DWORD                       dwSize;
    EM_NET_VTP_CALL_TYPE        emCallType;                     // The call type for checking
    fVTPCallStateCallBack       cbCallState;                    // The callback function
    LDWORD                      dwUser;                         // user's param
    const char*                 szTargetID;                     // The device ID for forward, NULL means not formard
}NET_IN_VTP_CALL_STATE_ATTACH;

// The output parameter of the status information of subscribes to video phone
typedef struct tagNET_OUT_VTP_CALL_STATE_ATTACH 
{
    DWORD                       dwSize;
}NET_OUT_VTP_CALL_STATE_ATTACH;


//////////////////////////////////////////////////////////////////////////
// ABILITY INFO
//////////////////////////////////////////////////////////////////////////

// Check the buses route CLIENT_CheckBusLine()input parameter
typedef struct tagNET_IN_CHECK_BUS_LINE 
{
    DWORD           dwSize;
    char            szLineID[DH_COMMON_STRING_32];          // Line marking
    char            szDesignation[DH_COMMON_STRING_32];     // Line serial number
    NET_LINE_DIRECTION  emDirection;                        // Line direction
    int             nSiteCount;                             // The total stations number on the route
    char            szDirectionName[DH_COMMON_STRING_32];   // Name of the line firection
    char            szVersion[DH_COMMON_STRING_32];         // Line version number
}NET_IN_CHECK_BUS_LINE;


// The status type of the bus line whether to update
typedef enum tagNET_BUS_LINE_UPDATE_TYPE
{
    NET_BUS_LINE_UPDATE_UNKNOWN = 0,
    NET_BUS_LINE_UPDATE_UPDATED,                            // Need update
    NET_BUS_LINE_UPDATE_NOT_UPDATED,                        // Not update
}NET_BUS_LINE_UPDATE_TYPE;

// Check the bus routeCLIENT_CheckBusLine()output parameter
typedef struct tagNET_OUT_CHECK_BUS_LINE
{
    DWORD           dwSize;
    char            szDeviceID[DH_COMMON_STRING_32];        // device ID
    char            szLineID[DH_COMMON_STRING_32];          // Line marking
    NET_BUS_LINE_UPDATE_TYPE emUpdate;                      // Indicator for whether need to update the status
}NET_OUT_CHECK_BUS_LINE;
#define MAX_SITE_GPS_POINT_NUM                      64      // Describe the bus stops the largest GPS point

// The shape type of the bus station
typedef enum tagNET_SITE_AREA_TYPE
{
    NET_SITE_AREA_ENCLOSURE = 0,                            // Type of the fence
    NET_SITE_AREA_CIRCLE,                                   // Circular
}NET_SITE_AREA_TYPE;

// Bus station information
typedef struct tagNET_BUS_SITE_INFO
{
    DWORD           dwSize;
    char            szLineID[DH_COMMON_STRING_32];          // Line marking
    int             nSiteCount;                             // Total number of the stations
    NET_LINE_DIRECTION  emDirection;                        // Line direction
    int             nStopSequence;                          // Station serial number
    char            szSiteID[DH_COMMON_STRING_32];          // Station ID
    char            szSiteName[DH_COMMON_STRING_32];        // Station name, you can use szSiteNameEx to input a longer name
    NET_SITE_AREA_TYPE emAreaType;                          // Station type
    int             nGPSPointNum;                           // valid GPS points number
    GPS_POINT       stuPoint[MAX_SITE_GPS_POINT_NUM];       // Station GPS information
    int             nRadius;                                // Radius of the station, unit: meter, valid only for the circulai
    int             nSpeedLimit;                            // Station speed limit, unit: km/H
    int             nDriveTime;                             // Time for the last station to current, unit:minute
    int             nTicketPrice;                           // Filp fares
    DH_TSECT        stuPowerSchedule[DH_TSCHE_DAY_NUM][DH_TSCHE_SEC_NUM]; // Time table, The first dimension of each element means ~ on Saturday and Sunday ,holidays
    char            szDesignation[DH_COMMON_STRING_64];     // Line serial number
    char            szDirectionName[DH_COMMON_STRING_64];   // Line direction name
    char            szVersion[DH_COMMON_STRING_32];
    BOOL            bUseExName;                             // Use new site name flag
    char            szSiteNameEx[DH_COMMON_STRING_128];     // site name with 128 bytes
}NET_BUS_SITE_INFO;

#define MAX_BUS_STOP_NUM                            32      // The max number of  the stations in a bus line

// Issue bus line CLIENT_DispatchBusLineInfo()Interface input parameters
typedef struct tagNET_IN_DISPATCH_BUS_LINE_INFO 
{
    DWORD           dwSize;
    int             nBusStopNum;                            // The number of bus line stations
    NET_BUS_SITE_INFO   stuStopInfo[MAX_BUS_STOP_NUM];      // The information of the bus line
}NET_IN_DISPATCH_BUS_LINE_INFO;

// Issue bus line  CLIENT_DispatchBusLineInfo()Interface output parameters
typedef struct tagNET_OUT_DISPATCH_BUS_LINE_INFO 
{
    DWORD           dwSize;
}NET_OUT_DISPATCH_BUS_LINE_INFO;

// Vehicle operation dispatching CLIENT_BusSchedule()Interface input parameters
typedef struct tagNET_IN_BUS_SCHEDULE_INFO 
{
    DWORD               dwSize;
    char                szLineID[DH_COMMON_STRING_64];      // Operating line identification
    NET_LINE_DIRECTION  emDirection;                        // Operating line direction
    NET_TIME_EX         stuStartTime;                       // Departure time 

}NET_IN_BUS_SCHEDULE_INFO;

// Vehicle operation dispatching CLIENT_BusSchedule()Interface output parameters
typedef struct tagNET_OUT_BUS_SCHEDULE_INFO 
{
    DWORD           dwSize;
}NET_OUT_BUS_SCHEDULE_INFO;

// Vehicle scheduling plan information
typedef struct tagNET_BUS_WORK_PLAN 
{
    DWORD               dwSize;
    char                szID[DH_COMMON_STRING_32];          // Scheduling record only, Up to 16 characters
    NET_TIME_EX         stuTime;                            // Departure time 
    char                szDriverName[DH_COMMON_STRING_64];  // The driver name
    char                szStartSite[DH_COMMON_STRING_64];   // Starting station name
    char                szEndSite[DH_COMMON_STRING_64];     // The terminal point name
    BOOL                bRunning;                           // Operation state, TRUE: run, FALSE:stop
}NET_BUS_WORK_PLAN;

// Issued by the vehicle scheduling plan CLIENT_DispatchWorkPlan Interface input parameters
typedef struct tagNET_IN_BUS_DISPATCH_WORK_PLAN 
{
    DWORD               dwSize;
    NET_BUS_WORK_PLAN   stuWorkPlan;                        // Vehicle scheduling plan information }NET_IN_BUS_DISPATCH_WORK_PLAN;
}NET_IN_BUS_DISPATCH_WORK_PLAN;

// Issued by the vehicle scheduling plan CLIENT_DispatchWorkPlan Interface output parameters
typedef struct tagNET_OUT_BUS_DISPATCH_WORK_PLAN 
{
    DWORD               dwSize;
}NET_OUT_BUS_DISPATCH_WORK_PLAN;

// Thermal image color 
typedef enum tagNET_THERMO_COLORIZATION 
{
    NET_THERMO_COLORIZATION_UNKNOWN,                        // Unknown 
    NET_THERMO_COLORIZATION_WHITE_HOT,                      // White 
    NET_THERMO_COLORIZATION_BLACK_HOT,                      // Black
    NET_THERMO_COLORIZATION_IRONBOW2,                       // Iron 
    NET_THERMO_COLORIZATION_ICEFIRE,                        // Icefire
	NET_THERMO_COLORIZATION_FUSION,                         // Fusion
	NET_THERMO_COLORIZATION_RAINBOW,                        // Rainbow
	NET_THERMO_COLORIZATION_GLOBOW,                         // Globow
	NET_THERMO_COLORIZATION_IRONBOW1,                       // Ironbow1
	NET_THERMO_COLORIZATION_SEPIA,                          // Sepia
	NET_THERMO_COLORIZATION_COLOR1,                         // Color1
	NET_THERMO_COLORIZATION_COLOR2,                         // Color2
	NET_THERMO_COLORIZATION_RAIN,                           // Rain
	NET_THERMO_COLORIZATION_RED_HOT,                        // RedHot
	NET_THERMO_COLORIZATION_GREEN_HOT,                      // GreenHot
} NET_THERMO_COLORIZATION;

// Thermal ROI mode 
typedef enum tagNET_THERMO_ROI 
{
    NET_THERMO_ROI_UNKNOWN,                                 // Unknown 
    NET_THERMO_ROI_FULL_SCREEN,                             // Full-screen 
    NET_THERMO_ROI_SKY,                                     // Top
    NET_THERMO_ROI_GROUND,                                  // Middle
    NET_THERMO_ROI_HORIZONTAL,                              // Bottom
    NET_THERMO_ROI_CENTER_75,                               // Center point 75%
    NET_THERMO_ROI_CENTER_50,                               // Center point 50%
    NET_THERMO_ROI_CENTER_25,                               // Center point 25%
    NET_THERMO_ROI_CUSTOM,                                  // Customized 
} NET_THERMO_ROI;

// Thermal mode 
typedef enum tagNET_THERMO_MODE 
{
    NET_THERMO_MODE_UNKNOWN,                                // Unknown
    NET_THERMO_MODE_DEFAULT,                                // Default
    NET_THERMO_MODE_INDOOR,                                 // Indoor 
    NET_THERMO_MODE_OUTDOOR,                                // Outdoor 
} NET_THERMO_MODE;

// Thermal optimized region 
typedef struct tagNET_THERMO_GRAPHY_OPT_REGION 
{
    BOOL                bOptimizedRegion;                   // Enable optimized region or not 
    int                 nOptimizedROIType;                  // Optimized region type. Please refer to NET_THERMO_ROI
    int                 nCustomRegion;                      // Customized region amount 
    NET_RECT            stCustomRegions[64];                // Customized region. Valid only when nOptimizedROIType is NET_THERMO_ROI_CUSTOM 
    char                Reserved[256];
} NET_THERMO_GRAPHY_OPTREGION;

// Thermal info 
typedef struct tagNET_THERMO_GRAPHY_INFO 
{
    int                         nBrightness;                // brightness
    int                         nSharpness;                 // Sharpness
    int                         nEZoom;                     // Zoom 
    int                         nThermographyGamma;         // Gamma value 
    int                         nColorization;              // Color. Refer to NET_THERMO_COLORIZATION
    int                         nSmartOptimizer;            // Optimized indicator 
    NET_THERMO_GRAPHY_OPTREGION stOptRegion;                // Optimized region 
    int                         nAgc;                       // Auto gain control 
    int                         nAgcMaxGain;                // Max auto gain
    int                         nAgcPlateau;                // Gain balance 
    char reserved[244];
} NET_THERMO_GRAPHY_INFO;

// CLIENT_QueryDevInfo port NET_QUERY_DEV_THERMO_GRAPHY_PRESET command to input parameter  
typedef struct tagNET_IN_THERMO_GET_PRESETINFO 
{
    DWORD               dwSize;
    int                 nChannel;                           // channel number
    NET_THERMO_MODE     emMode;                             // mode
} NET_IN_THERMO_GET_PRESETINFO;

// CLIENT_QueryDevInfo port NET_QUERY_DEV_THERMO_GRAPHY_PRESET command to output parameter
typedef struct tagNET_OUT_THERMO_GET_PRESETINFO 
{
    DWORD                       dwSize;
    NET_THERMO_GRAPHY_INFO      stInfo;                     // thermal info
} NET_OUT_THERMO_GET_PRESETINFO;

// CLIENT_QueryDevInfo port_QUERY_DEV_THERMO_GRAPHY_EXTSYSINFO command to input parameter
typedef struct tagNET_IN_THERMO_GET_EXTSYSINFO
{
    DWORD               dwSize;
    int                 nChannel;                           // channel number
} NET_IN_THERMO_GET_EXTSYSINFO;

// CLIENT_QueryDevInfo port  NET_QUERY_GET_LINKCHANNELS command to input parameter
typedef struct tagNET_IN_GET_LINKCHANNELS
{	
    DWORD               dwSize;
    int                 nChannel;	                        // channel number, check if there is related video channel in this video channel number
} NET_IN_GET_LINKCHANNELS;

#define NET_LINKCHANNEL_MAX     512                         // max linked video channel count
#define NET_LINKGROUP_MAX       64                          // max linked video channel group count

// CLIENT_QueryDevInfo port NET_QUERY_GET_LINKCHANNELS command to output parameter
typedef struct tagNET_OUT_GET_LINKCHANNELS
{	
    DWORD               dwSize;
    int                 nLinkedCnt;                         // related  number of video channel number
    int                 nLinked[NET_LINKCHANNEL_MAX];	    // related video channel number, including request channel number
} NET_OUT_GET_LINKCHANNELS;

// input param of CLIENT_QueryDevInfo according to NET_QUERY_GET_ALLLINKCHANNELS
typedef struct tagNET_IN_GET_ALLLINKCHANNELS
{	
    DWORD               dwSize;
} NET_IN_GET_ALLLINKCHANNELS;

// output param of CLIENT_QueryDevInfo according to NET_QUERY_GET_ALLLINKCHANNELS
typedef struct tagNET_OUT_GET_ALLLINKCHANNELS
{	
    DWORD               dwSize;
    int                 nGroupCnt;                                              // linked video channel group count in
    int                 nLinkedCnt[NET_LINKGROUP_MAX];                          // linked video channel counts in a group
    int                 nLinked[NET_LINKGROUP_MAX][NET_LINKCHANNEL_MAX];	    // linked video channel group info
                                                                                // first dimension: channel group, second dimension: video channel number
} NET_OUT_GET_ALLLINKCHANNELS;

// CLIENT_QueryDevInfo port NET_QUERY_GET_CAMERA_STATE command to input parameter
typedef struct tagNET_IN_GET_CAMERA_STATEINFO
{
    DWORD               dwSize;
    BOOL                bGetAllFlag;                                // if it is to check all the cameras status, if the member is TRUE, then nChannels member is unnecessary to set.
    int                 nValidNum;                                  // the member is valid when bGetAllFlag is FALSE, which means valid number of nChannels member
    int                 nChannels[DH_MAX_CAMERA_CHANNEL_NUM];       // The member is valid when bGetAllFlag is FALSE, it is to fill in the channel numbers in turn which needs inquiry. 
} NET_IN_GET_CAMERA_STATEINFO;

typedef enum tagEM_CAMERA_STATE_TYPE
{
    EM_CAMERA_STATE_TYPE_UNKNOWN,       // unknown 
    EM_CAMERA_STATE_TYPE_CONNECTING,    // connecting
    EM_CAMERA_STATE_TYPE_CONNECTED,     // connected
    EM_CAMERA_STATE_TYPE_UNCONNECT,     // unconnected
    EM_CAMERA_STATE_TYPE_EMPTY,         // channel is not configured, no info
    EM_CAMERA_STATE_TYPE_DISABLE,       // channel is configured, but it is forbidden. 
}EM_CAMERA_STATE_TYPE;

typedef struct tagNET_CAMERA_STATE_INFO
{
    int                     nChannel;           // camera channel number, -1 means invalid channel number
    EM_CAMERA_STATE_TYPE    emConnectionState;  // connection state
    char                    szReserved[1024];   // byte reserved
}NET_CAMERA_STATE_INFO;

// CLIENT_QueryDevInfo port NET_QUERY_GET_CAMERA_STATE command to output parameter
typedef struct tagNET_OUT_GET_CAMERA_STATEINFO 
{
    DWORD                       dwSize;
    int                         nValidNum;              // valid number of camera channel state, returned by sdk
    int                         nMaxNum;                // max number of array, filled in by user
    NET_CAMERA_STATE_INFO*      pCameraStateInfo;       // camera channel info array, distributed by user,apply to sizeof(NET_CAMERA_STATE_INFO)*nMaxNum
} NET_OUT_GET_CAMERA_STATEINFO;

// CLIENT_QueryDevInfo port NET_QUERY_GET_REMOTE_CHANNEL_AUDIO_ENCODE command to input parameter
typedef struct tagNET_IN_GET_REMOTE_CHANNEL_AUDIO_ENCODEINFO
{
    DWORD               dwSize;
    int                 nChannel;                                   // channel number
    int                 nStreamType;                                // stream type, 0: main stream; 1: sub stream 1; 2: sub stream 2; 3: sub stream 3; 4: talk stream
} NET_IN_GET_REMOTE_CHANNEL_AUDIO_ENCODEINFO;

// CLIENT_QueryDevInfo port NET_QUERY_GET_REMOTE_CHANNEL_AUDIO_ENCODE command to output parameter
typedef struct tagNET_OUT_GET_REMOTE_CHANNEL_AUDIO_ENCODEINFO
{
    DWORD                       dwSize;
    int                         nValidNum;                                  // valid number of audio coding
    DHDEV_TALKDECODE_INFO       stuListAudioEncode[MAX_AUDIO_ENCODE_NUM];   // audio coding list
} NET_OUT_GET_REMOTE_CHANNEL_AUDIO_ENCODEINFO;

// API CLIENT_QueryDevInfo  NET_QUERY_GET_COMM_PORT_INFO in param
typedef struct tagNET_IN_GET_COMM_PORT_INFO
{
    DWORD               dwSize;
} NET_IN_GET_COMM_PORT_INFO;

// API CLIENT_QueryDevInfo NET_QUERY_GET_COMM_PORT_INFO out param
//port type
typedef enum tagEM_COMM_PORT_TYPE_INFO
{
    EM_COMM_PORT_TYPE_UNKNOW = 0    ,
    EM_COMM_PORT_TYPE_RS232         ,
    EM_COMM_PORT_TYPE_RS485         ,
    EM_COMM_PORT_TYPE_RS422         ,
    EM_COMM_PORT_TYPE_RS485_422     ,
}EM_COMM_PORT_TYPE_INFO;

//port info
typedef struct  tagNET_COMM_PORT_INFO
{
    EM_COMM_PORT_TYPE_INFO  emCommPortType;     //Port Type
    int                     nCommPortNum;       //Port Number
    BYTE                    bReserved[1024];    //reserved bytes
}NET_COMM_PORT_INFO;

#define MAX_COMM_PORT_NUM       8
typedef struct tagNET_OUT_GET_COMM_PORT_INFO
{
    DWORD                       dwSize;
    int                         nPortInfosNum;                          //Port Infos Number
    NET_COMM_PORT_INFO          stCommPortInfos[MAX_COMM_PORT_NUM];     //Port Info
} NET_OUT_GET_COMM_PORT_INFO;

// external system info
typedef struct tagNET_THERMO_SYSINFO 
{
    char                szSerialNumber[64];                 // serial number
    char                szSoftwareVersion[64];              // software version
    char                szFirmwareVersion[64];              // firmware version
    char                szLibVersion[64];                   // library version
    char                reserved[256];
} NET_THERMO_SYSINFO;

// CLIENT_QueryDevInfo port NET_QUERY_DEV_THERMO_GRAPHY_EXTSYSINFO command to output parameter
typedef struct tagNET_OUT_THERMO_GET_EXTSYSINFO
{
    DWORD                       dwSize;
    NET_THERMO_SYSINFO          stInfo;                     // external system info
} NET_OUT_THERMO_GET_EXTSYSINFO;

// CLIENT_QueryDevInfo port NET_QUERY_DEV_THERMO_GRAPHY_OPTREGION command to input parameter
typedef struct tagNET_IN_THERMO_GET_OPTREGION
{
    DWORD               dwSize;
    int                 nChannel;                           // channel nunmber
} NET_IN_THERMO_GET_OPTREGION;

// CLIENT_QueryDevInfo port NET_QUERY_DEV_THERMO_GRAPHY_OPTREGION command to output parameter
typedef struct tagNET_OUT_THERMO_GET_OPTREGION
{
    DWORD                       dwSize;
    NET_THERMO_GRAPHY_OPTREGION stInfo;                     // optimize area info
} NET_OUT_THERMO_GET_OPTREGION;


// CLIENT_QueryDevInfo interface, input param of NET_QUERY_GET_VIDEOOUTPUTCHANNELS
typedef struct tagNET_IN_GET_VIDEOOUTPUTCHANNELS
{
    DWORD               dwSize;             // dwSize must be initialized,dwSize is sizeof(NET_IN_GET_VIDEOOUTPUTCHANNELS)
} NET_IN_GET_VIDEOOUTPUTCHANNELS;

// CLIENT_QueryDevInfo interface NET_QUERY_GET_VIDEOOUTPUTCHANNELS command to output parameter
typedef struct tagNET_OUT_GET_VIDEOOUTPUTCHANNELS
{
    DWORD               dwSize;             // dwSize must be initialized,dwSize is sizeof(NET_OUT_GET_VIDEOOUTPUTCHANNELS)
    int                 nMaxLocal;          // max local output channels,including mainboard and pluggable sub card channel
} NET_OUT_GET_VIDEOOUTPUTCHANNELS;


// CLIENT_QueryDevInfo interface NET_QUERY_GET_VIDEOINFO command to input parameter
typedef struct tagNET_IN_GET_VIDEOINFO
{
    DWORD               dwSize;             // dwSize must be initialized,dwSize is sizeof(NET_IN_GET_VIDEOINFO)
} NET_IN_GET_VIDEOINFO;

// Video channel state
typedef enum  tagNET_VIDEOCHANNEL_STATE
{
    NET_VIDEOCHANNEL_STATE_UNKNOWN,        // unknown state
    NET_VIDEOCHANNEL_STATE_IDLE,           // idle 
    NET_VIDEOCHANNEL_STATE_PLAY,           // play
    NET_VIDEOCHANNEL_STATE_MONITOR,        // monitor
    NET_VIDEOCHANNEL_STATE_TOUR,           // tour
} NET_VIDEOCHANNEL_STATE;

// video channel info
typedef struct tagNET_VIDEOCHANNELINFO
{
    BOOL                         bEnable;                // the state of channel enable, if state is true,GB28181 protocol will appear channel to server.
    NET_VIDEOCHANNEL_STATE       emVideoChannelState;    // video channel state
    int                          nNetflow;               // net flow(unit: kbps)
    int                          nBitrate;               // bit rate(unit: kbps)
    int                          nFrame;                 // Frame
    CAPTURE_SIZE                 emResolution;           // resolution
    BYTE                         byReserved[512];        // reserve 
} NET_VIDEOCHANNELINFO;


// CLIENT_QueryDevInfo interface NET_QUERY_GET_VIDEOINFO command to output param
typedef struct tagNET_OUT_GET_VIDEOINFO
{
    DWORD                    dwSize;                // dwSize must be initialized,dwSize is sizeof(NET_OUT_GET_VIDEOINFO)
    int                      nVideoInfoNum;         // the video info num which user need to gain,gain from 0 by user
    NET_VIDEOCHANNELINFO*    pNetVideoChannelInfo;  // the list of net video channel info,user malloc memory,the list num is in keeping with nVideoInfoNum, memory size is sizeof(NET_VIDEOCHANNELINFO)*nVideoInfoNum
    int                      nRetVideoInfoNum;      // return video info num actually by SDK
} NET_OUT_GET_VIDEOINFO;



// CLIENT_ControlDeviceEx port NET_CTRL_DEV_THERMO_GRAPHY_ENSHUTTER command to input parameter
typedef struct tagNET_IN_THERMO_EN_SHUTTER
{
    DWORD               dwSize;
    int                 nChannel;                           // channel number
    BOOL                bEnable;                            // switch shutter, TRUE on, FALSE off
} NET_IN_THERMO_EN_SHUTTER;

// CLIENT_ControlDeviceEx port NET_CTRL_DEV_THERMO_GRAPHY_ENSHUTTER command to output parameter
typedef struct tagNET_OUT_THERMO_EN_SHUTTER
{
    DWORD               dwSize;
} NET_OUT_THERMO_EN_SHUTTER;

// CLIENT_GetDevCaps port NET_THERMO_GRAPHY_CAPS command to input parameter
typedef struct tagNET_IN_THERMO_GETCAPS
{
    DWORD               dwSize;
    int                 nChannel;                           // channel number
} NET_IN_THERMO_GETCAPS;

// CLIENT_GetDevCaps port NET_THERMO_GRAPHY_CAPS command to output parameter
typedef struct tagNET_OUT_THERMO_GETCAPS
{
    DWORD               dwSize;
    DWORD               dwModes;                            // supported preset mode mask
    DWORD               dwColorization;                     // preset colorization mode mask
    DWORD               dwROIModes;                         // preset ROI mode mask
    RANGE               stBrightness;                       // brightness ability
    RANGE               stSharpness;                        // sharpness ability
    RANGE               stEZoom;                            // zoom ability
    RANGE               stThermographyGamma;                // Gamma ability
    RANGE               stSmartOptimizer;                   // optimize parameter ability
} NET_OUT_THERMO_GETCAPS;

// CLIENT_GetDevCaps port NET_RADIOMETRY_CAPS command to input parameter
typedef struct tagNET_IN_RADIOMETRY_GETCAPS
{
    DWORD               dwSize;
    int                 nChannel;                           // channel number
} NET_IN_RADIOMETRY_GETCAPS;

// temperature measurement mode type
typedef enum tagNET_RADIOMETRY_METERTYPE 
{
    NET_RADIOMETRY_METERTYPE_UNKNOWN,
    NET_RADIOMETRY_METERTYPE_SPOT,                          // spot
    NET_RADIOMETRY_METERTYPE_LINE,                          // line
    NET_RADIOMETRY_METERTYPE_AREA,                          // area
} NET_RADIOMETRY_METERTYPE;

// total number ability of spot, line and area
typedef struct tagNET_RADIOMETRY_TOTALNUM 
{
    DWORD               dwMaxNum;                           // max support number
    DWORD               dwMaxSpots;                         // max number of spot
    DWORD               dwMaxLines;                         // max number of line
    DWORD               dwMaxAreas;                         // max number of area
    char reserved[32]; 
} NET_RADIOMETRY_TOTALNUM;

// CLIENT_GetDevCaps port NET_RADIOMETRY_CAPS command to output parameter
typedef struct tagNET_OUT_RADIOMETRY_GETCAPS
{
    DWORD                       dwSize;
    NET_RADIOMETRY_TOTALNUM     stTotalNum;                 // total number abllity of spot, line and area
    DWORD                       dwMaxPresets;               // max number of temperature measurement preset
    DWORD                       dwMeterType;                // type mask of temperature measurement mode, refer to NET_RADIOMETRY_METERTYPE 
    RANGE                       stObjectEmissivity;         // emissivity related ability
    RANGE                       stObjectDistance;           // distance related ability
    RANGE                       stReflectedTemperature;     // reflected temperature related ability
    RANGE                       stRelativeHumidity;         // relative humidity related ability
    RANGE                       stAtmosphericTemperature;   // atmospheric temperature related ability
    int                         nStatisticsMinPeriod;       // min storage data interval of temperature measuring point statistics function, unit is second.
    float                       fIsothermMaxTemp;           // color code bar max temperature value
    float                       fIsothermMinTemp;           // color code bar min temperature value
} NET_OUT_RADIOMETRY_GETCAPS;

// temperature measurement info
typedef struct tagNET_RADIOMETRYINFO
{
    int                 nMeterType;                         // return to temperature measurement type, refer to NET_RADIOMETRY_METERTYPE 
    int                 nTemperUnit;                        // temperature unit (currently configured temperature unit), refer to NET_TEMPERATURE_UNIT 
    float               fTemperAver;                        // only return to this field when it is spot temperature or average temperature
    float               fTemperMax;                         // max temperature 
    float               fTemperMin;                         // min temperature 
    float               fTemperMid;                         // middle temperature value        
    float               fTemperStd;                         // standard deviation value
    char                reserved[64];
} NET_RADIOMETRYINFO;

// acquire thermal condition of temperature measurement   
typedef struct tagNET_RADIOMETRY_CONDITION
{
    int                 nPresetId;                          // preset number        
    int                 nRuleId;                            // rule number  
    int                 nMeterType;                         // temperature measurement type, refer to NET_RADIOMETRY_METERTYPE 
    char                szName[64];                         // name of temperature measurement, select from the name of temperature measurement configured rule
    int                 nChannel;                           // channel number
    char                reserved[256];
} NET_RADIOMETRY_CONDITION;

// CLIENT_QueryDevInfo port NET_QUERY_DEV_RADIOMETRY_POINT_TEMPER command to input parameter
typedef struct tagNET_IN_RADIOMETRY_GETPOINTTEMPER
{
    DWORD               dwSize;
    int                 nChannel;                           // channel number
    DH_POINT            stCoordinate;                       // temperature measurement spot coordinate, coordinate value 0~8192 
} NET_IN_RADIOMETRY_GETPOINTTEMPER;

// CLIENT_QueryDevInfo port NET_QUERY_DEV_RADIOMETRY_POINT_TEMPER command to output parameter
typedef struct tagNET_OUT_RADIOMETRY_GETPOINTTEMPER
{
    DWORD               dwSize;
    NET_RADIOMETRYINFO  stPointTempInfo;                    // acquire the parameter value of temperature measurement spot
} NET_OUT_RADIOMETRY_GETPOINTTEMPER;

// CLIENT_QueryDevInfo port NET_QUERY_DEV_RADIOMETRY_TEMPER command to input parameter
typedef struct tagNET_IN_RADIOMETRY_GETTEMPER
{
    DWORD                       dwSize;
    NET_RADIOMETRY_CONDITION    stCondition;                // acquire the temperature condition of temperature measurement 
} NET_IN_RADIOMETRY_GETTEMPER;

// CLIENT_QueryDevInfo port NET_QUERY_DEV_RADIOMETRY_TEMPER command to output parameter
typedef struct tagNET_OUT_RADIOMETRY_GETTEMPER
{
    DWORD               dwSize;
    NET_RADIOMETRYINFO  stTempInfo;                         // acquire parameter value of temperature measurement
} NET_OUT_RADIOMETRY_GETTEMPER;

// CLIENT_ControlDeviceEx port NET_CTRL_DEV_RADIOMETRY_SETOSDMARK command to input parameter
typedef struct tagNET_IN_RADIOMETRY_SETOSDMARK
{
    DWORD                       dwSize;
    NET_RADIOMETRY_CONDITION    stCondition;                // it needs to set the matching condition of osd temperature measurement 
} NET_IN_RADIOMETRY_SETOSDMARK;

// CLIENT_ControlDeviceEx port NET_CTRL_DEV_RADIOMETRY_SETOSDMARK command to output parameter
typedef struct tagNET_OUT_RADIOMETRY_SETOSDMARK
{
    DWORD               dwSize;
} NET_OUT_RADIOMETRY_SETOSDMARK;

//radiometry period
typedef enum tagEM_RADIOMETRY_PERIOD
{
	EM_RADIOMETRY_PERIOD_UNKNOWN,			 // unknown
    EM_RADIOMETRY_PERIOD_5  = 5,             // 5minutes,default
	EM_RADIOMETRY_PERIOD_10 = 10,            // 10minutes
	EM_RADIOMETRY_PERIOD_15 = 15,            // 15minutes
	EM_RADIOMETRY_PERIOD_30 = 30,            // 30minutes
}EM_RADIOMETRY_PERIOD;

// CLIENT_StartFind port NET_FIND_RADIOMETRY command to input parameter
typedef struct tagNET_IN_RADIOMETRY_STARTFIND
{   
    DWORD                  dwSize;
    NET_TIME               stStartTime;                       // query start time
    NET_TIME               stEndTime;                         // query end time
    int                    nMeterType;                        // query type
    int                    nChannel;                          // channel number
    EM_RADIOMETRY_PERIOD   emPeriod;                          // query period
} NET_IN_RADIOMETRY_STARTFIND;

// CLIENT_StartFind port NET_FIND_RADIOMETRY command to output parameter
typedef struct tagNET_OUT_RADIOMETRY_STARTFIND
{   
    DWORD               dwSize;
    int                 nFinderHanle;                      // acquired query handle
    int                 nTotalCount;                       // total number of result which conforms to the query condition
} NET_OUT_RADIOMETRY_STARTFIND;

#define NET_RADIOMETRY_DOFIND_MAX 32

// CLIENT_DoFind port NET_FIND_RADIOMETRY command to input parameter
typedef struct tagNET_IN_RADIOMETRY_DOFIND
{   
    DWORD               dwSize;
    int                 nFinderHanle;                      // query handle
    int                 nBeginNumber;                      // index number of query begin
    int                 nCount;                            // the number of query, max is NET_IN_RADIOMETRY_DOFIND_MAX 
} NET_IN_RADIOMETRY_DOFIND;

// return to query result
typedef struct tagNET_RADIOMETRY_QUERY
{
    NET_TIME            stTime;                            // record time
    int                 nPresetId;                         // preset number
    int                 nRuleId;                           // rule number
    char                szName[64];                        // query name
    DH_POINT            stCoordinate;                      // query temperature measurement coordinate
    int                 nChannel;                          // channel number
    NET_RADIOMETRYINFO  stTemperInfo;                      // temperature measurement info, currently nTemperMid, nTemperStd member invalid
    char                reserved[256];
} NET_RADIOMETRY_QUERY;

// CLIENT_DoFind port_FIND_RADIOMETRY command to output parameter
typedef struct tagNET_OUT_RADIOMETRY_DOFIND
{   
    DWORD                       dwSize;
    int                         nFound;                             // actual query spot number
    NET_RADIOMETRY_QUERY        stInfo[NET_RADIOMETRY_DOFIND_MAX];  // temperature statistics info 
} NET_OUT_RADIOMETRY_DOFIND;

// CLIENT_StopFind port_FIND_RADIOMETRY command to input parameter
typedef struct tagNET_IN_RADIOMETRY_STOPFIND
{   
    DWORD               dwSize;
    int                 nFinderHanle;                       // query handle 
} NET_IN_RADIOMETRY_STOPFIND;

// CLIENT_StopFind port_FIND_RADIOMETRY command to output parameter
typedef struct tagNET_OUT_RADIOMETRY_STOPFIND
{   
    DWORD               dwSize;
} NET_OUT_RADIOMETRY_STOPFIND;

// thermal temperature measurement spot alarm result value type 
typedef enum tagNET_RADIOMETRY_RESULT 
{
    NET_RADIOMETRY_RESULT_UNKNOWN,
    NET_RADIOMETRY_RESULT_VAL,                  // concrete value 
    NET_RADIOMETRY_RESULT_MAX,                  // max
    NET_RADIOMETRY_RESULT_MIN,                  // min
    NET_RADIOMETRY_RESULT_AVR,                  // average
    NET_RADIOMETRY_RESULT_STD,                  // standard
    NET_RADIOMETRY_RESULT_MID,                  // middle
    NET_RADIOMETRY_RESULT_ISO,                  // ISO
    NET_RADIOMETRY_RESULT_DIFF,                 // difference
    NET_RADIOMETRY_RESULT_SLOPE,                // slope
} NET_RADIOMETRY_RESULT;

// thermal temperature measurement spot alarm condition 
typedef enum tagNET_RADIOMETRY_ALARMCONTION 
{
    NET_RADIOMETRY_ALARMCONTION_UNKNOWN,
    NET_RADIOMETRY_ALARMCONTION_BELOW,          // below
    NET_RADIOMETRY_ALARMCONTION_MATCH,          // equal 
    NET_RADIOMETRY_ALARMCONTION_ABOVE,          // above
} NET_RADIOMETRY_ALARMCONTION;

// temperature unit
typedef enum tagNET_TEMPERATURE_UNIT
{
    NET_TEMPERATURE_UNIT_UNKNOWN,
    NET_TEMPERATURE_UNIT_CENTIGRADE,                // Centigrade
    NET_TEMPERATURE_UNIT_FAHRENHEIT,                // Fahrenheit
} NET_TEMPERATURE_UNIT;

// thermal temperature measurement spot temperature abnormity alarm 
typedef struct tagALARM_HEATIMG_TEMPER_INFO
{
    char                szName[64];                         // temperature abnormal spot name, select from the temperature measurement rule config
    int                 nAlarmId;                           // alarm number    
    int                 nResult;                            // alarm result value nValue type, refer to enumeration NET_RADIOMETRY_RESULT 
    int                 nAlarmContion;                      // alarm condition, refer to enumeration NET_RADIOMETRY_ALARMCONTION 
    float               fTemperatureValue;                  // alarm temperature value 
    int                 nTemperatureUnit;                   // temperature unit (currently configured temperature unit), refer to NET_TEMPERATURE_UNIT 
    DH_POINT            stCoordinate;                       // alarm spot coordinate, relative coordinate system, value is 0~8191 
    int                 nPresetID;                          // preset
    int                 nChannel;                           // channel number
    int                 nAction;                            // 0:start 1: stop
	DH_POLY_POINTS      stuAlarmCoordinates;				// alarm coordinates, the type is spot or line or polygon. replace stCoordinate field  
	char                reserved[156];
} ALARM_HEATIMG_TEMPER_INFO;

// thermal kindling point alarm 
typedef struct tagALARM_FIREWARNING_INFO
{
    int                 nPresetId;                          // deprecated, to get same info, use event DH_ALARM_FIREWARNING_INFO instead
    int                 nState;                             // 0-start, 1-stop 
    DH_RECT             stBoundingBox;                      // deprecated, to get same info, use event DH_ALARM_FIREWARNING_INFO instead 	
    int                 nTemperatureUnit;                   // deprecated, to get same info, use event DH_ALARM_FIREWARNING_INFO instead 
    float               fTemperature;                       // deprecated, to get same info, use event DH_ALARM_FIREWARNING_INFO instead 
    unsigned            nDistance;                          // deprecated, to get same info, use event DH_ALARM_FIREWARNING_INFO instead
	GPS_POINT           stGpsPoint;                         // deprecated, to get same info, use event DH_ALARM_FIREWARNING_INFO instead
    int                 nChannel;                           // channel id
    char                reserved[252];
} ALARM_FIREWARNING_INFO;

//PTZ control coordinate unit 
typedef struct tagPTZ_POSITION_UNIT
{
    int                  nPositionX;                        // PTZ horizontal angle,effective range[0,3599]
    int                  nPositionY;                        // PTZ vertical angle,effective range[-1800,1800]
    int                  nZoom;                             // PTZ aperture magnification,the effective range[0,128]
    char                 szReserve[32];                     // Reserved
}PTZ_POSITION_UNIT;

// firewarning info
typedef struct tagNET_FIREWARNING_INFO
{
    int                 nPresetId;                          // preset number is selected from the temperature measurement rule config refer to CFG_RADIOMETRY_RULE_INFO
    NET_RECT            stuBoundingBox;                     // kindling point rectangular box 
    int                 nTemperatureUnit;                   // temperature unit (currently configured temperature unit), refer to NET_TEMPERATURE_UNIT 
    float               fTemperature;                       // max spot temperature value provided by same frame detection and differential detection 
    unsigned            nDistance;                          // kindling point distance, unit is meter, 0 means invalid
    GPS_POINT           stuGpsPoint;                        // kindling point longitude and latitude
    PTZ_POSITION_UNIT   stuPTZPosition;                     // PTZ Info
    float               fAltitude;                          // Altitude(unit: metre)
    BYTE                reserved[208];
}NET_FIREWARNING_INFO;

// firewarning event info
typedef struct tagALARM_FIREWARNING_INFO_DETAIL
{
    int                     nChannel;                                           // channel id
    int                     nWarningInfoCount;                                  // warning info count
    NET_FIREWARNING_INFO    stuFireWarningInfo[MAX_FIREWARNING_INFO_NUM];       // warning info
    BYTE                    reserved[256];
}ALARM_FIREWARNING_INFO_DETAIL;

// face overheating
typedef struct tagALARM_FACE_OVERHEATING_INFO
{
    int                     nRelativeId;                          // picture id of face,refer to NET_FACE_INFO->nRelativeID
    int                     nTemperatureUnit;                     // temperature unit (currently configured temperature unit), refer to NET_TEMPERATURE_UNIT
    float                   fTemperature;                         // temperature value of the face     
    int                     nChannelID;                           // channel id
    BYTE                    reserved[124];
}ALARM_FACE_OVERHEATING_INFO;

// sensor status
typedef enum tagEM_SENSOR_ABNORMAL_STATUS
{
    NET_SENSOR_ABNORMAL_STATUS_UNKNOWN,
    NET_SENSOR_ABNORMAL_STATUS_SHORT,                              // short
    NET_SENSOR_ABNORMAL_STATUS_BREAK,                              // break
    NET_SENSOR_ABNORMAL_STATUS_INTRIDED,                           // intrided
}EM_SENSOR_ABNORMAL_STATUS;

//event type (DH_ALARM_SENSOR_ABNORMAL) 
typedef struct tagALARM_SENSOR_ABNORMAL_INFO
{
    int                                 nAction;                   // 0:start 1:stop
    int                                 nChannelID;                // channel id
    NET_TIME_EX                         stuTime;                   // UTC time
    EM_SENSOR_ABNORMAL_STATUS           emStatus;                  // sensor status
    BYTE                                byReserved[128];
}ALARM_SENSOR_ABNORMAL_INFO;

// heat map metadata info 
typedef struct tagNET_RADIOMETRY_METADATA
{
    int                 nHeight;                            // height 
    int                 nWidth;                             // width 
    int                 nChannel;                           // channel 
    NET_TIME            stTime;                             // acquire data time
    int                 nLength;                            // data size
    char                szSensorType[64];                   // module type
    int                 nUnzipParamR;                       // uncompressing parameter R 
    int                 nUnzipParamB;                       // uncompressing parameter B
    int                 nUnzipParamF;                       // uncompressing parameter F
    int                 nUnzipParamO;                       // uncompressing parameter O
    char                Reserved[256];
} NET_RADIOMETRY_METADATA;

// heat map data 
typedef struct tagNET_RADIOMETRY_DATA
{
    NET_RADIOMETRY_METADATA     stMetaData;                 // metadata
    BYTE*                       pbDataBuf;                  // heat map data buffer (for compressed data, it is temperature data of each pixel inside, it can use metadata info to compress)
    DWORD                       dwBufSize;                  // heat map data buffer size
    char                        reserved[512];
} NET_RADIOMETRY_DATA;

//////////////////////////////////////////////////////////////////////////
/// \fn temperature distribution data state callback function 
/// \brief 
/// \param  LLONG lAttachHandle [OUT] subscription handle, CLIENT_RadiometryAttach returned value
/// \param  NET_RADIOMETRY_DATA* pBuf [OUT] heat map data info
/// \param  int nBufLen [OUT] state info length
/// \param  param  LDWORD dwUser user data
/// \return none
///////////////////////////////////////////////////////////////////////////
typedef void (CALLBACK *fRadiometryAttachCB)(LLONG lAttachHandle, NET_RADIOMETRY_DATA* pBuf, int nBufLen, LDWORD dwUser);

// CLIENT_RadiometryAttach input parameter
typedef struct tagNET_IN_RADIOMETRY_ATTACH
{
    DWORD               dwSize;
    int                 nChannel;                           // video channel number -1, it means all
    fRadiometryAttachCB cbNotify;                           // state callback function pointer
    LDWORD              dwUser;                             // user data 
} NET_IN_RADIOMETRY_ATTACH;

// CLIENT_RadiometryAttach output parameter
typedef struct tagNET_OUT_RADIOMETRY_ATTACH
{
    DWORD               dwSize;
} NET_OUT_RADIOMETRY_ATTACH;

// subscribe temperature distribution data (heat map), user malloc and free memory of pInParam and pOutParam
CLIENT_NET_API LLONG CALL_METHOD CLIENT_RadiometryAttach(LLONG lLoginID, const NET_IN_RADIOMETRY_ATTACH* pInParam, NET_OUT_RADIOMETRY_ATTACH* pOutParam, int nWaitTime);

// cancel subscribing temperature distribution data, lAttachHandle is the returned value of CLIENT_RadiometryAttach
CLIENT_NET_API BOOL CALL_METHOD CLIENT_RadiometryDetach(LLONG lAttachHandle);

// CLIENT_RadiometryFetch input parameter
typedef struct tagNET_IN_RADIOMETRY_FETCH
{
    DWORD               dwSize;
    int                 nChannel;                           // channel number, channel number needs to be in accordance with subscription, -1 excluded
} NET_IN_RADIOMETRY_FETCH;

// CLIENT_RadiometryFetch output parameter
typedef struct tagNET_OUT_RADIOMETRY_FETCH 
{
    DWORD               dwSize;
    int                 nStatus;                            // 0: unknown, 1: free, 2: acquiring heat map
} NET_OUT_RADIOMETRY_FETCH;

// inform to start acquiring heat map data,user malloc and free memory of pInParam and pOutParam
CLIENT_NET_API BOOL CALL_METHOD CLIENT_RadiometryFetch(LLONG lLoginID, const NET_IN_RADIOMETRY_FETCH* pInParam, NET_OUT_RADIOMETRY_FETCH* pOutParam, int nWaitTime);

// radiometry data unachieving and conversion
/// \brief 
/// \param  pRadiometryData [IN] radiometry data, obtained from fRadiometryAttachCB 
/// \param  pGrayImg [IN, OUT] unachieved data, which is a grayscale image
///			you can pass NULL if this is not needed
///         you must ensure that the buffer is enough (no less than 'pixel number'*sizeof(unsigned short))
///         for each pixel, there is an unsigned short value representing the grayscale value (range: 0 ~ 16383)
///         and the ordering is top-left pixels in low memory address and lower-right pixels in high memory address 
/// \param  pTempForPixels [IN, OUT] temperature for each pixel
///			you can pass NULL if this is not needed
///         you must ensure that the buffer is enough (no less than 'pixel number'*sizeof(float))
///         for each pixel, there is a float value representing temperature at pixel position, in centigrade
///         and the ordering is top-left pixels in low memory address and lower-right pixels in high memory address 
/// \return TRUE for success, FALSE for failure
CLIENT_NET_API BOOL CALL_METHOD CLIENT_RadiometryDataParse(const NET_RADIOMETRY_DATA* pRadiometryData, unsigned short* pGrayImg, float* pTempForPixels);

// CLIENT_GetRadiometryYImage input parameter
typedef struct tagNET_IN_RADIOMETRY_Y_IMAGE
{
    DWORD                       dwSize;
    int                         nChannel;                               // channel id
}NET_IN_RADIOMETRY_Y_IMAGE;

// CLIENT_GetRadiometryYImage output parameter
typedef struct tagNET_OUT_RADIOMETRY_Y_IMAGE
{   
    DWORD                       dwSize;
    UINT                        nWidth;                                 // the width of image
    double                      PTS;                                    // timestamp (unit is millisecond)
    UINT                        nHeight;                                // the height of image
    char*                       pbDataBuf;                              // Y Date buffer(user alloc this memory, size is nBufLen)
    int                         nBufLen;                                // pbDataBuf max len
    int                         nBufRet;                                // pbDataBuf len
}NET_OUT_RADIOMETRY_Y_IMAGE;

// Get Y Image of thermal camera
CLIENT_NET_API BOOL CALL_METHOD CLIENT_GetRadiometryYImage(LLONG lLoginID, const NET_IN_RADIOMETRY_Y_IMAGE* pInParam, NET_OUT_RADIOMETRY_Y_IMAGE* pOutParam,int nWaitTime);

//////////////////////////////////////////////////////////////////////////
// query the information of flux
//////////////////////////////////////////////////////////////////////////

// input parameter for flux information
typedef struct tagNET_IN_SIM_CARD_FLUX_INFO
{	
	DWORD dwSize;                 // size of this struct
	int nCardIndex;               // sim number, start from zero
}NET_IN_SIM_CARD_FLUX_INFO;

#define NET_FLUX_RECORD_MAX_NUM 7      // the max count for information of flux

// information of flux structure
typedef struct tagNET_SIM_CARD_FLUX_RECORD
{	
	NET_TIME stuDate;            // date
	UINT nDaySendFlux;           // the up flux on that day, unit:0.1kb
	UINT nDayRecvFlux;           // the down flux on that day, unit:0.1kb
	BYTE byReserved[512];        // reserved bytes

}NET_SIM_CARD_FLUX_RECORD;

// output parameter for flux information
typedef struct tagNET_OUT_SIM_CARD_FLUX_INFO
{
	DWORD dwSize;                                        // size of this struct        
	UINT nTotalFlux;                                     // the total flux, unit: 0.1kb  
	UINT nSendFlux;                                      // the up flux, unit: 0.1kb 
	UINT nRecvFlux;                                      // the down flux, unit: 0.1kb
	NET_TIME stuTime;                                    // the date of writing in the total flux
	int nRecordNum;                                      // the number of records, this value must LE NET_RECORD_MAX_NUM
	NET_SIM_CARD_FLUX_RECORD stuRecord[NET_FLUX_RECORD_MAX_NUM];  // the records of the last seven days
	UINT nLastMonthTotalFlux;                                     // the total flux of last month , unit: 0.1kb

}NET_OUT_SIM_CARD_FLUX_INFO;

// get the flux informattion of the sim card,user malloc memory of pNetDataIn and pNetDataOut
CLIENT_NET_API BOOL CALL_METHOD CLIENT_GetSimCardFlux(LLONG lLoginID,const NET_IN_SIM_CARD_FLUX_INFO* pNetDataIn,NET_OUT_SIM_CARD_FLUX_INFO* pNetDataOut,int nWaitTime = 1000);

// set the work mode of door,user malloc memory of pNetDataIn and pNetDataOut
CLIENT_NET_API BOOL CALL_METHOD CLIENT_SetDoorWorkMode(LLONG lLoginID,const NET_IN_CTRL_ACCESS_SET_DOOR_WORK_MODE* pNetDataIn,NET_OUT_CTRL_ACCESS_SET_DOOR_WORK_MODE* pNetDataOut,int nWaitTime = NET_INTERFACE_DEFAULT_TIMEOUT);

// control the cabin's led,please refer to emType to ensure structure,then ensure memory size
CLIENT_NET_API BOOL CALL_METHOD CLIENT_ControlCabinLED(LLONG lLoginID, EM_CABIN_LED_CONTROL_TYPE emType,const void *pInParam, void* pOutParam, int nWaitTime = NET_INTERFACE_DEFAULT_TIMEOUT);


#define NET_SELECT_UPDATE_LOCK_MAX_NUM 32               // the max selection count when updating lock

// intput param for smart lock's channel info
typedef struct tagNET_IN_SELECT_UPDATE_LOCK_INFO
{
    DWORD dwSize;
    int nChannelNum;                                    // count of selected smart locks
    int nChannel[NET_SELECT_UPDATE_LOCK_MAX_NUM];       // array of selected smart lock channels
}NET_IN_SELECT_UPDATE_LOCK_INFO;

// output param for smart lock's channel info
typedef struct tagNET_OUT_SELECT_UPDATE_LOCK_INFO
{
    DWORD dwSize;   
}NET_OUT_SELECT_UPDATE_LOCK_INFO;

// select the smart lock(s) by channel(s),user malloc memory of pNetDataIn and pNetDataOut
CLIENT_NET_API BOOL CALL_METHOD CLIENT_SelectLockToUpdate(LLONG lLoginID,const NET_IN_SELECT_UPDATE_LOCK_INFO* pNetDataIn,NET_OUT_SELECT_UPDATE_LOCK_INFO* pNetDataOut,int nWaitTime = NET_INTERFACE_DEFAULT_TIMEOUT);

typedef enum tagEM_AUTHORITY_SUB_TYPE
{
    EM_AUTHORITY_SUB_TYPE_UNKNOWN,                            // Unknown
    EM_AUTHORITY_SUB_TYPE_SHUT_DOWN,                          // shut down device
    EM_AUTHORITY_SUB_TYPE_MONITOR,                            // monitor for all channel
    EM_AUTHORITY_SUB_TYPE_MONITOR_FOR_CHANNEL,                // monitor for some channel
    EM_AUTHORITY_SUB_TYPE_REPLAY,                             // replay for all channel
    EM_AUTHORITY_SUB_TYPE_REPLAY_FOR_CHANNEL,                 // replay for some channel
    EM_AUTHORITY_SUB_TYPE_BACKUP,                             // back up record file
    EM_AUTHORITY_SUB_TYPE_MHARDISK,                           // manage the hard disk
    EM_AUTHORITY_SUB_TYPE_MPTZ,                               // manage the PTZ(invalided if HasPtz is false)
    EM_AUTHORITY_SUB_TYPE_ACCOUNT,                            // account
    EM_AUTHORITY_SUB_TYPE_QUERY_LOG,                          // query log
    EM_AUTHORITY_SUB_TYPE_DEL_LOG,                            // delete log 
    EM_AUTHORITY_SUB_TYPE_SYS_UPDATE,                         // update system
    EM_AUTHORITY_SUB_TYPE_AUTO_MAINTAIN,                      // auto maintain
    EM_AUTHORITY_SUB_TYPE_GENERAL_CONF,                       // general config
    EM_AUTHORITY_SUB_TYPE_ENCODE_CONF,                        // encode config
    EM_AUTHORITY_SUB_TYPE_RECORD,                             // record file
    EM_AUTHORITY_SUB_TYPE_RECORD_CONF,                        // record config
    EM_AUTHORITY_SUB_TYPE_COM_CONF,                           // com config
    EM_AUTHORITY_SUB_TYPE_NET_CONF,                           // net config
    EM_AUTHORITY_SUB_TYPE_ALARM,                              // alarm I/O config
    EM_AUTHORITY_SUB_TYPE_ALARM_CONF,                         // alarm config
    EM_AUTHORITY_SUB_TYPE_VIDEO_CONFIG,                       // video config
    EM_AUTHORITY_SUB_TYPE_PTZ_CONFIG,                         // PTZ config
    EM_AUTHORITY_SUB_TYPE_OUTPUT_CONFIG,                      // video output config
    EM_AUTHORITY_SUB_TYPE_VIDEO_INPUT_CONFIG,                 // video input config
    EM_AUTHORITY_SUB_TYPE_DEFAULT_CONFIG,                     // default config
    EM_AUTHORITY_SUB_TYPE_BK_CONFIG,                          // backup config
    EM_AUTHORITY_SUB_TYPE_INTELLI_CONFIG,                     // intelli config
    EM_AUTHORITY_SUB_TYPE_REMOTE_DEVICE,                      // add or remove remote device
    EM_AUTHORITY_SUB_TYPE_ATM_POS,                            // atm
    EM_AUTHORITY_SUB_TYPE_OFFLINE_LOGINED_USER,               // kick user
    EM_AUTHORITY_SUB_TYPE_AUDIO_AUTH,                         // audio
    EM_AUTHORITY_SUB_TYPE_SPOT_SET,                           // set spot
    EM_AUTHORITY_SUB_TYPE_TVSET,                              // set tc
    EM_AUTHORITY_SUB_TYPE_IPSAN,                              // manage IPSAN
    EM_AUTHORITY_SUB_TYPE_CONFIG,                             // config
    EM_AUTHORITY_SUB_TYPE_ARMING,                             // arming
    EM_AUTHORITY_SUB_TYPE_DISARM,                             // disarming
    EM_AUTHORITY_SUB_TYPE_BYPASS,                             // bypass
    EM_AUTHORITY_SUB_TYPE_BYPASS_PARTIAL,                     // bypass fo some channel
    EM_AUTHORITY_SUB_TYPE_ALARM_BELL,                         // alarm bell
    EM_AUTHORITY_SUB_TYPE_ALARM_CONFIRM_PARTIAL,              // alarm config for some channel
    EM_AUTHORITY_SUB_TYPE_DEFENCE_CONFIG_PARTIAL,             // defence config for some channel
    EM_AUTHORITY_SUB_TYPE_ALARM_OUT_PARITAL,                  // alarm output config for some channel
    EM_AUTHORITY_SUB_TYPE_TALK,                               // talk
    EM_AUTHORITY_SUB_TYPE_WIRELESS_CONFIG,                    // wireless config
    EM_AUTHORITY_SUB_TYPE_MOBILE_CONFIG,                      // mobile config
    EM_AUTHORITY_SUB_TYPE_AUDIO_DETECT_CONFIG,                // audio detection config
    EM_AUTHORITY_SUB_TYPE_NET_PREVIEW_FOR_CHANNEL,            // net preview for some channel

    EM_AUTHORITY_SUB_TYPE_REBOOT,                             // reboot
    EM_AUTHORITY_SUB_TYPE_SYS_LENS,                           // system lens
    EM_AUTHORITY_SUB_TYPE_RAIN_BRUSH_CONFIG,                  // rain brush config
    EM_AUTHORITY_SUB_TYPE_LIGHTING_CONFIG,                    // lighting config
    EM_AUTHORITY_SUB_TYPE_ACCESS_CONTROL,                     // access control for all channel
    EM_AUTHORITY_SUB_TYPE_ACCESS_CONTROL_PARTIAL,             // access control for some channel
    EM_AUTHORITY_SUB_TYPE_DECODE_TOUR,                        // decode tour
    EM_AUTHORITY_SUB_TYPE_VIDEO_OUT,                          // video out
    EM_AUTHORITY_SUB_TYPE_PTZ_FOR_CHANNEL,                    // ptz for some channel
    EM_AUTHORITY_SUB_TYPE_MODIFY_LANGUANGE,                   // modify languange
    EM_AUTHORITY_SUB_TYPE_MODIFY_VIDEO,                       // modify video
    EM_AUTHORITY_SUB_TYPE_COURSE_RECORD_CONTROL,              // course record control
    EM_AUTHORITY_SUB_TYPE_COURSE_RECORD_SCHEDULE,             // course record schedule
    EM_AUTHORITY_SUB_TYPE_COURSE_RECORD_MODIFY_SCHEDULE,      // course record modify schedule
    EM_AUTHORITY_SUB_TYPE_COURSE_RECORD_INTELL_PARAM,         // course record intell param
    EM_AUTHORITY_SUB_TYPE_CAMERA_PERIPHERAL,                  // camera peripheral
    EM_AUTHORITY_SUB_TYPE_COURSE_RECORD_DOWNLOAD_MEDIA_FILE,  // course record download media file
    EM_AUTHORITY_SUB_TYPE_COURSE_RECORD_MODIFY_MEDIA_FILE,    // course record modify media file
    EM_AUTHORITY_SUB_TYPE_COURSE_RECORD_DELETE_MEDIA_FILE,    // course record delete media file
    EM_AUTHORITY_SUB_TYPE_COURSE_RECORD_BACKUP_TO_USB,        // course record backup to usb
    EM_AUTHORITY_SUB_TYPE_COURSE_RECORD_BACKUP_TO_FTP,        // course record backup to ftp
    EM_AUTHORITY_SUB_TYPE_COURSE_RECORD_EXPORT_ACCONUT,       // course record export account
    EM_AUTHORITY_SUB_TYPE_COURSE_RECORD_QUERY_ACCOUNT,        // course record query account
    EM_AUTHORITY_SUB_TYPE_COURSE_RECORD_MODIFY_ACCOUNT,       // course record modify account
    EM_AUTHORITY_SUB_TYPE_COURSE_RECORD_RESET_PASSWORD,       // course record reset password
    EM_AUTHORITY_SUB_TYPE_COURSE_RECORD_AUTHORITY,            // course record authority
}EM_AUTHORITY_SUB_TYPE;

typedef enum tagEM_AUTHORITY_MAIN_TYPE
{
    EM_AUTHORITY_MAIN_TYPE_UNKNOWN,            // Unknown
    EM_AUTHORITY_MAIN_TYPE_AUTH_USER_MAG,      // user manage
    EM_AUTHORITY_MAIN_TYPE_AUTH_SYS_CFG,       // system config
    EM_AUTHORITY_MAIN_TYPE_AUTH_OFFLINE_USER,  // offline user
    EM_AUTHORITY_MAIN_TYPE_AUTH_DF_UD,         // default and update
    EM_AUTHORITY_MAIN_TYPE_AUTH_PTZ_CTR,       // ptz control
    EM_AUTHORITY_MAIN_TYPE_AUTH_SYS_INFO,      // system info
    EM_AUTHORITY_MAIN_TYPE_AUTH_MANU_CTR,      // manual control
    EM_AUTHORITY_MAIN_TYPE_AUTH_BACKUP,        // back up file
    EM_AUTHORITY_MAIN_TYPE_AUTH_COLOR_SET,     // color set
    EM_AUTHORITY_MAIN_TYPE_AUTH_STORE_CFG,     // store config
    EM_AUTHORITY_MAIN_TYPE_AUTH_EVENT_CFG,     // event config
    EM_AUTHORITY_MAIN_TYPE_AUTH_NET_CFG,       // net config
    EM_AUTHORITY_MAIN_TYPE_AUTH_RMT_DEVICE,    // remote device
    EM_AUTHORITY_MAIN_TYPE_AUTH_DEL_LOG,       // delete log
    EM_AUTHORITY_MAIN_TYPE_AUTH_SHUTDOWN,      // shut down device
}EM_AUTHORITY_MAIN_TYPE;

#define NET_AUTHORITY_CLASS_MAX_NUM 256      // the max num for authority type

// authority info 
typedef struct tagNET_AUTH_CLASSIFY_INFO
{	    
    EM_AUTHORITY_MAIN_TYPE  emMainAuthority;    // the main authority
    EM_AUTHORITY_SUB_TYPE   emSubAuthority;     // the sub authority
    int                     nChannel;           // the channel
                                                // it is valid only when the emSubAuthority is EM_AUTHORITY_SUB_TYPE_MONITOR_FOR_CHANNEL,
                                                // EM_AUTHORITY_SUB_TYPE_REPLAY_FOR_CHANNEL,EM_AUTHORITY_SUB_TYPE_NET_PREVIEW_FOR_CHANNEL or EM_AUTHORITY_SUB_TYPE_PTZ_FOR_CHANNEL.

    BYTE                    byReserved[128];    // reserved
}NET_AUTH_CLASSIFY_INFO;

// input param for authority info
typedef struct tagNET_IN_GET_AUTHORITY_INFO_LIST
{
    DWORD dwSize;
}NET_IN_GET_AUTHORITY_INFO_LIST;

// output param for authority info
typedef struct tagNET_OUT_GET_AUTHORITY_INFO_LIST
{
    DWORD dwSize;
    int nRetAuthInfoCount;                                          // the returen num of authority info
    NET_AUTH_CLASSIFY_INFO stuAuthInfo[NET_AUTHORITY_CLASS_MAX_NUM];// the data of authority info

}NET_OUT_GET_AUTHORITY_INFO_LIST;

// get the list of authority info,user malloc memory of pNetDataIn and pNetDataOut
CLIENT_NET_API BOOL CALL_METHOD CLIENT_GetAuthClassifyList(LLONG lLoginID,const NET_IN_GET_AUTHORITY_INFO_LIST* pNetDataIn,NET_OUT_GET_AUTHORITY_INFO_LIST* pNetDataOut,int nWaitTime = NET_INTERFACE_DEFAULT_TIMEOUT);

//////////////////////////////////////////////////////////////////////////
// Capability Information
//////////////////////////////////////////////////////////////////////////

//////////////////////////////////////////////////////////////////////////
//	Search Type
typedef enum
{
	ABILITY_DYNAMIC_CONNECT = 1,        // dynamic connect capacity,corrresponding type is BOOL
	ABILITY_WATERMARK_CFG = 17,			// Watermark configuration capacity,the coresponding structure is DH_WATERMAKE_EN
	ABILITY_WIRELESS_CFG = 18,			// wireless  configuration capacity,the coresponding structure is DH_WIRELESS_EN
	ABILITY_DEVALL_INFO = 26,			// Device capacity list,the coresponding structure is DH_DEV_ENABLE_INFO 
	ABILITY_CARD_QUERY = 0x0100,		// Card number search capacity,the coresponding structure is DH_CARD_QUERY_EN 
	ABILITY_MULTIPLAY = 0x0101,			// Multiple-window preview capacity,the coresponding structure is  DH_MULTIPLAY_EN
	ABILITY_QUICK_QUERY_CFG = 0x0102,	// Fast query configuration Capabilities,the coresponding structure is DH_QUICK_QUERY_CFG_EN
	ABILITY_INFRARED = 0x0121,			// Wireless alarm capacity ,the coresponding structure is DH_WIRELESS_ALARM_INFO
	ABILITY_TRIGGER_MODE = 0x0131,		// Alarm activation mode function,corresponding type is int
	ABILITY_DISK_SUBAREA = 0x0141,		// Network hard disk partition,the coresponding structure is DH_DISK_SUBAREA_EN
	ABILITY_DSP_CFG = 0x0151,			// Query DSP Capabilities,the coresponding structure is DH_DSP_CFG
	ABILITY_STREAM_MEDIA = 0x0161,		// Query SIP,RTSP Capabilities,the coresponding structure is DH_STREAM_MEDIA_EN
	ABILITY_INTELLI_TRACKER = 0x0171,   // Search intelligent track capability,the coresponding structure is DH_INTELLI_TRACKER_EN
} DH_SYS_ABILITY;

//////////////////////////////////////////////////////////////////////////
//	The function list device supported 
enum 
{
	EN_FTP = 0,						// FTP bitwise, 1: send out record file;  2: Send out snapshot file
	EN_SMTP,						// SMTP bitwise,1: alarm send out text mail 2: Alarm send out image3:support HealthMail
	EN_NTP,							// NTP	 Bitwise:1:Adjust system time 
	EN_AUTO_MAINTAIN,				// Auto maintenance  Bitwise:1:reboot 2:close  3:delete file
	EN_VIDEO_COVER,					// Privacy mask Bitwise  :1:multiple-window privacy mask 
	EN_AUTO_REGISTER,				// Auto registration	Bitwise:1:SDK auto log in after registration 
	EN_DHCP,						// DHCP	Bitwise 1:DHCP
	EN_UPNP,						// UPNP	Bitwise 1:UPNP
	EN_COMM_SNIFFER,				// COM sniffer  Bitwise :1:CommATM
	EN_NET_SNIFFER,					// Network sniffer Bitwise : 1:NetSniffer
	EN_BURN,						// Burn function Bitwise 1:Search burn status 
	EN_VIDEO_MATRIX,				// Video matrix Bitwise  1:Support video matrix or not 2:Support SPOT video matrix or not
	EN_AUDIO_DETECT,				// Video detection Bitwise :1:Support video detection or not 
	EN_STORAGE_STATION,				// Storage position Bitwise:1:Ftp server (Ips) 2:SBM 3:NFS 16:DISK 17:Flash disk 
	EN_IPSSEARCH,					// IPS storage search  Bitwise  1:IPS storage search 	
	EN_SNAP,						// Snapshot Bitwise  1:Resoluiton 2:Frame rate 3:Snapshoot  4:Snapshoot file image; 5:Image quality 
	EN_DEFAULTNIC,					// Search default network card search  Bitwise  1:Support
	EN_SHOWQUALITY,					// Image quality configuration time in CBR mode 1:support 
	EN_CONFIG_IMEXPORT,				// Configuration import& emport function capacity.  Bitwise   1:support 
	EN_LOG,							// Support search log page by page or not. Bitwise 1:support 
	EN_SCHEDULE,					// Record setup capacity. Bitwise  1:Redandunce  2:Pre-record 3:Record period
	EN_NETWORK_TYPE,				// Network type. Bitwise 1:Wire Network 2:Wireless Network 3:CDMA/GPRS,4:CDMA/GPRS multi network card
	EN_MARK_IMPORTANTRECORD,		// Important record. Bitwise 1:Important record mark
	EN_ACFCONTROL,					// Frame rate control activities. Bitwise 1:support frame rate control activities;2:support timing alarm type activate frame rate control(it does not support dynamic detection), this ability mutually exclusive with ACF ability.
	EN_MULTIASSIOPTION,				// Multiple-channel extra stream. Bitwise:1:support three channel extra stream
	EN_DAVINCIMODULE,				// Component modules bitwise: 1.Separate processing the schedule 2.Standard I franme Interval setting
	EN_GPS,                         // GPS function bitwise:1:Gps locate function	
	EN_MULTIETHERNET,				// Support multi net card query   bitwise: 1: support
	EN_LOGIN_ATTRIBUTE,             // Login properties   bitwise: 1: support query login properties  
	EN_RECORD_GENERAL,				// Recording associated  bitwise: 1:Normal recording; 2:Alarm recording; 
									// 3:Motion detection recording;  4:Local storage; 5: Network storage ;  
									// 6:Redundancy storage;  7:Local emergency storage
	EN_JSON_CONFIG,					// Whether support Json configuration, bitwise: 1: support Json
	EN_HIDE_FUNCTION,				// Hide function:bitwise::1,hide PTZ function
	EN_DISK_DAMAGE,                 // Harddisk damage information support ability: bitwise:1,harddisk damage information
	EN_PLAYBACK_SPEED_CTRL,			// Support playback network transmission speed control, bitwise::1 support playback acceleration 
	EN_HOLIDAYSCHEDULE,				// Support holiday period setup : bitwise:1,Support holiday period setup 
	EN_FETCH_MONEY_TIMEOUT,			// ATM fetch money overtime
	EN_BACKUP_VIDEO_FORMAT,			// Device backup support format. DAV, ASF
	EN_QUERY_DISK_TYPE,             // backup disk type query
	EN_CONFIG_DISPLAY_OUTPUT,       // backup device output of display (such as VGA) configuration, by bit: 1: configuration on tour of frame segmentation 
	EN_SUBBITRATE_RECORD_CTRL,      // backup extra stream control configuration, by bit: 1-extra stream control configuration
	EN_IPV6,                        // backup IPV6 configuration, by bit:1-IPV6 configuration
   	EN_SNMP,                        // SNMP
	EN_QUERY_URL,                   // back up query device's URL info, by bit: 1-query device's config URL info
	EN_ISCSI,						// ISCSI
	EN_RAID,						// Raid
	EN_HARDDISK_INFO,				// Support disk info query
	EN_PICINPIC,                    // support picture in pictu,by bit:1,set; 2,preview , record , query record , download record
	EN_PLAYBACK_SPEED_CTRL_SUPPORT, // same to EN_PLAYBACK_SPEED_CTRL
	EN_LF_XDEV,						// support LF-X series of 24, 32, 64 channels, label their encode ability with sepcial calculation, by bit 1: able;
	EN_DSP_ENCODE_CAP,				// support F5 DSP encode
	EN_MULTICAST,                   // support different multicast config for different channel
	EM_NET_LIMIT,   				// query the limit ability of net, bitwise,1-limit size of net send code stream,2-user operate data encrypt,4-config data encrypt
	EM_COM422, 						// serial port 422
	EM_PROTOCAL_FRAMEWORK,			// support three generations of framework agrement or not(need actualize listMethod(),listService()),by F6 to visit
	EM_WRITE_DISK_OSD,				// write disk OSD overlying ,bitwise, 1-write disk OSD overlying configuration
	EM_DYNAMIC_MULTI_CONNECT,		// dynamic multi-connect,bitise,1-request reply video data
	EM_CLOUDSERVICE,  				// cloud service,bitwise,1- support private cloud service
	EM_RECORD_INFO,					// Video Information Report, by bit. 1-Active video information report, 2-Frame numbers inquiry support
	EN_DYNAMIC_REG,                 // Active Register Support, by bit. 1- Dynamic active register support. 2-IP and port allow 0 when SDK ask a dynamic link in active register
	EM_MULTI_PLAYBACK,              // Multi-channel Preview and Playback, by bit. 1-Multi-channel preview and playback support.
	EN_ENCODE_CHN,					// Encoding Channel, by bit. 1- Audio-only channel support
    EN_SEARCH_RECORD,               // Record search, by bit, 1-support sync search record, 2-support 3rd generation protocol search record
    EN_UPDATE_MD5,                  // Support MD5 check after update file send finish, 1- support MD5   2-support 3rd generation protocol upgrade
	EN_PROTOCOL3ToF6,               // protocol3 to F6, 1-support log 2.restore config by configManager.deleteFile protocol
};
typedef struct 
{
	DWORD IsFucEnable[512];			// Function list capacity set. Corresponding to the above mentioned enumeration. Use bit to represent sub-function.
} DH_DEV_ENABLE_INFO;

//////////////////////////////////////////////////////////////////////////
//Card number search function structure 
typedef struct 
{
	char		IsCardQueryEnable;
	char		iRev[3];
} DH_CARD_QUERY_EN;

//////////////////////////////////////////////////////////////////////////
//	Wireless capacity structure 
typedef struct 
{
	char		IsWirelessEnable;
	char		iRev[3];
} DH_WIRELESS_EN;

//////////////////////////////////////////////////////////////////////////
//	Image watermark capacity structure 
typedef struct 
{
	char		isSupportWM;		// 1:Support; 0 Do not support
	char		supportWhat;		// 0:Characrer watermark; 1:Image watermark; 2:Support character watermark and image watermark at the same time.
	char		reserved[2];
} DH_WATERMAKE_EN;

//////////////////////////////////////////////////////////////////////////
//	Multiple-window preview capacity structure 
typedef struct  
{
	int			nEnable;				// 1:Support;  0 :Do not support 
	DWORD		dwMultiPlayMask;		// Multiple-window preview mask 
	char		reserved[4];			// Reserved 
} DH_MULTIPLAY_EN;

//////////////////////////////////////////////////////////////////////////
//	Wireless alarm capacity structure 
typedef struct  
{
	BOOL		bSupport;				// Support or not 
	int			nAlarmInCount;			// Input amount
	int			nAlarmOutCount;			// Output amount 
	int			nRemoteAddrCount;		// Remote control amount 
	BYTE		reserved[32];
}DH_WIRELESS_ALARM_INFO;

//////////////////////////////////////////////////////////////////////////
// Network hard disk partition capacity structure
typedef struct 
{
	BOOL		bSupported;				// Support or not
	int			nSupportNum;			// Support the number of disk partition
	BYTE		bReserved[32];
} DH_DISK_SUBAREA_EN;

// DSP capabilities query ,use when DSP capabilities algorithm with ID 2.
typedef struct  
{
	BYTE bMainFrame[32];				//Use the resolution enumeration value (CAPTURE_SIZE) as index, the main code stream corresponds to the max resolution it support, if do not support, the value should be 0.
	BYTE bExtraFrame_1[32];				//Extra code stream1,use the same method as: bMainFrame
	BYTE bReserved[128];				//obligate for extra code stream 2 and 3.		
}DH_DSP_CFG_ITEM;

typedef struct  
{
	int nItemNum;						//Valid number of DH_DSP_CFG_ITEM, equals to channel number
	DH_DSP_CFG_ITEM	stuDspCfgItem[32];	//Main code stream Information
	BYTE bReserved[128];				//Retain
}DH_DSP_CFG; 

//////////////////////////////////////////////////////////////////////////
//	Fast query configuration capabilities struct 
typedef struct 
{
	char		IsQuickQueryEnable;    //1 is device support configuration command go back easily,please set enough configuration time to ensure reading the configuration in narrowband. Recommended for 60S
	char		iRev[3];
} DH_QUICK_QUERY_CFG_EN;

typedef struct  
{
	int			nStreamType;			// 0,null 1,SIP 2,RTSP
	BYTE		bReserved[16];			// Reserved
} DH_STREAM_MEDIA_EN;

//Search intelligent speed dome track capability 
typedef struct
{
	char		IsIntelliTrackerEnable;	// Has intelligent speed dome track capability if it is more than 0
	BYTE		bReserved[3];
}DH_INTELLI_TRACKER_EN;

#define NET_MAX_COMPOSITE_CHANNEL       256                 // Max number of composite channel 

// The input parameters of Getting Composite caps, corresponding to NET_COMPOSITE_CAPS
typedef struct tagNET_IN_COMPOSITE_CAPS 
{
    DWORD           dwSize;
    int             nChannelCount;                          // Channel count
    int             nChannels[NET_MAX_COMPOSITE_CHANNEL];   // Channel list
    int             nLayoutX;                               // window count of horizontal
    int             nLayoutY;                               // window count of vertical
} NET_IN_COMPOSITE_CAPS;

// The output parameters of Getting Composite caps, corresponding to NET_COMPOSITE_CAPS
typedef struct tagNET_OUT_COMPOSITE_CAPS 
{
    DWORD           dwSize;
    int             nSplitModeCount;                        // number of split mode 
    DH_SPLIT_MODE   emSplitModes[DH_MAX_SPLIT_MODE_NUM];    // list of split mode 
    int             nMaxFreeWindow;                         // max number of free window
} NET_OUT_COMPOSITE_CAPS;

#define MAX_REGISTER_ID_LEN       (256)
#define MAX_APP_ID_LEN            (256)
#define MAX_URL_LEN               (1024)
#define MAX_IOS_CERTIFICATE_LEN   (1024 * 14)
#define MAX_IOS_SECRET_KEY_LEN    (512)
#define MAX_MOBILE_CHANNEL_NUM    (256)

// Mobile server type
typedef enum tagEM_MOBILE_SERVER_TYPE
{
    EM_MOBILE_SERVER_TYPE_UNKNOWN,             // Unknown
    EM_MOBILE_SERVER_TYPE_APPLE,               // Apple
    EM_MOBILE_SERVER_TYPE_ANDROID,             // Android
    EM_MOBILE_SERVER_TYPE_SDKCLOUD,            // Private_CLOUD
} EM_MOBILE_SERVER_TYPE;

// Push server config info
typedef struct tagNET_PUSH_SERVER_INFO
{
    char                            szAddress[MAX_URL_LEN];                    // ip address
    int                             nPort;                                     // port
} NET_PUSH_SERVER_INFO;

// Event sub code type
typedef enum tagEM_EVENT_SUB_CODE
{
    EM_EVENT_SUB_CODE_UNKNOWN = -1,            // Unknown
    EM_EVENT_SUB_CODE_LOSS,                    // Loss
    EM_EVENT_SUB_CODE_COVER,                   // Cover
    EM_EVENT_SUB_CODE_FROZEN,                  // Frozen
    EM_EVENT_SUB_CODE_LIGHT,                   // Light
    EM_EVENT_SUB_CODE_DARK,                    // Dark
    EM_EVENT_SUB_CODE_SCENECHANGE,             // SceneChange
} EM_EVENT_SUB_CODE;

// Subscrbie info
typedef struct tagNET_SUBSCRIBE_INFO
{
    int                             nCode;                                     // Event code
    EM_EVENT_SUB_CODE               emSubCode;                                 // Sub Event code, invaild when it is EM_EVENT_SUB_CODE_UNKNOWN
    int                             nChnNum;                                   // channel number 
    int                             nIndexs[MAX_MOBILE_CHANNEL_NUM];           // channel aggregate
    char                            szNumber[DH_COMMON_STRING_16];             // effective when attach "CallNoAnswered " 
                                                                               // no responsion number (pushing notify to mobile when device call to these number)
    char                            szSound[DH_COMMON_STRING_64];              // played sound file when receiving event 
} NET_SUBSCRIBE_INFO;

// Push gateway type
typedef enum tagEM_PUSH_GATEWAY_TYPE
{
    EM_PUSH_GATEWAY_TYPE_UNKNOWN,              // Unknown
    EM_PUSH_GATEWAY_TYPE_APPLE_PUSH,           // ApplePush
    EM_PUSH_GATEWAY_TYPE_ANDROID_SDKPUSH,      // AndroidPrivatePush
    EM_PUSH_GATEWAY_TYPE_ANDROID_JPUSH,        // AndroidJPush
    EM_PUSH_GATEWAY_TYPE_ANDROID_GCM,          // AndroidGCM
} EM_PUSH_GATEWAY_TYPE;

// Mobile push notification, use RegisterID to subscribe
typedef struct tagNET_MOBILE_PUSH_NOTIFY
{
    DWORD                           dwSize;
    char                            szRegisterID[MAX_REGISTER_ID_LEN];         // register ID, 
    EM_MOBILE_SERVER_TYPE           emServerType;                              // server type 
    int                             nPeriodOfValidity;                         // Period Of Validity
    char                            szAuthServerAddr[MAX_URL_LEN];             // Android server address
    int                             nAuthServerPort;                           // Android server port 
    char                            szPushServerAddr[MAX_URL_LEN];             // push server address 
    int                             nPushServerPort;                           // push server port
    NET_PUSH_SERVER_INFO            stuPushServerMain;                         // push server config 
    NET_PUSH_SERVER_INFO            stuPushRedirectServer;                     // Push Redirect Server config
    char                            szDevName[DH_COMMON_STRING_64];            // Device name
    char                            szDevID[DH_COMMON_STRING_64];              // Device ID
    char                            szUser[DH_COMMON_STRING_64];               // User name
    char                            szPassword[DH_COMMON_STRING_64];           // Password
    char                            szCertificate[MAX_IOS_CERTIFICATE_LEN];    // Certificate
    char                            szSecretKey[MAX_IOS_SECRET_KEY_LEN];       // Secret Key
    int                             nSubScribeMax;                             // max number of subscribe 
    int                             nSubScribeNum;                             // number of subscribe
    NET_SUBSCRIBE_INFO             *pstuSubscribes;                            // subscribe info, user malloc memory,apply to sizeof(NET_SUBSCRIBE_INFO)*nSubScribeMax
} NET_MOBILE_PUSH_NOTIFY;

// Delete mobile push notification, CLIENT_DelMobilePushNotify's input param
typedef struct tagNET_MOBILE_PUSH_NOTIFY_DEL
{
    DWORD                           dwSize;
    char                            szRegisterID[MAX_REGISTER_ID_LEN];         // Register ID
} NET_MOBILE_PUSH_NOTIFY_DEL;

// Mobile push notification config,  use RegisterID + AppID to subscribe
typedef struct tagNET_MOBILE_PUSH_NOTIFY_CFG
{
    DWORD                           dwSize;
    char                            szRegisterID[MAX_REGISTER_ID_LEN];         // Register ID
    char                            szAppID[MAX_APP_ID_LEN];                   // AppID
    EM_MOBILE_SERVER_TYPE           emServerType;                              // Server Type
    EM_PUSH_GATEWAY_TYPE            emPushGatewayType;                         // Push Gateway Type
    int                             nPeriodOfValidity;                         // Period Of Validity
    char                            szAuthServerAddr[MAX_URL_LEN];             // Android server address
    int                             nAuthServerPort;                           // Android server port 
    char                            szPushServerAddr[MAX_URL_LEN];             // push server address
    int                             nPushServerPort;                           // push server port
    char                            szDevName[DH_COMMON_STRING_64];            // device name
    char                            szDevID[DH_COMMON_STRING_64];              // device ID
    char                            szUser[DH_COMMON_STRING_64];               // User
    char                            szPassword[DH_COMMON_STRING_64];           // Password
    char                            szCertificate[MAX_IOS_CERTIFICATE_LEN];    // Certificate
    char                            szSecretKey[MAX_IOS_SECRET_KEY_LEN];       // Secret Key
    int                             nSubScribeMax;                             // max number of subscribe 
    int                             nSubScribeNum;                             // number of subscribe 
    NET_SUBSCRIBE_INFO             *pstuSubscribes;                            // subscribe info, user malloc memory, apply to sizeof(NET_SUBSCRIBE_INFO)*nSubScribeMax
    NET_PUSH_SERVER_INFO            stuPushServerMain;                         // push server config info
    NET_PUSH_SERVER_INFO            stuPushRedirectServer;                     // push redirect server config info
} NET_MOBILE_PUSH_NOTIFY_CFG;

// Delete mobile push notification config, CLIENT_DelMobilePushNotify's input param
typedef struct tagNET_MOBILE_PUSH_NOTIFY_CFG_DEL
{
    DWORD                           dwSize;
    char                            szRegisterID[MAX_REGISTER_ID_LEN];         // Register ID
    char                            szAppID[MAX_APP_ID_LEN];                   // AppID
} NET_MOBILE_PUSH_NOTIFY_CFG_DEL;

// NetSDK optimize type
typedef enum tagEmOptimizeType
{
    EM_OPT_TYPE_DEFAULT     = 0,    // default to do nothing
    EM_OPT_TYPE_MOBILE_V1   = 1,    // optimized for mobile
    EM_OPT_TYPE_P2P_NETPARAM_V1 = 2,    // the set for P2P network param,  pParam's type is  NET_PARAM*
}EM_OPTIMIZE_TYPE;

typedef struct tagNET_IN_MOTIONMATCH_PARAM
{
    DWORD                 dwSize;                   // structure size, dwSize needs to make valuation as sizeof(NET_IN_MOTIONMATCH_PARAM) 
    BOOL                  bFileName;                // if filename can be used as valid query condition, if the filename is valid, then it doesn't need to fill in file info (stRecordInfo) 
    char                  szFileName[MAX_PATH];     // file name 
    NET_RECORDFILE_INFO   stuRecordInfo;            // file info 
    int                   nRegionNum;               // number of vertex in rule detection area
    DH_POINT              stuRegion[DH_MAX_DETECT_REGION_NUM];    // rule detection area, x-coordinate 0~21, y-coordinate 0~17
} NET_IN_MOTIONMATCH_PARAM;

typedef struct tagNET_TIME_BEGIN_END
{
    NET_TIME              stuBegin;                 // begin
    NET_TIME              stuEnd;                   // end
} NET_TIME_BEGIN_END;

#define NET_SMART_SEARTCH_TIME_SECTION_MAX (128)

typedef struct tagNET_OUT_MOTIONMATCH_PARAM
{
    DWORD                 dwSize;                   // structure size, dwSize needs to make valuation as sizeof(NET_OUT_MOTIONMATCH_PARAM) 
    int                   nTimeNum;                 // queried time quantum number
    NET_TIME_BEGIN_END    stuTime[NET_SMART_SEARTCH_TIME_SECTION_MAX];   // time quantum
} NET_OUT_MOTIONMATCH_PARAM;

// CLIENT_SnapPictureToFile port input parameter
typedef struct tagNET_IN_SNAP_PIC_TO_FILE_PARAM 
{
    DWORD       dwSize;
    SNAP_PARAMS stuParam;               // snapshot parameter, mode field is only snapshot for once, fail to support timed or continuous snapshot; except mobile DVR, other devices only support snapshot frequency of one snapshot per second
    char        szFilePath[MAX_PATH];   // write in file address
}NET_IN_SNAP_PIC_TO_FILE_PARAM;

// CLIENT_SnapPictureToFile port output parameter
typedef struct tagNET_OUT_SNAP_PIC_TO_FILE_PARAM 
{
    DWORD       dwSize;
    char*       szPicBuf;               // picture content, user memory allocation, memory size is dwPicBufLen
    DWORD       dwPicBufLen;            // picture content memory size, unit: byte
    DWORD       dwPicBufRetLen;         // returned picture size, unit:byte
}NET_OUT_SNAP_PIC_TO_FILE_PARAM;

//alarm event type DH_ALARM_BUS_SHARP_TURN (vehicle sharp turn event) corresponding data description info
typedef struct tagALARM_BUS_SHARP_TURN_INFO
{
    DWORD               dwSize;
    NET_GPS_STATUS_INFO stuGPSStatusInfo;       // GPS info 
} ALARM_BUS_SHARP_TURN_INFO;

//alarm event type DH_ALARM_BUS_SCRAM (vehicle scram event) corresponding data description info 
typedef struct tagALARM_BUS_SCRAM_INFO
{
    DWORD               dwSize;
    NET_GPS_STATUS_INFO stuGPSStatusInfo;       // GPS info 
} ALARM_BUS_SCRAM_INFO;

//alarm event type DH_ALARM_BUS_SHARP_ACCELERATE (vehicle sharp accelerate event) corresponding data description info
typedef struct tagALARM_BUS_SHARP_ACCELERATE_INFO
{
    DWORD               dwSize;
    NET_GPS_STATUS_INFO stuGPSStatusInfo;       // GPS info 
    NET_TIME_EX			stuTime;				// Event occurrence time
} ALARM_BUS_SHARP_ACCELERATE_INFO;

//alarm event type DH_ALARM_BUS_SHARP_DECELERATE(vehicle sharp decelerate event) corresponding data description info 
typedef struct tagALARM_BUS_SHARP_DECELERATE_INFO
{
    DWORD               dwSize;
    NET_GPS_STATUS_INFO stuGPSStatusInfo;       // GPS info 
    NET_TIME_EX			stuTime;				// Event occurrence time
} ALARM_BUS_SHARP_DECELERATE_INFO;


//access control card data operation type               
typedef enum tagNET_ACCESS_ACTION_TYPE        
{       
    NET_ACCESS_ACTION_TYPE_UNKNOWN  = 0,    // unknown               
    NET_ACCESS_ACTION_TYPE_INSERT   = 1,    // insert              
    NET_ACCESS_ACTION_TYPE_UPDATE   = 2,    // update              
    NET_ACCESS_ACTION_TYPE_REMOVE   = 3,    // remove               
}NET_ACCESS_ACTION_TYPE;   
        
//access control card data operation event        
typedef struct tagALARM_ACCESS_CARD_OPERATE_INFO       
{       
    DWORD                   dwSize;       
    NET_ACCESS_ACTION_TYPE  emActionType;                    // access control card data operation type               
    char                    szCardNo[DH_MAX_CARDINFO_LEN];   // access control card number               
    NET_THREE_STATUS_BOOL   emResult;                        // operation result, -1 is unknown, 0 is failure, 1 is success       
    int                     nChannelID;                      // channel ID, start from 0
}ALARM_ACCESS_CARD_OPERATE_INFO;

// alarm event typeDH_EVENT_TAKENAWAYDETECTION (object move event) corresponding data description info  
typedef struct tagALARM_TAKENAWAY_DETECTION_INFO
{
    DWORD               dwSize;    
    int                 nChannelID;                     // channel number
    double              PTS;                            // timestamp (unit is millisecond)
    NET_TIME_EX         UTC;                            // event occurrence time
    int                 nEventID;                       // event ID 
    int                 nEventAction;                   // event action, 0 means pulse event, 1 means continuity event begins, 2 means continuity event ends

    int                 nOccurrenceCount;               // rule triggered occurrence times
    int                 nLevel;                         // event level, GB30147 requirement item, o hint, 1 general, 2 alarm
	short				nPreserID;						// Event triggered preset period, starting from 1 (no unknown)
	char				szPresetName[64];				// Preset name for event triggered
} ALARM_TAKENAWAY_DETECTION_INFO;

// alarm event type DH_EVENT_VIDEOABNORMALDETECTION (video abnormity event) corresponding data description info
typedef struct tagALARM_VIDEOABNORMAL_DETECTION_INFO
{
    DWORD               dwSize;    
    int                 nChannelID;                     // channel number
    double              PTS;                            // timestamp (unit is millisecond)
    NET_TIME_EX         UTC;                            // event occurrence time 
    int                 nEventID;                       // event ID
    int                 nEventAction;                   // event type, 0 means pulse event, 1 means continuity event begins, 2 means continuity event ends

    int                 nType;                          // detection type, 0-video loss, 1-video tampering, 2-image freeze, 3-over brightness, 4-over darkness, 5-scene change
                                                        // 6-stripe detection, 7-noise detection, 8- color cast detection, 9-video fuzziness detection, 10-contrast abnormity detection 
                                                        // 11-Video motion, 12-video flicker, 13-video color, 14- defocus detection, 15-overexposure detection
    int                 nValue;                         // detection value, the higher the value, the poorer the video quality
    int                 nOccurrenceCount;               // occurrence times of rule trigger
} ALARM_VIDEOABNORMAL_DETECTION_INFO;

// Alarm event type DH_EVENT_LOCALALARM (external alarm event) corresponding data description info
typedef struct tagALARM_LOCAL_ALARM_INFO
{
    DWORD               dwSize;    
    int                 nChannelID;                     // channel number
    double              PTS;                            // timestamp (unit is millisecond)
    NET_TIME_EX         UTC;                            // Event occurrence time 
    int                 nEventID;                       // Event ID
    int                 nEventAction;                   // Event action, 0 means pulse event, 1 means continuous event begin, 2 means continuous event end;

    int                 nSenseType;                     // sensor type
    int                 nDefenceAreaType;               // protection area type, refer to EM_NET_DEFENCE_AREA_TYPE 
} ALARM_LOCAL_ALARM_INFO;

// alarm event type DH_EVENT_MOTIONDETECT (video motion detection event) corresponding data description info
typedef struct tagALARM_MOTIONDETECT_INFO
{
    DWORD               dwSize;    
    int                 nChannelID;                     // channel number
    double              PTS;                            // timestamp (unit is millisecond)
    NET_TIME_EX         UTC;                            // event occurrence time
    int                 nEventID;                       // event ID
    int                 nEventAction;                   // event action, 0 means pulse event, 1 means continuous event begin, 2 means continuous event end;
} ALARM_MOTIONDETECT_INFO;

// alarm event type DH_ALARM_POLICE_CHECK (police sign in/out event) corresponding data description info
typedef struct tagALARM_POLICE_CHECK_INFO
{
    DWORD               dwSize;
    int                 nChannelID;                     // channel number
    double              PTS;                            // timestamp (unit is millisecond)
    NET_TIME_EX         UTC;                            // event occurrence time
    int                 nEventID;                       // Event ID
    int                 nEventAction;                   // Event action 

    char                szId[32];                       // sign in person ID number
    char                szName[32];                     // sign in person name, UTF8
    int                 nSignType;                      // type, refer to NET_DRIVER_CHECK_TYPE 
    NET_GPS_STATUS_INFO stuGPSStatusInfo;               // GPS info 
    char                szComment[1024];                // comment info 
} ALARM_POLICE_CHECK_INFO;

// alarm event type DH_ALARM_NEW_FILE (new file event) corresponding data description info
typedef struct tagALARM_NEW_FILE_INFO
{
    DWORD           dwSize;
    int             nChannel;                           // snapshot channel number
    int             nEventID;                           // event ID
    DWORD           dwEvent;                            // event type
    int             FileSize;                           // file size, unit is byte
    int             nIndex;                             // event source channel 
    DWORD           dwStorPoint;                        // storage point
    char            szFileName[DH_COMMON_STRING_128];   // file name
}ALARM_NEW_FILE_INFO;


// the status type of PTZ
typedef enum tagEM_PTZ_STATUS_TYPE
{
    EM_PTZ_STATUS_UNKNOW,                    // unknown
    EM_PTZ_STATUS_NORMAL,                    // nomal
    EM_PTZ_STATUS_PANOC_FAILED,              // PanOCFailed
    EM_PTZ_STATUS_TILEOC_FAILED,             // TileOCFailed
    EM_PTZ_STATUS_OC_FAILED                  // OCFailed
} EM_PTZ_STATUS_TYPE;


// the info of Ptz Diagnoses(DH_ALARM_PTZ_DIAGNOSES)
typedef struct tagALARM_PTZ_DIAGNOSES_INFO
{
	int 				nChannel;			// video channel
	NET_TIME_EX			UTC;				// the time when event happen
	EM_PTZ_STATUS_TYPE	emPtzStat;			// the status of PTZ
	BYTE                Reserved[512];
} ALARM_PTZ_DIAGNOSES_INFO;

// event type DH_ALARM_FLASH_LIGHT_FAULT (FlashLightFault event) corresponding data description info
typedef struct tagALARM_FLASH_LIGHT_FAULT_INFO
{
    double              PTS;                            // PTS(ms)
    NET_TIME_EX         UTC;                            // the event happen time
    int                 nEventAction;                   // Envet Action, 1-start,2-stop
    int                 nLightNum;                      // flash light number
    int                 nDevID[MAX_FLASH_LIGHT_NUM];    // the fault flash light ID
    int                 nLane;                          // lane number
    BYTE			    bReserved[512];				    // reserved
}ALARM_FLASH_LIGHT_FAULT_INFO;

// event DH_ALARM_STROBOSCOPIC_LIGTHT_FAULT (StroboscopicLightFault event) corresponding data description info
typedef struct tagALARM_STROBOSCOPIC_LIGTHT_FAULT_INFO
{
    double              PTS;                                // PTS(ms)
    NET_TIME_EX         UTC;                                // the event happen time
    int                 nEventAction;                       // Envet Action, 1-start,2-stop
    int                 nLightNum;                          // stroboscopic light num 
    int                 nDevID[MAX_STROBOSCOPIC_LIGHT_NUM]; // the fault stroboscopic light ID
    int                 nLane;                              // lane number
    BYTE			    bReserved[512];				        // reserved
}ALARM_STROBOSCOPIC_LIGTHT_FAULT_INFO;

// upper limit type
typedef enum tagEM_UPPER_LIMIT_TYPE
{
    EM_UPPER_LIMIT_TYPE_UNKNOWN,                             // unknow
    EM_UPPER_LIMIT_TYPE_ENTER_OVER,                          // "EnterOver"
    EM_UPPER_LIMIT_TYPE_EXIT_OVER,                           // "ExitOver"
    EM_UPPER_LIMIT_TYPE_INSIDE_OVER,                         // "InsideOver"
} EM_UPPER_LIMIT_TYPE;

// event type DH_ALARM_HUMAM_NUMBER_STATISTIC 
typedef struct tagALARM_NUMBER_STATISTIC_INFO 
{
    double              PTS;                            // PTC(ms)
    NET_TIME_EX         UTC;                            // event time
    int                 nEventAction;                   // Envet Action, 1-start,2-stop
    int                 nNumber;                        // the number of object which is in the area
    int                 nEnteredNumber;                 // entered object number
    int                 nExitedNumber;                  // exited object number
    EM_UPPER_LIMIT_TYPE emUpperLimitType;               // upper limit type(EnterOver, ExitOver, InsideOver)
    BYTE                reserved[512];                  // reserved
}ALARM_HUMAN_NUMBER_STATISTIC_INFO;

typedef struct tagALARM_VIDEOUNFOCUS_INFO
{
    int                 nEventAction;                   // event action, 0 means pulse event, 1 means continuous event begin, 2 means continuous event end
    BYTE                reserved[64];                   // reserved
}ALARM_VIDEOUNFOCUS_INFO;
// status
typedef enum tagEM_DEFENCEMODE
{
    EM_DEFENCEMODE_UNKNOWN,             // "unknown"   
    EM_DEFENCEMODE_ARMING,              // "Arming"    
    EM_DEFENCEMODE_DISARMING,           // "Disarming" 
}EM_DEFENCEMODE;

// event type DH_ALARM_DEFENCE_ARMMODECHANGE (Defence arming status change event)
typedef struct tagALARM_DEFENCE_ARMMODECHANGE_INFO 
{
    EM_DEFENCEMODE       emDefenceStatus;              //status
    int                  nDefenceID;                   //DefenceID
    NET_TIME_EX          stuTime;                      //the event happen time
    BYTE                 reserved[512];                //reserverd
}ALARM_DEFENCE_ARMMODECHANGE_INFO;

// status
typedef enum tagEM_SUBSYSTEMMODE
{
    EM_SUBSYSTEMMODE_UNKNOWN,             // "unknown"   
    EM_SUBSYSTEMMODE_ACTIVE,              // "active"    
    EM_SUBSYSTEMMODE_INACTIVE,            // "inactive"  
    EM_SUBSYSTEMMODE_UNDISTRIBUTED,       // "undistributed" 
    EM_SUBSYSTEMMODE_ALLARMING,           // "AllArming" 
    EM_SUBSYSTEMMODE_ALLDISARMING,        // "AllDisarming" 
    EM_SUBSYSTEMMODE_PARTARMING,          // "PartArming" 
}EM_SUBSYSTEMMODE;

// event type DH_ALARM_SUBSYSTEM_ARMMODECHANGE (Subsystem arming status change event)
typedef struct tagALARM_SUBSYSTEM_ARMMODECHANGE_INFO 
{
    EM_SUBSYSTEMMODE    emSubsystemMode;            //status (support report AllArming, AllDisarming, PartArming)
    NET_TIME_EX         UTC;                        //the event happen time
    char                szSubSystemname[64];        //subsystem name
    int                 nSubSystemID;               //SubSystemID
    BYTE                reserved[512];              //reserved
}ALARM_SUBSYSTEM_ARMMODECHANGE_INFO;

// infrared detection information event (Corresponding ALARM_RFID_INFO)
typedef struct tagALARM_RFID_INFO
{
    int                         nAction;                // 0:start 1:stop
    int                         nIndex;                 // channel ID
    NET_TIME_EX                 stuEventTime;           // event time
    int                         nValidCount;            // RFID  valid number of RFID 
    char                        szRfid[10][12];         // RFID array
}ALARM_RFID_INFO;

//event type DH_ALARM_SMOKE_DETECTION
typedef struct tagALARM_SMOKE_DETECTION_INFO
{
    int                 nAction;                        // 0:start 1:stop
    char                szName[128];                    // event name
    double              PTS;                            // PTC(ms)
    NET_TIME_EX         stuTime;                        // event time
	int					nChannel;						// channel ID
    BYTE                reserved[124];                  // reserved               
}ALARM_SMOKE_DETECTION_INFO;

// patient detection type
typedef enum TagEM_PATIENTDETECTION_TYPE
{
    EM_PATIENTDETECTION_TYPE_UNKNOWN,                   // unknown
    EM_PATIENTDETECTION_TYPE_CROSS_REGION,              // someone cross region 
    EM_PATIENTDETECTION_TYPE_LIGHT_OFF,                 // light off
    EM_PATIENTDETECTION_TYPE_STOP_DETECTION,            // stop detection    
    EM_PATIENTDETECTION_TYPE_START_DETECTION,           // start detection
	EM_PATIENTDETECTION_TYPE_ESCAPE,                    // patient escape
	EM_PATIENTDETECTION_TYPE_ESCAPE_RELEASE,            // Release the patient escape
}EM_PATIENTDETECTION_TYPE;

// event type DH_ALARM_PATIENTDETECTION
typedef struct tagALARM_PATIENTDETECTION_INFO
{
    BYTE                        bEventAction;           // event action,0 pulse,1 durable events begin, 2 durable events end
    NET_TIME                    stuTime;                // event time
    EM_PATIENTDETECTION_TYPE    emDetectionType;        // detection type
    BYTE                        byReserved[256];        // reserved 
}ALARM_PATIENTDETECTION_INFO;

// radar high speed detection (DH_ALARM_RADAR_HIGH_SPEED)
typedef struct tagALARM_RADAR_HIGH_SPEED_INFO
{
    NET_TIME_EX                 stuTime;                    // event time
    float                       fSpeed;                     // speed(unit:km/h)
	char						szPlateNumber[16];			// plate 
    BYTE                        byReserved[1008];           // reserved 
}ALARM_RADAR_HIGH_SPEED_INFO;

// Polling Alarm  (DH_ALARM_POLLING_ALARM)
typedef struct tagALARM_POLLING_ALARM_INFO
{
    NET_TIME_EX                 stuTime;                    // event time
    BYTE                        byReserved[1024];           // reserved 
}ALARM_POLLING_ALARM_INFO;

// Traffic Strobe State
typedef enum tagEM_TRAFFICSTROBE_STATUS
{
    NET_TRAFFICSTROBE_STATUS_UNKNOWN,
	NET_TRAFFICSTROBE_STATUS_CLOSE_COMPLETE,                               // Traffic Strobe closed
	NET_TRAFFICSTROBE_STATUS_OPEN_COMPLETE,                                // Traffic Strobe opened
	NET_TRAFFICSTROBE_STATUS_NOT_CLOSE_COMPLETE,                           // Traffic Strobe not complete closed
	NET_TRAFFICSTROBE_STATUS_NOT_OPEN_COMPLETE,							   // Traffic Strobe not complete opened
} EM_TRAFFICSTROBE_STATUS;

// Traffic Strobe State info ( DH_ALARM_TRAFFICSTROBESTATE )
typedef struct tagALARM_TRAFFICSTROBESTATE_INFO
{
	BYTE                        bEventAction;               // Event Action,0=Impluse Event,1=Continued Event Start,2=Continued Event End;
    NET_TIME                    stuTime;                    // Alarm event time
	int                         nChannelID;                 // Channel No.
    EM_TRAFFICSTROBE_STATUS		emStatus;                   // Traffic Strobe State
    BYTE                        byReserved[1024];           // reserved 
}ALARM_TRAFFICSTROBESTATE_INFO;

// Paste detection info (Corresponding to DH_ALARM_PASTE_DETECTION)
typedef struct tagALARM_PASTE_DETECTION_INFO
{
    int                     nChannelID;                                 // ChannelId
    char                    szName[128];                                // event name
    double                  PTS;                                        // PTS(ms)
    NET_TIME_EX             UTC;                                        // the event happen time
    DH_MSG_OBJECT           stuObject;                                  // have being detected object
    int                     nDetectRegionNum;                           // rule detect region's point number
    DH_POINT                stuDetectRegion[DH_MAX_DETECT_REGION_NUM];  // rule detect region
    DH_MSG_HANDLE_EX        stuEventHandler;                            // Handle method
    BYTE                    bEventAction;                               // Event Action,0=Impluse Event,1=Continued Event Start,2=Continued Event End;
    BYTE                    bReserved[1023];                            // reserved
}ALARM_PASTE_DETECTION_INFO;

// TemperatureDifference Between Rule (DH_ALARM_BETWEENRULE_TEMP_DIFF)
typedef struct tagALARM_BETWEENRULE_DIFFTEMPER_INFO
{
    int                 nAction;                        // 0:start 1:stop
    int                 nOsdId;                         // config of TemperatureDifference OSD(OSDId)
    int                 nChannelID;                     // channel id
    NET_RADIOMETRY_ALARMCONTION     emAlarmContion;     // alarm condition
    int                 nRuleId1;                       // ThermometryRule1,refer to CFG_RADIOMETRY_RULE_INFO
    int                 nRuleId2;                       // ThermometryRule2,refer to CFG_RADIOMETRY_RULE_INFO
    float               fDiffValue;                     // Difference value between rules
    int                 nTemperatureUnit;               // TemperatureUnit,refer to NET_TEMPERATURE_UNIT
    UINT                nPresetID;                      // Preset id
    BYTE                reserved[128];                  // reserved  
}ALARM_BETWEENRULE_DIFFTEMPER_INFO;

// hotspot warning(DH_ALARM_HOTSPOT_WARNING)
typedef struct tagALARM_HOTSPOT_WARNING_INFO
{
    int                 nAction;                        // 0:start 1:stop
    int                 nChannelID;                     // channel id
    DH_POINT            stuCoordinate;                  // hotspot point,  0~8192
    float               fHotSpotValue;                  // hotspot value
    int                 nTemperatureUnit;               // temperature unit,refer to NET_TEMPERATURE_UNIT
}ALARM_HOTSPOT_WARNING_INFO;

// coldspot warning(DH_ALARM_COLDSPOT_WARNING)
typedef struct tagALARM_COLDSPOT_WARNING_INFO
{
    int                 nAction;                        // 0:start 1:stop
    int                 nChannelID;                     // channel id
    DH_POINT            stuCoordinate;                  // coldspot point,  0~8192
    float               fColdSpotValue;                 // coldspot value
    int                 nTemperatureUnit;               // temperature unit,refer to NET_TEMPERATURE_UNIT
}ALARM_COLDSPOT_WARNING_INFO;

// flow meter info (DH_ALARM_FLOW_METER)
typedef struct tagALARM_FLOW_METER_INFO 
{
    DWORD                   dwSize;
    float                   fInstantRate;                       // Instant Rate m/s
    float                   fInstantFlow;                       // Instant Flow m^3/h
    float                   fTotalFlow;                         // Total Flow m^3
    float                   fCurDayFlow;                        // Current day Flow m^3
    UINT                    unCurDayWorkingTime;                // Working time of current day ,unit: minute
    UINT                    unTotalWorktingTime;                // Working time in total, unit: minute
    NET_GPS_STATUS_INFO     stuGPS;                             // GPS information
    float                   fCurTemp;                           // the current temperature of the device, unit: Celsius degree
} ALARM_FLOW_METER_INFO;

// type of drop frame
typedef enum tagEM_BUF_DROP_FRAME_TYPE
{
    EM_BUF_DROP_FRAME_UNKOWN = 0,                       // frame type:unkonw
    EM_BUF_DROP_FRAME_DEVBUF,                           // frame type:main stream
    EM_BUF_DROP_FRAME_EXTBUF,                           // frame type:extra stream
    EM_BUF_DROP_FRAME_NETBUF,                           // frame type:net main stream 
    EM_BUF_DROP_FRAME_EXTRA1NETBUF                      // frame type:net extra1 stream
}EM_BUF_DROP_FRAME_TYPE;

// Video recond buffer drop frame event(DH_ALARM_BUF_DROP_FRAME)
typedef struct tagALARM_BUF_DROP_FRAME_INFO
{
    int                         nAction;                // 0:start 1:stop
    EM_BUF_DROP_FRAME_TYPE      emDropFrameType;        // type of drop frame
    BYTE                        reserved[1024];         // reserved
}ALARM_BUF_DROP_FRAME_INFO;


// Switch with master and slave (DH_ALARM_DCSSWITCH)
typedef struct tagALARM_DCSSWITCH_INFO
{
    int                         nAction;                // 0:start 1:stop
    char                        strMasterIP[32];        // Master device IP
    char                        strSlaveIP[32];         // Slave device IP
    NET_TIME_EX                 stuStartTime;           // the time when start switch
    BYTE                        reserved[1024];         // reserved
}ALARM_DCSSWITCH_INFO;

// Abnormal event when master broad'version and slave broad'version different (DH_ALARM_DOUBLE_DEV_VERSION_ABNORMAL)
typedef struct tagALARM_DOUBLE_DEV_VERSION_ABNORMAL_INFO
{
    int                         nAction;                // 0:start 1:stop
    char                        strMasterVer[64];       // master broad's version(when nAction is 0, this data is efficient)
    char                        strSlaveVer[64];        // slave broad's version(when nAction is 0, this data is efficient)
    BYTE                        reserved[1024];         // reserved
}ALARM_DOUBLE_DEV_VERSION_ABNORMAL_INFO;

// Radar connect state
typedef enum tagEM_RADAR_CONNECT_STATE
{
    EM_RADAR_CONNECT_STATE_UNKNOWN,     // unknown
    EM_RADAR_CONNECT_STATE_NORMAL,      // normal
    EM_RADAR_CONNECT_STATE_EXCEPTION,   // exception
}EM_RADAR_CONNECT_STATE;

// Radar connect state information event (DH_ALARM_RADAR_CONNECT_STATE)
typedef struct tagALARM_RADAR_CONNECT_STATE_INFO
{
    EM_RADAR_CONNECT_STATE      emRadarConnectState;    // Radar connect state
    BYTE                        reserved[1024];         // reserved
}ALARM_RADAR_CONNECT_STATE_INFO;

//CLIENT_SetMarkFileByTime input parameter
typedef enum tagEM_MARKFILE_MODE
{
	EM_MARK_FILE_BY_TIME_MODE,                              // add lock for record by time mode 
	EM_MARK_FILE_BY_NAME_MODE,                              // add lock for record by file mode 
}EM_MARKFILE_MODE;
typedef enum tagEM_MARKFILE_NAMEMADE_TYPE
{
	EM_MARKFILE_NAMEMADE_DEFAULT,							// default:need user pass record file name parameter szFilename
	EM_MARKFILE_NAMEMADE_JOINT,								// way of  file name split joint:pass the disk number ,pass the start cluster number,not need pass record file name
}EM_MARKFILE_NAMEMADE_TYPE;
//CLIENT_SetMarkFileByTimeEx input parameter
// Record file information
typedef struct tagNET_IN_SET_MARK_FILE
{
	DWORD               dwSize; 
	EM_MARKFILE_MODE	                    emLockMode;								// mode of lock for record,
	EM_MARKFILE_NAMEMADE_TYPE				emFileNameMadeType;						// way of create file name 
    int					nChannelID;								// channel ID
    char									szFilename[MAX_PATH];	                // file name
    unsigned int        nFramenum;								// the total number of file frames
    unsigned int        nSize;									// File length 
    NET_TIME            stuStartTime;							// Start time 
    NET_TIME            stuEndTime;								// End time 
    unsigned int        nDriveNo;								// HDD number
    unsigned int        nStartCluster;							// Initial cluster number
    BYTE                byRecordFileType;						// Recorded file type  0:general record;1:alarm record ;2:motion detection;3:card number record ;4:image 
    BYTE                byImportantRecID;						// 0:general record 1:Important record
    BYTE                byHint;									// Document Indexing
    BYTE                byRecType;								// 0-main stream record 1-sub1 stream record 2-sub2 stream record 3-sub3 stream record
} NET_IN_SET_MARK_FILE;
typedef struct tagNET_OUT_SET_MARK_FILE
{
    DWORD               dwSize; 
} NET_OUT_SET_MARK_FILE;
typedef struct tagNET_IN_SET_MARK_FILE_BY_TIME
{
    DWORD               dwSize;
    int                 nChannel;                             //the channel ID you want to lock, begin at 0, when the filed is -1,  it means all channels
    NET_TIME_EX         stuStartTime;                         //start time
    NET_TIME_EX         stuEndTime;                           //end time
    BOOL                bFlag;                                //action tag 	true : tag, false : clear
} NET_IN_SET_MARK_FILE_BY_TIME;

//CLIENT_SetMarkFileByTimeEx output parameter
typedef struct tagNET_OUT_SET_MARK_FILE_BY_TIME
{
    DWORD               dwSize; 
} NET_OUT_SET_MARK_FILE_BY_TIME;

//CLIENT_GetMarkInfo input parameter
typedef struct tagNET_IN_GET_MARK_INFO
{
    DWORD               dwSize;
} NET_IN_GET_MARK_INFO;

//CLIENT_GetMarkInfo output parameter 
typedef struct tagNET_OUT_GET_MARK_INFO
{
    DWORD               dwSize;
    int                 nTotalSpace;  //total capacity (unit M)
    int                 nMarkSpace;   //locked capacity of the video unit M 
} NET_OUT_GET_MARK_INFO;

//remove anti-submarine alarm input parameter 
typedef struct tagNET_IN_CLEAR_REPEAT_ENTER
{
    char         szCardNO[DH_MAX_CARDINFO_LEN];         // user card number
    BYTE         bReserved[1024];
}NET_IN_CLEAR_REPEAT_ENTER;

//remove anti-submarine alarm output parameter 
typedef struct tagNET_OUT_CLEAR_REPEAT_ENTER
{
    BYTE         bReserved[1024];
}NET_OUT_CLEAR_REPEAT_ENTER;

// alarm event type DH_ALARM_RECORD_LOSS (video loss event) corresponding data description info 
typedef struct tagALARM_RECORD_LOSS_INFO
{
    int                 nChannelID;                     // channel number
    int                 nEventAction;                   // event action, 0:pulse, 1:start
    char                szIP[DH_MAX_IPADDR_LEN_EX];     // video loss device IP
    int                 nPort;                          // video loss device port 
    BYTE                byReserved[1024];               // reserve
} ALARM_RECORD_LOSS_INFO;

// alarm event type DH_ALARM_VIDEO_FRAME_LOSS (video frame loss event) corresponding data description info
typedef struct tagALARM_VIDEO_FRAME_LOSS_INFO
{
    int                 nChannelID;                     // channel number
    int                 nEventAction;                   // event action, 0:Pulse,1:Start, 2: Stop 
    char                szIP[DH_MAX_IPADDR_LEN_EX];     // frame loss device IP 
    int                 nPort;                          // video loss device port 
    BYTE                byReserved[1024];               // reserve
} ALARM_VIDEO_FRAME_LOSS_INFO;

// alarm event type DH_ALARM_RECORD_VOLUME_FAILURE (disk volume abnormal) corresponding data description info 
typedef struct tagALARM_RECORD_VOLUME_FAILURE_INFO
{
    int                 nChannelID;                     // channel number
    int                 nEventAction;                   // event action, 0:Pulse, 1: Start, 2: Stop
    BYTE                byReserved[1024];               // reserve
} ALARM_RECORD_VOLUME_FAILURE_INFO;

// event type DH_EVENT_SNAP_UPLOAD (picture uploading finish event) corresponding data description info 
typedef struct tagEVENT_SNAP_UPLOAD_INFO
{
    int                 nChannelID;                     // channel number
    int                 nEventAction;                   // event action, 0: Pulse, 1: Start, 2: Stop
    BOOL                bSnapResult;                    // if the picture is uploaded successfully
    char                szFilePath[MAX_PATH];           // uploaded local picture name 
    BYTE                byReserved[1024];               // reserve 
}EVENT_SNAP_UPLOAD_INFO;

// alarm event type DH_ALARM_UPLOADPIC_FAILCOUNT (failed data number during uploading) corresponding data description info 
typedef struct tagALARM_UPLOADPIC_FAILCOUNT_INFO
{
    int                 nFailCount;                     // upload failure number
    BYTE                byReserved[1024];               // reserve
} ALARM_UPLOADPIC_FAILCOUNT_INFO;

// alarm event type DH_ALARM_HUMAN_INSIDE(Human inside) corresponding data description info
typedef struct tagALARM_HUMAN_INSIDE_INFO
{
    int                 nChannelID;                     // channel number
    int                 nEventAction;                   // event action, -1: unknown, 0: start, 1: stop
    int                 nHumanNumber;                   // Human number inside
    NET_TIME            stuUtcTime;                     // event time
    BYTE                byReserved[1000];               // reserve  
} ALARM_HUMAN_INSIDE_INFO;

// alarm event type DH_ALARM_HUMAN_TUMBLE_INSIDE(someone tumbles inside) corresponding data description info
typedef struct tagALARM_HUMAN_TUMBLE_INSIDE_INFO
{
    int                 nChannelID;                     // channel number
    int                 nEventAction;                   // event action, -1: unknown, 0: start, 1: stop
    NET_TIME            stuUtcTime;                     // event time
    BYTE                byReserved[1000];               // reserve
} ALARM_HUMAN_TUMBLE_INSIDE_INFO;

// alarm event type DH_ALARM_DISABLE_LOCKIN(Lock entry trigger event) corresponding data description info
typedef struct tagALARM_DISABLE_LOCKIN_INFO
{
    int                 nChannelID;                     // channel number
    NET_TIME            stuUtcTime;                     // event time
    BYTE                byReserved[1000];               // reserve
} ALARM_DISABLE_LOCKIN_INFO;


// alarm event type DH_ALARM_DISABLE_LOCKOUT(Lock go out trigger)  corresponding data description info
typedef struct tagALARM_DISABLE_LOCKOUT_INFO
{
    int                 nChannelID;                     // channel number
    NET_TIME            stuUtcTime;                     // event time
    BYTE                byReserved[1000];               // reserve
} ALARM_DISABLE_LOCKOUT_INFO;

//CLIENT_GetOperatorName input parameter 
typedef struct tagNET_IN_GET_OPERATOR_NAME
{
    BYTE                byReserved[1024];               // reserve
} NET_IN_GET_OPERATOR_NAME;

//CLIENT_GetOperatorName output parameter 
typedef struct tagNET_OUT_GET_OPERATOR_NAME
{
    char                szOpearatorName[DH_COMMON_STRING_64]; // operator username 
    BYTE                byReserved[1024];                     // reserve 
} NET_OUT_GET_OPERATOR_NAME;

// CLIENT_GetSelfCheckInfo input param
typedef struct tagNET_IN_GET_SELTCHECK_INFO
{
    DWORD               dwSize;                  // caller must initializes it before using, should be sizeof (NET_IN_GET_SELTCHECK_INFO)
} NET_IN_GET_SELTCHECK_INFO;

//IVS event type EVENT_IVS_PTZ_PRESET(PTZ turn to preset enet)data description
typedef struct tagDEV_EVENT_ALARM_PTZ_PRESET_INFO
{
    int                 nChannelID;                                 // channel id
    char                szName[DH_EVENT_NAME_LEN];                  // event name
    char                bReserved1[4];                              // keep align
    double              PTS;                                        // PTS
    NET_TIME_EX         UTC;                                        // event time
    int                 nEventID;                                   // event ID
    ///////////////////////////////up is common//////////////////////////////
    char                szPresetName[PTZ_PRESET_NAME_LEN];          // preset name
    PTZ_SPEED_UNIT      stuPos;                                     // preset ptz
    int                 nPresetID;                                  // preset id
    BYTE                bReserved[1024];                            // reserved
} DEV_EVENT_ALARM_PTZ_PRESET_INFO;

//IVS event type EVENT_IVS_RFID_INFO(event of infrared detect info)data description
typedef struct tagDEV_EVENT_ALARM_RFID_INFO
{
    int                 nChannelID;                                 // channel id
    char                szName[DH_EVENT_NAME_LEN];                  // event name
    char                bReserved1[4];                              // keep align
    double              PTS;                                        // PTS
    NET_TIME_EX         UTC;                                        // event time
    int                 nEventID;                                   // event ID
    ///////////////////////////////up is common//////////////////////////////
    int                 nAction;                                    // 0:start 1:stop
    int                 nValidCount;                                // the valid number of RFID identifier's array
    char                szRfid[10][12];                             // the array of RFID identifier
    BYTE                bReserved[1024];                            // reserved
} DEV_EVENT_ALARM_RFID_INFO;

// 
typedef struct tagNET_CAR_WEIGTH_INFO
{
	unsigned int		nAxleNum;									// Axle number
	unsigned int		nMaxAxleDistance;							// Max Distance of axle, unit(mm).
    unsigned int        nAxleWeightInfo[MAX_AXLE_NUM];              // Weight of each axle,unit(Kg)
    unsigned int        nAxleDistanceInfo[MAX_AXLE_NUM-1];          // The Distance between the alxe, unit(mm)
    unsigned int        nOverWeight;                                // Over weight info, unit(kg).
    BYTE                byReserved[512];                            // Reserved
}NET_CAR_WEIGHT_INFO;

// IVS evnent EVENT_IVS_QSYTRAFFICCARWEIGHT
typedef struct tagDEV_EVENT_QSYTRAFFICCARWEIGHT_INFO  
{
    NET_CAR_WEIGHT_INFO	                stCarWeightInfo;			// Car weight information
    DEV_EVENT_TRAFFICJUNCTION_INFO      stJunctionInfo;             // Traffic Junctioin information
}DEV_EVENT_QSYTRAFFICCARWEIGHT_INFO;

//the info of plate
typedef struct tagNET_PLATE_INFO
{
	char						szFrontPlateNumber[DH_MAX_PLATE_NUMBER_LEN];	// the number of front plate
	EM_NET_PLATE_COLOR_TYPE		emFrontPlateColor;								// the color of front plate
	char						szBackPlateNumber[DH_MAX_PLATE_NUMBER_LEN];		// the number of back plate
	EM_NET_PLATE_COLOR_TYPE		emBackPlateColor;								// the color of back plate
	BYTE						bReserved[1024];								// Reserved 
} NET_PLATE_INFO;

// the pic's snap time info from the front and back traffic gate
typedef struct tagNET_SNAP_TIME_INFO
{
    NET_TIME_EX stFrontGateSnapTime;    // the snapshot time info from the front traffic gate
    NET_TIME_EX stBackGateSnapTime;     // the snapshot time info from the back traffic gate
    BYTE  bReserved[128];       // Reserved bytes
}NET_SNAP_TIME_INFO;


// IVS event type EVENT_IVS_TRAFFIC_COMPAREPLATE(compare plate event)data description
typedef struct tagDEV_EVENT_TRAFFIC_COMPAREPLARE_INFO
{
	int					nChannelID;							// channel ID
	char				szName[DH_EVENT_NAME_LEN];			// event name
	int                 nTriggerType;                   	// Trigger Type, 0 vehicle inspection device, 1 radar, 2 video
	double				dbPTS;								// PTS(ms)
	NET_TIME_EX			stuUTC;								// the event happen time
	int					nEventID;							// event id

	DH_MSG_OBJECT		stuObject;							// have being detected object
	DH_MSG_OBJECT		stuVehicle;							// vehicle	info
	DH_EVENT_FILE_INFO  stuFileInfo;                        // event file info
	int					nMark;								// for marking the snap frame
	int					nSource;							// the source address of video analysis
	int					nFrameSequence;						// video frame number
	int					nSpeed;								// the speed of this car(unit:km/h)
	int					nLane;								// the corresponding lane nubmer
	int					nSequence;							// snap index,such as 3,2,1,1 means the last one,0 means there has some exception and snap stop
	DEV_EVENT_TRAFFIC_TRAFFICCAR_INFO stTrafficCar;         // Traffic vehicle info
	NET_PLATE_INFO		stuPlateInfo;						// the plate info
	EVENT_COMM_INFO     stCommInfo;                         // public info
	DWORD               dwSnapFlagMask;	                	// flag(by bit),see NET_RESERVED_COMMON
    NET_SNAP_TIME_INFO  stSnapTimeInfo;                     // the pic's snap time info from the front and back traffic gate
	BYTE				bReserved[820];					    // Reserved 
} DEV_EVENT_TRAFFIC_COMPAREPLATE_INFO;


typedef struct tagDEV_EVENT_SHOOTING_SCORE_RECOGNITION_INFO
{
    int							nChannelID;									// channel ID
    char						szName[DH_EVENT_NAME_LEN];					// event name
    double						dbPTS;										// PTS(ms)
    NET_TIME_EX					stuUTC;										// time of occurrence
    int							nEventID;									// event ID
    DH_EVENT_FILE_INFO			stuFileInfo;								// event file info
    BYTE						bEventAction;								// event action,0 pulse,1 durable events begin, 2 durable events end
	DH_POINT					stuDetectRegion;							// rule detect region
	int							nValidBulletHolesNum;						// bullet holes number
	NET_BULLET_HOLES_INFO       stuBulletHolesInfo[DH_MAX_BULLET_HOLES];	// detect bullet holes info 
    BYTE						byImageIndex;								// Serial number of the picture, in the same time (accurate to seconds) may have multiple images, starting from 0
    DWORD						dwSnapFlagMask;								// flag(by bit),see NET_RESERVED_COMMON    
    BYTE						bReserved[1024];							// Reserved
} DEV_EVENT_SHOOTING_SCORE_RECOGNITION_INFO;


// IVS event type EVENT_IVS_TRAFFIC_TRANSFINITE(traffic transfinite event)data description
typedef struct tagDEV_EVENT_TRAFFIC_TRANSFINITE_INFO
{
    int							nChannelID;									// channel ID
    char						szName[DH_EVENT_NAME_LEN];					// event name
    double						dbPTS;										// PTS(ms)
    NET_TIME_EX					stuUTC;										// time of occurrence
    int							nEventID;									// event ID
    DH_EVENT_FILE_INFO			stuFileInfo;								// event file info
    BYTE						bEventAction;								// event action,0 pulse,1 durable events begin, 2 durable events end
	
	char						szPlateNumber[16];							// plate number
    char                        szTime[20];									// transfinite time
	double						dbLong;										// length, unit:meter
    double						dbWidth;									// width, unit:meter
	double						dbHeight;									// Height, unit: meter
	char						szViolationCode[16];						// violate code
	char						szDescribe[132];						    // violate describe
	
    BYTE						byImageIndex;								// Serial number of the picture, in the same time (accurate to seconds) may have multiple images, starting from 0
    DWORD						dwSnapFlagMask;								// flag(by bit),see NET_RESERVED_COMMON    
    BYTE						bReserved[4096];							// Reserved
} DEV_EVENT_TRAFFIC_TRANSFINITE_INFO;

// information of the picture 
typedef struct tagDEV_EVENT_TRAFFIC_FCC_IMAGE 
{
	DWORD                dwOffSet;                      // current picture file's offset in the binary file, byte
	DWORD                dwLength;						// current picture file's size, byte
	WORD                 wWidth;                        // picture width, pixel
	WORD                 wHeight;                       // picture high, pixel
}DEV_EVENT_TRAFFIC_FCC_IMAGE;

//information of the object
typedef struct tagDEV_EVENT_TRAFFIC_FCC_OBJECT
{
	DEV_EVENT_TRAFFIC_FCC_IMAGE	stuImage;							// information of car picture
}DEV_EVENT_TRAFFIC_FCC_OBJECT;

//EVENT_IVS_TRAFFIC_FCC_INFO
typedef struct tagDEV_EVENT_TRAFFIC_FCC_INFO
{
	int                 nChannelID;                                 // channel ID
    char                szName[DH_EVENT_NAME_LEN];                  // event name
    DWORD				nTriggerID;									// Trigger ID
    double              PTS;                                        // PTS(ms)
    NET_TIME_EX         UTC;                                        // the event happen time
    int                 nEventID;                                   // event id
	///////////////////////////////up is common//////////////////////////////

	DWORD				dwNum;										// fuel truck nozzle number
	DWORD				dwLitre;									// refuel volume: 0.01 litre
	EM_REFUEL_TYPE		emType;										// refuel type {"98#","97#","95#","93#","90#","10#","5#","0#","-10#","-20#","-35#","-50#"}
	DWORD				dwMoney;									// refuel money spend: 0.01 RMB
	char				szPlateNumber[DH_COMMON_STRING_16];			// car number: "2016-05-23 10:31:17"
	char				szTime[DH_COMMON_STRING_32];				// Event occurrence time 
	DEV_EVENT_TRAFFIC_FCC_OBJECT	stuObject;						// information of car picture
	BYTE				bReserved[1024];							// Reserved
}DEV_EVENT_TRAFFIC_FCC_INFO;

typedef enum tagEM_PARKINGSPACE_STATUS
{
    EM_PARKINGSPACE_STATUS_UNKNOWN,
    EM_PARKINGSPACE_STATUS_BUSY,                            // busy
    EM_PARKINGSPACE_STATUS_FREE,                            // free
    EM_PARKINGSPACE_STATUS_OVERLINE,                        // overline
}EM_PARKINGSPACE_STATUS;

// Parking information matting
typedef struct tagNET_FEATUREPIC_AREA
{
	int					nRetCount;									// Return  coordinate count	
	DH_POINT			stuFeaturePicArea[10];						// Parking information coordinate  matting							
	BYTE				bReserved[256];					            // Reserved	
}NET_FEATUREPIC_AREA;

typedef struct tagDEV_EVENT_TRAFFIC_ANALYSE_PRESNAP_INFO
{
    int                 nChannelID;                                 // channel id
    char                szName[128];                                // envent name
    char                bReserved1[4];                              // keep align
    double              PTS;                                        // Time stamp (unit:ms)
    NET_TIME_EX         stuTime;                                    // Event occurrence time 
    int                 nEventID;                                   // Event ID
    int                 nGroupID;                                   // nGroupID event group ID. The nGroupID is the same for the snap process of the same object 
    int                 nCountInGroup;                              // nCountInGroup snap amount of one event group 
    int                 nIndexInGroup;                              // IndexInGroup snap SN of one event group
    DH_MSG_OBJECT       stuObject;                                  // plate info
    DH_MSG_OBJECT       stuVehicle;                                 // Vehicle info 
    int					nMark;								        // for marking the snap frame
    int					nSource;							        // the source address of video analysis
    int					nFrameSequence;						        // video frame number
    int					nLane;								        // the corresponding lane nubmer 
    int					nSequence;							        // snap index,such as 3,2,1,1 means the last one,0 means there has some exception and snap stop
    DEV_EVENT_TRAFFIC_TRAFFICCAR_INFO stTrafficCar;                 // Traffic vehicle info
    EM_PARKINGSPACE_STATUS   emParkingSpaceStatus;                  // all parking space status
    EVENT_COMM_INFO     stCommInfo;                                 // public info
	EM_PARKINGSPACE_STATUS   emCurParkingSpaceStatus;				// Parking spaces corresponding to the current parking spaces
	NET_FEATUREPIC_AREA		 stuParkingInfo;						// Parking information to support the existing matting coordinates, 4 vertices, the absolute coordinates, the range is determined by the resolution of the picture
    BYTE				bReserved[720];					        	// Reserved
}DEV_EVENT_TRAFFIC_ANALYSE_PRESNAP_INFO;

//IVS event type NEAR_DISTANCE_DETECTION(near distance detection event)data description
typedef struct tagDEV_EVENT_NEAR_DISTANCE_DETECTION_INFO
{
    int                 nChannelID;                                 // channel ID
    char                szName[DH_EVENT_NAME_LEN];                  // event name
    char                bReserved1[4];                              // keep align
    double              PTS;                                        // PTS(ms)
    NET_TIME_EX         UTC;                                        // time of occurrence
    int                 nEventID;                                   // event ID
    ///////////////////////////////up is common//////////////////////////////
    int                 nAction;                                    // event action, 0: Pulse, 1: Start, 2: Stop
	EVENT_INTELLI_COMM_INFO     stuIntelliCommInfo;                 // intelli comm info
    BYTE                bReserved[2048];                            // reserved
} DEV_EVENT_NEAR_DISTANCE_DETECTION_INFO;

// IVS event type EVENT_IVS_MAN_STAND_DETECTION(stereo standing event)data description
typedef struct tagDEV_EVENT_MANSTAND_DETECTION_INFO
{
	int                 		nChannelID;                 // channel ID
    char                		szName[DH_EVENT_NAME_LEN];  // event name
    char                		bReserved1[4];              // keep align, not reserved
    double              		PTS;                        // PTS(ms)
    NET_TIME_EX         		UTC;                        // time of occurrence
    int                 		nEventID;                   // event ID
    ///////////////////////////////up is common//////////////////////////////
    int                 		nAction;                    // event action, 0: Pulse, 1: Start, 2: Stop
	int							nManListCount;				// number of regional personnet list
	MAN_STAND_LIST_INFO			stuManList[MAX_MAN_LIST_COUNT];	// list of petsons within the region
	EVENT_INTELLI_COMM_INFO     stuIntelliCommInfo;         // intelli comm info
	char                		szReversed[2048];           // reserved
} DEV_EVENT_MANSTAND_DETECTION_INFO;

// IVS event type EVENT_IVS_MAN_NUM_DETECTION(the statistics of people in stereo vision event)data description
typedef struct tagDEV_EVENT_MANNUM_DETECTION_INFO
{
	int                 		nChannelID;                 // channel ID
    char                		szName[DH_EVENT_NAME_LEN];  // event name
    char                		bReserved1[4];              // keep align, not reserved
    double              		PTS;                        // PTS(ms)
    NET_TIME_EX         		UTC;                        // time of occurrence
    int                 		nEventID;                   // event ID
    ///////////////////////////////up is common//////////////////////////////
    int                 		nAction;                    // event action, 0: Pulse, 1: Start, 2: Stop
	int							nManListCount;				// number of regional personnet list
	MAN_NUM_LIST_INFO			stuManList[MAX_MAN_LIST_COUNT];	// list of petsons within the region
	EVENT_INTELLI_COMM_INFO     stuIntelliCommInfo;         // intelli comm info
	char                		szReversed[2048];           // reserved
} DEV_EVENT_MANNUM_DETECTION_INFO;


// Initialization parameter
typedef struct tagNETSDK_INIT_PARAM
{
    int                 nThreadNum;         // specify netsdk's normal network process thread number, zero means using default value
    BYTE                bReserved[1024];    // reserved
}NETSDK_INIT_PARAM, *LPNETSDK_INIT_PARAM;

// Overlay Type
typedef enum tagNET_EM_OSD_BLEND_TYPE
{
    NET_EM_OSD_BLEND_TYPE_UNKNOWN,                                  // unknow overlay type
    NET_EM_OSD_BLEND_TYPE_MAIN,                                     // Overlay to main stream 
    NET_EM_OSD_BLEND_TYPE_EXTRA1,                                   // Overlay to extra stream 1
    NET_EM_OSD_BLEND_TYPE_EXTRA2,                                   // Overlay to extra stream 2
    NET_EM_OSD_BLEND_TYPE_EXTRA3,                                   // Overlay to extra stream 3
    NET_EM_OSD_BLEND_TYPE_SNAPSHOT,                                 // Overlay to snap
    NET_EM_OSD_BLEND_TYPE_PREVIEW,                                  // Overlay to preview mode
}NET_EM_OSD_BLEND_TYPE;

// Encode widget-channel title 
typedef struct tagNET_OSD_CHANNEL_TITLE
{
	DWORD				    dwSize;
    NET_EM_OSD_BLEND_TYPE   emOsdBlendType;                 // Overlay Type, should set the value whether getting config  or setting config
    BOOL                    bEncodeBlend;                   // Overlay or not
	NET_COLOR_RGBA		    stuFrontColor;					// Foreground color
	NET_COLOR_RGBA		    stuBackColor;					// Background color 
	NET_RECT		    	stuRect;						// Zone. The coordinates value ranges from  0 to 8191. Only use left value and top value.The point (left,top) shall be the same as the point(right,bottom).
} NET_OSD_CHANNEL_TITLE;

// Encode widget-Time title
typedef struct tagNET_OSD_TIME_TITLE
{
	DWORD				    dwSize;
    NET_EM_OSD_BLEND_TYPE   emOsdBlendType;             // Overlay Type, should set the value whether getting config  or setting config
    BOOL                    bEncodeBlend;               // Overlay or not
	NET_COLOR_RGBA		    stuFrontColor;				// Foreground color
	NET_COLOR_RGBA		    stuBackColor;				// Background color
	NET_RECT 			    stuRect;					// Zone. The coordinates value ranges from  0 to 8191. Only use left value and top value.The point (left,top) shall be the same as the point(right,bottom).
	BOOL 				    bShowWeek;					// Display week or not 
} NET_OSD_TIME_TITLE;

// Encode widget-User-defined title information
typedef struct tagNET_CUSTOM_TITLE_INFO
{
    BOOL                    bEncodeBlend;                   // Overlay or not
	NET_COLOR_RGBA		    stuFrontColor;					// Foreground color
	NET_COLOR_RGBA		    stuBackColor;					// Background color
	NET_RECT			    stuRect;						// Zone. The coordinates value ranges from  0 to 8191. Only use left value and top value.The point (left,top) shall be the same as the point(right,bottom).
	char				    szText[CUSTOM_TITLE_LEN];		// Title contents
	BYTE                    byReserved[512];                // reserved 
} NET_CUSTOM_TITLE_INFO;

// Encode widget-User-defined title
typedef struct tagNET_OSD_CUSTOM_TITLE
{
	DWORD					dwSize;
	NET_EM_OSD_BLEND_TYPE   emOsdBlendType;                         // Overlay Type, should set the value whether getting config  or setting config
	int						nCustomTitleNum;						// User-defined title amount
	NET_CUSTOM_TITLE_INFO	stuCustomTitle[MAX_CUSTOM_TITLE_NUM];	// User-defined title
} NET_OSD_CUSTOM_TITLE;

// title text alignment type 
typedef enum tagEM_TITLE_TEXT_ALIGNTYPE
{
    EM_TEXT_ALIGNTYPE_INVALID,                              // Invalid alignment mathod
    EM_TEXT_ALIGNTYPE_LEFT,                                 // Left alignment 
    EM_TEXT_ALIGNTYPE_XCENTER,                              // X coordinate alignment 
    EM_TEXT_ALIGNTYPE_YCENTER,                              // Y coordinate alignment 
    EM_TEXT_ALIGNTYPE_CENTER,                               // Center
    EM_TEXT_ALIGNTYPE_RIGHT,                                // Right alignment 
    EM_TEXT_ALIGNTYPE_TOP,                                  // By top alignment 
    EM_TEXT_ALIGNTYPE_BOTTOM,                               // By bottom alignment 
    EM_TEXT_ALIGNTYPE_LEFTTOP,                              // By upper left alignment 
    EM_TEXT_ALIGNTYPE_CHANGELINE,                           // Next row alignment 
}EM_TITLE_TEXT_ALIGNTYPE;

// User-defined title text alignment 
typedef struct tagNET_OSD_CUSTOM_TITLE_TEXT_ALIGN
{
	DWORD					dwSize;
	int						nCustomTitleNum;						// User-defined title amount
    EM_TITLE_TEXT_ALIGNTYPE emTextAlign[MAX_CUSTOM_TITLE_NUM];      // User-defined title alignment info 
}NET_OSD_CUSTOM_TITLE_TEXT_ALIGN;

// Encode widget common info
typedef struct tagNET_OSD_COMM_INFO
{
	DWORD					dwSize;
	double					fFontSizeScale;			// overlay font size scale
													// if fFontSizeScale isn't 0, nFontSize is invalid 
													// otherwise nFontSize is valid
	int						nFontSize;				// global font size overlay to main stream, unit px, default is 24
	int						nFontSizeExtra1;		// global font size overlay to sub stream 1, unit px
	int						nFontSizeExtra2;		// global font size overlay to sub stream 2, unit px
	int						nFontSizeExtra3;		// global font size overlay to sub stream 3, unit px
	int						nFontSizeSnapshot;		// global font size overlay to snapshot stream, unit px
	int						nFontSizeMergeSnapshot; // combination picture overlay to snapshot stream, unit px
} NET_OSD_COMM_INFO;

// Encode widget - PTZZoom info
typedef struct tagNET_OSD_PTZZOOM_INFO
{
	DWORD					dwSize;
	BOOL					bMainBlend;				// Whether overlay to main stream  or not
	BOOL					bPreviewBlend;			// Whether overlay to preview stream or not
	NET_COLOR_RGBA		    stuFrontColor;			// front color
	NET_COLOR_RGBA		    stuBackColor;			// back color
	NET_RECT		    	stuRect;				// Zone. The coordinates value ranges from  0 to 8191. Only use left value and top value.The point (left,top) shall be the same as the point(right,bottom).
	int						nDisplayTime;			// the time of display,0 is means always show
} NET_OSD_PTZZOOM_INFO;

// Encode widget - GPS title info
typedef struct tagNET_OSD_GPS_TITLE
{
	DWORD					dwSize; 
	BOOL					bEncodeBlend;            //whether overlay to main stream or not
}NET_OSD_GPS_TITLE;

#define MAX_COUNT_PLANE		32	// max plane counts

// the osd info of people number plane 
typedef struct tagNET_STATPLAN_INFO
{
	int						nPlanID;				// plane ID
	BOOL					bEncodeBlend;			// blend main stream or not
	NET_COLOR_RGBA		    stuFrontColor;			// foreground color
	NET_COLOR_RGBA		    stuBackColor;			// background color
	EM_TITLE_TEXT_ALIGNTYPE emTextAlign;			// align type
	NET_RECT		    	stuRect;				// rect(0~8192)
	BOOL					bShowEnterNum;			// show the number of peoples entering or not
	BOOL					bShowExitNum;			// show the number of peoples leaving or not
	BYTE                    byReserved[128];        // reserved 
} NET_STATPLAN_INFO;

// the osd config of people number plane
typedef struct tagNET_OSD_NUMBER_STATPLAN
{
	DWORD					dwSize;
	int						nRetPlanCount;							// the actual count of plans
	NET_STATPLAN_INFO		stuStatplanInfo[MAX_COUNT_PLANE];		// OSD config
} NET_OSD_NUMBER_STATPLAN;
typedef enum tatNET_EM_AUDIO_FORAMT
{
	EM_AUDIO_FORMAT_UNKNOWN,							// unknown
	EM_AUDIO_FORMAT_G711A,                              // G711a
    EM_AUDIO_FORMAT_PCM,                                // PCM
    EM_AUDIO_FORMAT_G711U,                              // G711u
    EM_AUDIO_FORMAT_AMR,                                // AMR
    EM_AUDIO_FORMAT_AAC,                                // AAC
    
    EM_AUDIO_FORMAT_G726,								// G.726
    EM_AUDIO_FORMAT_G729,								// G.729
    EM_AUDIO_FORMAT_ADPCM,								// ADPCM
    EM_AUDIO_FORMAT_MPEG2,								// MPEG2
    EM_AUDIO_FORMAT_MPEG2L2,							// MPEG2-Layer2
    EM_AUDIO_FORMAT_OGG,								// OGG
    EM_AUDIO_FORMAT_MP3,								// MP3
    EM_AUDIO_FORMAT_G7221,								// G.722.1
} NET_EM_AUDIO_FORMAT;


// the type of video compression
typedef enum tagNET_EM_VIDEO_COMPRESSION
{
	EM_VIDEO_FORMAT_MPEG4,								// MPEG4
	EM_VIDEO_FORMAT_MS_MPEG4,							// MS-MPEG4
	EM_VIDEO_FORMAT_MPEG2,								// MPEG2
	EM_VIDEO_FORMAT_MPEG1,								// MPEG1
	EM_VIDEO_FORMAT_H263,								// H.263
	EM_VIDEO_FORMAT_MJPG,								// MJPG
	EM_VIDEO_FORMAT_FCC_MPEG4,							// FCC-MPEG4
	EM_VIDEO_FORMAT_H264,								// H.264
    EM_VIDEO_FORMAT_H265,								// H.265
	EM_VIDEO_FORMAT_SVAC,								// SVAC
} NET_EM_VIDEO_COMPRESSION;

// the type of BitRateControl
typedef enum tagNET_EM_BITRATE_CONTROL
{
	EM_BITRATE_CBR,									// CBR
	EM_BITRATE_VBR,									// VBR
} NET_EM_BITRATE_CONTROL;

// the type of image quality
typedef enum tagNET_EM_IMAGE_QUALITY
{
	EM_IMAGE_QUALITY_Q10 = 1,							// 10%
	EM_IMAGE_QUALITY_Q30,								// 30%
	EM_IMAGE_QUALITY_Q50,								// 50%
	EM_IMAGE_QUALITY_Q60,								// 60%
	EM_IMAGE_QUALITY_Q80,								// 80%
	EM_IMAGE_QUALITY_Q100,								// 100%
} NET_EM_IMAGE_QUALITY;

// H264 Encode Level
typedef enum tagNET_EM_H264_PROFILE_RANK
{
	EM_PROFILE_UNKNOWN,							   // unknow
	EM_PROFILE_BASELINE = 1,                       // Offer I/P Frame, Only support progressive(Progressive Scan)and CAVLC
	EM_PROFILE_MAIN,                               // Offer I/P/B Frame, Support progressiv and interlaced, Offer CAVLC or CABAC
	EM_PROFILE_EXTENDED,                           // Offer I/P/B/SP/SI Frame, Only support progressive(Progressive Scan)and CAVLC
	EM_PROFILE_HIGH,                               // FRExt Main_Profile Based on the new, o8x8 intra prediction(8x8 Intra-frame Predicdion), custom 
												   // Quant(custom quant ), lossless video coding(lossless video encoding), more yuv format
}NET_EM_H264_PROFILE_RANK;

// the type of stream
typedef enum tagNET_EM_FORMAT_TYPE
{
	EM_FORMAT_TYPE_UNKNOWN,				// unknow
	/*main stream*/
	EM_FORMAT_MAIN_NORMAL,				// main stream--normal
	EM_FORMAT_MAIN_MOVEEXAMINE,			// main stream--move examine
	EM_FORMAT_MAIN_ALARM,				// main stream--alarm

	/*extra stream*/
	EM_FORMAT_EXTRA1,					// extra stream 1
	EM_FORMAT_EXTRA2,					// extra stream 2
	EM_FORMAT_EXTRA3,					// extra stream 3
} NET_EM_FORMAT_TYPE;

// the type of pack
typedef enum tagNET_EM_PACK_TYPE
{
	EM_PACK_UNKOWN,				// UNKOWN
	EM_PACK_DHAV,				// DHAV
	EM_PACK_PS,					// ps
} NET_EM_PACK_TYPE;


// the video info of main(extra) stream
typedef struct tagNET_ENCODE_VIDEO_INFO
{
	DWORD						dwSize;
	NET_EM_FORMAT_TYPE			emFormatType;				// stream type, you need to set this value when get or set config
	BOOL						bVideoEnable;				// enable
	NET_EM_VIDEO_COMPRESSION 	emCompression;				// the type of video compression
	int							nWidth;						// the wigth of video 
	int							nHeight;					// the height of video
	NET_EM_BITRATE_CONTROL		emBitRateControl;			// the type of BitRateControl
	int							nBitRate;					// Video bit rate (kbps)
	float						nFrameRate;					// Frame Rate 
	int							nIFrameInterval;			// I frame interval(1-100). For example, 50 means there is I frame in each 49 B frame or P frame.
	NET_EM_IMAGE_QUALITY		emImageQuality;				// image quality
} NET_ENCODE_VIDEO_INFO;

// video pack info of main(extra) stream
typedef struct tagNET_ENCODE_VIDEO_PACK_INFO
{
	DWORD						dwSize;
	NET_EM_FORMAT_TYPE			emFormatType;				// stream type, you need to set this value when get or set config
	NET_EM_PACK_TYPE			emPackType;					// pack type 
} NET_ENCODE_VIDEO_PACK_INFO;

// video SVC info of main(extra) stream
typedef struct tagNET_ENCODE_VIDEO_SVC_INFO
{
	DWORD						dwSize;
	NET_EM_FORMAT_TYPE			emFormatType;				// stream type, you need to set this value when get or set config
	int							nSVC;						// SVC-T level
} NET_ENCODE_VIDEO_SVC_INFO;

//video profile info of main(extra) stream
typedef struct tagNET_ENCODE_VIDEO_PROFILE_INFO
{
	DWORD						dwSize;
	NET_EM_FORMAT_TYPE			emFormatType;				// stream type, you need to set this value when get or set config
	NET_EM_H264_PROFILE_RANK 	emProfile;                	// H.264 encode level
} NET_ENCODE_VIDEO_PROFILE_INFO;

// audio compression info of main(extra) stream
typedef struct tagNET_ENCODE_AUDIO_COMPRESSION_INFO
{
	DWORD						dwSize;
	BOOL						bAudioEnable;			// enable
	NET_EM_FORMAT_TYPE			emFormatType;			// stream type, you need to set this value when get or set config
	NET_EM_AUDIO_FORMAT			emCompression;			// the type of audio compression
} NET_ENCODE_AUDIO_COMPRESSION_INFO;

//audio  info of main(extra) stream
typedef struct tagNET_ENCODE_AUDIO_INFO
{
	DWORD						dwSize;
	NET_EM_FORMAT_TYPE			emFormatType;			// stream type, you need to set this value when get or set config
	int							nDepth;					// sampling depth
	int							nFrequency;				// sampling frequency
	int							nMode;					// audio mode
	int							nFrameType;				// audio frame type, 0-DHAV, 1-PS
	int							nPacketPeriod;			// audio packet period, ms
} NET_ENCODE_AUDIO_INFO;

// snap type
typedef enum tagNET_EM_SNAP_TYPE
{
	EM_SNAP_UNKNOWN,			// unknow
	EM_SNAP_NORMAL,				// normal
	EM_SNAP_MOVEEXAMINE,		// move examine
	EM_SNAP_ALARM,				// alarm
} NET_EM_SNAP_TYPE;

// the snap info
typedef struct tagNET_ENCODE_SNAP_INFO
{
	DWORD						dwSize;
	NET_EM_SNAP_TYPE			emSnapType;					// snap type
	BOOL						bSnapEnable;				// enable
	NET_EM_VIDEO_COMPRESSION 	emCompression;				// the video compression
	int							nWidth;						// the width of image
	int							nHeight;					// the height of image
	float						nFrameRate;					// frame rate
	int 						nQualityRange;				// the range of image quality
	NET_EM_IMAGE_QUALITY		emImageQuality;				// image quality
} NET_ENCODE_SNAP_INFO;

// snap time info
typedef struct tagNET_ENCODE_SNAP_TIME_INFO
{
	DWORD				dwSize;
	short           	shPicTimeInterval;             	// the time interval of snap, unit:second
	BYTE            	bPicIntervalHour;              	// the time interval of snap, unit:hour 
	DWORD           	dwTrigPicIntervalSecond;       	// the time interval of snap after alarm, unit:second
} NET_ENCODE_SNAP_TIME_INFO;

// config type
typedef enum tagNET_EM_CONFIG_TYPE
{
	NET_EM_CONFIG_DAYTIME,			// day time
	NET_EM_CONFIG_NIGHT,			// night
	NET_EM_CONFIG_NORMAL,			// normal
} NET_EM_CONFIG_TYPE;


// channel title info
typedef struct tagNET_ENCODE_CHANNELTITLE_INFO
{
	DWORD				dwSize;
	char				szChannelName[MAX_CHANNEL_NAME_LEN];	// channel title
} NET_ENCODE_CHANNELTITLE_INFO;

// the type of audio input
typedef enum tagNET_EM_AUDIOIN_SOURCE_TYPE
{
	NET_EM_AUDIOIN_SOURCE_UNKNOW,			// unkonw
	NET_EM_AUDIOIN_SOURCE_COAXIAL,			// Coaxial
	NET_EM_AUDIOIN_SOURCE_BNC,				// BNC
	NET_EM_AUDIOIN_SOURCE_HDCVI_BNC,		// HDCVI_BNC
	NET_EM_AUDIOIN_SOURCE_LINEIN,			// LineIn
	NET_EM_AUDIOIN_SOURCE_LINEIN1,			// LineIn1
	NET_EM_AUDIOIN_SOURCE_LINEIN2,			// LineIn2
	NET_EM_AUDIOIN_SOURCE_LINEIN3,			// LineIn3
	NET_EM_AUDIOIN_SOURCE_MIC,				// Mic
	NET_EM_AUDIOIN_SOURCE_MIC1,				// Mic1
	NET_EM_AUDIOIN_SOURCE_MIC2,				// Mic2
	NET_EM_AUDIOIN_SOURCE_MIC3,				// Mic3
	NET_EM_AUDIOIN_SOURCE_MICOUT,			// MicOut
	NET_EM_AUDIOIN_SOURCE_REMOTE,			// Remote
	NET_EM_AUDIOIN_SOURCE_REMOTE1,			// Remote1
	NET_EM_AUDIOIN_SOURCE_REMOTE2,			// Remote2
	NET_EM_AUDIOIN_SOURCE_REMOTE3,			// Remote3
} NET_EM_AUDIOIN_SOURCE_TYPE;

// the source info of audio input
typedef struct tagNET_ENCODE_AUDIO_SOURCE_INFO
{
	DWORD						dwSize;
	int 						nMaxAudioInSource;		// the max counts of audio input source
	int							nRetAudioInSource;		// the reality counts of audio input source
	NET_EM_AUDIOIN_SOURCE_TYPE	emAudioInSource[24];	// the source of  audio input
} NET_ENCODE_AUDIO_SOURCE_INFO;

// the denoise info of audio input
typedef struct tagNET_AUDIOIN_DENOISE_INFO
{
	DWORD				dwSize;
	BOOL				bEnable;			// enable
} NET_AUDIOIN_DENOISE_INFO;

// the volume config of audio input
typedef struct tagNET_AUDIOIN_VOLUME_INFO
{
	DWORD				dwSize;
	int					nVolume;			// the volume 
} NET_AUDIOIN_VOLUME_INFO;

// the volume config of audio output
typedef struct tagNET_AUDIOOUT_VOLUME_INFO
{
	DWORD				dwSize;
	int					nVolume;			// the volume
} NET_AUDIOOUT_VOLUME_INFO;

// switch mode
typedef enum tagNET_EM_SWITCH_MODE
{
	NET_EM_SWITCH_MODE_WIGHT,		// do not switch, use day time config
	NET_EM_SWITCH_MODE_LIGHT,		// switch by brightness
	NET_EM_SWITCH_MODE_TIME,		// switch by time
	NET_EM_SWITCH_MODE_NIGHT,		// do not switch, use night config
	NET_EM_SWITCH_MODE_NORMAL,		// use normal config
} NET_EM_SWITCH_MODE;

// roughly time of sunrise or sunset
typedef struct tagNET_SUN_TIME
{
	int				nHour;				// hour
	int				nMinute;			// minute
	int				nSecond;			// second
} NET_SUN_TIME;

// the config of switch mode
typedef struct tagNET_VIDEOIN_SWITCH_MODE_INFO
{
	DWORD				dwSize;
	NET_EM_SWITCH_MODE	emSwitchMode;			// switch mode

	/*roughly time of sunrise and sunset, use night config between sunset and sunrise, 
	it is valid when emSwitchMode is NET_EM_SWITCH_MODE_TIME*/
	NET_SUN_TIME		stuSunRiseTime;			// sunrise time
	NET_SUN_TIME		stuSunSetTime;			// sunset time
} NET_VIDEOIN_SWITCH_MODE_INFO;

// the color config of video input
typedef struct tagNET_VIDEOIN_COLOR_INFO
{
	DWORD				dwSize;
	NET_EM_CONFIG_TYPE	emCfgType;				// config type, you need set the value wether set or get config
	int					nBrightness;			// Brightness 0-100
	int					nContrast;				// nContrast 0-100
	int					nSaturation;			// Saturation 0-100
	int					nGamma;					// Gamma 0-100
} NET_VIDEOIN_COLOR_INFO;

// image options config of video input
typedef struct tagNET_VIDEOIN_IMAGE_INFO
{
	DWORD				dwSize;
	NET_EM_CONFIG_TYPE	emCfgType;				// config type, you need set the value wether set or get config
	BOOL				bMirror;				// mirror
	BOOL				bFlip;					// flip
	int					nRotate90;				// 0-not rotate, 1-clockwise 90 degrees, 2-anti-clockwise 90 degrees
} NET_VIDEOIN_IMAGE_INFO;

// image stable type
typedef enum tagNET_EM_STABLE_TYPE
{
	NET_EM_STABLE_OFF,				// off
	NET_EM_STABLE_ELEC,				// electronics
	NET_EM_STABLE_LIGHT,			// light
	NET_EM_STABLE_CONTORL = 4,		// control
} NET_EM_STABLE_TYPE;

// image stable config of video input
typedef struct tagNET_VIDEOIN_STABLE_INFO
{
	DWORD				dwSize;
	NET_EM_CONFIG_TYPE	emCfgType;			// config type, you need set the value wether set or get config
	NET_EM_STABLE_TYPE	emStableType;		// image stable type
} NET_VIDEOIN_STABLE_INFO;

// iris auto config of video input
typedef struct tagNET_VIDEOIN_IRISAUTO_INFO
{
	DWORD				dwSize;
	NET_EM_CONFIG_TYPE	emCfgType;		// config type, you need set the value wether set or get config
	BOOL				bIrisAuto;		// iris auto enable
} NET_VIDEOIN_IRISAUTO_INFO;

// image enhancement config of video input
typedef struct tagNET_VIDEOIN_IMAGEENHANCEMENT_INFO
{
	DWORD				dwSize;
	BOOL				bEnable;			// enable
	int					nLevel;				// image enhancement level
}NET_VIDEOIN_IMAGEENHANCEMENT_INFO;

//normal exposure config of video input
typedef struct tagNET_VIDEOIN_EXPOSURE_NORMAL_INFO
{
	DWORD					dwSize;
	NET_EM_CONFIG_TYPE		emCfgType;				// config type, you need set the value wether set or get config
	NET_EM_EXPOSURE_MODE	emExposureMode;			// exposure mode
	int						nAntiFlicker;			// anti flicker 0-Outdoor  1-50Hz 2-60Hz
	int						nCompensation;			// Compensation 0-100
	int						nGain;					// gain value 0-100
	int						nGainMin;				// the min value of Gain 0-100
	int						nGainMax;				// the max value of gain 0-100
	int						nExposureIris;			// the value of iris(0-100),  it is valid when mode is NET_EM_EXPOSURE_APERTUREFIRST
	double					dbExposureValue1;		// Auto exposure value min limit or manual axposure custom, unit is millisecond (0.1ms~80ms).
	double					dbExposureValue2;		// Auto exposure time max limit, unit is millisecond (0.1ms~80ms)
} NET_VIDEOIN_EXPOSURE_NORMAL_INFO;

// other exposure config of video input
typedef struct tagNET_VIDEOIN_EXPOSURE_OTHER_INFO
{
	DWORD					dwSize;
	NET_EM_CONFIG_TYPE		emCfgType;				// config type, you need set the value wether set or get config
	BOOL					bSlowShutter;			// slow shutter enable
	int						nSlowSpeed;				// slow speed level
	int						nSlowAutoExposure;		// slow exposure
	int						nRecoveryTime;			// recovery time,units:second. 0 means not recovery
	int						nIrisMin;				// iris value min limit(0-100)
	int						nIrisMax;				// iris value max limit(0-100)
} NET_VIDEOIN_EXPOSURE_OTHER_INFO;

// exposure shutter config of video input
typedef struct tagNET_VIDEOIN_EXPOSURE_SHUTTER_INFO
{
	DWORD					dwSize;
	BOOL					bAutoSyncPhase;			// auto syncphase enable
	float					fShutter;				// shutter value(0.1ms-80ms), it is valid when bAutoSyncPhase is true. the value mst be brtween ExposureValue1  and ExposureValue2
	int						nPhase;					// phase value,0~360
} NET_VIDEOIN_EXPOSURE_SHUTTER_INFO;



// back light config of video input
typedef struct tagNET_VIDEOIN_BACKLIGHT_INFO
{
	DWORD					dwSize;
	NET_EM_CONFIG_TYPE		emCfgType;				// config type, you need set the value wether set or get config
	NET_EM_BACK_MODE		emBlackMode;			// black mode
	NET_EM_BLACKLIGHT_MODE	emBlackLightMode;		// black light mode
	NET_RECT				stuBacklightRegion;     // the region of black light
	int						nWideDynamicRange;		// the value of widedynamic, it is valid when emBlackMode is NET_EM_BACKLIGHT_MODE_WIDEDYNAMIC
	int						nGlareInhibition;		// the value of glareinhibition, it is valid when emBlackMode is NET_EM_BACKLIGHT_MODE_GLAREINHIBITION
} NET_VIDEOIN_BACKLIGHT_INFO;


//the config of Intensity, valid when black mode is SSA
typedef struct tagNET_VIDEOIN_INTENSITY_INFO
{
	DWORD						dwSize;
	NET_EM_CONFIG_TYPE			emCfgType;				// config type, you need set the value wether set or get config
	NET_EM_BACK_INTENSITY_MODE	emIntensityMode;		// the mode of Intensity
	int							nIntensity;				// Intensity value[0-100] , valid when emIntensityMode==NET_EM_INTENSITY_MANUAL
} NET_VIDEOIN_INTENSITY_INFO;


// white balance config of video input
typedef struct tagNET_VIDEOIN_WHITEBALANCE_INFO
{
	DWORD						dwSize;
	NET_EM_CONFIG_TYPE			emCfgType;					// config type, you need set the value wether set or get config
	NET_EM_WHITEBALANCE_TYPE	emWhiteBalanceType;			// white balance type
	int							nGainRed;					// red gain (0-100), it is valid when emWhiteBalanceType is NET_EM_WHITEBALANCE_CUSTOM
	int							nGainBlue;					// blue gain (0-100), it is valid when emWhiteBalanceType is NET_EM_WHITEBALANCE_CUSTOM
	int							nGainGreen;					// green gain (0-100), it is valid when emWhiteBalanceType is NET_EM_WHITEBALANCE_CUSTOMCOLORTEMP
	int							nColorTemperature;			// color temperature level, it is valid when emWhiteBalanceType is NET_EM_WHITEBALANCE_CUSTOM
} NET_VIDEOIN_WHITEBALANCE_INFO;

// day or night type
typedef enum tagNET_EM_DAYNIGHT_TYPE
{
	NET_EM_DAYNIGHT_COLOR,				// always color
	NET_EM_DAYNIGHT_AUTO,				// according to the brightness auromatically switches
	NET_EM_DAYNIGHT_WHITEBLACK,			// always black and white
} NET_EM_DAYNIGHT_TYPE;

//day or night config of video input
typedef struct tagNET_VIDEOIN_DAYNIGHT_INFO
{
	DWORD						dwSize;
	NET_EM_CONFIG_TYPE			emCfgType;					// config type, you need set the value wether set or get config
	NET_EM_DAYNIGHT_TYPE		emDayNightType;				// day or night type
	int							nDayNightSensitivity;		// Sensitivity of day or night(1-3)
	int							nDayNightSwitchDelay;		// delay time of day or night switch(2s - 10s)
} NET_VIDEOIN_DAYNIGHT_INFO;

// ICR config of video input
typedef struct tagNET_VIDEOIN_DAYNIGHT_ICR_INFO
{
	DWORD						dwSize;
	NET_EM_CONFIG_TYPE			emCfgType;					// config type, you need set the value wether set or get config
	NET_EM_ICR_TYPE				emType;						// ICR switch type
} NET_VIDEOIN_DAYNIGHT_ICR_INFO;

//lighting mode
typedef enum tagNET_EM_LIGHTING_MODE
{
	NET_EM_LIGHTING_UNKNOW,			// unknow
	NET_EM_LIGHTING_MANUAL,			// manual
	NET_EM_LIGHTING_AUTO,			// auto
	NET_EM_LIGHTING_OFF,			// off
	NET_EM_LIGHTING_ZOOMPRIO,		// zoomprio
	NET_EM_LIGHTING_TIMING,			// timing
	NET_EM_LIGHTING_SMARTLIGHT,		// smart light(only used by SD)
} NET_EM_LIGHTING_MODE;

// lighting config of video input
typedef struct tagNET_VIDEOIN_LIGHTING_INFO
{
	DWORD						dwSize;
	NET_EM_LIGHTING_MODE		emLightMode;			// lighting mode
	int							nCorrection;			// correction of lighting
	int							nSensitive;				// Sensitive of lighting
	int							nNearLight;				// near light 0-100
	int							nFarLight;				// far light 0-100
} NET_VIDEOIN_LIGHTING_INFO;

// defog mode
typedef enum tagNET_EM_DEFOG_MODE
{
	NET_EM_DEFOG_UNKNOW,			// unknow
	NET_EM_DEFOG_OFF,				// off
	NET_EM_DEFOG_AUTO,				// auto
	NET_EM_DEFOG_MANAUL,			// manual
}NET_EM_DEFOG_MODE;

//intensity mode
typedef enum tagNET_EM_INTENSITY_MODE
{
	NET_EM_INTENSITY_MODE_UNKNOW,	//unknow
	NET_EM_INTENSITY_MODE_AUTO,  	//auto
	NET_EM_INTENSITY_MODE_MANUAL, 	//manual
}NET_EM_INTENSITY_MODE;


// defog config of video input
typedef struct tagNET_VIDEOIN_DEFOG_INFO
{
	DWORD							dwSize;
	NET_EM_CONFIG_TYPE				emCfgType;					// config type, you need set the value wether set or get config
	NET_EM_DEFOG_MODE				emDefogMode;				// defog mode
	int								nIntensity;					// Intensity 0-100
	NET_EM_INTENSITY_MODE			emIntensityMode;			// Intensity mode
	int								nLightIntensityLevel;		// Intensity light level(0-15)
	BOOL							bCamDefogEnable;			// enable
} NET_VIDEOIN_DEFOG_INFO;


// focus mode config of video input
typedef struct tagNET_VIDEOIN_FOCUSMODE_INFO
{
	DWORD							dwSize;
	NET_EM_CONFIG_TYPE				emCfgType;				// config type, you need set the value wether set or get config
	NET_EM_FOCUS_MODE				emFocusMode;			// focus mode
} NET_VIDEOIN_FOCUSMODE_INFO;

// sensitivity type
typedef enum tagNET_EM_SENSITIVITY_TYPE
{
	NET_EM_SENSITIVITY_HIGH,				// high
	NET_EM_SENSITIVITY_DEFAULT,				// default
	NET_EM_SENSITIVITY_LOW,					// low
} NET_EM_SENSITIVITY_TYPE;

// ICR type
typedef enum tagNET_EM_IRC_TYPE
{
	NET_EM_IRC_DONOT,			// do not correct
	NET_EM_IRC_DOIT,			// do correct
	NET_EM_IRC_AUTO,			// auto correct
} NET_EM_IRC_TYPE;


// focus options config of video input
typedef struct tagNET_VIDEOIN_FOCUSVALUE_INFO
{
	DWORD							dwSize;
	NET_EM_CONFIG_TYPE				emCfgType;				// config type, you need set the value wether set or get config
	NET_EM_SENSITIVITY_TYPE			emSensitivity;			// Sensitivity
	NET_EM_IRC_TYPE					emIRCorrection;			// IR correction
	NET_EM_FOCUS_LIMITSELECT_MODE	emFocusLimitSelectMode;	// focus limit select mode
	int								nFocusLimit;			// the value of focus limit
	BOOL							bAutoFocusTrace;		// auto focus trace
} NET_VIDEOIN_FOCUSVALUE_INFO;


// sharpness config of video input
typedef struct tagNET_VIDEOIN_SHARPNESS_INFO
{
	DWORD							dwSize;
	NET_EM_CONFIG_TYPE				emCfgType;					// config type, you need set the value wether set or get config
	NET_EM_SHARPNESS_MODE			emSharpnessMode;			// sharpness type
	int								nSharpness;					// value of sharpness(0-100)
	int								nLevel;						// sharpness inhibition level
} NET_VIDEOIN_SHARPNESS_INFO;

// 3D denoise type
typedef enum tagNET_EM_3D_TYPE
{
	NET_EM_3D_UNKONW,		// unknow
	NET_EM_3D_OFF,			// off
	NET_EM_3D_AUTO,			// auto
} NET_EM_3D_TYPE;

// algorithm denoise type
typedef enum tagNET_EM_ALGORITHM_TYPE
{
	NET_EM_ALGORITHM_UNKNOW,		// unknow
	NET_EM_ALGORITHM_OFF,			// off
	NET_EM_ALGORITHM_MANUAL,		// manual
}NET_EM_ALGORITHM_TYPE;

// algorithm denoise config of video input
typedef struct tagNET_DENOISE_ALGORITHM
{
	NET_EM_ALGORITHM_TYPE	emAlgorithmType;		// algorithm denoise type
	int						nTnfLevel;				// time domain level
	int						nSnfLevel;				// space domain level
	BYTE					bReserved[256];			// reserved bytes
} NET_DENOISE_ALGORITHM;

// comm denoise config of video input
typedef struct tagNET_VIDEOIN_DENOISE_INFO
{
	DWORD							dwSize;
	NET_EM_CONFIG_TYPE				emCfgType;					// config type, you need set the value wether set or get config
	BOOL							b2DEnable;					// 2D denoise enable
	int								n2DLevel;					// 2D denoise level(0-100), valid when b2DEnable==TRUE
	NET_DENOISE_ALGORITHM			stuAlgorithm;				// algorithm denoise options
} NET_VIDEOIN_DENOISE_INFO;

// 3D denoise config of video input
typedef struct tagNET_VIDEOIN_3D_DENOISE_INFO
{
	DWORD							dwSize;
	NET_EM_CONFIG_TYPE				emCfgType;					// config type, you need set the value wether set or get config
	NET_EM_3D_TYPE					em3DType;					// 3D denoise type
	int								nAutoLevel;					// 3D denoise type, valid when em3DType==NET_EM_3D_AUTO
} NET_VIDEOIN_3D_DENOISE_INFO;

// snap mode config
typedef struct tagNET_SNAP_MODE
{
    DWORD                           dwSize;
    int                             nMode;                      // 0:AUTO,1:manual operation,2:close
    BOOL                            bDirectStorageMode;         // TRUE Open. FALSE Close
}NET_SNAP_MODE;

// YueQing External lighting configuration(NET_EM_CFG_YUEQING_SUPPLYLIGHTING)
typedef struct tagNET_YUEQING_SUPPLYLIGHTING_INFO
{
	DWORD				dwSize;
	BOOL				bTimeEnable;					// Time   Enable
	BOOL				bManualEnable;					// Manual Enable
	DH_TSECT			stuTimeSection[DH_TSCHE_DAY_NUM][DH_TSCHE_SEC_NUM];	// time schedule
}NET_YUEQING_SUPPLYLIGHTING_INFO;


// the data of master-slave points
typedef struct tagNET_POINT_INFO
{
	DH_POINT	stuMasterPoint;		// the points of master, 8192 coordinate system
	DH_POINT	stuSlavePoint;		// the points of slave
	BYTE    byReserved[256];	// reserved bytes
}NET_POINT_INFO;

typedef struct tagNET_POINT_PAIR_EX
{
	int					nCaliSceneID;           // The serial number of the scene can be calibrated, and the panoramic image can be divided into several scenes
	PTZ_SPACE_UNIT		stuSlaveCenter;			// Calibration of the first goal, the ball machine and the first scenne to the scene to approximate the location of the ball at the center coordinates
	NET_POINT_INFO		stuPoint[16];			// After the scene fixed ball machine, record the corresponding relationship of the two pixels pixel coordinates
	int					nEffectiveNumber;		// stuPoint Effective number
	BYTE                byReserved[256];		// reserved bytes
} NET_POINT_PAIR_EX;

// Calibration value of master slave tracker
typedef struct tagNET_CALIBRATE_MATRIX_ELEMENT_EX
{
	char                szMasterSN[48];							// Group slave device serial number
    char                szSlaveSN[48];							// Device serial number
	PTZ_SPACE_UNIT		stuExpectRatio;							// Desired magnification(optional)
	NET_POINT_PAIR_EX	stuPointPairEx[8];						// Multi target tracking
	int					nEffectiveNumber;						// stuPointPairEx Effective number
	BYTE                byReserved[256];						// byReserved
} NET_CALIBRATE_MATRIX_ELEMENT_EX;

// The calibration matrix extension configuration of master-slave Multi purpose use(Corresponding NET_EM_CFG_CALIBRATE_MATRIX command)
typedef struct tagNET_MULTI_SENSOR_INFO
{	
	DWORD							 dwSize;
    DWORD							 nMaxCalibElement;			// the max counts of master-slave tracking mark matrix config
	DWORD							 nRetCalibElement;			// the counts of effective master-slave tracking mark matrix config
	NET_CALIBRATE_MATRIX_ELEMENT_EX	*pstCaliMatrixElementEx;	// memory by user
} NET_MULTI_SENSOR_INFO;

// burn CD coding plan, each channel a configuration(include picinpic channel)
typedef struct tagNET_ENCODE_PLAN_INFO
{
	DWORD				dwSize;
	BOOL                bEnable;                        // the channel is able or not,FALSE:without this channel,TRUE:this channel is available
	unsigned int        nExpectTime;                    // the business forecast time,int(s)
	CAPTURE_SIZE		emResolution;                   // video resolution
	unsigned int        nBitRate;                       // vide bit rate(kbps)
	unsigned int		nPacketSize;					// packet size of a single burning file(KB)
}NET_ENCODE_PLAN_INFO;

// Get information list input parameters(CLIENT_ScenicSpotGetPointInfos) 
typedef struct tagNET_IN_SCENICSPOT_GETPOINTINFOS_INFO
{
	DWORD				dwSize;
	int					nChannelID;						// Channel number
	int					nOffset;						// start form the first fewmax 255
	int					nLimit;							// Get the first structurebegin 0
}NET_IN_SCENICSPOT_GETPOINTINFOS_INFO;

typedef struct
{
    int                nHorizontalAngle;               // Horizontal Angle 
    int                nVerticalAngle;                 // Vertical Angle
    int                nMagnification;                 // Magnification
} POSTION;

typedef struct
{								
    int                nX;								// X coordinate(0~8191) 
    int                nY;								// Y coordinate(0~8191) 
} POINTCOORDINATE;											

typedef struct
{

	int					nIndex;											// information point number
	bool				bEnable;										// whether the information is effective
	bool				bTitleAttribute;								// Is there a child heading for this information pingt
	POSTION				stuPostion;										// tilt direction and magnificationHorizontal Angle 0~360, Vertical Angle -1800~1800, Magnification0~127
	POINTCOORDINATE		stuPoint;										// scenery position
	char				szTitleName[MAX_COMMON_STRING_64];				// Primary title
	BYTE				byTitleType;									// Level header name type
	BYTE                byReserved[256];								// reserved bytes
}POINTINFOS;

// Get information list output parameters(CLIENT_ScenicSpotGetPointInfos)
typedef struct tagNET_OUT_SCENICSPOT_GETPOINTINFOS_INFO
{
	DWORD				dwSize;
	int					nTotal;													// how many points of view
	int					nRetSceneNum;										    // Actual access to number
	POINTINFOS			stuPointInfos[MAX_SCENICSPOT_POINTS_NUM];				// The array length is equal to limit and the last set of queries may be less than limit		
}NET_OUT_SCENICSPOT_GETPOINTINFOS_INFO;

// set the scene, the original number will be covered input parameters (CLIENT_ScenicSpotSetPointInfo)
typedef struct tagNET_IN_SCENICSPOT_SETPOINTINFO_INFO
{
	DWORD				dwSize;
	int					nChannel;										// Channel
	int					nIndex;											// information point number
	bool				bEnable;										// whether the information is effective
	bool				bTitleAttribute;								// Is there a child heading for this information pingt
	POSTION				stuPosition;									// tilt direction and magnification
	POINTCOORDINATE		stuPoint;										// scenery position
	char				szTitleName[MAX_COMMON_STRING_64];				// Primary title
	BYTE				byTitleType;									// Level header name type
}NET_IN_SCENICSPOT_SETPOINTINFO_INFO;

// set the scene, the original number will be covered output parameters (CLIENT_ScenicSpotSetPointInfo)
typedef struct tagNET_OUT_SCENICSPOT_SETPOINTINFO_INFO
{
	DWORD				dwSize;
}NET_OUT_SCENICSPOT_SETPOINTINFO_INFO;

// Get the ability to support the scene points(CLIENT_ScenicSpotGetCaps input parameter) 
typedef struct tagNET_IN_SCENICSPOT_GETCAPS_INFO
{
	DWORD				dwSize;
	int					nChannel;					// channel number
}NET_IN_SCENICSPOT_GETCAPS_INFO;

// Capability Set
typedef struct tagNET_SCENICSPOT_CAPS_INFO
{
	BOOL						bEnable;			// Do you support feature points
	unsigned int				nTotalNum;			// How many poiints of view are supported at most
	unsigned int				nRegionNum;			// How many scene points does a single picture
	BYTE                		byReserved[256];	// reserved bytes
}NET_SCENICSPOT_CAPS_INFO;

// Get the ability to support the scene points(CLIENT_ScenicSpotGetCaps output parameter)
typedef struct tagNET_OUT_SCENICSPOT_GETCAPS_INFO
{
	DWORD						dwSize;
	NET_SCENICSPOT_CAPS_INFO	stuCaps;			// Capability Set information
}NET_OUT_SCENICSPOT_GETCAPS_INFO;

// sound effect type of alarm
typedef enum tagEM_ALARM_SOUND_EFFECT
{
    EM_ALARM_SOUND_EFFECT_UNKNOWN,                      // unknown effect
    EM_ALARM_SOUND_EFFECT_MUTE,                         // mute
    EM_ALARM_SOUND_EFFECT_NOTICE,                       // notice sound
    EM_ALARM_SOUND_EFFECT_ALERT,                        // alert sound
}EM_ALARM_SOUND_EFFECT;

// alarm sound config of device
typedef struct tagNET_ALARM_SOUND_INFO
{
    DWORD				dwSize;
    EM_ALARM_SOUND_EFFECT emSoundEffect;                // sound effect
}NET_ALARM_SOUND_INFO;

// stereo calibrate config
typedef struct tagNET_STEREO_CALIBRATE_INFO
{
	DWORD				dwSize;
	int					nCameraHeight;					// last saved calibrate heightunit: cm
	float				fCameraAngle;					// last saved calibrate angle unit: angle
} NET_STEREO_CALIBRATE_INFO;
// network connection mode
typedef enum tagEM_LINK_MODE
{
    EM_LINK_MODE_UNKNOWN,                                       // unknown mode
    EM_LINK_MODE_AUTO,                                          // auto select 
    EM_LINK_MODE_ADHOC,                                         // ADHOC
    EM_LINK_MODE_INFRASTRUCTURE,                                // infrastructure
}EM_LINK_MODE;

// Access point encryption mode
typedef enum tagEM_AP_DATA_ENCRYPT
{
    EM_AP_DATA_ENCRYPT_UNKNOWN,                           // UnKnown
    EM_AP_DATA_ENCRYPT_NONE,                              // NONE
    EM_AP_DATA_ENCRYPT_AUTO,                              // AUTO
    EM_AP_DATA_ENCRYPT_SHARED,                            // SHARED
    EM_AP_DATA_ENCRYPT_OPEN,                              // OPEN
    EM_AP_DATA_ENCRYPT_TKIP,                              // TKIP
    EM_AP_DATA_ENCRYPT_AES,                               // AES
} EM_AP_DATA_ENCRYPT;

// power mode
typedef enum tagEM_AP_POWER_MODE
{
    EM_AP_POWER_MODE_UNKNOWN,                             // UnKnown
    EM_AP_POWER_MODE_LOW,                                 // Low
    EM_AP_POWER_MODE_MIDDLE,                              // Middle
    EM_AP_POWER_MODE_HIGH,                                // High
}EM_AP_POWER_MODE;

// DHCP INFO
typedef struct tagNET_DHCP_INFO
{
    char                    szStartIP[NET_IPADDRSTR_LEN];       // start IP
    char                    szStopIP[NET_IPADDRSTR_LEN];        // stop IP
    int                     nLeaseTime;                         // Renewal time unit:minute
    BYTE                    byReserved[512];                    // reserved bytes
}NET_DHCP_INFO;

// WEP info
typedef struct tagNET_WEP_INFO
{
    int                     nKeyID;                             // key index :0~3
    char                    szKeys[MAX_WEP_KEY_NUM][MAX_COMMON_STRING_64];// keys array
    BYTE                    byReserved[256];                    // reserved bytes
}NET_WEP_INFO;


// WPA/WPA2 PSK info
typedef struct tagNET_WPA_PSK_INFO
{
    char                    szPassword[MAX_COMMON_STRING_64];  // password
    BYTE                    byReserved[256];                   // reserved bytes
}NET_WPA_PSK_INFO;

// EAP method
typedef enum tagEM_EAP_METHOD
{
    EM_EAP_METHOD_UNKNOWN,                                      // UnKnown
    EM_EAP_METHOD_PEAP,                                         // PEAP
    EM_EAP_METHOD_TLS,                                          // TLS
    EM_EAP_METHOD_TTLS,                                         // TTLS
} EM_EAP_METHOD;

// EAP authentication method
typedef enum tagEM_EAP_AUTH_TYPE
{
    EM_EAP_AUTH_TYPE_UNKNOWN,                                   // UnKnown
    EM_EAP_AUTH_TYPE_NONE,                                      // NONE
    EM_EAP_AUTH_TYPE_PAP,                                       // PAP
    EM_EAP_AUTH_TYPE_MSCHAP,                                    // MSCHAP
    EM_EAP_AUTH_TYPE_MSCHAPV2,                                  // MSCHAPV2
    EM_EAP_AUTH_TYPE_GTC,                                       // GTC
} EM_EAP_AUTH_TYPE;

// Acess point authentication method
typedef enum tagEM_AP_AUTHENTICATION
{
    EM_AP_AUTHENTICATION_UNKNOWN,                         // UnKnown
    EM_AP_AUTHENTICATION_OPEN,                            // OPEN
    EM_AP_AUTHENTICATION_WEP,                             // WEP
    EM_AP_AUTHENTICATION_WPAPSK,                          // WPA PSK
    EM_AP_AUTHENTICATION_WPA2PSK,                         // WPA2 PSK
    EM_AP_AUTHENTICATION_WPA,                             // WPA
    EM_AP_AUTHENTICATION_WPA2,                            // WPA2
} EM_AP_AUTHENTICATION;

// WPA/WPA2 info 
typedef struct tagNET_EAP_INFO
{
    EM_EAP_METHOD           emMethod;                           // EAP method
    EM_EAP_AUTH_TYPE        emAuthType;                         // EAP  authentication method
    char                    szIdentity[MAX_COMMON_STRING_64];   // identify
    char                    szAnonymousID[MAX_COMMON_STRING_64];// anonymousID
    char                    szPassword[MAX_COMMON_STRING_64];   // password
    char                    szCaCert[MAX_COMMON_STRING_512];    // CA certificate
    char                    szUserCert[MAX_COMMON_STRING_512];  // user  certificate
    BYTE                    byReserved[256];                    // reserved bytes
}NET_EAP_INFO;

// WIFI connection mode
typedef enum tagEM_WIFI_CONNECT_TYPE
{
    EM_WIFI_CONNECT_TYPE_UNKNOWN,                               // unknown
    EM_WIFI_CONNECT_TYPE_PUSH_BUTTON,                           // push button mode
    EM_WIFI_CONNECT_TYPE_PIN_ENROLLEE,                          // PINEnrollee mode
    EM_WIFI_CONNECT_TYPE_PIN_REG,                               // PIN REG mode ,not supported now
}EM_WIFI_CONNECT_TYPE;

// WIFI security configuration
typedef struct tagNET_WPS_INFO
{
    EM_WIFI_CONNECT_TYPE   emConnType;                          // connection mode
    char                   szPIN[MAX_COMMON_STRING_32];         // PIN code, supported in PINEnrollee mode
    BYTE                   byReserved[256];                     // reserved bytes
}NET_WPS_INFO;

// filter address list
typedef struct tagNET_FILTER_ADDRESS_LIST
{
    BOOL                    bEffective;                         // whether effective:true yes,false no
    char                    szMAC[DH_MACADDR_LEN];              // device mac address
    char                    szDescribeText[DH_COMMON_STRING_256]; // A detailed description of the MAC address  
    BYTE                    byReserved[256];                    // reserved bytes
}NET_FILTER_ADDRESS_LIST;

//the fitering policy of address list
typedef enum tagEM_ADDRLIST_FILTER_POLICY
{
    EM_ADDRLIST_FILTER_POLICY_UNKNOWN,                          // unknown
    EM_ADDRLIST_FILTER_POLICY_WHITE,                            // "white":list of valid address device connections
    EM_ADDRLIST_FILTER_POLICY_BLACK,                            // "black":list of valid address device connections in the forbidden list 
}EM_ADDRLIST_FILTER_POLICY;

// AccessPoint access fiter configuration
typedef struct tagNET_AP_FILTER
{
    BOOL                    bEnable;                            // enable ture:yes false:no
    int                     nMaxAccessNum;                      // maximum number of WIFI device connections
    EM_ADDRLIST_FILTER_POLICY emPolicy;                         // filter policy
    NET_FILTER_ADDRESS_LIST *pstuAddrList;                      // address filter list ,new/delete by user            
    int                     nMaxListNum;                        // the max of address number
    int                     nRetListNum;                        // return address number
    BYTE                    byReserved[256];                    // reserved bytes
}NET_AP_FILTER;

//Accesspoint work pattern type
typedef enum tagEM_WORKPATTERN
{
    EM_WORKPATTERN_UNKNOWN,                          // unknown
    EM_WORKPATTERN_2_4G,                             // 2.4G default
    EM_WORKPATTERN_5G,                               // 5G
}EM_WORKPATTERN;

// Accesspoint work pattern
typedef struct tagNET_AP_WORKPATTERN
{
    EM_WORKPATTERN          emWorkPatternType;                  // work pattern type:2.4G/5G
    BOOL                    bEnable;                            // WIFI enable switch: TRUE(enable), FALSE(disable)
    BOOL                    bHideSSID;                          // hide SSID,TRUE(yes),FALSE(no)
    char                    szSSID[DH_COMMON_STRING_32];        // SSID
    EM_LINK_MODE            emLinkMode;                         // link mode
    int                     nPriority;                          // priority(1-32)
    int                     nChannel;                           // channel(1-13)
    int                     nChannelBandwidth;                  // channel bandwidth,unit:M
    EM_AP_AUTHENTICATION    emAuthentication;                   // authentication
    EM_AP_DATA_ENCRYPT      emDataEncryption;                   // data encryption method
    EM_AP_POWER_MODE        emPowerMode;                        // power mode
    BOOL                    bDHCPEnable;                        // DHCP enable
    NET_DHCP_INFO           stuDHCP;                            // DHCP info
    char                    szIPAddresss[NET_IPADDRSTR_LEN];    // IP address
    char                    szSubnetMask[MAX_COMMON_STRING_16]; // subnetmask
    char                    szDefaultGateway[MAX_COMMON_STRING_16]; // default gateway
    NET_WEP_INFO            stuWEP;                             // WEP info
    NET_WPA_PSK_INFO        stuWPAPSK;                          // WPA/WPA2 PSK info
    NET_EAP_INFO            stuEAP;                             // WPA/WPA2 info
    NET_WPS_INFO            stuWPS;                             // WPS info
    NET_AP_FILTER           stuFilter;                          // access filter info
    BYTE                    byReserved[1024];                   // reserved bytes
}NET_AP_WORKPATTERN;

// Access Point name
typedef enum tagEM_ACCESSPOINT_NAME
{
    EM_ACCESSPOINT_NAME_UNKNOWN,                          // "Unknown"
    EM_ACCESSPOINT_NAME_ETH2,                             // "eth2" 
    EM_ACCESSPOINT_NAME_WALN0,                            // "wlan0" 
    EM_ACCESSPOINT_NAME_APRA0,                            // "apra0" 
}EM_ACCESSPOINT_NAME;

// AccessPoint info
typedef struct tagNET_ACCESSPOINT_INFO
{
    EM_ACCESSPOINT_NAME     emAccessPointName;                  // access point name    
    int                     nValidPatternNum;                   // valid pattern number
    NET_AP_WORKPATTERN      stuWorkPattern[MAX_WORKPATTERN_NUM];// work pattern configuration:2.4G(default)/5G
    BYTE                    byReserved[1024];                   // reserved bytes
}NET_ACCESSPOINT_INFO;

// WIFI server configuration(access point)
typedef struct tagNET_NETAPP_ACCESSPOINT
{
    DWORD                   dwSize;
    int                     nValidAPNum;                        // AccessPoints number
    NET_ACCESSPOINT_INFO    stuAccessPoints[NET_MAX_AP_NUM];    // Accesspoint info 
}NET_NETAPP_ACCESSPOINT;

// Railway record config
typedef struct tagNET_DEV_CAR_COACH_INFO
{
    DWORD                   dwSize;                         
    char                    szCarNo[MAX_CARNO_LEN];              // car no
    char                    szCoachNo[MAX_COACHNO_LEN];          // coach no
}NET_DEV_CAR_COACH_INFO;

// Snap format reference;
typedef enum tagEM_MEDIA_GLOBAL_SNAP_FORMAT_AS
{
	EM_MEDIA_GLOBAL_SNAP_FORMAT_AS_UNKNOWN,
	EM_MEDIA_GLOBAL_SNAP_FORMAT_AS_MAIN_FORMAT,                  // refer to main stream
	EM_MEDIA_GLOBAL_SNAP_FORMAT_AS_EXTRA_FORMAT,				 // refer to sub stream1
	EM_MEDIA_GLOBAL_SNAP_FORMAT_AS_EXTRA2_FORMAT,				 // refer to sub stream2
	EM_MEDIA_GLOBAL_SNAP_FORMAT_AS_EXTRA3_FORMAT,				 // refer to sub stream3
}EM_MEDIA_GLOBAL_SNAP_FORMAT_AS;

// Media global config
typedef struct tagNET_MEDIA_GLOBAL_INFO
{
	DWORD               dwSize;
	int                 nPacketType;                          // 0:by time,1:by size
	BYTE                byPacketLength;                       // Record packet time length. unit:minite. 1-255
	BYTE                byAlign[3];                           // align
	DWORD				dwPacketSize;                   	  // Record packet file size. unit:KB
	BOOL				bLogRecord;                           // Is wirte record's log
	BOOL				bLogEncode;                           // Is write abnormal encode's log
	EM_MEDIA_GLOBAL_SNAP_FORMAT_AS  emSnapFormatAs;           // Snap format reference; If device not support set snap format independently, snap format use reference
}NET_MEDIA_GLOBAL_INFO;

// NAS protocol type
typedef enum __EM_NAS_PROTOCOL
{
	NAS_PROTOCOL_UNKNOW = 0,
	NAS_PROTOCOL_FTP,			// FTP
	NAS_PROTOCOL_SMB,			// SMB
	NAS_PROTOCOL_NFS,			// NFS
	NAS_PROTOCOL_ISCSI,			// ISCSI
	NAS_PROTOCOL_CLOUD,			// CLOUD
}EM_NAS_PROTOCOL;

// NAS config
typedef struct tagNET_NAS_INFO
{
	DWORD               dwSize;
    BOOL                bEnable;									// Enable or not
	char				szName[MAX_COMMON_STRING_128];				// name
    char                szHostIp[DH_MAX_IPADDR_OR_DOMAIN_LEN];		// Host IP
    WORD                wHostPort;                          		// Host port 
	char				reserved[2];								// Reserved
    char                szDirName[DH_FTP_MAX_PATH];         		// FTP path
    char                szUserName[DH_FTP_USERNAME_LEN];    		// User name
    char                szPassword[DH_FTP_PASSWORD_LEN];    		// Password 
    int                 nInterval;                          		// Time interval between two near files.   	
	BOOL				bAnonymity;									// Anonymity enable or not, enable when byProtocol is 0
	EM_NAS_PROTOCOL     emProtocol;									// Protocol	
}NET_NAS_INFO;

// PPPoE config
typedef struct tagNET_PPPOE_INFO
{
	DWORD               dwSize; 
	BOOL				bEnable;								// Enable or not
    char                szUserName[MAX_USER_NAME_LEN];			// User name
	char				szPasswd[MAX_PWD_LEN];					// Password
}NET_PPPOE_INFO;

// Mail config
typedef struct tagNET_EMAIL_INFO
{
	DWORD               dwSize;		
	BOOL                bEnable;											// enable 0:false,    1:true
	char                szMailIPAddr[DH_MAX_DOMAIN_NAME_LEN];				// Mail server address(IP and domain name )   
    WORD                wMailPort;											// Mail server port     
	char				reserved[2];										// Reserved
    char                szSenderAddr[DH_MAX_MAIL_ADDR_LEN];					// From 
    char                szUserName[DH_MAX_MAIL_NAME_LEN];					// User name
    char                szUserPsw[DH_MAX_MAIL_NAME_LEN];					// Password
    char                szDestAddr[DH_MAX_MAIL_ADDR_LEN];					// To
    char                szSubject[DH_MAX_MAIL_SUBJECT_LEN];					// Subject
	char				*szReceivers[DH_MAX_MAIL_ADDR_LEN];					// Receiver list
	unsigned int		nReceiversNum;										// szReceivers num
	unsigned int		nReceiversNumRet;									// szReceivers num return
    BOOL                bSSLEnable;											// SSL enable
	BOOL				bTLSEnable;											// TLS enable
    WORD                wSendInterval;										// Send interval,[0,3600]s
    BYTE                bAnonymous;											// Anonymous Options[0,1], 0:FALSE,1:TRUE.
    BYTE                bAttachEnable;										// Attach Options[0,1], 0:FALSE,1:TRUE..
	BOOL				bHealthReport;										// Health report enable
	int					nHealthReportInterval;								// Health report interval
}NET_EMAIL_INFO;

// DDNS config
typedef struct tagNET_DDNS_INFO
{
	DWORD                dwSize;
    DWORD                dwDdnsServerNum; // DDNS server num
    DH_DDNS_SERVER_CFG   stuDdnsServer[DH_MAX_DDNS_NUM];
}NET_DDNS_INFO;

// Protocols manager config list
typedef struct tagNET_PROTOCOLS_MANAGER
{
	char				szDesc[DH_COMMON_STRING_128];       // Protocol description
	BOOL				bEnable;							// whether enbale or not
	char				szProtocol[DH_COMMON_STRING_128];   // protocol name,for example:DI_SMK
	char				szShowType[DH_COMMON_STRING_128];	// protocol type,for example:SmokingSensor
	BYTE				byReserved[256];					// Reserved
}NET_PROTOCOLS_MANAGER;

// Protocols manager config
typedef struct tagNET_SCADA_PROTOCOLS_MANAGER
{
	DWORD					dwSize;
	NET_PROTOCOLS_MANAGER*	pstProtocolsManager;		     // Protocols manager
	int						nProtocolsManagerNum;		     // num of ProtocolsManager
	int						nRetProtocolsManagerNum;         // Return actual num of ProtocolsManager
															 // When user get all Protocols managers at first,if nRetProtocolsManagerNum is greater than nProtocolsManagerNum,
															 // Then user get all Protocols managers at second,Suggest nProtocolsManagerNum is the same with nRetProtocolsManagerNum(Get by first).
}NET_SCADA_PROTOCOLS_MANAGER;

// device info config list
typedef struct tagNET_DEVICEIDSETS
{
	char			  szDevName[DH_COMMON_STRING_32];		// Device name
	char			  szDevType[DH_COMMON_STRING_32];		// Device type
	char			  szNewDeviceId[DH_COMMON_STRING_32];   // New device id config by user,the actual size of szNewDeviceId is the same as szOldDeviceId
	char			  szOldDeviceId[DH_COMMON_STRING_32];	// Default device id
	BYTE			  byReserved[128];						// Reserved
}NET_DEVICEIDSETS;

// device info config
typedef struct tagNET_SCADA_DEVICEINFO_CFG
{
	DWORD			    dwSize;
	char			    szCity[DH_COMMON_STRING_32];         // Current city
	char			    szProvince[DH_COMMON_STRING_32];     // Current province
	int				    nStation;							 // station type
	NET_DEVICEIDSETS*   pstDeviceIdSets;					 // Device info 
	int					nDeviceIdSetsNum;					 // num of DeviceIdSets
	int					nRetDeviceIdSetsNum;				 // Return num of DeviceIdSets
}NET_SCADA_DEVICEINFO_CFG;

// Link layer VPN type
typedef enum tagNET_EM_LINK_LAYER_VPN_TYPE
{
	NET_EM_TYPE_UNKNOWN = 0,     // unknown
	NET_EM_TYPE_PPTP,			 // VPN type:PPTP
	NET_EM_TYPE_L2TP,			 // VPN type:L2TP
}NET_EM_LINK_LAYER_VPN_TYPE;

// Link layer VPN config list
typedef struct tagNET_NETAPP_LINK_LAYER_VPN
{
	char						szName[MAX_LINK_NAME_LEN];				// link name
	NET_EM_LINK_LAYER_VPN_TYPE  emLinkLayerVPNType;						// link layer VPN type
	BOOL						bEnable;								// whether enbale or not
	char						szAddress[MAX_SERVER_ADDRESS_LEN];				// server IP
	char						szUserName[MAX_USER_NAME_LEN];			// user name
	char						szPassword[MAX_PWD_LEN];				// password
	BYTE						byReserved[1024];						// Reserved
}NET_NETAPP_LINK_LAYER_VPN;

// Link layer VPN config
typedef struct tagNET_NETAPP_LINK_LAYER_VPN_CFG
{
	DWORD			            dwSize;
	int							nRetLinkLayerVPNNum;					 // return num of Link layer VPN config
	NET_NETAPP_LINK_LAYER_VPN   stLinkLayerVPN[LINK_LAYER_VPN_NUM];   // Link layer VPN config array
}NET_NETAPP_LINK_LAYER_VPN_CFG;

// China tower config
typedef struct tagNET_VSP_CHINA_TOWER
{
	DWORD			dwSize;
	BOOL			bEnable;                              // whether enable or not,TRUE means on,FALSE means off
	char			szServerIP[MAX_SERVER_IP_LEN];     // Server IP
	int				nServerPort;						  // Server port,value range:0~65535
	int				nLoginInterval;						  // Login interval,Unit:second,value range:60~180
}NET_VSP_CHINA_TOWER;

// VTH password config(corresponding to NET_EM_CFG_VTH_PASSWORD)
typedef struct tagNET_CFG_VTH_PASSWORD_INFO
{
	DWORD		   dwSize;
	char		   szUser[8];					  // user password,must be 6 digits
}NET_CFG_VTH_PASSWORD_INFO;

// Sip server info
typedef struct tagNET_CFG_SIPSERVER_INFO
{
    DWORD                   dwSize; 
    char                    szAccoutName[MAX_COMMON_STRING_64];         // accout name
    char                    szAddres[DH_MAX_IPADDR_EX_LEN];             // address of sip server
    int                     nPort;                                      // port of sip server
    char                    szProxyAdress[DH_MAX_IPADDR_EX_LEN];        // address of proxy server
    char                    szProxyID[MAX_COMMON_STRING_64];            // id of proxy server
    int                     nProxyPort;                                 // port of proxy server
    char                    szUserID[MAX_COMMON_STRING_64];             // user id,it's usually cellphone number 
    int                     nUserType;                                  // user type,0:SIP client,2:vto is the server,3:VTNC is the server
    char                    szAuthID[MAX_USER_NAME_LEN];				// user authentication id
    char                    szAuthPassword[MAX_PWD_LEN];				// user authentication password
    char                    szSTUNServer[DH_MAX_IPADDR_EX_LEN];         // address of STUN server
    char                    szRegisterRealm[MAX_COMMON_STRING_64];      // register realm
    int                     nRegExpiration;                             // expireation of register,unit:S
    int                     nSipPort;                                   // sip port of local
    int                     nRTPPort;                                   // RTP port of local
    BOOL                    bUnregisterOnReboot;                        // enable of unregister onreboot
    char                    szDefaultCallNumber[MAX_COMMON_STRING_64];  // call number of default
    UINT                    nVideoStreamType;                           // video stream type,0main1extra12extra23extra3
    UINT                    nAudioStreamType;                           // audio stream type,0main1extra12extra23extra3
    BOOL                    bRouteEnable;                               // sip enable of cross route
    int                     nRouteCount;                                // route number
    char                    szRouteAddress[DH_MAX_IPADDR_EX_LEN][MAX_ROUTE_NUM];   // route address
    char                    szSIPServerLoginUserName[MAX_USER_NAME_LEN];// username of login VTNC
    char                    szSIPServerLoginPWD[MAX_PWD_LEN];			// password of login VTNC
    BOOL                    bMainVTO;                                   // Enable of be Main VTO
    char                    szRedundancyServer[DH_MAX_IPADDR_EX_LEN];   // address of redundancy sip server
    char                    szRedundancyUserName[MAX_USER_NAME_LEN];	// username of redundancy sip server
    char                    szRedundancyPassword[MAX_PWD_LEN];			// password of redundancy sip server
    char                    szAnalogNumberStart[MAX_COMMON_STRING_64];  // analog start number
    char                    szAnalogNumberEnd[MAX_COMMON_STRING_64];    // analog end number
	BOOL					bUserEnable;								// User Enable
}NET_CFG_SIPSERVER_INFO; 

//single ultrasonic configuration
typedef struct tagNET_CFG_SINGLE_ULTRASONIC
{
	BOOL      bEnable;                       //ultrasonic enable TRUE:yes FALSE:no
	UINT      nFreq;                         //ultrasonic frequency,unit(hz)
	int       nBurst;                        //burst
	int       nGain;                         //gain  value range [0,100]
	int       nDriveCurrent;                 //drive current,unit(mA)
	int       nMinValidDistance;             //min valid distance,unit(cm)
	int       nMaxValidDistance;             //max valid distance,unit(cm)
	int       nMinVaildSignalIntensity;      //min valid signal intensity
	int       nMaxVaildSignalIntensity;      //max valid signal intensity
	BYTE      byReserved[512];               //reserved bytes
}NET_CFG_SINGLE_ULTRASONIC;

//ultrasonic info
typedef struct tagNET_CFG_ULTRASONIC_INFO
{
	DWORD                      dwSize;
	int                        nMaxCount;         //max ultrasonic count 
	int                        nRetCount;         //return ultrasonic count
	NET_CFG_SINGLE_ULTRASONIC  *pstuULTRASONIC;   //ultrasonic info , new/malloc by user, delete/free by user
}NET_CFG_ULTRASONIC_INFO;

// AccessControl Delay strategy
typedef struct tagNET_CFG_ACCESS_DELAYSTRATEGY
{
    DWORD                   dwSize;
    BOOL                    bEnable;                                    // enable
    int                     nPreArrearsTerm;                            // Prepare Arrears TermunitDay
    int                     nArrearsTerm;                               // Arrears TermunitDay
}NET_CFG_ACCESS_DELAYSTRATEGY;

// server type
typedef enum tagNET_EM_REGISTAR_SERVER_TYPE
{
    NET_EM_REGISTAR_SERVER_TYPE_UNKNOWN,
    NET_EM_REGISTAR_SERVER_TYPE_VTO,
    NET_EM_REGISTAR_SERVER_TYPE_H500,
    NET_EM_REGISTAR_SERVER_TYPE_VTNC,
    NET_EM_REGISTAR_SERVER_TYPE_ZYCOO,
    NET_EM_REGISTAR_SERVER_TYPE_THIRDPARTY,
    NET_EM_REGISTAR_SERVER_TYPE_3CXSYSTEM,
    NET_EM_REGISTAR_SERVER_TYPE_ASTERISK,
}NET_EM_REGISTAR_SERVER_TYPE;

// Registar server info
typedef struct tagNET_CFG_REGISTAR_INFO
{
    DWORD                   dwSize;
    char                    szRegistarName[MAX_COMMON_STRING_64];    // name of registar serverexclusive
    NET_EM_REGISTAR_SERVER_TYPE emServerType;                        // type of registar server
    BOOL                    bRegistarEnable;                         // enable to registar
    char                    szAddress[DH_MAX_IPADDR_EX_LEN];         // registar address
    int                     nPort;                                   // registar port
	char                    szPassword[MAX_COMMON_STRING_64];  	     // registar password
}NET_CFG_REGISTAR_INFO;

//Front shield cover configuration(corresponding to NET_EM_CFG_AELENSMASK)
typedef struct tagNET_CFG_AELENSMASK_INFO
{
	DWORD		   dwSize;
	BOOL           bEnable;                       // enable(only be true can support open and close front shield cover)
	char           szName[MAX_COMMON_STRING_128]; // custom name
	BOOL           bStatus;                       // status(true:open  false:close)
}NET_CFG_AELENSMASK_INFO;

// recordextra config time schedule
typedef struct tagNET_CFG_RECORDEXTRA_INFO
{
	DWORD					dwSize;
	DH_TSECT                stuTimeSection[DH_N_SCHEDULE_TSECT][DH_N_REC_TSECT];            // The first dimension of the first 7 elements corresponds to 7 days a week,the eighth element corresponds to the holiday,Up to 6 time periods per day
}NET_CFG_RECORDEXTRA_INFO;
// Encode config
												// If it supports high definition setup, the high definition channel ranges from 0 to N-1. If your high definition channel number is more than N, the save operation may fail. 
// automatic arming schedule
typedef struct tagNET_CFG_ARMSCHEDULE_INFO
{
    DWORD					dwSize;
    BOOL                    bEnable;                                               // valid enable
    DH_TSECT                stuTimeSection[DH_N_SCHEDULE_TSECT][DH_N_REC_TSECT];   // The first dimension of the first 7 elements corresponds to 7 days a week,the eighth element corresponds to the holiday,Up to 6 time periods per day
}NET_CFG_ARMSCHEDULE_INFO;

// task of video diagmosis plan
typedef struct tagNET_PROJECT_TASK
{
	BOOL							bEnable;									// is it enable
	char							szTaskName[MAX_PATH];						// task name
	DH_TSECT                		stuTimeSection[DH_N_WEEKS][DH_N_REC_TSECT]; // time section of the task
	BOOL                            bIsCycle;                                   // is it cycle
	BYTE      						byReserved[512];      						// reserved bytes
}NET_PROJECT_TASK;

// plan of video diagmosis
typedef struct tagNET_VIDEODIAGNOSIS_PROJECT_INFO
{
	DWORD					dwSize;	
	char					szProjectName[MAX_PATH];		// project name
	int						nTotalTaskNum;					// User allocates total parameter number depending on capacity collection
	int						nReturnTaskNum;					// Returned real parameter number
	NET_PROJECT_TASK*		pstProjectTasks;				// User allocate nTotalProfileNum  count NET_PROJECT_TASK
} NET_VIDEODIAGNOSIS_PROJECT_INFO;


// Timeable information
typedef struct tagNET_CFG_TIME_SCHEDULE
{
	BOOL                    bEnableHoliday;                                                 // Whether to support the holiday configuration,default is not supported,returned TRUE if configuration is obtained, don't enable holiday configuration
	DH_TSECT                stuTimeSection[DH_N_SCHEDULE_TSECT][DH_N_REC_TSECT];            // The first dimension of the first 7 elements corresponds to 7 days a week,the eighth element corresponds to the holiday,Up to 6 time periods per day
} NET_CFG_TIME_SCHEDULE;

// Event title content structure
typedef struct tagNET_CFG_EVENT_TITLE
{
	char					szText[CFG_MAX_CHANNELNAME_LEN];
	DH_POINT				stuPoint;						// the upper left corner of the title, use 0-8191 relative coordinate system
	DH_SIZE     			stuSize;						// The width and height of the title,use 0-8191 relative coordinate system,one or two of them are 0 in accordance with the font adaptive width and height
	NET_COLOR_RGBA			stuFrontColor;					// Foreground color
	NET_COLOR_RGBA			stuBackColor;					// Background color
} NET_CFG_EVENT_TITLE;

// Mail attachment type
typedef enum tagNET_CFG_ATTACHMENT_TYPE
{
	NET_ATTACHMENT_TYPE_PIC,							// Picture attachments
	NET_ATTACHMENT_TYPE_VIDEO,							// video attachments
	NET_ATTACHMENT_TYPE_NUM,							// the total number of attachment of types
} NET_CFG_ATTACHMENT_TYPE;

// Mail details
typedef struct tagNET_CFG_MAIL_DETAIL
{
	NET_CFG_ATTACHMENT_TYPE		emAttachType;                 // Attachments type
	int							nMaxSize;                     // File cap sizeunit:kB
	int							nMaxTimeLength;               // Maximum recording time lengthunit:svalid for video
}NET_CFG_MAIL_DETAIL;

// Tour link configuration
typedef struct tagNET_CFG_TOURLINK
{
	BOOL					bEnable;									// Polling enabled
	DH_SPLIT_MODE	        emSplitMode;								// Split mode when polling
	int						nChannels[CFG_MAX_VIDEO_CHANNEL_NUM];		// Polling the list of channel numbers
	int						nChannelCount;								// Numbers of polling channels
} NET_CFG_TOURLINK;

// Access Control type
enum NET_EM_CFG_ACCESSCONTROLTYPE
{
	NET_EM_CFG_ACCESSCONTROLTYPE_NULL = 0,					// Not for operation
	NET_EM_CFG_ACCESSCONTROLTYPE_AUTO,						// Automaitc
	NET_EM_CFG_ACCESSCONTROLTYPE_OPEN,						// Open the door
	NET_EM_CFG_ACCESSCONTROLTYPE_CLOSE,						// Close the door
	NET_EM_CFG_ACCESSCONTROLTYPE_OPENALWAYS,				// Always open
	NET_EM_CFG_ACCESSCONTROLTYPE_CLOSEALWAYS,				// Always close
};

// Voice call originator
typedef enum tagNET_EM_CALLER_TYPE
{
	NET_EM_CALLER_DEVICE = 0,								// Device initiated
}NET_EM_CALLER_TYPE;

// Call protocol
typedef enum tagNET_EM_CALLER_PROTOCOL_TYPE
{
	NET_EM_CALLER_PROTOCOL_CELLULAR = 0,					// Mobile phone way
}NET_EM_CALLER_PROTOCOL_TYPE;

// Voice call linkage information
typedef struct tagNET_CFG_TALKBACK_INFO
{
	BOOL							bCallEnable;					// Voice call enable
	NET_EM_CALLER_TYPE				emCallerType;					// Voice call Originator
	NET_EM_CALLER_PROTOCOL_TYPE		emCallerProtocol;				// Voice call protocol
}NET_CFG_TALKBACK_INFO;

// Telephone alarm center linkage information
typedef struct tagNET_CFG_PSTN_ALARM_SERVER
{
	BOOL				bNeedReport;									// Whether to report to the telephone alarm center
	int					nServerCount;									// Number of telephone alarm servers					
	BYTE 				byDestination[CFG_MAX_PSTN_SERVER_NUM];			// Reported by the alarm center
}NET_CFG_PSTN_ALARM_SERVER;

typedef enum tagNET_EM_CFG_OPERATE_TYPE
{
    NET_EM_CFG_SNAP_MODE,                   // Snap Mode Config
    NET_EM_CFG_DEV_CAR_COACH,               // Railway record config,corresponding to struct NET_DEV_CAR_COACH_INFO
	NET_EM_CFG_YUEQING_SUPPLYLIGHTING,		// YueQing External lighting configuration, corresponding to struct NET_YUEQING_SUPPLYLIGHTING_INFO
	NET_EM_CFG_MEDIA_GLOBAL,                // media global config,corresponding to struct NET_MEDIA_GLOBAL_INFO

	/*********the configuration about OSD*************************************************************************************/
	NET_EM_CFG_CHANNELTITLE = 1000,         // Encode widget-channel title config, corresponding to struct NET_OSD_CHANNEL_TITLE, emOsdBlendType in struct must be set
	NET_EM_CFG_TIMETITLE,               	// Encode widget-Time title config, corresponding to NET_OSD_TIME_TITLE, emOsdBlendType in struct must be set
	NET_EM_CFG_CUSTOMTITLE,             	// Encode widget-Self-defined title config, corresponding to NET_OSD_CUSTOM_TITLE, emOsdBlendType  in struct must be set
	NET_EM_CFG_CUSTOMTITLETEXTALIGN,    	// Encode widget-Self-defined title alignment config, corresponding to NET_OSD_CUSTOM_TITLE_TEXT_ALIGN
	NET_EM_CFG_OSDCOMMINFO,             	// Encode widget-common info config, corresponding to NET_OSD_COMM_INFO
	NET_EM_CFG_OSD_PTZZOOM,					// Encode widget-PTZ zoom config, corresponding to NET_OSD_PTZZOOM_INFO
	NET_EM_CFG_GPSTITLE,					// Encode widget-GPS title,corresponding to NET_OSD_GPS_TITLE
	NET_EM_CFG_OSD_NUMBERSTATPLAN,			// Configuration of the statistical plane,which about number of people,  use this config when Class type is NumberStatPlan, correspinding to NET_OSD_NUMBER_PLAN

	/*********the configuration about Encode**********************************************************************************/
	NET_EM_CFG_ENCODE_VIDEO = 1100,			// Encode-video options config, corresponding to NET_ENCODE_VIDEO_INFO
	NET_EM_CFG_ENCODE_VIDEO_PACK,			// Encode-video pack options config, corresponding to NET_ENCODE_VIDEO_PACK_INFO
	NET_EM_CFG_ENCODE_VIDEO_SVC,			// Encode-video SVC options config, corresponding to NET_ENCODE_VIDEO_SVC_INFO
	NET_EM_CFG_ENCODE_VIDEO_PROFILE,		// Encode-video profile options config, corresponding to NET_ENCODE_VIDEO_PROFILE_INFO
	NET_EM_CFG_ENCODE_AUDIO_COMPRESSION,	// Encode-video audio compression options config, corresponding to NET_ENCODE_AUDIO_COMPRESSION_INFO
	NET_EM_CFG_ENCODE_AUDIO_INFO,			// Encode-video audio options config, corresponding to NET_ENCODE_AUDIO_INFO
	NET_EM_CFG_ENCODE_SNAP_INFO,			// Encode-video snap options config, corresponding to NET_ENCODE_SNAP_INFO
	NET_EM_CFG_ENCODE_SNAPTIME, 			// Encode-video snap time options config, corresponding to NET_ENCODE_SNAP_TIME_INFO
	NET_EM_CFG_ENCODE_CHANNELTITLE,			// Encode-video channel title options config, corresponding to NET_ENCODE_CHANNELTITLE_INFO
	
	/**********the configuration about audio**********************************************************************************/
	NET_EM_CFG_AUDIOIN_SOURCE = 1200,		// Audio-input audio source config, corresponding to NET_ENCODE_AUDIO_SOURCE_INFO
	NET_EM_CFG_AUDIOIN_DENOISE,				// Audio-denoise config, corresponding to NET_AUDIOIN_DENOISE_INFO
	NET_EM_CFG_AUDIOIN_VOLUME,				// Audio-volume of audio input, corresponding to NET_AUDIOIN_VOLUME_INFO
	NET_EM_CFG_AUDIOOUT_VOLUME,				// Audio-volume of audio output, corresponding to NET_AUDIOOUT_VOLUME_INFO
	
	/**********the configuration about video**********************************************************************************/
	NET_EM_CFG_VIDEOIN_SWITCHMODE = 1300,	// VideoIn-switch mode config, corresponding to NET_VIDEOIN_SWITCH_MODE_INFO
	NET_EM_CFG_VIDEOIN_COLOR,				// VideoIn-color options config, corresponding to NET_VIDEOIN_COLOR_INFO			
	NET_EM_CFG_VIDEOIN_IMAGE_OPT,			// VideoIn-image options config, corresponding to NET_VIDEOIN_IMAGE_INFO
	NET_EM_CFG_VIDEOIN_STABLE,				// VideoIn-stable config, corresponding to NET_VIDEOIN_STABLE_INFO
	NET_EM_CFG_VIDEOIN_IRISAUTO,			// VideoIn-auto iris config-, corresponding to NET_VIDEOIN_IRISAUTO_INFO
	NET_EM_CFG_VIDEOIN_IMAGEENHANCEMENT,	// VideoIn-image enhancement config, corresponding to NET_VIDEOIN_IMAGEENHANCEMENT_INFO
	NET_EM_CFG_VIDEOIN_EXPOSURE_NORMAL,		// VideoIn-normal exposure config, corresponding to NET_VIDEOIN_EXPOSURE_NORMAL_INFO
	NET_EM_CFG_VIDEOIN_EXPOSURE_OTHER,		// VideoIn-other exposure config, corresponding to NET_VIDEOIN_EXPOSURE_OTHER_INFO
	NET_EM_CFG_VIDEOIN_EXPOSURE_SHUTTER,	// VideoIn-exposure shutter config, corresponding to NET_VIDEOIN_EXPOSURE_SHUTTER_INFO
	NET_EM_CFG_VIDEOIN_BACKLIGHT,			// VideoIn-back light config, corresponding to NET_VIDEOIN_BACKLIGHT_INFO
	NET_EM_CFG_VIDEOIN_INTENSITY,			// VideoIn-Intensity config, corresponding to NET_VIDEOIN_INTENSITY_INFO
	NET_EM_CFG_VIDEOIN_LIGHTING,			// VideoIn-lighting config, corresponding to NET_VIDEOIN_LIGHTING_INFO
	NET_EM_CFG_VIDEOIN_DEFOG,				// VideoIn-defog config, corresponding to NET_VIDEOIN_DEFOG_INFO
	NET_EM_CFG_VIDEOIN_FOCUSMODE,			// VideoIn-focus mode config, corresponding to NET_VIDEOIN_FOCUSMODE_INFO
	NET_EM_CFG_VIDEOIN_FOCUSVALUE,			// VideoIn-focus options config, corresponding to NET_VIDEOIN_FOCUSVALUE_INFO
	NET_EM_CFG_VIDEOIN_WHITEBALANCE,		// VideoIn-white balance config, corresponding to NET_VIDEOIN_WHITEBALANCE_INFO
	NET_EM_CFG_VIDEOIN_DAYNIGHT,			// VideoIn-day night config, corresponding to NET_VIDEOIN_DAYNIGHT_INFO
	NET_EM_CFG_VIDEOIN_DAYNIGHT_ICR,		// VideoIn-ICR config, corresponding to NET_VIDEOIN_DAYNIGHT_ICR_INFO
	NET_EM_CFG_VIDEOIN_SHARPNESS,			// VideoIn-shrpness config, corresponding to NET_VIDEOIN_SHARPNESS_INFO
	NET_EM_CFG_VIDEOIN_COMM_DENOISE,		// VideoIn-comm denoise config, corresponding to NET_VIDEOIN_DENOISE_INFO
	NET_EM_CFG_VIDEOIN_3D_DENOISE,			// VideoIn-3D denoise config, corresponding to NET_VIDEOIN_3D_DENOISE_INFO

	/***********the configuration about case**********************************************************************************/
	NET_EM_CFG_ENCODE_PLAN = 1400,			// Case-the plan config of encode, corresponding to NET_ENCODE_PLAN_INFO

    /***********the configuration about alarm gateway ************************************************************************/
     NET_EM_CFG_ALARM_SOUND = 1500,         // alarm sound config of alarm gateway, corresponding to NET_ALARM_SOUND_INFO 
    
    /**********network application configuration***********************************************************************************************/
    NET_EM_CFG_ACCESS_POINT = 1600,         // WIFI server configuration(access point),corresponding to NET_NETAPP_ACCESSPOINT
	NET_EM_CFG_NAS			= 1700,			// NAS config, corresponding to NET_NAS_INFO 
	NET_EM_CFG_PPPOE,						// PPPOE configcorresponding to  NET_PPPOE_INFO  
	NET_EM_CFG_EMAIL,						// Email configcorresponding to  NET_EAMIL_INFO  
	NET_EM_CFG_DDNS,						// DDNS configcorresponding to  NET_DDNS_INFO 

	/**************SCADA configuration****************************************************************************************/
	NET_EM_CFG_SCADA_PROTOCOLS_MANAGER  = 1800,   	// Protocol manager config,corresponding to NET_SCADA_PROTOCOLS_MANAGER
	NET_EM_CFG_SCADA_DEVICEINFO_CFG,			  	// Device info config, corresponding to NET_SCADA_DEVICEINFO_CFG

	/**************NetApp configuration****************************************************************************************/
	NET_EM_CFG_NETAPP_LINK_LAYER_VPN = 1900,	  	// Link layer VPN config,correspongindg to NET_NETAPP_LINK_LAYER_VPN_CFG

	/**************China tower configuration****************************************************************************************/
	NET_EM_CFG_VSP_CHINA_TOWER = 2000,			  	// VSP china tower config,corresponding to NET_VSP_CHINA_TOWER 

	/**************intelligent configuration****************************************************************************************/
	NET_EM_CFG_STEREO_CALIBRATE = 2100,			  	// stero calibrate config, corresponding to NET_STEREO_CALIBRATE_INFO

	NET_EM_CFG_STEREO_CALIBRATEMATRIX_MULTISENSOR,	// Multi Sensor configuration, corresponding to NET_MULTI_SENSOR_INFO

	/**************radar configuration****************************************************************************************/
	NET_EM_CFG_RADAR            = 2200,				// radar configuration, corresponding to DEV_RADAR_CONFIG
	/**************VTH password configuration**************************************************************************************/
	NET_EM_CFG_VTH_PASSWORD		= 2300,				// VTH password configuration,corresponding to NET_CFG_VTH_PASSWORD_INFO
    NET_EM_CFG_REGISTAR         = 2301,             // registar server configuration,corresponding NET_CFG_REGISTAR_INFO
    NET_EM_CFG_SIP              = 2302,             // sip configuration, corresponding NET_CFG_SIPSERVER_INFO
	/**********Front shield cover configuration***********************************************************************************************/
	NET_EM_CFG_AELENSMASK       = 2400,             //Front shield cover configuration,corresponding to NET_CFG_AELENSMASK_INFO

	NET_EM_CFG_ULTRASONIC       = 2500,     		//ultrasonic configuration,corresponding to NET_CFG_ULTRASONIC_INFO 
    /**********ARC Configuration***********************************************************************************************/
    NET_EM_CFG_ARMSCHEDULE      = 2600,     		// automatic arming schedule configuration,corresponding to NET_CFG_ARMSCHEDULE_INFO
	/**********record snap configuration*******************************************************************************************/
    NET_EM_CFG_RECORDEXTRA		= 3610,				// RecodeExtra config, corresponding to NET_CFG_RECORDEXTRA_INFO
	
	/**********video diagnosis configuration*******************************************************************************************/
	NET_EM_VIDEODIAGNOSIS_PROJECT = 3700,			// video diagnosis plan config, corresponding to NET_VIDEODIAGNOSIS_PROJECT_INFO
} NET_EM_CFG_OPERATE_TYPE;


//////////////////////////////////////////////////////////////////////////
// RPC method name
//////////////////////////////////////////////////////////////////////////

#define RPC_METHOD_MONITORWALL_ADD      "monitorWallManager.addMonitorWall"     // add MonitorWall

/***********************************************************************
 ** Callback Function Definition 
 ***********************************************************************/

// Network disconnection callback function original shape 
typedef void (CALLBACK *fDisConnect)(LLONG lLoginID, char *pchDVRIP, LONG nDVRPort, LDWORD dwUser);

// network re-connection callback function original shape
typedef void (CALLBACK *fHaveReConnect)(LLONG lLoginID, char *pchDVRIP, LONG nDVRPort, LDWORD dwUser);

// The prototype of dynamic sub connection offline call function 
typedef void (CALLBACK *fSubDisConnect)(EM_INTERFACE_TYPE emInterfaceType, BOOL bOnline, LLONG lOperateHandle, LLONG lLoginID, LDWORD dwUser);

// monitor disconnect callback function
typedef void (CALLBACK *fRealPlayDisConnect)(LLONG lOperateHandle, EM_REALPLAY_DISCONNECT_EVENT_TYPE dwEventType, void* param, LDWORD dwUser);

// Real-time monitor data callback function original shape
// Obtain corresponding stream by filtering dwDataType, stream type refers to EM_REALDATA_FLAG
// The dwDataType value in stream transcoding refers to NET_DATA_CALL_BACK_VALUE
typedef void (CALLBACK *fRealDataCallBack)(LLONG lRealHandle, DWORD dwDataType, BYTE *pBuffer, DWORD dwBufSize, LDWORD dwUser);

// Real-time monitor data callback function original shape---extensive
// Obtain corresponding stream by filtering dwDataType, stream type refers to EM_REALDATA_FLAG
// The dwDataType value in stream transcoding refers to NET_DATA_CALL_BACK_VALUE
typedef void (CALLBACK *fRealDataCallBackEx)(LLONG lRealHandle, DWORD dwDataType, BYTE *pBuffer, DWORD dwBufSize, LONG param, LDWORD dwUser);


// Obtain corresponding stream by filtering dwDataType, stream type refers to EM_REALDATA_FLAG
// The dwDataType value in stream transcoding refers to NET_DATA_CALL_BACK_VALUE
typedef void (CALLBACK *fRealDataCallBackEx2)(LLONG lRealHandle, DWORD dwDataType, BYTE *pBuffer, DWORD dwBufSize, LLONG param, LDWORD dwUser);
// OSD callback function original shape 
typedef void (CALLBACK *fDrawCallBack)(LLONG lLoginID, LLONG lPlayHandle, HDC hDC, LDWORD dwUser);

// Playback process by time callback function original shape
typedef void (CALLBACK *fTimeDownLoadPosCallBack) (LLONG lPlayHandle, DWORD dwTotalSize, DWORD dwDownLoadSize, int index, NET_RECORDFILE_INFO recordfileinfo, LDWORD dwUser);

// Alarm message callback function original shape
typedef BOOL (CALLBACK *fMessCallBack)(LONG lCommand, LLONG lLoginID, char *pBuf, DWORD dwBufLen, char *pchDVRIP, LONG nDVRPort, LDWORD dwUser);
// explanation of new parameter: 
// bAlarmAckFlag : TRUE,the event is affirmable event;FALSE,the event is not affirmable event
// nEventID is used by CLIENT_AlarmAck, when bAlarmAckFlag is TRUE, this data is efficient
// pBuf memory was managed by SDK 
typedef BOOL (CALLBACK *fMessCallBackEx1)(LONG lCommand, LLONG lLoginID, char *pBuf, DWORD dwBufLen, char *pchDVRIP, LONG nDVRPort, BOOL bAlarmAckFlag, LONG nEventID, LDWORD dwUser);

// Listening server callback function original shape
typedef int (CALLBACK *fServiceCallBack)(LLONG lHandle, char *pIp, WORD wPort, LONG lCommand, void *pParam, DWORD dwParamLen, LDWORD dwUserData);

// Audio data of audio talk callback function original shape 
typedef void (CALLBACK *pfAudioDataCallBack)(LLONG lTalkHandle, char *pDataBuf, DWORD dwBufSize, BYTE byAudioFlag, LDWORD dwUser);

// Upgrade device callback function original shape
typedef void (CALLBACK *fUpgradeCallBack) (LLONG lLoginID, LLONG lUpgradechannel, int nTotalSize, int nSendSize, LDWORD dwUser);

//Upgrade device callback function original shape, support large file upgrade for example G bytes file.
typedef void (CALLBACK *fUpgradeCallBackEx) (LLONG lLoginID, LLONG lUpgradechannel, INT64 nTotalSize, INT64 nSendSize, LDWORD dwUser);

// Transparent COM callback function original shape
typedef void (CALLBACK *fTransComCallBack) (LLONG lLoginID, LLONG lTransComChannel, char *pBuffer, DWORD dwBufSize, LDWORD dwUser);

// Search device log data callback function original shape 
typedef void (CALLBACK *fLogDataCallBack)(LLONG lLoginID, char *pBuffer, DWORD dwBufSize, DWORD nTotalSize, BOOL bEnd, LDWORD dwUser);

// Snapshot callback function original shape 
// Encode Type 10: jpeg 0: number i frame of mpeg4 
typedef void (CALLBACK *fSnapRev)(LLONG lLoginID, BYTE *pBuf, UINT RevLen, UINT EncodeType, DWORD CmdSerial, LDWORD dwUser);

// GPS message subscription callback 
typedef void( CALLBACK *fGPSRev)(LLONG lLoginID, GPS_Info GpsInfo, LDWORD dwUserData);

// GPSGPS message subscription callback - extra
typedef void (CALLBACK *fGPSRevEx)(LLONG lLoginID, GPS_Info GpsInfo, ALARM_STATE_INFO stAlarmInfo, LDWORD dwUserData, void *reserved);

// GPSGPS message subscription callback - extra2
typedef void (CALLBACK *fGPSRevEx2)(LLONG lLoginID, const LPNET_GPS_LOCATION_INFO lpData, LDWORD dwUserData, void *reserved);

// GPS GPS subscription of temperature and humidity callback
typedef void (CALLBACK *fGPSTempHumidityRev)(LLONG lLoginID, GPS_TEMP_HUMIDITY_INFO GpsTHInfo, LDWORD dwUserData);

// Asynchronous data callback
typedef void (CALLBACK *fMessDataCallBack)(LLONG lCommand, LPNET_CALLBACK_DATA lpData, LDWORD dwUser);

// File Transfer callback
typedef void (CALLBACK *fTransFileCallBack)(LLONG lHandle, int nTransType, int nState, int nSendSize, int nTotalSize, LDWORD dwUser);

// intelligent analysis data callback;nSequence instruct the repeat picture's station,0 instruct the first time it appear, 2 instruct the last time it appear or it only appear once,1 instruct it will appear next time
// int nState = (int) reserved instruct current callback data's state, 0 means realtime data,1 means offline data,2 means send offline data over
typedef int  (CALLBACK *fAnalyzerDataCallBack)(LLONG lAnalyzerHandle, DWORD dwAlarmType, void* pAlarmInfo, BYTE *pBuffer, DWORD dwBufSize, LDWORD dwUser, int nSequence, void *reserved);

// Asynchronism search device call
typedef void (CALLBACK *fSearchDevicesCB)(DEVICE_NET_INFO_EX *pDevNetInfo, void* pUserData);

// Asynchronism register device call
// typedef void (CALLBACK *fHaveLogin)(LLONG lLoginID, char *pchDVRIP, LONG nDVRPort, BOOL bOnline, NET_DEVICEINFO_Ex stuDeviceInfo, int nError, LDWORD dwUser, void *reserved);

// playback YUV callback function
typedef void (CALLBACK *fYUVDataCallBack)(LLONG lPlaybackHandle, BYTE *pBuffer, DWORD dwBufSize, LONG param, LDWORD dwUser, void *reserved);

/************************************************************************
 ** Interface Definition 
 ***********************************************************************/

// SDK Initialization 
CLIENT_NET_API BOOL CALL_METHOD CLIENT_Init(fDisConnect cbDisConnect, LDWORD dwUser);

// SDK Initialization, specify initialization parameter through struct NETSDK_INIT_PARAM,
// e.g., netsdk's normal network process thread number.
// equivalent to CLIENT_Init when lpInitParam is NULL
CLIENT_NET_API BOOL CALL_METHOD CLIENT_InitEx(fDisConnect cbDisConnect, LDWORD dwUser, LPNETSDK_INIT_PARAM lpInitParam = NULL);

// SDK exit and clear
CLIENT_NET_API void CALL_METHOD CLIENT_Cleanup();

//------------------------------------------------------------------------

// Set re-connection callback function after disconnection. Internal SDK  auto connect again after disconnection 
CLIENT_NET_API void CALL_METHOD CLIENT_SetAutoReconnect(fHaveReConnect cbAutoConnect, LDWORD dwUser);

// Dynamic sub-set link disconnected callback function, the current monitoring and playback equipment SVR is a short connection
CLIENT_NET_API void CALL_METHOD CLIENT_SetSubconnCallBack(fSubDisConnect cbSubDisConnect, LDWORD dwUser);

// Return the function execution failure code
CLIENT_NET_API DWORD CALL_METHOD CLIENT_GetLastError(void);

// Set device connection timeout value and trial times 
CLIENT_NET_API void CALL_METHOD CLIENT_SetConnectTime(int nWaitTime, int nTryTimes);

// Set log in network environment 
CLIENT_NET_API void CALL_METHOD CLIENT_SetNetworkParam(NET_PARAM *pNetParam);

// Get SDK version information 
CLIENT_NET_API DWORD CALL_METHOD CLIENT_GetSDKVersion();

//------------------------------------------------------------------------

// Register to the device 
CLIENT_NET_API LLONG CALL_METHOD CLIENT_Login(const char *pchDVRIP, WORD wDVRPort, const char *pchUserName, const char *pchPassword, LPNET_DEVICEINFO lpDeviceInfo, int *error = 0);

// Expension interface nSpecCap =0 is login under TCP mode, void* pCapParam fill in NULL
//			           nSpecCap =2 is login with active registeration, void* pCapParam fill in NULL
//					   nSpecCap =3 is login under multicast mode, void* pCapParam fill in NULL
//			           nSpecCap =4 is login under UDP mode, void* pCapParam fill in NULL
//					   nSpecCap =6 is login with main connection,void* pCapParam fill in NULL
//					   nSpecCap =7 is SSL encription,void* pCapParam fill in NULL
//			           nSpecCap =9 is login in to remote device, enter void* pCapParam as remote device name
//					   nSpecCap = 12 is login under LDAP mode,void* pCapParam fill in NULL
//                     nSpecCap = 13 is login under AD mode,void* pCapParam fill in NULL
//                     nSpecCap = 14 is Radius login,void* pCapParam fill in NULL 
//                     nSpecCap = 15 is Socks5 login, now void* pCapParam fill in Socks5 server IP&&port&&ServerName&&ServerPassword string
//                     nSpecCap = 16 is proxy login, now void* pCapParam fill in SOCKET value
//                     nSpecCap = 19 is web privatepenestrating login,void* pCapParam fill in NULL
//                     nSpecCap = 20 mobile phone client login,void* pCapParam fill in NULL
CLIENT_NET_API LLONG CALL_METHOD CLIENT_LoginEx(const char *pchDVRIP, WORD wDVRPort, const char *pchUserName, const char *pchPassword, int nSpecCap, void* pCapParam, LPNET_DEVICEINFO lpDeviceInfo, int *error = 0);

// Login extension interface
CLIENT_NET_API LLONG CALL_METHOD CLIENT_LoginEx2(const char *pchDVRIP, WORD wDVRPort, const char *pchUserName, const char *pchPassword, EM_LOGIN_SPAC_CAP_TYPE emSpecCap, void* pCapParam, LPNET_DEVICEINFO_Ex lpDeviceInfo, int *error = 0);

// Asynchronism login device 
// nSpecCap = 0 TCP login,nSpecCap = 6 only main connect login
// nSpecCap = 19 is private penestrating login
CLIENT_NET_API LLONG CALL_METHOD CLIENT_StartLogin(char *pchDVRIP, WORD wDVRPort, char *pchUserName, char *pchPassword, int nSpecCap, void* pCapParam, fHaveLogin cbLogin, LDWORD dwUser);

// Only when CLIENT_StartLogin () callback errors for 11 when asynchronous log in using the interface to continue
CLIENT_NET_API LLONG CALL_METHOD CLIENT_StartLoginEx(NET_IN_STARTLOGINEX* pInParam, NET_OUT_STARTLOGINEX* pOutParam);

// stop login
CLIENT_NET_API BOOL CALL_METHOD CLIENT_StopLogin(LLONG lLoginID);

CLIENT_NET_API BOOL CALL_METHOD CLIENT_StopLoginEx(LLONG lLoginID);

// Log out the device 
CLIENT_NET_API BOOL CALL_METHOD CLIENT_Logout(LLONG lLoginID);


//------------------------------------------------------------------------

// get the protocol type from device
CLIENT_NET_API BOOL CALL_METHOD CLIENT_GetDevProtocolType(LLONG lLoginID, EM_DEV_PROTOCOL_TYPE *pemProtocolType);

//------------------------------------------------------------------------

// Begin real-time monitor 
CLIENT_NET_API LLONG CALL_METHOD CLIENT_RealPlay(LLONG lLoginID, int nChannelID, HWND hWnd);

// Begin real-time monitor--extensive
CLIENT_NET_API LLONG CALL_METHOD CLIENT_RealPlayEx(LLONG lLoginID, int nChannelID, HWND hWnd, DH_RealPlayType rType = DH_RType_Realplay);

//start real-time monitor
CLIENT_NET_API LLONG CALL_METHOD CLIENT_StartRealPlay(LLONG lLoginID, int nChannelID, HWND hWnd, DH_RealPlayType rType, fRealDataCallBackEx cbRealData, fRealPlayDisConnect cbDisconnect, LDWORD dwUser, DWORD dwWaitTime = 10000);

// Multiple-window preview 
CLIENT_NET_API LLONG CALL_METHOD CLIENT_MultiPlay(LLONG lLoginID, HWND hWnd);

// Stop multiple-window preview 
CLIENT_NET_API BOOL CALL_METHOD CLIENT_StopMultiPlay(LLONG lMultiHandle);

// Snapshot;hPlayHandle is monitor or playback handle
CLIENT_NET_API BOOL CALL_METHOD CLIENT_CapturePicture(LLONG hPlayHandle, const char *pchPicFileName);
CLIENT_NET_API BOOL CALL_METHOD CLIENT_CapturePictureEx(LLONG hPlayHandle, const char *pchPicFileName, NET_CAPTURE_FORMATS eFormat);

// Set real-time monitor data callback 
CLIENT_NET_API BOOL CALL_METHOD CLIENT_SetRealDataCallBack(LLONG lRealHandle, fRealDataCallBack cbRealData, LDWORD dwUser);

// real data flag, corresponding param dwFlag in CLIENT_SetRealDataCallBackEx
// supports '|' operator, like dwFlag = REALDATA_FLAG_RAW_DATA | REALDATA_FLAG_YUV_DATA
typedef enum tagEM_REALDATA_FLAG
{	
	REALDATA_FLAG_RAW_DATA				= 0x01, // raw data flag,		corresponding param dwDataType in fRealDataCallBack(Ex/Ex2) is 0, 0x01 = 0x01 << 0
	REALDATA_FLAG_DATA_WITH_FRAME_INFO	= 0x02, // data with frame info flag,	corresponding param dwDataType in fRealDataCallBack(Ex/Ex2) is 1, 0x02 = 0x01 << 1
	REALDATA_FLAG_YUV_DATA				= 0x04, // YUV data flag,		corresponding param dwDataType in fRealDataCallBack(Ex/Ex2) is 2, 0x04 = 0x01 << 2
	REALDATA_FLAG_PCM_AUDIO_DATA		= 0x08, // PCM audio data flag,	corresponding param dwDataType in fRealDataCallBack(Ex/Ex2) is 3, 0x08 = 0x01 << 3
} EM_REALDATA_FLAG;

// Set real-time monitor data callback--extensive 
CLIENT_NET_API BOOL CALL_METHOD CLIENT_SetRealDataCallBackEx(LLONG lRealHandle, fRealDataCallBackEx cbRealData, LDWORD dwUser, DWORD dwFlag);

// Set real-time monitor data callback--extensive 2
CLIENT_NET_API BOOL CALL_METHOD CLIENT_SetRealDataCallBackEx2(LLONG lRealHandle, fRealDataCallBackEx2 cbRealData, LDWORD dwUser, DWORD dwFlag);
// Set video fluency
CLIENT_NET_API BOOL	CALL_METHOD CLIENT_AdjustFluency(LLONG lRealHandle, int nLevel);

// Save data as file
CLIENT_NET_API BOOL CALL_METHOD CLIENT_SaveRealData(LLONG lRealHandle, const char *pchFileName);

// Stop saving data as file 
CLIENT_NET_API BOOL CALL_METHOD CLIENT_StopSaveRealData(LLONG lRealHandle);

// Set video property
CLIENT_NET_API BOOL CALL_METHOD CLIENT_ClientSetVideoEffect(LLONG lPlayHandle, unsigned char nBrightness, unsigned char nContrast, unsigned char nHue, unsigned char nSaturation);

// Get video property 
CLIENT_NET_API BOOL CALL_METHOD CLIENT_ClientGetVideoEffect(LLONG lPlayHandle, unsigned char *nBrightness, unsigned char *nContrast, unsigned char *nHue, unsigned char *nSaturation);

// Set screen overlay callback 
CLIENT_NET_API void CALL_METHOD CLIENT_RigisterDrawFun(fDrawCallBack cbDraw, LDWORD dwUser);

// Open audio 
CLIENT_NET_API BOOL CALL_METHOD CLIENT_OpenSound(LLONG hPlayHandle);

// Set volume;lPlayHandle is monitor handle or playabck handle 
CLIENT_NET_API BOOL CALL_METHOD CLIENT_SetVolume(LLONG lPlayHandle, int nVolume);

// If use HD image internal adjustment strategy, enable by default. This strategy is used, when quick play x4 and higher, only play 1 frame, When disable, play every frame. 
CLIENT_NET_API BOOL CALL_METHOD CLIENT_PlayEnableLargePicAdjustment(LLONG lPlayHandle, BOOL bEnable);

// Stop audio 
CLIENT_NET_API BOOL CALL_METHOD CLIENT_CloseSound();

// Stop real-time preview 
CLIENT_NET_API BOOL CALL_METHOD CLIENT_StopRealPlay(LLONG lRealHandle);

// stop real-time preview-extensive
CLIENT_NET_API BOOL CALL_METHOD CLIENT_StopRealPlayEx(LLONG lRealHandle);

//------------------------------------------------------------------------

// general PTZ control
CLIENT_NET_API BOOL CALL_METHOD CLIENT_PTZControl(LLONG lLoginID, int nChannelID, DWORD dwPTZCommand, DWORD dwStep, BOOL dwStop);

// Private PTZ control,user malloc param4's memory,please refer to corresponding structure of DH_EXTPTZ_ControlType
CLIENT_NET_API BOOL CALL_METHOD CLIENT_DHPTZControl(LLONG lLoginID, int nChannelID, DWORD dwPTZCommand, unsigned char param1, unsigned char param2, unsigned char param3, BOOL dwStop,void* param4=NULL);

//  private PTZ control extensive port. support 3D intelligent position
CLIENT_NET_API BOOL CALL_METHOD CLIENT_DHPTZControlEx(LLONG lLoginID, int nChannelID, DWORD dwPTZCommand, LONG lParam1, LONG lParam2, LONG lParam3, BOOL dwStop);

// private PTZ control expansion port , support 3D quick positioning, Fish eye ,user malloc param4's memory,please refer to corresponding structure of DH_EXTPTZ_ControlType
CLIENT_NET_API BOOL CALL_METHOD CLIENT_DHPTZControlEx2(LLONG lLoginID, int nChannelID, DWORD dwPTZCommand, LONG lParam1, LONG lParam2, LONG lParam3, BOOL dwStop , void* param4 = NULL);

// Area Scan interface,private PTZ control
CLIENT_NET_API BOOL CALL_METHOD CLIENT_DHPTZAreaScan(LLONG lLoginID ,DWORD dwPTZCommand, char* szInBuffer, DWORD dwInBufferSize,char* szOutBuffer = NULL, DWORD dwOutBufferSize = 0, int waittime=1000);

// CLIENT_DHPTZCalibrateSingleDirection inparam
typedef struct tagNET_IN_CALIBRATE_SINGLEDIRECTION
{
    DWORD                       dwSize;
    int                         nMode;                      // calibrate1level2vertical
}NET_IN_CALIBRATE_SINGLEDIRECTION;
//------------------------------------------------------------------------
// query record state everyday in the month
CLIENT_NET_API BOOL CALL_METHOD CLIENT_QueryRecordStatus(LLONG lLoginID, int nChannelId, int nRecordFileType, LPNET_TIME tmMonth, char* pchCardid, LPNET_RECORD_STATUS pRecordStatus, int waittime=1000);

// Search whether there is recorded file in specified period
CLIENT_NET_API BOOL CALL_METHOD CLIENT_QueryRecordTime(LLONG lLoginID, int nChannelId, int nRecordFileType, LPNET_TIME tmStart, LPNET_TIME tmEnd, char* pchCardid, BOOL *bResult, int waittime=1000);

// Search all recorded file sin the specified periods
CLIENT_NET_API BOOL CALL_METHOD CLIENT_QueryRecordFile(LLONG lLoginID, int nChannelId, int nRecordFileType, LPNET_TIME tmStart, LPNET_TIME tmEnd, char* pchCardid, LPNET_RECORDFILE_INFO nriFileinfo, int maxlen, int *filecount, int waittime=1000, BOOL bTime = FALSE);

// Async search period all record files
CLIENT_NET_API BOOL CALL_METHOD CLIENT_StartQueryRecordFile(LLONG lLoginID, NET_IN_START_QUERY_RECORDFILE *pInParam, NET_OUT_START_QUERY_RECORDFILE *pOutParam);

// search first 16 records
CLIENT_NET_API BOOL CALL_METHOD CLIENT_QuickQueryRecordFile(LLONG lLoginID, int nChannelId, int nRecordFileType, LPNET_TIME tmStart, LPNET_TIME tmEnd, char* pchCardid, LPNET_RECORDFILE_INFO nriFileinfo, int maxlen, int *filecount, int waittime=1000, BOOL bTime = FALSE);


// Query the first record time 
CLIENT_NET_API BOOL CALL_METHOD CLIENT_QueryFurthestRecordTime(LLONG lLoginID, int nRecordFileType, char *pchCardid, NET_FURTHEST_RECORD_TIME* pFurthrestTime, int nWaitTime);

// Begin searching recorded file
CLIENT_NET_API LLONG	CALL_METHOD CLIENT_FindFile(LLONG lLoginID, int nChannelId, int nRecordFileType, char* cardid, LPNET_TIME time_start, LPNET_TIME time_end, BOOL bTime, int waittime);

// Search recorded file 
CLIENT_NET_API int	CALL_METHOD CLIENT_FindNextFile(LLONG lFindHandle,LPNET_RECORDFILE_INFO lpFindData);

// Stop searching recorded file 
CLIENT_NET_API BOOL CALL_METHOD CLIENT_FindClose(LLONG lFindHandle);

// Playback in file 
CLIENT_NET_API LLONG CALL_METHOD CLIENT_PlayBackByRecordFile(LLONG lLoginID, LPNET_RECORDFILE_INFO lpRecordFile, HWND hWnd, fDownLoadPosCallBack cbDownLoadPos, LDWORD dwUserData);

// Playback in file--extensive
CLIENT_NET_API LLONG CALL_METHOD CLIENT_PlayBackByRecordFileEx(LLONG lLoginID, LPNET_RECORDFILE_INFO lpRecordFile, HWND hWnd, fDownLoadPosCallBack cbDownLoadPos, LDWORD dwPosUser, fDataCallBack fDownLoadDataCallBack, LDWORD dwDataUser);

// playback in file - disconnect callback
CLIENT_NET_API LLONG CALL_METHOD CLIENT_StartPlayBackByRecordFile(LLONG lLoginID,  LPNET_RECORDFILE_INFO lpRecordFile, HWND hWnd, 
															 fDownLoadPosCallBack cbDownLoadPos, LDWORD dwPosUser, 
															 fDataCallBack fDownLoadDataCallBack, LDWORD dwDataUser, 
															 fRealPlayDisConnect fDisConnectCallBack, LDWORD dwDisUser, DWORD dwWaitTime = 10000);

//Playback in file--Drop frame playback (not enough bandwidth can be used)
CLIENT_NET_API LLONG CALL_METHOD CLIENT_FramCotrolPlayBackByRecordFile(LLONG lLoginID, LPNET_RECORDFILE_INFO lpRecordFile, HWND hWnd, 
																   fDownLoadPosCallBack cbDownLoadPos, LDWORD dwPosUser, 
																   fDataCallBack fDownLoadDataCallBack, LDWORD dwDataUser, unsigned int nCutFrameRate);

// Playback in time
CLIENT_NET_API LLONG CALL_METHOD CLIENT_PlayBackByTime(LLONG lLoginID, int nChannelID, LPNET_TIME lpStartTime, LPNET_TIME lpStopTime, HWND hWnd, fDownLoadPosCallBack cbDownLoadPos, LDWORD dwPosUser);

// Playback in time--extensive
CLIENT_NET_API LLONG CALL_METHOD CLIENT_PlayBackByTimeEx(LLONG lLoginID, int nChannelID, LPNET_TIME lpStartTime, LPNET_TIME lpStopTime, HWND hWnd, 
													 fDownLoadPosCallBack cbDownLoadPos, LDWORD dwPosUser, 
													 fDataCallBack fDownLoadDataCallBack, LDWORD dwDataUser);

// Playback by time, support playback by direction compare to the old version,user malloc memory of pstNetIn and pstNetOut
CLIENT_NET_API LLONG CALL_METHOD CLIENT_PlayBackByTimeEx2(LLONG lLoginID, int nChannelID, 
                        NET_IN_PLAY_BACK_BY_TIME_INFO *pstNetIn, NET_OUT_PLAY_BACK_BY_TIME_INFO *pstNetOut);

// playback in time--disconnect callback
CLIENT_NET_API LLONG CALL_METHOD CLIENT_StartPlayBackByTime(LLONG lLoginID, int nChannelID, 
													   LPNET_TIME lpStartTime, LPNET_TIME lpStopTime, HWND hWnd, 
													   fDownLoadPosCallBack cbDownLoadPos, LDWORD dwPosUser,
													   fDataCallBack fDownLoadDataCallBack, LDWORD dwDataUser, 
													   fRealPlayDisConnect fDisConnectCallBack, LDWORD dwDisUser, DWORD dwWaitTime = 10000);

//Playback in time--Drop frame playback (not enough bandwidth can be used)
CLIENT_NET_API LLONG CALL_METHOD CLIENT_FramCotrolPlayBackByTime(LLONG lLoginID, int nChannelID, LPNET_TIME lpStartTime, LPNET_TIME lpStopTime, HWND hWnd, 
															 fDownLoadPosCallBack cbDownLoadPos, LDWORD dwPosUser, 
															 fDataCallBack fDownLoadDataCallBack, LDWORD dwDataUser, unsigned int nCutFrameRate);

//playback by synopsis file
CLIENT_NET_API LLONG CALL_METHOD CLIENT_PlayBackBySynopsisFile(LLONG lLoginID, 
															LPNET_SYNOPSISFILE_INFO lpRecordFile, 
															HWND hWnd, 
															fDownLoadPosCallBack cbDownLoadPos,
															LDWORD dwPosUser, 
															fDataCallBack fDownLoadDataCallBack,
															LDWORD dwDataUser,
															LDWORD dwUser);

// multi-channel preview palyabck
CLIENT_NET_API LLONG CALL_METHOD CLIENT_MultiPlayBack(LLONG lLoginID, NET_MULTI_PLAYBACK_PARAM *pParam);

// position record playback initial point
CLIENT_NET_API BOOL CALL_METHOD CLIENT_SeekPlayBack(LLONG lPlayHandle, unsigned int offsettime, unsigned int offsetbyte);

// Pause or restore file playback 
CLIENT_NET_API BOOL CALL_METHOD CLIENT_PausePlayBack(LLONG lPlayHandle, BOOL bPause);

// Fast playback file 
CLIENT_NET_API BOOL CALL_METHOD CLIENT_FastPlayBack(LLONG lPlayHandle);

// Slow playback file 
CLIENT_NET_API BOOL CALL_METHOD CLIENT_SlowPlayBack(LLONG lPlayHandle);

// Set playback speed
CLIENT_NET_API BOOL CALL_METHOD CLIENT_SetPlayBackSpeed(LLONG lPlayHandle, EM_PLAY_BACK_SPEED emSpeed);

// Step playback file 
CLIENT_NET_API BOOL CALL_METHOD CLIENT_StepPlayBack(LLONG lPlayHandle, BOOL bStop);

// Control playback direction--Forward or Backward playback
// bBackward FALSE:normal play TRUE:upend
CLIENT_NET_API BOOL CALL_METHOD CLIENT_PlayBackControlDirection(LLONG lPlayHandle, BOOL bBackward);

// Set playback frame rate 
CLIENT_NET_API BOOL CALL_METHOD CLIENT_SetFramePlayBack(LLONG lPlayHandle, int framerate);

// Get playback frame rate
CLIENT_NET_API BOOL CALL_METHOD CLIENT_GetFramePlayBack(LLONG lPlayHandle, int *fileframerate, int *playframerate);

// Restore ordinary playback
CLIENT_NET_API BOOL CALL_METHOD CLIENT_NormalPlayBack(LLONG lPlayHandle);

// smart search play back
CLIENT_NET_API BOOL CALL_METHOD CLIENT_SmartSearchPlayBack(LLONG lPlayHandle, LPIntelligentSearchPlay lpPlayBackParam);

// Get playback OSD time 
CLIENT_NET_API BOOL CALL_METHOD CLIENT_GetPlayBackOsdTime(LLONG lPlayHandle, LPNET_TIME lpOsdTime, LPNET_TIME lpStartTime, LPNET_TIME lpEndTime);

// Stop file playback 
CLIENT_NET_API BOOL CALL_METHOD CLIENT_StopPlayBack(LLONG lPlayHandle);

// Download recorded file 
CLIENT_NET_API LLONG CALL_METHOD CLIENT_DownloadByRecordFile(LLONG lLoginID,LPNET_RECORDFILE_INFO lpRecordFile, char *sSavedFileName, fDownLoadPosCallBack cbDownLoadPos, LDWORD dwUserData);

// Download video files - extension
// sSavedFileName Is not null, video data written to the corresponding file path; fDownLoadDataCallBack Is not null, video data through the callback function returns 
CLIENT_NET_API LLONG CALL_METHOD CLIENT_DownloadByRecordFileEx(LLONG lLoginID, LPNET_RECORDFILE_INFO lpRecordFile, char *sSavedFileName, 
                                                           fDownLoadPosCallBack cbDownLoadPos, LDWORD dwUserData, 
                                                           fDataCallBack fDownLoadDataCallBack, LDWORD dwDataUser, void* pReserved = NULL);

// Download file by time 
// nRecordFileType corresonding to emum EM_QUERY_RECORD_TYPE
CLIENT_NET_API LLONG CALL_METHOD CLIENT_DownloadByTime(LLONG lLoginID, int nChannelId, int nRecordFileType, LPNET_TIME tmStart, LPNET_TIME tmEnd, char *sSavedFileName, fTimeDownLoadPosCallBack cbTimeDownLoadPos, LDWORD dwUserData);

// Through the time to download the video - extension
// sSavedFileName Is not null, video data written to the corresponding file path; fDownLoadDataCallBack Is not null, video data through the callback function returns
CLIENT_NET_API LLONG CALL_METHOD CLIENT_DownloadByTimeEx(LLONG lLoginID, int nChannelId, int nRecordFileType, LPNET_TIME tmStart, LPNET_TIME tmEnd, char *sSavedFileName, 
                                                     fTimeDownLoadPosCallBack cbTimeDownLoadPos, LDWORD dwUserData, 
                                                     fDataCallBack fDownLoadDataCallBack, LDWORD dwDataUser, void* pReserved = NULL);

// Search record download process 
CLIENT_NET_API BOOL CALL_METHOD CLIENT_GetDownloadPos(LLONG lFileHandle, int *nTotalSize, int *nDownLoadSize);

// Stop record download 
CLIENT_NET_API BOOL CALL_METHOD CLIENT_StopDownload(LLONG lFileHandle);

//------------------------------------- realplay playback download by stream date type  ---------------------


#define NET_DATA_CALL_BACK_VALUE      1000       // dwDataType = NET_DATA_CALL_BACK_VALUE + emDataType
                                                 // dwDataType is the param in fRealDataCallBackEx and fDataCallBack
                                                 // emDataType is the enum in NET_IN_REALPLAY_BY_DATA_TYPE/NET_IN_PLAYBACK_BY_DATA_TYPE/NET_IN_DOWNLOAD_BY_DATA_TYPE                                       
                                                 
// stream date type
typedef enum tagEM_REAL_DATA_TYPE
{
    EM_REAL_DATA_TYPE_PRIVATE,       // private stream
    EM_REAL_DATA_TYPE_GBPS,          // Chinese standard ps stream 
    EM_REAL_DATA_TYPE_TS,            // TS stream
    EM_REAL_DATA_TYPE_MP4,           // MP4 file(the callback function supply the private stream data, the parama dwDataType is 0 )
}EM_REAL_DATA_TYPE;

// RealPlay By Stream Data Type (in param)
typedef struct tagNET_IN_REALPLAY_BY_DATA_TYPE
{
    DWORD               dwSize;                 // struct size
    int                 nChannelID;             // channel id
    HWND                hWnd;                   // play handle
    DH_RealPlayType     rType;                  // real play stream type 
    fRealDataCallBackEx cbRealData;             // realplay data callback function prototype
    EM_REAL_DATA_TYPE   emDataType;             // stream data type
    LDWORD              dwUser;                 // data user
    const char*         szSaveFileName;         // file name to convert
}NET_IN_REALPLAY_BY_DATA_TYPE;

// RealPlay By Stream Data Type (out param)
typedef struct tagNET_OUT_REALPLAY_BY_DATA_TYPE
{
    DWORD               dwSize;                 // struct size  
}NET_OUT_REALPLAY_BY_DATA_TYPE;

// RealPlay By Stream Data Type
CLIENT_NET_API LLONG CALL_METHOD CLIENT_RealPlayByDataType(LLONG lLoginID, const NET_IN_REALPLAY_BY_DATA_TYPE* pstInParam, NET_OUT_REALPLAY_BY_DATA_TYPE* pstOutParam, DWORD dwWaitTime);


// PlayBack By stream Data Type (in param)
typedef struct tagNET_IN_PLAYBACK_BY_DATA_TYPE
{
    DWORD               dwSize;                 // struct size
    int                 nChannelID;             // channel id
    NET_TIME            stStartTime;            // start time 
    NET_TIME            stStopTime;             // stop time
    HWND                hWnd;                   // play handle
    fDownLoadPosCallBack cbDownLoadPos;         // function prototype of play back progress 
    LDWORD              dwPosUser;              // pos user
    fDataCallBack       fDownLoadDataCallBack;  // Playback data callback function prototype
    EM_REAL_DATA_TYPE   emDataType;             // stream data type 
    LDWORD              dwDataUser;             // date user
    int                 nPlayDirection;         // Playback direction, 0:normal; 1:reverse;
}NET_IN_PLAYBACK_BY_DATA_TYPE;

// PlayBack By stream Data Type (out param)
typedef struct tagNET_OUT_PLAYBACK_BY_DATA_TYPE
{
    DWORD               dwSize;                 // struct size  
}NET_OUT_PLAYBACK_BY_DATA_TYPE;

// PlayBack By stream Data Type
CLIENT_NET_API LLONG CALL_METHOD CLIENT_PlayBackByDataType(LLONG lLoginID, const NET_IN_PLAYBACK_BY_DATA_TYPE* pstInParam, NET_OUT_PLAYBACK_BY_DATA_TYPE* pstOutParam, DWORD dwWaitTime);


// Download record by stream data type (in param)
typedef struct tagNET_IN_DOWNLOAD_BY_DATA_TYPE
{
    DWORD                       dwSize;                 // struct size
    int                         nChannelID;             // channel id     
    EM_QUERY_RECORD_TYPE        emRecordType;           // record type
    char*                       szSavedFileName;        // save file path
    NET_TIME                    stStartTime;            // start time
    NET_TIME                    stStopTime;             // stop time 
    fTimeDownLoadPosCallBack    cbDownLoadPos;          // function prototype of play back progress
    LDWORD                      dwPosUser;              // pos user
    fDataCallBack               fDownLoadDataCallBack;  // Playback data callback function prototype
    EM_REAL_DATA_TYPE           emDataType;             // stream data type 
    LDWORD                      dwDataUser;             // date user
}NET_IN_DOWNLOAD_BY_DATA_TYPE;

// Download record by stream data type (out param)
typedef struct tagNET_OUT_DOWNLOAD_BY_DATA_TYPE
{
    DWORD               dwSize;                 // struct size  
}NET_OUT_DOWNLOAD_BY_DATA_TYPE;

// Download record by stream data type
CLIENT_NET_API LLONG CALL_METHOD CLIENT_DownloadByDataType(LLONG lLoginID, const NET_IN_DOWNLOAD_BY_DATA_TYPE* pstInParam, NET_OUT_DOWNLOAD_BY_DATA_TYPE* pstOutParam, DWORD dwWaitTime);

//-------------------------------------------------------------------

//partial enlarged
CLIENT_NET_API BOOL CALL_METHOD CLIENT_SetDisplayRegion(LLONG lPlayHandle,DWORD nRegionNum, DH_DISPLAYRREGION *pSrcRect, HWND hDestWnd, BOOL bEnable);

// start to search record file frame info
CLIENT_NET_API BOOL    CALL_METHOD CLIENT_FindFrameInfo(LLONG lLoginID, NET_IN_FIND_FRAMEINFO_PRAM *pInParam, NET_OUT_FIND_FRAMEINFO_PRAM* pOutParam, int nWaitTime);

// search record file frame info, search by specified info item
CLIENT_NET_API BOOL    CALL_METHOD CLIENT_FindNextFrameInfo(LLONG lFindHandle, NET_IN_FINDNEXT_FRAMEINFO_PRAM *pInParam, NET_OUT_FINDNEXT_FRAMEINFO_PRAM* pOutParam, int nWaitTime);

// set file stream tags
CLIENT_NET_API BOOL    CALL_METHOD CLIENT_FileStreamSetTags(LLONG lFindHandle, NET_IN_FILE_STREAM_TAGS_INFO *pInParam, NET_OUT_FILE_STREAM_TAGS_INFO *pOutParam, int nWaitTime);

// get file stream tags
CLIENT_NET_API BOOL    CALL_METHOD CLIENT_FileStreamGetTags(LLONG lFindHandle, NET_IN_FILE_STREAM_GET_TAGS_INFO *pInParam, NET_OUT_FILE_STREAM_GET_TAGS_INFO *pOutParam, int nWaitTime);

// clear file stream tags
CLIENT_NET_API BOOL    CALL_METHOD CLIENT_FileStreamClearTags(LLONG lFindHandle, NET_IN_FILE_STREAM_TAGS_INFO *pInParam, NET_OUT_FILE_STREAM_TAGS_INFO *pOutParam, int nWaitTime);

// filter file stream tags
CLIENT_NET_API BOOL    CALL_METHOD CLIENT_FileStreamFilterTags(LLONG lFindHandle, NET_IN_FILE_STREAM_FILTER_TAGS_INFO *pInParam, NET_OUT_FILE_STREAM_GET_TAGS_INFO *pOutParam, int nWaitTime);

// end record file search
CLIENT_NET_API BOOL CALL_METHOD CLIENT_FindFrameInfoClose(LLONG lFindHandle);
// pInParam, pOutParam memory managed by user
CLIENT_NET_API BOOL CALL_METHOD CLIENT_FileStreamMotionMatch(LLONG lLoginID, const NET_IN_MOTIONMATCH_PARAM *pInParam, NET_OUT_MOTIONMATCH_PARAM* pOutParam, int nWaitTime);

// find tags' info, lFindHandle is get from CLIENT_FindFrameInfo, the tags in the result were set by CLIENT_TagManagerStartTag/CLIENT_TagManagerSetSubTag/CLIENT_TagManagerStopTag
CLIENT_NET_API BOOL CALL_METHOD CLIENT_FileStreamFindTags(LLONG lFindHandle, NET_IN_FINDTAGS_INFO *pInParam, NET_OUT_FINDTAGS_INFO *pOutParam, int nWaitTime);

// get tag state
CLIENT_NET_API BOOL CALL_METHOD CLIENT_TagManagerGetTagState(LLONG lLoginID, NET_IN_TAGMANAGER_GETTAGSTATE_INFO *pInParam, NET_OUT_TAGMANAGER_GETTAGSTATE_INFO *pOutParam, int nWaitTime);

// start tag
CLIENT_NET_API BOOL CALL_METHOD CLIENT_TagManagerStartTag(LLONG lLoginID, NET_IN_TAGMANAGER_STARTTAG_INFO *pInParam, NET_OUT_TAGMANAGER_STARTTAG_INFO *pOutParam, int nWaitTime);

// set a sub tag, sub tag's count can not greater than 20, otherwise it will failed 
CLIENT_NET_API BOOL CALL_METHOD CLIENT_TagManagerSetSubTag(LLONG lLoginID, NET_IN_TAGMANAGER_SETSUBTAG_INFO *pInParam, NET_OUT_TAGMANAGER_SETSUBTAG_INFO *pOutParam, int nWaitTime);

// stop a tag
CLIENT_NET_API BOOL CALL_METHOD CLIENT_TagManagerStopTag(LLONG lLoginID, NET_IN_TAGMANAGER_STOPTAG_INFO *pInParam, NET_OUT_TAGMANAGER_STOPTAG_INFO *pOutParam, int nWaitTime);

//------------------------------------------------------------------------

// Set alarm callback function 
CLIENT_NET_API void CALL_METHOD CLIENT_SetDVRMessCallBack(fMessCallBack cbMessage,LDWORD dwUser);

CLIENT_NET_API void CALL_METHOD CLIENT_SetDVRMessCallBackEx1(fMessCallBackEx1 cbMessage,LDWORD dwUser);


// subscribe alarm 
CLIENT_NET_API BOOL CALL_METHOD CLIENT_StartListen(LLONG lLoginID);

// subscribe alarm---extensive
CLIENT_NET_API BOOL CALL_METHOD CLIENT_StartListenEx(LLONG lLoginID);

// Stop subscribe alarm 
CLIENT_NET_API BOOL CALL_METHOD CLIENT_StopListen(LLONG lLoginID);

// alarm reset
CLIENT_NET_API BOOL CALL_METHOD CLIENT_AlarmReset(LLONG lLoginID, DWORD dwAlarmType, int nChannel, void* pReserved = NULL, int nWaitTime = 1000);

//------------------------------------------------------------------------

// actively registration function. enable service. nTimeout is invalid. 
CLIENT_NET_API LLONG CALL_METHOD CLIENT_ListenServer(char* ip, WORD port, int nTimeout, fServiceCallBack cbListen, LDWORD dwUserData);

// stop service
CLIENT_NET_API BOOL CALL_METHOD CLIENT_StopListenServer(LLONG lServerHandle);

// Respond the registration requestion from the device 
CLIENT_NET_API BOOL CALL_METHOD CLIENT_ResponseDevReg(char *devSerial, char* ip, WORD port, BOOL bAccept);

//------------------------------------------------------------------------

// Alarm upload function. Enable service. dwTimeOut paramter is invalid 
CLIENT_NET_API LLONG CALL_METHOD CLIENT_StartService(WORD wPort, char *pIp = NULL, fServiceCallBack pfscb = NULL, DWORD dwTimeOut = 0xffffffff, LDWORD dwUserData = 0);

// Stop service 
CLIENT_NET_API BOOL CALL_METHOD CLIENT_StopService(LLONG lHandle);

//------------------------------------------------------------------------

// Set audio talk mode(client-end mode or server mode), user malloc pValue's memory,please refer to the corresponding structure of EM_USEDEV_MODE
CLIENT_NET_API BOOL CALL_METHOD CLIENT_SetDeviceMode(LLONG lLoginID, EM_USEDEV_MODE emType, void* pValue);

// Enable audio talk 
CLIENT_NET_API LLONG CALL_METHOD CLIENT_StartTalkEx(LLONG lLoginID, pfAudioDataCallBack pfcb, LDWORD dwUser);

// Begin PC record 
CLIENT_NET_API BOOL CALL_METHOD CLIENT_RecordStart();

// Stop PC record 
CLIENT_NET_API BOOL CALL_METHOD CLIENT_RecordStop();


// start PC record(to CLIENT_RecordStart()extend)
CLIENT_NET_API BOOL CALL_METHOD CLIENT_RecordStartEx(LLONG lLoginID);

// end PC record(to CLIENT_RecordStop() extend)
CLIENT_NET_API BOOL CALL_METHOD CLIENT_RecordStopEx(LLONG lLoginID);


// Send out audio data to the device 
CLIENT_NET_API LONG CALL_METHOD CLIENT_TalkSendData(LLONG lTalkHandle, char *pSendBuf, DWORD dwBufSize);

// Decode audio data 
CLIENT_NET_API void CALL_METHOD CLIENT_AudioDec(char *pAudioDataBuf, DWORD dwBufSize);
CLIENT_NET_API BOOL CALL_METHOD CLIENT_AudioDecEx(LLONG lTalkHandle, char *pAudioDataBuf, DWORD dwBufSize);

// Set audio talk volume
CLIENT_NET_API BOOL CALL_METHOD CLIENT_SetAudioClientVolume(LLONG lTalkHandle, WORD wVolume);

// Stop audio talk 
CLIENT_NET_API BOOL CALL_METHOD CLIENT_StopTalkEx(LLONG lTalkHandle);

// add device into broadcast group 
CLIENT_NET_API BOOL CALL_METHOD CLIENT_AudioBroadcastAddDev(LLONG lLoginID);

// Remove device from the broadcast group 
CLIENT_NET_API BOOL CALL_METHOD CLIENT_AudioBroadcastDelDev(LLONG lLoginID);

// audio encode-initialization(specified standard format, private format)
CLIENT_NET_API int  CALL_METHOD CLIENT_InitAudioEncode(DH_AUDIO_FORMAT aft);

// Audio encode--data encode
CLIENT_NET_API int	CALL_METHOD	CLIENT_AudioEncode(LLONG lTalkHandle, BYTE *lpInBuf, DWORD *lpInLen, BYTE *lpOutBuf, DWORD *lpOutLen);

// audio encode--complete and then exit
CLIENT_NET_API int	CALL_METHOD	CLIENT_ReleaseAudioEncode();

//------------------------------------------------------------------------

// Search device log
CLIENT_NET_API BOOL CALL_METHOD CLIENT_QueryLog(LLONG lLoginID, char *pLogBuffer, int maxlen, int *nLogBufferlen, int waittime=3000);

// Search device log page by page.
CLIENT_NET_API BOOL CALL_METHOD CLIENT_QueryDeviceLog(LLONG lLoginID, QUERY_DEVICE_LOG_PARAM *pQueryParam, char *pLogBuffer, int nLogBufferLen, int *pRecLogNum, int waittime=3000);

// search device log item
CLIENT_NET_API BOOL CALL_METHOD CLIENT_QueryDevLogCount(LLONG lLoginID, NET_IN_GETCOUNT_LOG_PARAM *pInParam, NET_OUT_GETCOUNT_LOG_PARAM* pOutParam , int waittime=3000);

// Search channel record status, user malloc and free the memory of pRSBuffer
CLIENT_NET_API BOOL CALL_METHOD CLIENT_QueryRecordState(LLONG lLoginID, char *pRSBuffer, int maxlen, int *nRSBufferlen, int waittime=1000);

// Search channel extra record status (the returned byte number was equal to the channel number, every byte instruct the respond channel's state,0-stop,1-manual,2-schedule),user malloc and free the memory of pRSBuffer
CLIENT_NET_API BOOL CALL_METHOD CLIENT_QueryExtraRecordState(LLONG lLoginID, char *pRSBuffer, int maxlen, int *nRSBufferlen, void *pReserved, int waittime=1000);

// Search device status,user malloc pBuf memory,please refer to query device type to ensure structure,then apply to the memory size of structure 
CLIENT_NET_API BOOL CALL_METHOD CLIENT_QueryDevState(LLONG lLoginID, int nType, char *pBuf, int nBufLen, int *pRetLen, int waittime=1000);

// query remote device state,when nType = DH_DEVSTATE_ALARM_FRONTDISCONNECT,the number form 1.
CLIENT_NET_API BOOL CALL_METHOD CLIENT_QueryRemotDevState(LLONG lLoginID, int nType, int nChannelID, char *pBuf, int nBufLen, int *pRetLen, int waittime=1000);

// Search system capacity information ,user malloc pSysInfoBuffer's memory,please refer to the corresponding structure of DH_SYS_ABILITY.if nSystemType is ABILITY_DYNAMIC_CONNECT,memory is sizeof(BOOL) at least,if nSystemType is ABILITY_TRIGGER_MODE,memory size is sizeof(int) at least.
CLIENT_NET_API BOOL CALL_METHOD CLIENT_QuerySystemInfo(LLONG lLoginID, int nSystemType, char *pSysInfoBuffer, int maxlen, int *nSysInfolen, int waittime=1000);

// New Search system capacity information(by Json)
CLIENT_NET_API BOOL CALL_METHOD CLIENT_QueryNewSystemInfo(LLONG lLoginID, char* szCommand, int nChannelID, char* szOutBuffer, DWORD dwOutBufferSize, int *error, int waittime=1000);

// Get channel bit stream 
CLIENT_NET_API LONG CALL_METHOD CLIENT_GetStatiscFlux(LLONG lLoginID, LLONG lPlayHandle);

// Get PTZ information 
CLIENT_NET_API BOOL  CALL_METHOD CLIENT_GetPtzOptAttr(LLONG lLoginID,DWORD dwProtocolIndex,LPVOID lpOutBuffer,DWORD dwBufLen,DWORD *lpBytesReturned,int waittime=500);

// device capacity type, corresponding to CLIENT_GetDevCaps port
#define NET_DEV_CAP_SEQPOWER            0x01                // power timing capacity, pInBuf=NET_IN_CAP_SEQPOWER*, pOutBuf=NET_OUT_CAP_SEQPOWER*
#define NET_ENCODE_CFG_CAPS             0x02                // device encode config corresponding to capacity, pInBuf=NET_IN_ENCODE_CFG_CAPS*, pOutBuf= NET_OUT_ENCODE_CFG_CAPS*
#define NET_VIDEOIN_FISHEYE_CAPS        0x03                // Fish eye capacity , pInBuf=NET_IN_VIDEOIN_FISHEYE_CAPS*, pOutBuf=NET_OUT_VIDEOIN_FISHEYE_CAPS*
#define NET_COMPOSITE_CAPS              0x04                // Get composited caps by pointed window number, pInBuf=NET_IN_COMPOSITE_CAPS*, pOutBuf=NET_OUT_COMPOSITE_CAPS*
#define NET_VIDEO_DETECT_CAPS           0x05                // Get video detect device caps,pInBuf=NET_IN_VIDEO_DETECT_CAPS* , pOutBuf=NET_OUT_VIDEO_DETECT_CAPS*
#define NET_THERMO_GRAPHY_CAPS          0x06                // Thermal imaging camera capability, pInBuf=NET_IN_THERMO_GETCAPS*, pOutBuf=NET_OUT_THERMO_GETCAPS*
#define NET_RADIOMETRY_CAPS             0x07                // Thermal imaging radiometry capability, pInBuf=NET_IN_RADIOMETRY_GETCAPS*, pOutBuf=NET_OUT_RADIOMETRY_GETCAPS*
#define NET_POS_CAPS                    0x08                // POS capability, pInBuf = NET_IN_POS_GETCAPS *, pOutBuf = NET_OUT_POS_GETCAPS *
#define NET_USER_MNG_CAPS               0x09                // User manage capability, pInBuf = NET_IN_USER_MNG_GETCAPS *, pOutBuf = NET_OUT_USER_MNG_GETCAPS *
#define NET_MEDIAMANAGER_CAPS           0x0a                // query capabilities of VideoInput, pInBuf=NET_IN_MEDIAMANAGER_GETCAPS*, pOutBuf=NET_OUT_MEDIAMANAGER_GETCAPS*
#define	NET_VIDEO_MOSAIC_CAPS			0x0b				// query capabilities of video mosaic, pInBuf=NET_IN_MEDIA_VIDEOMOSAIC_GETCAPS*, pOutBuf=NET_OUT_MEDIA_VIDEOMOSAIC_GETCAPS*
#define NET_SNAP_CFG_CAPS               0x0c                // query capabilities of snap config, pInBuf=NET_IN_SNAP_CFG_CAPS*, pOutBuf= NET_OUT_SNAP_CFG_CAPS*
#define NET_VIDEOIN_CAPS                0x0d                // query capabilities of VideoInput, pInBUf = NET_IN_VIDEOIN_CAPS*, pOutBuf = NET_OUT_VIDEOIN_CAPS*
#define NET_FACE_BOARD_CAPS             0x0e                // query capabilities of face board, pInBuf = NET_IN_FACEBOARD_CAPS*, pOutBuf = NET_OUT_FACEBOARD_CAPS*
#define NET_EXTERNALSENSOR_CAPS			0x0f				// query capabilities of external sensor, pInBuf = NET_IN_EXTERNALSENSOR_CAPS*, pOutBuf = NET_OUT_EXTERNALSENSOR_CAPS*
#define NET_VIDEO_IMAGECONTROL_CAPS		0x10				// video image control capability, pInBuf = NET_IN_VIDEO_IMAGECONTROL_CAPS*, pOutBuf = NET_OUT_VIDEO_IMAGECONTROL_CAPS*
#define NET_VIDEOIN_EXPOSURE_CAPS		0x11				// video exposure capability, pInBuf = NET_IN_VIDEOIN_EXPOSURE_CAPS*, pOutBuf = NET_OUT_VIDEOIN_EXPOSURE_CAPS*
#define NET_VIDEOIN_DENOISE_CAPS		0x12				// video denoise capability, pInBuf = NET_IN_VIDEOIN_DENOISE_CAPS*, pOutBuf = NET_OUT_VIDEOIN_DENOISE_CAPS*
#define NET_VIDEOIN_BACKLIGHT_CAPS		0x13				// video back light capability, pInBuf = NET_IN_VIDEOIN_BACKLIGHT_CAPS*, pOutBuf = NET_OUT_VIDEOIN_BACKLIGHT_CAPS*
#define NET_VIDEOIN_WHITEBALANCE_CAPS	0x14				// video white balance capability, pInBuf = NET_IN_VIDEOIN_WHITEBALANCE_CAPS*, pOutBuf = NET_OUT_VIDEOIN_WHITEBALANCE_CAPS*
#define NET_VIDEOIN_DAYNIGHT_CAPS		0x15				// video day night capability, pInBuf = NET_IN_VIDEOIN_DAYNIGHT_CAPS*, pOutBuf = NET_OUT_VIDEOIN_DAYNIGHT_CAPS*
#define NET_VIDEOIN_ZOOM_CAPS			0x16				// video zoom capability, pInBuf = NET_IN_VIDEOIN_ZOOM_CAPS*, pOutBuf = NET_OUT_VIDEOIN_ZOOM_CAPS*
#define	NET_VIDEOIN_FOCUS_CAPS			0x17				// video focus capability, pInBuf = NET_IN_VIDEOIN_FOCUS_CAPS*, pOutBuf = NET_OUT_VIDEOIN_FOCUS_CAPS*
#define NET_VIDEOIN_SHARPNESS_CAPS		0x18				// video sharpness capability, pInBuf = NET_IN_VIDEOIN_SHARPNESS_CAPS*, pOutBuf = NET_OUT_VIDEOIN_SHARPNESS_CAPS*
#define NET_VIDEOIN_COLOR_CAPS			0x19				// video color capability, pInBuf = NET_IN_VIDEOIN_COLOR_CAPS*, pOutBuf = NET_OUT_VIDEOIN_COLOR_CAPS*
#define NET_GET_MASTERSLAVEGROUP_CAPS	0X1a				// Get service capability set, pInBuf = NET_IN_GET_MASTERSLAVEGROUP_CAPS*, pOutBuf = NET_OUT_GET_MASTERSLAVEGROUP_CAPS*
#define NET_FACERECOGNITIONSE_CAPS		0x1b				// Face Recognition Server Caps bility Query  pInBuf = NET_IN_FACERECOGNITIONSERVER_CAPSBILITYQUERY, pOutBuf = NET_OUT_FACERECOGNITIONSERVER_CAPSBILITYQUERY *
#define NET_STORAGE_CAPS				0x1c				// Get Storage capability, pInBuf = NET_IN_STORAGE_CAPS*, pOutBuf = NET_OUT_STORAGE_CAPS*
#define NET_VIDEOIN_RAWFRAME_CAPS		0x1d				// Get the video input extension capability set, pInBuf = NET_IN_VIDEOIN_RAWFRAME_CAPS*, pOutBuf = NET_OUT_VIDEOIN_RAWFRAME_CAPS*
// get device capacity,user malloc memory of pInBuf and pOutBuf,please refer to nType to ensure structure,then ensure memory size
CLIENT_NET_API BOOL  CALL_METHOD CLIENT_GetDevCaps(LLONG lLoginID, int nType, void* pInBuf, void* pOutBuf, int nWaitTime);

// device info type, corresponding to CLIENT_QueryDevInfo port
#define NET_QUERY_DEV_STORAGE_NAMES                 0x01                // search device storage module name list , pInBuf=NET_IN_STORAGE_DEV_NAMES *, pOutBuf=NET_OUT_STORAGE_DEV_NAMES *
#define NET_QUERY_DEV_STORAGE_INFOS                 0x02                // search device storage info list, pInBuf=NET_IN_STORAGE_DEV_INFOS*, pOutBuf= NET_OUT_STORAGE_DEV_INFOS *
#define NET_QUERY_RECENCY_JNNCTION_CAR_INFO         0x03                // search recent ANPR vehicle info port, pInBuf=NET_IN_GET_RECENCY_JUNCTION_CAR_INFO*, pOutBuf=NET_OUT_GET_RECENCY_JUNCTION_CAR_INFO*
#define NET_QUERY_LANES_STATE                       0x04                // search lane info ,pInBuf = NET_IN_GET_LANES_STATE , pOutBuf = NET_OUT_GET_LANES_STATE
#define NET_QUERY_DEV_FISHEYE_WININFO               0x05                // search Fish eye window info , pInBuf= NET_IN_FISHEYE_WININFO*, pOutBuf=NET_OUT_FISHEYE_WININFO *
#define NET_QUERY_DEV_REMOTE_DEVICE_INFO            0x06                // search remote device info , pInBuf= NET_IN_GET_DEVICE_INFO*, pOutBuf= NET_OUT_GET_DEVICE_INFO *
#define NET_QUERY_SYSTEM_INFO                       0x07                // search system info , pInBuf= NET_IN_SYSTEM_INFO*, pOutBuf= NET_OUT_SYSTEM_INFO*
#define NET_QUERY_REG_DEVICE_NET_INFO               0x08                // query active register device network connection , pInBuf=NET_IN_REGDEV_NET_INFO * , pOutBuf=NET_OUT_REGDEV_NET_INFO *
#define NET_QUERY_DEV_THERMO_GRAPHY_PRESET          0x09                // query thermal preset info , pInBuf= NET_IN_THERMO_GET_PRESET_INFO*, pOutBuf= NET_OUT_THERMO_GET_PRESET_INFO *
#define NET_QUERY_DEV_THERMO_GRAPHY_OPTREGION       0x0a                // query thermal ROI info, pInBuf= NET_IN_THERMO_GET_OPTREGION*, pOutBuf= NET_OUT_THERMO_GET_OPTREGION *
#define NET_QUERY_DEV_THERMO_GRAPHY_EXTSYSINFO      0x0b                // query thermal external system info, pInBuf= NET_IN_THERMO_GET_EXTSYSINFO*, pOutBuf= NET_OUT_THERMO_GET_EXTSYSINFO *
#define NET_QUERY_DEV_RADIOMETRY_POINT_TEMPER       0x0c                // query the parameter value of temperature measurement point, pInBuf= NET_IN_RADIOMETRY_GETPOINTTEMPER*, pOutBuf= NET_OUT_RADIOMETRY_GETPOINTTEMPER *
#define NET_QUERY_DEV_RADIOMETRY_TEMPER             0x0d                // query parameter value of temperature measurement item, pInBuf= NET_IN_RADIOMETRY_GETTEMPER*, pOutBuf= NET_OUT_RADIOMETRY_GETTEMPER *
#define NET_QUERY_GET_CAMERA_STATE                  0x0e                // query camera state, pInBuf= NET_IN_GET_CAMERA_STATEINFO*, pOutBuf= NET_OUT_GET_CAMERA_STATEINFO *
#define NET_QUERY_GET_REMOTE_CHANNEL_AUDIO_ENCODE   0x0f                // acquire remote communication audio coding mode  , pInBuf= NET_IN_GET_REMOTE_CHANNEL_AUDIO_ENCODEINFO*, pOutBuf= NET_OUT_GET_REMOTE_CHANNEL_AUDIO_ENCODEINFO *
#define NET_QUERY_GET_COMM_PORT_INFO                0x10                // get comm port Info, pInBuf=NET_IN_GET_COMM_PORT_INFO* , pOutBuf=NET_OUT_GET_COMM_PORT_INFO* 
#define NET_QUERY_GET_LINKCHANNELS                  0x11                // query video linked channels for a single channel 
#define NET_QUERY_GET_VIDEOOUTPUTCHANNELS           0x12                // query video decode channel count statistics info, pInBuf=NET_IN_GET_VIDEOOUTPUTCHANNELS*, pOutBuf=NET_OUT_GET_VIDEOOUTPUTCHANNELS*
#define NET_QUERY_GET_VIDEOINFO                     0x13                // query video decode channel info, pInBuf=NET_IN_GET_VIDEOINFO*, pOutBuf=NET_OUT_GET_VIDEOINFO*
#define NET_QUERY_GET_ALLLINKCHANNELS               0x14                // query all linked video channel groups, pInBuf=NET_IN_GET_ALLLINKCHANNELS* , pOutBuf=NET_OUT_GET_ALLLINKCHANNELS*
#define NET_QUERY_VIDEOCHANNELSINFO                 0x15                // query video channel info, pInBuf=NET_IN_GET_VIDEOCHANNELSINFO* , pOutBuf=NET_OUT_GET_VIDEOCHANNELSINFO*
#define NET_QUERY_TRAFFICRADAR_VERSION              0x16                // query traffic radar version, pInBuf=NET_IN_TRAFFICRADAR_VERSION* , pOutBuf=NET_OUT_TRAFFICRADAR_VERSION*
#define NET_QUERY_WORKGROUP_NAMES                   0x17                // query all workgroups' names, pInBuf=NET_IN_WORKGROUP_NAMES* , pOutBuf=NET_OUT_WORKGROUP_NAMES*
#define NET_QUERY_WORKGROUP_INFO                    0x18                // query workgroup info, pInBuf=NET_IN_WORKGROUP_INFO* , pOutBuf=NET_OUT_WORKGROUP_INFO*
#define NET_QUERY_WLAN_ACCESSPOINT                  0x19                // query wlan access point info, pInBuf=NET_IN_WLAN_ACCESSPOINT* , pOutBuf=NET_OUT_WLAN_ACCESSPOINT*
#define NET_QUERY_GPS_INFO							0x1a				// query device GPS information,pInBuf=NET_IN_DEV_GPS_INFO* , pOutBuf=NET_OUT_DEV_GPS_INFO*
#define NET_QUERY_IVS_REMOTE_DEVICE_INFO            0x1b                // query IVS related remote device info, pInBuf = NET_IN_IVS_REMOTE_DEV_INFO*, pOutBuf = NET_OUT_IVS_REMOTE_DEV_INFO*
#define NET_QUERY_SMART_SWITCH_INFO                 0x1c                // query smart switch info, pInBuf = NET_IN_SMART_SWITCH_INFO*,  pOutBuf = NET_OUT_SMART_SWITCH_INFO*
#define NET_QUERY_UPGRADE_STATE                     0x1d                // query upgrade state, pInBuf = NET_IN_UPGRADE_STATE*, pOutBuf = NET_OUT_UPGRADE_STATE* 
#define NET_QUERY_VIDEO_ENCODE_CAPS					0x1e				// query video encode caps, pInBuf = NET_IN_VIDEO_ENCODE_CAPS*, pOutBuf = NET_OUT_VIDEO_ENCODE_CAPS* 
#define NET_QUERY_AUDIO_ENCODE_CAPS					0x1f				// query audio encode caps, pInBuf = NET_IN_AUDIO_ENCODE_CAPS*, pOutBuf = NET_OUT_AUDIO_ENCODE_CAPS* 
#define NET_QUERY_AUDIO_IN_CAPS						0x20				// query audio input channel caps, pInBuf = NET_IN_AUDIO_IN_CAPS*, pOutBuf = NET_OUT_AUDIO_IN_CAPS* 
#define NET_QUERY_SMART_ENCODE_CAPS					0x21				// query Smart encode caps, pInBuf = NET_IN_SMART_ENCODE_CAPS*, pOutBuf = NET_OUT_SMART_ENCODE_CAPS* 
#define NET_QUERY_HARDDISK_TEMPERATURE				0X22				// query hard disk temperature,pInBu f= NET_IN_HDD_TEMPERATURE*,pOutBuf = NET_OUT_HDD_TEMPERATURE*
#define NET_QUERY_RAWFRAMEDATA						0x23				// Gets YUV data in the specified format, pInBuf = NET_IN_RAWFRAMEDATA*, pOutBuf = NET_OUT_RAWFRAMEDARA*
// search device info,user malloc memory of pInBuf and pOutBuf,please refer to nQueryType to ensure structure,then ensure memory size
CLIENT_NET_API BOOL  CALL_METHOD CLIENT_QueryDevInfo(LLONG lLoginID, int nQueryType, void* pInBuf, void* pOutBuf, void *pReserved = NULL , int nWaitTime = 1000);
//------------------------------------------------------------------------

// Reboot device 
CLIENT_NET_API BOOL CALL_METHOD CLIENT_RebootDev(LLONG lLoginID);

// Shut down devic e
CLIENT_NET_API BOOL CALL_METHOD CLIENT_ShutDownDev(LLONG lLoginID);

// Device control,user malloc param's memory,please refer to the corresponding structure of type 
CLIENT_NET_API BOOL CALL_METHOD CLIENT_ControlDevice(LLONG lLoginID, CtrlType type, void *param, int waittime = 1000);

// Device control asynchronism,user malloc param's memory,please refer to the corresponding structure of type
CLIENT_NET_API BOOL CALL_METHOD CLIENT_StartControlDevice(LLONG lLoginID, CtrlType type, void *param);

// Set channel record status,user mallloc memory of pInBuf and pOutBuf,please refer to corresponding structure of emType.
CLIENT_NET_API BOOL CALL_METHOD CLIENT_ControlDeviceEx(LLONG lLoginID, CtrlType emType, void* pInBuf, void* pOutBuf = NULL, int nWaitTime = 1000);

#define DH_POS_EXCHANGE_GOODS_MAX             4           //goods array number   
#define DH_POS_EXCHANGE_FUSSY_KEY_MAX		  4           //the key words'number of fuzzy pattern

// POS get exchange information   the key word type of Fuzzy pattern
typedef enum tagEM_NET_POS_EXCHANGE_FUSSY_KEY
{
    EM_NET_POS_EXCHANGE_KEY_NULL = 0,                      // not Fuzzy pattern
    EM_NET_POS_EXCHANGE_KEY_GOODS = 1,                     // key word = Goods(now only suport Goods when Fuzzy pattern)
} EM_NET_POS_EXCHANGE_FUSSY_KEY;
  
// Interface CLIENT_StartFind  command NET_FIND_POS_EXCHANGE in param       
typedef struct tagNET_IN_POSEXCHANGE_STARTFIND       
{       
    DWORD							dwSize;														//struct size       
    int								nChannel;													//channel number, 0 - first channel, -1 - all channel       
    NET_TIME_EX						stuStartTime;												//start time       
    NET_TIME_EX				        stuEndTime;													//end time       
    char							szGoods[DH_POS_EXCHANGE_GOODS_MAX][DH_COMMON_STRING_32];    //goods       
    EM_NET_POS_EXCHANGE_FUSSY_KEY   szFuzzyPattern[DH_POS_EXCHANGE_FUSSY_KEY_MAX];			    //Fuzzy pattern       
    int                             nPosId;                                                     //Pos id, -1 means all pos
} NET_IN_POSEXCHANGE_STARTFIND;       
        
// Interface CLIENT_StartFind command NET_FIND_POS_EXCHANGE out param       
typedef struct tagNET_OUT_POSEXCHANGE_STARTFIND       
{       
    DWORD                dwSize;                         // struct size       
    unsigned int		 nToken;						 // Token
    DWORD                dwTotalCount;                   // total count       
}NET_OUT_POSEXCHANGE_STARTFIND;       
        
// Interface CLIENT_DoFind command NET_FIND_POS_EXCHANGE in param       
typedef struct tagNET_IN_POSEXCHANGE_DOFIND       
{       
    DWORD                dwSize;                         // struct size       
    unsigned int		 nToken;						 // Search handle       
    unsigned int		 nBeginNumber;                   // begin number    0<=dwBeginNumber<=dwTotalCount-1       
    unsigned int         nCount;                         // count of get exchange information       
} NET_IN_POSEXCHANGE_DOFIND;       
        
// POS exchange information       
typedef struct tagNET_POSEXCHANGE_INFO       
{       
    NET_TIME            stuExchangeTime;                //exchange time       
}NET_POSEXCHANGE_INFO;       

// POS exchange info 
typedef struct tagNET_POSEXCHANGE_INFO_EX
{
    int                 nPosId;                         // Pos id
    char                cDetail[512];                   // exchange detail info
    int                 nDetailLen;                     // length of cDetail
    BYTE                reserved[1024];                 // reserved
} NET_POSEXCHANGE_INFO_EX;
        
// Interface CLIENT_DoFind command NET_FIND_POS_EXCHANGE out param       
typedef struct tagNET_OUT_POSEXCHANGE_DOFIND       
{       
    DWORD                       dwSize;                                             // struct size       
    DWORD                       dwFound;                                            // found number       
    NET_POSEXCHANGE_INFO        arrPOSExchangeInfo[DH_MAX_POS_EXCHANGE_INFO];       // pos exchange information array       
    NET_POSEXCHANGE_INFO_EX     stuInfoEx[DH_MAX_POS_EXCHANGE_INFO];                // exchange info detail
}NET_OUT_POSEXCHANGE_DOFIND;       
        
// Interface CLIENT_StopFind command NET_FIND_POS_EXCHANGE in param       
typedef struct tagNET_IN_POSEXCHANGE_STOPFIND       
{          
    DWORD               dwSize;                             // struct size       
    unsigned int        nToken;								// Search handle       
} NET_IN_POSEXCHANGE_STOPFIND;       
        
// Interface CLIENT_StopFind command NET_FIND_POS_EXCHANGE out param       
typedef struct tagNET_OUT_POSEXCHANGE_STOPFIND       
{          
    DWORD               dwSize;                             // struct size       
} NET_OUT_POSEXCHANGE_STOPFIND;       

typedef enum tagNET_FIND {
    NET_FIND_RADIOMETRY,                            // thermal temperature query, pInBuf= NET_IN_RADIOMETRY_*FIND*, pOutBuf= NET_OUT_RADIOMETRY_*FIND*    
	NET_FIND_POS_EXCHANGE,							// POS Exchange Info Find,pInBuf = NET_IN_POSEXCHANGE_*FIND*,pOutBuf= NET_OUT_POSEXCHANGE_*FIND*
} NET_FIND;

// decode callback type
typedef enum tagNET_VIDEOSTREAM_TYPE
{
	NET_VIDEOSTREAM_NORMAL,							// normal
	NET_VIDEOSTREAM_YUV,							// YUV
} NET_VIDEOSTREAM_TYPE;


#define MAX_EVENT_LINK_QUERY_CODE_NUMBER	8		// max number of query code
#define MAX_EVENT_LINK_NAME_NUMBER			16		// max number of event link name

typedef enum tagEM_NET_QUERY_CODE {
		EM_NET_QUERY_CODE_UNKNOWN,					// Unknown
		EM_NET_QUERY_CODE_CROSSLINEDETECTION,		// CrossLineDetection
		EM_NET_QUERY_CODE_CROSSREGIONDETECTION,		// CrossRegionDetection
		EM_NET_QUERY_CODE_TAKENAWAYDETECTION,		// TakenAwayDetection
		EM_NET_QUERY_CODE_LEFTDETECTION,			// LeftDetection
		EM_NET_QUERY_CODE_PARKINGDETECTION,			// ParkingDetection
		EM_NET_QUERY_CODE_WANDERDETECTION,			// WanderDetection
		EM_NET_QUERY_CODE_MOVEDETECTION,			// MoveDetection
		EM_NET_QUERY_CODE_FACEDETECTION,			// FaceDetection
		EM_NET_QUERY_CODE_RETROGRADEDETECTION,		// RetrogradeDetection
		EM_NET_QUERY_CODE_NUMBERSTAT,				// NumberStat
		EM_NET_QUERY_CODE_TRAFFICJUNCTION,			// TrafficJunction
		EM_NET_QUERY_CODE_RIOTERDETECTION,			// RioterDetection
		EM_NET_QUERY_CODE_ALARMLOCAL,				// AlarmLocal
		EM_NET_QUERY_CODE_VIDEOMOTION,				// VideoMotion
		EM_NET_QUERY_CODE_VIDEOBLIND,				// VideoBlind
		EM_NET_QUERY_CODE_AUDIODETECT,				// AudioDetect
		EM_NET_QUERY_CODE_AUDIOANOMALY,				// AudioAnomaly
		EM_NET_QUERY_CODE_AUDIOMUTATION,			// AudioMutation
		EM_NET_QUERY_CODE_SCENECHANGE,				// SceneChange
		EM_NET_QUERY_CODE_VIDEOUNFOCUS,				// VideoUnFocus
		EM_NET_QUERY_CODE_STORAGELOWSPACE,			// StorageLowSpace
		EM_NET_QUERY_CODE_STORAGEFAILURE,			// StorageFailure
		EM_NET_QUERY_CODE_LOGINFAILURE,				// LoginFailure
		EM_NET_QUERY_CODE_STORAGENOTEXIST,			// StorageNotExist
		EM_NET_QUERY_CODE_IPCONFLICT,				// IPConflict
		EM_NET_QUERY_CODE_NETABORT,					// NetAbort
		EM_NET_QUERY_CODE_HOTPLUG,					// HotPlug
		EM_NET_QUERY_CODE_STORAGEFORMAT,			// StorageFormat
		EM_NET_QUERY_CODE_EMERGENCYRECORDPULL,		// EmergencyRecordPull
} EM_NET_QUERY_CODE;

typedef enum tagEM_NET_LINK_NAME {
		EM_NET_LINK_NAME_UNKNOWN,				// Unknown
		EM_NET_LINK_NAME_DEFAULT,				// default, base on eventManager.getCaps
		EM_NET_LINK_NAME_TIMESECTIONENABLE,		// TimeSectionEnable
		EM_NET_LINK_NAME_RECORDENABLE,			// RecordEnable
		EM_NET_LINK_NAME_RECORDCLOUDENABLE,		// RecordCloudEnable
		EM_NET_LINK_NAME_ALARMOUTENABLE,		// AlarmOutEnable
		EM_NET_LINK_NAME_PTZLINKENABLE,			// PtzLinkEnable
		EM_NET_LINK_NAME_SNAPSHOTENABLE,		// SnapshotEnable
		EM_NET_LINK_NAME_SNAPSHOTCLOUDENABLE,	// SnapshotCloudEnable
		EM_NET_LINK_NAME_MAILENABLE,			// MailEnable
		EM_NET_LINK_NAME_BEEPENABLE,			// BeepEnable
		EM_NET_LINK_NAME_DEJITTERENABLE,		// DejitterEnable
		EM_NET_LINK_NAME_TIPENABLE,				// TipEnable
		EM_NET_LINK_NAME_MONITORTOURENABLE,		// MonitorTourEnable
		EM_NET_LINK_NAME_MMSENABLE,				// MMSEnable
		EM_NET_LINK_NAME_SUPPORTALARMBELL,		// SupportAlarmBell
		EM_NET_LINK_NAME_SUPPORTACCESSCONTROL,	// SupportAccessControl
		EM_NET_LINK_NAME_SIPCALLENABLE,			// SipCallEnable
		EM_NET_LINK_NAME_SUPPORTALARMSERVER,	// SupportAlarmServer
		EM_NET_LINK_NAME_SUPPORTPTZLINKDELAY,	// SupportPtzLinkDelay
		EM_NET_LINK_NAME_SUPPORTPSTNALARMSERVER,// SupportPSTNAlarmServer
		EM_NET_LINK_NAME_SUPPORTICR,			// SupportICR
		EM_NET_LINK_NAME_BEEPTIME,				// BeepTime
		EM_NET_LINK_NAME_DEJITTERRANGE,			// DejitterRange
		EM_NET_LINK_NAME_ALARMOUTLATCH,			// AlarmOutLatch
		EM_NET_LINK_NAME_RECORDLATCH,			// RecordLatch
		EM_NET_LINK_NAME_VOICEENABLE,			// VoiceEnable
		EM_NET_LINK_NAME_LOGENABLE,				// LogEnable
		EM_NET_LINK_NAME_SUPPORTLIGHTCONTROL,	// SupportLightControl
		EM_NET_LINK_NAME_SUPPORTVIDEOMATRIX,	// SupportVideoMatrix
} EM_NET_LINK_NAME;

// CLIENT_getEventLink in param
typedef struct tagNET_IN_EVENT_LINK
{   
    DWORD               dwSize;					// struct size
	int					nCodeNum;				// query code number
	EM_NET_QUERY_CODE	szCodeName[MAX_EVENT_LINK_QUERY_CODE_NUMBER];		// query code name
} NET_IN_EVENT_LINK;

// CLIENT_getEventLink out param
typedef struct tagNET_OUT_EVENT_LINK
{   
    DWORD               dwSize;					// struct size
	EM_NET_LINK_NAME	szLinkName[MAX_EVENT_LINK_QUERY_CODE_NUMBER][MAX_EVENT_LINK_NAME_NUMBER];		// event link names
} NET_OUT_EVENT_LINK;


// start find information,user malloc memory of pInBuf and pOutBuf,please refer to emType to ensure structure,then ensure memory size
CLIENT_NET_API BOOL CALL_METHOD CLIENT_StartFind(LLONG lLoginID, NET_FIND emType, void* pInBuf, void* pOutBuf, int nWaitTime = 1000);
// find information,user malloc memory of pInBuf and pOutBuf,please refer to emType to ensure structure,then ensure memory size
CLIENT_NET_API BOOL CALL_METHOD CLIENT_DoFind(LLONG lLoginID, NET_FIND emType, void* pInBuf, void* pOutBuf, int nWaitTime = 1000);
// stop find information,user malloc memory of pInBuf and pOutBuf,please refer to emType to ensure structure,then ensure memory size
CLIENT_NET_API BOOL CALL_METHOD CLIENT_StopFind(LLONG lLoginID, NET_FIND emType, void* pInBuf, void* pOutBuf, int nWaitTime = 1000);
//set channel record status
CLIENT_NET_API BOOL CALL_METHOD CLIENT_SetupRecordState(LLONG lLoginID, char *pRSBuffer, int nRSBufferlen);

// set channel extra record status
CLIENT_NET_API BOOL CALL_METHOD CLIENT_SetupExtraRecordState(LLONG lLoginID, char *pRSBuffer, int nRSBufferlen, void* pReserved);

// Search IO status
CLIENT_NET_API BOOL CALL_METHOD CLIENT_QueryIOControlState(LLONG lLoginID, DH_IOTYPE emType, 
                                           void *pState, int maxlen, int *nIOCount, int waittime=1000);

// IO control,user malloc memory of pState, please refer to corresponding structure of emType
CLIENT_NET_API BOOL CALL_METHOD CLIENT_IOControl(LLONG lLoginID, DH_IOTYPE emType, void *pState, int maxlen);

// Compulsive I frame
CLIENT_NET_API BOOL CALL_METHOD CLIENT_MakeKeyFrame(LLONG lLoginID, int nChannelID, int nSubChannel=0);

// public agency registration
typedef void (CALLBACK *fConnectMessCallBack)(LLONG lConnectHandle, NET_CLOUDSERVICE_CONNECT_RESULT* pConnectResult, void* pReserved, LDWORD dwUser);
//public agency registration, user malloc and free pConnectParm's memory,apply to sizeof(NET_CLOUDSERVICE_CONNECT_PARAM)
CLIENT_NET_API LLONG CALL_METHOD CLIENT_ConnectCloudService(LLONG lLoginID, NET_CLOUDSERVICE_CONNECT_PARAM* pConnectParm, fConnectMessCallBack pConnectMessCB, LDWORD dwUser, void* pReserved);
//------------------------------------------------------------------------

// Search user information,user malloc and free info's memory,apply to sizeof(USER_MANAGE_INFO) 
CLIENT_NET_API BOOL CALL_METHOD CLIENT_QueryUserInfo(LLONG lLoginID, USER_MANAGE_INFO *info, int waittime=1000);

// Search user information--extensive,user malloc and free info's memory,apply to sizeof(USER_MANAGE_INFO_EX) 
CLIENT_NET_API BOOL CALL_METHOD CLIENT_QueryUserInfoEx(LLONG lLoginID, USER_MANAGE_INFO_EX *info, int waittime=1000);

// Search device info--Max supports device of 64-ch
CLIENT_NET_API BOOL CALL_METHOD CLIENT_QueryUserInfoNew(LLONG lLoginID, USER_MANAGE_INFO_NEW *info, void* pReserved, int waittime = 1000);
// Device operation user,user malloc memory of opParam and subParam,please refer to nOperateType to ensure structure,then ensure memory size
CLIENT_NET_API BOOL CALL_METHOD CLIENT_OperateUserInfo(LLONG lLoginID, int nOperateType, void *opParam, void *subParam, int waittime=1000);

// Device operation user--extensive,user malloc memory of opParam and subParam,please refer to nOperateType to ensure structure,then ensure memory size
CLIENT_NET_API BOOL CALL_METHOD CLIENT_OperateUserInfoEx(LLONG lLoginID, int nOperateType, void *opParam, void *subParam, int waittime=1000);

// User operates the device--Max supports device of 64-ch,user malloc memory of opParam and subParam,please refer to nOperateType to ensure structure,then ensure memory size
CLIENT_NET_API BOOL CALL_METHOD CLIENT_OperateUserInfoNew(LLONG lLoginID, int nOperateType, void *opParam, void *subParam, void* pReserved, int waittime = 1000);

//------------------------------------------------------------------------

// Create transparent COM channel ,TransComType: high 2 bytes represent the serial number,low 2 bytes of serail type,type 0: serial,1:485
CLIENT_NET_API LLONG CALL_METHOD CLIENT_CreateTransComChannel(LLONG lLoginID, int TransComType, unsigned int baudrate, unsigned int databits, unsigned int stopbits, unsigned int parity, fTransComCallBack cbTransCom, LDWORD dwUser);

// Transparent COM send out data 
CLIENT_NET_API BOOL CALL_METHOD CLIENT_SendTransComData(LLONG lTransComChannel, char *pBuffer, DWORD dwBufSize);

// Release transparent COM channel 
CLIENT_NET_API BOOL CALL_METHOD CLIENT_DestroyTransComChannel(LLONG lTransComChannel);

// Query the status of a transparent serial port
CLIENT_NET_API BOOL CALL_METHOD CLIENT_QueryTransComParams(LLONG lLoginID, int TransComType, DH_COMM_STATE* pCommState, int nWaitTime = 500);

//------------------------------------------------------------------------

// Begin upgrading device program,user malloc pchFileName's memory,size is MAX_PATH 
CLIENT_NET_API LLONG CALL_METHOD CLIENT_StartUpgrade(LLONG lLoginID, char *pchFileName, fUpgradeCallBack cbUpgrade, LDWORD dwUser);

// Begin upgrading device program--extensive,user malloc pchFileName's memory,size is MAX_PATH 
CLIENT_NET_API LLONG CALL_METHOD CLIENT_StartUpgradeEx(LLONG lLoginID, EM_UPGRADE_TYPE emType, char *pchFileName, fUpgradeCallBack cbUpgrade, LDWORD dwUser);

// Begin upgrading device program--extensive,support large file  upgrade,for example G bytes file 
CLIENT_NET_API LLONG CALL_METHOD CLIENT_StartUpgradeEx2(LLONG lLoginID, EM_UPGRADE_TYPE emType, char *pchFileName, fUpgradeCallBackEx cbUpgrade, LDWORD dwUser);
// Send out data
CLIENT_NET_API BOOL CALL_METHOD CLIENT_SendUpgrade(LLONG lUpgradeID);

// Stop upgrading device program 
CLIENT_NET_API BOOL CALL_METHOD CLIENT_StopUpgrade(LLONG lUpgradeID);

//------------------------------------------------------------------------

// Search configuration information 
CLIENT_NET_API BOOL  CALL_METHOD CLIENT_GetDevConfig(LLONG lLoginID, DWORD dwCommand, LONG lChannel, LPVOID lpOutBuffer, DWORD dwOutBufferSize, LPDWORD lpBytesReturned,int waittime=500);

// Set configuration information 
CLIENT_NET_API BOOL  CALL_METHOD CLIENT_SetDevConfig(LLONG lLoginID, DWORD dwCommand, LONG lChannel, LPVOID lpInBuffer, DWORD dwInBufferSize, int waittime=500);

// New configuration interface, Search configuration information(using Json protocol, see configuration SDK)
CLIENT_NET_API BOOL CALL_METHOD CLIENT_GetNewDevConfig(LLONG lLoginID, char* szCommand, int nChannelID, char* szOutBuffer, DWORD dwOutBufferSize, int *error, int waittime=500);

// New configuration interface, Set configuration information(using Json protocol, see configuration SDK)
CLIENT_NET_API BOOL CALL_METHOD CLIENT_SetNewDevConfig(LLONG lLoginID, char* szCommand, int nChannelID, char* szInBuffer, DWORD dwInBufferSize, int *error, int *restart, int waittime=500);

// Delete configuration interface(Json format)
CLIENT_NET_API BOOL CALL_METHOD CLIENT_DeleteDevConfig(LLONG lLoginID, NET_IN_DELETECFG* pInParam, NET_OUT_DELETECFG* pOutParam, int waittime=500);

// Get the configuration member name interface(Json format)
CLIENT_NET_API BOOL CALL_METHOD CLIENT_GetMemberNames(LLONG lLoginID, NET_IN_MEMBERNAME* pInParam, NET_OUT_MEMBERNAME* pOutParam, int waittime=500);

// get net card info,user malloc the memory of lpInParam and lpOutParam,please refer to the corresponding structure of emType
CLIENT_NET_API BOOL CALL_METHOD CLIENT_QueryNetStat(LLONG lLoginID , EM_NET_QUERY_TYPE emType , void *lpInParam , int nInParamLen , void *lpOutParam , int nOutParamLen , int *pError = NULL , int nWaitTime=1000);

// Get VideoInAnalyse info(users apply for and release the memory of szOutBuffer)
CLIENT_NET_API BOOL CALL_METHOD CLIENT_GetVideoInAnalyse(LLONG 	lLoginID, char* szCommand, int nChannelID, EM_SCENE_CLASS_TYPE emClassType, char* szOutBuffer, DWORD dwOutBufferSize, int *error, int waittime);

// Get Event link name(users apply for and release the memory of pInParam and pOutParam)
CLIENT_NET_API BOOL CALL_METHOD CLIENT_GetEventLink(LLONG lLoginID, NET_IN_EVENT_LINK* pInParam, NET_OUT_EVENT_LINK* pOutParam, int nWaitTime);

//------------------------------------------------------------------------

// Search device channel name 
CLIENT_NET_API BOOL CALL_METHOD CLIENT_QueryChannelName(LLONG lLoginID, char *pChannelName, int maxlen, int *nChannelCount, int waittime=1000);

// Set device channel name,user malloc pbuf's memory,size is 512 byte at least
CLIENT_NET_API BOOL  CALL_METHOD CLIENT_SetupChannelName(LLONG lLoginID,char *pbuf, int nbuflen);

// Set device channel character overlay 
CLIENT_NET_API BOOL  CALL_METHOD CLIENT_SetupChannelOsdString(LLONG lLoginID, int nChannelNo, DH_CHANNEL_OSDSTRING* struOsdString, int nbuflen);

// Search device current time
CLIENT_NET_API BOOL CALL_METHOD CLIENT_QueryDeviceTime(LLONG lLoginID, LPNET_TIME pDeviceTime, int waittime=1000);

// Set device current time
CLIENT_NET_API BOOL CALL_METHOD CLIENT_SetupDeviceTime(LLONG lLoginID, LPNET_TIME pDeviceTime);

// Search device current time extension interface
CLIENT_NET_API BOOL CALL_METHOD CLIENT_QueryDeviceTimeEx(LLONG lLoginID, LPNET_TIME_EX pDeviceTime, int waittime=1000);

// Set device current time extension interface
CLIENT_NET_API BOOL CALL_METHOD CLIENT_SetupDeviceTimeEx(LLONG lLoginID, LPNET_TIME_EX pDeviceTime);

// Set device max bit stream
CLIENT_NET_API BOOL CALL_METHOD CLIENT_SetMaxFlux(LLONG lLoginID, WORD wFlux);

//------------------------------------------------------------------------

// Import configuration file,user malloc szFileName's memory,size is MAX_PATH
CLIENT_NET_API LLONG CALL_METHOD CLIENT_ImportConfigFile(LLONG lLoginID, char *szFileName, fDownLoadPosCallBack cbUploadPos, LDWORD dwUserData, DWORD param=0);

// Stop importing configuration file
CLIENT_NET_API BOOL CALL_METHOD CLIENT_StopImportCfgFile(LLONG lImportHandle);

// Exporting configuration file
CLIENT_NET_API LLONG CALL_METHOD CLIENT_ExportConfigFile(LLONG lLoginID, DH_CONFIG_FILE_TYPE emConfigFileType, char *szSavedFilePath, fDownLoadPosCallBack cbDownLoadPos, LDWORD dwUserData);

// top exporting configuration file
CLIENT_NET_API BOOL CALL_METHOD CLIENT_StopExportCfgFile(LLONG lExportHandle);

//------------------------------------------------------------------------

// Search device IP in DDBS by device name or device serial number
CLIENT_NET_API BOOL CALL_METHOD CLIENT_GetDVRIPByResolveSvr(char *pchDVRIP, WORD wDVRPort, BYTE *sDVRName, WORD wDVRNameLen, BYTE *sDVRSerialNumber, WORD wDVRSerialLen, char* sGetIP);

// Search IPC,NVS and etc in LAN 
CLIENT_NET_API BOOL CALL_METHOD CLIENT_SearchDevices(char* szBuf, int nBufLen, int* pRetLen, DWORD dwSearchTime, char* szLocalIp=NULL);

// asynchronism search IPC, NVS and etc in LAN,user malloc pUserData means user data: not support multi-thread
CLIENT_NET_API LLONG CALL_METHOD CLIENT_StartSearchDevices(fSearchDevicesCB cbSearchDevices, void* pUserData, char* szLocalIp=NULL);

// stop asynchronism search IPC, NVS and etc in LAN
CLIENT_NET_API BOOL CALL_METHOD CLIENT_StopSearchDevices(LLONG lSearchHandle);

// modify Device ip: not support multi-thread
CLIENT_NET_API BOOL CALL_METHOD CLIENT_ModifyDevice(DEVICE_NET_INFO_EX *pDevNetInfo, DWORD dwWaitTime, int *iError = NULL, char* szLocalIp = NULL, void *reserved = NULL);

// search device ip cross VLAN
CLIENT_NET_API BOOL CALL_METHOD CLIENT_SearchDevicesByIPs(DEVICE_IP_SEARCH_INFO* pIpSearchInfo, fSearchDevicesCB cbSearchDevices, LDWORD dwUserData, char* szLocalIp, DWORD dwWaitTime);
CLIENT_NET_API BOOL CALL_METHOD CLIENT_SetDeviceSearchParam(const NET_DEVICE_SEARCH_PARAM* pstParam);
//------------------------------------------------------------------------

// Platform embedded interface
CLIENT_NET_API BOOL CALL_METHOD CLIENT_GetPlatFormInfo(LLONG lLoginID, DWORD dwCommand, int nSubCommand, int nParam, LPVOID lpOutBuffer, DWORD dwOutBufferSize, LPDWORD lpBytesReturned,int waittime=500);
// lpInBuffer managed by user
CLIENT_NET_API BOOL CALL_METHOD CLIENT_SetPlatFormInfo(LLONG lLoginID, DWORD dwCommand, int nSubCommand, int nParam, LPVOID lpInBuffer, DWORD dwInBufferSize, int waittime=500);

// control focus 
//	dwFocusCommand = 0 focus adjucy
//	dwFocusCommand = 1continuous focus adjustment
//	dwFocusCommand = 2 auto adjust ,adjust to the best position,nFocus and nZoominvalid
CLIENT_NET_API BOOL CALL_METHOD CLIENT_FocusControl(LLONG lLoginID, int nChannelID, DWORD dwFocusCommand, double nFocus, double nZoom, void *reserved = NULL, int waittime=500);

///////////////////////////////Mobile DVR interface///////////////////////////////

// Set snapshot callback function 
CLIENT_NET_API void CALL_METHOD CLIENT_SetSnapRevCallBack(fSnapRev OnSnapRevMessage, LDWORD dwUser);

// Snapshot request
CLIENT_NET_API BOOL CALL_METHOD CLIENT_SnapPicture(LLONG lLoginID, SNAP_PARAMS par);

// Snapshot request--extensive
CLIENT_NET_API BOOL CALL_METHOD CLIENT_SnapPictureEx(LLONG lLoginID, SNAP_PARAMS *par, int *reserved = 0);

// Set GPS subscription callback function,user malloc and free memory of pInParam and  pOutParam
CLIENT_NET_API BOOL CALL_METHOD CLIENT_SnapPictureToFile(LLONG lLoginID, const NET_IN_SNAP_PIC_TO_FILE_PARAM* pInParam, NET_OUT_SNAP_PIC_TO_FILE_PARAM* pOutParam, int nWaitTime);
CLIENT_NET_API void CALL_METHOD CLIENT_SetSubcribeGPSCallBack(fGPSRev OnGPSMessage, LDWORD dwUser);

// Set GPS subscription callback function - extensive
CLIENT_NET_API void CALL_METHOD CLIENT_SetSubcribeGPSCallBackEX(fGPSRevEx OnGPSMessage, LDWORD dwUser);

// Set GPS subscription callback function - extensive2
CLIENT_NET_API void CALL_METHOD CLIENT_SetSubcribeGPSCallBackEX2(fGPSRevEx2 OnGPSMessage, LDWORD dwUser);

/**************************************************************************************
*   Funcname: CLIENT_SubcribeGPS
*   Purpose:GPS inforamtion subscribe
*   InputParam:   LLONG  :lLoginID    //login handle
*   InputParam:   BOOL   :bStart      //TRUE:subscribe  FALSE:cancle subscribe
*   InputParam:   LONG   :KeepTime    // subscribe time last (unit second) value:-1  means indefinite duration last


*   InputParam:   LONG   :InterTime   // GPS send rate in subscribe time
*   Return: BOOL
**************************************************************************************/
CLIENT_NET_API BOOL CALL_METHOD CLIENT_SubcribeGPS (LLONG lLoginID, BOOL bStart, LONG KeepTime, LONG InterTime);

// Set GPS subscription of temperature and humidity callback function
CLIENT_NET_API void CALL_METHOD CLIENT_SetSubcribeGPSTHCallBack(fGPSTempHumidityRev OnGPSMessage, LDWORD dwUser);

// GPS subscription of temperature and humidity
CLIENT_NET_API BOOL CALL_METHOD CLIENT_SubcribeGPSTempHumidity (LLONG lLoginID, BOOL bStart,	int InterTime, void* Reserved);

//GPS log query
CLIENT_NET_API BOOL CALL_METHOD CLIENT_QueryGPSLog(LLONG lLoginID,QUERY_GPS_LOG_PARAM *pQueryParam, char *pLogBuffer, int nLogBufferLen, int *pRecLogNum, BOOL *bContinue, int waittime);

// subscription task
CLIENT_NET_API BOOL CALL_METHOD CLIENT_AttachMission(LLONG lLoginID, NET_IN_ATTACH_MISSION_PARAM *pInParam, NET_OUT_ATTACH_MISSION_PARAM *pOutParam, int nWaitTime);

// cancen subscription
CLIENT_NET_API BOOL CALL_METHOD CLIENT_DetachMission(LLONG lAttachHandle);

// vehicle emergency alarm confirm
CLIENT_NET_API BOOL CALL_METHOD CLIENT_BusConfirmEvent(LLONG lLoginID, const NET_IN_BUS_CONFIRM_EVENT* pInParam, NET_OUT_BUS_CONFIRM_EVENT* pOutParam, int nWaitTime = 3000);

// set device address information
CLIENT_NET_API BOOL CALL_METHOD CLIENT_SetDevicePosition(LLONG lLoginID, const NET_IN_SET_DEVICE_POSITION* pInParam, NET_OUT_SET_DEVICE_POSITION* pOutParam, int nWaitTime);

// get device address information
CLIENT_NET_API BOOL CALL_METHOD CLIENT_GetDevicePosition(LLONG lLoginID, const NET_IN_GET_DEVICE_POSITION* pInParam, NET_OUT_GET_DEVICE_POSITION* pOutParam, int nWaitTime);

// snapshoot by time mode 
CLIENT_NET_API BOOL CALL_METHOD CLIENT_SnapPictureByEvent(LLONG lLoginID, const NET_IN_SNAP_BY_EVENT* pInParam, NET_OUT_SNAP_BY_EVENT* pOutParam, int nWaitTime);
//////////////////////////////NVD interface//////////////////////////////

// Query decoder information
CLIENT_NET_API BOOL CALL_METHOD CLIENT_QueryDecoderInfo(LLONG lLoginID, LPDEV_DECODER_INFO lpDecInfo, int waittime=1000);

// Query decoder TV information
CLIENT_NET_API BOOL CALL_METHOD CLIENT_QueryDecoderTVInfo(LLONG lLoginID, int nMonitorID, LPDEV_DECODER_TV lpMonitorInfo, int waittime=1000);

// Query decoder channel information
CLIENT_NET_API BOOL CALL_METHOD CLIENT_QueryDecEncoderInfo(LLONG lLoginID, int nDecoderID, LPDEV_ENCODER_INFO lpEncoderInfo, int waittime=1000);

// Set decoder TV enable
CLIENT_NET_API BOOL CALL_METHOD CLIENT_SetDecTVOutEnable(LLONG lLoginID, BYTE *pDecTVOutEnable, int nBufLen, int waittime=1000);

// set decoder tip layout enable, channel number start at 0
CLIENT_NET_API BOOL CALL_METHOD CLIENT_SetDecLayOutEnable(LLONG lLoginID, BYTE bDecLayOutEnable, int nChannel, int waittime=1000);
CLIENT_NET_API BOOL CALL_METHOD CLIENT_GetDecLayOutEnable(LLONG lLoginID, BYTE *pDecLayOutEnable, int nChannel, int waittime=1000);
//------------------------------------------------------------------------

// Set up asynchronous callback function
CLIENT_NET_API BOOL CALL_METHOD CLIENT_SetOperateCallBack(LLONG lLoginID, fMessDataCallBack cbMessData, LDWORD dwUser);

// Control decoder TV screen partition,Interface is asynchronous
CLIENT_NET_API LLONG CALL_METHOD CLIENT_CtrlDecTVScreen(LLONG lLoginID, int nMonitorID, BOOL bEnable, int nSplitType, BYTE *pEncoderChannel, int nBufLen, void* userdata=NULL);

// Switch decoder TV screen,Interface is asynchronous
// According to nMonitorID(TV ID) nWndID(Screen ID) Convert to nDecoderID(decoder channel) formula:nEncoderID nMonitorID*nSplitNum(Partition number)+nWndID
CLIENT_NET_API LLONG CALL_METHOD CLIENT_SwitchDecTVEncoder(LLONG lLoginID, int nDecoderID, LPDEV_ENCODER_INFO lpEncoderInfo, void* userdata=NULL);

//------------------------------------------------------------------------

// Add combination of screen
CLIENT_NET_API int CALL_METHOD CLIENT_AddTourCombin(LLONG lLoginID, int nMonitorID, int nSplitType, BYTE *pEncoderChannnel, int nBufLen, int waittime=1000);

// Delete combination of screen
CLIENT_NET_API BOOL CALL_METHOD CLIENT_DelTourCombin(LLONG lLoginID, int nMonitorID, int nCombinID, int waittime=1000);

// Modify combination of screen
CLIENT_NET_API BOOL CALL_METHOD CLIENT_SetTourCombin(LLONG lLoginID, int nMonitorID, int nCombinID, int nSplitType, BYTE *pEncoderChannel, int nBufLen, int waittime=1000);

// Query combination of screen,nCombinID: 0-32
CLIENT_NET_API BOOL CALL_METHOD CLIENT_QueryTourCombin(LLONG lLoginID, int nMonitorID, int nCombinID, LPDEC_COMBIN_INFO lpDecCombinInfo, int waittime=1000);

// Set up tour operation
CLIENT_NET_API BOOL CALL_METHOD CLIENT_SetDecoderTour(LLONG lLoginID, int nMonitorID, LPDEC_TOUR_COMBIN lpDecTourInfo, int waittime=1000);

// Query tour operation
CLIENT_NET_API BOOL CALL_METHOD CLIENT_QueryDecoderTour(LLONG lLoginID, int nMonitorID, LPDEC_TOUR_COMBIN lpDecTourInfo, int waittime=1000);

// Query the current flux information of decoding channel
CLIENT_NET_API BOOL CALL_METHOD CLIENT_QueryDecChannelFlux(LLONG lLoginID, int nDecoderID, LPDEV_DECCHANNEL_STATE lpChannelStateInfo, int waittime=1000);

// control decoder tour operation
CLIENT_NET_API BOOL CALL_METHOD CLIENT_CtrlDecoderTour(LLONG lLoginID, int nMonitorID, DEC_CTRL_TOUR_TYPE emActionParam, int waittime = 1000);
//------------------------------------------------------------------------

typedef void (CALLBACK *fDecPlayBackPosCallBack)(LLONG lLoginID, int nEncoderID, DWORD dwTotalSize, DWORD dwPlaySize, LDWORD dwUser);

// Set the playback progress callback function
CLIENT_NET_API BOOL CALL_METHOD CLIENT_SetDecPlaybackPos(LLONG lLoginID, fDecPlayBackPosCallBack cbPlaybackPos, LDWORD dwUser);

// Decoder TV screen playback,Interface is asynchronous
CLIENT_NET_API LLONG CALL_METHOD CLIENT_DecTVPlayback(LLONG lLoginID, int nDecoderID, DEC_PLAYBACK_MODE emPlaybackMode, LPVOID lpInBuffer, DWORD dwInBufferSize, void* userdata=NULL);

// Control TV screen playback
CLIENT_NET_API BOOL CALL_METHOD CLIENT_CtrlDecPlayback(LLONG lLoginID, int nDecoderID, DEC_CTRL_PLAYBACK_TYPE emCtrlType, int nValue, int waittime=1000);

///////////////////////////////intelligent device interface///////////////////////////////

// real load picture of intelligent analysis 
CLIENT_NET_API LLONG CALL_METHOD CLIENT_RealLoadPicture(LLONG lLoginID, int nChannelID, DWORD dwAlarmType, fAnalyzerDataCallBack cbAnalyzerData, LDWORD dwUser);

// real load picture of intelligent analysis(expand interface: 'bNeedPicFile == true' instruct load picture file, 'bNeedPicFile == false' instruct not load picture file ) 
CLIENT_NET_API LLONG CALL_METHOD CLIENT_RealLoadPictureEx(LLONG lLoginID, int nChannelID, 
													 DWORD dwAlarmType, 
													 BOOL bNeedPicFile, 
													 fAnalyzerDataCallBack cbAnalyzerData, 
													 LDWORD dwUser, 
													 void* Reserved);

// stop load picture of intelligent analysis
CLIENT_NET_API BOOL CALL_METHOD CLIENT_StopLoadPic(LLONG lAnalyzerHandle);

// Confirm if picture upload is complete--picture
// This port better not be in the same thread with  CLIENT_RealLoadPictureEx/CLIENT_RealLoadPicture's call function, otherwise will cause jam, and must call in another thread
CLIENT_NET_API BOOL CALL_METHOD CLIENT_SnapManagerConfirmUpload(LLONG lLoginID, const NET_IN_SNAP_MANAGER_CONFIRM_UPLOAD* pstInParam, NET_OUT_SNAP_MANAGER_CONFIRM_UPLOAD* pstOutParam, int waittime=1000);

// Search according to the criteria,user malloc and free pQueryCondition's memory,please refer to emType to ensure structure,then ensure memory size
CLIENT_NET_API LLONG	CALL_METHOD CLIENT_FindFileEx(LLONG lLoginID, EM_FILE_QUERY_TYPE emType, void* pQueryCondition, void *reserved, int waittime=1000);	

// Search file:nFilecount:the searched amount. The return value is media file amount. The search in the specified time completed if return <nFilecount.
CLIENT_NET_API int	CALL_METHOD CLIENT_FindNextFileEx(LLONG lFindHandle, int nFilecount, void* pMediaFileInfo, int maxlen, void *reserved, int waittime=1000);

// End the file search
CLIENT_NET_API BOOL CALL_METHOD CLIENT_FindCloseEx(LLONG lFindHandle);

// get matched search filter file total
CLIENT_NET_API BOOL CALL_METHOD CLIENT_GetTotalFileCount(LLONG lFindHandle, int* pTotalCount, void *reserved, int waittime=1000);

// set search jump filter, user malloc and free pOption's memory
CLIENT_NET_API BOOL CALL_METHOD CLIENT_SetFindingJumpOption(LLONG lFindHandle, NET_FINDING_JUMP_OPTION_INFO* pOption, void *reserved, int waittime=1000);

// Download the specified intelligent analytics data-image,user malloc and free lpMediaFileInfo's memory,please refer to the corresponding structure of emType
CLIENT_NET_API LLONG CALL_METHOD CLIENT_DownloadMediaFile(LLONG lLoginID,EM_FILE_QUERY_TYPE emType, void* lpMediaFileInfo, char *sSavedFileName, fDownLoadPosCallBack cbDownLoadPos, LDWORD dwUserData,  void *reserved);

// Stop downloading the data
CLIENT_NET_API BOOL CALL_METHOD CLIENT_StopDownloadMediaFile(LLONG lFileHandle);

// batch download file
CLIENT_NET_API BOOL CALL_METHOD CLIENT_DownLoadMultiFile(LLONG lLoginID, NET_IN_DOWNLOAD_MULTI_FILE *pstInParam, NET_OUT_DOWNLOAD_MULTI_FILE *pstOutParam, int waittime=1000);

// stop download
CLIENT_NET_API BOOL CALL_METHOD CLIENT_StopLoadMultiFile(LLONG lDownLoadHandle);

// download picture of intelligent analysis when being off line
CLIENT_NET_API LLONG CALL_METHOD CLIENT_LoadOffLineFile(LLONG lLoginID, int nChannelID, DWORD dwAlarmType, LPNET_TIME_EX lpStartTime, LPNET_TIME_EX lpEndTime, fAnalyzerDataCallBack cbAnalyzerData, LDWORD dwUser);

//Pause IVS data download(bPause=TRUE, it is to stop download , bPause=FALSE , it is to resume download.)
CLIENT_NET_API BOOL CALL_METHOD CLIENT_PauseLoadPic(LLONG lLoadHadle, BOOL bPause);

// traffic snap--snapshot by network,user malloc and free (pstInParam's and pstOutParam's) memory
CLIENT_NET_API BOOL CALL_METHOD CLIENT_TrafficSnapByNetwork(LLONG lLoginID, int nChannelID, NET_IN_SNAPSHOT* pstInParam, NET_OUT_SNAPSHOT* pstOutParam);
// start traffic flux state
// traffic control --forced the red light,user malloc and free (pstInParam's and pstOutParam's) memory
CLIENT_NET_API BOOL CALL_METHOD CLIENT_TrafficForceLightState(LLONG lLoginID, int nChannelID, NET_IN_FORCELIGHTSTATE* pstInParamg, NET_OUT_FORCELIGHTSTATE* pstOutParam, int waittime=1000);

//  balck/white list operation,user malloc and free (pstInParam's and pstOutParam's) memory
CLIENT_NET_API BOOL CALL_METHOD CLIENT_OperateTrafficList(LLONG lLoginID ,  NET_IN_OPERATE_TRAFFIC_LIST_RECORD* pstInParam , NET_OUT_OPERATE_TRAFFIC_LIST_RECORD *pstOutParam = NULL ,  int waittime = 1000);

// start traffic flux state,user malloc and free (pstInParam's and pstOutParam's) memory
CLIENT_NET_API LLONG CALL_METHOD CLIENT_StartTrafficFluxStat(LLONG lLoginID, NET_IN_TRAFFICFLUXSTAT* pstInParam,NET_OUT_TRAFFICFLUXSTAT* pstOutParam);

// stop traffic flux state
CLIENT_NET_API BOOL CALL_METHOD CLIENT_StopTrafficFluxStat(LLONG lFluxStatHandle);

// start find flux state,user malloc and free (pstInParam's and pstOutParam's) memory
CLIENT_NET_API LLONG CALL_METHOD CLIENT_StartFindFluxStat(LLONG lLoginID, NET_IN_TRAFFICSTARTFINDSTAT* pstInParam, NET_OUT_TRAFFICSTARTFINDSTAT* pstOutParam);

// do find flux state,user malloc and free (pstInParam's and pstOutParam's) memory
CLIENT_NET_API int	CALL_METHOD CLIENT_DoFindFluxStat(LLONG lFindHandle, NET_IN_TRAFFICDOFINDSTAT* pstInParam,NET_OUT_TRAFFICDOFINDSTAT* pstOutParam);

// stop find flux state
CLIENT_NET_API BOOL CALL_METHOD CLIENT_StopFindFluxStat(LLONG lFindHandle);
// start find number state,user malloc and free (pstInParam's and pstOutParam's) memory
CLIENT_NET_API LLONG CALL_METHOD CLIENT_StartFindNumberStat(LLONG lLoginID, NET_IN_FINDNUMBERSTAT* pstInParam, NET_OUT_FINDNUMBERSTAT* pstOutParam);

// do find number state,user malloc and free (pstInParam's and pstOutParam's) memory
CLIENT_NET_API int	CALL_METHOD CLIENT_DoFindNumberStat(LLONG lFindHandle, NET_IN_DOFINDNUMBERSTAT* pstInParam, NET_OUT_DOFINDNUMBERSTAT* pstOutParam);

// stop find number state
CLIENT_NET_API BOOL CALL_METHOD CLIENT_StopFindNumberStat(LLONG lFindHandle);

// subscribe video statistical summary,user malloc and free (pstInParam's and pstOutParam's) memory
CLIENT_NET_API LLONG CALL_METHOD CLIENT_AttachVideoStatSummary(LLONG lLoginID, const NET_IN_ATTACH_VIDEOSTAT_SUM* pInParam, NET_OUT_ATTACH_VIDEOSTAT_SUM* pOutParam, int nWaitTime);

// unsubscribe video statistical summary
CLIENT_NET_API BOOL CALL_METHOD CLIENT_DetachVideoStatSummary(LLONG lAttachHandle);

// Call of the analysis device method,user malloc and free (pstInParam's and pstOutParam's) memory,please refer to the corresponding structure of szCmd
CLIENT_NET_API BOOL CALL_METHOD CLIENT_OperateVideoAnalyseDevice(LLONG lLoginID, int nChannelID, char* szCmd, void *pstInParam, void *pstOutParam, int waittime=1000);

// Access to video analysis object,user malloc and free (pstInParam's and pstOutParam's) memory
CLIENT_NET_API BOOL CALL_METHOD CLIENT_VideoAnalyseGetTemplateImage(LLONG lLoginID, const NET_IN_VIDEOANALYSE_GETTEMPLATEIMAGE* pstInParam, NET_OUT_VIDEOANALYSE_GETTEMPLATEIMAGE* pstOutParam, int nWaitTime=1000);

// Set the video analysis object,user malloc and free (pstInParam's and pstOutParam's) memory
CLIENT_NET_API BOOL CALL_METHOD CLIENT_VideoAnalyseSetTemplateImage(LLONG lLoginID, const NET_IN_VIDEOANALYSE_SETTEMPLATEIMAGE* pstInParam, NET_OUT_VIDEOANALYSE_SETTEMPLATEIMAGE* pstOutParam, int nWaitTime=1000);

// serial device method call
CLIENT_NET_API BOOL CALL_METHOD CLIENT_OperateCommDevice(LLONG lLoginID, int nChannelID, char* szCmd, void *pstInParam, void *pstOutParam, int waittime=1000);

// Intelligent track speed dome control port.
CLIENT_NET_API BOOL CALL_METHOD CLIENT_ControlIntelliTracker(LLONG lLoginID, NET_IN_CONTROL_INTELLITRACKER* pstInParam, NET_OUT_CONTROL_INTELLITRACKER* pstOutParam);

// master-slave device method,look for CLIENT_OperateMasterSlaveDevice,user malloc and free (pstInParam's and pstOutParam's) memoryp,please refer to the corresponding structure of szCmd
CLIENT_NET_API BOOL CALL_METHOD CLIENT_OperateMasterSlaveDevice(LLONG lLoginID, int nChannelID, char* szCmd, void *pstInParam, void *pstOutParam, int waittime=1000);

// video splicing method call,user malloc and free (pstInParam's and pstOutParam's) memory,please refer to the corresponding structure of emType
CLIENT_NET_API BOOL CALL_METHOD CLIENT_OperateVideoJoin(LLONG lLoginID, NET_VIDEOJOIN_OPERATE_TYPE emType, void* pstInParam, void* pstOutParam, int nWaitTime);

////////////////////////////////  Video analysis /////////////////////////////////
// Real-time get video analysis result,user malloc and free (pstInParam's and pstOutParam's) memory
CLIENT_NET_API BOOL CALL_METHOD CLIENT_StartVideoDiagnosis(LLONG lLoginID, NET_IN_VIDEODIAGNOSIS *pstInParam, NET_OUT_VIDEODIAGNOSIS *pstOutParam);

// Stop video analysis result report
CLIENT_NET_API BOOL CALL_METHOD CLIENT_StopVideoDiagnosis(LLONG hDiagnosisHandle);

// Start video analysis result search,user malloc and free (pstInParam's and pstOutParam's) memory
CLIENT_NET_API BOOL CALL_METHOD CLIENT_StartFindDiagnosisResult(LLONG lLoginID, NET_IN_FIND_DIAGNOSIS* pstInParam, NET_OUT_FIND_DIAGNOSIS* pstOutParam);

// Get video analysis result info,user malloc and free (pstInParam's and pstOutParam's) memory
CLIENT_NET_API BOOL CALL_METHOD CLIENT_DoFindDiagnosisResult(LLONG hFindHandle,NET_IN_DIAGNOSIS_INFO* pstInParam, NET_OUT_DIAGNOSIS_INFO* pstOutParam);

// End video analysis result search
CLIENT_NET_API BOOL CALL_METHOD CLIENT_StopFindDiagnosis(LLONG hFindHandle);

// enable real time disgnosis plan,user malloc and free (pstInParam's and pstOutParam's) memory
CLIENT_NET_API BOOL CALL_METHOD CLIENT_StartRealTimeProject(LLONG lLoginID,NET_IN_START_RT_PROJECT_INFO* pstInParam, NET_OUT_START_RT_PROJECT_INFO* pstOutParam, int nWaitTime);

// stop real time diagnosis plan,user malloc and free (pstInParam's and pstOutParam's) memory
CLIENT_NET_API BOOL CALL_METHOD CLIENT_StopRealTimeProject(LLONG lLoginID,NET_IN_STOP_RT_PROJECT_INFO* pstInParam, NET_OUT_STOP_RT_PROJECT_INFO* pstOutParam, int nWaitTime);

// get parking space status,user malloc and free (pstInParam's and pstOutParam's) memory
CLIENT_NET_API BOOL CALL_METHOD CLIENT_GetParkingSpaceStatus(LLONG lLoginID, NET_IN_GET_PARKINGSPACE_STATUS* pstInParam, NET_OUT_GET_PARKINGSPACE_STATUS* pstOutParam);

// attach parking space data,user malloc and free (pstInParam's and pstOutParam's) memory
CLIENT_NET_API LLONG CALL_METHOD CLIENT_AttachParkingSpaceData(LLONG lLoginID, NET_IN_ATTACH_PARKINGSPACE* pstInParam, NET_OUT_ATTACH_PARKINGSPACE* pstOutParam);

// detach parking space data,user malloc and free (pstInParam's and pstOutParam's) memory
CLIENT_NET_API BOOL CALL_METHOD CLIENT_DetachParkingSpaceData(NET_IN_DETACH_PARKINGSPACE* pstInParam, NET_OUT_DETACH_PARKINGSPACE* pstOutParam);

// Subscribed to the car information,user malloc and free (pstInParam's and pstOutParam's) memory
CLIENT_NET_API LLONG CALL_METHOD CLIENT_AttachCarPassInfo(LLONG lLoginID, NET_IN_ATTACH_CAR_PASS_INFO* pstInParam, NET_OUT_ATTACH_CAR_PASS_INFO* pstOutParam , int nWaitTime);

// Unsubscribe car information lCarPassInfoHandle is CLIENT_AttachCarPassInfo returns a handle 
CLIENT_NET_API BOOL CALL_METHOD CLIENT_DetachCarPassInfo(LLONG lCarPassInfoHandle);

// order specific lane info ,user malloc and free (pstInParam's and pstOutParam's) memory
CLIENT_NET_API LLONG CALL_METHOD CLIENT_AttachLanesState(LLONG lLoginID, NET_IN_ATTACH_LANES_STATE* pstInParam, NET_OUT_ATTACH_LANES_STATE* pstOutParam, int nWaitTime);

// cancel order of specific lane info lLanesStateHandle is CLIENT_AttachLanesStatereturn handle
CLIENT_NET_API BOOL CALL_METHOD CLIENT_DetachLanesState(LLONG lLanesStateHandle);

// get car port light status,user malloc and free (pstInParam's and pstOutParam's) memory
CLIENT_NET_API BOOL CALL_METHOD CLIENT_GetCarPortLightStatus(LLONG lLoginID, NET_IN_GET_CARPORTLIGHT_STATUS* pstInParam, NET_OUT_GET_CARPORTLIGHT_STATUS* pstOutParam, int waittime=1000);

// ser car port light status,user malloc and free (pstInParam's and pstOutParam's) memory
CLIENT_NET_API BOOL CALL_METHOD CLIENT_SetCarPortLightStatus(LLONG lLoginID, NET_IN_SET_CARPORTLIGHT_STATUS* pstInParam, NET_OUT_SET_CARPORTLIGHT_STATUS* pstOutParam, int waittime=1000);

// subscribe intelligent analysis porogress, for video analysis source is record file, ,user malloc and free (pstInParam's and pstOutParam's) memory
CLIENT_NET_API BOOL CALL_METHOD CLIENT_AttachVideoAnalyseState(LLONG lLoginID, NET_IN_ATTACH_VIDEOANALYSE_STATE* pstInParam, NET_OUT_ATTACH_VIDEOANALYSE_STATE* pstOutParam, int nWaittime=1000);

// stop subscription
CLIENT_NET_API BOOL CALL_METHOD CLIENT_DetachVideoAnalyseState(LLONG lAttachHandle);

// subscribe front-end device talk state,user malloc and free (pstInParam's and pstOutParam's) memory
CLIENT_NET_API LLONG CALL_METHOD CLIENT_AttachTalkState(LLONG lLoginID, NET_IN_ATTACH_TALK_STATE* pstInParam, NET_OUT_ATTACH_TALK_STATE* pstOutParam, int nWaitTime = 1000);

// cancel subscribing front-end device talk state, lAttachHandle is the returned handle of  CLIENT_AttachTalkState
CLIENT_NET_API BOOL CALL_METHOD CLIENT_DetachTalkState(LLONG lAttachHandle);

////////////////////////////////synopsis video' interface////////////////////////////////

// add video synopsis task,user malloc and free (pstInParam's and pstOutParam's) memory
CLIENT_NET_API BOOL CALL_METHOD CLIENT_AddVideoSynopsisTask(LLONG lLoginID,	NET_IN_ADD_VIDEOSYNOPSIS* pstInParam, NET_OUT_ADD_VIDEOSYNOPSIS* pstOutParam);

// video Synopsis task pretreatment,user malloc and free (pstInParam's and pstOutParam's) memory
CLIENT_NET_API BOOL CALL_METHOD CLIENT_PreHandleVideoSynopsisTask(LLONG lLoginID,  const NET_IN_PRE_HANDLE_VIDEOSYNOPSIS* pstInParam, NET_OUT_PRE_HANDLE_VIDEOSYNOPSIS* pstOutParam);
// video Synopsis task handle ,user malloc and free (pstInParam's and pstOutParam's) memory
CLIENT_NET_API BOOL CALL_METHOD CLIENT_RunVideoSynopsisTask(LLONG lLoginID, const NET_IN_RUN_VIDEOSYNOPSIS_TASK* pstInParam, NET_OUT_RUN_VIDEOSYNOPSIS_TASK* pstOutParam);
// pause video Synopsis task,user malloc and free pstInParam's memory
CLIENT_NET_API BOOL CALL_METHOD CLIENT_PauseVideoSynopsisTask(LLONG lLoginID, NET_IN_PAUSE_VIDEOSYNOPSIS* pstInParam);

// remove video synopsis task,user malloc and free pstInParam's memory
CLIENT_NET_API BOOL CALL_METHOD CLIENT_RemoveVideoSynopsisTask(LLONG lLoginID, NET_IN_REMOVE_VIDEOSYNOPSIS* pstInParam);

// subscibe real load object data,user malloc and free (pstInParam's and pstOutParam's) memory
CLIENT_NET_API BOOL CALL_METHOD CLIENT_RealLoadObjectData(LLONG lLoginID, NET_IN_REALLOAD_OBJECTDATA* pstInParam, NET_OUT_REALLOAD_OBJECTDATA* pstOutParam);

// stop load object data,user malloc and free pstInParam's memory
CLIENT_NET_API BOOL CALL_METHOD CLIENT_StopLoadObjectData(LLONG lRealLoadHandle, NET_IN_STOPLOAD_OBJECTDATA* pstInParam);

// subscribe real synopsis progress state,user malloc and free (pstInParam's and pstOutParam's) memory
CLIENT_NET_API BOOL CALL_METHOD CLIENT_RealLoadSynopsisState(LLONG lLoginID, NET_IN_REALLAOD_SYNOPSISSTATE* pstInParam, NET_OUT_REALLOAD_SYNOPSISSTATE* pstOutParam);

// stop subscribe real synopsis progress state,user malloc and free pstInParam's memory
CLIENT_NET_API BOOL CALL_METHOD CLIENT_StopLoadSynopsisState(LLONG lRealLoadHandle, NET_IN_STOPLOAD_SYNOPSISSTATE* pstInParam);

// query real synopsis video state,user malloc and free (pstInParam's and pstOutParam's) memory
CLIENT_NET_API BOOL CALL_METHOD CLIENT_QueryVideoSynopsisInfo(LLONG lLoginID, NET_IN_QUERY_VIDEOSYNOPSIS* pstInParam, NET_OUT_QUERY_VIDEOSYNOPSIS* pstuOutParam);

// according to the query criteria query synopsis file,user malloc and free (pstInParam's and pstOutParam's) memory
CLIENT_NET_API BOOL	CALL_METHOD CLIENT_FindSynopsisFile(LLONG lLoginID, NET_IN_FIND_SYNOPSISFILE *pstInParam, NET_OUT_FIND_SYNOPSISFILE *pstOutParam);	

// query synopsis file,user malloc and free (pstInParam's and pstOutParam's) memory
CLIENT_NET_API BOOL	CALL_METHOD CLIENT_FindNextSynopsisFile(LLONG lFindHandle, NET_IN_FINDNEXT_SYNOPSISFILE *pstInParam, NET_OUT_FINDNEXT_SYNOPSISFILE *pstOutParam);

// close query synopsis file
CLIENT_NET_API BOOL CALL_METHOD CLIENT_SynopsisFindClose(LLONG lFindHandle);

// download synosis file,user malloc and free (pstInParam's and pstOutParam's) memory
CLIENT_NET_API BOOL CALL_METHOD CLIENT_DownLoadSynosisFile(LLONG lLoginID, NET_IN_DOWNLOAD_SYNOPSISFILE *pstInParam, NET_OUT_DOWNLOAD_SYNOPSISFILE *pstOutParam);

// stop load synosis file
CLIENT_NET_API BOOL CALL_METHOD CLIENT_StopLoadSynosisFile(LLONG lDownLoadHandle);

// according to the path to the file request video service,generate the file information,,user malloc and free pstInParam's memory
CLIENT_NET_API BOOL	CALL_METHOD	CLIENT_SetFilePathInfo(LLONG lLoginID, NET_IN_SET_FILEPATHINFO* pstInParam);

// attach add file state,user malloc and free (pstInParam's and pstOutParam's) memory
CLIENT_NET_API LLONG CALL_METHOD CLIENT_AttachAddFileState(LLONG lLoginID, const NET_IN_ADDFILE_STATE* pstInParam, NET_OUT_ADDFILE_STATE *pstOutParam, int nWaitTime = 1000);

// cancel attach add file state,  CLIENT_AttacAddFileState return lAttachHandle
CLIENT_NET_API LLONG CALL_METHOD CLIENT_DetachAddFileState(LLONG lAttachHandle);

/////////////////////////////////face recognition port/////////////////////////////////////////
// face recognition database info operation, including add and delete, ,user malloc and free (pstInParam's and pstOutParam's) memory
CLIENT_NET_API BOOL CALL_METHOD CLIENT_OperateFaceRecognitionDB(LLONG lLoginID, const NET_IN_OPERATE_FACERECONGNITIONDB* pstInParam, NET_OUT_OPERATE_FACERECONGNITIONDB *pstOutParam, int nWaitTime = 1000);

// by filter search face recognition result ,user malloc and free (pstInParam's and pstOutParam's) memory
CLIENT_NET_API BOOL CALL_METHOD CLIENT_StartFindFaceRecognition(LLONG lLoginID, const NET_IN_STARTFIND_FACERECONGNITION* pstInParam, NET_OUT_STARTFIND_FACERECONGNITION *pstOutParam, int nWaitTime = 1000);

// start face detection / registry multi channel query
CLIENT_NET_API BOOL CALL_METHOD CLIENT_StartMultiFindFaceRecognition(LLONG lLoginID, const NET_IN_STARTMULTIFIND_FACERECONGNITION* pstInParam, NET_OUT_STARTMULTIFIND_FACERECONGNITION *pstOutParam, int nWaitTime);

// search face recognition result:nFilecount: need search item,  return value is media file item  return value<nFilecount then corresponding period file search complete(search max of 20 records each time)
// user malloc and free (pstInParam's and pstOutParam's) memory
CLIENT_NET_API BOOL CALL_METHOD CLIENT_DoFindFaceRecognition(const NET_IN_DOFIND_FACERECONGNITION* pstInParam, NET_OUT_DOFIND_FACERECONGNITION *pstOutParam, int nWaitTime = 1000);

// end search
CLIENT_NET_API BOOL CALL_METHOD CLIENT_StopFindFaceRecognition(LLONG lFindHandle);

// multimodal face recognition
CLIENT_NET_API BOOL CALL_METHOD CLIENT_StartMultiFindFaceRecognitionRecord(LLONG lLoginID, const NET_IN_STARTMULTIFIND_FACERECONGNITIONRECORD* pstInParam, NET_OUT_STARTMULTIFIND_FACERECONGNITIONRECORD *pstOutParam, int nWaitTime);

// multi channel Search file:nFilecount:the searched amount. The return value is media file amount. The search in the specified time completed if return <nFilecount.
CLIENT_NET_API BOOL CALL_METHOD CLIENT_DoFindFaceRecognitionRecord(const NET_IN_DOFIND_FACERECONGNITIONRECORD* pstInParam, NET_OUT_DOFIND_FACERECONGNITIONRECORD *pstOutParam, int nWaitTime);

// end search
CLIENT_NET_API BOOL CALL_METHOD CLIENT_StopFindFaceRecognitionRecord(LLONG lFindHandle);

// face recognition(input one big graph, input big graph detected face picture),user malloc and free (pstInParam's and pstOutParam's) memory
CLIENT_NET_API BOOL CALL_METHOD CLIENT_DetectFace(LLONG lLoginID, const NET_IN_DETECT_FACE* pstInParam, NET_OUT_DETECT_FACE *pstOutParam, int nWaitTime = 1000);

// face recognition staff group operation, including add, modify and delete, ,user malloc and free (pstInParam's and pstOutParam's) memory
CLIENT_NET_API BOOL CALL_METHOD CLIENT_OperateFaceRecognitionGroup(LLONG lLoginID, const NET_IN_OPERATE_FACERECONGNITION_GROUP* pstInParam, NET_OUT_OPERATE_FACERECONGNITION_GROUP *pstOutParam, int nWaitTime = 1000);

// face recognition person type operation, including add, modify and delete, user malloc and free (pstInParam's and pstOutParam's) memory
CLIENT_NET_API BOOL CALL_METHOD CLIENT_OperateFaceRecognitionPersonType(LLONG lLoginID, const NET_IN_OPERATE_FACERECOGNITION_PERSON_TYPE* pstInParam, NET_OUT_OPERATE_FACERECOGNITION_PERSON_TYPE *pstOutParam, int nWaitTime);

// face recognition get person type, user malloc and free (pstInParam's and pstOutParam's) memory
CLIENT_NET_API BOOL CALL_METHOD CLIENT_GetFaceRecognitionPersonType(LLONG lLoginID, const NET_IN_GET_FACERECOGNITION_PERSON_TYPE* pstInParam, NET_OUT_GET_FACERECOGNITION_PERSON_TYPE *pstOutParam, int nWaitTime);

// search face recognition staff group info ,user malloc and free (pstInParam's and pstOutParam's) memory
CLIENT_NET_API BOOL CALL_METHOD CLIENT_FindGroupInfo(LLONG lLoginID, const NET_IN_FIND_GROUP_INFO* pstInParam, NET_OUT_FIND_GROUP_INFO *pstOutParam, int nWaitTime = 1000);

// arm channel staff group info ,user malloc and free (pstInParam's and pstOutParam's) memory
CLIENT_NET_API BOOL CALL_METHOD CLIENT_SetGroupInfoForChannel(LLONG lLoginID, const NET_IN_SET_GROUPINFO_FOR_CHANNEL* pstInParam, NET_OUT_SET_GROUPINFO_FOR_CHANNEL *pstOutParam, int nWaitTime = 1000);

// set picture information,user malloc and free (pstInParam's and pstOutParam's) memory
CLIENT_NET_API LLONG CALL_METHOD CLIENT_AttachFaceFindState(LLONG lLoginID, const NET_IN_FACE_FIND_STATE* pstInParam, NET_OUT_FACE_FIND_STATE *pstOutParam, int nWaitTime = 1000);
CLIENT_NET_API BOOL CALL_METHOD CLIENT_DetachFaceFindState(LLONG lAttachHandle);
CLIENT_NET_API BOOL CALL_METHOD CLIENT_FaceRecognitionSetSearchImageInfo(LLONG lLoginID, const NET_IN_FACE_RECOGNITION_SET_SEARCH_IMAGE_INFO* pstInParam, NET_OUT_FACE_RECOGNITION_SET_SEARCH_IMAGE_INFO *pstOutParam, int nWaitTime = 1000);
//////////////////////////////// burn the upload interface /////////////////////////////////

// open burning session, return to burning session handle,user malloc and free (pstInParam's and pstOutParam's) memory
CLIENT_NET_API LLONG CALL_METHOD CLIENT_StartBurnSession(LLONG lLoginID, const NET_IN_START_BURN_SESSION* pstInParam, NET_OUT_START_BURN_SESSION *pstOutParam, int nWaitTime);

// close burning session
CLIENT_NET_API BOOL CALL_METHOD CLIENT_StopBurnSession(LLONG lBurnSession);

// start burning,user malloc and free (pstInParam's and pstOutParam's) memory
CLIENT_NET_API BOOL CALL_METHOD CLIENT_StartBurn(LLONG lBurnSession, const NET_IN_START_BURN* pstInParam, NET_OUT_START_BURN *pstOutParam, int nWaitTime);

// stop burning
CLIENT_NET_API BOOL CALL_METHOD CLIENT_StopBurn(LLONG lBurnSession);

// pause/recover burning
CLIENT_NET_API BOOL CALL_METHOD CLIENT_PauseBurn(LLONG lBurnSession, BOOL bPause);

// key mark,user malloc and free (pstInParam's and pstOutParam's) memory
CLIENT_NET_API BOOL CALL_METHOD CLIENT_BurnMarkTag(LLONG lBurnSession, const NET_IN_BURN_MARK_TAG* pstInParam, NET_OUT_BURN_MARK_TAG *pstOutParam, int nWaitTime);

// change disk,user malloc and free (pstInParam's and pstOutParam's) memory
CLIENT_NET_API BOOL CALL_METHOD CLIENT_BurnChangeDisk(LLONG lBurnSession, const NET_IN_BURN_CHANGE_DISK* pstInParam, NET_OUT_BURN_CHANGE_DISK *pstOutParam, int nWaitTime);

// get burning status,user malloc and free (pstInParam's and pstOutParam's) memory
CLIENT_NET_API BOOL CALL_METHOD CLIENT_BurnGetState(LLONG lBurnSession, const NET_IN_BURN_GET_STATE* pstInParam, NET_OUT_BURN_GET_STATE *pstOutParam, int nWaitTime);

// attach burn state,user malloc and free (pstInParam's and pstOutParam's) memory
CLIENT_NET_API LLONG CALL_METHOD CLIENT_AttachBurnState(LLONG lLoginID, const NET_IN_ATTACH_STATE* pstInParam, NET_OUT_ATTACH_STATE *pstOutParam, int nWaitTime = 1000);

// cancel listening burning status, lAttachHandle is CLIENT_AttachBurnState return value
CLIENT_NET_API BOOL CALL_METHOD CLIENT_DetachBurnState(LLONG lAttachHandle);

// start burn upload,user malloc and free (pstInParam's and pstOutParam's) memory
CLIENT_NET_API LLONG CALL_METHOD CLIENT_StartUploadFileBurned(LLONG lLoginID, const NET_IN_FILEBURNED_START* pstInParam, NET_OUT_FILEBURNED_START *pstOutParam, int nWaitTime = 1000);

// send file burned,CLIENT_StartUploadFileBurned return lUploadHandle
CLIENT_NET_API BOOL	CALL_METHOD	CLIENT_SendFileBurned(LLONG lUploadHandle);

// stop upload 
CLIENT_NET_API BOOL	CALL_METHOD	CLIENT_StopUploadFileBurned(LLONG lUploadHandle);

// listening burning parity status,user malloc and free (pstInParam's and pstOutParam's) memory
CLIENT_NET_API LLONG CALL_METHOD CLIENT_AttachBurnCheckState(LLONG lLoginID, const NET_IN_ATTACH_BURN_CHECK* pstInParam, NET_OUT_ATTACH_BURN_CHECK* pstOutParam, int nWaitTime = 1000);

// cancle attach burn state,CLIENT_DetachBurnCheckState return lAttachHandle
CLIENT_NET_API BOOL CALL_METHOD CLIENT_DetachBurnCheckState(LLONG lAttachHandle);

// subscribe burning case info,user malloc and free (pInParam's and pOutParam's) memory
CLIENT_NET_API LLONG CALL_METHOD CLIENT_AttachBurnCase(LLONG lLoginID, const NET_IN_ATTACH_BURN_CASE* pInParam, NET_OUT_ATTACH_BURN_CASE* pOutParam, int nWaitTime);

// cancel subscribe burning case info, lAttachHandle is CLIENT_AttachBurnCase return value
CLIENT_NET_API BOOL CALL_METHOD CLIENT_DetachBurnCase(LLONG lAttachHandle);

// subscribe burning device state,user malloc and free (pInParam's and pOutParam's) memory
CLIENT_NET_API LLONG CALL_METHOD CLIENT_AttachBurnDevState(LLONG lLoginID, const NET_IN_ATTACH_BURN_DEV_STATE* pInParam, NET_OUT_ATTACH_BURN_DEV_STATE* pOutParam, int nWaitTime = 3000);

// cancel subscribe burning device state, lAttachHandle is CLIENT_AttachBurnDevState return value
CLIENT_NET_API BOOL CALL_METHOD CLIENT_DetachBurnDevState(LLONG lAttachHandle);

// body mind data info
typedef struct tagNET_BODY_MIND_DATA_INFO
{
    DWORD       dwSize;
    UINT        nECG;               // ecg value,unit: bpm
    UINT        nECGMax;            // ecg max value,unit: bpm
    UINT        nECGMin;            // ecg min value,unit: bpm
    UINT        nSPO2;              // spo2 value,unit: %
    UINT        nSPO2Max;           // spo2 max value,unit: %
    UINT        nSPO2Min;           // spo2 min value,unit: %
    UINT        nRESP;              // resp value,unit: BrPM
    UINT        nRESPMax;           // resp max value,unit: BrPM
    UINT        nRESPMin;           // resp min value,unit: BrPM
    UINT        nNIBP_SYS;          // system nibp value,unit: mmHg
    UINT        nNIBP_SYSMax;       // system nibp max value,unit: mmHg
    UINT        nNIBP_SYSMin;       // system nibp min value,unit: mmHg
    UINT        nNIBP_AVG;          // average nibp value,unit: mmHg
    UINT        nNIBP_AVGMax;       // average nibp max value,unit: mmHg
    UINT        nNIBP_AVGMin;       // average nibp min value,unit: mmHg
    UINT        nNIBP_DIA;          // diastole nibp value,unit: mmHg
    UINT        nNIBP_DIAMax;       // diastole nibp max value,unit: mmHg
    UINT        nNIBP_DIAMin;       // diastole nibp min value,unit: mmHg
} NET_BODY_MIND_DATA_INFO;

// call back of body mind data,lAttachHandle is th return value of CLIENT_AttachBodyMindData
typedef void (CALLBACK *fBodyMindDataCallBack) (LLONG lAttachHandle, const NET_BODY_MIND_DATA_INFO* pBuf, int nBufLen, LDWORD dwUser);

// the input param of CLIENT_AttachBodyMindData
typedef struct tagNET_IN_ATTACH_BODY_MIND_DATA
{
    DWORD                       dwSize;
    fBodyMindDataCallBack       cbBodyMindData;                    // call back function
    LDWORD                      dwUser;                            // user data
} NET_IN_ATTACH_BODY_MIND_DATA;

// the output param of CLIENT_AttachBodyMindData
typedef struct tagNET_OUT_ATTACH_BODY_MIND_DATA
{
    DWORD                dwSize;
} NET_OUT_ATTACH_BODY_MIND_DATA;

// the input param of CLIENT_DetachBodyMindData
typedef struct tagNET_IN_DETACH_BODY_MIND_DATA
{
    DWORD                dwSize;
} NET_IN_DETACH_BODY_MIND_DATA;

// the output param of CLIENT_DetachBodyMindData
typedef struct tagNET_OUT_DETACH_BODY_MIND_DATA
{
    DWORD                dwSize;
} NET_OUT_DETACH_BODY_MIND_DATA;


// attach the body mind data
CLIENT_NET_API LLONG CALL_METHOD CLIENT_AttachBodyMindData(LLONG lLoginID, const NET_IN_ATTACH_BODY_MIND_DATA* pstInParam, NET_OUT_ATTACH_BODY_MIND_DATA* pstOutParam, int nWaitTime);

// detach the body mind data,lAttachHandle is the return value of CLIENT_AttachBodyMindData
CLIENT_NET_API BOOL CALL_METHOD CLIENT_DetachBodyMindData(LLONG lAttachHandle, const NET_IN_DETACH_BODY_MIND_DATA* pstInParam, NET_OUT_DETACH_BODY_MIND_DATA *pstOutParam, int nWaitTime);


//////////////////////////////// logical device /////////////////////////////////////////

// attach camerastate,user malloc and free (pstInParam's and pstOutParam's) memory
CLIENT_NET_API LLONG CALL_METHOD CLIENT_AttachCameraState(LLONG lLoginID, const NET_IN_CAMERASTATE* pstInParam, NET_OUT_CAMERASTATE *pstOutParam, int nWaitTime = 3000);

// detach camera state,CLIENT_AttachCameraState return lAttachHandle
CLIENT_NET_API BOOL	CALL_METHOD	CLIENT_DetachCameraState(LLONG lAttachHandle);

//get all valid display source,user malloc and free (pInParam's and pOutParam's) memory
CLIENT_NET_API BOOL CALL_METHOD CLIENT_MatrixGetCameras(LLONG lLoginID, const DH_IN_MATRIX_GET_CAMERAS* pInParam, DH_OUT_MATRIX_GET_CAMERAS* pOutParam, int nWaitTime = 1000);

// add logical device,user malloc and free (pInParam's and pOutParam's) memory
CLIENT_NET_API BOOL CALL_METHOD CLIENT_MatrixAddCameras(LLONG lLoginID, const DH_IN_ADD_LOGIC_DEVICE_CAMERA* pInParam, DH_OUT_ADD_LOGIC_DEVICE_CAMERA* pOutParam, int nWaitTime = 1000);

// add logical device bydeviceinfo,user malloc and free (pInParam's and pOutParam's) memory
CLIENT_NET_API BOOL CALL_METHOD CLIENT_MatrixAddCamerasByDevice(LLONG lLoginID, const DH_IN_ADD_LOGIC_BYDEVICE_CAMERA* pInParam, DH_OUT_ADD_LOGIC_BYDEVICE_CAMERA* pOutParam, int nWaitTime = 1000);

// add logical cameras by group,user malloc and free (pInParam's and pOutParam's) memory
CLIENT_NET_API BOOL CALL_METHOD CLIENT_MatrixAddCamerasByGroup(LLONG lLoginID, const NET_IN_ADD_LOGIC_BYGROUP_CAMERA* pInParam, NET_OUT_ADD_LOGIC_BYGROUP_CAMERA* pOutParam, int nWaitTime = 1000);

////////////////////////////////  Matrix interface /////////////////////////////////

// Search product definition,user malloc and free pstuProdDef's memory
CLIENT_NET_API BOOL CALL_METHOD CLIENT_QueryProductionDefinition(LLONG lLoginID, DH_PRODUCTION_DEFNITION* pstuProdDef, int nWaitTime = 1000);

// Search matri sub card info,user malloc and free pstuCardList's memory
CLIENT_NET_API BOOL CALL_METHOD CLIENT_QueryMatrixCardInfo(LLONG lLoginID, DH_MATRIX_CARD_LIST* pstuCardList, int nWaitTime = 1000);

// Search system status,user malloc and free pstuStatus's memory
CLIENT_NET_API BOOL CALL_METHOD	CLIENT_QuerySystemStatus(LLONG lLoginID, DH_SYSTEM_STATUS* pstuStatus, int nWaitTime = 1000);

// Search split mode,user malloc and free pstuCaps's memory
CLIENT_NET_API BOOL CALL_METHOD CLIENT_GetSplitCaps(LLONG lLoginID, int nChannel, DH_SPLIT_CAPS* pstuCaps, int nWaitTime = 1000);

// Search/set display source,user malloc and free pstuSplitSrc's memory
CLIENT_NET_API BOOL CALL_METHOD CLIENT_GetSplitSource(LLONG lLoginID, int nChannel, int nWindow, DH_SPLIT_SOURCE* pstuSplitSrc, int nMaxCount, int* pnRetCount, int nWaitTime = 1000);
CLIENT_NET_API BOOL CALL_METHOD CLIENT_SetSplitSource(LLONG lLoginID, int nChannel, int nWindow, const DH_SPLIT_SOURCE* pstuSplitSrc, int nSrcCount, int nWaitTime = 1000);
// set display source, support output parameter,user malloc and free (pInparam's and pOutParam's)memory
CLIENT_NET_API BOOL CALL_METHOD CLIENT_SetSplitSourceEx(LLONG lLoginID, const NET_IN_SET_SPLIT_SOURCE* pInparam, NET_OUT_SET_SPLIT_SOURCE* pOutParam, int nWaitTime = 1000);

// Search/set split mode,user malloc and free pstuSplitInfo's memory
CLIENT_NET_API BOOL CALL_METHOD CLIENT_GetSplitMode(LLONG lLoginID, int nChannel, DH_SPLIT_MODE_INFO* pstuSplitInfo, int nWaitTime = 1000);
CLIENT_NET_API BOOL CALL_METHOD CLIENT_SetSplitMode(LLONG lLoginID, int nChannel, const DH_SPLIT_MODE_INFO* pstuSplitInfo, int nWaitTime = 1000);

// Search split group amount
CLIENT_NET_API BOOL CALL_METHOD CLIENT_GetSplitGroupCount(LLONG lLoginID, int nChannel, DH_SPLIT_MODE emSplitMode, int* pnGroupCount, int nWaitTime = 1000);

// Search video output capability,user malloc and free pstuCaps's memory
CLIENT_NET_API BOOL CALL_METHOD CLIENT_GetVideoOutCaps(LLONG lLoginID, int nChannel, DH_VIDEO_OUT_CAPS* pstuCaps, int nWaitTime = 1000);

// Set video output option,user malloc and free pstuVideoOut's memory
CLIENT_NET_API BOOL CALL_METHOD CLIENT_SetVideoOutOption(LLONG lLoginID, int nChannel, const DH_VIDEO_OUT_OPT* pstuVideoOut, int nWaitTime = 1000);

// Search the current window of video input channel,user malloc and free pstuWnds's memory
CLIENT_NET_API BOOL CALL_METHOD CLIENT_QueryVideoOutWindows(LLONG lLoginID, int nChannel, DH_VIDEO_OUT_WINDOW* pstuWnds, int nMaxWndCount, int* pnRetWndCount, int nWaitTime = 1000);

// set windown position,user malloc and free (pInParam's and pOutParam's)memory
CLIENT_NET_API BOOL CALL_METHOD	CLIENT_SetSplitWindowRect(LLONG lLoginID, const DH_IN_SPLIT_SET_RECT* pInParam, DH_OUT_SPLIT_SET_RECT* pOutParam, int nWaitTime = 1000);
CLIENT_NET_API BOOL	CALL_METHOD	CLIENT_GetSplitWindowRect(LLONG lLoginID, const DH_IN_SPLIT_GET_RECT* pInParam, DH_OUT_SPLIT_GET_RECT* pOutParam, int nWaitTime = 1000);

// open or close window,user malloc and free (pInParam's and pOutParam's)memory
CLIENT_NET_API BOOL CALL_METHOD CLIENT_OpenSplitWindow(LLONG lLoginID, const DH_IN_SPLIT_OPEN_WINDOW* pInParam, DH_OUT_SPLIT_OPEN_WINDOW* pOutParam, int nWaitTime = 1000);
CLIENT_NET_API BOOL CALL_METHOD CLIENT_CloseSplitWindow(LLONG lLoginID, const DH_IN_SPLIT_CLOSE_WINDOW* pInParam, DH_OUT_SPLIT_CLOSE_WINDOW* pOutParam, int nWaitTime = 1000);

// set window order,user malloc and free (pInParam's and pOutParam's)memory
CLIENT_NET_API BOOL CALL_METHOD CLIENT_SetSplitTopWindow(LLONG lLoginID, const DH_IN_SPLIT_SET_TOP_WINDOW* pInParam, DH_OUT_SPLIT_SET_TOP_WINDOW* pOutParam, int nWaitTime = 1000);

// get windows info,user malloc and free (pInParam's and pOutParam's)memory
CLIENT_NET_API BOOL CALL_METHOD CLIENT_GetSplitWindowsInfo(LLONG lLoginID, const DH_IN_SPLIT_GET_WINDOWS* pInParam, DH_OUT_SPLIT_GET_WINDOWS* pOutParam, int nWaitTime = 1000);

// load or save collection,user malloc and free (pInParam's and pOutParam's)memory
CLIENT_NET_API BOOL CALL_METHOD	CLIENT_LoadSplitCollection(LLONG lLoginID, const DH_IN_SPLIT_LOAD_COLLECTION* pInParam, DH_OUT_SPLIT_LOAD_COLLECTION* pOutParam, int nWaitTime = 1000);
CLIENT_NET_API BOOL CALL_METHOD	CLIENT_SaveSplitCollection(LLONG lLoginID, const DH_IN_SPLIT_SAVE_COLLECTION* pInParam, DH_OUT_SPLIT_SAVE_COLLECTION* pOutParam, int nWaitTime = 1000);

// get collection info,user malloc and free (pInParam's and pOutParam's)memory
CLIENT_NET_API BOOL CALL_METHOD CLIENT_GetSplitCollections(LLONG lLoginID, const DH_IN_SPLIT_GET_COLLECTIONS* pInParam, DH_OUT_SPLIT_GET_COLLECTIONS* pOutParam, int nWaitTime = 1000);

// rename collection,user malloc and free (pInParam's and pOutParam's)memory
CLIENT_NET_API BOOL	CALL_METHOD CLIENT_RenameSplitCollection(LLONG lLoginID, const DH_IN_SPLIT_RENAME_COLLECTION* pInParam, DH_OUT_SPLIT_RENAME_COLLECTION* pOutParam, int nWaitTime = 1000);

// delete collection,user malloc and free (pInParam's and pOutParam's)memory
CLIENT_NET_API BOOL CALL_METHOD CLIENT_DeleteSplitCollection(LLONG lLoginID, const DH_IN_SPLIT_DELETE_COLLECTION* pInParam, DH_OUT_SPLIT_DELETE_COLLECTION* pOutParam, int nWaitTime = 1000);

// decode policy,user malloc and free (pInParam's and pOutParam's)memory
CLIENT_NET_API BOOL CALL_METHOD CLIENT_SetDecodePolicy(LLONG lLoginID, const DH_IN_SET_DEC_POLICY* pInParam, DH_OUT_SET_DEC_POLICY* pOutParam, int nWaitTime = 1000);
CLIENT_NET_API BOOL CALL_METHOD CLIENT_GetDecodePolicy(LLONG lLoginID, const DH_IN_GET_DEC_POLICY* pInParam, DH_OUT_GET_DEC_POLICY* pOutParam, int nWaitTime = 1000);

// mode of audio output,user malloc and free (pInParam's and pOutParam's)memory
CLIENT_NET_API BOOL CALL_METHOD CLIENT_SetSplitAudioOuput(LLONG lLoginID, const DH_IN_SET_AUDIO_OUTPUT* pInParam, DH_OUT_SET_AUDIO_OUTPUT* pOutParam, int nWaitTime = 1000);
CLIENT_NET_API BOOL CALL_METHOD CLIENT_GetSplitAudioOuput(LLONG lLoginID, const DH_IN_GET_AUDIO_OUTPUT* pInParam, DH_OUT_GET_AUDIO_OUTPUT* pOutParam, int nWaitTime = 1000);

// set display source,user malloc and free (pInParam's and pOutParam's)memory
CLIENT_NET_API BOOL CALL_METHOD CLIENT_MatrixSetCameras(LLONG lLoginID, const DH_IN_MATRIX_SET_CAMERAS* pInParam, DH_OUT_MATRIX_SET_CAMERAS* pOutParam, int nWaitTime = 1000);

// get video in ability,user malloc and free (pInParam's and pOutParam's)memory
CLIENT_NET_API BOOL CALL_METHOD CLIENT_GetVideoInCaps(LLONG lLoginID, const DH_IN_GET_VIDEO_IN_CAPS* pInParam, DH_OUT_GET_VIDEO_IN_CAPS* pOutParam, int nWaitTime = 1000);

// get mode of video out,user malloc and free (pInParam's and pOutParam's)memory
CLIENT_NET_API BOOL CALL_METHOD CLIENT_EnumVideoOutModes(LLONG lLoginID, const DH_IN_ENUM_VIDEO_OUT_MODES* pInParam, DH_OUT_ENUM_VIDEO_OUT_MODES* pOutParam, int nWaitTime = 1000);

// get or set splite output OSD info,user malloc and free (pInParam's and pOutParam's)memory
CLIENT_NET_API BOOL CALL_METHOD CLIENT_GetSplitOSD(LLONG lLoginID, const DH_IN_SPLIT_GET_OSD* pInParam, DH_OUT_SPLIT_GET_OSD* pOutParam, int nWaitTime = 1000);
CLIENT_NET_API BOOL CALL_METHOD CLIENT_SetSplitOSD(LLONG lLoginID, const DH_IN_SPLIT_SET_OSD* pInParam, DH_OUT_SPLIT_SET_OSD* pOutParam, int nWaitTime = 1000);

// get or set splite output OSD info ----expand interface (the memory of pInParam and pOutParam applyed and released by user),user malloc and free (pInParam's and pOutParam's)memory
CLIENT_NET_API BOOL CALL_METHOD CLIENT_GetSplitOSDEx(LLONG lLoginID, const NET_IN_SPLIT_GET_OSD_EX* pInParam, NET_OUT_SPLIT_GET_OSD_EX* pOutParam, int nWaitTime = 1000);
CLIENT_NET_API BOOL CALL_METHOD CLIENT_SetSplitOSDEx(LLONG lLoginID, const NET_IN_SPLIT_SET_OSD_EX* pInParam, NET_OUT_SPLIT_SET_OSD_EX* pOutParam, int nWaitTime = 1000);

// set window tour display source,user malloc and free (pInParam's and pOutParam's)memory
CLIENT_NET_API BOOL CALL_METHOD CLIENT_SetTourSource(LLONG lLoginID, const NET_IN_SET_TOUR_SOURCE* pInParam, NET_OUT_SET_TOUR_SOURCE* pOutParam, int nWaitTime = 1000);
CLIENT_NET_API BOOL CALL_METHOD CLIENT_GetTourSource(LLONG lLoginID, const NET_IN_GET_TOUR_SOURCE* pInParam, NET_OUT_GET_TOUR_SOURCE* pOutParam, int nWaitTime);

// order tour status ,user malloc and free (pInParam's and pOutParam's)memory
CLIENT_NET_API LLONG CALL_METHOD CLIENT_AttachSplitTour(LLONG lLoginID, const NET_IN_ATTACH_SPLIT_TOUR* pInParam, NET_OUT_ATTACH_SPLIT_TOUR* pOutParam, int nWaitTime);
CLIENT_NET_API BOOL CALL_METHOD CLIENT_DetachSplitTour(LLONG lAttachHandle);

// low matrix switch,user malloc and free (pInParam's and pOutParam's)memory
CLIENT_NET_API BOOL CALL_METHOD CLIENT_MatrixSwitch(LLONG lLoginID, const NET_IN_MATRIX_SWITCH* pInParam, NET_OUT_MATRIX_SWITCH* pOutParam, int nWaitTime);

// set display source, support multiple window at same time,user malloc and free (pInParam's and pOutParam's)memory
CLIENT_NET_API BOOL CALL_METHOD CLIENT_SplitSetMultiSource(LLONG lLoginID, const NET_IN_SPLIT_SET_MULTI_SOURCE* pInParam, NET_OUT_SPLIT_SET_MULTI_SOURCE* pOutParam, int nWaitTime);

// video split operation,user malloc and free (pInParam's and pOutParam's)memory
CLIENT_NET_API BOOL CALL_METHOD CLIENT_OperateSplit(LLONG lLoginID, NET_SPLIT_OPERATE_TYPE emType, void* pInParam, void* pOutParam, int nWaitTime);
// attach windows source info, user malloc and free(pInParam's and pOutParam's) memory
CLIENT_NET_API LLONG CALL_METHOD CLIENT_AttachWindowsSource( LLONG lLoginID, const NET_IN_ATTACH_WINDOW_SOURCE* pInParam, NET_OUT_ATTACH_WINDOW_SOURCE* pOutParam, int nWaitTime );
// deteach windows  source info
CLIENT_NET_API BOOL CALL_METHOD CLIENT_DetachWindowsSource( LLONG lAttachHandle );

// enlarge region of the window, user malloc and free(pInParam's and pOutParam's) memory
CLIENT_NET_API LLONG CALL_METHOD CLIENT_WindowRegionEnlarge(LLONG lLoginID, const NET_IN_WINDOW_REGION_ENLARGE* pInParam, NET_OUT_WINDOW_REGION_ENLARGE* pOutParam, int nWaitTime);

// restore the window which is be enlarged, user malloc and free(pInParam's and pOutParam's) memory
CLIENT_NET_API LLONG CALL_METHOD CLIENT_WindowEnlargeReduction(LLONG lLoginID, const NET_IN_WINDOW_ENLARGE_REDUCTION* pInParam, NET_OUT_WINDOW_ENLARGE_REDUCTION* pOutParam, int nWaitTime);

// split window play operaton type 
typedef enum tagNET_PLAYER_OPERATE_TYPE
{
    NET_PLAYER_OPERATE_OPEN,                // open playuer,NET_IN_PLAYER_OPEN
    NET_PLAYER_OPERATE_CLOSE,               // close player,NET_IN_PLAYER_CLOSE
    NET_PLAYER_OPERATE_START,               // start play,NET_IN_PLAYER_START
    NET_PLAYER_OPERATE_STOP,                // stop play,NET_IN_PLAYER_STOP
    NET_PLAYER_OPERATE_PAUSE,               // pause/restore play,NET_IN_PLAYER_PAUSE
    NET_PLAYER_OPERATE_SEEK_TIME,           // go to specific time play,NET_IN_PLAYER_SEEK_TIME
    NET_PLAYER_OPERATE_STEP_FRAME,          // single frame playback, need to be used after pause,NET_IN_PLAYER_STEP_FRAME
    NET_PLAYER_OPERATE_GET_STATE,           // take current playback status,NET_IN_PLAYER_GET_STATE
    NET_PLAYER_OPERATE_GET_TIME,            // Get  current playback time,NET_IN_PLAYER_GET_TIME and NET_OUT_PLAYER_GET_TIME
    NET_PLAYER_OPERATE_GET_SPEED,           // Get play speed,NET_IN_PLAYER_GET_SPEED and NET_OUT_PLAYER_GET_SPEED
    NET_PLAYER_OPERATE_SET_SPEED,           // Set play speed,NET_IN_PLAYER_SET_SPEED
    NET_PLAYER_OPERATE_GET_VOLUME,          // Get volume,NET_IN_PLAYER_GET_VOLUME and NET_OUT_PLAYER_GET_VOLUME
    NET_PLAYER_OPERATE_SET_VOLUME,          // Set volume,NET_IN_PLAYER_SET_VOLUME
    NET_PLAYER_OPERATE_GET_PLAYLIST,        // Get file list,NET_IN_PLAYER_GET_PLAYLIST and NET_OUT_PLAYER_GET_PLAYLIST
    NET_PLAYER_OPERATE_GET_PLAYLIST_TS,     // Get current file list all period info,NET_IN_PLAYER_GET_PLAYLIST_TS and NET_OUT_PLAYER_GET_PLAYLIST_TS
}NET_PLAYER_OPERATE_TYPE;

// play criteria 
typedef struct tagNET_PLAYER_OPEN_CONDITION 
{
    DWORD               dwSize;
    char                szDevice[DH_DEV_ID_LEN];        // point remote device 
    int                 nChannel;                       // channel no.
    NET_TIME            stuStartTime;                   // start time
    NET_TIME            stuEndTime;                     // end time
    NET_STREAM_TYPE     emStreamType;                   // stream type 
    int                 nEventNum;                      // Event type number
    int                 nEvent[MAX_IVS_EVENT_NUM];      // Event type
}NET_PLAYER_OPEN_CONDITION;

//open player input parameter , corresponding to NET_PLAYER_OPERATE_OPEN
typedef struct tagNET_IN_PLAYER_OPEN 
{
	DWORD						dwSize;
	LLONG						lPlayerID;		// play example ID
	const char*					pszDevice;		// take record from local or remote end, as null represents to take record from local, 
												// it is valid when bDeviceInfo is FALSE
												// if from remote device, need to maintain with NET_PLAYER_OPEN_CONDITION szDevice identical
	NET_PLAYER_OPEN_CONDITION 	stuCondition;	// search criteria 
	BOOL						bDeviceInfo;	// it means that the "stuDeviceInfo" is valid or not
	DH_REMOTE_DEVICE			stuDeviceInfo;	// device info, it is valid when bDeviceInfo is TRUE
}NET_IN_PLAYER_OPEN;

// open player  output parameter , corresponding to NET_PLAYER_OPERATE_OPEN
typedef struct tagNET_OUT_PLAYER_OPEN 
{
  DWORD        dwSize;
}NET_OUT_PLAYER_OPEN;

// close player input parameter , corresponding to NET_PLAYER_OPERATE_CLOSE
typedef struct tagNET_IN_PLAYER_CLOSE
{
  DWORD        dwSize;
  LLONG        lPlayerID;           // play example ID
}NET_IN_PLAYER_CLOSE;

// close player output parameter , corresponding to NET_PLAYER_OPERATE_CLOSE
typedef struct tagNET_OUT_PLAYER_CLOSE
{
  DWORD        dwSize;
}NET_OUT_PLAYER_CLOSE;

//  start  play input parameter , corresponding to NET_PLAYER_OPERATE_START
typedef struct tagNET_IN_PLAYER_START
{
  DWORD        dwSize;
  LLONG        lPlayerID;           //  play example ID
}NET_IN_PLAYER_START;

//  start  play  output parameter , corresponding to NET_PLAYER_OPERATE_START
typedef struct tagNET_OUT_PLAYER_START
{
  DWORD        dwSize;
}NET_OUT_PLAYER_START;

// stop play input parameter , corresponding to NET_PLAYER_OPERATE_STOP
typedef struct tagNET_IN_PLAYER_STOP
{
  DWORD        dwSize;
  LLONG        lPlayerID;           //  play example ID
}NET_IN_PLAYER_STOP;

// stop play  output parameter , corresponding to NET_PLAYER_OPERATE_STOP
typedef struct tagNET_OUT_PLAYER_STOP
{
  DWORD        dwSize;
}NET_OUT_PLAYER_STOP;

//  pause /recover  play input parameter , corresponding to NET_PLAYER_OPERATE_PAUSE
typedef struct tagNET_IN_PLAYER_PAUSE
{
  DWORD        dwSize;
  LLONG        lPlayerID;         	// play example ID
  BOOL         bPause;             	// pause or not, TRUE- pause  play , FALSE-recover  play 
}NET_IN_PLAYER_PAUSE;

//  pause /recover  play  output parameter , corresponding to NET_PLAYER_OPERATE_PAUSE
typedef struct tagNET_OUT_PLAYER_PAUSE
{
  DWORD        dwSize;
}NET_OUT_PLAYER_PAUSE;

// go to specific time play input parameter , corresponding to NET_PLAYER_OPERATE_SEEK_TIME
typedef struct tagNET_IN_PLAYER_SEEK_TIME
{
  DWORD        dwSize;
  LLONG        lPlayerID;           // play example ID
  NET_TIME     stuTime;            	// go to time
}NET_IN_PLAYER_SEEK_TIME;

// go to specific time play  output parameter , corresponding to NET_PLAYER_OPERATE_SEEK_TIME
typedef struct tagNET_OUT_PLAYER_SEEK_TIME
{
  DWORD        dwSize;
}NET_OUT_PLAYER_SEEK_TIME;

// single frame play input parameter , corresponding to NET_PLAYER_OPERATE_STEP_FRAME
typedef struct tagNET_IN_PLAYER_STEP_FRAME
{
  DWORD        dwSize;
  LLONG        lPlayerID;           //  play example ID
  BOOL         bForward;            //  positive direction, TRUE-positive direction, FALSE-negative direction
}NET_IN_PLAYER_STEP_FRAME;

// single frame play  output parameter , corresponding to NET_PLAYER_OPERATE_STEP_FRAME
typedef struct tagNET_OUT_PLAYER_STEP_FRAME
{
  DWORD        dwSize;
}NET_OUT_PLAYER_STEP_FRAME;

//  play status 
typedef enum tagNET_PLAYER_STATE 
{
  NET_PLAYER_STATE_UNKNOWN,        	// unknown 
  NET_PLAYER_STATE_ERROR,         	// error occur
  NET_PLAYER_STATE_READING,        	// ready, may start palyabck
  NET_PLAYER_STATE_STANDBY,        	// current file playback stop, only can start playback from beginningart
  NET_PLAYER_STATE_RUNNING,        	// running
  NET_PLAYER_STATE_PAUSED,        	// paused , may continue from pause point
  NET_PLAYER_STATE_CLOSED,        	// closed, cannot play, must open and enter Reading status again
}NET_PLAYER_STATE;

// search  play status input parameter , corresponding to NET_PLAYER_OPERATE_GET_STATE
typedef struct tagNET_IN_PLAYER_GET_STATE
{
  DWORD        dwSize;
  LLONG        lPlayerID;           //  play example ID
}NET_IN_PLAYER_GET_STATE;

// search  play status  output parameter , corresponding to NET_PLAYER_OPERATE_GET_STATE
typedef struct tagNET_OUT_PLAYER_GET_STATE
{
  DWORD        dwSize;
  NET_PLAYER_STATE  emState;            // status 
}NET_OUT_PLAYER_GET_STATE;

// Get  current  playback timeinput parameter , corresponding to NET_PLAYER_OPERATE_GET_TIME
typedef struct tagNET_IN_PLAYER_GET_TIME
{
  DWORD        dwSize;
  LLONG        lPlayerID;           //  play example ID
}NET_IN_PLAYER_GET_TIME;

// Get  current  playback time output parameter , corresponding to NET_PLAYER_OPERATE_GET_TIME
typedef struct tagNET_OUT_PLAYER_GET_TIME
{
  DWORD        dwSize;
  NET_TIME     stuTime;            //  current  play time
}NET_OUT_PLAYER_GET_TIME;

// Get  play speed input parameter , corresponding to NET_PLAYER_OPERATE_GET_SPEED
typedef struct tagNET_IN_PLAYER_GET_SPEED
{
  DWORD        dwSize;
  LLONG        lPlayerID;           //  play example ID
}NET_IN_PLAYER_GET_SPEED;

// Get  play speed output parameter , corresponding to NET_PLAYER_OPERATE_GET_SPEED
typedef struct tagNET_OUT_PLAYER_GET_SPEED
{
  DWORD        dwSize;
  float        fSpeed;             	//  play speed, >0 positive direction  play , <0 direction  play 
									//  absolute value means speed, =1 nornal speed, >1 quick, <1 slow
}NET_OUT_PLAYER_GET_SPEED;

// set play speedinput parameter , corresponding to NET_PLAYER_OPERATE_SET_SPEED
typedef struct tagNET_IN_PLAYER_SET_SPEED
{
  DWORD        dwSize;
  LLONG        lPlayerID;           //  play example ID
  float        fSpeed;             	//  play speed, >0 positive direction  play , <0 direction  play 
									//  absolute value means speed, =1 normal speed, >1 quick, <1 slow
}NET_IN_PLAYER_SET_SPEED;

// set play speed output parameter , corresponding to NET_PLAYER_OPERATE_SET_SPEED
typedef struct tagNET_OUT_PLAYER_SET_SPPED
{
  DWORD        dwSize;
}NET_OUT_PLAYER_SET_SPEED;

// Get  play  volume input parameter , corresponding to NET_PLAYER_OPERATE_GET_VOLUME
typedef struct tagNET_IN_PLAYER_GET_VOLUME
{
  DWORD        dwSize;
  LLONG        lPlayerID;           //  play example ID
}NET_IN_PLAYER_GET_VOLUME;

// Get  play  volume  output parameter , corresponding to NET_PLAYER_OPERATE_GET_VOLUME
typedef struct tagNET_OUT_PLAYER_GET_VOLUME
{
  DWORD        dwSize;
  int          nVolume;            //  play  volume , 0~100
}NET_OUT_PLAYER_GET_VOLUME;

// set play  volume input parameter , corresponding to NET_PLAYER_OPERATE_SET_VOLUME
typedef struct tagNET_IN_PLAYER_SET_VOLUME
{
  DWORD        dwSize;
  LLONG        lPlayerID;           //  play example ID
  int          nVolume;             //  play  volume , 0~100
}NET_IN_PLAYER_SET_VOLUME;

// set play  volume  output parameter , corresponding to NET_PLAYER_OPERATE_SET_VOLUME
typedef struct tagNET_OUT_PLAYER_SET_VOLUME
{
  DWORD        dwSize;
}NET_OUT_PLAYER_SET_VOLUME;

//  play listinfo 
typedef struct tagNET_PLAYER_PLAYLIST
{
  DWORD        	dwSize;
  char         	szFilePath[MAX_PATH];      	// file path, select either Url
  char         	szUrl[MAX_PATH];        	// URL, and FilePath wither one
  DH_DEVICE_PROTOCOL emUrlProtocol;         // URL protocol type 
  NET_TIME      stuStartTime;          		// start time
  NET_TIME      stuEndTime;           		// end time
}NET_PLAYER_PLAYLIST;

// Get  current  play listinput parameter , corresponding to NET_PLAYER_OPERATE_GET_PLAYLIST
typedef struct tagNET_IN_PLAYER_GET_PLAYLIST
{
  DWORD         dwSize;
  LLONG         lPlayerID;           	// play example ID
  NET_TIME      stuStartTime;          	// start time
  NET_TIME      stuEndTime;           	// end time
  int           nLimit;             	// search file quantity limit, 0 means no limit
}NET_IN_PLAYER_GET_PLAYLIST;

// Get  current  play list output parameter , corresponding to NET_PLAYER_OPERATE_GET_PLAYLIST
typedef struct tagNET_OUT_PLAYER_GET_PLAYLIST
{
  DWORD       dwSize;
  NET_PLAYER_PLAYLIST* pstuPlayLists;  // play list, user allocate memory
  int         nMaxPlayListCount;       // max play list quantity, user fill in
  int         nRetPlayListCount;       // return play list quantity
}NET_OUT_PLAYER_GET_PLAYLIST;

// Get current file list all time period info input parameter, corresponding  NET_PLAYER_OPERATE_GET_PLAYLIST_TS
typedef struct tagNET_IN_PLAYER_GET_PLAYLIST_TS
{
    DWORD               dwSize;
    LLONG               lPlayerID;                      // Play actual ID
}NET_IN_PLAYER_GET_PLAYLIST_TS;

typedef struct tagNET_PLAYLIST_TIMESECTION
{
    DWORD               dwSize;
    int                 nEvent;                         // Record event type,  currently have
                                                        // EVENT_ALARM_COMMON, EVENT_ALARM_VIDEOBLIND, EVENT_ALARM_VIDEOLOSS, 
                                                        // EVENT_ALARM_MOTIONDETECT, EVENT_ALARM_LOCALALARM
    LPDH_TSECT          pstuTSs;                        // Time period info
    DWORD               unMaxTS;                        // Time period max number
    DWORD               unRetTS;                        // Actual returned time period number
}NET_PLAYLIST_TIMESECTION;

// Get current file list all time period info output parameters, corresponding  NET_PLAYER_OPERATE_GET_PLAYLIST_TS
typedef struct tagNET_OUT_PLAYER_GET_PLAYLIST_TS
{
    DWORD                    dwSize;
    DWORD                    dwEventNum;                // Record type quantity
    NET_PLAYLIST_TIMESECTION stuTS[MAX_IVS_EVENT_NUM];  // Current playback list time period info
}NET_OUT_PLAYER_GET_PLAYLIST_TS;

// split window  play ,user malloc and free (pInParam's and pOutParam's)memory,please refer to the corrsponding structure of emType
CLIENT_NET_API BOOL CALL_METHOD CLIENT_OperateSplitPlayer(LLONG lLoginID, NET_PLAYER_OPERATE_TYPE emType, void* pInParam, void* pOutParam, int nWaitTime);

// NET_ACCESS_LOCK_VER input parameter 
typedef struct tagNET_ACCESS_LOCK_VER_IN
{
    int                 nChannel;                       // channel number 
    char                reserved[128];                  // reserve 
} NET_ACCESS_LOCK_VER_IN;

// NET_ACCESS_LOCK_VER output parameter 
typedef struct tagNET_ACCESS_LOCK_VER_OUT
{
    char                szVersion[32];                  // version 
    NET_TIME            stuTime;                        // time 
    char                reserved[512];                  // reserve 
} NET_ACCESS_LOCK_VER_OUT;

// door lock version number, CLIENT_QueryDevState query type
typedef struct tagNET_ACCESS_LOCK_VER
{
    DWORD                    dwSize;
    NET_ACCESS_LOCK_VER_IN   stuIn;                     // input parameter 
    NET_ACCESS_LOCK_VER_OUT  stuOut;                    // output parameter 
} NET_ACCESS_LOCK_VER;

//////////////////////////////////// TV wall control  //////////////////////////////////////

// power control,user malloc memory of pInParam and pOutParam 
CLIENT_NET_API BOOL CALL_METHOD CLIENT_PowerControl(LLONG lLoginID, const DH_IN_WM_POWER_CTRL* pInParam, DH_OUT_WM_POWER_CTRL* pOutParam, int nWaitTime = 1000);

// get or set mode of display ,user malloc memory of pInParam and pOutParam
CLIENT_NET_API BOOL CALL_METHOD CLIENT_SetDisplayMode(LLONG lLoginID, const DH_IN_WM_SET_DISPLAY_MODE* pInParam, DH_OUT_WM_SET_DISPLAY_MODE* pOutParam, int nWaitTime = 1000);
CLIENT_NET_API BOOL CALL_METHOD CLIENT_GetDisplayMode(LLONG lLoginID, const DH_IN_WM_GET_DISPLAY_MODE* pInParam, DH_OUT_WM_GET_DISPLAY_MODE* pOutParam, int nWaitTime = 1000);

// load or save plan,user malloc memory of pInParam and pOutParam
CLIENT_NET_API BOOL CALL_METHOD	CLIENT_LoadMonitorWallCollection(LLONG lLoginID, const DH_IN_WM_LOAD_COLLECTION* pInParam, DH_OUT_WM_LOAD_COLLECTION* pOutParam, int nWaitTime = 1000);
CLIENT_NET_API BOOL CALL_METHOD	CLIENT_SaveMonitorWallCollection(LLONG lLoginID, const DH_IN_WM_SAVE_COLLECTION* pInParam, DH_OUT_WM_SAVE_COLLECTION* pOutParam, int nWaitTime = 1000);

// get plan of monitor wall,user malloc memory of pInParam and pOutParam
CLIENT_NET_API BOOL CALL_METHOD CLIENT_GetMonitorWallCollections(LLONG lLoginID, const DH_IN_WM_GET_COLLECTIONS* pInParam, DH_OUT_WM_GET_COLLECTIONS* pOutParam, int nWaitTime = 1000);

// rename monitor wall's plan,user malloc memory of pInParam and pOutParam
CLIENT_NET_API BOOL	CALL_METHOD CLIENT_RenameMonitorWallCollection(LLONG lLoginID, const DH_IN_WM_RENAME_COLLECTION* pInParam, DH_OUT_WM_RENAME_COLLECTION* pOutParam, int nWaitTime = 1000);

// get or set scene of monitor wall ,user malloc memory of pInParam and pOutParam
CLIENT_NET_API BOOL CALL_METHOD CLIENT_MonitorWallGetScene(LLONG lLoginID, const DH_IN_MONITORWALL_GET_SCENE* pInParam, DH_OUT_MONITORWALL_GET_SCENE* pOutParam, int nWaitTime = 1000);
CLIENT_NET_API BOOL CALL_METHOD CLIENT_MonitorWallSetScene(LLONG lLoginID, const DH_IN_MONITORWALL_SET_SCENE* pInParam, DH_OUT_MONITORWALL_SET_SCENE* pOutParam, int nWaitTime = 1000);

// get attribute caps of monitor wall
CLIENT_NET_API BOOL CALL_METHOD CLIENT_MonitorWallGetAttributeCaps(LLONG lLoginID, const DH_IN_MONITORWALL_GET_ARRT_CAPS* pInParam, DH_OUT_MONITORWALL_GET_ARRT_CAPS* pOutParam, int nWaitTime = 1000);

// auto adjust of monitor wall,user malloc memory of pInParam and pOutParam
CLIENT_NET_API BOOL CALL_METHOD CLIENT_MonitorWallAutoAdjust(LLONG lLoginID, const DH_IN_MONITORWALL_AUTO_ADJUST* pInParam, DH_OUT_MONITORWALL_AUTO_ADJUST* pOutParam, int nWaitTime = 1000);

// set attribute of monitor wall ,user malloc memory of pInParam and pOutParam
CLIENT_NET_API BOOL CALL_METHOD CLIENT_MonitorWallSetAttribute(LLONG lLoginID, const DH_IN_MONITORWALL_SET_ATTR* pInParam, DH_OUT_MONITORWALL_SET_ATTR* pOutParam, int nWaitTime = 1000);

// set mode of backlight,user malloc memory of pInParam and pOutParam
CLIENT_NET_API BOOL CALL_METHOD CLIENT_MonitorWallSetBackLight(LLONG lLoginID, const DH_IN_MONITORWALL_SET_BACK_LIGHT* pInParam, DH_OUT_MONITORWALL_SET_BACK_LIGHT* pOutParam, int nWaitTime = 1000);

// search/set screen on/off plan,user malloc memory of pInParam and pOutParam
CLIENT_NET_API BOOL CALL_METHOD CLIENT_MonitorWallGetPowerSchedule(LLONG lLoginID, const NET_IN_MW_GET_POWER_SCHEDULE* pInParam, NET_OUT_MW_GET_POWER_SCHEDULE* pOutParam, int nWaitTime);
CLIENT_NET_API BOOL CALL_METHOD CLIENT_MonitorWallSetPowerSchedule(LLONG lLoginID, const NET_IN_MW_SET_POWER_SCHEDULE* pInParam, NET_OUT_MW_SET_POWER_SCHEDULE* pOutParam, int nWaitTime);

// search/set screen control parameter,user malloc memory of pInParam and pOutParam
CLIENT_NET_API BOOL CALL_METHOD CLIENT_MonitorWallGetScrnCtrlParam(LLONG lLoginID, const NET_IN_MW_GET_SCRN_CTRL_PARAM* pInParam, NET_OUT_MW_GET_SCRN_CTRL_PARAM* pOutParam, int nWaitTime);
CLIENT_NET_API BOOL CALL_METHOD CLIENT_MonitorWallSetScrnCtrlParam(LLONG lLoginID, const NET_IN_MW_SET_SCRN_CTRL_PARAM* pInParam, NET_OUT_MW_SET_SCRN_CTRL_PARAM* pOutParam, int nWaitTime);

// search/set screen and window background color,user malloc memory of pInParam and pOutParam
CLIENT_NET_API BOOL CALL_METHOD CLIENT_MonitorWallGetBackgroudColor(LLONG lLoginID, const NET_IN_MW_GET_BACKGROUDND_COLOR* pInParam, NET_OUT_MW_GET_BACKGROUDND_COLOR* pOutParam, int nWaitTime);
CLIENT_NET_API BOOL CALL_METHOD CLIENT_MonitorWallSetBackgroudColor(LLONG lLoginID, const NET_IN_MW_SET_BACKGROUD_COLOR* pInParam, NET_OUT_MW_SET_BACKGROUD_COLOR* pOutParam, int nWaitTime);

// order scheme tour status,user malloc memory of pInParam and pOutParam
CLIENT_NET_API LLONG CALL_METHOD CLIENT_MonitorWallAttachTour(LLONG lLoginID, const NET_IN_WM_ATTACH_TOUR* pInParam, NET_OUT_WM_ATTACH_TOUR* pOutParam, int nWaitTime);
CLIENT_NET_API BOOL CALL_METHOD CLIENT_MonitorWallDetachTour(LLONG lAttachHandle);

// operate Monitor Wall,user malloc memory of pInParam and pOutParam,please refer to the corresponding structure of emType
CLIENT_NET_API BOOL CALL_METHOD CLIENT_OperateMonitorWall(LLONG lLoginID, NET_MONITORWALL_OPERATE_TYPE emType, void* pInParam, void* pOutParam, int nWaitTime);

// switch Monitor Wall display sigal,user malloc memory of pInParam and pOutParam
CLIENT_NET_API BOOL CALL_METHOD CLIENT_MonitorWallSwitchDisplaySignal(LLONG lLoginID, const NET_IN_MW_SWITCH_DISPLAY_SIGNAL* pInParam, NET_OUT_MW_SWITCH_DISPLAY_SIGNAL* pOutParam, int nWaitTime);

//////////////////////////////////////////////////////////////////////////////////////////////////////
///////////////////////////////// Interfaces of guid screen///////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////////////////////////////
#define MAX_PLAYDATES_COUNT		32		// the max count of dates
#define	MAX_SCREENTIME_COUNT	8		// the max count of switch screen time of guid screen
#define MAX_WINDOWS_COUNT		16		// the max count of windows of guid screen
#define MAX_ELEMENTS_COUNT		8		// the max count of elements of windows
#define MAX_ELEMENTTEXT_LENGTH	512		// the max length of text element
#define MAX_NOTE_COUNT			4		// the max count of notes
#define MAX_PROGRAMMES_COUNT	32		// the max count of programmes

// the type of screen date
typedef enum tagEM_SCREEN_DATE_TYPE
{
	EM_SCREEN_DATE_UNKNOWN,				// unknown
	EM_SCREEN_DATE_MONTH,				// per month
	EM_SCREEN_DATE_WEEK,				// per week
	EM_SCREEN_DATE_DAY,					// per day
	EM_SCREEN_DATE_CUSTOM,				// custom
} EM_SCREEN_DATE_TYPE;

// the time info of programme
typedef struct tagNET_PROGRAMME_TIME_INFO
{
	DWORD                dwHour;                  // hour
    DWORD                dwMinute;                // minute
    DWORD                dwSecond;                // second
} NET_PROGRAMME_TIME_INFO;

// the info of switch screen time
typedef struct tagNET_SCREEN_TIME_INFO
{
	BOOL					bEnable;							// is it enable
	EM_SCREEN_DATE_TYPE		emDateType;							// the type of screen date
	UINT					nDateCount;							// the count of screen date
	UINT					nPlayDates[MAX_PLAYDATES_COUNT];	// the data of screen date
	NET_PROGRAMME_TIME_INFO	stuOpenTime;						// the time of open screen
	NET_PROGRAMME_TIME_INFO	stuCloseTime;						// the time of close screen
	BYTE        			byReserved[128];                 	// reserved bytes
} NET_SCREEN_TIME_INFO;

// the rect info of window
typedef struct NET_GUIDESCREEN_WINDOW_RECT_INFO
{
	char				szWindowID[MAX_COMMON_STRING_64];			// window ID
	NET_RECT			stuRect;									// rect info
	BYTE        		byReserved[128];                 			 // reserved bytes
} NET_GUIDESCREEN_WINDOW_RECT_INFO;

// the status of switch screen
typedef enum tagEM_SCREEN_STATUS_TYPE
{
	EM_SCREEN_STATUS_UNKNOWN,		// unknown
	EM_SCREEN_STATUS_ON,			// on
	EM_SCREEN_STATUS_OFF,			// off
} EM_SCREEN_STATUS_TYPE;

// the configure of guid screen
typedef struct tagNET_GUIDESCREEN_ATTRIBUTE_INFO
{
	DWORD           					dwSize;
	char								szScreenID[MAX_COMMON_STRING_64];		// screen ID
	EM_SCREEN_STATUS_TYPE				emStatus;								// the status of switch screen
	BOOL								bIsForeverOpen;							// is forever open
	UINT								nScreenTime;							// count of switch screen time
	NET_SCREEN_TIME_INFO				stuScreenTime[MAX_SCREENTIME_COUNT];	// data of switch screen time
	UINT								nBright;								// bright of guid screen, 1-100 
	UINT								nContrast;								// contrast of guid screen, 1-100
	UINT								nSaturation;							// saturation of guid screen, 1-100
	UINT								nWidth;									// width of guid screen
	UINT								nHeight;								// height of guid screen
	UINT								nWindowsCount;							// count of windows
	NET_GUIDESCREEN_WINDOW_RECT_INFO	stuWindows[MAX_WINDOWS_COUNT];			// windows info
	BYTE        						byReserved[512];						// reserved bytes
} NET_GUIDESCREEN_ATTRIBUTE_INFO;

// input parameter of the interface of setting guid screen config
typedef struct tagNET_IN_SET_GUIDESCREEN_CFG
{
	DWORD           				dwSize;
	UINT							nScreenCount;		// the count of configuration of the guid screen, it's specified by user
	NET_GUIDESCREEN_ATTRIBUTE_INFO	*pstGuideScreenCfg;	// the information of guid screen, it's specified by user,
														// the size is (nScreenCount*NET_GUIDSCREEN_ATTRIBUTE_INFO)
} NET_IN_SET_GUIDESCREEN_CFG;

// output parameter of th interface of setting guid screen config
typedef struct tagNET_OUT_SET_GUIDESCREEN_CFG
{
	DWORD           				dwSize;
} NET_OUT_SET_GUIDESCREEN_CFG;

/**************************************************************************************
*   Funcname: CLIENT_SetGuideScreenCfg
*   Purpose: interface of setting guid screen config
*   InputParam:	LLONG							:lLoginID		// login handle
*   InputParam:	NET_IN_SET_GUIDESCREEN_CFG*		:pInParam 		// input parameter
*   OutParam:	NET_OUT_SET_GUIDESCREEN_CFG*	:pstOutPqram	// output parameter
*   InputParam:	int								:nWaitTime		// wait time
*   Return:		BOOL
*   Created:		%2017%:%10%:%11%  
*   Revision Record:    date:author:modify sth
**************************************************************************************/
CLIENT_NET_API BOOL CALL_METHOD CLIENT_SetGuideScreenCfg(LLONG lLoginID, const NET_IN_SET_GUIDESCREEN_CFG* pInParam, NET_OUT_SET_GUIDESCREEN_CFG *pstOutPqram, const int nWaitTime);


// the input parameter of the interface of getting guid screen config by ID
typedef struct tagNET_IN_GET_GUIDESCREEN_CFG_BYID
{
	DWORD           				dwSize;
	char							szScreenID[MAX_COMMON_STRING_64];		// screen ID
} NET_IN_GET_GUIDESCREEN_CFG_BYID;

// the output parameter of the interface of getting guid screen config by ID
typedef struct tagNET_OUT_GET_GUIDESCREEN_CFG_BYID
{
	DWORD           				dwSize;
	NET_GUIDESCREEN_ATTRIBUTE_INFO	stuGuideScreenCfg;			// the config of guid screen
} NET_OUT_GET_GUIDESCREEN_CFG_BYID;

/**************************************************************************************
*   Funcname: CLIENT_GetOneGuideScreenCfgById
*   Purpose: interface of getting guid screen config by ID
*   InputParam:	LLONG								:lLoginID	// login handle
*   InputParam:	NET_IN_GET_GUIDESCREEN_CFG_BYID*	:pInParam	// input parameter
*   OutPutParam:NET_OUT_GET_GUIDESCREEN_CFG_BYID*	:pOutParam 	// output parameter
*   InputParam:	int									:nWaitTime	// wait time
*   Return:		BOOL
*   Created:		%2017%:%10%:%11%  
*   Revision Record:    date:author:modify sth
**************************************************************************************/
CLIENT_NET_API BOOL CALL_METHOD CLIENT_GetOneGuideScreenCfgById(
										LLONG lLoginID, const NET_IN_GET_GUIDESCREEN_CFG_BYID *pInParam, 
										NET_OUT_GET_GUIDESCREEN_CFG_BYID* pOutParam, const int nWaitTime);



// the input parameter of the interface of getting config of all guid screen
typedef struct tagNET_IN_GET_ALL_GUIDESCREEN_CFG
{
	DWORD           				dwSize;
} NET_IN_GET_ALL_GUIDESCREEN_CFG;

// the output parameter of the interface of getting config of all guid screen
typedef struct tagNET_OUT_GET_ALL_GUIDESCREEN_CFG
{
	DWORD           				dwSize;
	UINT							nMaxScreen;					// max counts of guis screen, it's specified by user
	UINT							nRetScreen;					// the conut of guid screen actual return
	NET_GUIDESCREEN_ATTRIBUTE_INFO	*pstGuideScreenCfg;			// the config of all guid screen,  applyed by user
																// the size if (nMaxScreen *NET_GUIDSCREEN_ATTRIBUTE_INFO)
} NET_OUT_GET_ALL_GUIDESCREEN_CFG;

/**************************************************************************************
*   Funcname: CLIENT_GetAllGuideScreenCfg
*   Purpose: interface of getting config of all guid screen
*   InputParam:	LLONG								:lLoginID	// login handle
*   InputParam:	NET_IN_GET_ALL_GUIDESCREEN_CFG*		:pInParam	// input parameter
*   OutPutParam:NET_OUT_GET_ALL_GUIDESCREEN_CFG*	:pOutParam 	// output parameter
*   InputParam:	int									:nWaitTime	// wait time
*   Return:		BOOL
*   Created:		%2017%:%10%:%11%  
*   Revision Record:    date:author:modify sth
**************************************************************************************/
CLIENT_NET_API BOOL CALL_METHOD CLIENT_GetAllGuideScreenCfg(
										LLONG lLoginID, const NET_IN_GET_ALL_GUIDESCREEN_CFG *pInParam, 
										NET_OUT_GET_ALL_GUIDESCREEN_CFG* pOutParam, const int nWaitTime);

// the type of horizontal alignment
typedef enum tagEM_HORI_ALIGN_TYPE
{
	EM_HORI_ALIGN_UNKNOWN,				// unknown
	EM_HORI_ALIGN_LEFT,					// left
	EM_HORI_ALIGN_CENTER,				// center
	EM_HORI_ALIGN_RIGHT,				// right
} EM_HORI_ALIGN_TYPE;

// the type of vertical alignment
typedef enum tagEM_VERT_ALIGN_TYPE
{
	EM_VERT_ALIGN_UNKNOWN,				// unknown
	EM_VERT_ALIGN_UP,					// up
	EM_VERT_ALIGN_CENTER,				// center
	EM_VERT_ALIGN_DOWN,					// down
} EM_VERT_ALIGN_TYPE;

// enter(exit) style
typedef enum tagEM_PIC_STYLE_TYPE
{
	EM_PIC_STYLE_UNKNOWN,		// unknown
	EM_PIC_STYLE_DEFAULT,		// default
	EM_PIC_STYLE_LEFT,			// left
	EM_PIC_STYLE_RIGHT,			// right
	EM_PIC_STYLE_UP,			// up
	EM_PIC_STYLE_DOWN,			// down
} EM_PIC_STYLE_TYPE;


//  text info
typedef struct tagNET_GUIDESCREEN_TEXT_INFO
{
	char					szContent[MAX_ELEMENTTEXT_LENGTH];		// content
	UINT					nFontSize;								// font size
	NET_COLOR_RGBA			stuFontColor;							// font color
	char					szFontStyle[MAX_COMMON_STRING_32];		// font style
	double					dbLineHeight;							// height of line
	EM_HORI_ALIGN_TYPE		emHoriAlign;							// the type of horizontal alignment
	EM_VERT_ALIGN_TYPE		emVertAlign;							// the type of vertical alignment
	UINT					nStayTime;								// stay time, (the interval of enter or exit), unit:s
	EM_PIC_STYLE_TYPE		emEnterStyle;							// enter style
	EM_PIC_STYLE_TYPE		emExitStyle;							// exit style
	BYTE        			byReserved[128];                 		//  reserved bytes
} NET_GUIDESCREEN_TEXT_INFO;

// the note info of element
typedef struct tagNET_GUIDESCREEN_NOTE_INFO
{
	BOOL						bEnable;			// is it enable
	NET_GUIDESCREEN_TEXT_INFO	stuTextInfo;		// the text info
	NET_RECT					stuRect;			// rect info
	BYTE        				byReserved[128];    // reserved bytes
} NET_GUIDESCREEN_NOTE_INFO;

// the info of video element
typedef struct tagNET_VIDEO_ELEMENT_INFO
{
	char 						szName[MAX_COMMON_STRING_64];			// custon name
	BOOL						bFillerState;							// the filler state
	char						szPath[MAX_COMMON_STRING_128];			// the address of video file
	UINT						nPlayCount;								// play count
	UINT						nNote;									// the count notes
	NET_GUIDESCREEN_NOTE_INFO	stuNoteInfo[MAX_NOTE_COUNT];			// the notes of element
	BYTE        				byReserved[128];                 		// reserved bytes
} NET_VIDEO_ELEMENT_INFO;

// the info of picture element
typedef struct tagNET_PICTURE_ELEMENT_INFO
{
	char 						szName[MAX_COMMON_STRING_64];			// custon name
	BOOL						bFillerState;							// the filler state
	char						szPath[MAX_COMMON_STRING_128];			// the address of picture file
	UINT						nPlayTime;								// play time, unit:s
	UINT						nPlayCount;								// play count
	UINT						nDiaphaneity;							// Diaphaneity, 0-100
	UINT						nStayTime;								// stay time, unit:s
	EM_PIC_STYLE_TYPE			emEnterStyle;							// enter style
	EM_PIC_STYLE_TYPE			emExitStyle;							// exit style
	UINT						nNote;									// the count of notes
	NET_GUIDESCREEN_NOTE_INFO	stuNoteInfo[MAX_NOTE_COUNT];			// the notes of element
	BYTE        				byReserved[128];                 		// reserved bytes
} NET_PICTURE_ELEMENT_INFO;

// the info of text element
typedef struct tagNET_TEXT_ELEMENT_INFO
{
	char 						szName[MAX_COMMON_STRING_64];			// custon name
	BOOL						bFillerState;							// the filler state
	NET_GUIDESCREEN_TEXT_INFO	stuElementsText;						// text info
	UINT						nNote;									// the count of notes
	NET_GUIDESCREEN_NOTE_INFO	stuNoteInfo[MAX_NOTE_COUNT];			// the notes of element
	BYTE        				byReserved[128];                 		// reserved bytes
} NET_TEXT_ELEMENT_INFO;

// the info of place holder element
typedef struct tagNET_PLACEHOLDER_ELEMENT_INFO
{
	char 						szName[MAX_COMMON_STRING_64];			// custon name
	BOOL						bFillerState;							// the filler state
	UINT						nNote;									// the count of notes
	NET_GUIDESCREEN_NOTE_INFO	stuNoteInfo[MAX_NOTE_COUNT];			// the notes of element
	BYTE        				byReserved[128];                 		// reserved bytes
} NET_PLACEHOLDER_ELEMENT_INFO;

// capture type
typedef enum tagEM_CAPTURE_TYPE
{
	EM_CAPTURE_UNKNOWN,			// unknown
	EM_CAPTURE_VIDEO,			// video
	EM_CAPTURE_PICTURE,			// picture
} EM_CAPTURE_TYPE;

// the info of place capture element
typedef struct tagNET_CAPTURE_ELEMENT_INFO
{
	
	char 						szName[MAX_COMMON_STRING_64];			// custon name
	BOOL						bFillerState;							// the filler state
	char						szUserName[DH_USER_NAME_LEN_EX];		// user name
	char						szPassWord[DH_USER_PSW_LEN_EX];			// password
	char						szIP[DH_MAX_IPADDR_LEN_EX];				// IP address
	UINT						nPort;									// port 
	UINT						nChannel;								// channel
	EM_CAPTURE_TYPE				emCaptureType;							// capture type
	UINT						nPlayTime;								// play time, unit:s
	UINT						nNote;									// the count of notes
	NET_GUIDESCREEN_NOTE_INFO	stuNoteInfo[MAX_NOTE_COUNT];			// the notes of element
	BYTE        				byReserved[128];                 		// reserved bytes
} NET_CAPTURE_ELEMENT_INFO;

// the type of elemetns
typedef enum tagEM_ELEMENTS_TYPE
{
	EM_ELEMENTS_UNKNOWN,			// unknown
	EM_ELEMENTS_VIDEO,				// video element, corresponding to  NET_ELEMENTS_VIDEO_INFO
	EM_ELEMENTS_PICTURE,			// picture element, corresponding to NET_ELEMENTS_PICTURE_INFO
	EM_ELEMENTS_TEXT,				// text element, corresponding to NET_ELEMENTS_TEXT_INFO
	EM_ELEMENTS_PLACEHOLDER,		// place holder element, corresponding to NET_ELEMENTS_PLACEHOLDER_INFO
	EM_ELEMENTS_CAPTURE,			// capture element, corresponding to NET_ELEMENTS_CAPTURE_INFO
} EM_ELEMENTS_TYPE;

//the type of tour period
typedef enum tagEM_TOURPERIOD_TYPE
{
	EM_TOURPERIOD_UNKNOWN,		// unknown
	EM_TOURPERIOD_PROGRAMME,	// programme
	EM_TOURPERIOD_PLAN,			// programme plan
	EM_TOURPERIOD_CUSTOM,		// custom
} EM_TOURPERIOD_TYPE;


// windows info
typedef struct tagNET_GUIDESCREEN_WINDOW_INFO
{
	char				szWindowID[MAX_COMMON_STRING_64];			// windows ID
	NET_COLOR_RGBA		stuColor;									// background color
	UINT				nDiaphaneity;								// the diaphaneity of background(0-100)
	EM_TOURPERIOD_TYPE	emTourPeriodType;							// the type of tour period
	UINT				nTourPeriodTime;							// custom tour period time(unit:s)it's effective when emTourPeriodType is EM_TOURPERIOD_CUSTOM
	BOOL				bAutoPlay;									// is it auto play(it's effective when the element is video)
	BOOL				bLoopPlay;									// is it loop play(it's effective when the element is video)
	EM_ELEMENTS_TYPE	emElementsType;								// the type of elements
	UINT				nElementsCount;								// the count of elements
	char*				pstElementsBuf;								// the buf of elements
	UINT				nBufLen;									// the size of this buf
	BYTE        		byReserved[128];                 			// reserved bytes															
} NET_GUIDESCREEN_WINDOW_INFO;

// the ordinary programme info 
typedef struct tagNET_ORDINARY_INFO
{
	BOOL						bTempletState;							// is it templet
	char						szDescription[MAX_COMMON_STRING_128];	// the description of this programme
	UINT						nWhnCount;								// the count of windows
	NET_GUIDESCREEN_WINDOW_INFO	stuWindowsInfo[MAX_WINDOWS_COUNT];		// the info of windows
	BYTE        				byReserved[128];                 		// reserved bytes
} NET_ORDINARY_INFO;

// the programme info of the guid screen
typedef struct tagNET_PROGRAMME_INFO
{
	char						szProgrammeName[MAX_COMMON_STRING_64];	// programme name
	char						szProgrammeID[MAX_COMMON_STRING_64];	// programme ID
	char						szGuideScreenID[MAX_COMMON_STRING_64];	// guid screen ID
	BOOL						bEnable;								// is it enable	
	NET_ORDINARY_INFO			stuOrdinaryInfo;						// the ordinary programme info
	BYTE        				byReserved[512];                 		// reserved bytes
} NET_PROGRAMME_INFO;

// input paramer of the interface of getting programme info by ID
typedef struct tagNET_IN_GET_PROGRAMME_BYID
{
	DWORD           			dwSize;
	char						szProgrammeID[MAX_COMMON_STRING_64];	// programme ID
} NET_IN_GET_PROGRAMME_BYID;

// output paramer of the interface of getting programme info by ID
typedef struct tagNET_OUT_GET_PROGRAMME_BYID
{
	DWORD           			dwSize;
	NET_PROGRAMME_INFO			stuProgrammeInfo;			// programme info
} NET_OUT_GET_PROGRAMME_BYID;

/**************************************************************************************
*   Funcname: CLIENT_GetOneProgrammeById
*   Purpose: interface of getting programme info by ID
*   InputParam:	LLONG							:lLoginID	// login handle
*   InputParam:	NET_IN_GET_PROGRAMME_BYID*		:pInParam	// input parameter
*   OutPutParam:NET_OUT_GET_PROGRAMME_BYID*		:pOutParam 	// output parameter
*   InputParam:	int								:nWaitTime	// wait time
*   Return:		BOOL
*   Created:		%2017%:%10%:%11%  
*   Revision Record:    date:author:modify sth
**************************************************************************************/
CLIENT_NET_API BOOL CALL_METHOD CLIENT_GetOneProgrammeById(LLONG lLoginID, const NET_IN_GET_PROGRAMME_BYID *pInParam, NET_OUT_GET_PROGRAMME_BYID *pOutParam, const int nWaitTime);


// input parameter of the interface of getting all programmes
typedef struct tagNET_IN_GET_ALL_PROGRAMMES
{
	DWORD           			dwSize;
} NET_IN_GET_ALL_PROGRAMMES;

// output parameter of the interface of getting all programmes
typedef struct tagNET_OUT_GET_ALL_PROGRAMMES
{
	DWORD           			dwSize;
	UINT						nMaxCnt;					// the max count of pstProgrammeInfo
	UINT						nRetCnt;					// the count of pstProgrammeInfo actual return
	NET_PROGRAMME_INFO			*pstProgrammeInfo;			// th programmes info, this memory is applyed and released by user
} NET_OUT_GET_ALL_PROGRAMMES;

/**************************************************************************************
*   Funcname: CLIENT_GetAllProgrammes
*   Purpose: interface of getting all programmes
*   InputParam:	LLONG							:lLoginID		// login handle
*   InputParam:	NET_IN_GET_ALL_PROGRAMMES*		:pInParam		// input parameter
*   OutPutParam:NET_OUT_GET_ALL_PROGRAMMES*		:pOutParam 		// output parameter
*   InputParam:	int								:nWaitTime		// wait time
*   Return:		BOOL
*   Created:		%2017%:%10%:%11%  
*   Revision Record:    date:author:modify sth
**************************************************************************************/
CLIENT_NET_API BOOL CALL_METHOD CLIENT_GetAllProgrammes(
												LLONG lLoginID, const NET_IN_GET_ALL_PROGRAMMES* pInParam, 
												NET_OUT_GET_ALL_PROGRAMMES* pOutParam, const int nWaitTime);


// briefly info of programme
typedef struct tagNET_BRIEFLY_PROGRAMME_INFO
{
	char				szProgrammeName[MAX_COMMON_STRING_64];	// programme name
	char				szProgrammeID[MAX_COMMON_STRING_64];	// programme ID
	char				szGuideScreenID[MAX_COMMON_STRING_64];	// guid screen ID
	BOOL				bEnable;								// is it enable
	BOOL				bTempletState;							// is it templetState
	BYTE        		byReserved[512];                 			// reserved bytes	
} NET_BRIEFLY_PROGRAMME_INFO;

// input parameter of  the interface of getting all briefly programmes 
typedef struct tagNET_IN_GET_ALL_BRIEFLYPROGRAMMES
{
	DWORD           				dwSize;
} NET_IN_GET_ALL_BRIEFLYPROGRAMMES;

// output parameter of  the interface of getting all briefly programmes 
typedef struct tagNET_OUT_GET_ALL_BRIEFLYPROGRAMMES
{
	DWORD           				dwSize;
	UINT							nRetCnt;									// th count actual return
	NET_BRIEFLY_PROGRAMME_INFO		stuBriProgrammes[MAX_PROGRAMMES_COUNT];		// briefly info of programme
} NET_OUT_GET_ALL_BRIEFLYPROGRAMMES;


/**************************************************************************************
*   Funcname: CLIENT_GetAllBrieflyProgrammes
*   Purpose: interface of getting all briefly programmes 
*   InputParam:	LLONG									:lLoginID		// login handle
*   InputParam:	NET_IN_GET_ALL_BRIEFLYPROGRAMMES*		:pInParam		// input parameter
*   OutPutParam:NET_OUT_GET_ALL_BRIEFLYPROGRAMMES*		:pOutParam 		// output parameter
*   InputParam:	int										:nWaitTime		// wait time
*   Return:		BOOL
*   Created:		%2017%:%10%:%11%  
*   Revision Record:    date:author:modify sth
**************************************************************************************/
CLIENT_NET_API BOOL CALL_METHOD CLIENT_GetAllBrieflyProgrammes(
												LLONG lLoginID, const NET_IN_GET_ALL_BRIEFLYPROGRAMMES* pInParam, 
												NET_OUT_GET_ALL_BRIEFLYPROGRAMMES* pOutParam, const int nWaitTime);


// input parameter of the interface of adding a programme
typedef struct tagNET_IN_ADD_ONE_PROGRAMME
{
	DWORD           			dwSize;
	NET_PROGRAMME_INFO			stuProgrammeInfo;	// programme info
} NET_IN_ADD_ONE_PROGRAMME;


// output parameter  of the interface of adding a programme
typedef struct tagNET_OUT_ADD_ONE_PROGRAMME
{
	DWORD           			dwSize;
	char						szProgrammeID[MAX_COMMON_STRING_64];	// programme ID
} NET_OUT_ADD_ONE_PROGRAMME;

/**************************************************************************************
*   Funcname: CLIENT_AddOneProgramme
*   Purpose: interface of adding a programme
*   InputParam:	LLONG						:lLoginID		// login handle
*   InputParam:	NET_IN_ADD_ONE_PROGRAMME*	:pInParam 		// input parameter
*   OutPutParam:NET_OUT_ADD_ONE_PROGRAMME* 	:pOutParam		// output parameter
*   InputParam:	int							:nWaitTime		// wait time
*   Return:		BOOL
*   Created:		%2017%:%10%:%11%  
*   Revision Record:    date:author:modify sth
**************************************************************************************/
CLIENT_NET_API BOOL CALL_METHOD CLIENT_AddOneProgramme(LLONG lLoginID, const NET_IN_ADD_ONE_PROGRAMME* pInParam, NET_OUT_ADD_ONE_PROGRAMME *pOutParam, const int nWaitTime);


// input parameter of the interface of modify a programme
typedef struct tagNET_IN_MODIFY_ONE_PROGRAMME
{
	DWORD           			dwSize;
	NET_PROGRAMME_INFO			stuProgrammeInfo;	// programme info
} NET_IN_MODIFY_ONE_PROGRAMME;


// output parameter of the interface of modify a programme
typedef struct tagNET_OUT_MODIFY_ONE_PROGRAMME
{
	DWORD           			dwSize;
} NET_OUT_MODIFY_ONE_PROGRAMME;

/**************************************************************************************
*   Funcname: CLIENT_ModifyOneProgrammeByID
*   Purpose: interface of modify a programme
*   InputParam:	LLONG							:lLoginID		// login handle
*   InputParam:	NET_IN_MODIFY_ONE_PROGRAMME*	:pInParam 		// input parameter
*   OutputParam:NET_OUT_MODIFY_ONE_PROGRAMME*	:pOutParam 		// output parameter
*   InputParam:	int								:nWaitTime		// wait time
*   Return:		BOOL
*   Created:		%2017%:%10%:%11%  
*   Revision Record:    date:author:modify sth
**************************************************************************************/
CLIENT_NET_API BOOL CALL_METHOD CLIENT_ModifyOneProgrammeByID(LLONG lLoginID, const NET_IN_MODIFY_ONE_PROGRAMME* pInParam, NET_OUT_MODIFY_ONE_PROGRAMME* pOutParam, const int nWaitTime);


// input parameter of the interface of delete some programmes
typedef struct tagNET_IN_DEL_PROGRAMMES
{
	DWORD           dwSize;
	UINT			nProgrammeID;													// the count of programmes which are need to delete
	char			szProGrammeIdList[MAX_PROGRAMMES_COUNT][MAX_COMMON_STRING_64];	// the IDs of these programmes which are need to delete
} NET_IN_DEL_PROGRAMMES;

// output parameter of the interface of delete some programmes
typedef struct tagNET_OUT_DEL_PROGRAMMES
{
	DWORD           dwSize;
} NET_OUT_DEL_PROGRAMMES;

/**************************************************************************************
*   Funcname: CLIENT_DelMultiProgrammesById
*   Purpose: interface of delete some programmes
*   InputParam:	LLONG						:lLoginID	// login handle
*   InputParam:	NET_IN_DEL_PROGRAMMES*		:pInParam 	// input parameter
*   InputParam:	NET_OUT_DEL_PROGRAMMES*		:pOutParam 	// output parameter
*   InputParam:	int							:nWaitTime	// wait time
*   Return:		BOOL
*   Created:		%2017%:%10%:%11%  
*   Revision Record:    date:author:modify sth
**************************************************************************************/
CLIENT_NET_API BOOL CALL_METHOD CLIENT_DelMultiProgrammesById(
											LLONG lLoginID, const NET_IN_DEL_PROGRAMMES* pInParam, 
											NET_OUT_DEL_PROGRAMMES *pOutParam, const int nWaitTime);

// the infos of immediately programme plan
typedef struct tagNET_IMMEDIATELY_PLAN_INFO
{
	char			szPlanName[MAX_COMMON_STRING_64];			// programme plan name
	char			szPlanID[MAX_COMMON_STRING_64];				// programme plan ID
	BOOL			bEnable;									// is it enable
	UINT			nPlayTime;									// play time, unit:minute
	char			szProgrammeName[MAX_COMMON_STRING_64];		// programme name
	char			szProgrammeID[MAX_COMMON_STRING_64];		// programme ID
	BYTE        	byReserved[512];                 			// reserved bytes
} NET_IMMEDIATELY_PLAN_INFO;

// the data struct of programme
typedef struct tagNET_PROGRAMME_DATA
{
	DWORD                dwYear;                  // year
    DWORD                dwMonth;                 // month
    DWORD                dwDay;                	  // day
} NET_PROGRAMME_DATA;

// state of review
typedef enum tagEM_REVIES_STATE
{
	EM_REVIES_UNKNOWN,			// unknown
	EM_REVIES_PASS,				// pass 
	EM_REVIES_NOTPASS,			// not pass
} EM_REVIES_STATE;

// the info of programme in plan
typedef struct tagNET_PROGRAMME_OF_PLAN
{
	char						szProgrammeName[MAX_COMMON_STRING_64];	// programme name
	char						szProgrammeID[MAX_COMMON_STRING_64];	// programme ID
	BOOL						bIsBgProgramme;							// is it background
	NET_PROGRAMME_TIME_INFO		stuSatrtTime;							// start time
	NET_PROGRAMME_TIME_INFO		stuEndTime;								// end time
	BYTE        				byReserved[128];                 		// reserved bytes
} NET_PROGRAMME_OF_PLAN;

// the infos of timer programme plan
typedef struct tagNET_TIMER_PLAN_INFO
{
	char					szPlanName[MAX_COMMON_STRING_64];			// programme plan name
	char					szPlanID[MAX_COMMON_STRING_64];				// programme plan ID
	EM_SCREEN_DATE_TYPE		emDataType;									// the type of data
	UINT					nDataCount;									// the count of data
	UINT					nPlayDates[MAX_PLAYDATES_COUNT];			// the list of data
	NET_PROGRAMME_DATA		stuSatrtDate;								// start time
	NET_PROGRAMME_DATA		stuEndDate;									// end time
	EM_REVIES_STATE			emReviewState;								// state of review
	char					szReviewOpinion[MAX_COMMON_STRING_64];		// the options of review
	BOOL					bOverdue;									// is it overdue
	UINT					nProgrammes;								// the count of programmes
	NET_PROGRAMME_OF_PLAN	stuProgrammes[MAX_PROGRAMMES_COUNT];		// the list of programmes
	BYTE        			byReserved[512];                 			// resverd bytes
} NET_TIMER_PLAN_INFO;

// input parameter of the interface of getting all programme plans
typedef struct tagNET_IN_GET_ALL_PROGRAMMEPLANS
{
	DWORD           			dwSize;
} NET_IN_GET_ALL_PROGRAMMEPLANS;

// output parameter of the interface of getting all programme plans
typedef struct tagNET_OUT_GET_ALL_PROGRAMMEPLANS
{
	DWORD           			dwSize;
	UINT						nMaxPlanCnt;			// the max count of immediately programmes and timer programmes
	UINT						nRetImmCnt;				// the count of immediately programmes actual return
	NET_IMMEDIATELY_PLAN_INFO*	pstImmePlan;			// the info of immediately programmes
	UINT						nRetTimerCnt;			// the count of timer programmes actual return
	NET_TIMER_PLAN_INFO*		pstTimerPlan;			// the info of timer programmes
} NET_OUT_GET_ALL_PROGRAMMEPLANS;

/**************************************************************************************
*   Funcname: CLIENT_GetAllProgrammePlans
*   Purpose: interface of getting all programme plans
*   InputParam:	LLONG								:lLoginID		// login handle 
*   InputParam:	NET_IN_GET_ALL_PROGRAMMEPLANS*		:pInParam		// input parameter
*   OutPutParam:NET_OUT_GET_ALL_PROGRAMMEPLANS*		:pOutParam  	// output parameter
*   InputParam:	int									:nWaitTime		// wait time
*   Return:		BOOL
*   Created:		%2017%:%10%:%11%  
*   Revision Record:    date:author:modify sth
**************************************************************************************/
CLIENT_NET_API BOOL CALL_METHOD CLIENT_GetAllProgrammePlans(
											LLONG lLoginID, const NET_IN_GET_ALL_PROGRAMMEPLANS *pInParam,
											NET_OUT_GET_ALL_PROGRAMMEPLANS *pOutParam, const int nWaitTime);


// the type of programme plan
typedef enum tagEM_PROGRAMMEPLAN_TYPE
{
	EM_PROGRAMMEPLAN_UNKNOWN,		// unknown
	EM_PROGRAMMEPLAN_IMME,			// immediately plan
	EM_PROGRAMMEPLAN_TIMER,			// timer plan
} EM_PROGRAMMEPLAN_TYPE;

// input parameter of the interface of getting programme plan by ID
typedef struct tagNET_IN_GET_PROGRAMMEPLAN_BYID
{
	DWORD           			dwSize;
	char						szPlanID[MAX_COMMON_STRING_64];		// programme planID
} NET_IN_GET_PROGRAMMEPLAN_BYID;

// output parameter of the interface of getting programme plan by ID
typedef struct tagNET_OUT_GET_PROGRAMMEPLAN_BYID
{
	DWORD           			dwSize;
	EM_PROGRAMMEPLAN_TYPE		emPlanType;		// the type of programme plan
	NET_IMMEDIATELY_PLAN_INFO	stuImmePlan;	// the info of immediately programme, it's effective when emPlanType is EM_PROGRAMMEPLAN_IMME
	NET_TIMER_PLAN_INFO			stuTimerPlan;	// the info of timer programme,  it's effective when emPlanType is EM_PROGRAMMEPLAN_TIMER
} NET_OUT_GET_PROGRAMMEPLAN_BYID;

/**************************************************************************************
*   Funcname: CLIENT_GetOneProgrammePlanByID
*   Purpose: interface of getting programme plan by ID
*   InputParam:	LLONG								:lLoginID	// login handle
*   InputParam:	NET_IN_GET_PROGRAMMEPLAN_BYID*		:pInParam	// input parameter
*   OutPutParam:NET_OUT_GET_PROGRAMMEPLAN_BYID*		:pOutParam  // output parameter
*   InputParam:	int									:nWaitTime	// wait time
*   Return:		BOOL
*   Created:		%2017%:%10%:%11%  
*   Revision Record:    date:author:modify sth
**************************************************************************************/
CLIENT_NET_API BOOL CALL_METHOD CLIENT_GetOneProgrammePlanByID(
										LLONG lLoginID,  NET_IN_GET_PROGRAMMEPLAN_BYID *pInParam,
										NET_OUT_GET_PROGRAMMEPLAN_BYID* pOutParam, const int nWaitTime);

// input parameter of the interface of adding a  immediately programme plan
typedef struct tagNET_IN_ADD_IMME_PROGRAMMEPLAN
{
	DWORD           			dwSize;
	NET_IMMEDIATELY_PLAN_INFO	stuImmePlan;		// the info of immediately programme
} NET_IN_ADD_IMME_PROGRAMMEPLAN;

// input parameter of the interface of adding a  timer programme plan
typedef struct tagNET_IN_ADD_TIMER_PROGRAMMEPLAN
{
	DWORD           			dwSize;
	NET_TIMER_PLAN_INFO			stuTimerPlan;		// the info of timer programme
} NET_IN_ADD_TIMER_PROGRAMMEPLAN;

// output parameter of the interface of adding a programme plan
typedef struct tagNET_OUT_ADD_PROGRAMMEPLAN
{
	DWORD           		dwSize;
	char					szPlanID[MAX_COMMON_STRING_64];				// programme plan ID
} NET_OUT_ADD_PROGRAMMEPLAN;

/**************************************************************************************
*   Funcname: CLIENT_AddOneImmediProgrammePlan
*   Purpose: interface of adding a  immediately programme plan
*   InputParam:	LLONG							:lLoginID		// login handle
*   InputParam:	NET_IN_ADD_IMME_PROGRAMMEPLAN*	:pInParam  		// input parameter
*   OutPutParam:NET_OUT_ADD_PROGRAMMEPLAN*		:pstOutParam  	// output parameter
*   InputParam:	int								:nWaitTime		// wait time
*   Return:		BOOL
*   Created:		%2017%:%10%:%11%  
*   Revision Record:    date:author:modify sth
**************************************************************************************/
CLIENT_NET_API BOOL CALL_METHOD CLIENT_AddOneImmediProgrammePlan(
										LLONG lLoginID,  const NET_IN_ADD_IMME_PROGRAMMEPLAN* pInParam, 
										NET_OUT_ADD_PROGRAMMEPLAN* pOutParam, const int nWaitTime);


/**************************************************************************************
*   Funcname: CLIENT_AddOneTimerProgrammePlan
*   Purpose: interface of adding a  timer programme plan
*   InputParam:	LLONG								:lLoginID		// login handle
*   InputParam:	NET_IN_ADD_TIMER_PROGRAMMEPLAN*		:pInParam		// input parameter
*   OutPutParam:NET_OUT_ADD_PROGRAMMEPLAN*			:pOutParam  	// output parameter
*   InputParam:	int									:nWaitTime		// wait time
*   Return:		BOOL
*   Created:		%2017%:%10%:%11%  
*   Revision Record:    date:author:modify sth
**************************************************************************************/
CLIENT_NET_API BOOL CALL_METHOD CLIENT_AddOneTimerProgrammePlan(
										LLONG lLoginID,  const NET_IN_ADD_TIMER_PROGRAMMEPLAN* pInParam, 
										NET_OUT_ADD_PROGRAMMEPLAN* pOutParam, const int nWaitTime);

// input parameter of the interface of modify a  immediately programme plan
typedef struct tagNET_IN_MODIFY_IMME_PROGRAMMEPLAN
{
	DWORD           			dwSize;
	NET_IMMEDIATELY_PLAN_INFO	stuImmePlan;		// the info of immediately programme
} NET_IN_MODIFY_IMME_PROGRAMMEPLAN;

// output parameter of the interface of modify a  immediately programme plan
typedef struct tagNET_OUT_MODIFY_IMME_PROGRAMMEPLAN
{
	DWORD           			dwSize;
} NET_OUT_MODIFY_IMME_PROGRAMMEPLAN;


/**************************************************************************************
*   Funcname: CLIENT_ModifyOneImmediProgrammePlan
*   Purpose: interface of modify a  immediately programme plan
*   InputParam:	LLONG									:lLoginID	// login handle
*   InputParam:	NET_IN_MODIFY_IMME_PROGRAMMEPLAN*		:pInParam  	// input parameter
*   OutPutParam:NET_OUT_MODIFY_IMME_PROGRAMMEPLAN*		:pOutParam  // output parameter
*   InputParam:	int										:nWaitTime	// wait time
*   Return:		BOOL
*   Created:		%2017%:%10%:%11%  
*   Revision Record:    date:author:modify sth
**************************************************************************************/
CLIENT_NET_API BOOL CALL_METHOD CLIENT_ModifyOneImmediProgrammePlan(
										LLONG lLoginID,  const NET_IN_MODIFY_IMME_PROGRAMMEPLAN* pInParam,
										NET_OUT_MODIFY_IMME_PROGRAMMEPLAN* pOutParam, const int nWaitTime);


// input parameter of the interface of modify a  timer programme plan
typedef struct tagNET_IN_MODIFY_TIMER_PROGRAMMEPLAN
{
	DWORD           			dwSize;
	NET_TIMER_PLAN_INFO			stuTimerPlan;		// the info of timer programme
} NET_IN_MODIFY_TIMER_PROGRAMMEPLAN;

// output parameter of the interface of modify a  timer programme plan
typedef struct tagNET_OUT_MODIFY_TIMER_PROGRAMMEPLAN
{
	DWORD           			dwSize;
} NET_OUT_MODIFY_TIMER_PROGRAMMEPLAN;

/**************************************************************************************
*   Funcname: CLIENT_ModifyOneTimerProgrammePlan
*   Purpose: interface of modify a  timer programme plan
*   InputParam:	LLONG									:lLoginID	// login handle
*   InputParam:	NET_IN_MODIFY_TIMER_PROGRAMMEPLAN*		:pInParam  	// input parameter
*   OutputParam:NET_OUT_MODIFY_TIMER_PROGRAMMEPLAN*		:pOutParam  // output parameter
*   InputParam:	int										:nWaitTime	// wait time
*   Return:		BOOL
*   Created:		%2017%:%10%:%11%  
*   Revision Record:    date:author:modify sth
**************************************************************************************/
CLIENT_NET_API BOOL CALL_METHOD CLIENT_ModifyOneTimerProgrammePlan(
										LLONG lLoginID,  const NET_IN_MODIFY_TIMER_PROGRAMMEPLAN* pInParam, 
										NET_OUT_MODIFY_TIMER_PROGRAMMEPLAN* pOutParam, const int nWaitTime);

// input parameter of the interface of delete same programme plans
typedef struct tagNET_IN_DEL_PROGRAMMEPLANS
{
	DWORD		dwSize;
	UINT		nPlanID;												// the count of plans which are need to delete
	char		szPlanID[MAX_PROGRAMMES_COUNT][MAX_COMMON_STRING_64];	// the list of plans which are need to delete
} NET_IN_DEL_PROGRAMMEPLANS;

// output parameter of the interface of delete same programme plans
typedef struct tagNET_OUT_DEL_PROGRAMMEPLANS
{
	DWORD		dwSize;
} NET_OUT_DEL_PROGRAMMEPLANS;

/**************************************************************************************
*   Funcname: CLIENT_DelMultiProgrammePlans
*   Purpose: interface of delete same programme plans
*   InputParam:	LLONG							:lLoginID		// login handle
*   InputParam:	NET_IN_DEL_PROGRAMMEPLANS*		:pInParam  		// input parameter
*   InputParam:	NET_OUT_DEL_PROGRAMMEPLANS*		:pOutParam  	// output parameter
*   InputParam:	int								:nWaitTime		// wait time
*   Return:		BOOL
*   Created:		%2017%:%10%:%11%  
*   Revision Record:    date:author:modify sth
**************************************************************************************/
CLIENT_NET_API BOOL CALL_METHOD CLIENT_DelMultiProgrammePlans(
										LLONG lLoginID,  const NET_IN_DEL_PROGRAMMEPLANS* pInParam, 
										NET_OUT_DEL_PROGRAMMEPLANS* pOutParam, const int nWaitTime);

#define MAX_GD_COUNT	42

// the color type of light band
typedef enum tagEM_GD_COLOR_TYPE
{
	EM_GD_COLOR_RED,		// red 
	EM_GD_COLOR_GREEN,		// green
	EM_GD_COLOR_YELLOW,		// yellow
} EM_GD_COLOR_TYPE;

// input paramerter of interface of setting the status of light band
typedef struct tagNET_IN_SET_GD_STATUS
{
	DWORD				dwSize;
	char				szScreenID[MAX_COMMON_STRING_64];		// screen ID(windows ID)
	UINT				nGDNum;									// the count of light bands
	EM_GD_COLOR_TYPE	emStatus[MAX_GD_COUNT];					// the info of these light bands
} NET_IN_SET_GD_STATUS;

// output paramerter of interface of setting the status of light band
typedef struct tagNET_OUT_SET_GD_STATUS
{
	DWORD				dwSize;
} NET_OUT_SET_GD_STATUS;

/**************************************************************************************
*   Funcname: CLIENT_SetGuideScreenGDStatus
*   Purpose: interface of setting the status of light band
*   InputParam:	LLONG						:lLoginID			// login handle
*   InputParam:	NET_IN_SET_GD_STATUS*		:pInParam  		// input parameter
*   InputParam:	NET_OUT_SET_GD_STATUS*		:pOutParam  		// output parameter
*   InputParam:	int							:nWaitTime		// wait time
*   Return:		BOOL
*   Created:		%2017%:%10%:%11%  
*   Revision Record:    date:author:modify sth
**************************************************************************************/
CLIENT_NET_API BOOL CALL_METHOD CLIENT_SetGuideScreenGDStatus(
										LLONG lLoginID,  const NET_IN_SET_GD_STATUS* pInParam, 
										NET_OUT_SET_GD_STATUS* pOutParam, const int nWaitTime);



///////////////////////////////// directory management /////////////////////////////////////////

// add nodes,user malloc memory of pInParam and pOutParam
CLIENT_NET_API BOOL CALL_METHOD CLIENT_OrganizationAddNodes(LLONG lLoginID, const DH_IN_ORGANIZATION_ADD_NODES* pInParam, DH_OUT_ORGANIZATION_ADD_NODES* pOutParam, int nWaitTime = 1000);

// delete nodes,user malloc memory of pInParam and pOutParam
CLIENT_NET_API BOOL CALL_METHOD CLIENT_OrganizationDeleteNodes(LLONG lLoginID, const DH_IN_ORGANIZATION_DELETE_NODES* pInParam, DH_OUT_ORGANIZATION_DELETE_NODES* pOutParam, int nWaitTime = 1000);

// get info of nodes,user malloc memory of pInParam and pOutParam
CLIENT_NET_API BOOL CALL_METHOD CLIENT_OrganizationGetNodes(LLONG lLoginID, const DH_IN_ORGANIZATION_GET_NODES* pInParam, DH_OUT_ORGANIZATION_GET_NODES* pOutParam, int nWaitTime = 1000);

//set node,user malloc memory of pInParam and pOutParam
CLIENT_NET_API BOOL CALL_METHOD CLIENT_OrganizationSetNode(LLONG lLoginID, const DH_IN_ORGANIZATION_SET_NODE* pInParam, DH_OUT_ORGANIZATION_SET_NODE* pOutParam, int nWaitTime = 1000);


//////////////////////////////// network caught /////////////////////////////////

// start caught,user malloc memory of pInParam and pOutParam
CLIENT_NET_API LLONG CALL_METHOD CLIENT_StartSniffer(LLONG lLoginID, const DH_IN_START_SNIFFER* pInParam, DH_OUT_START_SNIFFER* pOutParam, int nWaitTime = 1000);

// stop caught
CLIENT_NET_API BOOL CALL_METHOD CLIENT_StopSniffer(LLONG lLoginID, LLONG lSnifferID);

// get caught state,user malloc memory of pInParam and pOutParam
CLIENT_NET_API BOOL CALL_METHOD CLIENT_GetSnifferInfo(LLONG lLoginID, const DH_IN_GET_SNIFFER_INFO* pInParam, DH_OUT_GET_SNIFFER_INFO* pOutParam, int nWaitTime = 1000);

//////////////////////////////// Play Library Related Port /////////////////////////////////

//Snapshot JPEG
CLIENT_NET_API BOOL CALL_METHOD  CLIENT_GetPicJPEG(LLONG lPlayHandle, BYTE* pJpegBuf, DWORD dwBufSize,DWORD *pJpegSize,int quality);


//////////////////////////////// manage remote file /////////////////////////////////

// create file,user malloc memory of pInParam and pOutParam
CLIENT_NET_API BOOL CALL_METHOD CLIENT_CreateRemoteFile(LLONG lLoginID, const DH_IN_CREATE_REMOTE_FILE* pInParam, DH_OUT_CREATE_REMOTE_FILE* pOutParam, int nWaitTime = 1000);

// remove file,user malloc memory of pInParam and pOutParam
CLIENT_NET_API BOOL CALL_METHOD CLIENT_RemoveRemoteFiles(LLONG lLoginID, const DH_IN_REMOVE_REMOTE_FILES* pInParam, DH_OUT_REMOVE_REMOTE_FILES* pOutParam, int nWaitTime = 1000);

// rename,user malloc memory of pInParam and pOutParam
CLIENT_NET_API BOOL CALL_METHOD CLIENT_RenameRemoteFile(LLONG lLoginID, const DH_IN_RENAME_REMOTE_FILE* pInParam, DH_OUT_RENAME_REMOTE_FILE* pOutParam, int nWaitTime = 1000);

// display files and subdirectories in a directory,user malloc memory of pInParam and pOutParam
CLIENT_NET_API BOOL CALL_METHOD CLIENT_ListRemoteFile(LLONG lLoginID, const DH_IN_LIST_REMOTE_FILE* pInParam, DH_OUT_LIST_REMOTE_FILE* pOutParam, int nWaitTime = 1000);

// sybcgribize fule upload,user malloc memory of pInParam and pOutParam
CLIENT_NET_API BOOL CALL_METHOD CLIENT_UploadRemoteFile(LLONG lLoginID, const DH_IN_UPLOAD_REMOTE_FILE* pInParam, DH_OUT_UPLOAD_REMOTE_FILE* pOutParam, int nWaitTime = 1000);

// CLIENT_StartUploadRemoteFile callback function, lUploadFileHandle is CLIENT_StartUploadRemoteFile return value
typedef void (CALLBACK *fUploadFileCallBack) (LLONG lUploadFileHandle, int nTotalSize, int nSendSize, LDWORD dwUser);
// start upload remote file,user malloc memory of pInParam and pOutParam
CLIENT_NET_API LLONG CALL_METHOD CLIENT_StartUploadRemoteFile(LLONG lLoginID, const DH_IN_UPLOAD_REMOTE_FILE* pInParam, DH_OUT_UPLOAD_REMOTE_FILE* pOutParam, fUploadFileCallBack cbUploadFile, LDWORD dwUser);

// stop upload remote file
CLIENT_NET_API BOOL CALL_METHOD CLIENT_StopUploadRemoteFile(LLONG lUploadFileID);

// remote project, play audio file on device end,user malloc memory of pInParam and pOutParam
CLIENT_NET_API BOOL CALL_METHOD CLIENT_PlayAudioFile(LLONG lLoginID, const NET_IN_PLAY_AUDIO_FILE* pInParam, NET_OUT_PLAY_AUDIO_FILE* pOutParam, int nWaitTime = 1000);

// file download, only for small file,user malloc memory of pInParam and pOutParam
CLIENT_NET_API BOOL CALL_METHOD CLIENT_DownloadRemoteFile(LLONG lLoginID, const DH_IN_DOWNLOAD_REMOTE_FILE* pInParam, DH_OUT_DOWNLOAD_REMOTE_FILE* pOutParam, int nWaitTime = 1000);

// set file property,user malloc memory of pInParam and pOutParam
CLIENT_NET_API BOOL CALL_METHOD CLIENT_SetFileAttribute(LLONG lLoginID, const DH_IN_SET_FILEATTRIBUTE* pInParam, DH_OUT_SET_FILEATTRIBUTE* pOutParam, int nWaitTime = 1000);

CLIENT_NET_API BOOL CALL_METHOD CLIENT_DownloadPieceFile(LLONG lLoginID, const NET_IN_DOWNLOAD_PIECE_FILE* pInParam, NET_OUT_DOWNLOAD_PIECE_FILE* pOutParam,int nWaitTime = 1000);
////////////////////////////////// manage storage device ////////////////////////////////////////

// Get ISCSI target list,user malloc memory of pInParam and pOutParam
CLIENT_NET_API BOOL CALL_METHOD	CLIENT_GetISCSITargets(LLONG lLoginID, const DH_IN_ISCSI_TARGETS* pInParam, DH_OUT_ISCSI_TARGETS* pOutParam, int nWaitTime = 1000);
// user malloc memory of pInParam and pOutParam
CLIENT_NET_API BOOL CALL_METHOD CLIENT_GetBitmap(LLONG lLoginID , const DH_IN_BITMAP* pInParam, DH_OUT_BITMAP* pOutParam, int nWaitTime = 1000);

// Get storage device name list,user malloc memory of pstuNames
CLIENT_NET_API BOOL CALL_METHOD	CLIENT_GetStorageDeviceNames(LLONG lLoginID, DH_STORAGE_DEVICE_NAME* pstuNames, int nMaxCount, int* pnRetCount, int nWaitTime = 1000);

// Get storage device info,user malloc memory of pDevice
CLIENT_NET_API BOOL CALL_METHOD	CLIENT_GetStorageDeviceInfo(LLONG lLoginID, const char* pszDevName, DH_STORAGE_DEVICE* pDevice, int nWaitTime = 1000);

// subscribe record file update info,user malloc memory of pInParam and pOutParam
CLIENT_NET_API LLONG CALL_METHOD CLIENT_AttachRecordInfo(LLONG lLoginID, const NET_IN_ATTACH_RECORD_INFO* pInParam, NET_OUT_ATTACH_RECORD_INFO* pOutParam, int nWaitTime = 1000);

// cancel subscribe record file update info, lAttachHandle is CLIENT_AttachRecordInfo return value
CLIENT_NET_API BOOL CALL_METHOD CLIENT_DetachRecordInfo(LLONG lAttachHandle);

// subscribe writing in remote torage info status,user malloc memory of pInParam and pOutParam
CLIENT_NET_API LLONG CALL_METHOD CLIENT_NetStorageAttachWriteInfo(LLONG lLoginID, const NET_IN_STORAGE_ATTACH_WRITE_INFO* pInParam, NET_OUT_STORAGE_ATTACH_WRITE_INFO* pOutParam, int nWaitTime);

// cancel writing in remote device info, lAttachHandle is the return value of CLIENT_NetStorageAttachWriteInfo 
CLIENT_NET_API BOOL CALL_METHOD CLIENT_NetStorageDetachWriteInfo(LLONG lAttachHandle);

// search remote storage wiring in info status,user malloc memory of pInParam and pOutParam
CLIENT_NET_API BOOL CALL_METHOD CLIENT_NetStorageGetWriteInfo(LLONG lLoginID, const NET_IN_STORAGE_GET_WRITE_INFO* pInParam, NET_OUT_STORAGE_GET_WRITE_INFO* pOutParam, int nWaitTime);

// RAID operation, different operation type corresponding different structures,user malloc memory of pInBuf and pOutBuf,please refer to the corresponding structure of emType
CLIENT_NET_API BOOL CALL_METHOD CLIENT_OperateRaid(LLONG lLoginID, NET_RAID_OPERATE_TYPE emType, void* pInBuf, void* pOutBuf, int nWaitTime);

/////////////////////////////////// cascade device ///////////////////////////////////////

// search cascade video,user malloc memory of pInParam and pOutParam
CLIENT_NET_API BOOL CALL_METHOD CLIENT_MatrixSearch(LLONG lLoginID, const DH_IN_MATRIX_SEARCH* pInParam, DH_OUT_MATRIX_SEARCH* pOutParam, int nWaitTime = 1000);

// get cascade tree,user malloc memory of pInParam and pOutParam
CLIENT_NET_API BOOL CALL_METHOD CLIENT_GetMatrixTree(LLONG lLoginID, const DH_IN_GET_MATRIX_TREE* pInParam, DH_OUT_GET_MATRIX_TREE* pOutParam, int nWaitTime = 1000);

// get superior cascade device list info,user malloc memory of pInParam and pOutParam
CLIENT_NET_API BOOL CALL_METHOD CLIENT_GetSuperiorMatrixList(LLONG lLoginID, const DH_IN_GET_SUPERIOR_MATRIX_LIST* pInParam, DH_OUT_GET_SUPERIOR_MATRIX_LIST* pOutParam, int nWaitTime = 1000);

/************************************************************************/
/*							backup record comes back
/************************************************************************/

// start record backup restore
CLIENT_NET_API LLONG CALL_METHOD CLIENT_StartRecordBackupRestore(LLONG lLoginID);

// stop record backup restore
CLIENT_NET_API void CALL_METHOD CLIENT_StopRecordBackupRestore(LLONG lRestoreID);

// add record backup restore task,user malloc memory of pInParam
CLIENT_NET_API BOOL CALL_METHOD CLIENT_AddRecordBackupRestoreTask(LLONG lRestoreID, const DH_IN_ADD_REC_BAK_RST_TASK* pInParam, int nWaitTime = 1000);

// remove record backup restore task,user malloc memory of pInParam
CLIENT_NET_API BOOL CALL_METHOD CLIENT_RemoveRecordBackupRestoreTask(LLONG lRestoreID, const DH_IN_REMOVE_REC_BAK_RST_TASK* pInParam, int nWaitTime = 1000);

// get record backup restore task,user malloc memory of pInParam and pOutParam
CLIENT_NET_API BOOL CALL_METHOD CLIENT_QueryRecordBackupRestoreTask(LLONG lRestoreID, const DH_IN_QUERY_REC_BAK_RST_TASK* pInParam, DH_OUT_QUERY_REC_BAK_RST_TASK* pOutParam, int nWaitTime = 1000);

//////////////////////////////// Encode Manager  ////////////////////////////////
// judicial burn in plan parameter coding,user malloc memory of pInParam and pOutParam
CLIENT_NET_API BOOL CALL_METHOD CLIENT_GetEncodePlan(LLONG lLoginID, const DH_IN_GET_ENCODE_PLAN* pInParam, DH_OUT_GET_ENCODE_PLAN* pOutParam, int nWaitTime = 1000);

// get expect time by resolution and CD capacity, pInParam and pOutParam application an release by use
CLIENT_NET_API BOOL CALL_METHOD CLIENT_GetEncodePlanByResolution(LLONG lLoginID, const NET_IN_GET_ENCODEPLANE_BY_RESOLUTION* pInParam, NET_OUT_GET_ENCODEPLANE_BY_RESOLUTION* pOutParam, int nWaitTime /*= 1000*/);

/************************************************************************/
/*                           database record operation                         */
/************************************************************************/
// by search filter search record,user malloc memory of pInParam and pOutParam
CLIENT_NET_API BOOL    CALL_METHOD CLIENT_FindRecord(LLONG lLoginID, NET_IN_FIND_RECORD_PARAM* pInParam, NET_OUT_FIND_RECORD_PARAM* pOutParam, int waittime=1000);    

// search record :nFilecount: need search items,  return value as media file items  return value<nFilecountas corresponding period file search complete,user malloc memory of pInParam and pOutParam
CLIENT_NET_API int    CALL_METHOD CLIENT_FindNextRecord(NET_IN_FIND_NEXT_RECORD_PARAM* pInParam, NET_OUT_FIND_NEXT_RECORD_PARAM* pOutParam, int waittime=1000);

// search record items,user malloc memory of pInParam and pOutParam
CLIENT_NET_API BOOL    CALL_METHOD CLIENT_QueryRecordCount(NET_IN_QUEYT_RECORD_COUNT_PARAM* pInParam, NET_OUT_QUEYT_RECORD_COUNT_PARAM* pOutParam, int waittime=1000);

// end record search
CLIENT_NET_API BOOL CALL_METHOD CLIENT_FindRecordClose(LLONG lFindHandle);

/************************************************************************/
/*                            PTZ metadata port subcribe
/************************************************************************/
// subscribe PTZ metadata port,user malloc memory of pstuInPtzStatusProc and pstuOutPtzStatusProc
CLIENT_NET_API LLONG CALL_METHOD CLIENT_AttachPTZStatusProc(LLONG lLoginID, NET_IN_PTZ_STATUS_PROC *pstuInPtzStatusProc,  NET_OUT_PTZ_STATUS_PROC *pstuOutPtzStatusProc, int nWaitTime = 3000);

// stop subscribe PTZ metadata port, lAttachHandle is CLIENT_AttachPTZStatusProc return value
CLIENT_NET_API BOOL    CALL_METHOD    CLIENT_DetachPTZStatusProc(LLONG lAttachHandle);

/************************************************************************/
/*                            PTZ visual domain subcribe
/************************************************************************/
// subscribe PTZ visual domain,user malloc memory of pstuInViewRange and pstuOutViewRange
CLIENT_NET_API LLONG CALL_METHOD CLIENT_AttachViewRangeState(LLONG lLoginID, NET_IN_VIEW_RANGE_STATE *pstuInViewRange, NET_OUT_VIEW_RANGE_STATE *pstuOutViewRange, int nWaitTime = 3000);

// stop subscribe PTZ visual domain, lAttachHandle is CLIENT_AttachViewRangeState return value
CLIENT_NET_API BOOL    CALL_METHOD    CLIENT_DetachViewRangeState(LLONG lAttachHandle);

// subscribe PTZ weather info,user malloc memory of pstuInParam and pstuOutParam
CLIENT_NET_API LLONG CALL_METHOD CLIENT_AttachWeatherInfo(LLONG lLoginID, NET_IN_WEATHER_INFO *pstuInParam, NET_OUT_WEATHER_INFO *pstuOutParam, int nWaitTime);

//stop subscribe PTZ weather info,lAttachHandle is CLIENT_AttachWeatherInfo return value
CLIENT_NET_API BOOL CALL_METHOD CLIENT_DetachWeatherInfo(LLONG lAttachHandle);

// configuration of set view range by object,user malloc memory of pInstuParam and pstuOutParam
CLIENT_NET_API BOOL CALL_METHOD CLIENT_SetViewRangeByObject(LLONG lLoginID, NET_IN_SET_VIEW_RANGE_BY_OBJECT *pInstuParam, NET_OUT_SET_VIEW_RANGE_BY_OBJECT *pstuOutParam,  int nWaitTime = NET_INTERFACE_DEFAULT_TIMEOUT);

// Get information list
CLIENT_NET_API BOOL CALL_METHOD CLIENT_ScenicSpotGetPointInfos(LLONG lLoginID, NET_IN_SCENICSPOT_GETPOINTINFOS_INFO *pInstuParam, NET_OUT_SCENICSPOT_GETPOINTINFOS_INFO *pstuOutParam,  int nWaitTime);

// set the scene, the original number will be covered
CLIENT_NET_API BOOL CALL_METHOD CLIENT_ScenicSpotSetPointInfo(LLONG lLoginID,  NET_IN_SCENICSPOT_SETPOINTINFO_INFO *pInstuParam,  NET_OUT_SCENICSPOT_SETPOINTINFO_INFO *pstuOutParam,  int nWaitTime);

// Get the ability to support the scene points
CLIENT_NET_API BOOL CALL_METHOD CLIENT_ScenicSpotGetCaps(LLONG lLoginID,  const NET_IN_SCENICSPOT_GETCAPS_INFO *pInstuParam,  NET_OUT_SCENICSPOT_GETCAPS_INFO *pstuOutParam,  int nWaitTime);

// Set Backup Sources
CLIENT_NET_API BOOL CALL_METHOD CLIENT_SetBackupSources(LLONG lLoginID, const NET_IN_SETBACKUP_SOURCES_INFO *pInstuParam, NET_OUT_SETBACKUP_SOURCES_INFO* pstuOutParam, int nWaitTime);
/************************************************************************/
/*                            BUS order                                   */
/************************************************************************/
// order Bus status,user malloc memory of pstuInBus and pstuOutBus
CLIENT_NET_API LLONG CALL_METHOD CLIENT_AttachBusState(LLONG lLoginID, NET_IN_BUS_ATTACH *pstuInBus, NET_OUT_BUS_ATTACH *pstuOutBus, int nWaitTime = 3000);

// stop order Bus status, lAttachHandle is CLIENT_AttachBusState return value
CLIENT_NET_API BOOL CALL_METHOD CLIENT_DetachBusState(LLONG lAttachHandle);

/************************************************************************/
/*                            analog alarm channel data subcribe
/************************************************************************/

// subscribe analog alarm channel data,user malloc memory of pInParam and pOutParam
CLIENT_NET_API LLONG CALL_METHOD CLIENT_AttachAnalogAlarmData(LLONG lLoginID, const NET_IN_ANALOGALARM_DATA* pInParam, NET_OUT_ANALOGALARM_DATA* pOutParam, int nWaitTime);

// stop subscribe analog alarm channel data
CLIENT_NET_API BOOL CALL_METHOD CLIENT_DetachAnalogAlarmData(LLONG lAttachHandle);

/************************************************************************/
/*               record  change port order              */
/************************************************************************/

// order record  change port,user malloc memory of pInParam and pOutParam
CLIENT_NET_API LLONG CALL_METHOD CLIENT_AttachRecordUpdater(LLONG lLoginID, const NET_IN_RECORDUPDATER_DATA* pInParam, NET_OUT_RECORDUPDATER_DATA* pOutParam, int nWaitTime);

// stoporder record  change port 
CLIENT_NET_API BOOL CALL_METHOD CLIENT_DetachRecordUpdater(LLONG lAttachHandle);

////////////////////////////Special Version Interface///////////////////////////////
// open log function,user malloc memory of pstLogPrintInfo 
CLIENT_NET_API BOOL CALL_METHOD CLIENT_LogOpen(LOG_SET_PRINT_INFO *pstLogPrintInfo);

// close log function
CLIENT_NET_API BOOL CALL_METHOD CLIENT_LogClose();

// Search device log--extensive,user malloc memory of pLogBuffer
CLIENT_NET_API BOOL CALL_METHOD CLIENT_QueryLogEx(LLONG lLoginID, DH_LOG_QUERY_TYPE logType, char *pLogBuffer, int maxlen, int *nLogBufferlen, void* reserved, int waittime=3000);

// Start query log(support BSC only),user malloc memory of pInParam and pOutParam
CLIENT_NET_API LLONG CALL_METHOD CLIENT_StartQueryLog(LLONG lLoginID, const NET_IN_START_QUERYLOG* pInParam, NET_OUT_START_QUERYLOG* pOutParam, int nWaitTime);

// Query next log(support BSC only),user malloc memory of pInParam and pOutParam
CLIENT_NET_API BOOL CALL_METHOD CLIENT_QueryNextLog(LLONG lLogID, NET_IN_QUERYNEXTLOG* pInParam, NET_OUT_QUERYNEXTLOG* pOutParam, int nWaitTime);

// Stop query log(support BSC only)
CLIENT_NET_API BOOL CALL_METHOD CLIENT_StopQueryLog(LLONG lLogID);

// Active registered redirect function,establish directed connections
CLIENT_NET_API LONG CALL_METHOD CLIENT_ControlConnectServer(LLONG lLoginID, char* RegServerIP, WORD RegServerPort, int TimeOut=3000);

// Establish active registered connection
CLIENT_NET_API BOOL CALL_METHOD CLIENT_ControlRegisterServer(LLONG lLoginID, LONG ConnectionID, int waittime=1000);

// Disconnected directional connection
CLIENT_NET_API BOOL CALL_METHOD CLIENT_ControlDisconnectRegServer(LLONG lLoginID, LONG ConnectionID);

// Query active registered server information
CLIENT_NET_API BOOL CALL_METHOD CLIENT_QueryControlRegServerInfo(LLONG lLoginID, LPDEV_SERVER_AUTOREGISTER lpRegServerInfo, int waittime=2000);

// according to CLIENT_ControlRegisterOtherServer ,input parameter
typedef struct tagNET_IN_REGISTER_INFO 
{
	DWORD               dwSize;										 	 // struct size
    char                szRegServerIP[DH_MAX_IPADDR_LEN_EX];             // register ip address
    int                 nRegServerPort;                                  // register port
} NET_IN_REGISTER_INFO;

// according to CLIENT_ControlRegisterOtherServer output parameter
typedef struct tagNET_OUT_REGISTER_INFO 
{
    DWORD       dwSize;			// struct size
}NET_OUT_REGISTER_INFO;

// auto register to server,vehicle group talk requirement,control device register to other server
CLIENT_NET_API BOOL CALL_METHOD CLIENT_ControlRegisterOtherServer(LLONG lLoginID, NET_IN_REGISTER_INFO* pInParam, NET_OUT_REGISTER_INFO* pOutParam , int waittime);

// Upload file
CLIENT_NET_API LLONG CALL_METHOD CLIENT_FileTransmit(LLONG lLoginID, int nTransType, char* szInBuf, int nInBufLen, fTransFileCallBack cbTransFile, LDWORD dwUserData, int waittime);

// web info trasmit
CLIENT_NET_API BOOL  CALL_METHOD CLIENT_TransmitInfoForWeb(LLONG lLoginID, char* szInBuffer, DWORD dwInBufferSize, char* szOutBuffer, DWORD dwOutBufferSize, void* pExtData, int waittime=500);


// watermark verify for picture *nResult = 0-means no verify *nResult = 1-means has verify
CLIENT_NET_API BOOL  CALL_METHOD CLIENT_WatermarkVerifyForPicture(char* szFilePath, int* nResult, void* pReserved);

// multi realplay,user malloc memory of pInBuf and pOutBuf
CLIENT_NET_API BOOL  CALL_METHOD CLIENT_MultiRealPlay(LLONG lLoginID, DHDEV_IN_MULTIPLAY_PARAM* pInBuf, int nInBufLen, DHDEV_OUT_MULTIPLAY_PARAM* pOutBuf, int nOutBufLen, int* pRetLen);
 
// stop multi realplay
CLIENT_NET_API BOOL  CALL_METHOD CLIENT_StopMultiRealPlay(LLONG* lRealHandles, int nNumOfHandles);

// when hwnd != null,set playback yuv data callback 
CLIENT_NET_API BOOL CALL_METHOD CLIENT_SetPlaybackYUVCallBack(LLONG lPlayHandle, fYUVDataCallBack cYUVData, LDWORD dwUser);

// get web configuration
CLIENT_NET_API BOOL CALL_METHOD CLIENT_GetNewDevConfigForWeb(LLONG lLoginID, char* szCommand, int nChannelID, char* szOutBuffer, DWORD dwOutBufferSize, int *error, int waittime=500);

// set configuration for web
CLIENT_NET_API BOOL CALL_METHOD CLIENT_SetNewDevConfigForWeb(LLONG lLoginID, char* szCommand, int nChannelID, char* szInBuffer, DWORD dwInBufferSize, int *error, int *restart, int waittime=500);

// set play strategy parameter, only valid for real time
CLIENT_NET_API BOOL CALL_METHOD CLIENT_SetPlayMethod(LLONG lRealHandle, int nStartTime, int nSlowTime, int nFastTime, int nFailedTime);

// close device auto registration link
CLIENT_NET_API BOOL CALL_METHOD CLIENT_CloseRegConnect(LLONG lHandle, char *pIp, WORD wPort, void *pParam);

// by file playback input parameter-proxy use
typedef struct tagNET_IN_PLAYBACK_BYFILE_PROXY 
{
    DWORD                   dwSize;
    LPNET_RECORDFILE_INFO   lpRecordFile;                   // record file info
    HWND                    hWnd;                           // window handle
    fDownLoadPosCallBack    cbDownLoadPos;                  // playback progress call
    LDWORD                  dwPosUser;                      // progress call user data
    fDataCallBack           fDownLoadDataCallBack;          // playback data call
    LDWORD                  dwDataUser;                     // data call user data
    char                    szDstIP[DH_COMMON_STRING_128];  // device IP
}NET_IN_PLAYBACK_BYFILE_PROXY;

// by file playback output parameter - proxy use
typedef struct tagNET_OUT_PLAYBACK_BYFILE_PROXY 
{
    DWORD                   dwSize;
}NET_OUT_PLAYBACK_BYFILE_PROXY;

// by time playback input parameter-proxy use
typedef struct tagNET_IN_PLAYBACK_BYTIME_PROXY 
{
    DWORD                   dwSize;
    int                     nChannelID;                     // channel no.
    LPNET_TIME              lpStartTime;                    // start time
    LPNET_TIME              lpStopTime;                     // end time
    HWND                    hWnd;                           // window handlw
    fDownLoadPosCallBack    cbDownLoadPos;                  // playback progress call
    LDWORD                  dwPosUser;                      // progress call user data
    fDataCallBack           fDownLoadDataCallBack;          // playback data call
    LDWORD                  dwDataUser;                     // data call user data
    char                    szDstIP[DH_COMMON_STRING_128];  // device IP
}NET_IN_PLAYBACK_BYTIME_PROXY;

// by time playback output parameter-proxy use
typedef struct tagNET_OUT_PLAYBACK_BYTIME_PROXY 
{
    DWORD                   dwSize;
}NET_OUT_PLAYBACK_BYTIME_PROXY;

// by file playback - attached device IP, proxy use only,user malloc memory of pInParam and pOutParam
CLIENT_NET_API LLONG CALL_METHOD CLIENT_PlayBackByRecordFileProxy(LLONG lLoginID, NET_IN_PLAYBACK_BYFILE_PROXY* pInParam, NET_OUT_PLAYBACK_BYFILE_PROXY* pOutParam);

// by time playback - attached device IP, proxy use only,user malloc memory of pInParam and pOutParam
CLIENT_NET_API LLONG CALL_METHOD CLIENT_PlayBackByTimeProxy(LLONG lLoginID, NET_IN_PLAYBACK_BYTIME_PROXY* pInParam, NET_OUT_PLAYBACK_BYTIME_PROXY* pOutParam);

// Import configuration file (JSON) pSendBuf: To send data, the user allocates memory, nSendBufLen: The length of the sent, reserved: retention parameter
CLIENT_NET_API BOOL CALL_METHOD CLIENT_ImportConfigFileJson(LLONG lLoginID, char *pSendBuf, int nSendBufLen, void* reserved=NULL, int nWaitTime=3000);
// Export the configuration file (JSON) pOutBuffer: Receive buffer, The user allocates memory, maxlen: Receive buffer length, nRetlen: The actual length derived, reserved: retention parameter
CLIENT_NET_API BOOL CALL_METHOD CLIENT_ExportConfigFileJson(LLONG lLoginID, char *pOutBuffer, int maxlen, int *nRetlen, void* reserved=NULL, int nWaitTime=3000);

CLIENT_NET_API BOOL CALL_METHOD CLIENT_SetSecurityKey(LLONG lPlayHandle, char* szKey, DWORD nKeyLen);

// RPC capacity search
CLIENT_NET_API BOOL CALL_METHOD CLIENT_QueryRpcMethod(LLONG lLoginID, const char* pszMethod, BOOL* pResult, int nWaitTime);

//////////////////////////////////////////////////////////////////////////
// Public transport business relevant interface
//////////////////////////////////////////////////////////////////////////

// check bus route,user malloc memory of pstInParam and pstOutParam
CLIENT_NET_API BOOL CALL_METHOD CLIENT_CheckBusLine(LLONG lLoginID, const NET_IN_CHECK_BUS_LINE* pstInParam, NET_OUT_CHECK_BUS_LINE* pstOutParam, int nWaitTime = 5000);

// send bus route info,user malloc memory of pstInParam and pstOutParam
CLIENT_NET_API BOOL CALL_METHOD CLIENT_DispatchBusLineInfo(LLONG lLoginID, const NET_IN_DISPATCH_BUS_LINE_INFO* pstInParam, NET_OUT_DISPATCH_BUS_LINE_INFO* pstOutParam, int nWaitTime = 5000);

// vehicle operation dispatch ,user malloc memory of pstInParam and pstOutParam
CLIENT_NET_API BOOL CALL_METHOD CLIENT_BusSchedule(LLONG lLoginID, const NET_IN_BUS_SCHEDULE_INFO* pstInParam, NET_OUT_BUS_SCHEDULE_INFO* pstOutParam, int nWaitTime = 5000);

// Issued by the vehicle scheduling plan,user malloc memory of pstInParam and pstOutParam
CLIENT_NET_API BOOL CALL_METHOD CLIENT_DispatchWorkPlan(LLONG lLoginID, const NET_IN_BUS_DISPATCH_WORK_PLAN* pstInParam, NET_OUT_BUS_DISPATCH_WORK_PLAN* pstOutParam, int nWaitTime);

// Weather forecast info
typedef struct tagNET_WEATHERFORCAST_INFO
{
	int								nHighTemperature;					// High temperaturethe day, unitDegree Celsius
	int								nLowTemperature;					// Low temperaturethe day, unit: Degree Celsius
	int								nCurrentTemperature;				// Current temperature, unit: Degree Celsius
	char							szWeatherState[64];					// Weather State
	int								nWeatherCode;						// Weather code 
	BYTE							byReserved[1024];					// reserved
}NET_WEATHERFORCAST_INFO;
// push weather forecast in Param
typedef struct tagNET_IN_PUSH_WEATHERFORCAST
{
	DWORD							dwSize;
	NET_WEATHERFORCAST_INFO			stuWeatherInfo;						// Weather forecast Infomation
}NET_IN_PUSH_WEATHERFORCAST;
// push weather forecast Out Param
typedef struct tagNET_OUT_PUSH_WEATHERFORCAST
{
	DWORD							dwSize;
}NET_OUT_PUSH_WEATHERFORCAST;

// Push weather forecast
CLIENT_NET_API BOOL CALL_METHOD CLIENT_PushWeatherForecast(LLONG lLoginID, const NET_IN_PUSH_WEATHERFORCAST* pstInParam, NET_OUT_PUSH_WEATHERFORCAST* pstOutParam, int nWaitTime);
//////////////////////////////////////////////////////////////////////////

// add feedback type after task
typedef enum tagNET_MISSION_FEEDBACK_TYPE
{
    NET_MISSION_FEEDBACK_NO = 0,                        // no feedback needed
    NET_MISSION_FEEDBACK_YES,                           // need feedback, shall replay DH_ALARM_MISSION_CONFIRM task confirmed event
}NET_MISSION_FEEDBACK_TYPE;

// message type (mobile)
typedef enum tagNET_MISSION_MESSAGE_TYPE
{
    NET_MISSION_MESSAGE_TEXT = 0 ,      //text info 
    NET_MISSION_MESSAGE_AD ,            //ad info 
    NET_MISSION_MESSAGE_BROAD ,         //broadcast info
    NET_MISSION_MESSAGE_DRIVER ,        //driver info
    NET_MISSION_MESSAGE_DISPATCH ,      //dispatch command
    NET_MISSION_MESSAGE_TRANSPARENT ,   //transparent data   
    NET_MISSION_MESSAGE_URGENT,         //emergency
    NET_MISSION_MESSAGE_MAX ,
}NET_MISSION_MESSAGE_TYPE;

// CLIENT_AddMission()port input parameter
typedef struct tagNET_IN_ADD_MISSION 
{
    DWORD           dwSize;
    char            szTitle[DH_COMMON_STRING_256];      // task name
    char            szMissionID[DH_COMMON_STRING_256];  // task ID
    char            szComment[DH_COMMON_STRING_256];    // replay content
    NET_MISSION_FEEDBACK_TYPE   emFeedBack;             // feedback type after adding
    NET_MISSION_MESSAGE_TYPE    emMessageType;          // message type 
	BOOL            bComment;                           // FALSE, szComment is walid, TRUE, szCommentEx is valid
	char            szCommentEx[DH_COMMON_STRING_2048]; // replay content, no null
	int				nChannel[DH_MAX_CHANNEL_COUNT];		// In which several channels are superimposed, Supports multi-channel simultaneous stacking
	int				nDuration;							// Overlay display duration, Unit:second; 0 indicates that it has been stacked until the next time
	int				nCount;								// nChannel length
}NET_IN_ADD_MISSION;

// CLIENT_AddMission() port output parameter
typedef struct tagNET_OUT_ADD_MISSION 
{
    DWORD           dwSize;
}NET_OUT_ADD_MISSION;
// user malloc memory of pstInParam and pstOutParam
CLIENT_NET_API BOOL CALL_METHOD CLIENT_AddMission(LLONG lLoginID, const NET_IN_ADD_MISSION* pstInParam, NET_OUT_ADD_MISSION* pstOutParam, int nWaitTime = 3000);

// CLIENT_ParkingControlStartFind port input parameter
typedef struct tagNET_IN_PARKING_CONTROL_START_FIND_PARAM
{
    DWORD                     dwSize;          // structure size
    BOOL                      bSearchCount;    // Search if record call is valid
    DWORD                     dwSearchCount;   // Search record item, value range is 1~100
    BOOL                      bBegin;          // Search if start time is valid
    NET_TIME                  stuBegin;        // Search start time
    BOOL                      bEnd;            // Search if end time is valid
    NET_TIME                  stuEnd;          // Search end time
    BOOL                      bCardType;       // If card type is valid
    NET_ECK_IC_CARD_USER_TYPE emCardType;      // Card type
    BOOL                      bFlag;           // If car pass mark is valid
    NET_ECK_CAR_PASS_FLAG     emFlag;          // Car pass mark
} NET_IN_PARKING_CONTROL_START_FIND_PARAM;

// CLIENT_ParkingControlStartFind port output parameter
typedef struct tagNET_OUT_PARKING_CONTROL_START_FIND_PARAM
{
    DWORD                     dwSize;          // structure size
    DWORD                     dwTotalCount;    // Result number match the search criteria
} NET_OUT_PARKING_CONTROL_START_FIND_PARAM;

// CLIENT_ParkingControlDoFind port input parameter
typedef struct tagNET_IN_PARKING_CONTROL_DO_FIND_PARAM
{
    DWORD                     dwSize;          // structure size
    DWORD                     dwFileCount;     // Currently record item number to search
} NET_IN_PARKING_CONTROL_DO_FIND_PARAM;

// Car pass record info
typedef struct tagNET_CAR_PASS_ITEM
{
    DWORD                      dwSize; 
    NET_TIME                   stuTime;          // Car pass time
    DWORD                      dwCardNo;         // Card no.
    NET_ECK_IC_CARD_USER_TYPE  emCardType;       // Card type
    NET_ECK_CAR_PASS_FLAG      emFlag;           // Car pass record type
} NET_CAR_PASS_ITEM;

// CLIENT_ParkingControlDoFind port output parameter
typedef struct tagNET_OUT_PARKING_CONTROL_DO_FIND_PARAM
{
    DWORD                     dwSize;          // structure size
    NET_CAR_PASS_ITEM        *pstuRecordList;  // Record list, user allocate memory,memory size is sizeof(NET_CAR_PASS_ITEM)*nMaxRecordNum
    int                       nMaxRecordNum;   // List record number
    int                       nRetRecordNum;   // Searched record items, current searched item number is lower than designated item number, search end
} NET_OUT_PARKING_CONTROL_DO_FIND_PARAM;

// Start car pass record search,user malloc memory of pInParam and pOutParam
CLIENT_NET_API LLONG CALL_METHOD CLIENT_ParkingControlStartFind(LLONG lLoginID, NET_IN_PARKING_CONTROL_START_FIND_PARAM* pInParam, NET_OUT_PARKING_CONTROL_START_FIND_PARAM* pOutParam, int waittime);

// Get car pass record,user malloc memory of pInParam and pOutParam
CLIENT_NET_API BOOL CALL_METHOD CLIENT_ParkingControlDoFind(LLONG lFindeHandle, NET_IN_PARKING_CONTROL_DO_FIND_PARAM* pInParam, NET_OUT_PARKING_CONTROL_DO_FIND_PARAM* pOutParam, int waittime);

// End car pass record search
CLIENT_NET_API BOOL CALL_METHOD CLIENT_ParkingControlStopFind(LLONG lFindHandle);

// Order car pass record data call function origin
typedef void (CALLBACK *fParkingControlRecordCallBack)(LLONG lLoginID, LLONG lAttachHandle, NET_CAR_PASS_ITEM *pInfo, int nBufLen, LDWORD dwUser);

// CLIENT_ParkingControlAttachRecord() port input parameter
typedef struct tagNET_IN_PARKING_CONTROL_PARAM
{
    DWORD                           dwSize;
    fParkingControlRecordCallBack   cbCallBack;                 // Data call function
    LDWORD                          dwUser;                     // User custom parameter
} NET_IN_PARKING_CONTROL_PARAM;

// CLIENT_ParkingControlDetachRecord() port output parameter
typedef struct tagNET_OUT_PARKING_CONTROL_PARAM
{
    DWORD    dwSize;
} NET_OUT_PARKING_CONTROL_PARAM;

// Car pass record order,user malloc memory of pInParam and pOutParam
CLIENT_NET_API LLONG CALL_METHOD CLIENT_ParkingControlAttachRecord(LLONG lLoginID, const NET_IN_PARKING_CONTROL_PARAM* pInParam, NET_OUT_PARKING_CONTROL_PARAM* pOutParam, int nWaitTime);

// Cancel car pass record order
CLIENT_NET_API BOOL CALL_METHOD CLIENT_ParkingControlDetachRecord(LLONG lAttachHandle);

// Vehicle detector status
typedef enum tagNET_CAR_DETECTOR_STATE
{
    NET_CAR_DETECTOR_STATE_UNKNOWN = -1,
    NET_CAR_DETECTOR_STATE_OFFLINE = 0,                   // Offline
    NET_CAR_DETECTOR_STATE_ONLINE,                        // Online
    NET_CAR_DETECTOR_STATE_LOOPOFFLINE,                   // Senser coil offline
} NET_CAR_DETECTOR_STATE;

// Sub device general status
typedef enum tagNET_AUXILIARY_DEV_STATE
{
    NET_AUXILIARY_DEV_STATE_UNKNOWN = -1,
    NET_AUXILIARY_DEV_STATE_OFFLINE = 0,                   // offline
    NET_AUXILIARY_DEV_STATE_ONLINE,                        // online
} NET_AUXILIARY_DEV_STATE;

// Device IC card quantity condition
typedef enum tagNET_IC_CARD_STATUS
{
    NET_IC_CARD_STATUS_UNKNOWN = -1,
    NET_IC_CARD_STATUS_NORMAL = 0,                         // Normal
    NET_IC_CARD_STATUS_NOCARD,                             // No card
    NET_IC_CARD_STATUS_LOWCARD,                            // Few
    NET_IC_CARD_STATUS_FULLCARDS,                          // Full
} NET_IC_CARD_STATUS;

// auxiliary work mode type
typedef enum tagNET_AUXILIARY_WORK_MODE
{
    NET_AUXILIARY_WORK_MODE_UNKNOWN = 0,
    NET_AUXILIARY_WORK_MODE_OFFLINE,                       // working offline
    NET_AUXILIARY_WORK_MODE_ONLINE,                        // working online
} NET_AUXILIARY_WORK_MODE;

// Alarm event type DH_ALARM_AUXILIARY_DEV_STATE(sub device status event)corresponding data description info
typedef struct tagALARM_AUXILIARY_DEV_STATE
{
    DWORD                   dwSize;
    NET_CAR_DETECTOR_STATE  emDetector;                   // Vehicle detector status
    NET_AUXILIARY_DEV_STATE emLed;                        // LED status lattice screen status
    NET_AUXILIARY_DEV_STATE emICDetectBoard;              // Panel swiping card board
    NET_AUXILIARY_DEV_STATE emICSenderBoard;              // Issuing card swiping cardboard
    NET_IC_CARD_STATUS      emICCardStatus;               // Device IC card quantity condition
    NET_AUXILIARY_WORK_MODE emWorkMode;                   // working mode
} ALARM_AUXILIARY_DEV_STATE;

// Parking issuing card swiping card type
typedef enum tagNET_PARKING_CARD_TYPE
{
    NET_PARKING_CARD_TYPE_UNKNOWN,
    NET_PARKING_CARD_TYPE_SEND,                             // Issue card
    NET_PARKING_CARD_TYPE_DETECT,                           // Swipe card
} NET_PARKING_CARD_TYPE;

// Alarm event type DH_ALARM_PARKING_CARD(parking swipe card event)corresponding data description info
typedef struct tagALARM_PARKING_CARD
{
    DWORD                   dwSize;
    NET_PARKING_CARD_TYPE   emType;                       // Type
    DWORD                   dwCardNo;                     // Card no.
    char                    szPlate[DH_COMMON_STRING_16]; // Plate
} ALARM_PARKING_CARD;

// The type of alarm source device
typedef enum tagEM_ALARM_TRANSMIT_DEV_SRC_TYPE
{
    EM_ALARM_TRANSMIT_DEV_SRC_TYPE_UNKNOWN,             // Unknown    
	EM_ALARM_TRANSMIT_DEV_SRC_TYPE_DIGIT,               // Digit   
    EM_ALARM_TRANSMIT_DEV_SRC_TYPE_ANALOG,              // Analog 
    EM_ALARM_TRANSMIT_DEV_SRC_TYPE_SMARTLOCK,           // SmartLock
}EM_ALARM_TRANSMIT_DEV_SRC_TYPE;

// alarm transmit alarm type
typedef enum tagEM_ALARM_TRANSMIT_ALARM_TYPE
{
    EM_ALARM_TRANSMIT_ALARM_TYPE_UNKNOWN,               // unknown
    EM_ALARM_TRANSMIT_ALARM_TYPE_PREVENTREMOVE,         // PreventRemove
    EM_ALARM_TRANSMIT_ALARM_TYPE_LOWERPOWER,            // LowerPower
    EM_ALARM_TRANSMIT_ALARM_TYPE_KEY,                   // Key
    EM_ALARM_TRANSMIT_ALARM_TYPE_DURESS,                // Duress
}EM_ALARM_TRANSMIT_ALARM_TYPE;

// Alarm transmission event (Corresponding structure ALARM_PROFILE_ALARM_TRANSMIT_INFO)
typedef struct tagALARM_PROFILE_ALARM_TRANSMIT_INFO
{
    DWORD                           dwSize;
    int					            nChannelID;								// channel number
    int					            nAction;								// 0:Start 1:Stop
    NET_TIME                        stuTime;                                // time-to-event
    NET_SENSE_METHOD                emSenseMethod;                          // Sensor Type
    char                            szUserID[DH_COMMON_STRING_128];         // Alarm users, to distinguish    
	EM_ALARM_TRANSMIT_DEV_SRC_TYPE  emDevSrcType;                           // The type of alarm source device 
    EM_ALARM_TRANSMIT_ALARM_TYPE    emAlarmType;                            // alarm transmit alarm type
}ALARM_PROFILE_ALARM_TRANSMIT_INFO;

// order monitor spot info,user malloc memory of pInParam and pOutParam
CLIENT_NET_API LLONG CALL_METHOD CLIENT_SCADAAttachInfo(LLONG lLoginID, const NET_IN_SCADA_ATTACH_INFO* pInParam, NET_OUT_SCADA_ATTACH_INFO* pOutParam, int nWaitTime);

// cancel monitor spot info order
CLIENT_NET_API BOOL CALL_METHOD CLIENT_SCADADetachInfo(LLONG lAttachHandle);

// scada get threshold,user malloc memory of pInParam and pOutParam
CLIENT_NET_API BOOL CALL_METHOD CLIENT_SCADAGetThreshold(LLONG lLoginID, const NET_IN_SCADA_GET_THRESHOLD* pInParam, NET_OUT_SCADA_GET_THRESHOLD* pOutParam, int nWaitTime);

// scada set threshold,user malloc memory of pInParam and pOutParam
CLIENT_NET_API BOOL CALL_METHOD CLIENT_SCADASetThreshold(LLONG lLoginID, const NET_IN_SCADA_SET_THRESHOLD* pInParam, NET_OUT_SCADA_SET_THRESHOLD* pOutParam, int nWaitTime);

// scada start find history info,user malloc memory of pInParam and pOutParam
CLIENT_NET_API LLONG CALL_METHOD CLIENT_StartFindSCADA(LLONG lLoginID, const NET_IN_SCADA_START_FIND* pInParam, NET_OUT_SCADA_START_FIND* pOutParam, int nWaitTime);

// scada do find history info,user malloc memory of pInParam and pOutParam
CLIENT_NET_API BOOL CALL_METHOD CLIENT_DoFindSCADA(LLONG lFindHandle, const NET_IN_SCADA_DO_FIND* pInParam, NET_OUT_SCADA_DO_FIND* pOutParam, int nWaitTime);

// scada stop find history info
CLIENT_NET_API BOOL CALL_METHOD CLIENT_StopFindSCADA(LLONG lFindHandle);

// scada attach alarm event,user malloc memory of pInParam and pOutParam
CLIENT_NET_API LLONG CALL_METHOD CLIENT_SCADAAlarmAttachInfo(LLONG lLoginID, const NET_IN_SCADA_ALARM_ATTACH_INFO* pInParam, NET_OUT_SCADA_ALARM_ATTACH_INFO* pOutParam, int nWaitTime = 3000);

// scada detach alarm event
CLIENT_NET_API BOOL CALL_METHOD CLIENT_SCADAAlarmDetachInfo(LLONG lAttachHandle);

// scada set point info,user malloc memory of pInParam and pOutParam
CLIENT_NET_API BOOL CALL_METHOD CLIENT_SCADASetInfo(LLONG lLoginID, const NET_IN_SCADA_POINT_SET_INFO_LIST* pInParam, NET_OUT_SCADA_POINT_SET_INFO_LIST* pOutParam, int nWaitTime = 3000);

//////////////////////////////////////////////////////////////////////////

// Order wireless code info port,user malloc memory of pstInBuf and pstOutBuf
CLIENT_NET_API LLONG CALL_METHOD CLIENT_AttachLowRateWPAN(LLONG lLoginID, const NET_IN_ATTACH_LOWRATEWPAN* pstInParam, NET_OUT_ATTACH_LOWRATEWPAN* pstOutParam, int nWaitTime = 3000);

// Cancel order wireless info port, lAttachHandle is CLIENT_AttachLowRateWPAN return value
CLIENT_NET_API BOOL CALL_METHOD CLIENT_DetachLowRateWPAN(LLONG lAttachHandle);

//Get play library port used to decode and display in preview, close resource type of gotten port,it will affect normal operaton
CLIENT_NET_API LONG CALL_METHOD CLIENT_GetRealPlayPort(LLONG lRealPlayHandle);

// Get play library port used to decode and display in preview, close resource type of gotten port,it will affect normal operaton
CLIENT_NET_API LONG CALL_METHOD  CLIENT_GetPlayBackPort(LLONG lPlayBackHandle);

// Parking Spaces of the intelligent parking systems 
typedef enum tagNET_ECK_PARK_STATE
{
    NET_ECK_PARK_STATE_UNKOWN,
    NET_ECK_PARK_STATE_PARK,                     // Parking cars
    NET_ECK_PARK_STATE_NOPARK,                   // No car parking Spaces
} NET_ECK_PARK_STATE;

// Parking information
typedef struct tagNET_PARK_INFO_ITEM
{
    DWORD                      dwSize; 
    char                       szParkNo[DH_COMMON_STRING_32];   // Parking number
    NET_ECK_PARK_STATE         emState;                         // Parking state
    DWORD                      dwScreenIndex;                   // GuideScreen index according to parking number
    DWORD                      dwFreeParkNum;                   // Free count of parking space
} NET_PARK_INFO_ITEM;

// Subscribe to the parking information callback function prototypes
typedef void (CALLBACK *fParkInfoCallBack)(LLONG lLoginID, LLONG lAttachHandle, NET_PARK_INFO_ITEM *pInfo, int nBufLen, LDWORD dwUser);

// Parking space detector type
typedef enum tagNET_ECK_PARK_DETECTOR_TYPE
{
    NET_ECK_PARK_DETECTOR_TYPE_SONIC,                     // ultrasonic detector
    NET_ECK_PARK_DETECTOR_TYPE_CAMERA,                    // The camera detector
    NET_ECK_PARK_DETECTOR_TYPE_ALL,
} NET_ECK_PARK_DETECTOR_TYPE;

// Parking space detector information query condition

typedef struct tagNET_PARK_INFO_FILTER
{
    DWORD                      dwSize; 
    DWORD                      dwNum;                                  // Parking space detector type number
    NET_ECK_PARK_DETECTOR_TYPE emType[NET_ECK_PARK_DETECTOR_TYPE_ALL]; // Parking space detector type 
} NET_PARK_INFO_FILTER;

// CLIENT_ParkingControlAttachParkInfo()Interface input parameters
typedef struct tagNET_IN_PARK_INFO_PARAM
{
    DWORD                           dwSize;
    NET_PARK_INFO_FILTER            stuFilter;
    fParkInfoCallBack               cbCallBack;                 // Data callback function
    LDWORD                          dwUser;                     // User defined parameters 
} NET_IN_PARK_INFO_PARAM;

// CLIENT_ParkingControlAttachParkInfo()Interface output parameters
 typedef struct tagNET_OUT_PARK_INFO_PARAM
{
    DWORD    dwSize;
} NET_OUT_PARK_INFO_PARAM;

// Parking state subscription,user malloc memory of pInParam and pOutParam
CLIENT_NET_API LLONG CALL_METHOD CLIENT_ParkingControlAttachParkInfo(LLONG lLoginID, const NET_IN_PARK_INFO_PARAM* pInParam, NET_OUT_PARK_INFO_PARAM* pOutParam, int nWaitTime);

// Cancel the parking state subscription
CLIENT_NET_API BOOL CALL_METHOD CLIENT_ParkingControlDetachParkInfo(LLONG lAttachHandle);

// Subscribe to the video phone status information,user malloc memory of pInBuf and pOutBuf
CLIENT_NET_API LLONG CALL_METHOD CLIENT_AttachVTPCallState(LLONG lLoginID, const NET_IN_VTP_CALL_STATE_ATTACH* pInParam, NET_OUT_VTP_CALL_STATE_ATTACH* pOutParam, int nWaitTime);

// Unsubscribe video phone status information
CLIENT_NET_API BOOL CALL_METHOD CLIENT_DetachVTPCallState(LLONG lCallStateHandle);

// set mobile push notification, use RegisterID to identify the config info,user malloc and free pstuCfg's memory 
CLIENT_NET_API BOOL CALL_METHOD CLIENT_SetMobileSubscribe(LLONG lLoginID, const NET_MOBILE_PUSH_NOTIFY *pstuCfg, int *nError, int *nRestart, int nWaitTime = 1000);

// delete mobile push notification, use RegisterID to identify the config info,user malloc and free memory of pstuIn and pstuOut
CLIENT_NET_API BOOL CALL_METHOD CLIENT_DelMobileSubscribe(LLONG lLoginID, const NET_MOBILE_PUSH_NOTIFY_DEL *pstuIn, NET_OUT_DELETECFG* pstuOut, int nWaitTime = 1000);

// set mobile push notification config, use RegisterID+AppID to identify the info, AppID cannot include ".", use "_" instead of ".",user malloc and free memory of pstuCfg
CLIENT_NET_API BOOL CALL_METHOD CLIENT_SetMobileSubscribeCfg(LLONG lLoginID, const NET_MOBILE_PUSH_NOTIFY_CFG *pstuCfg, int *nError, int *nRestart, int nWaitTime = 1000);

// delete mobile push notification config, use RegisterID+AppID to identify the info, AppID cannot include ".", use "_" instead of ".",user malloc and free memory of pstuIn and pstuOut
CLIENT_NET_API BOOL CALL_METHOD CLIENT_DelMobileSubscribeCfg(LLONG lLoginID, const NET_MOBILE_PUSH_NOTIFY_CFG_DEL *pstuIn, NET_OUT_DELETECFG* pstuOut, int nWaitTime = 1000);

////////////////////////////////////////////////////////////////////////////////
// Net applications, operation type
typedef enum tagEM_RPC_NETAPP_TYPE
{
    EM_RPC_NETAPP_TYPE_GET_NET_RESOURCE_STAT,      //  get statistics of net resource, pstuIn: NET_IN_NETAPP_GET_NET_RESOURCE_STAT*, pstuOut: NET_OUT_NETAPP_GET_NET_RESOURCE_STAT*
}EM_RPC_NETAPP_TYPE;

// EM_RPC_NETAPP_TYPE_GET_NET_RESOURCE_STAT, input param
typedef struct tagNET_IN_NETAPP_GET_NET_RESOURCE_STAT
{
    DWORD               dwSize;
}NET_IN_NETAPP_GET_NET_RESOURCE_STAT;

// EM_RPC_NETAPP_TYPE_GET_NET_RESOURCE_STAT, output param
typedef struct tagNET_OUT_NETAPP_GET_NET_RESOURCE_STAT
{
    DWORD               dwSize;
    int                 nIPChanneIn;             // IP channel in bandwidth, unit: kbps
    int                 nNetRemain;              // Net remain bandwidth, unit: kbps
    int                 nNetCapability;          // Net total bandwidth, unit: Mbps
    int                 nRemotePreview;          // Remote preview bandwidth, unit: kbps
    int                 nRemotePlayAndDownload;  // Remote playback and download bandwidth, unit: kbps
    int                 nRemoteSendRemain;       // Remote send remain bandwidth, unit: kbps
    int                 nRemoteSendCapability;   // Remote send total bandwidth, unit: Mbps
}NET_OUT_NETAPP_GET_NET_RESOURCE_STAT;

// network application component, company internal customized port 
// used to acquire network source data of front-end device,user malloc and free memory of pstuIn and pstuOut,please refer to corresponding structure of emType
CLIENT_NET_API BOOL CALL_METHOD CLIENT_RPC_NetApp(LLONG lLoginID, EM_RPC_NETAPP_TYPE emType, const void* pstuIn, void* pstuOut, int nWaitTime = 1000);

// Set optimize mode,user malloc and free pParam's memory,please refer to corresponding structure of emType
CLIENT_NET_API BOOL CALL_METHOD CLIENT_SetOptimizeMode(EM_OPTIMIZE_TYPE emType, void *pParam);

// panoramic PTZ operation type 
typedef enum tagEM_NET_WIDE_VIEW_CTRL
{
    EM_NET_WIDE_VIEW_CTRL_START,                      // begin to generate panoramic view, startGenerateWideView, structure NET_IN_WIDE_VIEW_GENERATE_START and NET_OUT_WIDE_VIEW_GENERATE_START
    EM_NET_WIDE_VIEW_CTRL_STOP,                       // stop generating panoramic view, stopGenerateWideView, structure NET_IN_WIDE_VIEW_GENERATE_STOP and NET_OUT_WIDE_VIEW_GENERATE_STOP
    EM_NET_WIDE_VIEW_CTRL_PROGRESS,                   // acquire current progress generated by panoramic view, getWideViewProgress, structure NET_IN_WIDE_VIEW_PROGRESS and NET_OUT_WIDE_VIEW_PROGRESS
    EM_NET_WIDE_VIEW_CTRL_IMAGEINFO,                  // acquire panoramic view info, getImageInfo, structure NET_IN_WIDE_VIEW_IMAGE and NET_OUT_WIDE_VIEW_IMAGE
    EM_NET_WIDE_VIEW_CTRL_WV_COORD,                   // switch from PTZ coordinate to panoramic view coordinate, getWideViewCoordinates, structure NET_IN_WIDE_VIEW_WV and NET_OUT_WIDE_VIEW_WV
    EM_NET_WIDE_VIEW_CTRL_PTZ_COORD,                  // switch from panoramic view to PTZ absolute coordinate, getPtzCoordinates, structure NET_IN_WIDE_VIEW_PTZ and NET_OUT_WIDE_VIEW_PTZ 
} EM_NET_WIDE_VIEW_CTRL;

// begin to generate panoramic view input parameter 
typedef struct tagNET_IN_WIDE_VIEW_GENERATE_START
{
    DWORD                    dwSize;
    int                      nChannel;                // channel number 
} NET_IN_WIDE_VIEW_GENERATE_START;

// begin to generate panoramic view output parameter 
typedef struct tagNET_OUT_WIDE_VIEW_GENERATE_START
{
    DWORD                    dwSize;
} NET_OUT_WIDE_VIEW_GENERATE_START;

//  stop generating panoramic view input parameter 
typedef struct tagNET_IN_WIDE_VIEW_GENERATE_STOP
{
    DWORD                    dwSize;
    int                      nChannel;                // channel number 
} NET_IN_WIDE_VIEW_GENERATE_STOP;

//  stop generating panoramic view output parameter 
typedef struct tagNET_OUT_WIDE_VIEW_GENERATE_STOP
{
    DWORD                    dwSize;
} NET_OUT_WIDE_VIEW_GENERATE_STOP;

//  acquire current progress input parameter generated by panoramic view
typedef struct tagNET_IN_WIDE_VIEW_PROGRESS
{
    DWORD                    dwSize;
    int                      nChannel;                // channel number 
} NET_IN_WIDE_VIEW_PROGRESS;

//  acquire current progress output parameter generated by panoramic view 
typedef struct tagNET_OUT_WIDE_VIEW_PROGRESS
{
    DWORD                    dwSize;
    int                      nProgress;               // current progress generated by panoramic view, range 0-100 
} NET_OUT_WIDE_VIEW_PROGRESS;

//  acquire panoramic view info input parameter 
typedef struct tagNET_IN_WIDE_VIEW_IMAGE
{
    DWORD                    dwSize;
    int                      nChannel;                // channel number 
} NET_IN_WIDE_VIEW_IMAGE;

//  panoramic view info 
typedef struct tagNET_WIDE_VIEW_IMAGE_INFO
{
    char                     szImageUrl[MAX_PATH];    // channel number 
} NET_WIDE_VIEW_IMAGE_INFO;

// acquire panoramic view info output parameter
typedef struct tagNET_OUT_WIDE_VIEW_IMAGE
{
    DWORD                    dwSize;
    NET_WIDE_VIEW_IMAGE_INFO stuImageInfo;             // panoramic view info 
} NET_OUT_WIDE_VIEW_IMAGE;

#define NET_WIDE_VIEW_COORDINATES_MAX (64)

//  switch from PTZ coordinate to panoramic view coordinate input parameter 
typedef struct tagNET_IN_WIDE_VIEW_WV
{
    DWORD                    dwSize;
    int                      nChannel;                // channel number 
    int                      nNum;                    // effective PTZ coordinate number 
    PTZ_SPEED_UNIT           stuPTZ[NET_WIDE_VIEW_COORDINATES_MAX];   // PTZ direction and zoom rate 
} NET_IN_WIDE_VIEW_WV;

//  switch from PTZ coordinate to panoramic view coordinate output parameter 
typedef struct tagNET_OUT_WIDE_VIEW_WV
{
    DWORD                    dwSize;
    NET_RECT                 stuRect[NET_WIDE_VIEW_COORDINATES_MAX];  // panoramic view coordinate 
} NET_OUT_WIDE_VIEW_WV;

//  switch from panoramic view to PTZ absolute coordinate input parameter 
typedef struct tagNET_IN_WIDE_VIEW_PTZ
{
    DWORD                    dwSize;
    int                      nChannel;                // channel number 
    int                      nNum;                    // effective panoramic coordinate number 
    NET_RECT                 stuRect[NET_WIDE_VIEW_COORDINATES_MAX];  // panoramic view coordinate 
} NET_IN_WIDE_VIEW_PTZ;

//  switch from panoramic view to PTZ absolute coordinate output parameter 
typedef struct tagNET_OUT_WIDE_VIEW_PTZ
{
    DWORD                    dwSize;
    PTZ_SPEED_UNIT           stuPTZ[NET_WIDE_VIEW_COORDINATES_MAX];   // PTZ direction and zoom rate 
} NET_OUT_WIDE_VIEW_PTZ;

// panoramic PTZ relevant operation port, operation type refers to EM_NET_WIDE_VIEW_CTRL,user malloc memory of pstuInParam and pstuOutParam,please refer to corresponding structure of emCtrlType
CLIENT_NET_API BOOL CALL_METHOD CLIENT_WideViewCtrl(LLONG lLoginID, EM_NET_WIDE_VIEW_CTRL emCtrlType, const void *pstuInParam, void *pstuOutParam, int nWaitTime = 1000);
// ,user malloc memory of pInParam and pOutParam
CLIENT_NET_API BOOL CALL_METHOD CLIENT_SetMarkFile(LLONG lLoginID,const NET_IN_SET_MARK_FILE* pInParam, NET_OUT_SET_MARK_FILE* pOutParam, int nWaitTime = NET_INTERFACE_DEFAULT_TIMEOUT );
CLIENT_NET_API BOOL CALL_METHOD CLIENT_SetMarkFileByTime(LLONG lLoginID, const NET_IN_SET_MARK_FILE_BY_TIME* pInParam, NET_OUT_SET_MARK_FILE_BY_TIME* pOutParam, int nWaitTime = 1000);

//record according to time stamp &&C5

//acquire sign video info ,user malloc memory of pInParam and pOutParam
CLIENT_NET_API BOOL CALL_METHOD CLIENT_GetMarkInfo(LLONG lLoginID, const NET_IN_GET_MARK_INFO* pInParam, NET_OUT_GET_MARK_INFO* pOutParam, int nWaitTime = 1000);

// record stream type 
typedef enum tagEM_NET_AUDIO_REC_STREAM
{
    EM_NET_AUDIO_REC_STREAM_UNKNOWN,
    EM_NET_AUDIO_REC_STREAM_MAIN,                    // Main
    EM_NET_AUDIO_REC_STREAM_EXTRA1,                  // Extra1
    EM_NET_AUDIO_REC_STREAM_EXTRA2,                  // Extra2
    EM_NET_AUDIO_REC_STREAM_EXTRA3,                  // Extra3
    EM_NET_AUDIO_REC_STREAM_TALKBACK,                // Talkback
} EM_NET_AUDIO_REC_STREAM;

// enable/disable audio record and acquire record name input parameter, corresponding command DH_CTRL_AUDIO_REC_START_NAME and DH_CTRL_AUDIO_REC_STOP_NAME 
typedef struct tagNET_IN_AUDIO_REC_MNG_NAME
{
    DWORD                    dwSize;                 // structure size 
    int                      nChannel;               // audio channel number 
    EM_NET_AUDIO_REC_STREAM  emStream;               // code stream type 
} NET_IN_AUDIO_REC_MNG_NAME;

// enable /disable audio record and acquire record name output parameter, corresponding command DH_CTRL_AUDIO_REC_START_NAME and DH_CTRL_AUDIO_REC_STOP_NAME 
typedef struct tagNET_OUT_AUDIO_REC_MNG_NAME
{
    DWORD                    dwSize;                 // structure size 
    char                     szFileName[MAX_PATH];   // recording file name 
} NET_OUT_AUDIO_REC_MNG_NAME;

// enable record type 
typedef enum tagEM_NET_REC_EVENT
{
    EM_NET_REC_EVENT_UNKNOWN,                       // unknown 
    EM_NET_REC_EVENT_ALARM,                         // Alarm
} EM_NET_REC_EVENT;

// enable/disable designated channel video input parameter, able to designate record type 
typedef struct tagNET_IN_REC_MNG_CTRL_MPT300
{
    DWORD                    dwSize;                 // the structure size
    int                      nChannel;               // channel number 
    EM_NET_REC_EVENT         emType;                 // record type, it is valid when nAction is 0 
    int                      nAction;                // 0:enable, 1:disable 
} NET_IN_REC_MNG_CTRL_MPT300;

// enable/disable designated channel record output parameter 
typedef struct tagNET_OUT_REC_MNG_CTRL_MPT300
{
    DWORD                    dwSize;                 // the structure size 
} NET_OUT_REC_MNG_CTRL_MPT300;

// realtime snapshot (manual snapshot) input parameter, corresponding command DH_CTRL_SNAP_MNG_SNAP_SHOT 
typedef struct tagNET_IN_SNAP_MNG_SHOT
{
    DWORD                    dwSize;                 // the structure size 
    int                      nChannel;               // channel number 
    int                      nTime;                  // continuous snapshot times, 0 means stopping snapshot, positive number means the number of continuous snapshot 
} NET_IN_SNAP_MNG_SHOT;

// realtime snapshot (manual snapshot) output parameter, corresponding command DH_CTRL_SNAP_MNG_SNAP_SHOT 
typedef struct tagNET_OUT_SNAP_MNG_SHOT
{
    DWORD                    dwSize;                 // the structure size
} NET_OUT_SNAP_MNG_SHOT;

// and close database/resume database input parameter, corresponding command DH_CTRL_LOG_STOP and DH_CTRL_LOG_RESUME
typedef struct tagNET_IN_LOG_MNG_CTRL
{
    DWORD                    dwSize;                 // the structure size 
} NET_IN_LOG_MNG_CTRL;

// and close database/resume database output parameter, corresponding command DH_CTRL_LOG_STOP and DH_CTRL_LOG_RESUME
typedef struct tagNET_OUT_LOG_MNG_CTRL
{
    DWORD                    dwSize;                 // the structure size 
} NET_OUT_LOG_MNG_CTRL;

// direct unvarnished transmission input parameter 
typedef struct tagNET_IN_TRANSMIT_DIRECTLY
{
    DWORD                    dwSize;                  // the structure size 
    char                    *szInBuffer;              // data transmitted to device 
    DWORD                    dwInBufferSize;          // data size transmitted to device
    NET_TRANSMIT_EXT_INFO    stExtData;               // data expansion transmitted to device, binary data 
} NET_IN_TRANSMIT_DIRECTLY;

// direct unvarnished output parameter 
typedef struct tagNET_OUT_TRANSMIT_DIRECTLY
{
    DWORD                    dwSize;                  // the structure size 
    char                    *szOutBuffer;             // device returned data 
    DWORD                    dwOutBufferSize;         // device returned data buffering size 
    DWORD                    dwRetBufferSize;         // actual returned data size 
} NET_OUT_TRANSMIT_DIRECTLY;

//remove anti-submarine alarm ,user malloc memory of pInParam and pOutParam
CLIENT_NET_API BOOL CALL_METHOD CLIENT_ClearRepeatEnter(LLONG lLoginID, const NET_IN_CLEAR_REPEAT_ENTER* pInParam, NET_OUT_CLEAR_REPEAT_ENTER* pOutParam, int nWaitTime = 1000);

//remove anti-submarine alarm ,user malloc memory of pInParam and pOutParam
CLIENT_NET_API BOOL CALL_METHOD CLIENT_GetOperatorName(LLONG lLoginID, const NET_IN_GET_OPERATOR_NAME* pInParam, NET_OUT_GET_OPERATOR_NAME* pOutParam, int nWaitTime = 1000);

// POS connection type
typedef enum tagEM_CONN_TYPE
{
    EM_CONN_TYPE_UNKNOWN,                            // unknown
    EM_CONN_TYPE_NET,                                // tcp/ip
    EM_CONN_TYPE_RS232,                              // RS232
    EM_CONN_TYPE_RS485,                              // RS485
} EM_CONN_TYPE;

// POS protocol type
typedef enum tagEM_CONN_PROT
{
    EM_CONN_PROT_UNKNOWN,                            // unknown
    EM_CONN_PROT_NONE,                               // custom
    EM_CONN_PROT_POS,                                // POS
} EM_CONN_PROT;

// POS status type
typedef enum tagEM_POS_STATUS
{
    EM_POS_STATUS_OTHER = -1,                        // other error
    EM_POS_STATUS_NO,                                // no error
    EM_POS_STATUS_PROT_FORMAT,                       // protocol format error
    EM_POS_STATUS_NET_ADDR_CONFLICT,                 // net address conflict
    EM_POS_STATUS_RS232_ADDR_CONFLICT,               // rs232 address conflict
    EM_POS_STATUS_RS485_ADDR_CONFLICT,               // rs485 address conflict
    EM_POS_STATUS_LINK_CHANNEL_CONFLICT,             // linked channel conflict
    EM_POS_STATUS_NOT_EXIST,                         // POS does not exist
    EM_POS_STATUS_NUM_LIMIT,                         // POS number is out of limit
    EM_POS_STATUS_NAME_CONFLICT,                     // POS name conflict
} EM_POS_STATUS;

// POS tcp/ip connection attribute
typedef struct tagNET_POS_NET_ATT
{
    char                     szSrcIp[DH_MAX_IPADDR_LEN_EX];              // POS ip addr
    int                      nSrcPort;                                   // POS port
    char                     szDstIp[DH_MAX_IPADDR_LEN_EX];              // NVR ip addr
    int                      nDstPort;                                   // NVR port
} NET_POS_NET_ATT;

// POS COM connection attribute, RS232, RS485
typedef struct tagNET_POS_COM_ATT
{
    int                      nCommChannel;                               // comm channel (sub index of CFG_CMD_COMM)
    int                      nAddress;                                   // RS485 address
    DH_COMM_PROP             stuAttr;                                    // RS485 attr
} NET_POS_COM_ATT;

// POS custom protocol
typedef struct tagNET_POS_CUSTOM_PROT
{
    char                     szStartStr[DH_COMMON_STRING_32];            // start string
    BOOL                     bAnyCharater;                               // whether use szStartStr as prefix, TRUE: use any string as prefix, FALSE: use szStartStr as prefix
    char                     szEndStr[DH_COMMON_STRING_32];              // end string
    char                     szLineDelimiter[DH_COMMON_STRING_32];       // line delimiter
    int                      nMoreLine;                                  // count of POS trade info lines
    char                     szIgnoreStr[DH_COMMON_STRING_32];           // string to ignore
    BOOL                     bCaseSensitive;                             // whether case sensitive
    BYTE                     reserved[1024];                             // reserved
} NET_POS_CUSTOM_PROT;

// Ptz link type
typedef enum tagEM_PTZ_LINK_TYPE
{
	EM_PTZ_LINK_TYPE_NONE,							// none
	EM_PTZ_LINK_TYPE_PRESET,						// "Preset" 
	EM_PTZ_LINK_TYPE_TOUR,							// "Tour" 
	EM_PTZ_LINK_TYPE_PATTERN,						// "Pattern" 
	EM_PTZ_LINK_TYPE_ZOOM,							// "Zoom" 
	EM_PTZ_LINK_TYPE_SINGLESCENE,					// "SingleScene" 
	EM_PTZ_LINK_TYPE_QUICKFOCUS,					// "QuickFocus" 
} EM_PTZ_LINK_TYPE;

// Linkage PTZ information
typedef struct tagNET_CFG_PTZ_LINK
{
	EM_PTZ_LINK_TYPE		emType;						// Linkage type
	int						nValue;						// The linkage value corresponds to the preset number,cruise number,and so on.
} NET_CFG_PTZ_LINK;

// pos ptz link
typedef struct tagNET_PTZ_LINK
{
	EM_PTZ_LINK_TYPE		emType;					// ptz link type
	int						nParam1;				// link param 1
	int						nParam2;				// link param 2
	int						nParam3;				// link param 3
	int						nChannelID;				// ptz link channelID
} NET_PTZ_LINK;


// monitor tour config
typedef struct tagNET_MONITOR_TOUR
{
	BOOL					bEnable;				// monitor tour enable
	int						nChannels[256];			// monitor tour channels 
	int						nChannelCount;			// monitor tour channel number
	BYTE					reserved[32];			// reserved
} NET_MONITOR_TOUR;


#define MAX_NUMBER_PTZLINK		256	// max number ptzlink
#define	MAX_NUMBER_TOUR			32	// max number of monitor Tour 

// POS event link
typedef struct tagNET_POS_EVENT_LINK
{
	DWORD					dwSize;									// struct size
	DH_TSECT				stuTimeSection[DH_TSCHE_DAY_NUM][DH_TSCHE_SEC_NUM];		// time schedule
	BOOL					bRecordEnable;							// RecordEnable
	BOOL					bRecordCloudEnable;						// RecordCloudEnable
	DWORD					dwRecordMask[DH_MAX_CHANNUM];			// record channels
	int						nRecordLatch;							// RecordLatch unit:second
	BOOL					bAlarmOutEnable;						// AlarmOutEnable
	DWORD					dwAlarmOutMask[DH_MAX_CHANNUM];			// Alarm Out channels
	int						nAlarmOutLatch;							// AlarmOutLatch unit:second
	int						nPtzLinkNum;							// Ptz Link Number
	NET_PTZ_LINK			stuPtzLink[MAX_NUMBER_PTZLINK];			// pos ptz link
	BOOL					bPtzLinkEnable;							// Ptz Link Enable
	int						nPtzLinkDelay;							// Ptz Link Delay unit:second
	BOOL					bSnapshotEnable;						// SnapshotEnable
	BOOL					bSnapshotCloudEnable;					// SnapshotCloudEnable
	DWORD					dwSnapshotMask[DH_MAX_CHANNUM];			// Snapshot channels
	int						nSnapshotPeriod;						// SnapshotPeriod
	int						nSnapshotTimes;							// SnapshotTimes
	BOOL					bTipEnable;								// TipEnable
	BOOL					bMailEnable;							// MailEnable
	BOOL					bMessageEnable;							// MessageEnable
	BOOL					bBeepEnable;							// BeepEnable
	BOOL					bVoiceEnable;							// VoiceEnable
	DWORD					dwVoiceMask[DH_MAX_CHANNUM];			// Voice channels
	char					szAudioFileName[MAX_PATH];				// AudioFileName
	int						nPlayTimes;								// PlayTimes
	
	int						nDejitter;								// Dejitter unit:second 
	BOOL					bLogEnable;								// LogEnable
	int						nMonitorTourNum;                        // MonitorTourNumber
	NET_MONITOR_TOUR		stuMonitorTour[MAX_NUMBER_TOUR];		// MonitorTour config
} NET_POS_EVENT_LINK;

// POS info
typedef struct tagNET_POS_INFO
{
    BOOL                     bEnable;                // enable
    DWORD                    dwPosId;                // POS ID, invalid wheth adding POS
    char                     szName[64];             // POS name
    EM_CONN_TYPE             emConnType;             // connection type
    NET_POS_NET_ATT          stuNetAtt;              // net connection attr, valid if emConnType is EM_CONN_TYPE_NET
    NET_POS_COM_ATT          stuComAtt;              // comm attr, valid if emConnType is EM_CONN_TYPE_RS232 or EM_CONN_TYPE_RS485
    EM_CONN_PROT             emConnProt;             // connection protocol type
    NET_POS_CUSTOM_PROT      stuCustom;              // valid if emConnProt is EM_CONN_PROT_NONE
    int                      nTimeOut;               // time out for recv msg, unit: sec
    int                      nLinkChannel[32];       // linked channel
    int                      nLinkChannelNum;        // valid count in nLinkChannel
    int                      nPlayBackTime;          // POS msg assocated video playback time, unit: sec

	BOOL					 bPreviewBlend;			 // preview blend
    BYTE                     reserved[1020];         // reserved
} NET_POS_INFO;

// The input parameters of adding POS Device parameter, corresponding to DH_CTRL_POS_ADD/DH_CTRL_POS_MODIFY
typedef struct tagNET_IN_POS_ADD
{
    DWORD                    dwSize;                 // struct size
    NET_POS_INFO             stuPosInfo;             // POS info
	NET_POS_EVENT_LINK		 stuPosEventLink;		 // POS EventLink 
} NET_IN_POS_ADD;

// The output parameters of adding POS Device parameter, corresponding to DH_CTRL_POS_ADD/DH_CTRL_POS_MODIFY
typedef struct tagNET_OUT_POS_ADD
{
    DWORD                    dwSize;                 // struct size
    DWORD                    dwPosId;                // POS ID
    EM_POS_STATUS            emStatus;               // Status
} NET_OUT_POS_ADD;

// The input parameters of removing POS Device parameter, corresponding to DH_CTRL_POS_REMOVE
typedef struct tagNET_IN_POS_REMOVE
{
    DWORD                    dwSize;                 // struct size
    DWORD                    dwPosId;                // POS ID
} NET_IN_POS_REMOVE;

// The input parameters of moving POS Device parameter, corresponding to DH_CTRL_POS_REMOVE
typedef struct tagNET_OUT_POS_REMOVE
{
    DWORD                    dwSize;                 // struct size
    EM_POS_STATUS            emStatus;               // Status
} NET_OUT_POS_REMOVE;

// The input parameters of multi-removing POS Device parameter, corresponding to DH_CTRL_POS_REMOVE_MULTI
typedef struct tagNET_IN_POS_REMOVE_MULTI
{
    DWORD                    dwSize;                 // struct size
    DWORD                    dwPosId[16];            // Pos ID, -1 means all
    int                      nPosNum;                // number of POS ID
} NET_IN_POS_REMOVE_MULTI;

// The output parameters of multi-removing POS Device parameter, corresponding to DH_CTRL_POS_REMOVE_MULTI
typedef struct tagNET_OUT_POS_REMOVE_MULTI
{
    DWORD                    dwSize;                 // struct size
    EM_POS_STATUS            emStatus;               // Status
    DWORD                    dwPosIdFailed[16];      // Failed POS ID
    int                      nPosNum;                // number of Failed POS ID
} NET_OUT_POS_REMOVE_MULTI;

// The input parameters of Getting all POS Devices parameter, corresponding to DH_DEVSTATE_GET_ALL_POS
typedef struct tagNET_POS_ALL_INFO
{
    DWORD                    dwSize;                 // struct size
	int						 nMaxNum;				 // POS max num
    int                      nRetPosNum;             // pos return number
    NET_POS_INFO             stuPos[256];            // pos information
	NET_POS_EVENT_LINK		 *pstuPosEventLink;		 // POS EventLink, it matches with stuPos, user malloc the memory,apply to sizeof(NET_POS_EVENT_LINK)*nMaxNum
} NET_POS_ALL_INFO;

// info of event type DH_ALARM_LOCK_BREAK (POS Manage Event)
typedef struct tagALARM_POS_MANAGE_INFO
{
    int                      nEventAction;                   // Event Action, 0:Pulse, 1:Start, 2:Stop
    int                      nType;                          // Type, 0:add, 1:remove, 2:modify
    DWORD                    dwPosId;                        // Pos ID
    char                     szName[DH_COMMON_STRING_64];    // Pos Name
    int                      nLinkChannel[32];               // Link Channels
    int                      nLinkChannelNum;                // number of link channel
    EM_CONN_PROT             emConnProt;                     // Connect protocol
    BYTE                     reserved[1024];                 // reserved
} ALARM_POS_MANAGE_INFO;

// info of event type DH_ALARM_LOCK_BREAK (lock break event)
typedef struct tagALARM_LOCK_BREAK_INFO
{
    int                      nDoor;                          // door channel, start from 0
    int                      nAction;                        // 0:start, 1:stop
    NET_TIME_EX              UTC;                            // event time
    BYTE                     reserved[1024];                 // revered
}ALARM_LOCK_BREAK_INFO;

// Pos data type
typedef enum tagEM_POS_DATA_TYPE
{
	EM_POS_DATA_TYPE_UNKNOWN,				// unknown
	EM_POS_DATA_TYPE_STORE_INF0,			// store info
	EM_POS_DATA_TYPE_PRODUCT_INFO,			// product info
}EM_POS_DATA_TYPE;

// Store info
typedef struct tagNET_STORE_INFO
{
	char					 szDealNum[MAX_DEAL_NUM_LEN];				// Store deal num,used to identify which ticket items belong to the product
	char					 szNo[MAX_STORE_NO_LEN];					// Store name No
	char					 szName[MAX_STORE_NAME_LEN];				// Store name
	char					 szEmPloyeeID[MAX_STORE_EMPLOYEE_ID_LEN];	// Store employee ID 
	unsigned int			 nSeviceType;								// Store sevice type,0-market,1-return goods
	unsigned int			 nAmount;									// Store amount, Unit:cent
	BYTE					 byReserved[1024];							// reserved
}NET_STORE_INFO;

// Product info
typedef struct tagNET_PRODUCT_INFO
{
	char					 szDealNum[MAX_DEAL_NUM_LEN];			// product deal num,used to identify which ticket items belong to the product
	char					 szNo[MAX_PRODUCT_NO_LEN];				// product name No
	char					 szName[MAX_PRODUCT_NAME_LEN];			// product name
	char					 szCategory[MAX_PRODUCT_CATEGORY_LEN];	// product category
	unsigned int			 nPrice;								// product price,Unit:point
	double					 dbQuantity;							// product quantity,piece or kg
	BYTE					 byReserved[1024];						// reserved
}NET_PRODUCT_INFO;


// POS trade info
typedef struct tagNET_POS_TRADE_INFO
{
    DWORD                    dwPosId;                        // POS ID
    NET_TIME                 stuTime;                        // trading time
    BYTE                    *pbyComment;                     // trading content
    DWORD                    dwCommentLen;                   // pbyComment length
	EM_POS_DATA_TYPE		 emDataType;					 // data type
	BOOL					 bEnd;							 // End flag,judge whether a small ticket is over.End value, true means a small ticket ends.
	void*					 pPosData;						 // pos data
															 // if emDataType is EM_POS_DATA_TYPE_STORE_INF0,transfer store info,corresponding to NET_STORE_INFO
															 // if emDataType is EM_POS_DATA_TYPE_PRODUCT_INFO,transfer product info,corresponding to NET_PRODUCT_INFO
	BYTE                     reserved[1012];				 // reserved
} NET_POS_TRADE_INFO;

// POS trade info callback prototype
typedef void (CALLBACK *fPosTradeCallBack)(LLONG lLoginID, LLONG lAttachHandle, NET_POS_TRADE_INFO *pInfo, int nBufLen, LDWORD dwUser);

// input param of attaching POS trade info
typedef struct tagNET_IN_POS_TRADE_ATTACH 
{
    DWORD                       dwSize;                 // struct size
    fPosTradeCallBack           cbCallState;            // trade info callback
    LDWORD                      dwUser;                 // callback user's param
} NET_IN_POS_TRADE_ATTACH;

// output param of attaching POS trade info
typedef struct tagNET_OUT_POS_TRADE_ATTACH 
{
    DWORD                       dwSize;                 // struct size
} NET_OUT_POS_TRADE_ATTACH;

// input param of CLIENT_GetDevCaps according to NET_POS_CAPS 
typedef struct tagNET_IN_POS_GETCAPS
{
    DWORD               dwSize;
} NET_IN_POS_GETCAPS;

// output param of CLIENT_GetDevCaps according to NET_POS_CAPS 
typedef struct tagNET_OUT_POS_GETCAPS
{
    DWORD               dwSize;
    EM_CONN_TYPE        emConnType[10];                    // connection type
    int                 nConnTypeNum;                      // connection type count in emConnType
    EM_CONN_PROT        emConnProt[10];                    // connect protocol type
    int                 nConnProtNum;                      // connect protocol count in emConnProt
    int                 nMaxPos;                           // max POS count supported
	BOOL				bSupportPosRecord;				   // support pos record

} NET_OUT_POS_GETCAPS;

// attach POS trade info,user malloc memory of pInParam and pOutParam
CLIENT_NET_API LLONG CALL_METHOD CLIENT_AttachPosTrade(LLONG lLoginID, const NET_IN_POS_TRADE_ATTACH* pInParam, NET_OUT_POS_TRADE_ATTACH* pOutParam, int nWaitTime);

// detach POS trade info
CLIENT_NET_API BOOL CALL_METHOD CLIENT_DetachPosTrade(LLONG lAttachHandle);

// CLIENT_GetDevCaps interface NET_USER_MNG_CAPS command parameter
typedef struct tagNET_IN_USER_MNG_GETCAPS
{
    DWORD               dwSize;
} NET_IN_USER_MNG_GETCAPS;

// CLIENT_GetDevCaps interface NET_USER_MNG_CAPS command parameter
typedef struct tagNET_OUT_USER_MNG_GETCAPS
{
    DWORD               dwSize;
    BOOL                bAccountLimitation;							// Account Limitation
    BOOL                bIndividualAccessFilter;					// Individual Access Filter
    DWORD               dwMaxPageSize;								// Max number of Page Size
	unsigned int		nMaxPwdLen;									// max passwd len
	unsigned int		nMinPwdLen;									// min passwd len	
	char				szType[MAX_PWD_BASIC_CHARS_ARRAY_LEN];		// basic chars supportedNumber,Lower case,Capital. such as :"Number,Lower,Upper"
	char				szCharList[MAX_PWD_SPEC_CHARS_ARRAY_LEN];	// special chars supported, as"~!@#$%^" 
	int					nCombine;									// chars combine style0-no special request, 1-special char must be included, 2-at least two style included (number, capital, lower case, special char)
} NET_OUT_USER_MNG_GETCAPS;

//CLIENT_GetDevCaps interface NET_EXTERNALSENSOR_CAPS command parameter
typedef struct tagNET_IN_EXTERNALSENSOR_CAPS
{
	DWORD				dwSize;
}NET_IN_EXTERNALSENSOR_CAPS;

// CLIENT_GetDevCaps interface NET_GET_MASTERSLAVEGROUP_CAPS command parameter
typedef struct tagNET_IN_GET_MASTERSLAVEGROUP_CAPS
{
	DWORD				dwSize;
}NET_IN_GET_MASTERSLAVEGROUP_CAPS;

typedef enum tagEM_MASTERSLAVEGROUP_MODE
{
	EM_MASTERSLAVEGROUP_MODE_UNKNOWN,		//  Unknown
	EM_MASTERSLAVEGROUP_MODE_COMMANDER,	    //  Commander
	EM_MASTERSLAVEGROUP_MODE_PROPOSER,      //  Proposer 
	EM_MASTERSLAVEGROUP_MODE_JUDGE,			//  Judge
	EM_MASTERSLAVEGROUP_MODE_HAWKEYE,		//  HawkEye
	EM_MASTERSLAVEGROUP_MODE_MULTISENSOR,	//  MultiSensor
	EM_MASTERSLAVEGROUP_MODE_GLOBALCAMERA,	//  GlobalCamera
	EM_MASTERSLAVEGROUP_MODE_NEWCOMMANDER,  //  NewCommander
	EM_MASTERSLAVEGROUP_MODE_NEWHAWEYE,     //  NewHawEye

}EM_MASTERSLAVEGROUP_MODE;

// CLIENT_GetDevCaps interface NET_GET_MASTERSLAVEGROUP_CAPS command parameter
typedef struct tagNET_OUT_GET_MASTERSLAVEGROUP_CAPS
{
	DWORD									dwSize;
	EM_MASTERSLAVEGROUP_MODE				emRole;
}NET_OUT_GET_MASTERSLAVEGROUP_CAPS;

// CLIENT_GetDevCaps interface NET_STORAGE_CAPS command parameter
typedef struct tagNET_IN_STORAGE_CAPS
{
	DWORD									dwSize;
}NET_IN_STORAGE_CAPS;

// Whether need to reboot after formatting
typedef enum tagNET_EM_FORMAT_NEEDREBOOT
{
	EM_FORMAT_NEEDREBOOT_UNKNOWN,					// unknown
	EM_FORMAT_NEEDREBOOT_NOREBOOT,					// doesn't need reboot
	EM_FORMAT_NEEDREBOOT_REBOOT						// Need reboot, call CLIENT_ControlDevice formatting using the command DH_CTRL_SDCARD or DH_CTRL_DISK Automatic restart
													// using the command DH_CTRL_FORMAT_PATITION requires client to restart 
}NET_EM_FORMAT_NEEDREBOOT;

// CLIENT_GetDevCaps interface NET_STORAGE_CAPS command parameter
typedef struct tagNET_OUT_STORAGE_CAPS
{
	DWORD									dwSize;
	NET_EM_FORMAT_NEEDREBOOT				emReboot;
}NET_OUT_STORAGE_CAPS;

// external sensor networking mode
typedef enum tagEM_SENSOR_NERWORKING_MODE
{
	EM_SENSOR_NERWORKING_MODE_UNKNOWN,			// unknow mode
	EM_SENSOR_NETWORKING_MODE_RS485,			// RS-485
	EM_SENSOR_NETWORKING_MODE_RFID,				// RFID
}EM_SENSOR_NERWORKING_MODE;
// CLIENT_GetDevCaps interface NET_EXTERNALSENSOR_CAPS command parameter
typedef struct tagNET_OUT_EXTERNALSENSOR_CAPS
{
	DWORD								dwSize;
	BOOL								bIsSupport;					// whether to support external sensor data acquisition
	EM_SENSOR_NERWORKING_MODE			emNetworkingMode;			// external sensor networking
	int									nChannel;					// the maximum number of channels supported
}NET_OUT_EXTERNALSENSOR_CAPS;

// Wireless control work mode
typedef enum tagEM_WIRELESS_CTRL_WORK_MODE
{
    EM_WIRELESS_CTRL_WORK_MODE_UNKNOWN,                     // unknown
    EM_WIRELESS_CTRL_WORK_MODE_NORMAL,                      // Normal
    EM_WIRELESS_CTRL_WORK_MODE_POLLING,                     // Polling
} EM_WIRELESS_CTRL_WORK_MODE;


// info of event type DH_ALARM_REMOTE_CTRL_STATUS ( Wireless control work Event)
typedef struct tagALARM_REMOTE_CTRL_STATUS
{
    NET_TIME_EX              UTC;                            // time of Event 
    DWORD                    dwID;                           // control ID
    char                     szName[32];                     // Name
    char                     szUser[32];                     // User
    EM_WIRELESS_CTRL_WORK_MODE emMode;                       // Work mode
    BYTE                     reserved[1024];                 // reserved
} ALARM_REMOTE_CTRL_STATUS;

typedef enum tagEM_PASSENGER_CARD_CHECK_TYPE
{
    EM_PASSENGER_CARD_CHECK_TYPE_UNKOWN = 0,                // Unkown
    EM_PASSENGER_CARD_CHECK_TYPE_SIGNIN,                    // Signin
    EM_PASSENGER_CARD_CHECK_TYPE_SIGNOUT,                   // Signout
    EM_PASSENGER_CARD_CHECK_TYPE_NORMAL,                    // Normal
}EM_PASSENGER_CARD_CHECK_TYPE;

// The description information of event type DH_ALARM_PASSENGER_CARD_CHECK corresponding data block
typedef struct tagALARM_PASSENGER_CARD_CHECK
{
    BOOL                     bEventConfirm;                  // Need confirm or not
    char                     szCardNum[DH_MAX_BUSCARD_NUM];  // Buscard number
    NET_GPS_STATUS_INFO      stuGPS;                         // GPS information
    NET_TIME_EX              UTC;                            // Time of card check
    int                      nTime;                          // UTC time. unit:second 
    EM_PASSENGER_CARD_CHECK_TYPE    emType;                  // Type of card check
    char                     szMac[DH_MAX_POS_MAC_NUM];      // POS Mac code(default "0000")
    BYTE                     reserved[1012];                 // reserved
} ALARM_PASSENGER_CARD_CHECK;

// The description information of event type DH_ALARM_BUS_TIRED_DRIVE_CHECK corresponding data block
typedef struct tagALARM_TIRED_DRIVE_CHECK_INFO
{
    BOOL                     bEventConfirm;                  // Need confirm or not
    int                      nAction;                        // 0:Start, 1:Stop
    int                      nDriveTime;                     // Drive time     unit:minutes
    NET_GPS_STATUS_INFO      stuGPS;                         // GPS information
    NET_TIME_EX              UTC;                            // Time of the event happens
    int                      nTime;                          // Time of the event happens  UTC time. unit:second
    BYTE                     reserved[1024];                 // reserved
} ALARM_TIRED_DRIVE_CHECK_INFO;


// info of event type DH_ALARM_SOUND ( Sound Event)
typedef struct tagALARM_SOUND
{
    int                      nChannel;                       // Channel
    NET_TIME_EX              UTC;                            // Time
    BYTE                     reserved[1024];                 // reserved
} ALARM_SOUND;

// The input parameters of Sound Alarm parameter, corresponding to DH_CTRL_SET_SOUND_ALARM
typedef struct tagNET_IN_SOUND_ALARM
{
    DWORD                    dwSize;                 // struct size
} NET_IN_SOUND_ALARM;

// The output parameters of Set FishEye parameter, corresponding to DH_CTRL_SET_SOUND_ALARM
typedef struct tagNET_OUT_SOUND_ALARM
{
    DWORD                    dwSize;                 // struct size
} NET_OUT_SOUND_ALARM;

// the output channesl which should to be silenced
typedef struct tagNET_SILENCE_CHANNEL
{
	int				nMatrix;			// the index of matrix
	int				nOutChannel;		// the counts of  output channels which should to be  silenced
	int				nOutPutChannel[DH_MAX_AUDIO_MATRIX_OUTPUT];	// the output channels which should to be  silenced
	BYTE			bReserved[512];	
}NET_SILENCE_CHANNEL;

//Audio matrix silence input parameter
typedef struct tagNET_IN_AUDIO_MATRIX_SILENCE
{
	DWORD					dwSize;				// the data is sizeof(NET_IN_AUDIO_MATRIX_SILENCE)
	BOOL					bEnable;			// enbale(true:close voice     false:open voice)
	int						nListCount;			// the counts of  output channels lists which should to be  silenced
	NET_SILENCE_CHANNEL*	pstSlienceChannel;	// the output channels lists which should to be silenced,memory size is sizeof(NET_SILENCE_CHANNEL)*nListCount
} NET_IN_AUDIO_MATRIX_SILENCE;





//Audio matrix silence output parameter
typedef struct tagNET_OUT_AUDIO_MATRIX_SILENCE
{
	DWORD					dwSize;		//the data is sizeof(NET_OUT_AUDIO_MATRIX_SILENCE)
} NET_OUT_AUDIO_MATRIX_SILENCE;

// target type
typedef enum tagEM_TARGET_TYPE
{
    EM_TARGET_TYPE_CLIENT,              // "client"
    EM_TARGET_TYPE_FTP_SERVER,          // "FTPServer"
}EM_TARGET_TYPE;

// fliter info
typedef struct tagNET_MANUAL_UPLOAD_FILTER_INFO
{
    EM_TARGET_TYPE           emTargetType;                      //target type, "Client" , "FTPServer"
    char                     szClientIP[DH_MAX_IPADDR_LEN];     //client IP address
    NET_TIME                 stuStartTime;                      //snap picture start time
    NET_TIME                 stuEndTime;                        //snap picture end time
}NET_MANUAL_UPLOAD_FILTER_INFO;

// event type DH_CTRL_MANUAL_UPLOAD_PICTURE, description of input parameter
typedef struct tagNET_IN_MANUAL_UPLOAD_PICTURE
{
    DWORD                                 dwSize;             // structure size
    NET_MANUAL_UPLOAD_FILTER_INFO         stuFilter;          // manual upload filter       
}NET_IN_MANUAL_UPLOAD_PICTURE;

// event type DH_CTRL_MANUAL_UPLOAD_PICTURE, description of output parameter
typedef struct tagNET_OUT_MANUAL_UPLOAD_PICTURE
{
    DWORD					dwSize;		                    // structure size
}NET_OUT_MANUAL_UPLOAD_PICTURE;

// description of reboot network decoding device input parameter, corresponding command DH_CTRL_REBOOT_NET_DECODING_DEV
typedef struct tagNET_IN_REBOOT_NET_DECODING_DEV
{
    DWORD               dwSize;                                 // structure size, need to be assigned sizeof(NET_IN_REBOOT_NET_DECODING_DEV)
    char				szDeviceID[DH_COMMON_STRING_64];	    // device ID
    int                 nRebootDelayTime;                       // reboot delay time, unit: second
}NET_IN_REBOOT_NET_DECODING_DEV;

//  description of input parameter,corresponding interface CLIENT_SetDefenceArmMode
typedef struct tagNET_IN_SET_DEFENCEMODE
{
    DWORD                   dwSize;
    char                    szPassword[DH_COMMON_STRING_64];           // login password
    int                     nChannel;                                  // channel id
    EM_DEFENCEMODE          emDefenceMode;                             // work status
}NET_IN_SET_DEFENCEMODE;

// description of output parameter,corresponding interface CLIENT_SetDefenceArmMode
typedef struct tagNET_OUT_SET_DEFENCEMODE 
{
    DWORD                   dwSize;
}NET_OUT_SET_DEFENCEMODE;

// description of input parameter,corresponding interface CLIENT_SetSubSystemArmMode
typedef struct tagNET_IN_SET_SUBSYSTEMMODE 
{
    DWORD                   dwSize;
    int                     nChannel;                                  // subsystem id,start 0
    char                    szPassword[DH_COMMON_STRING_64];           // login password
    EM_DEFENCEMODE          emDefenceMode;                             // work status
}NET_IN_SET_SUBSYSTEMMODE;

// description of output parameter,corresponding interface CLIENT_SetSubSystemArmMode
typedef struct tagNET_OUT_SET_SUBSYSTEMMODE
{
    DWORD               dwSize;
    DWORD               dwSourceNum;                        // alarm source input arming failed subsystem number 
    int                 nSource[ARM_DISARM_ZONE_MAX];       // alarm source input failed subsystem no.,subsystem no. start from 0
    DWORD               dwLinkNum;                          // link alarm arming failed subsystem number 
    int                 nLink[ARM_DISARM_ZONE_MAX];         // link alarm failed subsystem no., subsystem no. start from 0
}NET_OUT_SET_SUBSYSTEMMODE;

// description of input parameter,corresponding interface CLIENT_GetDefenceArmMode
typedef struct tagNET_IN_GET_DEFENCEMODE
{
    DWORD                    dwSize;
    int                      nDefenceNum;                            // Zone valid number
    int                      anDefence[DH_MAX_ALARMIN];              // Zone number to search
}NET_IN_GET_DEFENCEMODE;

// description of output parameter,corresponding interface CLIENT_GetDefenceArmMode
typedef struct tagNET_OUT_GET_DEFENCEMODE
{
    DWORD                    dwSize;
    int                      nDefenceNum;                            // Zone valid number
    EM_DEFENCEMODE           anDefenceState[DH_MAX_ALARMIN];         // Zone status info
}NET_OUT_GET_DEFENCEMODE;

// description of input parameter,corresponding interface CLIENT_GetSubSystemArmMode
typedef struct tagNET_IN_GET_SUBSYSTEMMODE
{
    DWORD                    dwSize;
}NET_IN_GET_SUBSYSTEMMODE;

// description of output parameter,corresponding interface  CLIENT_GetSubSystemArmMode
typedef struct tagNET_OUT_GET_SUBSYSTEMMODE
{
    DWORD                    dwSize;
    int                      nSubSystemNum;                            //subsystem valid number
    EM_SUBSYSTEMMODE         anSubSystemState[DH_MAX_ALARM_SUBSYSTEM_NUM];         // subsystem state info
}NET_OUT_GET_SUBSYSTEMMODE;

// description of Input parameter, corresponding interface CLIENT_ManualCheckPSTN
typedef struct tagNET_IN_PSTN_MANUALCHECK_STATE
{
    DWORD                    dwSize;
}NET_IN_PSTN_MANUALCHECK_STATE;

// description of Output parameter, corresponding interface CLIENT_ManualCheckPSTN
typedef struct tagNET_OUT_PSTN_MANUALCHECK_STATE
{
    DWORD                    dwSize;
    NET_THREE_STATUS_BOOL    emState;            //result 
}NET_OUT_PSTN_MANUALCHECK_STATE;

// description of reboot network decoding device ouput parameter, corresponding command DH_CTRL_REBOOT_NET_DECODING_DEV
typedef struct tagNET_OUT_REBOOT_NET_DECODING_DEV
{
    DWORD					dwSize;		                    // structure size, need to be assigned sizeof(NET_OUT_REBOOT_NET_DECODING_DEV)
}NET_OUT_REBOOT_NET_DECODING_DEV;

// event type DH_ALARM_DRIVE_AFTER_WORK (not work time event) data description
// ParkingControl set IC Sender input parameter, corresponding command DH_CTRL_SET_IC_SENDER
typedef struct tagNET_IN_SET_IC_SENDER
{
	DWORD					dwSize;				// the data is sizeof(NET_IN_SET_IC_SENDER)
	BOOL					bEnable;			// enbale(true: send IC card is allowed, false: forbidden)
}NET_IN_SET_IC_SENDER;
// ParkingControl set IC Sender input parameter, corresponding command DH_CTRL_SET_IC_SENDER
typedef struct tagNET_OUT_SET_IC_SENDER
{
	DWORD					dwSize;				// the data is sizeof(NET_OUT_SET_IC_SENDER)
}NET_OUT_SET_IC_SENDER;
typedef struct tagALARM_DVRIVE_AFTER_WORK
{
    BOOL                     bEventConfirm;                  // need confirm or not
    NET_GPS_STATUS_INFO      stuGPS;                         // GPS
    NET_TIME_EX              stuUtc;                         // check time
    DWORD                    dwUtc;                          // check time, value equal to stuUtc, format different, used to confirm
    BYTE                     reserved[1024];                 // reserved
} ALARM_DVRIVE_AFTER_WORK;

// event type DH_ALARM_UPLOAD_PIC_FAILED (break rules data upload failed) data description
typedef struct tagALARM_UPLOAD_PIC_FAILED_INFO
{
    int                      nAction;                        // 0:upload failed, 1:upload success after failed
    BYTE                     reserved[1024];                 // reserved
} ALARM_UPLOAD_PIC_FAILED_INFO;

// get city and road code info, CLIENT_QueryDevState, DH_DEVSTATE_GET_ROAD_LIST
typedef struct tagNET_ROAD_LIST_INFO
{
    DWORD                    dwSize;                 // structure size
    char                     szCity[32];             // city
    char                     szRoadList[2048][256];  // road code
    int                      nRoadNum;               // number of road code, first dimension of szRoadList
} NET_ROAD_LIST_INFO;

// wireless device type
typedef enum tagEM_WIRELESS_DEV_TYPE
{
    EM_WIRELESS_DEV_TYPE_UNKNOWN,                            // unknown
    EM_WIRELESS_DEV_TYPE_STATION,                            // "Station"
    EM_WIRELESS_DEV_TYPE_AP,                                 // "AccessPoint"
} EM_WIRELESS_DEV_TYPE;

// wireless authentication
typedef enum tagEM_WIRELESS_AUTHENTICATION
{
    EM_WIRELESS_AUTHENTICATION_UNKNOWN,                             // UnKnown
    EM_WIRELESS_AUTHENTICATION_OPEN,                                // OPEN
    EM_WIRELESS_AUTHENTICATION_SHARED,                              // SHARED
    EM_WIRELESS_AUTHENTICATION_WPA,                                 // WPA
    EM_WIRELESS_AUTHENTICATION_WPAPSK,                              // WPA-PSK
    EM_WIRELESS_AUTHENTICATION_WPA2,                                // WPA2
    EM_WIRELESS_AUTHENTICATION_WPA2PSK,                             // WPA2-PSK
    EM_WIRELESS_AUTHENTICATION_WPANONE,                             // WPA-NONE
    EM_WIRELESS_AUTHENTICATION_WPAPSK_WPA2PSK,                      // WPA-PSK|WPA2-PSK
    EM_WIRELESS_AUTHENTICATION_WPA_WPA2,                            // WPA|WPA2
    EM_WIRELESS_AUTHENTICATION_WPA_WPAPSK,                          // WPA | WPA-PSK
    EM_WIRELESS_AUTHENTICATION_WPA2_WPA2PSK,                        // WPA2|WPA2-PSK
    EM_WIRELESS_AUTHENTICATION_WPA_WPAPSK_WPA2_WPA2PSK,             // WPA|WPA-PSK|WPA2|WPA2-PSK
} EM_WIRELESS_AUTHENTICATION;

// Wireless data encryption 
typedef enum tagEM_WIRELESS_DATA_ENCRYPT
{
    EM_WIRELESS_DATA_ENCRYPT_UNKNOWN,                            // UnKnown
    EM_WIRELESS_DATA_ENCRYPT_NONE,                               // NONE
    EM_WIRELESS_DATA_ENCRYPT_WEP,                                // WEP
    EM_WIRELESS_DATA_ENCRYPT_TKIP,                               // TKIP
    EM_WIRELESS_DATA_ENCRYPT_AES,                                // AES(CCMP)
    EM_WIRELESS_DATA_ENCRYPT_TKIP_AES,                           // TKIP+AES
} EM_WIRELESS_DATA_ENCRYPT;

// wifi device info
typedef struct tagNET_WIFI_DEV_INFO
{
    char                     szMac[DH_MACADDR_LEN];          // mac address of wifi device
    int                      nLinkQuality;                   // link quality, 0~100
    NET_TIME_EX              stuEnterTime;                   // first searched time
    NET_TIME_EX              stuLeaveTime;                   // leave time
    int                      nSearchedCount;                 // searched count
    char                     szSSID[24];                     // SSID
    NET_TIME_EX              UTC;                            // event time
    EM_WIRELESS_DEV_TYPE     emDevType;                      // WIFI dev type
    int                      nChannel;                       // Wifi channel
    EM_WIRELESS_AUTHENTICATION emAuth;                       // authentication
    EM_WIRELESS_DATA_ENCRYPT emEncrypt;                      // data encrypt
    char                     szAPMac[DH_MACADDR_LEN];        // AP Mac
    int                      nAPChannel;                     // AP channel
    char                     szAPSSID[24];                   // AP SSID
    EM_WIRELESS_DATA_ENCRYPT emAPEncrypt;                    // AP data encrypt
	int						 nRssiQuality;                   // Signal strength
	char					 szManufacturer[MAX_MANUFACTURER_LEN];			// Manufacturer of mac address
	char					 szMacHistorySSID[MAX_MACHISTORY_SSID_NUM][MAX_MACHISTORY_SSID_LEN];     // History SSID list
	unsigned int			 nRetMacHistorySSIDNum;			 // actual num of mac history SSID
    BYTE                     reserved[264];                  // reserved
} NET_WIFI_DEV_INFO;

// basic wifi info
typedef struct tagNET_WIFI_BASIC_INFO
{
	UINT nPeriodUTC;      //start time of this notification period, indicates the same period; if device number notified in one notification is over the max value,several notification is needed.this value indicates the same notification period.
	int  nDeviceSum;      //the device sum notified during this period,several notifications during the same period have the same value. 
	int  nCurDeviceCount; //device number in one notification,the value should be the same as nWifiNum in struct ALARM_WIFI_SEARCH_INFO,several notifications during the same period sum up to nCurDeviceCount
	BYTE reserved[500];   //reserved
}NET_WIFI_BASIC_INFO;

// result of positioning
typedef enum tagNET_GPS_POSITION_RESULT
{				
	NET_GPS_POSITION_RESULT_UNKNOWN = 0,					// unknown
	NET_GPS_POSITION_RESULT_FAILED = 1,						// has GPS data,but orientation failed,data is inefficacy
	NET_GPS_POSITION_RESULT_SUCCEED = 2,					// has GPS data,and orientation succeed,data is efficacious
}NET_GPS_POSITION_RESULT;

// wifi place info
typedef struct tagNET_WIFI_GPS_INFO
{
	NET_GPS_POSITION_RESULT			emPositioningResult;	// result of positioning
	unsigned int					nLongitude;				// Longitude(unit:1/1000000 degree)
															// west Longitude: 0 - 180000000				practical value = 180*1000000 - dwLongitude
															// east Longitude: 180000000 - 360000000		practical value = dwLongitude - 180*1000000
															// eg: Longitude:300168866 (300168866 - 180*1000000)/1000000  equal east Longitude 120.168866 degree
	unsigned int					nLatidude;				// Latidude(unit:1/1000000 degree)
															// north Latidude: 0 - 90000000				practical value = 90*1000000 - dwLatidude
															// south Latidude: 90000000 - 180000000	practical value = dwLatidude - 90*1000000
															// eg: Latidude:120186268 (120186268 - 90*1000000)/1000000 equal south Latidude 30. 186268 degree
	unsigned int					nSpeed;					// speed(unit:0.001km/H)
	BYTE							reserved[112];			// reserved
}NET_WIFI_GPS_INFO;

// event type DH_ALARM_WIFI_SEARCH (search WIFI device) data description
typedef struct tagALARM_WIFI_SEARCH_INFO
{
    int                      nWifiNum;                       // WIFI number, indicate valid object in stuWifi
    NET_WIFI_DEV_INFO        stuWifi[1024];                  // wifi device info
	int                      nChannel;                       // channel 
	NET_WIFI_BASIC_INFO      stuWifiBasiInfo;				 // Wifi event basic info
	BOOL					 bGPSinfo;						 // is include GPS info 
	NET_WIFI_GPS_INFO		 stuWifiGPSInfo;				 // GPS info
    BYTE                     reserved[376];                  // reserved
} ALARM_WIFI_SEARCH_INFO;


// WIFI virtual informaion
typedef struct tagNET_WIFI_VIRTUALINFO
{
	NET_TIME_EX				stuAccessTime;								// access time, not accurate, just for your reference
	char					szSrcMac[DH_MACADDR_LEN];					// source MAC of virtual information,shown in capital letters,seperated by "-" 
    char					szDstMac[DH_MACADDR_LEN];					// destination MAC of virtual information,shown in capital letters,seperated by "-" 
	int						nProtocal;									// protocal code, which signals corresponding apps
	char					szUrl[DH_MAX_URL_LEN];						// url
	char					szDomain[MAX_VIRTUALINFO_DOMAIN_LEN];		// domain
	char					szTitle[MAX_VIRTUALINFO_TITLE_LEN];			// net title
	char					szUsrName[MAX_VIRTUALINFO_USERNAME_LEN];	// username
	char					szPassWord[MAX_VIRTUALINFO_PASSWORD_LEN];	// password
	char					szPhoneNum[MAX_VIRTUALINFO_PHONENUM_LEN];	// telephone number
	char					szImei[MAX_VIRTUALINFO_IMEI_LEN];			// international mobile device indicator
	char					szImsi[MAX_VIRTUALINFO_IMSI_LEN];			// international mobile user indicator
	char					szLatitude[MAX_VIRTUALINFO_LATITUDE_LEN];	// latitude
	char					szLongitude[MAX_VIRTUALINFO_LONGITUDE_LEN];	// longitude
	char					szSrcIP[DH_MAX_IPADDR_LEN_EX];				// source IP
	char					szDstIP[DH_MAX_IPADDR_LEN_EX];				// destination IP
	UINT					nSrcPort;									// source port 
	UINT					nDstPort;									// destination port
    char                    szSiteNum[MAX_COMMON_STRING_16];            // site number
    char                    szDevNum[MAX_COMMON_STRING_32];             // device Number
    char                    szUserID[MAX_COMMON_STRING_32];             // fictitious user ID
    char                    szIDFA[MAX_COMMON_STRING_64];               // IDFA of iphone
	BYTE                    reserved[368];								// reserved
}NET_WIFI_VIRTUALINFO;

// event type DH_ALARM_WIFI_VIRTUALINFO_SEARCH (search wifi virtual information)data description
typedef struct tagALARM_WIFI_VIRTUALINFO_SEARCH_INFO
{
    int                      nVirtualInfoNum;                // number of wifi virtual information,indicates valid number of stuVirtualInfo
    NET_WIFI_VIRTUALINFO     stuVirtualInfo[MAX_VIRTUALINFO_NUM];           // wifi virtual information
	int                      nChannel;                       // channel number
    BYTE                     reserved[512];                  // reserved
} ALARM_WIFI_VIRTUALINFO_SEARCH_INFO;

// event type DH_ALARM_USER_LOCK_EVENT (User Lock Alarm Event)data description
typedef struct tagALARM_USER_LOCK_EVENT_INFO
{	
    char                    szUser[DH_NEW_USER_NAME_LENGTH];       // user  name
	char					szGroup[DH_NEW_USER_NAME_LENGTH];	   // group name
	char                    szIP[DH_MAX_IPADDR_LEN_EX];            // device IP
	BYTE                    reserved[512];						   // reserved 
} ALARM_USER_LOCK_EVENT_INFO;

//wirelesstype
typedef enum tagNET_WIRELESSDEV_LOWPOWER_TYPE
{
    NET_WIRELESSDEV_UNKNOWN = 0,                                // unknowndevice
    NET_WIRELESSDEV_CONTROL = 1,                                // wirelesscontrol
    NET_WIRELESSDEV_DEFENCE = 2,                                // wirelessdefence
    NET_WIRELESSDEV_KEYBOARD = 3,                               // wirelessKeyBoard
    NET_WIRELESSDEV_MAGNETOMER = 4,                             // wirelessMagnetomer
	NET_WIRELESSDEV_ALARMBELL = 5,								// wirelessAlarmBell
} NET_WIRELESSDEV_LOWPOWER_TYPE;

//report event of lowpower wireless
typedef struct tagALARM_WIRELESSDEV_LOWPOWER_INFO
{
    NET_THREE_STATUS_BOOL         emResult;                  // lowpower event:true lowpower,false narmal, unknown Unknown
    NET_TIME                      stuTime;                   // event time
    int                           nId;                       // wirelessdevice ID
    NET_WIRELESSDEV_LOWPOWER_TYPE emType;                    // wirelessdevice type
    BYTE                          reserved[1024];            // reserved
} ALARM_WIRELESSDEV_LOWPOWER_INFO;

// event type DH_ALARM_BUS_PAD_SHUTDOWN(PAD shut down event) data description
typedef struct tagALARM_BUS_PAD_SHUTDOWN_INFO
{
    int                nDelayTime;                      // delay time for shut down, unit: sec
    BOOL               bConfirm;                        // need confirme to shut down, TRUE: yes, FALSE: no
    NET_TIME_EX        stuUtcTime;                      // event time
    DWORD              dwUtc;                           // event time, for confirm
    BYTE               byReserved[512];                 // revervsed
} ALARM_BUS_PAD_SHUTDOWN_INFO;

// heat picture data type
typedef enum tagEM_HEAT_PIC_DATA_TYPE
{
	EM_HEAT_PIC_DATA_TYPE_UNKNOWN,					// Unknown
	EM_HEAT_PIC_DATA_TYPE_GRAYDATA,					// Gray data
	EM_HEAT_PIC_DATA_TYPE_SOURCEDATA,				// Source data
}EM_HEAT_PIC_DATA_TYPE;
// get heatmap information, input arguments
typedef struct tagNET_IN_QUERY_HEAT_MAP
{
    int                      nChannel;                       // channel id
    NET_TIME_EX              stuBegin;                       // begin time
    NET_TIME_EX              stuEnd;                         // end time
    UINT                     nPlanID;                        // Plan ID,Speed Dome use,start from 1
	EM_HEAT_PIC_DATA_TYPE	 emDataType;					 // Desired Data type
    BYTE                     reserved[1016];                 // reserved
} NET_IN_QUERY_HEAT_MAP;

// get heatmap information, output arguments
typedef struct tagNET_OUT_QUERY_HEAT_MAP
{
    int                      nWidth;                         // picture width
    int                      nHeight;                        // picture height
    char *                   pBufData;                       // heatmap picture data, user alloc this memory, size is nBufLen
															 // if emDataType is EM_HEAT_PIC_DATA_TYPE_GRAYDATA,a Byte means a point
															 // if emDataType is EM_HEAT_PIC_DATA_TYPE_SOURCEDATA,four Bytes mean a point
    int                      nBufLen;                        // pBufData max len
    int                      nBufRet;                        // data len
    int                      nAverage;                       // average info
    UINT					 nPlanID;						 // plan id
	EM_HEAT_PIC_DATA_TYPE	 emDataType;					 // Obtained Data type
	UINT					 nPixelMax;						 // Max pixel value
	UINT					 nPixelMin;						 // Min pixel value
    BYTE                     reserved[1004];                 // reserved
} NET_OUT_QUERY_HEAT_MAP;

// get heatmap information, type DH_DEVSTATE_GET_HEAT_MAP
typedef struct tagNET_QUERY_HEAT_MAP
{
    DWORD                    dwSize;                         // size of this structure
    NET_IN_QUERY_HEAT_MAP    stuIn;                          // heatmap search condition
    NET_OUT_QUERY_HEAT_MAP   stuOut;                         // heatmap search result
} NET_QUERY_HEAT_MAP;

//input parameter when getting GPS information
typedef struct tagNET_IN_DEV_GPS_INFO
{
	DWORD                    dwSize;                         // size of NET_IN_DEV_GPS_INFO
    int                      nChannel;                       // device channel number
} NET_IN_DEV_GPS_INFO;

//GPS work state
typedef enum tagNET_GPS_WORK_STATUS
{
	EM_WORK_STATE_UNKNOWN = 0,			//unknown work status
	EM_NO_POSITIONING,					//no positioning
	EM_NO_DIFFERENTIAL_POSITIONING,		//not differential positioning
	EM_DIFFERENTIAL_POSITIONING,		//differential positioning
	EM_INVALID_PPS,						//invalid PPS
	EM_EVALUATING,						//evalusting
}NET_GPS_WORK_STATUS;

typedef struct tagNET_OUT_DEV_GPS_INFO
{
	DWORD                   dwSize;                          // size of NET_OUT_DEV_GPS_INFO
    NET_TIME				stuLocalTime;                    // current time
	double                  dbLongitude;                     // longtitude(unit:1/1000000 degree, range:0-360 degree)
    double                  dbLatitude;                      // latitude(unit:1/1000000 degree, range:0-180 degree)
	double                  dbAltitude;                      // altitude(unit:metre)
	double                  dbSpeed;                         // speed(unit:km/H)
	double                  dbBearing;                       // bearing angle(unit:degree)
	NET_THREE_STATUS_BOOL   emAntennasStatus;				 // antennas status(0:bad 1:good)
	NET_THREE_STATUS_BOOL   emPositioningResult;             // fix a position or not(0:no 1:yes)
	DWORD					dwSatelliteCount;				 // count of satellites
	NET_GPS_WORK_STATUS     emworkStatus;                    // work state
	int                     nAlarmCount;					 // count of alarm
	int                     nAlarmState[128];                // places that alarm occur,maybe multiple values
	float					fHDOP;							 // horizontal precision factor
} NET_OUT_DEV_GPS_INFO;

// GPS status info
typedef struct tagNET_SET_GPS_STATUS_INFO
{
	EM_DATE_SOURCE          emDateSource;						// date source
	NET_TIME				stuLocalTime;						// current time
	double                  dbLongitude;						// longtitude(unit: degree, range:0-360 degree)
    double                  dbLatitude;							// latitude(unit: degree, range:0-180 degree)
	double                  dbAltitude;							// altitude(unit:metre)
	double                  dbSpeed;							// speed(unit:km/H)
	double                  dbBearing;							// bearing angle(unit:degree)
	NET_THREE_STATUS_BOOL   emAntennasStatus;					// antennas status(0:bad 1:good)
	NET_THREE_STATUS_BOOL   emPositioningResult;				// fix a position or not(0:no 1:yes)
	int						nSatelliteCount;					// count of satellites
	NET_GPS_WORK_STATUS     emWorkStatus;						// work state
	int                     nAlarmCount;						// count of alarm
	int                     nAlarmState[128];					// places that alarm occur,maybe multiple values
	double					dbHDOP;								// horizontal precision factor
	BYTE					byReserved[1024];					// Reserved byte
} NET_SET_GPS_STATUS_INFO;

// CLIENT_SetGPSStatus interface paramter input
typedef struct tagNET_IN_SET_GPS_STATUS
{
	DWORD						dwSize;								
	BOOL						bEnable;						// whether use setup data
	int							nChannel;						// channel
	NET_SET_GPS_STATUS_INFO		stGPSInfo;						// GPS status info
} NET_IN_SET_GPS_STATUS;

// CLIENT_SetPositionStatus interface paramter output
typedef struct tagNET_OUT_SET_GPS_STATUS
{
	DWORD					dwSize;								
} NET_OUT_SET_GPS_STATUS;

// set GPS status information
CLIENT_NET_API BOOL CALL_METHOD CLIENT_SetGPSStatus(LLONG lLoginID, const NET_IN_SET_GPS_STATUS* pInParam, NET_OUT_SET_GPS_STATUS* pOutParam, int nWaitTime);

// Query IVS related remote device information input parameter
typedef struct tagNET_IN_IVS_REMOTE_DEV_INFO
{
    DWORD                   dwSize;                         // size of this structure
    int                     nChannel;                       // channel
}NET_IN_IVS_REMOTE_DEV_INFO;

// Query IVS related remote device information output parameter
typedef struct tagNET_OUT_IVS_REMOTE_DEV_INFO
{
    DWORD                   dwSize;                         // size of this structure
    int                     nPort;                          // device port
    char                    szIP[64];                       // device ip   
    char                    szUser[64];                     // user name
    char                    szPassword[64];                 // password
    char                    szAddress[128];	                // deployment address of equipment
}NET_OUT_IVS_REMOTE_DEV_INFO;

// Query smart switch information input parameter
typedef struct tagNET_IN_SMART_SWITCH_INFO
{
    DWORD                   dwSize;                         // size of this structure
    char                    szSerialNumber[DH_WIRELESS_DEVICE_SERIAL_NUMBER_MAX_LEN]; // serial number of device
}NET_IN_SMART_SWITCH_INFO;

// Query smart switch information output parameter
typedef struct tagNET_OUT_SMART_SWITCH_INFO
{
    DWORD                   dwSize;                         // size of this structure
    BOOL                    bSwitchEable;                   // switch state , TRUE - on, FALSE - off
    double                  dbCurrentPower;                 // current power, unit: W 
    double                  dbHistoryPowerUsed;             // history power used, unit: kw/h
    double                  dbTodayPowerUsed;               // today power used, unit: kw/h
    double                  dbMonthPowerUsed[NET_MAX_MONTH_NUM]; // month power userd, unit: kw/h
}NET_OUT_SMART_SWITCH_INFO; 

// Query upgrade-state input parameter
typedef struct tagNET_IN_UPGRADE_STATE
{
    DWORD                   dwSize;                         // size of this structure
}NET_IN_UPGRADE_STATE;

// stream type
enum CFG_EM_STREAM_TYPES
{
	CFG_EM_STREAMTYPE_ERR,                  // other
	CFG_EM_STREAMTYPE_MAIN,					// "Main"-main stream
	CFG_EM_STREAMTYPE_EXTRA_1,				// "Extra1"-sub stream1
	CFG_EM_STREAMTYPE_EXTRA_2,				// "Extra2"-sub stream2
	CFG_EM_STREAMTYPE_EXTRA_3,				// "Extra3"-sub stream3
	CFG_EM_STREAMTYPE_SNAPSHOT,				// "Snapshot"-snap stream
	CFG_EM_STREAMTYPE_TALKBACK,				// "Talkback"-talk stream
};

// query video encode caps input parameters
typedef struct tagNET_IN_VIDEO_ENCODE_CAPS
{
	DWORD					dwSize;                         // structure size
	int						nChannel;						// channel NO.
	int						nGroup;							// group NO.
	CFG_EM_STREAM_TYPES		stStreamType;					// stream type
}NET_IN_VIDEO_ENCODE_CAPS;

// query video encode caps output parameters
typedef struct tagNET_OUT_VIDEO_ENCODE_CAPS
{
	DWORD					dwSize;																// structure size
	int						nSvcEncodeTypesNum;													// number of video formats which supports SVC encode
	char					szSvcEncodeTypes[DH_COMMON_STRING_32][DH_COMMON_STRING_32];			// formats which supports SVC encode
	int						nGOPCustomEncodesNum;												// number of streams which supports self-defined I frames gap
	char					szGOPCustomEncodes[DH_COMMON_STRING_32][DH_COMMON_STRING_32];		// streams which supports self-defined I frames gap
	int						nMaxSVCTLevel;														// max number of level which SVC-T supports
}NET_OUT_VIDEO_ENCODE_CAPS;

// query audio encode caps input parameters
typedef struct tagNET_IN_AUDIO_ENCODE_CAPS
{
	DWORD					dwSize;                         // structure size
	int						nChannel;						// channel NO.
	CFG_EM_STREAM_TYPES		stStreamType;					// stream type
}NET_IN_AUDIO_ENCODE_CAPS;

// query audio encode caps output parameters
typedef struct tagNET_OUT_AUDIO_ENCODE_CAPS
{
	DWORD					dwSize;														// structure size
	BOOL					bSupportSourceSelect;										// support multiple audio inputs or not
	int						nSourceTypeNum;												// audio resource type number
	char					szSourceType[DH_COMMON_STRING_32][DH_COMMON_STRING_32];		// audio resource type list
}NET_OUT_AUDIO_ENCODE_CAPS;

// query audio input channel caps input parameters
typedef struct tagNET_IN_AUDIO_IN_CAPS
{
	DWORD					dwSize;                         // structure size
	int						nChannel;						// channel NO.
}NET_IN_AUDIO_IN_CAPS;

//  query audio input channel caps output parameters
typedef struct tagNET_IN_AUDIO_OUT_CAPS
{
	DWORD					dwSize;                         // structure size
	int						nMicNum;						// input mic number,>0 means supports talk input
	int						nLineInNum;						// input line number,>0 mean backing track input
}NET_OUT_AUDIO_IN_CAPS;

// Smart encode information
typedef struct tagSMART_ENCODE_INFO
{
	DWORD					dwSize;							    // structure size
	char					szCompression[MAX_COMMON_STRING_8]; // video encode format,as "H.264", "H.265"
	int						nPolicy;						    // Smart encode strategy
	bool					bEnable;						    // stream enable or not
	int						nWidth;							    // video width, optional
	int						nHeight;						    // video height, optional
	float					fFPS;							    // video frame rate, optional
}SMART_ENCODE_INFO;

// query Smart encode caps input parameters
typedef struct tagNET_IN_SMART_ENCODE_CAPS
{
	DWORD					dwSize;								// structure size
	int						nChannel;							// channel NO.
	int						nStreamNum;							// actural stream NO.
	SMART_ENCODE_INFO		stSmartEncodeInfo[MAX_STREAM_NUM];  // Smart encode information of each stream,if stream NO. is 3,array index 012 presents the encode information of main streamsub stream1sub stream2
}NET_IN_SMART_ENCODE_CAPS;

// Smart encode caps
typedef struct tagSMART_ENCODE_CAPS_INFO
{
	DWORD					dwSize;																// structure size
	int						nSmartEncodeCap;													// 1: supports open encode locate P frame function; 1: not support Smart encode; 2: not support encode
	
}SMART_ENCODE_CAPS_INFO;

//query Smart encode caps output parameters
typedef struct tagNET_OUT_SMART_ENCODE_CAPS
{
	DWORD						dwSize;																// structure size
	int							nSmartEncodeCapsNum;												// Smart encode caps group number
	SMART_ENCODE_CAPS_INFO		stSmartEncodeCaps[MAX_STREAM_NUM];									// encode caps of ezch stream smart encode information
}NET_OUT_SMART_ENCODE_CAPS;

// uprade packege type
typedef enum tagEM_UPGRADE_PACKAGE_TYPE
{
    EM_UPGRADE_PACKAGE_TYPE_UNKNOWN,                                // unknown 
    EM_UPGRADE_PACKAGE_TYPE_REGULAR,                                // regular
    EM_UPGRADE_PACKAGE_TYPE_EMERGENCY,                              // emergency
}EM_UPGRADE_PACKAGE_TYPE;

// uprade package and uprade state
typedef enum tagEM_UPGRADE_STATE
{
    EM_UPGRADE_STATE_UNKNOWN,                               // unknown
    EM_UPGRADE_STATE_NONE,                                  // check none
    EM_UPGRADE_STATE_INVALID,                               // package invalid
    EM_UPGRADE_STATE_NOT_ENOUGH_MEMORY,                     // out of memory
    EM_UPGRADE_STATE_DOWNLOADING,                           // downloading package
    EM_UPGRADE_STATE_DOWNLOAD_FAILED,                       // download failed
    EM_UPGRADE_STATE_DOWNLOAD_SUCCESSED,                    // download successed
    EM_UPGRADE_STATE_PREPARING,                             // preparing
    EM_UPGRADE_STATE_UPGRADING,                             // upgrading
    EM_UPGRADE_STATE_UPGRADE_FAILED,                        // upgrade failed
    EM_UPGRADE_STATE_UPGRADE_SUCCESSED,                     // upgrade successed
    EM_UPGRADE_STATE_UPGRADE_CANCELLED,                     // upgrade cancelled
	EM_UPGRADE_STATE_FILE_UNMATCH,                          // file unmatch
}EM_UPGRADE_STATE;

// event of deivce upgrade
typedef struct tagALARM_UPGRADE_STATE
{
    EM_UPGRADE_STATE        emState;                        // uprade package and uprade state
    int                     nProgress;                      // upgrade progress, 0 ~ 100
    char                    szFileName[DH_COMMON_STRING_64];// upgrading file name
    BYTE		            byReserved[1024];
}ALARM_UPGRADE_STATE;

typedef enum tagEM_LABELDATA_STATE
{
	EM_LABELDATA_STATE_ENTER,									 // Enter
	EM_LABELDATA_STATE_LEAVE,									 // Leave
}EM_LABELDATA_STATE;

// IPC added new(2017.4),RFID tag information acquisition event 
typedef struct tagALARM_LABELINFO
{
	int						nChannelID;							// channel ID
	char					szIndexIs[DH_COMMON_STRING_8];		// The meaning of the event index representation, such as no field, cannot be cascaded
	int						nVideoIndex;						// Video channel
	UINT					nACK;								// Confirm ID. 
	char					szReceiverID[DH_COMMON_STRING_16];	// Receiver ID 
	char					szLabelID[DH_COMMON_STRING_16];		// RFID Label
	NET_TIME_EX				stuDateTime;						// Acquisition time UTC
	EM_LABELDATA_STATE		emLabelDataState;					// Label Data StateEnterLeave
	BYTE					byReserve[1024];					// reserve
}ALARM_LABELINFO;

// Query upgrade-state output parameter
typedef struct tagNET_OUT_UPGRADE_STATE
{
    DWORD                   dwSize;                         // size of this structure
    char                    szOldVersion[DH_COMMON_STRING_64]; 	// old version
    char                    szNewVersion[DH_COMMON_STRING_64];  // new version
    EM_UPGRADE_STATE        emState;                        // upgrade package and upgrade state
    EM_UPGRADE_PACKAGE_TYPE emType;                         // upgrade package type
    int                     nProgress;                      // progress of upgrading 0 - 100
}NET_OUT_UPGRADE_STATE;

//query hard disk temperature input parameter
typedef struct tagNET_IN_HDD_TEMPERATURE
{
	DWORD					dwSize;
	char					szHardDiskName[DH_COMMON_STRING_16];		// hard disk name
	
}NET_IN_HDD_TEMPERATURE;

//query hard disk temperature output parameter
typedef struct tagNET_OUT_HDD_TEMPERATURE
{
	DWORD					dwSize;
	int						nID;										// attribute ID
	char					szName[DH_COMMON_STRING_64];				// attribute name
	int						nCurrent;									// attribute value
	int						nWorst;										// maximum error value
	int						nThreshold;									// threshod
	char					szRaw[DH_COMMON_STRING_32];					// actual value
	int						nPredict;									// state
	int						nSync;										// synchronization status,0 is adaptive;1 is synchronous priority, I/O priority to raid synchronous; 
																		// 2 is business priority, I/O priority to the hard disk to write data; 3 is balanced
}NET_OUT_HDD_TEMPERATURE;

// Get the YUV data input parameter in the specified format 
typedef struct tagNET_IN_RAWFRAMEDATA
{
	DWORD					dwSize;										
	int						nChannel;									// video input channel
	int						nSensorID;									// sensor ID
	char					szRawFrameType[DH_COMMON_STRING_32];		// YUV data format, The supported scope is obtained through the CLIENT_GetDevCaps function, NET_VIDEOIN_RAWFRAME_CAPS;
}NET_IN_RAWFRAMEDATA;

// Get the YUV data output parameter in the specified format
typedef struct tagNET_OUT_RAWFRAMEDATA
{
	DWORD					dwSize;
	UINT					nHeight;									// The height of the image returned
	UINT					nWidth;										// the width of the image returned
	UINT					nDataLen;									// YUV binary datauint: byte
	char*					pszBuffer;									// YUV binary dataThe user applies for memorynBufferLen
	int						nBufferLen;									// User application memory size
}NET_OUT_RAWFRAMEDARA;

//DoorWork mode
typedef enum tagNET_DOORWORK_MODE
{
    NET_DOORWORK_MODE_UNKNOWN = 0,                                // unknown
    NET_DOORWORK_MODE_NORMAL = 1,                                 // normal
    NET_DOORWORK_MODE_SHUTLOCK = 2,                               // shutlock
    NET_DOORWORK_MODE_UNUSED = 3,                                 // unused
    NET_DOORWORK_MODE_OPENDOORCONTINUE = 4,                       // open door continue
} NET_DOORWORK_MODE;

// Get DoorWork mode, type DH_DEVSTATE_GET_ACCESSCONTROLMODE
typedef struct tagNET_GET_DOORWORK_MODE
{
    DWORD                    dwSize;
    int                      nChannel;              // ChannelID
    NET_DOORWORK_MODE   emControlMode;              // DoorWork mode      
}NET_GET_DOORWORK_MODE;

// VT Event Type
typedef enum tagEM_AUDIO_CB_FLAG
{
    EM_AUDIO_CB_FLAG_UNKNOWN,
    EM_AUDIO_CB_FLAG_NEWCALL,                   // new call
    EM_AUDIO_CB_FLAG_REMOTE_HANGUP,             // remote hangup
    EM_AUDIO_CB_FLAG_DISCONNECT,                // disconnect
    EM_AUDIO_CB_FLAG_RING,                      // ring
} EM_AUDIO_CB_FLAG;

// VT callback function
typedef int (CALLBACK *pfVtEventCallBack)(LLONG instId, LLONG ulRegisterId, LLONG ulSessionId, int nEvent, char *pDataBuf, DWORD dwBufSize, LDWORD dwUser);

// new call action EM_AUDIO_CB_FLAG_NEWCALL
typedef enum tagEM_NEWCALL_ACTION
{
    EM_NEWCALL_ACTION_UNKNOWN,                  // unknown
    EM_NEWCALL_ACTION_REFUSE,                   // refuse
    EM_NEWCALL_ACTION_ACCEPT,                   // accept
} EM_NEWCALL_ACTION;

typedef enum tagEM_VT_PARAM_VALID
{
    EM_VT_PARAM_VALID_EVENT_CB     = 0x0001,
    EM_VT_PARAM_VALID_USER_DATA    = 0x0002,
    EM_VT_PARAM_VALID_MID_NUM      = 0x0004,
    EM_VT_PARAM_VALID_ACTION       = 0x0008,
    EM_VT_PARAM_VALID_WAITTIME     = 0x0010,
    EM_VT_PARAM_VALID_VIDEOWND     = 0x0020,
    EM_VT_PARAM_VALID_CSMODE       = 0x0040,
    EM_VT_PARAM_VALID_AUDIO_ENCODE = 0x0080,
    EM_VT_PARAM_VALID_LOCAL_IP     = 0x0100,
} EM_VT_PARAM_VALID;

// VT Talk Parameter
typedef struct tagNET_VT_TALK_PARAM
{
    DWORD               dwSize;                  // struct size
    int                 nValidFlag;              // Flag Valid, combination of EM_VT_PARAM_VALID 
    pfVtEventCallBack   pfEventCb;               // Event callback function, EM_VT_PARAM_VALID_EVENT_CB
    LDWORD              dwUser;                  // user information, EM_VT_PARAM_VALID_USER_DATA
    char                szPeerMidNum[16];        // peer middle number, 8 byte, EM_VT_PARAM_VALID_MID_NUM
    EM_NEWCALL_ACTION   emAction;                // Action, 0:unknown, 1:refuse, 2:accept, EM_VT_PARAM_VALID_ACTION
    int                 nWaitTime;               // waittime, EM_VT_PARAM_VALID_WAITTIME
    HWND                hVideoWnd;               // Video Window, EM_VT_PARAM_VALID_VIDEOWND
    BOOL                bClient;                 // Client/Server, TRUE:Client, FALSE:Server, EM_VT_PARAM_VALID_CSMODE
    DHDEV_TALKDECODE_INFO stAudioEncode;         // Audio encode info, EM_VT_PARAM_VALID_AUDIO_ENCODE
} NET_VT_TALK_PARAM;

// VTC Register VTO
CLIENT_NET_API LLONG CALL_METHOD CLIENT_VT_RegisterVto(const char *pszIp, int nPort = 13801);

// VTC Unregister VTO
CLIENT_NET_API BOOL CALL_METHOD CLIENT_VT_UnRegisterVto(LLONG ulLongId);

/////////////////////////////////// UAV interfaces ///////////////////////////////////////

// UAV fly info
typedef struct tagNET_UAV_FLY_INFO 
{
    float               fAltitude;                          // Altitude, with respect to horizon, Unit: meter
    float               fDistance;                          // Distance, with respect to remote controller, Unit: meter
    float               fHorizontalSpeed;                   // Horizontal Speed, Unit: m/s
    float               fVerticalSpeed;                     // Vertical Speed, Unit: m/s
    int                 nRCLinkQuality;                     // remote controller link quality, range: 0~100
    char                reserved[512];
} NET_UAV_FLY_INFO;

// callback for UAV info, lAttachHandle is the return value of CLIENT_AttachUavFly
typedef void (CALLBACK *fUavFlyCallBack) (LLONG lAttachHandle, NET_UAV_FLY_INFO* pBuf, DWORD dwBufLen, LDWORD dwUser);

// CLIENT_AttachUavFly input
typedef struct tagNET_IN_ATTACH_UAV_FLY 
{
    DWORD               dwSize;
    fUavFlyCallBack     cbUavFly;                           // fly info callback
    DWORD               dwUser;                             // user data                   
} NET_IN_ATTACH_UAV_FLY;

// CLIENT_AttachUavFly output
typedef struct tagNET_OUT_ATTACH_UAV_FLY 
{
    DWORD                dwSize;
} NET_OUT_ATTACH_UAV_FLY;

// subscribe UAV fly info,user malloc memory of pInParam and pOutParam
CLIENT_NET_API LLONG CALL_METHOD CLIENT_AttachUavFly(LLONG lLoginID, const NET_IN_ATTACH_UAV_FLY* pInParam, NET_OUT_ATTACH_UAV_FLY* pOutParam, int nWaitTime);

// unsubscribe UAV fly info
CLIENT_NET_API BOOL CALL_METHOD CLIENT_DetachUavFly(LLONG lAttachHandle);

////////////////////////////////////////////////////////////////////////////////

// Get device self check info,user malloc memory of pInParam and pOutParam
CLIENT_NET_API BOOL CALL_METHOD CLIENT_GetSelfCheckInfo(LLONG lLoginID, const NET_IN_GET_SELTCHECK_INFO* pInParam, NET_SELFCHECK_INFO* pOutParam, int nWaitTime = NET_INTERFACE_DEFAULT_TIMEOUT);

/////////////////////////////////Laser and EIS interface/////////////////////////////////

// open or close laser input
typedef struct tagLASER_IN_INFO
{
	DWORD		dwSize;				
	UINT		nChannelID;			// channel ID
} LASER_IN_INFO;

// open or close laser output
typedef struct tagLaser_OUT_INFO 
{
	DWORD		dwSize;			
} LASER_OUT_INFO;

// open or close EIS input
typedef struct tagEIS_IN_INFO 
{
	DWORD		dwSize;				
	UINT		nChannelID;			// channel ID
} EIS_IN_INFO;

// open or close EIS output
typedef struct tagEIS_OUT_INFO 
{
	DWORD		dwSize;			
} EIS_OUT_INFO;


// open laser interface,user malloc and free memory of pLaserDataIn and pLaserDataOut 
CLIENT_NET_API BOOL CALL_METHOD CLIENT_StartLaser(LLONG lLoginID,const LASER_IN_INFO* pLaserDataIn, LASER_OUT_INFO* pLaserDataOut,int nWaitTime = 1000);

// close laser interface,user malloc and free memory of pLaserDataIn and pLaserDataOut 
CLIENT_NET_API BOOL CALL_METHOD CLIENT_StopLaser(LLONG lLoginID,const LASER_IN_INFO* pLaserDataIn, LASER_OUT_INFO* pLaserDataOut,int nWaitTime = 1000);

// open EIS interface,user malloc and free memory of pEISDataIn and pEISDataOut 
CLIENT_NET_API BOOL CALL_METHOD CLIENT_StartEIS(LLONG lLoginID,const EIS_IN_INFO* pEISDataIn, EIS_OUT_INFO* pEISDataOut,int nWaitTime = 1000);

// close EIS interface,user malloc and free memory of pEISDataIn and pEISDataOut 
CLIENT_NET_API BOOL CALL_METHOD CLIENT_StopEIS(LLONG lLoginID,const EIS_IN_INFO* pEISDataIn, EIS_OUT_INFO* pEISDataOut,int nWaitTime = 1000);

// PSTN manually checking,user malloc and free memory of pInBuf and pOutBuf 
CLIENT_NET_API BOOL CALL_METHOD CLIENT_ManualCheckPSTN(LLONG lLoginID, const NET_IN_PSTN_MANUALCHECK_STATE* pInBuf, NET_OUT_PSTN_MANUALCHECK_STATE* pOutBuf, int nWaitTime = NET_INTERFACE_DEFAULT_TIMEOUT);
#define  MAX_NUM_EX_MODULE          16              // the max number of extend modules
#define  MAX_NUM_ADDR_IN_BUS        256             // the max number of address in the bus

// CLIENT_GetExModuleState input parameter
typedef struct tagNET_IN_EXMODULE_INFO 
{
    DWORD		dwSize;				                                        // size of the structure
} NET_IN_EXMODULE_INFO;

// the information of single extend module
typedef struct tagNET_OUT_EXMODULE_INFO
{
    NET_BUS_TYPE		emBusType;                                          // bus type
    int                 nChannelID;	                                        // bus number
    int                 nAddrCount;                                         // the number of address in actually
    int                 nAddr[MAX_NUM_ADDR_IN_BUS];                         // address in the bus
    char                reserved[1024];                                     // reserved
} NET_OUT_EXMODULE_INFO;

// CLIENT_GetExModuleState output parameter
typedef struct tagNET_OUT_EXMODULE_INFO_ALL
{
    DWORD		                dwSize;			                            // size of the structure
    int                         nExModuleCount;                             // the number of bus
    NET_OUT_EXMODULE_INFO       stuExModuleInfo[MAX_NUM_EX_MODULE];         // the array of extend modules's information
} NET_OUT_EXMODULE_INFO_ALL;

/**************************************************************************************
*   Funcname: CLIENT_GetExModuleState
*   Purpose:Get the information of extend modules
*   InputParam:   LLONG                        :lLoginID        //login handle
*   InputParam:   NET_IN_EXMODULE_INFO*        :pInParam        //input parameter,can't fill NULL,need set dwSize,user malloc and free pInParam's memory
*   InputParam:   NET_OUT_EXMODULE_INFO_ALL*   :pOutParam       //output parameter,can't fill NULL,need set dwSize,user malloc and free pOutParam's memory
*   InputParam:   int                          :nWaitTime       //wait time
*   Return: BOOL
**************************************************************************************/
CLIENT_NET_API BOOL CALL_METHOD CLIENT_GetExModuleState(LLONG lLoginID, const NET_IN_EXMODULE_INFO* pInParam, NET_OUT_EXMODULE_INFO_ALL* pOutParam, int nWaitTime = NET_INTERFACE_DEFAULT_TIMEOUT);

///////////////////////////////Master slave Group operate interface //////////////////////////
#define MASTERSALVE_CLASS_LEN           16

typedef struct tagNET_IN_MSGROUP_OPEN_INFO
{
    DWORD                  dwSize;     
    int                    nGroupID;                            // device group id
    int                    nSlaveID;                            // slave device id
}NET_IN_MSGROUP_OPEN_INFO;

typedef struct tagNET_OUT_MSGROUP_OPEN_INFO
{
    DWORD                  dwSize;         
    DWORD                  dwToken;                             // master slave group token
}NET_OUT_MSGROUP_OPEN_INFO;

typedef struct tagNET_IN_MSGROUP_LOCATE_INFO
{
    DWORD                   dwSize;          
    DWORD                   dwToken;                            // master slave group token
    DH_POINT                stuPoint;                           // master coordinate
    short                   snMasterInfo[3];                    // [abscissa, ordinate, radius]
}NET_IN_MSGROUP_LOCATE_INFO;

typedef struct tagNET_OUT_MSGROUP_LOCATE_INTO
{
    DWORD                   dwSize;        
    short                   snPTZ[3];                           // PTZ
}NET_OUT_MSGROUP_LOCATE_INTO;

typedef struct tagNET_IN_MSGROUP_TRACK_INFO
{
    DWORD                   dwSize;                        
    DWORD                   dwToken;                            // master slave group token
    char                    szClass[MASTERSALVE_CLASS_LEN];     // object type
    DWORD                   dwObjectID;                         // object id
}NET_IN_MSGROUP_TRACK_INFO;

typedef struct tagNET_OUT_MSGROUP_TRACK_INFO
{
    DWORD                   dwSize;        
}NET_OUT_MSGROUP_TRACK_INFO;

typedef struct tagNET_IN_MSGROUP_CLOSE_INFO
{
    DWORD                   dwSize;         
    DWORD                   dwToken;                            // master slave group token
}NET_IN_MSGROUP_CLOSE_INFO;

typedef struct tagNET_OUT_MSGROUP_CLOSE_INFO
{
    DWORD                   dwSize;        
}NET_OUT_MSGROUP_CLOSE_INFO;

// MasterSlaveGroup operation type EM_MSGROUP_OPERATE_RECTLOCATE input
typedef struct tagNET_IN_MSGROUP_RECTLOCATE_INFO
{
    DWORD                   dwSize;                            
    DWORD                   dwToken;                            // master slave group token, get from the EM_MSGROUP_OPERATE_OPEN operate type
    NET_RECT                stuRect;                            // the selected rectangle coordinates
    int                     nRectDirection;                     // the rectangle direction, 0-from top left to bottom right, zoom in; 1-from bottom right to top left, zoom out
}NET_IN_MSGROUP_RECTLOCATE_INFO;

// MasterSlaveGroup operation type EM_MSGROUP_OPERATE_RECTLOCATE output
typedef struct tagNET_OUT_MSGROUP_RECTLOCATE_INFO
{
    DWORD                   dwSize;                             
    int                     nPTZ[3];                            // PTZ
}NET_OUT_MSGROUP_RECTLOCATE_INFO;

// master slave group notify structure
typedef struct tagNET_MSGROUP_NOTIFY_INFO 
{
    DWORD                    dwSubscribeID;                     // subscribe id
    int                      nSlaveID;                          // slave device id
    char                     szClass[MASTERSALVE_CLASS_LEN];    // object type
    DWORD                    dwObjectID;                        // object id
    NET_RECT                 stuBoundingBox;                    // bounding box of tracking object
    BYTE                     byReserved[512];                   // reserved
}NET_MSGROUP_NOTIFY_INFO;

// notify callback function, lAttachHandle is the return value of CLIENT_AttachMasterSlaveGroup interface
typedef void (CALLBACK *fMasterSlaveCallBack)(LLONG lAttachHandle, NET_MSGROUP_NOTIFY_INFO* pBuf, DWORD dwBufLen, LDWORD dwUser);

// CLIENT_AttachMasterSlaveGroup input
typedef struct tagNET_IN_MSGROUP_ATTACH_INFO
{
    DWORD                    dwSize;                    
    fMasterSlaveCallBack     cbNotify;                          // callback function
    LDWORD                   dwUser;                            // user information
    int                      nGroupID;                          // device group id
}NET_IN_MSGROUP_ATTACH_INFO;

// CLIENT_AttachMasterSlaveGroup output
typedef struct tagNET_OUT_MSGROUP_ATTACH_INFO
{
    DWORD                    dwSize;               
}NET_OUT_MSGROUP_ATTACH_INFO;

// CLIENT_OperateMasterSlaveGroup operation type
typedef enum tagEM_MSGROUP_OPERATE_TYPE
{
    EM_MSGROUP_OPERATE_OPEN,                                    // Open operating handle, corresponding to  NET_IN_MSGROUP_OPEN_INFO and NET_OUT_MSGROUP_OPEN_INFO
    EM_MSGROUP_OPERATE_LOCATE,                                  // Locate master device and link the slave device, corresponding to NET_IN_MSGROUP_LOCATE_INFO and NET_OUT_MSGROUP_LOCATE_INTO
    EM_MSGROUP_OPERATE_TRACK,                                   // Do tracking, corresponding to NET_IN_MSGROUP_TRACK_INFO and NET_OUT_MSGROUP_TRACK_INFO
    EM_MSGROUP_OPERATE_CLOSE,                                   // Close the operation, corresponding to NET_IN_MSGROUP_CLOSE_INFO and NET_OUT_MSGROUP_CLOSE_INFO
    EM_MSGROUP_OPERATE_RECTLOCATE,                              // Select the rectangular box from the main device linkage from the slave, corresponding to NET_IN_MSGROUP_RECTLOCATE_INFO and NET_OUT_MSGROUP_RECTLOCATE_INFO
}EM_MSGROUP_OPERATE_TYPE;

// Master slave group operation interface,user malloc memory of pInParam and pOutParam,please refer to corresponding structure of emOperateType
CLIENT_NET_API BOOL CALL_METHOD CLIENT_OperateMasterSlaveGroup(LLONG lLoginID, EM_MSGROUP_OPERATE_TYPE emOperateType, void* pInParam, void* pOutParam, int nWaitTime = NET_INTERFACE_DEFAULT_TIMEOUT);

// Attach Master slave group interface,user malloc memory of pInParam and pOutParam
CLIENT_NET_API LLONG CALL_METHOD CLIENT_AttachMasterSlaveGroup(LLONG lLoginID, const NET_IN_MSGROUP_ATTACH_INFO *pInParam, NET_OUT_MSGROUP_ATTACH_INFO *pOutParam, int nWaitTime = NET_INTERFACE_DEFAULT_TIMEOUT);

// Detach Master slave group interface
CLIENT_NET_API BOOL CALL_METHOD CLIENT_DetachMasterSlaveGroup(LLONG lAttachHandle);
//////////////////////////////////////////////////////////////////////////

#define     MAX_TRAFFIC_LANE_NUM            16                  // max lane number
#define     MAX_CAMERA_PERIPHERAL_NUM       8                   // max peripherals of camera
#define     MAX_STORAGE_NUM                 8                   // max storage number
#define     MAX_PARTITION_NUM               8                   // max partition of storage

// partition information
typedef struct tagNET_PARTITION_INFO
{
    double                      dbTotalBytes;                   // total bytes of partition
    double                      dbUsedBytes;                    // used bytes of partition
    BOOL                        bError;                         // is partition error
    BYTE                        byReserved[64];                 // reserved
}NET_PARTITION_INFO;

// storage work state
typedef enum tagENUM_STORAGE_STATE
{
    ENUM_STORAGE_STATE_UNKONWN,                                 // unknown
    ENUM_STORGAE_STATE_READ_ERROR,                              // do read error
    ENUM_STORAGE_STATE_INITIALIZING,                            // initializing
    ENUM_STORAGE_STATE_READ_SUCCESS,                            // do read success
}ENUM_STORAGE_STATE;

// storage Device information
typedef struct tagNET_STORAGE_INFO
{
    ENUM_STORAGE_STATE       emState;                           // storage work state
    int                      nPartitonNum;                      // partition numbers
    NET_PARTITION_INFO       stuPartions[MAX_PARTITION_NUM];    // partition information
    BYTE                     byReserved[128];                   // reserved
}NET_STORAGE_INFO;

// network resource
typedef struct tagNET_RESOURCE_STATE
{
	int						nIPChanneIn;					// IP channel access speed, unit:kbps
	int						nNetRemain;						// network recevied residual capability, unit:kbps
	int						nNetCapability;					// total network recevied capability, unit:kbps
	int						nRemotePreview;					// remote preview capability, unit:kbps
	int						nRmtPlayDownload;				// remote playback and download capability, unit:kbps
	int						nRemoteSendRemain;				// remote send residual capability, unit:kbps
	int						nRemoteSendCapability;			// total remote send capability, unit:kbps
	BYTE                    byReserved[32];                 // reserved
} NET_RESOURCE_STATE;

// device work state
typedef struct tagNET_WORKSTATE
{
    BOOL                     bOnline;                           // is device online 
    char                     szFirmwareVersion[DH_COMMON_STRING_128];  // firmware version
    float                    fTemperature;                      // temperature, unit: centigrade
    float                    fPowerDissipation;                 // power dissipation, unit: W
    int                      nUtilizationOfCPU;                 // utilization of CPU
    
    int                      nStorageNum;                       // storage number of device
    NET_STORAGE_INFO         stuStorages[MAX_STORAGE_NUM];      // storage information
    int					 	 nUpTimeLast;						// last poewr up time, unit:second
    int					 	 nUpTimeTotal;						// total power up time, unit:second
    double					 dbMemInfoTotal;					// total memory size, unit:byte
    double					 dbMemInfoFree;						// free momery size, unit:byte
    BYTE					 byReserved1[4];					// keep align, not reserved
	char					 szDevType[32];						// device type
    NET_RESOURCE_STATE		 stuResourceStat;					// network resource state
    BYTE                     byReserved[8];                     // reserved
}NET_WORKSTATE;

// get device information, type DH_DEVSTATE_GET_WORK_STATE
typedef struct tagNET_QUERY_WORK_STATE
{
    DWORD                    dwSize;                            // size of structure
    NET_WORKSTATE            stuWorkState;                      // working state
}NET_QUERY_WORK_STATE;

// input parameter of get capsule lock state 
typedef struct tagNET_IN_QUERY_CAPSULE_LOCKSTATE
{
    DWORD                    dwSize;                            // size of structure
    int                      nChannel;                          // channel id
}NET_IN_QUERY_CAPSULE_LOCKSTATE;

// on line state
typedef enum tagNET_EM_ONLINESTATE
{
	NET_EM_ONLINESTATE_UNKNOWN,									 // unknown
    NET_EM_ONLINESTATE_ONLINE,                                   // online
    NET_EM_ONLINESTATE_OFFLINE,                                  // offfline
}NET_EM_ONLINESTATE;

// Lock state
typedef enum  tagNET_EM_LOCKSTATE
{
    NET_EM_LOCKSTATE_UNKNOWN,                                   // unknown
    NET_EM_LOCKSTATE_Open,                                      // open
    NET_EM_LOCKSTATE_CLOSE,                                     // close
    NET_EM_LOCKSTATE_ABNORMAL,                                  // abnormal
    NET_EM_LOCKSTATE_FAKELOCKED,                                // fakelocked
}NET_EM_LOCKSTATE;

// output parameter of get capsule lock state  
typedef struct tagNET_OUT_QUERY_CAPSULE_LOCKSTATE
{
    DWORD                       dwSize;                         // size of structure
    NET_EM_LOCKSTATE            emLockState;                    // lock state
    NET_EM_ONLINESTATE			emOnLineState;					// on line state
}NET_OUT_QUERY_CAPSULE_LOCKSTATE;

// input parameter(get human number in capsule)
typedef struct tagNET_IN_QUERY_CAPSULE_HUMANNUM
{
    DWORD                       dwSize;                         // size of structure
    int                         nChannel;                       // channel id
}NET_IN_QUERY_CAPSULE_HUMANNUM;

// output parameter(get human number in capsule)
typedef struct tagNET_OUT_QUERY_CAPSULE_HUMANNUM
{
    DWORD                       dwSize;                         // size of structure
    UINT                        nHumanNum;                      // human number
}NET_OUT_QUERY_CAPSULE_HUMANNUM;

// record mode
typedef enum tagNET_EM_RECORD_MODE
{
    NET_EM_RECORD_MODE_UNKNOWN,                                 // unknown
    NET_EM_RECORD_MODE_COURSE,                                  // course record
    NET_EM_RECORD_MODE_NORMAL,                                  // normal
}NET_EM_RECORD_MODE;

// In parameter, accrossoing interface CLIENT_SetCourseRecordMode
typedef struct tagNET_IN_SET_COURSE_RECORD_MODE
{
    DWORD                       dwSize;                         
    NET_EM_RECORD_MODE          emRecordMode;                   // record mode
    int                         nClassRoomID;                   // classroom id
}NET_IN_SET_COURSE_RECORD_MODE;

// out parameter,accrossoing interface CLIENT_SetCourseRecordMode
typedef struct tagNET_OUT_SET_COURSE_RECORD_MODE
{
    DWORD                       dwSize;                         
}NET_OUT_SET_COURSE_RECORD_MODE;

// In parameter, accrossoing interface CLIENT_GetCourseRecordMode
typedef struct tagNET_IN_GET_COURSE_RECORD_MODE
{
    DWORD                       dwSize;  
    int                         nClassRoomID;                   // classroom id
}NET_IN_GET_COURSE_RECORD_MODE;

// out parameter,accrossoing interface CLIENT_GetCourseRecordMode
typedef struct tagNET_OUT_GET_COURSE_RECORD_MODE
{
    DWORD                       dwSize; 
    NET_EM_RECORD_MODE          emRecordMode;                   // record mode
}NET_OUT_GET_COURSE_RECORD_MODE;

// device common information
typedef struct tagNET_DEVICE_COMMON_INFO
{
    char                     szVendor[DH_COMMON_STRING_32];     // vendor
    char                     szDviceType[DH_COMMON_STRING_32];  // device type
    char                     szVersion[DH_COMMON_STRING_128];   // device version
    char                     szSerialNo[DH_COMMON_STRING_128];  // serial No.
    BYTE                     byReserved[128];                   // reserved
}NET_DEVICE_COMMON_INFO;

// vehicle detector
typedef struct tagNET_VEHICLE_DETECTOR_INFO
{
    NET_DEVICE_COMMON_INFO   stuDevice;                         // device information
    int                      nWokingState;                      // working state: 0 - fault, 1 - normal, 2 - close 
    int                      nMatchState;                       // coil matching state: 0 - no match, 1 - match, 2 - no detect
    int                      nLaneNum;                          // lane number
    int                      nUsageOfLane[MAX_TRAFFIC_LANE_NUM];// usage of each lane, 0-100
    BYTE                     byReserved[128];                   // reserved
}NET_VEHICLE_DETECTOR_INFO;

// signal detector
typedef struct tagNET_SIGNAL_DETECTOR_INFO 
{
    NET_DEVICE_COMMON_INFO   stuDevice;                         // device information
    int                      nWokingState;                      // working state: 0 - fault, 1 - normal, 2 - close 
    int                      nMatchState;                       // coil matching state: 0 - no match, 1 - match, 2 - no detect
    BYTE                     byReserved[128];                   // reserved
}NET_SIGNAL_DETECTOR_INFO;

// stroboscopic lamp information
typedef struct tagNET_STROBOSCOPIC_LAMP_INFO
{
    NET_DEVICE_COMMON_INFO   stuDevice;                         // device information
    int                      nWokingState;                      // working state: 0 - fault, 1 - normal, 2 - close
    int                      nMatchState;                       // coil matching state: 0 - no match, 1 - match, 2 - no detect
    BYTE                     byReserved[128];                   // reserved
}NET_STROBOSCOPIC_LAMP_INFO;

// RS485 stroboscopic lamp information
typedef struct tagNET_RS485_STROBOSCOPIC_LAMP_INFO
{
    NET_DEVICE_COMMON_INFO   stuDevice;                         // device information
    int                      nWokingState;                      // working state: 0- fault, 1- normal, 2- close
    int                      nMatchState;                       // coil matching state: 0- no match, 1- match, 2- no detect

    int                      nPeripheralAddress;                // address of peripheral
    int                      nPeripheralStateCode;              // peripheral fault code: 1-no response, 2-lamp fault, 3-not connected to storbe signal     
    int                      nLuminance;                        // luminance, 1-20
    int                      nSensibility;                      // sensibility, 1-255
    float                    fPowerDissipation;                 // dissipation of power

    BYTE                     byReserved[128];                   // reserved
}NET_RS485_STROBOSCOPIC_LAMP_INFO;

// flash lamp information
typedef struct tagNET_FLASH_LAMP_INFO
{
    NET_DEVICE_COMMON_INFO   stuDevice;                         // device information
    int                      nWokingState;                      // working state: 0- fault, 1- normal, 2- close
    int                      nMatchState;                       // coil matching state: 0- no match, 1- match, 2- no detect
    BYTE                     byReserved[128];                   // reserved
}NET_FLASH_LAMP_INFO;

// RS485 flash lamp information
typedef struct tagNET_RS485_FLASH_LAMP_INFO
{
    NET_DEVICE_COMMON_INFO   stuDevice;                         // device information
    int                      nWokingState;                      // working state: 0- fault, 1- normal, 2- close
    int                      nMatchState;                       // coil matching state: 0- no match, 1- match, 2- no detect
    
    int                      nPeripheralAddress;                // address of peripheral
    int                      nPeripheralStateCode;              // peripheral fault code: 1-no response, 2-lamp fault, 3-not connected to storbe signal          
    int                      nLuminance;                        // luminance, 1-20
    int                      nSensibility;                      // sensibility, 1-255
    float                    fPowerDissipation;                 // dissipation of power
    
    BYTE                     byReserved[128];                   // reserved
}NET_RS485_FLASH_LAMP_INFO;

// RS485 steady lamp information
typedef struct tagNET_RS485_STEADY_LAMP_INFO
{
    NET_DEVICE_COMMON_INFO   stuDevice;                         // device information
    int                      nWokingState;                      // working state: 0- fault, 1- normal, 2- close
    int                      nMatchState;                       // coil matching state: 0- no match, 1- match, 2- no detect
    
    int                      nPeripheralAddress;                // address of peripheral
    int                      nPeripheralStateCode;              // peripheral fault code: 1-no response, 2-lamp fault, 3-not connected to storbe signal    
    int                      nLuminance;                        // luminance, 1-20
    int                      nSensibility;                      // sensibility, 1-255
    float                    fPowerDissipation;                 // dissipation of power
    
    BYTE                     byReserved[128];                   // reserved
}NET_RS485_STEADY_LAMP_INFO;

// camera peripheral information
typedef struct tagNET_CAMERA_PERIPHERAL_INFO
{
    int                         nVehicleDetectorNum;                                // amount of vehicle detector
    NET_VEHICLE_DETECTOR_INFO   stuVehicleDetectors[MAX_CAMERA_PERIPHERAL_NUM];     // vehicle detector information

    int                         nSignalDetectorNum;                                 // amount of signal detector
    NET_SIGNAL_DETECTOR_INFO    stuSignalDetectors[MAX_CAMERA_PERIPHERAL_NUM];      // signal detector information
    
    int                         nStroboscopicLampNum;                               // amount of stroboscopic lamp
    NET_STROBOSCOPIC_LAMP_INFO  stuStroboscopicLamps[MAX_CAMERA_PERIPHERAL_NUM];    // stroboscopic lamp information

    int                         nFlashLampNum;                                      // amount of flash lamp
    NET_FLASH_LAMP_INFO         stuFlashLamps[MAX_CAMERA_PERIPHERAL_NUM];           // flash lamp information

    int                         nRS485StroboscopicLampNum;                          // amount of RS485 flash lamp
    NET_RS485_STROBOSCOPIC_LAMP_INFO    stuRS485StroboscopicLamps[MAX_CAMERA_PERIPHERAL_NUM]; // RS485 flash lamp information
    
    int                         nRS485FlashLampNum;                                 // amount of RS485 flash lamp
    NET_RS485_FLASH_LAMP_INFO   stuRS485FlashLamps[MAX_CAMERA_PERIPHERAL_NUM];      // RS485 flash lamp

    int                         nRS485SteadyLampNum;                                // amount of RS485 steady lamp
    NET_RS485_STEADY_LAMP_INFO  stuRS485SteadyLamps[MAX_CAMERA_PERIPHERAL_NUM];     // RS485 Steady lamp

    BYTE                        byReserved[128];                                    // reserved
}NET_CAMERA_PERIPHERAL_INFO;

// camera work state 
typedef struct tagNET_REMOTECAMERA_NOTIFY_INFO
{
    int                         nChannelID;                     // channel id
    NET_WORKSTATE               stuNative;                      // camera work state
    NET_CAMERA_PERIPHERAL_INFO  stuPeripherals;                 // all peripherals's information
    BYTE                        byReserved[128];                // reserved
}NET_REMOTECAMERA_NOTIFY_INFO;

// camera work state callback function, lAttachHandle is the result of CLIENT_AttachRemoteCameraState 
typedef void (CALLBACK *fRemoteCameraStateCallBack)(LLONG lAttachHandle, NET_REMOTECAMERA_NOTIFY_INFO* pstuState, DWORD dwStateSize, LDWORD dwUser);

// CLIENT_AttachRemoteCameraState input
typedef struct tagNET_IN_REMOTECAMERA_ATTACH_INFO
{
    DWORD                       dwSize;                         // size of structure
    fRemoteCameraStateCallBack  cbNotify;                       // callback function
    LDWORD                      dwUser;                         // user information
}NET_IN_REMOTECAMERA_ATTACH_INFO;

// CLIENT_AttachRemoteCameraState output
typedef struct tagNET_OUT_REMOTECAMERA_ATTACH_INFO
{
    DWORD                    dwSize;                            // size of structure
}NET_OUT_REMOTECAMERA_ATTACH_INFO;

// attach remote camera state,user malloc memory of pInParam and pOutParam
CLIENT_NET_API LLONG CALL_METHOD CLIENT_AttachRemoteCameraState(LLONG lLoginID, const NET_IN_REMOTECAMERA_ATTACH_INFO *pInParam, NET_OUT_REMOTECAMERA_ATTACH_INFO *pOutParam, int nWaitTime);

// detach remote camera state
CLIENT_NET_API BOOL CALL_METHOD CLIENT_DetachRemoteCameraState(LLONG lAttachHandle);

// Patrol status
typedef enum tagNET_EM_PATROL_STATUS
{
    NET_EM_PATROL_STATUS_UNKNOWN,                               // Unknown
    NET_EM_PATROL_STATUS_BEGIN,                                 // Patrol start
    NET_EM_PATROL_STATUS_END,                                   // Patrol end
    NET_EM_PATROL_STATUS_FAIL,                                  // Patrol fail
}NET_EM_PATROL_STATUS;

// CLIENT_SendNotifyToDev inparam (corresponding to NET_EM_NOTIFY_PATROL_STATUS)
typedef struct tagNET_IN_PATROL_STATUS_INFO
{
    DWORD                       dwSize;                         // size of the structure
    NET_EM_PATROL_STATUS        emPatrolStatus;                 // Patrol status
    
}NET_IN_PATROL_STATUS_INFO;

// CLIENT_SendNotifyToDev outparam (corresponding to NET_EM_NOTIFY_PATROL_STATUS)
typedef struct tagNET_OUT_PATROL_STATUS_INFO
{
    DWORD                    dwSize;                            // size of the structure
}NET_OUT_PATROL_STATUS_INFO;

// Type of notify, corresponding to CLIENT_SendNotifyToDev 
typedef enum tagNET_EM_NOTIFY_TYPE
{
    NET_EM_NOTIFY_PATROL_STATUS = 1,                            // Send patrol status (corresponding to NET_IN_PATROL_STATUS_INFO, NET_OUT_PATROL_STATUS_INFO)
}NET_EM_NOTIFY_TYPE;

// Send notify to device interface,  pInParam :user allocate memory,  pOutParam :user allocate memory ,please refer to corresponding structure of emNotifyType
CLIENT_NET_API BOOL CALL_METHOD CLIENT_SendNotifyToDev(LLONG lLoginID, NET_EM_NOTIFY_TYPE emNotifyType, const void* pInParam, void* pOutParam, int nWaitTime = NET_INTERFACE_DEFAULT_TIMEOUT );

// heat map raw stream information
typedef struct tagNET_RAWSTREAM_NOTIFY_INFO 
{
    int                      nWidth;                            // width of picture
    int                      nHeight;                           // height of picture
    NET_TIME                 stuStartTime;                      // start time
    NET_TIME                 stuEndTime;                        // end time
    char*                    pStream;                           // stream
    int                      nLength;                           // length of raw stream
    BYTE                     byReserved[512];                   // reserved
}NET_RAWSTREAM_NOTIFY_INFO;

// heat map raw stream callback function, lAttachHandle is the result of CLIENT_AttachHeatMapRawStream 
typedef void (CALLBACK *fRawStreamCallBack)(LLONG lAttachHandle, NET_RAWSTREAM_NOTIFY_INFO* pBuf, DWORD dwBufLen, LDWORD dwUser);

// CLIENT_AttachHeatMapRawStream input
typedef struct tagNET_IN_RAWSTREAM_ATTACH_INFO
{
    DWORD                    dwSize;                            // size of structure
    fRawStreamCallBack       cbNotify;                          // callback function
    LDWORD                   dwUser;                            // user information
    int                      nChannel;                          // channel id
}NET_IN_RAWSTREAM_ATTACH_INFO;

// CLIENT_AttachHeatMapRawStream output
typedef struct tagNET_OUT_RAWSTREAM_ATTACH_INFO
{
    DWORD                    dwSize;                            // size of structure
}NET_OUT_RAWSTREAM_ATTACH_INFO;

// attach heat map raw stream,user malloc memory of pInParam and pOutParam
CLIENT_NET_API LLONG CALL_METHOD CLIENT_AttachHeatMapRawStream(LLONG lLoginID, const NET_IN_RAWSTREAM_ATTACH_INFO *pInParam, NET_OUT_RAWSTREAM_ATTACH_INFO *pOutParam, int nWaitTime);

// detach heat map raw stream 
CLIENT_NET_API BOOL CALL_METHOD CLIENT_DetachHeatMapRawStream(LLONG lAttachHandle);

//plate info
typedef struct tagNET_ANALYSERESULT
{
    char                      szPlateNumber[64];                // plate number
    NET_RECT                  stuRect;                          // plate boundingBox coordinate,absolute coordinate
    BYTE                      byReserved[128];                  // reserved 
}NET_ANALYSERESULT;

// CLIENT_SetIVSServerAnalyseResult input
typedef struct tagNET_IN_SET_ANALYSERESULT_INFO
{
    DWORD                     dwSize;                           // 
    int                       nChannel;                         // channel id,from 0
    int                       nAnalyseResultCount;              // count of plate analyse result
    NET_ANALYSERESULT         stuAnalyseResult[MAX_PLATE_NUM];  // plate info
	unsigned int			  nFrameSequence;					// frame sequence, Used to identify with the IVS,Required
	unsigned int			  nPhysicalLane;					// Physical parking space
}NET_IN_SET_ANALYSERESULT_INFO;

// CLIENT_SetIVSServerAnalyseResult output
typedef struct tagNET_OUT_SET_ANALYSERESULT_INFO
{
    DWORD                     dwSize;                           
}NET_OUT_SET_ANALYSERESULT_INFO;

// CLIENT_AttachEventRestore input
typedef struct tagNET_IN_ATTACH_EVENT_RESTORE
{
    DWORD                dwSize;															// size of structure
	char 				 szUuid[MAX_EVENT_RESTORE_UUID];									// client identifier
}NET_IN_ATTACH_EVENT_RESTORE;

// IVSServer set Analyseresult,use with event of EVENT_IVS_TRAFFIC_ANALYSE_PRESNAP,user malloc and free memory of pInBuf and pOutBuf
CLIENT_NET_API BOOL CALL_METHOD CLIENT_SetIVSServerAnalyseResult(LLONG lLoginID, const NET_IN_SET_ANALYSERESULT_INFO* pInBuf,NET_OUT_SET_ANALYSERESULT_INFO* pOutBuf,int nWaitTime);

// Get state of capsulelock,user malloc and free memory of pInBuf and pOutBuf
CLIENT_NET_API BOOL CALL_METHOD CLIENT_GetCapsuleLockState(LLONG lLoginID, const NET_IN_QUERY_CAPSULE_LOCKSTATE* pInBuf,NET_OUT_QUERY_CAPSULE_LOCKSTATE* pOutBuf, int nWaitTime);

// get human number in capsule,user malloc and free memory of pInBuf and pOutBuf
CLIENT_NET_API BOOL CALL_METHOD CLIENT_GetCapsuleHumanNum(LLONG lLoginID, const NET_IN_QUERY_CAPSULE_HUMANNUM* pInBuf, NET_OUT_QUERY_CAPSULE_HUMANNUM* pOutBuf, int nWaitTime);

// set course record mode,user malloc and free memory of pInBuf and pOutBuf
CLIENT_NET_API BOOL CALL_METHOD CLIENT_SetCourseRecordMode(LLONG lLoginID, const NET_IN_SET_COURSE_RECORD_MODE* pInBuf, NET_OUT_SET_COURSE_RECORD_MODE* pOutBuf, int nWaitTime);

// get course record mode,user malloc and free memory of pInBuf and pOutBuf
CLIENT_NET_API BOOL CALL_METHOD CLIENT_GetCourseRecordMode(LLONG lLoginID, const NET_IN_GET_COURSE_RECORD_MODE* pInBuf, NET_OUT_GET_COURSE_RECORD_MODE* pOutBuf, int nWaitTime);

// logic channel
typedef enum tagNET_EM_LOGIC_CHANNEL
{
    NET_EM_LOGIC_CHANNEL_INVALID = -1,                          // invalid
    NET_EM_LOGIC_CHANNEL_COMPOSITE,                             // composite channel id
    NET_EM_LOGIC_CHANNEL_PPT,                                   // PPT logic channel id
    NET_EM_LOGIC_CHANNEL_BLACKBOARD,                            // blackboard logic channel id
    NET_EM_LOGIC_CHANNEL_STUDENTFEATURE,                        // student feature logic channel id
    NET_EM_LOGIC_CHANNEL_STUDENTFULLVIEW,                       // student fullview logic channel id
    NET_EM_LOGIC_CHANNEL_TEACHERFEATURE,                        // teacher feature logic channel id
    NET_EM_LOGIC_CHANNEL_TEACHERFULLVIEW,                       // teacher fullview logic channel id
    NET_EM_LOGIC_CHANNEL_TEACHERDETECT,                         // teacher detect logic channel id
    NET_EM_LOGIC_CHANNEL_BLACKBOARDDETECT,                      // blackboard detect logic channel id
}NET_EM_LOGIC_CHANNEL;

// channel info
typedef struct  tagNET_LOGIC_CHANNEL_INFO
{
    int                         nRoomID;                        // classroom id
    NET_EM_LOGIC_CHANNEL        emLogicChannel;                 // logic channel id
    BYTE                        byReserved[32];                 // reserved
}NET_LOGIC_CHANNEL_INFO;

// in parameter, accrossoing interface CLIENT_GetRealPreviewChannel
typedef struct tagNET_IN_GET_REAL_PREVIEW_CHANNEL
{
    DWORD                       dwSize;
    int                         nChannelCount;                  // channel count
    NET_LOGIC_CHANNEL_INFO      stuChannelInfo[MAX_PREVIEW_CHANNEL_NUM];          // logicchannel info
}NET_IN_GET_REAL_PREVIEW_CHANNEL;

// out parameter, accrossoing interface CLIENT_GetRealPreviewChannel
typedef struct tagNET_OUT_GET_REAL_PREVIEW_CHANNEL
{
    DWORD                    dwSize;
    int                      nChannelNum;                       // channel number
    int                      nChannel[MAX_PREVIEW_CHANNEL_NUM]; // channel id
}NET_OUT_GET_REAL_PREVIEW_CHANNEL;

// get course preview channel,user malloc and free memory of pInBuf and pOutBuf
CLIENT_NET_API BOOL CALL_METHOD CLIENT_GetRealPreviewChannel(LLONG lLoginID, const NET_IN_GET_REAL_PREVIEW_CHANNEL* pInBuf, NET_OUT_GET_REAL_PREVIEW_CHANNEL* pOutBuf, int nWaitTime);

// in parameter, accrossoing interface CLIENT_GetDefaultRealChannel
typedef struct tagNET_IN_GET_DEFAULT_REAL_CHANNEL
{
    DWORD                       dwSize; 
    int                         nChannelCount;                                       // channel count
    NET_LOGIC_CHANNEL_INFO      stuChannelInfo[MAX_PREVIEW_CHANNEL_NUM];             // logic channel info
}NET_IN_GET_DEFAULT_REAL_CHANNEL;

// out parameter, accrossoing interface CLIENT_GetDefaultRealChannel
typedef struct tagNET_OUT_GET_DEFAULT_REAL_CHANNEL
{
    DWORD                    dwSize;
    int                      nChannelNum;                                           // channel count
    int                      nChannel[MAX_PREVIEW_CHANNEL_NUM];                     // channel id
}NET_OUT_GET_DEFAULT_REAL_CHANNEL;

// get course media default real channel id,user malloc and free memory of pInBuf and pOutBuf
CLIENT_NET_API BOOL CALL_METHOD CLIENT_GetDefaultRealChannel(LLONG lLoginID, const NET_IN_GET_DEFAULT_REAL_CHANNEL* pInBuf, NET_OUT_GET_DEFAULT_REAL_CHANNEL* pOutBuf, int nWaitTime);

// in parameter, accrossoing interface CLIENT_GetLogicChannel
typedef struct tagNET_IN_GET_COURSE_LOGIC_CHANNEL
{
    DWORD                       dwSize; 
    int                         nChannelNum;                       // channel count
    int                         nChannel[MAX_PREVIEW_CHANNEL_NUM]; // channel id
}NET_IN_GET_COURSE_LOGIC_CHANNEL;

// out parameter, accrossoing interface CLIENT_GetLogicChannel
typedef struct tagNET_OUT_GET_COURSE_LOGIC_CHANNEL
{
    DWORD                       dwSize;
    int                         nChannelCount;                                          // logic channel count 
    NET_LOGIC_CHANNEL_INFO      stuChannelInfo[MAX_PREVIEW_CHANNEL_NUM];                // logic channel info
}NET_OUT_GET_COURSE_LOGIC_CHANNEL;

// get course media logic channel id,user malloc and free memory of pInBuf and pOutBuf
CLIENT_NET_API BOOL CALL_METHOD CLIENT_GetLogicChannel(LLONG lLoginID, const NET_IN_GET_COURSE_LOGIC_CHANNEL* pInBuf,NET_OUT_GET_COURSE_LOGIC_CHANNEL* pOutBuf, int nWaitTime);

// out parameter, accrossoing interface CLIENT_SetBlindRealChannel
typedef struct tagNET_IN_SET_BLIND_REAL_CHANNEL
{
    DWORD                       dwSize; 
    int                         nChannelNum;                                            // channel count
    int                         nChannel[MAX_PREVIEW_CHANNEL_NUM];                      // channel id
    NET_LOGIC_CHANNEL_INFO      stuChannelInfo[MAX_PREVIEW_CHANNEL_NUM];                // logic channel info
}NET_IN_SET_BLIND_REAL_CHANNEL;

// out parameter, accrossoing interface CLIENT_SetBlindRealChannel
typedef struct tagNET_OUT_SET_BLIND_REAL_CHANNEL
{
    DWORD                       dwSize; 
}NET_OUT_SET_BLIND_REAL_CHANNEL;

// set blind of Real and logic channel ,user malloc and free memory of pInBuf and pOutBuf
CLIENT_NET_API BOOL CALL_METHOD CLIENT_SetBlindRealChannel(LLONG lLoginID, const NET_IN_SET_BLIND_REAL_CHANNEL* pInBuf,NET_OUT_SET_BLIND_REAL_CHANNEL* pOutBuf, int nWaitTime);

// out parameter, accrossoing interface CLIENT_GetInputChannelMedia
typedef struct tagNET_IN_GET_INPUT_CHANNEL_MEDIA
{
    DWORD                           dwSize; 
    int                             nChannelNum;                       // channel count
    int                             nChannel[MAX_PREVIEW_CHANNEL_NUM]; // channel id
}NET_IN_GET_INPUT_CHANNEL_MEDIA;

// input media
typedef enum tagNET_ENUM_INPUT_CHANNEL_MEDIA
{
    NET_ENUM_INPUT_MEDIA_UNKNOWN,                                       // unknown
    NET_ENUM_INPUT_MEDIA_VGA,                                           // VGA
    NET_ENUM_INPUT_MEDIA_HDMI,                                          // HDMI
}NET_ENUM_INPUT_CHANNEL_MEDIA;

// out parameter, accrossoing interface CLIENT_GetInputChannelMedia
typedef struct tagNET_OUT_GET_INPUT_CHANNEL_MEDIA
{
    DWORD                           dwSize;
    int                             nChannelNum;                              // channel count
    NET_ENUM_INPUT_CHANNEL_MEDIA    emInputMedia[MAX_PREVIEW_CHANNEL_NUM];    // input media 
}NET_OUT_GET_INPUT_CHANNEL_MEDIA;

// get course media input type,user malloc and free memory of pInBuf and pOutBuf
CLIENT_NET_API BOOL CALL_METHOD CLIENT_GetInputChannelMedia(LLONG lLoginID, const NET_IN_GET_INPUT_CHANNEL_MEDIA* pInBuf,NET_OUT_GET_INPUT_CHANNEL_MEDIA* pOutBuf, int nWaitTime);

//attach event restore,user malloc and free memory of pInParam 
CLIENT_NET_API LLONG CALL_METHOD CLIENT_AttachEventRestore(LLONG lLoginID, const NET_IN_ATTACH_EVENT_RESTORE *pInParam, int nWaitTime);

// detach event restore ,user malloc and free memory of pInParam
CLIENT_NET_API BOOL CALL_METHOD CLIENT_DetachEventRestore(LLONG lAttachHandle);

// CLIENT_GetLaserDistance input
typedef struct tagNET_IN_GET_LASER_DISTANCE
{
    DWORD                     dwSize;                           // size of structure
    int                       nChannel;                         // channel id,from 0
}NET_IN_GET_LASER_DISTANCE;

// CLIENT_GetLaserDistance output
typedef struct tagNET_OUT_GET_LASER_DISTANCE
{
    DWORD                     dwSize;                           // size of structure
    UINT                      unDistance;                       // Distance   unit:meter
}NET_OUT_GET_LASER_DISTANCE;

// get the distance which object in the center of picture,user malloc and free memory of pInBuf and pOutBuf
CLIENT_NET_API BOOL CALL_METHOD CLIENT_GetLaserDistance(LLONG lLoginID, const NET_IN_GET_LASER_DISTANCE* pInBuf, NET_OUT_GET_LASER_DISTANCE* pOutBuf, int nWaitTime = NET_INTERFACE_DEFAULT_TIMEOUT );

// CLIENT_GetAtomsphData input
typedef struct tagNET_IN_GET_ATOMSPHDATA
{
    DWORD                     dwSize;                           
}NET_IN_GET_ATOMSPHDATA;

// CLIENT_GetAtomsphData output
typedef struct tagNET_OUT_GET_ATOMSPHDATA
{
    DWORD                     dwSize;
    float                     fWindSpeed;                       // WindSpeed, unit m/s
    float                     fWindDirection;                   // WindDirection, unit degree
    float                     fHumidity;                        // Humidity, unit %
    float                     fTemperture;                      // Temperture, unit degree Celsius
    float                     fAirPressure;                     // AirPressure, unit hPa
    float                     fSunBathe;                        // SunBathe, unit w/m2
}NET_OUT_GET_ATOMSPHDATA;

// get atomsphere data,user malloc and free memory of pInBuf and pOutBuf
CLIENT_NET_API BOOL CALL_METHOD CLIENT_GetAtomsphData(LLONG lLoginID, const NET_IN_GET_ATOMSPHDATA* pInBuf, NET_OUT_GET_ATOMSPHDATA* pOutBuf,int nWaitTime);

// enable/disable designated channel video input parameter
typedef struct tagNET_IN_SET_COURSE_RECORD_STATE
{
    DWORD                    dwSize;                 // the structure size
    int                      nChannel;               // channel id
    int                      nAction;                // 0:enable 1:disable
}NET_IN_SET_COURSE_RECORD_STATE;

// enable/disable designated channel video out parameter
typedef struct tagNET_OUT_SET_COURSE_RECORD_STATE
{
    DWORD                    dwSize;                 // the structure size
}NET_OUT_SET_COURSE_RECORD_STATE;

// start or stop record,user malloc and free memory of pInBuf and pOutBuf
CLIENT_NET_API BOOL CALL_METHOD CLIENT_SetCourseRecordState(LLONG lLoginID, const NET_IN_SET_COURSE_RECORD_STATE *pInBuf, NET_OUT_SET_COURSE_RECORD_STATE *pOutBuf, int nWaitTime);

// open find coursemedia record in parameter
typedef struct tagNET_IN_QUERY_COURSEMEDIA_FILEOPEN
{
    DWORD                    dwSize;                            // the structure size
    NET_TIME                 stuStartTime;                      // start time
    NET_TIME                 stuEndTime;                        // end time
    char                     szKeyWord[DH_COMMON_STRING_128];   // key word
}NET_IN_QUERY_COURSEMEDIA_FILEOPEN;

// open find coursemedia record out parameter
typedef struct tagNET_OUT_QUERY_COURSEMEDIA_FILEOPEN
{
    DWORD                    dwSize;                            // the structure size
    UINT                     ntotalNum;                         // total num of resule
    UINT                     nfindID;                           // find id
}NET_OUT_QUERY_COURSEMEDIA_FILEOPEN;

// open find coursemedia record, user malloc and free memory of pInBuf and pOutBuf
CLIENT_NET_API BOOL CALL_METHOD CLIENT_OpenQueryCourseMediaFile(LLONG lLoginID, const NET_IN_QUERY_COURSEMEDIA_FILEOPEN *pInBuf, NET_OUT_QUERY_COURSEMEDIA_FILEOPEN *pOutBuf, int nWaitTime);

// find coursemedia record in parameter
typedef struct tagNET_IN_QUERY_COURSEMEDIA_FILE
{
    DWORD                    dwSize;                            // the structure size
    UINT                     nfindID;                           // find id
    int                      nOffset;                           // start find offset
    int                      nCount;                            // find count
}NET_IN_QUERY_COURSEMEDIA_FILE;

// record info
typedef struct tagNET_RECORD_INFO
{
    int                      nRealChannel;                                      // real channel id
    NET_TIME                 stuStartTime;                                      // start time
    NET_TIME                 stuEndTime;                                        // end time
    UINT                     nFileLen;                                          // record file lenth,low 32-bit,0:no record 
    UINT                     nFileLenEx;                                        // record file lenth,high 32-bit
    UINT                     nTime;                                             // record time unit:second
    UINT                     nFileType;                                         // file type 0,cut file 1, uncut file
    BYTE                     byReserved[64];                                    // reserved 
}NET_RECORD_INFO;

// course info
typedef struct tagNET_COURSE_INFO
{
    char                     szCourseName[DH_COMMON_STRING_64];                 // course name
    char                     szTeacherName[DH_COMMON_STRING_64];                // teacher name
    char                     szIntroduction[DH_COMMON_STRING_128];              // record introdution
    BYTE                     byReserved[64];                                    // reserved
}NET_COURSE_INFO;

// coursemedia info
typedef struct tagNET_COURSEMEDIA_FILE_INFO
{
    int                      nID;                                               // id,-1,no record
    NET_COURSE_INFO          stuCourseInfo;                                     // course info
    int                      nChannelNum;                                       // channel count
    int                      nRecordNum[64];                                    // record section num of logic channel 
    NET_RECORD_INFO          stuRecordInfo[64][16];                             // record info,first dimensional:logic channel id,second dimensional:record section num
    BYTE                     byReserved[128];                                   // reserve
}NET_COURSEMEDIA_FILE_INFO;

// find coursemedia record out parameter
typedef struct tagNET_OUT_QUERY_COURSEMEDIA_FILE
{
    DWORD                       dwSize;                                         // the structure size
    int                         nCountResult;                                   // resule num
    NET_COURSEMEDIA_FILE_INFO   stuCourseMediaFile[64];                         // record file info
}NET_OUT_QUERY_COURSEMEDIA_FILE;

// find coursemedia record,user malloc and free memory of pInBuf and pOutBuf
CLIENT_NET_API BOOL CALL_METHOD CLIENT_DoQueryCourseMediaFile(LLONG lLoginID, const NET_IN_QUERY_COURSEMEDIA_FILE *pInBuf, NET_OUT_QUERY_COURSEMEDIA_FILE *pOutBuf, int nWaitTime);

// close find coursemedia record in parameter
typedef struct tagNET_IN_QUERY_COURSEMEDIA_FILECLOSE
{
    DWORD                       dwSize;                                         // the structure size
    UINT                        nFindID;                                        // find id
}NET_IN_QUERY_COURSEMEDIA_FILECLOSE;

// close find coursemedia record out parameter
typedef struct tagNET_OUT_QUERY_COURSEMEDIA_FILECLOSE
{
    DWORD                       dwSize;                                         // the structure size
}NET_OUT_QUERY_COURSEMEDIA_FILECLOSE;

// close find coursemedia record,user malloc and free memory of pInBuf and pOutBuf
CLIENT_NET_API BOOL CALL_METHOD CLIENT_CloseQueryCourseMediaFile(LLONG lLoginID, const NET_IN_QUERY_COURSEMEDIA_FILECLOSE *pInBuf, NET_OUT_QUERY_COURSEMEDIA_FILECLOSE *pOutBuf, int nWaitTime);


// file transmit status
typedef enum tagNET_EM_AIO_FILE_STATUS
{
	NET_EM_AIO_FILE_STATUS_UNKNOWN,
	NET_EM_AIO_FILE_STATUS_BEGIN,				// Begin, 
	NET_EM_AIO_FILE_STATUS_UPLOADING,			// Uploading
	NET_EM_AIO_FILE_STATUS_END,					// End, 
}NET_EM_AIO_FILE_STATUS;

// file type
typedef enum tagNET_EM_AIO_FILE_TYPE
{
	NET_EM_AIO_FILE_TYPE_UNKNOWN,
	NET_EM_AIO_FILE_TYPE_UPSYSTEM,				// "upSystem"
	NET_EM_AIO_FILE_TYPE_UPAPP,					// "upApp"
}NET_EM_AIO_FILE_TYPE;

// CLIENT_UploadAIOFile's interface input param(upload file to the device)
typedef struct tagNET_IN_UPLOAD_AIO_FILE
{
    DWORD               dwSize;
    char*				pszFileSrc;                     // path of source file
    char*				pszFileDst;                     // path of goal file
    char*				pszFolderDst;                   // target file path: may be NULL, NULL device use default path
    unsigned int        nPacketLen;                     // file pack size(byte): 0 means not pack
	NET_EM_AIO_FILE_TYPE	emType;						// file type
} NET_IN_UPLOAD_AIO_FILE;

// CLIENT_UploadAIOFile's interface output param(upload file to the device)
typedef struct tagNET_OUT_UPLOAD_AIO_FILE
{
    DWORD               dwSize;
} NET_OUT_UPLOAD_AIO_FILE;

// fAttachBurnStateCB parameter
typedef struct tagNET_CB_AIOFILEPROC
{
    DWORD						dwSize;
    NET_EM_AIO_FILE_STATUS      emStatus;				// file transmit status
    NET_EM_AIO_FILE_TYPE		emType;					// file type

    unsigned int				dwLength;				// file size : unit byte
	char*						pbFileBuf;				// file buf 
	DWORD						dwBufSize;				// file buf size
}NET_CB_AIOFILEPROC;

// CLIENT_attachAIOFileProc() callback function,lAttachHandle is the return value of CLIENT_attachAIOFileProc
typedef void (CALLBACK *fAttachAIOFileprocCB) (LLONG lAttachHandle, NET_CB_AIOFILEPROC* pBuf, int nBufLen, LDWORD dwUser);

// CLIENT_attachAIOFileProc() input param
typedef struct tagNET_IN_ATTACH_AIO
{
    DWORD                   dwSize;
    fAttachAIOFileprocCB    cbNotify;						// callback function
    LDWORD                  dwUser;                         // user data
}NET_IN_ATTACH_AIO;

// CLIENT_attachAIOFileProc output param
typedef struct tagNET_OUT_ATTACH_AIO
{
    DWORD                   dwSize;
}NET_OUT_ATTACH_AIO;

// advert action
typedef enum tagNET_EM_ADVERT_ACTION
{
	NET_EM_ADVERT_ACTION_UNKNOWN	= 0,
	NET_EM_ADVERT_ACTION_START,				// start
	NET_EM_ADVERT_ACTION_STOP,				// stop
}NET_EM_ADVERT_ACTION;

// advert position
typedef enum tagNET_EM_ADVERT_POSITION
{
	NET_EM_ADVERT_POSITION_UNKNOWN	= 0,
	NET_EM_ADVERT_POSITION_TOP,				// top    
	NET_EM_ADVERT_POSITION_MIDDLE,			// middle 
	NET_EM_ADVERT_POSITION_BOTTOM,			// bottom 
}NET_EM_ADVERT_POSITION;

// CLIENT_SetAdvert input param(set advert)
typedef struct tagNET_IN_SET_ADVERT 
{
    DWORD						dwSize;
    NET_EM_ADVERT_ACTION        emAction;             // advert action
    int							nSpeed;               // advert play speed
    NET_EM_ADVERT_POSITION		emPosition;           // advert position
} NET_IN_SET_ADVERT;

// CLIENT_SetAdvert output param(set advert)
typedef struct tagNET_OUT_SET_ADVERT
{
    DWORD               dwSize;
} NET_OUT_SET_ADVERT;

// CLIENT_AIOScreen input param(get NVR screen)
typedef struct tagNET_IN_AIO_SCREEN
{
    DWORD						dwSize;
} NET_IN_AIO_SCREEN;

// CLIENT_AIOScreen input param(get NVR screen)
typedef struct tagNET_OUT_AIO_SCREEN
{
    DWORD               dwSize;
	char				szName[DH_COMMON_STRING_256];	// picture path
} NET_OUT_AIO_SCREEN;

// CLIENT_AIOandroidAdb input param(control android Adb)
typedef struct tagNET_IN_CONTROL_ANDROID_ADB 
{
    DWORD						dwSize;
	BOOL						bEnable;				// enable open android adb
} NET_IN_CONTROL_ANDROID_ADB;

// CLIENT_AIOandroidAdb output param(control android Adb)
typedef struct tagNET_OUT_CONTROL_ANDROID_ADB
{
    DWORD               dwSize;
} NET_OUT_CONTROL_ANDROID_ADB;

// Alarm EventHandle
typedef struct tagNET_CFG_ALARM_MSG_HANDLE
{
    DWORD                               dwSize;

	//Ability
    bool                				abChannelCount;                                 // Whether Support Channel Count
	bool                				abAlarmOutCount;                                // Whether Support AlarmOut Count
	
	bool								abRecordMask;                                   // Whether Support RecordMask
	bool								abRecordEnable;                                 // Whether Support RecordEnable
	bool								abRecordLatch;                                  // Whether Support RecordLatch
	
	bool								abAlarmOutMask;                                 // Whether Support AlarmOutMask
	
	bool								abAlarmOutEn;                                   // Whether Support AlarmOut Enable
	bool								abAlarmOutLatch;                                // Whether Support AlarmOut Latch
	bool								abExAlarmOutMask;                               // Whether Support ExAlarmOut Mask
	bool								abExAlarmOutEn;                                 // Whether Support ExAlarmOut Enable
	
	bool								abPtzLinkEn;                                    // Whether Support PTZ Link Enable
	bool								abTourMask;                                     // Whether Support Tour Mask
	bool								abTourEnable;                                   // Whether Support Tour Enable
	
	bool								abSnapshot;                                     // Whether Support Snapshot
	bool								abSnapshotEn;                                   // Whether Support Snapshot Enable
	bool								abSnapshotPeriod;                               // Whether Support Snapshot Period, Snap a picture every sceond
	bool								abSnapshotTimes;                                // Whether Support Snapshot Times
	bool								abTipEnable;                                    // Whether Support Local Message Box prompt
	
	bool								abMailEnable;                                   // Whether Support Send Mail Enable
	bool								abMessageEnable;                                // Whether Support Upload to the alarm center server
	bool								abBeepEnable;                                   // Whether Support Beep Enable
	bool								abVoiceEnable;                                  // Whether Support Voice Enable
	
	bool								abMatrixMask;                                   // Whether Support Matrix Mask
	bool								abMatrixEnable;                                 // Whether Support Matrix Enable
	bool								abEventLatch;                                   // Whether Support Event Latch
	bool								abLogEnable;                                    // Whether Support Log Enable
	
	bool								abDelay;                                        // Whether Support Delay
	bool								abVideoMessageEn;                               // Whether Support Add Message to video
	bool								abMMSEnable;                                    // Whether Support Send MMS Enable
	bool								abMessageToNetEn;                               // Whether Support Message to Net Enable
	bool								abTourSplit;                                    // Whether Support TourSplit
	
	bool								abSnapshotTitleEn;                              // Whether Support Snapshot Title Enable
	bool                				abPtzLinkEx;                                    // Whether Support PTZ Link Enable
	bool                				abSnapshotTitle;                                // Whether Support Add Snapshot Title
	bool                				abMailDetail;                                   // Whether Support Mail Detail
	bool                				abVideoTitleEn;                                 // Whether Support Add Video Title Enable, MainStream
	bool                				abVideoTitle;                                   // Whether Support Video Title
	
	bool                				abTour;                                         // Whether Support Tour
	bool                				abDBKeys;                                       // Whether Support Specifies the KeyWords that need to be Written to the DB in the event details
	bool                				abJpegSummary;                                  // Whether Support Add JPEG Summary
	bool                				abFlashEn;                                      // Whether Support Flash Enable
	bool                				abFlashLatch;                                   // Whether Support Flash Latch
	
    bool								abAudioFileName;                                // Whether Support Audio File Name
    bool								abAlarmBellEn;                                  // Whether Support AlarmBell Enable
    bool								abAccessControlEn;                              // Whether Support Access Control Enable
    bool								abAccessControl;                                // Whether Support Access Control
	
    bool								abTalkBack;	                                    // Whether Support TalkBack
    bool								abPSTNAlarmServer;                              // Whether Support PSTN Alarm Server
    bool								abAlarmBellLatch;                               // Whether Support AlarmBell Latch

	bool                                abPlayTimes;                                    // Whether Support PlayTimes
	bool                                abReboot;                                        // Whether Support Reboot enable
	bool                                abBeepTime;                                     // Whether Support BeepTime
	
	BYTE                                byReserved[68];                                 // reserved for ability

	//Information
	NET_CFG_TIME_SCHEDULE				stuTimeSection;									// Alarm Time Section

	int									nChannelCount;									// The video channel of the device
	int									nAlarmOutCount;									// The alarm output amount of the device

	DWORD								dwRecordMask[CFG_MAX_CHANNEL_COUNT];			// Subnet mask of the recording channel(use the bit to represent)
	BOOL								bRecordEnable;									// Record enable
	int									nRecordLatch;									// Record delay time(s)
	DWORD								dwAlarmOutMask[CFG_MAX_CHANNEL_COUNT];			// ubnet mask of alarm output channel

	BOOL								bAlarmOutEn;									// Alarm output enable
	int									nAlarmOutLatch;									// Alarm output delay time (s)
	DWORD								dwExAlarmOutMask[CFG_MAX_CHANNEL_COUNT];		// Subnet mask of extension alarm output channel 
	BOOL								bExAlarmOutEn;									// Extension alarm output enable

	NET_CFG_PTZ_LINK					stuPtzLink[CFG_MAX_VIDEO_CHANNEL_NUM];			// PTZ activation item
	BOOL								bPtzLinkEn;										// PTZ activation enable
	DWORD								dwTourMask[CFG_MAX_CHANNEL_COUNT];				// Subnet mask of tour channel
	BOOL								bTourEnable;									// Tour enable

	DWORD								dwSnapshot[CFG_MAX_CHANNEL_COUNT];				// Snapshot channel subnet mask
	BOOL								bSnapshotEn;									// Snapshot enable
	int									nSnapshotPeriod;								// Snapshot period(s)
	int									nSnapshotTimes;									// Snapshot times
	BOOL								bTipEnable;										// Local prompt dialogue box

	BOOL								bMailEnable;									// Send out emali. The image is sent out as the attachment.
	BOOL								bMessageEnable;									// Upload to the alarm server
	BOOL								bBeepEnable;							 		// Buzzer
	
	BOOL								bVoiceEnable;									// Audio prompt
	int                                 nPlayTimes;                                     // The Linkage Audio play times,bVoiceEnable=TRUE is effective

	DWORD								dwMatrixMask[CFG_MAX_CHANNEL_COUNT];			// Subnet mask of the activated video channel
	BOOL								bMatrixEnable;									// Activate the video matrix
	int									nEventLatch;									// Activation delay time (s),0-15
	BOOL								bLogEnable;										// Record log or not

	int									nDelay;											// Delay first and then becomes valid when set. Unit is second.
	BOOL								bVideoMessageEn;								// Overlay the prompt character to the video. The overlay character includes the event type, channel number. The unit is second.
	BOOL								bMMSEnable;										// Enable MMS
	BOOL								bMessageToNetEn;								// Send the message to the network enable
	int									nTourSplit;										// Tour split mod 0: 1tour; 1: 8tour

	BOOL								bSnapshotTitleEn;								// Enble osd
	int                 				nPtzLinkExNum;									// PTZ link configuration number
	NET_PTZ_LINK 						stuPtzLinkEx[CFG_MAX_VIDEO_CHANNEL_NUM];        // PTZ extend information
	int                 				nSnapTitleNum;									// Number of picture title
	NET_CFG_EVENT_TITLE 				stuSnapshotTitle[CFG_MAX_VIDEO_CHANNEL_NUM];	// Picture title content
	NET_CFG_MAIL_DETAIL 				stuMailDetail;									// Mail detial
	BOOL                				bVideoTitleEn;									// Whether overlay video title, mainly refers to the main stream
	int                 				nVideoTitleNum;									// Video title num
	NET_CFG_EVENT_TITLE 				stuVideoTitle[CFG_MAX_VIDEO_CHANNEL_NUM];		// Video title

	int                 				nTourNum;										// Tour num
	NET_CFG_TOURLINK    				stuTour[CFG_MAX_VIDEO_CHANNEL_NUM];				// Tour configuration
	int                 				nDBKeysNum;										// Specify the db keyword on the number of effective
	char                				szDBKeys[CFG_MAX_DBKEY_NUM][CFG_MAX_CHANNELNAME_LEN];	// The specify event detail information need write the BD keyword
	BYTE                				byJpegSummary[CFG_MAX_SUMMARY_LEN];						// The summary information of the jpeg image
	BOOL                				bFlashEnable;											// Whether enable flash
	int                 				nFlashLatch;											// Flash delay time (s),the time range:[10,300]

	char								szAudioFileName[MAX_PATH];						// The Absolute Path to the Linkage Audio Files
	BOOL								bAlarmBellEn;									// Warning Signal Enable
	BOOL								bAccessControlEn;								// Entrance Guard Enable
	DWORD								dwAccessControl;								// Class Number of Entrance Guard
	NET_EM_CFG_ACCESSCONTROLTYPE		emAccessControlType[CFG_MAX_ACCESSCONTROL_NUM];	// Entrance Guard Linkage Operation Information

	NET_CFG_TALKBACK_INFO				stuTalkback;									// Voice Call Linkage Information
	NET_CFG_PSTN_ALARM_SERVER			stuPSTNAlarmServer;								// Telephone Alarm Center Linkage Information
	int									nAlarmBellLatch;								// Police no. output delay time(10-300 s)

	BOOL                                bReboot;                                         // Rebot Enable,TRUE:enable FALSE:unenble
	int                                 nBeepTime;                                      // The Max Beep Time:36000 Stand for persistent buzz
} NET_CFG_ALARM_MSG_HANDLE;

// Register update file
CLIENT_NET_API LLONG CALL_METHOD CLIENT_attachAIOFileProc(LLONG lLoginID, const NET_IN_ATTACH_AIO* pstInParam, NET_OUT_ATTACH_AIO *pstOutParam, int nWaitTime);

// detach AIO fileproc,lAttachHandle is the return value of CLIENT_attachAIOFileProc
CLIENT_NET_API BOOL CALL_METHOD CLIENT_detachAIOFileproc(LLONG lAttachHandle);

// set NVR advert
CLIENT_NET_API LLONG CALL_METHOD CLIENT_SetAIOAdvert(LLONG lLoginID, const NET_IN_SET_ADVERT* pstInParam, NET_OUT_SET_ADVERT *pstOutParam, int nWaitTime );

// get NVR screen
CLIENT_NET_API LLONG CALL_METHOD CLIENT_AIOScreen(LLONG lLoginID, const NET_IN_AIO_SCREEN* pstInParam, NET_OUT_AIO_SCREEN *pstOutParam, int nWaitTime );

// CLIENT_StartUploadAIOFile callback function, lUploadFileHandle is CLIENT_StartUploadAIOFile return value
typedef void (CALLBACK *fUploadAIOFileCallBack) (LLONG lUploadFileHandle, int nTotalSize, int nSendSize, LDWORD dwUser, int nError);

// start upload AIO file
CLIENT_NET_API LLONG CALL_METHOD CLIENT_StartUploadAIOFile(LLONG lLoginID, const NET_IN_UPLOAD_AIO_FILE* pInParam, NET_OUT_UPLOAD_AIO_FILE* pOutParam, fUploadAIOFileCallBack cbUploadFile, LDWORD dwUser);

// stop upload AIO file
CLIENT_NET_API BOOL CALL_METHOD CLIENT_StopUploadAIOFile(LLONG lUploadFileID);

// control androidAdb open or close
CLIENT_NET_API LLONG CALL_METHOD CLIENT_ControlAndroidAdb(LLONG lLoginID, const NET_IN_CONTROL_ANDROID_ADB* pstInParam, NET_OUT_CONTROL_ANDROID_ADB *pstOutParam, int nWaitTime);

// configuration interface, Get configuration information(szOutBuffer malloc/release by user, type corresponding to enum NET_EM_CFG_OPERATE_TYPE)
CLIENT_NET_API BOOL CALL_METHOD CLIENT_GetConfig(LLONG lLoginID, NET_EM_CFG_OPERATE_TYPE emCfgOpType, int nChannelID,
                                                         void* szOutBuffer, DWORD dwOutBufferSize, int waittime=NET_INTERFACE_DEFAULT_TIMEOUT, void *reserve = NULL);

// configuration interface, Set configuration information(szInBuffer malloc/release by user, type corresponding to enum NET_EM_CFG_OPERATE_TYPE)
CLIENT_NET_API BOOL CALL_METHOD CLIENT_SetConfig(LLONG lLoginID, NET_EM_CFG_OPERATE_TYPE emCfgOpType, int nChannelID,
                                                         void* szInBuffer, DWORD dwInBufferSize, int waittime=NET_INTERFACE_DEFAULT_TIMEOUT, int *restart = NULL, void *reserve = NULL);

// set net AutoAdapt Encode policy, in parameter
typedef struct tagNET_IN_BUFFER_POLICY
{
    DWORD                   dwSize;             
    DH_RealPlayType         emRealPlayType;          // stream type,only support DH_RType_Realplay_0,DH_RType_Realplay_1,DH_RType_Realplay_Test
    UINT                    nPolicy;                 // 0:default,1:fluency, 2:actual time
}NET_IN_BUFFER_POLICY; 

// set net AutoAdapt Encode policy,user malloc and free memory of pInBuf 
CLIENT_NET_API BOOL CALL_METHOD CLIENT_SetRealplayBufferPolicy(LLONG lPlayHandle,NET_IN_BUFFER_POLICY* pInBuf,int nWaitTime);



//////////////////////////////////////////////////////////////////////////
// TestOSD operation type
typedef enum tagEM_TESTOSD_OPERATE_TYPE
{
    EM_TESTOSD_ADD_TEXT,          // add text info, corresponding to NET_IN_TESTOSD_ADD_TEXT and NET_OUT_TESTOSD_ADD_TEXT
    EM_TESTOSD_GET_TEXT,          // get text info, corresponding to NET_IN_TESTOSD_GET_TEXT and NET_OUT_TESTOSD_GET_TEXT
    EM_TESTOSD_MODIFY_TEXT,       // modify text info, corresponding to NET_IN_TESTOSD_MODIFY_TEXT and NET_OUT_TESTOSD_MODIFY_TEXT
    EM_TESTOSD_DELETE_TEXT,       // delete text info, corresponding to NET_IN_TESTOSD_DELETE_TEXT and NET_OUT_TESTOSD_DELETE_TEXT
    EM_TESTOSD_ADD_PICTURE,       // add picture info, corresponding to NET_IN_TESTOSD_ADD_PIC and NET_OUT_TESTOSD_ADD_PIC
    EM_TESTOSD_GET_PICTURE,       // get picture info, corresponding to NET_IN_TESTOSD_GET_PIC and NET_OUT_TESTOSD_GET_PIC
    EM_TESTOSD_DELETE_PICTURE,    // delete picture info, corresponding to NET_IN_TESTOSD_DELETE_PIC and NET_OUT_TESTOSD_DELETE_PIC
}EM_TESTOSD_OPERATE_TYPE;

// OSD text info
typedef struct tagNET_TEXT_OSD
{   
    char                    szText[DH_COMMON_STRING_256];   // text
    DH_POINT                stuPoint;                       // position
	NET_COLOR_RGBA			stuTextColor;					// text color
	int						nFontSize;						// the font size, 0-middle 1-small 2-big
    BYTE                    byReserved[44];                 // reserved
}NET_TEXT_OSD;

// OSD picture type
typedef enum tagEM_OSDPIC_TYPE
{
    EM_OSDPIC_UNKNOWN,                                      // unknown
    EM_OSDPIC_HEADONFACE,                                   // head face
    EM_OSDPIC_SIDEFACE,                                     // side face
    EM_OSDPIC_MAP,                                          // map 
}EM_OSDPIC_TYPE;

// OSD picture info
typedef struct tagNET_PICTURE_OSD
{
    EM_OSDPIC_TYPE          emPictureType;                  // OSD picture type
	NET_RECT				stuPictureLocation;				// Picture Location
    BYTE                    byReserved[48];                 // reserved
}NET_PICTURE_OSD;

// Operation type EM_TESTOSD_ADD_TEXT, input param
typedef struct tagNET_IN_TESTOSD_ADD_TEXT
{
    DWORD                   dwSize; 
    int                     nMaxCount;                     // max count of text info to allocate
    NET_TEXT_OSD            *pstuTextInfo;                 // text info, allocated by user
}NET_IN_TESTOSD_ADD_TEXT;

// Operation type EM_TESTOSD_ADD_TEXT, output param
typedef struct tagNET_OUT_TESTOSD_ADD_TEXT
{
    DWORD                   dwSize;
    int                     nReturnCount;                   // the valid return count
    int                     *pnTextID;                      // text info ID, allocated by user, the length is equal to nMaxCount
    int                     *pnReturnCode;                  // return code , 0-success 1-failure, allocated by user, the length is equal to nMaxCount
}NET_OUT_TESTOSD_ADD_TEXT;

// Operation type EM_TESTOSD_GET_TEXT, input param
typedef struct tagNET_IN_TESTOSD_GET_TEXT
{
    DWORD                   dwSize;
    int                     nMaxCount;                      // max count of text info to allocate
    int                     *pnTextID;                      // text info, allocated by user
}NET_IN_TESTOSD_GET_TEXT;

// Operation type EM_TESTOSD_GET_TEXT, output param
typedef struct tagNET_OUT_TESTOSD_GET_TEXT
{
    DWORD                   dwSize;
    int                     nReturnCount;                   // the valid return count
    NET_TEXT_OSD            *pstuTextInfo;                  // text info ID, allocated by user, the length is equal to nMaxCount
}NET_OUT_TESTOSD_GET_TEXT;

// Operation type EM_TESTOSD_MODIFY_TEXT, input param
typedef struct tagNET_IN_TESTOSD_MODIFY_TEXT
{
    DWORD                   dwSize;
    int                     nMaxCount;                      // max count of text info and text id to allocate
    int                     *pnTextID;                      // text info ID, allocated by user
    NET_TEXT_OSD            *pstuTextInfo;                  // text info, allocated by user
}NET_IN_TESTOSD_MODIFY_TEXT;

// Operation type EM_TESTOSD_MODIFY_TEXT, output param
typedef struct tagNET_OUT_TESTOSD_MODIFY_TEXT
{
    DWORD                   dwSize;
    int                     nReturnCount;                   // the valid return count
    int                     *pnReturnCode;                  // return code , 0-success 1-failure, allocated by user, the length is equal to nMaxCount
}NET_OUT_TESTOSD_MODIFY_TEXT;

// Operation type EM_TESTOSD_DELETE_TEXT, input param
typedef struct tagNET_IN_TESTOSD_DELETE_TEXT
{
    DWORD                   dwSize;
    int                     nMaxCount;                      // max count of text id to allocate
    int                     *pnTextID;                      // text info ID, allocated by user
}NET_IN_TESTOSD_DELETE_TEXT;

// Operation type EM_TESTOSD_DELETE_TEXT, output param
typedef struct tagNET_OUT_TESTOSD_DELETE_TEXT
{
    DWORD                   dwSize;
    int                     nReturnCount;                   // the valid return count
    int                     *pnReturnCode;                  // return code , 0-success 1-failure, allocated by user, the length is equal to nMaxCount
}NET_OUT_TESTOSD_DELETE_TEXT;

// Operation type EM_TESTOSD_ADD_PICTURE, input param
typedef struct tagNET_IN_TESTOSD_ADD_PIC
{
    DWORD                   dwSize;
    int                     nMaxCount;                      // max count of info to allocate
    char                    (*pszFilePath)[MAX_PATH];       // the file path
    NET_PICTURE_OSD         *pstuPicture;                   // picture info
}NET_IN_TESTOSD_ADD_PIC;

// Operation type EM_TESTOSD_ADD_PICTURE, output param
typedef struct tagNET_OUT_TESTOSD_ADD_PIC
{
    DWORD                   dwSize;
    int                     nReturnCount;                   // the valid return count
    int                     *pnReturnCode;                  // return code , 0-success 1-failure, allocated by user, the length is equal to nMaxCount
}NET_OUT_TESTOSD_ADD_PIC;

// Operation type EM_TESTOSD_GET_PICTURE, input param
typedef struct tagNET_IN_TESTOSD_GET_PIC
{
    DWORD                   dwSize;
    int                     nCount;                         // get mounts of picture 
}NET_IN_TESTOSD_GET_PIC;

// Operation type EM_TESTOSD_GET_PICTURE, output param
typedef struct tagNET_OUT_TESTOSD_GET_PIC
{
    DWORD                   dwSize;    
    int                     nReturnCount;                   // the valid return count
    char                    (*pszFilePath)[MAX_PATH];       // the file path
    NET_PICTURE_OSD         *pstuPicture;                   // the picture info
}NET_OUT_TESTOSD_GET_PIC;

// Operation type EM_TESTOSD_DELETE_PICTURE, input param
typedef struct tagNET_IN_TESTOSD_DELETE_PIC
{
    DWORD                   dwSize;
    int                     nMaxCount;                      // max count of info to allocate
    char                    (*pszFilePath)[MAX_PATH];       // the file path    
}NET_IN_TESTOSD_DELETE_PIC;

// Operation type EM_TESTOSD_DELETE_PICTURE, output param
typedef struct tagNET_OUT_TESTOSD_DELETE_PIC
{
    DWORD                   dwSize;
    int                     nReturnCount;                   // the valid return count
    int                     *pnReturnCode;                  // return code , 0-success 1-failure, allocated by user, the length is equal to nMaxCount
}NET_OUT_TESTOSD_DELETE_PIC;


// get alarm event log input param
typedef struct tagNET_IN_GET_ALARM_EVENT_LOG
{
	DWORD                   dwSize;
	UINT 	                nEventID;				        // event ID
}NET_IN_GET_ALARM_EVENT_LOG;

// get alarm event log output param
typedef struct tagNET_OUT_GET_ALARM_EVENT_LOG
{
	DWORD                   dwSize;
	DWORD			        dwAlarmType;			       // alarm event type
	BYTE 			       *byAlarmData;                   // alarm event data, allocated by user, at least 2* 1024 bytes
	int 			        nMaxDataLen;		           // max alarm data len
	int						nRetDataLen;                   // real retrun data len
}NET_OUT_GET_ALARM_EVENT_LOG;

// Get alarm event log
CLIENT_NET_API BOOL CALL_METHOD CLIENT_GetEventLog(LLONG lLoginID, const NET_IN_GET_ALARM_EVENT_LOG *pstuInParam, NET_OUT_GET_ALARM_EVENT_LOG *pstuOutParam, int nWaitTime);

// TestOSD operation interface

CLIENT_NET_API BOOL CALL_METHOD CLIENT_OperateTestOSD(LLONG lLoginID, EM_TESTOSD_OPERATE_TYPE emOperateType, void *pInParam, void *pOutParam, int nWaitTime);

// input of set lock or unlock
typedef struct tagNET_IN_COURSECOMPOSITE_LOCK_CONTROL
{
    DWORD                   dwSize;
    BOOL                    bLock;                         // TRUE lock,FALSE unlock
    int                     nClassRoomID;                  // Classroom id        
    int                     nLogicChannel;                 // logic channel
}NET_IN_COURSECOMPOSITE_LOCK_CONTROL;

// output of set lock or unlock
typedef struct tagNET_OUT_COURSECOMPOSITE_LOCK_CONTROL
{
    DWORD                   dwSize;
}NET_OUT_COURSECOMPOSITE_LOCK_CONTROL;

// input of get lock state
typedef struct tagNET_IN_COURSECOMPOSITE_GET_LOCKINFO
{
    DWORD                   dwSize;
    int                     nClassRoomID;                   // Classroom id 
    int                     nLogicChannel;                  // logic channel
}NET_IN_COURSECOMPOSITE_GET_LOCKINFO;

// output of get lock state
typedef struct tagNET_OUT_COURSECOMPOSITE_GET_LOCKINFO
{
    DWORD                   dwSize;             
    BOOL                    bState;                         // lock state TRUE locked FALSE unlock
}NET_OUT_COURSECOMPOSITE_GET_LOCKINFO;

// input of get composite channel info
typedef struct tagNET_IN_COURSECOMPOSITE_GET_INFO
{
    DWORD                   dwSize;                        
    int                     nClassRoomId;                  // classroom id
}NET_IN_COURSECOMPOSITE_GET_INFO;

// record mdoe
typedef enum tagEM_COMPOSITECHANNEL_MODE
{
    EM_COMPOSITECHANNEL_CUSTOMER = -1,                      // customer
    EM_COMPOSITECHANNEL_INVALID,                            // invalid
    EM_COMPOSITECHANNEL_MOVIE,                              // movie
    EM_COMPOSITECHANNEL_NORMALCY,                           // normal
    EM_COMPOSITECHANNEL_PICK,                               // pick
}EM_COMPOSITECHANNEL_MODE;

// composite channel info
typedef struct tagNET_COMPOSITECHANNEL_INFO
{
    char                        szCourseName[DH_COMMON_STRING_128];      // course name
    char                        szTeacherName[DH_COMMON_STRING_64];      // teacher name
    int  					    nCompositeChannelMode;                  // record mode 0:invalid;1:movie mode; 2:normal mode; 3:pick mode; <0:customer mode;
    int                         nChannelNum;                             // logic channel number
    BOOL                        bCanStream[MAX_COURSE_LOGIC_CHANNEL];    // disable to stream,number is logic channel id
    BYTE                        byReserved[256];                                                  
}NET_COMPOSITECHANNEL_INFO;

// output of get composite channel info
typedef struct tagNET_OUT_COURSECOMPOSITE_GET_INFO
{
    DWORD                       dwSize;  
    NET_COMPOSITECHANNEL_INFO   stuChannelInfo;                          // composite channel info
}NET_OUT_COURSECOMPOSITE_GET_INFO;

// input of set composite channel info
typedef struct tagNET_IN_COURSECOMPOSITE_SET_INFO
{
    DWORD                       dwSize;                        
    int                         nClassRoomId;                            // classroom id
    NET_COMPOSITECHANNEL_INFO   stuChannelInfo;                          // composite channel info
}NET_IN_COURSECOMPOSITE_SET_INFO;

// output of set composite channel info
typedef struct tagNET_OUT_COURSECOMPOSITE_SET_INFO
{
    DWORD                       dwSize;                        
}NET_OUT_COURSECOMPOSITE_SET_INFO;

// input of update composite channel info to time
typedef struct tagNET_IN_COURSECOMPOSITE_UPDATE_INFO
{
    DWORD                       dwSize; 
    int                         nClassRoomId;                        // classroom id
    NET_TIME                    stuTime;                             // query time                     
}NET_IN_COURSECOMPOSITE_UPDATE_INFO;

// output of update composite channel info to time
typedef struct tagNET_OUT_COURSECOMPOSITE_UPDATE_INFO
{
    DWORD                       dwSize;                     
}NET_OUT_COURSECOMPOSITE_UPDATE_INFO;

// enum of coursemedia operate 
typedef enum tagEM_COURSECOMPOSITE_OPERATE_TYPE
{
    EM_COURSECOMPOSITE_TYPE_LOCK_CONTROL,                   // control composite and logic channel lock or unlock, corresponding to pInParam = NET_IN_COURSECOMPOSITE_LOCK_CONTROL, pOutParam = NET_OUT_COURSECOMPOSITE_LOCK_CONTROL
    EM_COURSECOMPOSITE_TYPE_GET_LOCKINFO,                   // get composite and logic channel lock state, corresponding to pInParam = NET_IN_COURSECOMPOSITE_GET_LOCKINFO, pOutParam = NET_OUT_COURSECOMPOSITE_GET_LOCKINFO 
    EM_COURSECOMPOSITE_TYPE_GET_INFO,                       // get Compositechannel info,corresponding to pInParam = NET_IN_COURSECOMPOSITE_GET_INFO,pOutParam = NET_OUT_COURSECOMPOSITE_GET_INFO 
    EM_COURSECOMPOSITE_TYPE_SET_INFO,                       // set compositechannel info,corresponding to pInParam = NET_IN_COURSECOMPOSITE_SET_INFO,pOutParam = NET_OUT_COURSECOMPOSITE_SET_INFO 
    EM_COURSECOMPOSITE_TYPE_UPDATE_INFO,                    // update compositechannel info to time,corresponding to pInParam = NET_IN_COURSECOMPOSITE_UPDATE_INFO, pOutParam = NET_OUT_COURSECOMPOSITE_UPDATE_INFO
}EM_COURSECOMPOSITE_OPERATE_TYPE;

//interface of coursemedia Operate 
CLIENT_NET_API BOOL CALL_METHOD CLIENT_OperateCourseCompositeChannel(LLONG lLoginID, EM_COURSECOMPOSITE_OPERATE_TYPE emOperateType, void *pInParam, void *pOutParam, int nWaitTime);

// max number of windows
#define NET_MAX_WINDOWS_NUMBER	64

// max number of mode 
#define NET_MAX_MODE_NUMBER		64

// CourseCompositeChannelMode window info
typedef struct tagNET_COMPOSITECHANNELMODE_WINDOWS
{
	DH_RECT					stRect;					// window coordinate
	int						nLogicChannel;			// logic channel, -1 means this window not bind channel
	int						nZOrder;				// Z order
	BYTE                    byReserved[64];         // Reserved
}NET_COMPOSITECHANNELMODE_WINDOWS;

// CourseCompositeChannelMode info
typedef struct tagNET_COMPOSITECHANNELMODE_INFO
{
	char									szName[DH_COMMON_STRING_128];			// mode name
	int										nWindowNum;								// actual number of windows
	NET_COMPOSITECHANNELMODE_WINDOWS		stWindows[NET_MAX_WINDOWS_NUMBER];		// windows info
	BYTE									byReserved[64];							// Reserved
}NET_COMPOSITECHANNELMODE_INFO;

// input params of add CourseCompositeChannelMode
typedef struct tagNET_IN_COURSECOMPOSITE_CHANNEL_MODE_ADD
{
    DWORD							dwSize;								// struct size
    int								nCount;								// number of add mode
	NET_COMPOSITECHANNELMODE_INFO	stModeInfo[NET_MAX_MODE_NUMBER];	// mode info
}NET_IN_COURSECOMPOSITE_CHANNEL_MODE_ADD;

// result of add CourseCompositeChannelMode
typedef struct tagNET_COURSECOMPOSITE_CHANNEL_MODE_ADD_RESULT
{
	int						nMode;					// mode, 0 invalid, 1 movie mode, 2 normal mode, 3 boutique mode, <0 custom mode
	int						nReturnCode;			// return code: 1 success, 2 failure, 3 full, 4 resource lack
	BYTE                    byReserved[64];         // Reserved
}NET_COURSECOMPOSITE_CHANNEL_MODE_ADD_RESULT;

// output params of add CourseCompositeChannelMode
typedef struct tagNET_OUT_COURSECOMPOSITE_CHANNEL_MODE_ADD
{
    DWORD											dwSize;								// struct size
	int												nCount;								// actual number of result
	NET_COURSECOMPOSITE_CHANNEL_MODE_ADD_RESULT		stResult[NET_MAX_MODE_NUMBER];		// return results
}NET_OUT_COURSECOMPOSITE_CHANNEL_MODE_ADD;

// input params of delete CourseCompositeChannelMode
typedef struct tagNET_IN_COURSECOMPOSITE_CHANNEL_MODE_DELETE
{
	DWORD					dwSize;							// struct size
	int						nModeNum;						// number of delete mode
	int						nMode[NET_MAX_MODE_NUMBER];		// sequence number of mode
}NET_IN_COURSECOMPOSITE_CHANNEL_MODE_DELETE;

// output params of delete CourseCompositeChannelMode 
typedef struct tagNET_OUT_COURSECOMPOSITE_CHANNEL_MODE_DELETE
{
	DWORD					dwSize;									// struct size
	int						nReturnNum;								// number of delete mode
	int						nReturnCode[NET_MAX_MODE_NUMBER];		// return code: 1 success, 2 failure
}NET_OUT_COURSECOMPOSITE_CHANNEL_MODE_DELETE;

// input params of modify CourseCompositeChannelMode
typedef struct tagNET_IN_COURSECOMPOSITE_CHANNEL_MODE_MODIFY 
{
	DWORD							dwSize;								// struct size
	int								nModeNum;							// number of modify mode
	int								nMode[NET_MAX_MODE_NUMBER];			// sequence number of mode 
	NET_COMPOSITECHANNELMODE_INFO	stModeInfo[NET_MAX_MODE_NUMBER];	// mode info
}NET_IN_COURSECOMPOSITE_CHANNEL_MODE_MODIFY;

// output params of modify CourseCompositeChannelMode 
typedef struct tagNET_OUT_COURSECOMPOSITE_CHANNEL_MODE_MODIFY
{
	DWORD					dwSize;									// struct size
	int						nReturnNum;								// number of modify mode
	int						nReturnCode[NET_MAX_MODE_NUMBER];		// return code: 1 success, 2 failure, 3 full, 4 resource lack
}NET_OUT_COURSECOMPOSITE_CHANNEL_MODE_MODIFY;

// input params of get CourseCompositeChannelMode
typedef struct tagNET_IN_COURSECOMPOSITE_CHANNEL_MODE_GET 
{
	DWORD					dwSize;					// struct size
	int						nCount;					// number of get mode
}NET_IN_COURSECOMPOSITE_CHANNEL_MODE_GET;

// output params of get CourseCompositeChannelMode 
typedef struct tagNET_OUT_COURSECOMPOSITE_CHANNEL_MODE_GET
{
	DWORD							dwSize;								// struct size
	int								nReturnNum;							// actual number of get mode
	int								nMode[NET_MAX_MODE_NUMBER];			// sequence number of mode 
	NET_COMPOSITECHANNELMODE_INFO	stModeInfo[NET_MAX_MODE_NUMBER];	// mode information
}NET_OUT_COURSECOMPOSITE_CHANNEL_MODE_GET;

// CourseCompositeChannelMode operate type
typedef enum tagNET_COURSECOMPOSITE_MODE_OPERATE_TYPE
{
    NET_COURSECOMPOSITE_MODE_ADD,                   // add mode,	pInParam = NET_IN_COURSECOMPOSITE_CHANNEL_MODE_ADD,	   pOutParam = NET_OUT_COURSECOMPOSITE_CHANNEL_MODE_ADD
	NET_COURSECOMPOSITE_MODE_DELETE,                // delete mode, pInParam = NET_IN_COURSECOMPOSITE_CHANNEL_MODE_DELETE, pOutParam = NET_OUT_COURSECOMPOSITE_CHANNEL_MODE_DELETE
	NET_COURSECOMPOSITE_MODE_MODIFY,                // modify mode, pInParam = NET_IN_COURSECOMPOSITE_CHANNEL_MODE_MODIFY, pOutParam = NET_OUT_COURSECOMPOSITE_CHANNEL_MODE_MODIFY
	NET_COURSECOMPOSITE_MODE_GET,                   // get mode,	pInParam = NET_IN_COURSECOMPOSITE_CHANNEL_MODE_GET,	   pOutParam = NET_OUT_COURSECOMPOSITE_CHANNEL_MODE_GET
}NET_COURSECOMPOSITE_MODE_OPERATE_TYPE;

//interface of CourseCompositeChannelMode Operate 
CLIENT_NET_API BOOL CALL_METHOD CLIENT_OperateCourseCompositeChannelMode(LLONG lLoginID, NET_COURSECOMPOSITE_MODE_OPERATE_TYPE emOperateType, void *pInParam, void *pOutParam, int nWaitTime);

///////////////////////////////////////////////////////////////////////
#define NET_UPGRADE_COUNT_MAX    256  //max upgrade count number

// vendor type info
typedef enum tagENUM_VENDOR_TYPE
{
    ENUM_VENDOR_TYPE_UNKNOWN,              //unknown
	ENUM_VENDOR_TYPE_GENERAL,              //General 
	ENUM_VENDOR_TYPE_DH,                   //DH 
	ENUM_VENDOR_TYPE_OEM,                  //OEM
}ENUM_VENDOR_TYPE;

//standard type
typedef enum tagENUM_STANDARD_TYPE
{
    ENUM_STANDARD_TYPE_UNKNOWN,              //unknown
	ENUM_STANDARD_TYPE_PAL,                  //P-PAL
	ENUM_STANDARD_TYPE_NTSC,                 //N-NTSC
	ENUM_STANDARD_TYPE_PAL_NTSC,             //PN-PAL/NTSC(default:PAL)
	ENUM_STANDARD_TYPE_NTSC_PAL,             //NP-NTSC/PAL(default:NTSC)
	ENUM_STANDARD_TYPE_SECAM,                //S-SECAM
}ENUM_STANDARD_TYPE;


//get NVR/IPC serial input parameter, use connect mode:direct
typedef struct tagNET_IN_UPGRADER_GETSERIAL
{   
    DWORD               dwSize;					
}NET_IN_UPGRADER_GETSERIAL;

//upgrade serial info
typedef struct tagNET_UPGRADER_SERIAL_INO
{
    ENUM_VENDOR_TYPE    emVendor;               //vendor
    ENUM_STANDARD_TYPE  emStandard;             //video standard
    NET_TIME_EX  stuBuild;                      //build info
    char    szChip[DH_COMMON_STRING_16];        //chip
    char    szSerial[DH_COMMON_STRING_256];     //serial
    char    szLanguage[DH_COMMON_STRING_128];   //language
    char    szSn[DH_COMMON_STRING_64];          //sn
    char    szSWVersion[DH_COMMON_STRING_64];   //software version    
    char    szTag[DH_COMMON_STRING_256];        //tag
    char    szTag2[DH_COMMON_STRING_256];       //tag2
    char    reserved[1024];                         
}NET_UPGRADER_SERIAL_INO;

#define MAX_UPGRADER_SERIAL_INFO  8

//get NVR/IPC serial output parameter, use connect mode:direct
typedef struct tagNET_OUT_UPGRADER_GETSERIAL
{   
    DWORD                     dwSize;					              
    NET_UPGRADER_SERIAL_INO   stuSerialInfo[MAX_UPGRADER_SERIAL_INFO]; // serial info
    int                       nRetNum;                                 // return serial count
}NET_OUT_UPGRADER_GETSERIAL;

//get update serial
CLIENT_NET_API BOOL CALL_METHOD CLIENT_GetUpdateSerial(LLONG lLoginID, NET_IN_UPGRADER_GETSERIAL *pstuInGetSerial, NET_OUT_UPGRADER_GETSERIAL *pstuOutGetSerial, int nWaitTime);

// cloud upgrade check input parameter
typedef struct tagNET_IN_CLOUD_UPGRADER_CHECK
{
    DWORD                  dwSize;
    ENUM_VENDOR_TYPE       emVendor;                            //vendor
    ENUM_STANDARD_TYPE     emStandard;                          //video standard
    NET_TIME_EX            stuBuild;                            //compile time, for comparison versions
    char                   szUrl[DH_COMMON_STRING_1024];        // url
    char                   szClass[DH_COMMON_STRING_64];        // device class
    char                   szSerial[DH_COMMON_STRING_256];      // device hardware signal series
    char                   szLanguage[DH_COMMON_STRING_128];    // language
    char                   szSN[DH_COMMON_STRING_64];           // device serial
    char                   szSWVersion[DH_COMMON_STRING_64];    // software version
    char                   szTag1[DH_COMMON_STRING_256];        // reserved bytes1
    char                   szTag2[DH_COMMON_STRING_256];        // reserved bytes2
    char                   szAccessKeyId[DH_COMMON_STRING_128]; //Access Key ID
    char                   szSecretAccessKey[DH_COMMON_STRING_128];//Secret Access Key
}NET_IN_CLOUD_UPGRADER_CHECK;

// cloud upgrade check output parameter
typedef struct tagNET_OUT_CLOUD_UPGRADER_CHECK
{
    DWORD                  dwSize;
    BOOL                   bHasNewVersion;                       //has new version
    char                   szVersion[DH_COMMON_STRING_64];       // version info
    char                   szAttention[DH_COMMON_STRING_1024];   // prompt information
    char                   szPackageUrl[DH_COMMON_STRING_1024];  // package url
	char 			       szPackageId[DH_COMMON_STRING_64];     //package id
}NET_OUT_CLOUD_UPGRADER_CHECK;

// check if there is scalable software in the cloud
CLIENT_NET_API BOOL CALL_METHOD CLIENT_CloudUpgraderCheck(NET_IN_CLOUD_UPGRADER_CHECK *pIn, NET_OUT_CLOUD_UPGRADER_CHECK *pOut, DWORD dwWaitTime);

//cloud download state
typedef enum tagemCloudDownloadState
{
    emCloudDownloadState_Unknown,            //unknown
    emCloudDownloadState_Success,            //download success(need to call CLIENT_CloudUpgraderStop)
    emCloudDownloadState_Failed,             //download failed(don't need to call CLIENT_CloudUpgraderStop,it will keep trying)
    emCloudDownloadState_Downloading,        //downloading  
    emCloudDownloadState_NoEnoughDiskSpace,  //no enough disk space
}emCloudDownloadState;

//cloud download process callback function pointer
typedef void (CALLBACK *fCloudDownload_Process_callback)(LLONG lDownHandle, emCloudDownloadState emState, double dwDownloadSpeed, DWORD dwProgressPercentage, LDWORD dwUser);

// cloud upgrade download input parameter
typedef struct tagNET_IN_CLOUD_UPGRADER_DOWN
{
    DWORD                               dwSize;                              //structure size
    char                                szPackageUrl[DH_COMMON_STRING_1024]; //device upgrade package url
    char                                szSaveFile[DH_COMMON_STRING_1024];   //saved file path
    fCloudDownload_Process_callback     pfProcessCallback;                   //process callback
    LDWORD                              dwUser;                              //callback user data 
}NET_IN_CLOUD_UPGRADER_DOWN;

//cloud upgrade download output parameter
typedef struct tagNET_OUT_CLOUD_UPGRADER_DOWN
{
    DWORD                  dwSize;     //structure size
}NET_OUT_CLOUD_UPGRADER_DOWN;

//cloud upgrade download 
CLIENT_NET_API LLONG CALL_METHOD CLIENT_CloudUpgraderDownLoad(NET_IN_CLOUD_UPGRADER_DOWN *pIn, NET_OUT_CLOUD_UPGRADER_DOWN *pOut);

//stop cloud upgrade download
CLIENT_NET_API BOOL CALL_METHOD CLIENT_CloudUpgraderStop(LLONG lDownloadHandle);

//pause cloud upgrade download
CLIENT_NET_API BOOL CALL_METHOD CLIENT_CloudUpgraderPause(LLONG lDownloadHandle, BOOL bPause);

//upgrade report result
typedef enum tagNET_UPGRADE_REPORT_RESULT
{
    NET_UPGRADE_REPORT_RESULT_UNKNWON,     //unknown
    NET_UPGRADE_REPORT_RESULT_SUCCESS,     //success
    NET_UPGRADE_REPORT_RESULT_FAILED,      //failed
}NET_UPGRADE_REPORT_RESULT;

//upgrade report info
typedef struct tagNET_UPGRADE_REPORT
{
    int     nDeviceNum;                                                 //device number
    char    szDevSerial[NET_UPGRADE_COUNT_MAX][MAX_COMMON_STRING_64];   // device serial
    char    szPacketID[MAX_COMMON_STRING_128];                          //package id
    NET_UPGRADE_REPORT_RESULT emResult;                                 //upgrade result
    char    szCode[MAX_COMMON_STRING_128];                              //error code      
    char    reserved[256];
}NET_UPGRADE_REPORT;

//upgrade result report input parameter
typedef struct tagNET_IN_UPGRADE_REPORT
{
    DWORD                   dwSize;                                    //structure size
    int                     nCount;                                    //upgrade package count
    NET_UPGRADE_REPORT      *pstuUpgradeReport;                        //upgrade result info    
    char                    szAccessKeyId[DH_COMMON_STRING_128];       //Access Key ID
    char                    szSecretAccessKey[DH_COMMON_STRING_128];   //Secret Access Key    
    char                    szUrl[DH_COMMON_STRING_1024];              //url
}NET_IN_UPGRADE_REPORT;

//upgrade result report output parameter
typedef struct tagNET_OUT_UPGRADE_REPORT
{
    DWORD                  dwSize;
}NET_OUT_UPGRADE_REPORT;

//cloud upgrade result report
CLIENT_NET_API BOOL CALL_METHOD CLIENT_CloudUpgraderReport(NET_IN_UPGRADE_REPORT *pIn, NET_OUT_UPGRADE_REPORT *pOut, DWORD dwWaitTime);

// CLIENT_GetRecordState in parameter
typedef struct tagNET_IN_GET_RECORD_STATE
{
	DWORD				dwSize;				// struct size
	int					nChannel;			// channel 
} NET_IN_GET_RECORD_STATE;

// CLIENT_GetRecordState out parameter
typedef struct tagNET_OUT_GET_RECORD_STATE
{
	DWORD				dwSize;				// struct size
	BOOL				bState;				// record state
} NET_OUT_GET_RECORD_STATE;

// interface of get record state
CLIENT_NET_API BOOL CALL_METHOD CLIENT_GetRecordState(LLONG lLoginID, const NET_IN_GET_RECORD_STATE *pInParam, NET_OUT_GET_RECORD_STATE *pOutParam, int nWaitTime);

// input of get course record info
typedef struct tagNET_IN_COURSERECORD_GETINFO
{
    DWORD                       dwSize;
    int                         nClassRoomID;                           // classroom id,defult:0
}NET_IN_COURSERECORD_GETINFO;

// output of get course record info
typedef struct tagNET_OUT_COURSERECORD_GETINFO
{
    DWORD                       dwSize;
    int                         nChannelNum;                            // logic channel number
    int                         nCanRecord[MAX_COURSE_LOGIC_CHANNEL];   // 0:invalid,1:record,2unrecord,number is logic channel id
}NET_OUT_COURSERECORD_GETINFO;

// input of set course record info
typedef struct tagNET_IN_COURSERECORD_SETINFO
{
    DWORD                       dwSize;
    int                         nClassRoomID;                           // classroom id, defult:0
    int                         nChannelNum;                            // logic channel number
    int                         nCanRecord[MAX_COURSE_LOGIC_CHANNEL];   // 0:invalid,1:record,2unrecord,number is logic channel id
}NET_IN_COURSERECORD_SETINFO;

// output of set course record info
typedef struct tagNET_OUT_COURSERECORD_SETINFO
{
    DWORD                       dwSize;
}NET_OUT_COURSERECORD_SETINFO;

// input of update record info to  time
typedef struct tagNET_IN_COURSERECORD_UPDATE_INFO
{
    DWORD                       dwSize;
    int                         nClassRoomID;                           // classroom id, defult:0
    NET_TIME                    stuTime;                                // query time
}NET_IN_COURSERECORD_UPDATE_INFO;

// output of update record info to time
typedef struct tagNET_OUT_COURSERECORD_UPDATE_INFO
{
    DWORD                       dwSize;
}NET_OUT_COURSERECORD_UPDATE_INFO;

// input of get course recorded time
typedef struct tagNET_IN_COURSERECORD_GET_TIME
{
    DWORD                       dwSize;
    int                         nClassRoomID;                        // classroom id, defult:0
}NET_IN_COURSERECORD_GET_TIME;

// output of get course recorded time
typedef struct tagNET_OUT_COURSERECORD_GET_TIME
{
    DWORD                       dwSize;
    UINT                        nTime;                               // recorded time, unit: second
}NET_OUT_COURSERECORD_GET_TIME;

// operate of coursemedia record info
typedef enum tagEM_COURSERECORD_OPERATE_TYPE
{
    EM_COURSERECORDE_TYPE_GET_INFO,                       // get classroom  record info ,corresponding to pInParam = NET_IN_COURSERECORD_GETINFO,pOutParam = NET_OUT_COURSERECORD_GETINFO 
    EM_COURSERECORDE_TYPE_SET_INFO,                       // set classroom  record info,corresponding to pInParam = NET_IN_COURSERECORD_SETINFO,pOutParam = NET_OUT_COURSERECORD_SETINFO 
    EM_COURSERECORDE_TYPE_UPDATE_INFO,                    // update record info to time,corresponding to pInParam = NET_IN_COURSERECORD_UPDATE_INFO, pOutParam = NET_OUT_COURSERECORD_UPDATE_INFO
    EM_COURSERECORDE_TYPE_GET_TIME,                       // get course recorded time,corresponding to pInParam = NET_IN_COURSERECORD_GET_TIME, pOutParam = NET_OUT_COURSERECORD_GET_TIME
}EM_COURSERECORD_OPERATE_TYPE;

// interface of operate coursemedia record info
CLIENT_NET_API BOOL CALL_METHOD CLIENT_OperateCourseRecordManager(LLONG lLoginID, EM_COURSERECORD_OPERATE_TYPE emOperateType, void *pInParam, void *pOutParam, int nWaitTime);

// call back for attach config change
typedef void(CALLBACK *fConfigChangeCallBack)(LLONG lAttachHandle, char* szCommand, char* szBuffer, LDWORD dwUser);

// attach config change input param
typedef struct tagNET_IN_ATTACH_CONFIGCHANNGE
{
    DWORD                           dwSize;
    char*                           szCfgName;                // config name,see dhconfigsdk.h CLIENT_GetNewDevConfig and CLIENT_SetNewDevConfig port command 
    fConfigChangeCallBack           cbNotify;                 // callback config of changed
    LDWORD                          dwUser;                   // userinfo
}NET_IN_ATTACH_CONFIGCHANNGE;

// attach config change output param
typedef struct tagNET_OUT_ATTACH_CONFIGCHANNGE
{
    DWORD                           dwSize;
}NET_OUT_ATTACH_CONFIGCHANNGE;

// attach config change 
CLIENT_NET_API LLONG CALL_METHOD CLIENT_AttachConfigChange(LLONG lLoginID, NET_IN_ATTACH_CONFIGCHANNGE *pInBuf,NET_OUT_ATTACH_CONFIGCHANNGE *pOutBuf, int nWaitTime);

// detach config change  
CLIENT_NET_API BOOL CALL_METHOD CLIENT_DetachConfigChange(LLONG lAttachHandle);

//CLIENT_GetRingFileList in parameter
typedef struct tagNET_IN_GET_RINGFILELIST
{
	DWORD 		dwSize;
}NET_IN_GET_RINGFILELIST;

//CLIENT_GetRingFileList out parameter
typedef struct tagNET_OUT_GET_RINGFILELIST
{
	DWORD       dwSize;	
	UINT        nRingFileNum;                                         //return ring file num
	char        szRingFileList[MAX_RINGFILE_NUM][DH_COMMON_STRING_64];//ring file list info
}NET_OUT_GET_RINGFILELIST;

//get ring file list 
CLIENT_NET_API BOOL CALL_METHOD CLIENT_GetRingFileList(LLONG lLoginID, NET_IN_GET_RINGFILELIST *pstuInParam, NET_OUT_GET_RINGFILELIST *pstuOutParam, int nWaitTime);

// CLIENT_GetConnetionStatus in parameter
typedef struct tagNET_IN_GETCONNECTION_STATUS
{
    DWORD       dwSize;                                               
}NET_IN_GETCONNECTION_STATUS;

// CLIENT_GetConnetionStatus out parameter
typedef struct tagNET_OUT_GETCONNECTION_STATUS
{
    DWORD       dwSize;  
    int         nChannelNum;                                           // channel number
    int         nStatus[MAX_ALARMEXT_MODULE_NUM];                      // 0:not distributed 1:offfline 2:online
}NET_OUT_GETCONNECTION_STATUS;

// Get Alarm ext module status
CLIENT_NET_API BOOL CALL_METHOD CLIENT_GetConnectionStatus(LLONG lLoginID,const NET_IN_GETCONNECTION_STATUS* pstuInParam, NET_OUT_GETCONNECTION_STATUS *pstuOutParam, int nWaitTime);

#define MAX_ALARM_BOX_VIDEO_NUM		16

// CLIENT_GetAlarmBoxVideoConnectionStatus in parameter
typedef struct tagNET_IN_GETALARMBOXVIDEOCONNECTION_STATUS
{
    DWORD       dwSize;                                               
} NET_IN_GETALARMBOXVIDEOCONNECTION_STATUS;

// CLIENT_GetAlarmBoxVideoConnectionStatus out parameter
typedef struct tagNET_OUT_GETALARMBOXVIDEOCONNECTION_STATUS
{
    DWORD       dwSize;  
    int         nChannelNum;                                        // alarm box video channel number
    int         nStatus[MAX_ALARM_BOX_VIDEO_NUM];                   // alarm box video channel , 0:disconnect 1:offline 2:online
} NET_OUT_GETALARMBOXVIDEOCONNECTION_STATUS;

// Get Alarm Box Video connection status
CLIENT_NET_API BOOL CALL_METHOD CLIENT_GetAlarmBoxVideoConnectionStatus(LLONG lLoginID,const NET_IN_GETALARMBOXVIDEOCONNECTION_STATUS* pstuInParam, NET_OUT_GETALARMBOXVIDEOCONNECTION_STATUS *pstuOutParam, int nWaitTime);

//authority
typedef enum tagNET_ATTENDANCE_AUTHORITY
{
    NET_ATTENDANCE_AUTHORITY_UNKNOWN = -1,
    NET_ATTENDANCE_AUTHORITY_CUSTOMER,           //customer
    NET_ATTENDANCE_AUTHORITY_ADMINISTRATORS,     //administrators
}NET_ATTENDANCE_AUTHORITY;

#define MAX_ATTENDANCE_USERNAME_LEN 36

//attendance user info
typedef struct tagNET_ATTENDANCE_USERINFO
{
    char                     szUserID[MAX_COMMON_STRING_32];   //user id   
    char                     szUserName[MAX_ATTENDANCE_USERNAME_LEN]; //username
    char                     szCardNo[MAX_COMMON_STRING_32];   // cardno
    NET_ATTENDANCE_AUTHORITY emAuthority;                      // authority
    char                     szPassword[MAX_COMMON_STRING_32]; // password
    int                      nPhotoLength;                     // photo data len   
	BYTE					 byReserved[256];				   // reserved bytes    
}NET_ATTENDANCE_USERINFO;

//CLIENT_Attendance_InsertFingerByUserID input Parameter
typedef struct tagNET_IN_FINGERPRINT_INSERT_BY_USERID
{
	DWORD						dwSize;
    char 						szUserID[DH_COMMON_STRING_32];			//  user ID
    int 						nSinglePacketLen;						// length of single packet
    int 						nPacketCount;							// count of packet
	char*						szFingerPrintInfo;						// count is nSinglePacketLen*nPacketCount)
}NET_IN_FINGERPRINT_INSERT_BY_USERID;


//CLIENT_Attendance_InsertFingerByUserID output Parameter
typedef struct tagNET_OUT_FINGERPRINT_INSERT_BY_USERID
{
	DWORD						dwSize;
    int							nFingerPrintID[DH_MAX_FINGER_PRINT];	// fingerprint' ID array
	int 						nReturnedCount;							// the count of actually useful in array.
	int							nFailedCode;							// failed code 0:success  1:other error  2:limits on the fingerprint capabilities of this user
}NET_OUT_FINGERPRINT_INSERT_BY_USERID;

//CLIENT_Attendance_RemoveFingerByUserID input param
typedef struct tagNET_CTRL_IN_FINGERPRINT_REMOVE_BY_USERID
{
	DWORD						dwSize;
	char 						szUserID[DH_COMMON_STRING_32];			// User ID
}NET_CTRL_IN_FINGERPRINT_REMOVE_BY_USERID;

//CLIENT_Attendance_RemoveFingerByUserID output param
typedef struct tagNET_CTRL_OUT_FINGERPRINT_REMOVE_BY_USERID
{
	DWORD						dwSize;
}NET_CTRL_OUT_FINGERPRINT_REMOVE_BY_USERID;

//CLIENT_Attendance_RemoveFingerRecord input param
typedef struct tagNET_CTRL_IN_FINGERPRINT_REMOVE
{
	DWORD						dwSize;
	int							nFingerPrintID;							//fingerprint ID
}NET_CTRL_IN_FINGERPRINT_REMOVE;

//CLIENT_Attendance_RemoveFingerRecord output param
typedef struct tagNET_CTRL_OUT_FINGERPRINT_REMOVE
{
	DWORD						dwSize;
}NET_CTRL_OUT_FINGERPRINT_REMOVE;

//CLIENT_Attendance_GetFingerRecord input param
typedef struct tagNET_CTRL_IN_FINGERPRINT_GET
{
	DWORD						dwSize;
	int							nFingerPrintID;							//fingerprint ID
}NET_CTRL_IN_FINGERPRINT_GET;

//CLIENT_Attendance_GetFingerRecord output param
typedef struct tagNET_CTRL_OUT_FINGERPRINT_GET
{
	DWORD						dwSize;
	char 						szUserID[DH_COMMON_STRING_32];			//	user ID
	char 						szFingerPrintName[DH_COMMON_STRING_32]; // name of fingerprint
	int							nFingerPrintID;							// fingerprint ID
	int							nRetLength;								// the returned length of fingerprint's binary data.
	int							nMaxFingerDataLength;					// the max length of fingerprint's binary data.
	char*						szFingerPrintInfo;						// binary data of fingerprint.
}NET_CTRL_OUT_FINGERPRINT_GET;

//CLIENT_Attendance_FindUser input parameter
typedef struct tagNET_IN_ATTENDANCE_FINDUSER
{
	DWORD			dwSize;
	int				nOffset;										// query offset
	int				nPagedQueryCount;								// query countpaged queryNo more than 100
}NET_IN_ATTENDANCE_FINDUSER;

//CLIENT_Attendance_FindUser output parameter
typedef struct tagNET_OUT_ATTENDANCE_FINDUSER
{
	DWORD 							dwSize;
	int								nTotalUser;						// the total of users.
	int								nMaxUserCount;					// max number of user infors
	NET_ATTENDANCE_USERINFO*       	stuUserInfo;            		// user information; alloc memory by user, size is (sizeof(NET_ATTENDANCE_USERINFO)*nMaxUserCount) 
	int								nRetUserCount;					// returned user infor counts.
	int                           	nMaxPhotoDataLength;        	// max photo data len
	int								nRetPhoteLength;				// returned length of photo data
	BYTE *                        	pbyPhotoData;           		// photo data
}NET_OUT_ATTENDANCE_FINDUSER;

//CLIENT_Attendance_GetFingerByUserID input parameter
typedef struct tagNET_IN_FINGERPRINT_GETBYUSER
{
	DWORD						 dwSize;
	char                         szUserID[MAX_COMMON_STRING_32];      //user ID 
}NET_IN_FINGERPRINT_GETBYUSER;

//CLIENT_Attendance_GetFingerByUserID output parameter
typedef struct tagNET_OUT_FINGERPRINT_GETBYUSER
{
	DWORD						dwSize;
	int							nFingerPrintIDs[DH_MAX_FINGER_PRINT];	// array of fingerprint'ID
	int							nRetFingerPrintCount;					// returned count of fingerprint
	int							nSinglePacketLength;					// the length of single fingerprint packet data
	int							nMaxFingerDataLength;					// the max length of photo data
	int							nRetFingerDataLength;					// returned length of photo data
	BYTE*                       pbyFingerData;                       	// photo data	
}NET_OUT_FINGERPRINT_GETBYUSER;


// CLIENT_Attendance_AddUser  input parameter
typedef struct tagNET_IN_ATTENDANCE_ADDUSER
{
    DWORD                      dwSize;  
    NET_ATTENDANCE_USERINFO    stuUserInfo;           //user info
    BYTE                       *pbyPhotoData;         // photo data
}NET_IN_ATTENDANCE_ADDUSER;

// CLIENT_Attendance_AddUser output parameter
typedef struct tagNET_OUT_ATTENDANCE_ADDUSER
{
    DWORD         dwSize; 
}NET_OUT_ATTENDANCE_ADDUSER;

// attendance add user
CLIENT_NET_API BOOL CALL_METHOD CLIENT_Attendance_AddUser(LLONG lLoginID, NET_IN_ATTENDANCE_ADDUSER *pstuInAddUser, NET_OUT_ATTENDANCE_ADDUSER *pstuOutAddUser, int nWaitTime);

// CLIENT_Attendance_DelUser input parameter
typedef struct tagNET_IN_ATTENDANCE_DELUSER
{
    DWORD                      dwSize;  
    char                       szUserID[MAX_COMMON_STRING_32];      //userid
}NET_IN_ATTENDANCE_DELUSER;

// CLIENT_Attendance_DelUser output parameter
typedef struct tagNET_OUT_ATTENDANCE_DELUSER
{
    DWORD         dwSize;   
}NET_OUT_ATTENDANCE_DELUSER;

//attendance delete user
CLIENT_NET_API BOOL CALL_METHOD CLIENT_Attendance_DelUser(LLONG lLoginID, NET_IN_ATTENDANCE_DELUSER *pstuInDelUser, NET_OUT_ATTENDANCE_DELUSER *pstuOutDelUser, int nWaitTime);

// CLIENT_Attendance_ModifyUser  input parameter
typedef struct tagNET_IN_ATTENDANCE_ModifyUSER
{
    DWORD                      dwSize;  
    NET_ATTENDANCE_USERINFO    stuUserInfo;                         //user info
    BYTE                       *pbyPhotoData;                       // photo data
}NET_IN_ATTENDANCE_ModifyUSER;

// CLIENT_Attendance_ModifyUser output parameter
typedef struct tagNET_OUT_ATTENDANCE_ModifyUSER
{
    DWORD         dwSize;   
}NET_OUT_ATTENDANCE_ModifyUSER;

//attendance modify user
CLIENT_NET_API BOOL CALL_METHOD CLIENT_Attendance_ModifyUser(LLONG lLoginID, NET_IN_ATTENDANCE_ModifyUSER *pstuInModifyUser, NET_OUT_ATTENDANCE_ModifyUSER *pstuOutModifyUser, int nWaitTime);

// CLIENT_Attendance_GetUser  input parameter
typedef struct tagNET_IN_ATTENDANCE_GetUSER
{
    DWORD                        dwSize;  
    char                         szUserID[MAX_COMMON_STRING_32];      //userid
}NET_IN_ATTENDANCE_GetUSER;

// CLIENT_Attendance_GetUser output parameter
typedef struct tagNET_OUT_ATTENDANCE_GetUSER
{
    DWORD                         dwSize;   
    NET_ATTENDANCE_USERINFO       stuUserInfo;            //user info    
    int                           nMaxLength;             // max photo data len
    BYTE *                        pbyPhotoData;           // photo data
}NET_OUT_ATTENDANCE_GetUSER;

//attendance get user
CLIENT_NET_API BOOL CALL_METHOD CLIENT_Attendance_GetUser(LLONG lLoginID, NET_IN_ATTENDANCE_GetUSER *pstuInGetUser, NET_OUT_ATTENDANCE_GetUSER *pstuOutGetUser, int nWaitTime);

//attendance insert fingerprint by userID
CLIENT_NET_API BOOL CALL_METHOD CLIENT_Attendance_InsertFingerByUserID(LLONG lLoginID, NET_IN_FINGERPRINT_INSERT_BY_USERID* pstuInInsert, NET_OUT_FINGERPRINT_INSERT_BY_USERID* pstuOutInsert, int nWaitTime);

//attendance remove all fingerprint of one user
CLIENT_NET_API BOOL CALL_METHOD	CLIENT_Attendance_RemoveFingerByUserID(LLONG lLoginID, NET_CTRL_IN_FINGERPRINT_REMOVE_BY_USERID* pstuInRemove, NET_CTRL_OUT_FINGERPRINT_REMOVE_BY_USERID* pstuOutRemove, int nWaitTime);

//attendance get fingerprint record
CLIENT_NET_API BOOL CALL_METHOD	CLIENT_Attendance_GetFingerRecord(LLONG lLoginID, NET_CTRL_IN_FINGERPRINT_GET* pstuInGet, NET_CTRL_OUT_FINGERPRINT_GET* pstuOutGet, int nWaitTime);

//attendance remove fingerprint record.
CLIENT_NET_API BOOL CALL_METHOD CLIENT_Attendance_RemoveFingerRecord(LLONG lLoginID, NET_CTRL_IN_FINGERPRINT_REMOVE* pstuInRemove, NET_CTRL_OUT_FINGERPRINT_REMOVE* pstuOutRemove, int nWaitTime);

//attendance find user
CLIENT_NET_API BOOL CALL_METHOD CLIENT_Attendance_FindUser(LLONG lLoginID, NET_IN_ATTENDANCE_FINDUSER *pstuInFindUser, NET_OUT_ATTENDANCE_FINDUSER *pstuOutFindUser, int nWaitTime);

//attendance get fingerprint record by userID
CLIENT_NET_API BOOL CALL_METHOD CLIENT_Attendance_GetFingerByUserID(LLONG lLoginID, NET_IN_FINGERPRINT_GETBYUSER *pstuIn, NET_OUT_FINGERPRINT_GETBYUSER *pstuOut, int nWaitTime);


// Device Discovery subclassID type
typedef enum tagEM_NET_DEVICE_DISCOVERY_SUBCLASSID_TYPE
{
	EM_NET_DEVICE_DISCOVERY_SUBCLASSID_TYPE_UNKNOWN = -1,					// unknown
    EM_NET_DEVICE_DISCOVERY_SUBCLASSID_TYPE_NONE = 0,						// private
	EM_NET_DEVICE_DISCOVERY_SUBCLASSID_TYPE_ONVIF,                          // Onvif 
	EM_NET_DEVICE_DISCOVERY_SUBCLASSID_TYPE_UPNP,                           // UPnP 
}EM_NET_DEVICE_DISCOVERY_SUBCLASSID_TYPE;

// fAttachDeviceDiscoveryCB parameter
typedef struct tagNET_DEVICE_DISCOVERY_INFO
{
	DWORD				dwSize;
	char				szMachineName[DH_COMMON_STRING_32];					// MachineName
	char				szDeviceClass[DH_COMMON_STRING_32];					// DeviceClass
	char				szDeviceType[DH_COMMON_STRING_32];					// DeviceType
	char				szSerialNo[DH_COMMON_STRING_32];					// SerialNo
	char				szVersion[DH_COMMON_STRING_32];						// Version
	char				szMac[DH_COMMON_STRING_32];							// Mac Address
	int					nVideoInputChannels;								// VideoInputChannels
	int					nRemoteVideoInputChannels;							// RemoteVideoInputChannels
	int					nVideoOutputChannels;								// VideoOutputChannels
	int					nAlarmInputChannels;								// AlarmInputChannels
	int					nAlarmOutputChannels;								// AlarmOutputChannels
	int					nAudioInputChannels;								// AudioInputChannels
	int					nPort;												// Port
	char				szVendor[DH_COMMON_STRING_32];						// Vendor
	int					nInit;												// Init:0 donot have an initialization feature, 1 uninitialized, 2 initialized
	char				szIPv4Address[DH_COMMON_STRING_32];					// IPv4 Address
	char				szIPv4SubnetMask[DH_COMMON_STRING_32];				// IPv4 SubnetMask
	char				szIPv4DefaultGateway[DH_COMMON_STRING_32];			// IPv4 DefaultGateway
	BOOL				bIPv4DhcpEnable;									// IPv4 DhcpEnable
	char				szIPv6Address[DH_COMMON_STRING_32];					// IPv6 Address
	char				szIPv6DefaultGateway[DH_COMMON_STRING_32];			// IPv6 DefaultGateway
	char				szIPv6LinkLocalAddress[DH_COMMON_STRING_32];		// IPv6 LinkLocalAddress
	BOOL				bIPv6DhcpEnable;									// IPv6 DhcpEnable
	char				szProtocol[DH_COMMON_STRING_32];					// Protocol
}NET_DEVICE_DISCOVERY_INFO;


// Attach Device Discovery callback function,lAttachHandle is the returned value of CLIENT_AttachDeviceDiscovery, pDeviceInfo is the returned value of deviceInfo, nDeivceNum is the number of  pDeviceInfo
typedef void (CALLBACK *fAttachDeviceDiscoveryCB) (LLONG lAttachHandle, NET_DEVICE_DISCOVERY_INFO* pDeviceInfo, int nDeviceNum, LDWORD dwUser);


// CLIENT_AttachDeviceDiscovery input parameter
typedef struct tagNET_IN_ATTACH_DEVICE_DISCOVERY
{
    DWORD											dwSize;					
    EM_NET_DEVICE_DISCOVERY_SUBCLASSID_TYPE 		emSubClassID;			// subclassID
	fAttachDeviceDiscoveryCB						cbNotify;				// callback function 
	LDWORD											dwUser;					// User Data
} NET_IN_ATTACH_DEVICE_DISCOVERY;

// CLIENT_AttachDeviceDiscovery output parameter
typedef struct tagNET_OUT_ATTACH_DEVICE_DISCOVERY
{
    DWORD											dwSize;
} NET_OUT_ATTACH_DEVICE_DISCOVERY;

// CLIENT_StartDeviceDiscovery input parameter
typedef struct tagNET_IN_START_DEVICE_DISCOVERY 
{
    DWORD											dwSize;
	EM_NET_DEVICE_DISCOVERY_SUBCLASSID_TYPE 		emSubClassID;			// subclassID
    int												nTimeOut;				// timeout, unit:s
} NET_IN_START_DEVICE_DISCOVERY;

// CLIENT_StartDeviceDiscovery output parameter
typedef struct tagNET_OUT_START_DEVICE_DISCOVERY 
{
    DWORD											dwSize;
} NET_OUT_START_DEVICE_DISCOVERY;

// CLIENT_StopDeviceDiscovery input parameter
typedef struct tagNET_IN_STOP_DEVICE_DISCOVERY 
{
    DWORD											dwSize;
	EM_NET_DEVICE_DISCOVERY_SUBCLASSID_TYPE 		emSubClassID;			// SubClassID
} NET_IN_STOP_DEVICE_DISCOVERY;

// CLIENT_StopDeviceDiscovery output parameter
typedef struct tagNET_OUT_STOP_DEVICE_DISCOVERY 
{
    DWORD											dwSize;
} NET_OUT_STOP_DEVICE_DISCOVERY;

// Attach Device Discovery
CLIENT_NET_API LLONG CALL_METHOD CLIENT_AttachDeviceDiscovery(LLONG lLoginID, const NET_IN_ATTACH_DEVICE_DISCOVERY *pInParam, NET_OUT_ATTACH_DEVICE_DISCOVERY *pOutParam, int nWaitTime);

// Detach Device Discovery, lAttachHandle is the returned handle of CLIENT_AttachDeviceDiscovery
CLIENT_NET_API BOOL CALL_METHOD CLIENT_DetachDeviceDiscovery(LLONG lAttachHandle);

// Start Device Discovery
CLIENT_NET_API BOOL CALL_METHOD CLIENT_StartDeviceDiscovery(LLONG lLoginID, const NET_IN_START_DEVICE_DISCOVERY *pInParam, NET_OUT_START_DEVICE_DISCOVERY *pOutParam, int nWaitTime);

// Stop Device Discovery
CLIENT_NET_API BOOL CALL_METHOD CLIENT_StopDeviceDiscovery(LLONG lLoginID, const NET_IN_STOP_DEVICE_DISCOVERY *pInParam, NET_OUT_STOP_DEVICE_DISCOVERY *pOutParam, int nWaitTime);

///////////////////////////////////deprecated///////////////////////////////////

// Search system server setup. This interface is invalid now please use  CLIENT_GetDevConfig
CLIENT_NET_API BOOL CALL_METHOD CLIENT_QueryConfig(LLONG lLoginID, int nConfigType, char *pConfigbuf, int maxlen, int *nConfigbuflen, int waittime=1000);

// Set system server setup. This interface is invalid now please use  CLIENT_SetDevConfig
CLIENT_NET_API BOOL CALL_METHOD CLIENT_SetupConfig(LLONG lLoginID, int nConfigType, char *pConfigbuf, int nConfigbuflen, int waittime=1000);

// This interface is invalid now. 
CLIENT_NET_API BOOL CALL_METHOD CLIENT_Reset(LLONG lLoginID, BOOL bReset);

// Search COM protocol. This interface is invalid now please use  CLIENT_GetDevConfig
CLIENT_NET_API BOOL CALL_METHOD CLIENT_QueryComProtocol(LLONG lLoginID, int nProtocolType, char *pProtocolBuffer, int maxlen, int *nProtocollen, int waittime=1000);

// Begin audio talk. This interface is invalid now. Please use  CLIENT_StartTalkEx
CLIENT_NET_API BOOL CALL_METHOD CLIENT_StartTalk(LLONG lRealHandle, BOOL bCustomSend=false);

// Stop audio talk. This interface is invalid now , please use  CLIENT_StopTalkEx
CLIENT_NET_API BOOL CALL_METHOD CLIENT_StopTalk(LLONG lRealHandle);

// Send out self-defined audio talk data. This interface is invalid now, please use  CLIENT_TalkSendData
CLIENT_NET_API BOOL CALL_METHOD CLIENT_SendTalkData_Custom(LLONG lRealHandle, char *pBuffer, DWORD dwBufSize);

// Set real-time preview buffer size
CLIENT_NET_API BOOL CALL_METHOD CLIENT_SetPlayerBufNumber(LLONG lRealHandle, DWORD dwBufNum);

// Download file by time
CLIENT_NET_API BOOL CALL_METHOD CLIENT_GetFileByTime(LLONG lLoginID, int nChannelID, LPNET_TIME lpStartTime, LPNET_TIME lpStopTime, char *sSavedFileName);

// Network playback control 
CLIENT_NET_API BOOL CALL_METHOD CLIENT_PlayBackControl(LLONG lPlayHandle, DWORD dwControlCode, DWORD dwInValue, DWORD *lpOutValue);

// Search device working status .This interface is invalid now, please use  CLIENT_QueryDevState
CLIENT_NET_API BOOL CALL_METHOD CLIENT_GetDEVWorkState(LLONG lLoginID, LPNET_DEV_WORKSTATE lpWorkState, int waittime=1000);

// Asynchronism search device log 
CLIENT_NET_API BOOL CALL_METHOD CLIENT_QueryLogCallback(LLONG lLoginID, fLogDataCallBack cbLogData, LDWORD dwUser);

// open or close record for mpt300,user malloc and free memory of pstRecIn and pstRecOut
CLIENT_NET_API BOOL CALL_METHOD CLIENT_RecMngCtrlMpt300(LLONG lLoginID, const NET_IN_REC_MNG_CTRL_MPT300 *pstRecIn, NET_OUT_REC_MNG_CTRL_MPT300 *pstRecOut, int nWaitTime);

// Transmit Info Directly,user malloc and free memory of pstInTransmit and pstOutTransmit
CLIENT_NET_API BOOL CALL_METHOD CLIENT_TransmitInfoDirectly(LLONG lLoginID, const NET_IN_TRANSMIT_DIRECTLY *pstInTransmit, NET_OUT_TRANSMIT_DIRECTLY *pstOutTransmit, int nWaittime = 1000);

// call back for Transmit Info Directly
typedef void (CALLBACK *fTransmitInfoDirectlyCallBack)(LLONG lLoginID, LLONG lStartHandle, const char *pszOutBuf, DWORD dwBufLen, LDWORD dwUser);

// start Transmit Info Directly,user malloc and free memory of pstInTransmit and pstOutTransmit
CLIENT_NET_API LLONG CALL_METHOD CLIENT_StartTransmitInfoDirectly(LLONG lLoginID, const NET_IN_TRANSMIT_DIRECTLY *pstInTransmit, NET_OUT_TRANSMIT_DIRECTLY *pstOutTransmit, fTransmitInfoDirectlyCallBack cbDirectly, LDWORD dwUser, int nWaitTime = 1000);

// stop Transmit Info Directly,user malloc and free memory of pstInTransmit and pstOutTransmit
CLIENT_NET_API BOOL CALL_METHOD CLIENT_StopTransmitInfoDirectly(LLONG lStartHandle, const NET_IN_TRANSMIT_DIRECTLY *pstInTransmit, NET_OUT_TRANSMIT_DIRECTLY *pstOutTransmit, int nWaitTime = 1000);

// Set Defence Arm Mode,user malloc and free memory of pInBuf and pOutBuf
CLIENT_NET_API BOOL CALL_METHOD CLIENT_SetDefenceArmMode(LLONG lLoginID, NET_IN_SET_DEFENCEMODE* pInBuf, NET_OUT_SET_DEFENCEMODE* pOutBuf = NULL, int nWaitTime = 1000);

// Get Defence Arm Mode,user malloc and free memory of pInBuf and pOutBuf
CLIENT_NET_API BOOL CALL_METHOD CLIENT_GetDefenceArmMode(LLONG lLoginID, NET_IN_GET_DEFENCEMODE* pInBuf, NET_OUT_GET_DEFENCEMODE* pOutBuf, int nWaitTime = 1000);

// Set SubSystem Arm Mode,user malloc and free memory of pInBuf and pOutBuf
CLIENT_NET_API BOOL CALL_METHOD CLIENT_SetSubSystemArmMode(LLONG lLoginID, NET_IN_SET_SUBSYSTEMMODE* pInBuf, NET_OUT_SET_SUBSYSTEMMODE* pOutBuf = NULL, int nWaitTime = 1000);

// Get SubSystem Arm Mode,user malloc and free memory of pInBuf and pOutBuf
CLIENT_NET_API BOOL CALL_METHOD CLIENT_GetSubSystemArmMode(LLONG lLoginID, NET_IN_GET_SUBSYSTEMMODE* pInBuf, NET_OUT_GET_SUBSYSTEMMODE* pOutBuf, int nWaitTime = 1000);

// set mobile push notification, use RegisterID to identify the config info,user malloc and free pstuCfg's memory
CLIENT_NET_API BOOL CALL_METHOD CLIENT_SetMobilePushNotify(LLONG lLoginID, const NET_MOBILE_PUSH_NOTIFY *pstuCfg, int *nError, int *nRestart, int nWaitTime = 1000);

// delete mobile push notification, use RegisterID to identify the config info,user malloc and free memory of pstuIn and pstuOut
CLIENT_NET_API BOOL CALL_METHOD CLIENT_DelMobilePushNotify(LLONG lLoginID, const NET_MOBILE_PUSH_NOTIFY_DEL *pstuIn, NET_OUT_DELETECFG* pstuOut, int nWaitTime = 1000);

// set mobile push notification config, use RegisterID+AppID to identify the info, AppID cannot include ".", use "_" instead of ".",user malloc and free pstuCfg's memory
CLIENT_NET_API BOOL CALL_METHOD CLIENT_SetMobilePushNotifyCfg(LLONG lLoginID, const NET_MOBILE_PUSH_NOTIFY_CFG *pstuCfg, int *nError, int *nRestart, int nWaitTime = 1000);

// delete mobile push notification config, use RegisterID+AppID to identify the info, AppID cannot include ".", use "_" instead of ".",user malloc and free memory of pstuIn and pstuOut
CLIENT_NET_API BOOL CALL_METHOD CLIENT_DelMobilePushNotifyCfg(LLONG lLoginID, const NET_MOBILE_PUSH_NOTIFY_CFG_DEL *pstuIn, NET_OUT_DELETECFG* pstuOut, int nWaitTime = 1000);


// batch insert access control cards
// nInsertCount: number of cards to insert, no more than 20
// pInsertCards: cards to insert, NET_RECORDSET_ACCESS_CTL_CARD buffer allocated and filled by user, amount to nInsertCount
// nRecNo: record number after inserted (-1 for failed), int buffer allocated by user, coresponding to pInsertCards one by one, amount to nInsertCount
CLIENT_NET_API BOOL CALL_METHOD CLIENT_InsertAccessControlCards(LLONG lLoginID, int nInsertCount, const NET_RECORDSET_ACCESS_CTL_CARD* pInsertCards, int* pRecNo, void* reserved = NULL, int nWaitTime = NET_INTERFACE_DEFAULT_TIMEOUT);

//face image compare info
typedef struct tagNET_IMAGE_COMPARE_INFO
{
	DWORD 			   	    dwoffset;			// offset in binary data,unit:byte
	DWORD					dwLength;			// Image length,unit:byte
	DWORD					dwWidth;			// image width
	DWORD					dwHeight;			// image height
	BYTE                    byReserved[128];    // Reserved
} NET_IMAGE_COMPARE_INFO;

//CLIENT_MatchTwoFace input param
typedef struct tagNET_MATCH_TWO_FACE_IN
{
	DWORD					dwSize;
	NET_IMAGE_COMPARE_INFO  stuOriginalImage;	// original image
	NET_IMAGE_COMPARE_INFO  stuCompareImage;	// compare image
	char 			   	   *pSendBuf;			// two face image data
	DWORD 					dwSendBufLen;		// data length
} NET_MATCH_TWO_FACE_IN;

//CLIENT_MatchTwoFace output param
typedef struct tagNET_MATCH_TWO_FACE_OUT
{
	DWORD					dwSize;
	int						nSimilarity;		//Similarity ( expressed as a percentage, from 1 to 100)
} NET_MATCH_TWO_FACE_OUT;

// calculate the similarity of two face images,user malloc and free memory of pstInParam and pstOutParam
CLIENT_NET_API BOOL  CALL_METHOD CLIENT_MatchTwoFaceImage(LLONG lLoginID, const NET_MATCH_TWO_FACE_IN *pstInParam, NET_MATCH_TWO_FACE_OUT *pstOutParam, int nWaitTime = NET_INTERFACE_DEFAULT_TIMEOUT);

// the decode frame type
typedef enum tagEM_DECODE_FRAME_TYPE
{
    EM_FRAME_TYPE_UNKNOWN   = -1,               // unknown
    EM_FRAME_TYPE_VIDEO     = 0,				// video
    EM_FRAME_TYPE_AUDIO     = 1,				// audio
}EM_DECODE_FRAME_TYPE;

// the ex frame info
typedef struct tagNET_FRAME_INFO_EX
{
    DWORD                   dwSize;
    EM_DECODE_FRAME_TYPE    emFrameType;			//frame type
    int				        nFrameSeq;				//frame index
    int				        nStamp;					//stamp info,in ms
    int				        nWidth;					//frame width,it is zero when frame type is audio
    int 			        nHeight;				//frame height,it is zero when frame type is audio
    int				        nFrameRate;				//frame rate
    int				        nChannels;				//channel num
    int				        nBitPerSample;			//bit per sample
    int				        nSamplesPerSec;			//sample per second
    int				        nRemainData;			//length of buffer data
    NET_TIME_EX		        nDataTime;				//frame time info

}NET_FRAME_INFO_EX;

// the decode frame info
typedef struct tagNET_FRAME_DECODE_INFO 
{
    DWORD                   dwSize;
    EM_DECODE_FRAME_TYPE    emFrameType;			//frame type

    void*			        pAudioData;				//audio data
    int				        nAudioDataLen;			//audio data length

    void*			        pVideoData[3];			//YUV data
    int				        nStride[3];				//stride of YUV
    int				        nWidth[3];				//width of YUV
    int				        nHeight[3];				//height of YUV  
}NET_FRAME_DECODE_INFO;

// the call back function type for decode of frame
typedef void (CALLBACK *fDecCallBack)(LLONG lLoginID, LLONG lPlayHandle, NET_FRAME_DECODE_INFO* pFrameDecodeInfo, NET_FRAME_INFO_EX* pFrameInfo, LDWORD dwUserData, LLONG nReserved);

// set the call back function for decode of frame
CLIENT_NET_API void CALL_METHOD CLIENT_SetDecCallBack(fDecCallBack cbDecCallBack, LDWORD dwUserData, LLONG nReserved = 0);

//  set the call back function for decode of frame ---- extend
CLIENT_NET_API void CALL_METHOD CLIENT_SetDecCallBackEx(fDecCallBack cbDecCallBack, LDWORD dwUserData, NET_VIDEOSTREAM_TYPE emVideoStreamType, LLONG nReserved);

// get default config of device(only support EncodePlane config now, emCfgOpType == NET_EM_CFG_ENCODE_PLAN)
CLIENT_NET_API BOOL CALL_METHOD CLIENT_GetDevDefaultConfig(LLONG lLoginID, NET_EM_CFG_OPERATE_TYPE emCfgOpType, int nChannelID,
                                          LPVOID lpOutBuffer, DWORD dwOutBufferSize, int nWaittime/*=NET_INTERFACE_DEFAULT_TIMEOUT*/);

#define NET_CUSTOM_PARKING_SPACE_NUMBER_MAX_LEN 32                        // the max length of car custom  lane

// the light color of parking space
typedef enum tagEM_PARKINGSPACE_LIGHT_COLOR
{
    EM_PARKINGSPACE_LIGHT_COLOR_UNKNOWN,                                    // Unknown
    EM_PARKINGSPACE_LIGHT_COLOR_RED,                                        // red
    EM_PARKINGSPACE_LIGHT_COLOR_YELLOW,                                     // yellow    
    EM_PARKINGSPACE_LIGHT_COLOR_GREEN,                                      // green 
}EM_PARKINGSPACE_LIGHT_COLOR;

// the light state of parking space
typedef enum tagEM_PARKINGSPACE_LIGHT_STATE
{
    EM_PARKINGSPACE_LIGHT_STATE_UNKNOWN = -1,                               // Unknown  
    EM_PARKINGSPACE_LIGHT_STATE_OFF,                                        // off
    EM_PARKINGSPACE_LIGHT_STATE_ON,                                         // on
    EM_PARKINGSPACE_LIGHT_STATE_GLINT,                                      // glint
    EM_PARKINGSPACE_LIGHT_STATE_LAST_STATE,                                 // last state
}EM_PARKINGSPACE_LIGHT_STATE;

// the light plan info of parking space
typedef struct tagNET_PARKING_SPACE_LIGHT_PLAN_INFO
{
    EM_PARKINGSPACE_LIGHT_COLOR emColor;                                    // light color
    EM_PARKINGSPACE_LIGHT_STATE emState;                                    // light state
    int nKeepTime;                                                          // state keep time,in second,-1 means light on all the time
    BYTE byReserved[128];                                                   // reserved
}NET_PARKING_SPACE_LIGHT_PLAN_INFO;

// the in param for set the light plan
typedef struct tagNET_IN_SET_PARKING_SPACE_LIGHT_PLAN
{
    DWORD dwSize;                                                           // struct size                                                           
    int nPhysicalLane;                                                      // physical lane,user should use this memeber prior.negative means this member is invalid
    char szCustomParkNo[NET_CUSTOM_PARKING_SPACE_NUMBER_MAX_LEN];           // custom lane number
    int nLightPlanNum;                                                      // the num of light plan
    NET_PARKING_SPACE_LIGHT_PLAN_INFO* pstuLightPlan;                       // the light plan data pointer
}NET_IN_SET_PARKING_SPACE_LIGHT_PLAN;

// the out param for set the light plan
typedef struct tagNET_OUT_SET_PARKING_SPACE_LIGHT_PLAN
{
    DWORD dwSize;                                                           // struct size
}NET_OUT_SET_PARKING_SPACE_LIGHT_PLAN;


// set the parking space light plan
CLIENT_NET_API BOOL CALL_METHOD CLIENT_SetParkingSpaceLightPlan(LLONG lLoginID,const NET_IN_SET_PARKING_SPACE_LIGHT_PLAN* pNetDataIn,NET_OUT_SET_PARKING_SPACE_LIGHT_PLAN* pNetDataOut,int nWaitTime);


/************************************************************************
 ** ObjectStructlize protocol interface
 ***********************************************************************/
//structlize event image info
typedef struct tagDEV_ALARM_OBJECTSTRUCTLIZE_PIC_INFO
{
    char                     szCutPicPath[MAX_PATH];          // the path of cut pictrue
    DWORD                    dwWidth;                         // picture width, pixel
    DWORD                    dwHeight;                        // picture height, pixel
    BYTE                     byReserved[256];                 // Reserved
} DEV_ALARM_OBJECTSTRUCTLIZE_PIC_INFO;

//objectstructlize person condition info or result info
typedef struct tagDEV_PERSON_CONDITION_RESULT_FEATURE_INFO 
{
    EM_DEV_EVENT_FACEDETECT_SEX_TYPE        emSex;                      // sex
    BYTE                                    byAge;                      // age
    BYTE                                    byCall;                     // calling        ,0:unknow 1:NO  2:YES
    BYTE                                    byHat;                      // with hat       ,0:unknow 1:NO  2:YES
    BYTE                                    byBag;                      // with bag       ,0:unknow 1:NO  2:YES
    BYTE                                    byUmbrella;                 // with Umbrella  ,0:unknow 1:NO  2:YES
    BYTE                                    byGlasses;                  // with Glasses   ,0:unknow 1:NO  2:YES
    BYTE                                    byReserved1[2];             
    EM_DEV_EVENT_FACEDETECT_FEATURE_TYPE    emEmotion;                  // human emotion:smile,anger, sadness, disgust, fear, surprise, neutral, laugh
    NET_COLOR_RGBA                          stuUpperBodyColor;          // upper body color
    NET_COLOR_RGBA                          stuLowerBodyColor;          // lower body color
    BYTE                                    byReserved[256];            // Reserved
} DEV_PERSON_CONDITION_FEATURE_INFO,DEV_PERSON_RESULT_FEATURE_INFO;

//person feature detect event(command:EVENT_IVS_OBJECTSTRUCTLIZE_PERSON)
typedef struct tagDEV_EVENT_OBJECTSTRUCTLIZE_PERSON_INFO
{
    int                                    nChannelID;                           // channel ID
    char                                   szName[DH_EVENT_NAME_LEN];            // event name
	BYTE                                   bReserved1[4];                        //
    double                                 dbPTS;                                // PTS(ms)
    NET_TIME_EX                            stuUTC;                               // time of occurrence
    int                                    nEventID;                             // event ID
    DH_EVENT_FILE_INFO                     stuFileInfo;                          // event file info
    int                                    nEventAction;                         // event action,0 pulse,1 durable events begin, 2 durable 

	char                                   szFilePath[MAX_PATH];                 // file path
    DEV_PERSON_RESULT_FEATURE_INFO         stuPersonFeatrue;                     // person feature
    NET_RECT                               stuBoundingBoxRect;                   // BoundingBox Rect, 0~8192
    DEV_ALARM_OBJECTSTRUCTLIZE_PIC_INFO    stuImageInfo;                         // cut image info

    int                                    nImageIndex;                          // Serial number of the picture, in the same time (accurate to seconds) may have multiple images, starting from 0
    DWORD                                  dwSnapFlagMask;                       // flag(by bit),see NET_RESERVED_COMMON      
    BYTE                                   bReserved[1024];                      // Reserved 
} DEV_EVENT_OBJECTSTRUCTLIZE_PERSON_INFO;

//nonmotor feature detect event(command:EVENT_IVS_OBJECTSTRUCTLIZE_NONMOTOR)
typedef struct tagDEV_EVENT_OBJECTSTRUCTLIZE_NONMOTOR_INFO
{
    int                                    nChannelID;                           // channel ID
    char                                   szName[DH_EVENT_NAME_LEN];            // event name
	BYTE                                   bReserved1[4];
    double                                 dbPTS;                                // PTS(ms)
    NET_TIME_EX                            stuUTC;                               // time of occurrence
    int                                    nEventID;                             // event ID
    DH_EVENT_FILE_INFO                     stuFileInfo;                          // event file info
    int                                    nEventAction;                         // event action,0 pulse,1 durable events begin, 2 durable 
    
	char                                   szFilePath[MAX_PATH];                 // file path
    char                                   szPlateNumber[DH_MAX_PLATE_NUMBER_LEN];// plate number 
    DEV_PERSON_RESULT_FEATURE_INFO         stuPersonFeatrue;                     // person feature
    NET_RECT                               stuBoundingBoxRect;                   // BoundingBox Rect, 0~8192
    DEV_ALARM_OBJECTSTRUCTLIZE_PIC_INFO    stuImageInfo;                         // cut image info

    int                                    nImageIndex;                          // Serial number of the picture, in the same time (accurate to seconds) may have multiple images, starting from 0
    DWORD                                  dwSnapFlagMask;                       // flag(by bit),see NET_RESERVED_COMMON      
    BYTE                                   bReserved[1024];                      // Reserved 
} DEV_EVENT_OBJECTSTRUCTLIZE_NONMOTOR_INFO;

//tumble detection info , Corresponding to DH_ALARM_TUMBLE_DETECTION alarm
typedef struct tagALARM_TUMBLE_DETECTION_INFO       
{     
	int									nAction;							// Event operation. 1: Start, 2: Stop
	int									nChannelID;							// Channel No.
    double								PTS;								// Time stamp (Unit is ms)
    NET_TIME_EX							UTC;								// Event occurrence time 
    int									nEventID;							// Event ID
	int									UTCMS;								// Millisecond	
	
	EM_CLASS_TYPE						emClassType;						// class type
	int									nObjectID;							// object ID
	char								szObjectType[DH_COMMON_STRING_16];	// Object Type, support for the following:
																			// "Unknown", "Human", "Vehicle","Fire", "Smoke", "Plate", "HumanFace", 
																			// "Container", "Animal", "TrafficLight", "PastePaper", "HumanHead", "BulletHole",  "Entity"
	NET_RECT							stuBoundingBox;						// Bounding Box
	BYTE                				byReserved[1024];   				// reserved  
}ALARM_TUMBLE_DETECTION_INFO; 

// tumble detection event (command:EVENT_IVS_TUMBLE_DETECTION)
typedef struct tagDEV_EVENT_TUMBLE_DETECTION_INFO
{
    int                     nChannelID;                             // channel ID
    char                    szName[DH_EVENT_NAME_LEN];              // event name
    char                    bReserved1[4];                          // reserved
    double                  PTS;                                    // PTS(ms)
    NET_TIME_EX             UTC;                                    // time of occurrence
    int                     nEventID;                               // event ID
	int						UTCMS;									// millseconds

	EM_CLASS_TYPE			emClassType;							// class type
	int						nObjectID;								// Object ID
	char					szObjectType[DH_COMMON_STRING_16];		// Object Type, support for the following:
																	// "Unknown", "Human", "Vehicle","Fire", "Smoke", "Plate", "HumanFace", 
																	// "Container", "Animal", "TrafficLight", "PastePaper", "HumanHead", "BulletHole",  "Entity"
	NET_RECT				stuBoundingBox;							// Bounding Box
	
	BYTE                    bReserved[1024];                        // reserved
} DEV_EVENT_TUMBLE_DETECTION_INFO;
// citizen picture compare image info
typedef struct tagCITIZEN_PICTURE_COMPARE_IMAGE_INFO
{
    DWORD           dwOffSet;                       // data offset, unit:byte
    DWORD           dwFileLenth;                    // file lenght, unit:byte
    WORD            wWidth;                         // width, unit:pixel
    WORD            wHeight;                        // height, unit:pixel
    BYTE            byReserved[256];                // reserved
}CITIZEN_PICTURE_COMPARE_IMAGE_INFO;

// citizen picture compare event,command:EVENT_IVS_CITIZEN_PICTURE_COMPARE
typedef struct tagDEV_EVENT_CITIZEN_PICTURE_COMPARE_INFO
{
    //common info
    int                 nChannelID;                         // channel number,start with 0
    int                 nEventAction;					    // event action, 0-pulse, -1-unknown
    double              dbPTS;                              // time stamp(unit:ms)
    char                szName[DH_EVENT_NAME_LEN];          // event name   
    NET_TIME_EX         stuUTC;                             // evnet time(UTC)
    int                 nEventID;                           // evnet id

    //evnet data
    bool                bCompareResult;                     // compare result,true:success,false:failed
    BYTE                nSimilarity;                        // Similarity,unit:%,rang:[1,100]
    BYTE                nThreshold;                         // Threshold,rang:[1,100]
    EM_CITIZENIDCARD_SEX_TYPE emSex;                        // sex
    int                 nEthnicity;                         // Ethnicity(see DEV_EVENT_ALARM_CITIZENIDCARD_INFO::nEthnicity)
    char                szCitizen[DH_COMMON_STRING_64];     // citizen name    
    char                szAddress[DH_COMMON_STRING_256];    // address
    char                szNumber[DH_COMMON_STRING_64];      // citizen number
    char                szAuthority[DH_COMMON_STRING_256];  // Authority
    NET_TIME            stuBirth;                           // Birth(y-m-d)
    NET_TIME            stuValidityStart;                   // Validity start time(y-m-d)
    BOOL                bLongTimeValidFlag;                 // When the value is TRUE, the stop date menas always valid, at this time, the stuValidityEnd is null. 
                                                            // When the value is FALSE, refer to stuValidityEnd value for stop date.
    NET_TIME            stuValidityEnd;                     // Validity end time(y-m-d)
    CITIZEN_PICTURE_COMPARE_IMAGE_INFO stuImageInfo[2];     // image infoelememnt 0 indicate snap picelememnt 1 indicate citizen pic

    BYTE                byReserved[1024];                   // reserved

}DEV_EVENT_CITIZEN_PICTURE_COMPARE_INFO;

// object find scope
typedef struct tagDEV_OBJECT_FIND_SCOPE_INFO
{
    BOOL                bEnable;                    // enable
	int                 nMinValue;                  // min value
	int                 nMaxValue;                  // max value
} DEV_OBJECT_FIND_SCOPE_INFO;

// sunshiled find scope
typedef struct tagDEV_OBJECT_SUNSHIELD_FIND_SCOPE_INFO
{
    BYTE                                   byFindType;                 // find type:1 find by number 2 find by seat(required,other value is invalid)
	int                                    nMinSunShieldNum;           // sun  shiled min numbers value(option)byFindType  is required 1
    int                                    nMaxSunShieldNum;           // sun  shiled max numbers value(option)byFindType  is required 1
	NET_SUNSHADE_STATE                     emMasterShield;             // master driver seat shield status(option)byFindType  is required 2
    NET_SUNSHADE_STATE                     emSlaveShield;              // slave driver seat shield status(option)byFindType  is required 2
} DEV_OBJECT_SUNSHIELD_FIND_SCOPE_INFO;

//vehicle structlize find condition 
typedef struct tagDEV_VEHICLE_CONDITION_FEATURE_INFO 
{
    char                                   szPlateNumber[DH_MAX_PLATE_NUMBER_LEN];        // plate number 
    EM_NET_PLATE_TYPE                      emPlateType;                                   // plate type
	NET_COLOR_RGBA                         stuPlateColor;                                 // plate color
    EM_NET_VEHICLE_TYPE                    emVehicleType;                                 // vehicle type
	NET_COLOR_RGBA                         stuVehicleColor;                               // vehicle color
    char                                   szVehicleLogo[32];                             // vehicle logo
    WORD                                   wVehicleSubBrand;                              // specifies the sub-brand of vehicle,the real value can be found in a mapping table from the development manual
    WORD                                   wVehicleYearModel;                             // specifies the model years of vehicle. the real value can be found in a mapping table from the development manual
    EM_VEHICLE_DIRECTION                   emVehicleDirection;                            // vehicle snapped direction

    NET_SAFEBELT_STATE                     emMainSeat;                                    // main driver seat belt status(option)
    NET_SAFEBELT_STATE                     emSlaveSeat;                                   // slave driver seat belt status(option)
	DEV_OBJECT_SUNSHIELD_FIND_SCOPE_INFO   stuSunShieldInfo;                              // sun shiled find scope info(option)
    DEV_OBJECT_FIND_SCOPE_INFO             stuDropInfo;                                   // drop find scope info(option)
    DEV_OBJECT_FIND_SCOPE_INFO             stuPaperBoxInfo;                               // paperBox find scope info(option)
    DEV_OBJECT_FIND_SCOPE_INFO             stuAnnualInspectionInfo;                       // Annual Inspection find scope info(option)   
    BYTE                                   byReserved[256];                               // reserved
} DEV_VEHICLE_CONDITION_FEATURE_INFO;

//vehicle structlize find result
typedef struct tagDEV_VEHICLE_RESULT_FEATURE_INFO 
{
    char                                   szPlateNumber[DH_MAX_PLATE_NUMBER_LEN];        // plate number 
    EM_NET_PLATE_TYPE                      emPlateType;                                   // plate type
	NET_COLOR_RGBA                         stuPlateColor;                                 // plate color
    NET_COLOR_RGBA                         stuVehicleColor;                               // vehicle color
    char                                   szVehicleLogo[32];                             // vehicle logo
    WORD                                   wVehicleSubBrand;                              // specifies the sub-brand of vehicle,the real value can be found in a mapping table from the development manual
    WORD                                   wVehicleYearModel;                             // specifies the model years of vehicle. the real value can be found in a mapping table from the development manual
    int                                    nVehicleWindowNum;                             // valid vehicle window numbers                                        
    NET_RECT                               stuVehicleWindow[4];                           // vehicle window positon,mark 0 corresponding to front vehicle window,
	                                                                                      // mark 1 corresponding to back vehicle window,others marks corresponding to side vehicle window
    EM_VEHICLE_DIRECTION                   emVehicleDirection;                            // vehicle snapped direction
    
    NET_SAFEBELT_STATE                     emMainSeat;                                    // main driver seat belt status
    NET_SAFEBELT_STATE                     emSlaveSeat;                                   // slave driver seat belt status
    NET_RECT                               stuMasterShield;                               // main driver seat shiled position
    NET_RECT                               stuSlaveShield;                                // slave driver seat shiled position
    int                                    nAnnualInspectionNum;                          // annual Inspection number                                        
    NET_RECT                               stuAnnualInspection[8];                        // annual Inspection position
    int                                    nDropNum;                                      // drop number                                       
    NET_RECT                               stuDrop[4];                                    // drop position
    int                                    nPaperBoxNum;                                  // paper box number                                        
    NET_RECT                               stuPaperBox[4];                                // paper box position
    BYTE                                   byReserved[256];                               // Reserved
} DEV_VEHICLE_RESULT_FEATURE_INFO;

// feature picture type
typedef enum tagEM_PIC_FEATURE_TYPE
{
    EM_PIC_FEATURE_TYPE_UNKOWN,            // unknow
    EM_PIC_FEATURE_TYPE_GLOBAL_SENCE,      // global scene
    EM_PIC_FEATURE_TYPE_SMALL,             // small
}EM_PIC_FEATURE_TYPE;

// image reuqest info
typedef struct tagDEV_IMAGE_REQUEST_FEATURE_INFO
{
    EM_PIC_FEATURE_TYPE      emPicType;            // picture type(required)
    DWORD                    dwoffset;             // offset in binary data,unit:byte
    DWORD                    dwLength;             // Image length,unit:byte
    DWORD                    dwWidth;              // image width
    DWORD                    dwHeight;             // image height
	BOOL                     bEnableRect;          // enable rect find
    NET_RECT                 stuRect;              // (option)mark object rect when picture type is Global Scene.(absolute coordinate)
    BYTE                     byReserved[256];      // Reserved
} DEV_IMAGE_REQUEST_FEATURE_INFO;

// image result info
typedef struct tagDEV_IMAGE_RESULT_FEATURE_INFO
{
    DWORD                   dwSimilarity;                   // Similarity ( expressed as a percentage, from 1 to 100)
    NET_TIME                stuTime;                        // time

	DWORD                   dwObjectOffset;                 // object image offset in binary data,unit:byte( depend on NeedData value in request)
	DWORD                   dwObjectLength;                 // object Image length,unit:byte(depend on NeedData value in request)
    char                    szObjectFilePath[MAX_PATH];     // object Image file path(depend on NeedData value in request)

	DWORD                   dwSceneOffset;                  // global scene image offset in binary data,unit:byte(depend on NeedData value in request)
	DWORD                   dwSceneLength;                  // global scene Image length,unit:byte(depend on NeedData value in request)
    char                    szSceneFilePath[MAX_PATH];      // global scene Image file path(depend on NeedData value in request)

    NET_RECT                stuObjectRect;                  // object rect(absolute coordinate)
    BYTE                    byReserved[256];                // Reserved
} DEV_IMAGE_RESULT_FEATURE_INFO;


// CLIENT_DetectObjectStructlize input parameter
typedef struct tagNET_IN_OBJECTSTRUCTLIZE_DETECT_PIC
{
    DWORD                                    dwSize; 
    int					                     nWidth;              // image width
    int                                      nHeight;             // image height
    char                                     *pBuffer;            // image binary buffer address
    int                                      nBufferLen;          // image binary buffer length
}NET_IN_OBJECTSTRUCTLIZE_DETECT_PIC;

// detected object info
typedef struct tagNET_OBJECT_DETECTED_INFO
{
    DH_POINT                         stuPoint;                       // object position,(absolute coordinate)
    DWORD                            dwWidth;                        // image width
    DWORD                            dwHeight;                       // image height
    char                             szObjectType[16];               // object type, refer to the same field of NET_OBJFILTER_INFO 
    char                             szObjectSubType[32];            // object sub type, refer to the same field of NET_OBJFILTER_INFO
    DEV_PERSON_RESULT_FEATURE_INFO   stuPersonResult;                // detected person result
    DEV_VEHICLE_RESULT_FEATURE_INFO  stuVehicleResult;               // detected vehicle result
    BYTE                             byReserved[512];                // Reserved
} NET_OBJECT_DETECTED_INFO;

// CLIENT_DetectObjectStructlize output parameter
typedef struct tagNET_OUT_OBJECTSTRUCTLIZE_DETECT_PIC
{
    DWORD                          dwSize; 
    int                            nObjectNum;                                // detected objects numbers
    NET_OBJECT_DETECTED_INFO       stuObjectinfos[DH_MAX_OBJECT_LIST];        // detected objects
}NET_OUT_OBJECTSTRUCTLIZE_DETECT_PIC;

// objectstructlize finding status callback function, 
typedef struct tagNET_CB_OBJECTSTRUCTLIZE_FIND_STATE
{
    DWORD               dwSize;     
    int                 nToken;         //The search token received 
    int                 nProgress;      //Normal value: 0-100. 1=Searched token does not exist (When subscribe a search that does not exist or already finished)
    int                 nCurrentCount;  //The amount that match current criteria
}NET_OBJECTSTRUCTLIZE_FIND_STATE;

//ObjectStructlize Find State callback
typedef void (CALLBACK *fObjectStructlizeFindState)(LLONG lLoginID, LLONG lAttachHandle, NET_OBJECTSTRUCTLIZE_FIND_STATE* pstStates, int nStateNum, LDWORD dwUser);

//CLIENT_ObjectStructlizeAttachFindState 
typedef struct tagNET_IN_OBJECTSTRUCTLIZE_FIND_STATE
{
    DWORD                              dwSize;             // Structure size. Must fill in. 
    
    int                                nTokenNum;          // Search token. 0=subscribe all searched tasks.
    int                                nTokens[8];         // Search tokens
    fObjectStructlizeFindState         cbObjectSFindState; // Callback function 
    LDWORD                             dwUser;             // User data 
}NET_IN_OBJECTSTRUCTLIZE_FIND_STATE;

//CLIENT_ObjectStructlizeAttachFindState
typedef struct  tagNET_OUT_OBJECTSTRUCTLIZE_FIND_STATE
{
    DWORD           dwSize;
 }NET_OUT_OBJECTSTRUCTLIZE_FIND_STATE;

// CLIENT_StartFindObjectStructlize 
typedef struct tagNET_IN_STARTFIND_OBJECTSTRUCTLIZE
{
    DWORD                                      dwSize;
    int                                        nChannelNum;                    // channel number
    int                                        stuChannelID[16];               // channel ID
	BOOL                                       bTimeEnable;                    // enable time find
    NET_TIME                                   stStartTime;                    // (option)start time
    NET_TIME                                   stEndTime;                      // (option)end time
    //find condition
    char                                       szObjectType[16];               // (required)object type, refer to the same field of NET_OBJFILTER_INFO
    char                                       szObjectSubType[32];            // (option)object sub type, refer to the same field of NET_OBJFILTER_INFO
    int                                        nSimilarity;                    // Similarity ( expressed as a percentage, from 1 to 100)

    DEV_PERSON_CONDITION_FEATURE_INFO          stuPersonCondition;             // person find condition,multiple conditions linked by AND
    DEV_VEHICLE_CONDITION_FEATURE_INFO         stuVehicleCondition;            // vehicle find condition,multiple conditions linked by AND
    DEV_IMAGE_REQUEST_FEATURE_INFO             stuImageRequestInfo;            // image find condition
    int                                        nRectNum;                       // valid feature box
    NET_RECT                                   stuRect[4];                     // feature box rect
    
    // image binary data
    char                                       *pBuffer;                       // Buffer address
    int                                        nBufferLen;                     // Buffer data length        
}NET_IN_STARTFIND_OBJECTSTRUCTLIZE;

// CLIENT_StartFindObjectStructlize 
typedef struct tagNET_OUT_STARTFIND_OBJECTSTRUCTLIZE
{
    DWORD               dwSize;
    int                 nTotalCount;                   // Record number of returns that match the query criteria 
    
    LLONG               lFindHandle;                   // Query handle
    int                 nToken;                        // The search token received
}NET_OUT_STARTFIND_OBJECTSTRUCTLIZE;



// CLIENT_DoFindObjectStructlize input parameter
typedef struct tagNET_IN_DOFIND_OBJECTSTRUCTLIZE
{
    DWORD                          dwSize;
    LLONG                          lFindHandle;                // Query handle
    int                            nBeginNum;                  // Queries starting serial number,0<=beginNumber<= totalCount-1
    int                            nCount;                     // The current number of records you want to search for
    EM_NEEDED_PIC_RETURN_TYPE      emNeededData;               // needed data type    
}NET_IN_DOFIND_OBJECTSTRUCTLIZE;

// doFind detected object info
typedef struct tagNET_DODBFIND_OBJECT_DETECTED_INFO
{
    int                               nChannelID;                        // channel id
    char                              szObjectType[16];                  // object type, refer to the same field of NET_OBJFILTER_INFO
    char                              szObjectSubType[32];               // object sub type, refer to the same field of NET_OBJFILTER_INFO
    
    DEV_PERSON_RESULT_FEATURE_INFO    stuPersonResult;                   // detected person result
    DEV_VEHICLE_RESULT_FEATURE_INFO   stuVehicleResult;                  // detected vehicle result
    int                               nImageNum;                         // detected image numbers
    DEV_IMAGE_RESULT_FEATURE_INFO     stuImageResults[4];                // detected image result
    BYTE                              byReserved[512];                   // Reserved
} NET_DODBFIND_OBJECT_DETECTED_INFO;

// CLIENT_DoFindObjectStructlize output parameter
typedef struct tagNET_OUT_DOFIND_OBJECTSTRUCTLIZE
{
    DWORD                                  dwSize;
    int                                    nfoundNum;                               // detected found numbers
    int                                    nObjectNum;                              // detected object return numbers
    NET_DODBFIND_OBJECT_DETECTED_INFO      stuObjectinfos[DH_MAX_OBJECT_LIST];      // detected object infos
    
    // image binary data
    char                                   *pBuffer;                                // Buffer address
    int                                    nBufferLen;                              // Buffer data length 
	int                                    nBufRet;                                 // actually returned len
}NET_OUT_DOFIND_OBJECTSTRUCTLIZE;

// query image by image,user malloc and free (pstInParam's and pstOutParam's) memory
CLIENT_NET_API BOOL CALL_METHOD CLIENT_DetectObjectStructlize(LLONG lLoginID, const NET_IN_OBJECTSTRUCTLIZE_DETECT_PIC* pstInParam, NET_OUT_OBJECTSTRUCTLIZE_DETECT_PIC *pstOutParam, int nWaitTime );

//subscribe structlize find state,user malloc and free (pstInParam's and pstOutParam's) memory
CLIENT_NET_API LLONG CALL_METHOD CLIENT_AttachObjectStructlizeFindState(LLONG lLoginID, const NET_IN_OBJECTSTRUCTLIZE_FIND_STATE* pstInParam, NET_OUT_OBJECTSTRUCTLIZE_FIND_STATE *pstOutParam, int nWaitTime );

//detach structlize find state,lAttachHandle is the return value of CLIENT_ObjectStructlizeAttachFindState
CLIENT_NET_API BOOL CALL_METHOD CLIENT_DetachObjectStructlizeFindState(LLONG lAttachHandle);

//start structlize find by condition,user malloc and free (pstInParam's and pstOutParam's) memory
CLIENT_NET_API BOOL CALL_METHOD CLIENT_StartFindObjectStructlize(LLONG lLoginID, const NET_IN_STARTFIND_OBJECTSTRUCTLIZE* pstInParam, NET_OUT_STARTFIND_OBJECTSTRUCTLIZE *pstOutParam, int nWaitTime );

// search object structlize result:nFilecount: need search item,  return value is media file item  return value<nFilecount then corresponding period file search complete(search max of 20 records each time)
// user malloc and free (pstInParam's and pstOutParam's) memory
CLIENT_NET_API BOOL CALL_METHOD CLIENT_DoFindObjectStructlize(const NET_IN_DOFIND_OBJECTSTRUCTLIZE* pstInParam, NET_OUT_DOFIND_OBJECTSTRUCTLIZE *pstOutParam, int nWaitTime );

// stop structlize find by findhanle
CLIENT_NET_API BOOL CALL_METHOD CLIENT_StopFindObjectStructlize(LLONG lFindHandle);

//requires the granularity of the statistics returned
typedef enum tagEM_STATIS_GRANULARITY
{
	EM_STATIS_GRANULARITY_UNKNOWN,					// unknow mode
	EM_STATIS_GRANULARITY_BY_HOUR,					// by hour
	EM_STATIS_GRANULARITY_BY_DAY,					// by day
	EM_STATIS_GRANULARITY_BY_MONTH,					// by month
}EM_STATIS_GRANULARITY;
//CLIENT_StartFindExternalSensor interface input parameter
typedef struct tagNET_IN_STARTFIND_EXTERNALSENSOR
{
	DWORD						dwSize;
	NET_TIME					stStartTime;				// start time
	NET_TIME					stEndTime;					// end time
	EM_STATIS_GRANULARITY		emGranularity;				// requires the granularity of the statistics returned"Hour"/"Day"/"Month"
	UINT						nVideoChannel;				// video channel numberDVR/NVR docking required, monocular IPC can not fill or fill 0
	UINT						nIndex;						// external sensor channel number
	UINT						nRecordHour;				// sample time point, Unit:Hour, range:[0,23],"szGranularity" is "Day" or "Month"

}NET_IN_STARTFIND_EXTERNALSENSOR;

// CLIENT_StartFindExternalSensor interface output parameter
typedef struct tagNET_OUT_STARTFIND_EXTERNALSENSOR
{
	DWORD				dwSize;
	UINT				nToken;											// query token
	UINT				nTotalCount;									// the total number of items that meet the query
}NET_OUT_STARTFIND_EXTERNALSENSOR;

// start querying exteranl sensor data
CLIENT_NET_API LLONG CALL_METHOD CLIENT_StartFindExternalSensor(LLONG lLoginID, const NET_IN_STARTFIND_EXTERNALSENSOR* pstInParam, NET_OUT_STARTFIND_EXTERNALSENSOR* pstOutParam, int nWaitTime);

// CLIENT_DoFindExternalSensor interface input paramter
typedef struct tagNET_IN_DOFIND_EXTERNALSENSOR
{
	DWORD				dwSize;
	UINT				nToken;											// query token
	UINT				nBegin;											// query the start sequence numbersaid from begintake nCount to return
	UINT				nCount;											// the number of queries per query
}NET_IN_DOFIND_EXTERNALSENSOR;

typedef struct tagNET_FIND_EXTERNALSENSOR_INFO
{
	UINT				nVideoChannel;									// video channle 
	UINT				nIndex;											// external sensor channel
	NET_TIME			stRecordTime;									// external sensor data storage time
	double				dbData;											// external sensot data 
	BYTE				byReserve[1024];								// Reserved
}NET_FIND_EXTERNALSENSOR_INFO;

// CLIENT_DoFindExternalSensor interface input parameter
typedef struct tagNET_OUT_DOFIND_EXTERNALSENSOR
{
	DWORD								dwSize;
	UINT								nFound;							// the number of inquiries
	UINT								nMazNum;						// number of queries
	NET_FIND_EXTERNALSENSOR_INFO*		pstuInfo;						// external sensor data query time,
}NET_OUT_DOFIND_EXTERNALSENSOR;

// obtain external sensor data that matches the query criteria
CLIENT_NET_API BOOL CALL_METHOD CLIENT_DoFindExternalSensor(LLONG lFindHandle, const NET_IN_DOFIND_EXTERNALSENSOR* pstInParam, NET_OUT_DOFIND_EXTERNALSENSOR* pstOutParam, int nWaitTime);

// stop querying external sensor data
CLIENT_NET_API BOOL CALL_METHOD CLIENT_StopFindExternalSensor(LLONG lFindHandle);

typedef struct tagNET_CFG_TRACKBOX
{
	int				nVideoChannel;				// video channel
	BOOL			bTargetBox;					// show target box
	BOOL			bTrackBox;					// show track box
	BOOL			bTargetNumber;				// show target number
	BYTE			byReserved[1024];			// Reserved
}NET_CFG_TRACKBOX;

typedef struct tagNET_IN_GET_TRACKBOX
{
	DWORD			dwSize;
	int				nVideoChannel;				// video channel
}NET_IN_GET_TRACKBOX;

// Get the tracking box display configuration

typedef struct tagNET_OUT_GET_TRACKBOX
{
	DWORD				dwSize;
	NET_CFG_TRACKBOX	stuGetTrackBox;
}NET_OUT_GET_TRACKBOX;

// Get the tracking box display configuration
CLIENT_NET_API BOOL CALL_METHOD CLIENT_GetTrackBox(LLONG lLoginID, const NET_IN_GET_TRACKBOX* pstInParam, NET_OUT_GET_TRACKBOX *pstOutParam, int nWaitTime);

typedef struct tagNET_IN_SET_TRACKBOX
{
	DWORD				dwSize;
	NET_CFG_TRACKBOX	stuSetTrackBox;
}NET_IN_SET_TRACKBOX;

typedef struct tagNET_OUT_SET_TRACKBOX
{
	DWORD			dwSize;
}NET_OUT_SET_TRACKBOX;

// Set the tracking box display configuration
CLIENT_NET_API BOOL CALL_METHOD CLIENT_SetTrackBox(LLONG lLoginID, const NET_IN_SET_TRACKBOX* pstInParam, NET_OUT_SET_TRACKBOX *pstOutParam, int nWaitTime);

// Track Mode
typedef enum tagEM_TRACK_MODE
{
	EM_TRACK_MODE_UNKNOWN = -1,						// Unknown mode
	EM_TRACK_MODE_AUTO,								// Automatic tracking
	EM_TRACK_MODE_SINGLE,							// Single target tracking
	EM_TRACK_MODE_MANUAL,							// Manual control
	EM_TRACK_MODE_ALARM								// Alarm tracking
}EM_TRACK_MODE;

typedef struct tagNET_CFG_TRACKMODE
{
	int					nVideoChannel;				// video channel
	EM_TRACK_MODE		emTrackMode;				// Track mode
	BYTE				byReserved[1024];			// Reserved
}NET_CFG_TRACKMODE;

typedef struct tagNET_IN_GET_TRACKMODE
{
	DWORD				dwSize;
	int					nVideoChannel;				// video channel
}NET_IN_GET_TRACKMODE;

typedef struct tagNET_OUT_GET_TRACKMODE
{
	DWORD				dwSize;
	NET_CFG_TRACKMODE	stuGetTrackMode;
}NET_OUT_GET_TRACKMODE;

// Get the tracking mode
CLIENT_NET_API BOOL CALL_METHOD CLIENT_GetTrackMode(LLONG lLoginID, const NET_IN_GET_TRACKMODE* pstInParam, NET_OUT_GET_TRACKMODE *pstOutParam, int nWaitTime);

typedef struct tagNET_IN_SET_TRACKMODE
{
	DWORD				dwSize;
	NET_CFG_TRACKMODE	stuSetTrackMode;
}NET_IN_SET_TRACKMODE;

typedef struct tagNET_OUT_SET_TRACKMODE
{
	DWORD			dwSize;
}NET_OUT_SET_TRACKMODE;

// Set the tracking mode
CLIENT_NET_API BOOL CALL_METHOD CLIENT_SetTrackMode(LLONG lLoginID, const NET_IN_SET_TRACKMODE* pstInParam, NET_OUT_SET_TRACKMODE *pstOutParam, int nWaitTime);

typedef struct tagNET_CFG_LINKPOS
{
	int				nVideoChannel;					// video channel
	char			szBallip[DH_COMMON_STRING_64];	// Linkage ball machine IP address, can be ignored
	unsigned short	nX;								// mouse click tracker video nx coordinate normalized value or mouse frame select rectangel center, ranges: [0, 8192)
	unsigned short	nY;								// mouse click tracker video ny coordinate normalized value or mouse frame select rectangel center, ranges: [0, 8192)
	float			fZoom;							// fZoom is negative for click linkage, only linkage unchanged times. Greater than or equal to 0 means that box selection linkages
	BYTE			byReserved[1024];				// Reserved
}NET_CFG_LINKPOS;

typedef struct tagNET_IN_SET_LINKPOS
{
	DWORD				dwSize;
	NET_CFG_LINKPOS		stuSetLinkPos; 
}NET_IN_SET_LINKPOS;

typedef struct tagNET_OUT_SET_LINKPOS
{
	DWORD			dwSize;
}NET_OUT_SET_LINKPOS;

// Click the linkage and box selection linkage
CLIENT_NET_API BOOL CALL_METHOD CLIENT_SetLinkPos(LLONG lLoginID, const NET_IN_SET_LINKPOS* pstInParam, NET_OUT_SET_LINKPOS *pstOutParam, int nWaitTime);
// +++++++++++++++++++++++++++++++++++++++++++++++++++++Clound storage service related interface+++++++++++++++++++++++++++++++++++++++++++

// bucket name info
typedef struct tagNET_BUCKET_NAME_INFO
{
    char szName[DH_COMMON_STRING_64];                       // name
    BYTE byReserved[256];                                   // reserved
}NET_BUCKET_NAME_INFO;

// the input pararm of find bucket name info
typedef struct tagNET_IN_FIND_BUCKET_NAME
{
    DWORD dwSize;                                           // struct size
    UINT nOffset;                                           // offset
    UINT nCount;                                            // count of bucket name to find    
}NET_IN_FIND_BUCKET_NAME;

// the output pararm of find bucket name info
typedef struct tagNET_OUT_FIND_BUCKET_NAME
{
    DWORD dwSize;                                           // struct size
    UINT nTotalCount;                                       // the total bucket name count
    UINT nFindCount;                                        // the result count of find
    NET_BUCKET_NAME_INFO* pBucketName;                      // the buffer for bucket name
    UINT nNameCount;                                        // the buffer count  
}NET_OUT_FIND_BUCKET_NAME;

// the file name info in someone bucket
typedef struct tagNET_BUCKET_FILE_NAME_INFO
{
    char szName[DH_COMMON_STRING_1024];                     // file name
    BYTE byReserved[256];                                   // reserved
}NET_BUCKET_FILE_NAME_INFO;

// the input param of find the file name info in someone bucket
typedef struct tagNET_IN_FIND_BUCKET_FILE_NAME
{
    DWORD dwSize;                                           // struct size    
    UINT nOffset;                                           // offset
    UINT nCount;                                            // count of file to find
    char szBucketName[DH_COMMON_STRING_64];                 // the name of bucket    
}NET_IN_FIND_BUCKET_FILE_NAME;

// the output param of find the file name info in someone bucket
typedef struct tagNET_OUT_FIND_BUCKET_FILE_NAME
{
    DWORD dwSize;                                           // struct size
    UINT nTotalCount;                                       // the total file count
    UINT nFindCount;                                        // the result num of find
    NET_BUCKET_FILE_NAME_INFO* pFileName;                   // the buffer fo file name info
    UINT nFileCount;                                        // the buffer count
}NET_OUT_FIND_BUCKET_FILE_NAME;

// find the bucket name info in the efs
CLIENT_NET_API BOOL CALL_METHOD CLIENT_FindBucketName(LLONG lLoginID, const NET_IN_FIND_BUCKET_NAME* pstInParam, NET_OUT_FIND_BUCKET_NAME *pstOutParam, int nWaitTime );

// find the file name info by bucket name
CLIENT_NET_API BOOL CALL_METHOD CLIENT_FindBucketFileName(LLONG lLoginID, const NET_IN_FIND_BUCKET_FILE_NAME* pstInParam, NET_OUT_FIND_BUCKET_FILE_NAME *pstOutParam, int nWaitTime );

// +++++++++++++++++++++++++++++++++++++++++++++++++++++EFS Interface+++++++++++++++++++++++++++++++++++++++++++

// +++++++++++++++++++++++++++++++++++++++++++++++++++++voice broadcast interface+++++++++++++++++++++++++++++++++++++++++++
// voice broadcast callback function original shape
typedef void (CALLBACK *pfAudioBroadcastDataCallBack)(LLONG lSpeakHandle, char *pDataBuf, DWORD dwBufSize, int nToken, LDWORD dwUser);

// voice broadcast input parameters for the specified channels
typedef struct tagNET_IN_START_CHANNEL_INFO
{
	DWORD							dwSize;
	int 							nChannelCount;			// count of the specified channels
	int								nChannel[32];			// the list of the specified channels
	pfAudioBroadcastDataCallBack 	pfcb;					// voice broadcast callback function
	LDWORD 							dwUser;					// user data
} NET_IN_START_CHANNEL_INFO;

// voice broad cast output parameters for the specified channels
typedef struct tagNET_OUT_START_CHANNEL_INFO
{
	DWORD					dwSize;
	int						nToken;					// broadcast token
	int						nFrequency;				// audio sampling frequency
	int						nDepth;					// audio sampling depth
	DH_TALK_CODING_TYPE		emCompressionType;		// audio compression type
} NET_OUT_START_CHANNEL_INFO;

// the input parameters of CLIENT_TalkChannelSendData
typedef struct tagNET_TALK_CHANNEL_SENDDATA_IN
{
	DWORD					dwSize;
	char 			   	   *pSendBuf;		// audio data to be sent
	int 					nSendBufLen;	// the length of the audio data to be sent
} NET_TALK_CHANNEL_SENDDATA_IN;

// the output parameters of CLIENT_TalkChannelSendData
typedef struct tagNET_TALK_CHANNEL_SENDDATA_OUT
{
	DWORD					dwSize;
} NET_TALK_CHANNEL_SENDDATA_OUT;

// void broadcast on the specified channels
CLIENT_NET_API LLONG CALL_METHOD CLIENT_StartTalkChannel(LLONG lLoginID, const NET_IN_START_CHANNEL_INFO *pstInParam, NET_OUT_START_CHANNEL_INFO *pstOutParam, int nWaitTime);

// send voice data to the specified channels
CLIENT_NET_API BOOL  CALL_METHOD CLIENT_TalkChannelSendData(LLONG lSpeakHandle, const NET_TALK_CHANNEL_SENDDATA_IN *pstInParam, NET_TALK_CHANNEL_SENDDATA_OUT *pstOutParam, int nWaitTime);

// stop voice broadcast
CLIENT_NET_API BOOL  CALL_METHOD CLIENT_StopTalkChannel(LLONG lSpeakHandle);
// +++++++++++++++++++++++++++++++++++++++++++++++++++++voice broadcast interfaces+++++++++++++++++++++++++++++++++++++++++++

// the input parameters of CLIENT_InitDevAccount
typedef struct tagNET_IN_INIT_DEVICE_ACCOUNT
{
	DWORD					dwSize;										// struct size:assignment when init the struct
	char					szMac[DH_MACADDR_LEN];						// mac addr	
	char					szUserName[MAX_USER_NAME_LEN];				// user name
	char					szPwd[MAX_PWD_LEN];							// password
	char					szCellPhone[MAX_CELL_PHONE_NUMBER_LEN];		// cellphone
	char					szMail[MAX_MAIL_LEN];						// mail addr
	BYTE					byInitStatus;								// this field already abandoned
	BYTE					byPwdResetWay;								// the way supported for reset password:byPwdResetWay value of CLIENT_StartSearchDevices's , CLIENT_SearchDevicesByIPs's callback function and CLIENT_SearchDevices
																		// the meaning of this parameter refers to DEVICE_NET_INFO_EX, the value must be same as byPwdResetWay returned by CLIENT_StartSearchDevices,CLIENT_SearchDevicesByIPs and CLIENT_SearchDevices
																		// bit0 : 1-support reset password by cellphone, you should set cellphone in szCellPhone if you need to set cellphone
																		// bit1 : 1-support reset password by mail, you should set mail address in szMail if you need to set mail address
	BYTE					byReserved[2];								// reserved
}NET_IN_INIT_DEVICE_ACCOUNT;

// the output parameters of CLIENT_InitDevAccount
typedef struct tagNET_OUT_INIT_DEVICE_ACCOUNT
{
	DWORD					dwSize;// struct size:assignment when init the struct
}NET_OUT_INIT_DEVICE_ACCOUNT;

// init account
CLIENT_NET_API BOOL CALL_METHOD CLIENT_InitDevAccount(const NET_IN_INIT_DEVICE_ACCOUNT* pInitAccountIn, NET_OUT_INIT_DEVICE_ACCOUNT* pInitAccountOut, DWORD dwWaitTime, char* szLocalIp);

// init account by ip
CLIENT_NET_API BOOL CALL_METHOD CLIENT_InitDevAccountByIP(const NET_IN_INIT_DEVICE_ACCOUNT* pInitAccountIn, NET_OUT_INIT_DEVICE_ACCOUNT* pInitAccountOut, DWORD dwWaitTime, const char* szLocalIp,const char* szDeviceIP);

// video standard
typedef enum tagEM_VIDEO_STANDARD
{
    EM_VIDEO_STANDARD_UNKNOWN,                               
	EM_VIDEO_STANDARD_PAL,		// "PLA"                               
	EM_VIDEO_STANDARD_NTSC,		// "NTSC"
	EM_VIDEO_STANDARD_SECAM,	// "SECAM"
}EM_VIDEO_STANDARD;

//boot wizard flow input parameters
typedef struct tagNET_IN_INIT_DEVICE_ACCESS
{
	DWORD             dwSize;                       //struct size:assignment when init the struct
	char			  szMac[DH_MACADDR_LEN];	    // mac addr	
	char			  szUserName[MAX_USER_NAME_LEN];// user name
	char			  szPwd[MAX_PWD_LEN];		    // password
	BYTE              byNetAccess;                  // whether enable netaccess:0-not involved, not handled 1-disable 2-enable
	BYTE              byUpgradeCheck;               // whether enable upgradecheck 0-not invloved, not handled 1-disable 2-enable
	BYTE              byReserved[2];                // reserved bytes
	EM_VIDEO_STANDARD emVideoStandard;				// Video Standard
}NET_IN_INIT_DEVICE_ACCESS;

//boot wizard flow ouput parameters
typedef struct tagNET_OUT_INIT_DEVICE_ACCESS
{
	DWORD             dwSize;                   // struct size:assignment when init the struct
}NET_OUT_INIT_DEVICE_ACCESS;

//boot wizard flow, include function netaccess, upgradecheck 
CLIENT_NET_API BOOL CALL_METHOD CLIENT_InitDevAccess(const NET_IN_INIT_DEVICE_ACCESS* pInitAccessIn, NET_OUT_INIT_DEVICE_ACCESS* pInitAccessOut, DWORD dwWaitTime, char* szLocalIp);

// the input parameters of CLIENT_GetDescriptionForResetPwd
typedef struct tagNET_IN_DESCRIPTION_FOR_RESET_PWD
{
	DWORD					dwSize;								// struct size:assignment when init the struct
	char					szMac[DH_MACADDR_LEN];				// mac addr
	char					szUserName[MAX_USER_NAME_LEN];		// user name	
	BYTE					byInitStatus;						// init status:byInitStatus value of CLIENT_StartSearchDevices's callback functionCLIENT_SearchDevicesCLIENT_SearchDevicesByIPs
	BYTE					byReserved[3];						// reserved
}NET_IN_DESCRIPTION_FOR_RESET_PWD;

// the output parameters of CLIENT_GetDescriptionForResetPwd
typedef struct tagNET_OUT_DESCRIPTION_FOR_RESET_PWD
{
	DWORD			dwSize;										// struct size:assignment when init the struct
	char			szCellPhone[MAX_CELL_PHONE_NUMBER_LEN];		// cellphone
	char			szMailAddr[MAX_MAIL_LEN];					// mac addr
	char*			pQrCode;									// QR code:user malloc and free
	unsigned int	nQrCodeLen;									// QR code len user set(360 is enough for current)
	unsigned int    nQrCodeLenRet;								// QR code len device returns
}NET_OUT_DESCRIPTION_FOR_RESET_PWD;

// get description info for reset password
CLIENT_NET_API BOOL CALL_METHOD CLIENT_GetDescriptionForResetPwd(const NET_IN_DESCRIPTION_FOR_RESET_PWD* pDescriptionIn, NET_OUT_DESCRIPTION_FOR_RESET_PWD* pDescriptionOut, DWORD dwWaitTime, char* szLocalIp);

// the input parameters of CLIENT_ResetPwd
typedef struct tagNET_IN_RESET_PWD
{
	DWORD					dwSize;								// struct size:assignment when init the struct
	char					szMac[DH_MACADDR_LEN];				// mac addr	
	char					szUserName[MAX_USER_NAME_LEN];		// user name
	char					szPwd[MAX_PWD_LEN];					// pass word
	char					szSecurity[MAX_SECURITY_CODE_LEN];	// security code
	BYTE					byInitStaus;						// init status:byInitStatus value of CLIENT_StartSearchDevices's callback functionCLIENT_SearchDevicesCLIENT_SearchDevicesByIPs
	BYTE					byPwdResetWay;						// the way supported for reset password:byPwdResetWay value of CLIENT_StartSearchDevices's callback functionCLIENT_SearchDevicesCLIENT_SearchDevicesByIPs
	BYTE					byReserved[2];						// reserved					
}NET_IN_RESET_PWD;

// the output parameters of CLIENT_ResetPwd
typedef struct tagNET_OUT_RESET_PWD
{
	DWORD					dwSize;// struct size:assignment when init the struct
}NET_OUT_RESET_PWD;

// reset password 
CLIENT_NET_API BOOL CALL_METHOD CLIENT_ResetPwd(const NET_IN_RESET_PWD* pResetPwdIn, NET_OUT_RESET_PWD* pResetPwdOut, DWORD dwWaitTime, char* szLocalIp);

// account type
typedef enum tagEM_ACCOUNT_TYPE
{
    EM_ACCOUNT_TYPE_UNKNOWN,                                    // unknown
    EM_ACCOUNT_TYPE_SDK,                                        // sdk account
    EM_ACCOUNT_TYPE_WEB,                                        // web account
}EM_ACCOUNT_TYPE;

// input param of init account
typedef struct tagNET_IN_INIT_ACCOUNT
{
    DWORD					dwSize;								// struct size    
    char					szUser[MAX_USER_NAME_LEN];		    // user
    char					szPwd[MAX_PWD_LEN];					// password
    char                    szMac[DH_MACADDR_LEN];              // mac addr
    EM_ACCOUNT_TYPE         emAccountType;                      // account type
}NET_IN_INIT_ACCOUNT;

// output param of init account
typedef struct tagNET_OUT_INIT_ACCOUNT
{
    DWORD               dwSize;								    // struct size  		
}NET_OUT_INIT_ACCOUNT;

// input param of reset web account
typedef struct tagNET_IN_REST_WEB_PWD
{
    DWORD					dwSize;								// struct size  
    char					szWebUser[MAX_USER_NAME_LEN];		// web user
    char					szWebPwd[MAX_PWD_LEN];			    // web password
}NET_IN_REST_WEB_PWD;

// output param of reset web account
typedef struct tagNET_OUT_REST_WEB_PWD
{
    DWORD               dwSize;								    // struct size	
}NET_OUT_REST_WEB_PWD;

// init account
CLIENT_NET_API BOOL CALL_METHOD CLIENT_InitAccount(const NET_IN_INIT_ACCOUNT* pInitAccountIn, NET_OUT_INIT_ACCOUNT* pInitAccountOut, int nWaitTime, char* szLocalIp);

// reset web account
CLIENT_NET_API BOOL CALL_METHOD CLIENT_ResetWebPwd(LLONG lLoginID,const NET_IN_REST_WEB_PWD* pstInParam, NET_OUT_REST_WEB_PWD* pstOutParam, int nWaitTime);

// the input parameters of CLIENT_CheckAuthCode
typedef struct tagNET_IN_CHECK_AUTHCODE
{
	DWORD					dwSize;									// struct size:assignment when init the struct
	char					szMac[DH_MACADDR_LEN];					// mac addr
	char					szSecurity[MAX_SECURITY_CODE_LEN];		// security code
}NET_IN_CHECK_AUTHCODE;

// the output parameters of CLIENT_CheckAuthCode
typedef struct tagNET_OUT_CHECK_AUTHCODE
{
	DWORD					dwSize;// struct size:assignment when init the struct
}NET_OUT_CHECK_AUTHCODE;

// check security code
CLIENT_NET_API BOOL CALL_METHOD CLIENT_CheckAuthCode(const NET_IN_CHECK_AUTHCODE* pCheckAuthCodeIn, NET_OUT_CHECK_AUTHCODE* pCheckAuthCodeOut, DWORD dwWaitTime, char* szLocalIp);
// input parameters of CLIENT_SetEverest360MediaMode
typedef struct tagNET_IN_SET_EVEREST360_MEDIAMODE
{
    DWORD					dwSize;				
    UINT                    nMediaMode;                             // Media Mode,0:Snap,1:Record,2:Live
}NET_IN_SET_EVEREST360_MEDIAMODE;

// Output parameters of CLIENT_SetEverest360MediaMode
typedef struct tagNET_OUT_SET_EVEREST360_MEDIAMODE
{
    DWORD					dwSize;				
}NET_OUT_SET_EVEREST360_MEDIAMODE;

// Set MediaMode of Everest360 Sports Camera
CLIENT_NET_API BOOL CALL_METHOD CLIENT_SetEverest360MediaMode(LLONG lLoginID, const NET_IN_SET_EVEREST360_MEDIAMODE *pstInParam, NET_OUT_SET_EVEREST360_MEDIAMODE *pstOutParam, int nWaitTime);

// the input parameters of CLIENT_GetPwdSpecification
typedef struct tagNET_IN_PWD_SPECI
{
	DWORD		dwSize;					// struct size
	char		szMac[DH_MACADDR_LEN];	// mac address
}NET_IN_PWD_SPECI;

// the output parameters of CLIENT_GetPwdSpecification
typedef struct tagNET_OUT_PWD_SPECI
{
	DWORD	dwSize;										// struct size
	unsigned int	nMaxPwdLen;							// max passwd len
	unsigned int	nMinPwdLen;							// min passwd len	
	char	szType[MAX_PWD_BASIC_CHARS_ARRAY_LEN];		// basic chars supportedNumber,Lower case,Capital. such as :"Number,Lower,Upper"
	char	szCharList[MAX_PWD_SPEC_CHARS_ARRAY_LEN];	// special chars supported, as"~!@#$%^" 
	int		nCombine;									// chars combine style: 0-no special request, 1-special char must be included, 2-at least two style included (number, capital, lower case, special char)
}NET_OUT_PWD_SPECI;

// get passwd specification
CLIENT_NET_API BOOL CALL_METHOD CLIENT_GetPwdSpecification(const NET_IN_PWD_SPECI* pPwdSpeciIn, NET_OUT_PWD_SPECI* pPwdSpeciOut, DWORD dwWaitTime, char* szLocalIp);

// the input parameters of CLIENT_SetLowRateWPANPower
typedef struct tagNET_IN_SET_LOWRATEWPAN_POWER
{
    DWORD			dwSize;										// struct size
    int             nPower;                                     // power
    char            szSmartLockNum[MAX_COMMON_STRING_32];       // serial Number
}NET_IN_SET_LOWRATEWPAN_POWER;

// the output parameters of CLIENT_SetLowRateWPANPower
typedef struct tagNET_OUT_SET_LOWRATEWPAN_POWER
{
    DWORD			dwSize;										// struct size   
}NET_OUT_SET_LOWRATEWPAN_POWER;

// set power of LowRateWPAN
CLIENT_NET_API BOOL CALL_METHOD CLIENT_SetLowRateWPANPower(LLONG lLoginID, const NET_IN_SET_LOWRATEWPAN_POWER *pstInParam, NET_OUT_SET_LOWRATEWPAN_POWER *pstOutParam, int nWaitTime);

// operate type of accesscontroller
typedef enum tagNET_EM_ACCESS_CTL_MANAGER
{
    NET_EM_ACCESS_CTL_MANAGER_ADDDEVICE,                        // add accesssubcontroller, Corresponding to pstInParam = NET_IN_ACCESS_CTL_MANAGER_ADDDEVICE, pstOutParam = NET_OUT_ACCESS_CTL_MANAGER_ADDDEVICE
    NET_EM_ACCESS_CTL_MANAGER_MODIFYDEVICE,                     // modify accesssubcontroller, Corresponding to pstInParam = NET_IN_ACCESS_CTL_MANAGER_MODIFYDEVICE, pstOutParam = NET_OUT_ACCESS_CTL_MANAGER_MODIFYDEVICE
    NET_EM_ACCESS_CTL_MANAGER_REMOVEDEVICE,                     // remove accesssubcontroller, Corresponding to pstInParam = NET_IN_ACCESS_CTL_MANAGER_REMOVEDEVICE, pstOutParam = NET_OUT_ACCESS_CTL_MANAGER_REMOVEDEVICE
    NET_EM_ACCESS_CTL_GETSUBCONTROLLER_INFO,                    // get accesssubcontroller info, Corresponding to pstInParam = NET_IN_GET_SUB_CONTROLLER_INFO, pstOutParam = NET_OUT_GET_SUB_CONTROLLER_INFO
    NET_EM_ACCESS_CTL_GETSUBCONTROLLER_STATE,                   // get accesssubcontroller state, Corresponding to pstInParam = NET_IN_GET_SUB_CONTROLLER_STATE, pstOutParam = NET_OUT_GET_SUB_CONTROLLER_STATE
    NET_EM_ACCESS_CTL_SET_REPEAT_ENTERROUTE,                    // set repeat engterroute, Corresponding to pstInparam = NET_IN_SET_REPEAT_ENTERROUTE, pstOutParam = NET_OUT_SET_REPEAT_ENTERROUTE
    NET_EM_ACCESS_CTL_GET_REPEAT_ENTERROUTE,                    // get repeat enterroute, Corresponding to pstInparam = NET_IN_GET_REPEAT_ENTERROUTE, pstOutParam = NET_OUT_GET_REPEAT_ENTERROUTE
    NET_EM_ACCESS_CTL_SET_ABLOCK_ROUTE,                         // set ABLock route, Corresponding to pstInparam = NET_IN_SET_ABLOCK_ROUTE, pstOutParam = NET_OUT_SET_ABLOCK_ROUTE
    NET_EM_ACCESS_CTL_GET_ABLOCK_ROUTE,                         // get ABLock route, Corresponding to pstInparam = NET_IN_GET_ABLOCK_ROUTE, pstOutParam = NET_OUT_GET_ABLOCK_ROUTE     
    NET_EM_ACCESS_CTL_GET_LOGSTATUS,                            // get log synchronize status, Corresponding to pstInparam = NET_IN_GET_LOGSTATUS, pstOutParam = NET_OUT_GET_LOGSTATUS     
    NET_EM_ACCESS_CTL_SYNCHRO_OFFLINE_LOG,                      // synchro offline log, Corresponding to pstInparam = NET_IN_SYNCHRO_OFFLINE_LOG, pstOutParam = NET_OUT_SYNCHRO_OFFLINE_LOG 
    NET_EM_ACCESS_CTL_SYNCHRO_TIME,                             // synchro subController time,Corresponding to pstInparam = NET_IN_SYNCHRO_CONTROLLER_TIME, pstOutParam = NET_OUT_SYNCHRO_CONTROLLER_TIME 
}NET_EM_ACCESS_CTL_MANAGER;

// in parameter of add device
typedef struct tagNET_IN_ACCESS_CTL_MANAGER_ADDDEVICE
{
    DWORD           dwSize;
    int             nSubControllerID;                           // accesssubcontroller ID, range: 1~255
    char            szSubControllerName[MAX_COMMON_STRING_128]; // accesssubcontroller name
}NET_IN_ACCESS_CTL_MANAGER_ADDDEVICE;

// out parameter of add device
typedef struct tagNET_OUT_ACCESS_CTL_MANAGER_ADDDEVICE
{
    DWORD           dwSize;   
    int             nErrorCode;                                 // errorcode 0-success, 1-add repeat,2-add fullest,3-other fail
}NET_OUT_ACCESS_CTL_MANAGER_ADDDEVICE;

// in parameter of modify device
typedef struct tagNET_IN_ACCESS_CTL_MANAGER_MODIFYDEVICE
{
    DWORD           dwSize;
    int             nSubControllerID;                           // accesssubcontroller ID, range: 1~255, 0:local
    char            szSubControllerName[MAX_COMMON_STRING_128]; // accesssubcontroller name
}NET_IN_ACCESS_CTL_MANAGER_MODIFYDEVICE;

// out parameter of modify device
typedef struct tagNET_OUT_ACCESS_CTL_MANAGER_MODIFYDEVICE
{
    DWORD           dwSize;
}NET_OUT_ACCESS_CTL_MANAGER_MODIFYDEVICE;

// in parameter of remove device
typedef struct tagNET_IN_ACCESS_CTL_MANAGER_REMOVEDEVICE
{
    DWORD           dwSize;
    int             nSubControllerID[MAX_ACCESSSUBCONTROLLER_NUM]; // accesssubcontroller ID, range: 1~255
    int             nSubControllerNum;                          // accesssubcontroller count
}NET_IN_ACCESS_CTL_MANAGER_REMOVEDEVICE;

// result of remove device
typedef struct tagNET_REMOVEDEVICE_RESULT
{
    int             nSubControllerID;                           // accesssubcontroller ID
    int             nErrorCode;                                 // result, 0-success,1-fail 2-unallocation
    BYTE            byReserved[128];                            
}NET_REMOVEDEVICE_RESULT;

//  out parameter of remove device
typedef struct tagNET_OUT_ACCESS_CTL_MANAGER_REMOVEDEVICE
{
    DWORD                       dwSize;
    int                         nResultNum;
    NET_REMOVEDEVICE_RESULT     stuResult[MAX_ACCESSSUBCONTROLLER_NUM];     
}NET_OUT_ACCESS_CTL_MANAGER_REMOVEDEVICE;

// in parameter of Get subcontroller info
typedef struct tagNET_IN_GET_SUB_CONTROLLER_INFO
{
    DWORD                       dwSize;
    int                         nSubControllerID[MAX_ACCESSSUBCONTROLLER_NUM];  // accesssubcontroller ID, range:-1~255, -1:Get All,0:Local,1~255:other
    int                         nSubControllerNum;                              // accesssubcontroller number
}NET_IN_GET_SUB_CONTROLLER_INFO;


// property
typedef enum  tagNET_EM_ACCESS_PROPERTY
{
    NET_EM_ACCESS_PROPERTY_UNKNOWN,                                             // unknown
    NET_EM_ACCESS_PROPERTY_BIDIRECT,                                            // bidirect
    NET_EM_ACCESS_PROPERTY_UNIDIRECT,                                           // unidirect
}NET_ACCESS_PROPERTY;

// card reader info
typedef struct tagNET_CARDREAD_INFO 
{
    int                         nDoor;                                          // door num
    int                         nReadNum;                                       // reader count
    char                        szReadID[MAX_ACCESS_READER_NUM][DH_COMMON_STRING_32];                 // reader num
    BYTE                        byReserved[64];  
}NET_CARDREAD_INFO;

// subcontroller info
typedef struct tagNET_SUB_CONTROLLER_INFO 
{
    int                         nSubControllerID;                               // ControllerID, range:0~255,0:local
    char                        szSubControllerName[MAX_COMMON_STRING_128];     // Controller name
    NET_ACCESS_PROPERTY         emProperty;                                     // property
    char                        szDeviceType[MAX_COMMON_STRING_128];            // device type
    char                        szVesion[MAX_COMMON_STRING_128];                // device vesion
    int                         nDoorNum;                                       // door count
    NET_CARDREAD_INFO           stuReaderInfo[MAX_ACCESSDOOR_NUM];              // the card reader of door
    BYTE                        byReserved[128];
}NET_SUB_CONTROLLER_INFO;

// out parameter of Get subcontroller info
typedef struct tagNET_OUT_GET_SUB_CONTROLLER_INFO
{
    DWORD                       dwSize;
    NET_SUB_CONTROLLER_INFO     stuSubControllerInfo[MAX_ACCESSSUBCONTROLLER_NUM];  // controller info
    int                         nRetNum;                                            // controller count
}NET_OUT_GET_SUB_CONTROLLER_INFO;

// in parameter of Get subcontroller state
typedef struct tagNET_IN_GET_SUB_CONTROLLER_STATE
{
    DWORD                       dwSize;
    int                         nSubControllerID[MAX_ACCESSSUBCONTROLLER_NUM];      // accesssubcontroller ID, range:-1~255, -1:Get All,0:Local,1~255:other
    int                         nSubControllerNum;                                  // accesssubcontroller number
}NET_IN_GET_SUB_CONTROLLER_STATE;

// controller state info
typedef struct tagNET_SUB_CONTROLLER_STATE
{
    int                         nSubControllerID;                                   // accesssubcontroller ID
    int                         nState;                                             // 0:off-line 1:online 2:unallocation
	BYTE                        byReserved[128];
}NET_SUB_CONTROLLER_STATE;

// out parameter of Get subcontroller state
typedef struct tagNET_OUT_GET_SUB_CONTROLLER_STATE
{
    DWORD                       dwSize;
    NET_SUB_CONTROLLER_STATE    stuSubControllerState[MAX_ACCESSSUBCONTROLLER_NUM];  // accesssubcontroller state
    int                         nRetNum;                                             // accesssubcontroller count
}NET_OUT_GET_SUB_CONTROLLER_STATE;

// repeat enterroute point info
typedef struct tagNET_POINT_READER_INFO
{
    int                         nReaderNum;                                         // repeat enterroute point num
    char                        szReaderID[MAX_ACCESS_READER_NUM][DH_COMMON_STRING_32]; // reader of repeat enterroute point
    BYTE                        byReserved[128];
}NET_POINT_READER_INFO;

// in parameter of set repaet enter route
typedef struct tagNET_IN_SET_REPEAT_ENTERROUTE
{
    DWORD                       dwSize;
    UINT                        nIndex;                                              // index of route
    int                         nPointNum;                                           // point number of route
    NET_POINT_READER_INFO       stuPointInfo[MAX_ACCESS_POINT_NUM];                  // point info
    int                         nTimeSection;                                        // config of AccessTimeSchedule number, please query dhconfigsdk.h
    BOOL                        bFlag;                                               // enable
    char                        szRouteName[MAX_COMMON_STRING_128];                  // route name
}NET_IN_SET_REPEAT_ENTERROUTE;

// out parameter of set repaet enter route
typedef struct tagNET_OUT_SET_REPEAT_ENTERROUTE
{
    DWORD                       dwSize;
}NET_OUT_SET_REPEAT_ENTERROUTE;

// in parameter of get repaet enter route
typedef struct tagNET_IN_GET_REPEAT_ENTERROUTE
{
    DWORD                       dwSize;
    UINT                        nIndex;                                              // repeat enter route index
}NET_IN_GET_REPEAT_ENTERROUTE;

// out parameter of set repaet enter route
typedef struct tagNET_OUT_GET_REPEAT_ENTERROUTE
{
    DWORD                       dwSize;
    int                         nPointNum;                                           // point count of route
    NET_POINT_READER_INFO       stuPointInfo[MAX_ACCESS_POINT_NUM];                  // point info
    int                         nTimeSection;                                        // config of AccessTimeSchedule number, please query dhconfigsdk.h
    BOOL                        bFlag;                                               // enable
    char                        szRouteName[MAX_COMMON_STRING_128];                  // route name
}NET_OUT_GET_REPEAT_ENTERROUTE;

// AB lock route info
typedef struct tagNET_POINT_DOOR_INFO
{
    int                         nDoorNum;                                           // ABlock door number
    int                         nDoorID[MAX_ACCESSDOOR_NUM];                        // ABlock door id
    BYTE                        byReserved[128];
}NET_POINT_DOOR_INFO;

// in parameter of set ABlock  route
typedef struct tagNET_IN_SET_ABLOCK_ROUTE
{
    DWORD                       dwSize;
    UINT                        nIndex;                                              // ABlock route index
    int                         nPointNum;                                           // point number of ABlock
    NET_POINT_DOOR_INFO         stuPointInfo[MAX_ACCESS_POINT_NUM];                  // point info
    int                         nTimeSection;                                        // config of AccessTimeSchedule number, please query dhconfigsdk.h
    BOOL                        bFlag;                                               // enable
    char                        szRouteName[MAX_COMMON_STRING_128];                  // route name
}NET_IN_SET_ABLOCK_ROUTE;

//  out parameter of get ABlock  route
typedef struct tagNET_OUT_SET_ABLOCK_ROUTE
{
    DWORD                       dwSize;
}NET_OUT_SET_ABLOCK_ROUTE;

// in parameter of get ABlock  route
typedef struct tagNET_IN_GET_ABLOCK_ROUTE
{
    DWORD                       dwSize;
    UINT                        nIndex;                                              // ABlock route index
}NET_IN_GET_ABLOCK_ROUTE;

// out parameter of get ABlock  route
typedef struct tagNET_OUT_GET_ABLOCK_ROUTE
{
    DWORD                       dwSize;
    int                         nPointNum;                                           // point number of ABlock
    NET_POINT_DOOR_INFO         stuPointInfo[MAX_ACCESS_POINT_NUM];                  // point info
    int                         nTimeSection;                                        // config of AccessTimeSchedule number,please query dhconfigsdk.h
    BOOL                        bFlag;                                               // enable
    char                        szRouteName[MAX_COMMON_STRING_128];                  // route name
}NET_OUT_GET_ABLOCK_ROUTE;

// in parameter of get log status
typedef struct tagNET_IN_GET_LOGSTATUS
{
    DWORD                       dwSize;
    int                         nControllerID;                                        // accesssubcontroller ID,range:1 ~255
}NET_IN_GET_LOGSTATUS;

// out parameter of get log status
typedef struct tagNET_OUT_GET_LOGSTATUS
{
    DWORD                       dwSize;
    BOOL                        bsynchronize;                                         // true:synchronizing
}NET_OUT_GET_LOGSTATUS;

// in parameter of synchro offline log
typedef struct tagNET_IN_SYNCHRO_OFFLINE_LOG
{
    DWORD                       dwSize;
    int                         nControllerID;                                        // ControllerID range:0 ~255
}NET_IN_SYNCHRO_OFFLINE_LOG;

// out parameter of synchro offline log
typedef struct tagNET_OUT_SYNCHRO_OFFLINE_LOG
{
    DWORD                       dwSize;
}NET_OUT_SYNCHRO_OFFLINE_LOG;

// in parameter of synchro time
typedef struct tagNET_IN_SYNCHRO_CONTROLLER_TIME
{
    DWORD                       dwSize;
    int                         nSubControllerID[MAX_ACCESSSUBCONTROLLER_NUM];  // accesssubcontroller ID, range:-1~255, -1:all,0:Local,1~255:other
    int                         nSubControllerNum;                              // accesssubcontroller number    
}NET_IN_SYNCHRO_CONTROLLER_TIME;

// out parameter of synchro time
typedef struct tagNET_OUT_SYNCHRO_CONTROLLER_TIME
{
    DWORD                       dwSize;
    int                         nSubControllerID[MAX_ACCESSSUBCONTROLLER_NUM];  // accesssubcontroller info
    int                         nRetNum;                                        // return number of failed, 0: all success    
}NET_OUT_SYNCHRO_CONTROLLER_TIME;

// interface of operate AccessControlManager
CLIENT_NET_API BOOL CALL_METHOD CLIENT_OperateAccessControlManager(LLONG lLoginID, NET_EM_ACCESS_CTL_MANAGER emtype, const void *pstInParam, void *pstOutParam, int nWaitTime);

// the input parameters of CLIENT_SetSmartLockUserName
typedef struct tagNET_IN_SET_SMART_LOCK_USERNAME
{
    DWORD						dwSize;										// struct size
	NET_ACCESS_METHOD			emType;										// open smart lock type
    char						szSerialNumber[MAX_COMMON_STRING_32];       // smart lock serial number									
	char						szUserID[MAX_COMMON_STRING_16];				// user ID
	char						szName[MAX_COMMON_STRING_32];				// modified name
} NET_IN_SET_SMART_LOCK_USERNAME;

// the output parameters of CLIENT_SetSmartLockUserName
typedef struct tagNET_OUT_SET_SMART_LOCK_USERNAME
{
    DWORD						dwSize;										// struct size   
} NET_OUT_SET_SMART_LOCK_USERNAME;

// Modify the user information for smart locks
CLIENT_NET_API BOOL CALL_METHOD CLIENT_SetSmartLockUsername(LLONG lLoginID, const NET_IN_SET_SMART_LOCK_USERNAME *pstInParam, NET_OUT_SET_SMART_LOCK_USERNAME *pstOutParam, int nWaitTime);

#define MAX_NUMBER_REGISTER_INFO	32

// // the input parameters of CLIENT_GetSmartLockRegisterInfo (you can get at most32 user info for one time)
typedef struct tagNET_IN_GET_SMART_LOCK_REGISTER_INFO
{
    DWORD			dwSize;										// struct size
    char            szSerialNumber[MAX_COMMON_STRING_32];       // device serial number
	int				nOffset;									// the offset of user list
} NET_IN_GET_SMART_LOCK_REGISTER_INFO;

// user register info
typedef struct tagNET_SMART_LOCK_REGISTER_INFO
{
	NET_ACCESS_METHOD			emType;										// open smart lock type
	char						szUserID[MAX_COMMON_STRING_16];				// user id
	char						szName[MAX_COMMON_STRING_32];				// user name
	BYTE						byReserved[512];							// Reserve
} NET_SMART_LOCK_REGISTER_INFO;

// the output parameters of CLIENT_GetSmartLockRegisterInfo
typedef struct tagNET_OUT_GET_SMART_LOCK_REGISTER_INFO
{
    DWORD							dwSize;						// struct size   
	int								nTotalCount;				// total user number
	int								nReturnCount;				// return user number
	NET_SMART_LOCK_REGISTER_INFO	stuRegisterInfo[MAX_NUMBER_REGISTER_INFO];	// user register info
} NET_OUT_GET_SMART_LOCK_REGISTER_INFO;

// get register info of the user of smart lock  
CLIENT_NET_API BOOL CALL_METHOD CLIENT_GetSmartLockRegisterInfo(LLONG lLoginID, const NET_IN_GET_SMART_LOCK_REGISTER_INFO *pstInParam, NET_OUT_GET_SMART_LOCK_REGISTER_INFO *pstOutParam, int nWaitTime);


// user info
typedef struct tagMATRIX_CAMERA_USERS_INFO 
{	
	unsigned int				nChannel;					// channel
	char						szUser[MAX_USER_NAME_LEN];	// user name
    char						szPwd[MAX_PWD_LEN];			// passwd
	char						szReserve[512];				// reserved
}MATRIX_CAMERA_USERS_INFO;

// the intput parameter of CLIENT_MatrixGetCameraUsers
typedef struct tagNET_IN_MATRIX_GET_CAMERA_USERS
{
	DWORD						dwSize;						// struct size	
	unsigned int				nStartChannel;				// start channel
	unsigned int				nChannelCount;				// channel num
}NET_IN_MATRIX_GET_CAMERA_USERS;

// the output parameter of CLIENT_MatrixGetCameraUsers
typedef struct tagNET_OUT_MATRIX_GET_CAMERA_USERS
{
	DWORD						dwSize;						// struct size
	MATRIX_CAMERA_USERS_INFO*	pstuUserInfo;				// user info structuser alloc and free memoryequal to sizeof(MATRIX_CAMERA_USERS_INFO)*nChannelCount,nChannelCount is in NET_IN_MATRIX_GET_CAMERA_USERS																
	unsigned int				nChannelCountRet;			// channel num return 
}NET_OUT_MATRIX_GET_CAMERA_USERS;

// get user info of valid display source
CLIENT_NET_API BOOL CALL_METHOD CLIENT_MatrixGetCameraUsers(LLONG lLoginID, const NET_IN_MATRIX_GET_CAMERA_USERS* pInParam, NET_OUT_MATRIX_GET_CAMERA_USERS* pOutParam, DWORD dwWaitTime);

// +++++++++++++++++++++++++++++++++++++++++++++++++++++API for MPT exchange data with NVR+++++++++++++++++++++++++++++++++++++++++++

// MPT state info
typedef struct tagNET_MPT_STATUS
{
    EM_MPT_STATE eState;                                            // MPT state
    char szSerialNumber[DH_COMMON_STRING_32];                       // SN of MPT,it is valid when state is 2 or 3
    BYTE byReserved[256];                                           // reserved
}NET_MPT_STATUS;

// the input param for get the MPT status
typedef struct tagNET_IN_GET_MPT_STATUS
{
    DWORD					dwSize;									// struct size
    UINT                    nOffset;                                // offset of usb slot
    UINT                    nCount;                                 // usb slot num
}NET_IN_GET_MPT_STATUS;

#define MAX_MPT_STATUS_NUM (64)                                     // max num of mpt status info

// the output param for get the MPT status
typedef struct tagNET_OUT_GET_MPT_STATUS
{
    DWORD					dwSize;									// struct size
    UINT                    nTotalUsbNum;                           // total usb slot num
    UINT                    nStatusNum;                             // MPT state num
    NET_MPT_STATUS          stuStatus[MAX_MPT_STATUS_NUM];          // MPT state data
}NET_OUT_GET_MPT_STATUS;

// Get the MPT status
CLIENT_NET_API BOOL CALL_METHOD CLIENT_GetMPTStatus(LLONG lLoginID,const NET_IN_GET_MPT_STATUS* pstInParam, NET_OUT_GET_MPT_STATUS* pstOutParam, DWORD dwWaitTime);

//  the type of MPT status
typedef enum tagEM_MPTSTATUS_TYPE
{
	EM_MPTSTATUS_UNKNOWN,			// unknown
	EM_MPTSTATUS_MPTBASE,			// the status of MPT base,  pOutParam = NET_MPTBASE_STATUS_INFO
	EM_MPTSTATUS_EXTCAMERA,			// the status of external camera, pOutParam = NET_EXTCAMERA_STATUS_INFO
	EM_MPTSTATUS_NETCONNECT,		// the type of network connection, pOutParam = NET_NETCONNECT_STATUS_INFO
} EM_MPTSTATUS_TYPE;

// the type of MPT base status
typedef enum tagNEM_MPTBASE_TYPE
{
	NEM_MPTBASE_UNKNOWN,				// unknown
	NEM_MPTBASE_ON,						// ON
	NEM_MPTBASE_OFF,					// OFF
} NEM_MPTBASE_TYPE;

// the type of external camera status
typedef enum tagEM_EXTCAMERA_TYPE
{
	EM_EXTCAMERA_UNKNOWN,				// unkonwn
	EM_EXTCAMERA_NORMAL,				//Normal
	EM_EXTCAMERA_ABNORMAL,				//Abnormal
} EM_EXTCAMERA_TYPE;

// the type of network connection
typedef enum tagEM_NETCONNECT_TYPE
{
	EM_NETCONNECT_UNKNOWN,				// unknown
	EM_NETCONNECT_WIFI,					// WIFI
	EM_NETCONNECT_MOBILE,				// Mobile
	EM_NETCONNECT_CABLE,				// Cable
	EM_NETCONNECT_MPTBASE,				// MptBase
} EM_NETCONNECT_TYPE;

// the info of MPT base
typedef struct tagNET_MPTBASE_STATUS_INFO
{
	DWORD					dwSize;									// the size of this struct(set the value when init)
	NEM_MPTBASE_TYPE		emMptBase;								//  the type of MPT base status
} NET_MPTBASE_STATUS_INFO;

// the info of external camera
typedef struct tagNET_EXTCAMERA_STATUS_INFO
{
	DWORD					dwSize;									// the size of this struct(set the value when init)
	EM_EXTCAMERA_TYPE		emExtCamera;							// the type of external camera status
} NET_EXTCAMERA_STATUS_INFO;

// the info of network connection
typedef struct tagNET_NETCONNECT_STATUS_INFO
{
	DWORD					dwSize;									// the size of this struct(set the value when init)
	EM_NETCONNECT_TYPE		emNetConnect;							// the type of network connection
} NET_NETCONNECT_STATUS_INFO;


// query the MPT status info
CLIENT_NET_API BOOL CALL_METHOD	CLIENT_QueryMPTStatus(LLONG lLoginID, const EM_MPTSTATUS_TYPE emStatusType, void* pstOutParam, DWORD dwWaitTime);


#define MAX_POLICE_ID_NUM 32                                        // max police id num

// the police id info to be written to MPT
typedef struct tagNET_WRITE_POLICE_ID_INFO
{
    int                     nUsbSlot;                                               // usb slot index
    int                     nPoliceIDCount;                                         // police id num                         
    char                    szPoliceID[MAX_POLICE_ID_NUM][DH_COMMON_STRING_32];     // police id data
    BYTE                    byReserved[512];                                        // reserved
}NET_WRITE_POLICE_ID_INFO;

#define MAX_POLICE_ID_INFO_NUM 64                                                   // max police info num

// the input param for write police id info to MPT
typedef struct tagNET_IN_WRITE_POLICE_ID
{
    DWORD					    dwSize;						                        // struct size
    int                         nIDInfoNum;                                         // police id num
    NET_WRITE_POLICE_ID_INFO    stuIDInfo[MAX_POLICE_ID_INFO_NUM];                  // police id data
}NET_IN_WRITE_POLICE_ID;

// the output param for write police id info to MPT
typedef struct tagNET_OUT_WRITE_POLICE_ID
{
    DWORD					dwSize;									                // struct size
    int                     nResultCount;                                           // result num
    BOOL                    bResult[MAX_POLICE_ID_INFO_NUM];                        // result data, 0-success,1-fail
}NET_OUT_WRITE_POLICE_ID;

// write police id info to MPT
CLIENT_NET_API BOOL CALL_METHOD CLIENT_WritePoliceID(LLONG lLoginID,const NET_IN_WRITE_POLICE_ID* pstInParam, NET_OUT_WRITE_POLICE_ID* pstOutParam, DWORD dwWaitTime);

#define MAX_MPT_MEDIA_TYPE_NUM 64                                                   // max num of mpt record file type

// the input param for mark the MPT record file
typedef struct tagNET_IN_MARK_MPT_RECORD_FILE
{
    DWORD					dwSize;									// struct size
    int                     nChannelID;                             // channel id
    NET_TIME                stuStartTime;                           // start time
    NET_TIME                stuEndTime;                             // end time
    int                     nMediaTypeCount;                        // file type num                        
    int                     nMediaType[MAX_MPT_MEDIA_TYPE_NUM];     // file type data,0:all type, 1:jpg, 2:dav
    int                     nCrimeType;                             // crime type
    int                     nCrimeTag;                              // crime tag
    char                    szComment[DH_COMMON_STRING_256];        // comment for mark
}NET_IN_MARK_MPT_RECORD_FILE;

// the output param for mark the MPT record file
typedef struct tagNET_OUT_MARK_MPT_RECORD_FILE
{
    DWORD					dwSize;									// struct size   
}NET_OUT_MARK_MPT_RECORD_FILE;

// Mark the MPT record file
CLIENT_NET_API BOOL CALL_METHOD CLIENT_MarkMPTRecordFile(LLONG lLoginID,const NET_IN_MARK_MPT_RECORD_FILE* pstInParam, NET_OUT_MARK_MPT_RECORD_FILE* pstOutParam, DWORD dwWaitTime);

// +++++++++++++++++++++++++++++++++++++++++++++++++++++API for MPT exchange data with NVR+++++++++++++++++++++++++++++++++++++++++++

// +++++++++++++++++++++++++++++++++++++++++++++++++++++Device Diagnosis+++++++++++++++++++++++++++++++++++++++++++

// the input param for query diagnosis info
typedef struct tagNET_IN_QUERY_DIAGNOSIS_INFO
{
    DWORD dwSize;                                   // strcut size
    UINT nOffset;                                   // offset to query
    UINT nCount;                                    // count of query
}NET_IN_QUERY_DIAGNOSIS_INFO;

// diagnosis info item
typedef struct tagNET_DIAGNOSIS_ITEM_INFO
{
    NET_TIME stuOccurTime;                          // event occur time 
    UINT nFaultCode;                                // fault code 
    BYTE byReserved[256];                           // reserved
}NET_DIAGNOSIS_ITEM_INFO;

// the output param for query diagnosis info
typedef struct tagNET_OUT_QUERY_DIAGNOSIS_INFO
{
    DWORD dwSize;                                   // struct size
    UINT nTotalCount;                               // total item num    
    NET_DIAGNOSIS_ITEM_INFO* pstuItemInfo;          // diagnosis info data pointer
    UINT nMaxCount;                                 // the max count of item
    UINT nRetCount;                                 // return count form device
}NET_OUT_QUERY_DIAGNOSIS_INFO;

// query diagnosis info
CLIENT_NET_API BOOL CALL_METHOD CLIENT_QueryDiagnosisInfo(LLONG lLoginID,const NET_IN_QUERY_DIAGNOSIS_INFO* pstInParam, NET_OUT_QUERY_DIAGNOSIS_INFO* pstOutParam, int nWaitTime);


// the input param for reset system
typedef struct tagNET_IN_RESET_SYSTEM
{
    DWORD dwSize;
}NET_IN_RESET_SYSTEM;

// the output param for reset system
typedef struct tagNET_OUT_RESET_SYSTEM
{
    DWORD dwSize;
}NET_OUT_RESET_SYSTEM;

// reset the system(clear user data and reboot to device init state)
CLIENT_NET_API BOOL CALL_METHOD CLIENT_ResetSystem(LLONG lLoginID,const NET_IN_RESET_SYSTEM* pstInParam, NET_OUT_RESET_SYSTEM* pstOutParam, int nWaitTime);

// +++++++++++++++++++++++++++++++++++++++++++++++++++++Device Diagnosis+++++++++++++++++++++++++++++++++++++++++++

#define MAX_IMSI_LEN		32		// max len of IMSI
#define MAX_ICCID_LEN		32		// max len of ICCID

// the mode of wireless
typedef enum tagEM_WIRELESS_MODE
{
	EM_WIRELESS_UNKNOWN,
	EM_WIRELESS_3G,
	EM_WIRELESS_3G1,
	EM_WIRELESS_3G2,
	EM_WIRELESS_3G3,
} EM_WIRELESS_MODE;

// the input param for CLIENT_MobileGetIMSI
typedef struct tagNET_IN_GET_IMSI
{
	DWORD 				dwSize;
	EM_WIRELESS_MODE	emMode;		// the mode of wireless
} NET_IN_GET_IMSI;

// the output param for CLIENT_MobileGetIMSI
typedef struct tagNET_OUT_GET_IMSI
{
	DWORD 		dwSize;
	char 		szIMSI[MAX_IMSI_LEN];			// IMSI
	char		szICCID[MAX_ICCID_LEN];			// ICCID
} NET_OUT_GET_IMSI;

// Get the IMSI of the SIM card
CLIENT_NET_API BOOL CALL_METHOD CLIENT_MobileGetIMSI(LLONG lLoginID, const NET_IN_GET_IMSI* pstInParam, NET_OUT_GET_IMSI* pstOutParam, int nWaitTime);

#define MAX_ROOMNUM_COUNT		32		// max count of rooms
#define MAX_FACE_COUTN			20		// max count of face data

// the info of fae
typedef struct tagNET_FACE_RECORD_INFO
{
	char		szUserName[MAX_USER_NAME_LEN];						// user name
	int			nRoom;												// count of rooms
	char		szRoomNo[MAX_ROOMNUM_COUNT][DH_COMMON_STRING_16]; 	// list of rooms
	int 		nFaceData;											// count of face data
	char		szFaceData[MAX_FACE_COUTN][MAX_FACE_DATA_LEN];		// face data
    BYTE        byReserved[512];                                    // reserved
} NET_FACE_RECORD_INFO;

// the input param of adding face data
typedef struct tagNET_IN_ADD_FACE_INFO
{
	DWORD 					dwSize;
	char					szUserID[DH_MAX_USERID_LEN];	// user ID
	NET_FACE_RECORD_INFO	stuFaceInfo;					// face data
} NET_IN_ADD_FACE_INFO;

// the output param of adding face data
typedef struct tagNET_OUT_ADD_FACE_INFO
{
	DWORD 			dwSize;
} NET_OUT_ADD_FACE_INFO;


// the input param of getting face data
typedef struct tagNET_IN_GET_FACE_INFO
{
	DWORD 			dwSize;
	char			szUserID[DH_MAX_USERID_LEN];	// user ID
} NET_IN_GET_FACE_INFO;

// the out param of getting face data
typedef struct tagNET_OUT_GET_FACE_INFO
{
	DWORD 			dwSize;
	int 			nFaceData;											// count of face data
	char			szFaceData[MAX_FACE_COUTN][MAX_FACE_DATA_LEN];		// face data
} NET_OUT_GET_FACE_INFO;

// the input param to updata face data
typedef struct tagNET_IN_UPDATE_FACE_INFO
{
	DWORD 					dwSize;
	char					szUserID[DH_MAX_USERID_LEN];	// user ID
	NET_FACE_RECORD_INFO	stuFaceInfo;					// the info of face
} NET_IN_UPDATE_FACE_INFO;

// the output param to updata face data
typedef struct tagNET_OUT_UPDATE_FACE_INFO
{
	DWORD 			dwSize;
} NET_OUT_UPDATE_FACE_INFO;


// the input param of removing face data
typedef struct tagNET_IN_REMOVE_FACE_INFO
{
	DWORD 			dwSize;
	char			szUserID[DH_MAX_USERID_LEN];	// user ID
} NET_IN_REMOVE_FACE_INFO;

// the output param of removing face data
typedef struct tagNET_OUT_REMOVE_FACE_INFO
{
	DWORD 			dwSize;
} NET_OUT_REMOVE_FACE_INFO;

// the input param of clear face data
typedef struct tagNET_IN_CLEAR_FACE_INFO
{
	DWORD 			dwSize;
} NET_IN_CLEAR_FACE_INFO;

// the output param of clear face data
typedef struct tagNET_OUT_CLEAR_FACE_INFO
{
	DWORD 			dwSize;
} NET_OUT_CLEAR_FACE_INFO;

// the opreate type of face info
typedef enum tagEM_FACEINFO_OPREATE_TYPE
{
	EM_FACEINFO_OPREATE_ADD,				// add, pInbuf = NET_IN_ADD_FACE_INFO , pOutBuf = NET_OUT_ADD_FACE_INFO
	EM_FACEINFO_OPREATE_GET,				// get, pInBuf = NET_IN_GET_FACE_INFO , pOutBuf = NET_OUT_GET_FACE_INFO
	EM_FACEINFO_OPREATE_UPDATE,				// update, pInbuf = NET_IN_UPDATE_FACE_INFO , pOutBuf = NET_OUT_UPDATE_FACE_INFO
	EM_FACEINFO_OPREATE_REMOVE,				// remove, pInbuf = NET_IN_REMOVE_FACE_INFO , pOutBuf = NET_OUT_REMOVE_FACE_INFO
	EM_FACEINFO_OPREATE_CLEAR,				// clear, pInbuf = NET_IN_CLEAR_FACE_INFO, pOutBuf = NET_OUT_CLEAR_FACE_INFO
} EM_FACEINFO_OPREATE_TYPE;

/**************************************************************************************
*   Funcname: CLIENT_FaceInfoOpreate
*   Purpose:the opreation function of face info
*   InputParam:	LLONG		:lLoginID		// login handle
*   InputParam:     EM_FACEINFO_OPREATE_TYPE: emType //opreate type
*   InputParam:	void*		:pInParam	// the input param, the resource is maintained by user
*   OutputParam:	void*		:pOutParam	// the outtime param, the resource is maintained by user
*   InputParam:	int			:nWaitTime	// timeout
*   Return:		BOOL
*   Created:		%2017%:%08%:%30%  
*   Revision Record:    date:author:modify sth
**************************************************************************************/
CLIENT_NET_API BOOL CALL_METHOD CLIENT_FaceInfoOpreate(LLONG lLoginID, EM_FACEINFO_OPREATE_TYPE emType, void* pInParam, void* pOutParam, int nWaitTime = NET_INTERFACE_DEFAULT_TIMEOUT);

/**************************************************************************************
*   Funcname: CLIENT_RenderPrivateData
*   Purpose:display private data
*   InputParam:     LLONG		:lPlayHandle        	// play handle
*   InputParam:     BOOL		:bTrue        		// TRUE:display  FALSE:not display
*   Return:		BOOL:TRUE--sucess   FALSE--failed
*   Created:	       %2017%:%09%:%25%  
*   Revision Record:    date:author:modify sth
**************************************************************************************/
CLIENT_NET_API BOOL CALL_METHOD CLIENT_RenderPrivateData(LLONG lPlayHandle, BOOL bTrue);

#ifdef __cplusplus
}
#endif
#endif // DHNETSDK_H



